describe("UtilDevTools", function() {
	var utilDevTools;
	UtilDevTools();

	var method = {
		"apply": function(obj, args){
			obj.new = args;
		},
		"widget": "widget"
	};

	var developerWindow = {
		"name": "developer"
	};

	var setWindowExpectedValue = function(){
		window.expectedValue = 1;
	};

	beforeEach(function(){
		window.name = "main";
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		window.inCustomizationMode = true;
		module('ZeedhiApp');
		utilDevTools = UtilDevTools;
	});

	it("UtilDevTools should be a function.", function () {
		expect(typeof utilDevTools).toBe("function");
	});

	it("BuildInDeveloperModeMethod should return a function.", function () {
		expect(typeof utilDevTools.buildInDeveloperModeMethod() ).toBe("function");
	});

	it("BuildInCustomizationModeMethod should return a function.", function () {
		expect(typeof utilDevTools.buildInCustomizationModeMethod() ).toBe("function");
	});

	it("BuildDevToolsWindowMethod should return a function.", function () {
		expect(typeof utilDevTools.buildDevToolsWindowMethod() ).toBe("function");
	});

	it("GetDevToolsWindow when is a parent and there is a opener should return window.", function () {
		window.name = developerWindow.name;
		window.parent = window;
		window.opener = {};
		expect(utilDevTools.getDevToolsWindow().name).toBe(window.name);
	});

	it("GetDevToolsWindow when isn't a parent and there is a parent should return window.", function () {
		window.name = developerWindow.name;
		window.parent = {};
		expect(utilDevTools.getDevToolsWindow().name).toBe(window.name);
	});

	it("GetDevToolsWindow when there isn't a parent neither a opener should return null.", function () {
		window.opener = null;
		window.parent = null;
		expect(utilDevTools.getDevToolsWindow()).toBe(null);
	});

	it("GetDevToolsWindow when is a parent and there isn't a opener should return null.", function () {
		window.opener = null;
		window.parent = window;
		expect(utilDevTools.getDevToolsWindow()).toBe(null);
	});

	it("GetDevToolsWindow when isn't a parent and there isn't a parent should return null.", function () {
		window.parent = null;
		expect(utilDevTools.getDevToolsWindow()).toBe(null);
	});

	it("GetMainWindow when is a parent and there is a opener should return window.opener.", function () {
		window.parent = window;
		window.opener = { name: "opener" };
		expect(utilDevTools.getMainWindow().name).toBe(window.opener.name);
	});

	it("GetMainWindow when is a parent and there isn't a opener should return window.", function () {
		window.parent = window;
		window.opener = null;
		expect(utilDevTools.getMainWindow().name).toBe(window.name);
	});

	it("GetMainWindow when isn't a parent and there is a parent should return window.parent.", function () {
		window.parent = { name: "parent" };
		expect(utilDevTools.getMainWindow().name).toBe(window.parent.name);
	});

	it("GetMainWindow when there isn't a parent neither a opener should return window.", function () {
		window.parent = null;
		window.opener = null;
		expect(utilDevTools.getMainWindow().name).toBe(window.name);
	});

	it("BuildInDeveloperModeMethod should apply a method when window is inDeveloperMode and window isn't inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inDeveloperMode = true;
		window.inCustomizationMode = false;
		UtilDevTools.buildInDeveloperModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildInDeveloperModeMethod should apply a method when window isn't inDeveloperMode and window is inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inDeveloperMode = false;
		window.inCustomizationMode = true;
		UtilDevTools.buildInDeveloperModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildInDeveloperModeMethod should apply a method when window is inDeveloperMode and window is inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inDeveloperMode = true;
		window.inCustomizationMode = true;
		UtilDevTools.buildInDeveloperModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildInDeveloperModeMethod shouldn't apply a method when window isn't inDeveloperMode and window isn't inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inDeveloperMode = false;
		window.inCustomizationMode = false;
		UtilDevTools.buildInDeveloperModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildInCustomizationModeMethod should apply a method when a window is inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inCustomizationMode = true;
		UtilDevTools.buildInCustomizationModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildInCustomizationModeMethod shouldn't apply a method when a window isn't inCustomizationMode.", function () {
		window.expectedValue = 0;
		window.inCustomizationMode = false;
		UtilDevTools.buildInCustomizationModeMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildMainWindowMethod should apply a method when window hasn't a parent.", function () {
		window.expectedValue = 0;
		window.parent = null;
		UtilDevTools.buildMainWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildMainWindowMethod shouldn't apply a method when window has other window in parent.", function () {
		window.parent = developerWindow;
		window.expectedValue = 0;
		UtilDevTools.buildMainWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildMainWindowMethod should apply a method when window has itself in parent and window hasn't a opener.", function () {
		window.parent = window;
		window.opener = null;
		window.expectedValue = 0;
		UtilDevTools.buildMainWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildMainWindowMethod shouldn't apply a method when window has itself in parent and other window in opener property.", function () {
		window.parent = window;
		window.opener = developerWindow;
		window.expectedValue = 0;
		UtilDevTools.buildMainWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildMainWindowMethodMethod should apply a method when window has itself in parent and opener property.", function () {
		window.parent = window;
		window.opener = window;
		window.expectedValue = 0;
		UtilDevTools.buildMainWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildDevToolsWindowMethod shouldn't apply a method when window hasn't a parent property.", function () {
		window.expectedValue = 0;
		window.parent = null;
		UtilDevTools.buildDevToolsWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildDevToolsWindowMethod should apply a method when window has parent different from the window.", function () {
		window.parent = developerWindow;
		window.expectedValue = 0;
		UtilDevTools.buildDevToolsWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

	it("BuildDevToolsWindowMethod shouldn't apply a method when window has itself as parent and window hasn't a opener property.", function () {
		window.parent = window;
		window.opener = null;
		window.expectedValue = 0;
		UtilDevTools.buildDevToolsWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(0);
	});

	it("BuildDevToolsWindowMethod should apply a method when window has itself as parent and opener property.", function () {
		window.parent = window;
		window.opener = window;
		window.expectedValue = 0;
		UtilDevTools.buildDevToolsWindowMethod(setWindowExpectedValue)();
		expect(window.expectedValue).toBe(1);
	});

});
