function DTServerService($http, ScreenService, DTMessages, DeveloperService){
	var serverUrl = 'http://192.168.120.153:3000';
	var getContainerRoute = '/getContainer',
		getComponentRoute = '/getComponent',
		saveComponentRoute = '/saveComponent',
		deleteComponentRoute = "/deleteComponent",
		resetContainerRoute = "/resetContainer",
		getFileRoute = '/getFile',
		getAllFilesRoute = '/getAllFiles',
		saveFileRoute = '/saveFile',
		deleteFileRoute = "/deleteFile";

	var sendRequest = function(dataToSend, route, callback){
		return $http({
			url: serverUrl.concat(route),
			method: 'POST',
			data: dataToSend,
			timeout: 6000
		}).then(function(response){
			return callback(response);
		}).catch(function(err){
			ScreenService.notificationMessage(DTMessages.CANT_CONNECT, 'error');
			return {"success": false, "response": err };
		});
	};

	var deleteCustomization = function(id, route, errMsg, successMsg){
		var callback = function(response){
			var responseData = response.data.responseData;
			var wasDeleted = responseData.ok;
			if(wasDeleted){
				ScreenService.successNotification(successMsg);
			} else {
				ScreenService.notificationMessage(errMsg, 'error');
				console.error(response);
			}
			return wasDeleted;
		};
		return sendRequest({"id": id}, route, callback);
	};

	var isNewCustomId = function(customId){
		return customId.toString().indexOf('NEW') === 0;
	};

	this.getContainer = function(id){
		var criteria = { "id": id };
		var callback = function(response){
			var responseData = response.data || {};
			var responseItems = (responseData.success && responseData.responseData) || [];
			var data = {
				updatedItems: {},
				newItems: {}
			};
			responseItems.forEach(function(item){
				var itemId = item.id;
				var jsonItem = item.jsonItem;
				if(isNewCustomId(itemId)){
					jsonItem._path = item.path;
					data.newItems[itemId] = jsonItem;
				} else {
					data.updatedItems[itemId] = jsonItem;
				}
			});
			return data;
		};
		return sendRequest(criteria, getContainerRoute, callback);
	};

	this.getCustomComponent = function(id, containerId){
		var criteria = { "id": id, "containerId": containerId };
		var callback = function(response){
			var data = {};
			var responseData = response.data;
			if(responseData && responseData.success){
				var responseItems = responseData.responseData || [];
				responseItems.some(function(item){
					if(item.id == id){
						data = item.jsonItem;
						return true;
					}
				});
			}
			return data;
		};

		return sendRequest(criteria, getComponentRoute, callback);
	};

	this.getCustomFile = function(id){
		var callback = function(response){
			console.log(response);
		};
		return sendRequest({"id": id}, getFileRoute, callback);
	};

	this.getAllCustomFiles = function(){
		var callback = function(response){
			var responseData = response.data;
			var data = {};
			if(responseData && responseData.success){
				var responseItems = responseData.responseData || [];
				responseItems.forEach(function(item){
					data[item.id] = item.jsonItem;
				});
			}
			return data;
		};
		return sendRequest({}, getAllFilesRoute, callback);
	};

	this.saveCustomComponent = function(id, containerId, path, componentObject){
		var dataToSave = {
			"id": id,
			"jsonItem": componentObject,
			"containerId": containerId,
			"path": path || [],
			"email": DeveloperService.getLocalEmail()
		};
		var callback = function(response){
			var success = response.data && response.data.success;
			if(success){
				ScreenService.successNotification(DTMessages.CUSTOM_SAVED);
			} else {
				ScreenService.notificationMessage(DTMessages.CUSTOM_WASNT_SAVED, 'error');
			}
			return { "success": success };
		};
		return sendRequest(dataToSave, saveComponentRoute, callback);
	};

	this.deleteItemCustomization = function(id){
		return deleteCustomization(id, deleteComponentRoute, DTMessages.CUSTOM_ITEM_WASNT_DELETED, DTMessages.CUSTOM_ITEM_WAS_DELETED);
	};

	this.deleteCustomFile = function(fileName){
		deleteCustomization(fileName, deleteFileRoute, DTMessages.CUSTOM_FILE_WASNT_DELETED, DTMessages.CUSTOM_FILE_WAS_DELETED);
	};

	this.resetContainer = function(containerId){
		var callback = function(response){
			var responseData = response.data.responseData;
			var wasDeleted = responseData.ok;
			if(wasDeleted){
				ScreenService.successNotification(DTMessages.getMessageContainerWasReseted(response.n));
			} else {
				ScreenService.notificationMessage(DTMessages.CUSTOM_CONTAINER_WASNT_RESETED, 'error');
				console.error(response);
			}
			return wasDeleted;
		};
		return sendRequest({"containerId": containerId}, resetContainerRoute, callback);
	};
}
