function DeveloperToolsForm(metaDataFactory, ScreenService, templateManager, DevToolsService){
	var DEFAULT_TEMPLATES = ["field/text-edit.html", "widget/form.html", "container/window.html"];
	var DEFAULT_WIDGET = {
		"name": "widgetToForm",
		"label": "widgetToForm",
		"isVisible": true,
		"template": "widget/form.html",
		"actions": [],
	};
	var DEFAULT_CONTAINER = {
		"name": "containerForm",
		"label": "containerForm",
		"showHeader": false,
		"height": "100%",
		"isVisible": true,
		"showFooter": false,
		"footer": "component/footer.html",
		"template": "container/window.html",
		"addToHistory": false,
		"widgets": []
	};
	var CONFIG_TEMPLATES = {
		"string": {
			"template": "field/text-edit.html"
		},
		"number": {
			"template": "field/number.html"
		},
		"integer": {
			"template": "field/number.html"
		},
		"boolean": {
			"template": "field/checkbox.html",
			"extraProperties": {
				"valueToTrue": true,
				"valueToFalse": false
			}
		},
		"array": {
			"template": "field/textarea.html",
			"extraProperties": {
				"typeOfValue": "array",
				"textRows": 5
			}
		},
		"object": {
			"template": "field/textarea.html",
			"extraProperties": {
				"typeOfValue": "object",
				"textRows": 5
			}
		}
	};

	function hasPropertyInJson(json, propertyName){
		return Object.keys(json).indexOf(propertyName) >= 0;
	}
	this.hasPropertyInJson = hasPropertyInJson;

	function isCheckboxField(template){
		return template.indexOf('checkbox') >= 0;
	}
	this.isCheckboxField = isCheckboxField;

	function setDependenciesInField(dependencies, field){
		if(dependencies){
			var fieldName = field.name;
			var dependsOn = dependencies.dependsOn;
			var fieldDependsOn = dependsOn ? dependsOn[fieldName] : undefined;
			var dependencieFields = dependencies.dependenciesTree;
			var fieldDependencieFields = dependencieFields ? dependencieFields[fieldName] : undefined;
			if (fieldDependsOn){
				field.dependsOn = fieldDependsOn;
			}
			if(fieldDependencieFields && fieldDependencieFields.length){
				field.dependencieFields = fieldDependencieFields;
				field.events = [{
					"name":"FieldOnChange",
					"code": "DevToolsController.updateDependencyFieldsVisibility(args.owner.field.widget, " + '"' + fieldName + '"' + ");",
					"id": (+new Date())
				}];
			}
		}
		return field;
	}

	function createFieldToShowTemplate(template){
		var field = createField({ name: "template", type: "string", description: template });
		field.defaultValue = field.placeholder;
		field.readOnly = true;
		return field;
	}

	function valueFunctionOfObjectField(value){
		if(value !== undefined){
			if(typeof value == 'object'){
				value = JSON.stringify(value, false, 2);
			}
			this.widget.currentRow[this.name] = value;
			templateManager.updateTemplate();
		} else {
			value = this.widget.currentRow[this.name] || "";
			value = value ? JSON.parse(value) : value;
			return value;
		}
	}

	function prepareValueFunction(field){
		if(field.typeOfValue){
			field.value = valueFunctionOfObjectField;
		}
	}

	function getDefaultField(name, description, template){
		var fieldClass = isCheckboxField(template) ? "6 zh-dvtools" : "12 zh-dvtools";
		var field = {
			"class": fieldClass,
			"showOnForm": true,
			"isVisible": true,
			"template": template,
			"id": (+new Date()),
			"name": name,
			"label": name,
			"placeholder": description,
			"help": description,
			"title": description
		};
		return field;
	}

	function createField(fieldProperties, dependencies){
		var type = fieldProperties.type;
		var configTemplate = CONFIG_TEMPLATES[type] || CONFIG_TEMPLATES.string;
		var fieldTemplate = configTemplate.template;
		var extraProperties = configTemplate.extraProperties || {};
		var field = getDefaultField(fieldProperties.name, fieldProperties.description, fieldTemplate);

		if(!!fieldProperties.isRequired){
			field.validations = { "required": !!fieldProperties.isRequired };
		}
		if(fieldProperties.defaultValue !== undefined){
			field.defaultValue = fieldProperties.defaultValue;
		}
		Object.keys(extraProperties).forEach(function(key){
			field[key] = extraProperties[key];
		});
		field = setDependenciesInField(dependencies, field);
		field = metaDataFactory.fieldFactory(field);
		prepareValueFunction(field);
		return field;
	}

	function transformFieldValuesInObject(activeItemJson, widget){
		var valuesObject = widget && widget.isValid();
		if(valuesObject){
			var json = JSON.parse(activeItemJson);
			valuesObject = {};
			widget.fields.forEach(function(fieldInForm){
				var fieldName = fieldInForm.name;
				var hasInOldJson = hasPropertyInJson(json, fieldName);
				if(fieldInForm.isVisible){
					var fieldValue = fieldInForm.value();
					var defaultValue = fieldInForm.defaultValue;
					var save = (defaultValue !== undefined && fieldValue != defaultValue) || !!fieldValue;
					if(hasInOldJson || save){
						valuesObject[fieldName] = fieldValue;
					}
				}
			});
		}
		return valuesObject;
	}

	function getFieldProperties(key, docTemplate, jsonItem){
		var value = docTemplate.properties[key];
		var type = typeof jsonItem[key];
		var properties = {
			name: key,
			isRequired: ~docTemplate.required.indexOf(key),
			type: type,
			description: "",
			defaultValue: false
		};
		if(value){
			var valueType = value.type;
			if(Util.isArray(valueType)){
				var index = valueType.indexOf(type) || 0;
				valueType = valueType[index];
			}
			properties.type = valueType;
			properties.description = value.description;
			properties.defaultValue = value.hasOwnProperty("default") ? value.default : undefined;
		}
		return properties;
	}

	function getFields(fieldsNames, docTemplate, jsonItem, dependencies){
		var fields = [];
		fieldsNames.forEach(function(value){
			var fieldProperties = getFieldProperties(value, docTemplate, jsonItem);
			fields.push(createField(fieldProperties, dependencies));
		});
		fields.push(createFieldToShowTemplate(jsonItem.template));
		return fields;
	}

	function getDefaultWidget(fields){
		var widget = DEFAULT_WIDGET;
		widget.fields = fields;
		widget.id = (+new Date());
		return widget;
	}

	function getDefaultContainer(){
		var container = DEFAULT_CONTAINER;
		container.id = (+new Date());
		return container;
	}

	this.getFormWidgetAndContainer = function(fieldsNames, docTemplate, jsonItem, dependencies){
		var fields = getFields(fieldsNames, docTemplate, jsonItem, dependencies);
		var widget = getDefaultWidget(fields);
		var container = getDefaultContainer();
		container.widgets[0] = widget;
		container = metaDataFactory.containerFactory(container);
		widget = metaDataFactory.widgetFactory(widget, container);
		return {"widget": widget, "container": container};
	};

	function getFormDocTemplate(template, inOptionsFolder){
		var formType = inOptionsFolder ? "options/" : "";
		formType += template + ".json";
		var formItem = templateManager.getDocsUrl(formType);
		formType = formType.slice(0, formType.indexOf("/") + 1);
		return DevToolsService.requestMetadataDeveloper(formItem).then(function(data){
			return data.data;
		}).catch(function(data){
			var defaultTemplate = DEFAULT_TEMPLATES.filter(function(template){
				var templateType = template.split("/")[0];
				return formType.indexOf(templateType) >= 0;
			})[0];

			if(!inOptionsFolder && formItem.indexOf(defaultTemplate) < 0 && data.status == "404"){
				return getFormDocTemplate(defaultTemplate);
			} else {
				return false;
			}
		});
	}
	this.getFormDocTemplate = getFormDocTemplate;

	this.getFormDependencies = function(docDependencies, options){
		var dependencies = {};
		if(docDependencies && options){
			var dependsOn = {};
			var preparedDep = {};
			Object.keys(docDependencies).forEach(function(realDep){
				if(options[realDep]){
					var dependencieValues = options[realDep].dependencies;
					docDependencies[realDep].forEach(function(value){
						if(dependencieValues[value] !== undefined){
							dependsOn[realDep] = dependsOn[realDep] || {};
							dependsOn[realDep][value] = dependencieValues[value];
							preparedDep[value] = preparedDep[value] ? preparedDep[value].concat(realDep) : [realDep];
						}
					});
				}
			});
			dependencies = {"dependsOn": dependsOn, "dependenciesTree": preparedDep};
		}
		return dependencies;
	};

	this.getActiveFormState = function(activeItemJson, formWidget){
		var activeText = activeItemJson.text;
		var state = { valid: false };
		if(activeText){
			try {
				var objectField = transformFieldValuesInObject(activeText, formWidget);
				if(!objectField){
					var objectText = JSON.parse(activeText).label || JSON.parse(activeText).name;
					ScreenService.notificationMessage('Fill required fields in ' + objectText, 'error', 5000);
				} else {
					state.valid = true;
					state.value = JSON.stringify(objectField);
				}
			} catch(e){
				state.valid = false;
				ScreenService.notificationMessage(e, 'error', 5000);
			}
		}
		return state;
	};
}
