
// FILE: bower_components/zhuaclfrontend/assets/dist/main.js
// FILE: assets/js/util/ZhuAclConfig.js
/* global Configuration */
function ZhuAclConfig() {
	var self = this;
	this.RULE_ALLOW = 'A';
	this.RULE_READONLY = 'R';
	this.RULE_INVISIBLE = 'I';

	this.exportRulesModal = 'exportRulesModal';
	this.menuElementsListWidgetName = 'elementsListWidget';
	this.containerElementsListWidgetName = 'containerElementsModal';
	this.containerElementsTreeWidgetName = 'containerElementsTree';

	this.containerType = 'container';

	this.getDefaultDataSourceFilter = function(groupIds, projectId, organizationId){
		var dataFilter = [];
		var groupFilter = {
			"name": "GROUP_ID",
			"operator": "=",
			"value": groupIds
		};
		var projectFilter = {
			"name": "PROJECT_ID",
			"operator": "=",
			"value": projectId
		};

		if(organizationId !== undefined){
			var organizationFilter = {
				"name": "ORGANIZATION_ID",
				"operator": "=",
				"value": organizationId
			};
			dataFilter.push(organizationFilter);
		}

		dataFilter.push(groupFilter);
		dataFilter.push(projectFilter);
		return dataFilter;
	};

	this.getAclMap = function(type){
		var aclMap = {
			"container": {
				"aclTypeCod": "C",
				"childs": ["widgets"]
			},
			"widgets": {
				"aclTypeCod": "W",
				"childs": ["widgets", "fields", "actions"]
			},
			"fields": {
				"aclTypeCod": "F",
				"childs": []
			},
			"actions": {
				"aclTypeCod": "A",
				"childs": []
			}
		};
		return aclMap[type] || {};
	};

	this.getRuleType = function(isVisible, isEditable){
		var ruleType;
		if(!isVisible){
			ruleType = self.RULE_INVISIBLE;
		} else if(!isEditable){
			ruleType = self.RULE_READONLY;
		} else {
			ruleType = self.RULE_ALLOW;
		}
		return ruleType;
	};

	this.getRuleConfiguration = function(ruleType){
		var ruleConfig = {
			isVisible: ruleType != self.RULE_INVISIBLE,
			isEditable: ruleType == self.RULE_ALLOW
		};
		return ruleConfig;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ZhuAclConfig', ZhuAclConfig);
});

// FILE: assets/js/util/ZhuAclMessages.js
var ZhuAclMessages = {
	CANCEL: 'Cancelar',
	YES: 'Sim',
	NO: 'Não',
	VISIBLE: 'visíveis',
	EDITABLE: 'editáveis',
	INVISIBLE: 'invisíveis'
};
(function(){
	var mappedMessages = [
		"ACCESS_DENIED",
		"RULE_NOT_SAVED",
		"SUC_RULE_INSERTED",
		"ERR_RULE_INSERTED",
		"SUC_RULES_EXPORTED",
		"ERR_RULES_EXPORTED",
		"CANNOT_LOAD_RULES",
		"CONFIRMATION_MESSAGE",
		"CONFIRM_CHILDS_QUESTION",
		"CONFIRM_EXPORT",
		"NSHOW_WITHOUT_ID",
		"NSHOW_WITHOUT_CONTAINER_CONFIG",
		"NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG"
	];
	mappedMessages.forEach(function addMappedMessages(code){
		ZhuAclMessages[code] = 'zhuacl#' + code;
	});
})();

// FILE: assets/js/services/ZhuAccessButtonService.js
/* global _Notification, Util, ZeedhiServices */
function ZhuAccessButtonService(){
	this.insertButtonInProject = function(project){
		var notification = {
			"label": "Access Control",
			"name": "accessControl",
			"icon": "block",
			"isVisible": false,
			"tooltip": "Access Control",
			"isAclNotification": true,
			"events": [
				{
					"name": "NotificationEvent",
					"code": "ZhuAclController.openAclModalWidget();"
				}
			]
		};
		try {
			notification = Util.extend(new _Notification(), notification);
			project.notifications = project.notifications || [];
			project.notifications.push(notification);
		} catch (e){
			console.error('O Controle de acesso à elementos só funciona em versões acima da 1.40.x do Zeedhi Framework');
		}
	};

	this.disableButtonInProject = function(project){
		if(project.notifications){
			project.notifications = project.notifications.filter(function(notification){
				return !notification.isAclNotification;
			});
		}
	};

	this.enableButtonInProject = function(project){
		if(project.notifications){
			project.notifications.forEach(function(notification){
				if(notification.isAclNotification){
					notification.isVisible = true;
				}
			});
		}
	};
}

ZeedhiServices.service("ZhuAccessButtonService", function(ApplicationContext){
	var instance = new ZhuAccessButtonService();
	ApplicationContext.registerInstance("ZhuAccessButtonService", instance);
	return instance;
});


// FILE: assets/js/services/ZhuAclService.js
/* global Util, Configuration, ZhuAclMessages*/
function ZhuAclService(requestDataSourceEngine, restEngine, ZhuAclConfig){

	this.getZhuAclRules = function(projectId, groupIds){
		return requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', ZhuAclConfig.getDefaultDataSourceFilter(groupIds, projectId), 1, undefined, undefined, undefined, 120000).then(function(response){
			var dataset = response.dataset || {};
			var rules = dataset.zhu_access_control_rule_login;
			if(Util.isArray(rules)){
				var objToReturn = {
					aclRules: rules,
					defaultRule: dataset.zhu_access_control_default_rule.defaultRule,
				};
				return objToReturn;
			} else {
				throw {code: ZhuAclMessages.CANNOT_LOAD_RULES};
			}
		});
	};

	this.getWidgetMetadata = function(widgetName){
		var widgetUrl = 'zhuacl#json/'+ widgetName +'.json';
		return restEngine.requestMetaData(widgetUrl).then(function(metadata){
			return metadata.data;
		});
	};

	this.exportRules = function(profileFrom, profilesTo){
		var rows = [{
				changed: true,
				profileFrom: profileFrom,
				profilesTo: profilesTo
		}];
		return requestDataSourceEngine.setDataSource('zhuacl#/acl_zhu_acl_element_exportRules', rows);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclService', ZhuAclService);
});

// FILE: assets/js/controllers/ZhuAclController.js
/* global openWindow, Util, Configuration, ZhuAclMessages */
function ZhuAclController(ScreenService, templateManager, metaDataFactory, ZHPromise, ZhuAclService, ZhuAccessButtonService, ZhuAclConfig, MessageService, ApplicationContext, eventAggregator){
	var self = this;
	var user;
	var validProperties = ['widgets', 'fields', 'actions'];
	var loadedContainers = {};

	function User(userId, projectId, organizationId, defaultRule){
		var currentGroupId;
		var userDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;

		this.setCurrentGroupId = function(id){
			currentGroupId = id;
		};

		this.getDefaultRule = function(){
			return userDefaultRule;
		};

		this.getCurrentGroupId = function(){
			return currentGroupId;
		};

		this.getProjectId = function(){
			return projectId;
		};

		this.getUserId = function(){
			return userId;
		};

		this.getOrganizationId = function(){
			return organizationId;
		};
	}

	this.getDefaultRule = function(){
		return user.getDefaultRule();
	};

	this.getCurrentGroupId = function(){
		return user.getCurrentGroupId();
	};

	this.getProjectId = function(){
		return user.getProjectId();
	};

	var prepareLabelToName = function(label){
		var name = label.toLowerCase();
		name = name.replace(/\ /g, "_");
		return name.replace(/[^\w\s]/gi, '');
	};

	var translateLabel = function(label){
		return ScreenService.i18n(label);
	};

	this.getRulesFromDatasource = function(rulesData){
		var userId = user.getUserId();
		var rules = {};
		rulesData.forEach(function(row){
			var obj = {
				ruleRow: row,
				userGroups: {}
			};
			obj.userGroups[userId] = {
				rule: row.RULE
			};
			rules[row.ELEMENT_UNIQUE_ID] = obj;
		});
		return rules;
	};

	this.getElementRule = function(rules, itemId){
		var itemRule = {};
		if(rules[itemId]){
			itemRule = rules[itemId].ruleRow;
		}
		return itemRule;
	};

	this.getPlannedItemToInsertInDataSet = function(rules, item, itemType, parentId){
		var uniqueId = item.id;
		var itemRule = self.getElementRule(rules, uniqueId);
		var itemToInsert = {
			"UNIQUE_ID": uniqueId,
			"NAME": item.name || prepareLabelToName(item.label),
			"LABEL": translateLabel(item.label || item.name),
			"TYPE": itemType,
			"RULE": itemRule.RULE || this.getDefaultRule(),
			"RULE_ID": itemRule.ID,
			"PARENT_ID": parentId,
			"USER_ID": user.getUserId(),
			"PROJECT_ID": user.getProjectId(),
			"ORGANIZATION_ID": user.getOrganizationId()
		};

		itemToInsert.isVisible = item.aclIsVisible;
		itemToInsert.isEditable = !item.aclReadOnly && item.aclIsVisible;

		return itemToInsert;
	};

	var insertContainerElementsListWidget = function(container){
		return getFactoredWidget(container, ZhuAclConfig.containerElementsListWidgetName, true).then(function(factoredWidget){
			container.actions = container.actions || [];
			return factoredWidget;
		});
	};

	this.openAclModalWidget = function(){
		insertContainerElementsListWidget(templateManager.container).then(function(widget){
			ScreenService.openPopup(widget);
		});
	};

	var redirectToMainWindow = function(){
		var dashboardMenu = templateManager.project.getMenu('dashboard');
		var openLinkByMenu = ApplicationContext.LibUtilities && ApplicationContext.LibUtilities.openLinkByMenu;
		if(dashboardMenu && openLinkByMenu){
			openLinkByMenu(dashboardMenu);
		} else {
			openWindow(templateManager.project.mainWindow);
		}
	};

	var verifyContainerRule = function(rule){
		var containerIsRestrict = rule === ZhuAclConfig.RULE_INVISIBLE;
		if(containerIsRestrict){
			var actions = [{
				label: ScreenService.i18n("Ok"),
				code: redirectToMainWindow
			}];
			var dialogConfig = {
				type: 'ALERT',
				icon: 'exclamation',
				actions: actions
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.ACCESS_DENIED, {}, dialogConfig);
		}
		return !containerIsRestrict;
	};

	var mergeUndefinedRules = function(groupIds, itemRule, parentRules){
		parentRules = parentRules || {};

		var definedRule = itemRule.rule;
		var groupRules = parentRules.groupRules || {};

		if(!definedRule){
			definedRule = parentRules.rule || templateManager.project.aclDefaultRule;
		} else {
			var undefinedGroups = itemRule.undefinedGroups || [];
			groupIds.forEach(function(groupId){
				var parentRule = groupRules[groupId] || templateManager.project.aclDefaultRule;
				var isUndefinedGroup = undefinedGroups.indexOf(groupId) !== -1;
				var groupRule = isUndefinedGroup ? parentRule : definedRule;
				groupRules[groupId] = groupRule;

				if(definedRule !== ZhuAclConfig.RULE_ALLOW && groupRule !== ZhuAclConfig.RULE_INVISIBLE){
					definedRule = groupRule;
				}
			});
			itemRule.rule = definedRule;
		}

		return {rule: definedRule, groupRules: groupRules};
	};

	var updateContainerAcl = function(rules, groupIds, userId){
		var applyRule = function(item, parentRules){
			var id = item.id;
			var userRule = Util.getNestedProperty(rules, [id, 'userGroups', userId]) || {};
			var mergedItem = mergeUndefinedRules(groupIds, userRule, parentRules);
			var itemRule = mergedItem.rule;

			item.aclIsVisible = itemRule != ZhuAclConfig.RULE_INVISIBLE;
			item.aclReadOnly = itemRule != ZhuAclConfig.RULE_ALLOW;

			return mergedItem;
		};

		var updateItemAcl = function(item, type, parentRules){
			var mergedItem = applyRule(item, parentRules);

			ZhuAclConfig.getAclMap(type).childs.forEach(function(type){
				if(Util.isArray(item[type])){
					item[type].forEach(function(child){
						updateItemAcl(child, type, mergedItem);
					});
				}
			});
		};

		var changeNotificationVisibility = function(isVisible){
			templateManager.project.notifications.some(function(notification){
				var isAclNotification = notification.name === "accessControl";
				if(isAclNotification){
					notification.isVisible = isVisible;
				}
				return isAclNotification;
			});
		};

		var container = templateManager.container;
		var aclEnable = container.aclEnable;
		aclEnable = aclEnable !== undefined ? aclEnable : true;
		changeNotificationVisibility(aclEnable);

		if(aclEnable){
			var containerRule = applyRule(container);
			if(verifyContainerRule(containerRule.rule)){
				updateItemAcl(container, ZhuAclConfig.containerType);
			}
			loadedContainers[container.id] = true;
		}
	};

	var convertObjectRulesToArray = function(userId, objRules){
		var arrayRules = [];
		Object.keys(objRules).forEach(function(ruleId){
			var objRule = objRules[ruleId];
			var userRule = objRule.userGroups[userId];
			if(userRule){
				var rule = {
					"UNIQUE_ID": ruleId,
					"NAME": objRule.itemName,
					"RULE": userRule.rule
				};
				arrayRules.push(rule);
			}
		});
		return arrayRules;
	};

	var convertArrayRulesToObject = function(userId, arrayRules){
		var objRules = {};
		arrayRules.forEach(function(rule){
			var profileObjects = {};
			profileObjects[userId] = {
				'rule': rule.RULE,
				'undefinedGroups': rule.GROUP_ID || []
			};
			objRules[rule.UNIQUE_ID] = {
				'itemName': rule.NAME,
				'userGroups': profileObjects
			};
		});
		return objRules;
	};

	var subscribeContainerEvent = function(rules, groupIds, userId){
		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function applyACL(){
			var containerId = templateManager.container.id;
			if(!loadedContainers[containerId]){
				updateContainerAcl(rules, groupIds, userId);
			}
		});

	};

	var updateProjectAcl = function(loginRules, defaultRule, groupIds){
		var rules = {};
		var userId = user.getUserId();
		if(Util.isObject(loginRules)){
			rules = loginRules;
			loginRules = convertObjectRulesToArray(userId, rules);
		} else if(Util.isArray(loginRules)){
			rules = convertArrayRulesToObject(userId, loginRules);
		}

		templateManager.project.aclDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;
		templateManager.userId = userId;

		templateManager.aclRules = rules;
		updateContainerAcl(rules, groupIds, userId);
		subscribeContainerEvent(rules, groupIds, userId);
	};

	var createUser = function createUser(userId, projectId, organizationId, defaultRule){
		user = new User(userId, projectId, organizationId, defaultRule);
	};

	this.setUserRules = function(groupIds, projectId, userId, organizationId, options){
		loadedContainers = {};
		options = options || {};

		if(options.canConfigureAcl){
			ZhuAccessButtonService.enableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		} else {
			ZhuAccessButtonService.disableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		}

		if(options.rules){
			createUser(userId, projectId, organizationId, options.defaultRule);
			updateProjectAcl(options.rules, options.defaultRule, groupIds);
		} else {
			ZhuAclService.getZhuAclRules(projectId, groupIds).then(function(response){
				createUser(userId, projectId, organizationId, response.defaultRule);
				updateProjectAcl(response.aclRules, response.defaultRule, groupIds);
			}).catch(function(err){
				MessageService.showMessageByCode(err.code);
			});
		}
	};

	this.setElementsListFilter = function(container, groupId, widgetName){
		widgetName = widgetName || ZhuAclConfig.menuElementsListWidgetName;
		var elementsWidget = container.getWidget(widgetName);
		elementsWidget.dataSourceFilter = ZhuAclConfig.getDefaultDataSourceFilter(groupId, user.getProjectId(), user.getOrganizationId());
		user.setCurrentGroupId(groupId);
	};

	var getFactoredWidget = function(container, widgetName, pushInContainer){
		var widget = container.getWidget(widgetName);
		if(widget){
			return ZHPromise.when(widget);
		} else {
			return ZhuAclService.getWidgetMetadata(widgetName).then(function(metadata){
				var factoredWidget = metaDataFactory.widgetFactory(metadata, container);
				if(pushInContainer){
					container.widgets.push(factoredWidget);
				}
				return factoredWidget;
			});
		}
	};
	this.getFactoredWidget = getFactoredWidget;

	this.insertElementsListWidget = function(container, parentWidget, extraElementsPath){
		getFactoredWidget(container, ZhuAclConfig.menuElementsListWidgetName).then(function(factoredWidget) {
			if(parentWidget && !factoredWidget.parent){
				factoredWidget.parent = parentWidget;
				parentWidget.widgets = parentWidget.widgets || [];
				parentWidget.widgets.push(factoredWidget);
				factoredWidget.extraElementsPath = extraElementsPath;
			}
		});
	};

}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclController', ZhuAclController);
});


// FILE: assets/js/controllers/ZhuAclCRUDController.js
/* global Configuration, ZhuAclMessages */
function ZhuAclCRUDController(ScreenService, ZHPromise, ZhuAclConfig, ZhuAclController, MessageService){

	function defineQuestionToUse(rule){
		var question = ZhuAclMessages.VISIBLE;
		if(rule == ZhuAclConfig.RULE_INVISIBLE){
			question = ZhuAclMessages.INVISIBLE;
		} else if(rule == ZhuAclConfig.RULE_ALLOW){
			question = ZhuAclMessages.EDITABLE;
		}
		return {newVisibility: ScreenService.i18n(question)};
	}

	function deleteChildrensRules(elementsWidget, parentRow, groupId, parentRule){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					if(row.extraElement){
						row.RULE = parentRule;
					} else {
						row.__isDeleted = true;
						row.RULE_SAVE_CONTAINER = parentRule;
					}
					row.changed = true;
					row.GROUP_ID = groupId;
					deleteChildrensRules(elementsWidget, row, groupId, parentRule);
				}
			});
		}
	}

	function confirmChildsVisibility(elementsWidget, ruleType){
		var returnPromise = ZHPromise.when();
		if(elementsWidget.currentRow._childCount){
			var currentRow = elementsWidget.currentRow;
			var defer = ZHPromise.defer();
			var actions = [
				{
					label: ScreenService.i18n(ZhuAclMessages.YES),
					code: function(){
						currentRow.changed = true;
						deleteChildrensRules(elementsWidget, currentRow, currentRow.GROUP_ID, currentRow.RULE);
						defer.resolve();
					}
				},
				{
					label: ScreenService.i18n(ZhuAclMessages.NO),
					code: function(){
						var messageConfig = {
							type: 'confirmation',
							icon: 'question'
						};
						MessageService.showMessageByCode(ZhuAclMessages.CONFIRMATION_MESSAGE, {}, messageConfig);
						defer.reject();
					}
				}
			];
			var dialogConfig = {
				type: 'confirmation',
				icon: 'question',
				actions: actions
			};

			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_CHILDS_QUESTION, defineQuestionToUse(ruleType), dialogConfig);
			returnPromise = defer.promise;
		}

		return returnPromise;
	}

	function reloadChildrensRules(elementsWidget, parentRow, updatedRules, deletedRules){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					var rowId = row.UNIQUE_ID;
					row.isVisible = parentRow.isVisible;
					row.isEditable = parentRow.isEditable;

					if(deletedRules.indexOf(rowId) != -1){
						delete row.RULE_ID;
					}
					if(updatedRules[rowId] && updatedRules[rowId].containerId){
						row.CONTAINER_RULE_ID = updatedRules[rowId].containerId;
					}

					reloadChildrensRules(elementsWidget, row, updatedRules, deletedRules);
				}
				delete row.__isDeleted;
				delete row.changed;
			});
		}
	}

	function fixZeedhiBehavior(widget){
		widget.dataSource.data.forEach(function(data){
			data.isEditable = (data._childCount && data.isVisible) || data.isEditable;
		});
	}

	function isFatherVisible(parentRow, widget){
		return widget.dataSource.data.some(function(row){
			return (row.PARENT_ID == parentRow.UNIQUE_ID) && (row.isVisible || row.isEditable);
		});
	}

	function reloadParentsRules(row, widget){
		var parentRow = row._parent;
		if(parentRow){
			if(row.isVisible){
				parentRow.isEditable = parentRow.isEditable || row.isEditable || row.isVisible;
				parentRow.isVisible = true;
			} else {
				var updateFather = isFatherVisible(parentRow, widget);
				parentRow.isEditable = updateFather;
				parentRow.isVisible = updateFather;
			}
			reloadParentsRules(row._parent, widget);
		}
	}

	function clearRowState(row){
		row.changed = false;
		row.__isDeleted = false;
	}

	function reloadDataSourceAfterSave(elementsWidget, newRules){
		var currentRow = elementsWidget.currentRow;
		var currentRowId = currentRow.UNIQUE_ID;
		var updatedRule = newRules.updated[currentRowId];
		var deletedRules = newRules.deleted || [];
		if(updatedRule){
			if(Util.isObject(updatedRule)){
				currentRow.RULE_ID = updatedRule.menuId;
				currentRow.CONTAINER_RULE_ID = updatedRule.containerId;
			} else {
				currentRow.RULE_ID = updatedRule;
			}
			reloadChildrensRules(elementsWidget, currentRow, newRules.updated, deletedRules);
			if(!currentRow.extraElement){
				reloadParentsRules(currentRow, elementsWidget);
				fixZeedhiBehavior(elementsWidget);
			}
		}

		var updatedRules = Object.keys(newRules.updated);
		if(updatedRules.length){
			//updated all extraElements
			elementsWidget.dataSource.data.forEach(function(row){
				var ruleId = newRules.updated[row.UNIQUE_ID];
				if(ruleId && row.extraElement){
					row.RULE_ID = ruleId;
					row.RULE = currentRow.RULE;
					row.isVisible = currentRow.isVisible;
					row.isEditable = currentRow.isEditable;
				}
				clearRowState(row);
			});
		} else {
			elementsWidget.dataSource.data.forEach(clearRowState);
		}
	}

	function cancelSave(widget){
		widget.reload();
	}

	function changeElementRule(currentRow, isVisible, isEditable){
		currentRow.RULE = ZhuAclConfig.getRuleType(isVisible, isEditable);
		currentRow.isVisible = isVisible;
		currentRow.isEditable = isEditable;
		currentRow.GROUP_ID = currentRow.GROUP_ID || ZhuAclController.getCurrentGroupId();
	}

	function clearInlineChangedStatus(row){
		delete row.changed_isEditable;
		delete row.changed_isVisible;
	}

	function saveRule(widget, isEditable, isVisible){
		var currentRow = widget.currentRow;
		var ruleType = ZhuAclConfig.getRuleType(isVisible, isEditable);

		clearInlineChangedStatus(currentRow);
		changeElementRule(currentRow, isVisible, isEditable);

		confirmChildsVisibility(widget, ruleType).then(function(){
			widget.saveRow().then(function(response){
				reloadDataSourceAfterSave(widget, response.dataset.aclZhuAccessControlRule);
				MessageService.successNotificationByCode(ZhuAclMessages.SUC_RULE_INSERTED);
			}).catch(function(){
				cancelSave(widget);
				MessageService.notificationMessageByCode(ZhuAclMessages.ERR_RULE_INSERTED, {}, {type: "alert"});
			});
		}).catch(function(){
			cancelSave(widget);
			MessageService.notificationMessageByCode(ZhuAclMessages.RULE_NOT_SAVED, {}, {type: "alert"});
		});
	}

	this.changeIsEditable = function changeIsEditable(widget, row){
		var isEditable = row.isEditable;
		var isVisible = isEditable || row.isVisible;
		saveRule(widget, isEditable, isVisible);
	};

	this.changeIsVisible = function changeIsVisible(widget, row){
		var isVisible = row.isVisible;
		var isEditable = isVisible && row.isEditable;
		saveRule(widget, isEditable, isVisible);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclCRUDController', ZhuAclCRUDController);
});


// FILE: assets/js/controllers/ZhuAclExportRulesController.js
function ZhuAclExportRulesController(templateManager, MessageService, ScreenService, ZhuAclConfig, ZhuAclController, ZhuAclService){
	var self = this;
	var requestToExportRules = function(profileFrom, profilesToExport) {
		ZhuAclService.exportRules(profileFrom, profilesToExport)
			.then(self.closeExportRulesPopup)
			.catch(function(response){
				var errorCode = (response && (response.errorCode || response.error)) || ZhuAclMessages.ERR_RULES_EXPORTED;
				MessageService.errorNotificationByCode(errorCode);
			});
	};

	this.exportRules = function() {
		var exportRulesWidget = templateManager.container.getWidget(ZhuAclConfig.exportRulesModal);
		if(exportRulesWidget.isValid()){
			var currentRow = exportRulesWidget.currentRow;
			var dialogConfig = {
				type: 'CONFIRMATION',
				actions: [
					{
						label: ScreenService.i18n(ZhuAclMessages.YES),
						code: function() {
							requestToExportRules(currentRow.profileFrom, currentRow.profilesToExport);
						}
					},
					{ label: ScreenService.i18n(ZhuAclMessages.NO) }
				]
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_EXPORT, {}, dialogConfig);
		}
	};

	this.profileActionEventExportRules = function(groupId) {
		var container = templateManager.container;
		return ZhuAclController.getFactoredWidget(container, ZhuAclConfig.exportRulesModal, true).then(function(exportRulesWidget){
			ScreenService.openPopup(exportRulesWidget);
			exportRulesWidget.isVisible = true;
			exportRulesWidget.currentRow = {
				profileFrom: groupId
			};
			exportRulesWidget.fields[0].dataSourceFilter = [{
				name: 'ID',
				operator: '<>',
				value: groupId
			}];
		});
	};

	this.closeExportRulesPopup = function() {
		ScreenService.closePopup();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclExportRulesController', ZhuAclExportRulesController);
});

// FILE: assets/js/controllers/ZhuContainerElementsController.js
/* global Util, Configuration, ZhuAclMessages */
function ZhuContainerElementsController(templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var self = this;
	var idErrorMessages = "";

	var getContainerElementsTreeWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsTreeWidgetName);
	};

	var getContainerElementsListWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = "";
	};

	var showIdErrorsMessage = function(){
		if(idErrorMessages){
			var messageConfig = {
				type: 'alert',
				icon: 'exclamation'
			};
			MessageService.showMessageByCode(ZhuAclMessages.NSHOW_WITHOUT_ID, {elementsIds: idErrorMessages}, messageConfig);
		}
	};

	var getPlannedContainer = function(rules, item, itemType, parentId){
		var plannedItems = [];
		itemType = itemType || ZhuAclConfig.containerType;
		if(item && !item.isZhuAclElement){
			var uniqueId = item.id;
			if(uniqueId){
				var aclType = ZhuAclConfig.getAclMap(itemType);
				var itemToInsert = ZhuAclController.getPlannedItemToInsertInDataSet(rules, item, aclType.aclTypeCod, parentId);
				plannedItems.push(itemToInsert);
				aclType.childs.forEach(function(type){
					if(Util.isArray(item[type])){
						item[type].forEach(function(child){
							plannedItems = plannedItems.concat(getPlannedContainer(rules, child, type, uniqueId));
						});
					}
				});
			} else {
				var nameToShow = item.label || item.name;
				idErrorMessages += "<br> " + nameToShow + ";";
			}
		}
		return plannedItems;
	};

	this.reloadContainerElementsModal = function(){
		getContainerElementsListWidget().reload();
	};

	this.reloadContainerElementsTree = function(){
		getContainerElementsTreeWidget().reload();
	};

	this.selectAccessOnChange = function(selectedRow){
		if(selectedRow && selectedRow.SELECT_ACCESS){
			ZhuAclController.setElementsListFilter(templateManager.container, selectedRow.ID, ZhuAclConfig.containerElementsTreeWidgetName);
		}
		self.reloadContainerElementsTree();
	};

	var copyItem = function(itemToCopy, itemType, parentId){
		var item;
		if(itemToCopy && !itemToCopy.isZhuAclElement && !itemToCopy.ignoreAclElementsConfig){
			var uniqueId = itemToCopy.id;
			item = {
				id: uniqueId,
				name: itemToCopy.name,
				label: itemToCopy.label,
			};
			itemType = itemType || ZhuAclConfig.containerType;
			var aclType = ZhuAclConfig.getAclMap(itemType);
			aclType.childs.forEach(function(type){
				if(Util.isArray(itemToCopy[type])){
					item[type] = itemToCopy[type].map(function(child){
						return copyItem(child, type, uniqueId);
					}).filter(function(copiedChild){
						return copiedChild;
					});
				}
			});
		}
		return item;
	};

	this.containerElementsTreeAfterLoadDataSource = function(){
		clearIdErrorsMessage();
		var widget = getContainerElementsTreeWidget();
		var elementsListWidget = getContainerElementsListWidget();
		var elementsListRow = elementsListWidget && elementsListWidget.currentRow;
		var rulesData = Util.clone(widget.dataSource.data);
		widget.dataSource.data = [];

		if(elementsListRow && elementsListRow.SELECT_ACCESS){
			var rules = ZhuAclController.getRulesFromDatasource(rulesData);
			var container = copyItem(templateManager.container);
			if(container){
				aclEngine.processAcl(container, rules, templateManager.userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule());
				widget.dataSource.data = getPlannedContainer(rules, container);
				showIdErrorsMessage();
			}
		}
	};

	this.isEditableOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsEditable(widget, currentRow);
	};

	this.isVisibleOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsVisible(widget, currentRow);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuContainerElementsController', ZhuContainerElementsController);
});

// FILE: assets/js/controllers/ZhuMenuElementsController.js
function ZhuMenuElementsController(ScreenService, templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService, restEngine, ZHPromise){
	var MENU = "M";
	var GROUP_MENU = "G";
	var IGNORED_MENUS = ['zhHistory'];
	var idErrorMessages = {"errorMenus": "", "errorContainers": ""};

	var getWidgetElements = function(){
		return templateManager.container.getWidget(ZhuAclConfig.menuElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = {"errorMenus": "", "errorContainers": ""};
	};

	var showIdErrorsMessage = function(){
		var messageCode;
		var errorMenus = idErrorMessages.errorMenus;
		var errorContainers = idErrorMessages.errorContainers;
		var messageParameters = {
			elementsIds: errorMenus,
			elementsWithoutContainer: errorContainers
		};

		if(errorMenus && errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG;
		} else if(errorMenus){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID;
		} else if(errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_CONTAINER_CONFIG;
		}

		if(messageCode){
			MessageService.showMessageByCode(messageCode, messageParameters, { type: 'alert' });
		}
	};

	var addErrorMenuId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorMenus += "<br> " + nameToShow + ";";
	};

	var addErrorContainerId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorContainers += "<br> " + nameToShow + ";";
	};

	this.elementsActionEventEdit = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			currentRow.isEditable = !currentRow.isEditable;
			ZhuAclCRUDController.changeIsEditable(elementsWidget, currentRow);
		}
	};

	this.elementsActionEventView = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			currentRow.isVisible = !currentRow.isVisible;
			currentRow.isEditable = currentRow.isVisible;
			ZhuAclCRUDController.changeIsVisible(elementsWidget, currentRow);
		}
	};

	this.elementsActionEventBack = function(){
		var elementsWidgetParent = getWidgetElements().parent;
		return ScreenService.closeSwipe().then(function(){
			elementsWidgetParent.list();
			elementsWidgetParent.activate();
		});
	};

	var addContainerProperties = function(item, menuItem, rules){
		var containerId = menuItem.containerId;
		var containerLabel = menuItem.containerLabel || menuItem.containerName;
		if(containerId && containerLabel){
			var containerRule = ZhuAclController.getElementRule(rules, containerId);
			item.CONTAINER_ID = containerId;
			item.CONTAINER_NAME = menuItem.containerName || ZhuAclController.prepareLabelToName(containerLabel);
			item.CONTAINER_LABEL = containerLabel;
			item.CONTAINER_RULE_ID = containerRule.RULE_ID;
		} else if(menuItem.windowName && !menuItem.buildedMenu){
			addErrorContainerId(menuItem);
		}
	};

	var planningMenuItem = function(menuItem, parentMenuItem, rules){
		var item;
		var uniqueId = menuItem.id;
		if(uniqueId){
			var parentId = parentMenuItem && parentMenuItem.id;
			var itemType = !parentId ? GROUP_MENU : MENU;
			item = ZhuAclController.getPlannedItemToInsertInDataSet(rules, menuItem, itemType, parentId);
			item.buildedMenu = menuItem.buildedMenu;
			addContainerProperties(item, menuItem, rules);
		} else {
			addErrorMenuId(menuItem);
		}

		return item;
	};

	var addMenusInDataSource = function(widget, rules, menus, parentMenu){
		menus.forEach(function(menu, index){
			var item = planningMenuItem(menu, parentMenu, rules);
			if(item){
				widget.dataSource.data.push(item);
				if (menu.menus && menu.menus.length > 0){
					addMenusInDataSource(widget, rules, menu.menus, menu);
				}
			}
		});
	};

	var getProjectMenu = function(){
		var projectMenu = Util.clone(templateManager.project.groupMenu);
		return projectMenu.filter(function(menu){
			return IGNORED_MENUS.indexOf(menu.name) == -1;
		});
	};

	var extraRulesPromise;

	var loadExtraRules = function(extraElementsPath){
		if(!extraRulesPromise){
			if(extraElementsPath){
				extraRulesPromise = restEngine.requestMetaData(Util.getRightPath(extraElementsPath), {});
			} else {
				extraRulesPromise = ZHPromise.when();
			}
		}
	};

	var addExtraElements = function(widget, projectRules, defaultRule){
		var getRule = function(id){
			return (projectRules[id] && projectRules[id].ruleRow) || {};
		};

		var getVisibility = function(rule){
			return (rule || defaultRule) !== 'I';
		};

		extraRulesPromise.then(function(response){
			if(response && response.data){
				response.data.forEach(function(row){
					var rule = getRule(row.UNIQUE_ID);
					row.RULE = rule.RULE;
					row.RULE_ID = rule.ID;
					row.isVisible = getVisibility(rule.RULE);
					row.extraElement = true;
				});
				widget.dataSource.data = widget.dataSource.data.concat(response.data);
			}
		});
	};

	this.loadPlanningTree = function(){
		var elementsWidget = getWidgetElements();
		var userId = templateManager.userId;
		var defautRule = templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule();
		var rules = ZhuAclController.getRulesFromDatasource(elementsWidget.dataSource.data);
		var newMenu = getProjectMenu();

		clearIdErrorsMessage();
		loadExtraRules(elementsWidget.extraElementsPath);
		aclEngine.processMenuAcl(newMenu, rules, templateManager.containers, userId, defautRule);
		elementsWidget.dataSource.data = [];
		addMenusInDataSource(elementsWidget, rules, newMenu);
		addExtraElements(elementsWidget, rules, defautRule);
		showIdErrorsMessage();
	};

	this.reloadElementsWidget = function(){
		getWidgetElements().reload();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuMenuElementsController', ZhuMenuElementsController);
});

// FILE: assets/js/adapter/config.js
ZeedhiApp.run(function(templateManager, ZhuAccessButtonService) {
	templateManager.projectPromise.then(function(project){
		if(templateManager.project.aclEnable){
			ZhuAccessButtonService.insertButtonInProject(templateManager.project);
		}
	});
});


// FILE: bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js

// FILE: src/GridNavigationLibrary.js
function GridNavigationLibrary(ZhKeyboardService, templateManager, ScreenService) {

	var rowHeight = 35;

	function isTree(widget) {
		return widget.template == "/widget/tree/grid.html" || widget.template == "widget/tree/grid.html";
	}

	function _getFocusedElement() {
		var element = $(':focus');
		if (element.is('.zh-label-with-icon') ||
			element.parent().is('.zh-radio-label') ||
			element.parent().is('.zh-box-label') ||
			element.is('textarea') ||
			element.is('input')) {
			return element;
		} else {
			return undefined;
		}
	}

	function getActiveWidget(){
		var enabledWidget = true;
		var widget;
		if (!ScreenService.isSideMenuOpen() && Util.isUndefined(_getFocusedElement()) && !ScreenService.isAlertShown()) {
			widget = templateManager.container.getActiveWidget();
			enabledWidget = !widget.disableGridNavigation && !widget.inEditMode();
		}
		return enabledWidget && widget;
	}

	function filterVisibleRowsOnTree(widget) {
		return widget.plainTreeData.filter(function (row) {
			return Util.isUndefinedOrNull(row._parent) || row._parent._opened;
		});
	}

	function filterSearchVisibleRows(widget, data) {
		var fieldsArray = [];
		widget.fields.forEach(function (field) {
			if (field.isVisible && field.aclIsVisible) {
				fieldsArray.push(field.name);
			}
		});
		return data.filter(function (row) {
			var toReturn = false;
			fieldsArray.forEach(function (property) {
				if (row[property] && row[property].toString().toLowerCase().indexOf(widget.searchList.toString().toLowerCase()) != -1) {
					toReturn = true;
				}
			});
			return toReturn;
		});
	}

	function getDataSource(widget){
		var dataSource;
		if (isTree(widget) && !!widget.searchList) {
			dataSource = filterSearchVisibleRows(widget, widget.plainTreeData);
		} else if (isTree(widget) && !widget.searchList) {
			dataSource = filterVisibleRowsOnTree(widget);
		} else if (!isTree(widget) && !!widget.searchList) {
			dataSource = filterSearchVisibleRows(widget, widget.groupedData || widget.dataSource.data);
		} else if (!isTree(widget) && !widget.searchList) {
			dataSource = widget.groupedData || widget.dataSource.data;
		}
		return dataSource;
	}

	function getElementScope(element){
		var scope = element.scope();
		if (!scope) {
			element = element.parents('[zh-new-grid]');
			scope = element[0] ? element[0].__scope : null;
		}
		return scope;
	}

	templateManager.projectPromise.then(function() {
		var shortcutKeys = templateManager.project.shortcutKeys;
		var keys = {
			toogleRow : shortcutKeys.toogleRow || "space",
			openSwipe : shortcutKeys.openSwipe || "enter",
			openInlineEdit : shortcutKeys.openInlineEdit || "ctrl+space"
		};

		ZhKeyboardService.bind(keys.toogleRow, function () {
			var widget = getActiveWidget();
			if (widget) {
				if (widget.showCheckbox) {
					widget.toggle(widget.currentRow);
				}
				templateManager.updateTemplate();
			}
		});

		ZhKeyboardService.bind(keys.openSwipe, function () {
			var widget = getActiveWidget();
			if (widget) {
				if (widget.name == "select_multiple_widget") {
					widget.setSelected(widget.dataSource.getCheckedRows(), widget.selectField);
					widget.afterClosePopup(widget.selectField);
				} else if (widget.afterMoveRow) {
					widget.afterMoveRow();
				}
			}
		});

		ZhKeyboardService.bind(keys.openInlineEdit, function(event) {
			var widget = getActiveWidget();
			if(widget){
				var elem = $($('.currentRow').find('.inline-editable')[0]);
				event.preventDefault();

				if(widget.template.indexOf('grid') >= 0 && elem.attr('class') && elem.attr('class').indexOf('inline-editable') >= 0){
					elem.click();
				}
			}
		});

		ZhKeyboardService.bind("right", function() {
			var widget = getActiveWidget();
			if (widget) {
				if (isTree(widget) && !Util.isEmptyObject(widget.currentRow)) {
					var closedTreeParent = $('.currentRow').find('.tree-expand-icon.closed')[0];
					if (closedTreeParent) {
						closedTreeParent.click();
					}
				} else if (widget.currentRow.__group && widget.currentRow.__groupHeader) {
					widget.currentRow.__groupOpened = true;
					templateManager.updateTemplate();
				}
			}
		});

		ZhKeyboardService.bind("left", function () {
			var widget = getActiveWidget();
			if (widget) {
				if (isTree(widget) && !Util.isEmptyObject(widget.currentRow)) {
					widget.currentRow._opened = false;
					templateManager.updateTemplate();
				} else if (widget.currentRow.__group && widget.currentRow.__groupHeader) {
					widget.currentRow.__groupOpened = false;
					templateManager.updateTemplate();
				}
			}
		});

		ZhKeyboardService.bind('pageup', function (event) {
			var widget = getActiveWidget();
			if (widget) {
				event.preventDefault();

				var parent = $('.body').last();
				var widgetHeight = parent.height();
				var leftover = parent.height() % rowHeight;
				if (parent.siblings('.grid-vertical-scroll-control').length > 0){
					parent.siblings('.grid-vertical-scroll-control').scrollTop(parent.siblings('.grid-vertical-scroll-control').scrollTop() - widgetHeight + leftover);
				} else {
					parent.scrollTop(parent.scrollTop() - widgetHeight + leftover);
				}
			}
		});

		ZhKeyboardService.bind('pagedown', function (event) {
			var widget = getActiveWidget();
			if (widget) {
				event.preventDefault();

				var parent = $('.body').last();
				var widgetHeight = parent.height();
				var leftover = parent.height() % rowHeight;
				if (parent.siblings('.grid-vertical-scroll-control').length > 0){
					parent.siblings('.grid-vertical-scroll-control').scrollTop(parent.siblings('.grid-vertical-scroll-control').scrollTop() + widgetHeight - leftover);
				} else {
					parent.scrollTop(parent.scrollTop() + widgetHeight - leftover);
				}
			}
		});

		ZhKeyboardService.bind("up", function (event) {
			var widget = getActiveWidget();
			if (widget) {
				event.preventDefault();
				var activeGrid = angular.element($('.zh-new-grid-' + widget.name));
				var parent = activeGrid.find('.body');
				var scope = getElementScope(parent);
				if(scope){
					var dataSource = getDataSource(widget);
					pressUpKey(widget, dataSource, scope);
					templateManager.updateTemplate();
					scrollUp(parent);
				}
			}
		});

		ZhKeyboardService.bind("down", function (event) {
			var widget = getActiveWidget();
			if (widget) {
				event.preventDefault();
				var activeGrid = angular.element($('.zh-new-grid-' + widget.name));
				var parent = activeGrid.find('.body');
				var scope = getElementScope(parent);
				if(scope){
					var dataSource = getDataSource(widget);
					pressDownKey(widget, dataSource, scope);
					templateManager.updateTemplate();
					scrollDown(parent);
				}
			}
		});
	});

	function scrollUp(parent){
		var scrollControl = parent.siblings('.grid-vertical-scroll-control')[0];
		var row = parent.find(".currentRow:last");

		if (scrollControl) {
			if (row.length === 0 || row[0] == parent.find('.tr').first()[0]) {
				scrollControl.scrollTop -= rowHeight;
			}
		} else if (row.length > 0 && row.offset().top < parent.offset().top) {
			parent[0].scrollTop -= rowHeight;
		}
	}

	function pressUpKey(widget, dataSource, scope) {
		var previousCurrentRow = widget.currentRow;
		var index = dataSource.indexOf(widget.currentRow);
		if (Util.isEmptyObject(widget.currentRow) || index == -1) {
			widget.currentRow = dataSource[0] || {};
		} else if (index > 0) {
			var isClosed, row, found = false;
			do {
				index -= 1;
				row = dataSource[index];
				isClosed = row.__groupHeaders && row.__groupHeaders.some(function(header) {
					return header.__groupOpened === false;
				});
				found = !isClosed;
			} while (!found && index > 0);
			if (found) {
				widget.currentRow = dataSource[index];
			}
		}

		if (scope && scope.controller && scope.controller._rows) {
			if (scope.controller._rows.length > 0 &&
				!~scope.controller._rows.indexOf(widget.currentRow) &&
				!~scope.controller._rows.indexOf(previousCurrentRow)) {
				widget.currentRow = scope.controller._rows[0];
			}
		}
		if (widget.computeDescriptionOrder) widget.computeDescriptionOrder();
	}

	function scrollDown(parent){
		var scrollControl = parent.siblings('.grid-vertical-scroll-control')[0];
		var row = parent.find(".currentRow:last");
		if (scrollControl) {
			if (row.length === 0 || row[0] == parent.find('.tr').last()[0]) {
				scrollControl.scrollTop += rowHeight;
			}
		} else if (row.length > 0 && row.offset().top + rowHeight > parent.offset().top + parent.innerHeight()) {
			parent[0].scrollTop += rowHeight;
		}
	}

	function pressDownKey(widget, dataSource, scope) {
		var previousCurrentRow = widget.currentRow;
		var index = dataSource.indexOf(widget.currentRow);
		if (Util.isEmptyObject(widget.currentRow) || index == -1) {
			widget.currentRow = dataSource[0] || {};
		} else if (index != dataSource.length - 1) {
			var isClosed, row, found = false;
			do {
				index += 1;
				row = dataSource[index];
				isClosed = row.__groupHeaders && row.__groupHeaders.some(function(header) {
					return header.__groupOpened === false;
				});
				found = !isClosed;
			} while (!found && index < dataSource.length - 1);
			if (found) {
				widget.currentRow = dataSource[index];
			}
		}

		if (scope && scope.controller && scope.controller._rows) {
			if (scope.controller._rows.length > 0 &&
				!~scope.controller._rows.indexOf(widget.currentRow) &&
				!~scope.controller._rows.indexOf(previousCurrentRow)) {
				widget.currentRow = scope.controller._rows[scope.controller._rows.length - 1];
			}
		}
		if (widget.computeDescriptionOrder) widget.computeDescriptionOrder();
	}
}

Configuration(function (ContextRegister) {
	ContextRegister.register("GridNavigationLibrary", GridNavigationLibrary);
});

// FILE: bower_components/zh-preferences/dist/preferences.js
// FILE: src/ZhPreferences.js
// FILE: src/ZhPreferences.js
/**
 * Service to handle preferences storage
 *
 * @class
 * @param {RepositoryFactory} RepositoryFactory Service to factory repositories
 * @param {RequestEngine}     requestEngine     Service to make http requests
 * @param {RequestFactory}    requestFactory    Service to factory requests
 * @param {ZHPromise}         ZHPromise         Service to manipulate promises
 * @param {Query}             Query             Service to build queries
 */
function ZhPreferences(RepositoryFactory, requestEngine, requestFactory, ZHPromise, Query, $q) {

    var repositoryMap = {};

    var userWidgetConfiguration = {};

    /**
     * Register new repository to save preferences
     *
     * @param  {String} name Repository name used to save a preference
     */
    this.registerRepository = function(name) {
        var repositoryName = name.split('#').pop();
        repositoryMap[repositoryName] = RepositoryFactory.factory(name, 'MEMORY');
    };


    /**
     * Persist preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to persist
     *
     * @return {Promise}                 Preferences saved
     */
    this.savePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var returnedIds = response.dataset[preferenceType];
            return preferences.map(function(preference, index) {
                preference.__is_new = false;
                preference.changed = false;
                preference.ID = returnedIds[index].ID;
                return preference;
            });
        }).then(function(preferences) {
            return repositoryMap[preferenceType].save(preferences);
        });
    };

    /**
     * Delete preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to delete
     *
     * @return {Promise}                 Preferences deleted
     */
    this.deletePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var deletedIds = response.dataset[preferenceType].map(function(preference) {
                return preference.ID;
            });
            var query = Query.build().where('ID').in(deletedIds);

            return repositoryMap[preferenceType].remove(query);
        });
    };

    /**
     * Get local saved preferences from repository
     *
     * @param  {String} preferenceType Repository name
     * @param  {Query}  query          Conditions to filter the preferences
     *
     * @return {Promise}               Array of preferences
     */
    this.getPreferences = function(preferenceType, query) {
        var filterRequest = requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType,
            'requestType': 'filterData',
            'filter': query.where()
        });
        return requestEngine.doRequest(filterRequest)
            .then(function(response) {
                var preferences = response.dataset[preferenceType];
                var removePreferences = repositoryMap[preferenceType].remove(query);
                var savePreferences = repositoryMap[preferenceType].save(preferences);
                return ZHPromise.all([removePreferences, savePreferences]).then(function(){
                    return preferences;
                });
            })
            .catch(function(response) {
                throw new Error(response.data.error);
            });
    };

    /**
     * Get local saved preferences from repository filtering by widget and logged user
     *
     * @param  {String} preferenceType Repository name
     * @param  {String} widgetID       Widget identified
     *
     * @return {Promise}               Array of preferences
     */
    this.getWidgetPreferences = function(preferenceType, widgetID) {
        var query = Query.build().where('WIDGET_ID').equals(widgetID);
        return this.getPreferences(preferenceType, query);
    };

    /**
     * Get default preferences for user on specified widget
     *
     * @param  {String} widgetID Widget to get default preferences
     *
     * @return {Promise}         Widget default preferences
     */
    this.getDefaultWidgetPreferences = function(widgetID) {
        if (userWidgetConfiguration[widgetID] === undefined) {
            var query = Query.build().where('WIDGET_ID').equals(widgetID);

            userWidgetConfiguration[widgetID] = requestEngine.doRequest(requestFactory.factory({
                'serviceName': 'zh-preferences#/default-preferences',
                'requestType': 'filterData',
                'filter': query.where()
            })).then(function(response) {
                return response.dataset.user_widget.pop() || {
                    '__is_new': true,
                    'ID': null,
                    'WIDGET_ID': widgetID,
                    'DEFAULT_FILTER': null,
                    'DEFAULT_LAYOUT': null,
                    'DEFAULT_VIEW': null,
                    'LAST_MODIFIED_DATE': null
                };
            });
        }

        return userWidgetConfiguration[widgetID];
    };

    /**
     * Change default preference for user on specified widget
     *
     * @param  {String}  configWidget        Preference configuration widget
     * @param  {String}  property            Property that contains the default preference ID
     * @param  {Integer} preference          Preference configuration widget changed row
     *
     * @return {Promise}                     Widget default preferences
     */
    this.changeDefaultPreferences = function(configWidget, property, preference){
        var widgetID = configWidget.listWidget.id;
        var preferenceID = preference[property] ? preference.ID : null;

        userWidgetConfiguration[widgetID] = this.getDefaultWidgetPreferences(widgetID)
            .then(function(defaultPreferences) {
                defaultPreferences[property] = preferenceID;
                return ZHPromise.all([
                    defaultPreferences,
                    requestEngine.doRequest(requestFactory.factory({
                        'serviceName': 'zh-preferences#/default-preferences/save',
                        'requestType': 'dataSource',
                        'dataSource': [defaultPreferences]
                    }))
                ]);
            }).then(function(args) {
                var defaultPreferences = args[0];
                var response = args[1];
                defaultPreferences.ID = Util.getNestedProperty(response, ['dataset', 'user_widget', '0', 'ID']);

                configWidget.dataSource.data.forEach(function(row){
                    row.__is_new = false;
                    row[property] = row.ID == defaultPreferences[property];
                });
                return defaultPreferences;
            });

        return userWidgetConfiguration[widgetID];
    };

    function isMasterDetail(template){
        return template.indexOf('master_detail/grid.html') !== -1 ||
               template.indexOf('master_detail/list.html') !== -1 ||
               template.indexOf('master_detail/tree/grid.html') !== -1;
    }

    function isMasterDetailView(widget){
        return isMasterDetail(widget.template) && widget.mode == ViewMode.VIEW;
    }

    this.validateWidget = function(widget){
        if (!widget.id) {
            throw new Error('Widget\'s must have "id" to use Preferences. '+
                'Widget with name "'+widget.name+'" does not have an id.');
        }
        if (!widget.container.id) {
            throw new Error('Container\'s must have "id" to use Preferences.');
        }

        return !isMasterDetailView(widget);
    };

    var userPromise = ZHPromise.defer();

    /**
     * Invalidate logged user and clear preferences loaded
     */
    this.unsetUser = function() {
        userPromise = ZHPromise.defer();
        userWidgetConfiguration = {};

        Object.keys(repositoryMap).forEach(function(repositoryName) {
            repositoryMap[repositoryName].clearAll();
        });
    };

    /**
     * Set logged user
     *
     * @param {String} userID Logged user
     */
    this.setUserID = function(userID) {
        userPromise.resolve(userID);
    };

    /**
     * Get logged user ID
     *
     * @return {Promise} User identifier
     */
    this.getUserID = function() {
        return userPromise.promise;
    };


    /**
     * Generate preference file to download
     *
     * @param {Object} preference       Preference to be exported
     * @param {String} fileName         File name
     *
     * @return {Promise<Object>}        Preference exported
     */
    this.downloadPreferenceFile = function(preference, fileName) {
        var dataStr = "data:text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(preference,false,4));

        var linkElement = document.createElement('a');
        linkElement.setAttribute("href",     dataStr);
        linkElement.setAttribute("download", fileName);
        linkElement.click();
        linkElement.remove();

        return ZHPromise.when(preference);
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhPreferences', ZhPreferences);
});

// FILE: src/Config.js
(function(){

window.metaDataUrls = window.metaDataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-preferences';

var hasMetaData = window.metaDataUrls.some(function(metaDataUrl){
	return metaDataUrl.package === PACKAGE_NAME;
});

var hasServiceUrl = window.metaDataUrls.some(function(metaDataUrl){
    return metaDataUrl.package === PACKAGE_NAME;
});

if (!hasMetaData) {
    window.metaDataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-preferences/dist/assets/'
    });
}

if (!hasServiceUrl) {
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: bower_components/zh-filter-preferences/dist/filter-preferences.min.js
/*! 11-09-2019 */

function ZhFilterBuilder(Query){var defaultOperators={"field/between-dates.html":Query.BETWEEN,"field/calendar.html":Query.EQ,"field/date.html":Query.EQ,"field/datetime.html":Query.EQ,"field/interval.html":{default:Query.BETWEEN,start:Query.GTE,end:Query.LTE},"field/number-edit.html":Query.EQ,"field/number-keyboard.html":Query.EQ,"field/number.html":Query.EQ,"field/radio.html":Query.EQ,"field/select.html":Query.EQ,"field/select-simple.html":Query.EQ,"field/select-native.html":Query.EQ,"field/select-autocomplete.html":Query.EQ,"field/select-multiple.html":Query.IN,default:Query.LIKE_INSENSITIVE};var buildFilterByTemplate={"field/select.html":buildSelectFilter,"field/select-autocomplete.html":buildSelectFilter,"field/select-native.html":buildSelectNativeFilter,"field/select-multiple.html":buildSelectMultipleFilter,"field/interval.html":buildIntervalFilter,default:buildDefaultFilter};this.setDefaultOperator=function(template,operator){defaultOperators[template]=operator};this.setBuildFilterByTemplate=function(template,callback){buildFilterByTemplate[template]=callback};function getFieldTemplate(field){return field.template.split("#").pop()}function getFilterBuilderByTemplate(template){if(!buildFilterByTemplate[template]){template="default"}return buildFilterByTemplate[template]}function clearFilters(filters){var clearedFilters=[];filters.forEach(function(filter){var repeated=clearedFilters.some(function(_filter){return _filter.name===filter.name});if(!repeated){clearedFilters.push(filter)}});return clearedFilters}this.buildFilter=function(fields,row){var query=Query.build();fields.forEach(function(field){if(!field.skipSearch){var template=getFieldTemplate(field);getFilterBuilderByTemplate(template)(query,field,row)}});return clearFilters(query.where())};var operatorFlow={BETWEEN:function(query,value){query.between([value.start,value.end])},"NOT BETWEEN":function(query,value){query.notBetween([value.start,value.end])},LIKE:function(query,value){query.like("%"+value+"%")},LIKE_I:function(query,value){query.likeInsensitive("%"+value+"%")},LIKE_BEGINNING:function(query,value){query.likeInsensitive(value+"%")},"NOT LIKE":function(query,value){query.notLike("%"+value+"%")},"LIKE ALL":function(query,value){query.likeAll("%"+value+"%")},"<>":function(query,value){query.notEquals(value)},"=":function(query,value){query.equals(value)},">":function(query,value){query.greater(value)},"<":function(query,value){query.lower(value)},">=":function(query,value){query.greaterOrEqual(value)},"<=":function(query,value){query.lowerOrEqual(value)},IN:function(query,value){query.in(value)},"NOT IN":function(query,value){query.notIn(value)}};function buildFilter(query,fieldName,operator,value){query.where(fieldName);operatorFlow[operator](query,value)}function isEmptyArrayOrObject(obj){var name;for(name in obj){return false}return true}function existValue(value){var isObject=typeof value=="object";return isObject?!isEmptyArrayOrObject(value):!Util.isEmptyOrBlank(value)}function buildDefaultFilter(query,field,currentRow){var fieldName=field.name;var value=currentRow[fieldName];if(existValue(value)){var operator=getFieldOperator(field);buildFilter(query,fieldName,operator,value)}}function buildIntervalFilter(query,field,currentRow){var fieldName=field.name;var value=currentRow[fieldName];value=existValue(value)?Util.clone(value):{};var existStartValue=!Util.isEmptyOrBlank(value.start);var existEndValue=!Util.isEmptyOrBlank(value.end);if(existStartValue||existEndValue){var operator=getFieldOperator(field);if(existStartValue&&existEndValue){operator=operator.default}else if(existStartValue){operator=operator.start;value=value.start}else{operator=operator.end;value=value.end}buildFilter(query,fieldName,operator,value)}}function buildSelectWithValueFieldFilter(query,field,currentRow,operator){var fieldName=field.valueField;var value=currentRow[fieldName];if(existValue(value)){buildFilter(query,fieldName,operator,value)}}function buildSelectWithValueFieldsFilter(query,field,currentRow,operator){var fieldName=field.name;var value=currentRow[fieldName];if(existValue(value)){buildFilter(query,fieldName,operator,value)}}function buildSelectWithOutDataFilter(query,field,currentRow,operator){var outData=field.outData;Object.keys(outData).forEach(function(outDataKey){var fieldName=outData[outDataKey];var value=currentRow[fieldName];if(existValue(value)){buildFilter(query,fieldName,operator,value)}})}function buildSelectFilter(query,field,currentRow){var operator=getFieldOperator(field);if(field.valueField){buildSelectWithValueFieldFilter(query,field,currentRow,operator)}else if(field.valueFields){buildSelectWithValueFieldsFilter(query,field,currentRow,operator)}else if(field.outData){buildSelectWithOutDataFilter(query,field,currentRow,operator)}else{throw new Error("Invalid select configuration on field "+field.name)}}function buildSelectNativeWithValueFieldFilter(query,field,currentRow,operator){var fieldName=field.name;var value=currentRow[fieldName];if(existValue(value)){buildFilter(query,fieldName,operator,value)}}function buildSelectNativeFilter(query,field,currentRow){var operator=getFieldOperator(field);if(field.valueField){buildSelectNativeWithValueFieldFilter(query,field,currentRow,operator)}else{throw new Error("Invalid select-native configuration on field "+field.name)}}function buildSelectMultipleWithValueFieldFilter(query,field,currentRow,operator){var fieldName=field.name;var value=currentRow[fieldName];if(currentRow[fieldName+"_EXCEPT"]){value=currentRow[fieldName+"_EXCEPT"];operator="NOT IN"}if(existValue(value)){buildFilter(query,fieldName,operator,value)}}function buildSelectMultipleWithValueFieldsFilter(query,field,currentRow,operator){throw new Error("Filter on field select-multiple with property valuesFields is not supported yet. "+'Field "'+field.name+'"')}function buildSelectMultipleFilter(query,field,currentRow){var operator=getFieldOperator(field);if(field.valueField){buildSelectMultipleWithValueFieldFilter(query,field,currentRow,operator)}else if(field.valueFields){buildSelectMultipleWithValueFieldsFilter(query,field,currentRow,operator)}else{throw new Error("Invalid select-multiple configuration on field "+field.name)}}function getFieldOperator(field){var template=getFieldTemplate(field);return field.operator||defaultOperators[template]||defaultOperators["default"]}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterBuilder",ZhFilterBuilder)});function ZhFilterDescriptionBuilder(DecodeValueFieldService,ZhUtil,templateManager){this.buildFilterAppliedInfo=function(preferenceName){return[{label:"Filter",filterName:preferenceName}]};function getFilterWidget(id){return templateManager.container.factoredWidgets.filter(function(widget){return widget._filteredWidgetId==id})[0]}this.buildFilterInfo=function(fields,currentRow,searchCriteria,widget){var filterInfo=[];var filterWidget=getFilterWidget(widget&&widget.id);fields.forEach(function(field){buildDefaultFilterInfo(filterInfo,field,currentRow,filterWidget)});buildFilterInfoForSearchCriteria(fields,filterInfo,searchCriteria);return filterInfo};function buildFilterInfoForSearchCriteria(fields,filterInfo,searchCriteria){if(searchCriteria){Object.keys(searchCriteria).forEach(function(fieldName){if(searchCriteria[fieldName]!==""){var fieldLabel;if(fieldName=="_ALL_MAP"){fieldLabel="All fields"}else{fieldLabel=getFieldByName(fields,fieldName).label}filterInfo.push(buildFilterInfo(fieldLabel,searchCriteria[fieldName]))}})}}function getFieldByName(fields,fieldName){return fields.filter(function(field){return field.name===fieldName}).shift()}function buildFilterInfo(label,value){return{label:label,value:value}}function buildDefaultFilterInfo(filterInfo,field,currentRow,widget){var condition=!Util.isEmptyOrBlank(currentRow[field.name]);var filterField=widget&&widget.getField(field.name);var showInfo=filterField&&filterField.isVisible;if(condition&&showInfo){var value=DecodeValueFieldService.model(field,currentRow[field.name]);value=ZhUtil.evaluate(value);filterInfo.push(buildFilterInfo(field.label,value))}}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterDescriptionBuilder",ZhFilterDescriptionBuilder)});function ZhFilterPreferences(ZhPreferences,ZhFilterBuilder,ZhFilterDescriptionBuilder,templateManager,eventAggregator,ZhUtil,eventEngine,metaDataFactory,ZHPromise,MessageService,ScreenService,ZhKeyboardService,ApplicationContext,$timeout){var self=this;var originalFilterRows={};var originalFilter={ID:"ORIGINAL",USER_ID:0,ORGANIZATION_ID:0,LABEL_TO_PREFERENCE:"Original"};function shouldShowFilterAction(widget){return Util.get(widget,["floatingControl","searchAction"])!==false||Util.get(widget,["floatingControl","customizationAction","filter"])!==false}eventAggregator.subscribe("beforeProjectStartup",function(){ZhPreferences.registerRepository("zh-filter-preferences#filter");templateManager.project.customFloatingActions.push({name:"Filter Action",label:"Search",type:"filter",order:20,icon:"search",template:"zh-filter-preferences#floating-filter-card.html",expandable:true,isVisible:shouldShowFilterAction})});eventEngine.registerEvent("FilterBeforeApply","beforeFilterApply");eventEngine.registerEvent("FilterAfterApply","afterFilterApply");var filterRegisteredDataSources={};this.registerFilterDataSource=function(widgetID,dataSource){if(widgetID){filterRegisteredDataSources[widgetID]=dataSource}return filterRegisteredDataSources};this.updateFilters=function(widgetID,dataSource){if(!dataSource){dataSource=filterRegisteredDataSources[widgetID]}return this.getWidgetFilters(widgetID).then(function(filters){return this.addIsDefaultFilterToFilters(filters,widgetID).then(function(filters){dataSource.data=filters})}.bind(this))};function addIsDefaultFilterToFilter(defaultFilter,filter){filter.DEFAULT_FILTER=filter.ID===defaultFilter;return filter}this.addIsDefaultFilterToFilters=function(filters,widgetID){return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){var defaultFilter=defaultPreferences.DEFAULT_FILTER;return filters.map(addIsDefaultFilterToFilter.bind(this,defaultFilter))})};this.getWidgetFilters=function(widgetID){return ZhPreferences.getWidgetPreferences("filter",widgetID).then(function parseFilterRow(preferences){preferences.forEach(function(preference){preference.FILTER_ROW=JSON.parse(preference.FILTER_ROW)});return preferences})};this.getFilters=function(query){return ZhPreferences.getPreferences("filter",query)};this.saveFilters=function(filters){var cloneFilters=filters.map(function(filter){return ZhUtil.clearRow(filter)});return ZhPreferences.savePreferences("filter",cloneFilters).then(function(response){cloneFilters.forEach(function(clone,index){filters[index].ID=clone.ID;filters[index].__is_new=clone.__is_new;filters[index].changes=clone.changed});return ZHPromise.when(response)})};this.deleteFilters=function(filters){var cloneFilters=filters.map(function(filter){return ZhUtil.clearRow(filter)});return ZhPreferences.deletePreferences("filter",cloneFilters)};function applyOriginalFilterRow(widget,filter){filter.FILTER_ROW=Util.clone(originalFilterRows[widget.id]||{})}this.applyOriginalFilter=function(widget){applyOriginalFilterRow(widget,originalFilter);return this.applyFilter(widget,originalFilter)};this.setOriginalFilter=function(widgetId,filter){originalFilterRows[widgetId]=Util.clone(filter)};this.getOriginalFilter=function(widgetId,widget){var clearedFilter=ZhUtil.clearRow(originalFilterRows[widgetId]||{});var originalFilter={ID:"ORIGINAL",LABEL_TO_PREFERENCE:"Original",FILTER_ROW:Util.clone(clearedFilter)};return originalFilter};function getFilterConditions(filters){return filters.filter(function(filter){return filter.name.indexOf("SEARCHLIST_")===-1}).map(function(filter){filter.isCustomFilter=true;return filter})}function getSearchConditions(filters){var searchFields=Util.clone(filters);searchFields=searchFields.filter(function(filter){return~filter.name.indexOf("SEARCHLIST_")});return searchFields.map(function(filter){filter.name=filter.name.replace("SEARCHLIST_","");return filter})}function triggerBeforeFilterApplyEvent(widget,preference){if(widget.beforeFilterApply){return widget.beforeFilterApply({preference:preference})}}function triggerAfterFilterApplyEvent(widget,preference,filters){if(widget.afterFilterApply){widget.afterFilterApply({preference:preference,filters:filters})}}function prepareFilterRow(preference){var filterRow=preference.FILTER_ROW||{};filterRow=typeof preference.FILTER_ROW==="string"?JSON.parse(preference.FILTER_ROW):preference.FILTER_ROW;preference.FILTER_ROW=filterRow}function applyFilterInFilterPopup(widget,filterRow){self.createFilterWidget(widget,filterRow)}function mergeFilters(baseFilters,newFilters){var result=baseFilters.concat(newFilters).reduce(function(prev,current,index){var fieldName=current.name;if(fieldName in prev.keys){var replaceIndex=prev.keys[fieldName];prev.filters[replaceIndex]=current}else{prev.keys[fieldName]=index;prev.filters.push(current)}return prev},{filters:[],keys:{}});return result.filters}function buildDataSourceFilter(widget,filters){var filterConditions=getFilterConditions(filters);var originalFilters=getOriginalDataSourceFilters(widget.dataSourceFilter);return mergeFilters(originalFilters,filterConditions)}function applyFilterInDataSource(widget,filters){widget.dataSourceFilter=buildDataSourceFilter(widget,filters);var searchConditions=getSearchConditions(filters);if(widget.setSearchCriteria){widget.setSearchCriteria(searchConditions)}return searchConditions}function setAppliedFilter(widget,preference){widget.appliedFilter=preference}function applyFilterInWidget(widget,fields,filterRow,preference){self.updateFilterInfo(widget,fields,filterRow,preference,{});setAppliedFilter(widget,preference)}this.applyFilter=function(widget,preference,reloadWidget){reloadWidget=reloadWidget!==false;prepareFilterRow(preference);var event;if(reloadWidget){event=triggerBeforeFilterApplyEvent(widget,preference)}var filterRow=preference.CUSTOM_FILTER_ROW||preference.FILTER_ROW;var fields=self.factoryFields(null,widget.fields).filter(function removeInvisibleFields(field){return field.isVisible});var filters=ZhFilterBuilder.buildFilter(fields,filterRow);if(reloadWidget){if(!event||!event.isDefaultPrevented()){var searchConditions=applyFilterInDataSource(widget,filters);var searchCriteria=buildSearchCriteria(searchConditions);return widget.reload(ZhUtil.getLikeAllFilter(widget,searchCriteria)).then(function(){if(widget.dataSource){widget.dataSource.clearCheckedRows()}applyFilterInWidget(widget,fields,filterRow,preference);triggerAfterFilterApplyEvent(widget,preference,filters);applyFilterInFilterPopup(widget,preference.FILTER_ROW);return ZHPromise.when()}.bind(this))}else{return ZHPromise.reject()}}else{applyFilterInDataSource(widget,filters);applyFilterInWidget(widget,fields,filterRow,preference);applyFilterInFilterPopup(widget,preference.FILTER_ROW);return ZHPromise.when()}};this.updateFilterInfo=function(widget,fields,filterRow,preference,searchCriteria){var filterInfo;var filterName=preference.LABEL_TO_PREFERENCE;if((preference.ID==="ORIGINAL"||preference.ID==="EDITED")&&!filterName||preference.forceBuild){filterInfo=ZhFilterDescriptionBuilder.buildFilterInfo(fields,filterRow,searchCriteria,widget)}else{filterInfo=ZhFilterDescriptionBuilder.buildFilterAppliedInfo(filterName)}widget._filterInfo=filterInfo};function trimPercentageSymbol(value){if(value.charAt(0)==="%"){value=value.substr(1)}if(value.charAt(value.length-1)==="%"){value=value.substr(0,value.length-1)}return value}function buildSearchCriteria(conditions){var searchCriteria={};conditions.forEach(function(condition){searchCriteria[condition.name]=trimPercentageSymbol(condition.value)});return searchCriteria}function getOriginalDataSourceFilters(dataSourceFilter){return dataSourceFilter.filter(function(filter){return!filter.isCustomFilter})}function removeConflictingProperties(field){var conflictingProperties=["selectWidget","bindedEvents"];conflictingProperties.forEach(function(property){delete field[property]})}function factoryFields(gridFields,editing){var cloneField=function(field){var fieldClone=self.cloneObj(field);if(editing){fieldClone.fieldGroup="FILTER_FIELDS"}fieldClone.dataSource=field.dataSource;fieldClone.dataSourceFilter=field.dataSourceFilter;return fieldClone};return gridFields.filter(function(field){return!Util.isUndefined(field.template)}).map(cloneField)}function factorySearchFields(gridFields){var factoredFields=[{id:"SEARCHLIST__ALL_MAP",label:"All Fields",name:"SEARCHLIST__ALL_MAP",class:12,template:"field/text-edit.html",isVisible:true,showOnList:true,descending:true,fieldGroup:"MORE_FIELDS"}];var buildSearchListField=function(field){var fieldClone=self.cloneObj(field);fieldClone.id="SEARCHLIST_"+fieldClone.id;fieldClone.name="SEARCHLIST_"+fieldClone.name;fieldClone.class=6;fieldClone.isVisible=true;fieldClone.readOnly=false;fieldClone.validations={};fieldClone.template="field/text-edit.html";fieldClone.fieldGroup="MORE_FIELDS";return fieldClone};return factoredFields.concat(gridFields.filter(function(field){return!field.skipSearch&&Util.isUndefined(field.template)}).map(buildSearchListField))}function mergeFieldNewProperties(field,filterProperties){var keys=Object.keys(filterProperties);keys.forEach(function(property){field[property]=filterProperties[property]});if(field.dataSource&&!(field.dataSource instanceof DataSource)){field.dataSource=metaDataFactory.factoryDataSource(field.dataSource,field.dataSourceFilter,field)}delete field.filterProperties;return field}function removeEventListener(field,eventName,eventIndex){eventIndex=eventEngine.bindEvent(field,eventName);eventEngine.unbindEvent(field,eventName,eventIndex);eventIndex--;eventEngine.unbindEvent(field,eventName,eventIndex)}this.factoryFields=function(parentWidget,gridFields,editing){var fields=factoryFields(gridFields,editing);var searchFields=factorySearchFields(gridFields);var buildFilterProperties=function(field){if(field.filterProperties){field=mergeFieldNewProperties(field,field.filterProperties)}removeConflictingProperties(field);removeEventListener(field,"FieldOnChange");return metaDataFactory.fieldFactory(field,parentWidget)};return fields.concat(searchFields).map(buildFilterProperties)};this.downloadPreferenceFile=function(filter){var fileName=filter.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/gi,"").replace(/(?:^\w|[A-Z]|\b\w|\s+)/g,function(match,index){if(+match===0)return"";return index==0?match.toLowerCase():match.toUpperCase()})+".filter";var filterToExport={ID:filter.ID,LABEL_TO_PREFERENCE:filter.LABEL_TO_PREFERENCE,WIDGET_ID:filter.WIDGET_ID,FILTER_ROW:ZhUtil.clearRow(filter.FILTER_ROW),LAST_MODIFIED_DATE:filter.LAST_MODIFIED_DATE};return ZhPreferences.downloadPreferenceFile(filterToExport,fileName)};this.validateFilterImport=function(filter,configWidget,filterDataSource){filterDataSource=filterDataSource||configWidget.dataSource.data;if(configWidget.listWidget.id!=filter.WIDGET_ID){MessageService.showMessageByCode("zh-filter-preferences#import_error_widget_id");return false}var filtersWithSameId=filterDataSource.filter(function(row){return row.ID==filter.ID});if(filtersWithSameId.length>0){MessageService.showMessageByCode("zh-filter-preferences#import_error_id_violation");return false}var filtersWithSameLabel=filterDataSource.filter(function(row){return row.LABEL_TO_PREFERENCE==filter.LABEL_TO_PREFERENCE});if(filtersWithSameLabel.length>0){MessageService.showMessageByCode("zh-filter-preferences#import_error_name_violation");return false}return true};function openFilterWidget(widget,rowToBeSet){return createFilterWidget(widget,rowToBeSet).then(function(widgetToBeShown){widget.activate();ScreenService.openPopup(widgetToBeShown)})}this.openFilterWidget=openFilterWidget;var filterWidgets={};function forceSelectDataSourceToLoad(field,row){var data=Util.getNestedProperty(field,["dataSource","data","length"]);var currentRow=row[field.name];if(field.dataSource&&!data&&currentRow){var fieldName=field.valueField||field.name;field.dataSource.load([{name:fieldName,value:currentRow,operator:"IN"}])}}function forceFieldDataSourceToLoad(widget){widget.fields.forEach(function(field){forceSelectDataSourceToLoad(field,widget.currentRow)})}function createFilterWidget(widget,rowToBeSet){return ApplicationContext.ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForFilterPopup(widget).then(function(filterWidget){if(!widget.filterInitialized){filterWidget.listWidget=widget;widget.filterInitialized=true}filterWidget.label=ScreenService.i18n("Filter");var currentRow=rowToBeSet||widget.filterRowData;if(!currentRow){currentRow=widget.appliedFilter?widget.appliedFilter.FILTER_ROW:{}}filterWidget.currentRow=filterWidget.currentRow||{};Object.keys(currentRow).forEach(function(fieldName){filterWidget.currentRow[fieldName]=ZhUtil.evaluate(currentRow[fieldName])});forceFieldDataSourceToLoad(filterWidget);filterWidget.actions[0].isVisible=true;filterWidget.actions[1].isVisible=true;filterWidget.actions[2].isVisible=false;filterWidget.actions[3].isVisible=false;filterWidgets[widget.id]=filterWidget;return ZHPromise.when(filterWidget)}.bind(this))}this.createFilterWidget=createFilterWidget;function isCopiable(object){var isCopiable=Util.isUndefinedOrNull(object);if(!isCopiable){var constr=object.constructor;var types=[String,Number,Array,Object,Boolean];isCopiable=types.some(function(type){return type().constructor===constr})}return isCopiable}this.cloneObj=function(object){var clone={};Object.keys(object).forEach(function(property){if(isCopiable(object[property])){clone[property]=ZhUtil.clearRow(object[property])}});return clone};function hasPopupOpened(){var isMobiScrollOpen=$(".mbsc-mobiscroll").length>0;return!ScreenService.isSideMenuOpen()&&!isMobiScrollOpen&&!ScreenService.isAlertShown()}function isFilterPreferencesActive(widget){return Util.get(widget,["floatingControl","customizationAction","filter"])!==false}function isSearchActionActive(widget){return Util.get(widget,["floatingControl","searchAction"])!==false}function openWidgetFilter(){var activeWidget=templateManager.container.getActiveWidget();if(hasPopupOpened()&&isFilterPreferencesActive(activeWidget)&&ZhPreferences.validateWidget(activeWidget)){openFilterWidget(activeWidget)}return false}function openWidgetSearch(){var activeWidget=templateManager.container.getActiveWidget();var searchIsEnabled=isFilterPreferencesActive(activeWidget)||isSearchActionActive(activeWidget);if(searchIsEnabled&&hasPopupOpened()&&ZhPreferences.validateWidget(activeWidget)){activeWidget.openSearchAction()}return false}templateManager.projectPromise.then(function(){templateManager.project.shortcutKeys=templateManager.project.shortcutKeys||{};var openFilterKey=templateManager.project.shortcutKeys.openWidgetFilter||"mod+l";var openSearchKey=templateManager.project.shortcutKeys.openWidgetSearch||"mod+f";ZhKeyboardService.unbind(openFilterKey);ZhKeyboardService.unbind(openSearchKey);ZhKeyboardService.bind(openFilterKey,openWidgetFilter);ZhKeyboardService.bind(openSearchKey,openWidgetSearch)})}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterPreferences",ZhFilterPreferences)});function ZhFilterWidgetsBuilder(restEngine,templateManager,metaDataFactory,ZhFilterPreferences){var widgetsJson={};function getWidgetJSON(widgetName){var widgetPath="zh-filter-preferences#json/widgets/"+widgetName+".json";if(!widgetsJson[widgetPath]){widgetsJson[widgetPath]=restEngine.requestMetaData(widgetPath).then(function(response){return response.data})}return widgetsJson[widgetPath]}this.factoryWidget=function(widgetName){return getWidgetJSON(widgetName).then(function(widget){widget=Util.clone(widget);return metaDataFactory.widgetFactory(widget,templateManager.container)})};this.factoryFilterConfigurationWidget=function(){return this.factoryWidget("config")};this.factoryCreateFilterWidget=function(){return this.factoryWidget("createFilter")};this.factoryImportFilterWidget=function(){return this.factoryWidget("importFilter")};function copyFieldGroups(fieldGroups){var copiedObj=(fieldGroups||[]).map(function(fieldGroup){return Util.clone(fieldGroup,["__proto__","rows","widget"])});return copiedObj}function buildFilterWidgetFieldGroups(filterWidget,widget){var fieldGroups=(filterWidget.fieldGroups||[]).concat(copyFieldGroups(widget.fieldGroups));var moreFieldsFieldGroup=[];fieldGroups=fieldGroups.filter(function(fieldGroup){var isMoreFieldFieldGroup=fieldGroup.name==="MORE_FIELDS";if(isMoreFieldFieldGroup){moreFieldsFieldGroup=fieldGroup;moreFieldsFieldGroup.isVisible=!widget.hideMoreFieldsOnFilter}return!isMoreFieldFieldGroup});filterWidget.fieldGroups=fieldGroups.concat(moreFieldsFieldGroup)}this.factoryBuildFiltersConditionsWidget=function(baseWidget){return getWidgetJSON("createFilterConditions").then(function(widgetJson){widgetJson=Util.clone(widgetJson);buildFilterWidgetFieldGroups(widgetJson,baseWidget);var widget=metaDataFactory.widgetFactory(widgetJson,templateManager.container);widget.fields=widget.fields.concat(ZhFilterPreferences.factoryFields(widget,baseWidget.fields));widget._filteredWidgetId=baseWidget.id;return widget})}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterWidgetsBuilder",ZhFilterWidgetsBuilder)});function ZhFilterCreateFilterConditionsController(ZhFilterPreferences,ZhFilterWidgetsBuilder,MessageService,ScreenService,ZHPromise,ZhUtil,templateManager,$timeout){var createFilterConditionsWidgetPromise={};this.clearCachedFilterConditionsWidget=function(widget){if(widget&&widget.id){widget.filterInitialized=false;delete createFilterConditionsWidgetPromise[widget.id]}};this.getCreateFilterConditionsWidget=function(widget){var widgetId=widget.id;if(!createFilterConditionsWidgetPromise[widgetId]){createFilterConditionsWidgetPromise[widgetId]=ZhFilterWidgetsBuilder.factoryBuildFiltersConditionsWidget(widget)}return createFilterConditionsWidgetPromise[widgetId]};function saveFilterFields(widget){var allowedFields=widget.fields.map(function(field){return ZhFilterPreferences.cloneObj(field)});widget.__originalFilterStatus=allowedFields;widget.__originalCurrentRow=ZhFilterPreferences.cloneObj(ZhUtil.clearRow(widget.currentRow))}this.getCreateFilterConditionsWidgetForFilterPopup=function(widget){return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){createFilterConditionsWidget.actions[0].isVisible=true;createFilterConditionsWidget.actions[1].isVisible=true;createFilterConditionsWidget.actions[2].isVisible=false;createFilterConditionsWidget.actions[3].isVisible=false;var labelField=createFilterConditionsWidget.getField("LABEL_TO_PREFERENCE");if(labelField){labelField.isVisible=false}saveFilterFields(createFilterConditionsWidget);return createFilterConditionsWidget})};this.getCreateFilterConditionsWidgetForEditPreference=function(widget){return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){createFilterConditionsWidget.actions[0].isVisible=false;createFilterConditionsWidget.actions[1].isVisible=false;createFilterConditionsWidget.actions[2].isVisible=true;createFilterConditionsWidget.actions[3].isVisible=true;var labelField=createFilterConditionsWidget.getField("LABEL_TO_PREFERENCE");if(labelField){labelField.isVisible=true}saveFilterFields(createFilterConditionsWidget);return createFilterConditionsWidget})};this.saveAndApplyFilter=function(createFilterConditionsWidget){var clearRequiredFieldErrors=function(invalidFields){invalidFields.shift();invalidFields.forEach(function(invalidField){var fieldName;invalidField=invalidField.filter(function(invalidItem){fieldName=invalidItem.fieldName;return invalidItem.errorMessage!="Required field"});if(!invalidField.length){delete createFilterConditionsWidget.currentRow.$error[fieldName]}})};if(createFilterConditionsWidget.isValid(clearRequiredFieldErrors)){var currentRow=createFilterConditionsWidget.currentRow;var filter=createFilterConditionsWidget.preference;filter.LABEL_TO_PREFERENCE=currentRow.LABEL_TO_PREFERENCE;filter.FILTER_ROW=ZhUtil.clearRow(currentRow);delete filter.FILTER_ROW.LABEL_TO_PREFERENCE;filter.FILTER_ROW=JSON.stringify(filter.FILTER_ROW);var configWidget=createFilterConditionsWidget.configWidget;var listWidget=configWidget.listWidget;return ZhFilterPreferences.saveFilters([filter]).then(function(){filter.FILTER_ROW=JSON.parse(filter.FILTER_ROW);MessageService.successNotificationByCode("zh-filter-preferences#save_success");ScreenService.closePopup(true);if(createFilterConditionsWidget.isValid()){filter.forceBuild=true;listWidget.filterRowData=currentRow;ZhFilterPreferences.applyFilter(listWidget,filter);listWidget.activate()}else{listWidget.openFilterWidget(createFilterConditionsWidget.currentRow)}ZhFilterPreferences.updateFilters(listWidget.id,configWidget.dataSource)}).catch(function(error){MessageService.showMessageByCode("zh-filter-preferences#save_error",{ERROR:error.message})})}else{return ZHPromise.reject("Invalid params")}};function getRowToApply(appliedFilter,currentRow){var appliedRow=appliedFilter&&appliedFilter.FILTER_ROW||{};var row=ZhUtil.clearRow(currentRow);Object.keys(appliedRow).forEach(function(key){if(Util.isString(appliedRow[key])&&appliedRow[key].indexOf("@")===0){row[key]=appliedRow[key]}});return row}this.clearFilter=function(widget){widget.fields.filter(function(field){return!field.readOnly}).forEach(function(field){var template=field.template||"";field.clearValue(field);if(template.indexOf("field/checkbox.html")!==-1){widget.currentRow[field.name]=field.valueToFalse;field.checked=false;templateManager.updateTemplate()}else if(template.indexOf("field/select-native.html")!==-1){$timeout(function(){var dropdown=field.element.next();dropdown.find(".selected").removeClass("selected");dropdown.find(".current").text("")})}if(field.change){field.change()}})};this.applyFilter=function(createFilterConditionsWidget,filterName){if(createFilterConditionsWidget.isValid()){var currentRow=createFilterConditionsWidget.currentRow;var listWidget=createFilterConditionsWidget.listWidget;delete currentRow.LABEL_TO_PREFERENCE;var filter={ID:"EDITED",FILTER_ROW:getRowToApply(listWidget.appliedFilter,currentRow)};if(filterName){filter.LABEL_TO_PREFERENCE=filterName}return ZhFilterPreferences.applyFilter(listWidget,filter).then(function(){if(ScreenService.getPopupOpen()==createFilterConditionsWidget){ScreenService.closePopup(true);listWidget.activate()}})}else{if(ScreenService.getPopupOpen()!=createFilterConditionsWidget){ScreenService.openPopup(createFilterConditionsWidget)}return ZHPromise.reject()}};function restoreFilterFields(widget){if(widget.__originalFilterStatus){widget.fields.forEach(function(field,index){var __field=widget.__originalFilterStatus.filter(function(fieldOriginal){return fieldOriginal.name===field.name})[0];Object.keys(__field).forEach(function(key){widget.fields[index][key]=__field[key]})});widget.currentRow=widget.__originalCurrentRow;delete widget.__originalCurrentRow;delete widget.__originalFilterStatus}}this.closePopup=function(widget){restoreFilterFields(widget);ScreenService.closePopup();widget.listWidget.activate()}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterCreateFilterConditionsController",ZhFilterCreateFilterConditionsController)});function ZhFilterImportFilterController(MessageService,ZhFilterPreferences,ZhPreferences,ZhFilterWidgetsBuilder,ZHPromise,ScreenService){var importFilterWidgetPromise=null;this.getImportFilterWidget=function(){if(importFilterWidgetPromise===null){importFilterWidgetPromise=ZhFilterWidgetsBuilder.factoryImportFilterWidget()}return importFilterWidgetPromise};this.importPreference=function(importFilterWidget){if(!importFilterWidget.isValid()){return ZHPromise.reject({err:"WIDGET_INVALID"})}var filterFileData=importFilterWidget.fields[0].value()[0].b64File;filterFileData=filterFileData.split("base64,")[1];filterFileData=window.atob(filterFileData);filterFileData=decodeURIComponent(escape(filterFileData));var filter=JSON.parse(filterFileData);var configWidget=importFilterWidget.configWidget;var listWidget=configWidget.listWidget;if(!ZhFilterPreferences.validateFilterImport(filter,configWidget)){return ZHPromise.reject({err:"FILTER_FILE_INVALID"})}filter.__is_new=true;filter.__generate_id=false;filter.FILTER_ROW=JSON.stringify(filter.FILTER_ROW);return ZhPreferences.getUserID().then(function(userID){filter.USER_ID=userID;return ZhFilterPreferences.saveFilters([filter]).then(function(){filter.FILTER_ROW=JSON.parse(filter.FILTER_ROW);ScreenService.closePopup();ZhFilterPreferences.updateFilters(listWidget.id,configWidget.dataSource);configWidget.refreshWidget()}).catch(function(error){MessageService.showMessageByCode("zh-filter-preferences#save_error",{ERROR:error.message})})})}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterImportFilterController",ZhFilterImportFilterController)});function ZhFilterConfigWidgetController(ZhPreferences,ZhFilterPreferences,ZhFilterWidgetsBuilder,ScreenService,MessageService,ZHPromise,ZhFilterCreateFilterConditionsController,ZhFilterImportFilterController){var configWidgetPromise=null;this.getConfigWidget=function(listWidget,dataSource){if(configWidgetPromise===null){configWidgetPromise=ZhFilterWidgetsBuilder.factoryFilterConfigurationWidget()}var userID=ZhPreferences.getUserID();return ZHPromise.all([userID,configWidgetPromise]).then(function(args){var userID=args[0];var configWidget=args[1];configWidget.listWidget=listWidget;configWidget.dataSourceFilter=[{name:"WIDGET_ID",operator:"=",value:listWidget.id}];dataSource.data.forEach(function(row){row.APPLIED=row.ID==(listWidget.appliedFilter?listWidget.appliedFilter.ID:"ORIGINAL")});configWidget.dataSource=dataSource;return configWidget})};var hideMoreFields=function(groups,hide){var moreFieldsFieldGroup=[];return groups.filter(function(fieldGroup){delete fieldGroup.rows;var isMoreFieldFieldGroup=fieldGroup.name==="MORE_FIELDS";if(isMoreFieldFieldGroup){moreFieldsFieldGroup=fieldGroup;moreFieldsFieldGroup.isVisible=!hide}return!isMoreFieldFieldGroup}).concat(moreFieldsFieldGroup)};this.openCreateFilterWidget=function(configWidget){var label=configWidget.currentRow.LABEL_TO_PREFERENCE;return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget).then(function(createFilterConditionsWidget){var listWidget=configWidget.listWidget;createFilterConditionsWidget.label=ScreenService.i18n("Filter");createFilterConditionsWidget.configWidget=configWidget;if(!listWidget.filterInitialized){var fieldGroups=(createFilterConditionsWidget.fieldGroups||[]).concat(listWidget.fieldGroups||[]);createFilterConditionsWidget.fieldGroups=fieldGroups;listWidget.filterInitialized=true}createFilterConditionsWidget.fieldGroups=hideMoreFields(createFilterConditionsWidget.fieldGroups,listWidget.hideMoreFieldsOnFilter);return ZhPreferences.getUserID().then(function(userID){var preference={__is_new:true,ID:null,USER_ID:userID,LABEL_TO_PREFERENCE:label,WIDGET_ID:listWidget.id,LAST_MODIFIED_DATE:null};createFilterConditionsWidget.preference=preference;var labelField=createFilterConditionsWidget.getField("LABEL_TO_PREFERENCE");var fields=ZhFilterPreferences.factoryFields(createFilterConditionsWidget,listWidget.fields,true);createFilterConditionsWidget.fields=[labelField].concat(fields);labelField.isVisible=true;createFilterConditionsWidget.newRow();return ScreenService.openPopup(createFilterConditionsWidget)})})};this.editFilter=function(configWidget,filter){var listWidget=configWidget.listWidget;return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget).then(function(createFilterConditionsWidget){createFilterConditionsWidget.preference=filter;createFilterConditionsWidget.label=ScreenService.i18n("Filter")+" - "+filter.LABEL_TO_PREFERENCE;createFilterConditionsWidget.configWidget=configWidget;createFilterConditionsWidget.fieldGroups=hideMoreFields(createFilterConditionsWidget.fieldGroups||[],listWidget.hideMoreFieldsOnFilter);var labelField=createFilterConditionsWidget.getField("LABEL_TO_PREFERENCE");var fields=ZhFilterPreferences.factoryFields(createFilterConditionsWidget,listWidget.fields,true);createFilterConditionsWidget.fields=[labelField].concat(fields);labelField.isVisible=true;createFilterConditionsWidget.currentRow=filter.FILTER_ROW;createFilterConditionsWidget.currentRow.LABEL_TO_PREFERENCE=filter.LABEL_TO_PREFERENCE;return ScreenService.openPopup(createFilterConditionsWidget)})};this.deleteFilters=function(configWidget){var filtersToDelete=configWidget.getCheckedRows();var defaultFilter=filtersToDelete.filter(function(filter){return filter.DEFAULT_FILTER});if(defaultFilter.length){MessageService.showMessageByCode("zh-filter-preferences#delete_error_default");return}return MessageService.confirmMessageByCode("zh-filter-preferences#delete",{NR_OF_FILTERS:filtersToDelete.length}).then(function(){return ZhFilterPreferences.deleteFilters(filtersToDelete).then(function(){configWidget.dataSource.clearCheckedRows();return ZhFilterPreferences.updateFilters(configWidget.listWidget.id,configWidget.dataSource)}).catch(function(error){return MessageService.getMessageByCode("zh-filter-preferences#"+error.message).then(function(errorMessage){return MessageService.showMessageByCode("zh-filter-preferences#delete_error",{ERROR:ScreenService.i18n(errorMessage)})})})})};this.updateDefaultFilter=function(configWidget,filter){return ZhPreferences.changeDefaultPreferences(configWidget,"DEFAULT_FILTER",filter)};this.updateAppliedFilter=function(configWidget,filter){var listWidget=configWidget.listWidget;return ZhFilterPreferences.applyFilter(listWidget,filter).then(function(){configWidget.dataSource.data.forEach(function(row){row.APPLIED=row.ID==filter.ID})})};this.downloadPreferenceFile=function(filter){return ZhFilterPreferences.downloadPreferenceFile(filter)};this.importPreferenceFile=function(configWidget){return ZhFilterImportFilterController.getImportFilterWidget().then(function(importFilterWidget){importFilterWidget.newRow();importFilterWidget.configWidget=configWidget;return ScreenService.openPopup(importFilterWidget)})}}Configuration(function(ContextRegister){ContextRegister.register("ZhFilterConfigWidgetController",ZhFilterConfigWidgetController)});function ZhFilterFloatingCardController($scope,ApplicationContext,templateManager,metaDataFactory,ScreenService,Query,eventEngine,ZHPromise,ZhUtil){var widget=$scope.widget;var ZhFilterPreferences=ApplicationContext.ZhFilterPreferences;var ZhPreferences=ApplicationContext.ZhPreferences;var ZhFilterConfigWidgetController=ApplicationContext.ZhFilterConfigWidgetController;var ZhFilterCreateFilterConditionsController=ApplicationContext.ZhFilterCreateFilterConditionsController;var filterWidget;var dataSource=metaDataFactory.factoryDataSource({name:"zh-filter-preferences#/filter",localStorage:false,lazyLoad:true,rest:true,data:[]});dataSource.afterLoadDataSource(function(){return ZhFilterPreferences.addIsDefaultFilterToFilters(dataSource.data,widget.id).then(function(filters){dataSource.data=filters.map(function(filter){filter.FILTER_ROW=JSON.parse(filter.FILTER_ROW);return filter})})});$scope.customizationCard={label:"Filters",type:"filter",openFilterConfiguration:openFilterConfiguration,refreshFilters:refreshFilters,openFilterWidget:openFilterWidget,applyOriginalFilter:applyOriginalFilter,applyFilter:applyPreference,dataSource:dataSource};ZhFilterPreferences.registerFilterDataSource(widget.id,$scope.customizationCard.dataSource);widget.openFilterWidget=openFilterWidget;widget.refreshFilters=refreshFilters;function updateFilterRow(filterRow,searchCriteria){var changed=false;filterRow=filterRow||{};Object.keys(searchCriteria).forEach(function(key){var name="SEARCHLIST_"+key;var filterValue=filterRow[name];if(!Util.isEmptyOrBlank(filterValue)&&searchCriteria[key]!==filterValue){changed=true}filterRow[name]=searchCriteria[key]});filterRow.changed=changed;return filterRow}$scope.$watchCollection("widget.searchCriteria",function(searchCriteria){if(ZhPreferences.validateWidget(widget)){var filterRow=widget.appliedFilter&&widget.appliedFilter.FILTER_ROW;filterRow=updateFilterRow(filterRow,searchCriteria||{});if(filterRow.changed){widget.appliedFilter={ID:"EDITED",FILTER_ROW:filterRow}}delete filterRow.changed;if(widget.appliedFilter){ZhFilterPreferences.updateFilterInfo(widget,widget.fields,filterRow,widget.appliedFilter,searchCriteria)}}});function openFilterConfiguration(){var dataSource=$scope.customizationCard.dataSource;return ZhFilterConfigWidgetController.getConfigWidget(widget,dataSource).then(function(configWidget){return ScreenService.openPopup(configWidget)})}function refreshFilters(){return ZhPreferences.getUserID().then(function(){var query=Query.build().where("WIDGET_ID").equals(widget.id);return dataSource.load(query.where())})}function openFilterWidget(rowToBeSet){return ZhFilterPreferences.openFilterWidget(widget,rowToBeSet)}function applyOriginalFilter(){return ZhFilterPreferences.applyOriginalFilter(widget)}function applyPreference(filter,reloadWidget){ZhFilterPreferences.applyFilter(widget,filter,reloadWidget)}function getDefaultFilter(){return dataSource.data.filter(function(filter){return filter.DEFAULT_FILTER}).shift()}function applyDefaultFilter(){var isLazyLoad=Util.getNestedProperty(widget,["dataSource","lazyLoad"]);var filter=getDefaultFilter()||ZhFilterPreferences.getOriginalFilter(widget.id);applyPreference(filter,!isLazyLoad)}function getDataSourceFilterRow(){var filterRow={};(widget.dataSourceFilter||[]).filter(function removeCustoms(filter){return!filter.isCustomFilter}).forEach(function parseToFilterRow(filter){filterRow[filter.name]=filter.value});return filterRow}function initOriginalFilter(){var widgetId=widget.id;var filter=ZhFilterPreferences.getOriginalFilter(widgetId);if(Util.isEmptyObject(filter.FILTER_ROW)){var filterRow=getDataSourceFilterRow();widget.fields.forEach(function(field){var hasFilterDefaultValue=field.filterProperties&&field.filterProperties.hasOwnProperty("defaultValue");var defaultValue=hasFilterDefaultValue?field.filterProperties.defaultValue:field.defaultValue;if(!Util.isUndefinedOrNull(defaultValue)){filterRow[field.name]=defaultValue}});ZhFilterPreferences.setOriginalFilter(widgetId,filterRow)}}function isFilterPreferencesActive(activeWidget){return Util.get(activeWidget||widget,["floatingControl","customizationAction","filter"])!==false}$scope.$on("$destroy",function(){delete widget.openFilterWidget;delete widget.refreshFilters;delete dataSource.afterLoadDataSourcelistener});(function initFilterPreferences(){var filterIsActive=isFilterPreferencesActive();if(filterIsActive&&ZhPreferences.validateWidget(widget)){ZhFilterPreferences.updateFilters(widget.id,$scope.customizationCard.dataSource).then(function(){if(Util.isUndefinedOrNull(widget.appliedFilter)&&widget.filterInitialized!==true){initOriginalFilter();applyDefaultFilter()}})}})()}var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhFloatingControl",["$timeout","templateManager",ZhFilterFloatingControl]);function ZhFilterFloatingControl($timeout,templateManager){function checkFloatingCardOverflow(e){var actionElement=$(e.target).closest(".float-action");var floatingControl=actionElement.parents(".zh-floating-control");var widgetContainer=actionElement.closest(".container");var actionTop,actionBottom;var widgetTop,widgetBottom;if(!actionElement.hasClass("opened")){actionElement.removeClass("fix-position-top");actionElement.removeClass("fix-position-bottom");actionTop=actionElement.offset().top;widgetTop=widgetContainer.offset().top;widgetBottom=widgetTop+widgetContainer.height();if(floatingControl.hasClass("pos-top")){actionTop=actionTop-200+actionElement.height();if(actionTop<widgetTop){actionElement.addClass("fix-position-top")}}else{actionBottom=actionTop+200;if(actionBottom>widgetBottom){actionElement.addClass("fix-position-bottom")}}}}return{priority:10,link:function($scope,element,attrs,controller){var widget=$scope.widget;if(widget.floatingControl===false){return}var isFilterApplied=function(){return widget&&widget.appliedFilter&&Object.keys(widget.appliedFilter.FILTER_ROW).length>0};var defaultOptions={searchAction:Util.isUndefined(element.parent().attr("data-hide-search"))&&Util.isUndefined(element.parent().attr("hide-search")),openSearch:false};widget.floatingControl=Util.extend(defaultOptions,widget.floatingControl);var isFilterAppliedWithoutOriginal=function(){return widget._filterInfo&&widget._filterInfo.length>0&&widget.appliedFilter&&widget.appliedFilter.ID!="ORIGINAL"};widget.floatingControl.addStatus({priority:20,classString:"searching",icon:"more",conditionFunction:function(){return isFilterAppliedWithoutOriginal()},infoTemplate:function(){var hideFilterValue=templateManager.project.hideFilterValue||widget.hideFilterValue;var filterInfoValue=hideFilterValue?"</b> {{info.filterName}}":":</b> {{info.value}}";if(isFilterApplied()){widget.floatingControl.statuses[1].icon="search";if(widget.floatingControl.customizationAction&&widget.floatingControl.customizationAction.filter){icon="filter";method="openFilter"}else{icon="close-x";method="clearFilter"}return'<div data-zh-touchstart="'+method+'()"> <div class="searching-control-info"><span data-zh-icon="'+icon+' no-border icon-white"></span><div class="searching-control-info-details"><span ng-repeat="info in widget._filterInfo"><b>{{info.label|i18n}}'+filterInfoValue+'</span></div></div><div class="searching-control-info-text" data-zh-bind-translate="\'Filter applied\'"></div></div>'}widget.floatingControl.statuses[1].icon="more"},scope:$scope});if(templateManager.project.showEmptyFilter){widget.floatingControl.addStatus({priority:30,classString:"emptyFilter",icon:"more",conditionFunction:function(){return!isFilterAppliedWithoutOriginal()},infoTemplate:'<div data-zh-touchstart="openFilter()"> <div class="searching-control-info"><span data-zh-icon="filter no-border icon-white"></span></div><div class="searching-control-info-text"></div></div>',scope:$scope})}$scope.openSearchAction=function(){$timeout.cancel($scope._closeTimeout);if(!element.hasClass("opened")){$scope.toggleActionMenu()}$scope.closeOpenedActions();var searchAction=element.find(".search-action").closest(".float-action");if(searchAction&&searchAction.length>0){searchAction.find("span.search-action-icon").click();Util.selectElement(searchAction.find("input"))}};widget.openSearchAction=$scope.openSearchAction;$scope.$on("$destroy",function(){delete widget.openSearchAction;widget.floatingControl.statuses=[]});$scope.openFilter=function(){$scope.widget.openFilterWidget()};$scope.clearFilter=function(){$scope.widget.searchCriteria={}};$scope.togglePreferencesCard=function(e){checkFloatingCardOverflow(e);$scope.toggleAction(e)};$scope.clearSearch=function(event,fieldName){if(!fieldName){widget.searchCriteria={}}else{delete widget.searchCriteria[fieldName]}if(event){Util.selectElement($(event.target).closest(".floating-card-input").find("input"))}};$scope.toggleSearchFieldsList=function(event){if(!widget.floatingControl.customizationAction||!widget.floatingControl.customizationAction.filter){var el=$(event.target).closest(".floating-card").find(".floating-card-search-field-select");el.toggleClass("opened");if(el.hasClass("opened")){$(event.target).closest(".floating-card").find(".floating-card-select-field-options").removeClass("opened");$(event.target).closest(".floating-card").find(".zh-select-search-floating").removeClass("opened")}}};$scope.selectSearchField=function(field){$scope.searchField=field;element.find(".floating-card-search-field-select").removeClass("opened");element.find("input.zh-input-search-floating").focus()};$scope.filterFields=function(field){return field.aclIsVisible&&!field.skipSearch&&(field.isVisible&&field.showOnList||field.showOnFilter)};$scope.getFields=function(widget){return(widget.fields||[]).filter($scope.filterFields)};if(widget.floatingControl.defaultSearchField){$scope.searchField=widget.getField(widget.floatingControl.defaultSearchField)}if(widget.floatingControl.openSearch){$timeout(widget.openSearchAction,0,false)}}}}(function(){window.templateUrls=window.templateUrls||[];window.metadataUrls=window.metadataUrls||[];window.serviceUrls=window.serviceUrls||[];var PACKAGE_NAME="zh-filter-preferences";var hasTemplateUrl=Boolean(window.templateUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasMetadataUrl=Boolean(window.metadataUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasServiceUrl=Boolean(window.serviceUrls.find(function(value){return value.package===PACKAGE_NAME}));if(!hasTemplateUrl){window.templateUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-filter-preferences/dist/templates/"})}if(!hasMetadataUrl){window.metadataUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-filter-preferences/dist/assets/"})}if(!hasServiceUrl){window.serviceUrls.push({package:PACKAGE_NAME,baseUrl:"../backend/service/index.php/zh-preferences"})}})();

// FILE: bower_components/zh-layout-preferences/dist/layout-preferences.min.js
/*! 04-09-2018 */

function ZhLayoutPreferences(ZhPreferences,templateManager,eventAggregator,containerMerger,ZHPromise,eventEngine,ZhUtil,MessageService){var originalLayout={ID:"ORIGINAL",USER_ID:0,ORGANIZATION_ID:0,LAYOUT_CHANGES:"{}",LABEL_TO_PREFERENCE:"Original"};function shouldShowLayoutAction(widget){return Util.get(widget,["floatingControl","customizationAction","layout"])!==false}eventAggregator.subscribe("beforeProjectStartup",function(){ZhPreferences.registerRepository("zh-layout-preferences#layout");templateManager.project.customFloatingActions.push({name:"Layout Action",label:"Layout",type:"layout",order:20,icon:"layout",template:"zh-layout-preferences#floating-layout-card.html",expandable:true,isVisible:shouldShowLayoutAction})});eventEngine.registerEvent("LayoutBeforeApply","beforeLayoutApply");eventEngine.registerEvent("LayoutAfterApply","afterLayoutApply");var layoutRegisteredDataSources={};this.registerLayoutDataSource=function(widgetID,dataSource){if(widgetID){layoutRegisteredDataSources[widgetID]=dataSource}return layoutRegisteredDataSources};this.updateLayouts=function(widgetID,dataSource){if(!dataSource){dataSource=layoutRegisteredDataSources[widgetID]}return this.getWidgetLayouts(widgetID).then(function(layouts){return layouts}).then(function(layouts){return this.addIsDefaultLayoutToLayouts(layouts,widgetID).then(function(layouts){dataSource.data=layouts})}.bind(this))};this.addIsDefaultLayoutToLayouts=function(layouts,widgetID){return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){var defaultLayout=defaultPreferences.DEFAULT_LAYOUT;return layouts.map(addIsDefaultLayoutToLayout.bind(this,defaultLayout))})};function addIsDefaultLayoutToLayout(defaultLayout,layout){layout.DEFAULT_LAYOUT=layout.ID===defaultLayout;return layout}this.getWidgetLayouts=function(widgetID){return ZhPreferences.getWidgetPreferences("layout",widgetID)};this.getLayouts=function(query){return ZhPreferences.getPreferences("layout",query)};this.saveLayouts=function(layouts){var cloneLayouts=layouts.map(function(layout){return ZhUtil.clearRow(layout)});return ZhPreferences.savePreferences("layout",cloneLayouts).then(function(response){cloneLayouts.forEach(function(clone,index){layouts[index].ID=clone.ID;layouts[index].__is_new=clone.__is_new;layouts[index].changes=clone.changed});return ZHPromise.when(response)})};this.deleteLayouts=function(layouts){var cloneLayouts=layouts.map(function(layout){return ZhUtil.clearRow(layout)});return ZhPreferences.deletePreferences("layout",cloneLayouts)};function triggerBeforeLayoutApplyEvent(widget,layout){if(widget.beforeLayoutApply){widget.beforeLayoutApply({layout:layout})}}function triggerAfterLayoutApplyEvent(widget,layout){if(widget.afterLayoutApply){widget.afterLayoutApply({layout:layout})}}var appliedPreference={ID:"ORIGINAL"};this.applyOriginalLayout=function(widget){triggerBeforeLayoutApplyEvent(widget,originalLayout);containerMerger.restore(widget,"ORIGINAL");widget.redraw();widget.setAppliedLayout(originalLayout);appliedPreference=originalLayout;triggerAfterLayoutApplyEvent(widget,originalLayout);return ZHPromise.when()};this.applyLayout=function(widget,preference){triggerBeforeLayoutApplyEvent(widget,preference);if(widget.backupPDFOrientation===undefined){widget.backupPDFOrientation=widget.reportOrientation||"PORTRAIT"}widget.reportOrientation=preference.PDF_REPORT_ORIENTATION;var layoutChanges=prepareLayout(JSON.parse(preference.LAYOUT_CHANGES));containerMerger.merge(templateManager.container,layoutChanges,appliedPreference.ID);widget.redraw(true);widget.setAppliedLayout(preference);appliedPreference=preference;triggerAfterLayoutApplyEvent(widget,preference);return ZHPromise.when()};var propertyToBeVerified={container:["widgets"],widget:["widgets","fields"]};function prepareLayout(objToBeMerged,baseObject){baseObject=baseObject||templateManager.container;var childProperties=[];if(baseObject instanceof Container){childProperties=propertyToBeVerified.container}else if(baseObject instanceof Widget){childProperties=propertyToBeVerified.widget}childProperties.forEach(function prepareChildLayout(propertyName){var childBaseObj=baseObject[propertyName];var childObjToBeMerged=objToBeMerged[propertyName];if(childObjToBeMerged){if(childBaseObj&&childBaseObj.length){var existentIds=childBaseObj.map(function returnObjId(obj){return obj.id});objToBeMerged[propertyName]=childObjToBeMerged.filter(function filterChilds(obj){var objIndex=existentIds.indexOf(obj.id);if(objIndex!=-1){prepareLayout(obj,childBaseObj[objIndex])}return~objIndex})}else{objToBeMerged[propertyName]=[]}}});return objToBeMerged}this.downloadPreferenceFile=function(layout){var fileName=layout.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/gi,"").replace(/(?:^\w|[A-Z]|\b\w|\s+)/g,function(match,index){if(+match===0)return"";return index==0?match.toLowerCase():match.toUpperCase()})+".layout";var layoutToExport={ID:layout.ID,LABEL_TO_PREFERENCE:layout.LABEL_TO_PREFERENCE,WIDGET_ID:layout.WIDGET_ID,PDF_REPORT_ORIENTATION:layout.PDF_REPORT_ORIENTATION,LAYOUT_CHANGES:JSON.parse(layout.LAYOUT_CHANGES),LAST_MODIFIED_DATE:layout.LAST_MODIFIED_DATE};return ZhPreferences.downloadPreferenceFile(layoutToExport,fileName)};this.validateLayoutImport=function(layout,configWidget,layoutDataSource){layoutDataSource=layoutDataSource||configWidget.dataSource.data;if(configWidget.listWidget.id!=layout.WIDGET_ID){MessageService.showMessageByCode("zh-layout-preferences#import_error_widget_id");return false}var layoutsWithSameId=layoutDataSource.filter(function(row){return row.ID==layout.ID});if(layoutsWithSameId.length>0){MessageService.showMessageByCode("zh-layout-preferences#import_error_id_violation");return false}var layoutsWithSameLabel=layoutDataSource.filter(function(row){return row.LABEL_TO_PREFERENCE==layout.LABEL_TO_PREFERENCE});if(layoutsWithSameLabel.length>0){MessageService.showMessageByCode("zh-layout-preferences#import_error_name_violation");return false}return true}}Configuration(function(ContextRegister){ContextRegister.register("ZhLayoutPreferences",ZhLayoutPreferences)});function ZhLayoutWidgetsBuilder(restEngine,templateManager,metaDataFactory){function getWidgetJSON(widgetName){var widgetPath="zh-layout-preferences#json/widgets/"+widgetName+".json";return restEngine.requestMetaData(widgetPath).then(function(response){return response.data})}this.factoryWidget=function(widgetName){return getWidgetJSON(widgetName).then(function(widget){return metaDataFactory.widgetFactory(widget,templateManager.container)})};this.factoryLayoutConfigurationWidget=function(){return this.factoryWidget("config")};this.factoryCreateLayoutWidget=function(){return this.factoryWidget("createLayout")};this.factoryImportLayoutWidget=function(){return this.factoryWidget("importLayout")};this.factoryBuildLayoutOptionsWidget=function(){return this.factoryWidget("createLayoutOptions")};this.factoryBuildLayoutFieldOptionsWidget=function(){return this.factoryWidget("createLayoutFieldOptions")}}Configuration(function(ContextRegister){ContextRegister.register("ZhLayoutWidgetsBuilder",ZhLayoutWidgetsBuilder)});function ZhLayoutCreateLayoutOptionsController(MessageService,ZhLayoutPreferences,ScreenService,ZhLayoutWidgetsBuilder,metaDataFactory,ZHPromise){var fieldDefaultProperties={isVisible:true,showOnList:true,visibility:true,grouped:false,fixed:false};var createLayoutOptionsWidgetPromise=null;function removeConflictingProperties(field){var conflictingProperties=["selectWidget","inlineEditable"];conflictingProperties.forEach(function(property){delete field[property]})}function factoryFields(parentWidget,gridFields){return gridFields.filter(function(field){return!field.skipLayout}).map(function(field){var factoredField=metaDataFactory.fieldFactory(field,parentWidget);removeConflictingProperties(factoredField);return factoredField})}this.getCreateLayoutOptionsWidget=function(preference,configWidget){if(createLayoutOptionsWidgetPromise===null){createLayoutOptionsWidgetPromise=ZhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget()}return createLayoutOptionsWidgetPromise.then(function(layoutOptionsWidget){var label=preference.LABEL_TO_PREFERENCE;var listWidget=configWidget.listWidget;layoutOptionsWidget.preference=preference;layoutOptionsWidget.label=ScreenService.i18n("Layout");if(label){layoutOptionsWidget.label=layoutOptionsWidget.label+" - "+label}layoutOptionsWidget.layoutName=label;layoutOptionsWidget.reportTitle=listWidget.label;layoutOptionsWidget.reportOrientation=preference.PDF_REPORT_ORIENTATION;layoutOptionsWidget.dataSource.data=Util.clone(listWidget.dataSource.data.slice(0,2));layoutOptionsWidget.fields=factoryFields(layoutOptionsWidget,listWidget.fields);var layoutChanges=JSON.parse(preference.LAYOUT_CHANGES);var widgetDifferences=getEditingWidgetPath(layoutChanges.widgets,listWidget);layoutOptionsWidget.fields=layoutOptionsWidget.fields.map(function(field,index){delete field.__isSelected;delete field.__leftActions;delete field.__notSelectable;delete field.__rightActions;if(!Util.isEmptyOrBlank(widgetDifferences.fields)){var fieldPreference=filterObject(widgetDifferences.fields,field.id,"id");if(fieldPreference.obj&&Object.keys(fieldPreference.obj).length){return Util.extend(field,fieldPreference.obj)}}return Util.extend(fieldDefaultProperties,field,true)});return layoutOptionsWidget})};this.saveAndApplyLayout=function(createLayoutOptionsWidget){if(createLayoutOptionsWidget.gridConfigWidget.isValid()){var configWidget=createLayoutOptionsWidget.createNewLayoutWidget.configWidget;var listWidget=configWidget.listWidget;var fields=getFieldEditingWidgetAsDataSource(createLayoutOptionsWidget);var layout=createLayoutOptionsWidget.preference;var layoutChanges=JSON.parse(layout.LAYOUT_CHANGES);var editingWidget=getEditingWidgetPath(layoutChanges.widgets,listWidget);editingWidget.fields=fields;layout.PDF_REPORT_ORIENTATION=createLayoutOptionsWidget.reportOrientation;layout.LABEL_TO_PREFERENCE=createLayoutOptionsWidget.layoutName;layout.LAYOUT_CHANGES=JSON.stringify(layoutChanges);return ZhLayoutPreferences.saveLayouts([layout]).then(function(layouts){return ZhLayoutPreferences.applyLayout(listWidget,layouts[0])}).then(function(){ScreenService.closePopup(true);ZhLayoutPreferences.updateLayouts(listWidget.id,configWidget.dataSource);listWidget.activate()}).catch(function(error){MessageService.showMessageByCode("zh-layout-preferences#save_error",{ERROR:error.message})})}else{return ZHPromise.reject("Invalid params")}};this.updateFieldVisibility=function(field){field.isVisible=true;field.showOnList=field.isVisibleOnReport=field.visibility};function getFieldEditingWidgetAsDataSource(listWidget){return listWidget.fields.map(function(field){var data={id:field.id,name:field.name,label:field.label,labelOnReport:field.labelOnReport||null,expression:field.expression||null,align:field.align||null,isVisible:field.isVisible,showOnList:field.showOnList,minWidth:field.gridColumnSize||null,maxWidth:field.gridColumnSize||null,gridColumnSize:field.gridColumnSize||null,reportColumnSize:field.reportColumnSize||null,isVisibleOnReport:field.isVisibleOnReport,sequenceInList:field.sequenceInList,visibility:field.isVisible&&field.showOnList,grouped:field.grouped||false,groupOpened:field.groupOpened,fixed:field.fixed||false,skipLayout:field.skipLayout||false};return data}).filter(function(field){return!field.skipLayout})}function getEditingWidgetPath(widgetsArray,listWidget){var foundWidget=getWidgetObjectByID(widgetsArray,listWidget.id);if(!foundWidget){widgetsArray.some(function(eachWidget){foundWidget=getEditingWidgetPath(eachWidget.widgets,listWidget);return foundWidget})}return foundWidget}function filterObject(obj,value,key){var filtered=obj.filter(function(preference){return preference[key]==value});var toReturn={};toReturn.index=obj.indexOf(filtered[0]);toReturn.obj=filtered[0];return toReturn}function getWidgetObjectByID(widgets,widgetID){return widgets.filter(function(eachWidgetObject){return eachWidgetObject.id==widgetID}).shift()}}Configuration(function(ContextRegister){ContextRegister.register("ZhLayoutCreateLayoutOptionsController",ZhLayoutCreateLayoutOptionsController)});function ZhLayoutImportLayoutController(MessageService,ZhLayoutPreferences,ZhPreferences,ZhLayoutWidgetsBuilder,ZHPromise,ScreenService){var importLayoutWidgetPromise=null;this.getImportLayoutWidget=function(){if(importLayoutWidgetPromise===null){importLayoutWidgetPromise=ZhLayoutWidgetsBuilder.factoryImportLayoutWidget()}return importLayoutWidgetPromise};this.importPreference=function(importLayoutWidget){if(!importLayoutWidget.isValid()){return ZHPromise.reject({err:"WIDGET_INVALID"})}var layoutFileData=importLayoutWidget.fields[0].value()[0].b64File;layoutFileData=layoutFileData.split("base64,")[1];layoutFileData=window.atob(layoutFileData);layoutFileData=decodeURIComponent(escape(layoutFileData));var layout=JSON.parse(layoutFileData);var configWidget=importLayoutWidget.configWidget;var listWidget=configWidget.listWidget;if(!ZhLayoutPreferences.validateLayoutImport(layout,configWidget)){return ZHPromise.reject({err:"LAYOUT_FILE_INVALID"})}layout.__is_new=true;layout.__generate_id=false;layout.LAYOUT_CHANGES=JSON.stringify(layout.LAYOUT_CHANGES);return ZhPreferences.getUserID().then(function(userID){layout.USER_ID=userID;return ZhLayoutPreferences.saveLayouts([layout]).then(function(){ScreenService.closePopup();ZhLayoutPreferences.updateLayouts(listWidget.id,configWidget.dataSource);configWidget.refreshWidget()}).catch(function(error){MessageService.showMessageByCode("zh-layout-preferences#save_error",{ERROR:error.message})})})}}Configuration(function(ContextRegister){ContextRegister.register("ZhLayoutImportLayoutController",ZhLayoutImportLayoutController)});function ZhLayoutConfigController(ZhPreferences,ZhLayoutPreferences,ZhLayoutWidgetsBuilder,ScreenService,MessageService,ZHPromise,$timeout,ZhLayoutCreateLayoutOptionsController,ZhLayoutImportLayoutController){var configWidgetPromise=null;this.getConfigWidget=function(listWidget,dataSource){if(configWidgetPromise===null){configWidgetPromise=ZhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget()}var userID=ZhPreferences.getUserID();return ZHPromise.all([userID,configWidgetPromise]).then(function(args){var configWidget=args[1];configWidget.listWidget=listWidget;configWidget.dataSourceFilter=[{name:"WIDGET_ID",operator:"=",value:listWidget.id}];dataSource.data.forEach(function(row){row.APPLIED=row.ID==(listWidget.appliedLayout?listWidget.appliedLayout.ID:"ORIGINAL")});configWidget.dataSource=dataSource;return configWidget})};this.openCreateLayoutWidget=function(configWidget){var listWidget=configWidget.listWidget;var pdfReportOrientation=listWidget.reportOrientation||"PORTRAIT";return ZhPreferences.getUserID().then(function(userID){return{__is_new:true,ID:null,WIDGET_ID:listWidget.id,USER_ID:userID,LAYOUT_CHANGES:JSON.stringify(createDiffDataStructure(listWidget.container,listWidget)),PDF_REPORT_ORIENTATION:pdfReportOrientation,LAST_MODIFIED_DATE:null}}).then(function(preference){return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference,configWidget).then(function(createLayoutOptionsWidget){createLayoutOptionsWidget.createNewLayoutWidget={configWidget:configWidget};$timeout(function(){ScreenService.openPopup(createLayoutOptionsWidget)})})})};function createDiffDataStructure(container,widget){var containerDiff={};createWidgetsDataStructure(widget.id,container.widgets,containerDiff);return containerDiff}function createWidgetsDataStructure(widgetID,widgets,containerDiff){containerDiff.widgets=[];createWidgetsObjects(containerDiff.widgets,widgets,widgetID);if(Util.isArray(widgets)&&widgets.length>0){widgets.forEach(function(eachWidget){createWidgetsDataStructure(widgetID,eachWidget.widgets,getWidgetObjectByID(containerDiff.widgets,eachWidget.id))})}}function createWidgetsObjects(widgetsDiff,widgetsArray,widgetID){if(Util.isArray(widgetsArray)&&widgetsArray.length>0){widgetsArray.forEach(function(eachWidget){var properties={id:eachWidget.id};if(eachWidget.id==widgetID){properties.fields=[]}widgetsDiff.push(properties)})}}function getWidgetObjectByID(widgets,widgetId){return widgets.filter(function(eachWidgetObject){return eachWidgetObject.id==widgetId}).shift()}this.editLayout=function(configWidget,layout){return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(layout,configWidget).then(function(createLayoutOptionsWidget){createLayoutOptionsWidget.createNewLayoutWidget={configWidget:configWidget};$timeout(function(){ScreenService.openPopup(createLayoutOptionsWidget)})})};this.deleteLayouts=function(configWidget){var layoutsToDelete=configWidget.getCheckedRows();var defaultLayout=layoutsToDelete.filter(function(layout){return layout.DEFAULT_LAYOUT});if(defaultLayout.length){MessageService.showMessageByCode("zh-layout-preferences#delete_error_default");return}return MessageService.confirmMessageByCode("zh-layout-preferences#delete",{NR_OF_LAYOUTS:layoutsToDelete.length}).then(function(){return ZhLayoutPreferences.deleteLayouts(layoutsToDelete).then(function(){configWidget.dataSource.clearCheckedRows();return ZhLayoutPreferences.updateLayouts(configWidget.listWidget.id,configWidget.dataSource)}).catch(function(error){return MessageService.getMessageByCode("zh-layout-preferences#"+error.message).then(function(errorMessage){return MessageService.showMessageByCode("zh-layout-preferences#delete_error",{ERROR:ScreenService.i18n(errorMessage)})})})})};this.updateDefaultLayout=function(configWidget,layout){return ZhPreferences.changeDefaultPreferences(configWidget,"DEFAULT_LAYOUT",layout)};this.updateAppliedLayout=function(configWidget,layout){var listWidget=configWidget.listWidget;return ZhLayoutPreferences.applyLayout(listWidget,layout).then(function(){configWidget.dataSource.data.forEach(function(row){row.APPLIED=row.ID==layout.ID})})};this.downloadPreferenceFile=function(layout){return ZhLayoutPreferences.downloadPreferenceFile(layout)};this.importPreferenceFile=function(configWidget){return ZhLayoutImportLayoutController.getImportLayoutWidget().then(function(importLayoutWidget){importLayoutWidget.newRow();importLayoutWidget.configWidget=configWidget;return ScreenService.openPopup(importLayoutWidget)})}}Configuration(function(ContextRegister){ContextRegister.register("ZhLayoutConfigController",ZhLayoutConfigController)});function ZhLayoutFloatingCardController($scope,ApplicationContext,Query,ScreenService,metaDataFactory){var widget=$scope.widget;$scope.appliedPreference=null;var ZhLayoutPreferences=ApplicationContext.ZhLayoutPreferences;var ZhPreferences=ApplicationContext.ZhPreferences;var ZhLayoutConfigController=ApplicationContext.ZhLayoutConfigController;var fieldDefaultProperties={isVisible:true,showOnList:true,visibility:true,grouped:false,fixed:false};var dataSource=metaDataFactory.factoryDataSource({name:"zh-layout-preferences#/layout",localStorage:false,lazyLoad:true,rest:true,data:[]});dataSource.afterLoadDataSource(function(){return ZhLayoutPreferences.addIsDefaultLayoutToLayouts(dataSource.data,widget.id).then(function(layouts){dataSource.data=layouts})});$scope.customizationCard={label:"Layouts",type:"layout",openLayoutConfiguration:openLayoutConfiguration,refreshLayouts:refreshLayouts,applyOriginalLayout:applyOriginalLayout,applyLayout:applyPreference,dataSource:dataSource};ZhLayoutPreferences.registerLayoutDataSource(widget.id,$scope.customizationCard.dataSource);widget.setAppliedLayout=setAppliedLayout;$scope.$on("$destroy",function(){delete dataSource.afterLoadDataSourcelistener});function setAppliedLayout(layout){$scope.appliedPreference=layout;widget.appliedLayout=layout}function openLayoutConfiguration(){var dataSource=$scope.customizationCard.dataSource;return ZhLayoutConfigController.getConfigWidget(widget,dataSource).then(function(configWidget){return ScreenService.openPopup(configWidget)})}function refreshLayouts(){return ZhPreferences.getUserID().then(function(){var query=Query.build().where("WIDGET_ID").equals(widget.id);return $scope.customizationCard.dataSource.load(query.where())})}function applyOriginalLayout(){return ZhLayoutPreferences.applyOriginalLayout(widget)}function applyPreference(layout){return ZhLayoutPreferences.applyLayout(widget,layout)}function getDefaultLayout(){return dataSource.data.filter(function(layout){return layout.DEFAULT_LAYOUT}).shift()}function applyDefaultLayout(){var layout=getDefaultLayout();if(layout){applyPreference(layout)}else{applyOriginalLayout()}}(function initLayoutPreferences(){if(ZhPreferences.validateWidget(widget)){ZhLayoutPreferences.updateLayouts(widget.id,dataSource).then(function(){if(Util.isUndefinedOrNull(widget.appliedLayout)){applyDefaultLayout()}else{applyPreference(widget.appliedLayout)}})}})()}(function(){window.templateUrls=window.templateUrls||[];window.metadataUrls=window.metadataUrls||[];window.serviceUrls=window.serviceUrls||[];var PACKAGE_NAME="zh-layout-preferences";var hasTemplateUrl=Boolean(window.templateUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasMetadataUrl=Boolean(window.metadataUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasServiceUrl=Boolean(window.serviceUrls.find(function(value){return value.package===PACKAGE_NAME}));if(!hasTemplateUrl){window.templateUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-layout-preferences/dist/templates/"})}if(!hasMetadataUrl){window.metadataUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-layout-preferences/dist/assets/"})}if(!hasServiceUrl){window.serviceUrls.push({package:PACKAGE_NAME,baseUrl:"../backend/service/index.php/zh-preferences"})}})();

// FILE: bower_components/zh-view-preferences/dist/view-preferences.min.js
/*! 14-01-2019 */

function ZhViewPreferences(ZhPreferences,ZhFilterPreferences,ZhLayoutPreferences,templateManager,eventAggregator,ZHPromise,Query,eventEngine,ZhUtil,MessageService){var originalView={ID:"ORIGINAL",LABEL_TO_PREFERENCE:"Original",FILTER_ID:"ORIGINAL",LAYOUT_ID:"ORIGINAL"};function shouldShowViewAction(widget){return Util.get(widget,["floatingControl","customizationAction","view"])!==false}eventAggregator.subscribe("beforeProjectStartup",function(){ZhPreferences.registerRepository("zh-view-preferences#view");templateManager.project.customFloatingActions.push({name:"View Action",label:"View",type:"view",order:30,icon:"view",template:"zh-view-preferences#floating-view-card.html",expandable:true,isVisible:shouldShowViewAction})});eventEngine.registerEvent("ViewBeforeApply","beforeViewApply");eventEngine.registerEvent("ViewAfterApply","afterViewApply");this.updateViews=function(widgetID,dataSource){return this.getWidgetViews(widgetID).then(function(views){return views}).then(function(views){return this.addIsDefaultViewToViews(views,widgetID).then(function(views){dataSource.data=views})}.bind(this))};this.addIsDefaultViewToViews=function(views,widgetID){return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){var defaultView=defaultPreferences.DEFAULT_VIEW;return views.map(addIsDefaultViewToView.bind(this,defaultView))})};function addIsDefaultViewToView(defaultView,view){view.DEFAULT_VIEW=view.ID===defaultView;return view}this.getWidgetViews=function(widgetID){return ZhPreferences.getWidgetPreferences("view",widgetID)};this.saveViews=function(views){var cloneViews=views.map(function(view){return ZhUtil.clearRow(view)});return ZhPreferences.savePreferences("view",cloneViews).then(function(response){cloneViews.forEach(function(clone,index){views[index].ID=clone.ID;views[index].__is_new=clone.__is_new;views[index].changes=clone.changed});return ZHPromise.when(response)})};this.deleteViews=function(views){var cloneViews=views.map(function(view){return ZhUtil.clearRow(view)});return ZhPreferences.deletePreferences("view",cloneViews)};function triggerBeforeViewApplyEvent(widget,view){if(widget.beforeViewApply){widget.beforeViewApply({view:view})}}function triggerAfterViewApplyEvent(widget,view){if(widget.afterViewApply){widget.afterViewApply({view:view})}}this.applyOriginalView=function(widget){var filter=ZhFilterPreferences.applyOriginalFilter(widget);var layout=ZhLayoutPreferences.applyOriginalLayout(widget);triggerBeforeViewApplyEvent(widget,originalView);return ZHPromise.all([filter,layout]).then(function(){widget.setAppliedView(originalView);triggerAfterViewApplyEvent(widget,originalView)})};this.applyView=function(widget,preference,reloadWidget){reloadWidget=reloadWidget!==false;var filterQuery=Query.build().where("ID").equals(preference.FILTER_ID).where("WIDGET_ID").equals(widget.id);var layoutQuery=Query.build().where("ID").equals(preference.LAYOUT_ID).where("WIDGET_ID").equals(widget.id);var filter=ZhFilterPreferences.getFilters(filterQuery);var layout=ZhLayoutPreferences.getLayouts(layoutQuery);triggerBeforeViewApplyEvent(widget,preference);return ZHPromise.all([filter,layout]).then(function(args){var filter=args[0].pop();var layout=args[1].pop();var filterStatus=ZhFilterPreferences.applyFilter(widget,filter,reloadWidget);var layoutStatus=ZhLayoutPreferences.applyLayout(widget,layout);return ZHPromise.all([filterStatus,layoutStatus])}).then(function(){widget.setAppliedView(preference);triggerAfterViewApplyEvent(widget,preference)})};this.downloadPreferenceFile=function(view){var fileName=view.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/gi,"").replace(/(?:^\w|[A-Z]|\b\w|\s+)/g,function(match,index){if(+match===0)return"";return index==0?match.toLowerCase():match.toUpperCase()})+".view";var viewToExport={ID:view.ID,LABEL_TO_PREFERENCE:view.LABEL_TO_PREFERENCE,WIDGET_ID:view.WIDGET_ID,FILTER_ID:view.FILTER_ID,LAYOUT_ID:view.LAYOUT_ID,LAST_MODIFIED_DATE:view.LAST_MODIFIED_DATE};return ZhPreferences.downloadPreferenceFile(viewToExport,fileName)};function getObjectFromBase64(b64){b64=b64.split("base64,")[1];b64=window.atob(b64);b64=decodeURIComponent(escape(b64));return JSON.parse(b64)}this.validateViewImport=function(view,importViewWidget,layouts,filters,viewDataSource){var configWidget=importViewWidget.configWidget;viewDataSource=viewDataSource||configWidget.dataSource.data;var layoutFileField=importViewWidget.getField("layoutFile");var filterFileField=importViewWidget.getField("filterFile");var layoutFileFieldGroup=importViewWidget.fieldGroups[1];var filterFileFieldGroup=importViewWidget.fieldGroups[2];if(configWidget.listWidget.id!=view.WIDGET_ID){MessageService.showMessageByCode("zh-view-preferences#import_error_widget_id");return false}var viewsWithSameId=viewDataSource.filter(function(row){return row.ID==view.ID});if(viewsWithSameId.length>0){MessageService.showMessageByCode("zh-view-preferences#import_error_id_violation");return false}var viewsWithSameLabel=viewDataSource.filter(function(row){return row.LABEL_TO_PREFERENCE==view.LABEL_TO_PREFERENCE});if(viewsWithSameLabel.length>0){MessageService.showMessageByCode("zh-view-preferences#import_error_name_violation");return false}var selectedLayout=layouts.filter(function(layout){return layout.ID==view.LAYOUT_ID});if(selectedLayout.length==0&&layoutFileField.value().length==0){MessageService.showMessageByCode("zh-view-preferences#import_error_layout_dependency");layoutFileFieldGroup.opened=true;return false}if(layoutFileField.value().length>0){var layout=getObjectFromBase64(layoutFileField.value()[0].b64File);if(layout.ID!=view.LAYOUT_ID){MessageService.showMessageByCode("zh-view-preferences#import_error_layout_dependency2");return false}}var selectedFilter=filters.filter(function(filter){return filter.ID==view.FILTER_ID});if(selectedFilter.length==0&&filterFileField.value().length==0){MessageService.showMessageByCode("zh-view-preferences#import_error_filter_dependency");filterFileFieldGroup.opened=true;return false}if(filterFileField.value().length>0){var filter=getObjectFromBase64(filterFileField.value()[0].b64File);if(filter.ID!=view.FILTER_ID){MessageService.showMessageByCode("zh-view-preferences#import_error_filter_dependency2");return false}}return true}}Configuration(function(ContextRegister){ContextRegister.register("ZhViewPreferences",ZhViewPreferences)});function ZhViewWidgetsBuilder(restEngine,templateManager,metaDataFactory){function getWidgetJSON(widgetName){var widgetPath="zh-view-preferences#json/widgets/"+widgetName+".json";return restEngine.requestMetaData(widgetPath).then(function(response){return response.data})}this.factoryWidget=function(widgetName){return getWidgetJSON(widgetName).then(function(widget){return metaDataFactory.widgetFactory(widget,templateManager.container)})};this.factoryViewConfigurationWidget=function(methods){return this.factoryWidget("config")};this.factoryCreateViewWidget=function(methods){return this.factoryWidget("create-view")};this.factoryImportViewWidget=function(){return this.factoryWidget("importView")}}Configuration(function(ContextRegister){ContextRegister.register("ZhViewWidgetsBuilder",ZhViewWidgetsBuilder)});function ZhViewCreateViewController(ZhViewPreferences,ZhFilterPreferences,ZhLayoutPreferences,ZhViewWidgetsBuilder,ScreenService,MessageService,ZHPromise,ZhUtil){var createViewWidgetPromise=null;this.getCreateViewWidget=function(configWidget){if(createViewWidgetPromise===null){createViewWidgetPromise=ZhViewWidgetsBuilder.factoryCreateViewWidget()}var listWidget=configWidget.listWidget;var filters=ZhFilterPreferences.getWidgetFilters(listWidget.id);var layouts=ZhLayoutPreferences.getWidgetLayouts(listWidget.id);return ZHPromise.all([createViewWidgetPromise,filters,layouts]).then(function(args){var createViewWidget=args[0];var filters=args[1];var layouts=args[2];createViewWidget.configWidget=configWidget;createViewWidget.getField("FILTER_ID").dataSource.data=filters;createViewWidget.getField("LAYOUT_ID").dataSource.data=layouts;return createViewWidget})};this.cancelCreateView=function(createViewWidget){var configWidget=createViewWidget.configWidget;Util.extend(configWidget.currentRow,configWidget.backupRow);return ScreenService.closePopup()};this.saveAndApplyView=function(createViewWidget){if(createViewWidget.isValid()){var view=ZhUtil.clearRow(createViewWidget.currentRow);var configWidget=createViewWidget.configWidget;var listWidget=configWidget.listWidget;return ZhViewPreferences.saveViews([view]).then(function(){return ZhViewPreferences.applyView(listWidget,view)}).then(function(){ZhViewPreferences.updateViews(listWidget.id,configWidget.dataSource);ScreenService.closePopup(true);return listWidget.activate()}).catch(function(error){return MessageService.getMessageByCode("zh-view-preferences#"+error.message).then(function(errorMessage){return MessageService.showMessageByCode("zh-view-preferences#save_error",{ERROR:ScreenService.i18n(errorMessage)})})})}else{return ZHPromise.when()}}}Configuration(function(ContextRegister){ContextRegister.register("ZhViewCreateViewController",ZhViewCreateViewController)});function ZhViewImportViewController(MessageService,ZhViewPreferences,ZhPreferences,ZhFilterPreferences,ZhLayoutPreferences,ZhViewWidgetsBuilder,ZHPromise,ScreenService,$timeout){var importViewWidgetPromise=null;this.getImportViewWidget=function(){if(importViewWidgetPromise===null){importViewWidgetPromise=ZhViewWidgetsBuilder.factoryImportViewWidget()}return importViewWidgetPromise};function getObjectFromBase64(b64){b64=b64.split("base64,")[1];b64=window.atob(b64);b64=decodeURIComponent(escape(b64));return JSON.parse(b64)}function buildImportPromises(importViewWidget,userID,layouts,filters){var layoutFileField=importViewWidget.getField("layoutFile");var filterFileField=importViewWidget.getField("filterFile");var promises=[];if(layoutFileField.value().length>0){var layout=getObjectFromBase64(layoutFileField.value()[0].b64File);if(!ZhLayoutPreferences.validateLayoutImport(layout,importViewWidget.configWidget,layouts)){return[ZHPromise.reject()]}promises.push(function(){layout.__is_new=true;layout.__generate_id=false;layout.USER_ID=userID;layout.LAYOUT_CHANGES=JSON.stringify(layout.LAYOUT_CHANGES);return ZhLayoutPreferences.saveLayouts([layout]).then(function(){ZhLayoutPreferences.updateLayouts(importViewWidget.configWidget.listWidget.id)}).catch(function(error){MessageService.showMessageByCode("zh-layout-preferences#save_error",{ERROR:error.message});return ZHPromise.reject()})}())}if(filterFileField.value().length>0){var filter=getObjectFromBase64(filterFileField.value()[0].b64File);if(!ZhFilterPreferences.validateFilterImport(filter,importViewWidget.configWidget,filters)){return[ZHPromise.reject()]}promises.push(function(){filter.__is_new=true;filter.__generate_id=false;filter.USER_ID=userID;filter.FILTER_ROW=JSON.stringify(filter.FILTER_ROW);return ZhFilterPreferences.saveFilters([filter]).then(function(){ZhFilterPreferences.updateFilters(importViewWidget.configWidget.listWidget.id)}).catch(function(error){MessageService.showMessageByCode("zh-filter-preferences#save_error",{ERROR:error.message});return ZHPromise.reject()})}())}return promises}this.importPreference=function(importViewWidget){if(!importViewWidget.isValid()){return ZHPromise.reject({err:"WIDGET_INVALID"})}var viewFileField=importViewWidget.getField("viewFile");var view=getObjectFromBase64(viewFileField.value()[0].b64File);var configWidget=importViewWidget.configWidget;var listWidget=configWidget.listWidget;var filters=ZhFilterPreferences.getWidgetFilters(listWidget.id);var layouts=ZhLayoutPreferences.getWidgetLayouts(listWidget.id);return ZHPromise.all([filters,layouts]).then(function(args){filters=args[0];layouts=args[1];if(!ZhViewPreferences.validateViewImport(view,importViewWidget,layouts,filters)){return ZHPromise.reject({err:"VIEW_FILE_INVALID"})}return ZhPreferences.getUserID().then(function(userID){view.USER_ID=userID;view.__is_new=true;view.__generate_id=false;var importPromises=buildImportPromises(importViewWidget,userID,layouts,filters);return ZHPromise.all(importPromises).then(function(){return ZhViewPreferences.saveViews([view]).then(function(){ScreenService.closePopup();ZhViewPreferences.updateViews(listWidget.id,configWidget.dataSource);configWidget.refreshWidget()}).catch(function(error){MessageService.showMessageByCode("zh-view-preferences#save_error",{ERROR:error.message})})})})})}}Configuration(function(ContextRegister){ContextRegister.register("ZhViewImportViewController",ZhViewImportViewController)});function ZhViewConfigController(ZhPreferences,ZhViewPreferences,ZhViewWidgetsBuilder,ScreenService,MessageService,ZHPromise,ZhViewCreateViewController,ZhViewImportViewController){var configWidgetPromise=null;this.getConfigWidget=function(listWidget,dataSource){if(configWidgetPromise===null){configWidgetPromise=ZhViewWidgetsBuilder.factoryViewConfigurationWidget()}var userID=ZhPreferences.getUserID();return ZHPromise.all([userID,configWidgetPromise]).then(function(args){var userID=args[0];var configWidget=args[1];configWidget.listWidget=listWidget;configWidget.dataSourceFilter=[{name:"WIDGET_ID",operator:"=",value:listWidget.id}];dataSource.data.forEach(function(row){row.APPLIED=row.ID==(listWidget.appliedView?listWidget.appliedView.ID:"ORIGINAL")});configWidget.dataSource=dataSource;return configWidget})};this.openCreateViewWidget=function(configWidget){var createViewWidget=ZhViewCreateViewController.getCreateViewWidget(configWidget);var userID=ZhPreferences.getUserID();var listWidget=configWidget.listWidget;return ZHPromise.all([createViewWidget,userID]).then(function(args){var createViewWidget=args[0];var userID=args[1];createViewWidget.newRow();createViewWidget.currentRow.ID=null;createViewWidget.currentRow.USER_ID=userID;createViewWidget.currentRow.WIDGET_ID=listWidget.id;createViewWidget.currentRow.LAST_MODIFIED_DATE=null;createViewWidget.label=ScreenService.i18n("View");return ScreenService.openPopup(createViewWidget)})};this.editView=function(configWidget,view){return ZhViewCreateViewController.getCreateViewWidget(configWidget).then(function(createViewWidget){configWidget.backupRow=Util.clone(view,["__rightActions","__leftActions"]);createViewWidget.currentRow=view;createViewWidget.label=ScreenService.i18n("View")+" - "+view.LABEL_TO_PREFERENCE;return ScreenService.openPopup(createViewWidget)})};this.deleteViews=function(configWidget){var viewsToDelete=configWidget.getCheckedRows();var defaultView=viewsToDelete.filter(function(view){return view.DEFAULT_VIEW});if(defaultView.length){MessageService.showMessageByCode("zh-view-preferences#delete_error_default");return}return MessageService.confirmMessageByCode("zh-view-preferences#delete",{NR_OF_VIEWS:viewsToDelete.length}).then(function(){return ZhViewPreferences.deleteViews(viewsToDelete).then(function(){configWidget.dataSource.clearCheckedRows();ZhViewPreferences.updateViews(configWidget.listWidget.id,configWidget.dataSource)}).catch(function(error){MessageService.showMessageByCode("zh-view-preferences#delete_error",{ERROR:error.message})})})};this.updateDefaultView=function(configWidget,view){return ZhPreferences.changeDefaultPreferences(configWidget,"DEFAULT_VIEW",view)};this.updateAppliedView=function(configWidget,view){var listWidget=configWidget.listWidget;return ZhViewPreferences.applyView(listWidget,view).then(function(){configWidget.dataSource.data.forEach(function(row){row.APPLIED=row.ID==view.ID})})};this.downloadPreferenceFile=function(view){return ZhViewPreferences.downloadPreferenceFile(view)};this.importPreferenceFile=function(configWidget){return ZhViewImportViewController.getImportViewWidget().then(function(importViewWidget){importViewWidget.newRow();importViewWidget.configWidget=configWidget;return ScreenService.openPopup(importViewWidget)})}}Configuration(function(ContextRegister){ContextRegister.register("ZhViewConfigController",ZhViewConfigController)});function ZhViewFloatingCardController($scope,ApplicationContext,ScreenService,eventEngine,Query,metaDataFactory){var widget=$scope.widget;$scope.appliedPreference=null;var ZhViewConfigController=ApplicationContext.ZhViewConfigController;var ZhViewPreferences=ApplicationContext.ZhViewPreferences;var ZhPreferences=ApplicationContext.ZhPreferences;var dataSource=metaDataFactory.factoryDataSource({name:"zh-view-preferences#/view",localStorage:false,lazyLoad:true,rest:true,data:[]});dataSource.afterLoadDataSource(function(){return ZhViewPreferences.addIsDefaultViewToViews(dataSource.data,widget.id).then(function(views){dataSource.data=views})});$scope.customizationCard={label:"Views",type:"view",openViewConfiguration:openViewConfiguration,refreshViews:refreshViews,applyOriginalView:applyOriginalView,applyView:applyPreference,dataSource:dataSource};var filterApplyEventIndex=eventEngine.bindEvent(widget,"FilterAfterApply",function(){$scope.appliedPreference=null;widget.appliedView=null;if(widget.reportCustomData){delete widget.reportCustomData.name}});var layoutApplyEventIndex=eventEngine.bindEvent(widget,"LayoutAfterApply",function(){$scope.appliedPreference=null;widget.appliedView=null;if(widget.reportCustomData){delete widget.reportCustomData.name}});widget.setAppliedView=setAppliedView;$scope.$on("$destroy",function(){delete widget.setAppliedView;eventEngine.unbindEvent(widget,"FilterAfterApply",filterApplyEventIndex);eventEngine.unbindEvent(widget,"LayoutAfterApply",layoutApplyEventIndex);delete dataSource.afterLoadDataSourcelistener});function setAppliedView(view){$scope.appliedPreference=view;widget.appliedView=view;if(view.LABEL_TO_PREFERENCE){sessionStorage.setItem("view_"+widget.id,view.LABEL_TO_PREFERENCE)}widget.reportCustomData=widget.reportCustomData||{};if(view.ID!="ORIGINAL"){widget.reportCustomData.name=view.LABEL_TO_PREFERENCE}else{delete widget.reportCustomData.name}}function openViewConfiguration(){return ZhViewConfigController.getConfigWidget(widget,dataSource).then(function(configWidget){return ScreenService.openPopup(configWidget)})}function refreshViews(){return ZhPreferences.getUserID().then(function(userID){var query=Query.build().where("WIDGET_ID").equals(widget.id);return $scope.customizationCard.dataSource.load(query.where())})}function applyOriginalView(){return ZhViewPreferences.applyOriginalView(widget)}function applyPreference(preference,reloadWidget){return ZhViewPreferences.applyView(widget,preference,reloadWidget)}function getDefaultView(){return dataSource.data.filter(function(view){return view.DEFAULT_VIEW}).shift()}function applyDefaultView(){var view=getDefaultView();if(view){var filterIsLazyLoad=Util.getNestedProperty(widget,["dataSource","lazyLoad"]);applyPreference(view,!filterIsLazyLoad)}}(function initViewPreferences(){if(ZhPreferences.validateWidget(widget)){ZhViewPreferences.updateViews(widget.id,dataSource).then(function(){if(Util.isUndefinedOrNull(widget.appliedView)){applyDefaultView()}else{applyPreference(widget.appliedView)}})}})()}(function(){window.templateUrls=window.templateUrls||[];window.metadataUrls=window.metadataUrls||[];window.serviceUrls=window.serviceUrls||[];var PACKAGE_NAME="zh-view-preferences";var hasTemplateUrl=Boolean(window.templateUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasMetadataUrl=Boolean(window.metadataUrls.find(function(value){return value.package===PACKAGE_NAME}));var hasServiceUrl=Boolean(window.serviceUrls.find(function(value){return value.package===PACKAGE_NAME}));if(!hasTemplateUrl){window.templateUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-view-preferences/dist/templates/"})}if(!hasMetadataUrl){window.metadataUrls.push({package:PACKAGE_NAME,baseUrl:"bower_components/zh-view-preferences/dist/assets/"})}if(!hasServiceUrl){window.serviceUrls.push({package:PACKAGE_NAME,baseUrl:"../backend/service/index.php/zh-preferences"})}})();

// FILE: bower_components/zh-interval-component/dist/zh-interval-component.min.js
window.templateUrls=window.templateUrls||[];var packageName="zh-interval-component";var hasPackageRegistered=window.templateUrls.some(function(templateUrl){return templateUrl.package===packageName});if(!hasPackageRegistered){window.templateUrls.push({"package":packageName,baseUrl:"./bower_components/"+packageName+"/dist/templates/"})}function IntervalRegister(DecodeValueFieldService,validateEngine,ScreenService){var self=this;var DEFAULT_DISPLAY_FORMAT="YYYY-MM-DD";var DEFAULT_DATE_FORMAT="YYYY-MM-DD";self.decodeValueField=function(field,fieldRow){var displayValue="";var dateFormat=field.dateFormat||DEFAULT_DATE_FORMAT;var formattedDate=field.displayFormat||DEFAULT_DISPLAY_FORMAT;if(fieldRow){if(fieldRow.end||fieldRow.start||fieldRow.period){var beginDate=fieldRow.start?moment(fieldRow.start,dateFormat).format(formattedDate):formattedDate.replace(/[a-z]/gi,"_");var endDate=fieldRow.end?moment(fieldRow.end,dateFormat).format(formattedDate):formattedDate.replace(/[a-z]/gi,"_");if(!fieldRow.period){displayValue=ScreenService.i18n("From ")+beginDate+ScreenService.i18n(" to ")+endDate;return displayValue}else{var periodLabel=fieldRow.period.label;displayValue=ScreenService.i18n("From ")+beginDate+ScreenService.i18n(" to ")+endDate+" ("+periodLabel+") ";return displayValue}}}return ScreenService.i18n("From ")+formattedDate.replace(/[a-z]/gi,"_")+ScreenService.i18n(" to ")+formattedDate.replace(/[a-z]/gi,"_")};DecodeValueFieldService.register("interval.html",self.decodeValueField);function required(value,validationAttrs){return{valid:requiredStart(value,validationAttrs).valid&&requiredEnd(value,validationAttrs).valid,message:ScreenService.i18n(validationAttrs.message)||ScreenService.i18n("Required field")}}function requiredStart(value,validationAttrs){return buildRequiredMessage(value,validationAttrs,"start")}function requiredEnd(value,validationAttrs){return buildRequiredMessage(value,validationAttrs,"end")}function buildRequiredMessage(value,validationAttrs,property){return{valid:value&&Util.isObject(value)&&!Util.isEmptyOrBlank(value[property]),message:ScreenService.i18n(validationAttrs.message)||ScreenService.i18n("Required field")}}function completeInterval(value,validationAttrs){var localValue=Util.isObject(value)?value:{};return{valid:Util.isEmptyOrBlank(localValue.start)?Util.isEmptyOrBlank(localValue.end):!Util.isEmptyOrBlank(localValue.end),message:ScreenService.i18n(validationAttrs.message)||ScreenService.i18n("Incomplete interval")}}var registeredValidations=validateEngine.getMappedValidations();var validationsToRegister={requiredInterval:required,requiredIntervalStart:requiredStart,requiredIntervalEnd:requiredEnd,completeInterval:completeInterval};for(var validationName in validationsToRegister){if(!registeredValidations.hasOwnProperty(validationName)){validateEngine.registerValidation(validationName,validationsToRegister[validationName])}}}Configuration(IntervalRegister);function IntervalController($scope,$rootScope,ZhUtil,ScreenService,metaDataFactory,eventEngine,templateManager){var self=this;self.field=$scope.field;self.widget=$scope.widget;$scope.$root=$rootScope;var DEFAULT_DISPLAY_FORMAT="YYYY-MM-DD";var DEFAULT_DATE_FORMAT="YYYY-MM-DD";var DEFAULT_START_LABEL="Begin on";var DEFAULT_END_LABEL="End on";var INCLUSIVE_SUFFIX="inclusive";var sourceElement=self.field||self.widget;$scope.dateFormat=sourceElement.dateFormat||DEFAULT_DATE_FORMAT;$scope.displayFormat=sourceElement.displayFormat||DEFAULT_DISPLAY_FORMAT;$scope.startLabel=ScreenService.i18n(sourceElement.startLabel||DEFAULT_START_LABEL);$scope.endLabel=ScreenService.i18n(sourceElement.endLabel||DEFAULT_END_LABEL);var emptyDefault=$scope.displayFormat.replace(/[Y,M,D]/gi,"_");if(!eventEngine.hasEvent("IntervalOnInvalidInput")){eventEngine.registerEvent("IntervalOnInvalidInput","intervalOnInvalidInput")}function isStartField(fieldId){var startId="_START";return fieldId.lastIndexOf(startId)+startId.length==fieldId.length}function isEndField(fieldId){var endId="_END";return fieldId.lastIndexOf(endId)+endId.length==fieldId.length}$scope.dateCompleted=function(element){var elementId=element.attr("id");if(isStartField(elementId)){var endId="_END";var endElementId=elementId.replace(/_START([^_START]*)$/,endId+"$1");element.parent().find("#"+endElementId).focus()}};$scope.fillCurrentRow=function(element){var dateString=element.value;if(element.id===self.field.name+"_END"){$scope.fieldRow.end=getValueByDisplayValue(dateString)}else{$scope.fieldRow.start=getValueByDisplayValue(dateString)}if(moment($scope.fieldRow.start,$scope.dateFormat)._d.getTime()>moment($scope.fieldRow.end,$scope.dateFormat)._d.getTime()){$scope.fieldRow.end=null;element.value=emptyDefault;if($scope.field.intervalOnInvalidInput){$scope.field.intervalOnInvalidInput()}}};function getDisplayValueByValue(value){var inDisplayFormat=moment(value,$scope.displayFormat,true).isValid();var displayValue=inDisplayFormat?value:moment(value,$scope.dateFormat).format($scope.displayFormat);return displayValue}function getValueByDisplayValue(displayValue){var inDateFormat=moment(displayValue,$scope.dateFormat,true).isValid();var value=inDateFormat?displayValue:moment(displayValue,$scope.displayFormat).format($scope.dateFormat);return value}function buildIntervals(){var fieldIntervals=self.field.intervals;var finalIntervals=[];if(fieldIntervals===undefined){fieldIntervals=[{label:"Today",period:"today"},{label:"Yesterday",period:"yesterday"},{label:"Tomorrow",period:"tomorrow"},{label:"This Week",period:"this-week"},{label:"Last Week",period:"last-week"},{label:"Next Week",period:"next-week"},{label:"This Quarter",period:"this-quarter"},{label:"Last Quarter",period:"last-quarter"},{label:"Next Quarter",period:"next-quarter"},{label:"This Month",period:"this-month"},{label:"Last Month",period:"last-month"},{label:"Next Month",period:"next-month"},{label:"This Year",period:"this-year"},{label:"Last Year",period:"last-year"},{label:"Next Year",period:"next-year"}]}if(Util.isArray(fieldIntervals)){fieldIntervals.forEach(function(interval){try{var row;if(isCustomPeriod(interval.period)){row=ZhUtil.evaluate(interval.period);row.customized=true}else{row=buildObjectWithPeriodDetails(interval.period)}finalIntervals.push({_period_label:interval.label,_period:row,_period_name:interval.period})}catch(e){}})}return finalIntervals}function isCustomPeriod(periodName){return periodName[0]==="@"}function buildObjectWithPeriodDetails(word){var decoded=decodeWordExpression(word);var buildIntervalMethod=decoded.suffix===INCLUSIVE_SUFFIX?buildIntervalInclusive:buildInterval;return buildIntervalMethod(decoded.prefix,decoded.amount,decoded.unit)}function decodeWordExpression(word){var exceptionWords={today:"this-day",yesterday:"last-day",tomorrow:"next-day"};if(word in exceptionWords){word=exceptionWords[word]}var prefix,amount,unit,suffix;var splitedString=word.split("-");if(splitedString.length===2){prefix=splitedString[0];unit=splitedString[1]}else if(splitedString.length>2){prefix=splitedString[0];amount=splitedString[1];unit=splitedString[2];suffix=splitedString[3]}else{throw"Used period has an incorrect format."}return{prefix:prefix,amount:amount||1,unit:unit,suffix:suffix}}function buildInterval(prefix,amount,unit){var start=moment().startOf(unit),end=moment();switch(prefix){case"last":end.subtract(1,unit);start.subtract(amount,unit);break;case"next":start.add(1,unit);end.add(amount,unit);break}end.endOf(unit);return factoryInterval(start,end)}function buildIntervalInclusive(prefix,amount,unit){var start=moment().startOf(unit),end=moment();switch(prefix){case"last":start.subtract(amount-1,unit);break;case"next":end.add(amount-1,unit);break}end.endOf(unit);return factoryInterval(start,end)}function factoryInterval(start,end){return{start:start.format($scope.dateFormat),end:end.format($scope.dateFormat),customized:false}}function clearInputValues(field,id){var parentElement=angular.element(field.element[0]);var element=parentElement.find("#"+id)[0];element.value=emptyDefault}function clearInternalValues(field,id){if(!self.field.isReadOnly()){var start="";var end="";var formattedStart=emptyDefault;var formattedEnd=emptyDefault;var intervalField=self.field;var parentWidget=intervalField.widget;if(id){if(isStartField(id)){end=$scope.fieldRow.end;formattedEnd=$scope.fieldRow.formattedEnd}else if(isEndField(id)){start=$scope.fieldRow.start;formattedStart=$scope.fieldRow.formattedStart}clearInputValues(field,id)}delete parentWidget.currentRow[intervalField.name];$scope.fieldRow={start:start,end:end,formattedStart:formattedStart,formattedEnd:formattedEnd};intervalField.validate(intervalField.value());if(self.field.change){self.field.change()}}}function createIntervalWidget(){var actions=[{label:ScreenService.i18n("Cancel"),showAsAction:"back",closeIntervalWidget:self.closeIntervalWidget,isVisible:true,events:[{name:"ActionEvent",code:"args.owner.closeIntervalWidget()"}]},{label:ScreenService.i18n("OK"),showAsAction:"never",isVisible:true,applyIntervalToWidgetRow:self.applyIntervalToWidgetRow,closeIntervalWidget:self.closeIntervalWidget,events:[{name:"ActionEvent",code:"args.owner.closeIntervalWidget(); args.owner.applyIntervalToWidgetRow();"}]}];return metaDataFactory.widgetFactory({name:"Interval",label:self.field.label,template:"zh-interval-component#widget/interval.html",container:self.field.widget.container,isVisible:true,intervalField:self.field,startLabel:self.field.startLabel,endLabel:self.field.endLabel,dateFormat:$scope.dateFormat,actions:actions,parent:self.field,displayFormat:self.field.displayFormat||$scope.dateFormat,widgets:[{name:"Calendar",label:self.field.label,template:"widget/calendar.html",container:self.field.widget.container,actions:actions,dateFormat:$scope.dateFormat,displayFormat:self.field.displayFormat||$scope.dateFormat,isVisible:true,floatingControl:false,dataSource:{data:self.field.dataSource},showDescriptionOrder:false,events:[{name:"WidgetAfterMoveRow",code:"args.owner.parent.selectIntervalDate(args)"},{name:"WidgetOnEnter",code:"args.owner.parent.selectDatesBasedOnCurrentRow()"}]},{name:"List",label:self.field.label,template:"widget/list.html",container:self.field.widget.container,isVisible:true,actions:actions,showDescriptionOrder:false,floatingControl:false,fields:[{label:"Period",name:"_period_label",translate:true,showOnList:true}],dataSource:{data:buildIntervals()},events:[{name:"WidgetAfterMoveRow",code:"args.owner.parent.selectIntervalPeriod(args.owner)"}]}],events:[]},self.field.widget.container)}this.inputOnClick=function($event){$event.stopPropagation()};this.openIntervalWidget=function(){if(!self.field.isReadOnly()){self.field.intervalWidget=createIntervalWidget();self.batata=$scope.$root;applyIntervalWidgetInitialValues();resetIntervalValuesOnCurrentRow();createIntervalWidgetMethods();ScreenService.openPopup(self.field.intervalWidget).then(function(){if(self.field.intervalWidget===$scope.$root._currentPopup){$scope.$root._currentPopup=self.field.intervalWidget.widgets[0];$scope.$root._currentPopup.lastActiveWidget=self.field.intervalWidget.parent.widget}})}};function applyIntervalWidgetInitialValues(){var intervalWidget=self.field.intervalWidget;intervalWidget.isIntervalEmpty=true;intervalWidget.beingSelected="start";intervalWidget.setBeingSelected=setBeingSelected;intervalWidget.carouselWidget="calendar";intervalWidget.intervalData={};intervalWidget.intervalData.start=getInitialstartDate();intervalWidget.intervalData.end=getInitialEndDate();intervalWidget.widgets[0].currentDate=intervalWidget.intervalData.start;intervalWidget.formattedData={};intervalWidget.formattedData.start=getDisplayValueByValue(getInitialstartDate());intervalWidget.formattedData.end=getDisplayValueByValue(getInitialEndDate())}function getInitialstartDate(){return getInitialDate("start")}function getInitialEndDate(){return getInitialDate("end")}function getInitialDate(property){var field=self.field;var parentWidget=field.widget;var currentRowProperty=parentWidget.currentRow[field.name];return currentRowProperty?currentRowProperty[property]:moment().format($scope.dateFormat)}function resetIntervalValuesOnCurrentRow(){var intervalField=self.field;var parentWidget=intervalField.widget;if(!parentWidget.currentRow[intervalField.name]){parentWidget.currentRow[intervalField.name]={start:null,end:null,period:null}}}function setBeingSelected(value){self.field.intervalWidget.beingSelected=value}this.applyIntervalToWidgetRow=function(){var intervalField=self.field;var parentWidget=intervalField.widget;var intervalData=intervalField.intervalWidget.intervalData;var formattedData=intervalField.intervalWidget.formattedData;$scope.fieldRow={start:intervalData.start,end:intervalData.end,period:intervalData.period};parentWidget.currentRow[intervalField.name]=$scope.fieldRow;intervalField.validate(intervalField.value());if(intervalField.change){intervalField.change()}};this.closeIntervalWidget=function(){ScreenService.closePopup();setLastActivatedWidget(self.field.widget)};function createIntervalWidgetMethods(){self.field.intervalWidget.selectDatesBasedOnCurrentRow=function(){var field=self.field;var currentRow=field.widget.currentRow;var intervalWidget=field.intervalWidget;var intervalData=intervalWidget.intervalData;var formattedData=intervalWidget.formattedData;var startDate=currentRow[field.name]?currentRow[field.name].start:null;var endDate=currentRow[field.name]?currentRow[field.name].end:null;if(Util.isValidDate(startDate,$scope.dateFormat)&&Util.isValidDate(endDate,$scope.dateFormat)){intervalData.start=startDate;intervalData.end=endDate;formattedData.start=getDisplayValueByValue(startDate);formattedData.end=getDisplayValueByValue(endDate);selectIntervalOnCalendar()}else{intervalData.start=moment().format($scope.dateFormat);intervalData.end=moment().format($scope.dateFormat);formattedData.start=moment().format($scope.displayFormat);formattedData.end=moment().format($scope.displayFormat);selectIntervalOnCalendar()}};self.field.intervalWidget.selectIntervalDate=function(args){var intervalWidget=self.field.intervalWidget;var intervalData=intervalWidget.intervalData;var formattedData=intervalWidget.formattedData;var selectedDate=args.row;var momentSelectedDate=moment(args.row,$scope.dateFormat);var momentstartDate=moment(intervalData.start,$scope.dateFormat);var momentEndDate=moment(intervalData.end,$scope.dateFormat);if(intervalWidget.beingSelected!=="start"&&intervalWidget.beingSelected!=="end"){intervalWidget.beingSelected="start"}intervalData[intervalWidget.beingSelected]=selectedDate;formattedData[intervalWidget.beingSelected]=getDisplayValueByValue(selectedDate);var shouldToggleBeingSelected=true;if(intervalWidget.beingSelected==="start"&&momentSelectedDate.isAfter(momentEndDate)){intervalData.end=selectedDate;formattedData.end=getDisplayValueByValue(selectedDate)}else if(intervalWidget.beingSelected==="end"&&momentSelectedDate.isBefore(momentstartDate)){shouldToggleBeingSelected=false;intervalData.start=selectedDate;formattedData.start=getDisplayValueByValue(selectedDate)}if(shouldToggleBeingSelected){toggleBeingSelected()}clearOldInterval();selectIntervalOnCalendar();intervalWidget.isIntervalEmpty=false};self.field.intervalWidget.selectIntervalPeriod=function(listWidget){var intervalWidget=self.field.intervalWidget;var intervalData=intervalWidget.intervalData;intervalData.start=listWidget.currentRow._period.start;intervalData.end=listWidget.currentRow._period.end;intervalData.period={name:listWidget.currentRow._period_name,label:listWidget.currentRow._period_label,customized:listWidget.currentRow._period.customized};intervalWidget.formattedData.start=getDisplayValueByValue(intervalData.start);intervalWidget.formattedData.end=getDisplayValueByValue(intervalData.end);clearOldInterval();selectIntervalOnCalendar();ScreenService.closePopup();self.field.widget.activate();self.applyIntervalToWidgetRow()};self.field.intervalWidget.isCalendarCurrentWidget=function(){return self.field.intervalWidget.carouselWidget==="calendar"};self.field.intervalWidget.isPeriodSelectorCurrentWidget=function(){return self.field.intervalWidget.carouselWidget==="periodSelector"};self.field.intervalWidget.toggleCurrentWidget=function(){var intervalWidget=self.field.intervalWidget;if(intervalWidget.isPeriodSelectorCurrentWidget()){intervalWidget.carouselWidget="calendar";intervalWidget.beingSelected="start"}else{intervalWidget.carouselWidget="periodSelector";intervalWidget.beingSelected="none"}setLastActivatedWidget(self.field.widget);intervalWidget.nextWidget();intervalWidget.container.prepareActions()}}function setLastActivatedWidget(toBecomeActiveWidget){toBecomeActiveWidget.activate();toBecomeActiveWidget.lastActiveWidget=self.field.widget.parent}function toggleBeingSelected(){var intervalWidget=self.field.intervalWidget;intervalWidget.beingSelected=intervalWidget.beingSelected==="start"?"end":"start"}function clearOldInterval(){var calendarWidget=self.field.intervalWidget.widgets[0];calendarWidget.selectedDates=[]}function selectIntervalOnCalendar(){var intervalWidget=self.field.intervalWidget||self.widget;var calendarWidget=intervalWidget.widgets[0];var dateFormat=self.field.dateFormat;var startDate=intervalWidget.intervalData.start;var endDate=intervalWidget.intervalData.end;var momentstartDate=moment(startDate,dateFormat);var momentEndDate=moment(endDate,dateFormat);var momentCurrentDate=momentstartDate;while(momentCurrentDate.isBefore(momentEndDate)||momentCurrentDate.isSame(momentEndDate)){calendarWidget.setDateAsSelected(momentCurrentDate.format(dateFormat),dateFormat);momentCurrentDate.add(1,"day")}}this.getPeriodLabel=function(){var label="";var property=self.field.widget.currentRow[self.field.name];if(property&&property.period&&property.period.label){label="("+property.period.label+")"}return label};if(self.field){var field=self.field;var fieldName=field.name;var parentWidget=field.widget;field.value(self.field.value()||self.field.defaultValue);field.isRowSet=function(){var fieldName=this.name;var currentRow=this.widget.currentRow;var hasValue=currentRow[fieldName]&&currentRow[fieldName].formattedStart;var startSet=hasValue&&currentRow[fieldName].formattedStart.indexOf("_")===-1;var endSet=hasValue&&currentRow[fieldName].formattedEnd.indexOf("_")===-1;return startSet&&endSet};field.clearValue=clearInternalValues;field.getDatesByPeriodName=function(periodName,periodLabel){periodLabel=periodLabel||"";var date=isCustomPeriod(periodName)?ZhUtil.evaluate(periodName):buildObjectWithPeriodDetails(periodName);return{start:date.start,end:date.end,formattedStart:getDisplayValueByValue(date.start),formattedEnd:getDisplayValueByValue(date.end),period:{name:periodName,label:periodLabel||"",customized:date.customized}}};if(parentWidget.currentRow[fieldName]&&parentWidget.currentRow[fieldName].period&&parentWidget.currentRow[fieldName].period.name){var date=field.getDatesByPeriodName(parentWidget.currentRow[fieldName].period.name);parentWidget.currentRow[fieldName].start=date.start;parentWidget.currentRow[fieldName].end=date.end}$scope.$watch("fieldRow.start",function(newValue){parentWidget.currentRow[fieldName].formattedStart=newValue?getDisplayValueByValue(newValue):emptyDefault});$scope.$watch("fieldRow.end",function(newValue){parentWidget.currentRow[fieldName].formattedEnd=newValue?getDisplayValueByValue(newValue):emptyDefault})}$scope.isRequired=function(field){return field.validations&&(field.validations.requiredInterval||field.validations.requiredIntervalStart||field.validations.requiredIntervalEnd)}}

// FILE: bower_components/zh-grid-configuration/dist/zh-grid-configuration.js

// FILE: bower_components/sortablejs/Sortable.js
/**!
 * Sortable
 * @author	RubaXa   <trash@rubaxa.org>
 * @author	owenm    <owen23355@gmail.com>
 * @license MIT
 */

(function sortableModule(factory) {
	"use strict";

	if (typeof define === "function" && define.amd) {
		define(factory);
	}
	else if (typeof module != "undefined" && typeof module.exports != "undefined") {
		module.exports = factory();
	}
	else {
		/* jshint sub:true */
		window["Sortable"] = factory();
	}
})(function sortableFactory() {
	"use strict";

	if (typeof window === "undefined" || !window.document) {
		return function sortableError() {
			throw new Error("Sortable.js requires a window with a document");
		};
	}

	var dragEl,
		parentEl,
		ghostEl,
		cloneEl,
		rootEl,
		nextEl,
		lastDownEl,

		scrollEl,
		scrollParentEl,
		scrollCustomFn,

		oldIndex,
		newIndex,

		activeGroup,
		putSortable,

		autoScrolls = [],
		scrolling = false,

		awaitingDragStarted = false,
		ignoreNextClick = false,
		sortables = [],

		pointerElemChangedInterval,
		lastPointerElemX,
		lastPointerElemY,

		tapEvt,
		touchEvt,

		moved,


		lastTarget,
		lastDirection,
		pastFirstInvertThresh = false,
		isCircumstantialInvert = false,
		lastMode, // 'swap' or 'insert'

		targetMoveDistance,


		forRepaintDummy,
		realDragElRect, // dragEl rect after current animation

		/** @const */
		R_SPACE = /\s+/g,

		expando = 'Sortable' + (new Date).getTime(),

		win = window,
		document = win.document,
		parseInt = win.parseInt,
		setTimeout = win.setTimeout,

		$ = win.jQuery || win.Zepto,
		Polymer = win.Polymer,

		captureMode = {
			capture: false,
			passive: false
		},

		IE11OrLess = !!navigator.userAgent.match(/(?:Trident.*rv[ :]?11\.|msie|iemobile)/i),
		Edge = !!navigator.userAgent.match(/Edge/i),
		// FireFox = !!navigator.userAgent.match(/firefox/i),

		CSSFloatProperty = Edge || IE11OrLess ? 'cssFloat' : 'float',

		// This will not pass for IE9, because IE9 DnD only works on anchors
		supportDraggable = ('draggable' in document.createElement('div')),

		supportCssPointerEvents = (function() {
			// false when <= IE11
			if (IE11OrLess) {
				return false;
			}
			var el = document.createElement('x');
			el.style.cssText = 'pointer-events:auto';
			return el.style.pointerEvents === 'auto';
		})(),

		_silent = false,
		_alignedSilent = false,

		abs = Math.abs,
		min = Math.min,

		savedInputChecked = [],

		_detectDirection = function(el, options) {
			var elCSS = _css(el),
				elWidth = parseInt(elCSS.width),
				child1 = _getChild(el, 0, options),
				child2 = _getChild(el, 1, options),
				firstChildCSS = child1 && _css(child1),
				secondChildCSS = child2 && _css(child2),
				firstChildWidth = firstChildCSS && parseInt(firstChildCSS.marginLeft) + parseInt(firstChildCSS.marginRight) + _getRect(child1).width,
				secondChildWidth = secondChildCSS && parseInt(secondChildCSS.marginLeft) + parseInt(secondChildCSS.marginRight) + _getRect(child2).width;
			if (elCSS.display === 'flex') {
				return elCSS.flexDirection === 'column' || elCSS.flexDirection === 'column-reverse'
				? 'vertical' : 'horizontal';
			}
			return (child1 &&
				(
					firstChildCSS.display === 'block' ||
					firstChildCSS.display === 'flex' ||
					firstChildCSS.display === 'table' ||
					firstChildCSS.display === 'grid' ||
					firstChildWidth >= elWidth &&
					elCSS[CSSFloatProperty] === 'none' ||
					child2 &&
					elCSS[CSSFloatProperty] === 'none' &&
					firstChildWidth + secondChildWidth > elWidth
				) ?
				'vertical' : 'horizontal'
			);
		},

		/**
		 * Detects first nearest empty sortable to X and Y position using emptyInsertThreshold.
		 * @param  {Number} x      X position
		 * @param  {Number} y      Y position
		 * @return {HTMLElement}   Element of the first found nearest Sortable
		 */
		_detectNearestEmptySortable = function(x, y) {
			for (var i = 0; i < sortables.length; i++) {
				if (sortables[i].children.length) continue;

				var rect = _getRect(sortables[i]),
					threshold = sortables[i][expando].options.emptyInsertThreshold,
					insideHorizontally = x >= (rect.left - threshold) && x <= (rect.right + threshold),
					insideVertically = y >= (rect.top - threshold) && y <= (rect.bottom + threshold);

				if (insideHorizontally && insideVertically) {
					return sortables[i];
				}
			}
		},

		_isClientInRowColumn = function(x, y, el, axis, options) {
			var targetRect = _getRect(el),
				targetS1Opp = axis === 'vertical' ? targetRect.left : targetRect.top,
				targetS2Opp = axis === 'vertical' ? targetRect.right : targetRect.bottom,
				mouseOnOppAxis = axis === 'vertical' ? x : y;

			return targetS1Opp < mouseOnOppAxis && mouseOnOppAxis < targetS2Opp;
		},

		_isElInRowColumn = function(el1, el2, axis) {
			var el1Rect = el1 === dragEl && realDragElRect || _getRect(el1),
				el2Rect = el2 === dragEl && realDragElRect || _getRect(el2),
				el1S1Opp = axis === 'vertical' ? el1Rect.left : el1Rect.top,
				el1S2Opp = axis === 'vertical' ? el1Rect.right : el1Rect.bottom,
				el1OppLength = axis === 'vertical' ? el1Rect.width : el1Rect.height,
				el2S1Opp = axis === 'vertical' ? el2Rect.left : el2Rect.top,
				el2S2Opp = axis === 'vertical' ? el2Rect.right : el2Rect.bottom,
				el2OppLength = axis === 'vertical' ? el2Rect.width : el2Rect.height;

			return (
				el1S1Opp === el2S1Opp ||
				el1S2Opp === el2S2Opp ||
				(el1S1Opp + el1OppLength / 2) === (el2S1Opp + el2OppLength / 2)
			);
		},

		_getParentAutoScrollElement = function(el, includeSelf) {
			// skip to window
			if (!el || !el.getBoundingClientRect) return win;

			var elem = el;
			var gotSelf = false;
			do {
				// we don't need to get elem css if it isn't even overflowing in the first place (performance)
				if (elem.clientWidth < elem.scrollWidth || elem.clientHeight < elem.scrollHeight) {
					var elemCSS = _css(elem);
					if (
						elem.clientWidth < elem.scrollWidth && (elemCSS.overflowX == 'auto' || elemCSS.overflowX == 'scroll') ||
						elem.clientHeight < elem.scrollHeight && (elemCSS.overflowY == 'auto' || elemCSS.overflowY == 'scroll')
					) {
						if (!elem || !elem.getBoundingClientRect || elem === document.body) return win;

						if (gotSelf || includeSelf) return elem;
						gotSelf = true;
					}
				}
			/* jshint boss:true */
			} while (elem = elem.parentNode);

			return win;
		},

		_autoScroll = _throttle(function (/**Event*/evt, /**Object*/options, /**HTMLElement*/rootEl, /**Boolean*/isFallback) {
			// Bug: https://bugzilla.mozilla.org/show_bug.cgi?id=505521
			if (options.scroll) {
				var _this = rootEl ? rootEl[expando] : window,
					sens = options.scrollSensitivity,
					speed = options.scrollSpeed,

					x = evt.clientX,
					y = evt.clientY,

					winWidth = window.innerWidth,
					winHeight = window.innerHeight,

					scrollThisInstance = false;

				// Detect scrollEl
				if (scrollParentEl !== rootEl) {
					_clearAutoScrolls();

					scrollEl = options.scroll;
					scrollCustomFn = options.scrollFn;

					if (scrollEl === true) {
						scrollEl = _getParentAutoScrollElement(rootEl, true);
						scrollParentEl = scrollEl;
					}
				}


				var layersOut = 0;
				var currentParent = scrollEl;
				do {
					var	el = currentParent,
						rect = _getRect(el),

						top = rect.top,
						bottom = rect.bottom,
						left = rect.left,
						right = rect.right,

						width = rect.width,
						height = rect.height,

						scrollWidth,
						scrollHeight,

						css,

						vx,
						vy,

						canScrollX,
						canScrollY,

						scrollPosX,
						scrollPosY;


					if (el !== win) {
						scrollWidth = el.scrollWidth;
						scrollHeight = el.scrollHeight;

						css = _css(el);

						canScrollX = width < scrollWidth && (css.overflowX === 'auto' || css.overflowX === 'scroll');
						canScrollY = height < scrollHeight && (css.overflowY === 'auto' || css.overflowY === 'scroll');

						scrollPosX = el.scrollLeft;
						scrollPosY = el.scrollTop;
					} else {
						scrollWidth = document.documentElement.scrollWidth;
						scrollHeight = document.documentElement.scrollHeight;

						css = _css(document.documentElement);

						canScrollX = width < scrollWidth && (css.overflowX === 'auto' || css.overflowX === 'scroll' || css.overflowX === 'visible');
						canScrollY = height < scrollHeight && (css.overflowY === 'auto' || css.overflowY === 'scroll' || css.overflowY === 'visible');

						scrollPosX = document.documentElement.scrollLeft;
						scrollPosY = document.documentElement.scrollTop;
					}

					vx = canScrollX && (abs(right - x) <= sens && (scrollPosX + width) < scrollWidth) - (abs(left - x) <= sens && !!scrollPosX);

					vy = canScrollY && (abs(bottom - y) <= sens && (scrollPosY + height) < scrollHeight) - (abs(top - y) <= sens && !!scrollPosY);


					if (!autoScrolls[layersOut]) {
						for (var i = 0; i <= layersOut; i++) {
							if (!autoScrolls[i]) {
								autoScrolls[i] = {};
							}
						}
					}

					if (autoScrolls[layersOut].vx != vx || autoScrolls[layersOut].vy != vy || autoScrolls[layersOut].el !== el) {
						autoScrolls[layersOut].el = el;
						autoScrolls[layersOut].vx = vx;
						autoScrolls[layersOut].vy = vy;

						clearInterval(autoScrolls[layersOut].pid);

						if (el && (vx != 0 || vy != 0)) {
							scrollThisInstance = true;
							/* jshint loopfunc:true */
							autoScrolls[layersOut].pid = setInterval((function () {
								// emulate drag over during autoscroll (fallback), emulating native DnD behaviour
								if (isFallback && this.layer === 0) {
									Sortable.active._emulateDragOver(true);
								}
								var scrollOffsetY = autoScrolls[this.layer].vy ? autoScrolls[this.layer].vy * speed : 0;
								var scrollOffsetX = autoScrolls[this.layer].vx ? autoScrolls[this.layer].vx * speed : 0;

								if ('function' === typeof(scrollCustomFn)) {
									if (scrollCustomFn.call(_this, scrollOffsetX, scrollOffsetY, evt, touchEvt, autoScrolls[this.layer].el) !== 'continue') {
										return;
									}
								}
								if (autoScrolls[this.layer].el === win) {
									win.scrollTo(win.pageXOffset + scrollOffsetX, win.pageYOffset + scrollOffsetY);
								} else {
									autoScrolls[this.layer].el.scrollTop += scrollOffsetY;
									autoScrolls[this.layer].el.scrollLeft += scrollOffsetX;
								}
							}).bind({layer: layersOut}), 24);
						}
					}
					layersOut++;
				} while (options.bubbleScroll && currentParent !== win && (currentParent = _getParentAutoScrollElement(currentParent, false)));
				scrolling = scrollThisInstance; // in case another function catches scrolling as false in between when it is not
			}
		}, 30),

		_clearAutoScrolls = function () {
			autoScrolls.forEach(function(autoScroll) {
				clearInterval(autoScroll.pid);
			});
			autoScrolls = [];
		},

		_prepareGroup = function (options) {
			function toFn(value, pull) {
				return function(to, from, dragEl, evt) {
					var ret;
					var sameGroup = to.options.group.name &&
									from.options.group.name &&
									to.options.group.name === from.options.group.name;

					if (value == null && (pull || sameGroup)) {
						// Default pull value
						// Default pull and put value if same group
						return true;
					} else if (value == null || value === false) {
						return false;
					} else if (pull && value === 'clone') {
						return value;
					} else if (typeof value === 'function') {
						return toFn(value(to, from, dragEl, evt));
					} else {
						var otherGroup = (pull ? to : from).options.group.name;

						return (value === true ||
						(typeof value === 'string' && value === otherGroup) ||
						(value.join && value.indexOf(otherGroup) > -1));
					}
				};
			}

			var group = {};
			var originalGroup = options.group;

			if (!originalGroup || typeof originalGroup != 'object') {
				originalGroup = {name: originalGroup};
			}

			group.name = originalGroup.name;
			group.checkPull = toFn(originalGroup.pull, true);
			group.checkPut = toFn(originalGroup.put);
			group.revertClone = originalGroup.revertClone;

			options.group = group;
		},

		_checkAlignment = function(evt) {
			if (!dragEl || !dragEl.parentNode) return;
			dragEl.parentNode[expando] && dragEl.parentNode[expando]._computeIsAligned(evt);
		},

		_isTrueParentSortable = function(el, target) {
			var trueParent = target;
			while (!trueParent[expando]) {
				trueParent = trueParent.parentNode;
			}

			return el === trueParent;
		},

		_artificalBubble = function(sortable, originalEvt, method) {
			// Artificial IE bubbling
			var nextParent = sortable.parentNode;
			while (nextParent && !nextParent[expando]) {
				nextParent = nextParent.parentNode;
			}

			if (nextParent) {
				nextParent[expando][method](_extend(originalEvt, {
					artificialBubble: true
				}));
			}
		},

		_hideGhostForTarget = function() {
			if (!supportCssPointerEvents && ghostEl) {
				_css(ghostEl, 'display', 'none');
			}
		},

		_unhideGhostForTarget = function() {
			if (!supportCssPointerEvents && ghostEl) {
				_css(ghostEl, 'display', '');
			}
		};


	// #1184 fix - Prevent click event on fallback if dragged but item not changed position
	document.addEventListener('click', function(evt) {
		if (ignoreNextClick) {
			evt.preventDefault();
			evt.stopPropagation && evt.stopPropagation();
			evt.stopImmediatePropagation && evt.stopImmediatePropagation();
			ignoreNextClick = false;
			return false;
		}
	}, true);

	var nearestEmptyInsertDetectEvent = function(evt) {
		if (dragEl) {
			var nearest = _detectNearestEmptySortable(evt.clientX, evt.clientY);

			if (nearest) {
				nearest[expando]._onDragOver({
					clientX: evt.clientX,
					clientY: evt.clientY,
					target: nearest,
					rootEl: nearest
				});
			}
		}
	};
	// We do not want this to be triggered if completed (bubbling canceled), so only define it here
	document.addEventListener('dragover', nearestEmptyInsertDetectEvent);
	document.addEventListener('mousemove', nearestEmptyInsertDetectEvent);

	/**
	 * @class  Sortable
	 * @param  {HTMLElement}  el
	 * @param  {Object}       [options]
	 */
	function Sortable(el, options) {
		if (!(el && el.nodeType && el.nodeType === 1)) {
			throw 'Sortable: `el` must be HTMLElement, not ' + {}.toString.call(el);
		}

		this.el = el; // root element
		this.options = options = _extend({}, options);


		// Export instance
		el[expando] = this;

		// Default options
		var defaults = {
			group: null,
			sort: true,
			disabled: false,
			store: null,
			handle: null,
			scroll: true,
			scrollSensitivity: 30,
			scrollSpeed: 10,
			bubbleScroll: true,
			draggable: /[uo]l/i.test(el.nodeName) ? 'li' : '>*',
			swapThreshold: 1, // percentage; 0 <= x <= 1
			invertSwap: false, // invert always
			invertedSwapThreshold: null, // will be set to same as swapThreshold if default
			removeCloneOnHide: true,
			direction: function() {
				return _detectDirection(el, this.options);
			},
			ghostClass: 'sortable-ghost',
			chosenClass: 'sortable-chosen',
			dragClass: 'sortable-drag',
			ignore: 'a, img',
			filter: null,
			preventOnFilter: true,
			animation: 0,
			easing: null,
			setData: function (dataTransfer, dragEl) {
				dataTransfer.setData('Text', dragEl.textContent);
			},
			dropBubble: false,
			dragoverBubble: false,
			dataIdAttr: 'data-id',
			delay: 0,
			touchStartThreshold: parseInt(window.devicePixelRatio, 10) || 1,
			forceFallback: false,
			fallbackClass: 'sortable-fallback',
			fallbackOnBody: false,
			fallbackTolerance: 0,
			fallbackOffset: {x: 0, y: 0},
			supportPointer: Sortable.supportPointer !== false && (
				('PointerEvent' in window) ||
				window.navigator && ('msPointerEnabled' in window.navigator) // microsoft
			),
			emptyInsertThreshold: 5
		};


		// Set default options
		for (var name in defaults) {
			!(name in options) && (options[name] = defaults[name]);
		}

		_prepareGroup(options);

		// Bind all private methods
		for (var fn in this) {
			if (fn.charAt(0) === '_' && typeof this[fn] === 'function') {
				this[fn] = this[fn].bind(this);
			}
		}

		// Setup drag mode
		this.nativeDraggable = options.forceFallback ? false : supportDraggable;

		// Bind events
		if (options.supportPointer) {
			_on(el, 'pointerdown', this._onTapStart);
		} else {
			_on(el, 'mousedown', this._onTapStart);
			_on(el, 'touchstart', this._onTapStart);
		}

		if (this.nativeDraggable) {
			_on(el, 'dragover', this);
			_on(el, 'dragenter', this);
		}

		sortables.push(this.el);

		// Restore sorting
		options.store && options.store.get && this.sort(options.store.get(this) || []);
	}

	Sortable.prototype = /** @lends Sortable.prototype */ {
		constructor: Sortable,

		_computeIsAligned: function(evt) {
			var target;

			if (ghostEl && !supportCssPointerEvents) {
				_hideGhostForTarget();
				target = document.elementFromPoint(evt.clientX, evt.clientY);
				_unhideGhostForTarget();
			} else {
				target = evt.target;
			}

			target = _closest(target, this.options.draggable, this.el, false);
			if (_alignedSilent) return;
			if (!dragEl || dragEl.parentNode !== this.el) return;

			var children = this.el.children;
			for (var i = 0; i < children.length; i++) {
				// Don't change for target in case it is changed to aligned before onDragOver is fired
				if (_closest(children[i], this.options.draggable, this.el, false) && children[i] !== target) {
					children[i].sortableMouseAligned = _isClientInRowColumn(evt.clientX, evt.clientY, children[i], this._getDirection(evt, null), this.options);
				}
			}
			// Used for nulling last target when not in element, nothing to do with checking if aligned
			if (!_closest(target, this.options.draggable, this.el, true)) {
				lastTarget = null;
			}

			_alignedSilent = true;
			setTimeout(function() {
				_alignedSilent = false;
			}, 30);

		},

		_getDirection: function(evt, target) {
			return (typeof this.options.direction === 'function') ? this.options.direction.call(this, evt, target, dragEl) : this.options.direction;
		},

		_onTapStart: function (/** Event|TouchEvent */evt) {
			if (!evt.cancelable) return;

			var _this = this,
				el = this.el,
				options = this.options,
				preventOnFilter = options.preventOnFilter,
				type = evt.type,
				touch = evt.touches && evt.touches[0],
				target = (touch || evt).target,
				originalTarget = evt.target.shadowRoot && ((evt.path && evt.path[0]) || (evt.composedPath && evt.composedPath()[0])) || target,
				filter = options.filter,
				startIndex;

			_saveInputCheckedState(el);


			// IE: Calls events in capture mode if event element is nested. This ensures only correct element's _onTapStart goes through.
			// This process is also done in _onDragOver
			if (IE11OrLess && !evt.artificialBubble && !_isTrueParentSortable(el, target)) {
				return;
			}

			// Don't trigger start event when an element is been dragged, otherwise the evt.oldindex always wrong when set option.group.
			if (dragEl) {
				return;
			}

			if (/mousedown|pointerdown/.test(type) && evt.button !== 0 || options.disabled) {
				return; // only left button and enabled
			}

			// cancel dnd if original target is content editable
			if (originalTarget.isContentEditable) {
				return;
			}

			target = _closest(target, options.draggable, el, false);

			if (!target) {
				if (IE11OrLess) {
					_artificalBubble(el, evt, '_onTapStart');
				}
				return;
			}

			if (lastDownEl === target) {
				// Ignoring duplicate `down`
				return;
			}

			// Get the index of the dragged element within its parent
			startIndex = _index(target, options.draggable);

			// Check filter
			if (typeof filter === 'function') {
				if (filter.call(this, evt, target, this)) {
					_dispatchEvent(_this, originalTarget, 'filter', target, el, el, startIndex);
					preventOnFilter && evt.cancelable && evt.preventDefault();
					return; // cancel dnd
				}
			}
			else if (filter) {
				filter = filter.split(',').some(function (criteria) {
					criteria = _closest(originalTarget, criteria.trim(), el, false);

					if (criteria) {
						_dispatchEvent(_this, criteria, 'filter', target, el, el, startIndex);
						return true;
					}
				});

				if (filter) {
					preventOnFilter && evt.cancelable && evt.preventDefault();
					return; // cancel dnd
				}
			}

			if (options.handle && !_closest(originalTarget, options.handle, el, false)) {
				return;
			}

			// Prepare `dragstart`
			this._prepareDragStart(evt, touch, target, startIndex);
		},


		_handleAutoScroll: function(evt, fallback) {
			if (!dragEl || !this.options.scroll) return;
			var x = evt.clientX,
				y = evt.clientY,

				elem = document.elementFromPoint(x, y),
				_this = this;

			// IE does not seem to have native autoscroll,
			// Edge's autoscroll seems too conditional,
			// Firefox and Chrome are good
			if (fallback || Edge || IE11OrLess) {
				_autoScroll(evt, _this.options, elem, fallback);

				// Listener for pointer element change
				var ogElemScroller = _getParentAutoScrollElement(elem, true);
				if (
					scrolling &&
					(
						!pointerElemChangedInterval ||
						x !== lastPointerElemX ||
						y !== lastPointerElemY
					)
				) {

					pointerElemChangedInterval && clearInterval(pointerElemChangedInterval);
					// Detect for pointer elem change, emulating native DnD behaviour
					pointerElemChangedInterval = setInterval(function() {
						if (!dragEl) return;
						// could also check if scroll direction on newElem changes due to parent autoscrolling
						var newElem = _getParentAutoScrollElement(document.elementFromPoint(x, y), true);
						if (newElem !== ogElemScroller) {
							ogElemScroller = newElem;
							_clearAutoScrolls();
							_autoScroll(evt, _this.options, ogElemScroller, fallback);
						}
					}, 10);
					lastPointerElemX = x;
					lastPointerElemY = y;
				}

			} else {
				// if DnD is enabled (and browser has good autoscrolling), first autoscroll will already scroll, so get parent autoscroll of first autoscroll
				if (!_this.options.bubbleScroll || _getParentAutoScrollElement(elem, true) === window) {
					_clearAutoScrolls();
					return;
				}
				_autoScroll(evt, _this.options, _getParentAutoScrollElement(elem, false), false);
			}
		},

		_prepareDragStart: function (/** Event */evt, /** Touch */touch, /** HTMLElement */target, /** Number */startIndex) {
			var _this = this,
				el = _this.el,
				options = _this.options,
				ownerDocument = el.ownerDocument,
				dragStartFn;

			if (target && !dragEl && (target.parentNode === el)) {
				rootEl = el;
				dragEl = target;
				parentEl = dragEl.parentNode;
				nextEl = dragEl.nextSibling;
				lastDownEl = target;
				activeGroup = options.group;
				oldIndex = startIndex;

				tapEvt = {
					target: dragEl,
					clientX: (touch || evt).clientX,
					clientY: (touch || evt).clientY
				};

				this._lastX = (touch || evt).clientX;
				this._lastY = (touch || evt).clientY;

				dragEl.style['will-change'] = 'all';
				// undo animation if needed
				dragEl.style.transition = '';
				dragEl.style.transform = '';

				dragStartFn = function () {
					// Delayed drag has been triggered
					// we can re-enable the events: touchmove/mousemove
					_this._disableDelayedDrag();

					// Make the element draggable
					dragEl.draggable = _this.nativeDraggable;

					// Bind the events: dragstart/dragend
					_this._triggerDragStart(evt, touch);

					// Drag start event
					_dispatchEvent(_this, rootEl, 'choose', dragEl, rootEl, rootEl, oldIndex);

					// Chosen item
					_toggleClass(dragEl, options.chosenClass, true);
				};

				// Disable "draggable"
				options.ignore.split(',').forEach(function (criteria) {
					_find(dragEl, criteria.trim(), _disableDraggable);
				});

				if (options.supportPointer) {
					_on(ownerDocument, 'pointerup', _this._onDrop);
					_on(ownerDocument, 'pointercancel', _this._onDrop);
				} else {
					_on(ownerDocument, 'mouseup', _this._onDrop);
					_on(ownerDocument, 'touchend', _this._onDrop);
					_on(ownerDocument, 'touchcancel', _this._onDrop);
				}

				if (options.delay) {
					// If the user moves the pointer or let go the click or touch
					// before the delay has been reached:
					// disable the delayed drag
					_on(ownerDocument, 'mouseup', _this._disableDelayedDrag);
					_on(ownerDocument, 'touchend', _this._disableDelayedDrag);
					_on(ownerDocument, 'touchcancel', _this._disableDelayedDrag);
					_on(ownerDocument, 'mousemove', _this._delayedDragTouchMoveHandler);
					_on(ownerDocument, 'touchmove', _this._delayedDragTouchMoveHandler);
					options.supportPointer && _on(ownerDocument, 'pointermove', _this._delayedDragTouchMoveHandler);

					_this._dragStartTimer = setTimeout(dragStartFn, options.delay);
				} else {
					dragStartFn();
				}
			}
		},

		_delayedDragTouchMoveHandler: function (/** TouchEvent|PointerEvent **/e) {
			var touch = e.touches ? e.touches[0] : e;
			if (min(abs(touch.clientX - this._lastX), abs(touch.clientY - this._lastY))
					>= this.options.touchStartThreshold
			) {
				this._disableDelayedDrag();
			}
		},

		_disableDelayedDrag: function () {
			var ownerDocument = this.el.ownerDocument;

			clearTimeout(this._dragStartTimer);
			_off(ownerDocument, 'mouseup', this._disableDelayedDrag);
			_off(ownerDocument, 'touchend', this._disableDelayedDrag);
			_off(ownerDocument, 'touchcancel', this._disableDelayedDrag);
			_off(ownerDocument, 'mousemove', this._delayedDragTouchMoveHandler);
			_off(ownerDocument, 'touchmove', this._delayedDragTouchMoveHandler);
			_off(ownerDocument, 'pointermove', this._delayedDragTouchMoveHandler);
		},

		_triggerDragStart: function (/** Event */evt, /** Touch */touch) {
			touch = touch || (evt.pointerType == 'touch' ? evt : null);

			if (!this.nativeDraggable || touch) {
				if (this.options.supportPointer) {
					touch && _on(document, 'touchmove', _preventScroll); // must be touchmove to prevent scroll
					_on(document, 'pointermove', this._onTouchMove);
				} else if (touch) {
					_on(document, 'touchmove', _preventScroll);
					_on(document, 'touchmove', this._onTouchMove);
				} else {
					_on(document, 'mousemove', this._onTouchMove);
				}
			} else {
				_on(dragEl, 'dragend', this);
				_on(rootEl, 'dragstart', this._onDragStart);
			}

			try {
				if (document.selection) {
					// Timeout neccessary for IE9
					_nextTick(function () {
						document.selection.empty();
					});
				} else {
					window.getSelection().removeAllRanges();
				}
			} catch (err) {
			}
		},

		_dragStarted: function (fallback) {
			awaitingDragStarted = false;
			if (rootEl && dragEl) {
				if (this.nativeDraggable) {
					_on(document, 'dragover', this._handleAutoScroll);
					_on(document, 'dragover', _checkAlignment);
				}
				var options = this.options;

				// Apply effect
				!fallback && _toggleClass(dragEl, options.dragClass, false);
				_toggleClass(dragEl, options.ghostClass, true);

				// In case dragging an animated element
				_css(dragEl, 'transform', '');

				Sortable.active = this;

				fallback && this._appendGhost();

				// Drag start event
				_dispatchEvent(this, rootEl, 'start', dragEl, rootEl, rootEl, oldIndex);
			} else {
				this._nulling();
			}
		},

		_emulateDragOver: function (bypassLastTouchCheck) {
			if (touchEvt) {
				if (this._lastX === touchEvt.clientX && this._lastY === touchEvt.clientY && !bypassLastTouchCheck) {
					return;
				}
				this._lastX = touchEvt.clientX;
				this._lastY = touchEvt.clientY;

				_hideGhostForTarget();

				var target = document.elementFromPoint(touchEvt.clientX, touchEvt.clientY);
				var parent = target;

				while (target && target.shadowRoot) {
					target = target.shadowRoot.elementFromPoint(touchEvt.clientX, touchEvt.clientY);
					parent = target;
				}

				if (parent) {
					do {
						if (parent[expando]) {
							var inserted;

							inserted = parent[expando]._onDragOver({
								clientX: touchEvt.clientX,
								clientY: touchEvt.clientY,
								target: target,
								rootEl: parent
							});

							if (inserted && !this.options.dragoverBubble) {
								break;
							}
						}

						target = parent; // store last element
					}
					/* jshint boss:true */
					while (parent = parent.parentNode);
				}
				dragEl.parentNode[expando]._computeIsAligned(touchEvt);

				_unhideGhostForTarget();
			}
		},


		_onTouchMove: function (/**TouchEvent*/evt) {
			if (tapEvt) {
				if (!evt.cancelable) return;
				var	options = this.options,
					fallbackTolerance = options.fallbackTolerance,
					fallbackOffset = options.fallbackOffset,
					touch = evt.touches ? evt.touches[0] : evt,
					matrix = ghostEl && _matrix(ghostEl),
					scaleX = ghostEl && matrix && matrix.a,
					scaleY = ghostEl && matrix && matrix.d,
					dx = ((touch.clientX - tapEvt.clientX) + fallbackOffset.x) / (scaleX ? scaleX : 1),
					dy = ((touch.clientY - tapEvt.clientY) + fallbackOffset.y) / (scaleY ? scaleY : 1),
					translate3d = evt.touches ? 'translate3d(' + dx + 'px,' + dy + 'px,0)' : 'translate(' + dx + 'px,' + dy + 'px)';


				// only set the status to dragging, when we are actually dragging
				if (!Sortable.active && !awaitingDragStarted) {
					if (fallbackTolerance &&
						min(abs(touch.clientX - this._lastX), abs(touch.clientY - this._lastY)) < fallbackTolerance
					) {
						return;
					}
					this._onDragStart(evt, true);
				}

				this._handleAutoScroll(touch, true);


				moved = true;
				touchEvt = touch;


				_css(ghostEl, 'webkitTransform', translate3d);
				_css(ghostEl, 'mozTransform', translate3d);
				_css(ghostEl, 'msTransform', translate3d);
				_css(ghostEl, 'transform', translate3d);

				evt.cancelable && evt.preventDefault();
			}
		},

		_appendGhost: function () {
			if (!ghostEl) {
				var rect = _getRect(dragEl, this.options.fallbackOnBody ? document.body : rootEl, true),
					css = _css(dragEl),
					options = this.options;

				ghostEl = dragEl.cloneNode(true);

				_toggleClass(ghostEl, options.ghostClass, false);
				_toggleClass(ghostEl, options.fallbackClass, true);
				_toggleClass(ghostEl, options.dragClass, true);

				_css(ghostEl, 'box-sizing', 'border-box');
				_css(ghostEl, 'margin', 0);
				_css(ghostEl, 'top', rect.top);
				_css(ghostEl, 'left', rect.left);
				_css(ghostEl, 'width', rect.width);
				_css(ghostEl, 'height', rect.height);
				_css(ghostEl, 'opacity', '0.8');
				_css(ghostEl, 'position', 'fixed');
				_css(ghostEl, 'zIndex', '100000');
				_css(ghostEl, 'pointerEvents', 'none');

				options.fallbackOnBody && document.body.appendChild(ghostEl) || rootEl.appendChild(ghostEl);
			}
		},

		_onDragStart: function (/**Event*/evt, /**boolean*/fallback) {
			var _this = this;
			var dataTransfer = evt.dataTransfer;
			var options = _this.options;

			// Setup clone
			cloneEl = _clone(dragEl);

			cloneEl.draggable = false;
			cloneEl.style['will-change'] = '';

			this._hideClone();

			_toggleClass(cloneEl, _this.options.chosenClass, false);


			// #1143: IFrame support workaround
			_this._cloneId = _nextTick(function () {
				if (!_this.options.removeCloneOnHide) {
					rootEl.insertBefore(cloneEl, dragEl);
				}
				_dispatchEvent(_this, rootEl, 'clone', dragEl);
			});


			!fallback && _toggleClass(dragEl, options.dragClass, true);

			// Set proper drop events
			if (fallback) {
				ignoreNextClick = true;
				_this._loopId = setInterval(_this._emulateDragOver, 50);
			} else {
				// Undo what was set in _prepareDragStart before drag started
				_off(document, 'mouseup', _this._onDrop);
				_off(document, 'touchend', _this._onDrop);
				_off(document, 'touchcancel', _this._onDrop);
				_off(document, 'pointercancel', _this._onDrop);

				if (dataTransfer) {
					dataTransfer.effectAllowed = 'move';
					options.setData && options.setData.call(_this, dataTransfer, dragEl);
				}

				_on(document, 'drop', _this);

				// #1276 fix:
				_css(dragEl, 'transform', 'translateZ(0)');
			}

			awaitingDragStarted = true;

			_this._dragStartId = _nextTick(_this._dragStarted.bind(_this, fallback));
			_on(document, 'selectstart', _this);
		},

		// Returns true - if no further action is needed (either inserted or another condition)
		_onDragOver: function (/**Event*/evt) {
			var el = this.el,
				target = evt.target,
				dragRect,
				targetRect,
				revert,
				options = this.options,
				group = options.group,
				activeSortable = Sortable.active,
				isOwner = (activeGroup === group),
				canSort = options.sort,
				_this = this;

			if (_silent) return;

			// IE event order fix
			if (IE11OrLess && !evt.rootEl && !evt.artificialBubble && !_isTrueParentSortable(el, target)) {
				return;
			}

			// Return invocation when no further action is needed in another sortable
			function completed() {
				if (activeSortable) {
					// Set ghost class to new sortable's ghost class
					_toggleClass(dragEl, putSortable ? putSortable.options.ghostClass : activeSortable.options.ghostClass, false);
					_toggleClass(dragEl, options.ghostClass, true);
				}

				if (putSortable !== _this && _this !== Sortable.active) {
					putSortable = _this;
				} else if (_this === Sortable.active) {
					putSortable = null;
				}


				// Null lastTarget if it is not inside a previously swapped element
				if ((target === dragEl && !dragEl.animated) || (target === el && !target.animated)) {
					lastTarget = null;
				}
				// no bubbling and not fallback
				if (!options.dragoverBubble && !evt.rootEl && target !== document) {
					_this._handleAutoScroll(evt);
					dragEl.parentNode[expando]._computeIsAligned(evt);
				}

				!options.dragoverBubble && evt.stopPropagation && evt.stopPropagation();

				return true;
			}

			// Call when dragEl has been inserted
			function changed() {
				_dispatchEvent(_this, rootEl, 'change', target, el, rootEl, oldIndex, _index(dragEl, options.draggable), evt);
			}


			if (evt.preventDefault !== void 0) {
				evt.cancelable && evt.preventDefault();
			}


			moved = true;

			target = _closest(target, options.draggable, el, true);

			// target is dragEl or target is animated
			if (!!_closest(evt.target, null, dragEl, true) || target.animated) {
				return completed();
			}

			if (target !== dragEl) {
				ignoreNextClick = false;
			}

			if (activeSortable && !options.disabled &&
				(isOwner
					? canSort || (revert = !rootEl.contains(dragEl)) // Reverting item into the original list
					: (
						putSortable === this ||
						(
							(this.lastPutMode = activeGroup.checkPull(this, activeSortable, dragEl, evt)) &&
							group.checkPut(this, activeSortable, dragEl, evt)
						)
					)
				)
			) {
				var axis = this._getDirection(evt, target);

				dragRect = _getRect(dragEl);

				if (revert) {
					this._hideClone();
					parentEl = rootEl; // actualization

					if (nextEl) {
						rootEl.insertBefore(dragEl, nextEl);
					} else {
						rootEl.appendChild(dragEl);
					}

					return completed();
				}

				if ((el.children.length === 0) || (el.children[0] === ghostEl) ||
					_ghostIsLast(evt, axis, el) && !dragEl.animated
				) {
					//assign target only if condition is true
					if (el.children.length !== 0 && el.children[0] !== ghostEl && el === evt.target) {
						target = _lastChild(el);
					}

					if (target) {
						targetRect = _getRect(target);
					}

					if (isOwner) {
						activeSortable._hideClone();
					} else {
						activeSortable._showClone(this);
					}

					if (_onMove(rootEl, el, dragEl, dragRect, target, targetRect, evt, !!target) !== false) {
						el.appendChild(dragEl);
						parentEl = el; // actualization
						realDragElRect = null;

						changed();
						this._animate(dragRect, dragEl);
						target && this._animate(targetRect, target);
						return completed();
					}
				}
				else if (target && target !== dragEl && (target.parentNode[expando] !== void 0) && target !== el) {
					var direction = 0,
						targetBeforeFirstSwap,
						aligned = target.sortableMouseAligned,
						differentLevel = dragEl.parentNode !== el,
						scrolledPastTop = _isScrolledPast(target, axis === 'vertical' ? 'top' : 'left');

					if (lastTarget !== target) {
						lastMode = null;
						targetBeforeFirstSwap = _getRect(target)[axis === 'vertical' ? 'top' : 'left'];
						pastFirstInvertThresh = false;
					}

					// Reference: https://www.lucidchart.com/documents/view/10fa0e93-e362-4126-aca2-b709ee56bd8b/0
					if (
						_isElInRowColumn(dragEl, target, axis) && aligned ||
						differentLevel ||
						scrolledPastTop ||
						options.invertSwap ||
						lastMode === 'insert' ||
						// Needed, in the case that we are inside target and inserted because not aligned... aligned will stay false while inside
						// and lastMode will change to 'insert', but we must swap
						lastMode === 'swap'
					) {
						// New target that we will be inside
						if (lastMode !== 'swap') {
							isCircumstantialInvert = options.invertSwap || differentLevel || scrolling || scrolledPastTop;
						}

						direction = _getSwapDirection(evt, target, axis,
							options.swapThreshold, options.invertedSwapThreshold == null ? options.swapThreshold : options.invertedSwapThreshold,
							isCircumstantialInvert,
							lastTarget === target);
						lastMode = 'swap';
					} else {
						// Insert at position
						direction = _getInsertDirection(target, options);
						lastMode = 'insert';
					}
					if (direction === 0) return completed();

					realDragElRect = null;
					lastTarget = target;

					lastDirection = direction;

					targetRect = _getRect(target);

					var nextSibling = target.nextElementSibling,
						after = false;

					after = direction === 1;

					var moveVector = _onMove(rootEl, el, dragEl, dragRect, target, targetRect, evt, after);

					if (moveVector !== false) {
						if (moveVector === 1 || moveVector === -1) {
							after = (moveVector === 1);
						}

						_silent = true;
						setTimeout(_unsilent, 30);

						if (isOwner) {
							activeSortable._hideClone();
						} else {
							activeSortable._showClone(this);
						}

						if (after && !nextSibling) {
							el.appendChild(dragEl);
						} else {
							target.parentNode.insertBefore(dragEl, after ? nextSibling : target);
						}

						parentEl = dragEl.parentNode; // actualization

						// must be done before animation
						if (targetBeforeFirstSwap !== undefined && !isCircumstantialInvert) {
							targetMoveDistance = abs(targetBeforeFirstSwap - _getRect(target)[axis === 'vertical' ? 'top' : 'left']);
						}
						changed();
						!differentLevel && this._animate(targetRect, target);
						this._animate(dragRect, dragEl);

						return completed();
					}
				}

				if (el.contains(dragEl)) {
					return completed();
				}
			}

			if (IE11OrLess && !evt.rootEl) {
				_artificalBubble(el, evt, '_onDragOver');
			}

			return false;
		},

		_animate: function (prevRect, target) {
			var ms = this.options.animation;

			if (ms) {
				var currentRect = _getRect(target);

				if (target === dragEl) {
					realDragElRect = currentRect;
				}

				if (prevRect.nodeType === 1) {
					prevRect = _getRect(prevRect);
				}

				// Check if actually moving position
				if ((prevRect.left + prevRect.width / 2) !== (currentRect.left + currentRect.width / 2)
					|| (prevRect.top + prevRect.height / 2) !== (currentRect.top + currentRect.height / 2)
				) {
					var matrix = _matrix(this.el),
						scaleX = matrix && matrix.a,
						scaleY = matrix && matrix.d;

					_css(target, 'transition', 'none');
					_css(target, 'transform', 'translate3d('
						+ (prevRect.left - currentRect.left) / (scaleX ? scaleX : 1) + 'px,'
						+ (prevRect.top - currentRect.top) / (scaleY ? scaleY : 1) + 'px,0)'
					);

					forRepaintDummy = target.offsetWidth; // repaint
					_css(target, 'transition', 'transform ' + ms + 'ms' + (this.options.easing ? ' ' + this.options.easing : ''));
					_css(target, 'transform', 'translate3d(0,0,0)');
				}

				(typeof target.animated === 'number') && clearTimeout(target.animated);
				target.animated = setTimeout(function () {
					_css(target, 'transition', '');
					_css(target, 'transform', '');
					target.animated = false;
				}, ms);
			}
		},

		_offUpEvents: function () {
			var ownerDocument = this.el.ownerDocument;

			_off(document, 'touchmove', _preventScroll);
			_off(document, 'pointermove', _preventScroll);
			_off(document, 'touchmove', this._onTouchMove);
			_off(document, 'pointermove', this._onTouchMove);
			_off(ownerDocument, 'mouseup', this._onDrop);
			_off(ownerDocument, 'touchend', this._onDrop);
			_off(ownerDocument, 'pointerup', this._onDrop);
			_off(ownerDocument, 'touchcancel', this._onDrop);
			_off(ownerDocument, 'pointercancel', this._onDrop);
			_off(document, 'selectstart', this);
		},

		_onDrop: function (/**Event*/evt) {
			var el = this.el,
				options = this.options;

			awaitingDragStarted = false;
			scrolling = false;
			isCircumstantialInvert = false;
			pastFirstInvertThresh = false;

			clearInterval(this._loopId);

			clearInterval(pointerElemChangedInterval);
			_clearAutoScrolls();
			_cancelThrottle();

			clearTimeout(this._dragStartTimer);

			_cancelNextTick(this._cloneId);
			_cancelNextTick(this._dragStartId);

			// Unbind events
			_off(document, 'mousemove', this._onTouchMove);


			if (this.nativeDraggable) {
				_off(document, 'drop', this);
				_off(el, 'dragstart', this._onDragStart);
				_off(document, 'dragover', this._handleAutoScroll);
				_off(document, 'dragover', _checkAlignment);
			}

			this._offUpEvents();

			if (evt) {
				if (moved) {
					evt.cancelable && evt.preventDefault();
					!options.dropBubble && evt.stopPropagation();
				}

				ghostEl && ghostEl.parentNode && ghostEl.parentNode.removeChild(ghostEl);

				if (rootEl === parentEl || (putSortable && putSortable.lastPutMode !== 'clone')) {
					// Remove clone
					cloneEl && cloneEl.parentNode && cloneEl.parentNode.removeChild(cloneEl);
				}

				if (dragEl) {
					if (this.nativeDraggable) {
						_off(dragEl, 'dragend', this);
					}

					_disableDraggable(dragEl);
					dragEl.style['will-change'] = '';

					// Remove class's
					_toggleClass(dragEl, putSortable ? putSortable.options.ghostClass : this.options.ghostClass, false);
					_toggleClass(dragEl, this.options.chosenClass, false);

					// Drag stop event
					_dispatchEvent(this, rootEl, 'unchoose', dragEl, parentEl, rootEl, oldIndex, null, evt);

					if (rootEl !== parentEl) {
						newIndex = _index(dragEl, options.draggable);

						if (newIndex >= 0) {
							// Add event
							_dispatchEvent(null, parentEl, 'add', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

							// Remove event
							_dispatchEvent(this, rootEl, 'remove', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

							// drag from one list and drop into another
							_dispatchEvent(null, parentEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
							_dispatchEvent(this, rootEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
						}

						putSortable && putSortable.save();
					}
					else {
						if (dragEl.nextSibling !== nextEl) {
							// Get the index of the dragged element within its parent
							newIndex = _index(dragEl, options.draggable);

							if (newIndex >= 0) {
								// drag & drop within the same list
								_dispatchEvent(this, rootEl, 'update', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
								_dispatchEvent(this, rootEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
							}
						}
					}

					if (Sortable.active) {
						/* jshint eqnull:true */
						if (newIndex == null || newIndex === -1) {
							newIndex = oldIndex;
						}

						_dispatchEvent(this, rootEl, 'end', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

						// Save sorting
						this.save();
					}
				}

			}
			this._nulling();
		},

		_nulling: function() {
			rootEl =
			dragEl =
			parentEl =
			ghostEl =
			nextEl =
			cloneEl =
			lastDownEl =

			scrollEl =
			scrollParentEl =
			autoScrolls.length =

			pointerElemChangedInterval =
			lastPointerElemX =
			lastPointerElemY =

			tapEvt =
			touchEvt =

			moved =
			newIndex =
			oldIndex =

			lastTarget =
			lastDirection =

			forRepaintDummy =
			realDragElRect =

			putSortable =
			activeGroup =
			Sortable.active = null;

			savedInputChecked.forEach(function (el) {
				el.checked = true;
			});

			savedInputChecked.length = 0;
		},

		handleEvent: function (/**Event*/evt) {
			switch (evt.type) {
				case 'drop':
				case 'dragend':
					this._onDrop(evt);
					break;

				case 'dragenter':
				case 'dragover':
					if (dragEl) {
						this._onDragOver(evt);
						_globalDragOver(evt);
					}
					break;

				case 'selectstart':
					evt.preventDefault();
					break;
			}
		},


		/**
		 * Serializes the item into an array of string.
		 * @returns {String[]}
		 */
		toArray: function () {
			var order = [],
				el,
				children = this.el.children,
				i = 0,
				n = children.length,
				options = this.options;

			for (; i < n; i++) {
				el = children[i];
				if (_closest(el, options.draggable, this.el, false)) {
					order.push(el.getAttribute(options.dataIdAttr) || _generateId(el));
				}
			}

			return order;
		},


		/**
		 * Sorts the elements according to the array.
		 * @param  {String[]}  order  order of the items
		 */
		sort: function (order) {
			var items = {}, rootEl = this.el;

			this.toArray().forEach(function (id, i) {
				var el = rootEl.children[i];

				if (_closest(el, this.options.draggable, rootEl, false)) {
					items[id] = el;
				}
			}, this);

			order.forEach(function (id) {
				if (items[id]) {
					rootEl.removeChild(items[id]);
					rootEl.appendChild(items[id]);
				}
			});
		},


		/**
		 * Save the current sorting
		 */
		save: function () {
			var store = this.options.store;
			store && store.set && store.set(this);
		},


		/**
		 * For each element in the set, get the first element that matches the selector by testing the element itself and traversing up through its ancestors in the DOM tree.
		 * @param   {HTMLElement}  el
		 * @param   {String}       [selector]  default: `options.draggable`
		 * @returns {HTMLElement|null}
		 */
		closest: function (el, selector) {
			return _closest(el, selector || this.options.draggable, this.el, false);
		},


		/**
		 * Set/get option
		 * @param   {string} name
		 * @param   {*}      [value]
		 * @returns {*}
		 */
		option: function (name, value) {
			var options = this.options;

			if (value === void 0) {
				return options[name];
			} else {
				options[name] = value;

				if (name === 'group') {
					_prepareGroup(options);
				}
			}
		},


		/**
		 * Destroy
		 */
		destroy: function () {
			var el = this.el;

			el[expando] = null;

			_off(el, 'mousedown', this._onTapStart);
			_off(el, 'touchstart', this._onTapStart);
			_off(el, 'pointerdown', this._onTapStart);

			if (this.nativeDraggable) {
				_off(el, 'dragover', this);
				_off(el, 'dragenter', this);
			}
			// Remove draggable attributes
			Array.prototype.forEach.call(el.querySelectorAll('[draggable]'), function (el) {
				el.removeAttribute('draggable');
			});

			this._onDrop();

			sortables.splice(sortables.indexOf(this.el), 1);

			this.el = el = null;
		},

		_hideClone: function() {
			if (!cloneEl.cloneHidden) {
				_css(cloneEl, 'display', 'none');
				cloneEl.cloneHidden = true;
				if (cloneEl.parentNode && this.options.removeCloneOnHide) {
					cloneEl.parentNode.removeChild(cloneEl);
				}
			}
		},

		_showClone: function(putSortable) {
			if (putSortable.lastPutMode !== 'clone') {
				this._hideClone();
				return;
			}

			if (cloneEl.cloneHidden) {
				// show clone at dragEl or original position
				if (rootEl.contains(dragEl) && !this.options.group.revertClone) {
					rootEl.insertBefore(cloneEl, dragEl);
				} else if (nextEl) {
					rootEl.insertBefore(cloneEl, nextEl);
				} else {
					rootEl.appendChild(cloneEl);
				}

				if (this.options.group.revertClone) {
					this._animate(dragEl, cloneEl);
				}
				_css(cloneEl, 'display', '');
				cloneEl.cloneHidden = false;
			}
		}
	};

	function _closest(/**HTMLElement*/el, /**String*/selector, /**HTMLElement*/ctx, includeCTX) {
		if (el) {
			ctx = ctx || document;

			do {
				if ((selector === '>*' && el.parentNode === ctx) || _matches(el, selector) || (includeCTX && el === ctx)) {
					return el;
				}

				if (el === ctx) break;
				/* jshint boss:true */
			} while (el = _getParentOrHost(el));
		}

		return null;
	}


	function _getParentOrHost(el) {
		return (el.host && el !== document && el.host.nodeType)
			? el.host
			: el.parentNode;
	}


	function _globalDragOver(/**Event*/evt) {
		if (evt.dataTransfer) {
			evt.dataTransfer.dropEffect = 'move';
		}
		evt.cancelable && evt.preventDefault();
	}


	function _on(el, event, fn) {
		el.addEventListener(event, fn, captureMode);
	}


	function _off(el, event, fn) {
		el.removeEventListener(event, fn, captureMode);
	}


	function _toggleClass(el, name, state) {
		if (el && name) {
			if (el.classList) {
				el.classList[state ? 'add' : 'remove'](name);
			}
			else {
				var className = (' ' + el.className + ' ').replace(R_SPACE, ' ').replace(' ' + name + ' ', ' ');
				el.className = (className + (state ? ' ' + name : '')).replace(R_SPACE, ' ');
			}
		}
	}


	function _css(el, prop, val) {
		var style = el && el.style;

		if (style) {
			if (val === void 0) {
				if (document.defaultView && document.defaultView.getComputedStyle) {
					val = document.defaultView.getComputedStyle(el, '');
				}
				else if (el.currentStyle) {
					val = el.currentStyle;
				}

				return prop === void 0 ? val : val[prop];
			}
			else {
				if (!(prop in style) && prop.indexOf('webkit') === -1) {
					prop = '-webkit-' + prop;
				}

				style[prop] = val + (typeof val === 'string' ? '' : 'px');
			}
		}
	}

	function _matrix(el) {
		var appliedTransforms = '';
		do {
			var transform = _css(el, 'transform');

			if (transform && transform !== 'none') {
				appliedTransforms = transform + ' ' + appliedTransforms;
			}
			/* jshint boss:true */
		} while (el = el.parentNode);

		if (window.DOMMatrix) {
			return new DOMMatrix(appliedTransforms);
		} else if (window.WebKitCSSMatrix) {
			return new WebKitCSSMatrix(appliedTransforms);
		} else if (window.CSSMatrix) {
			return new CSSMatrix(appliedTransforms);
		}
	}


	function _find(ctx, tagName, iterator) {
		if (ctx) {
			var list = ctx.getElementsByTagName(tagName), i = 0, n = list.length;

			if (iterator) {
				for (; i < n; i++) {
					iterator(list[i], i);
				}
			}

			return list;
		}

		return [];
	}



	function _dispatchEvent(sortable, rootEl, name, targetEl, toEl, fromEl, startIndex, newIndex, originalEvt) {
		sortable = (sortable || rootEl[expando]);
		var evt,
			options = sortable.options,
			onName = 'on' + name.charAt(0).toUpperCase() + name.substr(1);
		// Support for new CustomEvent feature
		if (window.CustomEvent && !IE11OrLess && !Edge) {
			evt = new CustomEvent(name, {
				bubbles: true,
				cancelable: true
			});
		} else {
			evt = document.createEvent('Event');
			evt.initEvent(name, true, true);
		}

		evt.to = toEl || rootEl;
		evt.from = fromEl || rootEl;
		evt.item = targetEl || rootEl;
		evt.clone = cloneEl;

		evt.oldIndex = startIndex;
		evt.newIndex = newIndex;

		evt.originalEvent = originalEvt;

		if (rootEl) {
			rootEl.dispatchEvent(evt);
	        }

		if (options[onName]) {
			options[onName].call(sortable, evt);
		}
	}


	function _onMove(fromEl, toEl, dragEl, dragRect, targetEl, targetRect, originalEvt, willInsertAfter) {
		var evt,
			sortable = fromEl[expando],
			onMoveFn = sortable.options.onMove,
			retVal;
		// Support for new CustomEvent feature
		if (window.CustomEvent && !IE11OrLess && !Edge) {
			evt = new CustomEvent('move', {
				bubbles: true,
				cancelable: true
			});
		} else {
			evt = document.createEvent('Event');
			evt.initEvent('move', true, true);
		}

		evt.to = toEl;
		evt.from = fromEl;
		evt.dragged = dragEl;
		evt.draggedRect = dragRect;
		evt.related = targetEl || toEl;
		evt.relatedRect = targetRect || _getRect(toEl);
		evt.willInsertAfter = willInsertAfter;

		evt.originalEvent = originalEvt;

		fromEl.dispatchEvent(evt);

		if (onMoveFn) {
			retVal = onMoveFn.call(sortable, evt, originalEvt);
		}

		return retVal;
	}

	function _disableDraggable(el) {
		el.draggable = false;
	}

	function _unsilent() {
		_silent = false;
	}

	/**
	 * Gets nth child of el, ignoring hidden children, sortable's elements (does not ignore clone if it's visible)
	 * and non-draggable elements
	 * @param  {HTMLElement} el       The parent element
	 * @param  {Number} childNum      The index of the child
	 * @param  {Object} options       Parent Sortable's options
	 * @return {HTMLElement}          The child at index childNum, or null if not found
	 */
	function _getChild(el, childNum, options) {
		var currentChild = 0,
			i = 0,
			children = el.children;

		while (i < children.length) {
			if (
				children[i].style.display !== 'none' &&
				children[i] !== ghostEl &&
				children[i] !== dragEl &&
				_closest(children[i], options.draggable, el, false)
			) {
				if (currentChild === childNum) {
					return children[i];
				}
				currentChild++;
			}

			i++;
		}
		return null;
	}

	/**
	 * Gets the last child in the el, ignoring ghostEl
	 * @param  {HTMLElement} el       Parent element
	 * @return {HTMLElement}          The last child, ignoring ghostEl
	 */
	function _lastChild(el) {
		var last = el.lastElementChild;

		if (last === ghostEl) {
			last = el.children[el.childElementCount - 2];
		}

		return last || null;
	}

	function _ghostIsLast(evt, axis, el) {
		var elRect = _getRect(_lastChild(el)),
			mouseOnAxis = axis === 'vertical' ? evt.clientY : evt.clientX,
			mouseOnOppAxis = axis === 'vertical' ? evt.clientX : evt.clientY,
			targetS2 = axis === 'vertical' ? elRect.bottom : elRect.right,
			targetS1Opp = axis === 'vertical' ? elRect.left : elRect.top,
			targetS2Opp = axis === 'vertical' ? elRect.right : elRect.bottom;

		return (
			mouseOnOppAxis > targetS1Opp &&
			mouseOnOppAxis < targetS2Opp &&
			mouseOnAxis > targetS2
		);
	}

	function _getSwapDirection(evt, target, axis, swapThreshold, invertedSwapThreshold, invertSwap, isLastTarget) {
		var targetRect = _getRect(target),
			mouseOnAxis = axis === 'vertical' ? evt.clientY : evt.clientX,
			targetLength = axis === 'vertical' ? targetRect.height : targetRect.width,
			targetS1 = axis === 'vertical' ? targetRect.top : targetRect.left,
			targetS2 = axis === 'vertical' ? targetRect.bottom : targetRect.right,
			dragRect = _getRect(dragEl),
			invert = false;


		if (!invertSwap) {
			// Never invert or create dragEl shadow when target movemenet causes mouse to move past the end of regular swapThreshold
			if (isLastTarget && targetMoveDistance < targetLength * swapThreshold) { // multiplied only by swapThreshold because mouse will already be inside target by (1 - threshold) * targetLength / 2
				// check if past first invert threshold on side opposite of lastDirection
				if (!pastFirstInvertThresh &&
					(lastDirection === 1 ?
						(
							mouseOnAxis > targetS1 + targetLength * invertedSwapThreshold / 2
						) :
						(
							mouseOnAxis < targetS2 - targetLength * invertedSwapThreshold / 2
						)
					)
				)
				{
					// past first invert threshold, do not restrict inverted threshold to dragEl shadow
					pastFirstInvertThresh = true;
				}

				if (!pastFirstInvertThresh) {
					var dragS1 = axis === 'vertical' ? dragRect.top : dragRect.left,
						dragS2 = axis === 'vertical' ? dragRect.bottom : dragRect.right;
					// dragEl shadow (target move distance shadow)
					if (
						lastDirection === 1 ?
						(
							mouseOnAxis < targetS1 + targetMoveDistance // over dragEl shadow
						) :
						(
							mouseOnAxis > targetS2 - targetMoveDistance
						)
					)
					{
						return lastDirection * -1;
					}
				} else {
					invert = true;
				}
			} else {
				// Regular
				if (
					mouseOnAxis > targetS1 + (targetLength * (1 - swapThreshold) / 2) &&
					mouseOnAxis < targetS2 - (targetLength * (1 - swapThreshold) / 2)
				) {
					return ((mouseOnAxis > targetS1 + targetLength / 2) ? -1 : 1);
				}
			}
		}

		invert = invert || invertSwap;

		if (invert) {
			// Invert of regular
			if (
				mouseOnAxis < targetS1 + (targetLength * invertedSwapThreshold / 2) ||
				mouseOnAxis > targetS2 - (targetLength * invertedSwapThreshold / 2)
			)
			{
				return ((mouseOnAxis > targetS1 + targetLength / 2) ? 1 : -1);
			}
		}

		return 0;
	}

	/**
	 * Gets the direction dragEl must be swapped relative to target in order to make it
	 * seem that dragEl has been "inserted" into that element's position
	 * @param  {HTMLElement} target       The target whose position dragEl is being inserted at
	 * @param  {Object} options           options of the parent sortable
	 * @return {Number}                   Direction dragEl must be swapped
	 */
	function _getInsertDirection(target, options) {
		var dragElIndex = _index(dragEl, options.draggable),
			targetIndex = _index(target, options.draggable);

		if (dragElIndex < targetIndex) {
			return 1;
		} else {
			return -1;
		}
	}


	/**
	 * Generate id
	 * @param   {HTMLElement} el
	 * @returns {String}
	 * @private
	 */
	function _generateId(el) {
		var str = el.tagName + el.className + el.src + el.href + el.textContent,
			i = str.length,
			sum = 0;

		while (i--) {
			sum += str.charCodeAt(i);
		}

		return sum.toString(36);
	}

	/**
	 * Returns the index of an element within its parent for a selected set of
	 * elements
	 * @param  {HTMLElement} el
	 * @param  {selector} selector
	 * @return {number}
	 */
	function _index(el, selector) {
		var index = 0;

		if (!el || !el.parentNode) {
			return -1;
		}

		while (el && (el = el.previousElementSibling)) {
			if ((el.nodeName.toUpperCase() !== 'TEMPLATE') && el !== cloneEl) {
				index++;
			}
		}

		return index;
	}

	function _matches(/**HTMLElement*/el, /**String*/selector) {
		if (el) {
			try {
				if (el.matches) {
					return el.matches(selector);
				} else if (el.msMatchesSelector) {
					return el.msMatchesSelector(selector);
				} else if (el.webkitMatchesSelector) {
					return el.webkitMatchesSelector(selector);
				}
			} catch(_) {
				return false;
			}
		}

		return false;
	}

	var _throttleTimeout;
	function _throttle(callback, ms) {
		return function () {
			if (!_throttleTimeout) {
				var args = arguments,
					_this = this;

				_throttleTimeout = setTimeout(function () {
					if (args.length === 1) {
						callback.call(_this, args[0]);
					} else {
						callback.apply(_this, args);
					}

					_throttleTimeout = void 0;
				}, ms);
			}
		};
	}

	function _cancelThrottle() {
		clearTimeout(_throttleTimeout);
		_throttleTimeout = void 0;
	}

	function _extend(dst, src) {
		if (dst && src) {
			for (var key in src) {
				if (src.hasOwnProperty(key)) {
					dst[key] = src[key];
				}
			}
		}

		return dst;
	}

	function _clone(el) {
		if (Polymer && Polymer.dom) {
			return Polymer.dom(el).cloneNode(true);
		}
		else if ($) {
			return $(el).clone(true)[0];
		}
		else {
			return el.cloneNode(true);
		}
	}

	function _saveInputCheckedState(root) {
		savedInputChecked.length = 0;

		var inputs = root.getElementsByTagName('input');
		var idx = inputs.length;

		while (idx--) {
			var el = inputs[idx];
			el.checked && savedInputChecked.push(el);
		}
	}

	function _nextTick(fn) {
		return setTimeout(fn, 0);
	}

	function _cancelNextTick(id) {
		return clearTimeout(id);
	}


	function _preventScroll(evt) {
		if ((Sortable.active || awaitingDragStarted) && evt.cancelable) {
			evt.preventDefault();
		}
	}

	/**
	 * Returns the "bounding client rect" of given element
	 * @param  {HTMLElement} el                The element whose boundingClientRect is wanted
	 * @param  {[HTMLElement]} container       the parent the element will be placed in
	 * @param  {[Boolean]} adjustForTransform  Whether the rect should compensate for parent's transform
	 * (used for fixed positioning on el)
	 * @return {Object}                        The boundingClientRect of el
	 */
	function _getRect(el, container, adjustForTransform) {
		if (!el.getBoundingClientRect && el !== win) return;

		var elRect,
			top,
			left,
			bottom,
			right,
			height,
			width;

		if (el !== win) {
			elRect = el.getBoundingClientRect();
			top = elRect.top;
			left = elRect.left;
			bottom = elRect.bottom;
			right = elRect.right;
			height = elRect.height;
			width = elRect.width;
		} else {
			top = 0;
			left = 0;
			bottom = window.innerHeight;
			right = window.innerWidth;
			height = window.innerHeight;
			width = window.innerWidth;
		}

		if (adjustForTransform && el !== win) {
			// Adjust for translate()
			container = container || el.parentNode;

			// solves #1123 (see: https://stackoverflow.com/a/37953806/6088312)
			// Not needed on <= IE11
			if (!IE11OrLess) {
				do {
					if (container && container.getBoundingClientRect && _css(container, 'transform') !== 'none') {
						var containerRect = container.getBoundingClientRect();

						// Set relative to edges of padding box of container
						top -= containerRect.top + parseInt(_css(container, 'border-top-width'));
						left -= containerRect.left + parseInt(_css(container, 'border-left-width'));
						bottom = top + elRect.height;
						right = left + elRect.width;

						break;
					}
					/* jshint boss:true */
				} while (container = container.parentNode);
			}

			// Adjust for scale()
			var matrix = _matrix(el),
				scaleX = matrix && matrix.a,
				scaleY = matrix && matrix.d;

			if (matrix) {
				top /= scaleY;
				left /= scaleX;

				width /= scaleX;
				height /= scaleY;

				bottom = top + height;
				right = left + width;
			}
		}

		return {
			top: top,
			left: left,
			bottom: bottom,
			right: right,
			width: width,
			height: height
		};
	}


	/**
	 * Checks if a side of an element is scrolled past a side of it's parents
	 * @param  {HTMLElement}  el       The element who's side being scrolled out of view is in question
	 * @param  {String}       side     Side of the element in question ('top', 'left', 'right', 'bottom')
	 * @return {Boolean}               Whether the element is overflowing the viewport on the given side of it's parent
	 */
	function _isScrolledPast(el, side) {
		var parent = _getParentAutoScrollElement(parent, true),
			elSide = _getRect(el)[side];

		/* jshint boss:true */
		while (parent) {
			var parentSide = _getRect(parent)[side],
				visible;

			if (side === 'top' || side === 'left') {
				visible = elSide >= parentSide;
			} else {
				visible = elSide <= parentSide;
			}

			if (!visible) return true;

			if (parent === win) break;

			parent = _getParentAutoScrollElement(parent, false);
		}

		return false;
	}


	// Export utils
	Sortable.utils = {
		on: _on,
		off: _off,
		css: _css,
		find: _find,
		is: function (el, selector) {
			return !!_closest(el, selector, el, false);
		},
		extend: _extend,
		throttle: _throttle,
		closest: _closest,
		toggleClass: _toggleClass,
		clone: _clone,
		index: _index,
		nextTick: _nextTick,
		cancelNextTick: _cancelNextTick,
		detectDirection: _detectDirection,
		getChild: _getChild
	};


	/**
	 * Create sortable instance
	 * @param {HTMLElement}  el
	 * @param {Object}      [options]
	 */
	Sortable.create = function (el, options) {
		return new Sortable(el, options);
	};


	// Export
	Sortable.version = '1.8.0';
	return Sortable;
});


// FILE: src/config/config.js
(function () {

	window.templateUrls = window.templateUrls || [];
	var packageName = 'zh-grid-configuration';

	var hasPackageRegistered = window.templateUrls.some(function (templateUrl) {
		return templateUrl.package === packageName;
	});

	if (!hasPackageRegistered) {
		window.templateUrls.push({
			'package': packageName,
			'baseUrl': './bower_components/' + packageName + '/dist/templates/'
		});
	}
})();

// FILE: src/controllers/ZhGridConfigController.js
function ZhGridConfigController($scope, $timeout, $filter, ApplicationContext, ScreenService, metaDataFactory) {
	var self = this;
	var widget = $scope.widget;
	var widgetShowCheckbox = widget.showCheckbox;
	var GRID_MODE = 'grid';
	var REPORT_MODE = 'report';

	widget.reportShowOnlyVisibleFields = false;
	widget._selectedColumn = undefined;

	self.configMode = GRID_MODE;

	self.inGridMode = function(){
		return self.configMode == GRID_MODE;
	};
	$scope.inGridMode = self.inGridMode;

	self.inReportMode = function(){
		return self.configMode == REPORT_MODE;
	};
	$scope.inReportMode = self.inReportMode;

	self.setConfigMode = function(mode) {
		self.configMode = mode;
		widget.showCheckbox = self.inGridMode() && widgetShowCheckbox;
		widget._selectedColumn = undefined;

		$timeout(function() {
			$scope.updateFields(false);
		});
	};

	$scope.getConfigMode = function() {
		return self.configMode;
	};

	self.setReportOrientation = function(orientation) {
		widget.reportOrientation = orientation;

		$timeout(function() {
			$scope.calcColumnSize();
			widget.resizeGridColumns(true);
		});
	};

	self.getPageInfo = function() {
		return ScreenService.i18n("Page")+": 1<br />"+(new Date()).toLocaleString();
	};

	$scope.selectColumn = function(column) {
		if (widget._selectedColumn == column) {
			widget._selectedColumn = undefined;
		} else {
			widget._selectedColumn = column;
			if (Util.isUndefined(column.fixed)) {
				column.fixed = false;
			}

			$scope.updateFieldDetailsData(column);
		}
	};

	$scope.updateFieldDetailsData = function(column) {
		self.fieldDetailsWidget.dataSource.data[0] = {
			"LABEL": (self.inReportMode() && column.labelOnReport) || column.label,
			"ALIGNMENT": column.align,
			"EXPRESSION": column.expression
		};
		self.fieldDetailsWidget.moveToFirst();
		self.fieldDetailsWidget.getField("FIXED").setValue(column.fixed);
		self.fieldDetailsWidget.getField("OPENED").setValue(Util.isUndefinedOrNull(column.groupOpened) ? true : column.groupOpened);
		self.fieldDetailsWidget.getField("VISIBLE").setValue(self.inReportMode() ? column.isVisibleOnReport : column.showOnList);

		if (self.inReportMode()) {
			self.fieldDetailsWidget.getField("COLUMNSIZE").label = "Column Width (%)";
			if (!Util.isUndefinedOrNull(column.reportColumnSize)) {
				self.fieldDetailsWidget.getField("COLUMNSIZE").setValue(parseInt(column.reportColumnSize));
			}

			self.fieldDetailsWidget.getField("FIXED").isVisible = false;
		} else {
			self.fieldDetailsWidget.getField("COLUMNSIZE").label = "Column Width (px)";
			self.fieldDetailsWidget.getField("COLUMNSIZE").setValue(column.gridColumnSize);

			self.fieldDetailsWidget.getField("FIXED").isVisible = true;
		}

		self.fieldDetailsWidget.getField('ALIGNMENT').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('EXPRESSION').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('COLUMNSIZE').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('FIXED').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('VISIBLE').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('OPENED').isVisible = column.grouped && !self.inReportMode();
		self.fieldDetailsWidget.getField('LABEL').class = column.grouped && !self.inReportMode() ? 10 : 12;
	};

	$scope.calcColumnSize = function() {
		var totalWidth = $scope.getTotalWidth();

		widget._filteredFields.forEach(function(field) {
			field.minWidth = field.maxWidth = undefined;

			if (self.inReportMode()) {
				if (field.reportColumnSize) {
					field.minWidth = field.maxWidth = totalWidth * parseInt(field.reportColumnSize) / 100;
				}
			} else if (field.gridColumnSize) {
				field.minWidth = field.maxWidth = field.gridColumnSize;
			}
		});
	};

	$scope.calcColumnWidthProperties = function() {
		var totalWidth = $scope.getTotalWidth();

		widget._filteredFields.forEach(function(field) {
			if (field.minWidth) {
				if (self.inReportMode()) {
					field.reportColumnSize = Math.round(field.minWidth * 100 / totalWidth) + "%";
				} else {
					field.gridColumnSize = field.minWidth;
				}
			}
		});
	};

	$scope.updateFields = function(apply) {
		function _updateFields() {
			widget.updateFilteredFields();
			self.updateGroupedColumns();
			self.updateNotVisibleColumns();
			$scope.calcColumnSize();
		}

		if (apply !== false) {
			$scope.$apply(_updateFields);
		} else {
			_updateFields();
		}
	};

	widget.updateFilteredFields = function() {
		widget._filteredFields = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		widget._filteredFields = widget._filteredFields.filter(function(field) {
			return (self.inReportMode() ? field.isVisibleOnReport !== false : field.showOnList) && field.aclIsVisible && !field.grouped;
		});
	};

	self._groupedColumns = [];
	self.updateGroupedColumns = function() {
		self._groupedColumns = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		self._groupedColumns = self._groupedColumns.filter(function(field) {
			return field.grouped && field.aclIsVisible;
		});
	};

	self._notVisibleColumns = [];
	self.updateNotVisibleColumns = function() {
		self._notVisibleColumns = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		self._notVisibleColumns = self._notVisibleColumns.filter(function(field) {
			return !~widget._filteredFields.indexOf(field) && !~self._groupedColumns.indexOf(field) && field.aclIsVisible;
		});
	};

	$timeout(function() { $scope.updateFields(false); });

	self.layoutNameWidget = metaDataFactory.widgetFactory({
		"id": "grid-config-layout-name",
		"name": "grid-config-layout-name",
		"template": "widget/form-structure.html",
		"container": widget.container,
		"isVisible": true,
		"popupNoBlock": false,
		"showDescriptionOrder": false,
		"floatingControl": false,
		"dataSource": { "data": [] },
		"actions": [],
		"events": [],
		"fields": [
			{
				"label": "Layout name",
				"name": "LABEL",
				"class": "12",
				"showOnForm": true,
				"template": "field/text-edit.html",
				"events": [],
				"validations": {
					"required": {}
				}
			}
		]
	});
	self.layoutNameWidget.dataSource.data.push({ LABEL: widget.layoutName });
	self.layoutNameWidget.currentRow = self.layoutNameWidget.dataSource.data[0];

	$scope.$watch(function() { return self.layoutNameWidget.dataSource.data[0].LABEL; }, function(newValue) {
		if (!Util.isUndefinedOrNull(newValue)) {
			widget.layoutName = newValue;
		}
	});

	self.fieldDetailsWidget = metaDataFactory.widgetFactory({
		"id": "grid-config-field-details",
		"name": "grid-config-field-details",
		"label": "Field Details",
		"template": 'widget/form-structure.html',
		"container": widget.container,
		"isVisible": true,
		"popupNoBlock": false,
		"showDescriptionOrder": false,
		"floatingControl": false,
		"fieldGroups": [{
			"name": "FIELDGROUP",
			"opened": true,
			"collapsible": false,
			"label": "Column properties"
		}],
		"fields": [{
			"label": "Label",
			"name": "LABEL",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 12,
			"template": "field/text-edit.html"
		},{
			"label": "Alignment",
			"name": "ALIGNMENT",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/select-native.html",
			"dataSource": {
				"data": [
					{"value": "left"},
					{"value": "center"},
					{"value": "right"}
				]
			},
			"valueField": "value",
			"hideEmptyValue": true,
			"descriptionField": "value"
		},{
			"label": "Group expression",
			"name": "EXPRESSION",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/select-native.html",
			"dataSource": {
				"data": [
					{"value": "SUM"},
					{"value": "AVG"},
					{"value": "COUNT"},
					{"value": "MAX"},
					{"value": "MIN"}
				]
			},
			"valueField": "value",
			"hideEmptyValue": false,
			"descriptionField": "value"
		},{
			"label": "Column Width (%)",
			"name": "COLUMNSIZE",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/number.html",
			"spin": true
		},{
			"label": "Fixed",
			"name": "FIXED",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 1,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		},{
			"label": "Visible",
			"name": "VISIBLE",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 2,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		},{
			"label": "Opened",
			"name": "OPENED",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 2,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		}],
		"dataSource": { "data": [] },
		"actions": [],
		"events": []
	});

	var lastSelectedColumn;
	$scope.$watch(function() { return self.fieldDetailsWidget.dataSource.data[0]; }, function(newValue) {
		if (newValue && widget._selectedColumn && lastSelectedColumn == widget._selectedColumn) {
			var forceResize = false;

			if (self.inReportMode()) {
				widget._selectedColumn.labelOnReport = newValue.LABEL;

				widget._selectedColumn.reportColumnSize = undefined;
				if (!Util.isUndefinedOrNull(newValue.COLUMNSIZE) && widget._selectedColumn.reportColumnSize != newValue.COLUMNSIZE+"%") {
					widget._selectedColumn.reportColumnSize = newValue.COLUMNSIZE+"%";

					$scope.calcColumnSize();
					$scope.setFieldWidthFromElementWidth();
					$scope.calcColumnWidthProperties();
					forceResize = true;
				}

				if (widget._selectedColumn.isVisibleOnReport != newValue.VISIBLE) {
					widget._selectedColumn.isVisibleOnReport = newValue.VISIBLE;

					widget._selectedColumn = undefined;
					widget.updateFilteredFields();
					self.updateNotVisibleColumns();
					$scope.removeInvisibleColumns();
					forceResize = true;
				}
			} else {
				widget._selectedColumn.label = newValue.LABEL;

				widget._selectedColumn.gridColumnSize = undefined;
				if (!Util.isUndefinedOrNull(newValue.COLUMNSIZE) && widget._selectedColumn.gridColumnSize != newValue.COLUMNSIZE) {
					widget._selectedColumn.gridColumnSize = newValue.COLUMNSIZE;

					$scope.calcColumnSize();
					$scope.setFieldWidthFromElementWidth();
					$scope.calcColumnWidthProperties();
					forceResize = true;
				}

				if (widget._selectedColumn.showOnList != newValue.VISIBLE) {
					widget._selectedColumn.showOnList = newValue.VISIBLE;

					widget._selectedColumn = undefined;
					widget.updateFilteredFields();
					self.updateNotVisibleColumns();
					$scope.removeInvisibleColumns();
					forceResize = true;
				}
			}

			if (widget._selectedColumn) {
				widget._selectedColumn.align = newValue.ALIGNMENT;
				widget._selectedColumn.expression = newValue.EXPRESSION;
				widget._selectedColumn.groupOpened = newValue.OPENED;

				if (widget._selectedColumn.fixed != newValue.FIXED) {
					widget._selectedColumn.fixed = newValue.FIXED;

					widget.updateFilteredFields();
					var fixed = true;
					widget._filteredFields.forEach(function(field, idx) {
						field.sequenceInList = idx;
						fixed = fixed && field.fixed;
						field.fixed = fixed;
					});

					forceResize = true;
				}
			}

			if (forceResize) {
				widget.resizeGridColumns(true);
			}
		}
		lastSelectedColumn = widget._selectedColumn;
	}, true);

	self.previewPDF = function() {
		ApplicationContext.ZhReports.openReport(undefined, widget, undefined, undefined, 'PDF');
	};

	self.openSearch = function() {
		self.searchOpened = !self.searchOpened;
		if (self.searchOpened) {
			$('#zh-grid-config-not-visible-search input').focus();
		}
	};

	self.clearSearch = function() {
		delete self.notVisibleSearch;
	};

	self.getNotVisibleColumns = function() {
		return self._notVisibleColumns.filter(function(item) {
			return !self.notVisibleSearch || item.label.indexOfLatin(self.notVisibleSearch) > -1;
		}).sort(function(a, b) {
			a.label = a.label || "";
			b.label = b.label || "";
			var labelA = a.label.toUpperCase();
			var labelB = b.label.toUpperCase();
			if (labelA < labelB) {
				return -1;
			}
			if (labelA > labelB) {
				return 1;
			}
			return 0;
		});
	};

	widget.gridConfigWidget = self.layoutNameWidget;
}


// FILE: src/directives/ZhGridConfigDirective.js

(function () {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhGridConfig', zhGridConfig);

	function zhGridConfig(ApplicationContext) {
		return {
			restrict: 'A',
			link: function($scope, element) {
				var widget = $scope.widget;

				if (Util.isUndefinedOrNull(ApplicationContext.ZhReports)) {
					element.find('span.config-buttons').hide();
				}

				$scope.getTotalWidth = function() {
					return element.find('.header-content').outerWidth() - element.find('.header-content .th.grid-checkbox-column').outerWidth() - 10;
				};

				element.find('.body-content').off('mousewheel wheel DOMMouseScroll');

				var groupedColumns = element.find('.zh-grid-config-groups .sortable-columns')[0];
				var visibleColumns = element.find('.zh-grid-container .sortable-columns')[0];
				var notVisibleColumns = element.find('.zh-grid-config-not-visible .sortable-columns')[0];

				$scope.onStartSortable = function(evt) {
					var el = $(evt.item);
					var fieldName = el.attr('data-zh-field-name');
					var field = widget.getField(fieldName);

					if (field == widget._selectedColumn) {
						delete widget._selectedColumn;
					}
				};

				$scope.onAddSortable = function(evt) {
					var fieldName = $(evt.item).attr('data-zh-field-name');
					var field = widget.getField(fieldName);
					if (field) {
						if ($(evt.to).parent().hasClass('zh-grid-config-groups')) {
							field.showOnList = false;
							field.grouped = true;
							field.groupOpened = Util.isUndefinedOrNull(field.groupOpened) ? true : field.groupOpened;
						} else if ($(evt.to).parent().hasClass('zh-grid-config-not-visible')) {
							if ($scope.inReportMode()) {
								field.isVisibleOnReport = false;
							} else {
								field.showOnList = false;
							}
							field.grouped = false;
						} else {
							if ($scope.inReportMode()) {
								field.isVisibleOnReport = true;
							} else {
								field.showOnList = true;
							}
							field.isVisible = true;
							field.grouped = false;
						}
					}

					$scope.onUpdateSortable(evt);
				};

				$scope.onUpdateSortable = function(evt) {
					var fixed = true;
					$(evt.to).parent().find('.th').each(function(idx, item) {
						var fieldName = $(item).attr('data-zh-field-name');
						var field = widget.getField(fieldName);
						if (field) {
							if (!$(evt.to).parent().hasClass('zh-grid-config-not-visible')) {
								field.sequenceInList = idx;
							}

							if ($(evt.to).parent().hasClass('tr')) {
								fixed = fixed && field.fixed;
								field.fixed = fixed;
							}
						}
					});

					if (evt.type == "add") {
						$(evt.item).remove();
					}

					$scope.updateFields();
				};

				var sortableOptions = {
					group: "sortableList",
					onUpdate: $scope.onUpdateSortable,
					onAdd: $scope.onAddSortable,
					onStart: $scope.onStartSortable,
					animation: 100,
					handle: ".label-text",
					scroll: element.find('.body-scroll-control')[0]
				};

				var sortableItems = [];
				[groupedColumns, visibleColumns, notVisibleColumns].forEach(function(el) {
					sortableItems.push(Sortable.create(el, sortableOptions));
				});

				$scope.$on('$destroy', function() {
					sortableItems.forEach(function(item) {
						item.destroy();
					});
					sortableItems = [];

					delete $scope.onUpdateSortable;
					delete $scope.onAddSortable;
					delete $scope.onStartSortable;
				});

				element.on('touchend touchcancel mouseup pointercancel', function() {
					delete element[0]._tapEvt;
				});

				$scope.setFieldWidthFromElementWidth = function(force) {
					element.find('.header-content .sortable-columns .th').each(function(idx, item) {
						item = $(item);
						var itemField = widget.getField(item.attr('data-zh-field-name'));
						if (Util.isUndefinedOrNull(itemField.minWidth) || force) {
							itemField.minWidth = item.width();
							itemField.maxWidth = item.outerWidth();
						}
					});
				};

				element.on('touchmove mousemove', function(evt) {
					if (element[0]._tapEvt) {
						var tapElement = $(element[0]._tapEvt.target);
						var thElement = tapElement.closest('.th');
						var deltaX = evt.clientX - element[0]._tapEvt.clientX;

						if (deltaX === 0) {
							return;
						}

						if (thElement.offset().left + thElement.outerWidth() + deltaX > thElement.offsetParent().offset().left + thElement.offsetParent().outerWidth()) {
							return;
						}

						var field = widget.getField(thElement.attr('data-zh-field-name'));
						var width = thElement.width();
						var outerWidth = thElement.outerWidth();

						$scope.setFieldWidthFromElementWidth(true);

						if (tapElement.hasClass('resize-handle-left')) {
							if (thElement.prev().length) {
								thElement.width(width - deltaX);

								if (field) {
									field.minWidth = field.maxWidth = outerWidth - deltaX;
								}

								var prevElement = thElement.prev();
								if (prevElement) {
									var prevWidth = prevElement.width();
									var prevOuterWidth = prevElement.outerWidth();

									prevElement.width(prevWidth + deltaX);

									var prevField = widget.getField(prevElement.attr('data-zh-field-name'));
									if (prevField) {
										prevField.minWidth = prevField.maxWidth = prevOuterWidth + deltaX;
									}
								}
							}
						} else {
							thElement.width(width + deltaX);

							if (field) {
								field.minWidth = width + deltaX;
								field.maxWidth = outerWidth + deltaX;
							}
						}

						element[0]._tapEvt.clientX += deltaX;

						$scope.calcColumnWidthProperties();
						$scope.updateFieldDetailsData(field);

						widget.resizeGridColumns(true);
					}
				});

				$scope.removeInvisibleColumns = function() {
					var thItems = element.find('.header-content .sortable-columns .th');
					thItems.each(function(idx, th) {
						th = $(th);
						var fieldName = th.attr('data-zh-field-name');
						var field = widget.getField(fieldName);
						if (!field || widget._filteredFields.indexOf(field) == -1) {
							th.remove();
						}
					});
				};
			}
		};
	}

	ZeedhiDirectives.directive('zhGridConfigResizeHandle', zhGridConfigResizeHandle);

	function zhGridConfigResizeHandle() {
		return {
			restrict: 'A',
			link: function($scope, element) {
				var widget = $scope.widget;

				element.on('touchStart mousedown', function(evt) {
					element.closest('.zh-grid-config-container')[0]._tapEvt = evt;
				});

				element.on('touchend touchcancel mouseup pointercancel', function(evt) {
					delete element.closest('.zh-grid-config-container')[0]._tapEvt;
				});
			}
		};
	}
})();

// FILE: bower_components/zhulog/dist/main.min.js
/*! 13-05-2019 */

function ZhuLogService(){function createRows(data,value,parent){Object.keys(value).forEach(function(key){createRow(data,key,value[key],parent)})}function createRow(data,key,value,parent){var method;(method=Util.isObject(value)?factoryCreateObjectRowsMethod():Array.isArray(value)?factoryCreateArrayRowsMethod():createSimpleRow)(data,key,value,parent)}function factoryCreateObjectRowsMethod(){return factoryCreateObjectRows(": {}")}function factoryCreateArrayRowsMethod(){return factoryCreateObjectRows(": []")}function createSimpleRow(data,key,value,parent){"string"==typeof value&&(value='"'+value+'"'),value=key+": "+value,addData(key,value,parent,data)}function addData(key,value,parent,data){var id=parent+"#"+key;return data.push({ID:id,VALUE:value,PARENT:parent}),id}function factoryCreateObjectRows(emptySufix){return function(data,key,rowContent,parent){var rowValue=key;0===Object.keys(rowContent).length&&(rowValue+=emptySufix),createRows(data,rowContent,addData(key,rowValue,parent,data))}}this.getPlainContentRows=function(value){var data=[];return createRows(data,JSON.parse(value)),data}}function ZhuLogController(ZhuLogService,ScreenService,templateManager,metaDataFactory,ZhPreferences,ZhFilterPreferences,$timeout,requestEngine,requestFactory){function uniqid(){return Math.random().toString(10).substring(2,15)+Math.random().toString(10).substring(2,15)}function getContentWidget(){return templateManager.container.getWidget(CONTENT_WIDGET)}function getContentFormWidget(){return templateManager.container.getWidget(CONTENT_FORM_WIDGET)}function getContentGridWidget(){return templateManager.container.getWidget(CONTENT_GRID_WIDGET)}function getFilterWidget(){return templateManager.container.getWidget(CONTENT_FILTER_WIDGET)}function getZhuLogWidget(){return templateManager.container.getWidget(LOG_WIDGET_NAME)}function setStatusToContainersLabel(widget){requestEngine.doRequest(requestFactory.factory({serviceName:"zhulog#/logConfiguration/status",requestType:"filterData"})).then(function(response){widget.container.label=response.dataset.status[0]?"Log - Active":"Log - Inactive",widget.activate()})}var self=this,LOG_WIDGET_NAME="zhuLog",CONTENT_WIDGET="content",CONTENT_FORM_WIDGET="contentForm",CONTENT_GRID_WIDGET="contentGrid",CONTENT_FILTER_WIDGET="select";this.closeSwipe=function(widget){ScreenService.closeSwipe(),widget.container.getWidget(LOG_WIDGET_NAME).activate()},this.showActiveAll=function(widget){requestEngine.doRequest(requestFactory.factory({serviceName:"zhulog#/logConfiguration/areAllActive",requestType:"filterData"})).then(function(response){var status=response.dataset.status[0];widget.getAction("allowAll").isVisible=!status,widget.noMatchText=status?"All users allowed!":"No user allowed!",widget.activate()})},this.openPopUp=function(widget){var popUpDefaults={id:uniqid(),name:"controlPopUp",showDescriptionOrder:!1,label:"Allow User's Log",template:"widget/grid.html",showCheckbox:!0,isVisible:!0,itemsPerPage:30,events:[{name:"WidgetAfterSelectRow",code:"ZhuLogController.setPopUpAllowActionVisiblity(args.owner, true)"},{name:"WidgetAfterUnselectRow",code:"ZhuLogController.setPopUpAllowActionVisiblity(args.owner, false)"}],parent:widget,floatingControl:{searchAction:!0,customizationAction:{filter:!0,layout:!1,view:!1},xlsAction:!1,pdfAction:!1,csvAction:!1},dataSource:{rest:!0,localStorage:!1,data:[],name:"/operador",itemsPerPage:30},dataSourceFilter:[{name:"IDATIVO",operator:"=",value:"S"}],fields:[{id:uniqid(),name:"CDOPERADOR",label:"ID",operator:"NOT IN",template:"field/text.html",isVisible:!0,showOnList:!0},{id:uniqid(),name:"NMOPERADOR",label:"Name",template:"field/text.html",isVisible:!0,showOnList:!0},{id:uniqid(),name:"DSEMAILOPER",label:"Email",template:"field/text.html",isVisible:!0,showOnList:!0},{id:uniqid(),name:"NRORG",label:"Organization",template:"field/select.html",dataSource:{name:"/organization",rest:!0,localStorage:!1,data:[]},dataSourceFilter:[],valueField:"ID",descriptionField:"NAME",isVisible:!0,showOnList:!0}],actions:[{id:uniqid(),name:"allow",label:"Allow",isVisible:!1,showAsAction:"never",events:[{name:"ActionEvent",code:"ZhuLogController.setZhuLogConfiguration(args.owner.widget,allow)"}]},{id:uniqid(),name:"back",label:"Cancel",isVisible:!0,showAsAction:"back",events:[{name:"ActionEvent",code:"ScreenService.closePopup()"}]}]};widget.dataSource.data.length&&popUpDefaults.dataSourceFilter.push({name:"CDOPERADOR",operator:"NOT_IN",value:widget.dataSource.data.map(function(user){return user.USER_ID})});var popUp=metaDataFactory.widgetFactory(popUpDefaults,widget.container);ScreenService.openPopup(popUp)},this.setPopUpAllowActionVisiblity=function(widget,isVisible){widget.getAction("allow").isVisible=isVisible,widget.activate()},this.switchLogStatus=function(widget){var params={};requestEngine.doRequest(requestFactory.factory({serviceName:"zhulog#/logConfiguration/switch",requestType:"dataSource",dataSource:params})).then(function(response){widget.container.label=response.dataset.status[0]?"Log - Active":"Log - Inactive",widget.activate()})},this.setZhuLogConfiguration=function(widget,method){var checkedUsers=widget.getCheckedRows(),params={all:!1,deny:!1,allow:!1};switch(method){case"remove":params.deny=!0;break;case"all":params.all=!0}params.users=params.all?[]:checkedUsers,requestEngine.doRequest(requestFactory.factory({serviceName:"zhulog#/logConfiguration/update",requestType:"dataSource",dataSource:params})).then(function(){ScreenService.closePopup(!0),parentWidget=widget.container.getWidget("logConfigurationWidget"),self.showActiveAll(parentWidget),parentWidget.reload()})},this.openContent=function(widget){getContentFormWidget().currentRow=widget.currentRow;var rows=ZhuLogService.getPlainContentRows(widget.currentRow.CONTENT);getContentGridWidget().dataSource.data=rows,ScreenService.openSwipe(getContentWidget())};var startFilter=function(widget,currentRow){ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result){result&&ZhFilterPreferences.openFilterWidget(widget,currentRow)})},isAppliedFilter=function(widget){return!!widget.zhuLogAppliedFilter};this.zhuLogWidgetOnEnter=function(){var widget=getZhuLogWidget();$timeout(function(){if(widget.inListMode())if(isAppliedFilter(widget))widget.reload();else{var currentRow=widget.currentRowDefaultFilter||widget.currentRow;ZhFilterPreferences.setOriginalFilter(widget.id,currentRow),startFilter(widget,currentRow)}setStatusToContainersLabel(widget)},500,!1)},this.zhuLogFilterAfterApply=function(){getZhuLogWidget().zhuLogAppliedFilter=!0},this.openFilterPopUp=function(){ScreenService.openPopup(getFilterWidget())},this.setSelectValues=function(){var selectWidget=templateManager.container.getWidget("select"),select2=templateManager.container.getWidget("select").fields[1];"REQUEST"===selectWidget.currentRow.SNATIVE?(select2.label="Resquest type",select2.readOnly=!1,select2.dataSource.data=[],select2.dataSource.data.push({value:"Dataset"},{value:"Row"},{value:"Empty"},{value:"Filter"})):"RESPONSE"===selectWidget.currentRow.SNATIVE?(select2.label="Response type",select2.readOnly=!1,select2.dataSource.data=[],select2.dataSource.data.push({value:"Dataset"},{value:"Message"})):(select2.dataSource.data=[],this.clearFieldsValue(1),select2.readOnly=!0)},this.setTextEdit=function(){var selectWidget=templateManager.container.getWidget("select");selectWidget.actions[2].isVisible=!0,selectWidget.actions[3].isVisible=!0,"REQUEST"===selectWidget.currentRow.SNATIVE?("Dataset"!==selectWidget.currentRow.SNATIVE2&&"Row"!==selectWidget.currentRow.SNATIVE2||(selectWidget.fields[2].label="Column name",selectWidget.fields[3].label="Operator",selectWidget.fields[3].template="field/select-native.html",selectWidget.fields[4].label="Value",this.addFields()),"Empty"===selectWidget.currentRow.SNATIVE2&&(selectWidget.fields[2].label="Parameter",selectWidget.fields[3].label="Operator",selectWidget.fields[3].template="field/select-native.html",selectWidget.fields[4].label="Value",this.addFields()),"Filter"===selectWidget.currentRow.SNATIVE2&&(selectWidget.fields[2].label="Column name",selectWidget.fields[3].label="Column value",selectWidget.fields[3].template="field/text-edit.html",selectWidget.fields[4].label="Value",this.addFields()),null===selectWidget.currentRow.SNATIVE2&&this.removeExtraFields()):"RESPONSE"===selectWidget.currentRow.SNATIVE&&("Dataset"===selectWidget.currentRow.SNATIVE2&&(selectWidget.fields[2].label="Column name",selectWidget.fields[3].label="Operator",selectWidget.fields[3].template="field/select-native.html",selectWidget.fields[4].label="Value",this.addFields()),"Message"==selectWidget.currentRow.SNATIVE2&&(selectWidget.fields[2].label="Column name",selectWidget.fields[3].label="Operator",selectWidget.fields[3].template="field/select-native.html",selectWidget.fields[4].label="Value",this.addFields()),null===selectWidget.currentRow.SNATIVE2&&this.removeExtraFields())},this.removeExtraFields=function(){for(var selectWidget=templateManager.container.getWidget("select"),fields=selectWidget.fields,i=fields.length-1;i>6;i--)Util.isUndefined(fields[i])||(delete selectWidget.currentRow[fields[i].name],delete fields[i])},this.addFields=function(){for(var field,dataSource,selectWidget=templateManager.container.getWidget("select"),fields=templateManager.container.getWidget("select").fields,i=2;i<=6;i++)if(5!==i){var data;Util.isUndefined(fields[i].dataSource)||(data=fields[i].dataSource.data,dataSource={data:data}),field={label:fields[i].label,template:fields[i].template,name:"extraField"+fields.length,class:fields[i].class,events:fields[i].events,dataSource:dataSource,valueField:fields[i].valueField,descriptionField:fields[i].descriptionField,isVisible:!0,breakline:fields[i].breakline},metaDataFactory.fieldFactory(field,selectWidget,selectWidget)}else field={label:fields[i].label,template:fields[i].template,name:"extraField"+fields.length,class:fields[i].class,events:fields[i].events,dataSource:dataSource,valueField:fields[i].valueField,descriptionField:fields[i].descriptionField,isVisible:!1,breakline:fields[i].breakline},metaDataFactory.fieldFactory(field,selectWidget,selectWidget)},this.clearAllFields=function(){this.removeExtraFields(),this.clearFieldsValue(0)},this.clearFieldsValue=function(startingValue){for(var selectWidget=templateManager.container.getWidget("select"),i=startingValue;i<selectWidget.fields.length;i++)Util.isUndefined(selectWidget.fields[i])||""!==selectWidget.fields[i].value&&selectWidget.fields[i].value("");selectWidget.fields[1].readOnly=!0},this.clearFields=function(args){var fieldIndex=args.owner.field.sequenceInList;fieldIndex--;for(var selectWidget=templateManager.container.getWidget("select"),fields=selectWidget.fields,i=fieldIndex;i>=fieldIndex-4;i--)delete selectWidget.currentRow[fields[i].name],delete fields[i];templateManager.updateTemplate()},this.manageFields=function(args){var operatorIndex=args.owner.field.sequenceInList;operatorIndex--;var selectWidget=templateManager.container.getWidget("select"),selectedfield=selectWidget.fields[operatorIndex],selectedValue=selectedfield.value();"IS_NULL"===selectedValue||"IS_NOT_NULL"===selectedValue?(selectWidget.fields[operatorIndex+1].readOnly=!1,selectWidget.fields[operatorIndex+1].class=4,selectWidget.fields[operatorIndex+2].isVisible=!1,selectWidget.fields[operatorIndex+1].readOnly=!0):"BETWEEN"===selectedValue||"NOT_BETWEEN"===selectedValue?(selectWidget.fields[operatorIndex+1].class=2,selectWidget.fields[operatorIndex+1].label="Begin",selectWidget.fields[operatorIndex+2].isVisible=!0):(selectWidget.fields[operatorIndex+1].readOnly=!1,selectWidget.fields[operatorIndex+1].class=4,selectWidget.fields[operatorIndex+1].label="Value",selectWidget.fields[operatorIndex+2].isVisible=!1)},this.startFiltering=function(){for(var selectWidget=templateManager.container.getWidget("select"),fields=selectWidget.fields,i=9,j=0,params=[];i<fields.length;)if(Util.isUndefined(fields[i]))i+=5;else{var name=fields[i].value();i++;var operator=fields[i].value();i++;var value;"IS_NULL"===operator||"IS_NOT_NULL"===operator?(value="",i+=3):"BETWEEN"===operator||"NOT_BETWEEN"===operator?(value=[],value[0]=fields[i].value(),i++,value[1]=fields[i].value(),i+=2):(value=fields[i].value(),i+=3),params[j]={name:name,operator:operator,value:value},j++}}}function ZhuLogDecorator(ApplicationContext,templateManager){function getLogDecorator(){return void 0===logDecorator&&(logDecorator=templateManager.projectPromise.then(function(){var decoratorServiceName=templateManager.project.zhuLogDecoratorService;return ApplicationContext[decoratorServiceName]})),logDecorator}var logDecorator;this.afterLoadDataSource=function(args){getLogDecorator().then(function(logDecorator){if(logDecorator){var dataSource=args.owner.dataSource;dataSource.data=logDecorator.normalizeRows(dataSource.data)}})},this.beforeGridFilterApply=function(args){getLogDecorator().then(function(logDecorator){if(logDecorator){var _event=args.e;_event.filters=logDecorator.normalizeFilters(_event.preference.FILTER_ROW)}})},this.apply=function(containerName,container){return getLogDecorator().then(function(logDecorator){if(logDecorator&&"zhulog#log"===containerName){var logWidget=container.widgets[0];logWidget.events.push({name:"WidgetAfterLoadDataSource",code:"ZhuLogDecorator.afterLoadDataSource(args);"}),logWidget.events.push({name:"FilterBeforeApply",code:"ZhuLogDecorator.beforeGridFilterApply(args);"}),logDecorator.decorateWidget(logWidget)}return container})}}function ZhuLogHelper(){this.moveField=function(widget,fieldName,position){var index=-1;widget.fields.find(function(field){return index++,field.name===fieldName}),widget.fields.splice(position-1,0,widget.fields.splice(index,1)[0])},this.getField=function(widget,fieldName){var field=widget.fields.find(function(field){return field.name===fieldName});if(null===field)throw'Field "'+fieldName+'" not found';return field},this.addField=function(widget,field,position){position=(position||widget.fields.length)-1,widget.fields.splice(position,0,field)},this.removeField=function(widget,fieldName){widget.fields=widget.fields.filter(function(field){return field.name!==fieldName})},this.editField=function(widget,fieldName,editings){var field=this.getField(widget,fieldName);Util.extend(field,editings)},this.getFilters=function(filters,filterName){return filters.filter(function(filter){return filter.name===filterName})},this.addFilter=function(filters,filter){filters.push(filter)},this.removeFilter=function(filters,filterName){filters=filters.filter(function(filter){return filter.name!==filterName})}}Configuration(function(ContextRegister){ContextRegister.register("ZhuLogService",ZhuLogService)}),Configuration(function(ContextRegister){ContextRegister.register("ZhuLogController",ZhuLogController)}),Configuration(function(ContainerDecoratorProvider,ContextRegister,ApplicationContext,templateManager){ContextRegister.register("ZhuLogDecorator",ZhuLogDecorator),ContainerDecoratorProvider.registerProvider(new ZhuLogDecorator(ApplicationContext,templateManager))}),Configuration(function(ContextRegister){ContextRegister.register("ZhuLogHelper",ZhuLogHelper)});

// FILE: bower_components/zh-scheduler/dist/zh-scheduler.min.js
function ZhSchedulerService(eventEngine,ZhIconCalendarEvents,ZhCardsEvents){function registerWidgetOnDayClickEvent(){eventEngine.registerEvent("WidgetOnDayClick","schedulerCalendarDayClick"),ZhIconCalendarEvents.registerFunctionForDayClick("schedulerCalendarDayClick")}function registerWidgetOnEventClickEvent(){eventEngine.registerEvent("WidgetOnEventClick","schedulerCardClick"),ZhCardsEvents.registerFunctionForClick("schedulerCardClick")}!function(){registerWidgetOnDayClickEvent(),registerWidgetOnEventClickEvent()}()}function CardsController($scope,ZhCardsEvents){function updateCurrentRow(newValue){widget.currentRow=newValue}function callRegisteredClickEvents(){var registeredClickEvents=ZhCardsEvents.getRegisteredClickFunctions();registeredClickEvents.forEach(function(functionName){widget[functionName]&&widget[functionName]()})}var widget;!function(){widget=$scope.widget}(),this.buildBorderColor=function(card){var border=widget.color&&card[widget.color]?"5px solid "+card[widget.color]:"none";return{"border-left":border}},this.getFields=function(){return widget.fields},this.isFieldTitleType=function(field){return"title"===field.type},this.isFieldNormalType=function(field){return"title"!==field.type},this.getFieldContent=function(card,field){return card[field.name]},this.onClick=function(clickedCard){updateCurrentRow(clickedCard),callRegisteredClickEvents()},this.isThereResult=function(){return widget.dataSource&&widget.dataSource.data&&widget.dataSource.data.length>0},this.getNoResultText=function(){return widget.noMatchText||"No results were found."}}function ZhIconCalendarController($scope,eventAggregator,ZhIconCalendarEvents){function callOnDateClickEvents(){var dayClickEventFunctions=ZhIconCalendarEvents.getRegisteredFunctionsForDateClick();callEventFunctions(dayClickEventFunctions)}function isDayInCurrentMonth(day){return!day.lastMonth&&!day.nextMonth}function isDayBetweenMinAndMaxDate(day){var date=moment(day.date,$scope.dateFormat);return isDateAfterMinAllowed(date)&&isDateBeforeMaxAllowed(date)}function updateCurrentDateOnField(date){date=date||$scope.currentDate,calendarWidget.field.currentDate=date}function updateCalendarCurrentRow(newValue){calendarWidget.currentRow=newValue}function changeVisuallySelectedDay(clickedDay){lastDayClicked=clickedDay}function callOnMonthClickEvents(){var monthClickEventFunctions=ZhIconCalendarEvents.getRegisteredFunctionsForMonthClick();callEventFunctions(monthClickEventFunctions)}function isMonthBetweenMinAndMaxDate(month){var date=month.date,beginMonth=moment(date,CONTROL_FORMAT_MONTH).startOf("month"),endMonth=moment(date,CONTROL_FORMAT_MONTH).endOf("month");return isDateAfterMinAllowed(beginMonth)&&isDateBeforeMaxAllowed(endMonth)}function callOnYearClickEvents(){var yearClickEventFunctions=ZhIconCalendarEvents.getRegisteredFunctionsForYearClick();callEventFunctions(yearClickEventFunctions)}function isYearInCurrentDecade(year){return!year.lastDecade&&!year.nextDecade}function isYearBetweenMinAndMaxDate(year){var beginYear=moment(year.number,CONTROL_FORMAT_YEAR).startOf("year"),endYear=moment(year.number,CONTROL_FORMAT_YEAR).endOf("year");return isDateAfterMinAllowed(beginYear)&&isDateBeforeMaxAllowed(endYear)}function isYearInLastDecade(year){return!!year.lastDecade}function changeCurrentYearShownOnCalendar(totalOfYearsToChange){totalOfYearsToChange=totalOfYearsToChange||1,$scope.currentDate.add(10*totalOfYearsToChange,"year")}function callEventFunctions(functions){functions.forEach(function(functionName){calendarWidget[functionName]&&calendarWidget[functionName]()})}function isDateAfterMinAllowed(date){var isAfter=!0;return $scope.momentMinDate.isValid()&&(isAfter=date.isAfter($scope.momentMinDate)),isAfter}function isDateBeforeMaxAllowed(date){var isBefore=!0;return $scope.momentMaxDate.isValid()&&(isBefore=date.isBefore($scope.momentMaxDate)),isBefore}function isDayInLastMonth(day){return day.lastMonth}function changeCurrentMonthShownOnCalendar(totalOfMonthsToChange){totalOfMonthsToChange=totalOfMonthsToChange||1,$scope.currentDate.add(totalOfMonthsToChange,"month")}var calendarWidget,lastDayClicked,CONTROL_FORMAT_MONTH="YYYY-MM",CONTROL_FORMAT_YEAR="YYYY";!function(){calendarWidget=$scope.widget}(),this.onDateClick=function(clickedDay){if(isDayBetweenMinAndMaxDate(clickedDay))if(isDayInCurrentMonth(clickedDay))calendarWidget.field&&updateCurrentDateOnField($scope.currentDate),updateCalendarCurrentRow(clickedDay.date),changeVisuallySelectedDay(clickedDay);else{var totalOfMonthsToChange=isDayInLastMonth(clickedDay)?-1:1;changeCurrentMonthShownOnCalendar(totalOfMonthsToChange)}callOnDateClickEvents()},this.onMonthClick=function(clickedMonth){callOnMonthClickEvents(),isMonthBetweenMinAndMaxDate(clickedMonth)&&($scope.currentDate=moment(clickedMonth.date,CONTROL_FORMAT_MONTH),$scope.changeCalendarMode("day",!0))},this.onYearClick=function(clickedYear){if(callOnYearClickEvents(),isYearInCurrentDecade(clickedYear)&&isYearBetweenMinAndMaxDate(clickedYear))$scope.currentDate=moment(clickedYear.number,CONTROL_FORMAT_YEAR),$scope.changeCalendarMode("month",!0);else{var totalOfYearsToChange=isYearInLastDecade(clickedYear)?-1:1;changeCurrentYearShownOnCalendar(totalOfYearsToChange)}},this.isThereEventsOnDay=function(day){return day.events.length>0},this.getIconsFromDay=function(day){var iconProperty=calendarWidget.iconProperty,icons=[];return iconProperty&&(icons=day.events.map(function(item){return item[iconProperty]}),icons=icons.filter(function(item,position){return icons.indexOf(item)===position})),icons},this.isDaySelected=function(day){return day===lastDayClicked}}window.templateUrls=window.templateUrls||[];var packageName="zh-scheduler",hasPackageRegistered=window.templateUrls.some(function(templateUrl){return templateUrl["package"]===packageName});hasPackageRegistered||window.templateUrls.push({"package":packageName,baseUrl:"./bower_components/"+packageName+"/dist/templates/"}),ZeedhiServices.service("ZhCardsEvents",function(){function registerFunction(eventType,eventFunctionName){registeredEvents.hasOwnProperty(eventType)||(registeredEvents[eventType]=[]),registeredEvents[eventType].push(eventFunctionName)}var registeredEvents;!function(){registeredEvents={}}(),this.registerFunctionForClick=function(eventFunctionName){registerFunction("onClick",eventFunctionName)},this.getRegisteredClickFunctions=function(){return registeredEvents.onClick}}),ZeedhiServices.service("ZhIconCalendarEvents",function(){function registerFunction(eventType,eventFunctionName){calendarEvents.hasOwnProperty(eventType)||(calendarEvents[eventType]=[]),calendarEvents[eventType].push(eventFunctionName)}var calendarEvents;!function(){calendarEvents={onDateClick:["dateClick"],onMonthClick:["monthClick"],onYearClick:["yearClick"]}}(),this.registerFunctionForDayClick=function(eventFunctionName){registerFunction("onDateClick",eventFunctionName)},this.registerFunctionForMonthClick=function(eventFunctionName){registerFunction("onMonthClick",eventFunctionName)},this.registerFunctionForYearClick=function(eventFunctionName){registerFunction("onYearClick",eventFunctionName)},this.getRegisteredFunctionsForDateClick=function(){return calendarEvents.onDateClick},this.getRegisteredFunctionsForMonthClick=function(){return calendarEvents.onMonthClick},this.getRegisteredFunctionsForYearClick=function(){return calendarEvents.onYearClick}}),Configuration(function(ContextRegister){ContextRegister.register("ZhSchedulerService",ZhSchedulerService)});var ZhSchedulerController=function(_super){function ZhSchedulerController1($scope,$rootScope,$timeout,ScreenService,templateManager,ZHPromise,eventEngine,Query,CrosstabService,$filter,maskEngine,ZhUtil,$window,ApplicationContext,ZhFeatures,eventAggregator,DecodeValueFieldService,ZhIconCalendarEvents,ZhCardsEvents,metaDataFactory){function createCalendarWidgetMetaData(){calendarWidgetMetaData={container:getSchedulerContainer(),id:"ZhSchedulerCalendar",name:"ZH_SCHEDULER_CALENDAR",template:"zh-scheduler#widget/calendar.html",floatingControl:!1,dateFormat:schedulerWidget.dateFormat||"YYYY-MM-DD",dateField:schedulerWidget.dateField,iconProperty:schedulerWidget.iconProperty,dataSource:{rest:!1,localStorage:!1,data:schedulerWidget.dataSource.data},events:getCalendarEvents()}}function removeActionBackToCalendarFromScheduler(){templateManager.container.actions.forEach(function(widget,index){"ZH_SCHEDULER_BACK_TO_CALENDAR"===widget.name&&templateManager.container.actions.splice(index,1)})}function hideCalendar(){$scope.showCalendar(),removeActionBackToCalendarFromScheduler()}function getCalendarEvents(){var calendarEvents=getOnlyCalendarEventsFromScheduler();return calendarEvents=addDayClickEventForReloadEvents(calendarEvents),calendarEvents=addOnEnterEventForGetTodayEvents(calendarEvents)}function getOnlyCalendarEventsFromScheduler(){var calendarEvents=["WidgetOnDayClick","WidgetOnMonthClick","WidgetOnYearClick","WidgetOnChangeMonth","WidgetOnChangeYear","WidgetOnChangeDecade"];return schedulerWidget.events.filter(function(event){return calendarEvents.some(function(calendarEvent){return event.name===calendarEvent})})}function addDayClickEventForReloadEvents(events){return events.push({name:"WidgetOnDayClick",code:onDayClick}),events}function onDayClick(){$scope.isProbablyTabletDevice()&&($scope.hideCalendar(),addActionBackToCalendarOnScheduler()),prepareCardsWidgetToBeShown()}function prepareCardsWidgetToBeShown(){buildSevenDaysPeriod(),filterEventsDataByClickedDay()}function addOnEnterEventForGetTodayEvents(events){return events.push({name:"WidgetOnEnter",code:onEnterCalendar}),events}function addActionBackToCalendarOnScheduler(){templateManager.container.actions.push(Util.extend(new Action,backToCalendarActionMetaData))}function buildSevenDaysPeriod(){var currentDay=calendarWidget.currentRow,dateFormat=schedulerWidget.dateFormat;currentSevenDaysPeriod=[];for(var i=-3;i<4;i++){var day=moment(currentDay,dateFormat).add(i,"days").format(dateFormat);currentSevenDaysPeriod.push(day)}}function onEnterCalendar(){filterEventsDataByClickedDay()}function filterEventsDataByClickedDay(){var lastDayClicked=moment(calendarWidget.currentRow,schedulerWidget.dateFormat);cardsWidget.dataSource.data=schedulerWidget.dataSource.data.filter(function(data){var dayToCompare=moment(data[schedulerWidget.dateField],schedulerWidget.dateFormat);return dayToCompare.isSame(lastDayClicked,"day")})}function createCardsWidgetMetaData(){cardsWidgetMetaData={container:getSchedulerContainer(),id:"ZhSchedulerCards",name:"ZH_SCHEDULER_CARDS",template:"zh-scheduler#widget/cards.html",floatingControl:!1,color:schedulerWidget.color,fields:schedulerWidget.fields,noMatchText:schedulerWidget.noEventsText||"No events were found on this day.",events:getOnlyCardsEvents()}}function getOnlyCardsEvents(){var cardsEvents=["WidgetOnEventClick"];return schedulerWidget.events.filter(function(event){return cardsEvents.some(function(cardEvent){return event.name===cardEvent})})}function getSchedulerContainer(){return schedulerWidget.container}function factoryCalendarWidget(){calendarWidget=metaDataFactory.widgetFactory(calendarWidgetMetaData)}function factoryCardsWidget(){cardsWidget=metaDataFactory.widgetFactory(cardsWidgetMetaData)}function createBackToCalendarActionMetaData(){backToCalendarActionMetaData={name:"ZH_SCHEDULER_BACK_TO_CALENDAR",label:"Calendar",showAsAction:"back",isVisible:!0,widget:schedulerWidget,events:[{name:"ActionEvent",code:hideCalendar}]}}function addUsefulFunctionsToSchedulerWidget(){schedulerWidget.getCalendarWidget=function(){return calendarWidget},schedulerWidget.getEventsWidget=function(){return cardsWidget},schedulerWidget.reloadSchedulerWidget=function(){schedulerWidget.reload(),templateManager.updateTemplate()}}var calendarWidget,calendarWidgetMetaData,cardsWidget,cardsWidgetMetaData,backToCalendarActionMetaData,schedulerWidget=(ZhSchedulerController.__super__.constructor.apply(this,arguments),$scope.widget),currentSevenDaysPeriod=[];!function(){schedulerWidget.activate(),createCalendarWidgetMetaData(),factoryCalendarWidget(),createCardsWidgetMetaData(),factoryCardsWidget(),createBackToCalendarActionMetaData(),addUsefulFunctionsToSchedulerWidget()}(),this.getCalendarWidgetTemplate=function(){return templateManager.getTemplateURL("zh-scheduler#widget/calendar.html")},this.getCardsWidgetTemplate=function(){return templateManager.getTemplateURL("zh-scheduler#widget/cards.html")},this.getSevenDaysPeriodTemplate=function(){return templateManager.getTemplateURL("zh-scheduler#widget/seven-days-period.html")},this.getCalendarWidget=function(){return[calendarWidget]},this.getCardsWidget=function(){return[cardsWidget]},this.getDaysFromSevenDaysPeriod=function(){var dateFormat=schedulerWidget.dateFormat,onlyDays=[];return currentSevenDaysPeriod.forEach(function(day){day=moment(day,dateFormat).format(dateFormat),onlyDays.push(day)}),onlyDays},this.getOnlyWeekDayNamesFromSevenDaysPeriod=function(){var weekDayNames=moment.weekdaysShort(),dateFormat=schedulerWidget.dateFormat,onlyWeekDayNames=[];return currentSevenDaysPeriod.forEach(function(day){var dayIndex=moment(day,dateFormat).weekday();onlyWeekDayNames.push(weekDayNames[dayIndex])}),onlyWeekDayNames},this.isDaySameAsSelected=function(day){var dateFormat=schedulerWidget.dateFormat,selectedDay=moment(calendarWidget.currentRow,dateFormat).format(dateFormat);return day===selectedDay},this.onCurrentWeekDayClick=function(day){calendarWidget.currentRow=day,prepareCardsWidgetToBeShown()},this.getOnlyDayOfMonth=function(day){var dateFormat=schedulerWidget.dateFormat;return moment(day,dateFormat).date().toString()},this.shouldShowEvents=function(){return!$scope.calendarVisibilityStatus||!$scope.isProbablyTabletDevice()},$scope.$watchCollection(function(){return schedulerWidget.dataSource.data},function(newValue){filterEventsDataByClickedDay(),calendarWidget.dataSource.data=newValue})}return __extends(ZhSchedulerController1,_super),ZhSchedulerController1}(ListController);!function(){function zhIconCalendar(ZhUtil,ScreenService){return{restrict:"A",link:function($scope,element){function updateMaxAndMinDates(){$scope.maxDate="today"===$scope.widget.maxDate?moment().format($scope.dateFormat):$scope.widget.maxDate,$scope.momentMaxDate=moment($scope.maxDate,$scope.dateFormat),$scope.minDate="today"===$scope.widget.minDate?moment().format($scope.dateFormat):$scope.widget.minDate,$scope.momentMinDate=moment($scope.minDate,$scope.dateFormat)}function isBlockedDate(date){var dayMoment=moment(date,$scope.dateFormat);return $scope.momentMinDate.isValid()&&dayMoment.isBefore($scope.momentMinDate)||$scope.momentMaxDate.isValid()&&dayMoment.isAfter($scope.momentMaxDate)}function isBlockedMonth(monthDate){var monthMomentBegining=moment(monthDate,controlFormatMonth).startOf(UNIT_MONTH),monthMomentEnd=moment(monthDate,controlFormatMonth).endOf(UNIT_MONTH);return $scope.momentMaxDate.isValid()&&monthMomentBegining.isAfter($scope.momentMaxDate)||$scope.momentMinDate.isValid()&&monthMomentEnd.isBefore($scope.momentMinDate)}function isBlockedYear(yearNumber){var yearMomentBegining=moment(yearNumber,controlFormatYear).startOf(UNIT_YEAR),yearMomentEnd=moment(yearNumber,controlFormatYear).endOf(UNIT_YEAR);return $scope.momentMaxDate.isValid()&&yearMomentBegining.isAfter($scope.momentMaxDate)||$scope.momentMinDate.isValid()&&yearMomentEnd.isBefore($scope.momentMinDate)}function isSelectedDate(date){return widget.selectedDates.indexOf(date)!=-1}function changeCalendarMode(mode,changeToParameterMode){var newMode=mode;if(!changeToParameterMode)switch(mode){case MODE_DAY:newMode=MODE_MONTH;break;case MODE_MONTH:newMode=MODE_YEAR}$scope.calendarMode=newMode}function navigateYearMode(value){$scope.currentDate.add(10*value,UNIT_YEAR)}function navigateMonthMode(value){$scope.currentDate.add(value,UNIT_YEAR)}function navigateDayMode(value){$scope.currentDate.add(value,UNIT_MONTH)}function buildCalendar(mode){switch(mode){case MODE_YEAR:buildYearCalendar();break;case MODE_MONTH:buildMonthCalendar();break;case MODE_DAY:buildDayCalendar()}}function buildDayCalendar(){var current=$scope.currentDate.clone().startOf(UNIT_WEEK).day(0);current.format(controlFormat)===$scope.currentDate.clone().format(controlFormat)&&current.subtract(7,UNIT_DAY);var monthEvents={};$scope.widget.dataSource&&$scope.widget.dataSource.data&&$scope.widget.dataSource.data.filter(function(item){var month=moment(item[$scope.widget.dateField],$scope.dateFormat).get(UNIT_MONTH),currMonth=current.get(UNIT_MONTH),isPreviousMonth=month===currMonth,isCurrentMonth=month===(currMonth+1)%12,isNextMonth=month===(currMonth+2)%12;return isPreviousMonth||isCurrentMonth||isNextMonth}).forEach(function(item){var itemTime=moment(item[$scope.widget.dateField],$scope.dateFormat).startOf(UNIT_DAY).format(controlFormat);monthEvents[itemTime]||(monthEvents[itemTime]=[]),monthEvents[itemTime].push(item)}),$scope.weeks=[];for(var i=0;i<6;i++){for(var week={days:[]},j=0;j<7;j++)week.days.push({day:current.date(),date:current.format($scope.dateFormat),today:current.format(controlFormat)===$scope.today.format(controlFormat),lastMonth:current.month()<$scope.currentDate.month()&&current.year()==$scope.currentDate.year()||current.month()>$scope.currentDate.month()&&current.year()<$scope.currentDate.year(),nextMonth:current.month()>$scope.currentDate.month()&&current.year()==$scope.currentDate.year()||current.month()<$scope.currentDate.month()&&current.year()>$scope.currentDate.year(),events:monthEvents[current.format(controlFormat)]||[],selected:isSelectedDate(current.format(controlFormat))}),current.add(1,UNIT_DAY);$scope.weeks.push(week)}}function buildMonthCalendar(){var auxiliarRow=[];$scope.monthRows=[],$scope.currentDate.startOf(UNIT_YEAR);for(var year=$scope.currentDate.format(controlFormatYear),shortMonthNames=moment.monthsShort(),i=0;i<12;i++)auxiliarRow.push({date:moment("01/"+(i+1)+"/"+year,"DD/MM/YYYY").format(controlFormatMonth),selected:!1,name:shortMonthNames[i]}),4==auxiliarRow.length&&($scope.monthRows.push({months:auxiliarRow}),auxiliarRow=[])}function buildYearCalendar(){var auxiliarRow=[];$scope.yearRows=[];var currentYear=parseInt($scope.currentDate.format(controlFormatYear)),gridInitialYear=10*Math.floor(currentYear/10);$scope.currentDate=moment(gridInitialYear,controlFormatYear);for(var i=-1;i<11;i++)auxiliarRow.push({number:moment("01/01/"+(gridInitialYear+i),"DD/MM/YYYY").format(controlFormatYear),selected:!1,lastDecade:i==-1,nextDecade:10==i}),4==auxiliarRow.length&&($scope.yearRows.push({years:auxiliarRow}),auxiliarRow=[])}function triggerChangeEvents(currentMoment,oldMoment){var monthBeforeChange=oldMoment.month(),monthAfterChange=currentMoment.month(),yearBeforeChange=oldMoment.year(),yearAfterChange=currentMoment.year(),decadeBeforeChange=10*Math.floor(oldMoment.year()/10),decadeAfterChange=10*Math.floor(currentMoment.year()/10);monthBeforeChange!=monthAfterChange&&triggerMonthChangeEvent(),yearBeforeChange!=yearAfterChange&&triggerYearChangeEvent(),decadeBeforeChange!=decadeAfterChange&&triggerDecadeChangeEvent()}function triggerMonthChangeEvent(){widget.onChangeMonth&&widget.onChangeMonth()}function triggerYearChangeEvent(){widget.onChangeYear&&widget.onChangeYear()}function triggerDecadeChangeEvent(){widget.onChangeDecade&&widget.onChangeDecade()}var controlFormat="YYYY-MM-DD",controlFormatMonth="YYYY-MM",controlFormatYear="YYYY",MODE_DAY="day",MODE_MONTH="month",MODE_YEAR="year",UNIT_DAY="day",UNIT_WEEK="week",UNIT_MONTH="month",UNIT_YEAR="year",widget=$scope.widget;widget.selectedDates=[],$scope.calendarMode=widget.calendarMode||MODE_DAY,$scope.today=moment().startOf(UNIT_DAY),$scope.widget.currentRow=$scope.today.format($scope.widget.dateFormat),$scope.dateFormat=$scope.widget.dateFormat||"L",widget.field?widget.field.currentDate?$scope.currentDate=moment(widget.field.currentDate,widget.field.dateFormat):widget.field.defaultValue&&($scope.currentDate=moment(ZhUtil.evaluate(widget.field.defaultValue),$scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY)):widget.currentDate&&($scope.currentDate=moment(widget.currentDate,$scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY)),$scope.currentDate&&$scope.currentDate.isValid()||($scope.currentDate=moment().startOf(UNIT_MONTH).startOf(UNIT_DAY)),$scope.todayText=$scope.widget.todayText||ScreenService.i18n("Today"),$scope.weekDays=$scope.widget.weekDays||moment.weekdaysShort(),$scope.monthNames=$scope.widget.monthNames||moment.months(),$scope.maxDate="",$scope.momentMaxDate="",$scope.minDate="",$scope.momentMinDate="",widget.setDateAsSelected=function(dateStr,dateFormat){widget.selectedDates.push(moment(dateStr,dateFormat||widget.dateFormat||controlFormat).format(controlFormat))},updateMaxAndMinDates(),$scope.isWeekdayToday=function(weekdayIndex){var todayWeekday=moment().day();return weekdayIndex===todayWeekday},$scope.isBlockedDate=isBlockedDate,$scope.isBlockedMonth=isBlockedMonth,$scope.isBlockedYear=isBlockedYear,$scope.changeCalendarMode=changeCalendarMode,$scope.getCalendarHeading=function(mode){var heading="";switch(mode){case MODE_YEAR:heading=$scope.yearRows?$scope.yearRows[0].years[0].number+" - "+$scope.yearRows[2].years[3].number:"";break;case MODE_MONTH:heading=$scope.currentDate.format(controlFormatYear);break;case MODE_DAY:heading=$scope.currentDate.format("MMMM YYYY")}return heading},$scope.navigateCalendar=function(value){switch($scope.calendarMode){case MODE_YEAR:navigateYearMode(value);break;case MODE_MONTH:navigateMonthMode(value);break;case MODE_DAY:navigateDayMode(value)}},$scope.$watch("calendarMode",function(){buildCalendar($scope.calendarMode)}),$scope.$watch(function(){return $scope.currentDate._d.getTime()},function(newTimestamp,oldTimestamp){var currentMoment=moment(new Date(newTimestamp)),oldMoment=moment(new Date(oldTimestamp));buildCalendar($scope.calendarMode),triggerChangeEvents(currentMoment,oldMoment)}),$scope.$watchCollection("widget.selectedDates",function(){buildCalendar($scope.calendarMode)}),$scope.$watchCollection(function(){if($scope.widget.dataSource)return $scope.widget.dataSource.data},function(){buildCalendar($scope.calendarMode)}),$scope.$watch(function(){return $scope.widget.minDate},function(){$scope.widget.minDate&&updateMaxAndMinDates()}),$scope.$watch(function(){return $scope.widget.maxDate},function(){$scope.widget.maxDate&&updateMaxAndMinDates()}),$scope.widget.getYear=function(){return $scope.currentDate.get("year")},$scope.widget.getMonthName=function(){return $scope.monthNames[$scope.currentDate.get("month")]},$scope.widget.getMonth=function(){return $scope.currentDate.get("month")}}}}var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhIconCalendar",zhIconCalendar)}(),function(){function zhScheduler(){var scope,element,attributes,controller,linkFunction=function(theScope,theElement,theAttributes,theController){function getDomCalendarByParentElement(element){return element.find("section.zh-icon-calendar")}scope=theScope,element=theElement,attributes=theAttributes,controller=theController,scope.calendarVisibilityStatus=!0,scope.hideCalendar=function(){var domCalendar=getDomCalendarByParentElement(element);scope.setCalendarVisibilityStatus(!1),domCalendar.slideUp()},scope.showCalendar=function(){var domCalendar=getDomCalendarByParentElement(element);scope.setCalendarVisibilityStatus(!0),domCalendar.slideDown()},scope.isProbablyTabletDevice=function(){return $(document).width()<769},scope.setCalendarVisibilityStatus=function(status){scope.calendarVisibilityStatus=status}};return{restrict:"A",link:linkFunction}}var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhScheduler",zhScheduler)}();

// FILE: bower_components/zh-rich-text/dist/zh-rich-text.min.js
function richTextSanitize(htmlText){function customPolicy(tagName,attribs){if("img"==tagName&&attribs.length){for(var src,i=0;i<attribs.length;i+=2)if("src"==attribs[i]){src=attribs[i+1];break}if(src&&0===src.indexOf("data:image")){var attrs=html.sanitizeAttribs(tagName,attribs);return attrs.push("src"),attrs.push(src),{attribs:attrs}}}return basicPolicy(tagName,attribs)}var basicPolicy=html.makeTagPolicy();return htmlText=htmlText||"",html.sanitizeWithPolicy(htmlText,customPolicy)}function ZhRichTextController($scope){var field=$scope.field,toolbarDefaultOptions={undo:!0,redo:!0,bold:!0,italic:!0,underline:!0,superscript:!0,subscript:!0,strikethrough:!0,clear:!0,fontname:!0,fontsize:!0,color:!0,ul:!0,ol:!0,paragraph:!0,lineheight:!0,table:!0,picture:!0,codeview:!1};toolbarDefaultOptions=$.extend(toolbarDefaultOptions,field.toolbar||{}),$scope.editorOptions={height:field.height,dialogsInBody:!0,lang:"pt-BR",toolbar:[["edit",["undo","redo"]],["style",["bold","italic","underline","superscript","subscript","strikethrough","clear"]],["fontface",["fontname"]],["textsize",["fontsize"]],["fontclr",["color"]],["alignment",["ul","ol","paragraph","lineheight"]],["table",["table"]],["insert",["picture"]],["view",["codeview"]]],callbacks:{onInit:function(){$scope.onInitElement()},onChange:function(){'<p><span style="font-size: 14px;">\ufeff</span><br></p>'!==arguments[0]&&$scope.onInitElement(arguments)}}},$scope.getLabelToRichText=function(field){return field.help?this.templateManager.getTemplateURL("field/label-with-tooltip.html"):this.templateManager.getTemplateURL("field/label-static.html")},Object.keys(toolbarDefaultOptions).forEach(function(key){toolbarDefaultOptions[key]===!1&&$scope.editorOptions.toolbar.forEach(function(group,index){var groupButtons=group[1],buttonIdx=groupButtons.indexOf(key);buttonIdx>-1&&groupButtons.splice(buttonIdx,1),0===groupButtons.length&&$scope.editorOptions.toolbar.splice(index,1)})})}!function(){var O,c=void 0,n=!0,s=null,C=!1,J=["aliceblue,antiquewhite,aqua,aquamarine,azure,beige,bisque,black,blanchedalmond,blue,blueviolet,brown,burlywood,cadetblue,chartreuse,chocolate,coral,cornflowerblue,cornsilk,crimson,cyan,darkblue,darkcyan,darkgoldenrod,darkgray,darkgreen,darkkhaki,darkmagenta,darkolivegreen,darkorange,darkorchid,darkred,darksalmon,darkseagreen,darkslateblue,darkslategray,darkturquoise,darkviolet,deeppink,deepskyblue,dimgray,dodgerblue,firebrick,floralwhite,forestgreen,fuchsia,gainsboro,ghostwhite,gold,goldenrod,gray,green,greenyellow,honeydew,hotpink,indianred,indigo,ivory,khaki,lavender,lavenderblush,lawngreen,lemonchiffon,lightblue,lightcoral,lightcyan,lightgoldenrodyellow,lightgreen,lightgrey,lightpink,lightsalmon,lightseagreen,lightskyblue,lightslategray,lightsteelblue,lightyellow,lime,limegreen,linen,magenta,maroon,mediumaquamarine,mediumblue,mediumorchid,mediumpurple,mediumseagreen,mediumslateblue,mediumspringgreen,mediumturquoise,mediumvioletred,midnightblue,mintcream,mistyrose,moccasin,navajowhite,navy,oldlace,olive,olivedrab,orange,orangered,orchid,palegoldenrod,palegreen,paleturquoise,palevioletred,papayawhip,peachpuff,peru,pink,plum,powderblue,purple,red,rosybrown,royalblue,saddlebrown,salmon,sandybrown,seagreen,seashell,sienna,silver,skyblue,slateblue,slategray,snow,springgreen,steelblue,tan,teal,thistle,tomato,transparent,turquoise,violet,wheat,white,whitesmoke,yellow,yellowgreen".split(","),"all-scroll,col-resize,crosshair,default,e-resize,hand,help,move,n-resize,ne-resize,no-drop,not-allowed,nw-resize,pointer,progress,row-resize,s-resize,se-resize,sw-resize,text,vertical-text,w-resize,wait".split(","),"armenian,decimal,decimal-leading-zero,disc,georgian,lower-alpha,lower-greek,lower-latin,lower-roman,square,upper-alpha,upper-latin,upper-roman".split(","),"100,200,300,400,500,600,700,800,900,bold,bolder,lighter".split(","),"block-level,inline-level,table-caption,table-cell,table-column,table-column-group,table-footer-group,table-header-group,table-row,table-row-group".split(","),"condensed,expanded,extra-condensed,extra-expanded,narrower,semi-condensed,semi-expanded,ultra-condensed,ultra-expanded,wider".split(","),"inherit,inline,inline-block,inline-box,inline-flex,inline-grid,inline-list-item,inline-stack,inline-table,run-in".split(","),"behind,center-left,center-right,far-left,far-right,left-side,leftwards,right-side,rightwards".split(","),"large,larger,small,smaller,x-large,x-small,xx-large,xx-small".split(","),"dashed,dotted,double,groove,outset,ridge,solid".split(","),"ease,ease-in,ease-in-out,ease-out,linear,step-end,step-start".split(","),"at,closest-corner,closest-side,ellipse,farthest-corner,farthest-side".split(","),"baseline,middle,sub,super,text-bottom,text-top".split(","),"caption,icon,menu,message-box,small-caption,status-bar".split(","),"fast,faster,slow,slower,x-fast,x-slow".split(","),["above","below","higher","level","lower"],["cursive","fantasy","monospace","sans-serif","serif"],["loud","silent","soft","x-loud","x-soft"],["no-repeat","repeat-x","repeat-y","round","space"],["blink","line-through","overline","underline"],["block","flex","grid","table"],["high","low","x-high","x-low"],["nowrap","pre","pre-line","pre-wrap"],["absolute","relative","static"],["alternate","alternate-reverse","reverse"],["border-box","content-box","padding-box"],["capitalize","lowercase","uppercase"],["child","female","male"],["=","opacity"],["backwards","forwards"],["bidi-override","embed"],["bottom","top"],["break-all","keep-all"],["clip","ellipsis"],["contain","cover"],["continuous","digits"],["end","start"],["flat","preserve-3d"],["hide","show"],["horizontal","vertical"],["inside","outside"],["italic","oblique"],["left","right"],["ltr","rtl"],["no-content","no-display"],["paused","running"],["suppress","unrestricted"],["thick","thin"],[","],["/"],["all"],["always"],["auto"],["avoid"],["both"],["break-word"],["center"],["circle"],["code"],["collapse"],["contents"],["fixed"],["hidden"],["infinite"],["inset"],["invert"],["justify"],["list-item"],["local"],["medium"],["mix"],["none"],["normal"],["once"],["repeat"],["scroll"],["separate"],["small-caps"],["spell-out"],["to"],["visible"]],L={animation:{cssPropBits:517,cssLitGroup:[J[10],J[24],J[29],J[45],J[48],J[54],J[63],J[71],J[72]],cssFns:["cubic-bezier()","steps()"]},"animation-delay":{cssPropBits:5,cssLitGroup:[J[48]],cssFns:[]},"animation-direction":{cssPropBits:0,cssLitGroup:[J[24],J[48],J[72]],cssFns:[]},"animation-duration":"animation-delay","animation-fill-mode":{cssPropBits:0,cssLitGroup:[J[29],J[48],J[54],J[71]],cssFns:[]},"animation-iteration-count":{cssPropBits:5,cssLitGroup:[J[48],J[63]],cssFns:[]},"animation-name":{cssPropBits:512,cssLitGroup:[J[48],J[71]],cssFns:[]},"animation-play-state":{cssPropBits:0,cssLitGroup:[J[45],J[48]],cssFns:[]},"animation-timing-function":{cssPropBits:0,cssLitGroup:[J[10],J[48]],cssFns:["cubic-bezier()","steps()"]},appearance:{cssPropBits:0,cssLitGroup:[J[71]],cssFns:[]},azimuth:{cssPropBits:5,cssLitGroup:[J[7],J[42],J[56]],cssFns:[]},"backface-visibility":{cssPropBits:0,cssLitGroup:[J[59],J[62],J[80]],cssFns:[]},background:{cssPropBits:23,cssLitGroup:[J[0],J[18],J[25],J[31],J[34],J[42],J[48],J[49],J[52],J[56],J[61],J[68],J[71],J[74],J[75]],cssFns:"image(),linear-gradient(),radial-gradient(),repeating-linear-gradient(),repeating-radial-gradient(),rgb(),rgba()".split(",")},"background-attachment":{cssPropBits:0,cssLitGroup:[J[48],J[61],J[68],J[75]],cssFns:[]},"background-color":{cssPropBits:2,cssLitGroup:[J[0]],cssFns:["rgb()","rgba()"]},"background-image":{cssPropBits:16,cssLitGroup:[J[48],J[71]],cssFns:["image()","linear-gradient()","radial-gradient()","repeating-linear-gradient()","repeating-radial-gradient()"]},"background-position":{cssPropBits:5,cssLitGroup:[J[31],J[42],J[48],J[56]],cssFns:[]},"background-repeat":{cssPropBits:0,cssLitGroup:[J[18],J[48],J[74]],cssFns:[]},"background-size":{cssPropBits:5,cssLitGroup:[J[34],J[48],J[52]],cssFns:[]},border:{cssPropBits:7,cssLitGroup:[J[0],J[9],J[47],J[62],J[64],J[69],J[71]],cssFns:["rgb()","rgba()"]},"border-bottom":"border","border-bottom-color":"background-color","border-bottom-left-radius":{cssPropBits:5,cssFns:[]},"border-bottom-right-radius":"border-bottom-left-radius","border-bottom-style":{cssPropBits:0,cssLitGroup:[J[9],J[62],J[64],J[71]],cssFns:[]},"border-bottom-width":{cssPropBits:5,cssLitGroup:[J[47],J[69]],cssFns:[]},"border-collapse":{cssPropBits:0,cssLitGroup:[J[59],J[76]],cssFns:[]},"border-color":"background-color","border-left":"border","border-left-color":"background-color","border-left-style":"border-bottom-style","border-left-width":"border-bottom-width","border-radius":{cssPropBits:5,cssLitGroup:[J[49]],cssFns:[]},"border-right":"border","border-right-color":"background-color","border-right-style":"border-bottom-style","border-right-width":"border-bottom-width","border-spacing":"border-bottom-left-radius","border-style":"border-bottom-style","border-top":"border","border-top-color":"background-color","border-top-left-radius":"border-bottom-left-radius","border-top-right-radius":"border-bottom-left-radius","border-top-style":"border-bottom-style","border-top-width":"border-bottom-width","border-width":"border-bottom-width",bottom:{cssPropBits:5,cssLitGroup:[J[52]],cssFns:[]},box:{cssPropBits:0,cssLitGroup:[J[60],J[71],J[72]],cssFns:[]},"box-shadow":{cssPropBits:7,cssLitGroup:[J[0],J[48],J[64],J[71]],cssFns:["rgb()","rgba()"]},"box-sizing":{cssPropBits:0,cssLitGroup:[J[25]],cssFns:[]},"caption-side":{cssPropBits:0,cssLitGroup:[J[31]],cssFns:[]},clear:{cssPropBits:0,cssLitGroup:[J[42],J[54],J[71]],cssFns:[]},clip:{cssPropBits:0,cssLitGroup:[J[52]],cssFns:["rect()"]},color:"background-color",content:{cssPropBits:8,cssLitGroup:[J[71],J[72]],cssFns:[]},cue:{cssPropBits:16,cssLitGroup:[J[71]],cssFns:[]},"cue-after":"cue","cue-before":"cue",cursor:{cssPropBits:16,cssLitGroup:[J[1],J[48],J[52]],cssFns:[]},direction:{cssPropBits:0,cssLitGroup:[J[43]],cssFns:[]},display:{cssPropBits:0,cssLitGroup:[J[4],J[6],J[20],J[52],J[67],J[71]],cssFns:[]},"display-extras":{cssPropBits:0,cssLitGroup:[J[67],J[71]],cssFns:[]},"display-inside":{cssPropBits:0,cssLitGroup:[J[20],J[52]],cssFns:[]},"display-outside":{cssPropBits:0,cssLitGroup:[J[4],J[71]],cssFns:[]},elevation:{cssPropBits:5,cssLitGroup:[J[15]],cssFns:[]},"empty-cells":{cssPropBits:0,cssLitGroup:[J[38]],cssFns:[]},filter:{cssPropBits:0,cssFns:["alpha()"]},"float":{cssPropBits:0,cssLitGroup:[J[42],J[71]],cssFns:[]},font:{cssPropBits:73,cssLitGroup:[J[3],J[8],J[13],J[16],J[41],J[48],J[49],J[69],J[72],J[77]],cssFns:[]},"font-family":{cssPropBits:72,cssLitGroup:[J[16],J[48]],cssFns:[]},"font-size":{cssPropBits:1,cssLitGroup:[J[8],J[69]],cssFns:[]},"font-stretch":{cssPropBits:0,cssLitGroup:[J[5],J[72]],cssFns:[]},"font-style":{cssPropBits:0,cssLitGroup:[J[41],J[72]],cssFns:[]},"font-variant":{cssPropBits:0,cssLitGroup:[J[72],J[77]],cssFns:[]},"font-weight":{cssPropBits:0,cssLitGroup:[J[3],J[72]],cssFns:[]},height:"bottom",left:"bottom","letter-spacing":{cssPropBits:5,cssLitGroup:[J[72]],cssFns:[]},"line-height":{cssPropBits:1,cssLitGroup:[J[72]],cssFns:[]},"list-style":{cssPropBits:16,cssLitGroup:[J[2],J[40],J[57],J[71]],cssFns:["image()","linear-gradient()","radial-gradient()","repeating-linear-gradient()","repeating-radial-gradient()"]},"list-style-image":{cssPropBits:16,cssLitGroup:[J[71]],cssFns:["image()","linear-gradient()","radial-gradient()","repeating-linear-gradient()","repeating-radial-gradient()"]},"list-style-position":{cssPropBits:0,cssLitGroup:[J[40]],cssFns:[]},"list-style-type":{cssPropBits:0,cssLitGroup:[J[2],J[57],J[71]],cssFns:[]},margin:"bottom","margin-bottom":"bottom","margin-left":"bottom","margin-right":"bottom","margin-top":"bottom","max-height":{cssPropBits:1,cssLitGroup:[J[52],J[71]],cssFns:[]},"max-width":"max-height","min-height":{cssPropBits:1,cssLitGroup:[J[52]],cssFns:[]},"min-width":"min-height",opacity:{cssPropBits:1,cssFns:[]},outline:{cssPropBits:7,cssLitGroup:[J[0],J[9],J[47],J[62],J[64],J[65],J[69],J[71]],cssFns:["rgb()","rgba()"]},"outline-color":{cssPropBits:2,cssLitGroup:[J[0],J[65]],cssFns:["rgb()","rgba()"]},"outline-style":"border-bottom-style","outline-width":"border-bottom-width",overflow:{cssPropBits:0,cssLitGroup:[J[52],J[62],J[75],J[80]],cssFns:[]},"overflow-wrap":{cssPropBits:0,cssLitGroup:[J[55],J[72]],cssFns:[]},"overflow-x":{cssPropBits:0,cssLitGroup:[J[44],J[52],J[62],J[75],J[80]],cssFns:[]},"overflow-y":"overflow-x",padding:"opacity","padding-bottom":"opacity","padding-left":"opacity","padding-right":"opacity","padding-top":"opacity","page-break-after":{cssPropBits:0,cssLitGroup:[J[42],J[51],J[52],J[53]],cssFns:[]},"page-break-before":"page-break-after","page-break-inside":{cssPropBits:0,cssLitGroup:[J[52],J[53]],cssFns:[]},pause:"border-bottom-left-radius","pause-after":"border-bottom-left-radius","pause-before":"border-bottom-left-radius",perspective:{cssPropBits:5,cssLitGroup:[J[71]],cssFns:[]},"perspective-origin":{cssPropBits:5,cssLitGroup:[J[31],J[42],J[56]],cssFns:[]},pitch:{cssPropBits:5,cssLitGroup:[J[21],J[69]],cssFns:[]},"pitch-range":"border-bottom-left-radius","play-during":{cssPropBits:16,cssLitGroup:[J[52],J[70],J[71],J[74]],cssFns:[]},position:{cssPropBits:0,cssLitGroup:[J[23]],cssFns:[]},quotes:{cssPropBits:8,cssLitGroup:[J[71]],cssFns:[]},resize:{cssPropBits:0,cssLitGroup:[J[39],J[54],J[71]],cssFns:[]},richness:"border-bottom-left-radius",right:"bottom",speak:{cssPropBits:0,cssLitGroup:[J[71],J[72],J[78]],cssFns:[]},"speak-header":{cssPropBits:0,cssLitGroup:[J[51],J[73]],cssFns:[]},"speak-numeral":{cssPropBits:0,cssLitGroup:[J[35]],cssFns:[]},"speak-punctuation":{cssPropBits:0,cssLitGroup:[J[58],J[71]],cssFns:[]},"speech-rate":{cssPropBits:5,cssLitGroup:[J[14],J[69]],cssFns:[]},stress:"border-bottom-left-radius","table-layout":{cssPropBits:0,cssLitGroup:[J[52],J[61]],cssFns:[]},"text-align":{cssPropBits:0,cssLitGroup:[J[42],J[56],J[66]],cssFns:[]},"text-decoration":{cssPropBits:0,cssLitGroup:[J[19],J[71]],cssFns:[]},"text-indent":"border-bottom-left-radius","text-overflow":{cssPropBits:8,cssLitGroup:[J[33]],cssFns:[]},"text-shadow":"box-shadow","text-transform":{cssPropBits:0,cssLitGroup:[J[26],J[71]],cssFns:[]},"text-wrap":{cssPropBits:0,cssLitGroup:[J[46],J[71],J[72]],cssFns:[]},top:"bottom",transform:{cssPropBits:0,cssLitGroup:[J[71]],cssFns:"matrix(),perspective(),rotate(),rotate3d(),rotatex(),rotatey(),rotatez(),scale(),scale3d(),scalex(),scaley(),scalez(),skew(),skewx(),skewy(),translate(),translate3d(),translatex(),translatey(),translatez()".split(",")},"transform-origin":"perspective-origin","transform-style":{cssPropBits:0,cssLitGroup:[J[37]],cssFns:[]},transition:{cssPropBits:1029,cssLitGroup:[J[10],J[48],J[50],J[71]],cssFns:["cubic-bezier()","steps()"]},"transition-delay":"animation-delay","transition-duration":"animation-delay","transition-property":{cssPropBits:1024,cssLitGroup:[J[48],J[50]],cssFns:[]},"transition-timing-function":"animation-timing-function","unicode-bidi":{cssPropBits:0,cssLitGroup:[J[30],J[72]],cssFns:[]},"vertical-align":{cssPropBits:5,cssLitGroup:[J[12],J[31]],cssFns:[]},visibility:"backface-visibility","voice-family":{cssPropBits:8,cssLitGroup:[J[27],J[48]],cssFns:[]},volume:{cssPropBits:1,cssLitGroup:[J[17],J[69]],cssFns:[]},"white-space":{cssPropBits:0,cssLitGroup:[J[22],J[72]],cssFns:[]},width:"min-height","word-break":{cssPropBits:0,cssLitGroup:[J[32],J[72]],cssFns:[]},"word-spacing":"letter-spacing","word-wrap":"overflow-wrap","z-index":"bottom",zoom:"line-height","cubic-bezier()":"animation-delay","steps()":{cssPropBits:5,cssLitGroup:[J[36],J[48]],cssFns:[]},"image()":{cssPropBits:18,cssLitGroup:[J[0],J[48]],cssFns:["rgb()","rgba()"]},"linear-gradient()":{cssPropBits:7,cssLitGroup:[J[0],J[31],J[42],J[48],J[79]],cssFns:["rgb()","rgba()"]},"radial-gradient()":{cssPropBits:7,cssLitGroup:[J[0],J[11],J[31],J[42],J[48],J[56],J[57]],cssFns:["rgb()","rgba()"]},"repeating-linear-gradient()":"linear-gradient()","repeating-radial-gradient()":"radial-gradient()","rgb()":{cssPropBits:1,cssLitGroup:[J[48]],cssFns:[]},"rgba()":"rgb()","rect()":{cssPropBits:5,cssLitGroup:[J[48],J[52]],cssFns:[]},"alpha()":{cssPropBits:1,cssLitGroup:[J[28]],cssFns:[]},"matrix()":"animation-delay","perspective()":"border-bottom-left-radius","rotate()":"border-bottom-left-radius","rotate3d()":"animation-delay","rotatex()":"border-bottom-left-radius","rotatey()":"border-bottom-left-radius","rotatez()":"border-bottom-left-radius","scale()":"animation-delay","scale3d()":"animation-delay","scalex()":"border-bottom-left-radius","scaley()":"border-bottom-left-radius","scalez()":"border-bottom-left-radius","skew()":"animation-delay","skewx()":"border-bottom-left-radius","skewy()":"border-bottom-left-radius","translate()":"animation-delay","translate3d()":"animation-delay","translatex()":"border-bottom-left-radius","translatey()":"border-bottom-left-radius","translatez()":"border-bottom-left-radius"};for(O in L)"string"==typeof L[O]&&Object.hasOwnProperty.call(L,O)&&(L[O]=L[L[O]]);"undefined"!=typeof window&&(window.cssSchema=L);var U,X;!function(){function g(a){var b=parseInt(a.substring(1),16);return 65535<b?(b-=65536,String.fromCharCode(55296+(b>>10),56320+(1023&b))):b==b?String.fromCharCode(b):" ">a[1]?"":a[1]}function w(a,b){return'"'+a.replace(/[\u0000-\u001f\\\"<>]/g,b)+'"'}function M(a){return E[a]||(E[a]="\\"+a.charCodeAt(0).toString(16)+" ")}function x(a){return e[a]||(e[a]=("">a?"%0":"%")+a.charCodeAt(0).toString(16))}var E={"\\":"\\\\"},e={"\\":"%5c"},v=RegExp("\\uFEFF|U[+][0-9A-F?]{1,6}(?:-[0-9A-F]{1,6})?|url[(][\\t\\n\\f ]*(?:\"(?:'|[^'\"\\n\\f\\\\]|\\\\[\\s\\S])*\"|'(?:\"|[^'\"\\n\\f\\\\]|\\\\[\\s\\S])*'|(?:[\\t\\x21\\x23-\\x26\\x28-\\x5b\\x5d-\\x7e]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))*)[\\t\\n\\f ]*[)]|(?!url[(])-?(?:[a-zA-Z_]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))(?:[a-zA-Z0-9_-]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))*[(]|(?:@?-?(?:[a-zA-Z_]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))|#)(?:[a-zA-Z0-9_-]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))*|\"(?:'|[^'\"\\n\\f\\\\]|\\\\[\\s\\S])*\"|'(?:\"|[^'\"\\n\\f\\\\]|\\\\[\\s\\S])*'|[-+]?(?:[0-9]+(?:[.][0-9]+)?|[.][0-9]+)(?:%|-?(?:[a-zA-Z_]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))(?:[a-zA-Z0-9_-]|[\\u0080-\\ud7ff\\ue000-\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]|\\\\(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff]))*)?|<!--|-->|[\\t\\n\\f ]+|/(?:[*][^*]*[*]+(?:[^/][^*]*[*]+)*/|/[^\\n\\f]*)|[~|^$*]=|[^\"'\\\\/]|/(?![/*])","gi"),f=RegExp("\\\\(?:(?:[0-9a-fA-F]{1,6}[\\t\\n\\f ]?|[\\u0020-\\u007e\\u0080-\\ud7ff\\ue000\\ufffd]|[\\ud800-\\udbff][\\udc00-\\udfff])|[\\n\\f])","g"),a=RegExp("^url\\([\\t\\n\\f ]*[\"']?|[\"']?[\\t\\n\\f ]*\\)$","gi");X=function(a){return a.replace(f,g)},U=function(f){for(var f=("￿"+f).replace(/^\uffff/,""),f=f.replace(/\r\n?/g,"\n"),f=f.match(v)||[],b=0,h=" ",d=0,y=f.length;d<y;++d){var l=X(f[d]),V=l.length,g=l.charCodeAt(0),l=34==g||39==g?w(l.substring(1,V-1),M):47==g&&1<V||"\\"==l||"-->"==l||"<!--"==l||"\ufeff"==l||32>=g?" ":/url\(/i.test(l)?"url("+w(l.replace(a,""),x)+")":l;h==l&&" "==l||(f[b++]=h=l)}return f.length=b,f}}(),"undefined"!=typeof window&&(window.lexCss=U,window.decodeCss=X);var Y=function(){function g(d){return d=(""+d).match(i),d?new e(v(d[1]),v(d[2]),v(d[3]),v(d[4]),v(d[5]),v(d[6]),v(d[7])):s}function w(d,a){return"string"==typeof d?encodeURI(d).replace(a,M):s}function M(d){return d=d.charCodeAt(0),"%"+"0123456789ABCDEF".charAt(d>>4&15)+"0123456789ABCDEF".charAt(15&d)}function x(d){if(d===s)return s;for(var h,d=d.replace(/(^|\/)\.(?:\/|$)/g,"$1").replace(/\/{2,}/g,"/"),a=f;(h=d.replace(a,"$1"))!=d;d=h);return d}function E(d,h){var f=d.T(),b=h.K();b?f.ga(h.j):b=h.X(),b?f.da(h.n):b=h.Y(),b?f.ea(h.k):b=h.$();var g=h.g,i=x(g);if(b)f.ca(h.V()),i=i&&i.replace(a,"");else if(b=!!g){if(47!==i.charCodeAt(0))var i=x(f.g||"").replace(a,""),e=i.lastIndexOf("/")+1,i=x((e?i.substring(0,e):"")+x(g)).replace(a,"")}else i=i&&i.replace(a,""),i!==g&&f.G(i);return b?f.G(i):b=h.aa(),b?f.O(h.l):b=h.Z(),b&&f.fa(h.o),f}function e(d,a,h,f,b,g,i){this.j=d,this.n=a,this.k=h,this.h=f,this.g=b,this.l=g,this.o=i}function v(d){return"string"==typeof d&&0<d.length?d:s}var f=RegExp(/(\/|^)(?:[^.\/][^\/]*|\.{2,}(?:[^.\/][^\/]*)|\.{3,}[^\/]*)\/\.\.(?:\/|$)/),a=/^(?:\.\.\/)*(?:\.\.$)?/;e.prototype.toString=function(){var d=[];return s!==this.j&&d.push(this.j,":"),s!==this.k&&(d.push("//"),s!==this.n&&d.push(this.n,"@"),d.push(this.k),s!==this.h&&d.push(":",this.h.toString())),s!==this.g&&d.push(this.g),s!==this.l&&d.push("?",this.l),s!==this.o&&d.push("#",this.o),d.join("")},e.prototype.T=function(){return new e(this.j,this.n,this.k,this.h,this.g,this.l,this.o)},e.prototype.W=function(){return this.j&&decodeURIComponent(this.j).toLowerCase()},e.prototype.ga=function(d){this.j=d?d:s},e.prototype.K=function(){return s!==this.j},e.prototype.da=function(d){this.n=d?d:s},e.prototype.X=function(){return s!==this.n},e.prototype.ea=function(d){this.k=d?d:s,this.G(this.g)},e.prototype.Y=function(){return s!==this.k},e.prototype.V=function(){return this.h&&decodeURIComponent(this.h)},e.prototype.ca=function(d){if(d){if(d=Number(d),d!==(65535&d))throw Error("Bad port number "+d);this.h=""+d}else this.h=s},e.prototype.$=function(){return s!==this.h},e.prototype.U=function(){return this.g&&decodeURIComponent(this.g)},e.prototype.G=function(d){d?(d=""+d,this.g=!this.k||/^\//.test(d)?d:"/"+d):this.g=s},e.prototype.O=function(d){this.l=d?d:s},e.prototype.aa=function(){return s!==this.l},e.prototype.ba=function(d){if("object"==typeof d&&!(d instanceof Array)&&(d instanceof Object||"[object Array]"!==Object.prototype.toString.call(d))){var f,a=[],h=-1;for(f in d){var b=d[f];"string"==typeof b&&(a[++h]=f,a[++h]=b)}d=a}for(var a=[],h="",g=0;g<d.length;)f=d[g++],b=d[g++],a.push(h,encodeURIComponent(f.toString())),h="&",b&&a.push("=",encodeURIComponent(b.toString()));this.l=a.join("")},e.prototype.fa=function(d){this.o=d?d:s},e.prototype.Z=function(){return s!==this.o};var i=/^(?:([^:\/?#]+):)?(?:\/\/(?:([^\/?#]*)@)?([^\/?#:@]*)(?::([0-9]+))?)?([^?#]+)?(?:\?([^#]*))?(?:#(.*))?$/,b=/[#\/\?@]/g,h=/[\#\?]/g;return e.parse=g,e.create=function(d,a,f,g,i,Q,N){return d=new e(w(d,b),w(a,b),"string"==typeof f?encodeURIComponent(f):s,0<g?g.toString():s,w(i,h),s,"string"==typeof N?encodeURIComponent(N):s),Q&&("string"==typeof Q?d.O(Q.replace(/[^?&=0-9A-Za-z_\-~.%]/g,M)):d.ba(Q)),d},e.N=E,e.ma=x,e.ha={ua:function(d){return/\.html$/.test(g(d).U())?"text/html":"application/javascript"},N:function(d,a){return d?E(g(d),g(a)).toString():""+a}},e}();"undefined"!=typeof window&&(window.URI=Y);var aa=c,ba=c,da=c,Z=c;!function(){function g(a){return"string"==typeof a?'url("'+a.replace(e,w)+'")':'url("about:blank")'}function w(a){return v[a]}function M(a,d){return a?Y.ha.N(a,d):d}function x(h,d,b){if(!b)return s;var g=(""+h).match(f);return!g||g[1]&&!a.test(g[1])?s:b(h,d)}function E(a){return a.replace(/^-(?:apple|css|epub|khtml|moz|mso?|o|rim|wap|webkit|xv)-(?=[a-z])/,"")}var e=/[\n\f\r\"\'()*<>]/g,v={"\n":"%0a","\f":"%0c","\r":"%0d",'"':"%22","'":"%27","(":"%28",")":"%29","*":"%2a","<":"%3c",">":"%3e"},f=/^(?:([^:\/?# ]+):)?/,a=/^(?:https?|geo|mailto|sms|tel)$/i;aa=function(){var a={};return function y(f,b,i,e,N){var f=E(f),u=L[f];if(u&&"object"==typeof u){for(var k=u.cssPropBits,q=80&k,B=1536&k,F=NaN,r=0,o=0;r<b.length;++r){var R,v,P,S,D,w,j=b[r].toLowerCase(),I=j.charCodeAt(0);if(32===I)j="";else if(34===I)j=16===q?i?g(x(M(e,X(b[r].substring(1,j.length-1))),f,i)):"":8&k&&!(q&q-1)?j:"";else if("inherit"!==j){if(D=u.cssLitGroup){var G;if(!(G=u.cssLitMap)){G={};for(var K=D.length;0<=--K;)for(var A=D[K],T=A.length;0<=--T;)G[A[T]]=a;G=u.cssLitMap=G}D=G}else D=a;if(w=D,!(w[E(j)]===a))if(35===I&&/^#(?:[0-9a-f]{3}){1,2}$/.test(j))j=2&k?j:"";else if(48<=I&&57>=I)j=1&k?j:"";else if(R=j.charCodeAt(1),v=j.charCodeAt(2),P=48<=R&&57>=R,S=48<=v&&57>=v,43===I&&(P||46===R&&S))j=1&k?(P?"":"0")+j.substring(1):"";else if(45===I&&(P||46===R&&S))j=4&k?(P?"-":"-0")+j.substring(1):1&k?"0":"";else if(46===I&&P)j=1&k?"0"+j:"";else if('url("'===j.substring(0,5))j=i&&16&k?g(x(M(e,b[r].substring(5,j.length-2)),f,i)):"";else if("("===j.charAt(j.length-1))a:{for(D=b,G=r,j=1,K=G+1,I=D.length;K<I&&j;)A=D[K++],j+=")"===A?-1:/^[^"']*\($/.test(A);if(!j)for(j=D[G].toLowerCase(),I=E(j),D=D.splice(G,K-G,""),G=u.cssFns,K=0,A=G.length;K<A;++K)if(G[K].substring(0,I.length)==I){D[0]=D[D.length-1]="",y(G[K],D,i,e),j=j+D.join(" ")+")";break a}j=""}else j=B&&/^-?[a-z_][\w\-]*$/.test(j)&&!/__$/.test(j)?N&&512===B?b[r]+N:1024===B&&L[j]&&"number"==typeof L[j].oa?j:"":/^\w+$/.test(j)&&64===q&&8&k?F+1===o?(b[F]=b[F].substring(0,b[F].length-1)+" "+j+'"',""):(F=o,'"'+j+'"'):""}j&&(b[o++]=j)}1===o&&'url("about:blank")'===b[0]&&(o=0),b.length=o}else b.length=0}}();var i=RegExp("^(active|after|before|blank|checked|default|disabled|drop|empty|enabled|first|first-child|first-letter|first-line|first-of-type|fullscreen|focus|hover|in-range|indeterminate|invalid|last-child|last-of-type|left|link|only-child|only-of-type|optional|out-of-range|placeholder-shown|read-only|read-write|required|right|root|scope|user-error|valid|visited)$"),b={};b[">"]=b["+"]=b["~"]=b,ba=function(a,d,f){function g(k,r){function o(b,f,g){var y,e,k,l,o,m=n;y="",b<f&&(o=a[b],"*"===o?(++b,y=o):/^[a-zA-Z]/.test(o)&&(e=x(o.toLowerCase(),[]))&&("tagName"in e&&(o=e.tagName),++b,y=o));for(l=k=e="";m&&b<f;++b)if(o=a[b],"#"===o.charAt(0))/^#_|__$|[^\w#:\-]/.test(o)?m=C:e+=o+v;else if("."===o)++b<f&&/^[0-9A-Za-z:_\-]+$/.test(o=a[b])&&!/^_|__$/.test(o)?e+="."+o:m=C;else if(b+1<f&&"["===a[b]){++b;var H=a[b++].toLowerCase();o=$.m[y+"::"+H],o!==+o&&(o=$.m["*::"+H]);var W;d.ia?(W=d.ia(y,H),"string"!=typeof W&&(m=C,W=H),m&&o!==+o&&(o=$.d.NONE)):(W=H,o!==+o&&(m=C));var p=H="",ca=C;switch(/^[~^$*|]?=$/.test(a[b])&&(H=a[b++],p=a[b++],/^[0-9A-Za-z:_\-]+$/.test(p)?p='"'+p+'"':"]"===p&&(p='""',--b),/^"([^\"\\]|\\.)*"$/.test(p)||(m=C),(ca="i"===a[b])&&++b),"]"!==a[b]&&(++b,m=C),o){case $.d.CLASSES:case $.d.LOCAL_NAME:case $.d.NONE:break;case $.d.GLOBAL_NAME:case $.d.ID:case $.d.IDREF:"="!==H&&"~="!==H&&"$="!==H||'""'==p||ca?"|="===H||""===H||(m=C):p='"'+p.substring(1,p.length-1)+v+'"';break;case $.d.URI:case $.d.URI_FRAGMENT:""!==H&&(m=C);break;default:m=C}m&&(k+="["+W.replace(/[^\w-]/g,"\\$&")+H+p+(ca?" i]":"]"))}else{if(!(b<f&&":"===a[b]))break;if(o=a[++b],!i.test(o))break;l+=":"+o}return b!==f&&(m=C),m&&(b=(y+e).replace(/[^ .*#\w-]/g,"\\$&")+k+l+g)&&j.push(b),m}" "===a[k]&&++k,r-1!==k&&" "===a[r]&&--r;for(var j=[],l=k,q=n,u=k;q&&u<r;++u){var B=a[u];b[B]!==b&&" "!==B||(o(l,u,B)?l=u+1:q=C)}return o(l,r,"")||(q=C),q?(j.length&&(l=j.join(""),e!==s&&(l="."+e+" "+l),N.push(l)),n):!f||f(a.slice(k,r))}var k,B,e=d.na,v=d.L,x=d.Aa,N=[],u=0,q=0;for(k=0;k<a.length;++k)B=a[k],("("==B||"["==B?(++q,!n):")"==B||"]"==B?(q&&--q,!n):" "==a[k]&&(q||b[a[k-1]]===b||b[a[k+1]]===b))||(a[u++]=a[k]);for(a.length=u,u=a.length,k=q=0;k<u;++k)if(","===a[k]){if(!g(q,k))return s;q=k+1}return g(q,u)?N:s},function(){var a=/^\w/,d=RegExp("^(?:(?:(?:(?:only|not) )?(?:all|aural|braille|embossed|handheld|print|projection|screen|speech|tty|tv)|\\( (?:(?:min-|max-)?(?:(?:device-)?(?:aspect-ratio|height|width)|color(?:-index)?|monochrome|orientation|resolution)|grid|hover|luminosity|pointer|scan|script) (?:: -?(?:[a-z]\\w+(?:-\\w+)*|\\d+(?: / \\d+|(?:\\.\\d+)?(?:p[cxt]|[cem]m|in|dpi|dppx|dpcm|%)?)) )?\\))(?: and ?\\( (?:(?:min-|max-)?(?:(?:device-)?(?:aspect-ratio|height|width)|color(?:-index)?|monochrome|orientation|resolution)|grid|hover|luminosity|pointer|scan|script) (?:: -?(?:[a-z]\\w+(?:-\\w+)*|\\d+(?: / \\d+|(?:\\.\\d+)?(?:p[cxt]|[cem]m|in|dpi|dppx|dpcm|%)?)) )?\\))*)(?: , (?:(?:(?:(?:only|not) )?(?:all|aural|braille|embossed|handheld|print|projection|screen|speech|tty|tv)|\\( (?:(?:min-|max-)?(?:(?:device-)?(?:aspect-ratio|height|width)|color(?:-index)?|monochrome|orientation|resolution)|grid|hover|luminosity|pointer|scan|script) (?:: -?(?:[a-z]\\w+(?:-\\w+)*|\\d+(?: / \\d+|(?:\\.\\d+)?(?:p[cxt]|[cem]m|in|dpi|dppx|dpcm|%)?)) )?\\))(?: and ?\\( (?:(?:min-|max-)?(?:(?:device-)?(?:aspect-ratio|height|width)|color(?:-index)?|monochrome|orientation|resolution)|grid|hover|luminosity|pointer|scan|script) (?:: -?(?:[a-z]\\w+(?:-\\w+)*|\\d+(?: / \\d+|(?:\\.\\d+)?(?:p[cxt]|[cem]m|in|dpi|dppx|dpcm|%)?)) )?\\))*))*$","i");Z=function(b){for(var b=b.slice(),f=b.length,g=0,i=0;i<f;++i){var e=b[i];" "!=e&&(b[g++]=e)}return b.length=g,b=b.join(" "),b=b.length?d.test(b)?a.test(b)?b:"not all , "+b:"not all":""}}(),function(){function a(b){var i,d=/^\s*[']([^']*)[']\s*$/,f=/^\s*url\s*[(]["]([^"]*)["][)]\s*$/,g=/^\s*url\s*[(][']([^']*)['][)]\s*$/,h=/^\s*url\s*[(]([^)]*)[)]\s*$/;return(i=/^\s*["]([^"]*)["]\s*$/.exec(b))||(i=d.exec(b))||(i=f.exec(b))||(i=g.exec(b))||(i=h.exec(b))?i[1]:s}function b(f,g,i,e,v,w,u){function k(){r=F.length&&F[F.length-1]===s}var q=c,B=u||[0],F=[],r=C;return fa(g,{startStylesheet:function(){q=[]},endStylesheet:function(){},startAtrule:function(g,j){if(r)g=s;else if("@media"===g)q.push("@media"," ",Z(j));else if("@keyframes"===g||"@-webkit-keyframes"===g){var k=j[0];1!==j.length||/__$|[^\w\-]/.test(k)?g=s:(q.push(g," ",k+i.L),g="@keyframes")}else if("@import"===g&&0<j.length)if(g=s,"function"==typeof w){var l=Z(j.slice(1));if("not all"!==l){++B[0];var u=[];q.push(u);var E=x(M(f,a(j[0])),function(a){var f=b(E,a.qa,i,e,v,w,B);--B[0],a=l?{toString:function(){return"@media "+l+" {"+f.result+"}"}}:f.result,u[0]=a,w(a,!!B[0])},v)}}else window.console&&window.console.log("@import "+j.join(" ")+" elided");r=!g,F.push(g)},endAtrule:function(){F.pop(),r||q.push(";"),k()},startBlock:function(){r||q.push("{")},endBlock:function(){r||(q.push("}"),r=n)},startRuleset:function(a){if(!r){var b=c;"@keyframes"===F[F.length-1]?(b=a.join(" ").match(/^ *(?:from|to|\d+(?:\.\d+)?%) *(?:, *(?:from|to|\d+(?:\.\d+)?%) *)*$/i),r=!b,b&&(b=b[0].replace(/ +/g,""))):(a=ba(a,i),a&&a.length?b=a.join(", "):r=n),r||q.push(b,"{")}F.push(s)},endRuleset:function(){F.pop(),r||q.push("}"),k()},declaration:function(a,b){if(!r){var d=C,g=b.length;2<=g&&"!"===b[g-2]&&"important"===b[g-1].toLowerCase()&&(d=n,b.length-=2),aa(a,b,e,f,i.L),b.length&&q.push(a,":",b.join(" "),d?" !important;":";")}}}),{result:{toString:function(){return q.join("")}},va:!!B[0]}}da=function(a,f,g,i){return b(a,f,g,i,c,c).result.toString()}}()}(),"undefined"!=typeof window&&(window.sanitizeCssProperty=aa,window.sanitizeCssSelectorList=ba,window.sanitizeStylesheet=da,window.sanitizeMediaQuery=Z);var fa,ga;!function(){function g(f,a,g,b,h){for(var d=a++;a<g&&"{"!==f[a]&&";"!==f[a];)++a;if(a<g&&(h||";"===f[a])){var h=d+1,e=a;h<g&&" "===f[h]&&++h,e>h&&" "===f[e-1]&&--e,b.startAtrule&&b.startAtrule(f[d].toLowerCase(),f.slice(h,e)),a="{"===f[a]?w(f,a,g,b):a+1,b.endAtrule&&b.endAtrule()}return a}function w(f,a,i,b){for(++a,b.startBlock&&b.startBlock();a<i;){var h=f[a].charAt(0);if("}"==h){++a;break}a=" "===h||";"===h?a+1:"@"===h?g(f,a,i,b,C):"{"===h?w(f,a,i,b):M(f,a,i,b)}return b.endBlock&&b.endBlock(),a}function M(f,a,g,b){var h=a,d=x(f,a,g,n);if(0>d)return d=~d,d===h?d+1:d;var y=f[d];if("{"!==y)return d===h?d+1:d;for(a=d+1,
d>h&&" "===f[d-1]&&--d,b.startRuleset&&b.startRuleset(f.slice(h,d));a<g;){if(y=f[a],"}"===y){++a;break}a=" "===y?a+1:e(f,a,g,b)}return b.endRuleset&&b.endRuleset(),a}function x(f,a,g,b){for(var h,d=[],e=-1;a<g;++a)if(h=f[a].charAt(0),"["===h||"("===h)d[++e]=h;else if("]"===h&&"["===d[e]||")"===h&&"("===d[e])--e;else if("{"===h||"}"===h||";"===h||"@"===h||":"===h&&!b)break;return 0<=e&&(a=~(a+1)),a}function E(f,a,g){for(;a<g&&";"!==f[a]&&"}"!==f[a];)++a;return a<g&&";"===f[a]?a+1:a}function e(f,a,g,b){var h=f[a++];if(!v.test(h))return E(f,a,g);if(a<g&&" "===f[a]&&++a,a==g||":"!==f[a])return E(f,a,g);++a,a<g&&" "===f[a]&&++a;var d=x(f,a,g,C);if(0>d)d=~d;else{for(var e=[],l=0,w=a;w<d;++w)a=f[w]," "!==a&&(e[l++]=a);if(d<g){do{if(a=f[d],";"===a||"}"===a)break;l=0}while(++d<g);";"===a&&++d}l&&b.declaration&&b.declaration(h.toLowerCase(),e)}return d}fa=function(f,a){var e=U(f);a.startStylesheet&&a.startStylesheet();for(var b=0,h=e.length;b<h;)b=" "===e[b]?b+1:b<h?"@"===e[b].charAt(0)?g(e,b,h,a,n):M(e,b,h,a):b;a.endStylesheet&&a.endStylesheet()};var v=/^-?[a-z]/i;ga=function(g,a){for(var i=U(g),b=0,h=i.length;b<h;)b=" "!==i[b]?e(i,b,h,a):b+1}}(),"undefined"!=typeof window&&(window.parseCssStylesheet=fa,window.parseCssDeclarations=ga);var $={d:{NONE:0,URI:1,URI_FRAGMENT:11,SCRIPT:2,STYLE:3,HTML:12,ID:4,IDREF:5,IDREFS:6,GLOBAL_NAME:7,LOCAL_NAME:8,CLASSES:9,FRAME_TARGET:10,MEDIA_QUERY:13}};$.atype=$.d,$.m={"*::class":9,"*::dir":0,"*::draggable":0,"*::hidden":0,"*::id":4,"*::inert":0,"*::itemprop":0,"*::itemref":6,"*::itemscope":0,"*::lang":0,"*::onblur":2,"*::onchange":2,"*::onclick":2,"*::ondblclick":2,"*::onerror":2,"*::onfocus":2,"*::onkeydown":2,"*::onkeypress":2,"*::onkeyup":2,"*::onload":2,"*::onmousedown":2,"*::onmousemove":2,"*::onmouseout":2,"*::onmouseover":2,"*::onmouseup":2,"*::onreset":2,"*::onscroll":2,"*::onselect":2,"*::onsubmit":2,"*::ontouchcancel":2,"*::ontouchend":2,"*::ontouchenter":2,"*::ontouchleave":2,"*::ontouchmove":2,"*::ontouchstart":2,"*::onunload":2,"*::spellcheck":0,"*::style":3,"*::tabindex":0,"*::title":0,"*::translate":0,"a::accesskey":0,"a::coords":0,"a::href":1,"a::hreflang":0,"a::name":7,"a::onblur":2,"a::onfocus":2,"a::shape":0,"a::target":10,"a::type":0,"area::accesskey":0,"area::alt":0,"area::coords":0,"area::href":1,"area::nohref":0,"area::onblur":2,"area::onfocus":2,"area::shape":0,"area::target":10,"audio::controls":0,"audio::loop":0,"audio::mediagroup":5,"audio::muted":0,"audio::preload":0,"audio::src":1,"bdo::dir":0,"blockquote::cite":1,"br::clear":0,"button::accesskey":0,"button::disabled":0,"button::name":8,"button::onblur":2,"button::onfocus":2,"button::type":0,"button::value":0,"canvas::height":0,"canvas::width":0,"caption::align":0,"col::align":0,"col::char":0,"col::charoff":0,"col::span":0,"col::valign":0,"col::width":0,"colgroup::align":0,"colgroup::char":0,"colgroup::charoff":0,"colgroup::span":0,"colgroup::valign":0,"colgroup::width":0,"command::checked":0,"command::command":5,"command::disabled":0,"command::icon":1,"command::label":0,"command::radiogroup":0,"command::type":0,"data::value":0,"del::cite":1,"del::datetime":0,"details::open":0,"dir::compact":0,"div::align":0,"dl::compact":0,"fieldset::disabled":0,"font::color":0,"font::face":0,"font::size":0,"form::accept":0,"form::action":1,"form::autocomplete":0,"form::enctype":0,"form::method":0,"form::name":7,"form::novalidate":0,"form::onreset":2,"form::onsubmit":2,"form::target":10,"h1::align":0,"h2::align":0,"h3::align":0,"h4::align":0,"h5::align":0,"h6::align":0,"hr::align":0,"hr::noshade":0,"hr::size":0,"hr::width":0,"iframe::align":0,"iframe::frameborder":0,"iframe::height":0,"iframe::marginheight":0,"iframe::marginwidth":0,"iframe::width":0,"img::align":0,"img::alt":0,"img::border":0,"img::height":0,"img::hspace":0,"img::ismap":0,"img::name":7,"img::src":1,"img::usemap":11,"img::vspace":0,"img::width":0,"input::accept":0,"input::accesskey":0,"input::align":0,"input::alt":0,"input::autocomplete":0,"input::checked":0,"input::disabled":0,"input::inputmode":0,"input::ismap":0,"input::list":5,"input::max":0,"input::maxlength":0,"input::min":0,"input::multiple":0,"input::name":8,"input::onblur":2,"input::onchange":2,"input::onfocus":2,"input::onselect":2,"input::pattern":0,"input::placeholder":0,"input::readonly":0,"input::required":0,"input::size":0,"input::src":1,"input::step":0,"input::type":0,"input::usemap":11,"input::value":0,"ins::cite":1,"ins::datetime":0,"label::accesskey":0,"label::for":5,"label::onblur":2,"label::onfocus":2,"legend::accesskey":0,"legend::align":0,"li::type":0,"li::value":0,"map::name":7,"menu::compact":0,"menu::label":0,"menu::type":0,"meter::high":0,"meter::low":0,"meter::max":0,"meter::min":0,"meter::optimum":0,"meter::value":0,"ol::compact":0,"ol::reversed":0,"ol::start":0,"ol::type":0,"optgroup::disabled":0,"optgroup::label":0,"option::disabled":0,"option::label":0,"option::selected":0,"option::value":0,"output::for":6,"output::name":8,"p::align":0,"pre::width":0,"progress::max":0,"progress::min":0,"progress::value":0,"q::cite":1,"select::autocomplete":0,"select::disabled":0,"select::multiple":0,"select::name":8,"select::onblur":2,"select::onchange":2,"select::onfocus":2,"select::required":0,"select::size":0,"source::type":0,"table::align":0,"table::bgcolor":0,"table::border":0,"table::cellpadding":0,"table::cellspacing":0,"table::frame":0,"table::rules":0,"table::summary":0,"table::width":0,"tbody::align":0,"tbody::char":0,"tbody::charoff":0,"tbody::valign":0,"td::abbr":0,"td::align":0,"td::axis":0,"td::bgcolor":0,"td::char":0,"td::charoff":0,"td::colspan":0,"td::headers":6,"td::height":0,"td::nowrap":0,"td::rowspan":0,"td::scope":0,"td::valign":0,"td::width":0,"textarea::accesskey":0,"textarea::autocomplete":0,"textarea::cols":0,"textarea::disabled":0,"textarea::inputmode":0,"textarea::name":8,"textarea::onblur":2,"textarea::onchange":2,"textarea::onfocus":2,"textarea::onselect":2,"textarea::placeholder":0,"textarea::readonly":0,"textarea::required":0,"textarea::rows":0,"textarea::wrap":0,"tfoot::align":0,"tfoot::char":0,"tfoot::charoff":0,"tfoot::valign":0,"th::abbr":0,"th::align":0,"th::axis":0,"th::bgcolor":0,"th::char":0,"th::charoff":0,"th::colspan":0,"th::headers":6,"th::height":0,"th::nowrap":0,"th::rowspan":0,"th::scope":0,"th::valign":0,"th::width":0,"thead::align":0,"thead::char":0,"thead::charoff":0,"thead::valign":0,"tr::align":0,"tr::bgcolor":0,"tr::char":0,"tr::charoff":0,"tr::valign":0,"track::default":0,"track::kind":0,"track::label":0,"track::srclang":0,"ul::compact":0,"ul::type":0,"video::controls":0,"video::height":0,"video::loop":0,"video::mediagroup":5,"video::muted":0,"video::poster":1,"video::preload":0,"video::src":1,"video::width":0},$.ATTRIBS=$.m,$.c={OPTIONAL_ENDTAG:1,EMPTY:2,CDATA:4,RCDATA:8,UNSAFE:16,FOLDABLE:32,SCRIPT:64,STYLE:128,VIRTUALIZED:256},$.eflags=$.c,$.f={a:0,abbr:0,acronym:0,address:0,applet:272,area:2,article:0,aside:0,audio:0,b:0,base:274,basefont:274,bdi:0,bdo:0,big:0,blockquote:0,body:305,br:2,button:0,canvas:0,caption:0,center:0,cite:0,code:0,col:2,colgroup:1,command:2,data:0,datalist:0,dd:1,del:0,details:0,dfn:0,dialog:272,dir:0,div:0,dl:0,dt:1,em:0,fieldset:0,figcaption:0,figure:0,font:0,footer:0,form:0,frame:274,frameset:272,h1:0,h2:0,h3:0,h4:0,h5:0,h6:0,head:305,header:0,hgroup:0,hr:2,html:305,i:0,iframe:4,img:2,input:2,ins:0,isindex:274,kbd:0,keygen:274,label:0,legend:0,li:1,link:274,map:0,mark:0,menu:0,meta:274,meter:0,nav:0,nobr:0,noembed:276,noframes:276,noscript:276,object:272,ol:0,optgroup:0,option:1,output:0,p:1,param:274,pre:0,progress:0,q:0,s:0,samp:0,script:84,section:0,select:0,small:0,source:2,span:0,strike:0,strong:0,style:148,sub:0,summary:0,sup:0,table:0,tbody:1,td:1,textarea:8,tfoot:1,th:1,thead:1,time:0,title:280,tr:1,track:2,tt:0,u:0,ul:0,"var":0,video:0,wbr:2},$.ELEMENTS=$.f,$.Q={a:"HTMLAnchorElement",abbr:"HTMLElement",acronym:"HTMLElement",address:"HTMLElement",applet:"HTMLAppletElement",area:"HTMLAreaElement",article:"HTMLElement",aside:"HTMLElement",audio:"HTMLAudioElement",b:"HTMLElement",base:"HTMLBaseElement",basefont:"HTMLBaseFontElement",bdi:"HTMLElement",bdo:"HTMLElement",big:"HTMLElement",blockquote:"HTMLQuoteElement",body:"HTMLBodyElement",br:"HTMLBRElement",button:"HTMLButtonElement",canvas:"HTMLCanvasElement",caption:"HTMLTableCaptionElement",center:"HTMLElement",cite:"HTMLElement",code:"HTMLElement",col:"HTMLTableColElement",colgroup:"HTMLTableColElement",command:"HTMLCommandElement",data:"HTMLElement",datalist:"HTMLDataListElement",dd:"HTMLElement",del:"HTMLModElement",details:"HTMLDetailsElement",dfn:"HTMLElement",dialog:"HTMLDialogElement",dir:"HTMLDirectoryElement",div:"HTMLDivElement",dl:"HTMLDListElement",dt:"HTMLElement",em:"HTMLElement",fieldset:"HTMLFieldSetElement",figcaption:"HTMLElement",figure:"HTMLElement",font:"HTMLFontElement",footer:"HTMLElement",form:"HTMLFormElement",frame:"HTMLFrameElement",frameset:"HTMLFrameSetElement",h1:"HTMLHeadingElement",h2:"HTMLHeadingElement",h3:"HTMLHeadingElement",h4:"HTMLHeadingElement",h5:"HTMLHeadingElement",h6:"HTMLHeadingElement",head:"HTMLHeadElement",header:"HTMLElement",hgroup:"HTMLElement",hr:"HTMLHRElement",html:"HTMLHtmlElement",i:"HTMLElement",iframe:"HTMLIFrameElement",img:"HTMLImageElement",input:"HTMLInputElement",ins:"HTMLModElement",isindex:"HTMLUnknownElement",kbd:"HTMLElement",keygen:"HTMLKeygenElement",label:"HTMLLabelElement",legend:"HTMLLegendElement",li:"HTMLLIElement",link:"HTMLLinkElement",map:"HTMLMapElement",mark:"HTMLElement",menu:"HTMLMenuElement",meta:"HTMLMetaElement",meter:"HTMLMeterElement",nav:"HTMLElement",nobr:"HTMLElement",noembed:"HTMLElement",noframes:"HTMLElement",noscript:"HTMLElement",object:"HTMLObjectElement",ol:"HTMLOListElement",optgroup:"HTMLOptGroupElement",option:"HTMLOptionElement",output:"HTMLOutputElement",p:"HTMLParagraphElement",param:"HTMLParamElement",pre:"HTMLPreElement",progress:"HTMLProgressElement",q:"HTMLQuoteElement",s:"HTMLElement",samp:"HTMLElement",script:"HTMLScriptElement",section:"HTMLElement",select:"HTMLSelectElement",small:"HTMLElement",source:"HTMLSourceElement",span:"HTMLSpanElement",strike:"HTMLElement",strong:"HTMLElement",style:"HTMLStyleElement",sub:"HTMLElement",summary:"HTMLElement",sup:"HTMLElement",table:"HTMLTableElement",tbody:"HTMLTableSectionElement",td:"HTMLTableDataCellElement",textarea:"HTMLTextAreaElement",tfoot:"HTMLTableSectionElement",th:"HTMLTableHeaderCellElement",thead:"HTMLTableSectionElement",time:"HTMLTimeElement",title:"HTMLTitleElement",tr:"HTMLTableRowElement",track:"HTMLTrackElement",tt:"HTMLElement",u:"HTMLElement",ul:"HTMLUListElement","var":"HTMLElement",video:"HTMLVideoElement",wbr:"HTMLElement"},$.ELEMENT_DOM_INTERFACES=$.Q,$.P={NOT_LOADED:0,SAME_DOCUMENT:1,NEW_DOCUMENT:2},$.ueffects=$.P,$.J={"a::href":2,"area::href":2,"audio::src":1,"blockquote::cite":0,"command::icon":1,"del::cite":0,"form::action":2,"img::src":1,"input::src":1,"ins::cite":0,"q::cite":0,"video::poster":1,"video::src":1},$.URIEFFECTS=$.J,$.M={UNSANDBOXED:2,SANDBOXED:1,DATA:0},$.ltypes=$.M,$.I={"a::href":2,"area::href":2,"audio::src":2,"blockquote::cite":2,"command::icon":1,"del::cite":2,"form::action":2,"img::src":1,"input::src":1,"ins::cite":2,"q::cite":2,"video::poster":1,"video::src":2},$.LOADERTYPES=$.I,"undefined"!=typeof window&&(window.html4=$);var ha=function(g){function w(a){if(k.hasOwnProperty(a))return k[a];var b=a.match(q);return b?String.fromCharCode(parseInt(b[1],10)):(b=a.match(B))?String.fromCharCode(parseInt(b[1],16)):r&&F.test(a)?(r.innerHTML="&"+a+";",b=r.textContent,k[a]=b):"&"+a+";"}function M(a,b){return w(b)}function x(a){return a.replace(j,M)}function E(a){return(""+a).replace(R,"&amp;").replace(P,"&lt;").replace(S,"&gt;").replace(D,"&#34;")}function e(a){return a.replace(ia,"&amp;$1").replace(P,"&lt;").replace(S,"&gt;")}function v(b){var d={z:b.z||b.cdata,A:b.A||b.comment,B:b.B||b.endDoc,t:b.t||b.endTag,e:b.e||b.pcdata,F:b.F||b.rcdata,H:b.H||b.startDoc,w:b.w||b.startTag};return function(b,g){var f,H=/(<\/|<\!--|<[!?]|[&<>])/g;if(f=b+"",G)f=f.split(H);else{for(var j,e=[],h=0;(j=H.exec(f))!==s;)e.push(f.substring(h,j.index)),e.push(j[0]),h=j.index+j[0].length;e.push(f.substring(h)),f=e}a(d,f,0,{r:C,C:C},g)}}function f(b,d,g,f,t){return function(){a(b,d,g,f,t)}}function a(a,d,p,e,t){try{a.H&&0==p&&a.H(t);for(var h,z,j,k=d.length;p<k;){var o=d[p++],l=d[p];switch(o){case"&":I.test(l)?(a.e&&a.e("&"+l,t,A,f(a,d,p,e,t)),p++):a.e&&a.e("&amp;",t,A,f(a,d,p,e,t));break;case"</":if(h=/^([-\w:]+)[^\'\"]*/.exec(l))if(h[0].length===l.length&&">"===d[p+1])p+=2,j=h[1].toLowerCase(),a.t&&a.t(j,t,A,f(a,d,p,e,t));else{var m=d,q=p,r=a,u=t,v=A,y=e,w=b(m,q);w?(r.t&&r.t(w.name,u,v,f(r,m,q,y,u)),p=w.next):p=m.length}else a.e&&a.e("&lt;/",t,A,f(a,d,p,e,t));break;case"<":if(h=/^([-\w:]+)\s*\/?/.exec(l))if(h[0].length===l.length&&">"===d[p+1]){p+=2,j=h[1].toLowerCase(),a.w&&a.w(j,[],t,A,f(a,d,p,e,t));var B=g.f[j];B&K&&(p=i(d,{name:j,next:p,c:B},a,t,A,e))}else{var m=d,q=a,r=t,u=A,v=e,x=b(m,p);x?(q.w&&q.w(x.name,x.R,r,u,f(q,m,x.next,v,r)),p=x.c&K?i(m,x,q,r,u,v):x.next):p=m.length}else a.e&&a.e("&lt;",t,A,f(a,d,p,e,t));break;case"<!--":if(!e.C){for(z=p+1;z<k&&(">"!==d[z]||!/--$/.test(d[z-1]));z++);if(z<k){if(a.A){var D=d.slice(p,z).join("");a.A(D.substr(0,D.length-2),t,A,f(a,d,z+1,e,t))}p=z+1}else e.C=n}e.C&&a.e&&a.e("&lt;!--",t,A,f(a,d,p,e,t));break;case"<!":if(/^\w/.test(l)){if(!e.r){for(z=p+1;z<k&&">"!==d[z];z++);z<k?p=z+1:e.r=n}e.r&&a.e&&a.e("&lt;!",t,A,f(a,d,p,e,t))}else a.e&&a.e("&lt;!",t,A,f(a,d,p,e,t));break;case"<?":if(!e.r){for(z=p+1;z<k&&">"!==d[z];z++);z<k?p=z+1:e.r=n}e.r&&a.e&&a.e("&lt;?",t,A,f(a,d,p,e,t));break;case">":a.e&&a.e("&gt;",t,A,f(a,d,p,e,t));break;case"":break;default:a.e&&a.e(o,t,A,f(a,d,p,e,t))}}a.B&&a.B(t)}catch(E){if(E!==A)throw E}}function i(a,b,d,h,t,j){var z=a.length;T.hasOwnProperty(b.name)||(T[b.name]=RegExp("^"+b.name+"(?:[\\s\\/]|$)","i"));for(var i=T[b.name],k=b.next,l=b.next+1;l<z&&("</"!==a[l-1]||!i.test(a[l]));l++);if(l<z&&(l-=1),z=a.slice(k,l).join(""),b.c&g.c.CDATA)d.z&&d.z(z,h,t,f(d,a,l,j,h));else{if(!(b.c&g.c.RCDATA))throw Error("bug");d.F&&d.F(e(z),h,t,f(d,a,l,j,h))}return l}function b(a,b){var d=/^([-\w:]+)/.exec(a[b]),f={};f.name=d[1].toLowerCase(),f.c=g.f[f.name];for(var e=a[b].substr(d[0].length),h=b+1,j=a.length;h<j&&">"!==a[h];h++)e+=a[h];if(!(j<=h)){for(var i=[];""!==e;)if(d=ja.exec(e))if(d[4]&&!d[5]||d[6]&&!d[7]){for(var d=d[4]||d[6],l=C,e=[e,a[h++]];h<j;h++){if(l){if(">"===a[h])break}else 0<=a[h].indexOf(d)&&(l=n);e.push(a[h])}if(j<=h)break;e=e.join("")}else{var k,l=d[1].toLowerCase();if(d[2]){k=d[3];var m=k.charCodeAt(0);34!==m&&39!==m||(k=k.substr(1,k.length-2)),k=x(k.replace(o,""))}else k="";i.push(l,k),e=e.substr(d[0].length)}else e=e.replace(/^[\s\S][^a-z\s]*/,"");return f.R=i,f.next=h+1,f}}function h(a){function b(a,d){f||d.push(a)}var d,f;return v({startDoc:function(){d=[],f=C},startTag:function(b,e,h){if(!f&&g.f.hasOwnProperty(b)){var j=g.f[b];if(!(j&g.c.FOLDABLE)){var k=a(b,e);if(k){if("object"!=typeof k)throw Error("tagPolicy did not return object (old API?)");if(!("attribs"in k))throw Error("tagPolicy gave no attribs");e=k.attribs;var i;if("tagName"in k?(i=k.tagName,k=g.f[i]):(i=b,k=j),j&g.c.OPTIONAL_ENDTAG){var l=d[d.length-1];l&&l.D===b&&(l.v!==i||b!==i)&&h.push("</",l.v,">")}for(j&g.c.EMPTY||d.push({D:b,v:i}),h.push("<",i),b=0,l=e.length;b<l;b+=2){var m=e[b],o=e[b+1];o!==s&&o!==c&&h.push(" ",m,'="',E(o),'"')}h.push(">"),j&g.c.EMPTY&&!(k&g.c.EMPTY)&&h.push("</",i,">")}else f=!(j&g.c.EMPTY)}}},endTag:function(a,b){if(f)f=C;else if(g.f.hasOwnProperty(a)){var e=g.f[a];if(!(e&(g.c.EMPTY|g.c.FOLDABLE))){if(e&g.c.OPTIONAL_ENDTAG)for(e=d.length;0<=--e;){var h=d[e].D;if(h===a)break;if(!(g.f[h]&g.c.OPTIONAL_ENDTAG))return}else for(e=d.length;0<=--e&&d[e].D!==a;);if(!(0>e)){for(h=d.length;--h>e;){var j=d[h].v;g.f[j]&g.c.OPTIONAL_ENDTAG||b.push("</",j,">")}e<d.length&&(a=d[e].v),d.length=e,b.push("</",a,">")}}}},pcdata:b,rcdata:b,cdata:b,endDoc:function(a){for(;d.length;d.length--)a.push("</",d[d.length-1].v,">")}})}function d(a,b,d,e,f){if(!f)return s;try{var g=Y.parse(""+a);if(g&&(!g.K()||ka.test(g.W()))){var h=f(g,b,d,e);return h?h.toString():s}}catch(j){}return s}function y(a,b,d,f,e){if(d||a(b+" removed",{S:"removed",tagName:b}),f!==e){var g="changed";f&&!e?g="removed":!f&&e&&(g="added"),a(b+"."+d+" "+g,{S:g,tagName:b,la:d,oldValue:f,newValue:e})}}function l(a,b,d){return b=b+"::"+d,a.hasOwnProperty(b)?a[b]:(b="*::"+d,a.hasOwnProperty(b)?a[b]:void 0)}function V(a,b,f,e,h){for(var j=0;j<b.length;j+=2){var q,k=b[j],i=b[j+1],m=i,o=s;if(q=a+"::"+k,(g.m.hasOwnProperty(q)||(q="*::"+k,g.m.hasOwnProperty(q)))&&(o=g.m[q]),o!==s)switch(o){case g.d.NONE:break;case g.d.SCRIPT:i=s,h&&y(h,a,k,m,i);break;case g.d.STYLE:if("undefined"==typeof N){i=s,h&&y(h,a,k,m,i);break}var r=[];N(i,{declaration:function(a,b){var e=a.toLowerCase();u(e,b,f?function(a){return d(a,g.P.ja,g.M.ka,{TYPE:"CSS",CSS_PROP:e},f)}:s),b.length&&r.push(e+": "+b.join(" "))}}),i=0<r.length?r.join(" ; "):s,h&&y(h,a,k,m,i);break;case g.d.ID:case g.d.IDREF:case g.d.IDREFS:case g.d.GLOBAL_NAME:case g.d.LOCAL_NAME:case g.d.CLASSES:i=e?e(i):i,h&&y(h,a,k,m,i);break;case g.d.URI:i=d(i,l(g.J,a,k),l(g.I,a,k),{TYPE:"MARKUP",XML_ATTR:k,XML_TAG:a},f),h&&y(h,a,k,m,i);break;case g.d.URI_FRAGMENT:i&&"#"===i.charAt(0)?(i=i.substring(1),i=e?e(i):i,i!==s&&i!==c&&(i="#"+i)):i=s,h&&y(h,a,k,m,i);break;default:i=s,h&&y(h,a,k,m,i)}else i=s,h&&y(h,a,k,m,i);b[j+1]=i}return b}function ea(a,b,d){return function(e,f){return g.f[e]&g.c.UNSAFE?void(d&&y(d,e,c,c,c)):{attribs:V(e,f,a,b,d)}}}function Q(a,b){var d=[];return h(b)(a,d),d.join("")}var N,u;"undefined"!=typeof window&&(N=window.parseCssDeclarations,u=window.sanitizeCssProperty);var k={lt:"<",LT:"<",gt:">",GT:">",amp:"&",AMP:"&",quot:'"',apos:"'",nbsp:" "},q=/^#(\d+)$/,B=/^#x([0-9A-Fa-f]+)$/,F=/^[A-Za-z][A-za-z0-9]+$/,r="undefined"!=typeof window&&window.document?window.document.createElement("textarea"):s,o=/\0/g,j=/&(#[0-9]+|#[xX][0-9A-Fa-f]+|\w+);/g,I=/^(#[0-9]+|#[xX][0-9A-Fa-f]+|\w+);/,R=/&/g,ia=/&([^a-z#]|#(?:[^0-9x]|x(?:[^0-9a-f]|$)|$)|$)/gi,P=/[<]/g,S=/>/g,D=/\"/g,ja=/^\s*([-.:\w]+)(?:\s*(=)\s*((")[^"]*("|$)|(')[^']*('|$)|(?=[a-z][-\w]*\s*=)|[^"'\s]*))?/i,G=3==="a,b".split(/(,)/).length,K=g.c.CDATA|g.c.RCDATA,A={},T={},ka=/^(?:https?|geo|mailto|sms|tel)$/i,m={};return m.pa=m.escapeAttrib=E,m.ra=m.makeHtmlSanitizer=h,m.sa=m.makeSaxParser=v,m.ta=m.makeTagPolicy=ea,m.wa=m.normalizeRCData=e,m.xa=m.sanitize=function(a,b,d,e){return Q(a,ea(b,d,e))},m.ya=m.sanitizeAttribs=V,m.za=m.sanitizeWithPolicy=Q,m.Ba=m.unescapeEntities=x,m}($),la=ha.sanitize;"undefined"!=typeof window&&(window.html=ha,window.html_sanitize=la)}(),function(factory){"function"==typeof define&&define.amd?define(["jquery"],factory):"object"==typeof module&&module.exports?module.exports=factory(require("jquery")):factory(window.jQuery)}(function($){"use strict";var isSupportAmd="function"==typeof define&&define.amd,isFontInstalled=function(fontName){var testFontName="Comic Sans MS"===fontName?"Courier New":"Comic Sans MS",$tester=$("<div>").css({position:"absolute",left:"-9999px",top:"-9999px",fontSize:"200px"}).text("mmmmmmmmmwwwwwww").appendTo(document.body),originalWidth=$tester.css("fontFamily",testFontName).width(),width=$tester.css("fontFamily",fontName+","+testFontName).width();return $tester.remove(),originalWidth!==width},userAgent=navigator.userAgent,isMSIE=/MSIE|Trident/i.test(userAgent),browserVersion;if(isMSIE){var matches=/MSIE (\d+[.]\d+)/.exec(userAgent);matches&&(browserVersion=parseFloat(matches[1])),matches=/Trident\/.*rv:([0-9]{1,}[\.0-9]{0,})/.exec(userAgent),matches&&(browserVersion=parseFloat(matches[1]))}var isEdge=/Edge\/\d+/.test(userAgent),hasCodeMirror=!!window.CodeMirror;if(!hasCodeMirror&&isSupportAmd&&"undefined"!=typeof require)if("undefined"!=typeof require.resolve)try{require.resolve("codemirror"),hasCodeMirror=!0}catch(e){}else"undefined"!=typeof eval("require").specified&&(hasCodeMirror=eval("require").specified("codemirror"));var isSupportTouch="ontouchstart"in window||navigator.MaxTouchPoints>0||navigator.msMaxTouchPoints>0,agent={isMac:navigator.appVersion.indexOf("Mac")>-1,isMSIE:isMSIE,isEdge:isEdge,isFF:!isEdge&&/firefox/i.test(userAgent),isPhantom:/PhantomJS/i.test(userAgent),isWebkit:!isEdge&&/webkit/i.test(userAgent),isChrome:!isEdge&&/chrome/i.test(userAgent),isSafari:!isEdge&&/safari/i.test(userAgent),browserVersion:browserVersion,jqueryVersion:parseFloat($.fn.jquery),isSupportAmd:isSupportAmd,isSupportTouch:isSupportTouch,hasCodeMirror:hasCodeMirror,isFontInstalled:isFontInstalled,isW3CRangeSupport:!!document.createRange},func=function(){var eq=function(itemA){return function(itemB){return itemA===itemB}},eq2=function(itemA,itemB){return itemA===itemB},peq2=function(propName){return function(itemA,itemB){return itemA[propName]===itemB[propName]}},ok=function(){return!0},fail=function(){return!1},not=function(f){return function(){return!f.apply(f,arguments)}},and=function(fA,fB){return function(item){return fA(item)&&fB(item)}},self=function(a){return a},invoke=function(obj,method){return function(){return obj[method].apply(obj,arguments)}},idCounter=0,uniqueId=function(prefix){var id=++idCounter+"";return prefix?prefix+id:id},rect2bnd=function(rect){var $document=$(document);return{top:rect.top+$document.scrollTop(),left:rect.left+$document.scrollLeft(),width:rect.right-rect.left,height:rect.bottom-rect.top}},invertObject=function(obj){var inverted={};for(var key in obj)obj.hasOwnProperty(key)&&(inverted[obj[key]]=key);return inverted},namespaceToCamel=function(namespace,prefix){return prefix=prefix||"",prefix+namespace.split(".").map(function(name){return name.substring(0,1).toUpperCase()+name.substring(1)}).join("")},debounce=function(func,wait,immediate){var timeout;return function(){var context=this,args=arguments,later=function(){timeout=null,immediate||func.apply(context,args)},callNow=immediate&&!timeout;clearTimeout(timeout),timeout=setTimeout(later,wait),callNow&&func.apply(context,args)}};return{eq:eq,eq2:eq2,peq2:peq2,ok:ok,fail:fail,self:self,not:not,and:and,invoke:invoke,uniqueId:uniqueId,rect2bnd:rect2bnd,invertObject:invertObject,namespaceToCamel:namespaceToCamel,debounce:debounce}}(),list=function(){var head=function(array){return array[0]},last=function(array){return array[array.length-1]},initial=function(array){return array.slice(0,array.length-1)},tail=function(array){return array.slice(1)},find=function(array,pred){for(var idx=0,len=array.length;idx<len;idx++){var item=array[idx];if(pred(item))return item}},all=function(array,pred){for(var idx=0,len=array.length;idx<len;idx++)if(!pred(array[idx]))return!1;return!0},indexOf=function(array,item){return $.inArray(item,array)},contains=function(array,item){return indexOf(array,item)!==-1},sum=function(array,fn){return fn=fn||func.self,array.reduce(function(memo,v){return memo+fn(v)},0)},from=function(collection){for(var result=[],idx=-1,length=collection.length;++idx<length;)result[idx]=collection[idx];return result},isEmpty=function(array){return!array||!array.length},clusterBy=function(array,fn){if(!array.length)return[];var aTail=tail(array);return aTail.reduce(function(memo,v){var aLast=last(memo);return fn(last(aLast),v)?aLast[aLast.length]=v:memo[memo.length]=[v],memo},[[head(array)]])},compact=function(array){for(var aResult=[],idx=0,len=array.length;idx<len;idx++)array[idx]&&aResult.push(array[idx]);return aResult},unique=function(array){for(var results=[],idx=0,len=array.length;idx<len;idx++)contains(results,array[idx])||results.push(array[idx]);return results},next=function(array,item){var idx=indexOf(array,item);return idx===-1?null:array[idx+1]},prev=function(array,item){var idx=indexOf(array,item);return idx===-1?null:array[idx-1]};return{head:head,last:last,initial:initial,tail:tail,prev:prev,next:next,find:find,contains:contains,all:all,sum:sum,from:from,isEmpty:isEmpty,clusterBy:clusterBy,compact:compact,unique:unique}}(),NBSP_CHAR=String.fromCharCode(160),ZERO_WIDTH_NBSP_CHAR="\ufeff",dom=function(){var isEditable=function(node){return node&&$(node).hasClass("note-editable")},isControlSizing=function(node){return node&&$(node).hasClass("note-control-sizing")},makePredByNodeName=function(nodeName){return nodeName=nodeName.toUpperCase(),function(node){return node&&node.nodeName.toUpperCase()===nodeName}},isText=function(node){return node&&3===node.nodeType},isElement=function(node){return node&&1===node.nodeType},isVoid=function(node){return node&&/^BR|^IMG|^HR|^IFRAME|^BUTTON|^INPUT/.test(node.nodeName.toUpperCase())},isPara=function(node){return!isEditable(node)&&(node&&/^DIV|^P|^LI|^H[1-7]/.test(node.nodeName.toUpperCase()))},isHeading=function(node){return node&&/^H[1-7]/.test(node.nodeName.toUpperCase())},isPre=makePredByNodeName("PRE"),isLi=makePredByNodeName("LI"),isPurePara=function(node){return isPara(node)&&!isLi(node)},isTable=makePredByNodeName("TABLE"),isData=makePredByNodeName("DATA"),isInline=function(node){return!(isBodyContainer(node)||isList(node)||isHr(node)||isPara(node)||isTable(node)||isBlockquote(node)||isData(node))},isList=function(node){return node&&/^UL|^OL/.test(node.nodeName.toUpperCase())},isHr=makePredByNodeName("HR"),isCell=function(node){return node&&/^TD|^TH/.test(node.nodeName.toUpperCase())},isBlockquote=makePredByNodeName("BLOCKQUOTE"),isBodyContainer=function(node){return isCell(node)||isBlockquote(node)||isEditable(node)},isAnchor=makePredByNodeName("A"),isParaInline=function(node){return isInline(node)&&!!ancestor(node,isPara)},isBodyInline=function(node){return isInline(node)&&!ancestor(node,isPara)},isBody=makePredByNodeName("BODY"),isClosestSibling=function(nodeA,nodeB){return nodeA.nextSibling===nodeB||nodeA.previousSibling===nodeB},withClosestSiblings=function(node,pred){pred=pred||func.ok;var siblings=[];return node.previousSibling&&pred(node.previousSibling)&&siblings.push(node.previousSibling),siblings.push(node),node.nextSibling&&pred(node.nextSibling)&&siblings.push(node.nextSibling),siblings},blankHTML=agent.isMSIE&&agent.browserVersion<11?"&nbsp;":"<br>",nodeLength=function(node){return isText(node)?node.nodeValue.length:node?node.childNodes.length:0},isEmpty=function(node){var len=nodeLength(node);return 0===len||(!isText(node)&&1===len&&node.innerHTML===blankHTML||!(!list.all(node.childNodes,isText)||""!==node.innerHTML))},paddingBlankHTML=function(node){isVoid(node)||nodeLength(node)||(node.innerHTML=blankHTML)},ancestor=function(node,pred){for(;node;){if(pred(node))return node;if(isEditable(node))break;node=node.parentNode}return null},singleChildAncestor=function(node,pred){for(node=node.parentNode;node&&1===nodeLength(node);){if(pred(node))return node;if(isEditable(node))break;node=node.parentNode}return null},listAncestor=function(node,pred){pred=pred||func.fail;var ancestors=[];return ancestor(node,function(el){return isEditable(el)||ancestors.push(el),pred(el)}),ancestors},lastAncestor=function(node,pred){var ancestors=listAncestor(node);return list.last(ancestors.filter(pred))},commonAncestor=function(nodeA,nodeB){for(var ancestors=listAncestor(nodeA),n=nodeB;n;n=n.parentNode)if($.inArray(n,ancestors)>-1)return n;return null},listPrev=function(node,pred){pred=pred||func.fail;for(var nodes=[];node&&!pred(node);)nodes.push(node),node=node.previousSibling;return nodes},listNext=function(node,pred){pred=pred||func.fail;for(var nodes=[];node&&!pred(node);)nodes.push(node),node=node.nextSibling;return nodes},listDescendant=function(node,pred){var descendants=[];return pred=pred||func.ok,function fnWalk(current){node!==current&&pred(current)&&descendants.push(current);for(var idx=0,len=current.childNodes.length;idx<len;idx++)fnWalk(current.childNodes[idx])}(node),descendants},wrap=function(node,wrapperName){var parent=node.parentNode,wrapper=$("<"+wrapperName+">")[0];return parent.insertBefore(wrapper,node),wrapper.appendChild(node),wrapper},insertAfter=function(node,preceding){var next=preceding.nextSibling,parent=preceding.parentNode;return next?parent.insertBefore(node,next):parent.appendChild(node),node},appendChildNodes=function(node,aChild){return $.each(aChild,function(idx,child){node.appendChild(child)}),node},isLeftEdgePoint=function(point){return 0===point.offset},isRightEdgePoint=function(point){return point.offset===nodeLength(point.node)},isEdgePoint=function(point){return isLeftEdgePoint(point)||isRightEdgePoint(point)},isLeftEdgeOf=function(node,ancestor){for(;node&&node!==ancestor;){if(0!==position(node))return!1;node=node.parentNode}return!0},isRightEdgeOf=function(node,ancestor){if(!ancestor)return!1;for(;node&&node!==ancestor;){if(position(node)!==nodeLength(node.parentNode)-1)return!1;node=node.parentNode}return!0},isLeftEdgePointOf=function(point,ancestor){return isLeftEdgePoint(point)&&isLeftEdgeOf(point.node,ancestor)},isRightEdgePointOf=function(point,ancestor){return isRightEdgePoint(point)&&isRightEdgeOf(point.node,ancestor)},position=function(node){for(var offset=0;node=node.previousSibling;)offset+=1;return offset},hasChildren=function(node){return!!(node&&node.childNodes&&node.childNodes.length)},prevPoint=function(point,isSkipInnerOffset){var node,offset;if(0===point.offset){if(isEditable(point.node))return null;node=point.node.parentNode,offset=position(point.node)}else hasChildren(point.node)?(node=point.node.childNodes[point.offset-1],offset=nodeLength(node)):(node=point.node,offset=isSkipInnerOffset?0:point.offset-1);return{node:node,offset:offset}},nextPoint=function(point,isSkipInnerOffset){var node,offset;if(nodeLength(point.node)===point.offset){if(isEditable(point.node))return null;node=point.node.parentNode,offset=position(point.node)+1}else hasChildren(point.node)?(node=point.node.childNodes[point.offset],offset=0):(node=point.node,offset=isSkipInnerOffset?nodeLength(point.node):point.offset+1);return{node:node,offset:offset}},isSamePoint=function(pointA,pointB){return pointA.node===pointB.node&&pointA.offset===pointB.offset},isVisiblePoint=function(point){if(isText(point.node)||!hasChildren(point.node)||isEmpty(point.node))return!0;var leftNode=point.node.childNodes[point.offset-1],rightNode=point.node.childNodes[point.offset];return!(leftNode&&!isVoid(leftNode)||rightNode&&!isVoid(rightNode))},prevPointUntil=function(point,pred){for(;point;){if(pred(point))return point;point=prevPoint(point)}return null},nextPointUntil=function(point,pred){for(;point;){if(pred(point))return point;point=nextPoint(point)}return null},isCharPoint=function(point){if(!isText(point.node))return!1;var ch=point.node.nodeValue.charAt(point.offset-1);return ch&&" "!==ch&&ch!==NBSP_CHAR},walkPoint=function(startPoint,endPoint,handler,isSkipInnerOffset){for(var point=startPoint;point&&(handler(point),!isSamePoint(point,endPoint));){var isSkipOffset=isSkipInnerOffset&&startPoint.node!==point.node&&endPoint.node!==point.node;point=nextPoint(point,isSkipOffset)}},makeOffsetPath=function(ancestor,node){var ancestors=listAncestor(node,func.eq(ancestor));return ancestors.map(position).reverse()},fromOffsetPath=function(ancestor,offsets){for(var current=ancestor,i=0,len=offsets.length;i<len;i++)current=current.childNodes.length<=offsets[i]?current.childNodes[current.childNodes.length-1]:current.childNodes[offsets[i]];return current},splitNode=function(point,options){var isSkipPaddingBlankHTML=options&&options.isSkipPaddingBlankHTML,isNotSplitEdgePoint=options&&options.isNotSplitEdgePoint;if(isEdgePoint(point)&&(isText(point.node)||isNotSplitEdgePoint)){if(isLeftEdgePoint(point))return point.node;if(isRightEdgePoint(point))return point.node.nextSibling}if(isText(point.node))return point.node.splitText(point.offset);var childNode=point.node.childNodes[point.offset],clone=insertAfter(point.node.cloneNode(!1),point.node);return appendChildNodes(clone,listNext(childNode)),isSkipPaddingBlankHTML||(paddingBlankHTML(point.node),paddingBlankHTML(clone)),clone},splitTree=function(root,point,options){var ancestors=listAncestor(point.node,func.eq(root));return ancestors.length?1===ancestors.length?splitNode(point,options):ancestors.reduce(function(node,parent){
return node===point.node&&(node=splitNode(point,options)),splitNode({node:parent,offset:node?dom.position(node):nodeLength(parent)},options)}):null},splitPoint=function(point,isInline){var splitRoot,container,pred=isInline?isPara:isBodyContainer,ancestors=listAncestor(point.node,pred),topAncestor=list.last(ancestors)||point.node;pred(topAncestor)?(splitRoot=ancestors[ancestors.length-2],container=topAncestor):(splitRoot=topAncestor,container=splitRoot.parentNode);var pivot=splitRoot&&splitTree(splitRoot,point,{isSkipPaddingBlankHTML:isInline,isNotSplitEdgePoint:isInline});return pivot||container!==point.node||(pivot=point.node.childNodes[point.offset]),{rightNode:pivot,container:container}},create=function(nodeName){return document.createElement(nodeName)},createText=function(text){return document.createTextNode(text)},remove=function(node,isRemoveChild){if(node&&node.parentNode){if(node.removeNode)return node.removeNode(isRemoveChild);var parent=node.parentNode;if(!isRemoveChild){var i,len,nodes=[];for(i=0,len=node.childNodes.length;i<len;i++)nodes.push(node.childNodes[i]);for(i=0,len=nodes.length;i<len;i++)parent.insertBefore(nodes[i],node)}parent.removeChild(node)}},removeWhile=function(node,pred){for(;node&&!isEditable(node)&&pred(node);){var parent=node.parentNode;remove(node),node=parent}},replace=function(node,nodeName){if(node.nodeName.toUpperCase()===nodeName.toUpperCase())return node;var newNode=create(nodeName);return node.style.cssText&&(newNode.style.cssText=node.style.cssText),appendChildNodes(newNode,list.from(node.childNodes)),insertAfter(newNode,node),remove(node),newNode},isTextarea=makePredByNodeName("TEXTAREA"),value=function($node,stripLinebreaks){var val=isTextarea($node[0])?$node.val():$node.html();return stripLinebreaks?val.replace(/[\n\r]/g,""):val},html=function($node,isNewlineOnBlock){var markup=value($node);if(isNewlineOnBlock){var regexTag=/<(\/?)(\b(?!!)[^>\s]*)(.*?)(\s*\/?>)/g;markup=markup.replace(regexTag,function(match,endSlash,name){name=name.toUpperCase();var isEndOfInlineContainer=/^DIV|^TD|^TH|^P|^LI|^H[1-7]/.test(name)&&!!endSlash,isBlockNode=/^BLOCKQUOTE|^TABLE|^TBODY|^TR|^HR|^UL|^OL/.test(name);return match+(isEndOfInlineContainer||isBlockNode?"\n":"")}),markup=$.trim(markup)}return markup},posFromPlaceholder=function(placeholder){var $placeholder=$(placeholder),pos=$placeholder.offset(),height=$placeholder.outerHeight(!0);return{left:pos.left,top:pos.top+height}},attachEvents=function($node,events){Object.keys(events).forEach(function(key){$node.on(key,events[key])})},detachEvents=function($node,events){Object.keys(events).forEach(function(key){$node.off(key,events[key])})},isCustomStyleTag=function(node){return node&&!dom.isText(node)&&list.contains(node.classList,"note-styletag")};return{NBSP_CHAR:NBSP_CHAR,ZERO_WIDTH_NBSP_CHAR:ZERO_WIDTH_NBSP_CHAR,blank:blankHTML,emptyPara:"<p>"+blankHTML+"</p>",makePredByNodeName:makePredByNodeName,isEditable:isEditable,isControlSizing:isControlSizing,isText:isText,isElement:isElement,isVoid:isVoid,isPara:isPara,isPurePara:isPurePara,isHeading:isHeading,isInline:isInline,isBlock:func.not(isInline),isBodyInline:isBodyInline,isBody:isBody,isParaInline:isParaInline,isPre:isPre,isList:isList,isTable:isTable,isData:isData,isCell:isCell,isBlockquote:isBlockquote,isBodyContainer:isBodyContainer,isAnchor:isAnchor,isDiv:makePredByNodeName("DIV"),isLi:isLi,isBR:makePredByNodeName("BR"),isSpan:makePredByNodeName("SPAN"),isB:makePredByNodeName("B"),isU:makePredByNodeName("U"),isS:makePredByNodeName("S"),isI:makePredByNodeName("I"),isImg:makePredByNodeName("IMG"),isTextarea:isTextarea,isEmpty:isEmpty,isEmptyAnchor:func.and(isAnchor,isEmpty),isClosestSibling:isClosestSibling,withClosestSiblings:withClosestSiblings,nodeLength:nodeLength,isLeftEdgePoint:isLeftEdgePoint,isRightEdgePoint:isRightEdgePoint,isEdgePoint:isEdgePoint,isLeftEdgeOf:isLeftEdgeOf,isRightEdgeOf:isRightEdgeOf,isLeftEdgePointOf:isLeftEdgePointOf,isRightEdgePointOf:isRightEdgePointOf,prevPoint:prevPoint,nextPoint:nextPoint,isSamePoint:isSamePoint,isVisiblePoint:isVisiblePoint,prevPointUntil:prevPointUntil,nextPointUntil:nextPointUntil,isCharPoint:isCharPoint,walkPoint:walkPoint,ancestor:ancestor,singleChildAncestor:singleChildAncestor,listAncestor:listAncestor,lastAncestor:lastAncestor,listNext:listNext,listPrev:listPrev,listDescendant:listDescendant,commonAncestor:commonAncestor,wrap:wrap,insertAfter:insertAfter,appendChildNodes:appendChildNodes,position:position,hasChildren:hasChildren,makeOffsetPath:makeOffsetPath,fromOffsetPath:fromOffsetPath,splitTree:splitTree,splitPoint:splitPoint,create:create,createText:createText,remove:remove,removeWhile:removeWhile,replace:replace,html:html,value:value,posFromPlaceholder:posFromPlaceholder,attachEvents:attachEvents,detachEvents:detachEvents,isCustomStyleTag:isCustomStyleTag}}(),Context=function($note,options){var self=this,ui=$.summernote.ui;return this.memos={},this.modules={},this.layoutInfo={},this.options=options,this.initialize=function(){return this.layoutInfo=ui.createLayout($note,options),this._initialize(),$note.hide(),this},this.destroy=function(){this._destroy(),$note.removeData("summernote"),ui.removeLayout($note,this.layoutInfo)},this.reset=function(){var disabled=self.isDisabled();this.code(dom.emptyPara),this._destroy(),this._initialize(),disabled&&self.disable()},this._initialize=function(){var buttons=$.extend({},this.options.buttons);Object.keys(buttons).forEach(function(key){self.memo("button."+key,buttons[key])});var modules=$.extend({},this.options.modules,$.summernote.plugins||{});Object.keys(modules).forEach(function(key){self.module(key,modules[key],!0)}),Object.keys(this.modules).forEach(function(key){self.initializeModule(key)})},this._destroy=function(){Object.keys(this.modules).reverse().forEach(function(key){self.removeModule(key)}),Object.keys(this.memos).forEach(function(key){self.removeMemo(key)}),this.triggerEvent("destroy",this)},this.code=function(html){var isActivated=this.invoke("codeview.isActivated");return void 0===html?(this.invoke("codeview.sync"),isActivated?this.layoutInfo.codable.val():this.layoutInfo.editable.html()):(isActivated?this.layoutInfo.codable.val(html):this.layoutInfo.editable.html(html),$note.val(html),this.triggerEvent("change",html),void 0)},this.isDisabled=function(){return"false"===this.layoutInfo.editable.attr("contenteditable")},this.enable=function(){this.layoutInfo.editable.attr("contenteditable",!0),this.invoke("toolbar.activate",!0),this.triggerEvent("disable",!1)},this.disable=function(){this.invoke("codeview.isActivated")&&this.invoke("codeview.deactivate"),this.layoutInfo.editable.attr("contenteditable",!1),this.invoke("toolbar.deactivate",!0),this.triggerEvent("disable",!0)},this.triggerEvent=function(){var namespace=list.head(arguments),args=list.tail(list.from(arguments)),callback=this.options.callbacks[func.namespaceToCamel(namespace,"on")];callback&&callback.apply($note[0],args),$note.trigger("summernote."+namespace,args)},this.initializeModule=function(key){var module=this.modules[key];module.shouldInitialize=module.shouldInitialize||func.ok,module.shouldInitialize()&&(module.initialize&&module.initialize(),module.events&&dom.attachEvents($note,module.events))},this.module=function(key,ModuleClass,withoutIntialize){return 1===arguments.length?this.modules[key]:(this.modules[key]=new ModuleClass(this),void(withoutIntialize||this.initializeModule(key)))},this.removeModule=function(key){var module=this.modules[key];module.shouldInitialize()&&(module.events&&dom.detachEvents($note,module.events),module.destroy&&module.destroy()),delete this.modules[key]},this.memo=function(key,obj){return 1===arguments.length?this.memos[key]:void(this.memos[key]=obj)},this.removeMemo=function(key){this.memos[key]&&this.memos[key].destroy&&this.memos[key].destroy(),delete this.memos[key]},this.createInvokeHandlerAndUpdateState=function(namespace,value){return function(event){self.createInvokeHandler(namespace,value)(event),self.invoke("buttons.updateCurrentStyle")}},this.createInvokeHandler=function(namespace,value){return function(event){event.preventDefault();var $target=$(event.target);self.invoke(namespace,value||$target.closest("[data-value]").data("value"),$target)}},this.invoke=function(){var namespace=list.head(arguments),args=list.tail(list.from(arguments)),splits=namespace.split("."),hasSeparator=splits.length>1,moduleName=hasSeparator&&list.head(splits),methodName=hasSeparator?list.last(splits):list.head(splits),module=this.modules[moduleName||"editor"];return!moduleName&&this[methodName]?this[methodName].apply(this,args):module&&module[methodName]&&module.shouldInitialize()?module[methodName].apply(module,args):void 0},this.initialize()};$.fn.extend({summernote:function(){var type=$.type(list.head(arguments)),isExternalAPICalled="string"===type,hasInitOptions="object"===type,options=hasInitOptions?list.head(arguments):{};options=$.extend({},$.summernote.options,options),options.langInfo=$.extend(!0,{},$.summernote.lang["en-US"],$.summernote.lang[options.lang]),options.icons=$.extend(!0,{},$.summernote.options.icons,options.icons),options.tooltip="auto"===options.tooltip?!agent.isSupportTouch:options.tooltip,this.each(function(idx,note){var $note=$(note);if(!$note.data("summernote")){var context=new Context($note,options);$note.data("summernote",context),$note.data("summernote").triggerEvent("init",context.layoutInfo)}});var $note=this.first();if($note.length){var context=$note.data("summernote");if(isExternalAPICalled)return context.invoke.apply(context,list.from(arguments));options.focus&&context.invoke("editor.focus")}return this}});var Renderer=function(markup,children,options,callback){this.render=function($parent){var $node=$(markup);if(options&&options.contents&&$node.html(options.contents),options&&options.className&&$node.addClass(options.className),options&&options.data&&$.each(options.data,function(k,v){$node.attr("data-"+k,v)}),options&&options.click&&$node.on("click",options.click),children){var $container=$node.find(".note-children-container");children.forEach(function(child){child.render($container.length?$container:$node)})}return callback&&callback($node,options),options&&options.callback&&options.callback($node),$parent&&$parent.append($node),$node}},renderer={create:function(markup,callback){return function(){var children=$.isArray(arguments[0])?arguments[0]:[],options="object"==typeof arguments[1]?arguments[1]:arguments[0];return options&&options.children&&(children=options.children),new Renderer(markup,children,options,callback)}}},editor=renderer.create('<div class="note-editor note-frame panel panel-default"/>'),toolbar=renderer.create('<div class="note-toolbar panel-heading"/>'),editingArea=renderer.create('<div class="note-editing-area"/>'),codable=renderer.create('<textarea class="note-codable"/>'),editable=renderer.create('<div class="note-editable panel-body" contentEditable="true"/>'),statusbar=renderer.create(['<div class="note-statusbar">','  <div class="note-resizebar">','    <div class="note-icon-bar"/>','    <div class="note-icon-bar"/>','    <div class="note-icon-bar"/>',"  </div>","</div>"].join("")),airEditor=renderer.create('<div class="note-editor"/>'),airEditable=renderer.create('<div class="note-editable" contentEditable="true"/>'),buttonGroup=renderer.create('<div class="note-btn-group btn-group">'),dropdown=renderer.create('<div class="dropdown-menu">',function($node,options){var markup=$.isArray(options.items)?options.items.map(function(item){var value="string"==typeof item?item:item.value||"",content=options.template?options.template(item):item,option="object"==typeof item?item.option:void 0,dataValue='data-value="'+value+'"',dataOption=void 0!==option?' data-option="'+option+'"':"";return'<li><a href="#" '+(dataValue+dataOption)+">"+content+"</a></li>"}).join(""):options.items;$node.html(markup)}),dropdownCheck=renderer.create('<div class="dropdown-menu note-check">',function($node,options){var markup=$.isArray(options.items)?options.items.map(function(item){var value="string"==typeof item?item:item.value||"",content=options.template?options.template(item):item;return'<li><a href="#" data-value="'+value+'">'+icon(options.checkClassName)+" "+content+"</a></li>"}).join(""):options.items;$node.html(markup)}),palette=renderer.create('<div class="note-color-palette"/>',function($node,options){for(var contents=[],row=0,rowSize=options.colors.length;row<rowSize;row++){for(var eventName=options.eventName,colors=options.colors[row],buttons=[],col=0,colSize=colors.length;col<colSize;col++){var color=colors[col];buttons.push(['<button type="button" class="note-color-btn"','style="background-color:',color,'" ','data-event="',eventName,'" ','data-value="',color,'" ','title="',color,'" ','data-toggle="button" tabindex="-1"></button>'].join(""))}contents.push('<div class="note-color-row">'+buttons.join("")+"</div>")}$node.html(contents.join("")),options.tooltip&&$node.find(".note-color-btn").tooltip({container:"body",trigger:"hover",placement:"bottom"})}),dialog=renderer.create('<div class="modal" aria-hidden="false" tabindex="-1"/>',function($node,options){options.fade&&$node.addClass("fade"),$node.html(['<div class="modal-dialog">','  <div class="modal-content">',options.title?'    <div class="modal-header">      <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>      <h4 class="modal-title">'+options.title+"</h4>    </div>":"",'    <div class="modal-body">'+options.body+"</div>",options.footer?'    <div class="modal-footer">'+options.footer+"</div>":"","  </div>","</div>"].join(""))}),popover=renderer.create(['<div class="note-popover popover in">','  <div class="arrow"/>','  <div class="popover-content note-children-container"/>',"</div>"].join(""),function($node,options){var direction="undefined"!=typeof options.direction?options.direction:"bottom";$node.addClass(direction),options.hideArrow&&$node.find(".arrow").hide()}),icon=function(iconClassName,tagName){return tagName=tagName||"i","<"+tagName+' class="'+iconClassName+'"/>'},ui={editor:editor,toolbar:toolbar,editingArea:editingArea,codable:codable,editable:editable,statusbar:statusbar,airEditor:airEditor,airEditable:airEditable,buttonGroup:buttonGroup,dropdown:dropdown,dropdownCheck:dropdownCheck,palette:palette,dialog:dialog,popover:popover,icon:icon,options:{},button:function($node,options){return renderer.create('<button type="button" class="note-btn btn btn-default btn-sm" tabindex="-1">',function($node,options){options&&options.tooltip&&self.options.tooltip&&$node.attr({title:options.tooltip}).tooltip({container:"body",trigger:"hover",placement:"bottom"})})($node,options)},toggleBtn:function($btn,isEnable){$btn.toggleClass("disabled",!isEnable),$btn.attr("disabled",!isEnable)},toggleBtnActive:function($btn,isActive){$btn.toggleClass("active",isActive)},onDialogShown:function($dialog,handler){$dialog.one("shown.bs.modal",handler)},onDialogHidden:function($dialog,handler){$dialog.one("hidden.bs.modal",handler)},showDialog:function($dialog){$dialog.modal("show")},hideDialog:function($dialog){$dialog.modal("hide")},createLayout:function($note,options){self.options=options;var $editor=(options.airMode?ui.airEditor([ui.editingArea([ui.airEditable()])]):ui.editor([ui.toolbar(),ui.editingArea([ui.codable(),ui.editable()]),ui.statusbar()])).render();return $editor.insertAfter($note),{note:$note,editor:$editor,toolbar:$editor.find(".note-toolbar"),editingArea:$editor.find(".note-editing-area"),editable:$editor.find(".note-editable"),codable:$editor.find(".note-codable"),statusbar:$editor.find(".note-statusbar")}},removeLayout:function($note,layoutInfo){$note.html(layoutInfo.editable.html()),layoutInfo.editor.remove(),$note.show()}};$.summernote=$.summernote||{lang:{}},$.extend($.summernote.lang,{"en-US":{font:{bold:"Bold",italic:"Italic",underline:"Underline",clear:"Remove Font Style",height:"Line Height",name:"Font Family",strikethrough:"Strikethrough",subscript:"Subscript",superscript:"Superscript",size:"Font Size"},image:{image:"Picture",insert:"Insert Image",resizeFull:"Resize Full",resizeHalf:"Resize Half",resizeQuarter:"Resize Quarter",floatLeft:"Float Left",floatRight:"Float Right",floatNone:"Float None",shapeRounded:"Shape: Rounded",shapeCircle:"Shape: Circle",shapeThumbnail:"Shape: Thumbnail",shapeNone:"Shape: None",dragImageHere:"Drag image or text here",dropImage:"Drop image or Text",selectFromFiles:"Select from files",maximumFileSize:"Maximum file size",maximumFileSizeError:"Maximum file size exceeded.",url:"Image URL",remove:"Remove Image"},video:{video:"Video",videoLink:"Video Link",insert:"Insert Video",url:"Video URL?",providers:"(YouTube, Vimeo, Vine, Instagram, DailyMotion or Youku)"},link:{link:"Link",insert:"Insert Link",unlink:"Unlink",edit:"Edit",textToDisplay:"Text to display",url:"To what URL should this link go?",openInNewWindow:"Open in new window"},table:{table:"Table",addRowAbove:"Add row above",addRowBelow:"Add row below",addColLeft:"Add column left",addColRight:"Add column right",delRow:"Delete row",delCol:"Delete column",delTable:"Delete table"},hr:{insert:"Insert Horizontal Rule"},style:{style:"Style",p:"Normal",blockquote:"Quote",pre:"Code",h1:"Header 1",h2:"Header 2",h3:"Header 3",h4:"Header 4",h5:"Header 5",h6:"Header 6"},lists:{unordered:"Unordered list",ordered:"Ordered list"},options:{help:"Help",fullscreen:"Full Screen",codeview:"Code View"},paragraph:{paragraph:"Paragraph",outdent:"Outdent",indent:"Indent",left:"Align left",center:"Align center",right:"Align right",justify:"Justify full"},color:{recent:"Recent Color",more:"More Color",background:"Background Color",foreground:"Foreground Color",transparent:"Transparent",setTransparent:"Set transparent",reset:"Reset",resetToDefault:"Reset to default"},shortcut:{shortcuts:"Keyboard shortcuts",close:"Close",textFormatting:"Text formatting",action:"Action",paragraphFormatting:"Paragraph formatting",documentStyle:"Document Style",extraKeys:"Extra keys"},help:{insertParagraph:"Insert Paragraph",undo:"Undoes the last command",redo:"Redoes the last command",tab:"Tab",untab:"Untab",bold:"Set a bold style",italic:"Set a italic style",underline:"Set a underline style",strikethrough:"Set a strikethrough style",removeFormat:"Clean a style",justifyLeft:"Set left align",justifyCenter:"Set center align",justifyRight:"Set right align",justifyFull:"Set full align",insertUnorderedList:"Toggle unordered list",insertOrderedList:"Toggle ordered list",outdent:"Outdent on current paragraph",indent:"Indent on current paragraph",formatPara:"Change current block's format as a paragraph(P tag)",formatH1:"Change current block's format as H1",formatH2:"Change current block's format as H2",formatH3:"Change current block's format as H3",formatH4:"Change current block's format as H4",formatH5:"Change current block's format as H5",formatH6:"Change current block's format as H6",insertHorizontalRule:"Insert horizontal rule","linkDialog.show":"Show Link Dialog"},history:{undo:"Undo",redo:"Redo"},specialChar:{specialChar:"SPECIAL CHARACTERS",select:"Select Special characters"}}});var key=function(){var keyMap={BACKSPACE:8,TAB:9,ENTER:13,SPACE:32,DELETE:46,LEFT:37,UP:38,RIGHT:39,DOWN:40,NUM0:48,NUM1:49,NUM2:50,NUM3:51,NUM4:52,NUM5:53,NUM6:54,NUM7:55,NUM8:56,B:66,E:69,I:73,J:74,K:75,L:76,R:82,S:83,U:85,V:86,Y:89,Z:90,SLASH:191,LEFTBRACKET:219,BACKSLASH:220,RIGHTBRACKET:221};return{isEdit:function(keyCode){return list.contains([keyMap.BACKSPACE,keyMap.TAB,keyMap.ENTER,keyMap.SPACE,keyMap.DELETE],keyCode)},isMove:function(keyCode){return list.contains([keyMap.LEFT,keyMap.UP,keyMap.RIGHT,keyMap.DOWN],keyCode)},nameFromCode:func.invertObject(keyMap),code:keyMap}}(),range=function(){var textRangeToPoint=function(textRange,isStart){var offset,prevContainer,container=textRange.parentElement(),tester=document.body.createTextRange(),childNodes=list.from(container.childNodes);for(offset=0;offset<childNodes.length;offset++)if(!dom.isText(childNodes[offset])){if(tester.moveToElementText(childNodes[offset]),tester.compareEndPoints("StartToStart",textRange)>=0)break;prevContainer=childNodes[offset]}if(0!==offset&&dom.isText(childNodes[offset-1])){var textRangeStart=document.body.createTextRange(),curTextNode=null;textRangeStart.moveToElementText(prevContainer||container),textRangeStart.collapse(!prevContainer),curTextNode=prevContainer?prevContainer.nextSibling:container.firstChild;var pointTester=textRange.duplicate();pointTester.setEndPoint("StartToStart",textRangeStart);for(var textCount=pointTester.text.replace(/[\r\n]/g,"").length;textCount>curTextNode.nodeValue.length&&curTextNode.nextSibling;)textCount-=curTextNode.nodeValue.length,curTextNode=curTextNode.nextSibling;curTextNode.nodeValue;isStart&&curTextNode.nextSibling&&dom.isText(curTextNode.nextSibling)&&textCount===curTextNode.nodeValue.length&&(textCount-=curTextNode.nodeValue.length,curTextNode=curTextNode.nextSibling),container=curTextNode,offset=textCount}return{cont:container,offset:offset}},pointToTextRange=function(point){var textRangeInfo=function(container,offset){var node,isCollapseToStart;if(dom.isText(container)){var prevTextNodes=dom.listPrev(container,func.not(dom.isText)),prevContainer=list.last(prevTextNodes).previousSibling;node=prevContainer||container.parentNode,offset+=list.sum(list.tail(prevTextNodes),dom.nodeLength),isCollapseToStart=!prevContainer}else{if(node=container.childNodes[offset]||container,dom.isText(node))return textRangeInfo(node,0);offset=0,isCollapseToStart=!1}return{node:node,collapseToStart:isCollapseToStart,offset:offset}},textRange=document.body.createTextRange(),info=textRangeInfo(point.node,point.offset);return textRange.moveToElementText(info.node),textRange.collapse(info.collapseToStart),textRange.moveStart("character",info.offset),textRange},WrappedRange=function(sc,so,ec,eo){this.sc=sc,this.so=so,this.ec=ec,this.eo=eo;var nativeRange=function(){if(agent.isW3CRangeSupport){var w3cRange=document.createRange();return w3cRange.setStart(sc,so),w3cRange.setEnd(ec,eo),w3cRange}var textRange=pointToTextRange({node:sc,offset:so});return textRange.setEndPoint("EndToEnd",pointToTextRange({node:ec,offset:eo})),textRange};this.getPoints=function(){return{sc:sc,so:so,ec:ec,eo:eo}},this.getStartPoint=function(){return{node:sc,offset:so}},this.getEndPoint=function(){return{node:ec,offset:eo}},this.select=function(){var nativeRng=nativeRange();if(agent.isW3CRangeSupport){var selection=document.getSelection();selection.rangeCount>0&&selection.removeAllRanges(),selection.addRange(nativeRng)}else nativeRng.select();return this},this.scrollIntoView=function(container){var height=$(container).height();return container.scrollTop+height<this.sc.offsetTop&&(container.scrollTop+=Math.abs(container.scrollTop+height-this.sc.offsetTop)),this},this.normalize=function(){var getVisiblePoint=function(point,isLeftToRight){if(dom.isVisiblePoint(point)&&!dom.isEdgePoint(point)||dom.isVisiblePoint(point)&&dom.isRightEdgePoint(point)&&!isLeftToRight||dom.isVisiblePoint(point)&&dom.isLeftEdgePoint(point)&&isLeftToRight||dom.isVisiblePoint(point)&&dom.isBlock(point.node)&&dom.isEmpty(point.node))return point;var block=dom.ancestor(point.node,dom.isBlock);if((dom.isLeftEdgePointOf(point,block)||dom.isVoid(dom.prevPoint(point).node))&&!isLeftToRight||(dom.isRightEdgePointOf(point,block)||dom.isVoid(dom.nextPoint(point).node))&&isLeftToRight){if(dom.isVisiblePoint(point))return point;isLeftToRight=!isLeftToRight}var nextPoint=isLeftToRight?dom.nextPointUntil(dom.nextPoint(point),dom.isVisiblePoint):dom.prevPointUntil(dom.prevPoint(point),dom.isVisiblePoint);return nextPoint||point},endPoint=getVisiblePoint(this.getEndPoint(),!1),startPoint=this.isCollapsed()?endPoint:getVisiblePoint(this.getStartPoint(),!0);return new WrappedRange(startPoint.node,startPoint.offset,endPoint.node,endPoint.offset)},this.nodes=function(pred,options){pred=pred||func.ok;var includeAncestor=options&&options.includeAncestor,fullyContains=options&&options.fullyContains,startPoint=this.getStartPoint(),endPoint=this.getEndPoint(),nodes=[],leftEdgeNodes=[];return dom.walkPoint(startPoint,endPoint,function(point){if(!dom.isEditable(point.node)){var node;fullyContains?(dom.isLeftEdgePoint(point)&&leftEdgeNodes.push(point.node),dom.isRightEdgePoint(point)&&list.contains(leftEdgeNodes,point.node)&&(node=point.node)):node=includeAncestor?dom.ancestor(point.node,pred):point.node,node&&pred(node)&&nodes.push(node)}},!0),list.unique(nodes)},this.commonAncestor=function(){return dom.commonAncestor(sc,ec)},this.expand=function(pred){var startAncestor=dom.ancestor(sc,pred),endAncestor=dom.ancestor(ec,pred);if(!startAncestor&&!endAncestor)return new WrappedRange(sc,so,ec,eo);var boundaryPoints=this.getPoints();return startAncestor&&(boundaryPoints.sc=startAncestor,boundaryPoints.so=0),endAncestor&&(boundaryPoints.ec=endAncestor,boundaryPoints.eo=dom.nodeLength(endAncestor)),new WrappedRange(boundaryPoints.sc,boundaryPoints.so,boundaryPoints.ec,boundaryPoints.eo)},this.collapse=function(isCollapseToStart){return isCollapseToStart?new WrappedRange(sc,so,sc,so):new WrappedRange(ec,eo,ec,eo)},this.splitText=function(){var isSameContainer=sc===ec,boundaryPoints=this.getPoints();return dom.isText(ec)&&!dom.isEdgePoint(this.getEndPoint())&&ec.splitText(eo),dom.isText(sc)&&!dom.isEdgePoint(this.getStartPoint())&&(boundaryPoints.sc=sc.splitText(so),boundaryPoints.so=0,isSameContainer&&(boundaryPoints.ec=boundaryPoints.sc,boundaryPoints.eo=eo-so)),new WrappedRange(boundaryPoints.sc,boundaryPoints.so,boundaryPoints.ec,boundaryPoints.eo)},this.deleteContents=function(){if(this.isCollapsed())return this;var rng=this.splitText(),nodes=rng.nodes(null,{fullyContains:!0}),point=dom.prevPointUntil(rng.getStartPoint(),function(point){return!list.contains(nodes,point.node)}),emptyParents=[];return $.each(nodes,function(idx,node){var parent=node.parentNode;point.node!==parent&&1===dom.nodeLength(parent)&&emptyParents.push(parent),dom.remove(node,!1)}),$.each(emptyParents,function(idx,node){dom.remove(node,!1)}),new WrappedRange(point.node,point.offset,point.node,point.offset).normalize()};var makeIsOn=function(pred){return function(){var ancestor=dom.ancestor(sc,pred);return!!ancestor&&ancestor===dom.ancestor(ec,pred)}};this.isOnEditable=makeIsOn(dom.isEditable),this.isOnList=makeIsOn(dom.isList),this.isOnAnchor=makeIsOn(dom.isAnchor),this.isOnCell=makeIsOn(dom.isCell),this.isOnData=makeIsOn(dom.isData),this.isLeftEdgeOf=function(pred){if(!dom.isLeftEdgePoint(this.getStartPoint()))return!1;var node=dom.ancestor(this.sc,pred);return node&&dom.isLeftEdgeOf(this.sc,node)},this.isCollapsed=function(){return sc===ec&&so===eo},this.wrapBodyInlineWithPara=function(){if(dom.isBodyContainer(sc)&&dom.isEmpty(sc))return sc.innerHTML=dom.emptyPara,new WrappedRange(sc.firstChild,0,sc.firstChild,0);var rng=this.normalize();if(dom.isParaInline(sc)||dom.isPara(sc))return rng;var topAncestor;if(dom.isInline(rng.sc)){var ancestors=dom.listAncestor(rng.sc,func.not(dom.isInline));topAncestor=list.last(ancestors),dom.isInline(topAncestor)||(topAncestor=ancestors[ancestors.length-2]||rng.sc.childNodes[rng.so])}else topAncestor=rng.sc.childNodes[rng.so>0?rng.so-1:0];var inlineSiblings=dom.listPrev(topAncestor,dom.isParaInline).reverse();if(inlineSiblings=inlineSiblings.concat(dom.listNext(topAncestor.nextSibling,dom.isParaInline)),inlineSiblings.length){var para=dom.wrap(list.head(inlineSiblings),"p");dom.appendChildNodes(para,list.tail(inlineSiblings))}return this.normalize()},this.insertNode=function(node){var rng=this.wrapBodyInlineWithPara().deleteContents(),info=dom.splitPoint(rng.getStartPoint(),dom.isInline(node));return info.rightNode?info.rightNode.parentNode.insertBefore(node,info.rightNode):info.container.appendChild(node),node},this.pasteHTML=function(markup){var contentsContainer=$("<div></div>").html(markup)[0],childNodes=list.from(contentsContainer.childNodes),rng=this.wrapBodyInlineWithPara().deleteContents();return childNodes.reverse().map(function(childNode){return rng.insertNode(childNode)}).reverse()},this.toString=function(){var nativeRng=nativeRange();return agent.isW3CRangeSupport?nativeRng.toString():nativeRng.text},this.getWordRange=function(findAfter){var endPoint=this.getEndPoint();if(!dom.isCharPoint(endPoint))return this;var startPoint=dom.prevPointUntil(endPoint,function(point){return!dom.isCharPoint(point)});return findAfter&&(endPoint=dom.nextPointUntil(endPoint,function(point){return!dom.isCharPoint(point)})),new WrappedRange(startPoint.node,startPoint.offset,endPoint.node,endPoint.offset)},this.bookmark=function(editable){return{s:{path:dom.makeOffsetPath(editable,sc),offset:so},e:{path:dom.makeOffsetPath(editable,ec),offset:eo}}},this.paraBookmark=function(paras){return{s:{path:list.tail(dom.makeOffsetPath(list.head(paras),sc)),offset:so},e:{path:list.tail(dom.makeOffsetPath(list.last(paras),ec)),offset:eo}}},this.getClientRects=function(){var nativeRng=nativeRange();return nativeRng.getClientRects()}};return{create:function(sc,so,ec,eo){if(4===arguments.length)return new WrappedRange(sc,so,ec,eo);if(2===arguments.length)return ec=sc,eo=so,new WrappedRange(sc,so,ec,eo);var wrappedRange=this.createFromSelection();return wrappedRange||1!==arguments.length?wrappedRange:(wrappedRange=this.createFromNode(arguments[0]),wrappedRange.collapse(dom.emptyPara===arguments[0].innerHTML))},createFromSelection:function(){var sc,so,ec,eo;if(agent.isW3CRangeSupport){var selection=document.getSelection();if(!selection||0===selection.rangeCount)return null;if(dom.isBody(selection.anchorNode))return null;var nativeRng=selection.getRangeAt(0);sc=nativeRng.startContainer,so=nativeRng.startOffset,ec=nativeRng.endContainer,eo=nativeRng.endOffset}else{var textRange=document.selection.createRange(),textRangeEnd=textRange.duplicate();textRangeEnd.collapse(!1);var textRangeStart=textRange;textRangeStart.collapse(!0);var startPoint=textRangeToPoint(textRangeStart,!0),endPoint=textRangeToPoint(textRangeEnd,!1);dom.isText(startPoint.node)&&dom.isLeftEdgePoint(startPoint)&&dom.isTextNode(endPoint.node)&&dom.isRightEdgePoint(endPoint)&&endPoint.node.nextSibling===startPoint.node&&(startPoint=endPoint),sc=startPoint.cont,so=startPoint.offset,ec=endPoint.cont,eo=endPoint.offset}return new WrappedRange(sc,so,ec,eo)},createFromNode:function(node){var sc=node,so=0,ec=node,eo=dom.nodeLength(ec);return dom.isVoid(sc)&&(so=dom.listPrev(sc).length-1,sc=sc.parentNode),dom.isBR(ec)?(eo=dom.listPrev(ec).length-1,ec=ec.parentNode):dom.isVoid(ec)&&(eo=dom.listPrev(ec).length,ec=ec.parentNode),this.create(sc,so,ec,eo)},createFromNodeBefore:function(node){return this.createFromNode(node).collapse(!0)},createFromNodeAfter:function(node){return this.createFromNode(node).collapse()},createFromBookmark:function(editable,bookmark){var sc=dom.fromOffsetPath(editable,bookmark.s.path),so=bookmark.s.offset,ec=dom.fromOffsetPath(editable,bookmark.e.path),eo=bookmark.e.offset;return new WrappedRange(sc,so,ec,eo)},createFromParaBookmark:function(bookmark,paras){var so=bookmark.s.offset,eo=bookmark.e.offset,sc=dom.fromOffsetPath(list.head(paras),bookmark.s.path),ec=dom.fromOffsetPath(list.last(paras),bookmark.e.path);return new WrappedRange(sc,so,ec,eo)}}}(),async=function(){var readFileAsDataURL=function(file){return $.Deferred(function(deferred){$.extend(new FileReader,{onload:function(e){var dataURL=e.target.result;deferred.resolve(dataURL)},onerror:function(){deferred.reject(this)}}).readAsDataURL(file)}).promise()},createImage=function(url){return $.Deferred(function(deferred){var $img=$("<img>");$img.one("load",function(){$img.off("error abort"),deferred.resolve($img)}).one("error abort",function(){$img.off("load").detach(),deferred.reject($img)}).css({display:"none"}).appendTo(document.body).attr("src",url)}).promise()};return{readFileAsDataURL:readFileAsDataURL,createImage:createImage
}}(),History=function($editable){var stack=[],stackOffset=-1,editable=$editable[0],makeSnapshot=function(){var rng=range.create(editable),emptyBookmark={s:{path:[],offset:0},e:{path:[],offset:0}};return{contents:$editable.html(),bookmark:rng?rng.bookmark(editable):emptyBookmark}},applySnapshot=function(snapshot){null!==snapshot.contents&&$editable.html(snapshot.contents),null!==snapshot.bookmark&&range.createFromBookmark(editable,snapshot.bookmark).select()};this.rewind=function(){$editable.html()!==stack[stackOffset].contents&&this.recordUndo(),stackOffset=0,applySnapshot(stack[stackOffset])},this.reset=function(){stack=[],stackOffset=-1,$editable.html(""),this.recordUndo()},this.undo=function(){$editable.html()!==stack[stackOffset].contents&&this.recordUndo(),0<stackOffset&&(stackOffset--,applySnapshot(stack[stackOffset]))},this.redo=function(){stack.length-1>stackOffset&&(stackOffset++,applySnapshot(stack[stackOffset]))},this.recordUndo=function(){stackOffset++,stack.length>stackOffset&&(stack=stack.slice(0,stackOffset)),stack.push(makeSnapshot())}},Style=function(){var jQueryCSS=function($obj,propertyNames){if(agent.jqueryVersion<1.9){var result={};return $.each(propertyNames,function(idx,propertyName){result[propertyName]=$obj.css(propertyName)}),result}return $obj.css.call($obj,propertyNames)};this.fromNode=function($node){var properties=["font-family","font-size","text-align","list-style-type","line-height"],styleInfo=jQueryCSS($node,properties)||{};return styleInfo["font-size"]=parseInt(styleInfo["font-size"],10),styleInfo},this.stylePara=function(rng,styleInfo){$.each(rng.nodes(dom.isPara,{includeAncestor:!0}),function(idx,para){$(para).css(styleInfo)})},this.styleNodes=function(rng,options){rng=rng.splitText();var nodeName=options&&options.nodeName||"SPAN",expandClosestSibling=!(!options||!options.expandClosestSibling),onlyPartialContains=!(!options||!options.onlyPartialContains);if(rng.isCollapsed())return[rng.insertNode(dom.create(nodeName))];var pred=dom.makePredByNodeName(nodeName),nodes=rng.nodes(dom.isText,{fullyContains:!0}).map(function(text){return dom.singleChildAncestor(text,pred)||dom.wrap(text,nodeName)});if(expandClosestSibling){if(onlyPartialContains){var nodesInRange=rng.nodes();pred=func.and(pred,function(node){return list.contains(nodesInRange,node)})}return nodes.map(function(node){var siblings=dom.withClosestSiblings(node,pred),head=list.head(siblings),tails=list.tail(siblings);return $.each(tails,function(idx,elem){dom.appendChildNodes(head,elem.childNodes),dom.remove(elem)}),list.head(siblings)})}return nodes},this.current=function(rng){var $cont=$(dom.isElement(rng.sc)?rng.sc:rng.sc.parentNode),styleInfo=this.fromNode($cont);try{styleInfo=$.extend(styleInfo,{"font-bold":document.queryCommandState("bold")?"bold":"normal","font-italic":document.queryCommandState("italic")?"italic":"normal","font-underline":document.queryCommandState("underline")?"underline":"normal","font-subscript":document.queryCommandState("subscript")?"subscript":"normal","font-superscript":document.queryCommandState("superscript")?"superscript":"normal","font-strikethrough":document.queryCommandState("strikethrough")?"strikethrough":"normal","font-family":document.queryCommandValue("fontname")||styleInfo["font-family"]})}catch(e){}if(rng.isOnList()){var orderedTypes=["circle","disc","disc-leading-zero","square"],isUnordered=$.inArray(styleInfo["list-style-type"],orderedTypes)>-1;styleInfo["list-style"]=isUnordered?"unordered":"ordered"}else styleInfo["list-style"]="none";var para=dom.ancestor(rng.sc,dom.isPara);if(para&&para.style["line-height"])styleInfo["line-height"]=para.style.lineHeight;else{var lineHeight=parseInt(styleInfo["line-height"],10)/parseInt(styleInfo["font-size"],10);styleInfo["line-height"]=lineHeight.toFixed(1)}return styleInfo.anchor=rng.isOnAnchor()&&dom.ancestor(rng.sc,dom.isAnchor),styleInfo.ancestors=dom.listAncestor(rng.sc,dom.isEditable),styleInfo.range=rng,styleInfo}},Bullet=function(){var self=this;this.insertOrderedList=function(editable){this.toggleList("OL",editable)},this.insertUnorderedList=function(editable){this.toggleList("UL",editable)},this.indent=function(editable){var self=this,rng=range.create(editable).wrapBodyInlineWithPara(),paras=rng.nodes(dom.isPara,{includeAncestor:!0}),clustereds=list.clusterBy(paras,func.peq2("parentNode"));$.each(clustereds,function(idx,paras){var head=list.head(paras);dom.isLi(head)?self.wrapList(paras,head.parentNode.nodeName):$.each(paras,function(idx,para){$(para).css("marginLeft",function(idx,val){return(parseInt(val,10)||0)+25})})}),rng.select()},this.outdent=function(editable){var self=this,rng=range.create(editable).wrapBodyInlineWithPara(),paras=rng.nodes(dom.isPara,{includeAncestor:!0}),clustereds=list.clusterBy(paras,func.peq2("parentNode"));$.each(clustereds,function(idx,paras){var head=list.head(paras);dom.isLi(head)?self.releaseList([paras]):$.each(paras,function(idx,para){$(para).css("marginLeft",function(idx,val){return val=parseInt(val,10)||0,val>25?val-25:""})})}),rng.select()},this.toggleList=function(listName,editable){var rng=range.create(editable).wrapBodyInlineWithPara(),paras=rng.nodes(dom.isPara,{includeAncestor:!0}),bookmark=rng.paraBookmark(paras),clustereds=list.clusterBy(paras,func.peq2("parentNode"));if(list.find(paras,dom.isPurePara)){var wrappedParas=[];$.each(clustereds,function(idx,paras){wrappedParas=wrappedParas.concat(self.wrapList(paras,listName))}),paras=wrappedParas}else{var diffLists=rng.nodes(dom.isList,{includeAncestor:!0}).filter(function(listNode){return!$.nodeName(listNode,listName)});diffLists.length?$.each(diffLists,function(idx,listNode){dom.replace(listNode,listName)}):paras=this.releaseList(clustereds,!0)}range.createFromParaBookmark(bookmark,paras).select()},this.wrapList=function(paras,listName){var head=list.head(paras),last=list.last(paras),prevList=dom.isList(head.previousSibling)&&head.previousSibling,nextList=dom.isList(last.nextSibling)&&last.nextSibling,listNode=prevList||dom.insertAfter(dom.create(listName||"UL"),last);return paras=paras.map(function(para){return dom.isPurePara(para)?dom.replace(para,"LI"):para}),dom.appendChildNodes(listNode,paras),nextList&&(dom.appendChildNodes(listNode,list.from(nextList.childNodes)),dom.remove(nextList)),paras},this.releaseList=function(clustereds,isEscapseToBody){var releasedParas=[];return $.each(clustereds,function(idx,paras){var head=list.head(paras),last=list.last(paras),headList=isEscapseToBody?dom.lastAncestor(head,dom.isList):head.parentNode,lastList=headList.childNodes.length>1?dom.splitTree(headList,{node:last.parentNode,offset:dom.position(last)+1},{isSkipPaddingBlankHTML:!0}):null,middleList=dom.splitTree(headList,{node:head.parentNode,offset:dom.position(head)},{isSkipPaddingBlankHTML:!0});paras=isEscapseToBody?dom.listDescendant(middleList,dom.isLi):list.from(middleList.childNodes).filter(dom.isLi),!isEscapseToBody&&dom.isList(headList.parentNode)||(paras=paras.map(function(para){return dom.replace(para,"P")})),$.each(list.from(paras).reverse(),function(idx,para){dom.insertAfter(para,headList)});var rootLists=list.compact([headList,middleList,lastList]);$.each(rootLists,function(idx,rootList){var listNodes=[rootList].concat(dom.listDescendant(rootList,dom.isList));$.each(listNodes.reverse(),function(idx,listNode){dom.nodeLength(listNode)||dom.remove(listNode,!0)})}),releasedParas=releasedParas.concat(paras)}),releasedParas}},Typing=function(){var bullet=new Bullet;this.insertTab=function(rng,tabsize){var tab=dom.createText(new Array(tabsize+1).join(dom.NBSP_CHAR));rng=rng.deleteContents(),rng.insertNode(tab,!0),rng=range.create(tab,tabsize),rng.select()},this.insertParagraph=function(editable){var rng=range.create(editable);rng=rng.deleteContents(),rng=rng.wrapBodyInlineWithPara();var nextPara,splitRoot=dom.ancestor(rng.sc,dom.isPara);if(splitRoot){if(dom.isEmpty(splitRoot)&&dom.isLi(splitRoot))return void bullet.toggleList(splitRoot.parentNode.nodeName);if(dom.isEmpty(splitRoot)&&dom.isPara(splitRoot)&&dom.isBlockquote(splitRoot.parentNode))dom.insertAfter(splitRoot,splitRoot.parentNode),nextPara=splitRoot;else{nextPara=dom.splitTree(splitRoot,rng.getStartPoint());var emptyAnchors=dom.listDescendant(splitRoot,dom.isEmptyAnchor);emptyAnchors=emptyAnchors.concat(dom.listDescendant(nextPara,dom.isEmptyAnchor)),$.each(emptyAnchors,function(idx,anchor){dom.remove(anchor)}),(dom.isHeading(nextPara)||dom.isPre(nextPara)||dom.isCustomStyleTag(nextPara))&&dom.isEmpty(nextPara)&&(nextPara=dom.replace(nextPara,"p"))}}else{var next=rng.sc.childNodes[rng.so];nextPara=$(dom.emptyPara)[0],next?rng.sc.insertBefore(nextPara,next):rng.sc.appendChild(nextPara)}range.create(nextPara,0).normalize().select().scrollIntoView(editable)}},TableResultAction=function(startPoint,where,action,domTable){function setStartPoint(){return!startPoint||!startPoint.tagName||"td"!==startPoint.tagName.toLowerCase()&&"th"!==startPoint.tagName.toLowerCase()?void console.error("Impossible to identify start Cell point.",startPoint):(_startPoint.colPos=startPoint.cellIndex,startPoint.parentElement&&startPoint.parentElement.tagName&&"tr"===startPoint.parentElement.tagName.toLowerCase()?void(_startPoint.rowPos=startPoint.parentElement.rowIndex):void console.error("Impossible to identify start Row point.",startPoint))}function setVirtualTablePosition(rowIndex,cellIndex,baseRow,baseCell,isRowSpan,isColSpan,isVirtualCell){var objPosition={baseRow:baseRow,baseCell:baseCell,isRowSpan:isRowSpan,isColSpan:isColSpan,isVirtual:isVirtualCell};_virtualTable[rowIndex]||(_virtualTable[rowIndex]=[]),_virtualTable[rowIndex][cellIndex]=objPosition}function getActionCell(virtualTableCellObj,resultAction,virtualRowPosition,virtualColPosition){return{baseCell:virtualTableCellObj.baseCell,action:resultAction,virtualTable:{rowIndex:virtualRowPosition,cellIndex:virtualColPosition}}}function recoverCellIndex(rowIndex,cellIndex){if(!_virtualTable[rowIndex])return cellIndex;if(!_virtualTable[rowIndex][cellIndex])return cellIndex;for(var newCellIndex=cellIndex;_virtualTable[rowIndex][newCellIndex];)if(newCellIndex++,!_virtualTable[rowIndex][newCellIndex])return newCellIndex}function addCellInfoToVirtual(row,cell){var cellIndex=recoverCellIndex(row.rowIndex,cell.cellIndex),cellHasColspan=cell.colSpan>1,cellHasRowspan=cell.rowSpan>1,isThisSelectedCell=row.rowIndex===_startPoint.rowPos&&cell.cellIndex===_startPoint.colPos;setVirtualTablePosition(row.rowIndex,cellIndex,row,cell,cellHasRowspan,cellHasColspan,!1);var rowspanNumber=cell.attributes.rowSpan?parseInt(cell.attributes.rowSpan.value,10):0;if(rowspanNumber>1)for(var rp=1;rp<rowspanNumber;rp++){var rowspanIndex=row.rowIndex+rp;adjustStartPoint(rowspanIndex,cellIndex,cell,isThisSelectedCell),setVirtualTablePosition(rowspanIndex,cellIndex,row,cell,!0,cellHasColspan,!0)}var colspanNumber=cell.attributes.colSpan?parseInt(cell.attributes.colSpan.value,10):0;if(colspanNumber>1)for(var cp=1;cp<colspanNumber;cp++){var cellspanIndex=recoverCellIndex(row.rowIndex,cellIndex+cp);adjustStartPoint(row.rowIndex,cellspanIndex,cell,isThisSelectedCell),setVirtualTablePosition(row.rowIndex,cellspanIndex,row,cell,cellHasRowspan,!0,!0)}}function adjustStartPoint(rowIndex,cellIndex,cell,isSelectedCell){rowIndex===_startPoint.rowPos&&_startPoint.colPos>=cell.cellIndex&&cell.cellIndex<=cellIndex&&!isSelectedCell&&_startPoint.colPos++}function createVirtualTable(){for(var rows=domTable.rows,rowIndex=0;rowIndex<rows.length;rowIndex++)for(var cells=rows[rowIndex].cells,cellIndex=0;cellIndex<cells.length;cellIndex++)addCellInfoToVirtual(rows[rowIndex],cells[cellIndex])}function getDeleteResultActionToCell(cell){switch(where){case TableResultAction.where.Column:if(cell.isColSpan)return TableResultAction.resultAction.SubtractSpanCount;break;case TableResultAction.where.Row:if(!cell.isVirtual&&cell.isRowSpan)return TableResultAction.resultAction.AddCell;if(cell.isRowSpan)return TableResultAction.resultAction.SubtractSpanCount}return TableResultAction.resultAction.RemoveCell}function getAddResultActionToCell(cell){switch(where){case TableResultAction.where.Column:if(cell.isColSpan)return TableResultAction.resultAction.SumSpanCount;if(cell.isRowSpan&&cell.isVirtual)return TableResultAction.resultAction.Ignore;break;case TableResultAction.where.Row:if(cell.isRowSpan)return TableResultAction.resultAction.SumSpanCount;if(cell.isColSpan&&cell.isVirtual)return TableResultAction.resultAction.Ignore}return TableResultAction.resultAction.AddCell}function init(){setStartPoint(),createVirtualTable()}var _startPoint={colPos:0,rowPos:0},_virtualTable=[],_actionCellList=[];this.getActionList=function(){for(var fixedRow=where===TableResultAction.where.Row?_startPoint.rowPos:-1,fixedCol=where===TableResultAction.where.Column?_startPoint.colPos:-1,actualPosition=0,canContinue=!0;canContinue;){var rowPosition=fixedRow>=0?fixedRow:actualPosition,colPosition=fixedCol>=0?fixedCol:actualPosition,row=_virtualTable[rowPosition];if(!row)return canContinue=!1,_actionCellList;var cell=row[colPosition];if(!cell)return canContinue=!1,_actionCellList;var resultAction=TableResultAction.resultAction.Ignore;switch(action){case TableResultAction.requestAction.Add:resultAction=getAddResultActionToCell(cell);break;case TableResultAction.requestAction.Delete:resultAction=getDeleteResultActionToCell(cell)}_actionCellList.push(getActionCell(cell,resultAction,rowPosition,colPosition)),actualPosition++}return _actionCellList},init()};TableResultAction.where={Row:0,Column:1},TableResultAction.requestAction={Add:0,Delete:1},TableResultAction.resultAction={Ignore:0,SubtractSpanCount:1,RemoveCell:2,AddCell:3,SumSpanCount:4};var Table=function(){this.tab=function(rng,isShift){var cell=dom.ancestor(rng.commonAncestor(),dom.isCell),table=dom.ancestor(cell,dom.isTable),cells=dom.listDescendant(table,dom.isCell),nextCell=list[isShift?"prev":"next"](cells,cell);nextCell&&range.create(nextCell,0).select()},this.addRow=function(rng,position){for(var cell=dom.ancestor(rng.commonAncestor(),dom.isCell),currentTr=$(cell).closest("tr"),trAttributes=this.recoverAttributes(currentTr),html=$("<tr"+trAttributes+"></tr>"),vTable=new TableResultAction(cell,TableResultAction.where.Row,TableResultAction.requestAction.Add,$(currentTr).closest("table")[0]),actions=vTable.getActionList(),idCell=0;idCell<actions.length;idCell++){var currentCell=actions[idCell],tdAttributes=this.recoverAttributes(currentCell.baseCell);switch(currentCell.action){case TableResultAction.resultAction.AddCell:html.append("<td"+tdAttributes+">"+dom.blank+"</td>");break;case TableResultAction.resultAction.SumSpanCount:if("top"===position){var baseCellTr=currentCell.baseCell.parent,isTopFromRowSpan=(baseCellTr?currentCell.baseCell.closest("tr").rowIndex:0)<=currentTr[0].rowIndex;if(isTopFromRowSpan){var newTd=$("<div></div>").append($("<td"+tdAttributes+">"+dom.blank+"</td>").removeAttr("rowspan")).html();html.append(newTd);break}}var rowspanNumber=parseInt(currentCell.baseCell.rowSpan,10);rowspanNumber++,currentCell.baseCell.setAttribute("rowSpan",rowspanNumber)}}if("top"===position)currentTr.before(html);else{var cellHasRowspan=cell.rowSpan>1;if(cellHasRowspan){var lastTrIndex=currentTr[0].rowIndex+(cell.rowSpan-2);return void $($(currentTr).parent().find("tr")[lastTrIndex]).after($(html))}currentTr.after(html)}},this.addCol=function(rng,position){var cell=dom.ancestor(rng.commonAncestor(),dom.isCell),row=$(cell).closest("tr"),rowsGroup=$(row).siblings();rowsGroup.push(row);for(var vTable=new TableResultAction(cell,TableResultAction.where.Column,TableResultAction.requestAction.Add,$(row).closest("table")[0]),actions=vTable.getActionList(),actionIndex=0;actionIndex<actions.length;actionIndex++){var currentCell=actions[actionIndex],tdAttributes=this.recoverAttributes(currentCell.baseCell);switch(currentCell.action){case TableResultAction.resultAction.AddCell:"right"===position?$(currentCell.baseCell).after("<td"+tdAttributes+">"+dom.blank+"</td>"):$(currentCell.baseCell).before("<td"+tdAttributes+">"+dom.blank+"</td>");break;case TableResultAction.resultAction.SumSpanCount:if("right"===position){var colspanNumber=parseInt(currentCell.baseCell.colSpan,10);colspanNumber++,currentCell.baseCell.setAttribute("colSpan",colspanNumber)}else $(currentCell.baseCell).before("<td"+tdAttributes+">"+dom.blank+"</td>")}}},this.recoverAttributes=function(el){var resultStr="";if(!el)return resultStr;for(var attrList=el.attributes||[],i=0;i<attrList.length;i++)"id"!==attrList[i].name.toLowerCase()&&attrList[i].specified&&(resultStr+=" "+attrList[i].name+"='"+attrList[i].value+"'");return resultStr},this.deleteRow=function(rng){for(var cell=dom.ancestor(rng.commonAncestor(),dom.isCell),row=$(cell).closest("tr"),cellPos=row.children("td, th").index($(cell)),rowPos=row[0].rowIndex,vTable=new TableResultAction(cell,TableResultAction.where.Row,TableResultAction.requestAction.Delete,$(row).closest("table")[0]),actions=vTable.getActionList(),actionIndex=0;actionIndex<actions.length;actionIndex++)if(actions[actionIndex]){var baseCell=actions[actionIndex].baseCell,virtualPosition=actions[actionIndex].virtualTable,hasRowspan=baseCell.rowSpan&&baseCell.rowSpan>1,rowspanNumber=hasRowspan?parseInt(baseCell.rowSpan,10):0;switch(actions[actionIndex].action){case TableResultAction.resultAction.Ignore:continue;case TableResultAction.resultAction.AddCell:var nextRow=row.next("tr")[0];if(!nextRow)continue;var cloneRow=row[0].cells[cellPos];hasRowspan&&(rowspanNumber>2?(rowspanNumber--,nextRow.insertBefore(cloneRow,nextRow.cells[cellPos]),nextRow.cells[cellPos].setAttribute("rowSpan",rowspanNumber),nextRow.cells[cellPos].innerHTML=""):2===rowspanNumber&&(nextRow.insertBefore(cloneRow,nextRow.cells[cellPos]),nextRow.cells[cellPos].removeAttribute("rowSpan"),nextRow.cells[cellPos].innerHTML=""));continue;case TableResultAction.resultAction.SubtractSpanCount:hasRowspan&&(rowspanNumber>2?(rowspanNumber--,baseCell.setAttribute("rowSpan",rowspanNumber),virtualPosition.rowIndex!==rowPos&&baseCell.cellIndex===cellPos&&(baseCell.innerHTML="")):2===rowspanNumber&&(baseCell.removeAttribute("rowSpan"),virtualPosition.rowIndex!==rowPos&&baseCell.cellIndex===cellPos&&(baseCell.innerHTML="")));continue;case TableResultAction.resultAction.RemoveCell:continue}}row.remove()},this.deleteCol=function(rng){for(var cell=dom.ancestor(rng.commonAncestor(),dom.isCell),row=$(cell).closest("tr"),cellPos=row.children("td, th").index($(cell)),vTable=new TableResultAction(cell,TableResultAction.where.Column,TableResultAction.requestAction.Delete,$(row).closest("table")[0]),actions=vTable.getActionList(),actionIndex=0;actionIndex<actions.length;actionIndex++)if(actions[actionIndex])switch(actions[actionIndex].action){case TableResultAction.resultAction.Ignore:continue;case TableResultAction.resultAction.SubtractSpanCount:var baseCell=actions[actionIndex].baseCell,hasColspan=baseCell.colSpan&&baseCell.colSpan>1;if(hasColspan){var colspanNumber=baseCell.colSpan?parseInt(baseCell.colSpan,10):0;colspanNumber>2?(colspanNumber--,baseCell.setAttribute("colSpan",colspanNumber),baseCell.cellIndex===cellPos&&(baseCell.innerHTML="")):2===colspanNumber&&(baseCell.removeAttribute("colSpan"),baseCell.cellIndex===cellPos&&(baseCell.innerHTML=""))}continue;case TableResultAction.resultAction.RemoveCell:dom.remove(actions[actionIndex].baseCell,!0);continue}},this.createTable=function(colCount,rowCount,options){for(var tdHTML,tds=[],idxCol=0;idxCol<colCount;idxCol++)tds.push("<td>"+dom.blank+"</td>");tdHTML=tds.join("");for(var trHTML,trs=[],idxRow=0;idxRow<rowCount;idxRow++)trs.push("<tr>"+tdHTML+"</tr>");trHTML=trs.join("");var $table=$("<table>"+trHTML+"</table>");return options&&options.tableClassName&&$table.addClass(options.tableClassName),$table[0]},this.deleteTable=function(rng){var cell=dom.ancestor(rng.commonAncestor(),dom.isCell);$(cell).closest("table").remove()}},KEY_BOGUS="bogus",Editor=function(context){var self=this,$note=context.layoutInfo.note,$editor=context.layoutInfo.editor,$editable=context.layoutInfo.editable,options=context.options,lang=options.langInfo,editable=$editable[0],lastRange=null,style=new Style,table=new Table,typing=new Typing,bullet=new Bullet,history=new History($editable);this.initialize=function(){$editable.on("keydown",function(event){event.keyCode===key.code.ENTER&&context.triggerEvent("enter",event),context.triggerEvent("keydown",event),event.isDefaultPrevented()||(options.shortcuts?self.handleKeyMap(event):self.preventDefaultEditableShortCuts(event))}).on("keyup",function(event){context.triggerEvent("keyup",event)}).on("focus",function(event){context.triggerEvent("focus",event)}).on("blur",function(event){context.triggerEvent("blur",event)}).on("mousedown",function(event){context.triggerEvent("mousedown",event)}).on("mouseup",function(event){context.triggerEvent("mouseup",event)}).on("scroll",function(event){context.triggerEvent("scroll",event)}).on("paste",function(event){context.triggerEvent("paste",event)}),$editable.html(dom.html($note)||dom.emptyPara);var changeEventName=agent.isMSIE?"DOMCharacterDataModified DOMSubtreeModified DOMNodeInserted":"input";$editable.on(changeEventName,func.debounce(function(){context.triggerEvent("change",$editable.html())},250)),$editor.on("focusin",function(event){context.triggerEvent("focusin",event)}).on("focusout",function(event){context.triggerEvent("focusout",event)}),options.airMode||(options.width&&$editor.outerWidth(options.width),options.height&&$editable.outerHeight(options.height),options.maxHeight&&$editable.css("max-height",options.maxHeight),options.minHeight&&$editable.css("min-height",options.minHeight)),history.recordUndo()},this.destroy=function(){$editable.off()},this.handleKeyMap=function(event){var keyMap=options.keyMap[agent.isMac?"mac":"pc"],keys=[];event.metaKey&&keys.push("CMD"),event.ctrlKey&&!event.altKey&&keys.push("CTRL"),event.shiftKey&&keys.push("SHIFT");var keyName=key.nameFromCode[event.keyCode];keyName&&keys.push(keyName);var eventName=keyMap[keys.join("+")];eventName?(event.preventDefault(),context.invoke(eventName)):key.isEdit(event.keyCode)&&this.afterCommand()},this.preventDefaultEditableShortCuts=function(event){(event.ctrlKey||event.metaKey)&&list.contains([66,73,85],event.keyCode)&&event.preventDefault()},this.createRange=function(){return this.focus(),range.create(editable)},this.saveRange=function(thenCollapse){lastRange=this.createRange(),thenCollapse&&lastRange.collapse().select()},this.restoreRange=function(){lastRange&&(lastRange.select(),this.focus())},this.saveTarget=function(node){$editable.data("target",node)},this.clearTarget=function(){$editable.removeData("target")},this.restoreTarget=function(){return $editable.data("target")},this.currentStyle=function(){var rng=range.create();return rng&&(rng=rng.normalize()),rng?style.current(rng):style.fromNode($editable)},this.styleFromNode=function($node){return style.fromNode($node)},this.undo=function(){context.triggerEvent("before.command",$editable.html()),history.undo(),context.triggerEvent("change",$editable.html())},context.memo("help.undo",lang.help.undo),this.redo=function(){context.triggerEvent("before.command",$editable.html()),history.redo(),context.triggerEvent("change",$editable.html())},context.memo("help.redo",lang.help.redo);for(var beforeCommand=this.beforeCommand=function(){context.triggerEvent("before.command",$editable.html()),self.focus()},afterCommand=this.afterCommand=function(isPreventTrigger){history.recordUndo(),isPreventTrigger||context.triggerEvent("change",$editable.html())},commands=["bold","italic","underline","strikethrough","superscript","subscript","justifyLeft","justifyCenter","justifyRight","justifyFull","formatBlock","removeFormat","backColor","foreColor","fontName"],idx=0,len=commands.length;idx<len;idx++)this[commands[idx]]=function(sCmd){return function(value){beforeCommand(),document.execCommand(sCmd,!1,value),afterCommand(!0)}}(commands[idx]),context.memo("help."+commands[idx],lang.help[commands[idx]]);this.tab=function(){var rng=this.createRange();rng.isCollapsed()&&rng.isOnCell()?table.tab(rng):(beforeCommand(),typing.insertTab(rng,options.tabSize),afterCommand())},context.memo("help.tab",lang.help.tab),this.untab=function(){var rng=this.createRange();rng.isCollapsed()&&rng.isOnCell()&&table.tab(rng,!0)},context.memo("help.untab",lang.help.untab),this.wrapCommand=function(fn){return function(){beforeCommand(),fn.apply(self,arguments),afterCommand()}},this.insertParagraph=this.wrapCommand(function(){typing.insertParagraph(editable)}),context.memo("help.insertParagraph",lang.help.insertParagraph),this.insertOrderedList=this.wrapCommand(function(){bullet.insertOrderedList(editable)}),context.memo("help.insertOrderedList",lang.help.insertOrderedList),this.insertUnorderedList=this.wrapCommand(function(){bullet.insertUnorderedList(editable)}),context.memo("help.insertUnorderedList",lang.help.insertUnorderedList),this.indent=this.wrapCommand(function(){bullet.indent(editable)}),context.memo("help.indent",lang.help.indent),this.outdent=this.wrapCommand(function(){bullet.outdent(editable)}),context.memo("help.outdent",lang.help.outdent),this.insertImage=function(src,param){return async.createImage(src,param).then(function($image){beforeCommand(),"function"==typeof param?param($image):("string"==typeof param&&$image.attr("data-filename",param),$image.css("width",Math.min($editable.width(),$image.width()))),$image.show(),range.create(editable).insertNode($image[0]),range.createFromNodeAfter($image[0]).select(),afterCommand()}).fail(function(e){context.triggerEvent("image.upload.error",e)})},this.insertImages=function(files){$.each(files,function(idx,file){var filename=file.name;options.maximumImageFileSize&&options.maximumImageFileSize<file.size?context.triggerEvent("image.upload.error",lang.image.maximumFileSizeError):async.readFileAsDataURL(file).then(function(dataURL){return self.insertImage(dataURL,filename)}).fail(function(){context.triggerEvent("image.upload.error")})})},this.insertImagesOrCallback=function(files){var callbacks=options.callbacks;callbacks.onImageUpload?context.triggerEvent("image.upload",files):this.insertImages(files)},this.insertNode=this.wrapCommand(function(node){var rng=this.createRange();rng.insertNode(node),range.createFromNodeAfter(node).select()}),this.insertText=this.wrapCommand(function(text){var rng=this.createRange(),textNode=rng.insertNode(dom.createText(text));range.create(textNode,dom.nodeLength(textNode)).select()}),this.getSelectedText=function(){var rng=this.createRange();return rng.isOnAnchor()&&(rng=range.createFromNode(dom.ancestor(rng.sc,dom.isAnchor))),rng.toString()},this.pasteHTML=this.wrapCommand(function(markup){var contents=this.createRange().pasteHTML(markup);range.createFromNodeAfter(list.last(contents)).select()}),this.formatBlock=this.wrapCommand(function(tagName,$target){var onApplyCustomStyle=context.options.callbacks.onApplyCustomStyle;onApplyCustomStyle?onApplyCustomStyle.call(this,$target,context,this.onFormatBlock):this.onFormatBlock(tagName)}),this.onFormatBlock=function(tagName){tagName=agent.isMSIE?"<"+tagName+">":tagName,document.execCommand("FormatBlock",!1,tagName)},this.formatPara=function(){this.formatBlock("P")},context.memo("help.formatPara",lang.help.formatPara);for(var idx=1;idx<=6;idx++)this["formatH"+idx]=function(idx){return function(){this.formatBlock("H"+idx)}}(idx),context.memo("help.formatH"+idx,lang.help["formatH"+idx]);this.fontSize=function(value){var rng=this.createRange();if(rng&&rng.isCollapsed()){var currentStyle=this.currentStyle(),spans=style.styleNodes(rng),firstSpan=list.head(spans);$(spans).css({"font-size":value+"px"}),firstSpan&&!dom.nodeLength(firstSpan)&&(firstSpan.innerHTML=dom.ZERO_WIDTH_NBSP_CHAR,range.createFromNodeAfter(firstSpan.firstChild).select(),$editable.data(KEY_BOGUS,firstSpan)),this.fontName(currentStyle["font-family"])}else beforeCommand(),$(style.styleNodes(rng)).css({"font-size":value+"px"}),afterCommand()},this.insertHorizontalRule=this.wrapCommand(function(){var hrNode=this.createRange().insertNode(dom.create("HR"));hrNode.nextSibling&&range.create(hrNode.nextSibling,0).normalize().select()}),context.memo("help.insertHorizontalRule",lang.help.insertHorizontalRule),this.removeBogus=function(){var bogusNode=$editable.data(KEY_BOGUS);if(bogusNode){var textNode=list.find(list.from(bogusNode.childNodes),dom.isText),bogusCharIdx=textNode.nodeValue.indexOf(dom.ZERO_WIDTH_NBSP_CHAR);bogusCharIdx!==-1&&textNode.deleteData(bogusCharIdx,1),dom.isEmpty(bogusNode)&&dom.remove(bogusNode),$editable.removeData(KEY_BOGUS)}},this.lineHeight=this.wrapCommand(function(value){style.stylePara(this.createRange(),{lineHeight:value})}),this.unlink=function(){var rng=this.createRange();if(rng.isOnAnchor()){var anchor=dom.ancestor(rng.sc,dom.isAnchor);rng=range.createFromNode(anchor),rng.select(),beforeCommand(),document.execCommand("unlink"),afterCommand()}},this.createLink=this.wrapCommand(function(linkInfo){var linkUrl=linkInfo.url,linkText=linkInfo.text,isNewWindow=linkInfo.isNewWindow,rng=linkInfo.range||this.createRange(),isTextChanged=rng.toString()!==linkText;"string"==typeof linkUrl&&(linkUrl=linkUrl.trim()),linkUrl=options.onCreateLink?options.onCreateLink(linkUrl):/^[A-Za-z][A-Za-z0-9+-.]*\:[\/\/]?/.test(linkUrl)?linkUrl:"http://"+linkUrl;var anchors=[];if(isTextChanged){rng=rng.deleteContents();var anchor=rng.insertNode($("<A>"+linkText+"</A>")[0]);anchors.push(anchor)}else anchors=style.styleNodes(rng,{nodeName:"A",expandClosestSibling:!0,onlyPartialContains:!0});$.each(anchors,function(idx,anchor){$(anchor).attr("href",linkUrl),isNewWindow?$(anchor).attr("target","_blank"):$(anchor).removeAttr("target")});var startRange=range.createFromNodeBefore(list.head(anchors)),startPoint=startRange.getStartPoint(),endRange=range.createFromNodeAfter(list.last(anchors)),endPoint=endRange.getEndPoint();range.create(startPoint.node,startPoint.offset,endPoint.node,endPoint.offset).select()}),this.getLinkInfo=function(){var rng=this.createRange().expand(dom.isAnchor),$anchor=$(list.head(rng.nodes(dom.isAnchor))),linkInfo={range:rng,text:rng.toString(),url:$anchor.length?$anchor.attr("href"):""};return $anchor.length&&(linkInfo.isNewWindow="_blank"===$anchor.attr("target")),linkInfo},this.color=this.wrapCommand(function(colorInfo){var foreColor=colorInfo.foreColor,backColor=colorInfo.backColor;foreColor&&document.execCommand("foreColor",!1,foreColor),backColor&&document.execCommand("backColor",!1,backColor)}),this.insertTable=this.wrapCommand(function(dim){var dimension=dim.split("x"),rng=this.createRange().deleteContents();rng.insertNode(table.createTable(dimension[0],dimension[1],options))}),this.addRow=function(position){var rng=this.createRange($editable);rng.isCollapsed()&&rng.isOnCell()&&(beforeCommand(),table.addRow(rng,position),afterCommand())},this.addCol=function(position){var rng=this.createRange($editable);rng.isCollapsed()&&rng.isOnCell()&&(beforeCommand(),table.addCol(rng,position),afterCommand())},this.deleteRow=function(){var rng=this.createRange($editable);rng.isCollapsed()&&rng.isOnCell()&&(beforeCommand(),table.deleteRow(rng),afterCommand())},this.deleteCol=function(){var rng=this.createRange($editable);rng.isCollapsed()&&rng.isOnCell()&&(beforeCommand(),table.deleteCol(rng),afterCommand())},this.deleteTable=function(){var rng=this.createRange($editable);rng.isCollapsed()&&rng.isOnCell()&&(beforeCommand(),table.deleteTable(rng),afterCommand())},this.floatMe=this.wrapCommand(function(value){var $target=$(this.restoreTarget());$target.toggleClass("note-float-left","left"===value),$target.toggleClass("note-float-right","right"===value),$target.css("float",value)}),this.resize=this.wrapCommand(function(value){var $target=$(this.restoreTarget());$target.css({width:100*value+"%",height:""})}),this.resizeTo=function(pos,$target,bKeepRatio){var imageSize;if(bKeepRatio){var newRatio=pos.y/pos.x,ratio=$target.data("ratio");imageSize={
width:ratio>newRatio?pos.x:pos.y/ratio,height:ratio>newRatio?pos.x*ratio:pos.y}}else imageSize={width:pos.x,height:pos.y};$target.css(imageSize)},this.removeMedia=this.wrapCommand(function(){var $target=$(this.restoreTarget()).detach();context.triggerEvent("media.delete",$target,$editable)}),this.hasFocus=function(){return $editable.is(":focus")},this.focus=function(){this.hasFocus()||$editable.focus()},this.isEmpty=function(){return dom.isEmpty($editable[0])||dom.emptyPara===$editable.html()},this.empty=function(){context.invoke("code",dom.emptyPara)}},Clipboard=function(context){var self=this,$editable=context.layoutInfo.editable;this.events={"summernote.keydown":function(we,e){self.needKeydownHook()&&(e.ctrlKey||e.metaKey)&&e.keyCode===key.code.V&&(context.invoke("editor.saveRange"),self.$paste.focus(),setTimeout(function(){self.pasteByHook()},0))}},this.needKeydownHook=function(){return agent.isMSIE&&agent.browserVersion>10||agent.isFF},this.initialize=function(){this.needKeydownHook()?(this.$paste=$('<div tabindex="-1" />').attr("contenteditable",!0).css({position:"absolute",left:-1e5,opacity:0}),$editable.before(this.$paste),this.$paste.on("paste",function(event){context.triggerEvent("paste",event)})):$editable.on("paste",this.pasteByEvent)},this.destroy=function(){this.needKeydownHook()&&(this.$paste.remove(),this.$paste=null)},this.pasteByHook=function(){var node=this.$paste[0].firstChild,src=node&&node.src;if(dom.isImg(node)&&0===src.indexOf("data:")){for(var decodedData=atob(node.src.split(",")[1]),array=new Uint8Array(decodedData.length),i=0;i<decodedData.length;i++)array[i]=decodedData.charCodeAt(i);var blob=new Blob([array],{type:"image/png"});blob.name="clipboard.png",context.invoke("editor.restoreRange"),context.invoke("editor.focus"),context.invoke("editor.insertImagesOrCallback",[blob])}else{var pasteContent=$("<div />").html(this.$paste.html()).html();context.invoke("editor.restoreRange"),context.invoke("editor.focus"),pasteContent&&context.invoke("editor.pasteHTML",pasteContent)}this.$paste.empty()},this.pasteByEvent=function(event){var clipboardData=event.originalEvent.clipboardData;if(clipboardData&&clipboardData.items&&clipboardData.items.length){var item=list.head(clipboardData.items);"file"===item.kind&&item.type.indexOf("image/")!==-1&&context.invoke("editor.insertImagesOrCallback",[item.getAsFile()]),context.invoke("editor.afterCommand")}}},Dropzone=function(context){var $document=$(document),$editor=context.layoutInfo.editor,$editable=context.layoutInfo.editable,options=context.options,lang=options.langInfo,documentEventHandlers={},$dropzone=$(['<div class="note-dropzone">','  <div class="note-dropzone-message"/>',"</div>"].join("")).prependTo($editor),detachDocumentEvent=function(){Object.keys(documentEventHandlers).forEach(function(key){$document.off(key.substr(2).toLowerCase(),documentEventHandlers[key])}),documentEventHandlers={}};this.initialize=function(){options.disableDragAndDrop?(documentEventHandlers.onDrop=function(e){e.preventDefault()},$document.on("drop",documentEventHandlers.onDrop)):this.attachDragAndDropEvent()},this.attachDragAndDropEvent=function(){var collection=$(),$dropzoneMessage=$dropzone.find(".note-dropzone-message");documentEventHandlers.onDragenter=function(e){var isCodeview=context.invoke("codeview.isActivated"),hasEditorSize=$editor.width()>0&&$editor.height()>0;isCodeview||collection.length||!hasEditorSize||($editor.addClass("dragover"),$dropzone.width($editor.width()),$dropzone.height($editor.height()),$dropzoneMessage.text(lang.image.dragImageHere)),collection=collection.add(e.target)},documentEventHandlers.onDragleave=function(e){collection=collection.not(e.target),collection.length||$editor.removeClass("dragover")},documentEventHandlers.onDrop=function(){collection=$(),$editor.removeClass("dragover")},$document.on("dragenter",documentEventHandlers.onDragenter).on("dragleave",documentEventHandlers.onDragleave).on("drop",documentEventHandlers.onDrop),$dropzone.on("dragenter",function(){$dropzone.addClass("hover"),$dropzoneMessage.text(lang.image.dropImage)}).on("dragleave",function(){$dropzone.removeClass("hover"),$dropzoneMessage.text(lang.image.dragImageHere)}),$dropzone.on("drop",function(event){var dataTransfer=event.originalEvent.dataTransfer;dataTransfer&&dataTransfer.files&&dataTransfer.files.length?(event.preventDefault(),$editable.focus(),context.invoke("editor.insertImagesOrCallback",dataTransfer.files)):$.each(dataTransfer.types,function(idx,type){var content=dataTransfer.getData(type);type.toLowerCase().indexOf("text")>-1?context.invoke("editor.pasteHTML",content):$(content).each(function(){context.invoke("editor.insertNode",this)})})}).on("dragover",!1)},this.destroy=function(){detachDocumentEvent()}},CodeMirror;agent.hasCodeMirror&&(agent.isSupportAmd?require(["codemirror"],function(cm){CodeMirror=cm}):CodeMirror=window.CodeMirror);var Codeview=function(context){var $editor=context.layoutInfo.editor,$editable=context.layoutInfo.editable,$codable=context.layoutInfo.codable,options=context.options;this.sync=function(){var isCodeview=this.isActivated();isCodeview&&agent.hasCodeMirror&&$codable.data("cmEditor").save()},this.isActivated=function(){return $editor.hasClass("codeview")},this.toggle=function(){this.isActivated()?this.deactivate():this.activate(),context.triggerEvent("codeview.toggled")},this.activate=function(){if($codable.val(dom.html($editable,options.prettifyHtml)),$codable.height($editable.height()),context.invoke("toolbar.updateCodeview",!0),$editor.addClass("codeview"),$codable.focus(),agent.hasCodeMirror){var cmEditor=CodeMirror.fromTextArea($codable[0],options.codemirror);if(options.codemirror.tern){var server=new CodeMirror.TernServer(options.codemirror.tern);cmEditor.ternServer=server,cmEditor.on("cursorActivity",function(cm){server.updateArgHints(cm)})}cmEditor.setSize(null,$editable.outerHeight()),$codable.data("cmEditor",cmEditor)}},this.deactivate=function(){if(agent.hasCodeMirror){var cmEditor=$codable.data("cmEditor");$codable.val(cmEditor.getValue()),cmEditor.toTextArea()}var value=dom.value($codable,options.prettifyHtml)||dom.emptyPara,isChange=$editable.html()!==value;$editable.html(value),$editable.height(options.height?$codable.height():"auto"),$editor.removeClass("codeview"),isChange&&context.triggerEvent("change",$editable.html(),$editable),$editable.focus(),context.invoke("toolbar.updateCodeview",!1)},this.destroy=function(){this.isActivated()&&this.deactivate()}},EDITABLE_PADDING=24,Statusbar=function(context){var $document=$(document),$statusbar=context.layoutInfo.statusbar,$editable=context.layoutInfo.editable,options=context.options;this.initialize=function(){return options.airMode||options.disableResizeEditor?void this.destroy():void $statusbar.on("mousedown",function(event){event.preventDefault(),event.stopPropagation();var editableTop=$editable.offset().top-$document.scrollTop(),onMouseMove=function(event){var height=event.clientY-(editableTop+EDITABLE_PADDING);height=options.minheight>0?Math.max(height,options.minheight):height,height=options.maxHeight>0?Math.min(height,options.maxHeight):height,$editable.height(height)};$document.on("mousemove",onMouseMove).one("mouseup",function(){$document.off("mousemove",onMouseMove)})})},this.destroy=function(){$statusbar.off(),$statusbar.remove()}},Fullscreen=function(context){var self=this,$editor=context.layoutInfo.editor,$toolbar=context.layoutInfo.toolbar,$editable=context.layoutInfo.editable,$codable=context.layoutInfo.codable,$window=$(window),$scrollbar=$("html, body");this.resizeTo=function(size){$editable.css("height",size.h),$codable.css("height",size.h),$codable.data("cmeditor")&&$codable.data("cmeditor").setsize(null,size.h)},this.onResize=function(){self.resizeTo({h:$window.height()-$toolbar.outerHeight()})},this.toggle=function(){$editor.toggleClass("fullscreen"),this.isFullscreen()?($editable.data("orgHeight",$editable.css("height")),$window.on("resize",this.onResize).trigger("resize"),$scrollbar.css("overflow","hidden")):($window.off("resize",this.onResize),this.resizeTo({h:$editable.data("orgHeight")}),$scrollbar.css("overflow","visible")),context.invoke("toolbar.updateFullscreen",this.isFullscreen())},this.isFullscreen=function(){return $editor.hasClass("fullscreen")}},Handle=function(context){var self=this,$document=$(document),$editingArea=context.layoutInfo.editingArea,options=context.options;this.events={"summernote.mousedown":function(we,e){self.update(e.target)&&e.preventDefault()},"summernote.keyup summernote.scroll summernote.change summernote.dialog.shown":function(){self.update()},"summernote.disable":function(){self.hide()},"summernote.codeview.toggled":function(){self.update()}},this.initialize=function(){this.$handle=$(['<div class="note-handle">','<div class="note-control-selection">','<div class="note-control-selection-bg"></div>','<div class="note-control-holder note-control-nw"></div>','<div class="note-control-holder note-control-ne"></div>','<div class="note-control-holder note-control-sw"></div>','<div class="',options.disableResizeImage?"note-control-holder":"note-control-sizing",' note-control-se"></div>',options.disableResizeImage?"":'<div class="note-control-selection-info"></div>',"</div>","</div>"].join("")).prependTo($editingArea),this.$handle.on("mousedown",function(event){if(dom.isControlSizing(event.target)){event.preventDefault(),event.stopPropagation();var $target=self.$handle.find(".note-control-selection").data("target"),posStart=$target.offset(),scrollTop=$document.scrollTop(),onMouseMove=function(event){context.invoke("editor.resizeTo",{x:event.clientX-posStart.left,y:event.clientY-(posStart.top-scrollTop)},$target,!event.shiftKey),self.update($target[0])};$document.on("mousemove",onMouseMove).one("mouseup",function(e){e.preventDefault(),$document.off("mousemove",onMouseMove),context.invoke("editor.afterCommand")}),$target.data("ratio")||$target.data("ratio",$target.height()/$target.width())}}),this.$handle.on("wheel",function(e){e.preventDefault(),self.update()})},this.destroy=function(){this.$handle.remove()},this.update=function(target){if(context.isDisabled())return!1;var isImage=dom.isImg(target),$selection=this.$handle.find(".note-control-selection");if(context.invoke("imagePopover.update",target),isImage){var $image=$(target),position=$image.position(),pos={left:position.left+parseInt($image.css("marginLeft"),10),top:position.top+parseInt($image.css("marginTop"),10)},imageSize={w:$image.outerWidth(!1),h:$image.outerHeight(!1)};$selection.css({display:"block",left:pos.left,top:pos.top,width:imageSize.w,height:imageSize.h}).data("target",$image);var sizingText=imageSize.w+"x"+imageSize.h;$selection.find(".note-control-selection-info").text(sizingText),context.invoke("editor.saveTarget",target)}else this.hide();return isImage},this.hide=function(){context.invoke("editor.clearTarget"),this.$handle.children().hide()}},AutoLink=function(context){var self=this,defaultScheme="http://",linkPattern=/^([A-Za-z][A-Za-z0-9+-.]*\:[\/\/]?|mailto:[A-Z0-9._%+-]+@)?(www\.)?(.+)$/i;this.events={"summernote.keyup":function(we,e){e.isDefaultPrevented()||self.handleKeyup(e)},"summernote.keydown":function(we,e){self.handleKeydown(e)}},this.initialize=function(){this.lastWordRange=null},this.destroy=function(){this.lastWordRange=null},this.replace=function(){if(this.lastWordRange){var keyword=this.lastWordRange.toString(),match=keyword.match(linkPattern);if(match&&(match[1]||match[2])){var link=match[1]?keyword:defaultScheme+keyword,node=$("<a />").html(keyword).attr("href",link)[0];this.lastWordRange.insertNode(node),this.lastWordRange=null,context.invoke("editor.focus")}}},this.handleKeydown=function(e){if(list.contains([key.code.ENTER,key.code.SPACE],e.keyCode)){var wordRange=context.invoke("editor.createRange").getWordRange();this.lastWordRange=wordRange}},this.handleKeyup=function(e){list.contains([key.code.ENTER,key.code.SPACE],e.keyCode)&&this.replace()}},AutoSync=function(context){var $note=context.layoutInfo.note;this.events={"summernote.change":function(){$note.val(context.invoke("code"))}},this.shouldInitialize=function(){return dom.isTextarea($note[0])}},Placeholder=function(context){var self=this,$editingArea=context.layoutInfo.editingArea,options=context.options;this.events={"summernote.init summernote.change":function(){self.update()},"summernote.codeview.toggled":function(){self.update()}},this.shouldInitialize=function(){return!!options.placeholder},this.initialize=function(){this.$placeholder=$('<div class="note-placeholder">'),this.$placeholder.on("click",function(){context.invoke("focus")}).text(options.placeholder).prependTo($editingArea)},this.destroy=function(){this.$placeholder.remove()},this.update=function(){var isShow=!context.invoke("codeview.isActivated")&&context.invoke("editor.isEmpty");this.$placeholder.toggle(isShow)}},Buttons=function(context){var self=this,ui=$.summernote.ui,$toolbar=context.layoutInfo.toolbar,options=context.options,lang=options.langInfo,invertedKeyMap=func.invertObject(options.keyMap[agent.isMac?"mac":"pc"]),representShortcut=this.representShortcut=function(editorMethod){var shortcut=invertedKeyMap[editorMethod];return options.shortcuts&&shortcut?(agent.isMac&&(shortcut=shortcut.replace("CMD","⌘").replace("SHIFT","⇧")),shortcut=shortcut.replace("BACKSLASH","\\").replace("SLASH","/").replace("LEFTBRACKET","[").replace("RIGHTBRACKET","]")," ("+shortcut+")"):""};this.initialize=function(){this.addToolbarButtons(),this.addImagePopoverButtons(),this.addLinkPopoverButtons(),this.addTablePopoverButtons(),this.fontInstalledMap={}},this.destroy=function(){delete this.fontInstalledMap},this.isFontInstalled=function(name){return self.fontInstalledMap.hasOwnProperty(name)||(self.fontInstalledMap[name]=agent.isFontInstalled(name)||list.contains(options.fontNamesIgnoreCheck,name)),self.fontInstalledMap[name]},this.addToolbarButtons=function(){context.memo("button.style",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:ui.icon(options.icons.magic)+" "+ui.icon(options.icons.caret,"span"),tooltip:lang.style.style,data:{toggle:"dropdown"}}),ui.dropdown({className:"dropdown-style",items:context.options.styleTags,template:function(item){"string"==typeof item&&(item={tag:item,title:lang.style.hasOwnProperty(item)?lang.style[item]:item});var tag=item.tag,title=item.title,style=item.style?' style="'+item.style+'" ':"",className=item.className?' class="'+item.className+'"':"";return"<"+tag+style+className+">"+title+"</"+tag+">"},click:context.createInvokeHandler("editor.formatBlock")})]).render()}),context.memo("button.bold",function(){return ui.button({className:"note-btn-bold",contents:ui.icon(options.icons.bold),tooltip:lang.font.bold+representShortcut("bold"),click:context.createInvokeHandlerAndUpdateState("editor.bold")}).render()}),context.memo("button.italic",function(){return ui.button({className:"note-btn-italic",contents:ui.icon(options.icons.italic),tooltip:lang.font.italic+representShortcut("italic"),click:context.createInvokeHandlerAndUpdateState("editor.italic")}).render()}),context.memo("button.underline",function(){return ui.button({className:"note-btn-underline",contents:ui.icon(options.icons.underline),tooltip:lang.font.underline+representShortcut("underline"),click:context.createInvokeHandlerAndUpdateState("editor.underline")}).render()}),context.memo("button.clear",function(){return ui.button({contents:ui.icon(options.icons.eraser),tooltip:lang.font.clear+representShortcut("removeFormat"),click:context.createInvokeHandler("editor.removeFormat")}).render()}),context.memo("button.strikethrough",function(){return ui.button({className:"note-btn-strikethrough",contents:ui.icon(options.icons.strikethrough),tooltip:lang.font.strikethrough+representShortcut("strikethrough"),click:context.createInvokeHandlerAndUpdateState("editor.strikethrough")}).render()}),context.memo("button.superscript",function(){return ui.button({className:"note-btn-superscript",contents:ui.icon(options.icons.superscript),tooltip:lang.font.superscript,click:context.createInvokeHandlerAndUpdateState("editor.superscript")}).render()}),context.memo("button.subscript",function(){return ui.button({className:"note-btn-subscript",contents:ui.icon(options.icons.subscript),tooltip:lang.font.subscript,click:context.createInvokeHandlerAndUpdateState("editor.subscript")}).render()}),context.memo("button.fontname",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:'<span class="note-current-fontname"/> '+ui.icon(options.icons.caret,"span"),tooltip:lang.font.name,data:{toggle:"dropdown"}}),ui.dropdownCheck({className:"dropdown-fontname",checkClassName:options.icons.menuCheck,items:options.fontNames.filter(self.isFontInstalled),template:function(item){return'<span style="font-family:'+item+'">'+item+"</span>"},click:context.createInvokeHandlerAndUpdateState("editor.fontName")})]).render()}),context.memo("button.fontsize",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:'<span class="note-current-fontsize"/>'+ui.icon(options.icons.caret,"span"),tooltip:lang.font.size,data:{toggle:"dropdown"}}),ui.dropdownCheck({className:"dropdown-fontsize",checkClassName:options.icons.menuCheck,items:options.fontSizes,click:context.createInvokeHandlerAndUpdateState("editor.fontSize")})]).render()}),context.memo("button.color",function(){return ui.buttonGroup({className:"note-color",children:[ui.button({className:"note-current-color-button",contents:ui.icon(options.icons.font+" note-recent-color"),tooltip:lang.color.recent,click:function(e){var $button=$(e.currentTarget);context.invoke("editor.color",{backColor:$button.attr("data-backColor"),foreColor:$button.attr("data-foreColor")})},callback:function($button){var $recentColor=$button.find(".note-recent-color");$recentColor.css("background-color","#FFFF00"),$button.attr("data-backColor","#FFFF00")}}),ui.button({className:"dropdown-toggle",contents:ui.icon(options.icons.caret,"span"),tooltip:lang.color.more,data:{toggle:"dropdown"}}),ui.dropdown({items:["<li>",'<div class="btn-group">','  <div class="note-palette-title">'+lang.color.background+"</div>","  <div>",'    <button type="button" class="note-color-reset btn btn-default" data-event="backColor" data-value="inherit">',lang.color.transparent,"    </button>","  </div>",'  <div class="note-holder" data-event="backColor"/>',"</div>",'<div class="btn-group">','  <div class="note-palette-title">'+lang.color.foreground+"</div>","  <div>",'    <button type="button" class="note-color-reset btn btn-default" data-event="removeFormat" data-value="foreColor">',lang.color.resetToDefault,"    </button>","  </div>",'  <div class="note-holder" data-event="foreColor"/>',"</div>","</li>"].join(""),callback:function($dropdown){$dropdown.find(".note-holder").each(function(){var $holder=$(this);$holder.append(ui.palette({colors:options.colors,eventName:$holder.data("event"),tooltip:options.tooltip}).render())})},click:function(event){var $button=$(event.target),eventName=$button.data("event"),value=$button.data("value");if(eventName&&value){var key="backColor"===eventName?"background-color":"color",$color=$button.closest(".note-color").find(".note-recent-color"),$currentButton=$button.closest(".note-color").find(".note-current-color-button");$color.css(key,value),$currentButton.attr("data-"+eventName,value),context.invoke("editor."+eventName,value)}}})]}).render()}),context.memo("button.ul",function(){return ui.button({contents:ui.icon(options.icons.unorderedlist),tooltip:lang.lists.unordered+representShortcut("insertUnorderedList"),click:context.createInvokeHandler("editor.insertUnorderedList")}).render()}),context.memo("button.ol",function(){return ui.button({contents:ui.icon(options.icons.orderedlist),tooltip:lang.lists.ordered+representShortcut("insertOrderedList"),click:context.createInvokeHandler("editor.insertOrderedList")}).render()});var justifyLeft=ui.button({contents:ui.icon(options.icons.alignLeft),tooltip:lang.paragraph.left+representShortcut("justifyLeft"),click:context.createInvokeHandler("editor.justifyLeft")}),justifyCenter=ui.button({contents:ui.icon(options.icons.alignCenter),tooltip:lang.paragraph.center+representShortcut("justifyCenter"),click:context.createInvokeHandler("editor.justifyCenter")}),justifyRight=ui.button({contents:ui.icon(options.icons.alignRight),tooltip:lang.paragraph.right+representShortcut("justifyRight"),click:context.createInvokeHandler("editor.justifyRight")}),justifyFull=ui.button({contents:ui.icon(options.icons.alignJustify),tooltip:lang.paragraph.justify+representShortcut("justifyFull"),click:context.createInvokeHandler("editor.justifyFull")}),outdent=ui.button({contents:ui.icon(options.icons.outdent),tooltip:lang.paragraph.outdent+representShortcut("outdent"),click:context.createInvokeHandler("editor.outdent")}),indent=ui.button({contents:ui.icon(options.icons.indent),tooltip:lang.paragraph.indent+representShortcut("indent"),click:context.createInvokeHandler("editor.indent")});context.memo("button.justifyLeft",func.invoke(justifyLeft,"render")),context.memo("button.justifyCenter",func.invoke(justifyCenter,"render")),context.memo("button.justifyRight",func.invoke(justifyRight,"render")),context.memo("button.justifyFull",func.invoke(justifyFull,"render")),context.memo("button.outdent",func.invoke(outdent,"render")),context.memo("button.indent",func.invoke(indent,"render")),context.memo("button.paragraph",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:ui.icon(options.icons.alignLeft)+" "+ui.icon(options.icons.caret,"span"),tooltip:lang.paragraph.paragraph,data:{toggle:"dropdown"}}),ui.dropdown([ui.buttonGroup({className:"note-align",children:[justifyLeft,justifyCenter,justifyRight,justifyFull]}),ui.buttonGroup({className:"note-list",children:[outdent,indent]})])]).render()}),context.memo("button.height",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:ui.icon(options.icons.textHeight)+" "+ui.icon(options.icons.caret,"span"),tooltip:lang.font.height,data:{toggle:"dropdown"}}),ui.dropdownCheck({items:options.lineHeights,checkClassName:options.icons.menuCheck,className:"dropdown-line-height",click:context.createInvokeHandler("editor.lineHeight")})]).render()}),context.memo("button.table",function(){return ui.buttonGroup([ui.button({className:"dropdown-toggle",contents:ui.icon(options.icons.table)+" "+ui.icon(options.icons.caret,"span"),tooltip:lang.table.table,data:{toggle:"dropdown"}}),ui.dropdown({className:"note-table",items:['<div class="note-dimension-picker">','  <div class="note-dimension-picker-mousecatcher" data-event="insertTable" data-value="1x1"/>','  <div class="note-dimension-picker-highlighted"/>','  <div class="note-dimension-picker-unhighlighted"/>',"</div>",'<div class="note-dimension-display">1 x 1</div>'].join("")})],{callback:function($node){var $catcher=$node.find(".note-dimension-picker-mousecatcher");$catcher.css({width:options.insertTableMaxSize.col+"em",height:options.insertTableMaxSize.row+"em"}).mousedown(context.createInvokeHandler("editor.insertTable")).on("mousemove",self.tableMoveHandler)}}).render()}),context.memo("button.link",function(){return ui.button({contents:ui.icon(options.icons.link),tooltip:lang.link.link+representShortcut("linkDialog.show"),click:context.createInvokeHandler("linkDialog.show")}).render()}),context.memo("button.picture",function(){return ui.button({contents:ui.icon(options.icons.picture),tooltip:lang.image.image,click:context.createInvokeHandler("imageDialog.show")}).render()}),context.memo("button.video",function(){return ui.button({contents:ui.icon(options.icons.video),tooltip:lang.video.video,click:context.createInvokeHandler("videoDialog.show")}).render()}),context.memo("button.hr",function(){return ui.button({contents:ui.icon(options.icons.minus),tooltip:lang.hr.insert+representShortcut("insertHorizontalRule"),click:context.createInvokeHandler("editor.insertHorizontalRule")}).render()}),context.memo("button.fullscreen",function(){return ui.button({className:"btn-fullscreen",contents:ui.icon(options.icons.arrowsAlt),tooltip:lang.options.fullscreen,click:context.createInvokeHandler("fullscreen.toggle")}).render()}),context.memo("button.codeview",function(){return ui.button({className:"btn-codeview",contents:ui.icon(options.icons.code),tooltip:lang.options.codeview,click:context.createInvokeHandler("codeview.toggle")}).render()}),context.memo("button.redo",function(){return ui.button({contents:ui.icon(options.icons.redo),tooltip:lang.history.redo+representShortcut("redo"),click:context.createInvokeHandler("editor.redo")}).render()}),context.memo("button.undo",function(){return ui.button({contents:ui.icon(options.icons.undo),tooltip:lang.history.undo+representShortcut("undo"),click:context.createInvokeHandler("editor.undo")}).render()}),context.memo("button.help",function(){return ui.button({contents:ui.icon(options.icons.question),tooltip:lang.options.help,click:context.createInvokeHandler("helpDialog.show")}).render()})},this.addImagePopoverButtons=function(){context.memo("button.imageSize100",function(){return ui.button({contents:'<span class="note-fontsize-10">100%</span>',tooltip:lang.image.resizeFull,click:context.createInvokeHandler("editor.resize","1")}).render()}),context.memo("button.imageSize50",function(){return ui.button({contents:'<span class="note-fontsize-10">50%</span>',tooltip:lang.image.resizeHalf,click:context.createInvokeHandler("editor.resize","0.5")}).render()}),context.memo("button.imageSize25",function(){return ui.button({contents:'<span class="note-fontsize-10">25%</span>',tooltip:lang.image.resizeQuarter,click:context.createInvokeHandler("editor.resize","0.25")}).render()}),context.memo("button.floatLeft",function(){return ui.button({contents:ui.icon(options.icons.alignLeft),tooltip:lang.image.floatLeft,click:context.createInvokeHandler("editor.floatMe","left")}).render()}),context.memo("button.floatRight",function(){return ui.button({contents:ui.icon(options.icons.alignRight),tooltip:lang.image.floatRight,click:context.createInvokeHandler("editor.floatMe","right")}).render()}),context.memo("button.floatNone",function(){return ui.button({contents:ui.icon(options.icons.alignJustify),tooltip:lang.image.floatNone,click:context.createInvokeHandler("editor.floatMe","none")}).render()}),context.memo("button.removeMedia",function(){return ui.button({contents:ui.icon(options.icons.trash),tooltip:lang.image.remove,click:context.createInvokeHandler("editor.removeMedia")}).render()})},this.addLinkPopoverButtons=function(){context.memo("button.linkDialogShow",function(){return ui.button({contents:ui.icon(options.icons.link),tooltip:lang.link.edit,click:context.createInvokeHandler("linkDialog.show")}).render()}),context.memo("button.unlink",function(){return ui.button({contents:ui.icon(options.icons.unlink),tooltip:lang.link.unlink,click:context.createInvokeHandler("editor.unlink")}).render()})},this.addTablePopoverButtons=function(){context.memo("button.addRowUp",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.rowAbove),tooltip:lang.table.addRowAbove,click:context.createInvokeHandler("editor.addRow","top")}).render()}),context.memo("button.addRowDown",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.rowBelow),tooltip:lang.table.addRowBelow,click:context.createInvokeHandler("editor.addRow","bottom")}).render()}),context.memo("button.addColLeft",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.colBefore),tooltip:lang.table.addColLeft,click:context.createInvokeHandler("editor.addCol","left")}).render()}),context.memo("button.addColRight",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.colAfter),tooltip:lang.table.addColRight,click:context.createInvokeHandler("editor.addCol","right")}).render()}),context.memo("button.deleteRow",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.rowRemove),tooltip:lang.table.delRow,click:context.createInvokeHandler("editor.deleteRow")}).render()}),context.memo("button.deleteCol",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.colRemove),tooltip:lang.table.delCol,click:context.createInvokeHandler("editor.deleteCol")}).render()}),context.memo("button.deleteTable",function(){return ui.button({className:"btn-md",contents:ui.icon(options.icons.trash),tooltip:lang.table.delTable,click:context.createInvokeHandler("editor.deleteTable")}).render()})},this.build=function($container,groups){for(var groupIdx=0,groupLen=groups.length;groupIdx<groupLen;groupIdx++){for(var group=groups[groupIdx],groupName=group[0],buttons=group[1],$group=ui.buttonGroup({className:"note-"+groupName}).render(),idx=0,len=buttons.length;idx<len;idx++){var button=context.memo("button."+buttons[idx]);button&&$group.append("function"==typeof button?button(context):button)}$group.appendTo($container)}},this.updateCurrentStyle=function(){var styleInfo=context.invoke("editor.currentStyle");if(this.updateBtnStates({".note-btn-bold":function(){return"bold"===styleInfo["font-bold"]},".note-btn-italic":function(){return"italic"===styleInfo["font-italic"]},".note-btn-underline":function(){return"underline"===styleInfo["font-underline"]},".note-btn-subscript":function(){return"subscript"===styleInfo["font-subscript"]},".note-btn-superscript":function(){return"superscript"===styleInfo["font-superscript"]},".note-btn-strikethrough":function(){return"strikethrough"===styleInfo["font-strikethrough"]}}),styleInfo["font-family"]){var fontNames=styleInfo["font-family"].split(",").map(function(name){return name.replace(/[\'\"]/g,"").replace(/\s+$/,"").replace(/^\s+/,"")}),fontName=list.find(fontNames,self.isFontInstalled);$toolbar.find(".dropdown-fontname li a").each(function(){var isChecked=$(this).data("value")+""==fontName+"";this.className=isChecked?"checked":""}),$toolbar.find(".note-current-fontname").text(fontName)}if(styleInfo["font-size"]){var fontSize=styleInfo["font-size"];$toolbar.find(".dropdown-fontsize li a").each(function(){var isChecked=$(this).data("value")+""==fontSize+"";this.className=isChecked?"checked":""}),$toolbar.find(".note-current-fontsize").text(fontSize)}if(styleInfo["line-height"]){var lineHeight=styleInfo["line-height"];$toolbar.find(".dropdown-line-height li a").each(function(){var isChecked=$(this).data("value")+""==lineHeight+"";this.className=isChecked?"checked":""})}},this.updateBtnStates=function(infos){$.each(infos,function(selector,pred){ui.toggleBtnActive($toolbar.find(selector),pred())})},this.tableMoveHandler=function(event){var posOffset,PX_PER_EM=18,$picker=$(event.target.parentNode),$dimensionDisplay=$picker.next(),$catcher=$picker.find(".note-dimension-picker-mousecatcher"),$highlighted=$picker.find(".note-dimension-picker-highlighted"),$unhighlighted=$picker.find(".note-dimension-picker-unhighlighted");if(void 0===event.offsetX){var posCatcher=$(event.target).offset();posOffset={x:event.pageX-posCatcher.left,y:event.pageY-posCatcher.top}}else posOffset={x:event.offsetX,y:event.offsetY};var dim={c:Math.ceil(posOffset.x/PX_PER_EM)||1,r:Math.ceil(posOffset.y/PX_PER_EM)||1};$highlighted.css({width:dim.c+"em",height:dim.r+"em"}),$catcher.data("value",dim.c+"x"+dim.r),3<dim.c&&dim.c<options.insertTableMaxSize.col&&$unhighlighted.css({width:dim.c+1+"em"}),3<dim.r&&dim.r<options.insertTableMaxSize.row&&$unhighlighted.css({height:dim.r+1+"em"}),$dimensionDisplay.html(dim.c+" x "+dim.r)}},Toolbar=function(context){var ui=$.summernote.ui,$note=context.layoutInfo.note,$toolbar=context.layoutInfo.toolbar,options=context.options;this.shouldInitialize=function(){return!options.airMode},this.initialize=function(){options.toolbar=options.toolbar||[],options.toolbar.length?context.invoke("buttons.build",$toolbar,options.toolbar):$toolbar.hide(),options.toolbarContainer&&$toolbar.appendTo(options.toolbarContainer),
$note.on("summernote.keyup summernote.mouseup summernote.change",function(){context.invoke("buttons.updateCurrentStyle")}),context.invoke("buttons.updateCurrentStyle")},this.destroy=function(){$toolbar.children().remove()},this.updateFullscreen=function(isFullscreen){ui.toggleBtnActive($toolbar.find(".btn-fullscreen"),isFullscreen)},this.updateCodeview=function(isCodeview){ui.toggleBtnActive($toolbar.find(".btn-codeview"),isCodeview),isCodeview?this.deactivate():this.activate()},this.activate=function(isIncludeCodeview){var $btn=$toolbar.find("button");isIncludeCodeview||($btn=$btn.not(".btn-codeview")),ui.toggleBtn($btn,!0)},this.deactivate=function(isIncludeCodeview){var $btn=$toolbar.find("button");isIncludeCodeview||($btn=$btn.not(".btn-codeview")),ui.toggleBtn($btn,!1)}},LinkDialog=function(context){var self=this,ui=$.summernote.ui,$editor=context.layoutInfo.editor,options=context.options,lang=options.langInfo;this.initialize=function(){var $container=options.dialogsInBody?$(document.body):$editor,body='<div class="form-group"><label>'+lang.link.textToDisplay+'</label><input class="note-link-text form-control" type="text" /></div><div class="form-group"><label>'+lang.link.url+'</label><input class="note-link-url form-control" type="text" value="http://" /></div>'+(options.disableLinkTarget?"":'<div class="checkbox"><label for="sn-checkbox-open-in-new-window"><input type="checkbox" id="sn-checkbox-open-in-new-window" checked />'+lang.link.openInNewWindow+"</label></div>"),footer='<button href="#" class="btn btn-primary note-link-btn disabled" disabled>'+lang.link.insert+"</button>";this.$dialog=ui.dialog({className:"link-dialog",title:lang.link.insert,fade:options.dialogsFade,body:body,footer:footer}).render().appendTo($container)},this.destroy=function(){ui.hideDialog(this.$dialog),this.$dialog.remove()},this.bindEnterKey=function($input,$btn){$input.on("keypress",function(event){event.keyCode===key.code.ENTER&&$btn.trigger("click")})},this.toggleLinkBtn=function($linkBtn,$linkText,$linkUrl){ui.toggleBtn($linkBtn,$linkText.val()&&$linkUrl.val())},this.showLinkDialog=function(linkInfo){return $.Deferred(function(deferred){var $linkText=self.$dialog.find(".note-link-text"),$linkUrl=self.$dialog.find(".note-link-url"),$linkBtn=self.$dialog.find(".note-link-btn"),$openInNewWindow=self.$dialog.find("input[type=checkbox]");ui.onDialogShown(self.$dialog,function(){context.triggerEvent("dialog.shown"),linkInfo.url||(linkInfo.url=linkInfo.text),$linkText.val(linkInfo.text);var handleLinkTextUpdate=function(){self.toggleLinkBtn($linkBtn,$linkText,$linkUrl),linkInfo.text=$linkText.val()};$linkText.on("input",handleLinkTextUpdate).on("paste",function(){setTimeout(handleLinkTextUpdate,0)});var handleLinkUrlUpdate=function(){self.toggleLinkBtn($linkBtn,$linkText,$linkUrl),linkInfo.text||$linkText.val($linkUrl.val())};$linkUrl.on("input",handleLinkUrlUpdate).on("paste",function(){setTimeout(handleLinkUrlUpdate,0)}).val(linkInfo.url).trigger("focus"),self.toggleLinkBtn($linkBtn,$linkText,$linkUrl),self.bindEnterKey($linkUrl,$linkBtn),self.bindEnterKey($linkText,$linkBtn);var isChecked=void 0!==linkInfo.isNewWindow?linkInfo.isNewWindow:context.options.linkTargetBlank;$openInNewWindow.prop("checked",isChecked),$linkBtn.one("click",function(event){event.preventDefault(),deferred.resolve({range:linkInfo.range,url:$linkUrl.val(),text:$linkText.val(),isNewWindow:$openInNewWindow.is(":checked")}),self.$dialog.modal("hide")})}),ui.onDialogHidden(self.$dialog,function(){$linkText.off("input paste keypress"),$linkUrl.off("input paste keypress"),$linkBtn.off("click"),"pending"===deferred.state()&&deferred.reject()}),ui.showDialog(self.$dialog)}).promise()},this.show=function(){var linkInfo=context.invoke("editor.getLinkInfo");context.invoke("editor.saveRange"),this.showLinkDialog(linkInfo).then(function(linkInfo){context.invoke("editor.restoreRange"),context.invoke("editor.createLink",linkInfo)}).fail(function(){context.invoke("editor.restoreRange")})},context.memo("help.linkDialog.show",options.langInfo.help["linkDialog.show"])},LinkPopover=function(context){var self=this,ui=$.summernote.ui,options=context.options;this.events={"summernote.keyup summernote.mouseup summernote.change summernote.scroll":function(){self.update()},"summernote.disable summernote.dialog.shown":function(){self.hide()}},this.shouldInitialize=function(){return!list.isEmpty(options.popover.link)},this.initialize=function(){this.$popover=ui.popover({className:"note-link-popover",callback:function($node){var $content=$node.find(".popover-content");$content.prepend('<span><a target="_blank"></a>&nbsp;</span>')}}).render().appendTo("body");var $content=this.$popover.find(".popover-content");context.invoke("buttons.build",$content,options.popover.link)},this.destroy=function(){this.$popover.remove()},this.update=function(){if(!context.invoke("editor.hasFocus"))return void this.hide();var rng=context.invoke("editor.createRange");if(rng.isCollapsed()&&rng.isOnAnchor()){var anchor=dom.ancestor(rng.sc,dom.isAnchor),href=$(anchor).attr("href");this.$popover.find("a").attr("href",href).html(href);var pos=dom.posFromPlaceholder(anchor);this.$popover.css({display:"block",left:pos.left,top:pos.top})}else this.hide()},this.hide=function(){this.$popover.hide()}},ImageDialog=function(context){var self=this,ui=$.summernote.ui,$editor=context.layoutInfo.editor,options=context.options,lang=options.langInfo;this.initialize=function(){var $container=options.dialogsInBody?$(document.body):$editor,imageLimitation="";if(options.maximumImageFileSize){var unit=Math.floor(Math.log(options.maximumImageFileSize)/Math.log(1024)),readableSize=1*(options.maximumImageFileSize/Math.pow(1024,unit)).toFixed(2)+" "+" KMGTP"[unit]+"B";imageLimitation="<small>"+lang.image.maximumFileSize+" : "+readableSize+"</small>"}var body='<div class="form-group note-group-select-from-files"><label>'+lang.image.selectFromFiles+'</label><input class="note-image-input form-control" type="file" name="files" accept="image/*" multiple="multiple" />'+imageLimitation+'</div><div class="form-group note-group-image-url" style="overflow:auto;"><label>'+lang.image.url+'</label><input class="note-image-url form-control col-md-12" type="text" /></div>',footer='<button href="#" class="btn btn-primary note-image-btn disabled" disabled>'+lang.image.insert+"</button>";this.$dialog=ui.dialog({title:lang.image.insert,fade:options.dialogsFade,body:body,footer:footer}).render().appendTo($container)},this.destroy=function(){ui.hideDialog(this.$dialog),this.$dialog.remove()},this.bindEnterKey=function($input,$btn){$input.on("keypress",function(event){event.keyCode===key.code.ENTER&&$btn.trigger("click")})},this.show=function(){context.invoke("editor.saveRange"),this.showImageDialog().then(function(data){ui.hideDialog(self.$dialog),context.invoke("editor.restoreRange"),"string"==typeof data?context.invoke("editor.insertImage",data):context.invoke("editor.insertImagesOrCallback",data)}).fail(function(){context.invoke("editor.restoreRange")})},this.showImageDialog=function(){return $.Deferred(function(deferred){var $imageInput=self.$dialog.find(".note-image-input"),$imageUrl=self.$dialog.find(".note-image-url"),$imageBtn=self.$dialog.find(".note-image-btn");ui.onDialogShown(self.$dialog,function(){context.triggerEvent("dialog.shown"),$imageInput.replaceWith($imageInput.clone().on("change",function(){deferred.resolve(this.files||this.value)}).val("")),$imageBtn.click(function(event){event.preventDefault(),deferred.resolve($imageUrl.val())}),$imageUrl.on("keyup paste",function(){var url=$imageUrl.val();ui.toggleBtn($imageBtn,url)}).val("").trigger("focus"),self.bindEnterKey($imageUrl,$imageBtn)}),ui.onDialogHidden(self.$dialog,function(){$imageInput.off("change"),$imageUrl.off("keyup paste keypress"),$imageBtn.off("click"),"pending"===deferred.state()&&deferred.reject()}),ui.showDialog(self.$dialog)})}},ImagePopover=function(context){var self=this,ui=$.summernote.ui,$editable=context.layoutInfo.editable,editable=$editable[0],options=context.options;this.events={"summernote.disable":function(){self.hide()}},this.shouldInitialize=function(){return!list.isEmpty(options.popover.image)},this.initialize=function(){this.$popover=ui.popover({className:"note-image-popover"}).render().appendTo("body");var $content=this.$popover.find(".popover-content");context.invoke("buttons.build",$content,options.popover.image)},this.destroy=function(){this.$popover.remove()},this.update=function(target){if(dom.isImg(target)){var pos=dom.posFromPlaceholder(target),posEditor=dom.posFromPlaceholder(editable);this.$popover.css({display:"block",left:pos.left,top:Math.min(pos.top,posEditor.top)})}else this.hide()},this.hide=function(){this.$popover.hide()}},TablePopover=function(context){var self=this,ui=$.summernote.ui,options=context.options;this.events={"summernote.mousedown":function(we,e){self.update(e.target)},"summernote.keyup summernote.scroll summernote.change":function(){self.update()},"summernote.disable":function(){self.hide()}},this.shouldInitialize=function(){return!list.isEmpty(options.popover.table)},this.initialize=function(){this.$popover=ui.popover({className:"note-table-popover"}).render().appendTo("body");var $content=this.$popover.find(".popover-content");context.invoke("buttons.build",$content,options.popover.table),agent.isFF&&document.execCommand("enableInlineTableEditing",!1,!1)},this.destroy=function(){this.$popover.remove()},this.update=function(target){if(context.isDisabled())return!1;var isCell=dom.isCell(target);if(isCell){var pos=dom.posFromPlaceholder(target);this.$popover.css({display:"block",left:pos.left,top:pos.top})}else this.hide();return isCell},this.hide=function(){this.$popover.hide()}},VideoDialog=function(context){var self=this,ui=$.summernote.ui,$editor=context.layoutInfo.editor,options=context.options,lang=options.langInfo;this.initialize=function(){var $container=options.dialogsInBody?$(document.body):$editor,body='<div class="form-group row-fluid"><label>'+lang.video.url+' <small class="text-muted">'+lang.video.providers+'</small></label><input class="note-video-url form-control span12" type="text" /></div>',footer='<button href="#" class="btn btn-primary note-video-btn disabled" disabled>'+lang.video.insert+"</button>";this.$dialog=ui.dialog({title:lang.video.insert,fade:options.dialogsFade,body:body,footer:footer}).render().appendTo($container)},this.destroy=function(){ui.hideDialog(this.$dialog),this.$dialog.remove()},this.bindEnterKey=function($input,$btn){$input.on("keypress",function(event){event.keyCode===key.code.ENTER&&$btn.trigger("click")})},this.createVideoNode=function(url){var $video,ytRegExp=/^(?:https?:\/\/)?(?:www\.)?(?:youtu\.be\/|youtube\.com\/(?:embed\/|v\/|watch\?v=|watch\?.+&v=))((\w|-){11})(?:\S+)?$/,ytMatch=url.match(ytRegExp),igRegExp=/(?:www\.|\/\/)instagram\.com\/p\/(.[a-zA-Z0-9_-]*)/,igMatch=url.match(igRegExp),vRegExp=/\/\/vine\.co\/v\/([a-zA-Z0-9]+)/,vMatch=url.match(vRegExp),vimRegExp=/\/\/(player\.)?vimeo\.com\/([a-z]*\/)*(\d+)[?]?.*/,vimMatch=url.match(vimRegExp),dmRegExp=/.+dailymotion.com\/(video|hub)\/([^_]+)[^#]*(#video=([^_&]+))?/,dmMatch=url.match(dmRegExp),youkuRegExp=/\/\/v\.youku\.com\/v_show\/id_(\w+)=*\.html/,youkuMatch=url.match(youkuRegExp),qqRegExp=/\/\/v\.qq\.com.*?vid=(.+)/,qqMatch=url.match(qqRegExp),qqRegExp2=/\/\/v\.qq\.com\/x?\/?(page|cover).*?\/([^\/]+)\.html\??.*/,qqMatch2=url.match(qqRegExp2),mp4RegExp=/^.+.(mp4|m4v)$/,mp4Match=url.match(mp4RegExp),oggRegExp=/^.+.(ogg|ogv)$/,oggMatch=url.match(oggRegExp),webmRegExp=/^.+.(webm)$/,webmMatch=url.match(webmRegExp);if(ytMatch&&11===ytMatch[1].length){var youtubeId=ytMatch[1];$video=$("<iframe>").attr("frameborder",0).attr("src","//www.youtube.com/embed/"+youtubeId).attr("width","640").attr("height","360")}else if(igMatch&&igMatch[0].length)$video=$("<iframe>").attr("frameborder",0).attr("src","https://instagram.com/p/"+igMatch[1]+"/embed/").attr("width","612").attr("height","710").attr("scrolling","no").attr("allowtransparency","true");else if(vMatch&&vMatch[0].length)$video=$("<iframe>").attr("frameborder",0).attr("src",vMatch[0]+"/embed/simple").attr("width","600").attr("height","600").attr("class","vine-embed");else if(vimMatch&&vimMatch[3].length)$video=$("<iframe webkitallowfullscreen mozallowfullscreen allowfullscreen>").attr("frameborder",0).attr("src","//player.vimeo.com/video/"+vimMatch[3]).attr("width","640").attr("height","360");else if(dmMatch&&dmMatch[2].length)$video=$("<iframe>").attr("frameborder",0).attr("src","//www.dailymotion.com/embed/video/"+dmMatch[2]).attr("width","640").attr("height","360");else if(youkuMatch&&youkuMatch[1].length)$video=$("<iframe webkitallowfullscreen mozallowfullscreen allowfullscreen>").attr("frameborder",0).attr("height","498").attr("width","510").attr("src","//player.youku.com/embed/"+youkuMatch[1]);else if(qqMatch&&qqMatch[1].length||qqMatch2&&qqMatch2[2].length){var vid=qqMatch&&qqMatch[1].length?qqMatch[1]:qqMatch2[2];$video=$("<iframe webkitallowfullscreen mozallowfullscreen allowfullscreen>").attr("frameborder",0).attr("height","310").attr("width","500").attr("src","http://v.qq.com/iframe/player.html?vid="+vid+"&amp;auto=0")}else{if(!(mp4Match||oggMatch||webmMatch))return!1;$video=$("<video controls>").attr("src",url).attr("width","640").attr("height","360")}return $video.addClass("note-video-clip"),$video[0]},this.show=function(){var text=context.invoke("editor.getSelectedText");context.invoke("editor.saveRange"),this.showVideoDialog(text).then(function(url){ui.hideDialog(self.$dialog),context.invoke("editor.restoreRange");var $node=self.createVideoNode(url);$node&&context.invoke("editor.insertNode",$node)}).fail(function(){context.invoke("editor.restoreRange")})},this.showVideoDialog=function(text){return $.Deferred(function(deferred){var $videoUrl=self.$dialog.find(".note-video-url"),$videoBtn=self.$dialog.find(".note-video-btn");ui.onDialogShown(self.$dialog,function(){context.triggerEvent("dialog.shown"),$videoUrl.val(text).on("input",function(){ui.toggleBtn($videoBtn,$videoUrl.val())}).trigger("focus"),$videoBtn.click(function(event){event.preventDefault(),deferred.resolve($videoUrl.val())}),self.bindEnterKey($videoUrl,$videoBtn)}),ui.onDialogHidden(self.$dialog,function(){$videoUrl.off("input"),$videoBtn.off("click"),"pending"===deferred.state()&&deferred.reject()}),ui.showDialog(self.$dialog)})}},HelpDialog=function(context){var self=this,ui=$.summernote.ui,$editor=context.layoutInfo.editor,options=context.options,lang=options.langInfo;this.createShortCutList=function(){var keyMap=options.keyMap[agent.isMac?"mac":"pc"];return Object.keys(keyMap).map(function(key){var command=keyMap[key],$row=$('<div><div class="help-list-item"/></div>');return $row.append($("<label><kbd>"+key+"</kdb></label>").css({width:180,"margin-right":10})).append($("<span/>").html(context.memo("help."+command)||command)),$row.html()}).join("")},this.initialize=function(){var $container=options.dialogsInBody?$(document.body):$editor,body=['<p class="text-center">','<a href="http://summernote.org/" target="_blank">Summernote 0.8.7</a> · ','<a href="https://github.com/summernote/summernote" target="_blank">Project</a> · ','<a href="https://github.com/summernote/summernote/issues" target="_blank">Issues</a>',"</p>"].join("");this.$dialog=ui.dialog({title:lang.options.help,fade:options.dialogsFade,body:this.createShortCutList(),footer:body,callback:function($node){$node.find(".modal-body").css({"max-height":300,overflow:"scroll"})}}).render().appendTo($container)},this.destroy=function(){ui.hideDialog(this.$dialog),this.$dialog.remove()},this.showHelpDialog=function(){return $.Deferred(function(deferred){ui.onDialogShown(self.$dialog,function(){context.triggerEvent("dialog.shown"),deferred.resolve()}),ui.showDialog(self.$dialog)}).promise()},this.show=function(){context.invoke("editor.saveRange"),this.showHelpDialog().then(function(){context.invoke("editor.restoreRange")})}},AirPopover=function(context){var self=this,ui=$.summernote.ui,options=context.options,AIR_MODE_POPOVER_X_OFFSET=20;this.events={"summernote.keyup summernote.mouseup summernote.scroll":function(){self.update()},"summernote.disable summernote.change summernote.dialog.shown":function(){self.hide()},"summernote.focusout":function(we,e){agent.isFF||e.relatedTarget&&dom.ancestor(e.relatedTarget,func.eq(self.$popover[0]))||self.hide()}},this.shouldInitialize=function(){return options.airMode&&!list.isEmpty(options.popover.air)},this.initialize=function(){this.$popover=ui.popover({className:"note-air-popover"}).render().appendTo("body");var $content=this.$popover.find(".popover-content");context.invoke("buttons.build",$content,options.popover.air)},this.destroy=function(){this.$popover.remove()},this.update=function(){var styleInfo=context.invoke("editor.currentStyle");if(styleInfo.range&&!styleInfo.range.isCollapsed()){var rect=list.last(styleInfo.range.getClientRects());if(rect){var bnd=func.rect2bnd(rect);this.$popover.css({display:"block",left:Math.max(bnd.left+bnd.width/2,0)-AIR_MODE_POPOVER_X_OFFSET,top:bnd.top+bnd.height})}}else this.hide()},this.hide=function(){this.$popover.hide()}},HintPopover=function(context){var self=this,ui=$.summernote.ui,POPOVER_DIST=5,hint=context.options.hint||[],direction=context.options.hintDirection||"bottom",hints=$.isArray(hint)?hint:[hint];this.events={"summernote.keyup":function(we,e){e.isDefaultPrevented()||self.handleKeyup(e)},"summernote.keydown":function(we,e){self.handleKeydown(e)},"summernote.disable summernote.dialog.shown":function(){self.hide()}},this.shouldInitialize=function(){return hints.length>0},this.initialize=function(){this.lastWordRange=null,this.$popover=ui.popover({className:"note-hint-popover",hideArrow:!0,direction:""}).render().appendTo("body"),this.$popover.hide(),this.$content=this.$popover.find(".popover-content"),this.$content.on("click",".note-hint-item",function(){self.$content.find(".active").removeClass("active"),$(this).addClass("active"),self.replace()})},this.destroy=function(){this.$popover.remove()},this.selectItem=function($item){this.$content.find(".active").removeClass("active"),$item.addClass("active"),this.$content[0].scrollTop=$item[0].offsetTop-this.$content.innerHeight()/2},this.moveDown=function(){var $current=this.$content.find(".note-hint-item.active"),$next=$current.next();if($next.length)this.selectItem($next);else{var $nextGroup=$current.parent().next();$nextGroup.length||($nextGroup=this.$content.find(".note-hint-group").first()),this.selectItem($nextGroup.find(".note-hint-item").first())}},this.moveUp=function(){var $current=this.$content.find(".note-hint-item.active"),$prev=$current.prev();if($prev.length)this.selectItem($prev);else{var $prevGroup=$current.parent().prev();$prevGroup.length||($prevGroup=this.$content.find(".note-hint-group").last()),this.selectItem($prevGroup.find(".note-hint-item").last())}},this.replace=function(){var $item=this.$content.find(".note-hint-item.active");if($item.length){var node=this.nodeFromItem($item);this.lastWordRange.insertNode(node),range.createFromNode(node).collapse().select(),this.lastWordRange=null,this.hide(),context.triggerEvent("change",context.layoutInfo.editable.html(),context.layoutInfo.editable),context.invoke("editor.focus")}},this.nodeFromItem=function($item){var hint=hints[$item.data("index")],item=$item.data("item"),node=hint.content?hint.content(item):item;return"string"==typeof node&&(node=dom.createText(node)),node},this.createItemTemplates=function(hintIdx,items){var hint=hints[hintIdx];return items.map(function(item,idx){var $item=$('<div class="note-hint-item"/>');return $item.append(hint.template?hint.template(item):item+""),$item.data({index:hintIdx,item:item}),0===hintIdx&&0===idx&&$item.addClass("active"),$item})},this.handleKeydown=function(e){this.$popover.is(":visible")&&(e.keyCode===key.code.ENTER?(e.preventDefault(),this.replace()):e.keyCode===key.code.UP?(e.preventDefault(),this.moveUp()):e.keyCode===key.code.DOWN&&(e.preventDefault(),this.moveDown()))},this.searchKeyword=function(index,keyword,callback){var hint=hints[index];if(hint&&hint.match.test(keyword)&&hint.search){var matches=hint.match.exec(keyword);hint.search(matches[1],callback)}else callback()},this.createGroup=function(idx,keyword){var $group=$('<div class="note-hint-group note-hint-group-'+idx+'"/>');return this.searchKeyword(idx,keyword,function(items){items=items||[],items.length&&($group.html(self.createItemTemplates(idx,items)),self.show())}),$group},this.handleKeyup=function(e){if(list.contains([key.code.ENTER,key.code.UP,key.code.DOWN],e.keyCode)){if(e.keyCode===key.code.ENTER&&this.$popover.is(":visible"))return}else{var wordRange=context.invoke("editor.createRange").getWordRange(),keyword=wordRange.toString();if(hints.length&&keyword){this.$content.empty();var bnd=func.rect2bnd(list.last(wordRange.getClientRects()));bnd&&(this.$popover.hide(),this.lastWordRange=wordRange,hints.forEach(function(hint,idx){hint.match.test(keyword)&&self.createGroup(idx,keyword).appendTo(self.$content)}),"top"===direction?this.$popover.css({left:bnd.left,top:bnd.top-this.$popover.outerHeight()-POPOVER_DIST}):this.$popover.css({left:bnd.left,top:bnd.top+bnd.height+POPOVER_DIST}))}else this.hide()}},this.show=function(){this.$popover.show()},this.hide=function(){this.$popover.hide()}};$.summernote=$.extend($.summernote,{version:"0.8.7",ui:ui,dom:dom,plugins:{},options:{modules:{editor:Editor,clipboard:Clipboard,dropzone:Dropzone,codeview:Codeview,statusbar:Statusbar,fullscreen:Fullscreen,handle:Handle,hintPopover:HintPopover,autoLink:AutoLink,autoSync:AutoSync,placeholder:Placeholder,buttons:Buttons,toolbar:Toolbar,linkDialog:LinkDialog,linkPopover:LinkPopover,imageDialog:ImageDialog,imagePopover:ImagePopover,tablePopover:TablePopover,videoDialog:VideoDialog,helpDialog:HelpDialog,airPopover:AirPopover},buttons:{},lang:"en-US",toolbar:[["style",["style"]],["font",["bold","underline","clear"]],["fontname",["fontname"]],["color",["color"]],["para",["ul","ol","paragraph"]],["table",["table"]],["insert",["link","picture","video"]],["view",["fullscreen","codeview","help"]]],popover:{image:[["imagesize",["imageSize100","imageSize50","imageSize25"]],["float",["floatLeft","floatRight","floatNone"]],["remove",["removeMedia"]]],link:[["link",["linkDialogShow","unlink"]]],table:[["add",["addRowDown","addRowUp","addColLeft","addColRight"]],["delete",["deleteRow","deleteCol","deleteTable"]]],air:[["color",["color"]],["font",["bold","underline","clear"]],["para",["ul","paragraph"]],["table",["table"]],["insert",["link","picture"]]]},airMode:!1,width:null,height:null,linkTargetBlank:!0,focus:!1,tabSize:4,styleWithSpan:!0,shortcuts:!0,textareaAutoSync:!0,direction:null,tooltip:"auto",styleTags:["p","blockquote","pre","h1","h2","h3","h4","h5","h6"],fontNames:["Arial","Arial Black","Comic Sans MS","Courier New","Helvetica Neue","Helvetica","Impact","Lucida Grande","Tahoma","Times New Roman","Verdana"],fontSizes:["8","9","10","11","12","14","18","24","36"],colors:[["#000000","#424242","#636363","#9C9C94","#CEC6CE","#EFEFEF","#F7F7F7","#FFFFFF"],["#FF0000","#FF9C00","#FFFF00","#00FF00","#00FFFF","#0000FF","#9C00FF","#FF00FF"],["#F7C6CE","#FFE7CE","#FFEFC6","#D6EFD6","#CEDEE7","#CEE7F7","#D6D6E7","#E7D6DE"],["#E79C9C","#FFC69C","#FFE79C","#B5D6A5","#A5C6CE","#9CC6EF","#B5A5D6","#D6A5BD"],["#E76363","#F7AD6B","#FFD663","#94BD7B","#73A5AD","#6BADDE","#8C7BC6","#C67BA5"],["#CE0000","#E79439","#EFC631","#6BA54A","#4A7B8C","#3984C6","#634AA5","#A54A7B"],["#9C0000","#B56308","#BD9400","#397B21","#104A5A","#085294","#311873","#731842"],["#630000","#7B3900","#846300","#295218","#083139","#003163","#21104A","#4A1031"]],lineHeights:["1.0","1.2","1.4","1.5","1.6","1.8","2.0","3.0"],tableClassName:"table table-bordered",insertTableMaxSize:{col:10,row:10},dialogsInBody:!1,dialogsFade:!1,maximumImageFileSize:null,callbacks:{onInit:null,onFocus:null,onBlur:null,onEnter:null,onKeyup:null,onKeydown:null,onImageUpload:null,onImageUploadError:null},codemirror:{mode:"text/html",htmlMode:!0,lineNumbers:!0},keyMap:{pc:{ENTER:"insertParagraph","CTRL+Z":"undo","CTRL+Y":"redo",TAB:"tab","SHIFT+TAB":"untab","CTRL+B":"bold","CTRL+I":"italic","CTRL+U":"underline","CTRL+SHIFT+S":"strikethrough","CTRL+BACKSLASH":"removeFormat","CTRL+SHIFT+L":"justifyLeft","CTRL+SHIFT+E":"justifyCenter","CTRL+SHIFT+R":"justifyRight","CTRL+SHIFT+J":"justifyFull","CTRL+SHIFT+NUM7":"insertUnorderedList","CTRL+SHIFT+NUM8":"insertOrderedList","CTRL+LEFTBRACKET":"outdent","CTRL+RIGHTBRACKET":"indent","CTRL+NUM0":"formatPara","CTRL+NUM1":"formatH1","CTRL+NUM2":"formatH2","CTRL+NUM3":"formatH3","CTRL+NUM4":"formatH4","CTRL+NUM5":"formatH5","CTRL+NUM6":"formatH6","CTRL+ENTER":"insertHorizontalRule","CTRL+K":"linkDialog.show"},mac:{ENTER:"insertParagraph","CMD+Z":"undo","CMD+SHIFT+Z":"redo",TAB:"tab","SHIFT+TAB":"untab","CMD+B":"bold","CMD+I":"italic","CMD+U":"underline","CMD+SHIFT+S":"strikethrough","CMD+BACKSLASH":"removeFormat","CMD+SHIFT+L":"justifyLeft","CMD+SHIFT+E":"justifyCenter","CMD+SHIFT+R":"justifyRight","CMD+SHIFT+J":"justifyFull","CMD+SHIFT+NUM7":"insertUnorderedList","CMD+SHIFT+NUM8":"insertOrderedList","CMD+LEFTBRACKET":"outdent","CMD+RIGHTBRACKET":"indent","CMD+NUM0":"formatPara","CMD+NUM1":"formatH1","CMD+NUM2":"formatH2","CMD+NUM3":"formatH3","CMD+NUM4":"formatH4","CMD+NUM5":"formatH5","CMD+NUM6":"formatH6","CMD+ENTER":"insertHorizontalRule","CMD+K":"linkDialog.show"}},icons:{align:"note-icon-align",alignCenter:"note-icon-align-center",alignJustify:"note-icon-align-justify",alignLeft:"note-icon-align-left",alignRight:"note-icon-align-right",rowBelow:"note-icon-row-below",colBefore:"note-icon-col-before",colAfter:"note-icon-col-after",rowAbove:"note-icon-row-above",rowRemove:"note-icon-row-remove",colRemove:"note-icon-col-remove",indent:"note-icon-align-indent",outdent:"note-icon-align-outdent",arrowsAlt:"note-icon-arrows-alt",bold:"note-icon-bold",caret:"note-icon-caret",circle:"note-icon-circle",close:"note-icon-close",code:"note-icon-code",eraser:"note-icon-eraser",font:"note-icon-font",frame:"note-icon-frame",italic:"note-icon-italic",link:"note-icon-link",unlink:"note-icon-chain-broken",magic:"note-icon-magic",menuCheck:"note-icon-check",minus:"note-icon-minus",orderedlist:"note-icon-orderedlist",pencil:"note-icon-pencil",picture:"note-icon-picture",question:"note-icon-question",redo:"note-icon-redo",square:"note-icon-square",strikethrough:"note-icon-strikethrough",subscript:"note-icon-subscript",superscript:"note-icon-superscript",table:"note-icon-table",textHeight:"note-icon-text-height",trash:"note-icon-trash",underline:"note-icon-underline",undo:"note-icon-undo",unorderedlist:"note-icon-unorderedlist",video:"note-icon-video"}}})}),function($){$.extend($.summernote.lang,{"pt-BR":{font:{bold:"Negrito",italic:"Itálico",underline:"Sublinhado",clear:"Remover estilo da fonte",height:"Altura da linha",name:"Fonte",strikethrough:"Riscado",size:"Tamanho da fonte"},image:{image:"Imagem",insert:"Inserir imagem",resizeFull:"Redimensionar Completamente",resizeHalf:"Redimensionar pela Metade",resizeQuarter:"Redimensionar um Quarto",floatLeft:"Flutuar para Esquerda",floatRight:"Flutuar para Direira",floatNone:"Não Flutuar",dragImageHere:"Arraste uma imagem para cá",selectFromFiles:"Selecione a partir dos arquivos",url:"URL da imagem"},video:{video:"Vídeo",videoLink:"Link para vídeo",insert:"Inserir vídeo",url:"URL do vídeo?",providers:"(YouTube, Vimeo, Vine, Instagram, DailyMotion ou Youku)"},link:{link:"Link",insert:"Inserir link",unlink:"Remover link",edit:"Editar",textToDisplay:"Texto para exibir",url:"Para qual URL esse link leva?",openInNewWindow:"Abrir em uma nova janela"},table:{table:"Tabela",addRowAbove:"Adicionar linha acima",addRowBelow:"Adicionar linha abaixo",addColLeft:"Adicionar coluna a esquerda",addColRight:"Adicionar coluna a direita",delRow:"Excluir linha",delCol:"Excluir coluna"},hr:{insert:"Inserir linha horizontal"},style:{style:"Estilo",normal:"Normal",blockquote:"Citação",pre:"Código",h1:"Título 1",h2:"Título 2",h3:"Título 3",h4:"Título 4",h5:"Título 5",h6:"Título 6"},lists:{unordered:"Lista com marcadores",ordered:"Lista numerada"},options:{help:"Ajuda",fullscreen:"Tela cheia",codeview:"Ver código-fonte"},paragraph:{paragraph:"Parágrafo",outdent:"Menor tabulação",indent:"Maior tabulação",left:"Alinhar à esquerda",center:"Alinhar ao centro",right:"Alinha à direita",justify:"Justificado"},color:{recent:"Cor recente",more:"Mais cores",background:"Fundo",foreground:"Fonte",transparent:"Transparente",setTransparent:"Fundo transparente",reset:"Restaurar",resetToDefault:"Restaurar padrão"},shortcut:{shortcuts:"Atalhos do teclado",close:"Fechar",textFormatting:"Formatação de texto",action:"Ação",paragraphFormatting:"Formatação de parágrafo",documentStyle:"Estilo de documento"},history:{undo:"Desfazer",redo:"Refazer"},help:{insertParagraph:"Inserir Parágrafo",undo:"Desfazer o último comando",redo:"Refazer o último comando",tab:"Tab",untab:"Desfazer tab",bold:"Colocar em negrito",italic:"Colocar em itálico",underline:"Sublinhado",strikethrough:"Tachado",removeFormat:"Remover estilo",justifyLeft:"Alinhar à esquerda",justifyCenter:"Centralizar",justifyRight:"Alinhar à esquerda",justifyFull:"Justificar",insertUnorderedList:"Lista não ordenada",insertOrderedList:"Lista ordenada",outdent:"Recuar parágrafo atual",indent:"Avançar parágrafo atual",formatPara:"Alterar formato do bloco para parágrafo(tag P)",formatH1:"Alterar formato do bloco para H1",formatH2:"Alterar formato do bloco para H2",formatH3:"Alterar formato do bloco para H3",formatH4:"Alterar formato do bloco para H4",formatH5:"Alterar formato do bloco para H5",formatH6:"Alterar formato do bloco para H6",insertHorizontalRule:"Inserir régua horizontal","linkDialog.show":"Inserir um Hiperlink"}}})}(jQuery),angular.module("summernote",[]).controller("SummernoteController",["$scope","$attrs","$timeout",function($scope,$attrs,$timeout){"use strict";var currentElement,summernoteConfig=angular.copy($scope.summernoteConfig)||{};if(angular.isDefined($attrs.height)&&(summernoteConfig.height=+$attrs.height),angular.isDefined($attrs.minHeight)&&(summernoteConfig.minHeight=+$attrs.minHeight),angular.isDefined($attrs.maxHeight)&&(summernoteConfig.maxHeight=+$attrs.maxHeight),angular.isDefined($attrs.placeholder)&&(summernoteConfig.placeholder=$attrs.placeholder),angular.isDefined($attrs.focus)&&(summernoteConfig.focus=!0),angular.isDefined($attrs.airmode)&&(summernoteConfig.airMode=!0),angular.isDefined($attrs.lang)){if(!angular.isDefined($.summernote.lang[$attrs.lang]))throw new Error('"'+$attrs.lang+'" lang file must be exist.');summernoteConfig.lang=$attrs.lang}summernoteConfig.callbacks=summernoteConfig.callbacks||{},angular.isDefined($attrs.onInit)&&(summernoteConfig.callbacks.onInit=function(evt){$scope.init({evt:evt})}),angular.isDefined($attrs.onEnter)&&(summernoteConfig.callbacks.onEnter=function(evt){$scope.enter({evt:evt})}),angular.isDefined($attrs.onFocus)&&(summernoteConfig.callbacks.onFocus=function(evt){$scope.focus({evt:evt})}),angular.isDefined($attrs.onPaste)&&(summernoteConfig.callbacks.onPaste=function(evt){$scope.paste({evt:evt})}),angular.isDefined($attrs.onKeyup)&&(summernoteConfig.callbacks.onKeyup=function(evt){$scope.keyup({evt:evt})}),angular.isDefined($attrs.onKeydown)&&(summernoteConfig.callbacks.onKeydown=function(evt){$scope.keydown({evt:evt})}),angular.isDefined($attrs.onImageUpload)&&(summernoteConfig.callbacks.onImageUpload=function(files){$scope.imageUpload({files:files,editable:$scope.editable})}),angular.isDefined($attrs.onMediaDelete)&&(summernoteConfig.callbacks.onMediaDelete=function(target){var removedMedia={attrs:{}};removedMedia.tagName=target[0].tagName,angular.forEach(target[0].attributes,function(attr){removedMedia.attrs[attr.name]=attr.value}),$scope.mediaDelete({target:removedMedia})}),this.activate=function(scope,element,ngModel){var updateNgModel=function(){
var newValue=element.summernote("code");element.summernote("isEmpty")&&(newValue=""),ngModel&&ngModel.$viewValue!==newValue&&$timeout(function(){ngModel.$setViewValue(newValue)},0)},originalOnChange=summernoteConfig.callbacks.onChange;summernoteConfig.callbacks.onChange=function(contents){$timeout(function(){element.summernote("isEmpty")&&(contents=""),updateNgModel()},0),angular.isDefined($attrs.onChange)?$scope.change({contents:contents,editable:$scope.editable}):angular.isFunction(originalOnChange)&&originalOnChange.apply(this,arguments)},angular.isDefined($attrs.onBlur)&&(summernoteConfig.callbacks.onBlur=function(evt){!summernoteConfig.airMode&&element.blur(),$scope.blur({evt:evt})}),element.summernote(summernoteConfig);var unwatchNgModel,editor$=element.next(".note-editor");if(editor$.find(".note-toolbar").click(function(){updateNgModel(),editor$.hasClass("codeview")?(editor$.on("keyup",updateNgModel),ngModel&&(unwatchNgModel=scope.$watch(function(){return ngModel.$modelValue},function(newValue){editor$.find(".note-codable").val(newValue)}))):(editor$.off("keyup",updateNgModel),angular.isFunction(unwatchNgModel)&&unwatchNgModel())}),ngModel){var _sanitize=function(objectHtml){return objectHtml?richTextSanitize(objectHtml):objectHtml};ngModel.$render=function(){ngModel.$viewValue?element.summernote("code",_sanitize(ngModel.$viewValue)):element.summernote("empty")}}angular.isDefined($attrs.editable)&&($scope.editable=editor$.find(".note-editable")),angular.isDefined($attrs.editor)&&($scope.editor=element),currentElement=element,element.on("$destroy",function(){element.summernote("destroy"),$scope.summernoteDestroyed=!0})},$scope.$on("$destroy",function(){$scope.summernoteDestroyed||currentElement.summernote("destroy")})}]).directive("summernote",[function(){"use strict";return{restrict:"EA",transclude:"element",replace:!0,require:["summernote","?ngModel"],controller:"SummernoteController",scope:{summernoteConfig:"=config",editable:"=",editor:"=",init:"&onInit",enter:"&onEnter",focus:"&onFocus",blur:"&onBlur",paste:"&onPaste",keyup:"&onKeyup",keydown:"&onKeydown",change:"&onChange",imageUpload:"&onImageUpload",mediaDelete:"&onMediaDelete"},template:'<div class="summernote"></div>',link:function(scope,element,attrs,ctrls,transclude){var summernoteController=ctrls[0],ngModel=ctrls[1],_sanitize=function(objectHtml){return objectHtml?richTextSanitize(objectHtml):objectHtml};if(ngModel)var clearWatch=scope.$watch(function(){return ngModel.$viewValue},function(value){clearWatch(),element.append(_sanitize(value)),summernoteController.activate(scope,element,ngModel)},!0);else transclude(scope,function(clone,scope){element.append(_sanitize(clone.html()))}),summernoteController.activate(scope,element,ngModel)}}}]),function(){var ZeedhiApp=angular.module("ZeedhiApp");window.templateUrls=window.templateUrls||[];var packageName="zh-rich-text",hasPackageRegistered=window.templateUrls.some(function(templateUrl){return templateUrl["package"]===packageName});hasPackageRegistered||window.templateUrls.push({"package":packageName,baseUrl:"./bower_components/"+packageName+"/dist/templates/"}),ZeedhiApp.requires.push("summernote")}(),function(){function zhRichText(templateManager,ZHPromise,$timeout){return{restrict:"A",require:"?ngModel",link:function(scope,element,attrs,controller){function isEmptyText(text){return""===text||"<br>"==text||"<p><br></p>"==text}function setCursorPositionToEnd(node){node.focus();var textNode=node.firstChild,range=document.createRange();range.setStart(textNode,1),range.setEnd(textNode,1);var sel=window.getSelection();sel.removeAllRanges(),sel.addRange(range)}scope.onInitElement=function(){if(isEmptyText(element.summernote("code"))){var context=element.data("summernote");context&&scope.field&&(context.invoke("editor.fontSize",scope.field.defaultFontSize||14),context.invoke("editor.fontName",scope.field.defaultFontName||"Arial"),context.invoke("buttons.updateCurrentStyle"))}else{var editor=element.next().find(".note-editable")[0],bogusCharPosition=editor.innerHTML.indexOf("\ufeff");bogusCharPosition!==-1&&(editor.innerHTML=editor.innerHTML.slice(0,bogusCharPosition)+editor.innerHTML.slice(bogusCharPosition+1),setCursorPositionToEnd(editor))}},controller&&controller.$parsers.unshift(function(value){return value=value.replace(/[\ufeff]/g,"").replace(/<p>/g,"<div>").replace(/<p /g,"<div ").replace(/<\/p>/g,"</div>")})}}}var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhRichText",zhRichText)}(),function(){function registerRichTextDecoders(DecodeValueFieldService,decodeTemplateFieldService,$sce){decodeTemplateFieldService.registerViewTemplate("zh-rich-text#field/rich-text.html","zh-rich-text#field/rich-text-view.html"),DecodeValueFieldService.register("rich-text.html",richTextValueDecoder($sce))}function richTextValueDecoder($sce){return function(field,fieldRow){return $sce.trustAsHtml(richTextSanitize(fieldRow))}}Configuration(registerRichTextDecoders)}();

// FILE: bower_components/zh-image-uploader/dist/zh-image-uploader.min.js
function ImageUploaderController($scope,templateManager,ScreenService){function clearCurrentRow(){self.widget.currentRow[self.field.name]="",_callChangeEvent(),_callAfterDeselectFileEvent()}function _callChangeEvent(){self.field.change&&self.field.change()}function _callAfterDeselectFileEvent(){self.field.afterDeselectFile&&self.field.afterDeselectFile()}var self=this,widgetAccessorDescriptor={get:function(){return $scope.widget}},fieldAcessosDescriptor={get:function(){return $scope.field}};Object.defineProperty(self,"widget",widgetAccessorDescriptor),Object.defineProperty(self,"field",fieldAcessosDescriptor),this.getChooseFileText=function(){return self.field.chooseFileText||ScreenService.i18n("Choose a file...")},this.getNoFileSelectedText=function(){var currentFieldRow=self.widget.currentRow[self.field.name];return currentFieldRow&&"object"==typeof currentFieldRow?self.field.noFileSelectedText=currentFieldRow[0].name:currentFieldRow?self.field.noFileSelectedText=currentFieldRow:self.field.noFileSelectedText=ScreenService.i18n("No file selected."),self.field.noFileSelectedText||ScreenService.i18n("No file selected.")},this.getCurrentThumbnail=function(){var currentFieldRow=self.widget.currentRow[self.field.name],currentThumbnail="//:0";if("string"==typeof currentFieldRow){var root=templateManager.project.fileServiceBaseUrl||"";currentThumbnail=root+currentFieldRow}else Array.isArray(currentFieldRow)&&currentFieldRow.length>0&&(currentThumbnail=currentFieldRow[0].b64File);return currentThumbnail},this.getCurrentThumbnailStringify=function(){return JSON.stringify(self.getCurrentThumbnail())},this.isVisibleInCurrentFormTemplate=function(){},this.isEditMode=function(){return"edit"===self.widget.mode},this.clearImage=function(file){return file?(clearCurrentRow(),file.name=ScreenService.i18n("No file selected."),file.name):(clearCurrentRow(),self.field.noFileSelectedText=ScreenService.i18n("No file selected."),self.field.noFileSelectedText)}}function ImageUploaderRegister(DecodeValueFieldService,decodeTemplateFieldService){DecodeValueFieldService.registerType("zh-image-uploader#field/image-uploader.html","image"),decodeTemplateFieldService.registerViewTemplate("zh-image-uploader#field/image-uploader.html","zh-image-uploader#field/image-view.html")}window.templateUrls=window.templateUrls||[];var packageName="zh-image-uploader",hasPackageRegistered=window.templateUrls.some(function(templateUrl){return templateUrl["package"]===packageName});hasPackageRegistered||window.templateUrls.push({"package":packageName,baseUrl:"./bower_components/"+packageName+"/dist/templates/"}),function(){function zhImageUploader(templateManager,ZHPromise){var linkFunction=function(scope,element,attrs){function onFileSelected(){var files=this.files;files.length>0&&(_ensureCurrentRowPropertyExistence(),_setCurrentRowAsChanged(),_putFilesIntoPromises(files),_executeAllPromises()),this.value=""}function _ensureCurrentRowPropertyExistence(){widget.currentRow=widget.currentRow||{},widget.currentRow[field.name]=widget.currentRow[field.name]||[]}function _setCurrentRowAsChanged(){widget.currentRow.changed=!0}function _putFilesIntoPromises(files){for(var i in files)files.hasOwnProperty(i)&&files[i]instanceof File&&filesPromises.push(_buildFileData(files[i]))}function _buildFileData(file){var deferred=ZHPromise.defer(),fileReader=new FileReader;return fileReader.onload=function(event){widget.currentRow[field.name]=[_addExtraDataToFile(file)],field.fileList=widget.currentRow[field.name],deferred.resolve()},fileReader.readAsDataURL(file),deferred}function _addExtraDataToFile(file){return $.extend(!1,{b64File:event.target.result},file)}function _executeAllPromises(){ZHPromise.all(filesPromises).then(function(){_callAfterSelectFileEvent(),field.validate(field.value()),templateManager.updateTemplate()})}function _callAfterSelectFileEvent(){field.afterSelectFile&&field.afterSelectFile()}var field=(element.parent(),scope.field),widget=field.widget,filesPromises=[];field.fileList=[],element.shouldNotPreventDefaultOnClick=!0,element.parent().find("ul").nonbounce(),element.on("change",onFileSelected);var button=element.siblings("button"),input=element;button.on("click.file-upload touchstart.file-upload",function(e){input.trigger(e.type)})};return{restrict:"A",link:linkFunction}}var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhImageUploader",zhImageUploader)}(),Configuration(function(ContextRegister){ContextRegister.register("ImageUploaderRegister",ImageUploaderRegister)});

// FILE: bower_components/zh-reports/dist/zhReports.js

// FILE: src/FloatingReportCardController.js
function FloatingReportCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {

	//------ Control variables
	var editingWidget = $scope.$parent.widget;

    /**
     * Checks if the card is already opened and reconfigure the state of it if it not.
     *
     * @param  {jquery.Event} e   Event that triggered the click.
     */
	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			}
			else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}
			}
		}
	}

	/**
	 * Toggle the report action when it's template is configured as card.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 */
	$scope.toggleFloatReportCard = function(e) {
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};

	editingWidget.toggleFloatReportCard = $scope.toggleFloatReportCard;

	/**
	 * Change the report orientation.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 * @param  {String} orientation   Report orientation.
	 */
	$scope.setOrientation = function(e, orientation) {
		$scope.widget.reportOrientation = orientation;
		e.stopPropagation();
	};
}

// FILE: src/ZhReports.js
/* global Util, Configuration*/
function ZhReports(templateManager, ZHPromise, ScreenService, restEngine, ZhUtil, eventAggregator, $filter, DecodeValueFieldService, ZhLogger, eventEngine) {

    var reportConfig = {
        PDF:{
            uri: '/generateReport',
            type: 'PDF',
            action: 'pdfAction'
        },
        XLS:{
            uri: '/generateXLS',
            type: 'XLS',
            action: 'xlsAction'
        },
        CSV:{
            uri: '/generateCSV',
            type: 'CSV',
            action: 'csvAction'
        }
    };

    var maxVisibleColumns = {
        'LANDSCAPE': 15,
        'PORTRAIT': 10
    };

    var reportNameToReportFunction = {
        "=": "is equal to",
        "IN": "is among",
        "NOT_IN": "is not among",
        "!=": "is different of",
        ">": "is bigger than",
        "<": "is lower than",
        ">=": "is not lower than",
        "<=": "is not bigger than",
        "BETWEEN": ["is between", "and"],
        "NOT_BETWEEN": ["is lower than", "and bigger than"],
        "LIKE": "has",
        "LIKE_I": "has",
        "LIKE_ALL": "has any of",
        "NOT_LIKE": "does not have",
        "IS_NULL": "is not defined",
        "IS_NOT_NULL": "is defined",
        "ORDER_DESC": "ordered descending",
        "ORDER_ASC": "ordered ascending",
        "ORDER_BY": "ordered",
        "GROUP_BY": "grouped"
    };

    eventEngine.registerEvent('BeforeGenerateReport', 'beforeGenerateReport');

    function triggerBeforeGenerateReport(widget, metaData){
        if(widget.beforeGenerateReport){
            return widget.beforeGenerateReport({
                'metadata': metaData
            });
        }
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Field}  field  - Field with the data related to the filter.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    function getFieldFilterValue(field, filter) {
        return getFieldFilterDefaultText(filter, filter.value, ScreenService.i18n(reportNameToReportFunction[filter.operator]));
    }

    /**
     * Configure the default value for the variable if it's not defined.
     *
     * @param {Mixed} value        - The variable to be tested.
     * @param {Mixed} defaultValue - The value attributed the variable if it's not defined.
     *
     * @return {Mixed} The value that the variable should recieve.
     */
    function notUndefinedOrDefault(value, defaultValue) {
        return value === undefined ? defaultValue : value;
    }

    /**
     * Tests if the given type is PDF.
     *
     * @param {String} type - The type to be tested.
     *
     * @return {Boolean} Tells if the given type is PDF.
     */
    function isPdfFormat(type) {
        return type === reportConfig.PDF.type;
    }

     /**
     * Check which type of template need to be returned.
     *
     * @return {String} Template type to be used.
     */
    function checkZhReportActionDisplayMode() {
        if( !templateManager.project.zhReportActionDisplayMode ) {
            return 'zh-reports#floating-report-card.html';
        } else if (templateManager.project.zhReportActionDisplayMode === 'buttons') {
            return 'zh-reports#floating-report.html';
        }
    }

    eventAggregator.subscribe('beforeProjectStartup', function() {
        templateManager.project.customFloatingActions.push({
            'id': '#reportAction#',
            'name': 'reportAction',
            'label': ScreenService.i18n('Export'),
            "expandable": !templateManager.project.zhReportActionDisplayMode,
            'order': 10,
            'icon': 'file-export',
            'template': checkZhReportActionDisplayMode(),
            'isVisible': function(widget) {
                var show = true;
                if (widget.floatingControl) {
                    var showPdfAction = notUndefinedOrDefault(widget.floatingControl.pdfAction, true);
                    var showXlsAction = notUndefinedOrDefault(widget.floatingControl.xlsAction, true);
                    var showCsvAction = notUndefinedOrDefault(widget.floatingControl.csvAction, true);
                    show = showPdfAction || showXlsAction || showCsvAction;
                }

                return show;
            }
        });
    });

    /**
     * Checks if the propertie 'reportsUrl' is configured on the environment.json.
     *
     * @throws {Error} Message warning that the propertie should be configured at the file.
     */
    var checkConfiguration = function() {
        if (!(templateManager.project && templateManager.project.reportsUrl)) {
            throw (ScreenService.i18n(
                'Need report configuration in "environment.json": "reportsUrl"!'));
        }
    };

    /**
     * Returns the format of the column.
     *
     * @param {Field} column - The Field to test it's format.
     *
     * @return {Object} The Field's format.
     */
    var getFormatOfColumn = function(column) {
        var format = column.format;
        if (!format && column.mask) {
            column.mask.params = column.mask.params || {};
            format = {};
            format.params = column.mask.params;
            format.type = column.mask.type;
            if (Util.isString(format.params.mask) && Util.stringStartsWith(format.params.mask, '@') && Util.stringEndsWith(format.params.mask, '@')) {
                format.params.mask = ZhUtil.evaluate(format.params.mask);
            }
        }
        return format;
    };

    /**
     * Build a column to the metadata based on the given field.
     *
     * @param {Field}  column - Field base to create the column to the report.
     * @param {String} size   - The size of the Field, in percentage inside the report.
     *
     * @return {Object} Column to the metadata, with the necessary data.
     */
    var buildColumn = function(column, size) {
        if (Util.isUndefined(column.reportAlign)) {
            column.reportAlign = column.align || 'left';
        }
        return {
            align: column.reportAlign,
            description: ScreenService.i18n(column.labelOnReport || column.label),
            expression: column.expression,
            format: getFormatOfColumn(column),
            sequence: column.sequenceInList,
            size: size,
            xlsType: column.xlsType
        };
    };

    /**
     * Filter the widgetColumns by just the visible fields.
     *
     * @param {Array.<Field>}  widgetColumns               - Array to be filtered.
     * @param {Boolean}        reportShowOnlyVisibleFields - Tells if the report should show only visible fields.
     * @param {String}         [reportType='PDF']          - The type of the report.
     * @param {Array.<String>} [groups]                    - List of the field names that are grouped.
     *
     * @return {Array.<Field>} Filtered array with all fields that are visible on the report.
     */
    var filterVisibleColumns = function(widgetColumns, reportShowOnlyVisibleFields, reportType, groups) {
        var showGroupedColumns = !isPdfFormat(reportType);

        return widgetColumns.filter(function(column) {
            var isGroupedColumn = groups ? groups.indexOf(column.name) != -1 : column.grouped;
            var columnGroupVisibilitity = showGroupedColumns && isGroupedColumn;

            var isVisibleOnReport = Util.isUndefinedOrNull(column.isVisibleOnReport) || column.isVisibleOnReport;
            var columnVisibility = isVisibleOnReport && (!reportShowOnlyVisibleFields || (column.isVisible && column.showOnList));

            return column.aclIsVisible && (columnGroupVisibilitity || columnVisibility);
        });
    };

    /**
     * Returns the max visible columns on the report.
     *
     * @param  {Widget}    widget       -   Widget where the report is being generated.
     * @param  {String}    reportType   -   The type of the report.
     *
     * @return {Integer} The max of visible columns on the report.
     */
    var getMaxVisibleColumns = function(widget, reportType) {
        var maxColumns;
        if (reportType==reportConfig.PDF.type) {
            maxColumns = widget.maxColumns;
        } else {
            maxColumns = 99999;
        }
        return maxColumns;
    };

    /**
     * Builds the configuration of the field's display.
     *
     * @param {Widget}         widget             - Widget where the report is being generated.
     * @param {Array.<String>} [columnsToUse]     - The name of the fields that will be used on the
     * report generation.
     * @param {String}         [reportType='PDF'] - The type of the report.
     * @param {Array.<String>} [groups]           - List of the field names that are grouped.
     *
     * @return {Object} Object where each propertie is a object with the configuration of its display
     * on the report.
     */
    var _buildMetaDataColumns = function(widget, columnsToUse, reportType, groups) {
        var columns = {};
        var widgetColumns = widget.fields;
        var maxVisibleColCount = getMaxVisibleColumns(widget, reportType);
        if (widgetColumns) {
            widgetColumns = filterVisibleColumns(widgetColumns, widget.reportShowOnlyVisibleFields, reportType, groups);
            if (maxVisibleColCount && widgetColumns.length > maxVisibleColCount) {
                ZhLogger.warning(ScreenService.i18n(
                    "Number of columns too big to render the report correctly. It was limited to __maxVisibleColCount__ columns.",
                    {
                        'maxVisibleColCount': maxVisibleColCount
                    }
                ));
            }

            var remainingSize = 100;
            var columnsWithoutColumnSize = 0;

            widgetColumns.forEach(function(column) {
                if (column.reportColumnSize) {
                    remainingSize -= parseInt(column.reportColumnSize);
                }
                else {
                    columnsWithoutColumnSize++;
                }
            });

            if (remainingSize < 0) {
                ZhLogger.warning(ScreenService.i18n('Sum of columns width is bigger than 100%.'));
            }

            var size = Math.max(2, Math.floor(remainingSize / columnsWithoutColumnSize));
            var sum = 0;
            widgetColumns.forEach(function(column, index) {
                if (!columnsToUse || !columnsToUse[column.name]) {
                    var columnSize;
                    if (column.reportColumnSize) {
                        columnSize = column.reportColumnSize;
                    }
                    else {
                        if (index === (widgetColumns.length - 1)) {
                            columnSize = Math.max(2, remainingSize - sum);
                        }
                        else {
                            columnSize = size;
                            sum += size;
                        }
                    }

                    if (!columnSizeHasUnity(columnSize)) {
                        columnSize += '%';
                    }

                    columns[column.valueFieldOnReport || column.name] = buildColumn(column, columnSize);
                }
            });

        }
        return columns;
    };

    /**
     * Returns an array containing only the displayed value from field.
     *
     * @param {Object} object - Object containing the dataSource.data's row.
     *
     * @return {Object} Array containing only a object as {field.descriptionField: <displayedValue>}.
     */
    function getStaticPropertyValueFromObject(row) {
        var toReturn = {};
        toReturn[row[this.valueField]] = row[this.descriptionField];
        return toReturn;
    }

    /**
     * Builds the configuration of the field's staticData.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     * @param {Object} object - Object were the configuration field's display are gathered.
     *
     * @return {Object} Object where all the staticData from fields with DataSource are
     * gathered to be treated.
     */
    var _buildStaticDataColumns = function(widget, columns) {
        var widgetColumns = widget.fields;
        var staticData = {};
        if (widgetColumns) {
            widgetColumns.forEach(function(column) {
                if (columns[column.name] && column.dataSource && column.dataSource.data) {
                    staticData[column.name] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
                else if (columns[column.valueFieldOnReport] && column.dataSource && column.dataSource.data) {
                    staticData[column.valueFieldOnReport] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
            });

            if (Object.keys(staticData).length) {
                for (var data in staticData) {
                    staticData[data] = staticData[data].reduce(function(o1, o2) {
                        for (var key in o2) {
                            o1[key] = o2[key];
                        }
                        return o1;
                    }, {});
                }
            }
        }

        return staticData;
    };

    /**
     * Tests if the column size has a measurement unit.
     *
     * @param {String|Number} columnSize - The column size to be tested.
     *
     * @return {Boolean} Tells if the column size has a measurement unit.
     */
    function columnSizeHasUnity(columnSize) {
        return (/[\%px]/g).test(columnSize.toString());
    }

    /**
     * Build a report's group based on the given Field.
     *
     * @param {Field} column - The Field to be based on the cration of the column.
     *
     * @return {Object} Report's group.
     */
    var buildGroup = function(column) {
        return {
            name: column.name,
            field: column.name,
            isNumber: column.isNumber,
            descending: column.descending,
            format: getFormatOfColumn(column),
            internalSorts: column.internalSorts,
            header: {
                label: column.headerLabel || ScreenService.i18n(column.label),
                displayfield: column.name
            }
        };
    };

    /**
     * Build the metadata of the fields grouped.
     *
     * @param {Widget} widget - Widget used to create the grouped metadata.
     * @param {Array.<String>} [customGroups] - List of the field names that are grouped.
     *
     * @return {Array.<Object>} Metadata of grouped columns.
     */
    var _buildMetaDataGroups = function(widget, customGroups) {
        var buildedGroups = [];
        var groups = widget.groups;
        if (customGroups) {
            groups = customGroups.map(function(name) {
                return widget.getField(name);
            });
        }
        buildedGroups = groups.map(function(group) {
            return buildGroup(group);
        });
        return buildedGroups;
    };

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function getFieldFilterDefaultTextWithoutValue(text) {
        return ScreenService.i18n(text) + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function likeIFieldText(value, text) {
        return ScreenService.i18n(text) + ' \'' + value.replace(/%/g, '') + '\' case sensitive.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown in the header.
     */
    function valueFieldText(filter, value, text) {
        if (value[0] === "%" && value[value.length - 1] === "%") {
            value = value.substring(1, value.length - 1);
        }
        else {

            if (value[0] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(1, value.length);
                    text = "ends with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(1, value.length);
                    text = "does not ends with";
                }
            }
            if (value[value.length - 1] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "starts with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "does not starts with";
                }
            }
        }
        return ScreenService.i18n(text) + ' ' + value + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function oneOrMoreValuesFieldText(value, text) {
        if (Array.isArray(text)) {
            return ScreenService.i18n(text[0]) + ' ' + value[0] + ' ' + ScreenService.i18n(text[1]) + ' ' + value[1] + '.';
        }

        return ScreenService.i18n(text) + ' ' + value.join(', ') + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} filter- Filter used to create the report.
     * @param {Object} value - Value of the filter.
     * @param {String} text  - Text used to create the header of the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    var getFieldFilterDefaultText = function(filter, value, text) {
        if (Array.isArray(value)) {

            return oneOrMoreValuesFieldText(value, text);
        }

        if (filter.operator === "LIKE_I") {
            return likeIFieldText(value, text);
        }

        if (!value) {
            return getFieldFilterDefaultTextWithoutValue(text);
        }

        return valueFieldText(filter, value, text);


    };

    /**
     * Checks if the filter related to the given item should be displayed on the header of
     * the report.
     *
     * @param  {Field} field -  Field to be filter to be tested.
     *
     * @return {Boolean} Boolean telling if the filter related to the field should be displayed.
     */
    function filterIsVisibleOnHeader(field) {
        return field && field.isVisibleOnReport;
    }

    /**
     * Builds the metadata of the filter.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The metadatas of each filter in the widget.
     */
    var _buildMetaDataFilter = function(widget) {
        var filterData = [];
        if (widget.dataSourceFilter) {
            widget.dataSourceFilter.forEach(function(item) {
                var field = widget.getField(item.name);
                if (filterIsVisibleOnHeader(field)) {
                    filterData.push({
                        label: field.label,
                        value: ZhUtil.evaluate(getFieldFilterValue(field, item))
                    });
                }
            });
        }

        return filterData;
    };

    /**
     * Retrieves the image from the url and converts it to base64
     *
     * @param {String}   url    - The source of the image
     * @param {Function} widget - Widget where the report is being generated.
     */
    function convertImageToB64(url, callback) {
        var xhr = new XMLHttpRequest();

        xhr.onload = function() {
            var reader = new FileReader();
            reader.onloadend = function() {
                callback(reader.result);
            };
            reader.readAsDataURL(xhr.response);
        };

        xhr.open('GET', url);
        xhr.responseType = 'blob';
        xhr.send();
    }

    /**
     * Fetches the images if they exist.
     *
     * @param {Function} callback  - The callback to be executed after all the images where fetched.
     * @param {Widget}   widget    - Widget where the report is being generated.
     */
    function getAllImages(container, callback) {
        var imgs = {};
        var imgSrc = ['reportClientLogo', 'reportProductLogo', 'reportEnterpriseLogo'];
        var index = 0;

        if(!container[imgSrc[0]] && !container[imgSrc[1]] && !container[imgSrc[2]]){
            callback({});
            return;
        }

        imgSrc.forEach(function(imgName){
            if(container[imgName]){
                index++;
                convertImageToB64(container[imgName], function(imageData){
                    imgs[imgName] = imageData;
                    index--;

                    if(index === 0){
                        callback(imgs);
                    }
                });
            }
        });
    }

    /**
     * Builds the widget metadata.
     *
     * @param {Object} [customData]       - Customization data for the generation of the report.
     * @param {Widget} sourceWidget       - Widget where the report is being generated.
     * @param {String} [reportType='PDF'] - The type of the report.
     *
     * @return {Object} The widget's metadata for the generation of the report.
     */
    var buildMetaData = function(customData, sourceWidget, reportType, callback) {
        var widget = sourceWidget;
        var columns = _buildMetaDataColumns(widget, customData.columns, reportType, customData.groups);
        var staticColumns = _buildStaticDataColumns(widget, columns);
        var widgetLabel = ScreenService.i18n(widget.label);

        getAllImages(templateManager.project, function(images) {
            var widgetMetaData = {
                name: widget.name,
                title: customData.name || widgetLabel,
                reportName: customData.reportName || widget.name,
                reportClientLogo: images.reportClientLogo,
                reportProductLogo: images.reportProductLogo,
                reportEnterpriseLogo: images.reportEnterpriseLogo,
                widgetLabel: widgetLabel,
                version: '1.0',
                clientLogo: templateManager.project.reportClientLogo,
                productLogo: templateManager.project.reportProductLogo,
                enterpriseLogo: templateManager.project.reportEnterpriseLogo,
                columns: columns,
                staticData: staticColumns,
                groups: _buildMetaDataGroups(widget, customData.groups),
                filter: _buildMetaDataFilter(widget),
                xlsDefaultType: widget.xlsDefaultType,
                orientation: widget.reportOrientation || 'PORTRAIT'
            };

            callback(widgetMetaData);
        });
    };


    /**
     * Tells if the given field needs to have it's value decoded.
     *
     * @param {Field} field - Fild to test if it's value need to be decoded.
     *
     * @return {Boolean} Tells if the field's value need to be decoded
     */
    var shouldUseDecodeValueField = function(field) {
        var masksTypesToAvoid = ['currency', 'hour', 'float', 'percent', 'fix'];
        return !field.mask || masksTypesToAvoid.indexOf(field.mask.type) == -1;
    };

    /**
     * Get the grouped columns from the given metadata of grouped fields.
     *
     * @param {Object} groupsMetadata - Metadata of the grouped on the report.
     *
     * @return {Array.<Field>} The fields of the group.
     */
    var getGroupedColumns = function(groupsMetadata) {
        var columns = [];
        groupsMetadata.forEach(function(group) {
            columns.push(group.field);
        });

        return columns;
    };

    /**
     * Format the values of the dataset based on the field of the widget configured
     * the metaData.
     *
     * @param {Array.<Object>} dataSet  - The widget's dataset.
     * @param {Object}         metaData - Metadata of the report.
     * @param {Widget}         widget   - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The given dataset formatted for be used on the report.
     */
    var formatValuesForReport = function(dataSet, metaData, widget) {
        var columns = Object.keys(metaData.columns);
        columns = columns.concat(getGroupedColumns(metaData.groups));

        return dataSet.map(function(row) {
            if(row.__groupHeader || row.__groupFooter) return row;

            var formattedRow = {};
            columns.forEach(function(column) {
                var field = widget.getField(column);
                if (field && shouldUseDecodeValueField(field)) {
                    formattedRow[column] = DecodeValueFieldService.model(field, row[column]);
                }
                else {
                    formattedRow[column] = row[column];
                }
            });

            return formattedRow;
        });
    };

    /**
     * Builds the dataSet based on the given widget,
     *
     * @param {Object} metaData   - The metadata of the report.
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Array.<Object>}  Dataset from the widget formatted to the report.
     */
    var buildDataSet = function(metaData, widget, reportType) {
        return loadAllData(widget, reportType).then(function(dataSet) {
            return isPdfFormat(reportType) ? formatValuesForReport(dataSet, metaData, widget) : dataSet;
        });
    };

    /**
     * Get the datasorce's name without the initial '/'.
     *
     * @param {DataSource} dataSource - Widget's dataSource to get the name.
     *
     * @return {String} Datasorce name without the initial '/'.
     */
    function getDataSourceName(dataSource) {
        return dataSource.name.split('/').pop();
    }

    /**
     * Load all data from the widget's dataSource.
     *
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Promise.<Array>} Promise that will be resolved with the widget's datasource.
     */
    function loadAllData(widget, reportType) {
        var dataSource = widget.dataSource;
        var storageStrategy = dataSource.getStorageStrategy();
        var result;

        if (storageStrategy) {
            result = storageStrategy.getDataSource(dataSource.name, widget.getDataSourceFilter(), null).then(function(dataSet) {
                if (Util.isObject(dataSet)) {
                    dataToReturn = dataSet.dataset[getDataSourceName(dataSource)];
                }
                else {
                    dataToReturn = dataSet;
                }

                return isPdfFormat(reportType) ? widget.getUpdatedGroupData(dataToReturn) : dataToReturn;
            });
        }
        else {
            result = ZHPromise.when(widget.groupedData);
        }

        return result;
    }

    /**
     * Build the path where the file was created.
     *
     * @param {String} fileName - Name of the created file.
     *
     * @return {Object} Object with the ful path of the created file.
     */
    var buildFilePath = function(fileName) {
        var reportsUrl = templateManager.project.reportsUrl;
        return {
            path: reportsUrl + '/reports/' + fileName
        };
    };

    /**
     * Builds a successs callback.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccess = function(callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePath(data.messages[0].message));
            }
        };
    };

    /**
     * Build the file path of the generated file for the dataExporter
     *
     * @param {String} fileName - Name of the generated file.
     *
     * @return {Object} Object with the fukk path of the generated file.
     */
    var buildFilePathForDataExporter = function(exporterUrl, fileName) {
        return {
            path: exporterUrl + fileName
        };
    };

    /**
     * Builds a successs callback for the dataExporter.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccessForDataExporter = function(exporterUrl, callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePathForDataExporter(exporterUrl, data.messages[0].message));
            }
        };
    };

    /**
     * Returns the uri from a given report type.
     *
     * @param {String} reportType - Report type that will be used to get the uri.
     *
     * @return {String} - Uri from the given report type.
     */
    var getReportUri = function(reportType) {
        return reportConfig[reportType].uri;
    };

    /**
     * Format the dataset for the XLS and CSV files.
     *
     * @param {Object} columns - The metadata of each column of the report.
     * @param {Array}  dataSet - The datasource's data of the widget.
     *
     * @return {Array.<Object>} The dataSet to the XLS and CSV files
     */
    var formatDataSet = function(columns, dataSet) {
        var columnsName = Object.keys(columns);
        var rowValues = [];
        var formatedDataSet = [];

        dataSet.forEach(function(row) {
            columnsName.forEach(function(column) {
                rowValues.push(row[column]);
            });

            formatedDataSet.push(rowValues);
            rowValues = [];
        });

        return [columnsName].concat(formatedDataSet);
    };

    /**
     * Build the dataSet to be used on the backend to generate the report.
     *
     * @param {String} reportType - Defines what report type will be generate.
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} dataSet - Data used to create the reports.
     *
     * @return {Object} - Builded dataSet.
     */
    var buildDataSetToReportRow = function(reportType, metaData, dataSet) {
        if(reportType == reportConfig.PDF.type) {
            return dataSet.map(ZhUtil.clearRow);
        } else {
            return formatDataSet(metaData.columns, dataSet);
        }
    };

    /**
     * Build the reportRow to be used on the backend to generate the report.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} reportRow - Builded reportRow.
     */
    var buildReportRow = function(metaData, reportRowDataSet, widget) {
        var reportRow = {};
        reportRow.metaData = JSON.stringify(metaData);
        reportRow.dataSet = JSON.stringify(reportRowDataSet);
        reportRow.filter = JSON.stringify(widget.dataSourceFilter);
        return reportRow;
    };

    /**
     * Build the params to be used on the request.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} - Builded params.
     */
    var buildParamsToRequest = function(metaData, reportRowDataSet, widget) {
        return {
            'requestType': 'Row',
            'row': buildReportRow(metaData, reportRowDataSet, widget)
        };
    };

    /**
     * Build the request to generate the report.
     *
     * @param {String} reportType         - Defines what report type will be generate.
     * @param {Object} metaData           - Defines the report's structure.
     * @param {Object} reportRowDataSet   - Data used to create the reports.
     * @param {function} callback         - Callback called after sucess.
     * @param {function} callbackError    - Callback called after error.
     * @param {Widget} widget             - Widget to report.
     *
     * @return {Promisse} Request's promisse.
     */
    var buildRequest = function(reportType, metaData, reportRowDataSet, callback, callbackError, widget) {
        var uri = getReportUri(reportType);
        var params = buildParamsToRequest(metaData, reportRowDataSet, widget);
        var callbackSuccess = buildCallbackOfSuccess(callback);
        var requestType = 'post';
        var reportsUrl = templateManager.project.reportsUrl;

        return restEngine.request(uri,params,callbackSuccess,requestType,callbackError,reportsUrl);
    };

    /**
     * Process to be executed after the request.
     *
     * @param {Object}      response        - Request result.
     * @param {Promisse}    deferred        - Promisse that defines the result of report generation.
     * @param {String}      reportType      - Defines what report type will be generated.
     *
     */
    var afterRequest = function(widget, response, deferred, reportType) {
        if (response) {
            var file = buildFilePath(response.messages[0].message);

            deferred.resolve(file);

            if(widget.afterOpenreport){
                widget.afterOpenreport();
            }
        }
        else {
            deferred.reject();
            publishErrorEvent();
        }
    };

    /**
     * Generates the report making a requests to the server to generate the file.
     *
     * @param {Object}          metaData        - Metadata for the construction of the report.
     * @param {Array.<Object>}  dataSet         - The widget's dataset.
     * @param {Function}        callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}        callbackError   - Callback that will be executed if the request failed.
     * @param {String}          reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateReport = function(widget, metaData, dataSet, callback, callbackError, reportType) {
        var deferred = ZHPromise.defer();
        var reportRowDataSet = buildDataSetToReportRow(reportType, metaData, dataSet);

        buildRequest(reportType, metaData, reportRowDataSet, callback, callbackError, widget).then(function(response) {
            afterRequest(widget, response, deferred, reportType);
        }).catch(function(response) {
            deferred.reject(response);
            publishErrorEvent();
        });

        return deferred.promise;
    };

    /**
     * Returns the params to make the request.
     *
     * @param {Object} actionControl - The widget's actionControl.
     * @param {Widget} widget        - Widget where the report is being generated.
     * @param {Object} metaData      - Metadata for the construction of the report.
     *
     * @return {Object} Params to the request.
     */
    function getParams(actionControl, widget, metaData){
        var params = {
            'requestType': 'FilterData',
            'filter': widget.dataSourceFilter,
            'metaData': metaData
        };
        if(actionControl.dataSourceName){
            params.dataSourceName = actionControl.dataSourceName;
        } else if(actionControl.collectionName){
            params.collectionName = actionControl.collectionName;
        } else {
            var reportRow = {};
            reportRow.metaData = JSON.stringify(metaData);
            reportRow.dataSet = JSON.stringify(widget.dataSource.data);
            reportRow.filter = JSON.stringify(widget.dataSourceFilter);

            params = {
                'requestType': 'Row',
                'row': reportRow,
            };
        }
        return params;
    }

    /**
     * Generates the report making only one request to the server generate the file.
     *
     * @param {Object}   metaData           - Metadata for the construction of the report.
     * @param {Object}   actionControl      - The widget's actionControl.
     * @param {Object}   widget             - Widget where the report is being generated.
     * @param {Function} callback           - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function} callbackError      - Callback that will be executed if the request failed.
     * @param {String}   [reportType='PDF'] - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateServerSideReport = function(metaData, actionControl, widget, callback, callbackError, reportType) {
        var exporterUrl = templateManager.project.exporterUrl;
        var deferred = ZHPromise.defer();
        var uri = getReportUri(reportType);
        var params = getParams(actionControl, widget, metaData);

        restEngine.request(uri, params, buildCallbackOfSuccessForDataExporter(exporterUrl, callback), 'post', callbackError, exporterUrl)
            .then(
                function(response) {
                    afterRequest(widget, response, deferred, reportType);
                })
            .catch(
                function(response) {
                    deferred.reject(response);
                    publishErrorEvent();
                }
            );

        return deferred.promise;
    };

    /**
     * Publish a new event. Lauched after the report failed.
     */
    var publishErrorEvent = function() {
        eventAggregator.publish('afterReportError');
    };

    /**
     * Creates a metaData for the translate of fixed words on the report.
     *
     * @param {Object} metaData - Metada of the report.
     *
     * @return {Object} The metadata with the configuration of the i18n
     */
    var _i18nMetaData = function(metaData) {
        var words = {
            'Page': ScreenService.i18n('Page'),
            'Report': ScreenService.i18n('Report'),
            'Version': ScreenService.i18n('Version')
        };

        metaData.words = words;
        metaData.lang = templateManager.lang;

        Object.keys(metaData.columns).forEach(function(key) {
            var column = metaData.columns[key];

            column.description = ScreenService.i18n(column.description);
        });

        metaData.groups.forEach(function(group) {
            Object.keys(group).forEach(function(key) {
                var grp = group[key];
                if (grp) {
                    grp.label = ScreenService.i18n(grp.label);
                }
            });
        });

        return metaData;
    };

    /**
     * Get the action control's configuration.
     *
     * @param  {Widget} widget     - Widget where the report is being generated.
     * @param  {String} reportType - The type of the report.
     *
     * @return {Object | Boolean} A object containing the configuration of the floating action or true.
     */
    var getActionControl = function(widget, reportType) {
        return Util.getNestedProperty(widget, ['floatingControl', reportConfig[reportType].action]);
    };

    /**
     * Generates the report.
     *
     * @param {Object}      customData      - Informations to customize the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Function to be executed after the file be generated, reciving the name of the generated file.
     * @param {Function}    callbackError   - Function to be executed if happen a error while generating the file.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created o rejected.
     */
    this.generateReport = function generateReport(customData, sourceWidget, callback, callbackError, reportType) {
        if (Util.getNestedProperty(sourceWidget, ['dataSource', 'data', 'length'])) {
            customData = customData || {};
            checkConfiguration();
            var widget = sourceWidget;
            var promise;

            buildMetaData(customData, widget, reportType, function(metaDataResolved) {
                var metaDataInternationalized = _i18nMetaData(metaDataResolved);

                triggerBeforeGenerateReport(widget, metaDataInternationalized);

                var actionControl = getActionControl(widget, reportType);

                if (typeof actionControl === "object" && actionControl.runAt == 'server') {
                    promise = _generateServerSideReport(metaDataInternationalized, actionControl, widget, callback, callbackError, reportType);
                }
                else {
                    promise = buildDataSet(metaDataInternationalized, widget, reportType).then(function(dataSet) {
                        return _generateReport(widget, metaDataInternationalized, dataSet, callback, callbackError, reportType);
                    });
                }
            });

            return promise;
        }
        else {
            ScreenService.errorNotification("Can't create a report without data!");
        }
    };

    /**
     * Returns a callback to be executed if the report is created succefully.
     *
     * @param {Function} callback - Function executed after the test.
     *
     * @return {Function} Callback that tests if the screen is able to open the popup creating a fake popup and, warning the user if the windows's popups are blocked.
     */
    var _previewCallback = function(callback) {
        return function(data) {
            var popup_window = window.open(data.path, '_blank');
            if (!popup_window) {
                ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
            }
            if (callback) {
                callback(data);
            }
        };
    };

    /**
     * Creates the report and open it rigth after.
     *
     * @param {Object}      customData      - Custom data for the generation of the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}    callbackError   - Callback that will be executed if the request failed.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created or rejected
     */
    this.openReport = function openReport(customData, sourceWidget, callback, callbackError, reportType) {
        return this.generateReport(customData, sourceWidget, _previewCallback(callback), callbackError, reportType);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhReports', ZhReports);
});


// FILE: src/ZhReportsDirective.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

function ReportsDirective(ApplicationContext){
    return {
        priority: 2,
        link: function($scope) {
            /**
             * Generate the report of the given widget.
             * @param  {Widget}   widget  Widget where the report is being generated.
             * @param  {String}   type    The type of the report.
             */
            $scope.generateReport = function(widget, type) {
                ApplicationContext.ZhReports.openReport(widget.reportCustomData, widget, undefined, undefined, type);
            };
        }
    };
}

ZeedhiDirectives.directive(
    'zhFloatingControl', ['ApplicationContext', ReportsDirective]
);


// FILE: bower_components/devtools/dist/developer-enrichment.min.js
/*! 15-09-2017 */
var UtilDevTools=function(){};UtilDevTools.getZeedhiTypes=function(){var zeedhiTypes=["__metadata","widgets","fields"];if(!window.inCustomizationMode){zeedhiTypes.push("actions")}return zeedhiTypes};UtilDevTools.getIgnoredZeedhiTypes=function(){var zeedhiTypes=[];if(window.inCustomizationMode){zeedhiTypes.push("actions");zeedhiTypes.push("__original")}return zeedhiTypes};UtilDevTools.getZeedhiTypesSubitems=function(){var subitemsPropertiesMap={container:["widgets"],widget:["widgets","fields"],field:[],action:[],groupMenu:[],menu:[]};if(!window.inCustomizationMode){subitemsPropertiesMap.widget.push("actions")}return subitemsPropertiesMap};UtilDevTools.buildInDeveloperModeMethod=function(method){return function buildInDeveloperModeMethod(){var obj=this,args=arguments;if(window.inDeveloperMode||window.inCustomizationMode){method.apply(obj,args)}}};UtilDevTools.buildInCustomizationModeMethod=function(method){return function buildInCustomizationModeMethod(){var obj=this,args=arguments;if(window.inCustomizationMode){method.apply(obj,args)}}};UtilDevTools.buildDevToolsWindowMethod=function(method){return function buildDevToolsWindowMethod(){var obj=this,args=arguments,DevToolsWindow=window.parent?window:null;if(window.parent==window){DevToolsWindow=window.opener?window:null}if(DevToolsWindow){method.apply(obj,args)}}};UtilDevTools.buildMainWindowMethod=function(method){return function buildMainWindowMethod(){var obj=this,args=arguments,MainWindow=window.parent?window.parent:window;if(window.parent==window){MainWindow=window.opener?window.opener:window}if(MainWindow===window){method.apply(obj,args)}}};UtilDevTools.getDevToolsWindow=function(){var DevToolsWindow;if(window.parent==window){DevToolsWindow=window.opener?window:null}else{DevToolsWindow=window.parent?window:null}return DevToolsWindow};UtilDevTools.getMainWindow=function(){var MainWindow;if(window.parent==window){MainWindow=window.opener?window.opener:window}else{MainWindow=window.parent?window.parent:window}return MainWindow};function DTRenderService(metaDataFactory,templateManager,ApplicationContext){var DASHBOARD="container/dashboard.html";var typeMap={fields:{factoryMethod:metaDataFactory.fieldFactory},widgets:{factoryMethod:metaDataFactory.widgetFactory},actions:{factoryMethod:metaDataFactory.actionFactory}};var getWindowController=function(){return ApplicationContext.WindowController};var factoryObject=function(type,obj,objParent){if(typeMap[type]){obj=typeMap[type].factoryMethod(obj,objParent)}return obj};var getObjDetails=function(path,obj){var result={};if(path.length==2){var type=path[0];var position=path[1];result.parent=obj;result.position=position;result.objValue=obj[type][position]}else{var key=path.shift();var newObj=obj[key];result=getObjDetails(path,newObj)}return result};var getLastComponentIndex=function(parts){var index;if(parts.lastIndexOf("fields")>=0){index=parts.lastIndexOf("fields")}else if(parts.lastIndexOf("actions")>=0){index=parts.lastIndexOf("actions")}else{index=parts.lastIndexOf("widgets")}return index};var needToRefactory=function(lastIndexOfChangedComponent,option,parts){var result=false;var hasChanges=lastIndexOfChangedComponent!==-1;var isDeleted=option=="delete";var componentWasDeleted=isDeleted&&lastIndexOfChangedComponent!=parts.length-2;result=hasChanges&&(!isDeleted||componentWasDeleted);return result};var getPartsToComponets=function(lastIndexOfChangedComponent,parts){return parts.slice(0,lastIndexOfChangedComponent+2)};var replaceComponentAtPosition=function(parent,component,position){parent.splice(position,1,component);return parent};var updateContainerActions=function(){templateManager.container.actions=null;templateManager.container.prepareActions()};var updateContainer=function(path,option){var parts=path.split("/");parts.shift();var lastIndexOfChangedComponent=getLastComponentIndex(parts);var partsToComponent=getPartsToComponets(lastIndexOfChangedComponent,parts);var componentCollection=partsToComponent[lastIndexOfChangedComponent];if(needToRefactory(lastIndexOfChangedComponent,option,parts)){var preparedObj=getObjDetails(partsToComponent,templateManager.container);var parentToFactory=preparedObj.parent;var parentArray=parentToFactory[componentCollection]||[];var types=Object.keys(typeMap);if(types.indexOf(componentCollection)!==-1){if(componentCollection=="widgets"){parentToFactory=templateManager.container}var obj=factoryObject(componentCollection,preparedObj.objValue,parentToFactory);replaceComponentAtPosition(parentArray,obj,preparedObj.position)}}if(componentCollection=="actions"){updateContainerActions()}var needRenderContainer=templateManager.container.template==DASHBOARD;if(needRenderContainer){templateManager.container.__isVisited=true}};var visitorDiffCallback=function(val){if(val instanceof Widget){val.__isVisited=true}};var resetVisitedWidget=function(widget){if(widget.__isVisited&&widget.onRender){widget.onRender()}widget.__isVisited=false};var updateEditor=function(keysDiffs){getWindowController().clearInvalidItems();getWindowController().updateActiveItemInEditor(keysDiffs)};var updateTemplateManagerContainer=function(containerComming){var fileName=templateManager.container.__metadata.fileName;var indexPath=templateManager.container.__metadata.__indexPath;containerComming.fileName=fileName;containerComming.__indexPath=indexPath;ShowInDevTools.setTree(containerComming)};this.renderContainer=UtilDevTools.buildMainWindowMethod(function(newContainer){updateTemplateManagerContainer(newContainer);var diffs=diff.getDiff(templateManager.container.__metadata,newContainer);var keysDiffs=Object.keys(diffs);templateManager.container.__metadata=Util.clone(newContainer);diff.applyDiff(templateManager.container,diffs,visitorDiffCallback);keysDiffs.reverse().forEach(function(path){if(path!="/fileName"){var diffOp=diffs[path].operation;updateContainer(path,diffOp)}});updateEditor(keysDiffs);if(templateManager.container.__isVisited&&templateManager.container.onRender){templateManager.container.onRender();templateManager.container.__isVisited=false}templateManager.container.eachWidget(resetVisitedWidget)});var searchMenu=function(element,elementHistory){return element.groupMenu.find(function(menu){return menu.name==elementHistory})};var insertHistoryMenuInProject=function(elementToInsert,element){var elementHistory=element.historyMenuGroup;var insertHistory=elementToInsert.historyMenuGroup;if(elementHistory&&insertHistory){var historyMenu=searchMenu(element,elementHistory);historyMenu=historyMenu.menus.find(function(menu){return menu.historyMenu});var menu=searchMenu(elementToInsert,insertHistory);menu.menus.push(historyMenu)}};var setPropertiesInProject=function(currentProject,newProject,projectReceived){var indexPath=currentProject.__indexPath;newProject.__indexPath=indexPath;templateManager.project=metaDataFactory.projectFactory(newProject);templateManager.project.__indexPath=indexPath;templateManager.project.__menuMetadata=projectReceived.groupMenu;templateManager.updateTitle(currentProject.label);templateManager.updateTemplate()};function updateMetadataGroupMenu(devToolsWindow,newGroupMenu){var menuContainers=devToolsWindow.groupMenu.containers;menuContainers.zeedhi_project.groupMenu=Util.clone(newGroupMenu);menuContainers.__metadata.groupMenu=menuContainers.zeedhi_project.groupMenu}function insertDiffsInMenu(zeedhi_project,newGroupMenu,devToolsWindow){var diffs=diff.getDiff(zeedhi_project.groupMenu,newGroupMenu);diff.applyDiff(devToolsWindow.groupMenu.groupMenu,diffs,visitorDiffCallback);var insertGroupMenu=Object.keys(diffs).map(function(value){return"groupMenu"+value});updateEditor(insertGroupMenu)}function updateGroupMenu(newGroupMenu){var devToolsWindow=UtilDevTools.getMainWindow().getDevToolsWindow();var zeedhi_project=templateManager.containers.zeedhi_project;updateMetadataGroupMenu(devToolsWindow,newGroupMenu);insertDiffsInMenu(zeedhi_project,newGroupMenu,devToolsWindow);zeedhi_project.groupMenu=newGroupMenu}this.updateProjectConfig=UtilDevTools.buildMainWindowMethod(function(projectChanged){var currentProject=templateManager.project;var newProject=Util.clone(projectChanged);if(newProject.id==currentProject.id){insertHistoryMenuInProject(newProject,currentProject);updateGroupMenu(projectChanged.groupMenu);setPropertiesInProject(currentProject,newProject,projectChanged)}})}function SocketService(ZhSocketFactory,ZHPromise,ScreenService,templateManager,restEngine,ApplicationContext,DevToolsService,DTMessages,DTRenderService){var self=this;var CLIENT="browser";var NAMESPACE="zh_developer";var WSKEY="you can create!";var defaultHost="ws://ws.zeedhi.com";var isConnected=false;var webSocket=null;var clientEmail=null;var inDeveloperMode=false;var ERROR_TYPE="error";var SUCCESS_TYPE="success";var pendingChanges={};var getWindowController=function(){return ApplicationContext.WindowController};var getLoginData=function(email){var login={data:{}};login.id=(new Date).getTime();login.sender=CLIENT;if(email){var defer=ZHPromise.defer();clientEmail=login.login=email;defer.resolve(login);return defer.promise}else{return restEngine.requestMetaData(".developerConfig.json").then(function(obj){clientEmail=login.login=obj.data.email;DevToolsService.setLocalEmail(login.login);return login})}};var formatErrorMessage=function(path,code){var message="Error while writing "+path;if(DTMessages.FILE_ERROR_MAP[code]){message+=" : "+DTMessages.FILE_ERROR_MAP[code]}return message};var getContainerComming=function(data){var containerComming;try{containerComming=JSON.parse(data)}catch(e){var message=~data.indexOf("Syntax error")?data:e;ScreenService.notificationMessage(message,"error",5e3)}return containerComming};var isJsFile=function(fileName){return fileName.indexOf(".json")==-1&&fileName.indexOf(".js")!=-1};var sendMessageToServer=function(messageToSend,ignoreResponse){var data=messageToSend.data||{};var dataId=messageToSend.id;messageToSend.room=clientEmail;messageToSend.namespace=NAMESPACE;data.id=dataId;data.namespace=NAMESPACE;data.key=WSKEY;webSocket.send(JSON.stringify(messageToSend));if(ignoreResponse){return ZHPromise.when()}else{var defer=ZHPromise.defer();pendingChanges[dataId]={defer:defer,id:dataId};setTimeout(function(){if(pendingChanges[dataId]){defer.reject({path:"",code:"ETIMEDOUT"})}},15e3);return defer.promise}};var getPreparedPathname=function(){var pathname=window.location.pathname;var lastSlash=pathname.lastIndexOf("/");pathname=~lastSlash?pathname.slice(0,lastSlash):pathname;return pathname};var getMessageIndexPath=function(indexPath,pathname){if(pathname[0]=="/"){pathname=pathname.replace("/","")}indexPath=indexPath.replace(pathname,"");return indexPath};var getFileName=function(data){return data.fileName||data.__indexPath.split("/").pop()};var getDataToSend=function(data,mode){var eventName="changesByBrowser";var pathname=getPreparedPathname();var messageData={mode:mode||"json",fileName:getFileName(data),projectPath:pathname,indexPath:data.__indexPath,sender:CLIENT,email:DevToolsService.getLocalEmail(),project:DevToolsService.getProject(),event:eventName};delete data.fileName;delete data.__indexPath;messageData.data=mode=="files"?data.data:data;messageData.projectPath=pathname;messageData.indexPath=getMessageIndexPath(messageData.indexPath,pathname);var dataToSend={id:(new Date).getTime(),data:messageData,event:eventName};return dataToSend};var showNotificationInDevTools=function(messageToShow,type){var windowController=getWindowController();if(windowController&&windowController.isDevToolsOpened()){windowController.showNotificationInDevTools(messageToShow,type)}};var showMessageInWindows=function(messageToMain,messageToDevTools,type,showDialog){if(type!=ERROR_TYPE&&!showDialog){ScreenService[type+"Notification"](messageToMain,type)}else{ScreenService.showMessage(messageToMain,type)}showNotificationInDevTools(messageToDevTools,type)};var getBasicDataToSend=function(data,eventName){data=Util.isObject(data)?data:{data:data};data.sender=CLIENT;data.event=eventName;data.project=DevToolsService.getProject();data.email=DevToolsService.getLocalEmail();var dataToSend={id:(new Date).getTime(),event:eventName,data:data};return dataToSend};var sendDataLogin=function(login){setTimeout(function(){var eventsToStart=["createNamespace","createRoom","joinRoom"];eventsToStart.forEach(function(event){login.data.event=login.event=event;sendMessageToServer(login,true)});login.data.event=login.event="loginClient";sendMessageToServer(login).then(function(){ScreenService.successNotification(DTMessages.LOGIN_ACCOMPLISHED);sendPathToWatch(login);self.updateScriptTreeWhenReloaded()}).catch(function(err){ScreenService.showMessage(DTMessages.LOGIN_NOT_ACCOMPLISHED,"error")})},1e3)};var sendPathToWatch=function(dataToSend){var pathname=window.location.href.replace(window.location.origin,"");pathname=pathname.slice(0,pathname.indexOf(window.location.hash));pathname=pathname.slice(0,pathname.lastIndexOf("/"));dataToSend.data.projectPath=pathname;dataToSend.data.event=dataToSend.event="sendPathToWatch";sendMessageToServer(dataToSend,true)};var getHost=function(host){if(!host.match("ws://")||!host){host=defaultHost}return host};var getEmailByClient=function(client){if(client&&client.email){client=client.email}return client};var findFiles=function(data,callback){data=data||"new";var eventName="findFiles";var dataToSend=getBasicDataToSend(data,eventName);setTimeout(function(){sendMessageToServer(dataToSend).then(function(responseData){if(callback){callback(responseData)}}).catch(function(){ScreenService.showMessage(DTMessages.LOGIN_NOT_ACCOMPLISHED,"error")})},1e3)};var ResponseHandler=function(){this.receiveChanges=UtilDevTools.buildMainWindowMethod(function(data){var containerComming=getContainerComming(data);if(containerComming){if(templateManager.container&&templateManager.container.name===containerComming.name){DTRenderService.renderContainer(containerComming)}else if(containerComming.zeedhi_project){var project=containerComming.zeedhi_project;DTRenderService.updateProjectConfig(project)}else if(containerComming.fileName&&isJsFile(containerComming.fileName)){getWindowController().reloadMainWindow()}}});this.loginClient=function(data){data.success=true;this.response(data)};this.noOtherClientsConnected=function(){Object.keys(pendingChanges).forEach(function(pendingChangeId){pendingChanges[pendingChangeId].reject({});delete pendingChanges[pendingChangeId]});ScreenService.alertNotification(DTMessages.LOGIN_NO_OTHER_CLIENTS)};this.response=function(data){var responseId=data.id;var responseData=data.data||{};var method=data.isError?"reject":"resolve";if(pendingChanges[responseId]){responseData.id=responseId;pendingChanges[responseId].defer[method](responseData);delete pendingChanges[responseId]}}};this.updateScriptTreeWhenReloaded=function(){getWindowController().updateScriptTree()};this.reloadFiles=function(){findFiles("reload",getWindowController().setScriptsFiles)};this.loadFiles=function(){findFiles("new",getWindowController().setFilesToLocalStorage)};this.newFile=UtilDevTools.buildMainWindowMethod(function(message){if(message){var fileName=message.fileName.replace(".json","");var dataToSend={id:(new Date).getTime(),event:"newFile",data:{data:message,sender:CLIENT,email:DevToolsService.getLocalEmail(),project:DevToolsService.getProject(),event:"newFile"}};sendMessageToServer(dataToSend).then(function(){var messageToShow=DTMessages.CONTAINER_WAS_CREATED;showMessageInWindows(messageToShow,messageToShow,SUCCESS_TYPE);openWindow(fileName)}).catch(function(err){showMessageInWindows(formatErrorMessage(err.path,err.code),DTMessages.CHECK_MAIN_WINDOW,ERROR_TYPE)})}});this.changeInClient=UtilDevTools.buildMainWindowMethod(function(data,mode){var isInOtherPackage=data.__indexPath.indexOf("http://")!=-1||data.__indexPath.indexOf("https://")!=-1;if(isInOtherPackage){showMessageInWindows(DTMessages.FILE_CANT_SAVE,DTMessages.FILE_CANT_SAVE,ERROR_TYPE)}else{sendMessageToServer(getDataToSend(data,mode)).then(function(){showMessageInWindows(DTMessages.FILE_SAVED_SUCCESSFULLY,DTMessages.FILE_SAVED_SUCCESSFULLY,SUCCESS_TYPE)}).catch(function(err){showMessageInWindows(formatErrorMessage(err.path,err.code),DTMessages.CHECK_MAIN_WINDOW,ERROR_TYPE)})}});this.login=UtilDevTools.buildMainWindowMethod(function(email){getLoginData(email).then(function(login){if(isConnected){sendDataLogin(login)}else{webSocket.retryConnection();sendDataLogin(login)}}).catch(function(err){ScreenService.showMessage(err.status+" : File .developerConfig.json "+err.statusText,"error")});DevToolsService.setProject()});this.connect=UtilDevTools.buildMainWindowMethod(function(client){DevToolsService.inDeveloperModePromise.then(function(definedInDeveloperMode){inDeveloperMode=definedInDeveloperMode;if(inDeveloperMode||client){var email=getEmailByClient(client);webSocket=ZhSocketFactory.create(ResponseHandler,getHost(templateManager.project.developerWebSocketUrl));webSocket.addListener({onOpen:function(){isConnected=true;self.login(email)}},{connectionLost:function(){isConnected=false}})}})});this.connect();window.SocketServiceInstance=this}ZeedhiServices.service("DTMessages",function(ApplicationContext){var instance=new DTMessages;ApplicationContext.registerInstance("DTMessages",instance);return instance});ZeedhiServices.service("DTServerService",function(ApplicationContext,$http,ScreenService,DTMessages,DevToolsService){var instance=new DTServerService($http,ScreenService,DTMessages,DevToolsService);ApplicationContext.registerInstance("DTServerService",instance);return instance});ZeedhiServices.service("DTRenderService",function(ApplicationContext,templateManager,metaDataFactory){var instance=new DTRenderService(metaDataFactory,templateManager,ApplicationContext);ApplicationContext.registerInstance("DTRenderService",instance);return instance});ZeedhiServices.service("DTCustomizationService",function(ApplicationContext,templateManager,ZHPromise,DTServerService,DTMessages,DTRenderService){var instance=new DTCustomizationService(templateManager,ZHPromise,DTServerService,DTMessages,DTRenderService);ApplicationContext.registerInstance("DTCustomizationService",instance);return instance});ZeedhiApp.run(function(ContextRegister,ContainerDecoratorProvider,ZHPromise,templateManager,eventEngine,DevToolsService,DTCustomizationService){DevToolsService.inDeveloperModePromise.then(function(response){if(response.inCustomizationMode){ContainerDecoratorProvider.registerProvider(new DTCustomizationDecorator(ZHPromise,DTCustomizationService));ContextRegister.register("DTComponentCRUD",DTCustomComponentCRUD)}if(response.inDeveloperMode){eventEngine.registerEvent("ContainerOnRender","onRender");eventEngine.registerEvent("WidgetOnRender","onRender");ContainerDecoratorProvider.registerProvider(new ContainerDecoratorDeveloper(ZHPromise,templateManager));DeveloperMenuEnrichment.enrich(templateManager.project);if(!response.inCustomizationMode){ContextRegister.register("SocketService",SocketService);ContextRegister.register("DTComponentCRUD",DTComponentCRUD);ContextRegister.register("FileManagementService",FileManagementService)}}})});ZeedhiServices.service("DevToolsService",function(restEngine,localStorageEngine,templateManager){return new DeveloperService(restEngine,localStorageEngine,templateManager)});ZeedhiServices.service("DeveloperToolsForm",function(ApplicationContext,ScreenService,templateManager,DevToolsService,metaDataFactory){var instance=new DeveloperToolsForm(metaDataFactory,ScreenService,templateManager,DevToolsService);ApplicationContext.registerInstance("DeveloperToolsForm",instance);return instance});ZeedhiServices.service("DeveloperTreeService",function(ApplicationContext,DevToolsService){var instance=new DeveloperTreeService(DevToolsService);ApplicationContext.registerInstance("DeveloperTreeService",instance);return instance});ZeedhiServices.service("ShortcutDeveloperService",function(ApplicationContext,metaDataFactory){var instance=new ShortcutDeveloperService(metaDataFactory);ApplicationContext.registerInstance("ShortcutDeveloperService",instance);return instance});ZeedhiServices.service("ElementToMetadataService",function(ApplicationContext,templateManager){var instance=new ElementToMetadataService(templateManager);ApplicationContext.registerInstance("ElementToMetadataService",instance);return instance});ZeedhiServices.service("ElementFileService",function(ApplicationContext,DevToolsService){var instance=new ElementFileService(DevToolsService);ApplicationContext.registerInstance("ElementFileService",instance);return instance});function DevToolsController($scope,$element,ContextRegister,ScreenService,templateManager,eventAggregator,ShortcutDeveloperService,DevToolsService,DeveloperTreeService){var self=this;var ENTER_KEY=13;var DEVTOOLS_PACKAGE_NAME="devtools";var MainWindow=UtilDevTools.getMainWindow();ContextRegister.register("DevToolsController",this);function hasDevTemplateUrl(templates){return templates.some(function(template){return template.package==DEVTOOLS_PACKAGE_NAME})}function setCursorPositionInActiveItem(){var item=$scope.activeFolder||DevToolsService.getLocalActiveItem();var cursorPosition=$scope.currentEditor?$scope.currentEditor.getCursorPosition():{};DevToolsService.setLocalActiveItem(item.name,item.path,item.mode,item.parent,cursorPosition)}if(!hasDevTemplateUrl(templateManager.templateUrls)){templateManager.templateUrls.push({baseUrl:"./bower_components/devtools/templates/","package":DEVTOOLS_PACKAGE_NAME})}var recoverComponentId=UtilDevTools.buildInCustomizationModeMethod(function(item,state){if(!$scope.inFileMode()){var text=JSON.parse(state.value);text.id=item.id;state.value=JSON.stringify(text)}});this.inCustomizationMode=function(){return templateManager.inCustomizationMode};this.getTemplateURLHeader=function(){var template=this.inCustomizationMode()?"devtools#/dt-lite.html":"devtools#/dt-default.html";return templateManager.getTemplateURL(template)};this.setUnloadedItems=function(data){DeveloperTreeService.setUnloadedItems(data.mode,data.items,data.setItems)};this.getUnloadedItems=function(mode){return DeveloperTreeService.getUnloadedItems(mode)};this.newFile=function(obj){MainWindow.executeFunction("DTComponentCRUD","newFile",[obj])};this.hideCreateFile=function(){$scope.hideCreateFile()};this.addNewContainer=function(){$scope.onCreateFile()};this.changeHeightOfCodeEditor=Util.buildDebounceMethod(function(open){var code=$element.find(".code");if(code&&code[0]){var heightValue=$scope.isToSearchText()?"80px":"50px";var hasShortcut=$scope.isToSearchText()||$scope.isToCreateFile();open=open!==undefined?open:hasShortcut;if(!open){heightValue="0px";templateManager.updateTemplate();$scope.currentEditor.focus()}code[0].style.height="calc(100% - "+heightValue+")"}},100);this.openShortcutWidget=function(widgetName){var activeWidget=$scope.widget;if(activeWidget&&activeWidget.name==widgetName){$scope.widget.reload()}else{if(activeWidget){ShortcutDeveloperService.hideWidget(activeWidget)}var widgetAndContainer=ShortcutDeveloperService.getWidgetAndContainerToShow(widgetName);$scope.widget=widgetAndContainer.widget;$scope.templateManager.container=widgetAndContainer.container}};this.shouldShowShortcutWidget=function(){return $scope.isToSearchText()||$scope.isToFindLine()||$scope.isToCreateFile()};this.changeCaseSensitive=function(){var caseSensitive=$scope.widget.getField("caseSensitive");caseSensitive.value(!caseSensitive.value());caseSensitive.class=caseSensitive.value()?"01 zh-dvtools zh-case-sensitive":"01 zh-dvtools";$scope.findTextInEditor()};this.findTextInEditor=function(args,option){$scope.findTextInEditor(option,args.e.keyCode)};this.findLine=function(args){$scope.findLine(args)};var isEmptySelection=function(selection){var selectionDiff;if(selection.end&&selection.start){selectionDiff=selection.end.column-selection.start.column||selection.end.row-selection.start.row}return selectionDiff};var replaceInEditor=function(callback){callback=callback||function(){};var replaceWith=$scope.widget.getField("textToReplace").value();var selection=$scope.currentEditor.selection.getRange();if(replaceWith&&replaceWith.length>0&&isEmptySelection(selection)){$scope.currentEditor.session.replace(selection,replaceWith);$scope.markTextInTextEditor("next",undefined);callback()}};this.replaceText=function(){replaceInEditor()};this.replaceAllText=function(){var callback=self.replaceAllText;replaceInEditor(callback)};this.updateDependencyFieldsVisibility=function(widget,fieldChanged){eventAggregator.publish("updateDependencyFieldsVisibility",widget.getField(fieldChanged))};this.updateActiveItemInEditor=function(paths){$scope.updateActiveItem(paths)};this.setErrorsValidationInTree=function(errorsObject){$scope.setInvalidItems(errorsObject)};this.addErrorValidationInTree=function(errorObject){$scope.addInvalidItem(errorObject)};this.clearErrorsValidationInTree=function(){$scope.clearInvalidItems()};this.reloadErrorsValidationInTree=function(){$scope.reloadErrorsValidationInTree()};this.getActiveTreeItem=function(){return $scope.getActiveItem()};this.toggleInspectionMode=function(){window.inspection=!window.inspection};this.alternateTree=function(){window.hide=!window.hide};this.switchWindowMode=function(){setCursorPositionInActiveItem();MainWindow.changeWindowMode()};this.openPath=function(path,mode,cursorPosition){$scope.openPath(path,self,cursorPosition);if(mode){self.changeCurrentMode(mode)}};this.getOpenFormImageBasedOnFormVisibility=function(){return $scope.inFormMode()?"json-view":"form-view"};this.getArrowImageBasedOnTreeVisibility=function(){return window.hide?"show-tree":"hide-tree"};this.getTreeTemplateUrlBasedOnEditorMode=function(){var treeItemHtml=$scope.inFileMode()?"developer/tree-item-project.html":"developer/tree-item.html";return templateManager.getFrameworkTemplateURL(treeItemHtml)};this.getShortcutUrl=function(){var shortcutHtml="developer/zh-search.html";if($scope.isToFindLine()){shortcutHtml="developer/zh-find-line.html"}else if($scope.isToCreateFile()){shortcutHtml="developer/zh-create-file.html"}return templateManager.getFrameworkTemplateURL(shortcutHtml)};this.getEditorTemplateUrlBasedOnEditorMode=function(){var codeEditorHtml="developer/code-editor-json.html";if($scope.inFileMode()){codeEditorHtml="developer/code-editor-js.html"}else if($scope.inFormMode()){codeEditorHtml="developer/form-editor.html"}return templateManager.getFrameworkTemplateURL(codeEditorHtml)};this.saveComponentChange=function(){var activeItem=$scope.getActiveItem();var state=activeItem?$scope.getActiveComponentState():{valid:false};if(state.valid){var componentObject={};var indexPath=activeItem._indexPath;var fileName=indexPath?indexPath.substring(indexPath.lastIndexOf("/")):"";var activePath=$scope.getActiveComponentPath();activePath=activePath&&activePath.length?activePath:undefined;if($scope.inFileMode()){componentObject.data=$scope.lastScript}else if($scope.inMenuMode()){var ZEEDHI_PROJECT="zeedhi_project";activePath=activePath.indexOf(ZEEDHI_PROJECT)<0?[ZEEDHI_PROJECT].concat(activePath):activePath;componentObject.zeedhi_project=Util.clone(window.groupMenu.containers.zeedhi_project)}else{componentObject=Util.clone(window.currentContainer.__metadata);componentObject=ShowInDevTools.clear(componentObject)}recoverComponentId(activeItem.item,state);componentObject.fileName=fileName;componentObject.__indexPath=indexPath;var saveParams=[state.value,componentObject,activePath,$scope.inFileMode()];MainWindow.executeFunction("DTComponentCRUD","saveComponentChange",saveParams)}};this.saveNewComponent=function(defaultJsonValue,activeItemPath,completeJson){var saveNewParams=[defaultJsonValue,activeItemPath,completeJson];MainWindow.executeFunction("DTComponentCRUD","saveNewComponent",saveNewParams)};this.deleteComponent=function(activeItemPath,completeJson,deleteType){var deleteParams=[activeItemPath,completeJson,deleteType];MainWindow.executeFunction("DTComponentCRUD","deleteComponent",deleteParams)};this.deleteItemCustomization=function(){var itemId=$scope.getActiveItem().item.id;MainWindow.executeFunction("DTComponentCRUD","deleteComponentCustomization",[itemId])};this.deleteCustomFile=function(){var fileName=$scope.activeItem.item.name;MainWindow.executeFunction("DTComponentCRUD","deleteCustomFile",[fileName])};this.resetContainer=function(){MainWindow.executeFunction("DTComponentCRUD","resetContainer")};this.searchCustomization=function(){MainWindow.executeFunction("DTCustomizationService","renderCustomizationInMainWindow",window.currentContainer)};this.inspectSentComponent=function(inspectData){var mode=inspectData.mode=="form"?"json":inspectData.mode;if(mode){self.changeCurrentMode(mode)}self.openPath(inspectData.componentPath,mode,inspectData.cursorPosition)};this.changeCurrentMode=function(mode){$scope.setCurrentMode(mode,self)};this.clearActiveItem=function(){$scope.activeItem=undefined};this.showNotificationInDevTools=function(messageObj){ScreenService.notificationMessage(messageObj.message,messageObj.type)};this.receiveMessages=function(message){if(message.functionName){return this[message.functionName](message.args)}};this.createNewFile=function(key){if(!key||key===ENTER_KEY){$scope.createNewFile()}};this.toggleDevMenu=function(event){document.getElementById("devToolsMenu").classList.toggle("showDevMenu");event.stopPropagation()};this.hideDevMenu=function(){document.getElementById("devToolsMenu").classList.remove("showDevMenu")};function hasButtonMenu(element){return element.closest("#devToolsButtonMenu").length}this.clickOutMenu=function(event){if(!hasButtonMenu($(event.target))){self.hideDevMenu()}};if(!self.inCustomizationMode()){$(document).on("click",this.clickOutMenu);$scope.$on("$destroy",function(){$(document).off("click",this.clickOutMenu)})}$scope.$watch(function(){return window.hide},function(newValue){$scope.inHideMode=newValue});$scope.$watch(function(){return window.inspection},function(newValue){$scope.inInspectionMode=newValue})}function WindowController($scope,ContextRegister,templateManager,ApplicationContext,eventAggregator,DevToolsService,ScreenService,DeveloperTreeService,DTCustomizationService){ContextRegister.register("WindowController",this);var invalidItems=[];var self=this;var MainWindow=UtilDevTools.getMainWindow();var DevToolsWindow=UtilDevTools.getDevToolsWindow();var JS_MODE="files";var JSON_MODE="json";var MENU_MODE="menu";function getDevToolsController(){return ApplicationContext.DevToolsController}function isDevToolsInSameWindow(){return MainWindow.showDevtoolsInSameWindow}function prepareMessageAndSend(functionName,args){if(DevToolsWindow&&DevToolsWindow.sendMessageToDevTools){var messageToSend={functionName:functionName,args:args};return DevToolsWindow.sendMessageToDevTools(messageToSend)}}function getInvalidItems(){return invalidItems}function setErrorsValidationInTree(){prepareMessageAndSend("setErrorsValidationInTree",getInvalidItems())}function reloadErrorsValidationInDevToolsTree(){prepareMessageAndSend("reloadErrorsValidationInTree")}function getRigthComponentPath(componentPath,mode,activeItem){var condition=!componentPath&&activeItem;condition=condition&&(mode=="files"||activeItem.parent==DevToolsWindow.currentContainer.name);componentPath=componentPath||[];return condition?activeItem.path:componentPath}function enterInDevToolsWindow(componentPath,mode){var activeItem=DevToolsService.getLocalActiveItem();var cursorPosition=activeItem?activeItem.cursorPosition:{};mode=!mode&&activeItem?activeItem.mode:mode;componentPath=getRigthComponentPath(componentPath,mode,activeItem);setErrorsValidationInTree();if(mode!="files"){reloadErrorsValidationInDevToolsTree()}prepareMessageAndSend("inspectSentComponent",{componentPath:componentPath,mode:mode,cursorPosition:cursorPosition});if(!isDevToolsInSameWindow()){window.open("","DevTools").focus()}}function setUnloadedItems(mode,items){prepareMessageAndSend("setUnloadedItems",{mode:mode,items:items,setItems:true})}function getUnloadedItems(mode){return prepareMessageAndSend("getUnloadedItems",mode);
}function removeBackDotsOfPath(path){path=path.replace(/\\/g,"/");path=path.replace(/\/\//g,"/");path=path.replace(/\.\.\//g,"");return path}function prepareFileMatrix(files){var mainScripts=[];for(var i in files){var src=files[i];if(src){mainScripts.push({src:removeBackDotsOfPath(src),relativeUrl:src})}}mainScripts.sort(function(scriptA,scriptB){var strA=scriptA.src.toLowerCase();var strB=scriptB.src.toLowerCase();if(strA>strB){return 1}if(strA<strB){return-1}return 0});return mainScripts}function addFiles(files){if(files&&DevToolsWindow&&!DevToolsWindow.treeScript){DevToolsWindow.treeScript=null;files=prepareFileMatrix(files);var treeScript=FileTree.build(files);DevToolsWindow.treeScript=treeScript;setUnloadedItems(JS_MODE,treeScript);if(!getUnloadedItems(JS_MODE)){MainWindow.publishDevToolsReady()}}}this.updateScriptTree=function(){MainWindow.executeFunction("FileManagementService","reloadFiles")};this.setScriptsFiles=UtilDevTools.buildDevToolsWindowMethod(function(scriptsFiles){DevToolsService.setFilesToLocalStorage(scriptsFiles);addFiles(scriptsFiles)});function priorToBuildingTree(){MainWindow.executeFunction("FileManagementService","findFiles")}function prepareDevToolsWindow(mode){var condition=false;var activeItem=DevToolsService.getLocalActiveItem();mode=!mode&&activeItem?activeItem.mode:mode;DevToolsWindow=getDevToolsWindow();DevToolsWindow.currentContainer=templateManager.container;DeveloperTreeService.setTreeInWindow(MENU_MODE,DevToolsWindow);if(mode==JS_MODE){condition=DevToolsWindow.treeScript&&!getUnloadedItems(mode)}else if(DevToolsWindow.currentContainer){setUnloadedItems(JSON_MODE,DevToolsWindow.currentContainer);condition=!getUnloadedItems(JSON_MODE)}if(condition){MainWindow.publishDevToolsReady()}}function runDeveloperEditMode(args){eventAggregator.publish("developerEnableEditMode",args)}function runDeveloperHideMode(args){eventAggregator.publish("developerEnableHideMode",args)}function triggerResize(owner){var event=document.createEvent("Event");event.initEvent("resize",true,true);owner.dispatchEvent(event)}function getDevToolsWindow(){var devToolsWindow;if(isDevToolsInSameWindow()){devToolsWindow=window.frames["dev-tools"]}else{devToolsWindow=window.open(window.location+"?devtools=true","DevTools","width="+window.screen.width+", height="+window.screen.height)}return devToolsWindow}this.updateActiveItemInEditor=function(paths){prepareMessageAndSend("updateActiveItemInEditor",paths)};this.setFilesToLocalStorage=function(scriptsFiles){MainWindow.setFilesToLocalStorage(scriptsFiles)};this.addInvalidItem=function(invalidItem){invalidItems.push(invalidItem);prepareMessageAndSend("addErrorValidationInTree",invalidItem)};this.clearInvalidItems=function clearInvalidItems(){invalidItems=[];prepareMessageAndSend("clearErrorsValidationInTree")};this.isMainWindow=function(){return window===MainWindow};this.isDevToolsWindow=function(){return window===DevToolsWindow};this.isDevToolsOpened=function(){return!!DevToolsWindow};this.reloadMainWindow=function(){MainWindow.location.reload(true)};function closeDevToolsWindow(){MainWindow.closeDevToolsWindow()}function openDevToolsWindow(componentPath,mode){if(DTCustomizationService.getAccessEditMode()){var evt=eventAggregator.subscribe("devToolsReady",function(){enterInDevToolsWindow(componentPath,mode);eventAggregator.unSubscribe("devToolsReady",evt)});var files=DevToolsService.getFilesFromLocalStorage();prepareDevToolsWindow(mode);MainWindow.devtools=true;if(!files||!files.length){priorToBuildingTree()}else{addFiles(files)}DevToolsService.setLocalDevToolsFlag(MainWindow.devtools);templateManager.updateTemplate();triggerResize(MainWindow)}}function reopenDevToolsWindow(){closeDevToolsWindow();openDevToolsWindow();if(MainWindow.showDevtoolsInSameWindow){MainWindow.publishDevToolsReady()}}this.sendToDevTools=UtilDevTools.buildMainWindowMethod(function(componentPath){if(self.isDevToolsOpened()){enterInDevToolsWindow(componentPath)}else{openDevToolsWindow(componentPath)}window.inspection=false});this.showNotificationInDevTools=function(message,type){if(self.isMainWindow()&&self.isDevToolsOpened()){var args={message:message,type:type};prepareMessageAndSend("showNotificationInDevTools",args)}else{ScreenService.notificationMessage(message,type)}};this.showNotificationInMainWindow=function(message,type){if(self.isMainWindow()){ScreenService.notificationMessage(message,type)}else{MainWindow.showNotificationInMainWindow(message,type)}};Mousetrap.bind("mod+f12",function(){if(self.isMainWindow()&&!DevToolsWindow){openDevToolsWindow()}else{closeDevToolsWindow()}});if(self.isMainWindow()){$scope.$watch(function(){return templateManager.container},function(){if(DevToolsWindow){DevToolsWindow.currentContainer=templateManager.container;ShowInDevTools.setTree(DevToolsWindow.currentContainer.__metadata);setUnloadedItems(JSON_MODE,DevToolsWindow.currentContainer);self.clearInvalidItems()}else if(templateManager.container&&DevToolsService.getLocalDevToolsFlag()){openDevToolsWindow()}});MainWindow.executeFunction=function(instanceName,functionName,args){var instance=ApplicationContext[instanceName];if(instance){return instance[functionName].apply(this,args)}};MainWindow.getDevToolsWindow=function(){return DevToolsWindow};MainWindow.showConfirmMessageInMainWindow=function(txt,confirmFunction,failFunction){ScreenService.confirmMessage(txt,"confirmation",confirmFunction,failFunction)};MainWindow.showDialogInMainWindow=function(message,actions){ScreenService.showCustomDialog(message,"confirmation",null,actions)};MainWindow.showNotificationInMainWindow=function(message,type){ScreenService.notificationMessage(message,type)};MainWindow.onbeforeunload=function(){if(self.isDevToolsOpened()){MainWindow.closeDevToolsByMainWindow=true;DevToolsWindow.close()}};MainWindow.setFilesToLocalStorage=function(scriptsFiles){DevToolsService.setFilesToLocalStorage(scriptsFiles)};MainWindow.publishDevToolsReady=function(){eventAggregator.publish("devToolsReady")};MainWindow.closeDevToolsWindow=function(){if(DevToolsWindow&&!isDevToolsInSameWindow()){DevToolsWindow.close()}prepareMessageAndSend("clearActiveItem");MainWindow.focus();MainWindow.devtools=false;DevToolsService.setLocalDevToolsFlag(MainWindow.devtools);DevToolsWindow=null;templateManager.updateTemplate();triggerResize(MainWindow)};MainWindow.changeWindowMode=function(){MainWindow.showDevtoolsInSameWindow=!MainWindow.showDevtoolsInSameWindow;reopenDevToolsWindow()}}else{DeveloperTreeService.updateTree(MENU_MODE);DevToolsWindow.sendMessageToDevTools=function(message){templateManager.updateTemplate();var devToolsController=getDevToolsController();if(devToolsController){return devToolsController.receiveMessages(message)}};DevToolsWindow.publishFinishedRenderTreeItems=function(){eventAggregator.publish("finishedRenderTreeItems")};DevToolsWindow.onbeforeunload=function(){if(!MainWindow.closeDevToolsByMainWindow){closeDevToolsWindow()}}}var auxValue;Object.defineProperty(window,"inspection",{set:function(newValue){auxValue=newValue;if(self.isMainWindow()&&DevToolsWindow){if(DevToolsWindow.inspection!==newValue){DevToolsWindow.inspection=newValue}else{runDeveloperEditMode({enableEditMode:newValue})}}else if(self.isDevToolsWindow()&&MainWindow.inspection!==newValue){MainWindow.inspection=newValue}templateManager.updateTemplate()},get:function(){return auxValue}});var auxHideValue;Object.defineProperty(window,"hide",{set:function(newValue){auxHideValue=newValue;runDeveloperHideMode(newValue)},get:function(){return auxHideValue}})}function ContainerDecoratorDeveloper(ZHPromise,templateManager){this.apply=function(containerName,container){if(containerName&&container&&Util.isObject(container)){MetadataEnrichment.enrich(container);ShowInDevTools.setTree(container);container.events=container.events||[];container.events.push({name:"ContainerOnRender",code:"args.owner.render()"});var widgetEventFunction=function(widget){var widgetEvent={name:"WidgetOnRender",code:"args.owner.redraw()"};widget.events=widget.events||[];widget.events.push(widgetEvent)};var _eachWidget=function(widget,f){f(widget);if(widget.widgets){widget.widgets.forEach(function(widget){_eachWidget(widget,f)})}};if(container.widgets){container.widgets.forEach(function(widget){_eachWidget(widget,widgetEventFunction)})}}return ZHPromise.when(container)}}function DTCustomizationDecorator(ZHPromise,CustomizationService){this.apply=function(containerName,container){if(containerName&&container&&Util.isObject(container)){var defer=ZHPromise.defer();container.__original=Util.clone(container);CustomizationService.buildContainerCustomization(container).then(function(container){defer.resolve(container)});return defer.promise}else{return ZHPromise.when(container)}}}function DTComponentCRUD(SocketService,ElementToMetadataService,templateManager){var NEW="NEW";var FILE_MODE="files";var MENU_MODE="menu";var changeJsonAndSendToWebSocket=function(json,path,completeJson,option){var indexPath=completeJson.__indexPath;delete completeJson.__indexPath;var containerToSend=ElementToMetadataService.changeJson(json,path,completeJson,option);if(containerToSend.somethingChange){containerToSend.completeJson.__indexPath=indexPath;SocketService.changeInClient(containerToSend.completeJson)}};var changeFileAndSendToWebSocket=function(newText,componentObject){if(componentObject.data){var somethingChange=ElementToMetadataService.hasChanged(newText,componentObject.data);if(somethingChange){componentObject.data=newText;SocketService.changeInClient(componentObject,"files")}}};this.saveComponentChange=function(newText,componentObject,path,inFileMode){if(newText&&newText!==""){if(inFileMode){changeFileAndSendToWebSocket(newText,componentObject)}else{changeJsonAndSendToWebSocket(newText,path,componentObject)}}};this.saveNewComponent=function(json,path,completeJson){changeJsonAndSendToWebSocket(json,path,completeJson,NEW)};this.deleteComponent=function(path,completeJson,deleteType){changeJsonAndSendToWebSocket(false,path,completeJson,deleteType)};this.newFile=function(data){SocketService.newFile(data)}}function DTCustomComponentCRUD(ElementToMetadataService){var NEW="NEW";var sendMessageToCustomizationService=function(functionName,args){var MainWindow=UtilDevTools.getMainWindow();MainWindow.executeFunction("DTCustomizationService",functionName,args)};var clearComponents=function(components){var zeedhiTypes=UtilDevTools.getZeedhiTypes().concat(UtilDevTools.getIgnoredZeedhiTypes());components.forEach(function deleteComponentExtraProperties(componentObject){zeedhiTypes.forEach(function deleteSubitems(type){delete componentObject[type]});delete componentObject.__indexPath;delete componentObject.fileName})};var saveComponent=function(item,componentObject,pathToSave,inFileMode,option){var oldElement=ElementToMetadataService.getJsonElement(pathToSave||[],{__metadata:componentObject});var oldItem=oldElement.jsonElement;clearComponents([oldItem,item]);var changed=ElementToMetadataService.hasChanged(oldItem,item);if(changed){var args=[item,componentObject.id,pathToSave];sendMessageToCustomizationService("saveItemCustomization",args)}};this.saveComponentChange=function(newText,componentObject,pathToSave,inFileMode){var item=JSON.parse(newText);saveComponent(item,componentObject,pathToSave,inFileMode)};this.saveNewComponent=function(newText,pathToSave,componentObject,inFileMode){var item=JSON.parse(newText);item.id=item.id||"NEW"+(new Date).getTime();saveComponent(item,componentObject,pathToSave,inFileMode,NEW)};this.deleteComponent=function(path,completeJson,deleteType){var item=path.reduce(function(current,key){return current[key]},completeJson);console.log("deleteComponent "+item)};this.deleteCustomFile=function(fileName){sendMessageToCustomizationService("deleteCustomFile",[fileName])};this.resetContainer=function(){sendMessageToCustomizationService("resetContainer")};this.deleteComponentCustomization=function(itemId){sendMessageToCustomizationService("deleteItemCustomization",[itemId])};this.newFile=function(data){console.log("newFile")}}function DTCustomizationService(templateManager,ZHPromise,DTServerService,DTMessages,DTRenderService){var self=this;var zeedhiTypes=UtilDevTools.getZeedhiTypes();var constProperties=UtilDevTools.getIgnoredZeedhiTypes().concat(["__indexPath","fileName"]);var originalItems={};var customContainerItems={};var newItems={};var NEW="NEW";var resetCustomItems=function(){customContainerItems={};newItems={}};var sendConfirmMessage=function(message){var defer=ZHPromise.defer();UtilDevTools.getMainWindow().showConfirmMessageInMainWindow(message,defer.resolve,defer.reject);return defer.promise};var updateOriginalItemsPath=function(item,pathToSave){if(!isNewCustomId(item.id)){originalItems[item.id].item._path=pathToSave}};var saveCustom=function(item,containerId,pathToSave){DTServerService.saveCustomComponent(item.id,containerId,pathToSave,item).then(function(response){if(response.success){self.renderCustomizationInMainWindow();updateOriginalItemsPath(item,pathToSave)}})};var isNewCustomId=function(customId){return customId.toString().indexOf(NEW)>-1};var updateCustomItems=function(containerId){return DTServerService.getContainer(containerId).then(function(data){customContainerItems=data.updatedItems;newItems=data.newItems}).catch(function(err){customContainerItems={};newItems={};console.log(err)})};var applyObjectCustomization=function(obj){var customObj=Util.clone(customContainerItems[obj.id]);if(customObj){delete customObj._path;Object.keys(obj).forEach(function(key){if(customObj.hasOwnProperty(key)){obj[key]=customObj[key];delete customObj[key]}else if(zeedhiTypes.indexOf(key)==-1&&constProperties.indexOf(key)==-1){delete obj[key]}});Object.keys(customObj).forEach(function(key){obj[key]=customObj[key]})}return obj};var buildCustomizationItems=function(object,type,items){var clonedObj=Util.clone(object);items=items||{};if(object.hasOwnProperty("id")){zeedhiTypes.forEach(function(key){if(key!="__metadata"&&object[key]){object[key].forEach(function(item){items=buildCustomizationItems(item,key,items)})}delete clonedObj[key]});var custom=Util.clone(customContainerItems[object.id]);clonedObj._path=custom?custom._path:"";items[object.id]={item:clonedObj,type:type||"container"};object=applyObjectCustomization(object)}return items};var hasBeenAppliedInContainer=function(zeedhiArray,item){return zeedhiArray.some(function(oldItem,index){if(oldItem.id==item.id){zeedhiArray[index]=applyObjectCustomization(item);return true}return false})};var pushItemInArray=function(zeedhiArray,item,index){var hasUpdated=hasBeenAppliedInContainer(zeedhiArray,item);if(!hasUpdated){if(zeedhiArray.length<=index+1){zeedhiArray.push(item)}else{zeedhiArray.splice(index+1,0,item)}}};var applyItemInContainer=function(item,container){var path=item._path;delete item._path;path.reduce(function(current,key,index){if(index===path.length-1){pushItemInArray(current,item,key)}return current[key]},container)};var removeItemInContainer=function(item,container){var path=item._path;delete item._path;path.reduce(function(current,key,index){if(index===path.length-1){current.splice(key,1)}return current[key]},container)};var applyNewCustomItems=function(container){Object.keys(newItems).forEach(function(item,index){applyItemInContainer(Util.clone(newItems[item]),container)})};var applyContainerCustomization=function(container){return updateCustomItems(container.id).then(function(){var items=buildCustomizationItems(container);applyNewCustomItems(container);return{container:container,items:items}})};var isSetted=false;var accessEditMode=false;this.setAccessEditMode=function(hasAccess){if(!isSetted){accessEditMode=hasAccess;isSetted=true}};this.getAccessEditMode=function(){return!templateManager.inCustomizationMode||accessEditMode};this.buildContainerCustomization=function(container){return applyContainerCustomization(container).then(function(customization){originalItems=customization.items;return customization.container})};this.renderCustomizationInMainWindow=UtilDevTools.buildMainWindowMethod(function(container){container=container||templateManager.container;var metadata=Util.clone(container.__metadata);applyContainerCustomization(metadata).then(function(customization){DTRenderService.renderContainer(customization.container)})});this.deleteItemCustomization=function(itemId){sendConfirmMessage(DTMessages.CONFIRM_DELETE).then(function(){return DTServerService.deleteItemCustomization(itemId)}).then(function(wasDeleted){if(wasDeleted){var container=Util.clone(templateManager.container.__metadata);var item;if(isNewCustomId(itemId)){item=Util.clone(newItems)[itemId];delete newItems[itemId];removeItemInContainer(item,container)}else{item=Util.clone(originalItems)[itemId].item;delete customContainerItems[itemId];applyItemInContainer(item,container)}DTRenderService.renderContainer(container)}})};this.deleteCustomFile=function(fileName){sendConfirmMessage(DTMessages.CONFIRM_DELETE).then(function(){DTServerService.deleteCustomFile(fileName)})};this.resetContainer=function(){var queryToReset=templateManager.container.id;sendConfirmMessage(DTMessages.CONFIRM_RESET).then(function(){return DTServerService.resetContainer(queryToReset)}).then(function(wasDeleted){if(wasDeleted){resetCustomItems();DTRenderService.renderContainer(templateManager.container.__original)}})};this.saveItemCustomization=function(item,containerId,pathToSave){var itemId=item.id;DTServerService.getCustomComponent(itemId,containerId).then(function(newCustomComponent){var custom=isNewCustomId(itemId)?newItems[itemId]:customContainerItems[itemId];var oldCustom=custom||{};var needUpdate=oldCustom^newCustomComponent;var diffs=diff.getDiff(oldCustom,newCustomComponent);if(needUpdate||Object.keys(diffs).length){sendConfirmMessage(DTMessages.CUSTOM_OUTDATED).then(function(){saveCustom(item,containerId,pathToSave)}).catch(function(){sendConfirmMessage(DTMessages.RELOAD_CONTAINER).then(function(){self.renderCustomizationInMainWindow()})})}else{saveCustom(item,containerId,pathToSave)}})}}function DTServerService($http,ScreenService,DTMessages,DeveloperService){var serverUrl="http://192.168.120.153:3000";var getContainerRoute="/getContainer",getComponentRoute="/getComponent",saveComponentRoute="/saveComponent",deleteComponentRoute="/deleteComponent",resetContainerRoute="/resetContainer",getFileRoute="/getFile",getAllFilesRoute="/getAllFiles",saveFileRoute="/saveFile",deleteFileRoute="/deleteFile";var sendRequest=function(dataToSend,route,callback){return $http({url:serverUrl.concat(route),method:"POST",data:dataToSend,timeout:6e3}).then(function(response){return callback(response)}).catch(function(err){ScreenService.notificationMessage(DTMessages.CANT_CONNECT,"error");return{success:false,response:err}})};var deleteCustomization=function(id,route,errMsg,successMsg){var callback=function(response){var responseData=response.data.responseData;var wasDeleted=responseData.ok;if(wasDeleted){ScreenService.successNotification(successMsg)}else{ScreenService.notificationMessage(errMsg,"error");console.error(response)}return wasDeleted};return sendRequest({id:id},route,callback)};var isNewCustomId=function(customId){return customId.toString().indexOf("NEW")===0};this.getContainer=function(id){var criteria={id:id};var callback=function(response){var responseData=response.data||{};var responseItems=responseData.success&&responseData.responseData||[];var data={updatedItems:{},newItems:{}};responseItems.forEach(function(item){var itemId=item.id;var jsonItem=item.jsonItem;if(isNewCustomId(itemId)){jsonItem._path=item.path;data.newItems[itemId]=jsonItem}else{data.updatedItems[itemId]=jsonItem}});return data};return sendRequest(criteria,getContainerRoute,callback)};this.getCustomComponent=function(id,containerId){var criteria={id:id,containerId:containerId};var callback=function(response){var data={};var responseData=response.data;if(responseData&&responseData.success){var responseItems=responseData.responseData||[];responseItems.some(function(item){if(item.id==id){data=item.jsonItem;return true}})}return data};return sendRequest(criteria,getComponentRoute,callback)};this.getCustomFile=function(id){var callback=function(response){console.log(response)};return sendRequest({id:id},getFileRoute,callback)};this.getAllCustomFiles=function(){var callback=function(response){var responseData=response.data;var data={};if(responseData&&responseData.success){var responseItems=responseData.responseData||[];responseItems.forEach(function(item){data[item.id]=item.jsonItem})}return data};return sendRequest({},getAllFilesRoute,callback)};this.saveCustomComponent=function(id,containerId,path,componentObject){var dataToSave={id:id,jsonItem:componentObject,containerId:containerId,path:path||[],email:DeveloperService.getLocalEmail()};var callback=function(response){var success=response.data&&response.data.success;if(success){ScreenService.successNotification(DTMessages.CUSTOM_SAVED)}else{ScreenService.notificationMessage(DTMessages.CUSTOM_WASNT_SAVED,"error")}return{success:success}};return sendRequest(dataToSave,saveComponentRoute,callback)};this.deleteItemCustomization=function(id){return deleteCustomization(id,deleteComponentRoute,DTMessages.CUSTOM_ITEM_WASNT_DELETED,DTMessages.CUSTOM_ITEM_WAS_DELETED)};this.deleteCustomFile=function(fileName){deleteCustomization(fileName,deleteFileRoute,DTMessages.CUSTOM_FILE_WASNT_DELETED,DTMessages.CUSTOM_FILE_WAS_DELETED)};this.resetContainer=function(containerId){var callback=function(response){var responseData=response.data.responseData;var wasDeleted=responseData.ok;if(wasDeleted){ScreenService.successNotification(DTMessages.getMessageContainerWasReseted(response.n))}else{ScreenService.notificationMessage(DTMessages.CUSTOM_CONTAINER_WASNT_RESETED,"error");console.error(response)}return wasDeleted};return sendRequest({containerId:containerId},resetContainerRoute,callback)}}function DeveloperService(restEngine,localStorageEngine,templateManager){var DEV_CLIENT="developerTools";var DATE_TO_EXPIRE=36e5;var CONFIG_FILE=".developerConfig.json";var LOCALVAR_DSLOGIN="ds_login",LOCALVAR_FILES="devToolsScriptsFiles",LOCALVAR_DEVTOOLS="devTools",LOCALVAR_CLIENT_EMAIL="ds_clientEmail",LOCALVAR_ACTIVE_ITEM="devToolsActiveItem",LOCALVAR_PROJECT_OBJECT="ds_projectObject";this.getProject=function(){return localStorageEngine.getLocalVar(LOCALVAR_PROJECT_OBJECT)};this.setProject=function(){var project={id:templateManager.project.id,name:templateManager.project.label};localStorageEngine.setLocalVar(LOCALVAR_PROJECT_OBJECT,project)};this.setLocalDevToolsFlag=function(value){localStorageEngine.setLocalVar(LOCALVAR_DEVTOOLS,!!value)};this.setLocalActiveItem=function(itemName,path,mode,parent,cursorPosition){parent=parent||"";localStorageEngine.setLocalVar(LOCALVAR_ACTIVE_ITEM,{mode:mode,path:path,parent:parent,cursorPosition:cursorPosition||{},name:itemName})};this.setLocalEmail=function(email){localStorageEngine.setLocalVar(LOCALVAR_CLIENT_EMAIL,email)};this.getLocalEmail=function(){return localStorageEngine.getLocalVar(LOCALVAR_CLIENT_EMAIL)};this.getLocalDevToolsFlag=function(){return localStorageEngine.getLocalVar(LOCALVAR_DEVTOOLS)};this.getLocalActiveItem=function(){return localStorageEngine.getLocalVar(LOCALVAR_ACTIVE_ITEM)};this.clearActiveItem=function(){localStorageEngine.remove(LOCALVAR_ACTIVE_ITEM)};var setShowDevtoolsInSameWindow=function(){var MainWindow=UtilDevTools.getMainWindow();if(typeof MainWindow.showDevtoolsInSameWindow=="undefined"){MainWindow.showDevtoolsInSameWindow=templateManager.project.showDevtoolsInSameWindow}};var normalizeInCustomizationMode=function(){var inCustomizationMode=window.inCustomizationMode;if(typeof inCustomizationMode=="undefined"){inCustomizationMode=templateManager.project.inCustomizationMode;window.inCustomizationMode=inCustomizationMode}else{templateManager.project.inCustomizationMode=inCustomizationMode}templateManager.inCustomizationMode=inCustomizationMode;return inCustomizationMode};this.inDeveloperModePromise=templateManager.projectPromise.then(function(){setShowDevtoolsInSameWindow();var responseObj={};var inCustomizationMode=normalizeInCustomizationMode();var inDeveloperMode=inCustomizationMode;if(!inDeveloperMode){var defineInProject=typeof templateManager.inDeveloperMode=="undefined";inDeveloperMode=defineInProject?templateManager.project.inDeveloperMode:templateManager.inDeveloperMode}window.inDeveloperMode=inDeveloperMode;templateManager.inDeveloperMode=inDeveloperMode;templateManager.project.inDeveloperMode=inDeveloperMode;responseObj.inDeveloperMode=inDeveloperMode;responseObj.inCustomizationMode=inCustomizationMode;return responseObj});this.requestMetadataDeveloper=function(path){return restEngine.requestMetaData(path).then(function(data){return data}).catch(function(){return restEngine.requestMetaData(path,false,false,"./").then(function(data){return data})})};this.clearLocalDevToolsFlag=function(){localStorageEngine.remove(LOCALVAR_DEVTOOLS)};this.setFilesToLocalStorage=function(scriptsFiles){if(scriptsFiles){localStorageEngine.setLocalVar(LOCALVAR_FILES,{files:scriptsFiles,date:new Date})}};this.getFilesFromLocalStorage=function(){var files=[];var obj=localStorageEngine.getLocalVar(LOCALVAR_FILES);if(obj&&obj.date&&obj.files){var lastDate=new Date(obj.date);var newDate=new Date;if(newDate-lastDate<DATE_TO_EXPIRE){files=obj.files}}return files}}function DeveloperToolsForm(metaDataFactory,ScreenService,templateManager,DevToolsService){var DEFAULT_TEMPLATES=["field/text-edit.html","widget/form.html","container/window.html"];var DEFAULT_WIDGET={name:"widgetToForm",label:"widgetToForm",isVisible:true,template:"widget/form.html",actions:[]};var DEFAULT_CONTAINER={name:"containerForm",label:"containerForm",showHeader:false,height:"100%",isVisible:true,showFooter:false,footer:"component/footer.html",template:"container/window.html",addToHistory:false,widgets:[]};var CONFIG_TEMPLATES={string:{template:"field/text-edit.html"},number:{template:"field/number.html"},integer:{template:"field/number.html"},"boolean":{template:"field/checkbox.html",extraProperties:{valueToTrue:true,valueToFalse:false}},array:{template:"field/textarea.html",extraProperties:{typeOfValue:"array",textRows:5}},object:{template:"field/textarea.html",extraProperties:{typeOfValue:"object",textRows:5}}};function hasPropertyInJson(json,propertyName){return Object.keys(json).indexOf(propertyName)>=0}this.hasPropertyInJson=hasPropertyInJson;function isCheckboxField(template){return template.indexOf("checkbox")>=0}this.isCheckboxField=isCheckboxField;function setDependenciesInField(dependencies,field){if(dependencies){var fieldName=field.name;var dependsOn=dependencies.dependsOn;var fieldDependsOn=dependsOn?dependsOn[fieldName]:undefined;var dependencieFields=dependencies.dependenciesTree;var fieldDependencieFields=dependencieFields?dependencieFields[fieldName]:undefined;if(fieldDependsOn){field.dependsOn=fieldDependsOn}if(fieldDependencieFields&&fieldDependencieFields.length){field.dependencieFields=fieldDependencieFields;field.events=[{name:"FieldOnChange",code:"DevToolsController.updateDependencyFieldsVisibility(args.owner.field.widget, "+'"'+fieldName+'"'+");",id:+new Date}]}}return field}function createFieldToShowTemplate(template){var field=createField({name:"template",type:"string",description:template});field.defaultValue=field.placeholder;field.readOnly=true;return field}function valueFunctionOfObjectField(value){if(value!==undefined){if(typeof value=="object"){value=JSON.stringify(value,false,2)}this.widget.currentRow[this.name]=value;templateManager.updateTemplate()}else{value=this.widget.currentRow[this.name]||"";value=value?JSON.parse(value):value;return value}}function prepareValueFunction(field){if(field.typeOfValue){field.value=valueFunctionOfObjectField}}function getDefaultField(name,description,template){var fieldClass=isCheckboxField(template)?"6 zh-dvtools":"12 zh-dvtools";var field={"class":fieldClass,showOnForm:true,isVisible:true,template:template,id:+new Date,name:name,label:name,placeholder:description,help:description,title:description};return field}function createField(fieldProperties,dependencies){var type=fieldProperties.type;var configTemplate=CONFIG_TEMPLATES[type]||CONFIG_TEMPLATES.string;var fieldTemplate=configTemplate.template;var extraProperties=configTemplate.extraProperties||{};var field=getDefaultField(fieldProperties.name,fieldProperties.description,fieldTemplate);if(!!fieldProperties.isRequired){field.validations={required:!!fieldProperties.isRequired}}if(fieldProperties.defaultValue!==undefined){field.defaultValue=fieldProperties.defaultValue}Object.keys(extraProperties).forEach(function(key){field[key]=extraProperties[key]});field=setDependenciesInField(dependencies,field);field=metaDataFactory.fieldFactory(field);prepareValueFunction(field);return field}function transformFieldValuesInObject(activeItemJson,widget){var valuesObject=widget&&widget.isValid();if(valuesObject){var json=JSON.parse(activeItemJson);valuesObject={};widget.fields.forEach(function(fieldInForm){var fieldName=fieldInForm.name;var hasInOldJson=hasPropertyInJson(json,fieldName);if(fieldInForm.isVisible){var fieldValue=fieldInForm.value();var defaultValue=fieldInForm.defaultValue;var save=defaultValue!==undefined&&fieldValue!=defaultValue||!!fieldValue;if(hasInOldJson||save){valuesObject[fieldName]=fieldValue}}})}return valuesObject}function getFieldProperties(key,docTemplate,jsonItem){var value=docTemplate.properties[key];var type=typeof jsonItem[key];var properties={name:key,isRequired:~docTemplate.required.indexOf(key),type:type,description:"",defaultValue:false};if(value){var valueType=value.type;if(Util.isArray(valueType)){var index=valueType.indexOf(type)||0;valueType=valueType[index]}properties.type=valueType;properties.description=value.description;properties.defaultValue=value.hasOwnProperty("default")?value.default:undefined}return properties}function getFields(fieldsNames,docTemplate,jsonItem,dependencies){var fields=[];fieldsNames.forEach(function(value){var fieldProperties=getFieldProperties(value,docTemplate,jsonItem);fields.push(createField(fieldProperties,dependencies))});fields.push(createFieldToShowTemplate(jsonItem.template));return fields}function getDefaultWidget(fields){var widget=DEFAULT_WIDGET;widget.fields=fields;widget.id=+new Date;return widget}function getDefaultContainer(){var container=DEFAULT_CONTAINER;container.id=+new Date;return container}this.getFormWidgetAndContainer=function(fieldsNames,docTemplate,jsonItem,dependencies){var fields=getFields(fieldsNames,docTemplate,jsonItem,dependencies);var widget=getDefaultWidget(fields);var container=getDefaultContainer();container.widgets[0]=widget;container=metaDataFactory.containerFactory(container);widget=metaDataFactory.widgetFactory(widget,container);return{widget:widget,container:container}};function getFormDocTemplate(template,inOptionsFolder){var formType=inOptionsFolder?"options/":"";formType+=template+".json";var formItem=templateManager.getDocsUrl(formType);formType=formType.slice(0,formType.indexOf("/")+1);return DevToolsService.requestMetadataDeveloper(formItem).then(function(data){return data.data}).catch(function(data){var defaultTemplate=DEFAULT_TEMPLATES.filter(function(template){var templateType=template.split("/")[0];return formType.indexOf(templateType)>=0})[0];if(!inOptionsFolder&&formItem.indexOf(defaultTemplate)<0&&data.status=="404"){return getFormDocTemplate(defaultTemplate)}else{return false}})}this.getFormDocTemplate=getFormDocTemplate;this.getFormDependencies=function(docDependencies,options){var dependencies={};if(docDependencies&&options){var dependsOn={};var preparedDep={};Object.keys(docDependencies).forEach(function(realDep){if(options[realDep]){var dependencieValues=options[realDep].dependencies;docDependencies[realDep].forEach(function(value){if(dependencieValues[value]!==undefined){dependsOn[realDep]=dependsOn[realDep]||{};dependsOn[realDep][value]=dependencieValues[value];preparedDep[value]=preparedDep[value]?preparedDep[value].concat(realDep):[realDep];
}})}});dependencies={dependsOn:dependsOn,dependenciesTree:preparedDep}}return dependencies};this.getActiveFormState=function(activeItemJson,formWidget){var activeText=activeItemJson.text;var state={valid:false};if(activeText){try{var objectField=transformFieldValuesInObject(activeText,formWidget);if(!objectField){var objectText=JSON.parse(activeText).label||JSON.parse(activeText).name;ScreenService.notificationMessage("Fill required fields in "+objectText,"error",5e3)}else{state.valid=true;state.value=JSON.stringify(objectField)}}catch(e){state.valid=false;ScreenService.notificationMessage(e,"error",5e3)}}return state}}function DeveloperTreeService(DevToolsService){var self=this;var treeServices={};this.insertTreeMode=function(mode,service){treeServices[mode]=service};this.setLastItem=function(mode,item){treeServices[mode].setLastItem(item)};this.getLastItem=function(mode){return treeServices[mode].getLastItem()};this.getIndexPath=function(mode,item){var indexPath=item.__indexPath;if(!indexPath){var treeItems=self.getTreeItems(mode);indexPath=treeItems.length?treeItems[0].__indexPath:""}return indexPath};this.clearLastItems=function(){Object.keys(treeServices).forEach(function(mode){treeServices[mode].setLastItem()})};this.setActiveItem=function(item,mode,parent){var cursorPosition=item.cursorPosition||{row:0,column:0};self.setLastItem(mode,item);DevToolsService.setLocalActiveItem(item.item.name,item.path,mode,parent,cursorPosition)};this.setUnloadedItems=function(mode,items,setItems){if(setItems){items=treeServices[mode].getTotalUnloadedItems(items)}treeServices[mode].setUnloadedItems(items)};this.getUnloadedItems=function(mode){return treeServices[mode].getUnloadedItems()};this.restoreUnloadedItems=function(){Object.keys(treeServices).forEach(function(mode){if(!self.getUnloadedItems(mode)){var treeItem=treeServices[mode].getDefaultTreeItem();if(treeItem){self.setUnloadedItems(mode,treeItem,true)}}})};this.getItemMetadata=function(item,mode){return treeServices[mode].getItemMetadata(item)};this.getTreeItems=function(mode){return treeServices[mode].getTreeItems()};this.updateTree=function(mode,items){treeServices[mode].updateTree(items)};this.loadEmptyTrees=function(){Object.keys(treeServices).forEach(function(mode){var treeService=treeServices[mode];var treeItems=treeService.getTreeItems();var isLoaded=treeItems&&treeItems.length;if(!isLoaded){treeService.loadTree()}})};this.openTree=function(mode,pathToOpen,lastItem){if(treeServices[mode].openTree){lastItem=treeServices[mode].openTree(pathToOpen,lastItem)}else{pathToOpen.every(function(part,index){lastItem._opened=true;if(lastItem[part]){lastItem=lastItem[part];return true}return false})}return lastItem};this.setTreeInWindow=function(mode,window){treeServices[mode].setTreeInWindow(window)}}function ElementFileService(DevToolsService){var getPathToFile=function(metadataElement){var path=metadataElement.url.replace(/\/\//g,"/");return path.split("/")};var getFileElement=function(path,metadataElement){var url=metadataElement.url;var relativeUrl=metadataElement.relativeUrl;var script={fileElement:"",fileUrl:url,metadataElement:metadataElement,path:path};if(url&&relativeUrl){script=DevToolsService.requestMetadataDeveloper(relativeUrl).then(function(obj){return{fileElement:obj.data,fileUrl:url,__indexPath:url,path:path}})}return script};this.getElementToEditor=function(metadataElement){var path=getPathToFile(metadataElement);return getFileElement(path,metadataElement)}}function ElementToMetadataService(templateManager){var self=this;var devToolsWindow=window.opener||window.parent;var MENU_TYPE="menu";var GROUP_MENU_TYPE="groupMenu";var CONTAINER_TYPE="container";var PARENT_MENU="parentMenu";var DELETE="DELETE";var NEW="NEW";var DELETEWIDGET="DELETEWIDGET";var propertiesMap={action:{type:"actions",parent:"widget",condition:"showAsAction"},field:{type:"fields",parent:"widget"},column:{type:"fields",parent:"widget"},widget:{type:"widgets",parent:"container"},menu:{type:"menus",parent:"groupMenu"},groupMenu:{},container:{type:"container",parent:"widgets"}};this.getPropertyFromScope=function(scope){var property=false;if(devToolsWindow){var types=Object.keys(propertiesMap);types.every(function(type){var t=type.charAt(0).toUpperCase()+type.slice(1);var instance=devToolsWindow[t];if(instance&&scope instanceof instance){property=type;return false}else{return true}})}return property};var getProperty=function(scope){var property=CONTAINER_TYPE;var types=Object.keys(propertiesMap);types.every(function(type){if(scope[type]){property=type;return false}else{return true}});return property};var elementHasProperties=function(element,propertiesArray){return propertiesArray.some(function(property){return element.hasOwnProperty(property)})};var getElementType=function(element,isMenuType){var elementType;if(isMenuType){elementType="menu"}else if(element[0]&&getMenuType(element[0])){elementType="groupMenu"}return elementType};var getMenuType=function(element){var WINDOW_NAME="windowName";var MENUS="menus";var isMenu=elementHasProperties(element,[WINDOW_NAME,MENUS]);var parentIsMenu=element.parent&&element.parent.menus;var isMenuType=isMenu||parentIsMenu;return getElementType(element,isMenuType)};var getMetadataElement=function(scope){var property=CONTAINER_TYPE;var completeMetadata={element:templateManager.container,type:property};if(scope){completeMetadata.type=self.getPropertyFromScope(scope)||getMenuType(scope);if(!completeMetadata.type){completeMetadata.type=getProperty(scope);completeMetadata.element=scope[completeMetadata.type]}else{completeMetadata.element=scope}}return completeMetadata};var addItemToPath=function(path,parent,parentType,item){var position=parent[parentType].indexOf(item);path.unshift(position);path.unshift(parentType);return path};var getCompletePathToElement=function(metadataElement,type,_path){var path=_path||[];if(type==GROUP_MENU_TYPE){path.splice(0,1,GROUP_MENU_TYPE)}else if(type!==CONTAINER_TYPE){var elementProperties=propertiesMap[type];var elementType=elementProperties.type;var parentType=elementProperties.parent;var parent=metadataElement.parent||metadataElement[parentType];if(!parent&&type==MENU_TYPE){parent=metadataElement[PARENT_MENU]||templateManager.project;elementType=GROUP_MENU_TYPE}if(parent){parentType=parent==metadataElement.parent?type:parentType;path=addItemToPath(path,parent,elementType,metadataElement);getCompletePathToElement(parent,parentType,path)}}return path};this.getJsonElement=function(path,container){var jsonElement=container.__metadata||{};path.every(function(part){if(!jsonElement[part]){jsonElement="";return false}jsonElement=jsonElement[part];return true});var jsonAndPath={jsonElement:Util.clone(jsonElement),path:path};return jsonElement?jsonAndPath:false};this.getPathToMetadata=function(scope){var pathToMetadata=[];var metadataElement=getMetadataElement(scope);var element=metadataElement.element;var type=metadataElement.type;if(element&&type&&type!=CONTAINER_TYPE){pathToMetadata=getCompletePathToElement(element,type)}return pathToMetadata};this.getElementToMetadata=function(scope,container){var path=self.getPathToMetadata(scope,container);return self.getJsonElement(path,container)};var hasChanged=function(oldObj,newObj){var diffs=Object.keys(diff.getDiff(oldObj,newObj));var condition=~diffs.indexOf("/fileName")?diffs.length>1:diffs.length>=1;return condition};this.hasChanged=hasChanged;var deleteComponentInFather=function(option,index,path,somethingChange,current,key){var pathLength=path.length;var indexToCompare=option==DELETE?pathLength-3:pathLength-2;var currentChanged;if(index===indexToCompare){currentChanged=option==DELETE?current[key][path[pathLength-2]]:current.widgets;somethingChange=true;currentChanged=currentChanged.filter(function(value,index){return index!=path[pathLength-1]});if(option==DELETE){current[key][path[pathLength-2]]=currentChanged}else{current.widgets=currentChanged}}return{current:current,somethingChange:somethingChange}};var insertNewItem=function(current,item,index){if(current.length<=index+1){current.push(item)}else{current.splice(index+1,0,item)}};this.changeJson=function(json,path,completeJson,option){var somethingChange=false;var object=json?JSON.parse(json):json;if(path&&path.length>0){path.reduce(function(current,key,index){if(option==DELETE||option==DELETEWIDGET){var changedIn=deleteComponentInFather(option,index,path,somethingChange,current,key);current=changedIn.current;somethingChange=changedIn.somethingChange}else if(index===path.length-1){if(option==NEW){insertNewItem(current,object,path[path.length-1]);somethingChange=true}else{somethingChange=hasChanged(current[key],object);current[key]=object}}return current[key]},completeJson)}else{somethingChange=hasChanged(completeJson,object);completeJson=object}return{somethingChange:somethingChange,completeJson:completeJson}}}function FileManagementService(SocketService){this.findFiles=function(){SocketService.loadFiles()};this.reloadFiles=function(){SocketService.reloadFiles()};this.newJsonFile=function(data){if(data){SocketService.newFile(data)}}}(function(){function FileTreeService(ElementFileService){var self=this;var lastItem;var treeItems=[];var DevToolsWindow=UtilDevTools.getDevToolsWindow();var fileUnloadedItems;this.setUnloadedItems=function(unloadedItems){fileUnloadedItems=unloadedItems};this.getUnloadedItems=function(){return fileUnloadedItems};this.setLastItem=function(item){lastItem=item;lastItem.path.shift()};this.getLastItem=function(){return lastItem};var setTreeItems=function(items){treeItems=items};this.getTreeItems=function(){return treeItems};this.getItemMetadata=function(item){var parent=self.getTreeItems()[0];return ElementFileService.getElementToEditor(item,parent)};this.loadTree=function(){var tree=DevToolsWindow.treeScript||{};tree=Object.keys(tree).map(function(key){return tree[key]});setTreeItems(tree)};this.updateTree=self.loadTree;this.getDefaultTreeItem=function(){var treeScript=Util.clone(window.treeScript);return FileTree.unsetParent(treeScript)};this.getTotalUnloadedItems=function(script){var hasSomeUnloadedItems=!script._element;var getTotalItems=function(item){var total=Object.keys(item).reduce(function(total,property){var child=item[property];if(child&&child.showInDevTools){total++;hasSomeUnloadedItems=hasSomeUnloadedItems||!child.element;return total+getTotalItems(child)}else{return total}},0);return total};var totalUnloadedItems=getTotalItems(script);totalUnloadedItems=hasSomeUnloadedItems?totalUnloadedItems:0;return totalUnloadedItems}}Configuration(function(DeveloperTreeService,ElementFileService){var instance=new FileTreeService(ElementFileService);DeveloperTreeService.insertTreeMode("files",instance)})})();(function(){function JsonTreeService(ElementToMetadataService){var self=this;var lastItem;var treeItems=[];var jsonUnloadedItems;this.setUnloadedItems=function(unloadedItems){jsonUnloadedItems=unloadedItems};this.getUnloadedItems=function(){return jsonUnloadedItems};this.setLastItem=function(item){lastItem=item};this.getLastItem=function(){return lastItem};var setTreeItems=function(items){treeItems=items};this.getTreeItems=function(){return treeItems};this.getItemMetadata=function(item){var parent=self.getTreeItems()[0];return ElementToMetadataService.getElementToMetadata(item,parent)};this.openTree=function(pathToOpen,lastItem){if(pathToOpen.length>=2&&lastItem){var type=pathToOpen.shift();var index=pathToOpen.shift();lastItem._opened=true;if(lastItem[type][index]){lastItem=lastItem[type][index];return self.openTree(pathToOpen,lastItem)}}return lastItem};this.loadTree=function(newContainer){var tree=[];if(newContainer){tree.push(newContainer)}setTreeItems(tree)};this.updateTree=self.loadTree;this.getDefaultTreeItem=function(){return window.currentContainer};this.getTotalUnloadedItems=function(container){var childrenMapProperties={container:["widgets"],widgets:["widgets","fields","actions"]};var hasSomeUnloadedItems=!container._element;var getTotalItems=function(item,type,initValue){var total=initValue;if(childrenMapProperties[type]){total+=childrenMapProperties[type].reduce(function(total,property){var children=item[property];if(children&&children.length){total+=children.reduce(function(total,child){if(child.showInDevTools){total++;hasSomeUnloadedItems=hasSomeUnloadedItems||!child._element;return total+getTotalItems(child,property,0)}else{return total}},0)}return total},0)}return total};var totalUnloadedItems=getTotalItems(container,"container",1);totalUnloadedItems=hasSomeUnloadedItems||window.opener?totalUnloadedItems:0;return totalUnloadedItems}}Configuration(function(DeveloperTreeService,ElementToMetadataService){var instance=new JsonTreeService(ElementToMetadataService);DeveloperTreeService.insertTreeMode("json",instance);DeveloperTreeService.insertTreeMode("form",instance)})})();(function(){function MenuTreeService(ElementToMetadataService,templateManager){var self=this;var lastItem;var treeItems=[];var MENU_PROPERTY={type:"menus",parent:"groupMenu"};var menuUnloadedItems;this.setUnloadedItems=function(unloadedItems){menuUnloadedItems=unloadedItems};this.getUnloadedItems=function(){return menuUnloadedItems};this.setTreeInWindow=function(devToolsWindow){var menu={};menu.groupMenu=Util.clone(templateManager.containers.zeedhi_project.groupMenu);menu.__indexPath=templateManager.project.__indexPath;menu.label="groupMenu";devToolsWindow.groupMenu=menu};this.setLastItem=function(item){lastItem=item};this.getLastItem=function(){return lastItem};var setTreeItems=function(items){treeItems=items};this.getTreeItems=function(){return treeItems};var setContainersInGroupMenu=function(groupMenu){groupMenu.containers=groupMenu.containers||templateManager.containers;groupMenu.containers.__metadata=Util.clone(groupMenu.containers.zeedhi_project)};this.getItemMetadata=function(item){var groupMenu=UtilDevTools.getDevToolsWindow().groupMenu;setContainersInGroupMenu(groupMenu);return ElementToMetadataService.getElementToMetadata(item,groupMenu.containers)};this.openTree=function(pathToOpen,lastItem){if(pathToOpen.length>=2&&lastItem){var type=pathToOpen.shift();var index=pathToOpen.shift();lastItem._opened=true;var item=lastItem[type]||lastItem;if(item[index]){lastItem=item[index];return self.openTree(pathToOpen,lastItem)}}return lastItem};var setParentInTree=function(parent,groupMenuElements){var element=parent[MENU_PROPERTY.type]||parent[MENU_PROPERTY.parent];var parentProperty=groupMenuElements?"parentMenu":"parent";if(Array.isArray(element)){element.forEach(function(subElement){subElement[parentProperty]=parent;setParentInTree(subElement)})}return parent};this.loadTree=function(devToolsWindow){devToolsWindow=devToolsWindow||UtilDevTools.getDevToolsWindow();var hasDevToolsWindowAndGroupMenu=devToolsWindow&&devToolsWindow.groupMenu;if(hasDevToolsWindowAndGroupMenu){ShowInDevTools.setTree(devToolsWindow.groupMenu);var newGroupMenu=setParentInTree(devToolsWindow.groupMenu,true);setTreeItems([newGroupMenu])}};this.updateTree=self.loadTree;this.getDefaultTreeItem=function(){return window.groupMenu};this.getTotalUnloadedItems=function(groupMenu){var PROJECT="project";var GROUPMENU_MAP={project:["groupMenu"],groupMenu:["menus"],menus:["menus"]};var hasSomeUnloadedItems=!groupMenu._element;var getTotalItems=function(item,type,initValue){var total=initValue;var itemProperties=GROUPMENU_MAP[type];if(itemProperties){total+=itemProperties.reduce(function(total,property){var children=item[property];if(children&&children.length){total+=children.reduce(function(total,child){if(child&&child.showInDevTools){total++;hasSomeUnloadedItems=hasSomeUnloadedItems||!child._element;return total+getTotalItems(child,property,0)}else{return total}},0)}return total},0)}return total};var totalUnloadedItems=getTotalItems(groupMenu,PROJECT,1);var isOpener=window.opener;totalUnloadedItems=hasSomeUnloadedItems||isOpener?totalUnloadedItems:0;return totalUnloadedItems}}Configuration(function(DeveloperTreeService,ElementToMetadataService,templateManager){var instance=new MenuTreeService(ElementToMetadataService,templateManager);DeveloperTreeService.insertTreeMode("menu",instance)})})();function ShortcutDeveloperService(metaDataFactory){var shortcut_widgets={};var DEFAULT_WIDGET={isVisible:true,template:"widget/form.html",actions:[]};var DEFAULT_CONTAINER={showHeader:false,height:"100%",isVisible:true,showFooter:false,footer:"component/footer.html",template:"container/window.html",addToHistory:false,widgets:[]};function createFields(fieldsToCreate){var fields=Object.keys(fieldsToCreate).map(function(value){var field=fieldsToCreate[value];field.id=+new Date;field.name=value;field=metaDataFactory.fieldFactory(field);return field});return fields}function getDefaultWidget(widgetName,fields){var widget=DEFAULT_WIDGET;widget.fields=createFields(fields);widget.name=widgetName;widget.label=widgetName;widget.id=+new Date;return widget}function getDefaultContainer(widgetName){var container=DEFAULT_CONTAINER;container.name="container"+widgetName;container.label="container"+widgetName;container.id=+new Date;return container}this.createWidgetAndContainer=function(widgetName,fields,hideFunction){if(!shortcut_widgets[widgetName]){var widget=getDefaultWidget(widgetName,fields);var container=getDefaultContainer(widgetName);widget.hideWidget=hideFunction;container.widgets[0]=widget;container=metaDataFactory.containerFactory(container);shortcut_widgets[widgetName]=metaDataFactory.widgetFactory(widget,container)}};this.hideWidget=function(widget){var widgetToHide=widget?shortcut_widgets[widget.name]:undefined;if(widgetToHide){widgetToHide.hideWidget()}};this.getWidgetAndContainerToShow=function(widgetName){var widgetAndContainer={};var shortcutWidget=shortcut_widgets[widgetName];if(shortcutWidget){widgetAndContainer.widget=shortcutWidget;widgetAndContainer.container=shortcutWidget.container}return widgetAndContainer}}(function(){var diff=function(){var getType=function(x){var type=typeof x;if(type==="object"&&Object.prototype.toString.call(x).match(/array/i)){type="array"}return type};var findDiff=function(ob1,ob2,path,result){var val1,val2,newpath,key;var type1=getType(ob1);var type2=getType(ob2);if(path===null||typeof path!=="string"){path="/"}if(result===null||typeof result!=="object"){result={}}if(ob1===null||ob2===null){if(ob1!==ob2){if(type1==="undefined"){result[path]={operation:"add",value:ob2}}else if(type2==="undefined"){result[path]={operation:"delete"}}else{result[path]={operation:"update",value:ob2}}}}else if(type1!==type2||type1!=="object"&&type1!=="array"||type2!=="object"&&type2!=="array"){if(ob1!==ob2){result[path]={operation:"update",value:ob2}}}else{for(key in ob1){newpath=path==="/"?path+key:path+"/"+key;val1=ob1[key];val2=ob2[key];if(val1===null||val1===undefined||val2===null||val2===undefined){if(val1!==val2){if(typeof val1==="undefined"){result[newpath]={operation:"add",value:val2}}else if(typeof val2==="undefined"){result[newpath]={operation:"delete"}}else{result[newpath]={operation:"update",value:val2}}}}else{if(getType(val1)!==getType(val2)){result[newpath]={operation:"update",value:val2}}else{if(typeof val1==="object"){findDiff(val1,val2,newpath,result)}else{if(val1!==val2){result[newpath]={operation:"update",value:val2}}}}}}for(key in ob2){newpath=path==="/"?path+key:path+"/"+key;val1=ob1[key];val2=ob2[key];if(val1!==val2){if(typeof val1==="undefined"){result[newpath]={operation:"add",value:val2}}}}}return result};var setValueByPath=function(ob,path,value,visitorCallback){if(!path.match(/^\//)){throw"diff path is not valid"}var keys=path.split("/");keys.shift();var val=ob;var length=keys.length;for(var i=0;i<length;i++){if(val===null||keys[i].length<1){throw"Invalid data"}if(i!==length-1){val=val[keys[i]];if(visitorCallback){visitorCallback(val)}}else{val[keys[i]]=value}}return ob};var deleteValueByPath=function(ob,path,visitorCallback){var keys=path.split("/");keys.shift();var val=ob;var length=keys.length;for(var i=0;i<length;i++){if(i!==length-1){if(val[keys[i]]===null){throw"invalid data"}val=val[keys[i]];if(visitorCallback){visitorCallback(val)}}else{if(getType(val)==="object"){delete val[keys[i]]}else{var index=parseInt(keys[i]);while(val.length>index){val.pop()}}}}return ob};var applyDiff=function(ob1,diff,visitorCallback){var path,diffOb,op;if(diff===null){throw"No diff object is provided, Nothing to apply"}for(var key in diff){path=key;diffOb=diff[key];op=diffOb.operation;if(op.match(/add|update|delete/)){if(op==="add"){if(path==="/"){ob1=diffOb.value;break}setValueByPath(ob1,path,diffOb.value,visitorCallback)}else if(op==="update"){if(path==="/"){ob1=diffOb.value;break}setValueByPath(ob1,path,diffOb.value,visitorCallback)}else{if(path==="/"){ob1=null;break}deleteValueByPath(ob1,path,visitorCallback)}}else{throw"malformed diff object"}}return ob1};return{getDiff:function(ob1,ob2){var result=findDiff(ob1,ob2);return result},applyDiff:function(ob,diff,visitorCallback){var result=applyDiff(ob,diff,visitorCallback);return result}}}();if(typeof module!=="undefined"&&typeof module.exports!=="undefined"){module.exports=diff}else{window.diff=diff}})();function DTMessages(){this.CANT_CONNECT="Can't connect.";this.RELOAD_CONTAINER="Deseja atualizar o container?";this.CHECK_MAIN_WINDOW="An error occurred. Please check your main window.";this.PROJECT_ID_SAVED="Project id saved successfully!";this.CONTAINER_PROJECT_ID_FAIL="The containers.json doesn't have a project id.";this.CONTAINER_WAS_CREATED="Container was created successfully";this.LOGIN_NOT_ACCOMPLISHED="Login not accomplished! Please, try connect the server.";this.LOGIN_ACCOMPLISHED="Login accomplished with success!";this.LOGIN_NO_OTHER_CLIENTS="No other clients connected";this.CONFIRM_RESET="Are you sure you want to reset?";this.CONFIRM_DELETE="Are you sure you want to delete?";this.CUSTOM_OUTDATED="Customization is outdated. Do you want to save anyway?";this.CUSTOM_SAVED="Customization saved with success.";this.CUSTOM_WASNT_SAVED="Customization not saved.";this.CUSTOM_ITEM_WAS_DELETED="Item customization was deleted.";this.CUSTOM_ITEM_WASNT_DELETED="Item customization wasn't deleted.";this.CUSTOM_CONTAINER_WASNT_RESETED="Container customization wasn't reseted.";this.CUSTOM_FILE_WAS_DELETED="Customization file was deleted.";this.CUSTOM_FILE_WASNT_DELETED="Customization file wasn't deleted.";this.FILE_INVALID_NAME="Invalid file name";this.FILE_INVALID_PATH="Invalid path";this.FILE_CANT_SAVE="You can't save this file, Because it is out of your file system";this.FILE_SAVED_SUCCESSFULLY="File saved successfully";this.FILE_ERROR_MAP={ECONTAINERSAVE:"Cannot save containers.json file",EPIPE:"Broken pipe.",EPERM:"Operation not permitted.",EACCES:"Permission denied.",EEXIST:"File exists.",EISDIR:"Is a directory.",EMFILE:"Too many open files in system.",ENOENT:"No such file or directory.",ENOTDIR:"Not a directory.",ENOTEMPTY:"Directory not empty.",ETIMEDOUT:"Operation timed out.",EADDRINUSE:"Address already in use.",ECONNRESET:"Connection reset by peer.",ECONNREFUSED:"Connection refused."};this.getMessageContainerWasReseted=function(qtd){var message="Container customization reseted.";if(qtd){message+=" ["+qtd+" objects was deleted]"}return message}}var DeveloperMenuEnrichment={enrich:function(project){var groupMenu=Util.clone(project.groupMenu);var showMenuInDevTools=function(groupMenu){groupMenu.forEach(function(group){group.showInDevTools=true;delete group.parent;if(group.menus){showMenuInDevTools(group.menus)}})};showMenuInDevTools(groupMenu);project.__menuMetadata={groupMenu:Util.clone(groupMenu),__indexPath:project.__indexPath}}};var tree={};function getOnlyObjects(obj){return Object.keys(obj).filter(function(property){return property!="parent"&&typeof obj[property]=="object"})}function clearTree(){tree={}}var FileTree={isFileAllowed:function(fileName){var filesToSave=[".js",".html",".css"];return filesToSave.some(function(type){return fileName.substring(fileName.length-type.length)==type})},unsetParent:function(obj){var unsetParent=function(_obj){if(typeof _obj=="object"){delete _obj.parent;var properties=getOnlyObjects(_obj);properties.forEach(function(property){_obj[property]=unsetParent(_obj[property])})}return _obj};return unsetParent(obj)},build:function(paths){var getUrl=function(path,name){return path.slice(0,path.indexOf(name)+name.length)};var buildNodeOfTree=function(parent,parentName,path,pathName,relativeUrl){if(!parent.hasOwnProperty(pathName)){var name=pathName;var nodeOfTree={};if(FileTree.isFileAllowed(pathName)){name=path.replace(/^.*[\\\/]/,"");pathName=name;nodeOfTree.relativeUrl=relativeUrl;nodeOfTree.isFile=true}nodeOfTree.name=nodeOfTree.label=name;nodeOfTree.isFolder=!nodeOfTree.isFile;nodeOfTree.showInDevTools=true;nodeOfTree.url=getUrl(path,name);nodeOfTree.parent=parentName;parent[pathName]=nodeOfTree}return parent[pathName]};clearTree();paths.forEach(function(fileObj){var obj=tree;var path=fileObj.src;var arrayPath=path.split("/");arrayPath.forEach(function(pathName,index){obj=buildNodeOfTree(obj,arrayPath[index-1],path,pathName,fileObj.relativeUrl)})});return tree}};var MetadataEnrichment={enrich:function(obj){obj.__metadata=Util.clone(obj)}};var properties=["__metadata","widgets","fields","actions","groupMenu","menus"];function isZeedhiObject(objName){return~properties.indexOf(objName)}function isDeveloperObject(obj,objName){return!objName||obj.hasOwnProperty("isFolder")}function isZeedhiOrDeveloperObject(obj,objName){return isDeveloperObject(obj,objName)||isZeedhiObject(objName)}function getProperties(obj){var props=properties.filter(function(property){return obj.hasOwnProperty(property)});props=props.length?props:Object.keys(obj);return props}function applyFunctionToObject(obj,f,objName){if(Array.isArray(obj)){obj.map(function(object){applyFunctionToObject(object,f,objName)})}else if(Util.isObject(obj)&&isZeedhiOrDeveloperObject(obj,objName)){obj=f(obj);getProperties(obj).forEach(function(property){if(obj[property]){obj[property]=applyFunctionToObject(obj[property],f,property)}})}return obj}function setPropertyShowInDevTools(obj){obj.showInDevTools=true;return obj}function deletePropertyShowInDevTools(obj){if(obj.showInDevTools)delete obj.showInDevTools;return obj}var ShowInDevTools={setTree:function(object){return applyFunctionToObject(object,setPropertyShowInDevTools)},clear:function(object){return applyFunctionToObject(object,deletePropertyShowInDevTools)}};ZeedhiChart.directive("zhCodeEditor",function(ScreenService,eventAggregator,DeveloperToolsForm){return{restrict:"A",link:function($scope,$element,$attr,$ctrl){function getKeepSession(newItem,oldItem){var keepSession=false;if(newItem&&oldItem&&newItem.path&&oldItem.path){keepSession=newItem.path==oldItem.path&&newItem.mode==oldItem.mode}return keepSession}function clearSession(text){var newSession=ace.createEditSession(text||"",new $scope.currentModeClass);$scope.currentEditor.setSession(newSession);$scope.currentEditor.getSession().setTabSize(2)}function getErrorLog(errors){var log="";errors.forEach(function(error,index){if(error){log+=error.reason+"In line: "+error.line;log+="\n"}else if(errors[index-1]&&errors[index-1].code=="E043"){log=errors[index-1].reason+"\n"+log}});return log}function getItemAttributes(item){var attributes=["text","path","mode"];var objectItem={};item.forEach(function(value,index){var indexName=attributes[index];if(indexName=="path"){value=item[index]?item[index].join("/"):""}objectItem[indexName]=value});return objectItem}function isScriptWithValidation(path){return!(~path.indexOf(".css")||~path.indexOf(".html"))}function isValidJson(text){jsonlint.parse(text)}function isValidJs(text){var validation=isScriptWithValidation($scope.getActiveItem()._indexPath);if(validation){JSHINT(text,{eqnull:true},{jquery:true});if(JSHINT.errors.length>0){throw new Error(getErrorLog(JSHINT.errors))}}}function isValid(text,validFunction){var annotations=$scope.currentEditor?$scope.currentEditor.getSession().getAnnotations():[];var aceLintErrors=annotations.filter(function(annotation){return annotation.type==="error"});var isValid=!aceLintErrors.length;if(isValid&&text){try{validFunction(text);isValid=true}catch(e){ScreenService.errorNotification(e.message);isValid=false}}return isValid}function setAnnotations(annotations){$scope.currentEditor.getSession().setAnnotations(annotations)}function getActiveComponentState(){var state={};if($scope.inFormMode()){state=DeveloperToolsForm.getActiveFormState($scope.activeItem,$scope.formWidget)}else{var text=$scope.currentEditor?$scope.currentEditor.getValue():"";var validFunction=$scope.inFileMode()?isValidJs:isValidJson;state={valid:isValid(text,validFunction),value:text}}return state}function activeEditorSession(text,cursorPosition,items){if($scope.currentEditor){cursorPosition=cursorPosition||{row:0,column:0};if(!getKeepSession(items.newItem,items.oldItem)){text=$scope.getTextAndOpenLastPath(text,items);clearSession(text)}else if($scope.currentEditor.getValue()!=text){$scope.currentEditor.setValue(text)}$scope.currentEditor.gotoLine(cursorPosition.row+1,cursorPosition.column);$scope.currentEditor.focus();$scope.textIsReady=true}}$scope.setAnnotations=setAnnotations;$scope.getActiveComponentState=getActiveComponentState;$scope.activeEditorSession=activeEditorSession;$scope.clearEditor=function(){if($scope.currentEditor){clearSession();$scope.currentEditor.setValue("");$scope.currentEditor.gotoLine(0,0)}};$scope.$watch(function(){return $element.children()[0]},function(){$scope.$evalAsync(function(){if($element.children()[0]){$scope.initEditor()}})});function watchActiveItem(){return $scope.activeItem?[$scope.activeItem.text,$scope.activeItem.path,$scope.currentMode]:[$scope.activeItem]}function activeItemHandler(newItem,oldItem){var activeItem=$scope.activeItem;if(activeItem){var text=activeItem.text;var annotations=activeItem.errorMessage?[activeItem.errorMessage]:[];if($scope.inFormMode()&&text){eventAggregator.publish("creatingForm",JSON.parse(text))}else if(text||text===""){setAnnotations(annotations);newItem=getItemAttributes(newItem);oldItem=getItemAttributes(oldItem);var items={newItem:newItem,oldItem:oldItem};activeEditorSession(text,activeItem.cursorPosition,items)}}$scope.refreshSearch(newItem.path,oldItem.path)}$scope.$watch(watchActiveItem,Util.buildDebounceMethod(activeItemHandler,200),true);Mousetrap.bind("mod+s",function(event){event.preventDefault();$scope.DevToolsCtrl.saveComponentChange()})}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhComponentsTree",function($timeout,templateManager,ElementToMetadataService,eventAggregator,ElementFileService,DevToolsService,DeveloperTreeService){return{restrict:"A",link:function($scope,$element,$attributes,$controller){$scope.lastScript="";var ACTIVE_ITEM_CLASS="active-item";var ERROR_ITEM_CLASS="error-validation";var FILE_TYPE="files";var JSON_TYPE="json";var MENU_TYPE="menu";var sidebar=$element;var invalidItems=[];var unloadedInvalidItems=[];var DevToolsWindow=window;var subitemsPropertiesMap=UtilDevTools.getZeedhiTypesSubitems();function getItemMetadataByPath(path){var container=DeveloperTreeService.getTreeItems($scope.getCurrentMode());return ElementToMetadataService.getJsonElement(path,container[0])}$scope.getItemMetadataByPath=getItemMetadataByPath;function getParent(element,elementType,position){var classElement=".tree-item."+elementType;var child=$(element.children("ul"));var parents=child.find("span"+classElement);parents=parents.length?parents.parent():child.children("li");return $(parents[position])}function setTreeItemAsSelected(item){var element=item._element;if(element){var unRegister=$scope.$watch(function(){return element.hasClass(getItemType(item))||element.hasClass(getFileSystemType(item))},function(){if(element.hasClass(getItemType(item))||element.hasClass(getFileSystemType(item))){$("ul#containers-list").find("."+ACTIVE_ITEM_CLASS).removeClass(ACTIVE_ITEM_CLASS);element.addClass(ACTIVE_ITEM_CLASS);unRegister()}})}
}var loadInvalidItems=Util.buildDebounceMethod(function(){unloadedInvalidItems.forEach(function(invalidItem){addInvalidItem(invalidItem)});unloadedInvalidItems=[]},200);function setActiveModeTab(){$(".tabs").find(".active").removeClass("active");$element.find("#"+$scope.currentMode).addClass("active");if($scope.inJsonMode()){loadInvalidItems()}else if($scope.inFileMode()){unloadedInvalidItems=invalidItems;invalidItems=[]}}var removeExtraProperties=UtilDevTools.buildInCustomizationModeMethod(function removeExtraProperties(json){var zeedhiTypes=UtilDevTools.getZeedhiTypes().concat(UtilDevTools.getIgnoredZeedhiTypes());zeedhiTypes.forEach(function deleteSubitems(type){delete json[type]});delete json.id});function clearJSONAndStringify(json){if(json){removeExtraProperties(json);delete json.__indexPath;delete json.fileName;var cleanJSON=ShowInDevTools.clear(json);return JSON.stringify(cleanJSON,null,"	")}}function getAnnotationMessage(element){element=Array.isArray(element)&&element[0]?element[0]:null;var message=element?element.scope().errorMessage:"";if(message){message={row:0,text:message,type:"warning"}}return message}function getItemText(itemMetadata){var text=$scope.inFileMode()?itemMetadata.fileElement:itemMetadata.jsonElement;return $scope.inFileMode()?text:clearJSONAndStringify(Util.clone(text))}$scope.getItemText=getItemText;function isSamePath(activePath,newPath){var hasActivePathInNewPath=newPath.indexOf(activePath)>=0;return!activePath||hasActivePathInNewPath}function getProperties(item){var type=getItemType(item);var properties=type?subitemsPropertiesMap[type]:[];if(!type){var files=[];var folders=[];var exceptions=["parent","parentMenu","$$hashKey","_element","_opened","__metadata"];Object.keys(item).forEach(function(key){if(Util.isObject(item[key])&&item[key].isFolder){folders.push(key)}else if(exceptions.indexOf(key)<0){files.push(key)}});properties=folders.concat(files)}return properties}function getElement(path,parent){parent=parent?parent:$(".tree-item.container").parent();var parentElement=parent.children("span");if(path.length>=2){path=Util.clone(path);var elementType=path.shift().slice(0,-1);var position=path.shift();var newParent=getParent(parent,elementType,position);parentElement=getElement(path,newParent)}return parentElement}function openLastPath(){var lastItem=DeveloperTreeService.getLastItem($scope.getCurrentMode());if(lastItem&&lastItem.path){$scope.openPath(lastItem.path)}}function validModes(newMode,oldMode){var modeChange=newMode!=oldMode;var hasModes=newMode&&oldMode;var fromForm=newMode=="json"&&oldMode=="form";return hasModes&&modeChange&&!fromForm}function forceModeFile(activeMode){return activeMode=="css"||activeMode=="html"?"files":activeMode}function getTextAndOpenLastPath(text,items){var newMode=forceModeFile(items.newItem.mode);var oldMode=forceModeFile(items.oldItem.mode);if(validModes(newMode,oldMode)){openLastPath();text=""}return text}$scope.getTextAndOpenLastPath=getTextAndOpenLastPath;function unsetElementOfTree(obj){if(typeof obj=="object"){delete obj._element;getProperties(obj).forEach(function(property){if(Array.isArray(obj[property])){obj[property].forEach(function(child){unsetElementOfTree(child)})}else if(typeof obj[property]=="object"){unsetElementOfTree(obj[property])}})}}function itemHasChildren(item){return getSubitems(item).length>0}$scope.itemHasChildren=itemHasChildren;function replaceElementOfActiveItem(){var item=$scope.activeItem.item;var needReplace=$scope.activeItem.cursorPosition||item._element&&item._element.is(":hidden");if(!item.isFolder&&item._element&&needReplace){var element=getElement($scope.activeItem.path);item._element=element.length>0?element:item._element}}function getModeClass(path){return path.substring(path.lastIndexOf(".")+1,path.length)}function updateClassMode(indexPath){var classMode=getModeClass(indexPath);classMode=classMode=="js"?"javascript":classMode;$scope.currentModeClass=ace.require("ace/mode/".concat(classMode)).Mode}function setActiveItemObject(item,itemMetadata,cursorPosition){var path=itemMetadata.path;var text=getItemText(itemMetadata);var indexPath=DeveloperTreeService.getIndexPath($scope.getCurrentMode(),itemMetadata);var annotationMessage=getAnnotationMessage(item._element);$scope.lastScript=text;$scope.activeItem={item:item,text:text,path:path,errorMessage:annotationMessage,cursorPosition:cursorPosition,_indexPath:indexPath};updateClassMode(indexPath);var itemParent=!$scope.inFileMode()?DevToolsWindow.currentContainer.name:"";DeveloperTreeService.setActiveItem($scope.activeItem,$scope.getCurrentMode(),itemParent);replaceElementOfActiveItem();setTreeItemAsSelected(item)}function setActiveFolder(item){var activeFolder=false;if(item.isFolder){var pathSeparator="/";var path=item.url.split(pathSeparator);var severalPaths=item.url.indexOf(pathSeparator)>=0;if(severalPaths){path.shift()}activeFolder={name:item.name,path:path,mode:FILE_TYPE,parent:""};DeveloperTreeService.setLastItem(FILE_TYPE,activeFolder)}$scope.activeFolder=activeFolder}function setActiveItem(item,itemMetadata,cursorPosition){setActiveFolder(item);if(!item.isFolder){itemMetadata=itemMetadata||DeveloperTreeService.getItemMetadata(item,$scope.getCurrentMode());if(itemMetadata){if(itemMetadata.then){itemMetadata.then(function(obj){setActiveItemObject(item,obj,cursorPosition)})}else if(itemMetadata.path){setActiveItemObject(item,itemMetadata,cursorPosition)}}}}$scope.setActiveItem=setActiveItem;function updateActiveItem(paths){var activeItem=getActiveItem();if(activeItem){var activePath=activeItem.path.join("/");paths.some(function(path){if(isSamePath(activePath,path)){var activeItemMetadata=getItemMetadataByPath(activeItem.path);var cursorPosition=$scope.currentEditor.getCursorPosition();setActiveItem(activeItem.item,activeItemMetadata,cursorPosition);return true}else{return false}})}}$scope.updateActiveItem=updateActiveItem;function getActiveItem(){return $scope.activeItem}$scope.getActiveItem=getActiveItem;function getItemType(item){return ElementToMetadataService.getPropertyFromScope(item)}$scope.getItemType=getItemType;function getFileSystemType(item){var icon="code-file";item=item||{};if(!item.isFile){icon=item._opened?"open-folder":"closed-folder"}return icon}$scope.getFileSystemType=getFileSystemType;function getSubitems(item){var subitems=[];if(item){getProperties(item).forEach(function(property){if(item[property]&&Array.isArray(item[property])){subitems=subitems.concat(item[property])}else if(typeof item[property]=="object"){subitems.push(item[property])}})}return subitems.filter(function(subitem){return subitem.showInDevTools})}$scope.getSubitems=getSubitems;function toggleItem(item,DevToolsController,cursorPosition){if(item){if(itemHasChildren(item)){item._opened=!item._opened;templateManager.updateTemplate()}if(!item.isFolder){if(DevToolsController&&$scope.textIsReady){DevToolsController.saveComponentChange()}$scope.textIsReady=false}setActiveItem(item,false,cursorPosition)}}$scope.toggleItem=toggleItem;function getItems(){return DeveloperTreeService.getTreeItems($scope.getCurrentMode())}$scope.getItems=getItems;function restoreJSONTreeToDefaultState(tree){if(!$scope.inFileMode||!$scope.inFileMode()){$scope.activeItem=undefined;DevToolsService.clearActiveItem();$scope.closeSubtree(tree);sidebar.find("."+ACTIVE_ITEM_CLASS).removeClass(ACTIVE_ITEM_CLASS)}DeveloperTreeService.setLastItem(JSON_TYPE,undefined)}function addInvalidItem(errorObject){var path=Util.clone(errorObject.path);var element=getElement(path);if($scope.inFileMode()||!element){unloadedInvalidItems.push(errorObject)}else{var invalidItem={path:path,element:element,errorMessage:errorObject.errorMessage};invalidItems.push(invalidItem);element.addClass(ERROR_ITEM_CLASS);if(element.scope()){element.scope().errorMessage=invalidItem.errorMessage}if(element.hasClass(ACTIVE_ITEM_CLASS)){var annotation=getAnnotationMessage(element);$scope.setAnnotations([annotation])}}}$scope.addInvalidItem=addInvalidItem;function clearInvalidItems(){invalidItems.forEach(function(item){item.element.removeClass(ERROR_ITEM_CLASS);if(item.element.scope()){item.element.scope().errorMessage=""}});invalidItems=[];$scope.setAnnotations([]);requestAnimationFrame(function(){})}$scope.clearInvalidItems=clearInvalidItems;$scope.setInvalidItems=function(errorsObject){clearInvalidItems();errorsObject.forEach(function(object){addInvalidItem(object)})};$scope.reloadErrorsValidationInTree=Util.buildDebounceMethod(function(){invalidItems.forEach(function(item){item.element.addClass(ERROR_ITEM_CLASS)})},200);$scope.closeSubtree=function(item){var subitems=getSubitems(item);subitems.forEach(function(child){$scope.closeSubtree(child)});if(itemHasChildren(item)){item._opened=false}templateManager.updateTemplate()};$scope.openPath=function(pathToOpen,DevToolsController,cursorPosition){DeveloperTreeService.loadEmptyTrees();var currentMode=$scope.getCurrentMode();var lastItem=DeveloperTreeService.getTreeItems(currentMode)[0];$scope.closeSubtree(lastItem);if(pathToOpen&&pathToOpen.length>=0&&lastItem){if(pathToOpen.length===0){lastItem._opened=true}else{lastItem=DeveloperTreeService.openTree(currentMode,pathToOpen,lastItem)}toggleItem(lastItem,DevToolsController,cursorPosition)}};$scope.getActiveComponentPath=function(){return $scope.activeItem?$scope.activeItem.path:null};$scope.clearLastItem=function(){DeveloperTreeService.clearLastItems()};function toggleSideMenu(){var isSideMenuOpen=UtilDevTools.getMainWindow().executeFunction("ScreenService","isSideMenuOpen");var openSideMenu=$scope.inMenuMode()&&!isSideMenuOpen;var closeSideMenu=$scope.lastMode=="menu"&&isSideMenuOpen;if(openSideMenu||closeSideMenu){UtilDevTools.getMainWindow().executeFunction("ScreenService","toggleSideMenu")}}$scope.$watch("currentMode",function(){var formModeIsNotRelated=$scope.lastMode&&$scope.lastMode!="form"&&!$scope.inFormMode();if(formModeIsNotRelated){unsetElementOfTree(DeveloperTreeService.getTreeItems($scope.lastMode))}if($scope.inFileMode()){DeveloperTreeService.updateTree(FILE_TYPE)}toggleSideMenu();DeveloperTreeService.restoreUnloadedItems()});function initMode(){var currentMode=$scope.getCurrentMode();var activeItem=DevToolsService.getLocalActiveItem()||{};var lastActiveItem=DeveloperTreeService.getLastItem(currentMode);var path=activeItem.mode==currentMode?activeItem.path:[];path=path||[];if(lastActiveItem&&Object.keys(lastActiveItem).length>0){path=lastActiveItem.path||path}else{$scope.activeItem=undefined}$scope.textIsReady=false;$scope.openPath(path,$scope.DevToolsCtrl,activeItem.cursorPosition)}eventAggregator.subscribe("finishedRenderTreeItems",function(){initMode();setActiveModeTab()});var oldValue=window.currentContainer;var auxValue;Object.defineProperty(window,"currentContainer",{set:function(newValue){DeveloperTreeService.updateTree(JSON_TYPE,newValue);if(newValue){restoreJSONTreeToDefaultState(newValue);if($scope.initEditor)$scope.initEditor();if($scope.clearEditor)$scope.clearEditor();$timeout(function(){$scope.$apply()},0,false);auxValue=newValue}},get:function(){return auxValue}});window.currentContainer=oldValue}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhCreateFile",function(ScreenService,templateManager,restEngine,ZhKeyboardService,ShortcutDeveloperService,DTMessages){return{link:function($scope,$element,$attributes,$controller){var createFile=false;var defaultContainer;var WIDGET_NAME="widgetToNewFile";function getActiveJsonPath(){var defaultPath=$scope.DevToolsCtrl.getActiveTreeItem();defaultPath=defaultPath._indexPath;defaultPath=defaultPath.slice(0,defaultPath.lastIndexOf("/")+1).replace(".","");return defaultPath}function setPathCreateFile(){$scope.widget.getField("pathCreateFile").value(getActiveJsonPath())}function setDefaultContainer(){var jsonLocation=templateManager.getDocsUrl("defaultJsonValue.json");restEngine.requestMetaData(jsonLocation).then(function(data){defaultContainer=JSON.stringify(data.data[0].container,null,"	")})}function getFileName(widget){var fileName=widget.getField("nameCreateFile").value()||"";fileName=fileName.trim();var isJsonName=fileName.lastIndexOf(".")==fileName.toLowerCase().indexOf(".json");fileName=isJsonName&&fileName.indexOf(".")>0?fileName:"";if(fileName===""){ScreenService.notificationMessage(DTMessages.FILE_INVALID_NAME,"error")}return fileName}function getLocalPath(widget){var localPath=widget.getField("pathCreateFile").value();if(localPath){localPath=localPath.trim();localPath.replace("\\","/");if(localPath[0]!="/"){localPath="/"+localPath}if(localPath[localPath.length-1]!="/"){localPath+="/"}}else{ScreenService.notificationMessage(DTMessages.FILE_INVALID_PATH,"error")}return localPath}function hideCreateFile(){createFile=false}$scope.hideCreateFile=hideCreateFile;$scope.isToCreateFile=function(){return createFile&&$scope.inJsonMode()};$scope.createNewFile=function(){var localPath=getLocalPath($scope.widget);if(localPath){var fileName=getFileName($scope.widget);if(fileName){var pathToCreate=localPath+fileName;var obj={pathToCreate:pathToCreate,fileName:fileName,defaultContainer:defaultContainer};$scope.DevToolsCtrl.newFile(obj);createFile=false}}};function onCreateFile(){createFile=!$scope.inJsonMode()?false:!createFile;return false}$scope.onCreateFile=onCreateFile;ZhKeyboardService.bind("mod+b",onCreateFile);$scope.$watch(function(){return $scope.isToCreateFile()},function(){$scope.DevToolsCtrl.changeHeightOfCodeEditor();if(createFile){$scope.DevToolsCtrl.openShortcutWidget(WIDGET_NAME);setPathCreateFile()}});function buildCreateFileWidget(){var FIELDS_CONFIG={pathCreateFile:{"class":"05 zh-dvtools",template:"field/text-edit.html",placeholder:"path",events:[{name:"FieldOnKeyup",code:"DevToolsController.createNewFile(args.e.keyCode);",id:+new Date}]},nameCreateFile:{"class":"05 zh-dvtools",template:"field/text-edit.html",focused:true,placeholder:"fileName.json",events:[{name:"FieldOnKeyup",code:"DevToolsController.createNewFile(args.e.keyCode);",id:+new Date}]},confirmCreateFile:{"class":"01 zh-dvtools",template:"field/button.html",label:"✓",events:[{name:"FieldOnClick",code:"DevToolsController.createNewFile();",id:+new Date}]},hideCreateFile:{"class":"01 zh-dvtools",template:"field/button.html",label:"✗",events:[{name:"FieldOnClick",code:"DevToolsController.hideCreateFile(args);",id:+new Date}]}};ShortcutDeveloperService.createWidgetAndContainer(WIDGET_NAME,FIELDS_CONFIG,hideCreateFile)}buildCreateFileWidget();setDefaultContainer()}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhDeleteComponents",function(ApplicationContext,templateManager,restEngine,ScreenService){var DELETE="DELETE";var DELETEWIDGET="DELETEWIDGET";return{link:function($scope,$element,$attributes,$controller){function isDevWindow(){return ApplicationContext.WindowController.isDevToolsWindow()}function deleteComponent(DevToolsController){var activeItemPath=DevToolsController.getActiveTreeItem().path;var type=activeItemPath[activeItemPath.length-2];if(type&&isDevWindow()){var completeJson=Util.clone(window.currentContainer.__metadata);ShowInDevTools.clear(completeJson);var deleteType=DELETE;if(type=="widgets"){deleteType=DELETEWIDGET}DevToolsController.deleteComponent(activeItemPath,completeJson,deleteType);setItemToOpen(activeItemPath,DevToolsController)}else if(!activeItemPath.length){ScreenService.notificationMessage("Impossível deletar um container dessa maneira.","information")}}$scope.deleteComponent=deleteComponent;function setItemToOpen(activeItemPath,DevToolsController){if(activeItemPath[activeItemPath.length-1]===0){activeItemPath.length=activeItemPath.length-2}else{var lastPosition=activeItemPath.length-1;activeItemPath[lastPosition]=activeItemPath[lastPosition]-1;if(activeItemPath[lastPosition]<0){activeItemPath[lastPosition]=0}}$scope.textIsReady=false;DevToolsController.openPath(activeItemPath)}}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhFindLine",function(templateManager,ZhKeyboardService,ShortcutDeveloperService){return{link:function($scope,$element,$attributes,$controller){var toFindLine=false;var ENTER_KEY_CODE=13;var ESC_KEY_CODE=27;var WIDGET_NAME="widgetOfFindLine";function hideFindLine(){toFindLine=false;templateManager.updateTemplate()}$scope.isToFindLine=function(){return toFindLine&&!$scope.inFormMode()};$scope.findLine=function(args){var key=args.e.keyCode;var line=$scope.widget.getField("findLine").value();if(line&&(key<37||key>40)){line=line-1;$scope.currentEditor.gotoLine(line,0);$scope.currentEditor.selection.moveTo(line,0);$scope.currentEditor.centerSelection()}if(key==ENTER_KEY_CODE||key==ESC_KEY_CODE){hideFindLine()}};function changeToFindLineText(event,key){toFindLine=$scope.inFormMode()?false:!toFindLine;return false}ZhKeyboardService.bind("mod+g",changeToFindLineText);$scope.changeToFindLineText=changeToFindLineText;$scope.$watch(function(){return $scope.isToFindLine()},function(){if(toFindLine){$scope.DevToolsCtrl.openShortcutWidget(WIDGET_NAME)}});function createFindLineWidget(){var FIELDS_CONFIG={findLine:{"class":"12 zh-find",template:"field/number.html",placeholder:"00",focused:true,events:[{name:"FieldOnKeyup",code:"DevToolsController.findLine(args);",id:+new Date}]}};ShortcutDeveloperService.createWidgetAndContainer(WIDGET_NAME,FIELDS_CONFIG,hideFindLine)}createFindLineWidget()}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhForm",function(templateManager,eventAggregator,DeveloperToolsForm){return{link:function($scope,$element,$attributes,$controller){var preparedDependencies;function setInitialDependencieFieldsVisibility(valuesInForm){Object.keys(preparedDependencies.dependsOn).forEach(function(fieldName,algo){var field=$scope.widget.getField(fieldName);if(field){var expectedValues={invalids:[]};var dependsOn=field.dependsOn;Object.keys(dependsOn).forEach(function(fieldDependName){expectedValues[fieldDependName]=dependsOn[fieldDependName];valuesInForm[fieldDependName]=valuesInForm[fieldDependName]===undefined?false:valuesInForm[fieldDependName];if(dependsOn[fieldDependName]!==valuesInForm[fieldDependName]){field.isVisible=false;expectedValues.invalids.push(fieldDependName)}});field.expectedValues=expectedValues}})}function setJsonValuesInFields(jsonItem){var valuesInForm={};$scope.widget.fields.forEach(function(field){var fieldName=field.name;var value=jsonItem[fieldName];if(!DeveloperToolsForm.hasPropertyInJson(jsonItem,fieldName)&&field.hasOwnProperty("defaultValue")){value=field.defaultValue}else if(DeveloperToolsForm.isCheckboxField(field.template)){field.saveValue=value;value=!!field.saveValue}field.value(value);valuesInForm[fieldName]=value});if(preparedDependencies.dependsOn){setInitialDependencieFieldsVisibility(valuesInForm)}}function createFormWidgetAndContainer(fields,docTemplate,jsonItem){var formAndContainer=DeveloperToolsForm.getFormWidgetAndContainer(fields,docTemplate,jsonItem,preparedDependencies);$scope.formWidget=formAndContainer.widget;$scope.widget=formAndContainer.widget;$scope.templateManager.container=formAndContainer.container;if(jsonItem){setJsonValuesInFields(jsonItem)}}$scope.createFormWidgetAndContainer=createFormWidgetAndContainer;var getDocAndCreateForm=Util.buildDebounceMethod(function(item){var jsonItem=item;DeveloperToolsForm.getFormDocTemplate(jsonItem.template).then(function(docTemplate){if(docTemplate){DeveloperToolsForm.getFormDocTemplate(jsonItem.template,true).then(function(options){var keysConcat=Object.keys(jsonItem).concat(Object.keys(docTemplate.properties));preparedDependencies=DeveloperToolsForm.getFormDependencies(docTemplate.dependencies,options);keysConcat=keysConcat.filter(function(value,valueIndex){return keysConcat.indexOf(value)==valueIndex&&value!="template"});createFormWidgetAndContainer(keysConcat,docTemplate,jsonItem,preparedDependencies)})}})},200,true);function updateFieldsOfDependency(fieldChanged){var fieldChangedName=fieldChanged.name;fieldChanged.dependencieFields.forEach(function(fieldName){var dependencieField=$scope.widget.getField(fieldName);if(dependencieField){var expectedValues=dependencieField.expectedValues;var invalidValues=expectedValues.invalids;var fieldIndexInInvalids=invalidValues.indexOf(fieldChangedName);var hasFieldInInvalids=fieldIndexInInvalids>=0;var fieldValueIsValid=fieldChanged.value()===expectedValues[fieldChangedName];if(fieldValueIsValid&&fieldChanged.isVisible){if(hasFieldInInvalids){invalidValues.splice(fieldIndexInInvalids,1)}}else if(!hasFieldInInvalids){invalidValues.push(fieldChangedName)}dependencieField.isVisible=!invalidValues.length;if(dependencieField.dependencieFields){updateFieldsOfDependency(dependencieField)}}})}eventAggregator.subscribe("creatingForm",function(item){getDocAndCreateForm(item)});eventAggregator.subscribe("updateDependencyFieldsVisibility",function(widgetAndFields){updateFieldsOfDependency(widgetAndFields)})}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhHideDeveloperTree",function(eventAggregator,templateManager){return{link:function($scope,$element,$attributes,$controller){var CODE_ELEMENT_SELECTOR=".editor";var TREE_ELEMENT_SELECTOR=".components-tree";var SHOW="show";var HIDE="hide";var visibilityOfTree;var treeIsHidden=function(){return visibilityOfTree===HIDE};$scope.treeIsHidden=treeIsHidden;var getElement=function(elementName){return $element.find(elementName)};var setWidthInElement=function(elementName,width){getElement(elementName).width(width);templateManager.updateTemplate()};var changeCodeContainerWidth=function(){var currentWidth=window.innerWidth;var widthOfComponentsTree=200;var finalWidth=treeIsHidden()?currentWidth-1:currentWidth-widthOfComponentsTree;setWidthInElement(CODE_ELEMENT_SELECTOR,finalWidth)};var changeTreeVisibility=function(){var widthOfTree=199;var finalWidth=treeIsHidden()?0:widthOfTree;setWidthInElement(TREE_ELEMENT_SELECTOR,finalWidth)};eventAggregator.subscribe("developerEnableHideMode",function(args){visibilityOfTree=args?HIDE:SHOW;changeCodeContainerWidth();changeTreeVisibility()});$(window).resize(function(){var width=window.innerWidth;var treeWidth=getElement(TREE_ELEMENT_SELECTOR).width()+1;width=width-treeWidth;setWidthInElement(CODE_ELEMENT_SELECTOR,width)})}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhInspection",function($timeout,eventEngine,ApplicationContext,$window,eventAggregator,ElementToMetadataService){return{link:function($scope,$element,$attrs,$controller){var DEVELOPER_INSPECTION_MODE_CLASS="developer-inspection";var getBody=function(){return $window.document.body};var addEditingClass=function(element){$(element).addClass(DEVELOPER_INSPECTION_MODE_CLASS)};var removeEditingClass=function(element){$(element).removeClass(DEVELOPER_INSPECTION_MODE_CLASS)};var getWindowController=function(){return ApplicationContext.WindowController};var sendElementPathToDevTools=function(scope){var pathToMetadata=ElementToMetadataService.getPathToMetadata(scope);getWindowController().sendToDevTools(pathToMetadata,"json")};var openWidgetToEdit=function(elem){var body=getBody();var $scope=angular.element(elem.target).scope();body.removeEventListener("click",openWidgetToEdit);sendElementPathToDevTools($scope);eventEngine.restoreDispatchEvent();removeEditingClass(body)};var enableEditMode=function(){var body=getBody();$timeout(function(){body.addEventListener("click",openWidgetToEdit)});addEditingClass(body);eventEngine.stopDispatchEvent("Entering in edit mode.")};var disableEditMode=function(){var body=getBody();body.removeEventListener("click",openWidgetToEdit);removeEditingClass(body);eventEngine.restoreDispatchEvent()};var runDeveloperEditMode=UtilDevTools.buildMainWindowMethod(function(args){window.inspection=!window.inspection;eventAggregator.publish("developerEnableEditMode",{enableEditMode:window.inspection})});Mousetrap.bind("mod+f11",runDeveloperEditMode);eventAggregator.subscribe("developerEnableEditMode",function(message){var enable=message?message.enableEditMode:true;if(enable){enableEditMode()}else{disableEditMode()}})}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhNewComponents",function(templateManager,DevToolsService){return{link:function($scope){function getDefaultValue(type){var getJsonLocation=templateManager.getDocsUrl("defaultJsonValue.json");return DevToolsService.requestMetadataDeveloper(getJsonLocation).then(function(data){return JSON.stringify(data.data[0][type])})}function addNewComponent(DevToolsController){var activeItemPath=DevToolsController.getActiveTreeItem().path;var type=activeItemPath[activeItemPath.length-2];if(type){var completeJson=Util.clone(window.currentContainer.__metadata);ShowInDevTools.clear(completeJson);getDefaultValue(type).then(function(defaultJsonValue){DevToolsController.saveNewComponent(defaultJsonValue,activeItemPath,completeJson)})}}$scope.addNewComponent=UtilDevTools.buildDevToolsWindowMethod(addNewComponent)}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhSearch",function(templateManager,ZhKeyboardService,ShortcutDeveloperService){return{link:function($scope,$element,$attrs,$controller){var toSearchText=false;var ENTER_KEY_CODE=13;var ESC_KEY_CODE=27;var WIDGET_NAME="widgetOfSearch";var OPTION_NEXT="next";var OPTION_REFRESH="refresh";var OPTION_PREV="prev";var OPTION_REPLACE="replace";var OPTION_REPLACE_ALL="replaceAll";var SEARCH_FUNCTIONS={next:getNextPosition,prev:getPrevPosition,refresh:getUpdatedPosition};var samePath=false;function getPositionsOfWord(value,textToSearch,rowIndex){var positions=[];textToSearch=textToSearch.replace(/\\/g,"\\\\");var regex=new RegExp(textToSearch,"g");value.replace(regex,function(valueInArray,index){positions.push({row:rowIndex,column:index})});return positions}function getTextPositions(textToSearch){var text=$scope.currentEditor.getValue();var items=[];if(text){if(!$scope.widget.getField("caseSensitive").value()){text=text.toLowerCase();textToSearch=textToSearch.toLowerCase()}text.split("\n").forEach(function(value,index){if(value.indexOf(textToSearch)>=0){items=items.concat(getPositionsOfWord(value,textToSearch,index))}})}return items}function getPrevPosition(items,row,column,text){column=column-text.length;var lastIndex=items.length-1;var lastItemInText=items[lastIndex];var prevItem={};var prevItemWasFound=items.reverse().some(function(value,index){prevItem=value;prevItem.index=lastIndex--;return value.row==row&&value.column<column+1||value.row<row});return prevItemWasFound?prevItem:lastItemInText}function getNextPosition(items,row,column){var nextItem;var wasFound=!items.every(function(value,index){nextItem=value;nextItem.index=index;return!(value.row==row&&value.column>column||value.row>row)});return wasFound?nextItem:items[0]}function getUpdatedPosition(items,row,column){var positions={row:row,column:column,index:-1};if(samePath){items.every(function(itemPosition,index){var itemWasBeforeCursor=itemPosition.row<row||itemPosition.row==row&&itemPosition.column<=column;if(itemWasBeforeCursor){positions.index=index}return itemWasBeforeCursor})}return positions}function getPositionOfNewWord(positions,row){var column=$scope.currentEditor.selection.getAllRanges()[0].start.column;var position;var wasFound=!positions.every(function(value,index){position=value;position.index=index;return!(value.row>row||value.row==row&&value.column>=column)});return wasFound?position:positions[0]}function getTextRowAndColumn(positions,option,textToSearch,key){var cursorPosition=$scope.currentEditor.getCursorPosition();var findIn=getPositionOfNewWord;if(!key||key==ENTER_KEY_CODE){option=option||OPTION_NEXT;findIn=SEARCH_FUNCTIONS[option]}return findIn(positions,cursorPosition.row,cursorPosition.column-1,textToSearch)}function selectTextInEditor(row,column,textToSearch){$scope.currentEditor.gotoLine(row+1,column);$scope.currentEditor.selection.selectTo(row,textToSearch.length+column);$scope.currentEditor.centerSelection()}function setFoundsFieldPlaceholder(index,total){var foundsField=$scope.widget.getField("founds");foundsField.placeholder=index+" of "+total;templateManager.updateTemplate()}function findText(textToSearch,option,key){var positions=getTextPositions(textToSearch);var positionsLength=positions.length;var index=0;if(positionsLength>0){var rowAndColumn=getTextRowAndColumn(positions,option,textToSearch,key);if(option!=OPTION_REFRESH){selectTextInEditor(rowAndColumn.row,rowAndColumn.column,textToSearch)}index=rowAndColumn.index+1}setFoundsFieldPlaceholder(index,positionsLength);templateManager.updateTemplate()}function hideSearch(){toSearchText=false;templateManager.updateTemplate()}$scope.isToSearchText=function(){return toSearchText&&!$scope.inFormMode()};$scope.refreshSearch=function(newItemPath,oldItemPath){samePath=newItemPath==oldItemPath;if($scope.isToSearchText()){$scope.findTextInEditor(OPTION_REFRESH)}};function markTextInTextEditor(option,key){var text=$scope.widget.getField("textToSearch").value();if(key==ESC_KEY_CODE){hideSearch()}else if(!text){setFoundsFieldPlaceholder(0,0)}else if(!key||key<37||key>40){findText(text,option,key)}}$scope.markTextInTextEditor=markTextInTextEditor;$scope.findTextInEditor=Util.buildDebounceMethod(markTextInTextEditor,200);function changeToSearchText(event,key){toSearchText=$scope.inFormMode()?false:!toSearchText;templateManager.updateTemplate();return false}ZhKeyboardService.bind("mod+f",changeToSearchText);ZhKeyboardService.bind("mod+r",changeToSearchText);$scope.changeToSearchText=changeToSearchText;$scope.$watch(function(){return $scope.isToSearchText()},function(){$scope.DevToolsCtrl.changeHeightOfCodeEditor();if(toSearchText){$scope.DevToolsCtrl.openShortcutWidget(WIDGET_NAME)}});function createSearchWidget(){var FIELDS_CONFIG={textToSearch:{"class":"05 zh-dvtools",template:"field/text-edit.html",placeholder:"Find",focused:true,events:[{name:"FieldOnKeyup",code:"DevToolsController.findTextInEditor(args,'"+OPTION_NEXT+"');",id:+new Date}]},searchUp:{"class":"01 zh-dvtools",template:"field/button.html",label:"<",events:[{name:"FieldOnClick",code:"DevToolsController.findTextInEditor(args,'"+OPTION_PREV+"');",id:+new Date}]},searchDown:{"class":"01 zh-dvtools",template:"field/button.html",label:">",events:[{name:"FieldOnClick",code:"DevToolsController.findTextInEditor(args, '"+OPTION_NEXT+"');",id:+new Date}]},caseSensitive:{"class":"01 zh-dvtools",template:"field/button.html",label:"Aa",events:[{name:"FieldOnClick",code:"DevToolsController.changeCaseSensitive();",id:+new Date}]},founds:{"class":"02 zh-dvtools",template:"field/text-edit.html",readOnly:true,placeholder:"0 of 0",events:[]},textToReplace:{"class":"05 zh-dvtools",template:"field/text-edit.html",placeholder:"Replace with",focused:true,events:[{name:"FieldOnKeyup",code:"DevToolsController.findTextInEditor(args,'"+OPTION_NEXT+"');",id:+new Date}]},replaceOk:{"class":"01 zh-dvtools",template:"field/button.html",label:"✓",events:[{name:"FieldOnClick",code:"DevToolsController.replaceText();",id:+new Date}]},replaceAll:{"class":"01 zh-dvtools",template:"field/button.html",label:"all",events:[{name:"FieldOnClick",code:"DevToolsController.replaceAllText();",id:+new Date}]}};ShortcutDeveloperService.createWidgetAndContainer(WIDGET_NAME,FIELDS_CONFIG,hideSearch)}createSearchWidget()}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhTabEditor",function(eventAggregator,ScreenService,templateManager,DevToolsService){return{link:function($scope,$element,$attributes,$controller){var developerCommands=["Ctrl-F","Command-F","Ctrl-G","Command-G","Ctrl-B","Command-B"];var MODE_JSON="json";var MODE_FORM="form";var MODE_MENU="menu";var MODE_JS="files";ace.config.setDefaultValue("session","useWorker",false);$scope.currentMode=getInitialCurrentMode();$scope.currentEditor=null;$scope.currentModeClass=null;$scope.lastMode="";function getInitialCurrentMode(){
var mode=MODE_JSON;var localDevtools=DevToolsService.getLocalDevToolsFlag();var localActiveItem=DevToolsService.getLocalActiveItem();if(localDevtools&&localActiveItem&&localActiveItem.mode){mode=localActiveItem.mode==MODE_FORM?MODE_JSON:localActiveItem.mode}return mode}function disableAceCommands(){developerCommands.forEach(function(command){$scope.currentEditor.commands.bindKey(command,null)})}function updateCurrentEditor(){var codeEditor=inFileMode()?"code-editor-js":"code-editor-json";var modeClass=inFileMode()?"ace/mode/javascript":"ace/mode/json";$scope.currentEditor=ace.edit(codeEditor);$scope.currentModeClass=ace.require(modeClass).Mode;disableAceCommands()}function actionIsVisibleInContainer(actionName){return actionName!="plus"&&actionName!="trash"}function setCurrentMode(mode,devTools){if($scope.currentMode!=mode){devTools.saveComponentChange();$scope.lastMode=$scope.currentMode;$scope.currentMode=mode}}$scope.setCurrentMode=setCurrentMode;function inFileMode(){return $scope.currentMode==MODE_JS}$scope.inFileMode=inFileMode;function inJsonMode(){return $scope.currentMode==MODE_JSON}$scope.inJsonMode=inJsonMode;function inFormMode(){return $scope.currentMode==MODE_FORM}$scope.inFormMode=inFormMode;function inMenuMode(){return $scope.currentMode==MODE_MENU}$scope.inMenuMode=inMenuMode;function closeMenu(DevToolsCtrl){var mode=$scope.lastMode||MODE_JSON;setCurrentMode(mode,DevToolsCtrl)}$scope.closeMenu=closeMenu;function initEditor(mode){$scope.currentMode=mode||$scope.currentMode;if(!inFormMode()){updateCurrentEditor();$scope.currentEditor.setTheme("ace/theme/chaos");$scope.currentEditor.getSession().setTabSize(2);$scope.currentEditor.session.setMode(new $scope.currentModeClass);$scope.currentEditor.$blockScrolling=Infinity;$scope.currentEditor.on("focus",function(){$element.find("textarea").addClass("mousetrap")});$scope.currentEditor.on("change",function(){if($scope.isToSearchText()){$scope.refreshSearch()}});$scope.DevToolsCtrl.changeHeightOfCodeEditor()}}$scope.initEditor=initEditor;$scope.getCurrentMode=function(){return $scope.currentMode||MODE_JSON};$scope.actionIsVisible=function(action,devTools){var activeItem=devTools.getActiveTreeItem();var condition=false;var template=activeItem?Util.getNestedProperty(activeItem,["item","template"]):undefined;if(template){var isContainer=~template.indexOf("container");condition=!isContainer||actionIsVisibleInContainer(action)}return condition};$scope.changeFormVisibility=function(devTools){var mode=inFormMode()?MODE_JSON:MODE_FORM;setCurrentMode(mode,devTools)};$scope.$watch("currentEditor",function(){var componentPath=$scope.getActiveComponentPath();if($scope.inJsonMode()&&$scope.lastMode==MODE_FORM&&componentPath){var text=$scope.getItemText($scope.getItemMetadataByPath(componentPath));$scope.activeItem.text=text}})}}});ZeedhiChart.directive("zhTreeItem",function(DeveloperTreeService){return{restrict:"A",link:function($scope,$element,$attributes,$controller){$scope.item._element=$element.find(".tree-item.wrapper");var DevWindow=window;var MainWindow=opener?opener:DevWindow.parent;function decrementContAndPublishEvents(cont){cont--;if(!cont){DevWindow.publishFinishedRenderTreeItems();MainWindow.publishDevToolsReady()}return cont}var unloadedItems=DeveloperTreeService.getUnloadedItems($scope.currentMode);DeveloperTreeService.setUnloadedItems($scope.currentMode,decrementContAndPublishEvents(unloadedItems))}}});var ZeedhiDirectives=angular.module("ZeedhiDirectives");ZeedhiDirectives.directive("zhSchemaValidator",function(templateManager,DevToolsService,ElementToMetadataService,ApplicationContext){return{link:function link($scope,element,attrs){function getWindowController(){return ApplicationContext.WindowController}function getAndFormatType(path){var type="container";var pathLength=path&&path.length;if(pathLength>=2){type=path[pathLength-2];var lastChar=type.length-1;if(type[lastChar]=="s"){type=type.substring(0,lastChar)}}return type}function getElementName(jsonElem,type){return jsonElem.label||jsonElem.name||jsonElem.id||type.toUpperCase()}function getFormatedDataPaht(dataPath){dataPath=dataPath||"";if(dataPath){dataPath=dataPath.replace(/^\//,"");dataPath="In "+dataPath+" - "}return dataPath}function formatErrorsMessage(errors,path,jsonElement){var type=getAndFormatType(path);var message="In "+type;message+=" "+getElementName(jsonElement,type);message+="("+path.join("/")+"): ";errors.forEach(function(error){var dataPath=getFormatedDataPaht(error.dataPath);message+="\n	"+dataPath+error.message});return message}function showErrorsInTree(path,message){var type=getAndFormatType(path);var object=$scope[type];if(object){var invalidItem={path:path,errorMessage:message};getWindowController().addInvalidItem(invalidItem)}}function showErrors(errors,path,jsonElement){var message=formatErrorsMessage(errors,path,jsonElement);console.warn(message);showErrorsInTree(path,message)}function clearDependencieErrors(errors,schema){errors=errors.filter(function(error){var propertieKeys="";if(schema.properties[error.params.missing]){propertieKeys=Object.keys(schema.properties[error.params.missing])}return error.code!=304||propertieKeys.indexOf("default")<0});return errors}var validateIfInDevMode=UtilDevTools.buildInDeveloperModeMethod(function(){var schemaFileName=attrs.zhSchemaValidator;var schemaPath=templateManager.getDocsUrl(schemaFileName);var metadataElement=ElementToMetadataService.getElementToMetadata($scope,templateManager.container);var jsonElement=metadataElement.jsonElement;if(jsonElement){DevToolsService.requestMetadataDeveloper(schemaPath).then(function(schema){var errors=tv4.validateMultiple(jsonElement,schema.data).errors||[];errors=clearDependencieErrors(errors,schema.data);if(errors.length){showErrors(errors,metadataElement.path,jsonElement)}})}});UtilDevTools.buildMainWindowMethod(validateIfInDevMode)()}}});var jsonlint=function(){var parser={trace:function trace(){},yy:{},symbols_:{error:2,JSONString:3,STRING:4,JSONNumber:5,NUMBER:6,JSONNullLiteral:7,NULL:8,JSONBooleanLiteral:9,TRUE:10,FALSE:11,JSONText:12,JSONValue:13,EOF:14,JSONObject:15,JSONArray:16,"{":17,"}":18,JSONMemberList:19,JSONMember:20,":":21,",":22,"[":23,"]":24,JSONElementList:25,$accept:0,$end:1},terminals_:{2:"error",4:"STRING",6:"NUMBER",8:"NULL",10:"TRUE",11:"FALSE",14:"EOF",17:"{",18:"}",21:":",22:",",23:"[",24:"]"},productions_:[0,[3,1],[5,1],[7,1],[9,1],[9,1],[12,2],[13,1],[13,1],[13,1],[13,1],[13,1],[13,1],[15,2],[15,3],[20,3],[19,1],[19,3],[16,2],[16,3],[25,1],[25,3]],performAction:function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$){var $0=$$.length-1;switch(yystate){case 1:this.$=yytext.replace(/\\(\\|")/g,"$"+"1").replace(/\\n/g,"\n").replace(/\\r/g,"\r").replace(/\\t/g,"	").replace(/\\v/g,"").replace(/\\f/g,"\f").replace(/\\b/g,"\b");break;case 2:this.$=Number(yytext);break;case 3:this.$=null;break;case 4:this.$=true;break;case 5:this.$=false;break;case 6:return this.$=$$[$0-1];break;case 13:this.$={};break;case 14:this.$=$$[$0-1];break;case 15:this.$=[$$[$0-2],$$[$0]];break;case 16:this.$={};this.$[$$[$0][0]]=$$[$0][1];break;case 17:this.$=$$[$0-2];$$[$0-2][$$[$0][0]]=$$[$0][1];break;case 18:this.$=[];break;case 19:this.$=$$[$0-1];break;case 20:this.$=[$$[$0]];break;case 21:this.$=$$[$0-2];$$[$0-2].push($$[$0]);break}},table:[{3:5,4:[1,12],5:6,6:[1,13],7:3,8:[1,9],9:4,10:[1,10],11:[1,11],12:1,13:2,15:7,16:8,17:[1,14],23:[1,15]},{1:[3]},{14:[1,16]},{14:[2,7],18:[2,7],22:[2,7],24:[2,7]},{14:[2,8],18:[2,8],22:[2,8],24:[2,8]},{14:[2,9],18:[2,9],22:[2,9],24:[2,9]},{14:[2,10],18:[2,10],22:[2,10],24:[2,10]},{14:[2,11],18:[2,11],22:[2,11],24:[2,11]},{14:[2,12],18:[2,12],22:[2,12],24:[2,12]},{14:[2,3],18:[2,3],22:[2,3],24:[2,3]},{14:[2,4],18:[2,4],22:[2,4],24:[2,4]},{14:[2,5],18:[2,5],22:[2,5],24:[2,5]},{14:[2,1],18:[2,1],21:[2,1],22:[2,1],24:[2,1]},{14:[2,2],18:[2,2],22:[2,2],24:[2,2]},{3:20,4:[1,12],18:[1,17],19:18,20:19},{3:5,4:[1,12],5:6,6:[1,13],7:3,8:[1,9],9:4,10:[1,10],11:[1,11],13:23,15:7,16:8,17:[1,14],23:[1,15],24:[1,21],25:22},{1:[2,6]},{14:[2,13],18:[2,13],22:[2,13],24:[2,13]},{18:[1,24],22:[1,25]},{18:[2,16],22:[2,16]},{21:[1,26]},{14:[2,18],18:[2,18],22:[2,18],24:[2,18]},{22:[1,28],24:[1,27]},{22:[2,20],24:[2,20]},{14:[2,14],18:[2,14],22:[2,14],24:[2,14]},{3:20,4:[1,12],20:29},{3:5,4:[1,12],5:6,6:[1,13],7:3,8:[1,9],9:4,10:[1,10],11:[1,11],13:30,15:7,16:8,17:[1,14],23:[1,15]},{14:[2,19],18:[2,19],22:[2,19],24:[2,19]},{3:5,4:[1,12],5:6,6:[1,13],7:3,8:[1,9],9:4,10:[1,10],11:[1,11],13:31,15:7,16:8,17:[1,14],23:[1,15]},{18:[2,17],22:[2,17]},{18:[2,15],22:[2,15]},{22:[2,21],24:[2,21]}],defaultActions:{16:[2,6]},parseError:function parseError(str,hash){throw new Error(str)},parse:function parse(input){var self=this,stack=[0],vstack=[null],lstack=[],table=this.table,yytext="",yylineno=0,yyleng=0,recovering=0,TERROR=2,EOF=1;this.lexer.setInput(input);this.lexer.yy=this.yy;this.yy.lexer=this.lexer;if(typeof this.lexer.yylloc=="undefined")this.lexer.yylloc={};var yyloc=this.lexer.yylloc;lstack.push(yyloc);if(typeof this.yy.parseError==="function")this.parseError=this.yy.parseError;function popStack(n){stack.length=stack.length-2*n;vstack.length=vstack.length-n;lstack.length=lstack.length-n}function lex(){var token;token=self.lexer.lex()||1;if(typeof token!=="number"){token=self.symbols_[token]||token}return token}var symbol,preErrorSymbol,state,action,a,r,yyval={},p,len,newState,expected;while(true){state=stack[stack.length-1];if(this.defaultActions[state]){action=this.defaultActions[state]}else{if(symbol==null)symbol=lex();action=table[state]&&table[state][symbol]}_handle_error:if(typeof action==="undefined"||!action.length||!action[0]){if(!recovering){expected=[];for(p in table[state])if(this.terminals_[p]&&p>2){expected.push("'"+this.terminals_[p]+"'")}var errStr="";if(this.lexer.showPosition){errStr="Parse error on line "+(yylineno+1)+":\n"+this.lexer.showPosition()+"\nExpecting "+expected.join(", ")+", got '"+this.terminals_[symbol]+"'"}else{errStr="Parse error on line "+(yylineno+1)+": Unexpected "+(symbol==1?"end of input":"'"+(this.terminals_[symbol]||symbol)+"'")}this.parseError(errStr,{text:this.lexer.match,token:this.terminals_[symbol]||symbol,line:this.lexer.yylineno,loc:yyloc,expected:expected})}if(recovering==3){if(symbol==EOF){throw new Error(errStr||"Parsing halted.")}yyleng=this.lexer.yyleng;yytext=this.lexer.yytext;yylineno=this.lexer.yylineno;yyloc=this.lexer.yylloc;symbol=lex()}while(1){if(TERROR.toString()in table[state]){break}if(state==0){throw new Error(errStr||"Parsing halted.")}popStack(1);state=stack[stack.length-1]}preErrorSymbol=symbol;symbol=TERROR;state=stack[stack.length-1];action=table[state]&&table[state][TERROR];recovering=3}if(action[0]instanceof Array&&action.length>1){throw new Error("Parse Error: multiple actions possible at state: "+state+", token: "+symbol)}switch(action[0]){case 1:stack.push(symbol);vstack.push(this.lexer.yytext);lstack.push(this.lexer.yylloc);stack.push(action[1]);symbol=null;if(!preErrorSymbol){yyleng=this.lexer.yyleng;yytext=this.lexer.yytext;yylineno=this.lexer.yylineno;yyloc=this.lexer.yylloc;if(recovering>0)recovering--}else{symbol=preErrorSymbol;preErrorSymbol=null}break;case 2:len=this.productions_[action[1]][1];yyval.$=vstack[vstack.length-len];yyval._$={first_line:lstack[lstack.length-(len||1)].first_line,last_line:lstack[lstack.length-1].last_line,first_column:lstack[lstack.length-(len||1)].first_column,last_column:lstack[lstack.length-1].last_column};r=this.performAction.call(yyval,yytext,yyleng,yylineno,this.yy,action[1],vstack,lstack);if(typeof r!=="undefined"){return r}if(len){stack=stack.slice(0,-1*len*2);vstack=vstack.slice(0,-1*len);lstack=lstack.slice(0,-1*len)}stack.push(this.productions_[action[1]][0]);vstack.push(yyval.$);lstack.push(yyval._$);newState=table[stack[stack.length-2]][stack[stack.length-1]];stack.push(newState);break;case 3:return true}}return true}};var lexer=function(){var lexer={EOF:1,parseError:function parseError(str,hash){if(this.yy.parseError){this.yy.parseError(str,hash)}else{throw new Error(str)}},setInput:function(input){this._input=input;this._more=this._less=this.done=false;this.yylineno=this.yyleng=0;this.yytext=this.matched=this.match="";this.conditionStack=["INITIAL"];this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0};return this},input:function(){var ch=this._input[0];this.yytext+=ch;this.yyleng++;this.match+=ch;this.matched+=ch;var lines=ch.match(/\n/);if(lines)this.yylineno++;this._input=this._input.slice(1);return ch},unput:function(ch){this._input=ch+this._input;return this},more:function(){this._more=true;return this},less:function(n){this._input=this.match.slice(n)+this._input},pastInput:function(){var past=this.matched.substr(0,this.matched.length-this.match.length);return(past.length>20?"...":"")+past.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var next=this.match;if(next.length<20){next+=this._input.substr(0,20-next.length)}return(next.substr(0,20)+(next.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var pre=this.pastInput();var c=new Array(pre.length+1).join("-");return pre+this.upcomingInput()+"\n"+c+"^"},next:function(){if(this.done){return this.EOF}if(!this._input)this.done=true;var token,match,tempMatch,index,col,lines;if(!this._more){this.yytext="";this.match=""}var rules=this._currentRules();for(var i=0;i<rules.length;i++){tempMatch=this._input.match(this.rules[rules[i]]);if(tempMatch&&(!match||tempMatch[0].length>match[0].length)){match=tempMatch;index=i;if(!this.options.flex)break}}if(match){lines=match[0].match(/\n.*/g);if(lines)this.yylineno+=lines.length;this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:lines?lines[lines.length-1].length-1:this.yylloc.last_column+match[0].length};this.yytext+=match[0];this.match+=match[0];this.yyleng=this.yytext.length;this._more=false;this._input=this._input.slice(match[0].length);this.matched+=match[0];token=this.performAction.call(this,this.yy,this,rules[index],this.conditionStack[this.conditionStack.length-1]);if(this.done&&this._input)this.done=false;if(token)return token;else return}if(this._input===""){return this.EOF}else{this.parseError("Lexical error on line "+(this.yylineno+1)+". Unrecognized text.\n"+this.showPosition(),{text:"",token:null,line:this.yylineno})}},lex:function lex(){var r=this.next();if(typeof r!=="undefined"){return r}else{return this.lex()}},begin:function begin(condition){this.conditionStack.push(condition)},popState:function popState(){return this.conditionStack.pop()},_currentRules:function _currentRules(){return this.conditions[this.conditionStack[this.conditionStack.length-1]].rules},topState:function(){return this.conditionStack[this.conditionStack.length-2]},pushState:function begin(condition){this.begin(condition)}};lexer.options={};lexer.performAction=function anonymous(yy,yy_,$avoiding_name_collisions,YY_START){var YYSTATE=YY_START;switch($avoiding_name_collisions){case 0:break;case 1:return 6;break;case 2:yy_.yytext=yy_.yytext.substr(1,yy_.yyleng-2);return 4;break;case 3:return 17;break;case 4:return 18;break;case 5:return 23;break;case 6:return 24;break;case 7:return 22;break;case 8:return 21;break;case 9:return 10;break;case 10:return 11;break;case 11:return 8;break;case 12:return 14;break;case 13:return"INVALID";break}};lexer.rules=[/^(?:\s+)/,/^(?:(-?([0-9]|[1-9][0-9]+))(\.[0-9]+)?([eE][-+]?[0-9]+)?\b)/,/^(?:"(?:\\[\\"bfnrt\/]|\\u[a-fA-F0-9]{4}|[^\\\0-\x09\x0a-\x1f"])*")/,/^(?:\{)/,/^(?:\})/,/^(?:\[)/,/^(?:\])/,/^(?:,)/,/^(?::)/,/^(?:true\b)/,/^(?:false\b)/,/^(?:null\b)/,/^(?:$)/,/^(?:.)/];lexer.conditions={INITIAL:{rules:[0,1,2,3,4,5,6,7,8,9,10,11,12,13],inclusive:true}};return lexer}();parser.lexer=lexer;return parser}();if(typeof require!=="undefined"&&typeof exports!=="undefined"){exports.parser=jsonlint;exports.parse=function(){return jsonlint.parse.apply(jsonlint,arguments)};exports.main=function commonjsMain(args){if(!args[1])throw new Error("Usage: "+args[0]+" FILE");if(typeof process!=="undefined"){var source=require("fs").readFileSync(require("path").join(process.cwd(),args[1]),"utf8")}else{var cwd=require("file").path(require("file").cwd());var source=cwd.join(args[1]).read({charset:"utf-8"})}return exports.parser.parse(source)};if(typeof module!=="undefined"&&require.main===module){exports.main(typeof process!=="undefined"?process.argv.slice(1):require("system").args)}}(function(global,factory){if(typeof define==="function"&&define.amd){define([],factory)}else if(typeof module!=="undefined"&&module.exports){module.exports=factory()}else{global.tv4=factory()}})(this,function(){if(!Object.keys){Object.keys=function(){var hasOwnProperty=Object.prototype.hasOwnProperty,hasDontEnumBug=!{toString:null}.propertyIsEnumerable("toString"),dontEnums=["toString","toLocaleString","valueOf","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","constructor"],dontEnumsLength=dontEnums.length;return function(obj){if(typeof obj!=="object"&&typeof obj!=="function"||obj===null){throw new TypeError("Object.keys called on non-object")}var result=[];for(var prop in obj){if(hasOwnProperty.call(obj,prop)){result.push(prop)}}if(hasDontEnumBug){for(var i=0;i<dontEnumsLength;i++){if(hasOwnProperty.call(obj,dontEnums[i])){result.push(dontEnums[i])}}}return result}}()}if(!Object.create){Object.create=function(){function F(){}return function(o){if(arguments.length!==1){throw new Error("Object.create implementation only accepts one parameter.")}F.prototype=o;return new F}}()}if(!Array.isArray){Array.isArray=function(vArg){return Object.prototype.toString.call(vArg)==="[object Array]"}}if(!Array.prototype.indexOf){Array.prototype.indexOf=function(searchElement){if(this===null){throw new TypeError}var t=Object(this);var len=t.length>>>0;if(len===0){return-1}var n=0;if(arguments.length>1){n=Number(arguments[1]);if(n!==n){n=0}else if(n!==0&&n!==Infinity&&n!==-Infinity){n=(n>0||-1)*Math.floor(Math.abs(n))}}if(n>=len){return-1}var k=n>=0?n:Math.max(len-Math.abs(n),0);for(;k<len;k++){if(k in t&&t[k]===searchElement){return k}}return-1}}if(!Object.isFrozen){Object.isFrozen=function(obj){var key="tv4_test_frozen_key";while(obj.hasOwnProperty(key)){key+=Math.random()}try{obj[key]=true;delete obj[key];return false}catch(e){return true}}}var uriTemplateGlobalModifiers={"+":true,"#":true,".":true,"/":true,";":true,"?":true,"&":true};var uriTemplateSuffices={"*":true};function notReallyPercentEncode(string){return encodeURI(string).replace(/%25[0-9][0-9]/g,function(doubleEncoded){return"%"+doubleEncoded.substring(3)})}function uriTemplateSubstitution(spec){var modifier="";if(uriTemplateGlobalModifiers[spec.charAt(0)]){modifier=spec.charAt(0);spec=spec.substring(1)}var separator="";var prefix="";var shouldEscape=true;var showVariables=false;var trimEmptyString=false;if(modifier==="+"){shouldEscape=false}else if(modifier==="."){prefix=".";separator="."}else if(modifier==="/"){prefix="/";separator="/"}else if(modifier==="#"){prefix="#";shouldEscape=false}else if(modifier===";"){prefix=";";separator=";";showVariables=true;trimEmptyString=true}else if(modifier==="?"){prefix="?";separator="&";showVariables=true}else if(modifier==="&"){prefix="&";separator="&";showVariables=true}var varNames=[];var varList=spec.split(",");var varSpecs=[];var varSpecMap={};for(var i=0;i<varList.length;i++){var varName=varList[i];var truncate=null;if(varName.indexOf(":")!==-1){var parts=varName.split(":");varName=parts[0];truncate=parseInt(parts[1],10)}var suffices={};while(uriTemplateSuffices[varName.charAt(varName.length-1)]){suffices[varName.charAt(varName.length-1)]=true;varName=varName.substring(0,varName.length-1)}var varSpec={truncate:truncate,name:varName,suffices:suffices};varSpecs.push(varSpec);varSpecMap[varName]=varSpec;varNames.push(varName)}var subFunction=function(valueFunction){var result="";var startIndex=0;for(var i=0;i<varSpecs.length;i++){var varSpec=varSpecs[i];var value=valueFunction(varSpec.name);if(value===null||value===undefined||Array.isArray(value)&&value.length===0||typeof value==="object"&&Object.keys(value).length===0){startIndex++;continue}if(i===startIndex){result+=prefix}else{result+=separator||","}if(Array.isArray(value)){if(showVariables){result+=varSpec.name+"="}for(var j=0;j<value.length;j++){if(j>0){result+=varSpec.suffices["*"]?separator||",":",";if(varSpec.suffices["*"]&&showVariables){result+=varSpec.name+"="}}result+=shouldEscape?encodeURIComponent(value[j]).replace(/!/g,"%21"):notReallyPercentEncode(value[j])}}else if(typeof value==="object"){if(showVariables&&!varSpec.suffices["*"]){result+=varSpec.name+"="}var first=true;for(var key in value){if(!first){result+=varSpec.suffices["*"]?separator||",":","}first=false;result+=shouldEscape?encodeURIComponent(key).replace(/!/g,"%21"):notReallyPercentEncode(key);result+=varSpec.suffices["*"]?"=":",";result+=shouldEscape?encodeURIComponent(value[key]).replace(/!/g,"%21"):notReallyPercentEncode(value[key])}}else{if(showVariables){result+=varSpec.name;if(!trimEmptyString||value!==""){result+="="}}if(varSpec.truncate!=null){value=value.substring(0,varSpec.truncate)}result+=shouldEscape?encodeURIComponent(value).replace(/!/g,"%21"):notReallyPercentEncode(value)}}return result};subFunction.varNames=varNames;return{prefix:prefix,substitution:subFunction}}function UriTemplate(template){if(!(this instanceof UriTemplate)){return new UriTemplate(template)}var parts=template.split("{");var textParts=[parts.shift()];var prefixes=[];var substitutions=[];var varNames=[];while(parts.length>0){var part=parts.shift();var spec=part.split("}")[0];var remainder=part.substring(spec.length+1);var funcs=uriTemplateSubstitution(spec);substitutions.push(funcs.substitution);prefixes.push(funcs.prefix);textParts.push(remainder);varNames=varNames.concat(funcs.substitution.varNames)}this.fill=function(valueFunction){var result=textParts[0];for(var i=0;i<substitutions.length;i++){var substitution=substitutions[i];result+=substitution(valueFunction);result+=textParts[i+1]}return result};this.varNames=varNames;this.template=template}UriTemplate.prototype={toString:function(){return this.template},fillFromObject:function(obj){return this.fill(function(varName){return obj[varName]})}};var ValidatorContext=function ValidatorContext(parent,collectMultiple,errorReporter,checkRecursive,trackUnknownProperties){this.missing=[];this.missingMap={};this.formatValidators=parent?Object.create(parent.formatValidators):{};this.schemas=parent?Object.create(parent.schemas):{};this.collectMultiple=collectMultiple;this.errors=[];this.handleError=collectMultiple?this.collectError:this.returnError;if(checkRecursive){this.checkRecursive=true;this.scanned=[];this.scannedFrozen=[];this.scannedFrozenSchemas=[];this.scannedFrozenValidationErrors=[];this.validatedSchemasKey="tv4_validation_id";this.validationErrorsKey="tv4_validation_errors_id"}if(trackUnknownProperties){this.trackUnknownProperties=true;this.knownPropertyPaths={};this.unknownPropertyPaths={}}this.errorReporter=errorReporter||defaultErrorReporter("en");if(typeof this.errorReporter==="string"){throw new Error("debug")}this.definedKeywords={};if(parent){for(var key in parent.definedKeywords){this.definedKeywords[key]=parent.definedKeywords[key].slice(0)}}};ValidatorContext.prototype.defineKeyword=function(keyword,keywordFunction){this.definedKeywords[keyword]=this.definedKeywords[keyword]||[];this.definedKeywords[keyword].push(keywordFunction)};ValidatorContext.prototype.createError=function(code,messageParams,dataPath,schemaPath,subErrors,data,schema){var error=new ValidationError(code,messageParams,dataPath,schemaPath,subErrors);error.message=this.errorReporter(error,data,schema);return error};ValidatorContext.prototype.returnError=function(error){return error};ValidatorContext.prototype.collectError=function(error){if(error){this.errors.push(error)}return null};ValidatorContext.prototype.prefixErrors=function(startIndex,dataPath,schemaPath){for(var i=startIndex;i<this.errors.length;i++){this.errors[i]=this.errors[i].prefixWith(dataPath,schemaPath)}return this};ValidatorContext.prototype.banUnknownProperties=function(data,schema){for(var unknownPath in this.unknownPropertyPaths){var error=this.createError(ErrorCodes.UNKNOWN_PROPERTY,{path:unknownPath},unknownPath,"",null,data,schema);var result=this.handleError(error);if(result){return result}}return null};ValidatorContext.prototype.addFormat=function(format,validator){if(typeof format==="object"){for(var key in format){this.addFormat(key,format[key])}return this}this.formatValidators[format]=validator};ValidatorContext.prototype.resolveRefs=function(schema,urlHistory){if(schema["$ref"]!==undefined){urlHistory=urlHistory||{};if(urlHistory[schema["$ref"]]){return this.createError(ErrorCodes.CIRCULAR_REFERENCE,{urls:Object.keys(urlHistory).join(", ")},"","",null,undefined,schema)}urlHistory[schema["$ref"]]=true;schema=this.getSchema(schema["$ref"],urlHistory)}return schema};ValidatorContext.prototype.getSchema=function(url,urlHistory){var schema;if(this.schemas[url]!==undefined){schema=this.schemas[url];return this.resolveRefs(schema,urlHistory)}var baseUrl=url;var fragment="";if(url.indexOf("#")!==-1){fragment=url.substring(url.indexOf("#")+1);baseUrl=url.substring(0,url.indexOf("#"))}if(typeof this.schemas[baseUrl]==="object"){schema=this.schemas[baseUrl];var pointerPath=decodeURIComponent(fragment);if(pointerPath===""){return this.resolveRefs(schema,urlHistory)}else if(pointerPath.charAt(0)!=="/"){return undefined}var parts=pointerPath.split("/").slice(1);for(var i=0;i<parts.length;i++){var component=parts[i].replace(/~1/g,"/").replace(/~0/g,"~");if(schema[component]===undefined){schema=undefined;break}schema=schema[component]}if(schema!==undefined){return this.resolveRefs(schema,urlHistory)}}if(this.missing[baseUrl]===undefined){this.missing.push(baseUrl);this.missing[baseUrl]=baseUrl;this.missingMap[baseUrl]=baseUrl}};ValidatorContext.prototype.searchSchemas=function(schema,url){if(Array.isArray(schema)){for(var i=0;i<schema.length;i++){this.searchSchemas(schema[i],url)}}else if(schema&&typeof schema==="object"){if(typeof schema.id==="string"){if(isTrustedUrl(url,schema.id)){if(this.schemas[schema.id]===undefined){this.schemas[schema.id]=schema}}}for(var key in schema){if(key!=="enum"){if(typeof schema[key]==="object"){this.searchSchemas(schema[key],url)}else if(key==="$ref"){var uri=getDocumentUri(schema[key]);if(uri&&this.schemas[uri]===undefined&&this.missingMap[uri]===undefined){this.missingMap[uri]=uri}}}}}};ValidatorContext.prototype.addSchema=function(url,schema){if(typeof url!=="string"||typeof schema==="undefined"){if(typeof url==="object"&&typeof url.id==="string"){schema=url;url=schema.id}else{return}}if(url===getDocumentUri(url)+"#"){url=getDocumentUri(url)}this.schemas[url]=schema;delete this.missingMap[url];normSchema(schema,url);this.searchSchemas(schema,url)};ValidatorContext.prototype.getSchemaMap=function(){var map={};for(var key in this.schemas){map[key]=this.schemas[key]}return map};ValidatorContext.prototype.getSchemaUris=function(filterRegExp){var list=[];for(var key in this.schemas){if(!filterRegExp||filterRegExp.test(key)){list.push(key)}}return list};ValidatorContext.prototype.getMissingUris=function(filterRegExp){var list=[];for(var key in this.missingMap){if(!filterRegExp||filterRegExp.test(key)){list.push(key)}}return list};ValidatorContext.prototype.dropSchemas=function(){this.schemas={};this.reset()};ValidatorContext.prototype.reset=function(){this.missing=[];this.missingMap={};this.errors=[]};ValidatorContext.prototype.validateAll=function(data,schema,dataPathParts,schemaPathParts,dataPointerPath){var topLevel;schema=this.resolveRefs(schema);if(!schema){return null}else if(schema instanceof ValidationError){this.errors.push(schema);return schema}var startErrorCount=this.errors.length;var frozenIndex,scannedFrozenSchemaIndex=null,scannedSchemasIndex=null;if(this.checkRecursive&&data&&typeof data==="object"){topLevel=!this.scanned.length;if(data[this.validatedSchemasKey]){var schemaIndex=data[this.validatedSchemasKey].indexOf(schema);if(schemaIndex!==-1){this.errors=this.errors.concat(data[this.validationErrorsKey][schemaIndex]);return null}}if(Object.isFrozen(data)){frozenIndex=this.scannedFrozen.indexOf(data);if(frozenIndex!==-1){var frozenSchemaIndex=this.scannedFrozenSchemas[frozenIndex].indexOf(schema);if(frozenSchemaIndex!==-1){this.errors=this.errors.concat(this.scannedFrozenValidationErrors[frozenIndex][frozenSchemaIndex]);return null}}}this.scanned.push(data);if(Object.isFrozen(data)){if(frozenIndex===-1){frozenIndex=this.scannedFrozen.length;this.scannedFrozen.push(data);this.scannedFrozenSchemas.push([])}scannedFrozenSchemaIndex=this.scannedFrozenSchemas[frozenIndex].length;this.scannedFrozenSchemas[frozenIndex][scannedFrozenSchemaIndex]=schema;this.scannedFrozenValidationErrors[frozenIndex][scannedFrozenSchemaIndex]=[]}else{if(!data[this.validatedSchemasKey]){try{Object.defineProperty(data,this.validatedSchemasKey,{value:[],configurable:true});Object.defineProperty(data,this.validationErrorsKey,{value:[],configurable:true})}catch(e){data[this.validatedSchemasKey]=[];data[this.validationErrorsKey]=[]}}scannedSchemasIndex=data[this.validatedSchemasKey].length;data[this.validatedSchemasKey][scannedSchemasIndex]=schema;data[this.validationErrorsKey][scannedSchemasIndex]=[]}}var errorCount=this.errors.length;var error=this.validateBasic(data,schema,dataPointerPath)||this.validateNumeric(data,schema,dataPointerPath)||this.validateString(data,schema,dataPointerPath)||this.validateArray(data,schema,dataPointerPath)||this.validateObject(data,schema,dataPointerPath)||this.validateCombinations(data,schema,dataPointerPath)||this.validateHypermedia(data,schema,dataPointerPath)||this.validateFormat(data,schema,dataPointerPath)||this.validateDefinedKeywords(data,schema,dataPointerPath)||null;if(topLevel){while(this.scanned.length){var item=this.scanned.pop();delete item[this.validatedSchemasKey]}this.scannedFrozen=[];this.scannedFrozenSchemas=[]}if(error||errorCount!==this.errors.length){while(dataPathParts&&dataPathParts.length||schemaPathParts&&schemaPathParts.length){var dataPart=dataPathParts&&dataPathParts.length?""+dataPathParts.pop():null;var schemaPart=schemaPathParts&&schemaPathParts.length?""+schemaPathParts.pop():null;if(error){error=error.prefixWith(dataPart,schemaPart)}this.prefixErrors(errorCount,dataPart,schemaPart)}}if(scannedFrozenSchemaIndex!==null){this.scannedFrozenValidationErrors[frozenIndex][scannedFrozenSchemaIndex]=this.errors.slice(startErrorCount)}else if(scannedSchemasIndex!==null){data[this.validationErrorsKey][scannedSchemasIndex]=this.errors.slice(startErrorCount)}return this.handleError(error)};ValidatorContext.prototype.validateFormat=function(data,schema){if(typeof schema.format!=="string"||!this.formatValidators[schema.format]){return null}var errorMessage=this.formatValidators[schema.format].call(null,data,schema);if(typeof errorMessage==="string"||typeof errorMessage==="number"){return this.createError(ErrorCodes.FORMAT_CUSTOM,{message:errorMessage},"","/format",null,data,schema)}else if(errorMessage&&typeof errorMessage==="object"){return this.createError(ErrorCodes.FORMAT_CUSTOM,{message:errorMessage.message||"?"},errorMessage.dataPath||"",errorMessage.schemaPath||"/format",null,data,schema)}return null};ValidatorContext.prototype.validateDefinedKeywords=function(data,schema,dataPointerPath){for(var key in this.definedKeywords){if(typeof schema[key]==="undefined"){continue}var validationFunctions=this.definedKeywords[key];for(var i=0;i<validationFunctions.length;i++){var func=validationFunctions[i];var result=func(data,schema[key],schema,dataPointerPath);if(typeof result==="string"||typeof result==="number"){return this.createError(ErrorCodes.KEYWORD_CUSTOM,{key:key,message:result},"","",null,data,schema).prefixWith(null,key)}else if(result&&typeof result==="object"){var code=result.code;
if(typeof code==="string"){if(!ErrorCodes[code]){throw new Error("Undefined error code (use defineError): "+code)}code=ErrorCodes[code]}else if(typeof code!=="number"){code=ErrorCodes.KEYWORD_CUSTOM}var messageParams=typeof result.message==="object"?result.message:{key:key,message:result.message||"?"};var schemaPath=result.schemaPath||"/"+key.replace(/~/g,"~0").replace(/\//g,"~1");return this.createError(code,messageParams,result.dataPath||null,schemaPath,null,data,schema)}}}return null};function recursiveCompare(A,B){if(A===B){return true}if(A&&B&&typeof A==="object"&&typeof B==="object"){if(Array.isArray(A)!==Array.isArray(B)){return false}else if(Array.isArray(A)){if(A.length!==B.length){return false}for(var i=0;i<A.length;i++){if(!recursiveCompare(A[i],B[i])){return false}}}else{var key;for(key in A){if(B[key]===undefined&&A[key]!==undefined){return false}}for(key in B){if(A[key]===undefined&&B[key]!==undefined){return false}}for(key in A){if(!recursiveCompare(A[key],B[key])){return false}}}return true}return false}ValidatorContext.prototype.validateBasic=function validateBasic(data,schema,dataPointerPath){var error;if(error=this.validateType(data,schema,dataPointerPath)){return error.prefixWith(null,"type")}if(error=this.validateEnum(data,schema,dataPointerPath)){return error.prefixWith(null,"type")}return null};ValidatorContext.prototype.validateType=function validateType(data,schema){if(schema.type===undefined){return null}var dataType=typeof data;if(data===null){dataType="null"}else if(Array.isArray(data)){dataType="array"}var allowedTypes=schema.type;if(!Array.isArray(allowedTypes)){allowedTypes=[allowedTypes]}for(var i=0;i<allowedTypes.length;i++){var type=allowedTypes[i];if(type===dataType||type==="integer"&&dataType==="number"&&data%1===0){return null}}return this.createError(ErrorCodes.INVALID_TYPE,{type:dataType,expected:allowedTypes.join("/")},"","",null,data,schema)};ValidatorContext.prototype.validateEnum=function validateEnum(data,schema){if(schema["enum"]===undefined){return null}for(var i=0;i<schema["enum"].length;i++){var enumVal=schema["enum"][i];if(recursiveCompare(data,enumVal)){return null}}return this.createError(ErrorCodes.ENUM_MISMATCH,{value:typeof JSON!=="undefined"?JSON.stringify(data):data},"","",null,data,schema)};ValidatorContext.prototype.validateNumeric=function validateNumeric(data,schema,dataPointerPath){return this.validateMultipleOf(data,schema,dataPointerPath)||this.validateMinMax(data,schema,dataPointerPath)||this.validateNaN(data,schema,dataPointerPath)||null};var CLOSE_ENOUGH_LOW=Math.pow(2,-51);var CLOSE_ENOUGH_HIGH=1-CLOSE_ENOUGH_LOW;ValidatorContext.prototype.validateMultipleOf=function validateMultipleOf(data,schema){var multipleOf=schema.multipleOf||schema.divisibleBy;if(multipleOf===undefined){return null}if(typeof data==="number"){var remainder=data/multipleOf%1;if(remainder>=CLOSE_ENOUGH_LOW&&remainder<CLOSE_ENOUGH_HIGH){return this.createError(ErrorCodes.NUMBER_MULTIPLE_OF,{value:data,multipleOf:multipleOf},"","",null,data,schema)}}return null};ValidatorContext.prototype.validateMinMax=function validateMinMax(data,schema){if(typeof data!=="number"){return null}if(schema.minimum!==undefined){if(data<schema.minimum){return this.createError(ErrorCodes.NUMBER_MINIMUM,{value:data,minimum:schema.minimum},"","/minimum",null,data,schema)}if(schema.exclusiveMinimum&&data===schema.minimum){return this.createError(ErrorCodes.NUMBER_MINIMUM_EXCLUSIVE,{value:data,minimum:schema.minimum},"","/exclusiveMinimum",null,data,schema)}}if(schema.maximum!==undefined){if(data>schema.maximum){return this.createError(ErrorCodes.NUMBER_MAXIMUM,{value:data,maximum:schema.maximum},"","/maximum",null,data,schema)}if(schema.exclusiveMaximum&&data===schema.maximum){return this.createError(ErrorCodes.NUMBER_MAXIMUM_EXCLUSIVE,{value:data,maximum:schema.maximum},"","/exclusiveMaximum",null,data,schema)}}return null};ValidatorContext.prototype.validateNaN=function validateNaN(data,schema){if(typeof data!=="number"){return null}if(isNaN(data)===true||data===Infinity||data===-Infinity){return this.createError(ErrorCodes.NUMBER_NOT_A_NUMBER,{value:data},"","/type",null,data,schema)}return null};ValidatorContext.prototype.validateString=function validateString(data,schema,dataPointerPath){return this.validateStringLength(data,schema,dataPointerPath)||this.validateStringPattern(data,schema,dataPointerPath)||null};ValidatorContext.prototype.validateStringLength=function validateStringLength(data,schema){if(typeof data!=="string"){return null}if(schema.minLength!==undefined){if(data.length<schema.minLength){return this.createError(ErrorCodes.STRING_LENGTH_SHORT,{length:data.length,minimum:schema.minLength},"","/minLength",null,data,schema)}}if(schema.maxLength!==undefined){if(data.length>schema.maxLength){return this.createError(ErrorCodes.STRING_LENGTH_LONG,{length:data.length,maximum:schema.maxLength},"","/maxLength",null,data,schema)}}return null};ValidatorContext.prototype.validateStringPattern=function validateStringPattern(data,schema){if(typeof data!=="string"||typeof schema.pattern!=="string"&&!(schema.pattern instanceof RegExp)){return null}var regexp;if(schema.pattern instanceof RegExp){regexp=schema.pattern}else{var body,flags="";var literal=schema.pattern.match(/^\/(.+)\/([img]*)$/);if(literal){body=literal[1];flags=literal[2]}else{body=schema.pattern}regexp=new RegExp(body,flags)}if(!regexp.test(data)){return this.createError(ErrorCodes.STRING_PATTERN,{pattern:schema.pattern},"","/pattern",null,data,schema)}return null};ValidatorContext.prototype.validateArray=function validateArray(data,schema,dataPointerPath){if(!Array.isArray(data)){return null}return this.validateArrayLength(data,schema,dataPointerPath)||this.validateArrayUniqueItems(data,schema,dataPointerPath)||this.validateArrayItems(data,schema,dataPointerPath)||null};ValidatorContext.prototype.validateArrayLength=function validateArrayLength(data,schema){var error;if(schema.minItems!==undefined){if(data.length<schema.minItems){error=this.createError(ErrorCodes.ARRAY_LENGTH_SHORT,{length:data.length,minimum:schema.minItems},"","/minItems",null,data,schema);if(this.handleError(error)){return error}}}if(schema.maxItems!==undefined){if(data.length>schema.maxItems){error=this.createError(ErrorCodes.ARRAY_LENGTH_LONG,{length:data.length,maximum:schema.maxItems},"","/maxItems",null,data,schema);if(this.handleError(error)){return error}}}return null};ValidatorContext.prototype.validateArrayUniqueItems=function validateArrayUniqueItems(data,schema){if(schema.uniqueItems){for(var i=0;i<data.length;i++){for(var j=i+1;j<data.length;j++){if(recursiveCompare(data[i],data[j])){var error=this.createError(ErrorCodes.ARRAY_UNIQUE,{match1:i,match2:j},"","/uniqueItems",null,data,schema);if(this.handleError(error)){return error}}}}}return null};ValidatorContext.prototype.validateArrayItems=function validateArrayItems(data,schema,dataPointerPath){if(schema.items===undefined){return null}var error,i;if(Array.isArray(schema.items)){for(i=0;i<data.length;i++){if(i<schema.items.length){if(error=this.validateAll(data[i],schema.items[i],[i],["items",i],dataPointerPath+"/"+i)){return error}}else if(schema.additionalItems!==undefined){if(typeof schema.additionalItems==="boolean"){if(!schema.additionalItems){error=this.createError(ErrorCodes.ARRAY_ADDITIONAL_ITEMS,{},"/"+i,"/additionalItems",null,data,schema);if(this.handleError(error)){return error}}}else if(error=this.validateAll(data[i],schema.additionalItems,[i],["additionalItems"],dataPointerPath+"/"+i)){return error}}}}else{for(i=0;i<data.length;i++){if(error=this.validateAll(data[i],schema.items,[i],["items"],dataPointerPath+"/"+i)){return error}}}return null};ValidatorContext.prototype.validateObject=function validateObject(data,schema,dataPointerPath){if(typeof data!=="object"||data===null||Array.isArray(data)){return null}return this.validateObjectMinMaxProperties(data,schema,dataPointerPath)||this.validateObjectRequiredProperties(data,schema,dataPointerPath)||this.validateObjectProperties(data,schema,dataPointerPath)||this.validateObjectDependencies(data,schema,dataPointerPath)||null};ValidatorContext.prototype.validateObjectMinMaxProperties=function validateObjectMinMaxProperties(data,schema){var keys=Object.keys(data);var error;if(schema.minProperties!==undefined){if(keys.length<schema.minProperties){error=this.createError(ErrorCodes.OBJECT_PROPERTIES_MINIMUM,{propertyCount:keys.length,minimum:schema.minProperties},"","/minProperties",null,data,schema);if(this.handleError(error)){return error}}}if(schema.maxProperties!==undefined){if(keys.length>schema.maxProperties){error=this.createError(ErrorCodes.OBJECT_PROPERTIES_MAXIMUM,{propertyCount:keys.length,maximum:schema.maxProperties},"","/maxProperties",null,data,schema);if(this.handleError(error)){return error}}}return null};ValidatorContext.prototype.validateObjectRequiredProperties=function validateObjectRequiredProperties(data,schema){if(schema.required!==undefined){for(var i=0;i<schema.required.length;i++){var key=schema.required[i];if(data[key]===undefined){var error=this.createError(ErrorCodes.OBJECT_REQUIRED,{key:key},"","/required/"+i,null,data,schema);if(this.handleError(error)){return error}}}}return null};ValidatorContext.prototype.validateObjectProperties=function validateObjectProperties(data,schema,dataPointerPath){var error;for(var key in data){var keyPointerPath=dataPointerPath+"/"+key.replace(/~/g,"~0").replace(/\//g,"~1");var foundMatch=false;if(schema.properties!==undefined&&schema.properties[key]!==undefined){foundMatch=true;if(error=this.validateAll(data[key],schema.properties[key],[key],["properties",key],keyPointerPath)){return error}}if(schema.patternProperties!==undefined){for(var patternKey in schema.patternProperties){var regexp=new RegExp(patternKey);if(regexp.test(key)){foundMatch=true;if(error=this.validateAll(data[key],schema.patternProperties[patternKey],[key],["patternProperties",patternKey],keyPointerPath)){return error}}}}if(!foundMatch){if(schema.additionalProperties!==undefined){if(this.trackUnknownProperties){this.knownPropertyPaths[keyPointerPath]=true;delete this.unknownPropertyPaths[keyPointerPath]}if(typeof schema.additionalProperties==="boolean"){if(!schema.additionalProperties){error=this.createError(ErrorCodes.OBJECT_ADDITIONAL_PROPERTIES,{key:key},"","/additionalProperties",null,data,schema).prefixWith(key,null);if(this.handleError(error)){return error}}}else{if(error=this.validateAll(data[key],schema.additionalProperties,[key],["additionalProperties"],keyPointerPath)){return error}}}else if(this.trackUnknownProperties&&!this.knownPropertyPaths[keyPointerPath]){this.unknownPropertyPaths[keyPointerPath]=true}}else if(this.trackUnknownProperties){this.knownPropertyPaths[keyPointerPath]=true;delete this.unknownPropertyPaths[keyPointerPath]}}return null};ValidatorContext.prototype.validateObjectDependencies=function validateObjectDependencies(data,schema,dataPointerPath){var error;if(schema.dependencies!==undefined){for(var depKey in schema.dependencies){if(data[depKey]!==undefined){var dep=schema.dependencies[depKey];if(typeof dep==="string"){if(data[dep]===undefined){error=this.createError(ErrorCodes.OBJECT_DEPENDENCY_KEY,{key:depKey,missing:dep},"","",null,data,schema).prefixWith(null,depKey).prefixWith(null,"dependencies");if(this.handleError(error)){return error}}}else if(Array.isArray(dep)){for(var i=0;i<dep.length;i++){var requiredKey=dep[i];if(data[requiredKey]===undefined){error=this.createError(ErrorCodes.OBJECT_DEPENDENCY_KEY,{key:depKey,missing:requiredKey},"","/"+i,null,data,schema).prefixWith(null,depKey).prefixWith(null,"dependencies");if(this.handleError(error)){return error}}}}else{if(error=this.validateAll(data,dep,[],["dependencies",depKey],dataPointerPath)){return error}}}}}return null};ValidatorContext.prototype.validateCombinations=function validateCombinations(data,schema,dataPointerPath){return this.validateAllOf(data,schema,dataPointerPath)||this.validateAnyOf(data,schema,dataPointerPath)||this.validateOneOf(data,schema,dataPointerPath)||this.validateNot(data,schema,dataPointerPath)||null};ValidatorContext.prototype.validateAllOf=function validateAllOf(data,schema,dataPointerPath){if(schema.allOf===undefined){return null}var error;for(var i=0;i<schema.allOf.length;i++){var subSchema=schema.allOf[i];if(error=this.validateAll(data,subSchema,[],["allOf",i],dataPointerPath)){return error}}return null};ValidatorContext.prototype.validateAnyOf=function validateAnyOf(data,schema,dataPointerPath){if(schema.anyOf===undefined){return null}var errors=[];var startErrorCount=this.errors.length;var oldUnknownPropertyPaths,oldKnownPropertyPaths;if(this.trackUnknownProperties){oldUnknownPropertyPaths=this.unknownPropertyPaths;oldKnownPropertyPaths=this.knownPropertyPaths}var errorAtEnd=true;for(var i=0;i<schema.anyOf.length;i++){if(this.trackUnknownProperties){this.unknownPropertyPaths={};this.knownPropertyPaths={}}var subSchema=schema.anyOf[i];var errorCount=this.errors.length;var error=this.validateAll(data,subSchema,[],["anyOf",i],dataPointerPath);if(error===null&&errorCount===this.errors.length){this.errors=this.errors.slice(0,startErrorCount);if(this.trackUnknownProperties){for(var knownKey in this.knownPropertyPaths){oldKnownPropertyPaths[knownKey]=true;delete oldUnknownPropertyPaths[knownKey]}for(var unknownKey in this.unknownPropertyPaths){if(!oldKnownPropertyPaths[unknownKey]){oldUnknownPropertyPaths[unknownKey]=true}}errorAtEnd=false;continue}return null}if(error){errors.push(error.prefixWith(null,""+i).prefixWith(null,"anyOf"))}}if(this.trackUnknownProperties){this.unknownPropertyPaths=oldUnknownPropertyPaths;this.knownPropertyPaths=oldKnownPropertyPaths}if(errorAtEnd){errors=errors.concat(this.errors.slice(startErrorCount));this.errors=this.errors.slice(0,startErrorCount);return this.createError(ErrorCodes.ANY_OF_MISSING,{},"","/anyOf",errors,data,schema)}};ValidatorContext.prototype.validateOneOf=function validateOneOf(data,schema,dataPointerPath){if(schema.oneOf===undefined){return null}var validIndex=null;var errors=[];var startErrorCount=this.errors.length;var oldUnknownPropertyPaths,oldKnownPropertyPaths;if(this.trackUnknownProperties){oldUnknownPropertyPaths=this.unknownPropertyPaths;oldKnownPropertyPaths=this.knownPropertyPaths}for(var i=0;i<schema.oneOf.length;i++){if(this.trackUnknownProperties){this.unknownPropertyPaths={};this.knownPropertyPaths={}}var subSchema=schema.oneOf[i];var errorCount=this.errors.length;var error=this.validateAll(data,subSchema,[],["oneOf",i],dataPointerPath);if(error===null&&errorCount===this.errors.length){if(validIndex===null){validIndex=i}else{this.errors=this.errors.slice(0,startErrorCount);return this.createError(ErrorCodes.ONE_OF_MULTIPLE,{index1:validIndex,index2:i},"","/oneOf",null,data,schema)}if(this.trackUnknownProperties){for(var knownKey in this.knownPropertyPaths){oldKnownPropertyPaths[knownKey]=true;delete oldUnknownPropertyPaths[knownKey]}for(var unknownKey in this.unknownPropertyPaths){if(!oldKnownPropertyPaths[unknownKey]){oldUnknownPropertyPaths[unknownKey]=true}}}}else if(error){errors.push(error)}}if(this.trackUnknownProperties){this.unknownPropertyPaths=oldUnknownPropertyPaths;this.knownPropertyPaths=oldKnownPropertyPaths}if(validIndex===null){errors=errors.concat(this.errors.slice(startErrorCount));this.errors=this.errors.slice(0,startErrorCount);return this.createError(ErrorCodes.ONE_OF_MISSING,{},"","/oneOf",errors,data,schema)}else{this.errors=this.errors.slice(0,startErrorCount)}return null};ValidatorContext.prototype.validateNot=function validateNot(data,schema,dataPointerPath){if(schema.not===undefined){return null}var oldErrorCount=this.errors.length;var oldUnknownPropertyPaths,oldKnownPropertyPaths;if(this.trackUnknownProperties){oldUnknownPropertyPaths=this.unknownPropertyPaths;oldKnownPropertyPaths=this.knownPropertyPaths;this.unknownPropertyPaths={};this.knownPropertyPaths={}}var error=this.validateAll(data,schema.not,null,null,dataPointerPath);var notErrors=this.errors.slice(oldErrorCount);this.errors=this.errors.slice(0,oldErrorCount);if(this.trackUnknownProperties){this.unknownPropertyPaths=oldUnknownPropertyPaths;this.knownPropertyPaths=oldKnownPropertyPaths}if(error===null&&notErrors.length===0){return this.createError(ErrorCodes.NOT_PASSED,{},"","/not",null,data,schema)}return null};ValidatorContext.prototype.validateHypermedia=function validateCombinations(data,schema,dataPointerPath){if(!schema.links){return null}var error;for(var i=0;i<schema.links.length;i++){var ldo=schema.links[i];if(ldo.rel==="describedby"){var template=new UriTemplate(ldo.href);var allPresent=true;for(var j=0;j<template.varNames.length;j++){if(!(template.varNames[j]in data)){allPresent=false;break}}if(allPresent){var schemaUrl=template.fillFromObject(data);var subSchema={$ref:schemaUrl};if(error=this.validateAll(data,subSchema,[],["links",i],dataPointerPath)){return error}}}}};function parseURI(url){var m=String(url).replace(/^\s+|\s+$/g,"").match(/^([^:\/?#]+:)?(\/\/(?:[^:@]*(?::[^:@]*)?@)?(([^:\/?#]*)(?::(\d*))?))?([^?#]*)(\?[^#]*)?(#[\s\S]*)?/);return m?{href:m[0]||"",protocol:m[1]||"",authority:m[2]||"",host:m[3]||"",hostname:m[4]||"",port:m[5]||"",pathname:m[6]||"",search:m[7]||"",hash:m[8]||""}:null}function resolveUrl(base,href){function removeDotSegments(input){var output=[];input.replace(/^(\.\.?(\/|$))+/,"").replace(/\/(\.(\/|$))+/g,"/").replace(/\/\.\.$/,"/../").replace(/\/?[^\/]*/g,function(p){if(p==="/.."){output.pop()}else{output.push(p)}});return output.join("").replace(/^\//,input.charAt(0)==="/"?"/":"")}href=parseURI(href||"");base=parseURI(base||"");return!href||!base?null:(href.protocol||base.protocol)+(href.protocol||href.authority?href.authority:base.authority)+removeDotSegments(href.protocol||href.authority||href.pathname.charAt(0)==="/"?href.pathname:href.pathname?(base.authority&&!base.pathname?"/":"")+base.pathname.slice(0,base.pathname.lastIndexOf("/")+1)+href.pathname:base.pathname)+(href.protocol||href.authority||href.pathname?href.search:href.search||base.search)+href.hash}function getDocumentUri(uri){return uri.split("#")[0]}function normSchema(schema,baseUri){if(schema&&typeof schema==="object"){if(baseUri===undefined){baseUri=schema.id}else if(typeof schema.id==="string"){baseUri=resolveUrl(baseUri,schema.id);schema.id=baseUri}if(Array.isArray(schema)){for(var i=0;i<schema.length;i++){normSchema(schema[i],baseUri)}}else{if(typeof schema["$ref"]==="string"){schema["$ref"]=resolveUrl(baseUri,schema["$ref"])}for(var key in schema){if(key!=="enum"){normSchema(schema[key],baseUri)}}}}}function defaultErrorReporter(language){language=language||"en";var errorMessages=languages[language];return function(error){var messageTemplate=errorMessages[error.code]||ErrorMessagesDefault[error.code];if(typeof messageTemplate!=="string"){return"Unknown error code "+error.code+": "+JSON.stringify(error.messageParams)}var messageParams=error.params;return messageTemplate.replace(/\{([^{}]*)\}/g,function(whole,varName){var subValue=messageParams[varName];return typeof subValue==="string"||typeof subValue==="number"?subValue:whole})}}var ErrorCodes={INVALID_TYPE:0,ENUM_MISMATCH:1,ANY_OF_MISSING:10,ONE_OF_MISSING:11,ONE_OF_MULTIPLE:12,NOT_PASSED:13,NUMBER_MULTIPLE_OF:100,NUMBER_MINIMUM:101,NUMBER_MINIMUM_EXCLUSIVE:102,NUMBER_MAXIMUM:103,NUMBER_MAXIMUM_EXCLUSIVE:104,NUMBER_NOT_A_NUMBER:105,STRING_LENGTH_SHORT:200,STRING_LENGTH_LONG:201,STRING_PATTERN:202,OBJECT_PROPERTIES_MINIMUM:300,OBJECT_PROPERTIES_MAXIMUM:301,OBJECT_REQUIRED:302,OBJECT_ADDITIONAL_PROPERTIES:303,OBJECT_DEPENDENCY_KEY:304,ARRAY_LENGTH_SHORT:400,ARRAY_LENGTH_LONG:401,ARRAY_UNIQUE:402,ARRAY_ADDITIONAL_ITEMS:403,FORMAT_CUSTOM:500,KEYWORD_CUSTOM:501,CIRCULAR_REFERENCE:600,UNKNOWN_PROPERTY:1e3};var ErrorCodeLookup={};for(var key in ErrorCodes){ErrorCodeLookup[ErrorCodes[key]]=key}var ErrorMessagesDefault={INVALID_TYPE:"Invalid type: {type} (expected {expected})",ENUM_MISMATCH:"No enum match for: {value}",ANY_OF_MISSING:'Data does not match any schemas from "anyOf"',ONE_OF_MISSING:'Data does not match any schemas from "oneOf"',ONE_OF_MULTIPLE:'Data is valid against more than one schema from "oneOf": indices {index1} and {index2}',NOT_PASSED:'Data matches schema from "not"',NUMBER_MULTIPLE_OF:"Value {value} is not a multiple of {multipleOf}",NUMBER_MINIMUM:"Value {value} is less than minimum {minimum}",NUMBER_MINIMUM_EXCLUSIVE:"Value {value} is equal to exclusive minimum {minimum}",NUMBER_MAXIMUM:"Value {value} is greater than maximum {maximum}",NUMBER_MAXIMUM_EXCLUSIVE:"Value {value} is equal to exclusive maximum {maximum}",NUMBER_NOT_A_NUMBER:"Value {value} is not a valid number",STRING_LENGTH_SHORT:"String is too short ({length} chars), minimum {minimum}",STRING_LENGTH_LONG:"String is too long ({length} chars), maximum {maximum}",STRING_PATTERN:"String does not match pattern: {pattern}",OBJECT_PROPERTIES_MINIMUM:"Too few properties defined ({propertyCount}), minimum {minimum}",OBJECT_PROPERTIES_MAXIMUM:"Too many properties defined ({propertyCount}), maximum {maximum}",OBJECT_REQUIRED:"Missing required property: {key}",OBJECT_ADDITIONAL_PROPERTIES:"Additional properties not allowed",OBJECT_DEPENDENCY_KEY:"Dependency failed - key must exist: {missing} (due to key: {key})",ARRAY_LENGTH_SHORT:"Array is too short ({length}), minimum {minimum}",ARRAY_LENGTH_LONG:"Array is too long ({length}), maximum {maximum}",ARRAY_UNIQUE:"Array items are not unique (indices {match1} and {match2})",ARRAY_ADDITIONAL_ITEMS:"Additional items not allowed",FORMAT_CUSTOM:"Format validation failed ({message})",KEYWORD_CUSTOM:"Keyword failed: {key} ({message})",CIRCULAR_REFERENCE:"Circular $refs: {urls}",UNKNOWN_PROPERTY:"Unknown property (not in schema)"};function ValidationError(code,params,dataPath,schemaPath,subErrors){Error.call(this);if(code===undefined){throw new Error("No error code supplied: "+schemaPath)}this.message="";this.params=params;this.code=code;this.dataPath=dataPath||"";this.schemaPath=schemaPath||"";this.subErrors=subErrors||null;var err=new Error(this.message);this.stack=err.stack||err.stacktrace;if(!this.stack){try{throw err}catch(err){this.stack=err.stack||err.stacktrace}}}ValidationError.prototype=Object.create(Error.prototype);ValidationError.prototype.constructor=ValidationError;ValidationError.prototype.name="ValidationError";ValidationError.prototype.prefixWith=function(dataPrefix,schemaPrefix){if(dataPrefix!==null){dataPrefix=dataPrefix.replace(/~/g,"~0").replace(/\//g,"~1");this.dataPath="/"+dataPrefix+this.dataPath}if(schemaPrefix!==null){schemaPrefix=schemaPrefix.replace(/~/g,"~0").replace(/\//g,"~1");this.schemaPath="/"+schemaPrefix+this.schemaPath}if(this.subErrors!==null){for(var i=0;i<this.subErrors.length;i++){this.subErrors[i].prefixWith(dataPrefix,schemaPrefix)}}return this};function isTrustedUrl(baseUrl,testUrl){if(testUrl.substring(0,baseUrl.length)===baseUrl){var remainder=testUrl.substring(baseUrl.length);if(testUrl.length>0&&testUrl.charAt(baseUrl.length-1)==="/"||remainder.charAt(0)==="#"||remainder.charAt(0)==="?"){return true}}return false}var languages={};function createApi(language){var globalContext=new ValidatorContext;var currentLanguage;var customErrorReporter;var api={setErrorReporter:function(reporter){if(typeof reporter==="string"){return this.language(reporter)}customErrorReporter=reporter;return true},addFormat:function(){globalContext.addFormat.apply(globalContext,arguments)},language:function(code){if(!code){return currentLanguage}if(!languages[code]){code=code.split("-")[0]}if(languages[code]){currentLanguage=code;return code}return false},addLanguage:function(code,messageMap){var key;for(key in ErrorCodes){if(messageMap[key]&&!messageMap[ErrorCodes[key]]){messageMap[ErrorCodes[key]]=messageMap[key]}}var rootCode=code.split("-")[0];if(!languages[rootCode]){languages[code]=messageMap;languages[rootCode]=messageMap}else{languages[code]=Object.create(languages[rootCode]);for(key in messageMap){if(typeof languages[rootCode][key]==="undefined"){languages[rootCode][key]=messageMap[key]}languages[code][key]=messageMap[key]}}return this},freshApi:function(language){var result=createApi();if(language){result.language(language)}return result},validate:function(data,schema,checkRecursive,banUnknownProperties){var def=defaultErrorReporter(currentLanguage);var errorReporter=customErrorReporter?function(error,data,schema){return customErrorReporter(error,data,schema)||def(error,data,schema)}:def;var context=new ValidatorContext(globalContext,false,errorReporter,checkRecursive,banUnknownProperties);if(typeof schema==="string"){schema={$ref:schema}}context.addSchema("",schema);var error=context.validateAll(data,schema,null,null,"");if(!error&&banUnknownProperties){error=context.banUnknownProperties(data,schema)}this.error=error;this.missing=context.missing;this.valid=error===null;return this.valid},validateResult:function(){var result={};this.validate.apply(result,arguments);return result},validateMultiple:function(data,schema,checkRecursive,banUnknownProperties){var def=defaultErrorReporter(currentLanguage);var errorReporter=customErrorReporter?function(error,data,schema){return customErrorReporter(error,data,schema)||def(error,data,schema)}:def;var context=new ValidatorContext(globalContext,true,errorReporter,checkRecursive,banUnknownProperties);if(typeof schema==="string"){schema={$ref:schema}}context.addSchema("",schema);context.validateAll(data,schema,null,null,"");if(banUnknownProperties){context.banUnknownProperties(data,schema)}var result={};result.errors=context.errors;result.missing=context.missing;result.valid=result.errors.length===0;return result},addSchema:function(){return globalContext.addSchema.apply(globalContext,arguments)},getSchema:function(){return globalContext.getSchema.apply(globalContext,arguments)},getSchemaMap:function(){return globalContext.getSchemaMap.apply(globalContext,arguments)},getSchemaUris:function(){return globalContext.getSchemaUris.apply(globalContext,arguments)},getMissingUris:function(){return globalContext.getMissingUris.apply(globalContext,arguments)},dropSchemas:function(){globalContext.dropSchemas.apply(globalContext,arguments)},defineKeyword:function(){globalContext.defineKeyword.apply(globalContext,arguments)},defineError:function(codeName,codeNumber,defaultMessage){if(typeof codeName!=="string"||!/^[A-Z]+(_[A-Z]+)*$/.test(codeName)){throw new Error("Code name must be a string in UPPER_CASE_WITH_UNDERSCORES")}if(typeof codeNumber!=="number"||codeNumber%1!==0||codeNumber<1e4){throw new Error("Code number must be an integer > 10000")}if(typeof ErrorCodes[codeName]!=="undefined"){throw new Error("Error already defined: "+codeName+" as "+ErrorCodes[codeName])}if(typeof ErrorCodeLookup[codeNumber]!=="undefined"){throw new Error("Error code already used: "+ErrorCodeLookup[codeNumber]+" as "+codeNumber)}ErrorCodes[codeName]=codeNumber;ErrorCodeLookup[codeNumber]=codeName;ErrorMessagesDefault[codeName]=ErrorMessagesDefault[codeNumber]=defaultMessage;for(var langCode in languages){var language=languages[langCode];if(language[codeName]){language[codeNumber]=language[codeNumber]||language[codeName]}}},reset:function(){globalContext.reset();this.error=null;this.missing=[];this.valid=true},missing:[],error:null,valid:true,normSchema:normSchema,resolveUrl:resolveUrl,getDocumentUri:getDocumentUri,errorCodes:ErrorCodes};api.language(language||"en");return api}var tv4=createApi();tv4.addLanguage("en-gb",ErrorMessagesDefault);tv4.tv4=tv4;return tv4});var JSHINT;if(typeof window==="undefined")window={};(function(){var require;require=function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);var f=new Error("Cannot find module '"+o+"'");throw f.code="MODULE_NOT_FOUND",f}var l=n[o]={exports:{}};t[o][0].call(l.exports,function(e){var n=t[o][1][e];return s(n?n:e)},l,l.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s}({1:[function(require,module,exports){var identifierStartTable=[];for(var i=0;i<128;i++){identifierStartTable[i]=i===36||i>=65&&i<=90||i===95||i>=97&&i<=122}var identifierPartTable=[];for(var i=0;i<128;i++){identifierPartTable[i]=identifierStartTable[i]||i>=48&&i<=57}module.exports={asciiIdentifierStartTable:identifierStartTable,asciiIdentifierPartTable:identifierPartTable}},{}],2:[function(require,module,exports){var str="768,769,770,771,772,773,774,775,776,777,778,779,780,781,782,783,784,785,786,787,788,789,790,791,792,793,794,795,796,797,798,799,800,801,802,803,804,805,806,807,808,809,810,811,812,813,814,815,816,817,818,819,820,821,822,823,824,825,826,827,828,829,830,831,832,833,834,835,836,837,838,839,840,841,842,843,844,845,846,847,848,849,850,851,852,853,854,855,856,857,858,859,860,861,862,863,864,865,866,867,868,869,870,871,872,873,874,875,876,877,878,879,1155,1156,1157,1158,1159,1425,1426,1427,1428,1429,1430,1431,1432,1433,1434,1435,1436,1437,1438,1439,1440,1441,1442,1443,1444,1445,1446,1447,1448,1449,1450,1451,1452,1453,1454,1455,1456,1457,1458,1459,1460,1461,1462,1463,1464,1465,1466,1467,1468,1469,1471,1473,1474,1476,1477,1479,1552,1553,1554,1555,1556,1557,1558,1559,1560,1561,1562,1611,1612,1613,1614,1615,1616,1617,1618,1619,1620,1621,1622,1623,1624,1625,1626,1627,1628,1629,1630,1631,1632,1633,1634,1635,1636,1637,1638,1639,1640,1641,1648,1750,1751,1752,1753,1754,1755,1756,1759,1760,1761,1762,1763,1764,1767,1768,1770,1771,1772,1773,1776,1777,1778,1779,1780,1781,1782,1783,1784,1785,1809,1840,1841,1842,1843,1844,1845,1846,1847,1848,1849,1850,1851,1852,1853,1854,1855,1856,1857,1858,1859,1860,1861,1862,1863,1864,1865,1866,1958,1959,1960,1961,1962,1963,1964,1965,1966,1967,1968,1984,1985,1986,1987,1988,1989,1990,1991,1992,1993,2027,2028,2029,2030,2031,2032,2033,2034,2035,2070,2071,2072,2073,2075,2076,2077,2078,2079,2080,2081,2082,2083,2085,2086,2087,2089,2090,2091,2092,2093,2137,2138,2139,2276,2277,2278,2279,2280,2281,2282,2283,2284,2285,2286,2287,2288,2289,2290,2291,2292,2293,2294,2295,2296,2297,2298,2299,2300,2301,2302,2304,2305,2306,2307,2362,2363,2364,2366,2367,2368,2369,2370,2371,2372,2373,2374,2375,2376,2377,2378,2379,2380,2381,2382,2383,2385,2386,2387,2388,2389,2390,2391,2402,2403,2406,2407,2408,2409,2410,2411,2412,2413,2414,2415,2433,2434,2435,2492,2494,2495,2496,2497,2498,2499,2500,2503,2504,2507,2508,2509,2519,2530,2531,2534,2535,2536,2537,2538,2539,2540,2541,2542,2543,2561,2562,2563,2620,2622,2623,2624,2625,2626,2631,2632,2635,2636,2637,2641,2662,2663,2664,2665,2666,2667,2668,2669,2670,2671,2672,2673,2677,2689,2690,2691,2748,2750,2751,2752,2753,2754,2755,2756,2757,2759,2760,2761,2763,2764,2765,2786,2787,2790,2791,2792,2793,2794,2795,2796,2797,2798,2799,2817,2818,2819,2876,2878,2879,2880,2881,2882,2883,2884,2887,2888,2891,2892,2893,2902,2903,2914,2915,2918,2919,2920,2921,2922,2923,2924,2925,2926,2927,2946,3006,3007,3008,3009,3010,3014,3015,3016,3018,3019,3020,3021,3031,3046,3047,3048,3049,3050,3051,3052,3053,3054,3055,3073,3074,3075,3134,3135,3136,3137,3138,3139,3140,3142,3143,3144,3146,3147,3148,3149,3157,3158,3170,3171,3174,3175,3176,3177,3178,3179,3180,3181,3182,3183,3202,3203,3260,3262,3263,3264,3265,3266,3267,3268,3270,3271,3272,3274,3275,3276,3277,3285,3286,3298,3299,3302,3303,3304,3305,3306,3307,3308,3309,3310,3311,3330,3331,3390,3391,3392,3393,3394,3395,3396,3398,3399,3400,3402,3403,3404,3405,3415,3426,3427,3430,3431,3432,3433,3434,3435,3436,3437,3438,3439,3458,3459,3530,3535,3536,3537,3538,3539,3540,3542,3544,3545,3546,3547,3548,3549,3550,3551,3570,3571,3633,3636,3637,3638,3639,3640,3641,3642,3655,3656,3657,3658,3659,3660,3661,3662,3664,3665,3666,3667,3668,3669,3670,3671,3672,3673,3761,3764,3765,3766,3767,3768,3769,3771,3772,3784,3785,3786,3787,3788,3789,3792,3793,3794,3795,3796,3797,3798,3799,3800,3801,3864,3865,3872,3873,3874,3875,3876,3877,3878,3879,3880,3881,3893,3895,3897,3902,3903,3953,3954,3955,3956,3957,3958,3959,3960,3961,3962,3963,3964,3965,3966,3967,3968,3969,3970,3971,3972,3974,3975,3981,3982,3983,3984,3985,3986,3987,3988,3989,3990,3991,3993,3994,3995,3996,3997,3998,3999,4000,4001,4002,4003,4004,4005,4006,4007,4008,4009,4010,4011,4012,4013,4014,4015,4016,4017,4018,4019,4020,4021,4022,4023,4024,4025,4026,4027,4028,4038,4139,4140,4141,4142,4143,4144,4145,4146,4147,4148,4149,4150,4151,4152,4153,4154,4155,4156,4157,4158,4160,4161,4162,4163,4164,4165,4166,4167,4168,4169,4182,4183,4184,4185,4190,4191,4192,4194,4195,4196,4199,4200,4201,4202,4203,4204,4205,4209,4210,4211,4212,4226,4227,4228,4229,4230,4231,4232,4233,4234,4235,4236,4237,4239,4240,4241,4242,4243,4244,4245,4246,4247,4248,4249,4250,4251,4252,4253,4957,4958,4959,5906,5907,5908,5938,5939,5940,5970,5971,6002,6003,6068,6069,6070,6071,6072,6073,6074,6075,6076,6077,6078,6079,6080,6081,6082,6083,6084,6085,6086,6087,6088,6089,6090,6091,6092,6093,6094,6095,6096,6097,6098,6099,6109,6112,6113,6114,6115,6116,6117,6118,6119,6120,6121,6155,6156,6157,6160,6161,6162,6163,6164,6165,6166,6167,6168,6169,6313,6432,6433,6434,6435,6436,6437,6438,6439,6440,6441,6442,6443,6448,6449,6450,6451,6452,6453,6454,6455,6456,6457,6458,6459,6470,6471,6472,6473,6474,6475,6476,6477,6478,6479,6576,6577,6578,6579,6580,6581,6582,6583,6584,6585,6586,6587,6588,6589,6590,6591,6592,6600,6601,6608,6609,6610,6611,6612,6613,6614,6615,6616,6617,6679,6680,6681,6682,6683,6741,6742,6743,6744,6745,6746,6747,6748,6749,6750,6752,6753,6754,6755,6756,6757,6758,6759,6760,6761,6762,6763,6764,6765,6766,6767,6768,6769,6770,6771,6772,6773,6774,6775,6776,6777,6778,6779,6780,6783,6784,6785,6786,6787,6788,6789,6790,6791,6792,6793,6800,6801,6802,6803,6804,6805,6806,6807,6808,6809,6912,6913,6914,6915,6916,6964,6965,6966,6967,6968,6969,6970,6971,6972,6973,6974,6975,6976,6977,6978,6979,6980,6992,6993,6994,6995,6996,6997,6998,6999,7000,7001,7019,7020,7021,7022,7023,7024,7025,7026,7027,7040,7041,7042,7073,7074,7075,7076,7077,7078,7079,7080,7081,7082,7083,7084,7085,7088,7089,7090,7091,7092,7093,7094,7095,7096,7097,7142,7143,7144,7145,7146,7147,7148,7149,7150,7151,7152,7153,7154,7155,7204,7205,7206,7207,7208,7209,7210,7211,7212,7213,7214,7215,7216,7217,7218,7219,7220,7221,7222,7223,7232,7233,7234,7235,7236,7237,7238,7239,7240,7241,7248,7249,7250,7251,7252,7253,7254,7255,7256,7257,7376,7377,7378,7380,7381,7382,7383,7384,7385,7386,7387,7388,7389,7390,7391,7392,7393,7394,7395,7396,7397,7398,7399,7400,7405,7410,7411,7412,7616,7617,7618,7619,7620,7621,7622,7623,7624,7625,7626,7627,7628,7629,7630,7631,7632,7633,7634,7635,7636,7637,7638,7639,7640,7641,7642,7643,7644,7645,7646,7647,7648,7649,7650,7651,7652,7653,7654,7676,7677,7678,7679,8204,8205,8255,8256,8276,8400,8401,8402,8403,8404,8405,8406,8407,8408,8409,8410,8411,8412,8417,8421,8422,8423,8424,8425,8426,8427,8428,8429,8430,8431,8432,11503,11504,11505,11647,11744,11745,11746,11747,11748,11749,11750,11751,11752,11753,11754,11755,11756,11757,11758,11759,11760,11761,11762,11763,11764,11765,11766,11767,11768,11769,11770,11771,11772,11773,11774,11775,12330,12331,12332,12333,12334,12335,12441,12442,42528,42529,42530,42531,42532,42533,42534,42535,42536,42537,42607,42612,42613,42614,42615,42616,42617,42618,42619,42620,42621,42655,42736,42737,43010,43014,43019,43043,43044,43045,43046,43047,43136,43137,43188,43189,43190,43191,43192,43193,43194,43195,43196,43197,43198,43199,43200,43201,43202,43203,43204,43216,43217,43218,43219,43220,43221,43222,43223,43224,43225,43232,43233,43234,43235,43236,43237,43238,43239,43240,43241,43242,43243,43244,43245,43246,43247,43248,43249,43264,43265,43266,43267,43268,43269,43270,43271,43272,43273,43302,43303,43304,43305,43306,43307,43308,43309,43335,43336,43337,43338,43339,43340,43341,43342,43343,43344,43345,43346,43347,43392,43393,43394,43395,43443,43444,43445,43446,43447,43448,43449,43450,43451,43452,43453,43454,43455,43456,43472,43473,43474,43475,43476,43477,43478,43479,43480,43481,43561,43562,43563,43564,43565,43566,43567,43568,43569,43570,43571,43572,43573,43574,43587,43596,43597,43600,43601,43602,43603,43604,43605,43606,43607,43608,43609,43643,43696,43698,43699,43700,43703,43704,43710,43711,43713,43755,43756,43757,43758,43759,43765,43766,44003,44004,44005,44006,44007,44008,44009,44010,44012,44013,44016,44017,44018,44019,44020,44021,44022,44023,44024,44025,64286,65024,65025,65026,65027,65028,65029,65030,65031,65032,65033,65034,65035,65036,65037,65038,65039,65056,65057,65058,65059,65060,65061,65062,65075,65076,65101,65102,65103,65296,65297,65298,65299,65300,65301,65302,65303,65304,65305,65343";
var arr=str.split(",").map(function(code){return parseInt(code,10)});module.exports=arr},{}],3:[function(require,module,exports){var str="170,181,186,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,248,249,250,251,252,253,254,255,256,257,258,259,260,261,262,263,264,265,266,267,268,269,270,271,272,273,274,275,276,277,278,279,280,281,282,283,284,285,286,287,288,289,290,291,292,293,294,295,296,297,298,299,300,301,302,303,304,305,306,307,308,309,310,311,312,313,314,315,316,317,318,319,320,321,322,323,324,325,326,327,328,329,330,331,332,333,334,335,336,337,338,339,340,341,342,343,344,345,346,347,348,349,350,351,352,353,354,355,356,357,358,359,360,361,362,363,364,365,366,367,368,369,370,371,372,373,374,375,376,377,378,379,380,381,382,383,384,385,386,387,388,389,390,391,392,393,394,395,396,397,398,399,400,401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,416,417,418,419,420,421,422,423,424,425,426,427,428,429,430,431,432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,447,448,449,450,451,452,453,454,455,456,457,458,459,460,461,462,463,464,465,466,467,468,469,470,471,472,473,474,475,476,477,478,479,480,481,482,483,484,485,486,487,488,489,490,491,492,493,494,495,496,497,498,499,500,501,502,503,504,505,506,507,508,509,510,511,512,513,514,515,516,517,518,519,520,521,522,523,524,525,526,527,528,529,530,531,532,533,534,535,536,537,538,539,540,541,542,543,544,545,546,547,548,549,550,551,552,553,554,555,556,557,558,559,560,561,562,563,564,565,566,567,568,569,570,571,572,573,574,575,576,577,578,579,580,581,582,583,584,585,586,587,588,589,590,591,592,593,594,595,596,597,598,599,600,601,602,603,604,605,606,607,608,609,610,611,612,613,614,615,616,617,618,619,620,621,622,623,624,625,626,627,628,629,630,631,632,633,634,635,636,637,638,639,640,641,642,643,644,645,646,647,648,649,650,651,652,653,654,655,656,657,658,659,660,661,662,663,664,665,666,667,668,669,670,671,672,673,674,675,676,677,678,679,680,681,682,683,684,685,686,687,688,689,690,691,692,693,694,695,696,697,698,699,700,701,702,703,704,705,710,711,712,713,714,715,716,717,718,719,720,721,736,737,738,739,740,748,750,880,881,882,883,884,886,887,890,891,892,893,902,904,905,906,908,910,911,912,913,914,915,916,917,918,919,920,921,922,923,924,925,926,927,928,929,931,932,933,934,935,936,937,938,939,940,941,942,943,944,945,946,947,948,949,950,951,952,953,954,955,956,957,958,959,960,961,962,963,964,965,966,967,968,969,970,971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1006,1007,1008,1009,1010,1011,1012,1013,1015,1016,1017,1018,1019,1020,1021,1022,1023,1024,1025,1026,1027,1028,1029,1030,1031,1032,1033,1034,1035,1036,1037,1038,1039,1040,1041,1042,1043,1044,1045,1046,1047,1048,1049,1050,1051,1052,1053,1054,1055,1056,1057,1058,1059,1060,1061,1062,1063,1064,1065,1066,1067,1068,1069,1070,1071,1072,1073,1074,1075,1076,1077,1078,1079,1080,1081,1082,1083,1084,1085,1086,1087,1088,1089,1090,1091,1092,1093,1094,1095,1096,1097,1098,1099,1100,1101,1102,1103,1104,1105,1106,1107,1108,1109,1110,1111,1112,1113,1114,1115,1116,1117,1118,1119,1120,1121,1122,1123,1124,1125,1126,1127,1128,1129,1130,1131,1132,1133,1134,1135,1136,1137,1138,1139,1140,1141,1142,1143,1144,1145,1146,1147,1148,1149,1150,1151,1152,1153,1162,1163,1164,1165,1166,1167,1168,1169,1170,1171,1172,1173,1174,1175,1176,1177,1178,1179,1180,1181,1182,1183,1184,1185,1186,1187,1188,1189,1190,1191,1192,1193,1194,1195,1196,1197,1198,1199,1200,1201,1202,1203,1204,1205,1206,1207,1208,1209,1210,1211,1212,1213,1214,1215,1216,1217,1218,1219,1220,1221,1222,1223,1224,1225,1226,1227,1228,1229,1230,1231,1232,1233,1234,1235,1236,1237,1238,1239,1240,1241,1242,1243,1244,1245,1246,1247,1248,1249,1250,1251,1252,1253,1254,1255,1256,1257,1258,1259,1260,1261,1262,1263,1264,1265,1266,1267,1268,1269,1270,1271,1272,1273,1274,1275,1276,1277,1278,1279,1280,1281,1282,1283,1284,1285,1286,1287,1288,1289,1290,1291,1292,1293,1294,1295,1296,1297,1298,1299,1300,1301,1302,1303,1304,1305,1306,1307,1308,1309,1310,1311,1312,1313,1314,1315,1316,1317,1318,1319,1329,1330,1331,1332,1333,1334,1335,1336,1337,1338,1339,1340,1341,1342,1343,1344,1345,1346,1347,1348,1349,1350,1351,1352,1353,1354,1355,1356,1357,1358,1359,1360,1361,1362,1363,1364,1365,1366,1369,1377,1378,1379,1380,1381,1382,1383,1384,1385,1386,1387,1388,1389,1390,1391,1392,1393,1394,1395,1396,1397,1398,1399,1400,1401,1402,1403,1404,1405,1406,1407,1408,1409,1410,1411,1412,1413,1414,1415,1488,1489,1490,1491,1492,1493,1494,1495,1496,1497,1498,1499,1500,1501,1502,1503,1504,1505,1506,1507,1508,1509,1510,1511,1512,1513,1514,1520,1521,1522,1568,1569,1570,1571,1572,1573,1574,1575,1576,1577,1578,1579,1580,1581,1582,1583,1584,1585,1586,1587,1588,1589,1590,1591,1592,1593,1594,1595,1596,1597,1598,1599,1600,1601,1602,1603,1604,1605,1606,1607,1608,1609,1610,1646,1647,1649,1650,1651,1652,1653,1654,1655,1656,1657,1658,1659,1660,1661,1662,1663,1664,1665,1666,1667,1668,1669,1670,1671,1672,1673,1674,1675,1676,1677,1678,1679,1680,1681,1682,1683,1684,1685,1686,1687,1688,1689,1690,1691,1692,1693,1694,1695,1696,1697,1698,1699,1700,1701,1702,1703,1704,1705,1706,1707,1708,1709,1710,1711,1712,1713,1714,1715,1716,1717,1718,1719,1720,1721,1722,1723,1724,1725,1726,1727,1728,1729,1730,1731,1732,1733,1734,1735,1736,1737,1738,1739,1740,1741,1742,1743,1744,1745,1746,1747,1749,1765,1766,1774,1775,1786,1787,1788,1791,1808,1810,1811,1812,1813,1814,1815,1816,1817,1818,1819,1820,1821,1822,1823,1824,1825,1826,1827,1828,1829,1830,1831,1832,1833,1834,1835,1836,1837,1838,1839,1869,1870,1871,1872,1873,1874,1875,1876,1877,1878,1879,1880,1881,1882,1883,1884,1885,1886,1887,1888,1889,1890,1891,1892,1893,1894,1895,1896,1897,1898,1899,1900,1901,1902,1903,1904,1905,1906,1907,1908,1909,1910,1911,1912,1913,1914,1915,1916,1917,1918,1919,1920,1921,1922,1923,1924,1925,1926,1927,1928,1929,1930,1931,1932,1933,1934,1935,1936,1937,1938,1939,1940,1941,1942,1943,1944,1945,1946,1947,1948,1949,1950,1951,1952,1953,1954,1955,1956,1957,1969,1994,1995,1996,1997,1998,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024,2025,2026,2036,2037,2042,2048,2049,2050,2051,2052,2053,2054,2055,2056,2057,2058,2059,2060,2061,2062,2063,2064,2065,2066,2067,2068,2069,2074,2084,2088,2112,2113,2114,2115,2116,2117,2118,2119,2120,2121,2122,2123,2124,2125,2126,2127,2128,2129,2130,2131,2132,2133,2134,2135,2136,2208,2210,2211,2212,2213,2214,2215,2216,2217,2218,2219,2220,2308,2309,2310,2311,2312,2313,2314,2315,2316,2317,2318,2319,2320,2321,2322,2323,2324,2325,2326,2327,2328,2329,2330,2331,2332,2333,2334,2335,2336,2337,2338,2339,2340,2341,2342,2343,2344,2345,2346,2347,2348,2349,2350,2351,2352,2353,2354,2355,2356,2357,2358,2359,2360,2361,2365,2384,2392,2393,2394,2395,2396,2397,2398,2399,2400,2401,2417,2418,2419,2420,2421,2422,2423,2425,2426,2427,2428,2429,2430,2431,2437,2438,2439,2440,2441,2442,2443,2444,2447,2448,2451,2452,2453,2454,2455,2456,2457,2458,2459,2460,2461,2462,2463,2464,2465,2466,2467,2468,2469,2470,2471,2472,2474,2475,2476,2477,2478,2479,2480,2482,2486,2487,2488,2489,2493,2510,2524,2525,2527,2528,2529,2544,2545,2565,2566,2567,2568,2569,2570,2575,2576,2579,2580,2581,2582,2583,2584,2585,2586,2587,2588,2589,2590,2591,2592,2593,2594,2595,2596,2597,2598,2599,2600,2602,2603,2604,2605,2606,2607,2608,2610,2611,2613,2614,2616,2617,2649,2650,2651,2652,2654,2674,2675,2676,2693,2694,2695,2696,2697,2698,2699,2700,2701,2703,2704,2705,2707,2708,2709,2710,2711,2712,2713,2714,2715,2716,2717,2718,2719,2720,2721,2722,2723,2724,2725,2726,2727,2728,2730,2731,2732,2733,2734,2735,2736,2738,2739,2741,2742,2743,2744,2745,2749,2768,2784,2785,2821,2822,2823,2824,2825,2826,2827,2828,2831,2832,2835,2836,2837,2838,2839,2840,2841,2842,2843,2844,2845,2846,2847,2848,2849,2850,2851,2852,2853,2854,2855,2856,2858,2859,2860,2861,2862,2863,2864,2866,2867,2869,2870,2871,2872,2873,2877,2908,2909,2911,2912,2913,2929,2947,2949,2950,2951,2952,2953,2954,2958,2959,2960,2962,2963,2964,2965,2969,2970,2972,2974,2975,2979,2980,2984,2985,2986,2990,2991,2992,2993,2994,2995,2996,2997,2998,2999,3000,3001,3024,3077,3078,3079,3080,3081,3082,3083,3084,3086,3087,3088,3090,3091,3092,3093,3094,3095,3096,3097,3098,3099,3100,3101,3102,3103,3104,3105,3106,3107,3108,3109,3110,3111,3112,3114,3115,3116,3117,3118,3119,3120,3121,3122,3123,3125,3126,3127,3128,3129,3133,3160,3161,3168,3169,3205,3206,3207,3208,3209,3210,3211,3212,3214,3215,3216,3218,3219,3220,3221,3222,3223,3224,3225,3226,3227,3228,3229,3230,3231,3232,3233,3234,3235,3236,3237,3238,3239,3240,3242,3243,3244,3245,3246,3247,3248,3249,3250,3251,3253,3254,3255,3256,3257,3261,3294,3296,3297,3313,3314,3333,3334,3335,3336,3337,3338,3339,3340,3342,3343,3344,3346,3347,3348,3349,3350,3351,3352,3353,3354,3355,3356,3357,3358,3359,3360,3361,3362,3363,3364,3365,3366,3367,3368,3369,3370,3371,3372,3373,3374,3375,3376,3377,3378,3379,3380,3381,3382,3383,3384,3385,3386,3389,3406,3424,3425,3450,3451,3452,3453,3454,3455,3461,3462,3463,3464,3465,3466,3467,3468,3469,3470,3471,3472,3473,3474,3475,3476,3477,3478,3482,3483,3484,3485,3486,3487,3488,3489,3490,3491,3492,3493,3494,3495,3496,3497,3498,3499,3500,3501,3502,3503,3504,3505,3507,3508,3509,3510,3511,3512,3513,3514,3515,3517,3520,3521,3522,3523,3524,3525,3526,3585,3586,3587,3588,3589,3590,3591,3592,3593,3594,3595,3596,3597,3598,3599,3600,3601,3602,3603,3604,3605,3606,3607,3608,3609,3610,3611,3612,3613,3614,3615,3616,3617,3618,3619,3620,3621,3622,3623,3624,3625,3626,3627,3628,3629,3630,3631,3632,3634,3635,3648,3649,3650,3651,3652,3653,3654,3713,3714,3716,3719,3720,3722,3725,3732,3733,3734,3735,3737,3738,3739,3740,3741,3742,3743,3745,3746,3747,3749,3751,3754,3755,3757,3758,3759,3760,3762,3763,3773,3776,3777,3778,3779,3780,3782,3804,3805,3806,3807,3840,3904,3905,3906,3907,3908,3909,3910,3911,3913,3914,3915,3916,3917,3918,3919,3920,3921,3922,3923,3924,3925,3926,3927,3928,3929,3930,3931,3932,3933,3934,3935,3936,3937,3938,3939,3940,3941,3942,3943,3944,3945,3946,3947,3948,3976,3977,3978,3979,3980,4096,4097,4098,4099,4100,4101,4102,4103,4104,4105,4106,4107,4108,4109,4110,4111,4112,4113,4114,4115,4116,4117,4118,4119,4120,4121,4122,4123,4124,4125,4126,4127,4128,4129,4130,4131,4132,4133,4134,4135,4136,4137,4138,4159,4176,4177,4178,4179,4180,4181,4186,4187,4188,4189,4193,4197,4198,4206,4207,4208,4213,4214,4215,4216,4217,4218,4219,4220,4221,4222,4223,4224,4225,4238,4256,4257,4258,4259,4260,4261,4262,4263,4264,4265,4266,4267,4268,4269,4270,4271,4272,4273,4274,4275,4276,4277,4278,4279,4280,4281,4282,4283,4284,4285,4286,4287,4288,4289,4290,4291,4292,4293,4295,4301,4304,4305,4306,4307,4308,4309,4310,4311,4312,4313,4314,4315,4316,4317,4318,4319,4320,4321,4322,4323,4324,4325,4326,4327,4328,4329,4330,4331,4332,4333,4334,4335,4336,4337,4338,4339,4340,4341,4342,4343,4344,4345,4346,4348,4349,4350,4351,4352,4353,4354,4355,4356,4357,4358,4359,4360,4361,4362,4363,4364,4365,4366,4367,4368,4369,4370,4371,4372,4373,4374,4375,4376,4377,4378,4379,4380,4381,4382,4383,4384,4385,4386,4387,4388,4389,4390,4391,4392,4393,4394,4395,4396,4397,4398,4399,4400,4401,4402,4403,4404,4405,4406,4407,4408,4409,4410,4411,4412,4413,4414,4415,4416,4417,4418,4419,4420,4421,4422,4423,4424,4425,4426,4427,4428,4429,4430,4431,4432,4433,4434,4435,4436,4437,4438,4439,4440,4441,4442,4443,4444,4445,4446,4447,4448,4449,4450,4451,4452,4453,4454,4455,4456,4457,4458,4459,4460,4461,4462,4463,4464,4465,4466,4467,4468,4469,4470,4471,4472,4473,4474,4475,4476,4477,4478,4479,4480,4481,4482,4483,4484,4485,4486,4487,4488,4489,4490,4491,4492,4493,4494,4495,4496,4497,4498,4499,4500,4501,4502,4503,4504,4505,4506,4507,4508,4509,4510,4511,4512,4513,4514,4515,4516,4517,4518,4519,4520,4521,4522,4523,4524,4525,4526,4527,4528,4529,4530,4531,4532,4533,4534,4535,4536,4537,4538,4539,4540,4541,4542,4543,4544,4545,4546,4547,4548,4549,4550,4551,4552,4553,4554,4555,4556,4557,4558,4559,4560,4561,4562,4563,4564,4565,4566,4567,4568,4569,4570,4571,4572,4573,4574,4575,4576,4577,4578,4579,4580,4581,4582,4583,4584,4585,4586,4587,4588,4589,4590,4591,4592,4593,4594,4595,4596,4597,4598,4599,4600,4601,4602,4603,4604,4605,4606,4607,4608,4609,4610,4611,4612,4613,4614,4615,4616,4617,4618,4619,4620,4621,4622,4623,4624,4625,4626,4627,4628,4629,4630,4631,4632,4633,4634,4635,4636,4637,4638,4639,4640,4641,4642,4643,4644,4645,4646,4647,4648,4649,4650,4651,4652,4653,4654,4655,4656,4657,4658,4659,4660,4661,4662,4663,4664,4665,4666,4667,4668,4669,4670,4671,4672,4673,4674,4675,4676,4677,4678,4679,4680,4682,4683,4684,4685,4688,4689,4690,4691,4692,4693,4694,4696,4698,4699,4700,4701,4704,4705,4706,4707,4708,4709,4710,4711,4712,4713,4714,4715,4716,4717,4718,4719,4720,4721,4722,4723,4724,4725,4726,4727,4728,4729,4730,4731,4732,4733,4734,4735,4736,4737,4738,4739,4740,4741,4742,4743,4744,4746,4747,4748,4749,4752,4753,4754,4755,4756,4757,4758,4759,4760,4761,4762,4763,4764,4765,4766,4767,4768,4769,4770,4771,4772,4773,4774,4775,4776,4777,4778,4779,4780,4781,4782,4783,4784,4786,4787,4788,4789,4792,4793,4794,4795,4796,4797,4798,4800,4802,4803,4804,4805,4808,4809,4810,4811,4812,4813,4814,4815,4816,4817,4818,4819,4820,4821,4822,4824,4825,4826,4827,4828,4829,4830,4831,4832,4833,4834,4835,4836,4837,4838,4839,4840,4841,4842,4843,4844,4845,4846,4847,4848,4849,4850,4851,4852,4853,4854,4855,4856,4857,4858,4859,4860,4861,4862,4863,4864,4865,4866,4867,4868,4869,4870,4871,4872,4873,4874,4875,4876,4877,4878,4879,4880,4882,4883,4884,4885,4888,4889,4890,4891,4892,4893,4894,4895,4896,4897,4898,4899,4900,4901,4902,4903,4904,4905,4906,4907,4908,4909,4910,4911,4912,4913,4914,4915,4916,4917,4918,4919,4920,4921,4922,4923,4924,4925,4926,4927,4928,4929,4930,4931,4932,4933,4934,4935,4936,4937,4938,4939,4940,4941,4942,4943,4944,4945,4946,4947,4948,4949,4950,4951,4952,4953,4954,4992,4993,4994,4995,4996,4997,4998,4999,5000,5001,5002,5003,5004,5005,5006,5007,5024,5025,5026,5027,5028,5029,5030,5031,5032,5033,5034,5035,5036,5037,5038,5039,5040,5041,5042,5043,5044,5045,5046,5047,5048,5049,5050,5051,5052,5053,5054,5055,5056,5057,5058,5059,5060,5061,5062,5063,5064,5065,5066,5067,5068,5069,5070,5071,5072,5073,5074,5075,5076,5077,5078,5079,5080,5081,5082,5083,5084,5085,5086,5087,5088,5089,5090,5091,5092,5093,5094,5095,5096,5097,5098,5099,5100,5101,5102,5103,5104,5105,5106,5107,5108,5121,5122,5123,5124,5125,5126,5127,5128,5129,5130,5131,5132,5133,5134,5135,5136,5137,5138,5139,5140,5141,5142,5143,5144,5145,5146,5147,5148,5149,5150,5151,5152,5153,5154,5155,5156,5157,5158,5159,5160,5161,5162,5163,5164,5165,5166,5167,5168,5169,5170,5171,5172,5173,5174,5175,5176,5177,5178,5179,5180,5181,5182,5183,5184,5185,5186,5187,5188,5189,5190,5191,5192,5193,5194,5195,5196,5197,5198,5199,5200,5201,5202,5203,5204,5205,5206,5207,5208,5209,5210,5211,5212,5213,5214,5215,5216,5217,5218,5219,5220,5221,5222,5223,5224,5225,5226,5227,5228,5229,5230,5231,5232,5233,5234,5235,5236,5237,5238,5239,5240,5241,5242,5243,5244,5245,5246,5247,5248,5249,5250,5251,5252,5253,5254,5255,5256,5257,5258,5259,5260,5261,5262,5263,5264,5265,5266,5267,5268,5269,5270,5271,5272,5273,5274,5275,5276,5277,5278,5279,5280,5281,5282,5283,5284,5285,5286,5287,5288,5289,5290,5291,5292,5293,5294,5295,5296,5297,5298,5299,5300,5301,5302,5303,5304,5305,5306,5307,5308,5309,5310,5311,5312,5313,5314,5315,5316,5317,5318,5319,5320,5321,5322,5323,5324,5325,5326,5327,5328,5329,5330,5331,5332,5333,5334,5335,5336,5337,5338,5339,5340,5341,5342,5343,5344,5345,5346,5347,5348,5349,5350,5351,5352,5353,5354,5355,5356,5357,5358,5359,5360,5361,5362,5363,5364,5365,5366,5367,5368,5369,5370,5371,5372,5373,5374,5375,5376,5377,5378,5379,5380,5381,5382,5383,5384,5385,5386,5387,5388,5389,5390,5391,5392,5393,5394,5395,5396,5397,5398,5399,5400,5401,5402,5403,5404,5405,5406,5407,5408,5409,5410,5411,5412,5413,5414,5415,5416,5417,5418,5419,5420,5421,5422,5423,5424,5425,5426,5427,5428,5429,5430,5431,5432,5433,5434,5435,5436,5437,5438,5439,5440,5441,5442,5443,5444,5445,5446,5447,5448,5449,5450,5451,5452,5453,5454,5455,5456,5457,5458,5459,5460,5461,5462,5463,5464,5465,5466,5467,5468,5469,5470,5471,5472,5473,5474,5475,5476,5477,5478,5479,5480,5481,5482,5483,5484,5485,5486,5487,5488,5489,5490,5491,5492,5493,5494,5495,5496,5497,5498,5499,5500,5501,5502,5503,5504,5505,5506,5507,5508,5509,5510,5511,5512,5513,5514,5515,5516,5517,5518,5519,5520,5521,5522,5523,5524,5525,5526,5527,5528,5529,5530,5531,5532,5533,5534,5535,5536,5537,5538,5539,5540,5541,5542,5543,5544,5545,5546,5547,5548,5549,5550,5551,5552,5553,5554,5555,5556,5557,5558,5559,5560,5561,5562,5563,5564,5565,5566,5567,5568,5569,5570,5571,5572,5573,5574,5575,5576,5577,5578,5579,5580,5581,5582,5583,5584,5585,5586,5587,5588,5589,5590,5591,5592,5593,5594,5595,5596,5597,5598,5599,5600,5601,5602,5603,5604,5605,5606,5607,5608,5609,5610,5611,5612,5613,5614,5615,5616,5617,5618,5619,5620,5621,5622,5623,5624,5625,5626,5627,5628,5629,5630,5631,5632,5633,5634,5635,5636,5637,5638,5639,5640,5641,5642,5643,5644,5645,5646,5647,5648,5649,5650,5651,5652,5653,5654,5655,5656,5657,5658,5659,5660,5661,5662,5663,5664,5665,5666,5667,5668,5669,5670,5671,5672,5673,5674,5675,5676,5677,5678,5679,5680,5681,5682,5683,5684,5685,5686,5687,5688,5689,5690,5691,5692,5693,5694,5695,5696,5697,5698,5699,5700,5701,5702,5703,5704,5705,5706,5707,5708,5709,5710,5711,5712,5713,5714,5715,5716,5717,5718,5719,5720,5721,5722,5723,5724,5725,5726,5727,5728,5729,5730,5731,5732,5733,5734,5735,5736,5737,5738,5739,5740,5743,5744,5745,5746,5747,5748,5749,5750,5751,5752,5753,5754,5755,5756,5757,5758,5759,5761,5762,5763,5764,5765,5766,5767,5768,5769,5770,5771,5772,5773,5774,5775,5776,5777,5778,5779,5780,5781,5782,5783,5784,5785,5786,5792,5793,5794,5795,5796,5797,5798,5799,5800,5801,5802,5803,5804,5805,5806,5807,5808,5809,5810,5811,5812,5813,5814,5815,5816,5817,5818,5819,5820,5821,5822,5823,5824,5825,5826,5827,5828,5829,5830,5831,5832,5833,5834,5835,5836,5837,5838,5839,5840,5841,5842,5843,5844,5845,5846,5847,5848,5849,5850,5851,5852,5853,5854,5855,5856,5857,5858,5859,5860,5861,5862,5863,5864,5865,5866,5870,5871,5872,5888,5889,5890,5891,5892,5893,5894,5895,5896,5897,5898,5899,5900,5902,5903,5904,5905,5920,5921,5922,5923,5924,5925,5926,5927,5928,5929,5930,5931,5932,5933,5934,5935,5936,5937,5952,5953,5954,5955,5956,5957,5958,5959,5960,5961,5962,5963,5964,5965,5966,5967,5968,5969,5984,5985,5986,5987,5988,5989,5990,5991,5992,5993,5994,5995,5996,5998,5999,6000,6016,6017,6018,6019,6020,6021,6022,6023,6024,6025,6026,6027,6028,6029,6030,6031,6032,6033,6034,6035,6036,6037,6038,6039,6040,6041,6042,6043,6044,6045,6046,6047,6048,6049,6050,6051,6052,6053,6054,6055,6056,6057,6058,6059,6060,6061,6062,6063,6064,6065,6066,6067,6103,6108,6176,6177,6178,6179,6180,6181,6182,6183,6184,6185,6186,6187,6188,6189,6190,6191,6192,6193,6194,6195,6196,6197,6198,6199,6200,6201,6202,6203,6204,6205,6206,6207,6208,6209,6210,6211,6212,6213,6214,6215,6216,6217,6218,6219,6220,6221,6222,6223,6224,6225,6226,6227,6228,6229,6230,6231,6232,6233,6234,6235,6236,6237,6238,6239,6240,6241,6242,6243,6244,6245,6246,6247,6248,6249,6250,6251,6252,6253,6254,6255,6256,6257,6258,6259,6260,6261,6262,6263,6272,6273,6274,6275,6276,6277,6278,6279,6280,6281,6282,6283,6284,6285,6286,6287,6288,6289,6290,6291,6292,6293,6294,6295,6296,6297,6298,6299,6300,6301,6302,6303,6304,6305,6306,6307,6308,6309,6310,6311,6312,6314,6320,6321,6322,6323,6324,6325,6326,6327,6328,6329,6330,6331,6332,6333,6334,6335,6336,6337,6338,6339,6340,6341,6342,6343,6344,6345,6346,6347,6348,6349,6350,6351,6352,6353,6354,6355,6356,6357,6358,6359,6360,6361,6362,6363,6364,6365,6366,6367,6368,6369,6370,6371,6372,6373,6374,6375,6376,6377,6378,6379,6380,6381,6382,6383,6384,6385,6386,6387,6388,6389,6400,6401,6402,6403,6404,6405,6406,6407,6408,6409,6410,6411,6412,6413,6414,6415,6416,6417,6418,6419,6420,6421,6422,6423,6424,6425,6426,6427,6428,6480,6481,6482,6483,6484,6485,6486,6487,6488,6489,6490,6491,6492,6493,6494,6495,6496,6497,6498,6499,6500,6501,6502,6503,6504,6505,6506,6507,6508,6509,6512,6513,6514,6515,6516,6528,6529,6530,6531,6532,6533,6534,6535,6536,6537,6538,6539,6540,6541,6542,6543,6544,6545,6546,6547,6548,6549,6550,6551,6552,6553,6554,6555,6556,6557,6558,6559,6560,6561,6562,6563,6564,6565,6566,6567,6568,6569,6570,6571,6593,6594,6595,6596,6597,6598,6599,6656,6657,6658,6659,6660,6661,6662,6663,6664,6665,6666,6667,6668,6669,6670,6671,6672,6673,6674,6675,6676,6677,6678,6688,6689,6690,6691,6692,6693,6694,6695,6696,6697,6698,6699,6700,6701,6702,6703,6704,6705,6706,6707,6708,6709,6710,6711,6712,6713,6714,6715,6716,6717,6718,6719,6720,6721,6722,6723,6724,6725,6726,6727,6728,6729,6730,6731,6732,6733,6734,6735,6736,6737,6738,6739,6740,6823,6917,6918,6919,6920,6921,6922,6923,6924,6925,6926,6927,6928,6929,6930,6931,6932,6933,6934,6935,6936,6937,6938,6939,6940,6941,6942,6943,6944,6945,6946,6947,6948,6949,6950,6951,6952,6953,6954,6955,6956,6957,6958,6959,6960,6961,6962,6963,6981,6982,6983,6984,6985,6986,6987,7043,7044,7045,7046,7047,7048,7049,7050,7051,7052,7053,7054,7055,7056,7057,7058,7059,7060,7061,7062,7063,7064,7065,7066,7067,7068,7069,7070,7071,7072,7086,7087,7098,7099,7100,7101,7102,7103,7104,7105,7106,7107,7108,7109,7110,7111,7112,7113,7114,7115,7116,7117,7118,7119,7120,7121,7122,7123,7124,7125,7126,7127,7128,7129,7130,7131,7132,7133,7134,7135,7136,7137,7138,7139,7140,7141,7168,7169,7170,7171,7172,7173,7174,7175,7176,7177,7178,7179,7180,7181,7182,7183,7184,7185,7186,7187,7188,7189,7190,7191,7192,7193,7194,7195,7196,7197,7198,7199,7200,7201,7202,7203,7245,7246,7247,7258,7259,7260,7261,7262,7263,7264,7265,7266,7267,7268,7269,7270,7271,7272,7273,7274,7275,7276,7277,7278,7279,7280,7281,7282,7283,7284,7285,7286,7287,7288,7289,7290,7291,7292,7293,7401,7402,7403,7404,7406,7407,7408,7409,7413,7414,7424,7425,7426,7427,7428,7429,7430,7431,7432,7433,7434,7435,7436,7437,7438,7439,7440,7441,7442,7443,7444,7445,7446,7447,7448,7449,7450,7451,7452,7453,7454,7455,7456,7457,7458,7459,7460,7461,7462,7463,7464,7465,7466,7467,7468,7469,7470,7471,7472,7473,7474,7475,7476,7477,7478,7479,7480,7481,7482,7483,7484,7485,7486,7487,7488,7489,7490,7491,7492,7493,7494,7495,7496,7497,7498,7499,7500,7501,7502,7503,7504,7505,7506,7507,7508,7509,7510,7511,7512,7513,7514,7515,7516,7517,7518,7519,7520,7521,7522,7523,7524,7525,7526,7527,7528,7529,7530,7531,7532,7533,7534,7535,7536,7537,7538,7539,7540,7541,7542,7543,7544,7545,7546,7547,7548,7549,7550,7551,7552,7553,7554,7555,7556,7557,7558,7559,7560,7561,7562,7563,7564,7565,7566,7567,7568,7569,7570,7571,7572,7573,7574,7575,7576,7577,7578,7579,7580,7581,7582,7583,7584,7585,7586,7587,7588,7589,7590,7591,7592,7593,7594,7595,7596,7597,7598,7599,7600,7601,7602,7603,7604,7605,7606,7607,7608,7609,7610,7611,7612,7613,7614,7615,7680,7681,7682,7683,7684,7685,7686,7687,7688,7689,7690,7691,7692,7693,7694,7695,7696,7697,7698,7699,7700,7701,7702,7703,7704,7705,7706,7707,7708,7709,7710,7711,7712,7713,7714,7715,7716,7717,7718,7719,7720,7721,7722,7723,7724,7725,7726,7727,7728,7729,7730,7731,7732,7733,7734,7735,7736,7737,7738,7739,7740,7741,7742,7743,7744,7745,7746,7747,7748,7749,7750,7751,7752,7753,7754,7755,7756,7757,7758,7759,7760,7761,7762,7763,7764,7765,7766,7767,7768,7769,7770,7771,7772,7773,7774,7775,7776,7777,7778,7779,7780,7781,7782,7783,7784,7785,7786,7787,7788,7789,7790,7791,7792,7793,7794,7795,7796,7797,7798,7799,7800,7801,7802,7803,7804,7805,7806,7807,7808,7809,7810,7811,7812,7813,7814,7815,7816,7817,7818,7819,7820,7821,7822,7823,7824,7825,7826,7827,7828,7829,7830,7831,7832,7833,7834,7835,7836,7837,7838,7839,7840,7841,7842,7843,7844,7845,7846,7847,7848,7849,7850,7851,7852,7853,7854,7855,7856,7857,7858,7859,7860,7861,7862,7863,7864,7865,7866,7867,7868,7869,7870,7871,7872,7873,7874,7875,7876,7877,7878,7879,7880,7881,7882,7883,7884,7885,7886,7887,7888,7889,7890,7891,7892,7893,7894,7895,7896,7897,7898,7899,7900,7901,7902,7903,7904,7905,7906,7907,7908,7909,7910,7911,7912,7913,7914,7915,7916,7917,7918,7919,7920,7921,7922,7923,7924,7925,7926,7927,7928,7929,7930,7931,7932,7933,7934,7935,7936,7937,7938,7939,7940,7941,7942,7943,7944,7945,7946,7947,7948,7949,7950,7951,7952,7953,7954,7955,7956,7957,7960,7961,7962,7963,7964,7965,7968,7969,7970,7971,7972,7973,7974,7975,7976,7977,7978,7979,7980,7981,7982,7983,7984,7985,7986,7987,7988,7989,7990,7991,7992,7993,7994,7995,7996,7997,7998,7999,8000,8001,8002,8003,8004,8005,8008,8009,8010,8011,8012,8013,8016,8017,8018,8019,8020,8021,8022,8023,8025,8027,8029,8031,8032,8033,8034,8035,8036,8037,8038,8039,8040,8041,8042,8043,8044,8045,8046,8047,8048,8049,8050,8051,8052,8053,8054,8055,8056,8057,8058,8059,8060,8061,8064,8065,8066,8067,8068,8069,8070,8071,8072,8073,8074,8075,8076,8077,8078,8079,8080,8081,8082,8083,8084,8085,8086,8087,8088,8089,8090,8091,8092,8093,8094,8095,8096,8097,8098,8099,8100,8101,8102,8103,8104,8105,8106,8107,8108,8109,8110,8111,8112,8113,8114,8115,8116,8118,8119,8120,8121,8122,8123,8124,8126,8130,8131,8132,8134,8135,8136,8137,8138,8139,8140,8144,8145,8146,8147,8150,8151,8152,8153,8154,8155,8160,8161,8162,8163,8164,8165,8166,8167,8168,8169,8170,8171,8172,8178,8179,8180,8182,8183,8184,8185,8186,8187,8188,8305,8319,8336,8337,8338,8339,8340,8341,8342,8343,8344,8345,8346,8347,8348,8450,8455,8458,8459,8460,8461,8462,8463,8464,8465,8466,8467,8469,8473,8474,8475,8476,8477,8484,8486,8488,8490,8491,8492,8493,8495,8496,8497,8498,8499,8500,8501,8502,8503,8504,8505,8508,8509,8510,8511,8517,8518,8519,8520,8521,8526,8544,8545,8546,8547,8548,8549,8550,8551,8552,8553,8554,8555,8556,8557,8558,8559,8560,8561,8562,8563,8564,8565,8566,8567,8568,8569,8570,8571,8572,8573,8574,8575,8576,8577,8578,8579,8580,8581,8582,8583,8584,11264,11265,11266,11267,11268,11269,11270,11271,11272,11273,11274,11275,11276,11277,11278,11279,11280,11281,11282,11283,11284,11285,11286,11287,11288,11289,11290,11291,11292,11293,11294,11295,11296,11297,11298,11299,11300,11301,11302,11303,11304,11305,11306,11307,11308,11309,11310,11312,11313,11314,11315,11316,11317,11318,11319,11320,11321,11322,11323,11324,11325,11326,11327,11328,11329,11330,11331,11332,11333,11334,11335,11336,11337,11338,11339,11340,11341,11342,11343,11344,11345,11346,11347,11348,11349,11350,11351,11352,11353,11354,11355,11356,11357,11358,11360,11361,11362,11363,11364,11365,11366,11367,11368,11369,11370,11371,11372,11373,11374,11375,11376,11377,11378,11379,11380,11381,11382,11383,11384,11385,11386,11387,11388,11389,11390,11391,11392,11393,11394,11395,11396,11397,11398,11399,11400,11401,11402,11403,11404,11405,11406,11407,11408,11409,11410,11411,11412,11413,11414,11415,11416,11417,11418,11419,11420,11421,11422,11423,11424,11425,11426,11427,11428,11429,11430,11431,11432,11433,11434,11435,11436,11437,11438,11439,11440,11441,11442,11443,11444,11445,11446,11447,11448,11449,11450,11451,11452,11453,11454,11455,11456,11457,11458,11459,11460,11461,11462,11463,11464,11465,11466,11467,11468,11469,11470,11471,11472,11473,11474,11475,11476,11477,11478,11479,11480,11481,11482,11483,11484,11485,11486,11487,11488,11489,11490,11491,11492,11499,11500,11501,11502,11506,11507,11520,11521,11522,11523,11524,11525,11526,11527,11528,11529,11530,11531,11532,11533,11534,11535,11536,11537,11538,11539,11540,11541,11542,11543,11544,11545,11546,11547,11548,11549,11550,11551,11552,11553,11554,11555,11556,11557,11559,11565,11568,11569,11570,11571,11572,11573,11574,11575,11576,11577,11578,11579,11580,11581,11582,11583,11584,11585,11586,11587,11588,11589,11590,11591,11592,11593,11594,11595,11596,11597,11598,11599,11600,11601,11602,11603,11604,11605,11606,11607,11608,11609,11610,11611,11612,11613,11614,11615,11616,11617,11618,11619,11620,11621,11622,11623,11631,11648,11649,11650,11651,11652,11653,11654,11655,11656,11657,11658,11659,11660,11661,11662,11663,11664,11665,11666,11667,11668,11669,11670,11680,11681,11682,11683,11684,11685,11686,11688,11689,11690,11691,11692,11693,11694,11696,11697,11698,11699,11700,11701,11702,11704,11705,11706,11707,11708,11709,11710,11712,11713,11714,11715,11716,11717,11718,11720,11721,11722,11723,11724,11725,11726,11728,11729,11730,11731,11732,11733,11734,11736,11737,11738,11739,11740,11741,11742,11823,12293,12294,12295,12321,12322,12323,12324,12325,12326,12327,12328,12329,12337,12338,12339,12340,12341,12344,12345,12346,12347,12348,12353,12354,12355,12356,12357,12358,12359,12360,12361,12362,12363,12364,12365,12366,12367,12368,12369,12370,12371,12372,12373,12374,12375,12376,12377,12378,12379,12380,12381,12382,12383,12384,12385,12386,12387,12388,12389,12390,12391,12392,12393,12394,12395,12396,12397,12398,12399,12400,12401,12402,12403,12404,12405,12406,12407,12408,12409,12410,12411,12412,12413,12414,12415,12416,12417,12418,12419,12420,12421,12422,12423,12424,12425,12426,12427,12428,12429,12430,12431,12432,12433,12434,12435,12436,12437,12438,12445,12446,12447,12449,12450,12451,12452,12453,12454,12455,12456,12457,12458,12459,12460,12461,12462,12463,12464,12465,12466,12467,12468,12469,12470,12471,12472,12473,12474,12475,12476,12477,12478,12479,12480,12481,12482,12483,12484,12485,12486,12487,12488,12489,12490,12491,12492,12493,12494,12495,12496,12497,12498,12499,12500,12501,12502,12503,12504,12505,12506,12507,12508,12509,12510,12511,12512,12513,12514,12515,12516,12517,12518,12519,12520,12521,12522,12523,12524,12525,12526,12527,12528,12529,12530,12531,12532,12533,12534,12535,12536,12537,12538,12540,12541,12542,12543,12549,12550,12551,12552,12553,12554,12555,12556,12557,12558,12559,12560,12561,12562,12563,12564,12565,12566,12567,12568,12569,12570,12571,12572,12573,12574,12575,12576,12577,12578,12579,12580,12581,12582,12583,12584,12585,12586,12587,12588,12589,12593,12594,12595,12596,12597,12598,12599,12600,12601,12602,12603,12604,12605,12606,12607,12608,12609,12610,12611,12612,12613,12614,12615,12616,12617,12618,12619,12620,12621,12622,12623,12624,12625,12626,12627,12628,12629,12630,12631,12632,12633,12634,12635,12636,12637,12638,12639,12640,12641,12642,12643,12644,12645,12646,12647,12648,12649,12650,12651,12652,12653,12654,12655,12656,12657,12658,12659,12660,12661,12662,12663,12664,12665,12666,12667,12668,12669,12670,12671,12672,12673,12674,12675,12676,12677,12678,12679,12680,12681,12682,12683,12684,12685,12686,12704,12705,12706,12707,12708,12709,12710,12711,12712,12713,12714,12715,12716,12717,12718,12719,12720,12721,12722,12723,12724,12725,12726,12727,12728,12729,12730,12784,12785,12786,12787,12788,12789,12790,12791,12792,12793,12794,12795,12796,12797,12798,12799,13312,13313,13314,13315,13316,13317,13318,13319,13320,13321,13322,13323,13324,13325,13326,13327,13328,13329,13330,13331,13332,13333,13334,13335,13336,13337,13338,13339,13340,13341,13342,13343,13344,13345,13346,13347,13348,13349,13350,13351,13352,13353,13354,13355,13356,13357,13358,13359,13360,13361,13362,13363,13364,13365,13366,13367,13368,13369,13370,13371,13372,13373,13374,13375,13376,13377,13378,13379,13380,13381,13382,13383,13384,13385,13386,13387,13388,13389,13390,13391,13392,13393,13394,13395,13396,13397,13398,13399,13400,13401,13402,13403,13404,13405,13406,13407,13408,13409,13410,13411,13412,13413,13414,13415,13416,13417,13418,13419,13420,13421,13422,13423,13424,13425,13426,13427,13428,13429,13430,13431,13432,13433,13434,13435,13436,13437,13438,13439,13440,13441,13442,13443,13444,13445,13446,13447,13448,13449,13450,13451,13452,13453,13454,13455,13456,13457,13458,13459,13460,13461,13462,13463,13464,13465,13466,13467,13468,13469,13470,13471,13472,13473,13474,13475,13476,13477,13478,13479,13480,13481,13482,13483,13484,13485,13486,13487,13488,13489,13490,13491,13492,13493,13494,13495,13496,13497,13498,13499,13500,13501,13502,13503,13504,13505,13506,13507,13508,13509,13510,13511,13512,13513,13514,13515,13516,13517,13518,13519,13520,13521,13522,13523,13524,13525,13526,13527,13528,13529,13530,13531,13532,13533,13534,13535,13536,13537,13538,13539,13540,13541,13542,13543,13544,13545,13546,13547,13548,13549,13550,13551,13552,13553,13554,13555,13556,13557,13558,13559,13560,13561,13562,13563,13564,13565,13566,13567,13568,13569,13570,13571,13572,13573,13574,13575,13576,13577,13578,13579,13580,13581,13582,13583,13584,13585,13586,13587,13588,13589,13590,13591,13592,13593,13594,13595,13596,13597,13598,13599,13600,13601,13602,13603,13604,13605,13606,13607,13608,13609,13610,13611,13612,13613,13614,13615,13616,13617,13618,13619,13620,13621,13622,13623,13624,13625,13626,13627,13628,13629,13630,13631,13632,13633,13634,13635,13636,13637,13638,13639,13640,13641,13642,13643,13644,13645,13646,13647,13648,13649,13650,13651,13652,13653,13654,13655,13656,13657,13658,13659,13660,13661,13662,13663,13664,13665,13666,13667,13668,13669,13670,13671,13672,13673,13674,13675,13676,13677,13678,13679,13680,13681,13682,13683,13684,13685,13686,13687,13688,13689,13690,13691,13692,13693,13694,13695,13696,13697,13698,13699,13700,13701,13702,13703,13704,13705,13706,13707,13708,13709,13710,13711,13712,13713,13714,13715,13716,13717,13718,13719,13720,13721,13722,13723,13724,13725,13726,13727,13728,13729,13730,13731,13732,13733,13734,13735,13736,13737,13738,13739,13740,13741,13742,13743,13744,13745,13746,13747,13748,13749,13750,13751,13752,13753,13754,13755,13756,13757,13758,13759,13760,13761,13762,13763,13764,13765,13766,13767,13768,13769,13770,13771,13772,13773,13774,13775,13776,13777,13778,13779,13780,13781,13782,13783,13784,13785,13786,13787,13788,13789,13790,13791,13792,13793,13794,13795,13796,13797,13798,13799,13800,13801,13802,13803,13804,13805,13806,13807,13808,13809,13810,13811,13812,13813,13814,13815,13816,13817,13818,13819,13820,13821,13822,13823,13824,13825,13826,13827,13828,13829,13830,13831,13832,13833,13834,13835,13836,13837,13838,13839,13840,13841,13842,13843,13844,13845,13846,13847,13848,13849,13850,13851,13852,13853,13854,13855,13856,13857,13858,13859,13860,13861,13862,13863,13864,13865,13866,13867,13868,13869,13870,13871,13872,13873,13874,13875,13876,13877,13878,13879,13880,13881,13882,13883,13884,13885,13886,13887,13888,13889,13890,13891,13892,13893,13894,13895,13896,13897,13898,13899,13900,13901,13902,13903,13904,13905,13906,13907,13908,13909,13910,13911,13912,13913,13914,13915,13916,13917,13918,13919,13920,13921,13922,13923,13924,13925,13926,13927,13928,13929,13930,13931,13932,13933,13934,13935,13936,13937,13938,13939,13940,13941,13942,13943,13944,13945,13946,13947,13948,13949,13950,13951,13952,13953,13954,13955,13956,13957,13958,13959,13960,13961,13962,13963,13964,13965,13966,13967,13968,13969,13970,13971,13972,13973,13974,13975,13976,13977,13978,13979,13980,13981,13982,13983,13984,13985,13986,13987,13988,13989,13990,13991,13992,13993,13994,13995,13996,13997,13998,13999,14000,14001,14002,14003,14004,14005,14006,14007,14008,14009,14010,14011,14012,14013,14014,14015,14016,14017,14018,14019,14020,14021,14022,14023,14024,14025,14026,14027,14028,14029,14030,14031,14032,14033,14034,14035,14036,14037,14038,14039,14040,14041,14042,14043,14044,14045,14046,14047,14048,14049,14050,14051,14052,14053,14054,14055,14056,14057,14058,14059,14060,14061,14062,14063,14064,14065,14066,14067,14068,14069,14070,14071,14072,14073,14074,14075,14076,14077,14078,14079,14080,14081,14082,14083,14084,14085,14086,14087,14088,14089,14090,14091,14092,14093,14094,14095,14096,14097,14098,14099,14100,14101,14102,14103,14104,14105,14106,14107,14108,14109,14110,14111,14112,14113,14114,14115,14116,14117,14118,14119,14120,14121,14122,14123,14124,14125,14126,14127,14128,14129,14130,14131,14132,14133,14134,14135,14136,14137,14138,14139,14140,14141,14142,14143,14144,14145,14146,14147,14148,14149,14150,14151,14152,14153,14154,14155,14156,14157,14158,14159,14160,14161,14162,14163,14164,14165,14166,14167,14168,14169,14170,14171,14172,14173,14174,14175,14176,14177,14178,14179,14180,14181,14182,14183,14184,14185,14186,14187,14188,14189,14190,14191,14192,14193,14194,14195,14196,14197,14198,14199,14200,14201,14202,14203,14204,14205,14206,14207,14208,14209,14210,14211,14212,14213,14214,14215,14216,14217,14218,14219,14220,14221,14222,14223,14224,14225,14226,14227,14228,14229,14230,14231,14232,14233,14234,14235,14236,14237,14238,14239,14240,14241,14242,14243,14244,14245,14246,14247,14248,14249,14250,14251,14252,14253,14254,14255,14256,14257,14258,14259,14260,14261,14262,14263,14264,14265,14266,14267,14268,14269,14270,14271,14272,14273,14274,14275,14276,14277,14278,14279,14280,14281,14282,14283,14284,14285,14286,14287,14288,14289,14290,14291,14292,14293,14294,14295,14296,14297,14298,14299,14300,14301,14302,14303,14304,14305,14306,14307,14308,14309,14310,14311,14312,14313,14314,14315,14316,14317,14318,14319,14320,14321,14322,14323,14324,14325,14326,14327,14328,14329,14330,14331,14332,14333,14334,14335,14336,14337,14338,14339,14340,14341,14342,14343,14344,14345,14346,14347,14348,14349,14350,14351,14352,14353,14354,14355,14356,14357,14358,14359,14360,14361,14362,14363,14364,14365,14366,14367,14368,14369,14370,14371,14372,14373,14374,14375,14376,14377,14378,14379,14380,14381,14382,14383,14384,14385,14386,14387,14388,14389,14390,14391,14392,14393,14394,14395,14396,14397,14398,14399,14400,14401,14402,14403,14404,14405,14406,14407,14408,14409,14410,14411,14412,14413,14414,14415,14416,14417,14418,14419,14420,14421,14422,14423,14424,14425,14426,14427,14428,14429,14430,14431,14432,14433,14434,14435,14436,14437,14438,14439,14440,14441,14442,14443,14444,14445,14446,14447,14448,14449,14450,14451,14452,14453,14454,14455,14456,14457,14458,14459,14460,14461,14462,14463,14464,14465,14466,14467,14468,14469,14470,14471,14472,14473,14474,14475,14476,14477,14478,14479,14480,14481,14482,14483,14484,14485,14486,14487,14488,14489,14490,14491,14492,14493,14494,14495,14496,14497,14498,14499,14500,14501,14502,14503,14504,14505,14506,14507,14508,14509,14510,14511,14512,14513,14514,14515,14516,14517,14518,14519,14520,14521,14522,14523,14524,14525,14526,14527,14528,14529,14530,14531,14532,14533,14534,14535,14536,14537,14538,14539,14540,14541,14542,14543,14544,14545,14546,14547,14548,14549,14550,14551,14552,14553,14554,14555,14556,14557,14558,14559,14560,14561,14562,14563,14564,14565,14566,14567,14568,14569,14570,14571,14572,14573,14574,14575,14576,14577,14578,14579,14580,14581,14582,14583,14584,14585,14586,14587,14588,14589,14590,14591,14592,14593,14594,14595,14596,14597,14598,14599,14600,14601,14602,14603,14604,14605,14606,14607,14608,14609,14610,14611,14612,14613,14614,14615,14616,14617,14618,14619,14620,14621,14622,14623,14624,14625,14626,14627,14628,14629,14630,14631,14632,14633,14634,14635,14636,14637,14638,14639,14640,14641,14642,14643,14644,14645,14646,14647,14648,14649,14650,14651,14652,14653,14654,14655,14656,14657,14658,14659,14660,14661,14662,14663,14664,14665,14666,14667,14668,14669,14670,14671,14672,14673,14674,14675,14676,14677,14678,14679,14680,14681,14682,14683,14684,14685,14686,14687,14688,14689,14690,14691,14692,14693,14694,14695,14696,14697,14698,14699,14700,14701,14702,14703,14704,14705,14706,14707,14708,14709,14710,14711,14712,14713,14714,14715,14716,14717,14718,14719,14720,14721,14722,14723,14724,14725,14726,14727,14728,14729,14730,14731,14732,14733,14734,14735,14736,14737,14738,14739,14740,14741,14742,14743,14744,14745,14746,14747,14748,14749,14750,14751,14752,14753,14754,14755,14756,14757,14758,14759,14760,14761,14762,14763,14764,14765,14766,14767,14768,14769,14770,14771,14772,14773,14774,14775,14776,14777,14778,14779,14780,14781,14782,14783,14784,14785,14786,14787,14788,14789,14790,14791,14792,14793,14794,14795,14796,14797,14798,14799,14800,14801,14802,14803,14804,14805,14806,14807,14808,14809,14810,14811,14812,14813,14814,14815,14816,14817,14818,14819,14820,14821,14822,14823,14824,14825,14826,14827,14828,14829,14830,14831,14832,14833,14834,14835,14836,14837,14838,14839,14840,14841,14842,14843,14844,14845,14846,14847,14848,14849,14850,14851,14852,14853,14854,14855,14856,14857,14858,14859,14860,14861,14862,14863,14864,14865,14866,14867,14868,14869,14870,14871,14872,14873,14874,14875,14876,14877,14878,14879,14880,14881,14882,14883,14884,14885,14886,14887,14888,14889,14890,14891,14892,14893,14894,14895,14896,14897,14898,14899,14900,14901,14902,14903,14904,14905,14906,14907,14908,14909,14910,14911,14912,14913,14914,14915,14916,14917,14918,14919,14920,14921,14922,14923,14924,14925,14926,14927,14928,14929,14930,14931,14932,14933,14934,14935,14936,14937,14938,14939,14940,14941,14942,14943,14944,14945,14946,14947,14948,14949,14950,14951,14952,14953,14954,14955,14956,14957,14958,14959,14960,14961,14962,14963,14964,14965,14966,14967,14968,14969,14970,14971,14972,14973,14974,14975,14976,14977,14978,14979,14980,14981,14982,14983,14984,14985,14986,14987,14988,14989,14990,14991,14992,14993,14994,14995,14996,14997,14998,14999,15000,15001,15002,15003,15004,15005,15006,15007,15008,15009,15010,15011,15012,15013,15014,15015,15016,15017,15018,15019,15020,15021,15022,15023,15024,15025,15026,15027,15028,15029,15030,15031,15032,15033,15034,15035,15036,15037,15038,15039,15040,15041,15042,15043,15044,15045,15046,15047,15048,15049,15050,15051,15052,15053,15054,15055,15056,15057,15058,15059,15060,15061,15062,15063,15064,15065,15066,15067,15068,15069,15070,15071,15072,15073,15074,15075,15076,15077,15078,15079,15080,15081,15082,15083,15084,15085,15086,15087,15088,15089,15090,15091,15092,15093,15094,15095,15096,15097,15098,15099,15100,15101,15102,15103,15104,15105,15106,15107,15108,15109,15110,15111,15112,15113,15114,15115,15116,15117,15118,15119,15120,15121,15122,15123,15124,15125,15126,15127,15128,15129,15130,15131,15132,15133,15134,15135,15136,15137,15138,15139,15140,15141,15142,15143,15144,15145,15146,15147,15148,15149,15150,15151,15152,15153,15154,15155,15156,15157,15158,15159,15160,15161,15162,15163,15164,15165,15166,15167,15168,15169,15170,15171,15172,15173,15174,15175,15176,15177,15178,15179,15180,15181,15182,15183,15184,15185,15186,15187,15188,15189,15190,15191,15192,15193,15194,15195,15196,15197,15198,15199,15200,15201,15202,15203,15204,15205,15206,15207,15208,15209,15210,15211,15212,15213,15214,15215,15216,15217,15218,15219,15220,15221,15222,15223,15224,15225,15226,15227,15228,15229,15230,15231,15232,15233,15234,15235,15236,15237,15238,15239,15240,15241,15242,15243,15244,15245,15246,15247,15248,15249,15250,15251,15252,15253,15254,15255,15256,15257,15258,15259,15260,15261,15262,15263,15264,15265,15266,15267,15268,15269,15270,15271,15272,15273,15274,15275,15276,15277,15278,15279,15280,15281,15282,15283,15284,15285,15286,15287,15288,15289,15290,15291,15292,15293,15294,15295,15296,15297,15298,15299,15300,15301,15302,15303,15304,15305,15306,15307,15308,15309,15310,15311,15312,15313,15314,15315,15316,15317,15318,15319,15320,15321,15322,15323,15324,15325,15326,15327,15328,15329,15330,15331,15332,15333,15334,15335,15336,15337,15338,15339,15340,15341,15342,15343,15344,15345,15346,15347,15348,15349,15350,15351,15352,15353,15354,15355,15356,15357,15358,15359,15360,15361,15362,15363,15364,15365,15366,15367,15368,15369,15370,15371,15372,15373,15374,15375,15376,15377,15378,15379,15380,15381,15382,15383,15384,15385,15386,15387,15388,15389,15390,15391,15392,15393,15394,15395,15396,15397,15398,15399,15400,15401,15402,15403,15404,15405,15406,15407,15408,15409,15410,15411,15412,15413,15414,15415,15416,15417,15418,15419,15420,15421,15422,15423,15424,15425,15426,15427,15428,15429,15430,15431,15432,15433,15434,15435,15436,15437,15438,15439,15440,15441,15442,15443,15444,15445,15446,15447,15448,15449,15450,15451,15452,15453,15454,15455,15456,15457,15458,15459,15460,15461,15462,15463,15464,15465,15466,15467,15468,15469,15470,15471,15472,15473,15474,15475,15476,15477,15478,15479,15480,15481,15482,15483,15484,15485,15486,15487,15488,15489,15490,15491,15492,15493,15494,15495,15496,15497,15498,15499,15500,15501,15502,15503,15504,15505,15506,15507,15508,15509,15510,15511,15512,15513,15514,15515,15516,15517,15518,15519,15520,15521,15522,15523,15524,15525,15526,15527,15528,15529,15530,15531,15532,15533,15534,15535,15536,15537,15538,15539,15540,15541,15542,15543,15544,15545,15546,15547,15548,15549,15550,15551,15552,15553,15554,15555,15556,15557,15558,15559,15560,15561,15562,15563,15564,15565,15566,15567,15568,15569,15570,15571,15572,15573,15574,15575,15576,15577,15578,15579,15580,15581,15582,15583,15584,15585,15586,15587,15588,15589,15590,15591,15592,15593,15594,15595,15596,15597,15598,15599,15600,15601,15602,15603,15604,15605,15606,15607,15608,15609,15610,15611,15612,15613,15614,15615,15616,15617,15618,15619,15620,15621,15622,15623,15624,15625,15626,15627,15628,15629,15630,15631,15632,15633,15634,15635,15636,15637,15638,15639,15640,15641,15642,15643,15644,15645,15646,15647,15648,15649,15650,15651,15652,15653,15654,15655,15656,15657,15658,15659,15660,15661,15662,15663,15664,15665,15666,15667,15668,15669,15670,15671,15672,15673,15674,15675,15676,15677,15678,15679,15680,15681,15682,15683,15684,15685,15686,15687,15688,15689,15690,15691,15692,15693,15694,15695,15696,15697,15698,15699,15700,15701,15702,15703,15704,15705,15706,15707,15708,15709,15710,15711,15712,15713,15714,15715,15716,15717,15718,15719,15720,15721,15722,15723,15724,15725,15726,15727,15728,15729,15730,15731,15732,15733,15734,15735,15736,15737,15738,15739,15740,15741,15742,15743,15744,15745,15746,15747,15748,15749,15750,15751,15752,15753,15754,15755,15756,15757,15758,15759,15760,15761,15762,15763,15764,15765,15766,15767,15768,15769,15770,15771,15772,15773,15774,15775,15776,15777,15778,15779,15780,15781,15782,15783,15784,15785,15786,15787,15788,15789,15790,15791,15792,15793,15794,15795,15796,15797,15798,15799,15800,15801,15802,15803,15804,15805,15806,15807,15808,15809,15810,15811,15812,15813,15814,15815,15816,15817,15818,15819,15820,15821,15822,15823,15824,15825,15826,15827,15828,15829,15830,15831,15832,15833,15834,15835,15836,15837,15838,15839,15840,15841,15842,15843,15844,15845,15846,15847,15848,15849,15850,15851,15852,15853,15854,15855,15856,15857,15858,15859,15860,15861,15862,15863,15864,15865,15866,15867,15868,15869,15870,15871,15872,15873,15874,15875,15876,15877,15878,15879,15880,15881,15882,15883,15884,15885,15886,15887,15888,15889,15890,15891,15892,15893,15894,15895,15896,15897,15898,15899,15900,15901,15902,15903,15904,15905,15906,15907,15908,15909,15910,15911,15912,15913,15914,15915,15916,15917,15918,15919,15920,15921,15922,15923,15924,15925,15926,15927,15928,15929,15930,15931,15932,15933,15934,15935,15936,15937,15938,15939,15940,15941,15942,15943,15944,15945,15946,15947,15948,15949,15950,15951,15952,15953,15954,15955,15956,15957,15958,15959,15960,15961,15962,15963,15964,15965,15966,15967,15968,15969,15970,15971,15972,15973,15974,15975,15976,15977,15978,15979,15980,15981,15982,15983,15984,15985,15986,15987,15988,15989,15990,15991,15992,15993,15994,15995,15996,15997,15998,15999,16000,16001,16002,16003,16004,16005,16006,16007,16008,16009,16010,16011,16012,16013,16014,16015,16016,16017,16018,16019,16020,16021,16022,16023,16024,16025,16026,16027,16028,16029,16030,16031,16032,16033,16034,16035,16036,16037,16038,16039,16040,16041,16042,16043,16044,16045,16046,16047,16048,16049,16050,16051,16052,16053,16054,16055,16056,16057,16058,16059,16060,16061,16062,16063,16064,16065,16066,16067,16068,16069,16070,16071,16072,16073,16074,16075,16076,16077,16078,16079,16080,16081,16082,16083,16084,16085,16086,16087,16088,16089,16090,16091,16092,16093,16094,16095,16096,16097,16098,16099,16100,16101,16102,16103,16104,16105,16106,16107,16108,16109,16110,16111,16112,16113,16114,16115,16116,16117,16118,16119,16120,16121,16122,16123,16124,16125,16126,16127,16128,16129,16130,16131,16132,16133,16134,16135,16136,16137,16138,16139,16140,16141,16142,16143,16144,16145,16146,16147,16148,16149,16150,16151,16152,16153,16154,16155,16156,16157,16158,16159,16160,16161,16162,16163,16164,16165,16166,16167,16168,16169,16170,16171,16172,16173,16174,16175,16176,16177,16178,16179,16180,16181,16182,16183,16184,16185,16186,16187,16188,16189,16190,16191,16192,16193,16194,16195,16196,16197,16198,16199,16200,16201,16202,16203,16204,16205,16206,16207,16208,16209,16210,16211,16212,16213,16214,16215,16216,16217,16218,16219,16220,16221,16222,16223,16224,16225,16226,16227,16228,16229,16230,16231,16232,16233,16234,16235,16236,16237,16238,16239,16240,16241,16242,16243,16244,16245,16246,16247,16248,16249,16250,16251,16252,16253,16254,16255,16256,16257,16258,16259,16260,16261,16262,16263,16264,16265,16266,16267,16268,16269,16270,16271,16272,16273,16274,16275,16276,16277,16278,16279,16280,16281,16282,16283,16284,16285,16286,16287,16288,16289,16290,16291,16292,16293,16294,16295,16296,16297,16298,16299,16300,16301,16302,16303,16304,16305,16306,16307,16308,16309,16310,16311,16312,16313,16314,16315,16316,16317,16318,16319,16320,16321,16322,16323,16324,16325,16326,16327,16328,16329,16330,16331,16332,16333,16334,16335,16336,16337,16338,16339,16340,16341,16342,16343,16344,16345,16346,16347,16348,16349,16350,16351,16352,16353,16354,16355,16356,16357,16358,16359,16360,16361,16362,16363,16364,16365,16366,16367,16368,16369,16370,16371,16372,16373,16374,16375,16376,16377,16378,16379,16380,16381,16382,16383,16384,16385,16386,16387,16388,16389,16390,16391,16392,16393,16394,16395,16396,16397,16398,16399,16400,16401,16402,16403,16404,16405,16406,16407,16408,16409,16410,16411,16412,16413,16414,16415,16416,16417,16418,16419,16420,16421,16422,16423,16424,16425,16426,16427,16428,16429,16430,16431,16432,16433,16434,16435,16436,16437,16438,16439,16440,16441,16442,16443,16444,16445,16446,16447,16448,16449,16450,16451,16452,16453,16454,16455,16456,16457,16458,16459,16460,16461,16462,16463,16464,16465,16466,16467,16468,16469,16470,16471,16472,16473,16474,16475,16476,16477,16478,16479,16480,16481,16482,16483,16484,16485,16486,16487,16488,16489,16490,16491,16492,16493,16494,16495,16496,16497,16498,16499,16500,16501,16502,16503,16504,16505,16506,16507,16508,16509,16510,16511,16512,16513,16514,16515,16516,16517,16518,16519,16520,16521,16522,16523,16524,16525,16526,16527,16528,16529,16530,16531,16532,16533,16534,16535,16536,16537,16538,16539,16540,16541,16542,16543,16544,16545,16546,16547,16548,16549,16550,16551,16552,16553,16554,16555,16556,16557,16558,16559,16560,16561,16562,16563,16564,16565,16566,16567,16568,16569,16570,16571,16572,16573,16574,16575,16576,16577,16578,16579,16580,16581,16582,16583,16584,16585,16586,16587,16588,16589,16590,16591,16592,16593,16594,16595,16596,16597,16598,16599,16600,16601,16602,16603,16604,16605,16606,16607,16608,16609,16610,16611,16612,16613,16614,16615,16616,16617,16618,16619,16620,16621,16622,16623,16624,16625,16626,16627,16628,16629,16630,16631,16632,16633,16634,16635,16636,16637,16638,16639,16640,16641,16642,16643,16644,16645,16646,16647,16648,16649,16650,16651,16652,16653,16654,16655,16656,16657,16658,16659,16660,16661,16662,16663,16664,16665,16666,16667,16668,16669,16670,16671,16672,16673,16674,16675,16676,16677,16678,16679,16680,16681,16682,16683,16684,16685,16686,16687,16688,16689,16690,16691,16692,16693,16694,16695,16696,16697,16698,16699,16700,16701,16702,16703,16704,16705,16706,16707,16708,16709,16710,16711,16712,16713,16714,16715,16716,16717,16718,16719,16720,16721,16722,16723,16724,16725,16726,16727,16728,16729,16730,16731,16732,16733,16734,16735,16736,16737,16738,16739,16740,16741,16742,16743,16744,16745,16746,16747,16748,16749,16750,16751,16752,16753,16754,16755,16756,16757,16758,16759,16760,16761,16762,16763,16764,16765,16766,16767,16768,16769,16770,16771,16772,16773,16774,16775,16776,16777,16778,16779,16780,16781,16782,16783,16784,16785,16786,16787,16788,16789,16790,16791,16792,16793,16794,16795,16796,16797,16798,16799,16800,16801,16802,16803,16804,16805,16806,16807,16808,16809,16810,16811,16812,16813,16814,16815,16816,16817,16818,16819,16820,16821,16822,16823,16824,16825,16826,16827,16828,16829,16830,16831,16832,16833,16834,16835,16836,16837,16838,16839,16840,16841,16842,16843,16844,16845,16846,16847,16848,16849,16850,16851,16852,16853,16854,16855,16856,16857,16858,16859,16860,16861,16862,16863,16864,16865,16866,16867,16868,16869,16870,16871,16872,16873,16874,16875,16876,16877,16878,16879,16880,16881,16882,16883,16884,16885,16886,16887,16888,16889,16890,16891,16892,16893,16894,16895,16896,16897,16898,16899,16900,16901,16902,16903,16904,16905,16906,16907,16908,16909,16910,16911,16912,16913,16914,16915,16916,16917,16918,16919,16920,16921,16922,16923,16924,16925,16926,16927,16928,16929,16930,16931,16932,16933,16934,16935,16936,16937,16938,16939,16940,16941,16942,16943,16944,16945,16946,16947,16948,16949,16950,16951,16952,16953,16954,16955,16956,16957,16958,16959,16960,16961,16962,16963,16964,16965,16966,16967,16968,16969,16970,16971,16972,16973,16974,16975,16976,16977,16978,16979,16980,16981,16982,16983,16984,16985,16986,16987,16988,16989,16990,16991,16992,16993,16994,16995,16996,16997,16998,16999,17000,17001,17002,17003,17004,17005,17006,17007,17008,17009,17010,17011,17012,17013,17014,17015,17016,17017,17018,17019,17020,17021,17022,17023,17024,17025,17026,17027,17028,17029,17030,17031,17032,17033,17034,17035,17036,17037,17038,17039,17040,17041,17042,17043,17044,17045,17046,17047,17048,17049,17050,17051,17052,17053,17054,17055,17056,17057,17058,17059,17060,17061,17062,17063,17064,17065,17066,17067,17068,17069,17070,17071,17072,17073,17074,17075,17076,17077,17078,17079,17080,17081,17082,17083,17084,17085,17086,17087,17088,17089,17090,17091,17092,17093,17094,17095,17096,17097,17098,17099,17100,17101,17102,17103,17104,17105,17106,17107,17108,17109,17110,17111,17112,17113,17114,17115,17116,17117,17118,17119,17120,17121,17122,17123,17124,17125,17126,17127,17128,17129,17130,17131,17132,17133,17134,17135,17136,17137,17138,17139,17140,17141,17142,17143,17144,17145,17146,17147,17148,17149,17150,17151,17152,17153,17154,17155,17156,17157,17158,17159,17160,17161,17162,17163,17164,17165,17166,17167,17168,17169,17170,17171,17172,17173,17174,17175,17176,17177,17178,17179,17180,17181,17182,17183,17184,17185,17186,17187,17188,17189,17190,17191,17192,17193,17194,17195,17196,17197,17198,17199,17200,17201,17202,17203,17204,17205,17206,17207,17208,17209,17210,17211,17212,17213,17214,17215,17216,17217,17218,17219,17220,17221,17222,17223,17224,17225,17226,17227,17228,17229,17230,17231,17232,17233,17234,17235,17236,17237,17238,17239,17240,17241,17242,17243,17244,17245,17246,17247,17248,17249,17250,17251,17252,17253,17254,17255,17256,17257,17258,17259,17260,17261,17262,17263,17264,17265,17266,17267,17268,17269,17270,17271,17272,17273,17274,17275,17276,17277,17278,17279,17280,17281,17282,17283,17284,17285,17286,17287,17288,17289,17290,17291,17292,17293,17294,17295,17296,17297,17298,17299,17300,17301,17302,17303,17304,17305,17306,17307,17308,17309,17310,17311,17312,17313,17314,17315,17316,17317,17318,17319,17320,17321,17322,17323,17324,17325,17326,17327,17328,17329,17330,17331,17332,17333,17334,17335,17336,17337,17338,17339,17340,17341,17342,17343,17344,17345,17346,17347,17348,17349,17350,17351,17352,17353,17354,17355,17356,17357,17358,17359,17360,17361,17362,17363,17364,17365,17366,17367,17368,17369,17370,17371,17372,17373,17374,17375,17376,17377,17378,17379,17380,17381,17382,17383,17384,17385,17386,17387,17388,17389,17390,17391,17392,17393,17394,17395,17396,17397,17398,17399,17400,17401,17402,17403,17404,17405,17406,17407,17408,17409,17410,17411,17412,17413,17414,17415,17416,17417,17418,17419,17420,17421,17422,17423,17424,17425,17426,17427,17428,17429,17430,17431,17432,17433,17434,17435,17436,17437,17438,17439,17440,17441,17442,17443,17444,17445,17446,17447,17448,17449,17450,17451,17452,17453,17454,17455,17456,17457,17458,17459,17460,17461,17462,17463,17464,17465,17466,17467,17468,17469,17470,17471,17472,17473,17474,17475,17476,17477,17478,17479,17480,17481,17482,17483,17484,17485,17486,17487,17488,17489,17490,17491,17492,17493,17494,17495,17496,17497,17498,17499,17500,17501,17502,17503,17504,17505,17506,17507,17508,17509,17510,17511,17512,17513,17514,17515,17516,17517,17518,17519,17520,17521,17522,17523,17524,17525,17526,17527,17528,17529,17530,17531,17532,17533,17534,17535,17536,17537,17538,17539,17540,17541,17542,17543,17544,17545,17546,17547,17548,17549,17550,17551,17552,17553,17554,17555,17556,17557,17558,17559,17560,17561,17562,17563,17564,17565,17566,17567,17568,17569,17570,17571,17572,17573,17574,17575,17576,17577,17578,17579,17580,17581,17582,17583,17584,17585,17586,17587,17588,17589,17590,17591,17592,17593,17594,17595,17596,17597,17598,17599,17600,17601,17602,17603,17604,17605,17606,17607,17608,17609,17610,17611,17612,17613,17614,17615,17616,17617,17618,17619,17620,17621,17622,17623,17624,17625,17626,17627,17628,17629,17630,17631,17632,17633,17634,17635,17636,17637,17638,17639,17640,17641,17642,17643,17644,17645,17646,17647,17648,17649,17650,17651,17652,17653,17654,17655,17656,17657,17658,17659,17660,17661,17662,17663,17664,17665,17666,17667,17668,17669,17670,17671,17672,17673,17674,17675,17676,17677,17678,17679,17680,17681,17682,17683,17684,17685,17686,17687,17688,17689,17690,17691,17692,17693,17694,17695,17696,17697,17698,17699,17700,17701,17702,17703,17704,17705,17706,17707,17708,17709,17710,17711,17712,17713,17714,17715,17716,17717,17718,17719,17720,17721,17722,17723,17724,17725,17726,17727,17728,17729,17730,17731,17732,17733,17734,17735,17736,17737,17738,17739,17740,17741,17742,17743,17744,17745,17746,17747,17748,17749,17750,17751,17752,17753,17754,17755,17756,17757,17758,17759,17760,17761,17762,17763,17764,17765,17766,17767,17768,17769,17770,17771,17772,17773,17774,17775,17776,17777,17778,17779,17780,17781,17782,17783,17784,17785,17786,17787,17788,17789,17790,17791,17792,17793,17794,17795,17796,17797,17798,17799,17800,17801,17802,17803,17804,17805,17806,17807,17808,17809,17810,17811,17812,17813,17814,17815,17816,17817,17818,17819,17820,17821,17822,17823,17824,17825,17826,17827,17828,17829,17830,17831,17832,17833,17834,17835,17836,17837,17838,17839,17840,17841,17842,17843,17844,17845,17846,17847,17848,17849,17850,17851,17852,17853,17854,17855,17856,17857,17858,17859,17860,17861,17862,17863,17864,17865,17866,17867,17868,17869,17870,17871,17872,17873,17874,17875,17876,17877,17878,17879,17880,17881,17882,17883,17884,17885,17886,17887,17888,17889,17890,17891,17892,17893,17894,17895,17896,17897,17898,17899,17900,17901,17902,17903,17904,17905,17906,17907,17908,17909,17910,17911,17912,17913,17914,17915,17916,17917,17918,17919,17920,17921,17922,17923,17924,17925,17926,17927,17928,17929,17930,17931,17932,17933,17934,17935,17936,17937,17938,17939,17940,17941,17942,17943,17944,17945,17946,17947,17948,17949,17950,17951,17952,17953,17954,17955,17956,17957,17958,17959,17960,17961,17962,17963,17964,17965,17966,17967,17968,17969,17970,17971,17972,17973,17974,17975,17976,17977,17978,17979,17980,17981,17982,17983,17984,17985,17986,17987,17988,17989,17990,17991,17992,17993,17994,17995,17996,17997,17998,17999,18000,18001,18002,18003,18004,18005,18006,18007,18008,18009,18010,18011,18012,18013,18014,18015,18016,18017,18018,18019,18020,18021,18022,18023,18024,18025,18026,18027,18028,18029,18030,18031,18032,18033,18034,18035,18036,18037,18038,18039,18040,18041,18042,18043,18044,18045,18046,18047,18048,18049,18050,18051,18052,18053,18054,18055,18056,18057,18058,18059,18060,18061,18062,18063,18064,18065,18066,18067,18068,18069,18070,18071,18072,18073,18074,18075,18076,18077,18078,18079,18080,18081,18082,18083,18084,18085,18086,18087,18088,18089,18090,18091,18092,18093,18094,18095,18096,18097,18098,18099,18100,18101,18102,18103,18104,18105,18106,18107,18108,18109,18110,18111,18112,18113,18114,18115,18116,18117,18118,18119,18120,18121,18122,18123,18124,18125,18126,18127,18128,18129,18130,18131,18132,18133,18134,18135,18136,18137,18138,18139,18140,18141,18142,18143,18144,18145,18146,18147,18148,18149,18150,18151,18152,18153,18154,18155,18156,18157,18158,18159,18160,18161,18162,18163,18164,18165,18166,18167,18168,18169,18170,18171,18172,18173,18174,18175,18176,18177,18178,18179,18180,18181,18182,18183,18184,18185,18186,18187,18188,18189,18190,18191,18192,18193,18194,18195,18196,18197,18198,18199,18200,18201,18202,18203,18204,18205,18206,18207,18208,18209,18210,18211,18212,18213,18214,18215,18216,18217,18218,18219,18220,18221,18222,18223,18224,18225,18226,18227,18228,18229,18230,18231,18232,18233,18234,18235,18236,18237,18238,18239,18240,18241,18242,18243,18244,18245,18246,18247,18248,18249,18250,18251,18252,18253,18254,18255,18256,18257,18258,18259,18260,18261,18262,18263,18264,18265,18266,18267,18268,18269,18270,18271,18272,18273,18274,18275,18276,18277,18278,18279,18280,18281,18282,18283,18284,18285,18286,18287,18288,18289,18290,18291,18292,18293,18294,18295,18296,18297,18298,18299,18300,18301,18302,18303,18304,18305,18306,18307,18308,18309,18310,18311,18312,18313,18314,18315,18316,18317,18318,18319,18320,18321,18322,18323,18324,18325,18326,18327,18328,18329,18330,18331,18332,18333,18334,18335,18336,18337,18338,18339,18340,18341,18342,18343,18344,18345,18346,18347,18348,18349,18350,18351,18352,18353,18354,18355,18356,18357,18358,18359,18360,18361,18362,18363,18364,18365,18366,18367,18368,18369,18370,18371,18372,18373,18374,18375,18376,18377,18378,18379,18380,18381,18382,18383,18384,18385,18386,18387,18388,18389,18390,18391,18392,18393,18394,18395,18396,18397,18398,18399,18400,18401,18402,18403,18404,18405,18406,18407,18408,18409,18410,18411,18412,18413,18414,18415,18416,18417,18418,18419,18420,18421,18422,18423,18424,18425,18426,18427,18428,18429,18430,18431,18432,18433,18434,18435,18436,18437,18438,18439,18440,18441,18442,18443,18444,18445,18446,18447,18448,18449,18450,18451,18452,18453,18454,18455,18456,18457,18458,18459,18460,18461,18462,18463,18464,18465,18466,18467,18468,18469,18470,18471,18472,18473,18474,18475,18476,18477,18478,18479,18480,18481,18482,18483,18484,18485,18486,18487,18488,18489,18490,18491,18492,18493,18494,18495,18496,18497,18498,18499,18500,18501,18502,18503,18504,18505,18506,18507,18508,18509,18510,18511,18512,18513,18514,18515,18516,18517,18518,18519,18520,18521,18522,18523,18524,18525,18526,18527,18528,18529,18530,18531,18532,18533,18534,18535,18536,18537,18538,18539,18540,18541,18542,18543,18544,18545,18546,18547,18548,18549,18550,18551,18552,18553,18554,18555,18556,18557,18558,18559,18560,18561,18562,18563,18564,18565,18566,18567,18568,18569,18570,18571,18572,18573,18574,18575,18576,18577,18578,18579,18580,18581,18582,18583,18584,18585,18586,18587,18588,18589,18590,18591,18592,18593,18594,18595,18596,18597,18598,18599,18600,18601,18602,18603,18604,18605,18606,18607,18608,18609,18610,18611,18612,18613,18614,18615,18616,18617,18618,18619,18620,18621,18622,18623,18624,18625,18626,18627,18628,18629,18630,18631,18632,18633,18634,18635,18636,18637,18638,18639,18640,18641,18642,18643,18644,18645,18646,18647,18648,18649,18650,18651,18652,18653,18654,18655,18656,18657,18658,18659,18660,18661,18662,18663,18664,18665,18666,18667,18668,18669,18670,18671,18672,18673,18674,18675,18676,18677,18678,18679,18680,18681,18682,18683,18684,18685,18686,18687,18688,18689,18690,18691,18692,18693,18694,18695,18696,18697,18698,18699,18700,18701,18702,18703,18704,18705,18706,18707,18708,18709,18710,18711,18712,18713,18714,18715,18716,18717,18718,18719,18720,18721,18722,18723,18724,18725,18726,18727,18728,18729,18730,18731,18732,18733,18734,18735,18736,18737,18738,18739,18740,18741,18742,18743,18744,18745,18746,18747,18748,18749,18750,18751,18752,18753,18754,18755,18756,18757,18758,18759,18760,18761,18762,18763,18764,18765,18766,18767,18768,18769,18770,18771,18772,18773,18774,18775,18776,18777,18778,18779,18780,18781,18782,18783,18784,18785,18786,18787,18788,18789,18790,18791,18792,18793,18794,18795,18796,18797,18798,18799,18800,18801,18802,18803,18804,18805,18806,18807,18808,18809,18810,18811,18812,18813,18814,18815,18816,18817,18818,18819,18820,18821,18822,18823,18824,18825,18826,18827,18828,18829,18830,18831,18832,18833,18834,18835,18836,18837,18838,18839,18840,18841,18842,18843,18844,18845,18846,18847,18848,18849,18850,18851,18852,18853,18854,18855,18856,18857,18858,18859,18860,18861,18862,18863,18864,18865,18866,18867,18868,18869,18870,18871,18872,18873,18874,18875,18876,18877,18878,18879,18880,18881,18882,18883,18884,18885,18886,18887,18888,18889,18890,18891,18892,18893,18894,18895,18896,18897,18898,18899,18900,18901,18902,18903,18904,18905,18906,18907,18908,18909,18910,18911,18912,18913,18914,18915,18916,18917,18918,18919,18920,18921,18922,18923,18924,18925,18926,18927,18928,18929,18930,18931,18932,18933,18934,18935,18936,18937,18938,18939,18940,18941,18942,18943,18944,18945,18946,18947,18948,18949,18950,18951,18952,18953,18954,18955,18956,18957,18958,18959,18960,18961,18962,18963,18964,18965,18966,18967,18968,18969,18970,18971,18972,18973,18974,18975,18976,18977,18978,18979,18980,18981,18982,18983,18984,18985,18986,18987,18988,18989,18990,18991,18992,18993,18994,18995,18996,18997,18998,18999,19000,19001,19002,19003,19004,19005,19006,19007,19008,19009,19010,19011,19012,19013,19014,19015,19016,19017,19018,19019,19020,19021,19022,19023,19024,19025,19026,19027,19028,19029,19030,19031,19032,19033,19034,19035,19036,19037,19038,19039,19040,19041,19042,19043,19044,19045,19046,19047,19048,19049,19050,19051,19052,19053,19054,19055,19056,19057,19058,19059,19060,19061,19062,19063,19064,19065,19066,19067,19068,19069,19070,19071,19072,19073,19074,19075,19076,19077,19078,19079,19080,19081,19082,19083,19084,19085,19086,19087,19088,19089,19090,19091,19092,19093,19094,19095,19096,19097,19098,19099,19100,19101,19102,19103,19104,19105,19106,19107,19108,19109,19110,19111,19112,19113,19114,19115,19116,19117,19118,19119,19120,19121,19122,19123,19124,19125,19126,19127,19128,19129,19130,19131,19132,19133,19134,19135,19136,19137,19138,19139,19140,19141,19142,19143,19144,19145,19146,19147,19148,19149,19150,19151,19152,19153,19154,19155,19156,19157,19158,19159,19160,19161,19162,19163,19164,19165,19166,19167,19168,19169,19170,19171,19172,19173,19174,19175,19176,19177,19178,19179,19180,19181,19182,19183,19184,19185,19186,19187,19188,19189,19190,19191,19192,19193,19194,19195,19196,19197,19198,19199,19200,19201,19202,19203,19204,19205,19206,19207,19208,19209,19210,19211,19212,19213,19214,19215,19216,19217,19218,19219,19220,19221,19222,19223,19224,19225,19226,19227,19228,19229,19230,19231,19232,19233,19234,19235,19236,19237,19238,19239,19240,19241,19242,19243,19244,19245,19246,19247,19248,19249,19250,19251,19252,19253,19254,19255,19256,19257,19258,19259,19260,19261,19262,19263,19264,19265,19266,19267,19268,19269,19270,19271,19272,19273,19274,19275,19276,19277,19278,19279,19280,19281,19282,19283,19284,19285,19286,19287,19288,19289,19290,19291,19292,19293,19294,19295,19296,19297,19298,19299,19300,19301,19302,19303,19304,19305,19306,19307,19308,19309,19310,19311,19312,19313,19314,19315,19316,19317,19318,19319,19320,19321,19322,19323,19324,19325,19326,19327,19328,19329,19330,19331,19332,19333,19334,19335,19336,19337,19338,19339,19340,19341,19342,19343,19344,19345,19346,19347,19348,19349,19350,19351,19352,19353,19354,19355,19356,19357,19358,19359,19360,19361,19362,19363,19364,19365,19366,19367,19368,19369,19370,19371,19372,19373,19374,19375,19376,19377,19378,19379,19380,19381,19382,19383,19384,19385,19386,19387,19388,19389,19390,19391,19392,19393,19394,19395,19396,19397,19398,19399,19400,19401,19402,19403,19404,19405,19406,19407,19408,19409,19410,19411,19412,19413,19414,19415,19416,19417,19418,19419,19420,19421,19422,19423,19424,19425,19426,19427,19428,19429,19430,19431,19432,19433,19434,19435,19436,19437,19438,19439,19440,19441,19442,19443,19444,19445,19446,19447,19448,19449,19450,19451,19452,19453,19454,19455,19456,19457,19458,19459,19460,19461,19462,19463,19464,19465,19466,19467,19468,19469,19470,19471,19472,19473,19474,19475,19476,19477,19478,19479,19480,19481,19482,19483,19484,19485,19486,19487,19488,19489,19490,19491,19492,19493,19494,19495,19496,19497,19498,19499,19500,19501,19502,19503,19504,19505,19506,19507,19508,19509,19510,19511,19512,19513,19514,19515,19516,19517,19518,19519,19520,19521,19522,19523,19524,19525,19526,19527,19528,19529,19530,19531,19532,19533,19534,19535,19536,19537,19538,19539,19540,19541,19542,19543,19544,19545,19546,19547,19548,19549,19550,19551,19552,19553,19554,19555,19556,19557,19558,19559,19560,19561,19562,19563,19564,19565,19566,19567,19568,19569,19570,19571,19572,19573,19574,19575,19576,19577,19578,19579,19580,19581,19582,19583,19584,19585,19586,19587,19588,19589,19590,19591,19592,19593,19594,19595,19596,19597,19598,19599,19600,19601,19602,19603,19604,19605,19606,19607,19608,19609,19610,19611,19612,19613,19614,19615,19616,19617,19618,19619,19620,19621,19622,19623,19624,19625,19626,19627,19628,19629,19630,19631,19632,19633,19634,19635,19636,19637,19638,19639,19640,19641,19642,19643,19644,19645,19646,19647,19648,19649,19650,19651,19652,19653,19654,19655,19656,19657,19658,19659,19660,19661,19662,19663,19664,19665,19666,19667,19668,19669,19670,19671,19672,19673,19674,19675,19676,19677,19678,19679,19680,19681,19682,19683,19684,19685,19686,19687,19688,19689,19690,19691,19692,19693,19694,19695,19696,19697,19698,19699,19700,19701,19702,19703,19704,19705,19706,19707,19708,19709,19710,19711,19712,19713,19714,19715,19716,19717,19718,19719,19720,19721,19722,19723,19724,19725,19726,19727,19728,19729,19730,19731,19732,19733,19734,19735,19736,19737,19738,19739,19740,19741,19742,19743,19744,19745,19746,19747,19748,19749,19750,19751,19752,19753,19754,19755,19756,19757,19758,19759,19760,19761,19762,19763,19764,19765,19766,19767,19768,19769,19770,19771,19772,19773,19774,19775,19776,19777,19778,19779,19780,19781,19782,19783,19784,19785,19786,19787,19788,19789,19790,19791,19792,19793,19794,19795,19796,19797,19798,19799,19800,19801,19802,19803,19804,19805,19806,19807,19808,19809,19810,19811,19812,19813,19814,19815,19816,19817,19818,19819,19820,19821,19822,19823,19824,19825,19826,19827,19828,19829,19830,19831,19832,19833,19834,19835,19836,19837,19838,19839,19840,19841,19842,19843,19844,19845,19846,19847,19848,19849,19850,19851,19852,19853,19854,19855,19856,19857,19858,19859,19860,19861,19862,19863,19864,19865,19866,19867,19868,19869,19870,19871,19872,19873,19874,19875,19876,19877,19878,19879,19880,19881,19882,19883,19884,19885,19886,19887,19888,19889,19890,19891,19892,19893,19968,19969,19970,19971,19972,19973,19974,19975,19976,19977,19978,19979,19980,19981,19982,19983,19984,19985,19986,19987,19988,19989,19990,19991,19992,19993,19994,19995,19996,19997,19998,19999,20000,20001,20002,20003,20004,20005,20006,20007,20008,20009,20010,20011,20012,20013,20014,20015,20016,20017,20018,20019,20020,20021,20022,20023,20024,20025,20026,20027,20028,20029,20030,20031,20032,20033,20034,20035,20036,20037,20038,20039,20040,20041,20042,20043,20044,20045,20046,20047,20048,20049,20050,20051,20052,20053,20054,20055,20056,20057,20058,20059,20060,20061,20062,20063,20064,20065,20066,20067,20068,20069,20070,20071,20072,20073,20074,20075,20076,20077,20078,20079,20080,20081,20082,20083,20084,20085,20086,20087,20088,20089,20090,20091,20092,20093,20094,20095,20096,20097,20098,20099,20100,20101,20102,20103,20104,20105,20106,20107,20108,20109,20110,20111,20112,20113,20114,20115,20116,20117,20118,20119,20120,20121,20122,20123,20124,20125,20126,20127,20128,20129,20130,20131,20132,20133,20134,20135,20136,20137,20138,20139,20140,20141,20142,20143,20144,20145,20146,20147,20148,20149,20150,20151,20152,20153,20154,20155,20156,20157,20158,20159,20160,20161,20162,20163,20164,20165,20166,20167,20168,20169,20170,20171,20172,20173,20174,20175,20176,20177,20178,20179,20180,20181,20182,20183,20184,20185,20186,20187,20188,20189,20190,20191,20192,20193,20194,20195,20196,20197,20198,20199,20200,20201,20202,20203,20204,20205,20206,20207,20208,20209,20210,20211,20212,20213,20214,20215,20216,20217,20218,20219,20220,20221,20222,20223,20224,20225,20226,20227,20228,20229,20230,20231,20232,20233,20234,20235,20236,20237,20238,20239,20240,20241,20242,20243,20244,20245,20246,20247,20248,20249,20250,20251,20252,20253,20254,20255,20256,20257,20258,20259,20260,20261,20262,20263,20264,20265,20266,20267,20268,20269,20270,20271,20272,20273,20274,20275,20276,20277,20278,20279,20280,20281,20282,20283,20284,20285,20286,20287,20288,20289,20290,20291,20292,20293,20294,20295,20296,20297,20298,20299,20300,20301,20302,20303,20304,20305,20306,20307,20308,20309,20310,20311,20312,20313,20314,20315,20316,20317,20318,20319,20320,20321,20322,20323,20324,20325,20326,20327,20328,20329,20330,20331,20332,20333,20334,20335,20336,20337,20338,20339,20340,20341,20342,20343,20344,20345,20346,20347,20348,20349,20350,20351,20352,20353,20354,20355,20356,20357,20358,20359,20360,20361,20362,20363,20364,20365,20366,20367,20368,20369,20370,20371,20372,20373,20374,20375,20376,20377,20378,20379,20380,20381,20382,20383,20384,20385,20386,20387,20388,20389,20390,20391,20392,20393,20394,20395,20396,20397,20398,20399,20400,20401,20402,20403,20404,20405,20406,20407,20408,20409,20410,20411,20412,20413,20414,20415,20416,20417,20418,20419,20420,20421,20422,20423,20424,20425,20426,20427,20428,20429,20430,20431,20432,20433,20434,20435,20436,20437,20438,20439,20440,20441,20442,20443,20444,20445,20446,20447,20448,20449,20450,20451,20452,20453,20454,20455,20456,20457,20458,20459,20460,20461,20462,20463,20464,20465,20466,20467,20468,20469,20470,20471,20472,20473,20474,20475,20476,20477,20478,20479,20480,20481,20482,20483,20484,20485,20486,20487,20488,20489,20490,20491,20492,20493,20494,20495,20496,20497,20498,20499,20500,20501,20502,20503,20504,20505,20506,20507,20508,20509,20510,20511,20512,20513,20514,20515,20516,20517,20518,20519,20520,20521,20522,20523,20524,20525,20526,20527,20528,20529,20530,20531,20532,20533,20534,20535,20536,20537,20538,20539,20540,20541,20542,20543,20544,20545,20546,20547,20548,20549,20550,20551,20552,20553,20554,20555,20556,20557,20558,20559,20560,20561,20562,20563,20564,20565,20566,20567,20568,20569,20570,20571,20572,20573,20574,20575,20576,20577,20578,20579,20580,20581,20582,20583,20584,20585,20586,20587,20588,20589,20590,20591,20592,20593,20594,20595,20596,20597,20598,20599,20600,20601,20602,20603,20604,20605,20606,20607,20608,20609,20610,20611,20612,20613,20614,20615,20616,20617,20618,20619,20620,20621,20622,20623,20624,20625,20626,20627,20628,20629,20630,20631,20632,20633,20634,20635,20636,20637,20638,20639,20640,20641,20642,20643,20644,20645,20646,20647,20648,20649,20650,20651,20652,20653,20654,20655,20656,20657,20658,20659,20660,20661,20662,20663,20664,20665,20666,20667,20668,20669,20670,20671,20672,20673,20674,20675,20676,20677,20678,20679,20680,20681,20682,20683,20684,20685,20686,20687,20688,20689,20690,20691,20692,20693,20694,20695,20696,20697,20698,20699,20700,20701,20702,20703,20704,20705,20706,20707,20708,20709,20710,20711,20712,20713,20714,20715,20716,20717,20718,20719,20720,20721,20722,20723,20724,20725,20726,20727,20728,20729,20730,20731,20732,20733,20734,20735,20736,20737,20738,20739,20740,20741,20742,20743,20744,20745,20746,20747,20748,20749,20750,20751,20752,20753,20754,20755,20756,20757,20758,20759,20760,20761,20762,20763,20764,20765,20766,20767,20768,20769,20770,20771,20772,20773,20774,20775,20776,20777,20778,20779,20780,20781,20782,20783,20784,20785,20786,20787,20788,20789,20790,20791,20792,20793,20794,20795,20796,20797,20798,20799,20800,20801,20802,20803,20804,20805,20806,20807,20808,20809,20810,20811,20812,20813,20814,20815,20816,20817,20818,20819,20820,20821,20822,20823,20824,20825,20826,20827,20828,20829,20830,20831,20832,20833,20834,20835,20836,20837,20838,20839,20840,20841,20842,20843,20844,20845,20846,20847,20848,20849,20850,20851,20852,20853,20854,20855,20856,20857,20858,20859,20860,20861,20862,20863,20864,20865,20866,20867,20868,20869,20870,20871,20872,20873,20874,20875,20876,20877,20878,20879,20880,20881,20882,20883,20884,20885,20886,20887,20888,20889,20890,20891,20892,20893,20894,20895,20896,20897,20898,20899,20900,20901,20902,20903,20904,20905,20906,20907,20908,20909,20910,20911,20912,20913,20914,20915,20916,20917,20918,20919,20920,20921,20922,20923,20924,20925,20926,20927,20928,20929,20930,20931,20932,20933,20934,20935,20936,20937,20938,20939,20940,20941,20942,20943,20944,20945,20946,20947,20948,20949,20950,20951,20952,20953,20954,20955,20956,20957,20958,20959,20960,20961,20962,20963,20964,20965,20966,20967,20968,20969,20970,20971,20972,20973,20974,20975,20976,20977,20978,20979,20980,20981,20982,20983,20984,20985,20986,20987,20988,20989,20990,20991,20992,20993,20994,20995,20996,20997,20998,20999,21000,21001,21002,21003,21004,21005,21006,21007,21008,21009,21010,21011,21012,21013,21014,21015,21016,21017,21018,21019,21020,21021,21022,21023,21024,21025,21026,21027,21028,21029,21030,21031,21032,21033,21034,21035,21036,21037,21038,21039,21040,21041,21042,21043,21044,21045,21046,21047,21048,21049,21050,21051,21052,21053,21054,21055,21056,21057,21058,21059,21060,21061,21062,21063,21064,21065,21066,21067,21068,21069,21070,21071,21072,21073,21074,21075,21076,21077,21078,21079,21080,21081,21082,21083,21084,21085,21086,21087,21088,21089,21090,21091,21092,21093,21094,21095,21096,21097,21098,21099,21100,21101,21102,21103,21104,21105,21106,21107,21108,21109,21110,21111,21112,21113,21114,21115,21116,21117,21118,21119,21120,21121,21122,21123,21124,21125,21126,21127,21128,21129,21130,21131,21132,21133,21134,21135,21136,21137,21138,21139,21140,21141,21142,21143,21144,21145,21146,21147,21148,21149,21150,21151,21152,21153,21154,21155,21156,21157,21158,21159,21160,21161,21162,21163,21164,21165,21166,21167,21168,21169,21170,21171,21172,21173,21174,21175,21176,21177,21178,21179,21180,21181,21182,21183,21184,21185,21186,21187,21188,21189,21190,21191,21192,21193,21194,21195,21196,21197,21198,21199,21200,21201,21202,21203,21204,21205,21206,21207,21208,21209,21210,21211,21212,21213,21214,21215,21216,21217,21218,21219,21220,21221,21222,21223,21224,21225,21226,21227,21228,21229,21230,21231,21232,21233,21234,21235,21236,21237,21238,21239,21240,21241,21242,21243,21244,21245,21246,21247,21248,21249,21250,21251,21252,21253,21254,21255,21256,21257,21258,21259,21260,21261,21262,21263,21264,21265,21266,21267,21268,21269,21270,21271,21272,21273,21274,21275,21276,21277,21278,21279,21280,21281,21282,21283,21284,21285,21286,21287,21288,21289,21290,21291,21292,21293,21294,21295,21296,21297,21298,21299,21300,21301,21302,21303,21304,21305,21306,21307,21308,21309,21310,21311,21312,21313,21314,21315,21316,21317,21318,21319,21320,21321,21322,21323,21324,21325,21326,21327,21328,21329,21330,21331,21332,21333,21334,21335,21336,21337,21338,21339,21340,21341,21342,21343,21344,21345,21346,21347,21348,21349,21350,21351,21352,21353,21354,21355,21356,21357,21358,21359,21360,21361,21362,21363,21364,21365,21366,21367,21368,21369,21370,21371,21372,21373,21374,21375,21376,21377,21378,21379,21380,21381,21382,21383,21384,21385,21386,21387,21388,21389,21390,21391,21392,21393,21394,21395,21396,21397,21398,21399,21400,21401,21402,21403,21404,21405,21406,21407,21408,21409,21410,21411,21412,21413,21414,21415,21416,21417,21418,21419,21420,21421,21422,21423,21424,21425,21426,21427,21428,21429,21430,21431,21432,21433,21434,21435,21436,21437,21438,21439,21440,21441,21442,21443,21444,21445,21446,21447,21448,21449,21450,21451,21452,21453,21454,21455,21456,21457,21458,21459,21460,21461,21462,21463,21464,21465,21466,21467,21468,21469,21470,21471,21472,21473,21474,21475,21476,21477,21478,21479,21480,21481,21482,21483,21484,21485,21486,21487,21488,21489,21490,21491,21492,21493,21494,21495,21496,21497,21498,21499,21500,21501,21502,21503,21504,21505,21506,21507,21508,21509,21510,21511,21512,21513,21514,21515,21516,21517,21518,21519,21520,21521,21522,21523,21524,21525,21526,21527,21528,21529,21530,21531,21532,21533,21534,21535,21536,21537,21538,21539,21540,21541,21542,21543,21544,21545,21546,21547,21548,21549,21550,21551,21552,21553,21554,21555,21556,21557,21558,21559,21560,21561,21562,21563,21564,21565,21566,21567,21568,21569,21570,21571,21572,21573,21574,21575,21576,21577,21578,21579,21580,21581,21582,21583,21584,21585,21586,21587,21588,21589,21590,21591,21592,21593,21594,21595,21596,21597,21598,21599,21600,21601,21602,21603,21604,21605,21606,21607,21608,21609,21610,21611,21612,21613,21614,21615,21616,21617,21618,21619,21620,21621,21622,21623,21624,21625,21626,21627,21628,21629,21630,21631,21632,21633,21634,21635,21636,21637,21638,21639,21640,21641,21642,21643,21644,21645,21646,21647,21648,21649,21650,21651,21652,21653,21654,21655,21656,21657,21658,21659,21660,21661,21662,21663,21664,21665,21666,21667,21668,21669,21670,21671,21672,21673,21674,21675,21676,21677,21678,21679,21680,21681,21682,21683,21684,21685,21686,21687,21688,21689,21690,21691,21692,21693,21694,21695,21696,21697,21698,21699,21700,21701,21702,21703,21704,21705,21706,21707,21708,21709,21710,21711,21712,21713,21714,21715,21716,21717,21718,21719,21720,21721,21722,21723,21724,21725,21726,21727,21728,21729,21730,21731,21732,21733,21734,21735,21736,21737,21738,21739,21740,21741,21742,21743,21744,21745,21746,21747,21748,21749,21750,21751,21752,21753,21754,21755,21756,21757,21758,21759,21760,21761,21762,21763,21764,21765,21766,21767,21768,21769,21770,21771,21772,21773,21774,21775,21776,21777,21778,21779,21780,21781,21782,21783,21784,21785,21786,21787,21788,21789,21790,21791,21792,21793,21794,21795,21796,21797,21798,21799,21800,21801,21802,21803,21804,21805,21806,21807,21808,21809,21810,21811,21812,21813,21814,21815,21816,21817,21818,21819,21820,21821,21822,21823,21824,21825,21826,21827,21828,21829,21830,21831,21832,21833,21834,21835,21836,21837,21838,21839,21840,21841,21842,21843,21844,21845,21846,21847,21848,21849,21850,21851,21852,21853,21854,21855,21856,21857,21858,21859,21860,21861,21862,21863,21864,21865,21866,21867,21868,21869,21870,21871,21872,21873,21874,21875,21876,21877,21878,21879,21880,21881,21882,21883,21884,21885,21886,21887,21888,21889,21890,21891,21892,21893,21894,21895,21896,21897,21898,21899,21900,21901,21902,21903,21904,21905,21906,21907,21908,21909,21910,21911,21912,21913,21914,21915,21916,21917,21918,21919,21920,21921,21922,21923,21924,21925,21926,21927,21928,21929,21930,21931,21932,21933,21934,21935,21936,21937,21938,21939,21940,21941,21942,21943,21944,21945,21946,21947,21948,21949,21950,21951,21952,21953,21954,21955,21956,21957,21958,21959,21960,21961,21962,21963,21964,21965,21966,21967,21968,21969,21970,21971,21972,21973,21974,21975,21976,21977,21978,21979,21980,21981,21982,21983,21984,21985,21986,21987,21988,21989,21990,21991,21992,21993,21994,21995,21996,21997,21998,21999,22000,22001,22002,22003,22004,22005,22006,22007,22008,22009,22010,22011,22012,22013,22014,22015,22016,22017,22018,22019,22020,22021,22022,22023,22024,22025,22026,22027,22028,22029,22030,22031,22032,22033,22034,22035,22036,22037,22038,22039,22040,22041,22042,22043,22044,22045,22046,22047,22048,22049,22050,22051,22052,22053,22054,22055,22056,22057,22058,22059,22060,22061,22062,22063,22064,22065,22066,22067,22068,22069,22070,22071,22072,22073,22074,22075,22076,22077,22078,22079,22080,22081,22082,22083,22084,22085,22086,22087,22088,22089,22090,22091,22092,22093,22094,22095,22096,22097,22098,22099,22100,22101,22102,22103,22104,22105,22106,22107,22108,22109,22110,22111,22112,22113,22114,22115,22116,22117,22118,22119,22120,22121,22122,22123,22124,22125,22126,22127,22128,22129,22130,22131,22132,22133,22134,22135,22136,22137,22138,22139,22140,22141,22142,22143,22144,22145,22146,22147,22148,22149,22150,22151,22152,22153,22154,22155,22156,22157,22158,22159,22160,22161,22162,22163,22164,22165,22166,22167,22168,22169,22170,22171,22172,22173,22174,22175,22176,22177,22178,22179,22180,22181,22182,22183,22184,22185,22186,22187,22188,22189,22190,22191,22192,22193,22194,22195,22196,22197,22198,22199,22200,22201,22202,22203,22204,22205,22206,22207,22208,22209,22210,22211,22212,22213,22214,22215,22216,22217,22218,22219,22220,22221,22222,22223,22224,22225,22226,22227,22228,22229,22230,22231,22232,22233,22234,22235,22236,22237,22238,22239,22240,22241,22242,22243,22244,22245,22246,22247,22248,22249,22250,22251,22252,22253,22254,22255,22256,22257,22258,22259,22260,22261,22262,22263,22264,22265,22266,22267,22268,22269,22270,22271,22272,22273,22274,22275,22276,22277,22278,22279,22280,22281,22282,22283,22284,22285,22286,22287,22288,22289,22290,22291,22292,22293,22294,22295,22296,22297,22298,22299,22300,22301,22302,22303,22304,22305,22306,22307,22308,22309,22310,22311,22312,22313,22314,22315,22316,22317,22318,22319,22320,22321,22322,22323,22324,22325,22326,22327,22328,22329,22330,22331,22332,22333,22334,22335,22336,22337,22338,22339,22340,22341,22342,22343,22344,22345,22346,22347,22348,22349,22350,22351,22352,22353,22354,22355,22356,22357,22358,22359,22360,22361,22362,22363,22364,22365,22366,22367,22368,22369,22370,22371,22372,22373,22374,22375,22376,22377,22378,22379,22380,22381,22382,22383,22384,22385,22386,22387,22388,22389,22390,22391,22392,22393,22394,22395,22396,22397,22398,22399,22400,22401,22402,22403,22404,22405,22406,22407,22408,22409,22410,22411,22412,22413,22414,22415,22416,22417,22418,22419,22420,22421,22422,22423,22424,22425,22426,22427,22428,22429,22430,22431,22432,22433,22434,22435,22436,22437,22438,22439,22440,22441,22442,22443,22444,22445,22446,22447,22448,22449,22450,22451,22452,22453,22454,22455,22456,22457,22458,22459,22460,22461,22462,22463,22464,22465,22466,22467,22468,22469,22470,22471,22472,22473,22474,22475,22476,22477,22478,22479,22480,22481,22482,22483,22484,22485,22486,22487,22488,22489,22490,22491,22492,22493,22494,22495,22496,22497,22498,22499,22500,22501,22502,22503,22504,22505,22506,22507,22508,22509,22510,22511,22512,22513,22514,22515,22516,22517,22518,22519,22520,22521,22522,22523,22524,22525,22526,22527,22528,22529,22530,22531,22532,22533,22534,22535,22536,22537,22538,22539,22540,22541,22542,22543,22544,22545,22546,22547,22548,22549,22550,22551,22552,22553,22554,22555,22556,22557,22558,22559,22560,22561,22562,22563,22564,22565,22566,22567,22568,22569,22570,22571,22572,22573,22574,22575,22576,22577,22578,22579,22580,22581,22582,22583,22584,22585,22586,22587,22588,22589,22590,22591,22592,22593,22594,22595,22596,22597,22598,22599,22600,22601,22602,22603,22604,22605,22606,22607,22608,22609,22610,22611,22612,22613,22614,22615,22616,22617,22618,22619,22620,22621,22622,22623,22624,22625,22626,22627,22628,22629,22630,22631,22632,22633,22634,22635,22636,22637,22638,22639,22640,22641,22642,22643,22644,22645,22646,22647,22648,22649,22650,22651,22652,22653,22654,22655,22656,22657,22658,22659,22660,22661,22662,22663,22664,22665,22666,22667,22668,22669,22670,22671,22672,22673,22674,22675,22676,22677,22678,22679,22680,22681,22682,22683,22684,22685,22686,22687,22688,22689,22690,22691,22692,22693,22694,22695,22696,22697,22698,22699,22700,22701,22702,22703,22704,22705,22706,22707,22708,22709,22710,22711,22712,22713,22714,22715,22716,22717,22718,22719,22720,22721,22722,22723,22724,22725,22726,22727,22728,22729,22730,22731,22732,22733,22734,22735,22736,22737,22738,22739,22740,22741,22742,22743,22744,22745,22746,22747,22748,22749,22750,22751,22752,22753,22754,22755,22756,22757,22758,22759,22760,22761,22762,22763,22764,22765,22766,22767,22768,22769,22770,22771,22772,22773,22774,22775,22776,22777,22778,22779,22780,22781,22782,22783,22784,22785,22786,22787,22788,22789,22790,22791,22792,22793,22794,22795,22796,22797,22798,22799,22800,22801,22802,22803,22804,22805,22806,22807,22808,22809,22810,22811,22812,22813,22814,22815,22816,22817,22818,22819,22820,22821,22822,22823,22824,22825,22826,22827,22828,22829,22830,22831,22832,22833,22834,22835,22836,22837,22838,22839,22840,22841,22842,22843,22844,22845,22846,22847,22848,22849,22850,22851,22852,22853,22854,22855,22856,22857,22858,22859,22860,22861,22862,22863,22864,22865,22866,22867,22868,22869,22870,22871,22872,22873,22874,22875,22876,22877,22878,22879,22880,22881,22882,22883,22884,22885,22886,22887,22888,22889,22890,22891,22892,22893,22894,22895,22896,22897,22898,22899,22900,22901,22902,22903,22904,22905,22906,22907,22908,22909,22910,22911,22912,22913,22914,22915,22916,22917,22918,22919,22920,22921,22922,22923,22924,22925,22926,22927,22928,22929,22930,22931,22932,22933,22934,22935,22936,22937,22938,22939,22940,22941,22942,22943,22944,22945,22946,22947,22948,22949,22950,22951,22952,22953,22954,22955,22956,22957,22958,22959,22960,22961,22962,22963,22964,22965,22966,22967,22968,22969,22970,22971,22972,22973,22974,22975,22976,22977,22978,22979,22980,22981,22982,22983,22984,22985,22986,22987,22988,22989,22990,22991,22992,22993,22994,22995,22996,22997,22998,22999,23000,23001,23002,23003,23004,23005,23006,23007,23008,23009,23010,23011,23012,23013,23014,23015,23016,23017,23018,23019,23020,23021,23022,23023,23024,23025,23026,23027,23028,23029,23030,23031,23032,23033,23034,23035,23036,23037,23038,23039,23040,23041,23042,23043,23044,23045,23046,23047,23048,23049,23050,23051,23052,23053,23054,23055,23056,23057,23058,23059,23060,23061,23062,23063,23064,23065,23066,23067,23068,23069,23070,23071,23072,23073,23074,23075,23076,23077,23078,23079,23080,23081,23082,23083,23084,23085,23086,23087,23088,23089,23090,23091,23092,23093,23094,23095,23096,23097,23098,23099,23100,23101,23102,23103,23104,23105,23106,23107,23108,23109,23110,23111,23112,23113,23114,23115,23116,23117,23118,23119,23120,23121,23122,23123,23124,23125,23126,23127,23128,23129,23130,23131,23132,23133,23134,23135,23136,23137,23138,23139,23140,23141,23142,23143,23144,23145,23146,23147,23148,23149,23150,23151,23152,23153,23154,23155,23156,23157,23158,23159,23160,23161,23162,23163,23164,23165,23166,23167,23168,23169,23170,23171,23172,23173,23174,23175,23176,23177,23178,23179,23180,23181,23182,23183,23184,23185,23186,23187,23188,23189,23190,23191,23192,23193,23194,23195,23196,23197,23198,23199,23200,23201,23202,23203,23204,23205,23206,23207,23208,23209,23210,23211,23212,23213,23214,23215,23216,23217,23218,23219,23220,23221,23222,23223,23224,23225,23226,23227,23228,23229,23230,23231,23232,23233,23234,23235,23236,23237,23238,23239,23240,23241,23242,23243,23244,23245,23246,23247,23248,23249,23250,23251,23252,23253,23254,23255,23256,23257,23258,23259,23260,23261,23262,23263,23264,23265,23266,23267,23268,23269,23270,23271,23272,23273,23274,23275,23276,23277,23278,23279,23280,23281,23282,23283,23284,23285,23286,23287,23288,23289,23290,23291,23292,23293,23294,23295,23296,23297,23298,23299,23300,23301,23302,23303,23304,23305,23306,23307,23308,23309,23310,23311,23312,23313,23314,23315,23316,23317,23318,23319,23320,23321,23322,23323,23324,23325,23326,23327,23328,23329,23330,23331,23332,23333,23334,23335,23336,23337,23338,23339,23340,23341,23342,23343,23344,23345,23346,23347,23348,23349,23350,23351,23352,23353,23354,23355,23356,23357,23358,23359,23360,23361,23362,23363,23364,23365,23366,23367,23368,23369,23370,23371,23372,23373,23374,23375,23376,23377,23378,23379,23380,23381,23382,23383,23384,23385,23386,23387,23388,23389,23390,23391,23392,23393,23394,23395,23396,23397,23398,23399,23400,23401,23402,23403,23404,23405,23406,23407,23408,23409,23410,23411,23412,23413,23414,23415,23416,23417,23418,23419,23420,23421,23422,23423,23424,23425,23426,23427,23428,23429,23430,23431,23432,23433,23434,23435,23436,23437,23438,23439,23440,23441,23442,23443,23444,23445,23446,23447,23448,23449,23450,23451,23452,23453,23454,23455,23456,23457,23458,23459,23460,23461,23462,23463,23464,23465,23466,23467,23468,23469,23470,23471,23472,23473,23474,23475,23476,23477,23478,23479,23480,23481,23482,23483,23484,23485,23486,23487,23488,23489,23490,23491,23492,23493,23494,23495,23496,23497,23498,23499,23500,23501,23502,23503,23504,23505,23506,23507,23508,23509,23510,23511,23512,23513,23514,23515,23516,23517,23518,23519,23520,23521,23522,23523,23524,23525,23526,23527,23528,23529,23530,23531,23532,23533,23534,23535,23536,23537,23538,23539,23540,23541,23542,23543,23544,23545,23546,23547,23548,23549,23550,23551,23552,23553,23554,23555,23556,23557,23558,23559,23560,23561,23562,23563,23564,23565,23566,23567,23568,23569,23570,23571,23572,23573,23574,23575,23576,23577,23578,23579,23580,23581,23582,23583,23584,23585,23586,23587,23588,23589,23590,23591,23592,23593,23594,23595,23596,23597,23598,23599,23600,23601,23602,23603,23604,23605,23606,23607,23608,23609,23610,23611,23612,23613,23614,23615,23616,23617,23618,23619,23620,23621,23622,23623,23624,23625,23626,23627,23628,23629,23630,23631,23632,23633,23634,23635,23636,23637,23638,23639,23640,23641,23642,23643,23644,23645,23646,23647,23648,23649,23650,23651,23652,23653,23654,23655,23656,23657,23658,23659,23660,23661,23662,23663,23664,23665,23666,23667,23668,23669,23670,23671,23672,23673,23674,23675,23676,23677,23678,23679,23680,23681,23682,23683,23684,23685,23686,23687,23688,23689,23690,23691,23692,23693,23694,23695,23696,23697,23698,23699,23700,23701,23702,23703,23704,23705,23706,23707,23708,23709,23710,23711,23712,23713,23714,23715,23716,23717,23718,23719,23720,23721,23722,23723,23724,23725,23726,23727,23728,23729,23730,23731,23732,23733,23734,23735,23736,23737,23738,23739,23740,23741,23742,23743,23744,23745,23746,23747,23748,23749,23750,23751,23752,23753,23754,23755,23756,23757,23758,23759,23760,23761,23762,23763,23764,23765,23766,23767,23768,23769,23770,23771,23772,23773,23774,23775,23776,23777,23778,23779,23780,23781,23782,23783,23784,23785,23786,23787,23788,23789,23790,23791,23792,23793,23794,23795,23796,23797,23798,23799,23800,23801,23802,23803,23804,23805,23806,23807,23808,23809,23810,23811,23812,23813,23814,23815,23816,23817,23818,23819,23820,23821,23822,23823,23824,23825,23826,23827,23828,23829,23830,23831,23832,23833,23834,23835,23836,23837,23838,23839,23840,23841,23842,23843,23844,23845,23846,23847,23848,23849,23850,23851,23852,23853,23854,23855,23856,23857,23858,23859,23860,23861,23862,23863,23864,23865,23866,23867,23868,23869,23870,23871,23872,23873,23874,23875,23876,23877,23878,23879,23880,23881,23882,23883,23884,23885,23886,23887,23888,23889,23890,23891,23892,23893,23894,23895,23896,23897,23898,23899,23900,23901,23902,23903,23904,23905,23906,23907,23908,23909,23910,23911,23912,23913,23914,23915,23916,23917,23918,23919,23920,23921,23922,23923,23924,23925,23926,23927,23928,23929,23930,23931,23932,23933,23934,23935,23936,23937,23938,23939,23940,23941,23942,23943,23944,23945,23946,23947,23948,23949,23950,23951,23952,23953,23954,23955,23956,23957,23958,23959,23960,23961,23962,23963,23964,23965,23966,23967,23968,23969,23970,23971,23972,23973,23974,23975,23976,23977,23978,23979,23980,23981,23982,23983,23984,23985,23986,23987,23988,23989,23990,23991,23992,23993,23994,23995,23996,23997,23998,23999,24000,24001,24002,24003,24004,24005,24006,24007,24008,24009,24010,24011,24012,24013,24014,24015,24016,24017,24018,24019,24020,24021,24022,24023,24024,24025,24026,24027,24028,24029,24030,24031,24032,24033,24034,24035,24036,24037,24038,24039,24040,24041,24042,24043,24044,24045,24046,24047,24048,24049,24050,24051,24052,24053,24054,24055,24056,24057,24058,24059,24060,24061,24062,24063,24064,24065,24066,24067,24068,24069,24070,24071,24072,24073,24074,24075,24076,24077,24078,24079,24080,24081,24082,24083,24084,24085,24086,24087,24088,24089,24090,24091,24092,24093,24094,24095,24096,24097,24098,24099,24100,24101,24102,24103,24104,24105,24106,24107,24108,24109,24110,24111,24112,24113,24114,24115,24116,24117,24118,24119,24120,24121,24122,24123,24124,24125,24126,24127,24128,24129,24130,24131,24132,24133,24134,24135,24136,24137,24138,24139,24140,24141,24142,24143,24144,24145,24146,24147,24148,24149,24150,24151,24152,24153,24154,24155,24156,24157,24158,24159,24160,24161,24162,24163,24164,24165,24166,24167,24168,24169,24170,24171,24172,24173,24174,24175,24176,24177,24178,24179,24180,24181,24182,24183,24184,24185,24186,24187,24188,24189,24190,24191,24192,24193,24194,24195,24196,24197,24198,24199,24200,24201,24202,24203,24204,24205,24206,24207,24208,24209,24210,24211,24212,24213,24214,24215,24216,24217,24218,24219,24220,24221,24222,24223,24224,24225,24226,24227,24228,24229,24230,24231,24232,24233,24234,24235,24236,24237,24238,24239,24240,24241,24242,24243,24244,24245,24246,24247,24248,24249,24250,24251,24252,24253,24254,24255,24256,24257,24258,24259,24260,24261,24262,24263,24264,24265,24266,24267,24268,24269,24270,24271,24272,24273,24274,24275,24276,24277,24278,24279,24280,24281,24282,24283,24284,24285,24286,24287,24288,24289,24290,24291,24292,24293,24294,24295,24296,24297,24298,24299,24300,24301,24302,24303,24304,24305,24306,24307,24308,24309,24310,24311,24312,24313,24314,24315,24316,24317,24318,24319,24320,24321,24322,24323,24324,24325,24326,24327,24328,24329,24330,24331,24332,24333,24334,24335,24336,24337,24338,24339,24340,24341,24342,24343,24344,24345,24346,24347,24348,24349,24350,24351,24352,24353,24354,24355,24356,24357,24358,24359,24360,24361,24362,24363,24364,24365,24366,24367,24368,24369,24370,24371,24372,24373,24374,24375,24376,24377,24378,24379,24380,24381,24382,24383,24384,24385,24386,24387,24388,24389,24390,24391,24392,24393,24394,24395,24396,24397,24398,24399,24400,24401,24402,24403,24404,24405,24406,24407,24408,24409,24410,24411,24412,24413,24414,24415,24416,24417,24418,24419,24420,24421,24422,24423,24424,24425,24426,24427,24428,24429,24430,24431,24432,24433,24434,24435,24436,24437,24438,24439,24440,24441,24442,24443,24444,24445,24446,24447,24448,24449,24450,24451,24452,24453,24454,24455,24456,24457,24458,24459,24460,24461,24462,24463,24464,24465,24466,24467,24468,24469,24470,24471,24472,24473,24474,24475,24476,24477,24478,24479,24480,24481,24482,24483,24484,24485,24486,24487,24488,24489,24490,24491,24492,24493,24494,24495,24496,24497,24498,24499,24500,24501,24502,24503,24504,24505,24506,24507,24508,24509,24510,24511,24512,24513,24514,24515,24516,24517,24518,24519,24520,24521,24522,24523,24524,24525,24526,24527,24528,24529,24530,24531,24532,24533,24534,24535,24536,24537,24538,24539,24540,24541,24542,24543,24544,24545,24546,24547,24548,24549,24550,24551,24552,24553,24554,24555,24556,24557,24558,24559,24560,24561,24562,24563,24564,24565,24566,24567,24568,24569,24570,24571,24572,24573,24574,24575,24576,24577,24578,24579,24580,24581,24582,24583,24584,24585,24586,24587,24588,24589,24590,24591,24592,24593,24594,24595,24596,24597,24598,24599,24600,24601,24602,24603,24604,24605,24606,24607,24608,24609,24610,24611,24612,24613,24614,24615,24616,24617,24618,24619,24620,24621,24622,24623,24624,24625,24626,24627,24628,24629,24630,24631,24632,24633,24634,24635,24636,24637,24638,24639,24640,24641,24642,24643,24644,24645,24646,24647,24648,24649,24650,24651,24652,24653,24654,24655,24656,24657,24658,24659,24660,24661,24662,24663,24664,24665,24666,24667,24668,24669,24670,24671,24672,24673,24674,24675,24676,24677,24678,24679,24680,24681,24682,24683,24684,24685,24686,24687,24688,24689,24690,24691,24692,24693,24694,24695,24696,24697,24698,24699,24700,24701,24702,24703,24704,24705,24706,24707,24708,24709,24710,24711,24712,24713,24714,24715,24716,24717,24718,24719,24720,24721,24722,24723,24724,24725,24726,24727,24728,24729,24730,24731,24732,24733,24734,24735,24736,24737,24738,24739,24740,24741,24742,24743,24744,24745,24746,24747,24748,24749,24750,24751,24752,24753,24754,24755,24756,24757,24758,24759,24760,24761,24762,24763,24764,24765,24766,24767,24768,24769,24770,24771,24772,24773,24774,24775,24776,24777,24778,24779,24780,24781,24782,24783,24784,24785,24786,24787,24788,24789,24790,24791,24792,24793,24794,24795,24796,24797,24798,24799,24800,24801,24802,24803,24804,24805,24806,24807,24808,24809,24810,24811,24812,24813,24814,24815,24816,24817,24818,24819,24820,24821,24822,24823,24824,24825,24826,24827,24828,24829,24830,24831,24832,24833,24834,24835,24836,24837,24838,24839,24840,24841,24842,24843,24844,24845,24846,24847,24848,24849,24850,24851,24852,24853,24854,24855,24856,24857,24858,24859,24860,24861,24862,24863,24864,24865,24866,24867,24868,24869,24870,24871,24872,24873,24874,24875,24876,24877,24878,24879,24880,24881,24882,24883,24884,24885,24886,24887,24888,24889,24890,24891,24892,24893,24894,24895,24896,24897,24898,24899,24900,24901,24902,24903,24904,24905,24906,24907,24908,24909,24910,24911,24912,24913,24914,24915,24916,24917,24918,24919,24920,24921,24922,24923,24924,24925,24926,24927,24928,24929,24930,24931,24932,24933,24934,24935,24936,24937,24938,24939,24940,24941,24942,24943,24944,24945,24946,24947,24948,24949,24950,24951,24952,24953,24954,24955,24956,24957,24958,24959,24960,24961,24962,24963,24964,24965,24966,24967,24968,24969,24970,24971,24972,24973,24974,24975,24976,24977,24978,24979,24980,24981,24982,24983,24984,24985,24986,24987,24988,24989,24990,24991,24992,24993,24994,24995,24996,24997,24998,24999,25000,25001,25002,25003,25004,25005,25006,25007,25008,25009,25010,25011,25012,25013,25014,25015,25016,25017,25018,25019,25020,25021,25022,25023,25024,25025,25026,25027,25028,25029,25030,25031,25032,25033,25034,25035,25036,25037,25038,25039,25040,25041,25042,25043,25044,25045,25046,25047,25048,25049,25050,25051,25052,25053,25054,25055,25056,25057,25058,25059,25060,25061,25062,25063,25064,25065,25066,25067,25068,25069,25070,25071,25072,25073,25074,25075,25076,25077,25078,25079,25080,25081,25082,25083,25084,25085,25086,25087,25088,25089,25090,25091,25092,25093,25094,25095,25096,25097,25098,25099,25100,25101,25102,25103,25104,25105,25106,25107,25108,25109,25110,25111,25112,25113,25114,25115,25116,25117,25118,25119,25120,25121,25122,25123,25124,25125,25126,25127,25128,25129,25130,25131,25132,25133,25134,25135,25136,25137,25138,25139,25140,25141,25142,25143,25144,25145,25146,25147,25148,25149,25150,25151,25152,25153,25154,25155,25156,25157,25158,25159,25160,25161,25162,25163,25164,25165,25166,25167,25168,25169,25170,25171,25172,25173,25174,25175,25176,25177,25178,25179,25180,25181,25182,25183,25184,25185,25186,25187,25188,25189,25190,25191,25192,25193,25194,25195,25196,25197,25198,25199,25200,25201,25202,25203,25204,25205,25206,25207,25208,25209,25210,25211,25212,25213,25214,25215,25216,25217,25218,25219,25220,25221,25222,25223,25224,25225,25226,25227,25228,25229,25230,25231,25232,25233,25234,25235,25236,25237,25238,25239,25240,25241,25242,25243,25244,25245,25246,25247,25248,25249,25250,25251,25252,25253,25254,25255,25256,25257,25258,25259,25260,25261,25262,25263,25264,25265,25266,25267,25268,25269,25270,25271,25272,25273,25274,25275,25276,25277,25278,25279,25280,25281,25282,25283,25284,25285,25286,25287,25288,25289,25290,25291,25292,25293,25294,25295,25296,25297,25298,25299,25300,25301,25302,25303,25304,25305,25306,25307,25308,25309,25310,25311,25312,25313,25314,25315,25316,25317,25318,25319,25320,25321,25322,25323,25324,25325,25326,25327,25328,25329,25330,25331,25332,25333,25334,25335,25336,25337,25338,25339,25340,25341,25342,25343,25344,25345,25346,25347,25348,25349,25350,25351,25352,25353,25354,25355,25356,25357,25358,25359,25360,25361,25362,25363,25364,25365,25366,25367,25368,25369,25370,25371,25372,25373,25374,25375,25376,25377,25378,25379,25380,25381,25382,25383,25384,25385,25386,25387,25388,25389,25390,25391,25392,25393,25394,25395,25396,25397,25398,25399,25400,25401,25402,25403,25404,25405,25406,25407,25408,25409,25410,25411,25412,25413,25414,25415,25416,25417,25418,25419,25420,25421,25422,25423,25424,25425,25426,25427,25428,25429,25430,25431,25432,25433,25434,25435,25436,25437,25438,25439,25440,25441,25442,25443,25444,25445,25446,25447,25448,25449,25450,25451,25452,25453,25454,25455,25456,25457,25458,25459,25460,25461,25462,25463,25464,25465,25466,25467,25468,25469,25470,25471,25472,25473,25474,25475,25476,25477,25478,25479,25480,25481,25482,25483,25484,25485,25486,25487,25488,25489,25490,25491,25492,25493,25494,25495,25496,25497,25498,25499,25500,25501,25502,25503,25504,25505,25506,25507,25508,25509,25510,25511,25512,25513,25514,25515,25516,25517,25518,25519,25520,25521,25522,25523,25524,25525,25526,25527,25528,25529,25530,25531,25532,25533,25534,25535,25536,25537,25538,25539,25540,25541,25542,25543,25544,25545,25546,25547,25548,25549,25550,25551,25552,25553,25554,25555,25556,25557,25558,25559,25560,25561,25562,25563,25564,25565,25566,25567,25568,25569,25570,25571,25572,25573,25574,25575,25576,25577,25578,25579,25580,25581,25582,25583,25584,25585,25586,25587,25588,25589,25590,25591,25592,25593,25594,25595,25596,25597,25598,25599,25600,25601,25602,25603,25604,25605,25606,25607,25608,25609,25610,25611,25612,25613,25614,25615,25616,25617,25618,25619,25620,25621,25622,25623,25624,25625,25626,25627,25628,25629,25630,25631,25632,25633,25634,25635,25636,25637,25638,25639,25640,25641,25642,25643,25644,25645,25646,25647,25648,25649,25650,25651,25652,25653,25654,25655,25656,25657,25658,25659,25660,25661,25662,25663,25664,25665,25666,25667,25668,25669,25670,25671,25672,25673,25674,25675,25676,25677,25678,25679,25680,25681,25682,25683,25684,25685,25686,25687,25688,25689,25690,25691,25692,25693,25694,25695,25696,25697,25698,25699,25700,25701,25702,25703,25704,25705,25706,25707,25708,25709,25710,25711,25712,25713,25714,25715,25716,25717,25718,25719,25720,25721,25722,25723,25724,25725,25726,25727,25728,25729,25730,25731,25732,25733,25734,25735,25736,25737,25738,25739,25740,25741,25742,25743,25744,25745,25746,25747,25748,25749,25750,25751,25752,25753,25754,25755,25756,25757,25758,25759,25760,25761,25762,25763,25764,25765,25766,25767,25768,25769,25770,25771,25772,25773,25774,25775,25776,25777,25778,25779,25780,25781,25782,25783,25784,25785,25786,25787,25788,25789,25790,25791,25792,25793,25794,25795,25796,25797,25798,25799,25800,25801,25802,25803,25804,25805,25806,25807,25808,25809,25810,25811,25812,25813,25814,25815,25816,25817,25818,25819,25820,25821,25822,25823,25824,25825,25826,25827,25828,25829,25830,25831,25832,25833,25834,25835,25836,25837,25838,25839,25840,25841,25842,25843,25844,25845,25846,25847,25848,25849,25850,25851,25852,25853,25854,25855,25856,25857,25858,25859,25860,25861,25862,25863,25864,25865,25866,25867,25868,25869,25870,25871,25872,25873,25874,25875,25876,25877,25878,25879,25880,25881,25882,25883,25884,25885,25886,25887,25888,25889,25890,25891,25892,25893,25894,25895,25896,25897,25898,25899,25900,25901,25902,25903,25904,25905,25906,25907,25908,25909,25910,25911,25912,25913,25914,25915,25916,25917,25918,25919,25920,25921,25922,25923,25924,25925,25926,25927,25928,25929,25930,25931,25932,25933,25934,25935,25936,25937,25938,25939,25940,25941,25942,25943,25944,25945,25946,25947,25948,25949,25950,25951,25952,25953,25954,25955,25956,25957,25958,25959,25960,25961,25962,25963,25964,25965,25966,25967,25968,25969,25970,25971,25972,25973,25974,25975,25976,25977,25978,25979,25980,25981,25982,25983,25984,25985,25986,25987,25988,25989,25990,25991,25992,25993,25994,25995,25996,25997,25998,25999,26000,26001,26002,26003,26004,26005,26006,26007,26008,26009,26010,26011,26012,26013,26014,26015,26016,26017,26018,26019,26020,26021,26022,26023,26024,26025,26026,26027,26028,26029,26030,26031,26032,26033,26034,26035,26036,26037,26038,26039,26040,26041,26042,26043,26044,26045,26046,26047,26048,26049,26050,26051,26052,26053,26054,26055,26056,26057,26058,26059,26060,26061,26062,26063,26064,26065,26066,26067,26068,26069,26070,26071,26072,26073,26074,26075,26076,26077,26078,26079,26080,26081,26082,26083,26084,26085,26086,26087,26088,26089,26090,26091,26092,26093,26094,26095,26096,26097,26098,26099,26100,26101,26102,26103,26104,26105,26106,26107,26108,26109,26110,26111,26112,26113,26114,26115,26116,26117,26118,26119,26120,26121,26122,26123,26124,26125,26126,26127,26128,26129,26130,26131,26132,26133,26134,26135,26136,26137,26138,26139,26140,26141,26142,26143,26144,26145,26146,26147,26148,26149,26150,26151,26152,26153,26154,26155,26156,26157,26158,26159,26160,26161,26162,26163,26164,26165,26166,26167,26168,26169,26170,26171,26172,26173,26174,26175,26176,26177,26178,26179,26180,26181,26182,26183,26184,26185,26186,26187,26188,26189,26190,26191,26192,26193,26194,26195,26196,26197,26198,26199,26200,26201,26202,26203,26204,26205,26206,26207,26208,26209,26210,26211,26212,26213,26214,26215,26216,26217,26218,26219,26220,26221,26222,26223,26224,26225,26226,26227,26228,26229,26230,26231,26232,26233,26234,26235,26236,26237,26238,26239,26240,26241,26242,26243,26244,26245,26246,26247,26248,26249,26250,26251,26252,26253,26254,26255,26256,26257,26258,26259,26260,26261,26262,26263,26264,26265,26266,26267,26268,26269,26270,26271,26272,26273,26274,26275,26276,26277,26278,26279,26280,26281,26282,26283,26284,26285,26286,26287,26288,26289,26290,26291,26292,26293,26294,26295,26296,26297,26298,26299,26300,26301,26302,26303,26304,26305,26306,26307,26308,26309,26310,26311,26312,26313,26314,26315,26316,26317,26318,26319,26320,26321,26322,26323,26324,26325,26326,26327,26328,26329,26330,26331,26332,26333,26334,26335,26336,26337,26338,26339,26340,26341,26342,26343,26344,26345,26346,26347,26348,26349,26350,26351,26352,26353,26354,26355,26356,26357,26358,26359,26360,26361,26362,26363,26364,26365,26366,26367,26368,26369,26370,26371,26372,26373,26374,26375,26376,26377,26378,26379,26380,26381,26382,26383,26384,26385,26386,26387,26388,26389,26390,26391,26392,26393,26394,26395,26396,26397,26398,26399,26400,26401,26402,26403,26404,26405,26406,26407,26408,26409,26410,26411,26412,26413,26414,26415,26416,26417,26418,26419,26420,26421,26422,26423,26424,26425,26426,26427,26428,26429,26430,26431,26432,26433,26434,26435,26436,26437,26438,26439,26440,26441,26442,26443,26444,26445,26446,26447,26448,26449,26450,26451,26452,26453,26454,26455,26456,26457,26458,26459,26460,26461,26462,26463,26464,26465,26466,26467,26468,26469,26470,26471,26472,26473,26474,26475,26476,26477,26478,26479,26480,26481,26482,26483,26484,26485,26486,26487,26488,26489,26490,26491,26492,26493,26494,26495,26496,26497,26498,26499,26500,26501,26502,26503,26504,26505,26506,26507,26508,26509,26510,26511,26512,26513,26514,26515,26516,26517,26518,26519,26520,26521,26522,26523,26524,26525,26526,26527,26528,26529,26530,26531,26532,26533,26534,26535,26536,26537,26538,26539,26540,26541,26542,26543,26544,26545,26546,26547,26548,26549,26550,26551,26552,26553,26554,26555,26556,26557,26558,26559,26560,26561,26562,26563,26564,26565,26566,26567,26568,26569,26570,26571,26572,26573,26574,26575,26576,26577,26578,26579,26580,26581,26582,26583,26584,26585,26586,26587,26588,26589,26590,26591,26592,26593,26594,26595,26596,26597,26598,26599,26600,26601,26602,26603,26604,26605,26606,26607,26608,26609,26610,26611,26612,26613,26614,26615,26616,26617,26618,26619,26620,26621,26622,26623,26624,26625,26626,26627,26628,26629,26630,26631,26632,26633,26634,26635,26636,26637,26638,26639,26640,26641,26642,26643,26644,26645,26646,26647,26648,26649,26650,26651,26652,26653,26654,26655,26656,26657,26658,26659,26660,26661,26662,26663,26664,26665,26666,26667,26668,26669,26670,26671,26672,26673,26674,26675,26676,26677,26678,26679,26680,26681,26682,26683,26684,26685,26686,26687,26688,26689,26690,26691,26692,26693,26694,26695,26696,26697,26698,26699,26700,26701,26702,26703,26704,26705,26706,26707,26708,26709,26710,26711,26712,26713,26714,26715,26716,26717,26718,26719,26720,26721,26722,26723,26724,26725,26726,26727,26728,26729,26730,26731,26732,26733,26734,26735,26736,26737,26738,26739,26740,26741,26742,26743,26744,26745,26746,26747,26748,26749,26750,26751,26752,26753,26754,26755,26756,26757,26758,26759,26760,26761,26762,26763,26764,26765,26766,26767,26768,26769,26770,26771,26772,26773,26774,26775,26776,26777,26778,26779,26780,26781,26782,26783,26784,26785,26786,26787,26788,26789,26790,26791,26792,26793,26794,26795,26796,26797,26798,26799,26800,26801,26802,26803,26804,26805,26806,26807,26808,26809,26810,26811,26812,26813,26814,26815,26816,26817,26818,26819,26820,26821,26822,26823,26824,26825,26826,26827,26828,26829,26830,26831,26832,26833,26834,26835,26836,26837,26838,26839,26840,26841,26842,26843,26844,26845,26846,26847,26848,26849,26850,26851,26852,26853,26854,26855,26856,26857,26858,26859,26860,26861,26862,26863,26864,26865,26866,26867,26868,26869,26870,26871,26872,26873,26874,26875,26876,26877,26878,26879,26880,26881,26882,26883,26884,26885,26886,26887,26888,26889,26890,26891,26892,26893,26894,26895,26896,26897,26898,26899,26900,26901,26902,26903,26904,26905,26906,26907,26908,26909,26910,26911,26912,26913,26914,26915,26916,26917,26918,26919,26920,26921,26922,26923,26924,26925,26926,26927,26928,26929,26930,26931,26932,26933,26934,26935,26936,26937,26938,26939,26940,26941,26942,26943,26944,26945,26946,26947,26948,26949,26950,26951,26952,26953,26954,26955,26956,26957,26958,26959,26960,26961,26962,26963,26964,26965,26966,26967,26968,26969,26970,26971,26972,26973,26974,26975,26976,26977,26978,26979,26980,26981,26982,26983,26984,26985,26986,26987,26988,26989,26990,26991,26992,26993,26994,26995,26996,26997,26998,26999,27000,27001,27002,27003,27004,27005,27006,27007,27008,27009,27010,27011,27012,27013,27014,27015,27016,27017,27018,27019,27020,27021,27022,27023,27024,27025,27026,27027,27028,27029,27030,27031,27032,27033,27034,27035,27036,27037,27038,27039,27040,27041,27042,27043,27044,27045,27046,27047,27048,27049,27050,27051,27052,27053,27054,27055,27056,27057,27058,27059,27060,27061,27062,27063,27064,27065,27066,27067,27068,27069,27070,27071,27072,27073,27074,27075,27076,27077,27078,27079,27080,27081,27082,27083,27084,27085,27086,27087,27088,27089,27090,27091,27092,27093,27094,27095,27096,27097,27098,27099,27100,27101,27102,27103,27104,27105,27106,27107,27108,27109,27110,27111,27112,27113,27114,27115,27116,27117,27118,27119,27120,27121,27122,27123,27124,27125,27126,27127,27128,27129,27130,27131,27132,27133,27134,27135,27136,27137,27138,27139,27140,27141,27142,27143,27144,27145,27146,27147,27148,27149,27150,27151,27152,27153,27154,27155,27156,27157,27158,27159,27160,27161,27162,27163,27164,27165,27166,27167,27168,27169,27170,27171,27172,27173,27174,27175,27176,27177,27178,27179,27180,27181,27182,27183,27184,27185,27186,27187,27188,27189,27190,27191,27192,27193,27194,27195,27196,27197,27198,27199,27200,27201,27202,27203,27204,27205,27206,27207,27208,27209,27210,27211,27212,27213,27214,27215,27216,27217,27218,27219,27220,27221,27222,27223,27224,27225,27226,27227,27228,27229,27230,27231,27232,27233,27234,27235,27236,27237,27238,27239,27240,27241,27242,27243,27244,27245,27246,27247,27248,27249,27250,27251,27252,27253,27254,27255,27256,27257,27258,27259,27260,27261,27262,27263,27264,27265,27266,27267,27268,27269,27270,27271,27272,27273,27274,27275,27276,27277,27278,27279,27280,27281,27282,27283,27284,27285,27286,27287,27288,27289,27290,27291,27292,27293,27294,27295,27296,27297,27298,27299,27300,27301,27302,27303,27304,27305,27306,27307,27308,27309,27310,27311,27312,27313,27314,27315,27316,27317,27318,27319,27320,27321,27322,27323,27324,27325,27326,27327,27328,27329,27330,27331,27332,27333,27334,27335,27336,27337,27338,27339,27340,27341,27342,27343,27344,27345,27346,27347,27348,27349,27350,27351,27352,27353,27354,27355,27356,27357,27358,27359,27360,27361,27362,27363,27364,27365,27366,27367,27368,27369,27370,27371,27372,27373,27374,27375,27376,27377,27378,27379,27380,27381,27382,27383,27384,27385,27386,27387,27388,27389,27390,27391,27392,27393,27394,27395,27396,27397,27398,27399,27400,27401,27402,27403,27404,27405,27406,27407,27408,27409,27410,27411,27412,27413,27414,27415,27416,27417,27418,27419,27420,27421,27422,27423,27424,27425,27426,27427,27428,27429,27430,27431,27432,27433,27434,27435,27436,27437,27438,27439,27440,27441,27442,27443,27444,27445,27446,27447,27448,27449,27450,27451,27452,27453,27454,27455,27456,27457,27458,27459,27460,27461,27462,27463,27464,27465,27466,27467,27468,27469,27470,27471,27472,27473,27474,27475,27476,27477,27478,27479,27480,27481,27482,27483,27484,27485,27486,27487,27488,27489,27490,27491,27492,27493,27494,27495,27496,27497,27498,27499,27500,27501,27502,27503,27504,27505,27506,27507,27508,27509,27510,27511,27512,27513,27514,27515,27516,27517,27518,27519,27520,27521,27522,27523,27524,27525,27526,27527,27528,27529,27530,27531,27532,27533,27534,27535,27536,27537,27538,27539,27540,27541,27542,27543,27544,27545,27546,27547,27548,27549,27550,27551,27552,27553,27554,27555,27556,27557,27558,27559,27560,27561,27562,27563,27564,27565,27566,27567,27568,27569,27570,27571,27572,27573,27574,27575,27576,27577,27578,27579,27580,27581,27582,27583,27584,27585,27586,27587,27588,27589,27590,27591,27592,27593,27594,27595,27596,27597,27598,27599,27600,27601,27602,27603,27604,27605,27606,27607,27608,27609,27610,27611,27612,27613,27614,27615,27616,27617,27618,27619,27620,27621,27622,27623,27624,27625,27626,27627,27628,27629,27630,27631,27632,27633,27634,27635,27636,27637,27638,27639,27640,27641,27642,27643,27644,27645,27646,27647,27648,27649,27650,27651,27652,27653,27654,27655,27656,27657,27658,27659,27660,27661,27662,27663,27664,27665,27666,27667,27668,27669,27670,27671,27672,27673,27674,27675,27676,27677,27678,27679,27680,27681,27682,27683,27684,27685,27686,27687,27688,27689,27690,27691,27692,27693,27694,27695,27696,27697,27698,27699,27700,27701,27702,27703,27704,27705,27706,27707,27708,27709,27710,27711,27712,27713,27714,27715,27716,27717,27718,27719,27720,27721,27722,27723,27724,27725,27726,27727,27728,27729,27730,27731,27732,27733,27734,27735,27736,27737,27738,27739,27740,27741,27742,27743,27744,27745,27746,27747,27748,27749,27750,27751,27752,27753,27754,27755,27756,27757,27758,27759,27760,27761,27762,27763,27764,27765,27766,27767,27768,27769,27770,27771,27772,27773,27774,27775,27776,27777,27778,27779,27780,27781,27782,27783,27784,27785,27786,27787,27788,27789,27790,27791,27792,27793,27794,27795,27796,27797,27798,27799,27800,27801,27802,27803,27804,27805,27806,27807,27808,27809,27810,27811,27812,27813,27814,27815,27816,27817,27818,27819,27820,27821,27822,27823,27824,27825,27826,27827,27828,27829,27830,27831,27832,27833,27834,27835,27836,27837,27838,27839,27840,27841,27842,27843,27844,27845,27846,27847,27848,27849,27850,27851,27852,27853,27854,27855,27856,27857,27858,27859,27860,27861,27862,27863,27864,27865,27866,27867,27868,27869,27870,27871,27872,27873,27874,27875,27876,27877,27878,27879,27880,27881,27882,27883,27884,27885,27886,27887,27888,27889,27890,27891,27892,27893,27894,27895,27896,27897,27898,27899,27900,27901,27902,27903,27904,27905,27906,27907,27908,27909,27910,27911,27912,27913,27914,27915,27916,27917,27918,27919,27920,27921,27922,27923,27924,27925,27926,27927,27928,27929,27930,27931,27932,27933,27934,27935,27936,27937,27938,27939,27940,27941,27942,27943,27944,27945,27946,27947,27948,27949,27950,27951,27952,27953,27954,27955,27956,27957,27958,27959,27960,27961,27962,27963,27964,27965,27966,27967,27968,27969,27970,27971,27972,27973,27974,27975,27976,27977,27978,27979,27980,27981,27982,27983,27984,27985,27986,27987,27988,27989,27990,27991,27992,27993,27994,27995,27996,27997,27998,27999,28000,28001,28002,28003,28004,28005,28006,28007,28008,28009,28010,28011,28012,28013,28014,28015,28016,28017,28018,28019,28020,28021,28022,28023,28024,28025,28026,28027,28028,28029,28030,28031,28032,28033,28034,28035,28036,28037,28038,28039,28040,28041,28042,28043,28044,28045,28046,28047,28048,28049,28050,28051,28052,28053,28054,28055,28056,28057,28058,28059,28060,28061,28062,28063,28064,28065,28066,28067,28068,28069,28070,28071,28072,28073,28074,28075,28076,28077,28078,28079,28080,28081,28082,28083,28084,28085,28086,28087,28088,28089,28090,28091,28092,28093,28094,28095,28096,28097,28098,28099,28100,28101,28102,28103,28104,28105,28106,28107,28108,28109,28110,28111,28112,28113,28114,28115,28116,28117,28118,28119,28120,28121,28122,28123,28124,28125,28126,28127,28128,28129,28130,28131,28132,28133,28134,28135,28136,28137,28138,28139,28140,28141,28142,28143,28144,28145,28146,28147,28148,28149,28150,28151,28152,28153,28154,28155,28156,28157,28158,28159,28160,28161,28162,28163,28164,28165,28166,28167,28168,28169,28170,28171,28172,28173,28174,28175,28176,28177,28178,28179,28180,28181,28182,28183,28184,28185,28186,28187,28188,28189,28190,28191,28192,28193,28194,28195,28196,28197,28198,28199,28200,28201,28202,28203,28204,28205,28206,28207,28208,28209,28210,28211,28212,28213,28214,28215,28216,28217,28218,28219,28220,28221,28222,28223,28224,28225,28226,28227,28228,28229,28230,28231,28232,28233,28234,28235,28236,28237,28238,28239,28240,28241,28242,28243,28244,28245,28246,28247,28248,28249,28250,28251,28252,28253,28254,28255,28256,28257,28258,28259,28260,28261,28262,28263,28264,28265,28266,28267,28268,28269,28270,28271,28272,28273,28274,28275,28276,28277,28278,28279,28280,28281,28282,28283,28284,28285,28286,28287,28288,28289,28290,28291,28292,28293,28294,28295,28296,28297,28298,28299,28300,28301,28302,28303,28304,28305,28306,28307,28308,28309,28310,28311,28312,28313,28314,28315,28316,28317,28318,28319,28320,28321,28322,28323,28324,28325,28326,28327,28328,28329,28330,28331,28332,28333,28334,28335,28336,28337,28338,28339,28340,28341,28342,28343,28344,28345,28346,28347,28348,28349,28350,28351,28352,28353,28354,28355,28356,28357,28358,28359,28360,28361,28362,28363,28364,28365,28366,28367,28368,28369,28370,28371,28372,28373,28374,28375,28376,28377,28378,28379,28380,28381,28382,28383,28384,28385,28386,28387,28388,28389,28390,28391,28392,28393,28394,28395,28396,28397,28398,28399,28400,28401,28402,28403,28404,28405,28406,28407,28408,28409,28410,28411,28412,28413,28414,28415,28416,28417,28418,28419,28420,28421,28422,28423,28424,28425,28426,28427,28428,28429,28430,28431,28432,28433,28434,28435,28436,28437,28438,28439,28440,28441,28442,28443,28444,28445,28446,28447,28448,28449,28450,28451,28452,28453,28454,28455,28456,28457,28458,28459,28460,28461,28462,28463,28464,28465,28466,28467,28468,28469,28470,28471,28472,28473,28474,28475,28476,28477,28478,28479,28480,28481,28482,28483,28484,28485,28486,28487,28488,28489,28490,28491,28492,28493,28494,28495,28496,28497,28498,28499,28500,28501,28502,28503,28504,28505,28506,28507,28508,28509,28510,28511,28512,28513,28514,28515,28516,28517,28518,28519,28520,28521,28522,28523,28524,28525,28526,28527,28528,28529,28530,28531,28532,28533,28534,28535,28536,28537,28538,28539,28540,28541,28542,28543,28544,28545,28546,28547,28548,28549,28550,28551,28552,28553,28554,28555,28556,28557,28558,28559,28560,28561,28562,28563,28564,28565,28566,28567,28568,28569,28570,28571,28572,28573,28574,28575,28576,28577,28578,28579,28580,28581,28582,28583,28584,28585,28586,28587,28588,28589,28590,28591,28592,28593,28594,28595,28596,28597,28598,28599,28600,28601,28602,28603,28604,28605,28606,28607,28608,28609,28610,28611,28612,28613,28614,28615,28616,28617,28618,28619,28620,28621,28622,28623,28624,28625,28626,28627,28628,28629,28630,28631,28632,28633,28634,28635,28636,28637,28638,28639,28640,28641,28642,28643,28644,28645,28646,28647,28648,28649,28650,28651,28652,28653,28654,28655,28656,28657,28658,28659,28660,28661,28662,28663,28664,28665,28666,28667,28668,28669,28670,28671,28672,28673,28674,28675,28676,28677,28678,28679,28680,28681,28682,28683,28684,28685,28686,28687,28688,28689,28690,28691,28692,28693,28694,28695,28696,28697,28698,28699,28700,28701,28702,28703,28704,28705,28706,28707,28708,28709,28710,28711,28712,28713,28714,28715,28716,28717,28718,28719,28720,28721,28722,28723,28724,28725,28726,28727,28728,28729,28730,28731,28732,28733,28734,28735,28736,28737,28738,28739,28740,28741,28742,28743,28744,28745,28746,28747,28748,28749,28750,28751,28752,28753,28754,28755,28756,28757,28758,28759,28760,28761,28762,28763,28764,28765,28766,28767,28768,28769,28770,28771,28772,28773,28774,28775,28776,28777,28778,28779,28780,28781,28782,28783,28784,28785,28786,28787,28788,28789,28790,28791,28792,28793,28794,28795,28796,28797,28798,28799,28800,28801,28802,28803,28804,28805,28806,28807,28808,28809,28810,28811,28812,28813,28814,28815,28816,28817,28818,28819,28820,28821,28822,28823,28824,28825,28826,28827,28828,28829,28830,28831,28832,28833,28834,28835,28836,28837,28838,28839,28840,28841,28842,28843,28844,28845,28846,28847,28848,28849,28850,28851,28852,28853,28854,28855,28856,28857,28858,28859,28860,28861,28862,28863,28864,28865,28866,28867,28868,28869,28870,28871,28872,28873,28874,28875,28876,28877,28878,28879,28880,28881,28882,28883,28884,28885,28886,28887,28888,28889,28890,28891,28892,28893,28894,28895,28896,28897,28898,28899,28900,28901,28902,28903,28904,28905,28906,28907,28908,28909,28910,28911,28912,28913,28914,28915,28916,28917,28918,28919,28920,28921,28922,28923,28924,28925,28926,28927,28928,28929,28930,28931,28932,28933,28934,28935,28936,28937,28938,28939,28940,28941,28942,28943,28944,28945,28946,28947,28948,28949,28950,28951,28952,28953,28954,28955,28956,28957,28958,28959,28960,28961,28962,28963,28964,28965,28966,28967,28968,28969,28970,28971,28972,28973,28974,28975,28976,28977,28978,28979,28980,28981,28982,28983,28984,28985,28986,28987,28988,28989,28990,28991,28992,28993,28994,28995,28996,28997,28998,28999,29000,29001,29002,29003,29004,29005,29006,29007,29008,29009,29010,29011,29012,29013,29014,29015,29016,29017,29018,29019,29020,29021,29022,29023,29024,29025,29026,29027,29028,29029,29030,29031,29032,29033,29034,29035,29036,29037,29038,29039,29040,29041,29042,29043,29044,29045,29046,29047,29048,29049,29050,29051,29052,29053,29054,29055,29056,29057,29058,29059,29060,29061,29062,29063,29064,29065,29066,29067,29068,29069,29070,29071,29072,29073,29074,29075,29076,29077,29078,29079,29080,29081,29082,29083,29084,29085,29086,29087,29088,29089,29090,29091,29092,29093,29094,29095,29096,29097,29098,29099,29100,29101,29102,29103,29104,29105,29106,29107,29108,29109,29110,29111,29112,29113,29114,29115,29116,29117,29118,29119,29120,29121,29122,29123,29124,29125,29126,29127,29128,29129,29130,29131,29132,29133,29134,29135,29136,29137,29138,29139,29140,29141,29142,29143,29144,29145,29146,29147,29148,29149,29150,29151,29152,29153,29154,29155,29156,29157,29158,29159,29160,29161,29162,29163,29164,29165,29166,29167,29168,29169,29170,29171,29172,29173,29174,29175,29176,29177,29178,29179,29180,29181,29182,29183,29184,29185,29186,29187,29188,29189,29190,29191,29192,29193,29194,29195,29196,29197,29198,29199,29200,29201,29202,29203,29204,29205,29206,29207,29208,29209,29210,29211,29212,29213,29214,29215,29216,29217,29218,29219,29220,29221,29222,29223,29224,29225,29226,29227,29228,29229,29230,29231,29232,29233,29234,29235,29236,29237,29238,29239,29240,29241,29242,29243,29244,29245,29246,29247,29248,29249,29250,29251,29252,29253,29254,29255,29256,29257,29258,29259,29260,29261,29262,29263,29264,29265,29266,29267,29268,29269,29270,29271,29272,29273,29274,29275,29276,29277,29278,29279,29280,29281,29282,29283,29284,29285,29286,29287,29288,29289,29290,29291,29292,29293,29294,29295,29296,29297,29298,29299,29300,29301,29302,29303,29304,29305,29306,29307,29308,29309,29310,29311,29312,29313,29314,29315,29316,29317,29318,29319,29320,29321,29322,29323,29324,29325,29326,29327,29328,29329,29330,29331,29332,29333,29334,29335,29336,29337,29338,29339,29340,29341,29342,29343,29344,29345,29346,29347,29348,29349,29350,29351,29352,29353,29354,29355,29356,29357,29358,29359,29360,29361,29362,29363,29364,29365,29366,29367,29368,29369,29370,29371,29372,29373,29374,29375,29376,29377,29378,29379,29380,29381,29382,29383,29384,29385,29386,29387,29388,29389,29390,29391,29392,29393,29394,29395,29396,29397,29398,29399,29400,29401,29402,29403,29404,29405,29406,29407,29408,29409,29410,29411,29412,29413,29414,29415,29416,29417,29418,29419,29420,29421,29422,29423,29424,29425,29426,29427,29428,29429,29430,29431,29432,29433,29434,29435,29436,29437,29438,29439,29440,29441,29442,29443,29444,29445,29446,29447,29448,29449,29450,29451,29452,29453,29454,29455,29456,29457,29458,29459,29460,29461,29462,29463,29464,29465,29466,29467,29468,29469,29470,29471,29472,29473,29474,29475,29476,29477,29478,29479,29480,29481,29482,29483,29484,29485,29486,29487,29488,29489,29490,29491,29492,29493,29494,29495,29496,29497,29498,29499,29500,29501,29502,29503,29504,29505,29506,29507,29508,29509,29510,29511,29512,29513,29514,29515,29516,29517,29518,29519,29520,29521,29522,29523,29524,29525,29526,29527,29528,29529,29530,29531,29532,29533,29534,29535,29536,29537,29538,29539,29540,29541,29542,29543,29544,29545,29546,29547,29548,29549,29550,29551,29552,29553,29554,29555,29556,29557,29558,29559,29560,29561,29562,29563,29564,29565,29566,29567,29568,29569,29570,29571,29572,29573,29574,29575,29576,29577,29578,29579,29580,29581,29582,29583,29584,29585,29586,29587,29588,29589,29590,29591,29592,29593,29594,29595,29596,29597,29598,29599,29600,29601,29602,29603,29604,29605,29606,29607,29608,29609,29610,29611,29612,29613,29614,29615,29616,29617,29618,29619,29620,29621,29622,29623,29624,29625,29626,29627,29628,29629,29630,29631,29632,29633,29634,29635,29636,29637,29638,29639,29640,29641,29642,29643,29644,29645,29646,29647,29648,29649,29650,29651,29652,29653,29654,29655,29656,29657,29658,29659,29660,29661,29662,29663,29664,29665,29666,29667,29668,29669,29670,29671,29672,29673,29674,29675,29676,29677,29678,29679,29680,29681,29682,29683,29684,29685,29686,29687,29688,29689,29690,29691,29692,29693,29694,29695,29696,29697,29698,29699,29700,29701,29702,29703,29704,29705,29706,29707,29708,29709,29710,29711,29712,29713,29714,29715,29716,29717,29718,29719,29720,29721,29722,29723,29724,29725,29726,29727,29728,29729,29730,29731,29732,29733,29734,29735,29736,29737,29738,29739,29740,29741,29742,29743,29744,29745,29746,29747,29748,29749,29750,29751,29752,29753,29754,29755,29756,29757,29758,29759,29760,29761,29762,29763,29764,29765,29766,29767,29768,29769,29770,29771,29772,29773,29774,29775,29776,29777,29778,29779,29780,29781,29782,29783,29784,29785,29786,29787,29788,29789,29790,29791,29792,29793,29794,29795,29796,29797,29798,29799,29800,29801,29802,29803,29804,29805,29806,29807,29808,29809,29810,29811,29812,29813,29814,29815,29816,29817,29818,29819,29820,29821,29822,29823,29824,29825,29826,29827,29828,29829,29830,29831,29832,29833,29834,29835,29836,29837,29838,29839,29840,29841,29842,29843,29844,29845,29846,29847,29848,29849,29850,29851,29852,29853,29854,29855,29856,29857,29858,29859,29860,29861,29862,29863,29864,29865,29866,29867,29868,29869,29870,29871,29872,29873,29874,29875,29876,29877,29878,29879,29880,29881,29882,29883,29884,29885,29886,29887,29888,29889,29890,29891,29892,29893,29894,29895,29896,29897,29898,29899,29900,29901,29902,29903,29904,29905,29906,29907,29908,29909,29910,29911,29912,29913,29914,29915,29916,29917,29918,29919,29920,29921,29922,29923,29924,29925,29926,29927,29928,29929,29930,29931,29932,29933,29934,29935,29936,29937,29938,29939,29940,29941,29942,29943,29944,29945,29946,29947,29948,29949,29950,29951,29952,29953,29954,29955,29956,29957,29958,29959,29960,29961,29962,29963,29964,29965,29966,29967,29968,29969,29970,29971,29972,29973,29974,29975,29976,29977,29978,29979,29980,29981,29982,29983,29984,29985,29986,29987,29988,29989,29990,29991,29992,29993,29994,29995,29996,29997,29998,29999,30000,30001,30002,30003,30004,30005,30006,30007,30008,30009,30010,30011,30012,30013,30014,30015,30016,30017,30018,30019,30020,30021,30022,30023,30024,30025,30026,30027,30028,30029,30030,30031,30032,30033,30034,30035,30036,30037,30038,30039,30040,30041,30042,30043,30044,30045,30046,30047,30048,30049,30050,30051,30052,30053,30054,30055,30056,30057,30058,30059,30060,30061,30062,30063,30064,30065,30066,30067,30068,30069,30070,30071,30072,30073,30074,30075,30076,30077,30078,30079,30080,30081,30082,30083,30084,30085,30086,30087,30088,30089,30090,30091,30092,30093,30094,30095,30096,30097,30098,30099,30100,30101,30102,30103,30104,30105,30106,30107,30108,30109,30110,30111,30112,30113,30114,30115,30116,30117,30118,30119,30120,30121,30122,30123,30124,30125,30126,30127,30128,30129,30130,30131,30132,30133,30134,30135,30136,30137,30138,30139,30140,30141,30142,30143,30144,30145,30146,30147,30148,30149,30150,30151,30152,30153,30154,30155,30156,30157,30158,30159,30160,30161,30162,30163,30164,30165,30166,30167,30168,30169,30170,30171,30172,30173,30174,30175,30176,30177,30178,30179,30180,30181,30182,30183,30184,30185,30186,30187,30188,30189,30190,30191,30192,30193,30194,30195,30196,30197,30198,30199,30200,30201,30202,30203,30204,30205,30206,30207,30208,30209,30210,30211,30212,30213,30214,30215,30216,30217,30218,30219,30220,30221,30222,30223,30224,30225,30226,30227,30228,30229,30230,30231,30232,30233,30234,30235,30236,30237,30238,30239,30240,30241,30242,30243,30244,30245,30246,30247,30248,30249,30250,30251,30252,30253,30254,30255,30256,30257,30258,30259,30260,30261,30262,30263,30264,30265,30266,30267,30268,30269,30270,30271,30272,30273,30274,30275,30276,30277,30278,30279,30280,30281,30282,30283,30284,30285,30286,30287,30288,30289,30290,30291,30292,30293,30294,30295,30296,30297,30298,30299,30300,30301,30302,30303,30304,30305,30306,30307,30308,30309,30310,30311,30312,30313,30314,30315,30316,30317,30318,30319,30320,30321,30322,30323,30324,30325,30326,30327,30328,30329,30330,30331,30332,30333,30334,30335,30336,30337,30338,30339,30340,30341,30342,30343,30344,30345,30346,30347,30348,30349,30350,30351,30352,30353,30354,30355,30356,30357,30358,30359,30360,30361,30362,30363,30364,30365,30366,30367,30368,30369,30370,30371,30372,30373,30374,30375,30376,30377,30378,30379,30380,30381,30382,30383,30384,30385,30386,30387,30388,30389,30390,30391,30392,30393,30394,30395,30396,30397,30398,30399,30400,30401,30402,30403,30404,30405,30406,30407,30408,30409,30410,30411,30412,30413,30414,30415,30416,30417,30418,30419,30420,30421,30422,30423,30424,30425,30426,30427,30428,30429,30430,30431,30432,30433,30434,30435,30436,30437,30438,30439,30440,30441,30442,30443,30444,30445,30446,30447,30448,30449,30450,30451,30452,30453,30454,30455,30456,30457,30458,30459,30460,30461,30462,30463,30464,30465,30466,30467,30468,30469,30470,30471,30472,30473,30474,30475,30476,30477,30478,30479,30480,30481,30482,30483,30484,30485,30486,30487,30488,30489,30490,30491,30492,30493,30494,30495,30496,30497,30498,30499,30500,30501,30502,30503,30504,30505,30506,30507,30508,30509,30510,30511,30512,30513,30514,30515,30516,30517,30518,30519,30520,30521,30522,30523,30524,30525,30526,30527,30528,30529,30530,30531,30532,30533,30534,30535,30536,30537,30538,30539,30540,30541,30542,30543,30544,30545,30546,30547,30548,30549,30550,30551,30552,30553,30554,30555,30556,30557,30558,30559,30560,30561,30562,30563,30564,30565,30566,30567,30568,30569,30570,30571,30572,30573,30574,30575,30576,30577,30578,30579,30580,30581,30582,30583,30584,30585,30586,30587,30588,30589,30590,30591,30592,30593,30594,30595,30596,30597,30598,30599,30600,30601,30602,30603,30604,30605,30606,30607,30608,30609,30610,30611,30612,30613,30614,30615,30616,30617,30618,30619,30620,30621,30622,30623,30624,30625,30626,30627,30628,30629,30630,30631,30632,30633,30634,30635,30636,30637,30638,30639,30640,30641,30642,30643,30644,30645,30646,30647,30648,30649,30650,30651,30652,30653,30654,30655,30656,30657,30658,30659,30660,30661,30662,30663,30664,30665,30666,30667,30668,30669,30670,30671,30672,30673,30674,30675,30676,30677,30678,30679,30680,30681,30682,30683,30684,30685,30686,30687,30688,30689,30690,30691,30692,30693,30694,30695,30696,30697,30698,30699,30700,30701,30702,30703,30704,30705,30706,30707,30708,30709,30710,30711,30712,30713,30714,30715,30716,30717,30718,30719,30720,30721,30722,30723,30724,30725,30726,30727,30728,30729,30730,30731,30732,30733,30734,30735,30736,30737,30738,30739,30740,30741,30742,30743,30744,30745,30746,30747,30748,30749,30750,30751,30752,30753,30754,30755,30756,30757,30758,30759,30760,30761,30762,30763,30764,30765,30766,30767,30768,30769,30770,30771,30772,30773,30774,30775,30776,30777,30778,30779,30780,30781,30782,30783,30784,30785,30786,30787,30788,30789,30790,30791,30792,30793,30794,30795,30796,30797,30798,30799,30800,30801,30802,30803,30804,30805,30806,30807,30808,30809,30810,30811,30812,30813,30814,30815,30816,30817,30818,30819,30820,30821,30822,30823,30824,30825,30826,30827,30828,30829,30830,30831,30832,30833,30834,30835,30836,30837,30838,30839,30840,30841,30842,30843,30844,30845,30846,30847,30848,30849,30850,30851,30852,30853,30854,30855,30856,30857,30858,30859,30860,30861,30862,30863,30864,30865,30866,30867,30868,30869,30870,30871,30872,30873,30874,30875,30876,30877,30878,30879,30880,30881,30882,30883,30884,30885,30886,30887,30888,30889,30890,30891,30892,30893,30894,30895,30896,30897,30898,30899,30900,30901,30902,30903,30904,30905,30906,30907,30908,30909,30910,30911,30912,30913,30914,30915,30916,30917,30918,30919,30920,30921,30922,30923,30924,30925,30926,30927,30928,30929,30930,30931,30932,30933,30934,30935,30936,30937,30938,30939,30940,30941,30942,30943,30944,30945,30946,30947,30948,30949,30950,30951,30952,30953,30954,30955,30956,30957,30958,30959,30960,30961,30962,30963,30964,30965,30966,30967,30968,30969,30970,30971,30972,30973,30974,30975,30976,30977,30978,30979,30980,30981,30982,30983,30984,30985,30986,30987,30988,30989,30990,30991,30992,30993,30994,30995,30996,30997,30998,30999,31000,31001,31002,31003,31004,31005,31006,31007,31008,31009,31010,31011,31012,31013,31014,31015,31016,31017,31018,31019,31020,31021,31022,31023,31024,31025,31026,31027,31028,31029,31030,31031,31032,31033,31034,31035,31036,31037,31038,31039,31040,31041,31042,31043,31044,31045,31046,31047,31048,31049,31050,31051,31052,31053,31054,31055,31056,31057,31058,31059,31060,31061,31062,31063,31064,31065,31066,31067,31068,31069,31070,31071,31072,31073,31074,31075,31076,31077,31078,31079,31080,31081,31082,31083,31084,31085,31086,31087,31088,31089,31090,31091,31092,31093,31094,31095,31096,31097,31098,31099,31100,31101,31102,31103,31104,31105,31106,31107,31108,31109,31110,31111,31112,31113,31114,31115,31116,31117,31118,31119,31120,31121,31122,31123,31124,31125,31126,31127,31128,31129,31130,31131,31132,31133,31134,31135,31136,31137,31138,31139,31140,31141,31142,31143,31144,31145,31146,31147,31148,31149,31150,31151,31152,31153,31154,31155,31156,31157,31158,31159,31160,31161,31162,31163,31164,31165,31166,31167,31168,31169,31170,31171,31172,31173,31174,31175,31176,31177,31178,31179,31180,31181,31182,31183,31184,31185,31186,31187,31188,31189,31190,31191,31192,31193,31194,31195,31196,31197,31198,31199,31200,31201,31202,31203,31204,31205,31206,31207,31208,31209,31210,31211,31212,31213,31214,31215,31216,31217,31218,31219,31220,31221,31222,31223,31224,31225,31226,31227,31228,31229,31230,31231,31232,31233,31234,31235,31236,31237,31238,31239,31240,31241,31242,31243,31244,31245,31246,31247,31248,31249,31250,31251,31252,31253,31254,31255,31256,31257,31258,31259,31260,31261,31262,31263,31264,31265,31266,31267,31268,31269,31270,31271,31272,31273,31274,31275,31276,31277,31278,31279,31280,31281,31282,31283,31284,31285,31286,31287,31288,31289,31290,31291,31292,31293,31294,31295,31296,31297,31298,31299,31300,31301,31302,31303,31304,31305,31306,31307,31308,31309,31310,31311,31312,31313,31314,31315,31316,31317,31318,31319,31320,31321,31322,31323,31324,31325,31326,31327,31328,31329,31330,31331,31332,31333,31334,31335,31336,31337,31338,31339,31340,31341,31342,31343,31344,31345,31346,31347,31348,31349,31350,31351,31352,31353,31354,31355,31356,31357,31358,31359,31360,31361,31362,31363,31364,31365,31366,31367,31368,31369,31370,31371,31372,31373,31374,31375,31376,31377,31378,31379,31380,31381,31382,31383,31384,31385,31386,31387,31388,31389,31390,31391,31392,31393,31394,31395,31396,31397,31398,31399,31400,31401,31402,31403,31404,31405,31406,31407,31408,31409,31410,31411,31412,31413,31414,31415,31416,31417,31418,31419,31420,31421,31422,31423,31424,31425,31426,31427,31428,31429,31430,31431,31432,31433,31434,31435,31436,31437,31438,31439,31440,31441,31442,31443,31444,31445,31446,31447,31448,31449,31450,31451,31452,31453,31454,31455,31456,31457,31458,31459,31460,31461,31462,31463,31464,31465,31466,31467,31468,31469,31470,31471,31472,31473,31474,31475,31476,31477,31478,31479,31480,31481,31482,31483,31484,31485,31486,31487,31488,31489,31490,31491,31492,31493,31494,31495,31496,31497,31498,31499,31500,31501,31502,31503,31504,31505,31506,31507,31508,31509,31510,31511,31512,31513,31514,31515,31516,31517,31518,31519,31520,31521,31522,31523,31524,31525,31526,31527,31528,31529,31530,31531,31532,31533,31534,31535,31536,31537,31538,31539,31540,31541,31542,31543,31544,31545,31546,31547,31548,31549,31550,31551,31552,31553,31554,31555,31556,31557,31558,31559,31560,31561,31562,31563,31564,31565,31566,31567,31568,31569,31570,31571,31572,31573,31574,31575,31576,31577,31578,31579,31580,31581,31582,31583,31584,31585,31586,31587,31588,31589,31590,31591,31592,31593,31594,31595,31596,31597,31598,31599,31600,31601,31602,31603,31604,31605,31606,31607,31608,31609,31610,31611,31612,31613,31614,31615,31616,31617,31618,31619,31620,31621,31622,31623,31624,31625,31626,31627,31628,31629,31630,31631,31632,31633,31634,31635,31636,31637,31638,31639,31640,31641,31642,31643,31644,31645,31646,31647,31648,31649,31650,31651,31652,31653,31654,31655,31656,31657,31658,31659,31660,31661,31662,31663,31664,31665,31666,31667,31668,31669,31670,31671,31672,31673,31674,31675,31676,31677,31678,31679,31680,31681,31682,31683,31684,31685,31686,31687,31688,31689,31690,31691,31692,31693,31694,31695,31696,31697,31698,31699,31700,31701,31702,31703,31704,31705,31706,31707,31708,31709,31710,31711,31712,31713,31714,31715,31716,31717,31718,31719,31720,31721,31722,31723,31724,31725,31726,31727,31728,31729,31730,31731,31732,31733,31734,31735,31736,31737,31738,31739,31740,31741,31742,31743,31744,31745,31746,31747,31748,31749,31750,31751,31752,31753,31754,31755,31756,31757,31758,31759,31760,31761,31762,31763,31764,31765,31766,31767,31768,31769,31770,31771,31772,31773,31774,31775,31776,31777,31778,31779,31780,31781,31782,31783,31784,31785,31786,31787,31788,31789,31790,31791,31792,31793,31794,31795,31796,31797,31798,31799,31800,31801,31802,31803,31804,31805,31806,31807,31808,31809,31810,31811,31812,31813,31814,31815,31816,31817,31818,31819,31820,31821,31822,31823,31824,31825,31826,31827,31828,31829,31830,31831,31832,31833,31834,31835,31836,31837,31838,31839,31840,31841,31842,31843,31844,31845,31846,31847,31848,31849,31850,31851,31852,31853,31854,31855,31856,31857,31858,31859,31860,31861,31862,31863,31864,31865,31866,31867,31868,31869,31870,31871,31872,31873,31874,31875,31876,31877,31878,31879,31880,31881,31882,31883,31884,31885,31886,31887,31888,31889,31890,31891,31892,31893,31894,31895,31896,31897,31898,31899,31900,31901,31902,31903,31904,31905,31906,31907,31908,31909,31910,31911,31912,31913,31914,31915,31916,31917,31918,31919,31920,31921,31922,31923,31924,31925,31926,31927,31928,31929,31930,31931,31932,31933,31934,31935,31936,31937,31938,31939,31940,31941,31942,31943,31944,31945,31946,31947,31948,31949,31950,31951,31952,31953,31954,31955,31956,31957,31958,31959,31960,31961,31962,31963,31964,31965,31966,31967,31968,31969,31970,31971,31972,31973,31974,31975,31976,31977,31978,31979,31980,31981,31982,31983,31984,31985,31986,31987,31988,31989,31990,31991,31992,31993,31994,31995,31996,31997,31998,31999,32000,32001,32002,32003,32004,32005,32006,32007,32008,32009,32010,32011,32012,32013,32014,32015,32016,32017,32018,32019,32020,32021,32022,32023,32024,32025,32026,32027,32028,32029,32030,32031,32032,32033,32034,32035,32036,32037,32038,32039,32040,32041,32042,32043,32044,32045,32046,32047,32048,32049,32050,32051,32052,32053,32054,32055,32056,32057,32058,32059,32060,32061,32062,32063,32064,32065,32066,32067,32068,32069,32070,32071,32072,32073,32074,32075,32076,32077,32078,32079,32080,32081,32082,32083,32084,32085,32086,32087,32088,32089,32090,32091,32092,32093,32094,32095,32096,32097,32098,32099,32100,32101,32102,32103,32104,32105,32106,32107,32108,32109,32110,32111,32112,32113,32114,32115,32116,32117,32118,32119,32120,32121,32122,32123,32124,32125,32126,32127,32128,32129,32130,32131,32132,32133,32134,32135,32136,32137,32138,32139,32140,32141,32142,32143,32144,32145,32146,32147,32148,32149,32150,32151,32152,32153,32154,32155,32156,32157,32158,32159,32160,32161,32162,32163,32164,32165,32166,32167,32168,32169,32170,32171,32172,32173,32174,32175,32176,32177,32178,32179,32180,32181,32182,32183,32184,32185,32186,32187,32188,32189,32190,32191,32192,32193,32194,32195,32196,32197,32198,32199,32200,32201,32202,32203,32204,32205,32206,32207,32208,32209,32210,32211,32212,32213,32214,32215,32216,32217,32218,32219,32220,32221,32222,32223,32224,32225,32226,32227,32228,32229,32230,32231,32232,32233,32234,32235,32236,32237,32238,32239,32240,32241,32242,32243,32244,32245,32246,32247,32248,32249,32250,32251,32252,32253,32254,32255,32256,32257,32258,32259,32260,32261,32262,32263,32264,32265,32266,32267,32268,32269,32270,32271,32272,32273,32274,32275,32276,32277,32278,32279,32280,32281,32282,32283,32284,32285,32286,32287,32288,32289,32290,32291,32292,32293,32294,32295,32296,32297,32298,32299,32300,32301,32302,32303,32304,32305,32306,32307,32308,32309,32310,32311,32312,32313,32314,32315,32316,32317,32318,32319,32320,32321,32322,32323,32324,32325,32326,32327,32328,32329,32330,32331,32332,32333,32334,32335,32336,32337,32338,32339,32340,32341,32342,32343,32344,32345,32346,32347,32348,32349,32350,32351,32352,32353,32354,32355,32356,32357,32358,32359,32360,32361,32362,32363,32364,32365,32366,32367,32368,32369,32370,32371,32372,32373,32374,32375,32376,32377,32378,32379,32380,32381,32382,32383,32384,32385,32386,32387,32388,32389,32390,32391,32392,32393,32394,32395,32396,32397,32398,32399,32400,32401,32402,32403,32404,32405,32406,32407,32408,32409,32410,32411,32412,32413,32414,32415,32416,32417,32418,32419,32420,32421,32422,32423,32424,32425,32426,32427,32428,32429,32430,32431,32432,32433,32434,32435,32436,32437,32438,32439,32440,32441,32442,32443,32444,32445,32446,32447,32448,32449,32450,32451,32452,32453,32454,32455,32456,32457,32458,32459,32460,32461,32462,32463,32464,32465,32466,32467,32468,32469,32470,32471,32472,32473,32474,32475,32476,32477,32478,32479,32480,32481,32482,32483,32484,32485,32486,32487,32488,32489,32490,32491,32492,32493,32494,32495,32496,32497,32498,32499,32500,32501,32502,32503,32504,32505,32506,32507,32508,32509,32510,32511,32512,32513,32514,32515,32516,32517,32518,32519,32520,32521,32522,32523,32524,32525,32526,32527,32528,32529,32530,32531,32532,32533,32534,32535,32536,32537,32538,32539,32540,32541,32542,32543,32544,32545,32546,32547,32548,32549,32550,32551,32552,32553,32554,32555,32556,32557,32558,32559,32560,32561,32562,32563,32564,32565,32566,32567,32568,32569,32570,32571,32572,32573,32574,32575,32576,32577,32578,32579,32580,32581,32582,32583,32584,32585,32586,32587,32588,32589,32590,32591,32592,32593,32594,32595,32596,32597,32598,32599,32600,32601,32602,32603,32604,32605,32606,32607,32608,32609,32610,32611,32612,32613,32614,32615,32616,32617,32618,32619,32620,32621,32622,32623,32624,32625,32626,32627,32628,32629,32630,32631,32632,32633,32634,32635,32636,32637,32638,32639,32640,32641,32642,32643,32644,32645,32646,32647,32648,32649,32650,32651,32652,32653,32654,32655,32656,32657,32658,32659,32660,32661,32662,32663,32664,32665,32666,32667,32668,32669,32670,32671,32672,32673,32674,32675,32676,32677,32678,32679,32680,32681,32682,32683,32684,32685,32686,32687,32688,32689,32690,32691,32692,32693,32694,32695,32696,32697,32698,32699,32700,32701,32702,32703,32704,32705,32706,32707,32708,32709,32710,32711,32712,32713,32714,32715,32716,32717,32718,32719,32720,32721,32722,32723,32724,32725,32726,32727,32728,32729,32730,32731,32732,32733,32734,32735,32736,32737,32738,32739,32740,32741,32742,32743,32744,32745,32746,32747,32748,32749,32750,32751,32752,32753,32754,32755,32756,32757,32758,32759,32760,32761,32762,32763,32764,32765,32766,32767,32768,32769,32770,32771,32772,32773,32774,32775,32776,32777,32778,32779,32780,32781,32782,32783,32784,32785,32786,32787,32788,32789,32790,32791,32792,32793,32794,32795,32796,32797,32798,32799,32800,32801,32802,32803,32804,32805,32806,32807,32808,32809,32810,32811,32812,32813,32814,32815,32816,32817,32818,32819,32820,32821,32822,32823,32824,32825,32826,32827,32828,32829,32830,32831,32832,32833,32834,32835,32836,32837,32838,32839,32840,32841,32842,32843,32844,32845,32846,32847,32848,32849,32850,32851,32852,32853,32854,32855,32856,32857,32858,32859,32860,32861,32862,32863,32864,32865,32866,32867,32868,32869,32870,32871,32872,32873,32874,32875,32876,32877,32878,32879,32880,32881,32882,32883,32884,32885,32886,32887,32888,32889,32890,32891,32892,32893,32894,32895,32896,32897,32898,32899,32900,32901,32902,32903,32904,32905,32906,32907,32908,32909,32910,32911,32912,32913,32914,32915,32916,32917,32918,32919,32920,32921,32922,32923,32924,32925,32926,32927,32928,32929,32930,32931,32932,32933,32934,32935,32936,32937,32938,32939,32940,32941,32942,32943,32944,32945,32946,32947,32948,32949,32950,32951,32952,32953,32954,32955,32956,32957,32958,32959,32960,32961,32962,32963,32964,32965,32966,32967,32968,32969,32970,32971,32972,32973,32974,32975,32976,32977,32978,32979,32980,32981,32982,32983,32984,32985,32986,32987,32988,32989,32990,32991,32992,32993,32994,32995,32996,32997,32998,32999,33000,33001,33002,33003,33004,33005,33006,33007,33008,33009,33010,33011,33012,33013,33014,33015,33016,33017,33018,33019,33020,33021,33022,33023,33024,33025,33026,33027,33028,33029,33030,33031,33032,33033,33034,33035,33036,33037,33038,33039,33040,33041,33042,33043,33044,33045,33046,33047,33048,33049,33050,33051,33052,33053,33054,33055,33056,33057,33058,33059,33060,33061,33062,33063,33064,33065,33066,33067,33068,33069,33070,33071,33072,33073,33074,33075,33076,33077,33078,33079,33080,33081,33082,33083,33084,33085,33086,33087,33088,33089,33090,33091,33092,33093,33094,33095,33096,33097,33098,33099,33100,33101,33102,33103,33104,33105,33106,33107,33108,33109,33110,33111,33112,33113,33114,33115,33116,33117,33118,33119,33120,33121,33122,33123,33124,33125,33126,33127,33128,33129,33130,33131,33132,33133,33134,33135,33136,33137,33138,33139,33140,33141,33142,33143,33144,33145,33146,33147,33148,33149,33150,33151,33152,33153,33154,33155,33156,33157,33158,33159,33160,33161,33162,33163,33164,33165,33166,33167,33168,33169,33170,33171,33172,33173,33174,33175,33176,33177,33178,33179,33180,33181,33182,33183,33184,33185,33186,33187,33188,33189,33190,33191,33192,33193,33194,33195,33196,33197,33198,33199,33200,33201,33202,33203,33204,33205,33206,33207,33208,33209,33210,33211,33212,33213,33214,33215,33216,33217,33218,33219,33220,33221,33222,33223,33224,33225,33226,33227,33228,33229,33230,33231,33232,33233,33234,33235,33236,33237,33238,33239,33240,33241,33242,33243,33244,33245,33246,33247,33248,33249,33250,33251,33252,33253,33254,33255,33256,33257,33258,33259,33260,33261,33262,33263,33264,33265,33266,33267,33268,33269,33270,33271,33272,33273,33274,33275,33276,33277,33278,33279,33280,33281,33282,33283,33284,33285,33286,33287,33288,33289,33290,33291,33292,33293,33294,33295,33296,33297,33298,33299,33300,33301,33302,33303,33304,33305,33306,33307,33308,33309,33310,33311,33312,33313,33314,33315,33316,33317,33318,33319,33320,33321,33322,33323,33324,33325,33326,33327,33328,33329,33330,33331,33332,33333,33334,33335,33336,33337,33338,33339,33340,33341,33342,33343,33344,33345,33346,33347,33348,33349,33350,33351,33352,33353,33354,33355,33356,33357,33358,33359,33360,33361,33362,33363,33364,33365,33366,33367,33368,33369,33370,33371,33372,33373,33374,33375,33376,33377,33378,33379,33380,33381,33382,33383,33384,33385,33386,33387,33388,33389,33390,33391,33392,33393,33394,33395,33396,33397,33398,33399,33400,33401,33402,33403,33404,33405,33406,33407,33408,33409,33410,33411,33412,33413,33414,33415,33416,33417,33418,33419,33420,33421,33422,33423,33424,33425,33426,33427,33428,33429,33430,33431,33432,33433,33434,33435,33436,33437,33438,33439,33440,33441,33442,33443,33444,33445,33446,33447,33448,33449,33450,33451,33452,33453,33454,33455,33456,33457,33458,33459,33460,33461,33462,33463,33464,33465,33466,33467,33468,33469,33470,33471,33472,33473,33474,33475,33476,33477,33478,33479,33480,33481,33482,33483,33484,33485,33486,33487,33488,33489,33490,33491,33492,33493,33494,33495,33496,33497,33498,33499,33500,33501,33502,33503,33504,33505,33506,33507,33508,33509,33510,33511,33512,33513,33514,33515,33516,33517,33518,33519,33520,33521,33522,33523,33524,33525,33526,33527,33528,33529,33530,33531,33532,33533,33534,33535,33536,33537,33538,33539,33540,33541,33542,33543,33544,33545,33546,33547,33548,33549,33550,33551,33552,33553,33554,33555,33556,33557,33558,33559,33560,33561,33562,33563,33564,33565,33566,33567,33568,33569,33570,33571,33572,33573,33574,33575,33576,33577,33578,33579,33580,33581,33582,33583,33584,33585,33586,33587,33588,33589,33590,33591,33592,33593,33594,33595,33596,33597,33598,33599,33600,33601,33602,33603,33604,33605,33606,33607,33608,33609,33610,33611,33612,33613,33614,33615,33616,33617,33618,33619,33620,33621,33622,33623,33624,33625,33626,33627,33628,33629,33630,33631,33632,33633,33634,33635,33636,33637,33638,33639,33640,33641,33642,33643,33644,33645,33646,33647,33648,33649,33650,33651,33652,33653,33654,33655,33656,33657,33658,33659,33660,33661,33662,33663,33664,33665,33666,33667,33668,33669,33670,33671,33672,33673,33674,33675,33676,33677,33678,33679,33680,33681,33682,33683,33684,33685,33686,33687,33688,33689,33690,33691,33692,33693,33694,33695,33696,33697,33698,33699,33700,33701,33702,33703,33704,33705,33706,33707,33708,33709,33710,33711,33712,33713,33714,33715,33716,33717,33718,33719,33720,33721,33722,33723,33724,33725,33726,33727,33728,33729,33730,33731,33732,33733,33734,33735,33736,33737,33738,33739,33740,33741,33742,33743,33744,33745,33746,33747,33748,33749,33750,33751,33752,33753,33754,33755,33756,33757,33758,33759,33760,33761,33762,33763,33764,33765,33766,33767,33768,33769,33770,33771,33772,33773,33774,33775,33776,33777,33778,33779,33780,33781,33782,33783,33784,33785,33786,33787,33788,33789,33790,33791,33792,33793,33794,33795,33796,33797,33798,33799,33800,33801,33802,33803,33804,33805,33806,33807,33808,33809,33810,33811,33812,33813,33814,33815,33816,33817,33818,33819,33820,33821,33822,33823,33824,33825,33826,33827,33828,33829,33830,33831,33832,33833,33834,33835,33836,33837,33838,33839,33840,33841,33842,33843,33844,33845,33846,33847,33848,33849,33850,33851,33852,33853,33854,33855,33856,33857,33858,33859,33860,33861,33862,33863,33864,33865,33866,33867,33868,33869,33870,33871,33872,33873,33874,33875,33876,33877,33878,33879,33880,33881,33882,33883,33884,33885,33886,33887,33888,33889,33890,33891,33892,33893,33894,33895,33896,33897,33898,33899,33900,33901,33902,33903,33904,33905,33906,33907,33908,33909,33910,33911,33912,33913,33914,33915,33916,33917,33918,33919,33920,33921,33922,33923,33924,33925,33926,33927,33928,33929,33930,33931,33932,33933,33934,33935,33936,33937,33938,33939,33940,33941,33942,33943,33944,33945,33946,33947,33948,33949,33950,33951,33952,33953,33954,33955,33956,33957,33958,33959,33960,33961,33962,33963,33964,33965,33966,33967,33968,33969,33970,33971,33972,33973,33974,33975,33976,33977,33978,33979,33980,33981,33982,33983,33984,33985,33986,33987,33988,33989,33990,33991,33992,33993,33994,33995,33996,33997,33998,33999,34000,34001,34002,34003,34004,34005,34006,34007,34008,34009,34010,34011,34012,34013,34014,34015,34016,34017,34018,34019,34020,34021,34022,34023,34024,34025,34026,34027,34028,34029,34030,34031,34032,34033,34034,34035,34036,34037,34038,34039,34040,34041,34042,34043,34044,34045,34046,34047,34048,34049,34050,34051,34052,34053,34054,34055,34056,34057,34058,34059,34060,34061,34062,34063,34064,34065,34066,34067,34068,34069,34070,34071,34072,34073,34074,34075,34076,34077,34078,34079,34080,34081,34082,34083,34084,34085,34086,34087,34088,34089,34090,34091,34092,34093,34094,34095,34096,34097,34098,34099,34100,34101,34102,34103,34104,34105,34106,34107,34108,34109,34110,34111,34112,34113,34114,34115,34116,34117,34118,34119,34120,34121,34122,34123,34124,34125,34126,34127,34128,34129,34130,34131,34132,34133,34134,34135,34136,34137,34138,34139,34140,34141,34142,34143,34144,34145,34146,34147,34148,34149,34150,34151,34152,34153,34154,34155,34156,34157,34158,34159,34160,34161,34162,34163,34164,34165,34166,34167,34168,34169,34170,34171,34172,34173,34174,34175,34176,34177,34178,34179,34180,34181,34182,34183,34184,34185,34186,34187,34188,34189,34190,34191,34192,34193,34194,34195,34196,34197,34198,34199,34200,34201,34202,34203,34204,34205,34206,34207,34208,34209,34210,34211,34212,34213,34214,34215,34216,34217,34218,34219,34220,34221,34222,34223,34224,34225,34226,34227,34228,34229,34230,34231,34232,34233,34234,34235,34236,34237,34238,34239,34240,34241,34242,34243,34244,34245,34246,34247,34248,34249,34250,34251,34252,34253,34254,34255,34256,34257,34258,34259,34260,34261,34262,34263,34264,34265,34266,34267,34268,34269,34270,34271,34272,34273,34274,34275,34276,34277,34278,34279,34280,34281,34282,34283,34284,34285,34286,34287,34288,34289,34290,34291,34292,34293,34294,34295,34296,34297,34298,34299,34300,34301,34302,34303,34304,34305,34306,34307,34308,34309,34310,34311,34312,34313,34314,34315,34316,34317,34318,34319,34320,34321,34322,34323,34324,34325,34326,34327,34328,34329,34330,34331,34332,34333,34334,34335,34336,34337,34338,34339,34340,34341,34342,34343,34344,34345,34346,34347,34348,34349,34350,34351,34352,34353,34354,34355,34356,34357,34358,34359,34360,34361,34362,34363,34364,34365,34366,34367,34368,34369,34370,34371,34372,34373,34374,34375,34376,34377,34378,34379,34380,34381,34382,34383,34384,34385,34386,34387,34388,34389,34390,34391,34392,34393,34394,34395,34396,34397,34398,34399,34400,34401,34402,34403,34404,34405,34406,34407,34408,34409,34410,34411,34412,34413,34414,34415,34416,34417,34418,34419,34420,34421,34422,34423,34424,34425,34426,34427,34428,34429,34430,34431,34432,34433,34434,34435,34436,34437,34438,34439,34440,34441,34442,34443,34444,34445,34446,34447,34448,34449,34450,34451,34452,34453,34454,34455,34456,34457,34458,34459,34460,34461,34462,34463,34464,34465,34466,34467,34468,34469,34470,34471,34472,34473,34474,34475,34476,34477,34478,34479,34480,34481,34482,34483,34484,34485,34486,34487,34488,34489,34490,34491,34492,34493,34494,34495,34496,34497,34498,34499,34500,34501,34502,34503,34504,34505,34506,34507,34508,34509,34510,34511,34512,34513,34514,34515,34516,34517,34518,34519,34520,34521,34522,34523,34524,34525,34526,34527,34528,34529,34530,34531,34532,34533,34534,34535,34536,34537,34538,34539,34540,34541,34542,34543,34544,34545,34546,34547,34548,34549,34550,34551,34552,34553,34554,34555,34556,34557,34558,34559,34560,34561,34562,34563,34564,34565,34566,34567,34568,34569,34570,34571,34572,34573,34574,34575,34576,34577,34578,34579,34580,34581,34582,34583,34584,34585,34586,34587,34588,34589,34590,34591,34592,34593,34594,34595,34596,34597,34598,34599,34600,34601,34602,34603,34604,34605,34606,34607,34608,34609,34610,34611,34612,34613,34614,34615,34616,34617,34618,34619,34620,34621,34622,34623,34624,34625,34626,34627,34628,34629,34630,34631,34632,34633,34634,34635,34636,34637,34638,34639,34640,34641,34642,34643,34644,34645,34646,34647,34648,34649,34650,34651,34652,34653,34654,34655,34656,34657,34658,34659,34660,34661,34662,34663,34664,34665,34666,34667,34668,34669,34670,34671,34672,34673,34674,34675,34676,34677,34678,34679,34680,34681,34682,34683,34684,34685,34686,34687,34688,34689,34690,34691,34692,34693,34694,34695,34696,34697,34698,34699,34700,34701,34702,34703,34704,34705,34706,34707,34708,34709,34710,34711,34712,34713,34714,34715,34716,34717,34718,34719,34720,34721,34722,34723,34724,34725,34726,34727,34728,34729,34730,34731,34732,34733,34734,34735,34736,34737,34738,34739,34740,34741,34742,34743,34744,34745,34746,34747,34748,34749,34750,34751,34752,34753,34754,34755,34756,34757,34758,34759,34760,34761,34762,34763,34764,34765,34766,34767,34768,34769,34770,34771,34772,34773,34774,34775,34776,34777,34778,34779,34780,34781,34782,34783,34784,34785,34786,34787,34788,34789,34790,34791,34792,34793,34794,34795,34796,34797,34798,34799,34800,34801,34802,34803,34804,34805,34806,34807,34808,34809,34810,34811,34812,34813,34814,34815,34816,34817,34818,34819,34820,34821,34822,34823,34824,34825,34826,34827,34828,34829,34830,34831,34832,34833,34834,34835,34836,34837,34838,34839,34840,34841,34842,34843,34844,34845,34846,34847,34848,34849,34850,34851,34852,34853,34854,34855,34856,34857,34858,34859,34860,34861,34862,34863,34864,34865,34866,34867,34868,34869,34870,34871,34872,34873,34874,34875,34876,34877,34878,34879,34880,34881,34882,34883,34884,34885,34886,34887,34888,34889,34890,34891,34892,34893,34894,34895,34896,34897,34898,34899,34900,34901,34902,34903,34904,34905,34906,34907,34908,34909,34910,34911,34912,34913,34914,34915,34916,34917,34918,34919,34920,34921,34922,34923,34924,34925,34926,34927,34928,34929,34930,34931,34932,34933,34934,34935,34936,34937,34938,34939,34940,34941,34942,34943,34944,34945,34946,34947,34948,34949,34950,34951,34952,34953,34954,34955,34956,34957,34958,34959,34960,34961,34962,34963,34964,34965,34966,34967,34968,34969,34970,34971,34972,34973,34974,34975,34976,34977,34978,34979,34980,34981,34982,34983,34984,34985,34986,34987,34988,34989,34990,34991,34992,34993,34994,34995,34996,34997,34998,34999,35000,35001,35002,35003,35004,35005,35006,35007,35008,35009,35010,35011,35012,35013,35014,35015,35016,35017,35018,35019,35020,35021,35022,35023,35024,35025,35026,35027,35028,35029,35030,35031,35032,35033,35034,35035,35036,35037,35038,35039,35040,35041,35042,35043,35044,35045,35046,35047,35048,35049,35050,35051,35052,35053,35054,35055,35056,35057,35058,35059,35060,35061,35062,35063,35064,35065,35066,35067,35068,35069,35070,35071,35072,35073,35074,35075,35076,35077,35078,35079,35080,35081,35082,35083,35084,35085,35086,35087,35088,35089,35090,35091,35092,35093,35094,35095,35096,35097,35098,35099,35100,35101,35102,35103,35104,35105,35106,35107,35108,35109,35110,35111,35112,35113,35114,35115,35116,35117,35118,35119,35120,35121,35122,35123,35124,35125,35126,35127,35128,35129,35130,35131,35132,35133,35134,35135,35136,35137,35138,35139,35140,35141,35142,35143,35144,35145,35146,35147,35148,35149,35150,35151,35152,35153,35154,35155,35156,35157,35158,35159,35160,35161,35162,35163,35164,35165,35166,35167,35168,35169,35170,35171,35172,35173,35174,35175,35176,35177,35178,35179,35180,35181,35182,35183,35184,35185,35186,35187,35188,35189,35190,35191,35192,35193,35194,35195,35196,35197,35198,35199,35200,35201,35202,35203,35204,35205,35206,35207,35208,35209,35210,35211,35212,35213,35214,35215,35216,35217,35218,35219,35220,35221,35222,35223,35224,35225,35226,35227,35228,35229,35230,35231,35232,35233,35234,35235,35236,35237,35238,35239,35240,35241,35242,35243,35244,35245,35246,35247,35248,35249,35250,35251,35252,35253,35254,35255,35256,35257,35258,35259,35260,35261,35262,35263,35264,35265,35266,35267,35268,35269,35270,35271,35272,35273,35274,35275,35276,35277,35278,35279,35280,35281,35282,35283,35284,35285,35286,35287,35288,35289,35290,35291,35292,35293,35294,35295,35296,35297,35298,35299,35300,35301,35302,35303,35304,35305,35306,35307,35308,35309,35310,35311,35312,35313,35314,35315,35316,35317,35318,35319,35320,35321,35322,35323,35324,35325,35326,35327,35328,35329,35330,35331,35332,35333,35334,35335,35336,35337,35338,35339,35340,35341,35342,35343,35344,35345,35346,35347,35348,35349,35350,35351,35352,35353,35354,35355,35356,35357,35358,35359,35360,35361,35362,35363,35364,35365,35366,35367,35368,35369,35370,35371,35372,35373,35374,35375,35376,35377,35378,35379,35380,35381,35382,35383,35384,35385,35386,35387,35388,35389,35390,35391,35392,35393,35394,35395,35396,35397,35398,35399,35400,35401,35402,35403,35404,35405,35406,35407,35408,35409,35410,35411,35412,35413,35414,35415,35416,35417,35418,35419,35420,35421,35422,35423,35424,35425,35426,35427,35428,35429,35430,35431,35432,35433,35434,35435,35436,35437,35438,35439,35440,35441,35442,35443,35444,35445,35446,35447,35448,35449,35450,35451,35452,35453,35454,35455,35456,35457,35458,35459,35460,35461,35462,35463,35464,35465,35466,35467,35468,35469,35470,35471,35472,35473,35474,35475,35476,35477,35478,35479,35480,35481,35482,35483,35484,35485,35486,35487,35488,35489,35490,35491,35492,35493,35494,35495,35496,35497,35498,35499,35500,35501,35502,35503,35504,35505,35506,35507,35508,35509,35510,35511,35512,35513,35514,35515,35516,35517,35518,35519,35520,35521,35522,35523,35524,35525,35526,35527,35528,35529,35530,35531,35532,35533,35534,35535,35536,35537,35538,35539,35540,35541,35542,35543,35544,35545,35546,35547,35548,35549,35550,35551,35552,35553,35554,35555,35556,35557,35558,35559,35560,35561,35562,35563,35564,35565,35566,35567,35568,35569,35570,35571,35572,35573,35574,35575,35576,35577,35578,35579,35580,35581,35582,35583,35584,35585,35586,35587,35588,35589,35590,35591,35592,35593,35594,35595,35596,35597,35598,35599,35600,35601,35602,35603,35604,35605,35606,35607,35608,35609,35610,35611,35612,35613,35614,35615,35616,35617,35618,35619,35620,35621,35622,35623,35624,35625,35626,35627,35628,35629,35630,35631,35632,35633,35634,35635,35636,35637,35638,35639,35640,35641,35642,35643,35644,35645,35646,35647,35648,35649,35650,35651,35652,35653,35654,35655,35656,35657,35658,35659,35660,35661,35662,35663,35664,35665,35666,35667,35668,35669,35670,35671,35672,35673,35674,35675,35676,35677,35678,35679,35680,35681,35682,35683,35684,35685,35686,35687,35688,35689,35690,35691,35692,35693,35694,35695,35696,35697,35698,35699,35700,35701,35702,35703,35704,35705,35706,35707,35708,35709,35710,35711,35712,35713,35714,35715,35716,35717,35718,35719,35720,35721,35722,35723,35724,35725,35726,35727,35728,35729,35730,35731,35732,35733,35734,35735,35736,35737,35738,35739,35740,35741,35742,35743,35744,35745,35746,35747,35748,35749,35750,35751,35752,35753,35754,35755,35756,35757,35758,35759,35760,35761,35762,35763,35764,35765,35766,35767,35768,35769,35770,35771,35772,35773,35774,35775,35776,35777,35778,35779,35780,35781,35782,35783,35784,35785,35786,35787,35788,35789,35790,35791,35792,35793,35794,35795,35796,35797,35798,35799,35800,35801,35802,35803,35804,35805,35806,35807,35808,35809,35810,35811,35812,35813,35814,35815,35816,35817,35818,35819,35820,35821,35822,35823,35824,35825,35826,35827,35828,35829,35830,35831,35832,35833,35834,35835,35836,35837,35838,35839,35840,35841,35842,35843,35844,35845,35846,35847,35848,35849,35850,35851,35852,35853,35854,35855,35856,35857,35858,35859,35860,35861,35862,35863,35864,35865,35866,35867,35868,35869,35870,35871,35872,35873,35874,35875,35876,35877,35878,35879,35880,35881,35882,35883,35884,35885,35886,35887,35888,35889,35890,35891,35892,35893,35894,35895,35896,35897,35898,35899,35900,35901,35902,35903,35904,35905,35906,35907,35908,35909,35910,35911,35912,35913,35914,35915,35916,35917,35918,35919,35920,35921,35922,35923,35924,35925,35926,35927,35928,35929,35930,35931,35932,35933,35934,35935,35936,35937,35938,35939,35940,35941,35942,35943,35944,35945,35946,35947,35948,35949,35950,35951,35952,35953,35954,35955,35956,35957,35958,35959,35960,35961,35962,35963,35964,35965,35966,35967,35968,35969,35970,35971,35972,35973,35974,35975,35976,35977,35978,35979,35980,35981,35982,35983,35984,35985,35986,35987,35988,35989,35990,35991,35992,35993,35994,35995,35996,35997,35998,35999,36000,36001,36002,36003,36004,36005,36006,36007,36008,36009,36010,36011,36012,36013,36014,36015,36016,36017,36018,36019,36020,36021,36022,36023,36024,36025,36026,36027,36028,36029,36030,36031,36032,36033,36034,36035,36036,36037,36038,36039,36040,36041,36042,36043,36044,36045,36046,36047,36048,36049,36050,36051,36052,36053,36054,36055,36056,36057,36058,36059,36060,36061,36062,36063,36064,36065,36066,36067,36068,36069,36070,36071,36072,36073,36074,36075,36076,36077,36078,36079,36080,36081,36082,36083,36084,36085,36086,36087,36088,36089,36090,36091,36092,36093,36094,36095,36096,36097,36098,36099,36100,36101,36102,36103,36104,36105,36106,36107,36108,36109,36110,36111,36112,36113,36114,36115,36116,36117,36118,36119,36120,36121,36122,36123,36124,36125,36126,36127,36128,36129,36130,36131,36132,36133,36134,36135,36136,36137,36138,36139,36140,36141,36142,36143,36144,36145,36146,36147,36148,36149,36150,36151,36152,36153,36154,36155,36156,36157,36158,36159,36160,36161,36162,36163,36164,36165,36166,36167,36168,36169,36170,36171,36172,36173,36174,36175,36176,36177,36178,36179,36180,36181,36182,36183,36184,36185,36186,36187,36188,36189,36190,36191,36192,36193,36194,36195,36196,36197,36198,36199,36200,36201,36202,36203,36204,36205,36206,36207,36208,36209,36210,36211,36212,36213,36214,36215,36216,36217,36218,36219,36220,36221,36222,36223,36224,36225,36226,36227,36228,36229,36230,36231,36232,36233,36234,36235,36236,36237,36238,36239,36240,36241,36242,36243,36244,36245,36246,36247,36248,36249,36250,36251,36252,36253,36254,36255,36256,36257,36258,36259,36260,36261,36262,36263,36264,36265,36266,36267,36268,36269,36270,36271,36272,36273,36274,36275,36276,36277,36278,36279,36280,36281,36282,36283,36284,36285,36286,36287,36288,36289,36290,36291,36292,36293,36294,36295,36296,36297,36298,36299,36300,36301,36302,36303,36304,36305,36306,36307,36308,36309,36310,36311,36312,36313,36314,36315,36316,36317,36318,36319,36320,36321,36322,36323,36324,36325,36326,36327,36328,36329,36330,36331,36332,36333,36334,36335,36336,36337,36338,36339,36340,36341,36342,36343,36344,36345,36346,36347,36348,36349,36350,36351,36352,36353,36354,36355,36356,36357,36358,36359,36360,36361,36362,36363,36364,36365,36366,36367,36368,36369,36370,36371,36372,36373,36374,36375,36376,36377,36378,36379,36380,36381,36382,36383,36384,36385,36386,36387,36388,36389,36390,36391,36392,36393,36394,36395,36396,36397,36398,36399,36400,36401,36402,36403,36404,36405,36406,36407,36408,36409,36410,36411,36412,36413,36414,36415,36416,36417,36418,36419,36420,36421,36422,36423,36424,36425,36426,36427,36428,36429,36430,36431,36432,36433,36434,36435,36436,36437,36438,36439,36440,36441,36442,36443,36444,36445,36446,36447,36448,36449,36450,36451,36452,36453,36454,36455,36456,36457,36458,36459,36460,36461,36462,36463,36464,36465,36466,36467,36468,36469,36470,36471,36472,36473,36474,36475,36476,36477,36478,36479,36480,36481,36482,36483,36484,36485,36486,36487,36488,36489,36490,36491,36492,36493,36494,36495,36496,36497,36498,36499,36500,36501,36502,36503,36504,36505,36506,36507,36508,36509,36510,36511,36512,36513,36514,36515,36516,36517,36518,36519,36520,36521,36522,36523,36524,36525,36526,36527,36528,36529,36530,36531,36532,36533,36534,36535,36536,36537,36538,36539,36540,36541,36542,36543,36544,36545,36546,36547,36548,36549,36550,36551,36552,36553,36554,36555,36556,36557,36558,36559,36560,36561,36562,36563,36564,36565,36566,36567,36568,36569,36570,36571,36572,36573,36574,36575,36576,36577,36578,36579,36580,36581,36582,36583,36584,36585,36586,36587,36588,36589,36590,36591,36592,36593,36594,36595,36596,36597,36598,36599,36600,36601,36602,36603,36604,36605,36606,36607,36608,36609,36610,36611,36612,36613,36614,36615,36616,36617,36618,36619,36620,36621,36622,36623,36624,36625,36626,36627,36628,36629,36630,36631,36632,36633,36634,36635,36636,36637,36638,36639,36640,36641,36642,36643,36644,36645,36646,36647,36648,36649,36650,36651,36652,36653,36654,36655,36656,36657,36658,36659,36660,36661,36662,36663,36664,36665,36666,36667,36668,36669,36670,36671,36672,36673,36674,36675,36676,36677,36678,36679,36680,36681,36682,36683,36684,36685,36686,36687,36688,36689,36690,36691,36692,36693,36694,36695,36696,36697,36698,36699,36700,36701,36702,36703,36704,36705,36706,36707,36708,36709,36710,36711,36712,36713,36714,36715,36716,36717,36718,36719,36720,36721,36722,36723,36724,36725,36726,36727,36728,36729,36730,36731,36732,36733,36734,36735,36736,36737,36738,36739,36740,36741,36742,36743,36744,36745,36746,36747,36748,36749,36750,36751,36752,36753,36754,36755,36756,36757,36758,36759,36760,36761,36762,36763,36764,36765,36766,36767,36768,36769,36770,36771,36772,36773,36774,36775,36776,36777,36778,36779,36780,36781,36782,36783,36784,36785,36786,36787,36788,36789,36790,36791,36792,36793,36794,36795,36796,36797,36798,36799,36800,36801,36802,36803,36804,36805,36806,36807,36808,36809,36810,36811,36812,36813,36814,36815,36816,36817,36818,36819,36820,36821,36822,36823,36824,36825,36826,36827,36828,36829,36830,36831,36832,36833,36834,36835,36836,36837,36838,36839,36840,36841,36842,36843,36844,36845,36846,36847,36848,36849,36850,36851,36852,36853,36854,36855,36856,36857,36858,36859,36860,36861,36862,36863,36864,36865,36866,36867,36868,36869,36870,36871,36872,36873,36874,36875,36876,36877,36878,36879,36880,36881,36882,36883,36884,36885,36886,36887,36888,36889,36890,36891,36892,36893,36894,36895,36896,36897,36898,36899,36900,36901,36902,36903,36904,36905,36906,36907,36908,36909,36910,36911,36912,36913,36914,36915,36916,36917,36918,36919,36920,36921,36922,36923,36924,36925,36926,36927,36928,36929,36930,36931,36932,36933,36934,36935,36936,36937,36938,36939,36940,36941,36942,36943,36944,36945,36946,36947,36948,36949,36950,36951,36952,36953,36954,36955,36956,36957,36958,36959,36960,36961,36962,36963,36964,36965,36966,36967,36968,36969,36970,36971,36972,36973,36974,36975,36976,36977,36978,36979,36980,36981,36982,36983,36984,36985,36986,36987,36988,36989,36990,36991,36992,36993,36994,36995,36996,36997,36998,36999,37000,37001,37002,37003,37004,37005,37006,37007,37008,37009,37010,37011,37012,37013,37014,37015,37016,37017,37018,37019,37020,37021,37022,37023,37024,37025,37026,37027,37028,37029,37030,37031,37032,37033,37034,37035,37036,37037,37038,37039,37040,37041,37042,37043,37044,37045,37046,37047,37048,37049,37050,37051,37052,37053,37054,37055,37056,37057,37058,37059,37060,37061,37062,37063,37064,37065,37066,37067,37068,37069,37070,37071,37072,37073,37074,37075,37076,37077,37078,37079,37080,37081,37082,37083,37084,37085,37086,37087,37088,37089,37090,37091,37092,37093,37094,37095,37096,37097,37098,37099,37100,37101,37102,37103,37104,37105,37106,37107,37108,37109,37110,37111,37112,37113,37114,37115,37116,37117,37118,37119,37120,37121,37122,37123,37124,37125,37126,37127,37128,37129,37130,37131,37132,37133,37134,37135,37136,37137,37138,37139,37140,37141,37142,37143,37144,37145,37146,37147,37148,37149,37150,37151,37152,37153,37154,37155,37156,37157,37158,37159,37160,37161,37162,37163,37164,37165,37166,37167,37168,37169,37170,37171,37172,37173,37174,37175,37176,37177,37178,37179,37180,37181,37182,37183,37184,37185,37186,37187,37188,37189,37190,37191,37192,37193,37194,37195,37196,37197,37198,37199,37200,37201,37202,37203,37204,37205,37206,37207,37208,37209,37210,37211,37212,37213,37214,37215,37216,37217,37218,37219,37220,37221,37222,37223,37224,37225,37226,37227,37228,37229,37230,37231,37232,37233,37234,37235,37236,37237,37238,37239,37240,37241,37242,37243,37244,37245,37246,37247,37248,37249,37250,37251,37252,37253,37254,37255,37256,37257,37258,37259,37260,37261,37262,37263,37264,37265,37266,37267,37268,37269,37270,37271,37272,37273,37274,37275,37276,37277,37278,37279,37280,37281,37282,37283,37284,37285,37286,37287,37288,37289,37290,37291,37292,37293,37294,37295,37296,37297,37298,37299,37300,37301,37302,37303,37304,37305,37306,37307,37308,37309,37310,37311,37312,37313,37314,37315,37316,37317,37318,37319,37320,37321,37322,37323,37324,37325,37326,37327,37328,37329,37330,37331,37332,37333,37334,37335,37336,37337,37338,37339,37340,37341,37342,37343,37344,37345,37346,37347,37348,37349,37350,37351,37352,37353,37354,37355,37356,37357,37358,37359,37360,37361,37362,37363,37364,37365,37366,37367,37368,37369,37370,37371,37372,37373,37374,37375,37376,37377,37378,37379,37380,37381,37382,37383,37384,37385,37386,37387,37388,37389,37390,37391,37392,37393,37394,37395,37396,37397,37398,37399,37400,37401,37402,37403,37404,37405,37406,37407,37408,37409,37410,37411,37412,37413,37414,37415,37416,37417,37418,37419,37420,37421,37422,37423,37424,37425,37426,37427,37428,37429,37430,37431,37432,37433,37434,37435,37436,37437,37438,37439,37440,37441,37442,37443,37444,37445,37446,37447,37448,37449,37450,37451,37452,37453,37454,37455,37456,37457,37458,37459,37460,37461,37462,37463,37464,37465,37466,37467,37468,37469,37470,37471,37472,37473,37474,37475,37476,37477,37478,37479,37480,37481,37482,37483,37484,37485,37486,37487,37488,37489,37490,37491,37492,37493,37494,37495,37496,37497,37498,37499,37500,37501,37502,37503,37504,37505,37506,37507,37508,37509,37510,37511,37512,37513,37514,37515,37516,37517,37518,37519,37520,37521,37522,37523,37524,37525,37526,37527,37528,37529,37530,37531,37532,37533,37534,37535,37536,37537,37538,37539,37540,37541,37542,37543,37544,37545,37546,37547,37548,37549,37550,37551,37552,37553,37554,37555,37556,37557,37558,37559,37560,37561,37562,37563,37564,37565,37566,37567,37568,37569,37570,37571,37572,37573,37574,37575,37576,37577,37578,37579,37580,37581,37582,37583,37584,37585,37586,37587,37588,37589,37590,37591,37592,37593,37594,37595,37596,37597,37598,37599,37600,37601,37602,37603,37604,37605,37606,37607,37608,37609,37610,37611,37612,37613,37614,37615,37616,37617,37618,37619,37620,37621,37622,37623,37624,37625,37626,37627,37628,37629,37630,37631,37632,37633,37634,37635,37636,37637,37638,37639,37640,37641,37642,37643,37644,37645,37646,37647,37648,37649,37650,37651,37652,37653,37654,37655,37656,37657,37658,37659,37660,37661,37662,37663,37664,37665,37666,37667,37668,37669,37670,37671,37672,37673,37674,37675,37676,37677,37678,37679,37680,37681,37682,37683,37684,37685,37686,37687,37688,37689,37690,37691,37692,37693,37694,37695,37696,37697,37698,37699,37700,37701,37702,37703,37704,37705,37706,37707,37708,37709,37710,37711,37712,37713,37714,37715,37716,37717,37718,37719,37720,37721,37722,37723,37724,37725,37726,37727,37728,37729,37730,37731,37732,37733,37734,37735,37736,37737,37738,37739,37740,37741,37742,37743,37744,37745,37746,37747,37748,37749,37750,37751,37752,37753,37754,37755,37756,37757,37758,37759,37760,37761,37762,37763,37764,37765,37766,37767,37768,37769,37770,37771,37772,37773,37774,37775,37776,37777,37778,37779,37780,37781,37782,37783,37784,37785,37786,37787,37788,37789,37790,37791,37792,37793,37794,37795,37796,37797,37798,37799,37800,37801,37802,37803,37804,37805,37806,37807,37808,37809,37810,37811,37812,37813,37814,37815,37816,37817,37818,37819,37820,37821,37822,37823,37824,37825,37826,37827,37828,37829,37830,37831,37832,37833,37834,37835,37836,37837,37838,37839,37840,37841,37842,37843,37844,37845,37846,37847,37848,37849,37850,37851,37852,37853,37854,37855,37856,37857,37858,37859,37860,37861,37862,37863,37864,37865,37866,37867,37868,37869,37870,37871,37872,37873,37874,37875,37876,37877,37878,37879,37880,37881,37882,37883,37884,37885,37886,37887,37888,37889,37890,37891,37892,37893,37894,37895,37896,37897,37898,37899,37900,37901,37902,37903,37904,37905,37906,37907,37908,37909,37910,37911,37912,37913,37914,37915,37916,37917,37918,37919,37920,37921,37922,37923,37924,37925,37926,37927,37928,37929,37930,37931,37932,37933,37934,37935,37936,37937,37938,37939,37940,37941,37942,37943,37944,37945,37946,37947,37948,37949,37950,37951,37952,37953,37954,37955,37956,37957,37958,37959,37960,37961,37962,37963,37964,37965,37966,37967,37968,37969,37970,37971,37972,37973,37974,37975,37976,37977,37978,37979,37980,37981,37982,37983,37984,37985,37986,37987,37988,37989,37990,37991,37992,37993,37994,37995,37996,37997,37998,37999,38000,38001,38002,38003,38004,38005,38006,38007,38008,38009,38010,38011,38012,38013,38014,38015,38016,38017,38018,38019,38020,38021,38022,38023,38024,38025,38026,38027,38028,38029,38030,38031,38032,38033,38034,38035,38036,38037,38038,38039,38040,38041,38042,38043,38044,38045,38046,38047,38048,38049,38050,38051,38052,38053,38054,38055,38056,38057,38058,38059,38060,38061,38062,38063,38064,38065,38066,38067,38068,38069,38070,38071,38072,38073,38074,38075,38076,38077,38078,38079,38080,38081,38082,38083,38084,38085,38086,38087,38088,38089,38090,38091,38092,38093,38094,38095,38096,38097,38098,38099,38100,38101,38102,38103,38104,38105,38106,38107,38108,38109,38110,38111,38112,38113,38114,38115,38116,38117,38118,38119,38120,38121,38122,38123,38124,38125,38126,38127,38128,38129,38130,38131,38132,38133,38134,38135,38136,38137,38138,38139,38140,38141,38142,38143,38144,38145,38146,38147,38148,38149,38150,38151,38152,38153,38154,38155,38156,38157,38158,38159,38160,38161,38162,38163,38164,38165,38166,38167,38168,38169,38170,38171,38172,38173,38174,38175,38176,38177,38178,38179,38180,38181,38182,38183,38184,38185,38186,38187,38188,38189,38190,38191,38192,38193,38194,38195,38196,38197,38198,38199,38200,38201,38202,38203,38204,38205,38206,38207,38208,38209,38210,38211,38212,38213,38214,38215,38216,38217,38218,38219,38220,38221,38222,38223,38224,38225,38226,38227,38228,38229,38230,38231,38232,38233,38234,38235,38236,38237,38238,38239,38240,38241,38242,38243,38244,38245,38246,38247,38248,38249,38250,38251,38252,38253,38254,38255,38256,38257,38258,38259,38260,38261,38262,38263,38264,38265,38266,38267,38268,38269,38270,38271,38272,38273,38274,38275,38276,38277,38278,38279,38280,38281,38282,38283,38284,38285,38286,38287,38288,38289,38290,38291,38292,38293,38294,38295,38296,38297,38298,38299,38300,38301,38302,38303,38304,38305,38306,38307,38308,38309,38310,38311,38312,38313,38314,38315,38316,38317,38318,38319,38320,38321,38322,38323,38324,38325,38326,38327,38328,38329,38330,38331,38332,38333,38334,38335,38336,38337,38338,38339,38340,38341,38342,38343,38344,38345,38346,38347,38348,38349,38350,38351,38352,38353,38354,38355,38356,38357,38358,38359,38360,38361,38362,38363,38364,38365,38366,38367,38368,38369,38370,38371,38372,38373,38374,38375,38376,38377,38378,38379,38380,38381,38382,38383,38384,38385,38386,38387,38388,38389,38390,38391,38392,38393,38394,38395,38396,38397,38398,38399,38400,38401,38402,38403,38404,38405,38406,38407,38408,38409,38410,38411,38412,38413,38414,38415,38416,38417,38418,38419,38420,38421,38422,38423,38424,38425,38426,38427,38428,38429,38430,38431,38432,38433,38434,38435,38436,38437,38438,38439,38440,38441,38442,38443,38444,38445,38446,38447,38448,38449,38450,38451,38452,38453,38454,38455,38456,38457,38458,38459,38460,38461,38462,38463,38464,38465,38466,38467,38468,38469,38470,38471,38472,38473,38474,38475,38476,38477,38478,38479,38480,38481,38482,38483,38484,38485,38486,38487,38488,38489,38490,38491,38492,38493,38494,38495,38496,38497,38498,38499,38500,38501,38502,38503,38504,38505,38506,38507,38508,38509,38510,38511,38512,38513,38514,38515,38516,38517,38518,38519,38520,38521,38522,38523,38524,38525,38526,38527,38528,38529,38530,38531,38532,38533,38534,38535,38536,38537,38538,38539,38540,38541,38542,38543,38544,38545,38546,38547,38548,38549,38550,38551,38552,38553,38554,38555,38556,38557,38558,38559,38560,38561,38562,38563,38564,38565,38566,38567,38568,38569,38570,38571,38572,38573,38574,38575,38576,38577,38578,38579,38580,38581,38582,38583,38584,38585,38586,38587,38588,38589,38590,38591,38592,38593,38594,38595,38596,38597,38598,38599,38600,38601,38602,38603,38604,38605,38606,38607,38608,38609,38610,38611,38612,38613,38614,38615,38616,38617,38618,38619,38620,38621,38622,38623,38624,38625,38626,38627,38628,38629,38630,38631,38632,38633,38634,38635,38636,38637,38638,38639,38640,38641,38642,38643,38644,38645,38646,38647,38648,38649,38650,38651,38652,38653,38654,38655,38656,38657,38658,38659,38660,38661,38662,38663,38664,38665,38666,38667,38668,38669,38670,38671,38672,38673,38674,38675,38676,38677,38678,38679,38680,38681,38682,38683,38684,38685,38686,38687,38688,38689,38690,38691,38692,38693,38694,38695,38696,38697,38698,38699,38700,38701,38702,38703,38704,38705,38706,38707,38708,38709,38710,38711,38712,38713,38714,38715,38716,38717,38718,38719,38720,38721,38722,38723,38724,38725,38726,38727,38728,38729,38730,38731,38732,38733,38734,38735,38736,38737,38738,38739,38740,38741,38742,38743,38744,38745,38746,38747,38748,38749,38750,38751,38752,38753,38754,38755,38756,38757,38758,38759,38760,38761,38762,38763,38764,38765,38766,38767,38768,38769,38770,38771,38772,38773,38774,38775,38776,38777,38778,38779,38780,38781,38782,38783,38784,38785,38786,38787,38788,38789,38790,38791,38792,38793,38794,38795,38796,38797,38798,38799,38800,38801,38802,38803,38804,38805,38806,38807,38808,38809,38810,38811,38812,38813,38814,38815,38816,38817,38818,38819,38820,38821,38822,38823,38824,38825,38826,38827,38828,38829,38830,38831,38832,38833,38834,38835,38836,38837,38838,38839,38840,38841,38842,38843,38844,38845,38846,38847,38848,38849,38850,38851,38852,38853,38854,38855,38856,38857,38858,38859,38860,38861,38862,38863,38864,38865,38866,38867,38868,38869,38870,38871,38872,38873,38874,38875,38876,38877,38878,38879,38880,38881,38882,38883,38884,38885,38886,38887,38888,38889,38890,38891,38892,38893,38894,38895,38896,38897,38898,38899,38900,38901,38902,38903,38904,38905,38906,38907,38908,38909,38910,38911,38912,38913,38914,38915,38916,38917,38918,38919,38920,38921,38922,38923,38924,38925,38926,38927,38928,38929,38930,38931,38932,38933,38934,38935,38936,38937,38938,38939,38940,38941,38942,38943,38944,38945,38946,38947,38948,38949,38950,38951,38952,38953,38954,38955,38956,38957,38958,38959,38960,38961,38962,38963,38964,38965,38966,38967,38968,38969,38970,38971,38972,38973,38974,38975,38976,38977,38978,38979,38980,38981,38982,38983,38984,38985,38986,38987,38988,38989,38990,38991,38992,38993,38994,38995,38996,38997,38998,38999,39000,39001,39002,39003,39004,39005,39006,39007,39008,39009,39010,39011,39012,39013,39014,39015,39016,39017,39018,39019,39020,39021,39022,39023,39024,39025,39026,39027,39028,39029,39030,39031,39032,39033,39034,39035,39036,39037,39038,39039,39040,39041,39042,39043,39044,39045,39046,39047,39048,39049,39050,39051,39052,39053,39054,39055,39056,39057,39058,39059,39060,39061,39062,39063,39064,39065,39066,39067,39068,39069,39070,39071,39072,39073,39074,39075,39076,39077,39078,39079,39080,39081,39082,39083,39084,39085,39086,39087,39088,39089,39090,39091,39092,39093,39094,39095,39096,39097,39098,39099,39100,39101,39102,39103,39104,39105,39106,39107,39108,39109,39110,39111,39112,39113,39114,39115,39116,39117,39118,39119,39120,39121,39122,39123,39124,39125,39126,39127,39128,39129,39130,39131,39132,39133,39134,39135,39136,39137,39138,39139,39140,39141,39142,39143,39144,39145,39146,39147,39148,39149,39150,39151,39152,39153,39154,39155,39156,39157,39158,39159,39160,39161,39162,39163,39164,39165,39166,39167,39168,39169,39170,39171,39172,39173,39174,39175,39176,39177,39178,39179,39180,39181,39182,39183,39184,39185,39186,39187,39188,39189,39190,39191,39192,39193,39194,39195,39196,39197,39198,39199,39200,39201,39202,39203,39204,39205,39206,39207,39208,39209,39210,39211,39212,39213,39214,39215,39216,39217,39218,39219,39220,39221,39222,39223,39224,39225,39226,39227,39228,39229,39230,39231,39232,39233,39234,39235,39236,39237,39238,39239,39240,39241,39242,39243,39244,39245,39246,39247,39248,39249,39250,39251,39252,39253,39254,39255,39256,39257,39258,39259,39260,39261,39262,39263,39264,39265,39266,39267,39268,39269,39270,39271,39272,39273,39274,39275,39276,39277,39278,39279,39280,39281,39282,39283,39284,39285,39286,39287,39288,39289,39290,39291,39292,39293,39294,39295,39296,39297,39298,39299,39300,39301,39302,39303,39304,39305,39306,39307,39308,39309,39310,39311,39312,39313,39314,39315,39316,39317,39318,39319,39320,39321,39322,39323,39324,39325,39326,39327,39328,39329,39330,39331,39332,39333,39334,39335,39336,39337,39338,39339,39340,39341,39342,39343,39344,39345,39346,39347,39348,39349,39350,39351,39352,39353,39354,39355,39356,39357,39358,39359,39360,39361,39362,39363,39364,39365,39366,39367,39368,39369,39370,39371,39372,39373,39374,39375,39376,39377,39378,39379,39380,39381,39382,39383,39384,39385,39386,39387,39388,39389,39390,39391,39392,39393,39394,39395,39396,39397,39398,39399,39400,39401,39402,39403,39404,39405,39406,39407,39408,39409,39410,39411,39412,39413,39414,39415,39416,39417,39418,39419,39420,39421,39422,39423,39424,39425,39426,39427,39428,39429,39430,39431,39432,39433,39434,39435,39436,39437,39438,39439,39440,39441,39442,39443,39444,39445,39446,39447,39448,39449,39450,39451,39452,39453,39454,39455,39456,39457,39458,39459,39460,39461,39462,39463,39464,39465,39466,39467,39468,39469,39470,39471,39472,39473,39474,39475,39476,39477,39478,39479,39480,39481,39482,39483,39484,39485,39486,39487,39488,39489,39490,39491,39492,39493,39494,39495,39496,39497,39498,39499,39500,39501,39502,39503,39504,39505,39506,39507,39508,39509,39510,39511,39512,39513,39514,39515,39516,39517,39518,39519,39520,39521,39522,39523,39524,39525,39526,39527,39528,39529,39530,39531,39532,39533,39534,39535,39536,39537,39538,39539,39540,39541,39542,39543,39544,39545,39546,39547,39548,39549,39550,39551,39552,39553,39554,39555,39556,39557,39558,39559,39560,39561,39562,39563,39564,39565,39566,39567,39568,39569,39570,39571,39572,39573,39574,39575,39576,39577,39578,39579,39580,39581,39582,39583,39584,39585,39586,39587,39588,39589,39590,39591,39592,39593,39594,39595,39596,39597,39598,39599,39600,39601,39602,39603,39604,39605,39606,39607,39608,39609,39610,39611,39612,39613,39614,39615,39616,39617,39618,39619,39620,39621,39622,39623,39624,39625,39626,39627,39628,39629,39630,39631,39632,39633,39634,39635,39636,39637,39638,39639,39640,39641,39642,39643,39644,39645,39646,39647,39648,39649,39650,39651,39652,39653,39654,39655,39656,39657,39658,39659,39660,39661,39662,39663,39664,39665,39666,39667,39668,39669,39670,39671,39672,39673,39674,39675,39676,39677,39678,39679,39680,39681,39682,39683,39684,39685,39686,39687,39688,39689,39690,39691,39692,39693,39694,39695,39696,39697,39698,39699,39700,39701,39702,39703,39704,39705,39706,39707,39708,39709,39710,39711,39712,39713,39714,39715,39716,39717,39718,39719,39720,39721,39722,39723,39724,39725,39726,39727,39728,39729,39730,39731,39732,39733,39734,39735,39736,39737,39738,39739,39740,39741,39742,39743,39744,39745,39746,39747,39748,39749,39750,39751,39752,39753,39754,39755,39756,39757,39758,39759,39760,39761,39762,39763,39764,39765,39766,39767,39768,39769,39770,39771,39772,39773,39774,39775,39776,39777,39778,39779,39780,39781,39782,39783,39784,39785,39786,39787,39788,39789,39790,39791,39792,39793,39794,39795,39796,39797,39798,39799,39800,39801,39802,39803,39804,39805,39806,39807,39808,39809,39810,39811,39812,39813,39814,39815,39816,39817,39818,39819,39820,39821,39822,39823,39824,39825,39826,39827,39828,39829,39830,39831,39832,39833,39834,39835,39836,39837,39838,39839,39840,39841,39842,39843,39844,39845,39846,39847,39848,39849,39850,39851,39852,39853,39854,39855,39856,39857,39858,39859,39860,39861,39862,39863,39864,39865,39866,39867,39868,39869,39870,39871,39872,39873,39874,39875,39876,39877,39878,39879,39880,39881,39882,39883,39884,39885,39886,39887,39888,39889,39890,39891,39892,39893,39894,39895,39896,39897,39898,39899,39900,39901,39902,39903,39904,39905,39906,39907,39908,39909,39910,39911,39912,39913,39914,39915,39916,39917,39918,39919,39920,39921,39922,39923,39924,39925,39926,39927,39928,39929,39930,39931,39932,39933,39934,39935,39936,39937,39938,39939,39940,39941,39942,39943,39944,39945,39946,39947,39948,39949,39950,39951,39952,39953,39954,39955,39956,39957,39958,39959,39960,39961,39962,39963,39964,39965,39966,39967,39968,39969,39970,39971,39972,39973,39974,39975,39976,39977,39978,39979,39980,39981,39982,39983,39984,39985,39986,39987,39988,39989,39990,39991,39992,39993,39994,39995,39996,39997,39998,39999,40000,40001,40002,40003,40004,40005,40006,40007,40008,40009,40010,40011,40012,40013,40014,40015,40016,40017,40018,40019,40020,40021,40022,40023,40024,40025,40026,40027,40028,40029,40030,40031,40032,40033,40034,40035,40036,40037,40038,40039,40040,40041,40042,40043,40044,40045,40046,40047,40048,40049,40050,40051,40052,40053,40054,40055,40056,40057,40058,40059,40060,40061,40062,40063,40064,40065,40066,40067,40068,40069,40070,40071,40072,40073,40074,40075,40076,40077,40078,40079,40080,40081,40082,40083,40084,40085,40086,40087,40088,40089,40090,40091,40092,40093,40094,40095,40096,40097,40098,40099,40100,40101,40102,40103,40104,40105,40106,40107,40108,40109,40110,40111,40112,40113,40114,40115,40116,40117,40118,40119,40120,40121,40122,40123,40124,40125,40126,40127,40128,40129,40130,40131,40132,40133,40134,40135,40136,40137,40138,40139,40140,40141,40142,40143,40144,40145,40146,40147,40148,40149,40150,40151,40152,40153,40154,40155,40156,40157,40158,40159,40160,40161,40162,40163,40164,40165,40166,40167,40168,40169,40170,40171,40172,40173,40174,40175,40176,40177,40178,40179,40180,40181,40182,40183,40184,40185,40186,40187,40188,40189,40190,40191,40192,40193,40194,40195,40196,40197,40198,40199,40200,40201,40202,40203,40204,40205,40206,40207,40208,40209,40210,40211,40212,40213,40214,40215,40216,40217,40218,40219,40220,40221,40222,40223,40224,40225,40226,40227,40228,40229,40230,40231,40232,40233,40234,40235,40236,40237,40238,40239,40240,40241,40242,40243,40244,40245,40246,40247,40248,40249,40250,40251,40252,40253,40254,40255,40256,40257,40258,40259,40260,40261,40262,40263,40264,40265,40266,40267,40268,40269,40270,40271,40272,40273,40274,40275,40276,40277,40278,40279,40280,40281,40282,40283,40284,40285,40286,40287,40288,40289,40290,40291,40292,40293,40294,40295,40296,40297,40298,40299,40300,40301,40302,40303,40304,40305,40306,40307,40308,40309,40310,40311,40312,40313,40314,40315,40316,40317,40318,40319,40320,40321,40322,40323,40324,40325,40326,40327,40328,40329,40330,40331,40332,40333,40334,40335,40336,40337,40338,40339,40340,40341,40342,40343,40344,40345,40346,40347,40348,40349,40350,40351,40352,40353,40354,40355,40356,40357,40358,40359,40360,40361,40362,40363,40364,40365,40366,40367,40368,40369,40370,40371,40372,40373,40374,40375,40376,40377,40378,40379,40380,40381,40382,40383,40384,40385,40386,40387,40388,40389,40390,40391,40392,40393,40394,40395,40396,40397,40398,40399,40400,40401,40402,40403,40404,40405,40406,40407,40408,40409,40410,40411,40412,40413,40414,40415,40416,40417,40418,40419,40420,40421,40422,40423,40424,40425,40426,40427,40428,40429,40430,40431,40432,40433,40434,40435,40436,40437,40438,40439,40440,40441,40442,40443,40444,40445,40446,40447,40448,40449,40450,40451,40452,40453,40454,40455,40456,40457,40458,40459,40460,40461,40462,40463,40464,40465,40466,40467,40468,40469,40470,40471,40472,40473,40474,40475,40476,40477,40478,40479,40480,40481,40482,40483,40484,40485,40486,40487,40488,40489,40490,40491,40492,40493,40494,40495,40496,40497,40498,40499,40500,40501,40502,40503,40504,40505,40506,40507,40508,40509,40510,40511,40512,40513,40514,40515,40516,40517,40518,40519,40520,40521,40522,40523,40524,40525,40526,40527,40528,40529,40530,40531,40532,40533,40534,40535,40536,40537,40538,40539,40540,40541,40542,40543,40544,40545,40546,40547,40548,40549,40550,40551,40552,40553,40554,40555,40556,40557,40558,40559,40560,40561,40562,40563,40564,40565,40566,40567,40568,40569,40570,40571,40572,40573,40574,40575,40576,40577,40578,40579,40580,40581,40582,40583,40584,40585,40586,40587,40588,40589,40590,40591,40592,40593,40594,40595,40596,40597,40598,40599,40600,40601,40602,40603,40604,40605,40606,40607,40608,40609,40610,40611,40612,40613,40614,40615,40616,40617,40618,40619,40620,40621,40622,40623,40624,40625,40626,40627,40628,40629,40630,40631,40632,40633,40634,40635,40636,40637,40638,40639,40640,40641,40642,40643,40644,40645,40646,40647,40648,40649,40650,40651,40652,40653,40654,40655,40656,40657,40658,40659,40660,40661,40662,40663,40664,40665,40666,40667,40668,40669,40670,40671,40672,40673,40674,40675,40676,40677,40678,40679,40680,40681,40682,40683,40684,40685,40686,40687,40688,40689,40690,40691,40692,40693,40694,40695,40696,40697,40698,40699,40700,40701,40702,40703,40704,40705,40706,40707,40708,40709,40710,40711,40712,40713,40714,40715,40716,40717,40718,40719,40720,40721,40722,40723,40724,40725,40726,40727,40728,40729,40730,40731,40732,40733,40734,40735,40736,40737,40738,40739,40740,40741,40742,40743,40744,40745,40746,40747,40748,40749,40750,40751,40752,40753,40754,40755,40756,40757,40758,40759,40760,40761,40762,40763,40764,40765,40766,40767,40768,40769,40770,40771,40772,40773,40774,40775,40776,40777,40778,40779,40780,40781,40782,40783,40784,40785,40786,40787,40788,40789,40790,40791,40792,40793,40794,40795,40796,40797,40798,40799,40800,40801,40802,40803,40804,40805,40806,40807,40808,40809,40810,40811,40812,40813,40814,40815,40816,40817,40818,40819,40820,40821,40822,40823,40824,40825,40826,40827,40828,40829,40830,40831,40832,40833,40834,40835,40836,40837,40838,40839,40840,40841,40842,40843,40844,40845,40846,40847,40848,40849,40850,40851,40852,40853,40854,40855,40856,40857,40858,40859,40860,40861,40862,40863,40864,40865,40866,40867,40868,40869,40870,40871,40872,40873,40874,40875,40876,40877,40878,40879,40880,40881,40882,40883,40884,40885,40886,40887,40888,40889,40890,40891,40892,40893,40894,40895,40896,40897,40898,40899,40900,40901,40902,40903,40904,40905,40906,40907,40908,40960,40961,40962,40963,40964,40965,40966,40967,40968,40969,40970,40971,40972,40973,40974,40975,40976,40977,40978,40979,40980,40981,40982,40983,40984,40985,40986,40987,40988,40989,40990,40991,40992,40993,40994,40995,40996,40997,40998,40999,41000,41001,41002,41003,41004,41005,41006,41007,41008,41009,41010,41011,41012,41013,41014,41015,41016,41017,41018,41019,41020,41021,41022,41023,41024,41025,41026,41027,41028,41029,41030,41031,41032,41033,41034,41035,41036,41037,41038,41039,41040,41041,41042,41043,41044,41045,41046,41047,41048,41049,41050,41051,41052,41053,41054,41055,41056,41057,41058,41059,41060,41061,41062,41063,41064,41065,41066,41067,41068,41069,41070,41071,41072,41073,41074,41075,41076,41077,41078,41079,41080,41081,41082,41083,41084,41085,41086,41087,41088,41089,41090,41091,41092,41093,41094,41095,41096,41097,41098,41099,41100,41101,41102,41103,41104,41105,41106,41107,41108,41109,41110,41111,41112,41113,41114,41115,41116,41117,41118,41119,41120,41121,41122,41123,41124,41125,41126,41127,41128,41129,41130,41131,41132,41133,41134,41135,41136,41137,41138,41139,41140,41141,41142,41143,41144,41145,41146,41147,41148,41149,41150,41151,41152,41153,41154,41155,41156,41157,41158,41159,41160,41161,41162,41163,41164,41165,41166,41167,41168,41169,41170,41171,41172,41173,41174,41175,41176,41177,41178,41179,41180,41181,41182,41183,41184,41185,41186,41187,41188,41189,41190,41191,41192,41193,41194,41195,41196,41197,41198,41199,41200,41201,41202,41203,41204,41205,41206,41207,41208,41209,41210,41211,41212,41213,41214,41215,41216,41217,41218,41219,41220,41221,41222,41223,41224,41225,41226,41227,41228,41229,41230,41231,41232,41233,41234,41235,41236,41237,41238,41239,41240,41241,41242,41243,41244,41245,41246,41247,41248,41249,41250,41251,41252,41253,41254,41255,41256,41257,41258,41259,41260,41261,41262,41263,41264,41265,41266,41267,41268,41269,41270,41271,41272,41273,41274,41275,41276,41277,41278,41279,41280,41281,41282,41283,41284,41285,41286,41287,41288,41289,41290,41291,41292,41293,41294,41295,41296,41297,41298,41299,41300,41301,41302,41303,41304,41305,41306,41307,41308,41309,41310,41311,41312,41313,41314,41315,41316,41317,41318,41319,41320,41321,41322,41323,41324,41325,41326,41327,41328,41329,41330,41331,41332,41333,41334,41335,41336,41337,41338,41339,41340,41341,41342,41343,41344,41345,41346,41347,41348,41349,41350,41351,41352,41353,41354,41355,41356,41357,41358,41359,41360,41361,41362,41363,41364,41365,41366,41367,41368,41369,41370,41371,41372,41373,41374,41375,41376,41377,41378,41379,41380,41381,41382,41383,41384,41385,41386,41387,41388,41389,41390,41391,41392,41393,41394,41395,41396,41397,41398,41399,41400,41401,41402,41403,41404,41405,41406,41407,41408,41409,41410,41411,41412,41413,41414,41415,41416,41417,41418,41419,41420,41421,41422,41423,41424,41425,41426,41427,41428,41429,41430,41431,41432,41433,41434,41435,41436,41437,41438,41439,41440,41441,41442,41443,41444,41445,41446,41447,41448,41449,41450,41451,41452,41453,41454,41455,41456,41457,41458,41459,41460,41461,41462,41463,41464,41465,41466,41467,41468,41469,41470,41471,41472,41473,41474,41475,41476,41477,41478,41479,41480,41481,41482,41483,41484,41485,41486,41487,41488,41489,41490,41491,41492,41493,41494,41495,41496,41497,41498,41499,41500,41501,41502,41503,41504,41505,41506,41507,41508,41509,41510,41511,41512,41513,41514,41515,41516,41517,41518,41519,41520,41521,41522,41523,41524,41525,41526,41527,41528,41529,41530,41531,41532,41533,41534,41535,41536,41537,41538,41539,41540,41541,41542,41543,41544,41545,41546,41547,41548,41549,41550,41551,41552,41553,41554,41555,41556,41557,41558,41559,41560,41561,41562,41563,41564,41565,41566,41567,41568,41569,41570,41571,41572,41573,41574,41575,41576,41577,41578,41579,41580,41581,41582,41583,41584,41585,41586,41587,41588,41589,41590,41591,41592,41593,41594,41595,41596,41597,41598,41599,41600,41601,41602,41603,41604,41605,41606,41607,41608,41609,41610,41611,41612,41613,41614,41615,41616,41617,41618,41619,41620,41621,41622,41623,41624,41625,41626,41627,41628,41629,41630,41631,41632,41633,41634,41635,41636,41637,41638,41639,41640,41641,41642,41643,41644,41645,41646,41647,41648,41649,41650,41651,41652,41653,41654,41655,41656,41657,41658,41659,41660,41661,41662,41663,41664,41665,41666,41667,41668,41669,41670,41671,41672,41673,41674,41675,41676,41677,41678,41679,41680,41681,41682,41683,41684,41685,41686,41687,41688,41689,41690,41691,41692,41693,41694,41695,41696,41697,41698,41699,41700,41701,41702,41703,41704,41705,41706,41707,41708,41709,41710,41711,41712,41713,41714,41715,41716,41717,41718,41719,41720,41721,41722,41723,41724,41725,41726,41727,41728,41729,41730,41731,41732,41733,41734,41735,41736,41737,41738,41739,41740,41741,41742,41743,41744,41745,41746,41747,41748,41749,41750,41751,41752,41753,41754,41755,41756,41757,41758,41759,41760,41761,41762,41763,41764,41765,41766,41767,41768,41769,41770,41771,41772,41773,41774,41775,41776,41777,41778,41779,41780,41781,41782,41783,41784,41785,41786,41787,41788,41789,41790,41791,41792,41793,41794,41795,41796,41797,41798,41799,41800,41801,41802,41803,41804,41805,41806,41807,41808,41809,41810,41811,41812,41813,41814,41815,41816,41817,41818,41819,41820,41821,41822,41823,41824,41825,41826,41827,41828,41829,41830,41831,41832,41833,41834,41835,41836,41837,41838,41839,41840,41841,41842,41843,41844,41845,41846,41847,41848,41849,41850,41851,41852,41853,41854,41855,41856,41857,41858,41859,41860,41861,41862,41863,41864,41865,41866,41867,41868,41869,41870,41871,41872,41873,41874,41875,41876,41877,41878,41879,41880,41881,41882,41883,41884,41885,41886,41887,41888,41889,41890,41891,41892,41893,41894,41895,41896,41897,41898,41899,41900,41901,41902,41903,41904,41905,41906,41907,41908,41909,41910,41911,41912,41913,41914,41915,41916,41917,41918,41919,41920,41921,41922,41923,41924,41925,41926,41927,41928,41929,41930,41931,41932,41933,41934,41935,41936,41937,41938,41939,41940,41941,41942,41943,41944,41945,41946,41947,41948,41949,41950,41951,41952,41953,41954,41955,41956,41957,41958,41959,41960,41961,41962,41963,41964,41965,41966,41967,41968,41969,41970,41971,41972,41973,41974,41975,41976,41977,41978,41979,41980,41981,41982,41983,41984,41985,41986,41987,41988,41989,41990,41991,41992,41993,41994,41995,41996,41997,41998,41999,42000,42001,42002,42003,42004,42005,42006,42007,42008,42009,42010,42011,42012,42013,42014,42015,42016,42017,42018,42019,42020,42021,42022,42023,42024,42025,42026,42027,42028,42029,42030,42031,42032,42033,42034,42035,42036,42037,42038,42039,42040,42041,42042,42043,42044,42045,42046,42047,42048,42049,42050,42051,42052,42053,42054,42055,42056,42057,42058,42059,42060,42061,42062,42063,42064,42065,42066,42067,42068,42069,42070,42071,42072,42073,42074,42075,42076,42077,42078,42079,42080,42081,42082,42083,42084,42085,42086,42087,42088,42089,42090,42091,42092,42093,42094,42095,42096,42097,42098,42099,42100,42101,42102,42103,42104,42105,42106,42107,42108,42109,42110,42111,42112,42113,42114,42115,42116,42117,42118,42119,42120,42121,42122,42123,42124,42192,42193,42194,42195,42196,42197,42198,42199,42200,42201,42202,42203,42204,42205,42206,42207,42208,42209,42210,42211,42212,42213,42214,42215,42216,42217,42218,42219,42220,42221,42222,42223,42224,42225,42226,42227,42228,42229,42230,42231,42232,42233,42234,42235,42236,42237,42240,42241,42242,42243,42244,42245,42246,42247,42248,42249,42250,42251,42252,42253,42254,42255,42256,42257,42258,42259,42260,42261,42262,42263,42264,42265,42266,42267,42268,42269,42270,42271,42272,42273,42274,42275,42276,42277,42278,42279,42280,42281,42282,42283,42284,42285,42286,42287,42288,42289,42290,42291,42292,42293,42294,42295,42296,42297,42298,42299,42300,42301,42302,42303,42304,42305,42306,42307,42308,42309,42310,42311,42312,42313,42314,42315,42316,42317,42318,42319,42320,42321,42322,42323,42324,42325,42326,42327,42328,42329,42330,42331,42332,42333,42334,42335,42336,42337,42338,42339,42340,42341,42342,42343,42344,42345,42346,42347,42348,42349,42350,42351,42352,42353,42354,42355,42356,42357,42358,42359,42360,42361,42362,42363,42364,42365,42366,42367,42368,42369,42370,42371,42372,42373,42374,42375,42376,42377,42378,42379,42380,42381,42382,42383,42384,42385,42386,42387,42388,42389,42390,42391,42392,42393,42394,42395,42396,42397,42398,42399,42400,42401,42402,42403,42404,42405,42406,42407,42408,42409,42410,42411,42412,42413,42414,42415,42416,42417,42418,42419,42420,42421,42422,42423,42424,42425,42426,42427,42428,42429,42430,42431,42432,42433,42434,42435,42436,42437,42438,42439,42440,42441,42442,42443,42444,42445,42446,42447,42448,42449,42450,42451,42452,42453,42454,42455,42456,42457,42458,42459,42460,42461,42462,42463,42464,42465,42466,42467,42468,42469,42470,42471,42472,42473,42474,42475,42476,42477,42478,42479,42480,42481,42482,42483,42484,42485,42486,42487,42488,42489,42490,42491,42492,42493,42494,42495,42496,42497,42498,42499,42500,42501,42502,42503,42504,42505,42506,42507,42508,42512,42513,42514,42515,42516,42517,42518,42519,42520,42521,42522,42523,42524,42525,42526,42527,42538,42539,42560,42561,42562,42563,42564,42565,42566,42567,42568,42569,42570,42571,42572,42573,42574,42575,42576,42577,42578,42579,42580,42581,42582,42583,42584,42585,42586,42587,42588,42589,42590,42591,42592,42593,42594,42595,42596,42597,42598,42599,42600,42601,42602,42603,42604,42605,42606,42623,42624,42625,42626,42627,42628,42629,42630,42631,42632,42633,42634,42635,42636,42637,42638,42639,42640,42641,42642,42643,42644,42645,42646,42647,42656,42657,42658,42659,42660,42661,42662,42663,42664,42665,42666,42667,42668,42669,42670,42671,42672,42673,42674,42675,42676,42677,42678,42679,42680,42681,42682,42683,42684,42685,42686,42687,42688,42689,42690,42691,42692,42693,42694,42695,42696,42697,42698,42699,42700,42701,42702,42703,42704,42705,42706,42707,42708,42709,42710,42711,42712,42713,42714,42715,42716,42717,42718,42719,42720,42721,42722,42723,42724,42725,42726,42727,42728,42729,42730,42731,42732,42733,42734,42735,42775,42776,42777,42778,42779,42780,42781,42782,42783,42786,42787,42788,42789,42790,42791,42792,42793,42794,42795,42796,42797,42798,42799,42800,42801,42802,42803,42804,42805,42806,42807,42808,42809,42810,42811,42812,42813,42814,42815,42816,42817,42818,42819,42820,42821,42822,42823,42824,42825,42826,42827,42828,42829,42830,42831,42832,42833,42834,42835,42836,42837,42838,42839,42840,42841,42842,42843,42844,42845,42846,42847,42848,42849,42850,42851,42852,42853,42854,42855,42856,42857,42858,42859,42860,42861,42862,42863,42864,42865,42866,42867,42868,42869,42870,42871,42872,42873,42874,42875,42876,42877,42878,42879,42880,42881,42882,42883,42884,42885,42886,42887,42888,42891,42892,42893,42894,42896,42897,42898,42899,42912,42913,42914,42915,42916,42917,42918,42919,42920,42921,42922,43000,43001,43002,43003,43004,43005,43006,43007,43008,43009,43011,43012,43013,43015,43016,43017,43018,43020,43021,43022,43023,43024,43025,43026,43027,43028,43029,43030,43031,43032,43033,43034,43035,43036,43037,43038,43039,43040,43041,43042,43072,43073,43074,43075,43076,43077,43078,43079,43080,43081,43082,43083,43084,43085,43086,43087,43088,43089,43090,43091,43092,43093,43094,43095,43096,43097,43098,43099,43100,43101,43102,43103,43104,43105,43106,43107,43108,43109,43110,43111,43112,43113,43114,43115,43116,43117,43118,43119,43120,43121,43122,43123,43138,43139,43140,43141,43142,43143,43144,43145,43146,43147,43148,43149,43150,43151,43152,43153,43154,43155,43156,43157,43158,43159,43160,43161,43162,43163,43164,43165,43166,43167,43168,43169,43170,43171,43172,43173,43174,43175,43176,43177,43178,43179,43180,43181,43182,43183,43184,43185,43186,43187,43250,43251,43252,43253,43254,43255,43259,43274,43275,43276,43277,43278,43279,43280,43281,43282,43283,43284,43285,43286,43287,43288,43289,43290,43291,43292,43293,43294,43295,43296,43297,43298,43299,43300,43301,43312,43313,43314,43315,43316,43317,43318,43319,43320,43321,43322,43323,43324,43325,43326,43327,43328,43329,43330,43331,43332,43333,43334,43360,43361,43362,43363,43364,43365,43366,43367,43368,43369,43370,43371,43372,43373,43374,43375,43376,43377,43378,43379,43380,43381,43382,43383,43384,43385,43386,43387,43388,43396,43397,43398,43399,43400,43401,43402,43403,43404,43405,43406,43407,43408,43409,43410,43411,43412,43413,43414,43415,43416,43417,43418,43419,43420,43421,43422,43423,43424,43425,43426,43427,43428,43429,43430,43431,43432,43433,43434,43435,43436,43437,43438,43439,43440,43441,43442,43471,43520,43521,43522,43523,43524,43525,43526,43527,43528,43529,43530,43531,43532,43533,43534,43535,43536,43537,43538,43539,43540,43541,43542,43543,43544,43545,43546,43547,43548,43549,43550,43551,43552,43553,43554,43555,43556,43557,43558,43559,43560,43584,43585,43586,43588,43589,43590,43591,43592,43593,43594,43595,43616,43617,43618,43619,43620,43621,43622,43623,43624,43625,43626,43627,43628,43629,43630,43631,43632,43633,43634,43635,43636,43637,43638,43642,43648,43649,43650,43651,43652,43653,43654,43655,43656,43657,43658,43659,43660,43661,43662,43663,43664,43665,43666,43667,43668,43669,43670,43671,43672,43673,43674,43675,43676,43677,43678,43679,43680,43681,43682,43683,43684,43685,43686,43687,43688,43689,43690,43691,43692,43693,43694,43695,43697,43701,43702,43705,43706,43707,43708,43709,43712,43714,43739,43740,43741,43744,43745,43746,43747,43748,43749,43750,43751,43752,43753,43754,43762,43763,43764,43777,43778,43779,43780,43781,43782,43785,43786,43787,43788,43789,43790,43793,43794,43795,43796,43797,43798,43808,43809,43810,43811,43812,43813,43814,43816,43817,43818,43819,43820,43821,43822,43968,43969,43970,43971,43972,43973,43974,43975,43976,43977,43978,43979,43980,43981,43982,43983,43984,43985,43986,43987,43988,43989,43990,43991,43992,43993,43994,43995,43996,43997,43998,43999,44000,44001,44002,44032,44033,44034,44035,44036,44037,44038,44039,44040,44041,44042,44043,44044,44045,44046,44047,44048,44049,44050,44051,44052,44053,44054,44055,44056,44057,44058,44059,44060,44061,44062,44063,44064,44065,44066,44067,44068,44069,44070,44071,44072,44073,44074,44075,44076,44077,44078,44079,44080,44081,44082,44083,44084,44085,44086,44087,44088,44089,44090,44091,44092,44093,44094,44095,44096,44097,44098,44099,44100,44101,44102,44103,44104,44105,44106,44107,44108,44109,44110,44111,44112,44113,44114,44115,44116,44117,44118,44119,44120,44121,44122,44123,44124,44125,44126,44127,44128,44129,44130,44131,44132,44133,44134,44135,44136,44137,44138,44139,44140,44141,44142,44143,44144,44145,44146,44147,44148,44149,44150,44151,44152,44153,44154,44155,44156,44157,44158,44159,44160,44161,44162,44163,44164,44165,44166,44167,44168,44169,44170,44171,44172,44173,44174,44175,44176,44177,44178,44179,44180,44181,44182,44183,44184,44185,44186,44187,44188,44189,44190,44191,44192,44193,44194,44195,44196,44197,44198,44199,44200,44201,44202,44203,44204,44205,44206,44207,44208,44209,44210,44211,44212,44213,44214,44215,44216,44217,44218,44219,44220,44221,44222,44223,44224,44225,44226,44227,44228,44229,44230,44231,44232,44233,44234,44235,44236,44237,44238,44239,44240,44241,44242,44243,44244,44245,44246,44247,44248,44249,44250,44251,44252,44253,44254,44255,44256,44257,44258,44259,44260,44261,44262,44263,44264,44265,44266,44267,44268,44269,44270,44271,44272,44273,44274,44275,44276,44277,44278,44279,44280,44281,44282,44283,44284,44285,44286,44287,44288,44289,44290,44291,44292,44293,44294,44295,44296,44297,44298,44299,44300,44301,44302,44303,44304,44305,44306,44307,44308,44309,44310,44311,44312,44313,44314,44315,44316,44317,44318,44319,44320,44321,44322,44323,44324,44325,44326,44327,44328,44329,44330,44331,44332,44333,44334,44335,44336,44337,44338,44339,44340,44341,44342,44343,44344,44345,44346,44347,44348,44349,44350,44351,44352,44353,44354,44355,44356,44357,44358,44359,44360,44361,44362,44363,44364,44365,44366,44367,44368,44369,44370,44371,44372,44373,44374,44375,44376,44377,44378,44379,44380,44381,44382,44383,44384,44385,44386,44387,44388,44389,44390,44391,44392,44393,44394,44395,44396,44397,44398,44399,44400,44401,44402,44403,44404,44405,44406,44407,44408,44409,44410,44411,44412,44413,44414,44415,44416,44417,44418,44419,44420,44421,44422,44423,44424,44425,44426,44427,44428,44429,44430,44431,44432,44433,44434,44435,44436,44437,44438,44439,44440,44441,44442,44443,44444,44445,44446,44447,44448,44449,44450,44451,44452,44453,44454,44455,44456,44457,44458,44459,44460,44461,44462,44463,44464,44465,44466,44467,44468,44469,44470,44471,44472,44473,44474,44475,44476,44477,44478,44479,44480,44481,44482,44483,44484,44485,44486,44487,44488,44489,44490,44491,44492,44493,44494,44495,44496,44497,44498,44499,44500,44501,44502,44503,44504,44505,44506,44507,44508,44509,44510,44511,44512,44513,44514,44515,44516,44517,44518,44519,44520,44521,44522,44523,44524,44525,44526,44527,44528,44529,44530,44531,44532,44533,44534,44535,44536,44537,44538,44539,44540,44541,44542,44543,44544,44545,44546,44547,44548,44549,44550,44551,44552,44553,44554,44555,44556,44557,44558,44559,44560,44561,44562,44563,44564,44565,44566,44567,44568,44569,44570,44571,44572,44573,44574,44575,44576,44577,44578,44579,44580,44581,44582,44583,44584,44585,44586,44587,44588,44589,44590,44591,44592,44593,44594,44595,44596,44597,44598,44599,44600,44601,44602,44603,44604,44605,44606,44607,44608,44609,44610,44611,44612,44613,44614,44615,44616,44617,44618,44619,44620,44621,44622,44623,44624,44625,44626,44627,44628,44629,44630,44631,44632,44633,44634,44635,44636,44637,44638,44639,44640,44641,44642,44643,44644,44645,44646,44647,44648,44649,44650,44651,44652,44653,44654,44655,44656,44657,44658,44659,44660,44661,44662,44663,44664,44665,44666,44667,44668,44669,44670,44671,44672,44673,44674,44675,44676,44677,44678,44679,44680,44681,44682,44683,44684,44685,44686,44687,44688,44689,44690,44691,44692,44693,44694,44695,44696,44697,44698,44699,44700,44701,44702,44703,44704,44705,44706,44707,44708,44709,44710,44711,44712,44713,44714,44715,44716,44717,44718,44719,44720,44721,44722,44723,44724,44725,44726,44727,44728,44729,44730,44731,44732,44733,44734,44735,44736,44737,44738,44739,44740,44741,44742,44743,44744,44745,44746,44747,44748,44749,44750,44751,44752,44753,44754,44755,44756,44757,44758,44759,44760,44761,44762,44763,44764,44765,44766,44767,44768,44769,44770,44771,44772,44773,44774,44775,44776,44777,44778,44779,44780,44781,44782,44783,44784,44785,44786,44787,44788,44789,44790,44791,44792,44793,44794,44795,44796,44797,44798,44799,44800,44801,44802,44803,44804,44805,44806,44807,44808,44809,44810,44811,44812,44813,44814,44815,44816,44817,44818,44819,44820,44821,44822,44823,44824,44825,44826,44827,44828,44829,44830,44831,44832,44833,44834,44835,44836,44837,44838,44839,44840,44841,44842,44843,44844,44845,44846,44847,44848,44849,44850,44851,44852,44853,44854,44855,44856,44857,44858,44859,44860,44861,44862,44863,44864,44865,44866,44867,44868,44869,44870,44871,44872,44873,44874,44875,44876,44877,44878,44879,44880,44881,44882,44883,44884,44885,44886,44887,44888,44889,44890,44891,44892,44893,44894,44895,44896,44897,44898,44899,44900,44901,44902,44903,44904,44905,44906,44907,44908,44909,44910,44911,44912,44913,44914,44915,44916,44917,44918,44919,44920,44921,44922,44923,44924,44925,44926,44927,44928,44929,44930,44931,44932,44933,44934,44935,44936,44937,44938,44939,44940,44941,44942,44943,44944,44945,44946,44947,44948,44949,44950,44951,44952,44953,44954,44955,44956,44957,44958,44959,44960,44961,44962,44963,44964,44965,44966,44967,44968,44969,44970,44971,44972,44973,44974,44975,44976,44977,44978,44979,44980,44981,44982,44983,44984,44985,44986,44987,44988,44989,44990,44991,44992,44993,44994,44995,44996,44997,44998,44999,45000,45001,45002,45003,45004,45005,45006,45007,45008,45009,45010,45011,45012,45013,45014,45015,45016,45017,45018,45019,45020,45021,45022,45023,45024,45025,45026,45027,45028,45029,45030,45031,45032,45033,45034,45035,45036,45037,45038,45039,45040,45041,45042,45043,45044,45045,45046,45047,45048,45049,45050,45051,45052,45053,45054,45055,45056,45057,45058,45059,45060,45061,45062,45063,45064,45065,45066,45067,45068,45069,45070,45071,45072,45073,45074,45075,45076,45077,45078,45079,45080,45081,45082,45083,45084,45085,45086,45087,45088,45089,45090,45091,45092,45093,45094,45095,45096,45097,45098,45099,45100,45101,45102,45103,45104,45105,45106,45107,45108,45109,45110,45111,45112,45113,45114,45115,45116,45117,45118,45119,45120,45121,45122,45123,45124,45125,45126,45127,45128,45129,45130,45131,45132,45133,45134,45135,45136,45137,45138,45139,45140,45141,45142,45143,45144,45145,45146,45147,45148,45149,45150,45151,45152,45153,45154,45155,45156,45157,45158,45159,45160,45161,45162,45163,45164,45165,45166,45167,45168,45169,45170,45171,45172,45173,45174,45175,45176,45177,45178,45179,45180,45181,45182,45183,45184,45185,45186,45187,45188,45189,45190,45191,45192,45193,45194,45195,45196,45197,45198,45199,45200,45201,45202,45203,45204,45205,45206,45207,45208,45209,45210,45211,45212,45213,45214,45215,45216,45217,45218,45219,45220,45221,45222,45223,45224,45225,45226,45227,45228,45229,45230,45231,45232,45233,45234,45235,45236,45237,45238,45239,45240,45241,45242,45243,45244,45245,45246,45247,45248,45249,45250,45251,45252,45253,45254,45255,45256,45257,45258,45259,45260,45261,45262,45263,45264,45265,45266,45267,45268,45269,45270,45271,45272,45273,45274,45275,45276,45277,45278,45279,45280,45281,45282,45283,45284,45285,45286,45287,45288,45289,45290,45291,45292,45293,45294,45295,45296,45297,45298,45299,45300,45301,45302,45303,45304,45305,45306,45307,45308,45309,45310,45311,45312,45313,45314,45315,45316,45317,45318,45319,45320,45321,45322,45323,45324,45325,45326,45327,45328,45329,45330,45331,45332,45333,45334,45335,45336,45337,45338,45339,45340,45341,45342,45343,45344,45345,45346,45347,45348,45349,45350,45351,45352,45353,45354,45355,45356,45357,45358,45359,45360,45361,45362,45363,45364,45365,45366,45367,45368,45369,45370,45371,45372,45373,45374,45375,45376,45377,45378,45379,45380,45381,45382,45383,45384,45385,45386,45387,45388,45389,45390,45391,45392,45393,45394,45395,45396,45397,45398,45399,45400,45401,45402,45403,45404,45405,45406,45407,45408,45409,45410,45411,45412,45413,45414,45415,45416,45417,45418,45419,45420,45421,45422,45423,45424,45425,45426,45427,45428,45429,45430,45431,45432,45433,45434,45435,45436,45437,45438,45439,45440,45441,45442,45443,45444,45445,45446,45447,45448,45449,45450,45451,45452,45453,45454,45455,45456,45457,45458,45459,45460,45461,45462,45463,45464,45465,45466,45467,45468,45469,45470,45471,45472,45473,45474,45475,45476,45477,45478,45479,45480,45481,45482,45483,45484,45485,45486,45487,45488,45489,45490,45491,45492,45493,45494,45495,45496,45497,45498,45499,45500,45501,45502,45503,45504,45505,45506,45507,45508,45509,45510,45511,45512,45513,45514,45515,45516,45517,45518,45519,45520,45521,45522,45523,45524,45525,45526,45527,45528,45529,45530,45531,45532,45533,45534,45535,45536,45537,45538,45539,45540,45541,45542,45543,45544,45545,45546,45547,45548,45549,45550,45551,45552,45553,45554,45555,45556,45557,45558,45559,45560,45561,45562,45563,45564,45565,45566,45567,45568,45569,45570,45571,45572,45573,45574,45575,45576,45577,45578,45579,45580,45581,45582,45583,45584,45585,45586,45587,45588,45589,45590,45591,45592,45593,45594,45595,45596,45597,45598,45599,45600,45601,45602,45603,45604,45605,45606,45607,45608,45609,45610,45611,45612,45613,45614,45615,45616,45617,45618,45619,45620,45621,45622,45623,45624,45625,45626,45627,45628,45629,45630,45631,45632,45633,45634,45635,45636,45637,45638,45639,45640,45641,45642,45643,45644,45645,45646,45647,45648,45649,45650,45651,45652,45653,45654,45655,45656,45657,45658,45659,45660,45661,45662,45663,45664,45665,45666,45667,45668,45669,45670,45671,45672,45673,45674,45675,45676,45677,45678,45679,45680,45681,45682,45683,45684,45685,45686,45687,45688,45689,45690,45691,45692,45693,45694,45695,45696,45697,45698,45699,45700,45701,45702,45703,45704,45705,45706,45707,45708,45709,45710,45711,45712,45713,45714,45715,45716,45717,45718,45719,45720,45721,45722,45723,45724,45725,45726,45727,45728,45729,45730,45731,45732,45733,45734,45735,45736,45737,45738,45739,45740,45741,45742,45743,45744,45745,45746,45747,45748,45749,45750,45751,45752,45753,45754,45755,45756,45757,45758,45759,45760,45761,45762,45763,45764,45765,45766,45767,45768,45769,45770,45771,45772,45773,45774,45775,45776,45777,45778,45779,45780,45781,45782,45783,45784,45785,45786,45787,45788,45789,45790,45791,45792,45793,45794,45795,45796,45797,45798,45799,45800,45801,45802,45803,45804,45805,45806,45807,45808,45809,45810,45811,45812,45813,45814,45815,45816,45817,45818,45819,45820,45821,45822,45823,45824,45825,45826,45827,45828,45829,45830,45831,45832,45833,45834,45835,45836,45837,45838,45839,45840,45841,45842,45843,45844,45845,45846,45847,45848,45849,45850,45851,45852,45853,45854,45855,45856,45857,45858,45859,45860,45861,45862,45863,45864,45865,45866,45867,45868,45869,45870,45871,45872,45873,45874,45875,45876,45877,45878,45879,45880,45881,45882,45883,45884,45885,45886,45887,45888,45889,45890,45891,45892,45893,45894,45895,45896,45897,45898,45899,45900,45901,45902,45903,45904,45905,45906,45907,45908,45909,45910,45911,45912,45913,45914,45915,45916,45917,45918,45919,45920,45921,45922,45923,45924,45925,45926,45927,45928,45929,45930,45931,45932,45933,45934,45935,45936,45937,45938,45939,45940,45941,45942,45943,45944,45945,45946,45947,45948,45949,45950,45951,45952,45953,45954,45955,45956,45957,45958,45959,45960,45961,45962,45963,45964,45965,45966,45967,45968,45969,45970,45971,45972,45973,45974,45975,45976,45977,45978,45979,45980,45981,45982,45983,45984,45985,45986,45987,45988,45989,45990,45991,45992,45993,45994,45995,45996,45997,45998,45999,46000,46001,46002,46003,46004,46005,46006,46007,46008,46009,46010,46011,46012,46013,46014,46015,46016,46017,46018,46019,46020,46021,46022,46023,46024,46025,46026,46027,46028,46029,46030,46031,46032,46033,46034,46035,46036,46037,46038,46039,46040,46041,46042,46043,46044,46045,46046,46047,46048,46049,46050,46051,46052,46053,46054,46055,46056,46057,46058,46059,46060,46061,46062,46063,46064,46065,46066,46067,46068,46069,46070,46071,46072,46073,46074,46075,46076,46077,46078,46079,46080,46081,46082,46083,46084,46085,46086,46087,46088,46089,46090,46091,46092,46093,46094,46095,46096,46097,46098,46099,46100,46101,46102,46103,46104,46105,46106,46107,46108,46109,46110,46111,46112,46113,46114,46115,46116,46117,46118,46119,46120,46121,46122,46123,46124,46125,46126,46127,46128,46129,46130,46131,46132,46133,46134,46135,46136,46137,46138,46139,46140,46141,46142,46143,46144,46145,46146,46147,46148,46149,46150,46151,46152,46153,46154,46155,46156,46157,46158,46159,46160,46161,46162,46163,46164,46165,46166,46167,46168,46169,46170,46171,46172,46173,46174,46175,46176,46177,46178,46179,46180,46181,46182,46183,46184,46185,46186,46187,46188,46189,46190,46191,46192,46193,46194,46195,46196,46197,46198,46199,46200,46201,46202,46203,46204,46205,46206,46207,46208,46209,46210,46211,46212,46213,46214,46215,46216,46217,46218,46219,46220,46221,46222,46223,46224,46225,46226,46227,46228,46229,46230,46231,46232,46233,46234,46235,46236,46237,46238,46239,46240,46241,46242,46243,46244,46245,46246,46247,46248,46249,46250,46251,46252,46253,46254,46255,46256,46257,46258,46259,46260,46261,46262,46263,46264,46265,46266,46267,46268,46269,46270,46271,46272,46273,46274,46275,46276,46277,46278,46279,46280,46281,46282,46283,46284,46285,46286,46287,46288,46289,46290,46291,46292,46293,46294,46295,46296,46297,46298,46299,46300,46301,46302,46303,46304,46305,46306,46307,46308,46309,46310,46311,46312,46313,46314,46315,46316,46317,46318,46319,46320,46321,46322,46323,46324,46325,46326,46327,46328,46329,46330,46331,46332,46333,46334,46335,46336,46337,46338,46339,46340,46341,46342,46343,46344,46345,46346,46347,46348,46349,46350,46351,46352,46353,46354,46355,46356,46357,46358,46359,46360,46361,46362,46363,46364,46365,46366,46367,46368,46369,46370,46371,46372,46373,46374,46375,46376,46377,46378,46379,46380,46381,46382,46383,46384,46385,46386,46387,46388,46389,46390,46391,46392,46393,46394,46395,46396,46397,46398,46399,46400,46401,46402,46403,46404,46405,46406,46407,46408,46409,46410,46411,46412,46413,46414,46415,46416,46417,46418,46419,46420,46421,46422,46423,46424,46425,46426,46427,46428,46429,46430,46431,46432,46433,46434,46435,46436,46437,46438,46439,46440,46441,46442,46443,46444,46445,46446,46447,46448,46449,46450,46451,46452,46453,46454,46455,46456,46457,46458,46459,46460,46461,46462,46463,46464,46465,46466,46467,46468,46469,46470,46471,46472,46473,46474,46475,46476,46477,46478,46479,46480,46481,46482,46483,46484,46485,46486,46487,46488,46489,46490,46491,46492,46493,46494,46495,46496,46497,46498,46499,46500,46501,46502,46503,46504,46505,46506,46507,46508,46509,46510,46511,46512,46513,46514,46515,46516,46517,46518,46519,46520,46521,46522,46523,46524,46525,46526,46527,46528,46529,46530,46531,46532,46533,46534,46535,46536,46537,46538,46539,46540,46541,46542,46543,46544,46545,46546,46547,46548,46549,46550,46551,46552,46553,46554,46555,46556,46557,46558,46559,46560,46561,46562,46563,46564,46565,46566,46567,46568,46569,46570,46571,46572,46573,46574,46575,46576,46577,46578,46579,46580,46581,46582,46583,46584,46585,46586,46587,46588,46589,46590,46591,46592,46593,46594,46595,46596,46597,46598,46599,46600,46601,46602,46603,46604,46605,46606,46607,46608,46609,46610,46611,46612,46613,46614,46615,46616,46617,46618,46619,46620,46621,46622,46623,46624,46625,46626,46627,46628,46629,46630,46631,46632,46633,46634,46635,46636,46637,46638,46639,46640,46641,46642,46643,46644,46645,46646,46647,46648,46649,46650,46651,46652,46653,46654,46655,46656,46657,46658,46659,46660,46661,46662,46663,46664,46665,46666,46667,46668,46669,46670,46671,46672,46673,46674,46675,46676,46677,46678,46679,46680,46681,46682,46683,46684,46685,46686,46687,46688,46689,46690,46691,46692,46693,46694,46695,46696,46697,46698,46699,46700,46701,46702,46703,46704,46705,46706,46707,46708,46709,46710,46711,46712,46713,46714,46715,46716,46717,46718,46719,46720,46721,46722,46723,46724,46725,46726,46727,46728,46729,46730,46731,46732,46733,46734,46735,46736,46737,46738,46739,46740,46741,46742,46743,46744,46745,46746,46747,46748,46749,46750,46751,46752,46753,46754,46755,46756,46757,46758,46759,46760,46761,46762,46763,46764,46765,46766,46767,46768,46769,46770,46771,46772,46773,46774,46775,46776,46777,46778,46779,46780,46781,46782,46783,46784,46785,46786,46787,46788,46789,46790,46791,46792,46793,46794,46795,46796,46797,46798,46799,46800,46801,46802,46803,46804,46805,46806,46807,46808,46809,46810,46811,46812,46813,46814,46815,46816,46817,46818,46819,46820,46821,46822,46823,46824,46825,46826,46827,46828,46829,46830,46831,46832,46833,46834,46835,46836,46837,46838,46839,46840,46841,46842,46843,46844,46845,46846,46847,46848,46849,46850,46851,46852,46853,46854,46855,46856,46857,46858,46859,46860,46861,46862,46863,46864,46865,46866,46867,46868,46869,46870,46871,46872,46873,46874,46875,46876,46877,46878,46879,46880,46881,46882,46883,46884,46885,46886,46887,46888,46889,46890,46891,46892,46893,46894,46895,46896,46897,46898,46899,46900,46901,46902,46903,46904,46905,46906,46907,46908,46909,46910,46911,46912,46913,46914,46915,46916,46917,46918,46919,46920,46921,46922,46923,46924,46925,46926,46927,46928,46929,46930,46931,46932,46933,46934,46935,46936,46937,46938,46939,46940,46941,46942,46943,46944,46945,46946,46947,46948,46949,46950,46951,46952,46953,46954,46955,46956,46957,46958,46959,46960,46961,46962,46963,46964,46965,46966,46967,46968,46969,46970,46971,46972,46973,46974,46975,46976,46977,46978,46979,46980,46981,46982,46983,46984,46985,46986,46987,46988,46989,46990,46991,46992,46993,46994,46995,46996,46997,46998,46999,47000,47001,47002,47003,47004,47005,47006,47007,47008,47009,47010,47011,47012,47013,47014,47015,47016,47017,47018,47019,47020,47021,47022,47023,47024,47025,47026,47027,47028,47029,47030,47031,47032,47033,47034,47035,47036,47037,47038,47039,47040,47041,47042,47043,47044,47045,47046,47047,47048,47049,47050,47051,47052,47053,47054,47055,47056,47057,47058,47059,47060,47061,47062,47063,47064,47065,47066,47067,47068,47069,47070,47071,47072,47073,47074,47075,47076,47077,47078,47079,47080,47081,47082,47083,47084,47085,47086,47087,47088,47089,47090,47091,47092,47093,47094,47095,47096,47097,47098,47099,47100,47101,47102,47103,47104,47105,47106,47107,47108,47109,47110,47111,47112,47113,47114,47115,47116,47117,47118,47119,47120,47121,47122,47123,47124,47125,47126,47127,47128,47129,47130,47131,47132,47133,47134,47135,47136,47137,47138,47139,47140,47141,47142,47143,47144,47145,47146,47147,47148,47149,47150,47151,47152,47153,47154,47155,47156,47157,47158,47159,47160,47161,47162,47163,47164,47165,47166,47167,47168,47169,47170,47171,47172,47173,47174,47175,47176,47177,47178,47179,47180,47181,47182,47183,47184,47185,47186,47187,47188,47189,47190,47191,47192,47193,47194,47195,47196,47197,47198,47199,47200,47201,47202,47203,47204,47205,47206,47207,47208,47209,47210,47211,47212,47213,47214,47215,47216,47217,47218,47219,47220,47221,47222,47223,47224,47225,47226,47227,47228,47229,47230,47231,47232,47233,47234,47235,47236,47237,47238,47239,47240,47241,47242,47243,47244,47245,47246,47247,47248,47249,47250,47251,47252,47253,47254,47255,47256,47257,47258,47259,47260,47261,47262,47263,47264,47265,47266,47267,47268,47269,47270,47271,47272,47273,47274,47275,47276,47277,47278,47279,47280,47281,47282,47283,47284,47285,47286,47287,47288,47289,47290,47291,47292,47293,47294,47295,47296,47297,47298,47299,47300,47301,47302,47303,47304,47305,47306,47307,47308,47309,47310,47311,47312,47313,47314,47315,47316,47317,47318,47319,47320,47321,47322,47323,47324,47325,47326,47327,47328,47329,47330,47331,47332,47333,47334,47335,47336,47337,47338,47339,47340,47341,47342,47343,47344,47345,47346,47347,47348,47349,47350,47351,47352,47353,47354,47355,47356,47357,47358,47359,47360,47361,47362,47363,47364,47365,47366,47367,47368,47369,47370,47371,47372,47373,47374,47375,47376,47377,47378,47379,47380,47381,47382,47383,47384,47385,47386,47387,47388,47389,47390,47391,47392,47393,47394,47395,47396,47397,47398,47399,47400,47401,47402,47403,47404,47405,47406,47407,47408,47409,47410,47411,47412,47413,47414,47415,47416,47417,47418,47419,47420,47421,47422,47423,47424,47425,47426,47427,47428,47429,47430,47431,47432,47433,47434,47435,47436,47437,47438,47439,47440,47441,47442,47443,47444,47445,47446,47447,47448,47449,47450,47451,47452,47453,47454,47455,47456,47457,47458,47459,47460,47461,47462,47463,47464,47465,47466,47467,47468,47469,47470,47471,47472,47473,47474,47475,47476,47477,47478,47479,47480,47481,47482,47483,47484,47485,47486,47487,47488,47489,47490,47491,47492,47493,47494,47495,47496,47497,47498,47499,47500,47501,47502,47503,47504,47505,47506,47507,47508,47509,47510,47511,47512,47513,47514,47515,47516,47517,47518,47519,47520,47521,47522,47523,47524,47525,47526,47527,47528,47529,47530,47531,47532,47533,47534,47535,47536,47537,47538,47539,47540,47541,47542,47543,47544,47545,47546,47547,47548,47549,47550,47551,47552,47553,47554,47555,47556,47557,47558,47559,47560,47561,47562,47563,47564,47565,47566,47567,47568,47569,47570,47571,47572,47573,47574,47575,47576,47577,47578,47579,47580,47581,47582,47583,47584,47585,47586,47587,47588,47589,47590,47591,47592,47593,47594,47595,47596,47597,47598,47599,47600,47601,47602,47603,47604,47605,47606,47607,47608,47609,47610,47611,47612,47613,47614,47615,47616,47617,47618,47619,47620,47621,47622,47623,47624,47625,47626,47627,47628,47629,47630,47631,47632,47633,47634,47635,47636,47637,47638,47639,47640,47641,47642,47643,47644,47645,47646,47647,47648,47649,47650,47651,47652,47653,47654,47655,47656,47657,47658,47659,47660,47661,47662,47663,47664,47665,47666,47667,47668,47669,47670,47671,47672,47673,47674,47675,47676,47677,47678,47679,47680,47681,47682,47683,47684,47685,47686,47687,47688,47689,47690,47691,47692,47693,47694,47695,47696,47697,47698,47699,47700,47701,47702,47703,47704,47705,47706,47707,47708,47709,47710,47711,47712,47713,47714,47715,47716,47717,47718,47719,47720,47721,47722,47723,47724,47725,47726,47727,47728,47729,47730,47731,47732,47733,47734,47735,47736,47737,47738,47739,47740,47741,47742,47743,47744,47745,47746,47747,47748,47749,47750,47751,47752,47753,47754,47755,47756,47757,47758,47759,47760,47761,47762,47763,47764,47765,47766,47767,47768,47769,47770,47771,47772,47773,47774,47775,47776,47777,47778,47779,47780,47781,47782,47783,47784,47785,47786,47787,47788,47789,47790,47791,47792,47793,47794,47795,47796,47797,47798,47799,47800,47801,47802,47803,47804,47805,47806,47807,47808,47809,47810,47811,47812,47813,47814,47815,47816,47817,47818,47819,47820,47821,47822,47823,47824,47825,47826,47827,47828,47829,47830,47831,47832,47833,47834,47835,47836,47837,47838,47839,47840,47841,47842,47843,47844,47845,47846,47847,47848,47849,47850,47851,47852,47853,47854,47855,47856,47857,47858,47859,47860,47861,47862,47863,47864,47865,47866,47867,47868,47869,47870,47871,47872,47873,47874,47875,47876,47877,47878,47879,47880,47881,47882,47883,47884,47885,47886,47887,47888,47889,47890,47891,47892,47893,47894,47895,47896,47897,47898,47899,47900,47901,47902,47903,47904,47905,47906,47907,47908,47909,47910,47911,47912,47913,47914,47915,47916,47917,47918,47919,47920,47921,47922,47923,47924,47925,47926,47927,47928,47929,47930,47931,47932,47933,47934,47935,47936,47937,47938,47939,47940,47941,47942,47943,47944,47945,47946,47947,47948,47949,47950,47951,47952,47953,47954,47955,47956,47957,47958,47959,47960,47961,47962,47963,47964,47965,47966,47967,47968,47969,47970,47971,47972,47973,47974,47975,47976,47977,47978,47979,47980,47981,47982,47983,47984,47985,47986,47987,47988,47989,47990,47991,47992,47993,47994,47995,47996,47997,47998,47999,48000,48001,48002,48003,48004,48005,48006,48007,48008,48009,48010,48011,48012,48013,48014,48015,48016,48017,48018,48019,48020,48021,48022,48023,48024,48025,48026,48027,48028,48029,48030,48031,48032,48033,48034,48035,48036,48037,48038,48039,48040,48041,48042,48043,48044,48045,48046,48047,48048,48049,48050,48051,48052,48053,48054,48055,48056,48057,48058,48059,48060,48061,48062,48063,48064,48065,48066,48067,48068,48069,48070,48071,48072,48073,48074,48075,48076,48077,48078,48079,48080,48081,48082,48083,48084,48085,48086,48087,48088,48089,48090,48091,48092,48093,48094,48095,48096,48097,48098,48099,48100,48101,48102,48103,48104,48105,48106,48107,48108,48109,48110,48111,48112,48113,48114,48115,48116,48117,48118,48119,48120,48121,48122,48123,48124,48125,48126,48127,48128,48129,48130,48131,48132,48133,48134,48135,48136,48137,48138,48139,48140,48141,48142,48143,48144,48145,48146,48147,48148,48149,48150,48151,48152,48153,48154,48155,48156,48157,48158,48159,48160,48161,48162,48163,48164,48165,48166,48167,48168,48169,48170,48171,48172,48173,48174,48175,48176,48177,48178,48179,48180,48181,48182,48183,48184,48185,48186,48187,48188,48189,48190,48191,48192,48193,48194,48195,48196,48197,48198,48199,48200,48201,48202,48203,48204,48205,48206,48207,48208,48209,48210,48211,48212,48213,48214,48215,48216,48217,48218,48219,48220,48221,48222,48223,48224,48225,48226,48227,48228,48229,48230,48231,48232,48233,48234,48235,48236,48237,48238,48239,48240,48241,48242,48243,48244,48245,48246,48247,48248,48249,48250,48251,48252,48253,48254,48255,48256,48257,48258,48259,48260,48261,48262,48263,48264,48265,48266,48267,48268,48269,48270,48271,48272,48273,48274,48275,48276,48277,48278,48279,48280,48281,48282,48283,48284,48285,48286,48287,48288,48289,48290,48291,48292,48293,48294,48295,48296,48297,48298,48299,48300,48301,48302,48303,48304,48305,48306,48307,48308,48309,48310,48311,48312,48313,48314,48315,48316,48317,48318,48319,48320,48321,48322,48323,48324,48325,48326,48327,48328,48329,48330,48331,48332,48333,48334,48335,48336,48337,48338,48339,48340,48341,48342,48343,48344,48345,48346,48347,48348,48349,48350,48351,48352,48353,48354,48355,48356,48357,48358,48359,48360,48361,48362,48363,48364,48365,48366,48367,48368,48369,48370,48371,48372,48373,48374,48375,48376,48377,48378,48379,48380,48381,48382,48383,48384,48385,48386,48387,48388,48389,48390,48391,48392,48393,48394,48395,48396,48397,48398,48399,48400,48401,48402,48403,48404,48405,48406,48407,48408,48409,48410,48411,48412,48413,48414,48415,48416,48417,48418,48419,48420,48421,48422,48423,48424,48425,48426,48427,48428,48429,48430,48431,48432,48433,48434,48435,48436,48437,48438,48439,48440,48441,48442,48443,48444,48445,48446,48447,48448,48449,48450,48451,48452,48453,48454,48455,48456,48457,48458,48459,48460,48461,48462,48463,48464,48465,48466,48467,48468,48469,48470,48471,48472,48473,48474,48475,48476,48477,48478,48479,48480,48481,48482,48483,48484,48485,48486,48487,48488,48489,48490,48491,48492,48493,48494,48495,48496,48497,48498,48499,48500,48501,48502,48503,48504,48505,48506,48507,48508,48509,48510,48511,48512,48513,48514,48515,48516,48517,48518,48519,48520,48521,48522,48523,48524,48525,48526,48527,48528,48529,48530,48531,48532,48533,48534,48535,48536,48537,48538,48539,48540,48541,48542,48543,48544,48545,48546,48547,48548,48549,48550,48551,48552,48553,48554,48555,48556,48557,48558,48559,48560,48561,48562,48563,48564,48565,48566,48567,48568,48569,48570,48571,48572,48573,48574,48575,48576,48577,48578,48579,48580,48581,48582,48583,48584,48585,48586,48587,48588,48589,48590,48591,48592,48593,48594,48595,48596,48597,48598,48599,48600,48601,48602,48603,48604,48605,48606,48607,48608,48609,48610,48611,48612,48613,48614,48615,48616,48617,48618,48619,48620,48621,48622,48623,48624,48625,48626,48627,48628,48629,48630,48631,48632,48633,48634,48635,48636,48637,48638,48639,48640,48641,48642,48643,48644,48645,48646,48647,48648,48649,48650,48651,48652,48653,48654,48655,48656,48657,48658,48659,48660,48661,48662,48663,48664,48665,48666,48667,48668,48669,48670,48671,48672,48673,48674,48675,48676,48677,48678,48679,48680,48681,48682,48683,48684,48685,48686,48687,48688,48689,48690,48691,48692,48693,48694,48695,48696,48697,48698,48699,48700,48701,48702,48703,48704,48705,48706,48707,48708,48709,48710,48711,48712,48713,48714,48715,48716,48717,48718,48719,48720,48721,48722,48723,48724,48725,48726,48727,48728,48729,48730,48731,48732,48733,48734,48735,48736,48737,48738,48739,48740,48741,48742,48743,48744,48745,48746,48747,48748,48749,48750,48751,48752,48753,48754,48755,48756,48757,48758,48759,48760,48761,48762,48763,48764,48765,48766,48767,48768,48769,48770,48771,48772,48773,48774,48775,48776,48777,48778,48779,48780,48781,48782,48783,48784,48785,48786,48787,48788,48789,48790,48791,48792,48793,48794,48795,48796,48797,48798,48799,48800,48801,48802,48803,48804,48805,48806,48807,48808,48809,48810,48811,48812,48813,48814,48815,48816,48817,48818,48819,48820,48821,48822,48823,48824,48825,48826,48827,48828,48829,48830,48831,48832,48833,48834,48835,48836,48837,48838,48839,48840,48841,48842,48843,48844,48845,48846,48847,48848,48849,48850,48851,48852,48853,48854,48855,48856,48857,48858,48859,48860,48861,48862,48863,48864,48865,48866,48867,48868,48869,48870,48871,48872,48873,48874,48875,48876,48877,48878,48879,48880,48881,48882,48883,48884,48885,48886,48887,48888,48889,48890,48891,48892,48893,48894,48895,48896,48897,48898,48899,48900,48901,48902,48903,48904,48905,48906,48907,48908,48909,48910,48911,48912,48913,48914,48915,48916,48917,48918,48919,48920,48921,48922,48923,48924,48925,48926,48927,48928,48929,48930,48931,48932,48933,48934,48935,48936,48937,48938,48939,48940,48941,48942,48943,48944,48945,48946,48947,48948,48949,48950,48951,48952,48953,48954,48955,48956,48957,48958,48959,48960,48961,48962,48963,48964,48965,48966,48967,48968,48969,48970,48971,48972,48973,48974,48975,48976,48977,48978,48979,48980,48981,48982,48983,48984,48985,48986,48987,48988,48989,48990,48991,48992,48993,48994,48995,48996,48997,48998,48999,49000,49001,49002,49003,49004,49005,49006,49007,49008,49009,49010,49011,49012,49013,49014,49015,49016,49017,49018,49019,49020,49021,49022,49023,49024,49025,49026,49027,49028,49029,49030,49031,49032,49033,49034,49035,49036,49037,49038,49039,49040,49041,49042,49043,49044,49045,49046,49047,49048,49049,49050,49051,49052,49053,49054,49055,49056,49057,49058,49059,49060,49061,49062,49063,49064,49065,49066,49067,49068,49069,49070,49071,49072,49073,49074,49075,49076,49077,49078,49079,49080,49081,49082,49083,49084,49085,49086,49087,49088,49089,49090,49091,49092,49093,49094,49095,49096,49097,49098,49099,49100,49101,49102,49103,49104,49105,49106,49107,49108,49109,49110,49111,49112,49113,49114,49115,49116,49117,49118,49119,49120,49121,49122,49123,49124,49125,49126,49127,49128,49129,49130,49131,49132,49133,49134,49135,49136,49137,49138,49139,49140,49141,49142,49143,49144,49145,49146,49147,49148,49149,49150,49151,49152,49153,49154,49155,49156,49157,49158,49159,49160,49161,49162,49163,49164,49165,49166,49167,49168,49169,49170,49171,49172,49173,49174,49175,49176,49177,49178,49179,49180,49181,49182,49183,49184,49185,49186,49187,49188,49189,49190,49191,49192,49193,49194,49195,49196,49197,49198,49199,49200,49201,49202,49203,49204,49205,49206,49207,49208,49209,49210,49211,49212,49213,49214,49215,49216,49217,49218,49219,49220,49221,49222,49223,49224,49225,49226,49227,49228,49229,49230,49231,49232,49233,49234,49235,49236,49237,49238,49239,49240,49241,49242,49243,49244,49245,49246,49247,49248,49249,49250,49251,49252,49253,49254,49255,49256,49257,49258,49259,49260,49261,49262,49263,49264,49265,49266,49267,49268,49269,49270,49271,49272,49273,49274,49275,49276,49277,49278,49279,49280,49281,49282,49283,49284,49285,49286,49287,49288,49289,49290,49291,49292,49293,49294,49295,49296,49297,49298,49299,49300,49301,49302,49303,49304,49305,49306,49307,49308,49309,49310,49311,49312,49313,49314,49315,49316,49317,49318,49319,49320,49321,49322,49323,49324,49325,49326,49327,49328,49329,49330,49331,49332,49333,49334,49335,49336,49337,49338,49339,49340,49341,49342,49343,49344,49345,49346,49347,49348,49349,49350,49351,49352,49353,49354,49355,49356,49357,49358,49359,49360,49361,49362,49363,49364,49365,49366,49367,49368,49369,49370,49371,49372,49373,49374,49375,49376,49377,49378,49379,49380,49381,49382,49383,49384,49385,49386,49387,49388,49389,49390,49391,49392,49393,49394,49395,49396,49397,49398,49399,49400,49401,49402,49403,49404,49405,49406,49407,49408,49409,49410,49411,49412,49413,49414,49415,49416,49417,49418,49419,49420,49421,49422,49423,49424,49425,49426,49427,49428,49429,49430,49431,49432,49433,49434,49435,49436,49437,49438,49439,49440,49441,49442,49443,49444,49445,49446,49447,49448,49449,49450,49451,49452,49453,49454,49455,49456,49457,49458,49459,49460,49461,49462,49463,49464,49465,49466,49467,49468,49469,49470,49471,49472,49473,49474,49475,49476,49477,49478,49479,49480,49481,49482,49483,49484,49485,49486,49487,49488,49489,49490,49491,49492,49493,49494,49495,49496,49497,49498,49499,49500,49501,49502,49503,49504,49505,49506,49507,49508,49509,49510,49511,49512,49513,49514,49515,49516,49517,49518,49519,49520,49521,49522,49523,49524,49525,49526,49527,49528,49529,49530,49531,49532,49533,49534,49535,49536,49537,49538,49539,49540,49541,49542,49543,49544,49545,49546,49547,49548,49549,49550,49551,49552,49553,49554,49555,49556,49557,49558,49559,49560,49561,49562,49563,49564,49565,49566,49567,49568,49569,49570,49571,49572,49573,49574,49575,49576,49577,49578,49579,49580,49581,49582,49583,49584,49585,49586,49587,49588,49589,49590,49591,49592,49593,49594,49595,49596,49597,49598,49599,49600,49601,49602,49603,49604,49605,49606,49607,49608,49609,49610,49611,49612,49613,49614,49615,49616,49617,49618,49619,49620,49621,49622,49623,49624,49625,49626,49627,49628,49629,49630,49631,49632,49633,49634,49635,49636,49637,49638,49639,49640,49641,49642,49643,49644,49645,49646,49647,49648,49649,49650,49651,49652,49653,49654,49655,49656,49657,49658,49659,49660,49661,49662,49663,49664,49665,49666,49667,49668,49669,49670,49671,49672,49673,49674,49675,49676,49677,49678,49679,49680,49681,49682,49683,49684,49685,49686,49687,49688,49689,49690,49691,49692,49693,49694,49695,49696,49697,49698,49699,49700,49701,49702,49703,49704,49705,49706,49707,49708,49709,49710,49711,49712,49713,49714,49715,49716,49717,49718,49719,49720,49721,49722,49723,49724,49725,49726,49727,49728,49729,49730,49731,49732,49733,49734,49735,49736,49737,49738,49739,49740,49741,49742,49743,49744,49745,49746,49747,49748,49749,49750,49751,49752,49753,49754,49755,49756,49757,49758,49759,49760,49761,49762,49763,49764,49765,49766,49767,49768,49769,49770,49771,49772,49773,49774,49775,49776,49777,49778,49779,49780,49781,49782,49783,49784,49785,49786,49787,49788,49789,49790,49791,49792,49793,49794,49795,49796,49797,49798,49799,49800,49801,49802,49803,49804,49805,49806,49807,49808,49809,49810,49811,49812,49813,49814,49815,49816,49817,49818,49819,49820,49821,49822,49823,49824,49825,49826,49827,49828,49829,49830,49831,49832,49833,49834,49835,49836,49837,49838,49839,49840,49841,49842,49843,49844,49845,49846,49847,49848,49849,49850,49851,49852,49853,49854,49855,49856,49857,49858,49859,49860,49861,49862,49863,49864,49865,49866,49867,49868,49869,49870,49871,49872,49873,49874,49875,49876,49877,49878,49879,49880,49881,49882,49883,49884,49885,49886,49887,49888,49889,49890,49891,49892,49893,49894,49895,49896,49897,49898,49899,49900,49901,49902,49903,49904,49905,49906,49907,49908,49909,49910,49911,49912,49913,49914,49915,49916,49917,49918,49919,49920,49921,49922,49923,49924,49925,49926,49927,49928,49929,49930,49931,49932,49933,49934,49935,49936,49937,49938,49939,49940,49941,49942,49943,49944,49945,49946,49947,49948,49949,49950,49951,49952,49953,49954,49955,49956,49957,49958,49959,49960,49961,49962,49963,49964,49965,49966,49967,49968,49969,49970,49971,49972,49973,49974,49975,49976,49977,49978,49979,49980,49981,49982,49983,49984,49985,49986,49987,49988,49989,49990,49991,49992,49993,49994,49995,49996,49997,49998,49999,50000,50001,50002,50003,50004,50005,50006,50007,50008,50009,50010,50011,50012,50013,50014,50015,50016,50017,50018,50019,50020,50021,50022,50023,50024,50025,50026,50027,50028,50029,50030,50031,50032,50033,50034,50035,50036,50037,50038,50039,50040,50041,50042,50043,50044,50045,50046,50047,50048,50049,50050,50051,50052,50053,50054,50055,50056,50057,50058,50059,50060,50061,50062,50063,50064,50065,50066,50067,50068,50069,50070,50071,50072,50073,50074,50075,50076,50077,50078,50079,50080,50081,50082,50083,50084,50085,50086,50087,50088,50089,50090,50091,50092,50093,50094,50095,50096,50097,50098,50099,50100,50101,50102,50103,50104,50105,50106,50107,50108,50109,50110,50111,50112,50113,50114,50115,50116,50117,50118,50119,50120,50121,50122,50123,50124,50125,50126,50127,50128,50129,50130,50131,50132,50133,50134,50135,50136,50137,50138,50139,50140,50141,50142,50143,50144,50145,50146,50147,50148,50149,50150,50151,50152,50153,50154,50155,50156,50157,50158,50159,50160,50161,50162,50163,50164,50165,50166,50167,50168,50169,50170,50171,50172,50173,50174,50175,50176,50177,50178,50179,50180,50181,50182,50183,50184,50185,50186,50187,50188,50189,50190,50191,50192,50193,50194,50195,50196,50197,50198,50199,50200,50201,50202,50203,50204,50205,50206,50207,50208,50209,50210,50211,50212,50213,50214,50215,50216,50217,50218,50219,50220,50221,50222,50223,50224,50225,50226,50227,50228,50229,50230,50231,50232,50233,50234,50235,50236,50237,50238,50239,50240,50241,50242,50243,50244,50245,50246,50247,50248,50249,50250,50251,50252,50253,50254,50255,50256,50257,50258,50259,50260,50261,50262,50263,50264,50265,50266,50267,50268,50269,50270,50271,50272,50273,50274,50275,50276,50277,50278,50279,50280,50281,50282,50283,50284,50285,50286,50287,50288,50289,50290,50291,50292,50293,50294,50295,50296,50297,50298,50299,50300,50301,50302,50303,50304,50305,50306,50307,50308,50309,50310,50311,50312,50313,50314,50315,50316,50317,50318,50319,50320,50321,50322,50323,50324,50325,50326,50327,50328,50329,50330,50331,50332,50333,50334,50335,50336,50337,50338,50339,50340,50341,50342,50343,50344,50345,50346,50347,50348,50349,50350,50351,50352,50353,50354,50355,50356,50357,50358,50359,50360,50361,50362,50363,50364,50365,50366,50367,50368,50369,50370,50371,50372,50373,50374,50375,50376,50377,50378,50379,50380,50381,50382,50383,50384,50385,50386,50387,50388,50389,50390,50391,50392,50393,50394,50395,50396,50397,50398,50399,50400,50401,50402,50403,50404,50405,50406,50407,50408,50409,50410,50411,50412,50413,50414,50415,50416,50417,50418,50419,50420,50421,50422,50423,50424,50425,50426,50427,50428,50429,50430,50431,50432,50433,50434,50435,50436,50437,50438,50439,50440,50441,50442,50443,50444,50445,50446,50447,50448,50449,50450,50451,50452,50453,50454,50455,50456,50457,50458,50459,50460,50461,50462,50463,50464,50465,50466,50467,50468,50469,50470,50471,50472,50473,50474,50475,50476,50477,50478,50479,50480,50481,50482,50483,50484,50485,50486,50487,50488,50489,50490,50491,50492,50493,50494,50495,50496,50497,50498,50499,50500,50501,50502,50503,50504,50505,50506,50507,50508,50509,50510,50511,50512,50513,50514,50515,50516,50517,50518,50519,50520,50521,50522,50523,50524,50525,50526,50527,50528,50529,50530,50531,50532,50533,50534,50535,50536,50537,50538,50539,50540,50541,50542,50543,50544,50545,50546,50547,50548,50549,50550,50551,50552,50553,50554,50555,50556,50557,50558,50559,50560,50561,50562,50563,50564,50565,50566,50567,50568,50569,50570,50571,50572,50573,50574,50575,50576,50577,50578,50579,50580,50581,50582,50583,50584,50585,50586,50587,50588,50589,50590,50591,50592,50593,50594,50595,50596,50597,50598,50599,50600,50601,50602,50603,50604,50605,50606,50607,50608,50609,50610,50611,50612,50613,50614,50615,50616,50617,50618,50619,50620,50621,50622,50623,50624,50625,50626,50627,50628,50629,50630,50631,50632,50633,50634,50635,50636,50637,50638,50639,50640,50641,50642,50643,50644,50645,50646,50647,50648,50649,50650,50651,50652,50653,50654,50655,50656,50657,50658,50659,50660,50661,50662,50663,50664,50665,50666,50667,50668,50669,50670,50671,50672,50673,50674,50675,50676,50677,50678,50679,50680,50681,50682,50683,50684,50685,50686,50687,50688,50689,50690,50691,50692,50693,50694,50695,50696,50697,50698,50699,50700,50701,50702,50703,50704,50705,50706,50707,50708,50709,50710,50711,50712,50713,50714,50715,50716,50717,50718,50719,50720,50721,50722,50723,50724,50725,50726,50727,50728,50729,50730,50731,50732,50733,50734,50735,50736,50737,50738,50739,50740,50741,50742,50743,50744,50745,50746,50747,50748,50749,50750,50751,50752,50753,50754,50755,50756,50757,50758,50759,50760,50761,50762,50763,50764,50765,50766,50767,50768,50769,50770,50771,50772,50773,50774,50775,50776,50777,50778,50779,50780,50781,50782,50783,50784,50785,50786,50787,50788,50789,50790,50791,50792,50793,50794,50795,50796,50797,50798,50799,50800,50801,50802,50803,50804,50805,50806,50807,50808,50809,50810,50811,50812,50813,50814,50815,50816,50817,50818,50819,50820,50821,50822,50823,50824,50825,50826,50827,50828,50829,50830,50831,50832,50833,50834,50835,50836,50837,50838,50839,50840,50841,50842,50843,50844,50845,50846,50847,50848,50849,50850,50851,50852,50853,50854,50855,50856,50857,50858,50859,50860,50861,50862,50863,50864,50865,50866,50867,50868,50869,50870,50871,50872,50873,50874,50875,50876,50877,50878,50879,50880,50881,50882,50883,50884,50885,50886,50887,50888,50889,50890,50891,50892,50893,50894,50895,50896,50897,50898,50899,50900,50901,50902,50903,50904,50905,50906,50907,50908,50909,50910,50911,50912,50913,50914,50915,50916,50917,50918,50919,50920,50921,50922,50923,50924,50925,50926,50927,50928,50929,50930,50931,50932,50933,50934,50935,50936,50937,50938,50939,50940,50941,50942,50943,50944,50945,50946,50947,50948,50949,50950,50951,50952,50953,50954,50955,50956,50957,50958,50959,50960,50961,50962,50963,50964,50965,50966,50967,50968,50969,50970,50971,50972,50973,50974,50975,50976,50977,50978,50979,50980,50981,50982,50983,50984,50985,50986,50987,50988,50989,50990,50991,50992,50993,50994,50995,50996,50997,50998,50999,51000,51001,51002,51003,51004,51005,51006,51007,51008,51009,51010,51011,51012,51013,51014,51015,51016,51017,51018,51019,51020,51021,51022,51023,51024,51025,51026,51027,51028,51029,51030,51031,51032,51033,51034,51035,51036,51037,51038,51039,51040,51041,51042,51043,51044,51045,51046,51047,51048,51049,51050,51051,51052,51053,51054,51055,51056,51057,51058,51059,51060,51061,51062,51063,51064,51065,51066,51067,51068,51069,51070,51071,51072,51073,51074,51075,51076,51077,51078,51079,51080,51081,51082,51083,51084,51085,51086,51087,51088,51089,51090,51091,51092,51093,51094,51095,51096,51097,51098,51099,51100,51101,51102,51103,51104,51105,51106,51107,51108,51109,51110,51111,51112,51113,51114,51115,51116,51117,51118,51119,51120,51121,51122,51123,51124,51125,51126,51127,51128,51129,51130,51131,51132,51133,51134,51135,51136,51137,51138,51139,51140,51141,51142,51143,51144,51145,51146,51147,51148,51149,51150,51151,51152,51153,51154,51155,51156,51157,51158,51159,51160,51161,51162,51163,51164,51165,51166,51167,51168,51169,51170,51171,51172,51173,51174,51175,51176,51177,51178,51179,51180,51181,51182,51183,51184,51185,51186,51187,51188,51189,51190,51191,51192,51193,51194,51195,51196,51197,51198,51199,51200,51201,51202,51203,51204,51205,51206,51207,51208,51209,51210,51211,51212,51213,51214,51215,51216,51217,51218,51219,51220,51221,51222,51223,51224,51225,51226,51227,51228,51229,51230,51231,51232,51233,51234,51235,51236,51237,51238,51239,51240,51241,51242,51243,51244,51245,51246,51247,51248,51249,51250,51251,51252,51253,51254,51255,51256,51257,51258,51259,51260,51261,51262,51263,51264,51265,51266,51267,51268,51269,51270,51271,51272,51273,51274,51275,51276,51277,51278,51279,51280,51281,51282,51283,51284,51285,51286,51287,51288,51289,51290,51291,51292,51293,51294,51295,51296,51297,51298,51299,51300,51301,51302,51303,51304,51305,51306,51307,51308,51309,51310,51311,51312,51313,51314,51315,51316,51317,51318,51319,51320,51321,51322,51323,51324,51325,51326,51327,51328,51329,51330,51331,51332,51333,51334,51335,51336,51337,51338,51339,51340,51341,51342,51343,51344,51345,51346,51347,51348,51349,51350,51351,51352,51353,51354,51355,51356,51357,51358,51359,51360,51361,51362,51363,51364,51365,51366,51367,51368,51369,51370,51371,51372,51373,51374,51375,51376,51377,51378,51379,51380,51381,51382,51383,51384,51385,51386,51387,51388,51389,51390,51391,51392,51393,51394,51395,51396,51397,51398,51399,51400,51401,51402,51403,51404,51405,51406,51407,51408,51409,51410,51411,51412,51413,51414,51415,51416,51417,51418,51419,51420,51421,51422,51423,51424,51425,51426,51427,51428,51429,51430,51431,51432,51433,51434,51435,51436,51437,51438,51439,51440,51441,51442,51443,51444,51445,51446,51447,51448,51449,51450,51451,51452,51453,51454,51455,51456,51457,51458,51459,51460,51461,51462,51463,51464,51465,51466,51467,51468,51469,51470,51471,51472,51473,51474,51475,51476,51477,51478,51479,51480,51481,51482,51483,51484,51485,51486,51487,51488,51489,51490,51491,51492,51493,51494,51495,51496,51497,51498,51499,51500,51501,51502,51503,51504,51505,51506,51507,51508,51509,51510,51511,51512,51513,51514,51515,51516,51517,51518,51519,51520,51521,51522,51523,51524,51525,51526,51527,51528,51529,51530,51531,51532,51533,51534,51535,51536,51537,51538,51539,51540,51541,51542,51543,51544,51545,51546,51547,51548,51549,51550,51551,51552,51553,51554,51555,51556,51557,51558,51559,51560,51561,51562,51563,51564,51565,51566,51567,51568,51569,51570,51571,51572,51573,51574,51575,51576,51577,51578,51579,51580,51581,51582,51583,51584,51585,51586,51587,51588,51589,51590,51591,51592,51593,51594,51595,51596,51597,51598,51599,51600,51601,51602,51603,51604,51605,51606,51607,51608,51609,51610,51611,51612,51613,51614,51615,51616,51617,51618,51619,51620,51621,51622,51623,51624,51625,51626,51627,51628,51629,51630,51631,51632,51633,51634,51635,51636,51637,51638,51639,51640,51641,51642,51643,51644,51645,51646,51647,51648,51649,51650,51651,51652,51653,51654,51655,51656,51657,51658,51659,51660,51661,51662,51663,51664,51665,51666,51667,51668,51669,51670,51671,51672,51673,51674,51675,51676,51677,51678,51679,51680,51681,51682,51683,51684,51685,51686,51687,51688,51689,51690,51691,51692,51693,51694,51695,51696,51697,51698,51699,51700,51701,51702,51703,51704,51705,51706,51707,51708,51709,51710,51711,51712,51713,51714,51715,51716,51717,51718,51719,51720,51721,51722,51723,51724,51725,51726,51727,51728,51729,51730,51731,51732,51733,51734,51735,51736,51737,51738,51739,51740,51741,51742,51743,51744,51745,51746,51747,51748,51749,51750,51751,51752,51753,51754,51755,51756,51757,51758,51759,51760,51761,51762,51763,51764,51765,51766,51767,51768,51769,51770,51771,51772,51773,51774,51775,51776,51777,51778,51779,51780,51781,51782,51783,51784,51785,51786,51787,51788,51789,51790,51791,51792,51793,51794,51795,51796,51797,51798,51799,51800,51801,51802,51803,51804,51805,51806,51807,51808,51809,51810,51811,51812,51813,51814,51815,51816,51817,51818,51819,51820,51821,51822,51823,51824,51825,51826,51827,51828,51829,51830,51831,51832,51833,51834,51835,51836,51837,51838,51839,51840,51841,51842,51843,51844,51845,51846,51847,51848,51849,51850,51851,51852,51853,51854,51855,51856,51857,51858,51859,51860,51861,51862,51863,51864,51865,51866,51867,51868,51869,51870,51871,51872,51873,51874,51875,51876,51877,51878,51879,51880,51881,51882,51883,51884,51885,51886,51887,51888,51889,51890,51891,51892,51893,51894,51895,51896,51897,51898,51899,51900,51901,51902,51903,51904,51905,51906,51907,51908,51909,51910,51911,51912,51913,51914,51915,51916,51917,51918,51919,51920,51921,51922,51923,51924,51925,51926,51927,51928,51929,51930,51931,51932,51933,51934,51935,51936,51937,51938,51939,51940,51941,51942,51943,51944,51945,51946,51947,51948,51949,51950,51951,51952,51953,51954,51955,51956,51957,51958,51959,51960,51961,51962,51963,51964,51965,51966,51967,51968,51969,51970,51971,51972,51973,51974,51975,51976,51977,51978,51979,51980,51981,51982,51983,51984,51985,51986,51987,51988,51989,51990,51991,51992,51993,51994,51995,51996,51997,51998,51999,52000,52001,52002,52003,52004,52005,52006,52007,52008,52009,52010,52011,52012,52013,52014,52015,52016,52017,52018,52019,52020,52021,52022,52023,52024,52025,52026,52027,52028,52029,52030,52031,52032,52033,52034,52035,52036,52037,52038,52039,52040,52041,52042,52043,52044,52045,52046,52047,52048,52049,52050,52051,52052,52053,52054,52055,52056,52057,52058,52059,52060,52061,52062,52063,52064,52065,52066,52067,52068,52069,52070,52071,52072,52073,52074,52075,52076,52077,52078,52079,52080,52081,52082,52083,52084,52085,52086,52087,52088,52089,52090,52091,52092,52093,52094,52095,52096,52097,52098,52099,52100,52101,52102,52103,52104,52105,52106,52107,52108,52109,52110,52111,52112,52113,52114,52115,52116,52117,52118,52119,52120,52121,52122,52123,52124,52125,52126,52127,52128,52129,52130,52131,52132,52133,52134,52135,52136,52137,52138,52139,52140,52141,52142,52143,52144,52145,52146,52147,52148,52149,52150,52151,52152,52153,52154,52155,52156,52157,52158,52159,52160,52161,52162,52163,52164,52165,52166,52167,52168,52169,52170,52171,52172,52173,52174,52175,52176,52177,52178,52179,52180,52181,52182,52183,52184,52185,52186,52187,52188,52189,52190,52191,52192,52193,52194,52195,52196,52197,52198,52199,52200,52201,52202,52203,52204,52205,52206,52207,52208,52209,52210,52211,52212,52213,52214,52215,52216,52217,52218,52219,52220,52221,52222,52223,52224,52225,52226,52227,52228,52229,52230,52231,52232,52233,52234,52235,52236,52237,52238,52239,52240,52241,52242,52243,52244,52245,52246,52247,52248,52249,52250,52251,52252,52253,52254,52255,52256,52257,52258,52259,52260,52261,52262,52263,52264,52265,52266,52267,52268,52269,52270,52271,52272,52273,52274,52275,52276,52277,52278,52279,52280,52281,52282,52283,52284,52285,52286,52287,52288,52289,52290,52291,52292,52293,52294,52295,52296,52297,52298,52299,52300,52301,52302,52303,52304,52305,52306,52307,52308,52309,52310,52311,52312,52313,52314,52315,52316,52317,52318,52319,52320,52321,52322,52323,52324,52325,52326,52327,52328,52329,52330,52331,52332,52333,52334,52335,52336,52337,52338,52339,52340,52341,52342,52343,52344,52345,52346,52347,52348,52349,52350,52351,52352,52353,52354,52355,52356,52357,52358,52359,52360,52361,52362,52363,52364,52365,52366,52367,52368,52369,52370,52371,52372,52373,52374,52375,52376,52377,52378,52379,52380,52381,52382,52383,52384,52385,52386,52387,52388,52389,52390,52391,52392,52393,52394,52395,52396,52397,52398,52399,52400,52401,52402,52403,52404,52405,52406,52407,52408,52409,52410,52411,52412,52413,52414,52415,52416,52417,52418,52419,52420,52421,52422,52423,52424,52425,52426,52427,52428,52429,52430,52431,52432,52433,52434,52435,52436,52437,52438,52439,52440,52441,52442,52443,52444,52445,52446,52447,52448,52449,52450,52451,52452,52453,52454,52455,52456,52457,52458,52459,52460,52461,52462,52463,52464,52465,52466,52467,52468,52469,52470,52471,52472,52473,52474,52475,52476,52477,52478,52479,52480,52481,52482,52483,52484,52485,52486,52487,52488,52489,52490,52491,52492,52493,52494,52495,52496,52497,52498,52499,52500,52501,52502,52503,52504,52505,52506,52507,52508,52509,52510,52511,52512,52513,52514,52515,52516,52517,52518,52519,52520,52521,52522,52523,52524,52525,52526,52527,52528,52529,52530,52531,52532,52533,52534,52535,52536,52537,52538,52539,52540,52541,52542,52543,52544,52545,52546,52547,52548,52549,52550,52551,52552,52553,52554,52555,52556,52557,52558,52559,52560,52561,52562,52563,52564,52565,52566,52567,52568,52569,52570,52571,52572,52573,52574,52575,52576,52577,52578,52579,52580,52581,52582,52583,52584,52585,52586,52587,52588,52589,52590,52591,52592,52593,52594,52595,52596,52597,52598,52599,52600,52601,52602,52603,52604,52605,52606,52607,52608,52609,52610,52611,52612,52613,52614,52615,52616,52617,52618,52619,52620,52621,52622,52623,52624,52625,52626,52627,52628,52629,52630,52631,52632,52633,52634,52635,52636,52637,52638,52639,52640,52641,52642,52643,52644,52645,52646,52647,52648,52649,52650,52651,52652,52653,52654,52655,52656,52657,52658,52659,52660,52661,52662,52663,52664,52665,52666,52667,52668,52669,52670,52671,52672,52673,52674,52675,52676,52677,52678,52679,52680,52681,52682,52683,52684,52685,52686,52687,52688,52689,52690,52691,52692,52693,52694,52695,52696,52697,52698,52699,52700,52701,52702,52703,52704,52705,52706,52707,52708,52709,52710,52711,52712,52713,52714,52715,52716,52717,52718,52719,52720,52721,52722,52723,52724,52725,52726,52727,52728,52729,52730,52731,52732,52733,52734,52735,52736,52737,52738,52739,52740,52741,52742,52743,52744,52745,52746,52747,52748,52749,52750,52751,52752,52753,52754,52755,52756,52757,52758,52759,52760,52761,52762,52763,52764,52765,52766,52767,52768,52769,52770,52771,52772,52773,52774,52775,52776,52777,52778,52779,52780,52781,52782,52783,52784,52785,52786,52787,52788,52789,52790,52791,52792,52793,52794,52795,52796,52797,52798,52799,52800,52801,52802,52803,52804,52805,52806,52807,52808,52809,52810,52811,52812,52813,52814,52815,52816,52817,52818,52819,52820,52821,52822,52823,52824,52825,52826,52827,52828,52829,52830,52831,52832,52833,52834,52835,52836,52837,52838,52839,52840,52841,52842,52843,52844,52845,52846,52847,52848,52849,52850,52851,52852,52853,52854,52855,52856,52857,52858,52859,52860,52861,52862,52863,52864,52865,52866,52867,52868,52869,52870,52871,52872,52873,52874,52875,52876,52877,52878,52879,52880,52881,52882,52883,52884,52885,52886,52887,52888,52889,52890,52891,52892,52893,52894,52895,52896,52897,52898,52899,52900,52901,52902,52903,52904,52905,52906,52907,52908,52909,52910,52911,52912,52913,52914,52915,52916,52917,52918,52919,52920,52921,52922,52923,52924,52925,52926,52927,52928,52929,52930,52931,52932,52933,52934,52935,52936,52937,52938,52939,52940,52941,52942,52943,52944,52945,52946,52947,52948,52949,52950,52951,52952,52953,52954,52955,52956,52957,52958,52959,52960,52961,52962,52963,52964,52965,52966,52967,52968,52969,52970,52971,52972,52973,52974,52975,52976,52977,52978,52979,52980,52981,52982,52983,52984,52985,52986,52987,52988,52989,52990,52991,52992,52993,52994,52995,52996,52997,52998,52999,53000,53001,53002,53003,53004,53005,53006,53007,53008,53009,53010,53011,53012,53013,53014,53015,53016,53017,53018,53019,53020,53021,53022,53023,53024,53025,53026,53027,53028,53029,53030,53031,53032,53033,53034,53035,53036,53037,53038,53039,53040,53041,53042,53043,53044,53045,53046,53047,53048,53049,53050,53051,53052,53053,53054,53055,53056,53057,53058,53059,53060,53061,53062,53063,53064,53065,53066,53067,53068,53069,53070,53071,53072,53073,53074,53075,53076,53077,53078,53079,53080,53081,53082,53083,53084,53085,53086,53087,53088,53089,53090,53091,53092,53093,53094,53095,53096,53097,53098,53099,53100,53101,53102,53103,53104,53105,53106,53107,53108,53109,53110,53111,53112,53113,53114,53115,53116,53117,53118,53119,53120,53121,53122,53123,53124,53125,53126,53127,53128,53129,53130,53131,53132,53133,53134,53135,53136,53137,53138,53139,53140,53141,53142,53143,53144,53145,53146,53147,53148,53149,53150,53151,53152,53153,53154,53155,53156,53157,53158,53159,53160,53161,53162,53163,53164,53165,53166,53167,53168,53169,53170,53171,53172,53173,53174,53175,53176,53177,53178,53179,53180,53181,53182,53183,53184,53185,53186,53187,53188,53189,53190,53191,53192,53193,53194,53195,53196,53197,53198,53199,53200,53201,53202,53203,53204,53205,53206,53207,53208,53209,53210,53211,53212,53213,53214,53215,53216,53217,53218,53219,53220,53221,53222,53223,53224,53225,53226,53227,53228,53229,53230,53231,53232,53233,53234,53235,53236,53237,53238,53239,53240,53241,53242,53243,53244,53245,53246,53247,53248,53249,53250,53251,53252,53253,53254,53255,53256,53257,53258,53259,53260,53261,53262,53263,53264,53265,53266,53267,53268,53269,53270,53271,53272,53273,53274,53275,53276,53277,53278,53279,53280,53281,53282,53283,53284,53285,53286,53287,53288,53289,53290,53291,53292,53293,53294,53295,53296,53297,53298,53299,53300,53301,53302,53303,53304,53305,53306,53307,53308,53309,53310,53311,53312,53313,53314,53315,53316,53317,53318,53319,53320,53321,53322,53323,53324,53325,53326,53327,53328,53329,53330,53331,53332,53333,53334,53335,53336,53337,53338,53339,53340,53341,53342,53343,53344,53345,53346,53347,53348,53349,53350,53351,53352,53353,53354,53355,53356,53357,53358,53359,53360,53361,53362,53363,53364,53365,53366,53367,53368,53369,53370,53371,53372,53373,53374,53375,53376,53377,53378,53379,53380,53381,53382,53383,53384,53385,53386,53387,53388,53389,53390,53391,53392,53393,53394,53395,53396,53397,53398,53399,53400,53401,53402,53403,53404,53405,53406,53407,53408,53409,53410,53411,53412,53413,53414,53415,53416,53417,53418,53419,53420,53421,53422,53423,53424,53425,53426,53427,53428,53429,53430,53431,53432,53433,53434,53435,53436,53437,53438,53439,53440,53441,53442,53443,53444,53445,53446,53447,53448,53449,53450,53451,53452,53453,53454,53455,53456,53457,53458,53459,53460,53461,53462,53463,53464,53465,53466,53467,53468,53469,53470,53471,53472,53473,53474,53475,53476,53477,53478,53479,53480,53481,53482,53483,53484,53485,53486,53487,53488,53489,53490,53491,53492,53493,53494,53495,53496,53497,53498,53499,53500,53501,53502,53503,53504,53505,53506,53507,53508,53509,53510,53511,53512,53513,53514,53515,53516,53517,53518,53519,53520,53521,53522,53523,53524,53525,53526,53527,53528,53529,53530,53531,53532,53533,53534,53535,53536,53537,53538,53539,53540,53541,53542,53543,53544,53545,53546,53547,53548,53549,53550,53551,53552,53553,53554,53555,53556,53557,53558,53559,53560,53561,53562,53563,53564,53565,53566,53567,53568,53569,53570,53571,53572,53573,53574,53575,53576,53577,53578,53579,53580,53581,53582,53583,53584,53585,53586,53587,53588,53589,53590,53591,53592,53593,53594,53595,53596,53597,53598,53599,53600,53601,53602,53603,53604,53605,53606,53607,53608,53609,53610,53611,53612,53613,53614,53615,53616,53617,53618,53619,53620,53621,53622,53623,53624,53625,53626,53627,53628,53629,53630,53631,53632,53633,53634,53635,53636,53637,53638,53639,53640,53641,53642,53643,53644,53645,53646,53647,53648,53649,53650,53651,53652,53653,53654,53655,53656,53657,53658,53659,53660,53661,53662,53663,53664,53665,53666,53667,53668,53669,53670,53671,53672,53673,53674,53675,53676,53677,53678,53679,53680,53681,53682,53683,53684,53685,53686,53687,53688,53689,53690,53691,53692,53693,53694,53695,53696,53697,53698,53699,53700,53701,53702,53703,53704,53705,53706,53707,53708,53709,53710,53711,53712,53713,53714,53715,53716,53717,53718,53719,53720,53721,53722,53723,53724,53725,53726,53727,53728,53729,53730,53731,53732,53733,53734,53735,53736,53737,53738,53739,53740,53741,53742,53743,53744,53745,53746,53747,53748,53749,53750,53751,53752,53753,53754,53755,53756,53757,53758,53759,53760,53761,53762,53763,53764,53765,53766,53767,53768,53769,53770,53771,53772,53773,53774,53775,53776,53777,53778,53779,53780,53781,53782,53783,53784,53785,53786,53787,53788,53789,53790,53791,53792,53793,53794,53795,53796,53797,53798,53799,53800,53801,53802,53803,53804,53805,53806,53807,53808,53809,53810,53811,53812,53813,53814,53815,53816,53817,53818,53819,53820,53821,53822,53823,53824,53825,53826,53827,53828,53829,53830,53831,53832,53833,53834,53835,53836,53837,53838,53839,53840,53841,53842,53843,53844,53845,53846,53847,53848,53849,53850,53851,53852,53853,53854,53855,53856,53857,53858,53859,53860,53861,53862,53863,53864,53865,53866,53867,53868,53869,53870,53871,53872,53873,53874,53875,53876,53877,53878,53879,53880,53881,53882,53883,53884,53885,53886,53887,53888,53889,53890,53891,53892,53893,53894,53895,53896,53897,53898,53899,53900,53901,53902,53903,53904,53905,53906,53907,53908,53909,53910,53911,53912,53913,53914,53915,53916,53917,53918,53919,53920,53921,53922,53923,53924,53925,53926,53927,53928,53929,53930,53931,53932,53933,53934,53935,53936,53937,53938,53939,53940,53941,53942,53943,53944,53945,53946,53947,53948,53949,53950,53951,53952,53953,53954,53955,53956,53957,53958,53959,53960,53961,53962,53963,53964,53965,53966,53967,53968,53969,53970,53971,53972,53973,53974,53975,53976,53977,53978,53979,53980,53981,53982,53983,53984,53985,53986,53987,53988,53989,53990,53991,53992,53993,53994,53995,53996,53997,53998,53999,54000,54001,54002,54003,54004,54005,54006,54007,54008,54009,54010,54011,54012,54013,54014,54015,54016,54017,54018,54019,54020,54021,54022,54023,54024,54025,54026,54027,54028,54029,54030,54031,54032,54033,54034,54035,54036,54037,54038,54039,54040,54041,54042,54043,54044,54045,54046,54047,54048,54049,54050,54051,54052,54053,54054,54055,54056,54057,54058,54059,54060,54061,54062,54063,54064,54065,54066,54067,54068,54069,54070,54071,54072,54073,54074,54075,54076,54077,54078,54079,54080,54081,54082,54083,54084,54085,54086,54087,54088,54089,54090,54091,54092,54093,54094,54095,54096,54097,54098,54099,54100,54101,54102,54103,54104,54105,54106,54107,54108,54109,54110,54111,54112,54113,54114,54115,54116,54117,54118,54119,54120,54121,54122,54123,54124,54125,54126,54127,54128,54129,54130,54131,54132,54133,54134,54135,54136,54137,54138,54139,54140,54141,54142,54143,54144,54145,54146,54147,54148,54149,54150,54151,54152,54153,54154,54155,54156,54157,54158,54159,54160,54161,54162,54163,54164,54165,54166,54167,54168,54169,54170,54171,54172,54173,54174,54175,54176,54177,54178,54179,54180,54181,54182,54183,54184,54185,54186,54187,54188,54189,54190,54191,54192,54193,54194,54195,54196,54197,54198,54199,54200,54201,54202,54203,54204,54205,54206,54207,54208,54209,54210,54211,54212,54213,54214,54215,54216,54217,54218,54219,54220,54221,54222,54223,54224,54225,54226,54227,54228,54229,54230,54231,54232,54233,54234,54235,54236,54237,54238,54239,54240,54241,54242,54243,54244,54245,54246,54247,54248,54249,54250,54251,54252,54253,54254,54255,54256,54257,54258,54259,54260,54261,54262,54263,54264,54265,54266,54267,54268,54269,54270,54271,54272,54273,54274,54275,54276,54277,54278,54279,54280,54281,54282,54283,54284,54285,54286,54287,54288,54289,54290,54291,54292,54293,54294,54295,54296,54297,54298,54299,54300,54301,54302,54303,54304,54305,54306,54307,54308,54309,54310,54311,54312,54313,54314,54315,54316,54317,54318,54319,54320,54321,54322,54323,54324,54325,54326,54327,54328,54329,54330,54331,54332,54333,54334,54335,54336,54337,54338,54339,54340,54341,54342,54343,54344,54345,54346,54347,54348,54349,54350,54351,54352,54353,54354,54355,54356,54357,54358,54359,54360,54361,54362,54363,54364,54365,54366,54367,54368,54369,54370,54371,54372,54373,54374,54375,54376,54377,54378,54379,54380,54381,54382,54383,54384,54385,54386,54387,54388,54389,54390,54391,54392,54393,54394,54395,54396,54397,54398,54399,54400,54401,54402,54403,54404,54405,54406,54407,54408,54409,54410,54411,54412,54413,54414,54415,54416,54417,54418,54419,54420,54421,54422,54423,54424,54425,54426,54427,54428,54429,54430,54431,54432,54433,54434,54435,54436,54437,54438,54439,54440,54441,54442,54443,54444,54445,54446,54447,54448,54449,54450,54451,54452,54453,54454,54455,54456,54457,54458,54459,54460,54461,54462,54463,54464,54465,54466,54467,54468,54469,54470,54471,54472,54473,54474,54475,54476,54477,54478,54479,54480,54481,54482,54483,54484,54485,54486,54487,54488,54489,54490,54491,54492,54493,54494,54495,54496,54497,54498,54499,54500,54501,54502,54503,54504,54505,54506,54507,54508,54509,54510,54511,54512,54513,54514,54515,54516,54517,54518,54519,54520,54521,54522,54523,54524,54525,54526,54527,54528,54529,54530,54531,54532,54533,54534,54535,54536,54537,54538,54539,54540,54541,54542,54543,54544,54545,54546,54547,54548,54549,54550,54551,54552,54553,54554,54555,54556,54557,54558,54559,54560,54561,54562,54563,54564,54565,54566,54567,54568,54569,54570,54571,54572,54573,54574,54575,54576,54577,54578,54579,54580,54581,54582,54583,54584,54585,54586,54587,54588,54589,54590,54591,54592,54593,54594,54595,54596,54597,54598,54599,54600,54601,54602,54603,54604,54605,54606,54607,54608,54609,54610,54611,54612,54613,54614,54615,54616,54617,54618,54619,54620,54621,54622,54623,54624,54625,54626,54627,54628,54629,54630,54631,54632,54633,54634,54635,54636,54637,54638,54639,54640,54641,54642,54643,54644,54645,54646,54647,54648,54649,54650,54651,54652,54653,54654,54655,54656,54657,54658,54659,54660,54661,54662,54663,54664,54665,54666,54667,54668,54669,54670,54671,54672,54673,54674,54675,54676,54677,54678,54679,54680,54681,54682,54683,54684,54685,54686,54687,54688,54689,54690,54691,54692,54693,54694,54695,54696,54697,54698,54699,54700,54701,54702,54703,54704,54705,54706,54707,54708,54709,54710,54711,54712,54713,54714,54715,54716,54717,54718,54719,54720,54721,54722,54723,54724,54725,54726,54727,54728,54729,54730,54731,54732,54733,54734,54735,54736,54737,54738,54739,54740,54741,54742,54743,54744,54745,54746,54747,54748,54749,54750,54751,54752,54753,54754,54755,54756,54757,54758,54759,54760,54761,54762,54763,54764,54765,54766,54767,54768,54769,54770,54771,54772,54773,54774,54775,54776,54777,54778,54779,54780,54781,54782,54783,54784,54785,54786,54787,54788,54789,54790,54791,54792,54793,54794,54795,54796,54797,54798,54799,54800,54801,54802,54803,54804,54805,54806,54807,54808,54809,54810,54811,54812,54813,54814,54815,54816,54817,54818,54819,54820,54821,54822,54823,54824,54825,54826,54827,54828,54829,54830,54831,54832,54833,54834,54835,54836,54837,54838,54839,54840,54841,54842,54843,54844,54845,54846,54847,54848,54849,54850,54851,54852,54853,54854,54855,54856,54857,54858,54859,54860,54861,54862,54863,54864,54865,54866,54867,54868,54869,54870,54871,54872,54873,54874,54875,54876,54877,54878,54879,54880,54881,54882,54883,54884,54885,54886,54887,54888,54889,54890,54891,54892,54893,54894,54895,54896,54897,54898,54899,54900,54901,54902,54903,54904,54905,54906,54907,54908,54909,54910,54911,54912,54913,54914,54915,54916,54917,54918,54919,54920,54921,54922,54923,54924,54925,54926,54927,54928,54929,54930,54931,54932,54933,54934,54935,54936,54937,54938,54939,54940,54941,54942,54943,54944,54945,54946,54947,54948,54949,54950,54951,54952,54953,54954,54955,54956,54957,54958,54959,54960,54961,54962,54963,54964,54965,54966,54967,54968,54969,54970,54971,54972,54973,54974,54975,54976,54977,54978,54979,54980,54981,54982,54983,54984,54985,54986,54987,54988,54989,54990,54991,54992,54993,54994,54995,54996,54997,54998,54999,55000,55001,55002,55003,55004,55005,55006,55007,55008,55009,55010,55011,55012,55013,55014,55015,55016,55017,55018,55019,55020,55021,55022,55023,55024,55025,55026,55027,55028,55029,55030,55031,55032,55033,55034,55035,55036,55037,55038,55039,55040,55041,55042,55043,55044,55045,55046,55047,55048,55049,55050,55051,55052,55053,55054,55055,55056,55057,55058,55059,55060,55061,55062,55063,55064,55065,55066,55067,55068,55069,55070,55071,55072,55073,55074,55075,55076,55077,55078,55079,55080,55081,55082,55083,55084,55085,55086,55087,55088,55089,55090,55091,55092,55093,55094,55095,55096,55097,55098,55099,55100,55101,55102,55103,55104,55105,55106,55107,55108,55109,55110,55111,55112,55113,55114,55115,55116,55117,55118,55119,55120,55121,55122,55123,55124,55125,55126,55127,55128,55129,55130,55131,55132,55133,55134,55135,55136,55137,55138,55139,55140,55141,55142,55143,55144,55145,55146,55147,55148,55149,55150,55151,55152,55153,55154,55155,55156,55157,55158,55159,55160,55161,55162,55163,55164,55165,55166,55167,55168,55169,55170,55171,55172,55173,55174,55175,55176,55177,55178,55179,55180,55181,55182,55183,55184,55185,55186,55187,55188,55189,55190,55191,55192,55193,55194,55195,55196,55197,55198,55199,55200,55201,55202,55203,55216,55217,55218,55219,55220,55221,55222,55223,55224,55225,55226,55227,55228,55229,55230,55231,55232,55233,55234,55235,55236,55237,55238,55243,55244,55245,55246,55247,55248,55249,55250,55251,55252,55253,55254,55255,55256,55257,55258,55259,55260,55261,55262,55263,55264,55265,55266,55267,55268,55269,55270,55271,55272,55273,55274,55275,55276,55277,55278,55279,55280,55281,55282,55283,55284,55285,55286,55287,55288,55289,55290,55291,63744,63745,63746,63747,63748,63749,63750,63751,63752,63753,63754,63755,63756,63757,63758,63759,63760,63761,63762,63763,63764,63765,63766,63767,63768,63769,63770,63771,63772,63773,63774,63775,63776,63777,63778,63779,63780,63781,63782,63783,63784,63785,63786,63787,63788,63789,63790,63791,63792,63793,63794,63795,63796,63797,63798,63799,63800,63801,63802,63803,63804,63805,63806,63807,63808,63809,63810,63811,63812,63813,63814,63815,63816,63817,63818,63819,63820,63821,63822,63823,63824,63825,63826,63827,63828,63829,63830,63831,63832,63833,63834,63835,63836,63837,63838,63839,63840,63841,63842,63843,63844,63845,63846,63847,63848,63849,63850,63851,63852,63853,63854,63855,63856,63857,63858,63859,63860,63861,63862,63863,63864,63865,63866,63867,63868,63869,63870,63871,63872,63873,63874,63875,63876,63877,63878,63879,63880,63881,63882,63883,63884,63885,63886,63887,63888,63889,63890,63891,63892,63893,63894,63895,63896,63897,63898,63899,63900,63901,63902,63903,63904,63905,63906,63907,63908,63909,63910,63911,63912,63913,63914,63915,63916,63917,63918,63919,63920,63921,63922,63923,63924,63925,63926,63927,63928,63929,63930,63931,63932,63933,63934,63935,63936,63937,63938,63939,63940,63941,63942,63943,63944,63945,63946,63947,63948,63949,63950,63951,63952,63953,63954,63955,63956,63957,63958,63959,63960,63961,63962,63963,63964,63965,63966,63967,63968,63969,63970,63971,63972,63973,63974,63975,63976,63977,63978,63979,63980,63981,63982,63983,63984,63985,63986,63987,63988,63989,63990,63991,63992,63993,63994,63995,63996,63997,63998,63999,64000,64001,64002,64003,64004,64005,64006,64007,64008,64009,64010,64011,64012,64013,64014,64015,64016,64017,64018,64019,64020,64021,64022,64023,64024,64025,64026,64027,64028,64029,64030,64031,64032,64033,64034,64035,64036,64037,64038,64039,64040,64041,64042,64043,64044,64045,64046,64047,64048,64049,64050,64051,64052,64053,64054,64055,64056,64057,64058,64059,64060,64061,64062,64063,64064,64065,64066,64067,64068,64069,64070,64071,64072,64073,64074,64075,64076,64077,64078,64079,64080,64081,64082,64083,64084,64085,64086,64087,64088,64089,64090,64091,64092,64093,64094,64095,64096,64097,64098,64099,64100,64101,64102,64103,64104,64105,64106,64107,64108,64109,64112,64113,64114,64115,64116,64117,64118,64119,64120,64121,64122,64123,64124,64125,64126,64127,64128,64129,64130,64131,64132,64133,64134,64135,64136,64137,64138,64139,64140,64141,64142,64143,64144,64145,64146,64147,64148,64149,64150,64151,64152,64153,64154,64155,64156,64157,64158,64159,64160,64161,64162,64163,64164,64165,64166,64167,64168,64169,64170,64171,64172,64173,64174,64175,64176,64177,64178,64179,64180,64181,64182,64183,64184,64185,64186,64187,64188,64189,64190,64191,64192,64193,64194,64195,64196,64197,64198,64199,64200,64201,64202,64203,64204,64205,64206,64207,64208,64209,64210,64211,64212,64213,64214,64215,64216,64217,64256,64257,64258,64259,64260,64261,64262,64275,64276,64277,64278,64279,64285,64287,64288,64289,64290,64291,64292,64293,64294,64295,64296,64298,64299,64300,64301,64302,64303,64304,64305,64306,64307,64308,64309,64310,64312,64313,64314,64315,64316,64318,64320,64321,64323,64324,64326,64327,64328,64329,64330,64331,64332,64333,64334,64335,64336,64337,64338,64339,64340,64341,64342,64343,64344,64345,64346,64347,64348,64349,64350,64351,64352,64353,64354,64355,64356,64357,64358,64359,64360,64361,64362,64363,64364,64365,64366,64367,64368,64369,64370,64371,64372,64373,64374,64375,64376,64377,64378,64379,64380,64381,64382,64383,64384,64385,64386,64387,64388,64389,64390,64391,64392,64393,64394,64395,64396,64397,64398,64399,64400,64401,64402,64403,64404,64405,64406,64407,64408,64409,64410,64411,64412,64413,64414,64415,64416,64417,64418,64419,64420,64421,64422,64423,64424,64425,64426,64427,64428,64429,64430,64431,64432,64433,64467,64468,64469,64470,64471,64472,64473,64474,64475,64476,64477,64478,64479,64480,64481,64482,64483,64484,64485,64486,64487,64488,64489,64490,64491,64492,64493,64494,64495,64496,64497,64498,64499,64500,64501,64502,64503,64504,64505,64506,64507,64508,64509,64510,64511,64512,64513,64514,64515,64516,64517,64518,64519,64520,64521,64522,64523,64524,64525,64526,64527,64528,64529,64530,64531,64532,64533,64534,64535,64536,64537,64538,64539,64540,64541,64542,64543,64544,64545,64546,64547,64548,64549,64550,64551,64552,64553,64554,64555,64556,64557,64558,64559,64560,64561,64562,64563,64564,64565,64566,64567,64568,64569,64570,64571,64572,64573,64574,64575,64576,64577,64578,64579,64580,64581,64582,64583,64584,64585,64586,64587,64588,64589,64590,64591,64592,64593,64594,64595,64596,64597,64598,64599,64600,64601,64602,64603,64604,64605,64606,64607,64608,64609,64610,64611,64612,64613,64614,64615,64616,64617,64618,64619,64620,64621,64622,64623,64624,64625,64626,64627,64628,64629,64630,64631,64632,64633,64634,64635,64636,64637,64638,64639,64640,64641,64642,64643,64644,64645,64646,64647,64648,64649,64650,64651,64652,64653,64654,64655,64656,64657,64658,64659,64660,64661,64662,64663,64664,64665,64666,64667,64668,64669,64670,64671,64672,64673,64674,64675,64676,64677,64678,64679,64680,64681,64682,64683,64684,64685,64686,64687,64688,64689,64690,64691,64692,64693,64694,64695,64696,64697,64698,64699,64700,64701,64702,64703,64704,64705,64706,64707,64708,64709,64710,64711,64712,64713,64714,64715,64716,64717,64718,64719,64720,64721,64722,64723,64724,64725,64726,64727,64728,64729,64730,64731,64732,64733,64734,64735,64736,64737,64738,64739,64740,64741,64742,64743,64744,64745,64746,64747,64748,64749,64750,64751,64752,64753,64754,64755,64756,64757,64758,64759,64760,64761,64762,64763,64764,64765,64766,64767,64768,64769,64770,64771,64772,64773,64774,64775,64776,64777,64778,64779,64780,64781,64782,64783,64784,64785,64786,64787,64788,64789,64790,64791,64792,64793,64794,64795,64796,64797,64798,64799,64800,64801,64802,64803,64804,64805,64806,64807,64808,64809,64810,64811,64812,64813,64814,64815,64816,64817,64818,64819,64820,64821,64822,64823,64824,64825,64826,64827,64828,64829,64848,64849,64850,64851,64852,64853,64854,64855,64856,64857,64858,64859,64860,64861,64862,64863,64864,64865,64866,64867,64868,64869,64870,64871,64872,64873,64874,64875,64876,64877,64878,64879,64880,64881,64882,64883,64884,64885,64886,64887,64888,64889,64890,64891,64892,64893,64894,64895,64896,64897,64898,64899,64900,64901,64902,64903,64904,64905,64906,64907,64908,64909,64910,64911,64914,64915,64916,64917,64918,64919,64920,64921,64922,64923,64924,64925,64926,64927,64928,64929,64930,64931,64932,64933,64934,64935,64936,64937,64938,64939,64940,64941,64942,64943,64944,64945,64946,64947,64948,64949,64950,64951,64952,64953,64954,64955,64956,64957,64958,64959,64960,64961,64962,64963,64964,64965,64966,64967,65008,65009,65010,65011,65012,65013,65014,65015,65016,65017,65018,65019,65136,65137,65138,65139,65140,65142,65143,65144,65145,65146,65147,65148,65149,65150,65151,65152,65153,65154,65155,65156,65157,65158,65159,65160,65161,65162,65163,65164,65165,65166,65167,65168,65169,65170,65171,65172,65173,65174,65175,65176,65177,65178,65179,65180,65181,65182,65183,65184,65185,65186,65187,65188,65189,65190,65191,65192,65193,65194,65195,65196,65197,65198,65199,65200,65201,65202,65203,65204,65205,65206,65207,65208,65209,65210,65211,65212,65213,65214,65215,65216,65217,65218,65219,65220,65221,65222,65223,65224,65225,65226,65227,65228,65229,65230,65231,65232,65233,65234,65235,65236,65237,65238,65239,65240,65241,65242,65243,65244,65245,65246,65247,65248,65249,65250,65251,65252,65253,65254,65255,65256,65257,65258,65259,65260,65261,65262,65263,65264,65265,65266,65267,65268,65269,65270,65271,65272,65273,65274,65275,65276,65313,65314,65315,65316,65317,65318,65319,65320,65321,65322,65323,65324,65325,65326,65327,65328,65329,65330,65331,65332,65333,65334,65335,65336,65337,65338,65345,65346,65347,65348,65349,65350,65351,65352,65353,65354,65355,65356,65357,65358,65359,65360,65361,65362,65363,65364,65365,65366,65367,65368,65369,65370,65382,65383,65384,65385,65386,65387,65388,65389,65390,65391,65392,65393,65394,65395,65396,65397,65398,65399,65400,65401,65402,65403,65404,65405,65406,65407,65408,65409,65410,65411,65412,65413,65414,65415,65416,65417,65418,65419,65420,65421,65422,65423,65424,65425,65426,65427,65428,65429,65430,65431,65432,65433,65434,65435,65436,65437,65438,65439,65440,65441,65442,65443,65444,65445,65446,65447,65448,65449,65450,65451,65452,65453,65454,65455,65456,65457,65458,65459,65460,65461,65462,65463,65464,65465,65466,65467,65468,65469,65470,65474,65475,65476,65477,65478,65479,65482,65483,65484,65485,65486,65487,65490,65491,65492,65493,65494,65495,65498,65499,65500";
var arr=str.split(",").map(function(code){return parseInt(code,10)});module.exports=arr},{}],4:[function(require,module,exports){var util=require("util/");var pSlice=Array.prototype.slice;var hasOwn=Object.prototype.hasOwnProperty;var assert=module.exports=ok;assert.AssertionError=function AssertionError(options){this.name="AssertionError";this.actual=options.actual;this.expected=options.expected;this.operator=options.operator;if(options.message){this.message=options.message;this.generatedMessage=false}else{this.message=getMessage(this);this.generatedMessage=true}var stackStartFunction=options.stackStartFunction||fail;if(Error.captureStackTrace){Error.captureStackTrace(this,stackStartFunction)}else{var err=new Error;if(err.stack){var out=err.stack;var fn_name=stackStartFunction.name;var idx=out.indexOf("\n"+fn_name);if(idx>=0){var next_line=out.indexOf("\n",idx+1);out=out.substring(next_line+1)}this.stack=out}}};util.inherits(assert.AssertionError,Error);function replacer(key,value){if(util.isUndefined(value)){return""+value}if(util.isNumber(value)&&!isFinite(value)){return value.toString()}if(util.isFunction(value)||util.isRegExp(value)){return value.toString()}return value}function truncate(s,n){if(util.isString(s)){return s.length<n?s:s.slice(0,n)}else{return s}}function getMessage(self){return truncate(JSON.stringify(self.actual,replacer),128)+" "+self.operator+" "+truncate(JSON.stringify(self.expected,replacer),128)}function fail(actual,expected,message,operator,stackStartFunction){throw new assert.AssertionError({message:message,actual:actual,expected:expected,operator:operator,stackStartFunction:stackStartFunction})}assert.fail=fail;function ok(value,message){if(!value)fail(value,true,message,"==",assert.ok)}assert.ok=ok;assert.equal=function equal(actual,expected,message){if(actual!=expected)fail(actual,expected,message,"==",assert.equal)};assert.notEqual=function notEqual(actual,expected,message){if(actual==expected){fail(actual,expected,message,"!=",assert.notEqual)}};assert.deepEqual=function deepEqual(actual,expected,message){if(!_deepEqual(actual,expected)){fail(actual,expected,message,"deepEqual",assert.deepEqual)}};function _deepEqual(actual,expected){if(actual===expected){return true}else if(util.isBuffer(actual)&&util.isBuffer(expected)){if(actual.length!=expected.length)return false;for(var i=0;i<actual.length;i++){if(actual[i]!==expected[i])return false}return true}else if(util.isDate(actual)&&util.isDate(expected)){return actual.getTime()===expected.getTime()}else if(util.isRegExp(actual)&&util.isRegExp(expected)){return actual.source===expected.source&&actual.global===expected.global&&actual.multiline===expected.multiline&&actual.lastIndex===expected.lastIndex&&actual.ignoreCase===expected.ignoreCase}else if(!util.isObject(actual)&&!util.isObject(expected)){return actual==expected}else{return objEquiv(actual,expected)}}function isArguments(object){return Object.prototype.toString.call(object)=="[object Arguments]"}function objEquiv(a,b){if(util.isNullOrUndefined(a)||util.isNullOrUndefined(b))return false;if(a.prototype!==b.prototype)return false;if(util.isPrimitive(a)||util.isPrimitive(b)){return a===b}var aIsArgs=isArguments(a),bIsArgs=isArguments(b);if(aIsArgs&&!bIsArgs||!aIsArgs&&bIsArgs)return false;if(aIsArgs){a=pSlice.call(a);b=pSlice.call(b);return _deepEqual(a,b)}var ka=objectKeys(a),kb=objectKeys(b),key,i;if(ka.length!=kb.length)return false;ka.sort();kb.sort();for(i=ka.length-1;i>=0;i--){if(ka[i]!=kb[i])return false}for(i=ka.length-1;i>=0;i--){key=ka[i];if(!_deepEqual(a[key],b[key]))return false}return true}assert.notDeepEqual=function notDeepEqual(actual,expected,message){if(_deepEqual(actual,expected)){fail(actual,expected,message,"notDeepEqual",assert.notDeepEqual)}};assert.strictEqual=function strictEqual(actual,expected,message){if(actual!==expected){fail(actual,expected,message,"===",assert.strictEqual)}};assert.notStrictEqual=function notStrictEqual(actual,expected,message){if(actual===expected){fail(actual,expected,message,"!==",assert.notStrictEqual)}};function expectedException(actual,expected){if(!actual||!expected){return false}if(Object.prototype.toString.call(expected)=="[object RegExp]"){return expected.test(actual)}else if(actual instanceof expected){return true}else if(expected.call({},actual)===true){return true}return false}function _throws(shouldThrow,block,expected,message){var actual;if(util.isString(expected)){message=expected;expected=null}try{block()}catch(e){actual=e}message=(expected&&expected.name?" ("+expected.name+").":".")+(message?" "+message:".");if(shouldThrow&&!actual){fail(actual,expected,"Missing expected exception"+message)}if(!shouldThrow&&expectedException(actual,expected)){fail(actual,expected,"Got unwanted exception"+message)}if(shouldThrow&&actual&&expected&&!expectedException(actual,expected)||!shouldThrow&&actual){throw actual}}assert.throws=function(block,error,message){_throws.apply(this,[true].concat(pSlice.call(arguments)))};assert.doesNotThrow=function(block,message){_throws.apply(this,[false].concat(pSlice.call(arguments)))};assert.ifError=function(err){if(err){throw err}};var objectKeys=Object.keys||function(obj){var keys=[];for(var key in obj){if(hasOwn.call(obj,key))keys.push(key)}return keys}},{"util/":9}],5:[function(require,module,exports){function EventEmitter(){this._events=this._events||{};this._maxListeners=this._maxListeners||undefined}module.exports=EventEmitter;EventEmitter.EventEmitter=EventEmitter;EventEmitter.prototype._events=undefined;EventEmitter.prototype._maxListeners=undefined;EventEmitter.defaultMaxListeners=10;EventEmitter.prototype.setMaxListeners=function(n){if(!isNumber(n)||n<0||isNaN(n))throw TypeError("n must be a positive number");this._maxListeners=n;return this};EventEmitter.prototype.emit=function(type){var er,handler,len,args,i,listeners;if(!this._events)this._events={};if(type==="error"){if(!this._events.error||isObject(this._events.error)&&!this._events.error.length){er=arguments[1];if(er instanceof Error){throw er}throw TypeError('Uncaught, unspecified "error" event.')}}handler=this._events[type];if(isUndefined(handler))return false;if(isFunction(handler)){switch(arguments.length){case 1:handler.call(this);break;case 2:handler.call(this,arguments[1]);break;case 3:handler.call(this,arguments[1],arguments[2]);break;default:len=arguments.length;args=new Array(len-1);for(i=1;i<len;i++)args[i-1]=arguments[i];handler.apply(this,args)}}else if(isObject(handler)){len=arguments.length;args=new Array(len-1);for(i=1;i<len;i++)args[i-1]=arguments[i];listeners=handler.slice();len=listeners.length;for(i=0;i<len;i++)listeners[i].apply(this,args)}return true};EventEmitter.prototype.addListener=function(type,listener){var m;if(!isFunction(listener))throw TypeError("listener must be a function");if(!this._events)this._events={};if(this._events.newListener)this.emit("newListener",type,isFunction(listener.listener)?listener.listener:listener);if(!this._events[type])this._events[type]=listener;else if(isObject(this._events[type]))this._events[type].push(listener);else this._events[type]=[this._events[type],listener];if(isObject(this._events[type])&&!this._events[type].warned){var m;if(!isUndefined(this._maxListeners)){m=this._maxListeners}else{m=EventEmitter.defaultMaxListeners}if(m&&m>0&&this._events[type].length>m){this._events[type].warned=true;console.error("(node) warning: possible EventEmitter memory "+"leak detected. %d listeners added. "+"Use emitter.setMaxListeners() to increase limit.",this._events[type].length);if(typeof console.trace==="function"){console.trace()}}}return this};EventEmitter.prototype.on=EventEmitter.prototype.addListener;EventEmitter.prototype.once=function(type,listener){if(!isFunction(listener))throw TypeError("listener must be a function");var fired=false;function g(){this.removeListener(type,g);if(!fired){fired=true;listener.apply(this,arguments)}}g.listener=listener;this.on(type,g);return this};EventEmitter.prototype.removeListener=function(type,listener){var list,position,length,i;if(!isFunction(listener))throw TypeError("listener must be a function");if(!this._events||!this._events[type])return this;list=this._events[type];length=list.length;position=-1;if(list===listener||isFunction(list.listener)&&list.listener===listener){delete this._events[type];if(this._events.removeListener)this.emit("removeListener",type,listener)}else if(isObject(list)){for(i=length;i-->0;){if(list[i]===listener||list[i].listener&&list[i].listener===listener){position=i;break}}if(position<0)return this;if(list.length===1){list.length=0;delete this._events[type]}else{list.splice(position,1)}if(this._events.removeListener)this.emit("removeListener",type,listener)}return this};EventEmitter.prototype.removeAllListeners=function(type){var key,listeners;if(!this._events)return this;if(!this._events.removeListener){if(arguments.length===0)this._events={};else if(this._events[type])delete this._events[type];return this}if(arguments.length===0){for(key in this._events){if(key==="removeListener")continue;this.removeAllListeners(key)}this.removeAllListeners("removeListener");this._events={};return this}listeners=this._events[type];if(isFunction(listeners)){this.removeListener(type,listeners)}else{while(listeners.length)this.removeListener(type,listeners[listeners.length-1])}delete this._events[type];return this};EventEmitter.prototype.listeners=function(type){var ret;if(!this._events||!this._events[type])ret=[];else if(isFunction(this._events[type]))ret=[this._events[type]];else ret=this._events[type].slice();return ret};EventEmitter.listenerCount=function(emitter,type){var ret;if(!emitter._events||!emitter._events[type])ret=0;else if(isFunction(emitter._events[type]))ret=1;else ret=emitter._events[type].length;return ret};function isFunction(arg){return typeof arg==="function"}function isNumber(arg){return typeof arg==="number"}function isObject(arg){return typeof arg==="object"&&arg!==null}function isUndefined(arg){return arg===void 0}},{}],6:[function(require,module,exports){var process=module.exports={};var queue=[];var draining=false;function drainQueue(){if(draining){return}draining=true;var currentQueue;var len=queue.length;while(len){currentQueue=queue;queue=[];var i=-1;while(++i<len){currentQueue[i]()}len=queue.length}draining=false}process.nextTick=function(fun){queue.push(fun);if(!draining){setTimeout(drainQueue,0)}};process.title="browser";process.browser=true;process.env={};process.argv=[];process.version="";process.versions={};function noop(){}process.on=noop;process.addListener=noop;process.once=noop;process.off=noop;process.removeListener=noop;process.removeAllListeners=noop;process.emit=noop;process.binding=function(name){throw new Error("process.binding is not supported")};process.cwd=function(){return"/"};process.chdir=function(dir){throw new Error("process.chdir is not supported")};process.umask=function(){return 0}},{}],7:[function(require,module,exports){if(typeof Object.create==="function"){module.exports=function inherits(ctor,superCtor){ctor.super_=superCtor;ctor.prototype=Object.create(superCtor.prototype,{constructor:{value:ctor,enumerable:false,writable:true,configurable:true}})}}else{module.exports=function inherits(ctor,superCtor){ctor.super_=superCtor;var TempCtor=function(){};TempCtor.prototype=superCtor.prototype;ctor.prototype=new TempCtor;ctor.prototype.constructor=ctor}}},{}],8:[function(require,module,exports){module.exports=function isBuffer(arg){return arg&&typeof arg==="object"&&typeof arg.copy==="function"&&typeof arg.fill==="function"&&typeof arg.readUInt8==="function"}},{}],9:[function(require,module,exports){(function(process,global){var formatRegExp=/%[sdj%]/g;exports.format=function(f){if(!isString(f)){var objects=[];for(var i=0;i<arguments.length;i++){objects.push(inspect(arguments[i]))}return objects.join(" ")}var i=1;var args=arguments;var len=args.length;var str=String(f).replace(formatRegExp,function(x){if(x==="%%")return"%";if(i>=len)return x;switch(x){case"%s":return String(args[i++]);case"%d":return Number(args[i++]);case"%j":try{return JSON.stringify(args[i++])}catch(_){return"[Circular]"}default:return x}});for(var x=args[i];i<len;x=args[++i]){if(isNull(x)||!isObject(x)){str+=" "+x}else{str+=" "+inspect(x)}}return str};exports.deprecate=function(fn,msg){if(isUndefined(global.process)){return function(){return exports.deprecate(fn,msg).apply(this,arguments)}}if(process.noDeprecation===true){return fn}var warned=false;function deprecated(){if(!warned){if(process.throwDeprecation){throw new Error(msg)}else if(process.traceDeprecation){console.trace(msg)}else{console.error(msg)}warned=true}return fn.apply(this,arguments)}return deprecated};var debugs={};var debugEnviron;exports.debuglog=function(set){if(isUndefined(debugEnviron))debugEnviron=process.env.NODE_DEBUG||"";set=set.toUpperCase();if(!debugs[set]){if(new RegExp("\\b"+set+"\\b","i").test(debugEnviron)){var pid=process.pid;debugs[set]=function(){var msg=exports.format.apply(exports,arguments);console.error("%s %d: %s",set,pid,msg)}}else{debugs[set]=function(){}}}return debugs[set]};function inspect(obj,opts){var ctx={seen:[],stylize:stylizeNoColor};if(arguments.length>=3)ctx.depth=arguments[2];if(arguments.length>=4)ctx.colors=arguments[3];if(isBoolean(opts)){ctx.showHidden=opts}else if(opts){exports._extend(ctx,opts)}if(isUndefined(ctx.showHidden))ctx.showHidden=false;if(isUndefined(ctx.depth))ctx.depth=2;if(isUndefined(ctx.colors))ctx.colors=false;if(isUndefined(ctx.customInspect))ctx.customInspect=true;if(ctx.colors)ctx.stylize=stylizeWithColor;return formatValue(ctx,obj,ctx.depth)}exports.inspect=inspect;inspect.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]};inspect.styles={special:"cyan",number:"yellow","boolean":"yellow",undefined:"grey","null":"bold",string:"green",date:"magenta",regexp:"red"};function stylizeWithColor(str,styleType){var style=inspect.styles[styleType];if(style){return"["+inspect.colors[style][0]+"m"+str+"["+inspect.colors[style][1]+"m"}else{return str}}function stylizeNoColor(str,styleType){return str}function arrayToHash(array){var hash={};array.forEach(function(val,idx){hash[val]=true});return hash}function formatValue(ctx,value,recurseTimes){if(ctx.customInspect&&value&&isFunction(value.inspect)&&value.inspect!==exports.inspect&&!(value.constructor&&value.constructor.prototype===value)){var ret=value.inspect(recurseTimes,ctx);if(!isString(ret)){ret=formatValue(ctx,ret,recurseTimes)}return ret}var primitive=formatPrimitive(ctx,value);if(primitive){return primitive}var keys=Object.keys(value);var visibleKeys=arrayToHash(keys);if(ctx.showHidden){keys=Object.getOwnPropertyNames(value)}if(isError(value)&&(keys.indexOf("message")>=0||keys.indexOf("description")>=0)){return formatError(value)}if(keys.length===0){if(isFunction(value)){var name=value.name?": "+value.name:"";return ctx.stylize("[Function"+name+"]","special")}if(isRegExp(value)){return ctx.stylize(RegExp.prototype.toString.call(value),"regexp")}if(isDate(value)){return ctx.stylize(Date.prototype.toString.call(value),"date")}if(isError(value)){return formatError(value)}}var base="",array=false,braces=["{","}"];if(isArray(value)){array=true;braces=["[","]"]}if(isFunction(value)){var n=value.name?": "+value.name:"";base=" [Function"+n+"]"}if(isRegExp(value)){base=" "+RegExp.prototype.toString.call(value)}if(isDate(value)){base=" "+Date.prototype.toUTCString.call(value)}if(isError(value)){base=" "+formatError(value)}if(keys.length===0&&(!array||value.length==0)){return braces[0]+base+braces[1]}if(recurseTimes<0){if(isRegExp(value)){return ctx.stylize(RegExp.prototype.toString.call(value),"regexp")}else{return ctx.stylize("[Object]","special")}}ctx.seen.push(value);var output;if(array){output=formatArray(ctx,value,recurseTimes,visibleKeys,keys)}else{output=keys.map(function(key){return formatProperty(ctx,value,recurseTimes,visibleKeys,key,array)})}ctx.seen.pop();return reduceToSingleString(output,base,braces)}function formatPrimitive(ctx,value){if(isUndefined(value))return ctx.stylize("undefined","undefined");if(isString(value)){var simple="'"+JSON.stringify(value).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return ctx.stylize(simple,"string")}if(isNumber(value))return ctx.stylize(""+value,"number");if(isBoolean(value))return ctx.stylize(""+value,"boolean");if(isNull(value))return ctx.stylize("null","null")}function formatError(value){return"["+Error.prototype.toString.call(value)+"]"}function formatArray(ctx,value,recurseTimes,visibleKeys,keys){var output=[];for(var i=0,l=value.length;i<l;++i){if(hasOwnProperty(value,String(i))){output.push(formatProperty(ctx,value,recurseTimes,visibleKeys,String(i),true))}else{output.push("")}}keys.forEach(function(key){if(!key.match(/^\d+$/)){output.push(formatProperty(ctx,value,recurseTimes,visibleKeys,key,true))}});return output}function formatProperty(ctx,value,recurseTimes,visibleKeys,key,array){var name,str,desc;desc=Object.getOwnPropertyDescriptor(value,key)||{value:value[key]};if(desc.get){if(desc.set){str=ctx.stylize("[Getter/Setter]","special")}else{str=ctx.stylize("[Getter]","special")}}else{if(desc.set){str=ctx.stylize("[Setter]","special")}}if(!hasOwnProperty(visibleKeys,key)){name="["+key+"]"}if(!str){if(ctx.seen.indexOf(desc.value)<0){if(isNull(recurseTimes)){str=formatValue(ctx,desc.value,null)}else{str=formatValue(ctx,desc.value,recurseTimes-1)}if(str.indexOf("\n")>-1){if(array){str=str.split("\n").map(function(line){return"  "+line}).join("\n").substr(2)}else{str="\n"+str.split("\n").map(function(line){return"   "+line}).join("\n")}}}else{str=ctx.stylize("[Circular]","special")}}if(isUndefined(name)){if(array&&key.match(/^\d+$/)){return str}name=JSON.stringify(""+key);if(name.match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)){name=name.substr(1,name.length-2);name=ctx.stylize(name,"name")}else{name=name.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'");name=ctx.stylize(name,"string")}}return name+": "+str}function reduceToSingleString(output,base,braces){var numLinesEst=0;var length=output.reduce(function(prev,cur){numLinesEst++;if(cur.indexOf("\n")>=0)numLinesEst++;return prev+cur.replace(/\u001b\[\d\d?m/g,"").length+1},0);if(length>60){return braces[0]+(base===""?"":base+"\n ")+" "+output.join(",\n  ")+" "+braces[1]}return braces[0]+base+" "+output.join(", ")+" "+braces[1]}function isArray(ar){return Array.isArray(ar)}exports.isArray=isArray;function isBoolean(arg){return typeof arg==="boolean"}exports.isBoolean=isBoolean;function isNull(arg){return arg===null}exports.isNull=isNull;function isNullOrUndefined(arg){return arg==null}exports.isNullOrUndefined=isNullOrUndefined;function isNumber(arg){return typeof arg==="number"}exports.isNumber=isNumber;function isString(arg){return typeof arg==="string"}exports.isString=isString;function isSymbol(arg){return typeof arg==="symbol"}exports.isSymbol=isSymbol;function isUndefined(arg){return arg===void 0}exports.isUndefined=isUndefined;function isRegExp(re){return isObject(re)&&objectToString(re)==="[object RegExp]"}exports.isRegExp=isRegExp;function isObject(arg){return typeof arg==="object"&&arg!==null}exports.isObject=isObject;function isDate(d){return isObject(d)&&objectToString(d)==="[object Date]"}exports.isDate=isDate;function isError(e){return isObject(e)&&(objectToString(e)==="[object Error]"||e instanceof Error)}exports.isError=isError;function isFunction(arg){return typeof arg==="function"}exports.isFunction=isFunction;function isPrimitive(arg){return arg===null||typeof arg==="boolean"||typeof arg==="number"||typeof arg==="string"||typeof arg==="symbol"||typeof arg==="undefined"}exports.isPrimitive=isPrimitive;exports.isBuffer=require("./support/isBuffer");function objectToString(o){return Object.prototype.toString.call(o)}function pad(n){return n<10?"0"+n.toString(10):n.toString(10)}var months=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function timestamp(){var d=new Date;var time=[pad(d.getHours()),pad(d.getMinutes()),pad(d.getSeconds())].join(":");return[d.getDate(),months[d.getMonth()],time].join(" ")}exports.log=function(){console.log("%s - %s",timestamp(),exports.format.apply(exports,arguments))};exports.inherits=require("inherits");exports._extend=function(origin,add){if(!add||!isObject(add))return origin;var keys=Object.keys(add);var i=keys.length;while(i--){origin[keys[i]]=add[keys[i]]}return origin};function hasOwnProperty(obj,prop){return Object.prototype.hasOwnProperty.call(obj,prop)}}).call(this,require("_process"),typeof global!=="undefined"?global:typeof self!=="undefined"?self:typeof window!=="undefined"?window:{})},{"./support/isBuffer":8,_process:6,inherits:7}],10:[function(require,module,exports){(function(global){var util=require("util");var assert=require("assert");var now=require("date-now");var slice=Array.prototype.slice;var console;var times={};if(typeof global!=="undefined"&&global.console){console=global.console}else if(typeof window!=="undefined"&&window.console){console=window.console}else{console={}}var functions=[[log,"log"],[info,"info"],[warn,"warn"],[error,"error"],[time,"time"],[timeEnd,"timeEnd"],[trace,"trace"],[dir,"dir"],[consoleAssert,"assert"]];for(var i=0;i<functions.length;i++){var tuple=functions[i];var f=tuple[0];var name=tuple[1];if(!console[name]){console[name]=f}}module.exports=console;function log(){}function info(){console.log.apply(console,arguments)}function warn(){console.log.apply(console,arguments)}function error(){console.warn.apply(console,arguments)}function time(label){times[label]=now()}function timeEnd(label){var time=times[label];if(!time){throw new Error("No such label: "+label)}var duration=now()-time;console.log(label+": "+duration+"ms")}function trace(){var err=new Error;err.name="Trace";err.message=util.format.apply(null,arguments);console.error(err.stack)}function dir(object){console.log(util.inspect(object)+"\n")}function consoleAssert(expression){if(!expression){var arr=slice.call(arguments,1);assert.ok(false,util.format.apply(null,arr))}}}).call(this,typeof global!=="undefined"?global:typeof self!=="undefined"?self:typeof window!=="undefined"?window:{})},{assert:4,"date-now":11,util:9}],11:[function(require,module,exports){module.exports=now;function now(){return(new Date).getTime()}},{}],12:[function(require,module,exports){(function(global){(function(){var undefined;var VERSION="3.7.0";var BIND_FLAG=1,BIND_KEY_FLAG=2,CURRY_BOUND_FLAG=4,CURRY_FLAG=8,CURRY_RIGHT_FLAG=16,PARTIAL_FLAG=32,PARTIAL_RIGHT_FLAG=64,ARY_FLAG=128,REARG_FLAG=256;var DEFAULT_TRUNC_LENGTH=30,DEFAULT_TRUNC_OMISSION="...";var HOT_COUNT=150,HOT_SPAN=16;var LAZY_DROP_WHILE_FLAG=0,LAZY_FILTER_FLAG=1,LAZY_MAP_FLAG=2;var FUNC_ERROR_TEXT="Expected a function";var PLACEHOLDER="__lodash_placeholder__";var argsTag="[object Arguments]",arrayTag="[object Array]",boolTag="[object Boolean]",dateTag="[object Date]",errorTag="[object Error]",funcTag="[object Function]",mapTag="[object Map]",numberTag="[object Number]",objectTag="[object Object]",regexpTag="[object RegExp]",setTag="[object Set]",stringTag="[object String]",weakMapTag="[object WeakMap]";var arrayBufferTag="[object ArrayBuffer]",float32Tag="[object Float32Array]",float64Tag="[object Float64Array]",int8Tag="[object Int8Array]",int16Tag="[object Int16Array]",int32Tag="[object Int32Array]",uint8Tag="[object Uint8Array]",uint8ClampedTag="[object Uint8ClampedArray]",uint16Tag="[object Uint16Array]",uint32Tag="[object Uint32Array]";var reEmptyStringLeading=/\b__p \+= '';/g,reEmptyStringMiddle=/\b(__p \+=) '' \+/g,reEmptyStringTrailing=/(__e\(.*?\)|\b__t\)) \+\n'';/g;var reEscapedHtml=/&(?:amp|lt|gt|quot|#39|#96);/g,reUnescapedHtml=/[&<>"'`]/g,reHasEscapedHtml=RegExp(reEscapedHtml.source),reHasUnescapedHtml=RegExp(reUnescapedHtml.source);var reEscape=/<%-([\s\S]+?)%>/g,reEvaluate=/<%([\s\S]+?)%>/g,reInterpolate=/<%=([\s\S]+?)%>/g;var reIsDeepProp=/\.|\[(?:[^[\]]+|(["'])(?:(?!\1)[^\n\\]|\\.)*?)\1\]/,reIsPlainProp=/^\w*$/,rePropName=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\n\\]|\\.)*?)\2)\]/g;var reRegExpChars=/[.*+?^${}()|[\]\/\\]/g,reHasRegExpChars=RegExp(reRegExpChars.source);var reComboMark=/[\u0300-\u036f\ufe20-\ufe23]/g;var reEscapeChar=/\\(\\)?/g;var reEsTemplate=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g;var reFlags=/\w*$/;var reHasHexPrefix=/^0[xX]/;var reIsHostCtor=/^\[object .+?Constructor\]$/;var reLatin1=/[\xc0-\xd6\xd8-\xde\xdf-\xf6\xf8-\xff]/g;var reNoMatch=/($^)/;var reUnescapedString=/['\n\r\u2028\u2029\\]/g;var reWords=function(){var upper="[A-Z\\xc0-\\xd6\\xd8-\\xde]",lower="[a-z\\xdf-\\xf6\\xf8-\\xff]+";return RegExp(upper+"+(?="+upper+lower+")|"+upper+"?"+lower+"|"+upper+"+|[0-9]+","g")}();var whitespace=" 	\f \ufeff"+"\n\r\u2028\u2029"+" ᠎             　";var contextProps=["Array","ArrayBuffer","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Math","Number","Object","RegExp","Set","String","_","clearTimeout","document","isFinite","parseInt","setTimeout","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","window"];var templateCounter=-1;var typedArrayTags={};typedArrayTags[float32Tag]=typedArrayTags[float64Tag]=typedArrayTags[int8Tag]=typedArrayTags[int16Tag]=typedArrayTags[int32Tag]=typedArrayTags[uint8Tag]=typedArrayTags[uint8ClampedTag]=typedArrayTags[uint16Tag]=typedArrayTags[uint32Tag]=true;typedArrayTags[argsTag]=typedArrayTags[arrayTag]=typedArrayTags[arrayBufferTag]=typedArrayTags[boolTag]=typedArrayTags[dateTag]=typedArrayTags[errorTag]=typedArrayTags[funcTag]=typedArrayTags[mapTag]=typedArrayTags[numberTag]=typedArrayTags[objectTag]=typedArrayTags[regexpTag]=typedArrayTags[setTag]=typedArrayTags[stringTag]=typedArrayTags[weakMapTag]=false;var cloneableTags={};cloneableTags[argsTag]=cloneableTags[arrayTag]=cloneableTags[arrayBufferTag]=cloneableTags[boolTag]=cloneableTags[dateTag]=cloneableTags[float32Tag]=cloneableTags[float64Tag]=cloneableTags[int8Tag]=cloneableTags[int16Tag]=cloneableTags[int32Tag]=cloneableTags[numberTag]=cloneableTags[objectTag]=cloneableTags[regexpTag]=cloneableTags[stringTag]=cloneableTags[uint8Tag]=cloneableTags[uint8ClampedTag]=cloneableTags[uint16Tag]=cloneableTags[uint32Tag]=true;cloneableTags[errorTag]=cloneableTags[funcTag]=cloneableTags[mapTag]=cloneableTags[setTag]=cloneableTags[weakMapTag]=false;var debounceOptions={leading:false,maxWait:0,trailing:false};var deburredLetters={"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss"};var htmlEscapes={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","`":"&#96;"};var htmlUnescapes={"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'","&#96;":"`"};var objectTypes={"function":true,object:true};var stringEscapes={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"};var freeExports=objectTypes[typeof exports]&&exports&&!exports.nodeType&&exports;var freeModule=objectTypes[typeof module]&&module&&!module.nodeType&&module;var freeGlobal=freeExports&&freeModule&&typeof global=="object"&&global&&global.Object&&global;var freeSelf=objectTypes[typeof self]&&self&&self.Object&&self;var freeWindow=objectTypes[typeof window]&&window&&window.Object&&window;var moduleExports=freeModule&&freeModule.exports===freeExports&&freeExports;var root=freeGlobal||freeWindow!==(this&&this.window)&&freeWindow||freeSelf||this;function baseCompareAscending(value,other){if(value!==other){var valIsReflexive=value===value,othIsReflexive=other===other;if(value>other||!valIsReflexive||value===undefined&&othIsReflexive){return 1}if(value<other||!othIsReflexive||other===undefined&&valIsReflexive){return-1}}return 0}function baseFindIndex(array,predicate,fromRight){var length=array.length,index=fromRight?length:-1;while(fromRight?index--:++index<length){if(predicate(array[index],index,array)){return index}}return-1}function baseIndexOf(array,value,fromIndex){if(value!==value){return indexOfNaN(array,fromIndex)}var index=fromIndex-1,length=array.length;while(++index<length){if(array[index]===value){return index}}return-1}function baseIsFunction(value){return typeof value=="function"||false}function baseToString(value){if(typeof value=="string"){return value}return value==null?"":value+""}function charAtCallback(string){return string.charCodeAt(0)}function charsLeftIndex(string,chars){var index=-1,length=string.length;while(++index<length&&chars.indexOf(string.charAt(index))>-1){}return index}function charsRightIndex(string,chars){var index=string.length;while(index--&&chars.indexOf(string.charAt(index))>-1){}return index}function compareAscending(object,other){return baseCompareAscending(object.criteria,other.criteria)||object.index-other.index}function compareMultiple(object,other,orders){var index=-1,objCriteria=object.criteria,othCriteria=other.criteria,length=objCriteria.length,ordersLength=orders.length;while(++index<length){var result=baseCompareAscending(objCriteria[index],othCriteria[index]);if(result){if(index>=ordersLength){return result}return result*(orders[index]?1:-1)}}return object.index-other.index}function deburrLetter(letter){return deburredLetters[letter]}function escapeHtmlChar(chr){return htmlEscapes[chr]}function escapeStringChar(chr){return"\\"+stringEscapes[chr]}function indexOfNaN(array,fromIndex,fromRight){var length=array.length,index=fromIndex+(fromRight?0:-1);while(fromRight?index--:++index<length){var other=array[index];if(other!==other){return index}}return-1}function isObjectLike(value){return!!value&&typeof value=="object"}function isSpace(charCode){return charCode<=160&&(charCode>=9&&charCode<=13)||charCode==32||charCode==160||charCode==5760||charCode==6158||charCode>=8192&&(charCode<=8202||charCode==8232||charCode==8233||charCode==8239||charCode==8287||charCode==12288||charCode==65279)}function replaceHolders(array,placeholder){var index=-1,length=array.length,resIndex=-1,result=[];while(++index<length){if(array[index]===placeholder){array[index]=PLACEHOLDER;result[++resIndex]=index}}return result}function sortedUniq(array,iteratee){var seen,index=-1,length=array.length,resIndex=-1,result=[];while(++index<length){var value=array[index],computed=iteratee?iteratee(value,index,array):value;if(!index||seen!==computed){seen=computed;result[++resIndex]=value}}return result}function trimmedLeftIndex(string){var index=-1,length=string.length;while(++index<length&&isSpace(string.charCodeAt(index))){}return index}function trimmedRightIndex(string){var index=string.length;while(index--&&isSpace(string.charCodeAt(index))){}return index}function unescapeHtmlChar(chr){return htmlUnescapes[chr]}function runInContext(context){context=context?_.defaults(root.Object(),context,_.pick(root,contextProps)):root;var Array=context.Array,Date=context.Date,Error=context.Error,Function=context.Function,Math=context.Math,Number=context.Number,Object=context.Object,RegExp=context.RegExp,String=context.String,TypeError=context.TypeError;var arrayProto=Array.prototype,objectProto=Object.prototype,stringProto=String.prototype;var document=(document=context.window)&&document.document;var fnToString=Function.prototype.toString;var hasOwnProperty=objectProto.hasOwnProperty;var idCounter=0;var objToString=objectProto.toString;var oldDash=context._;var reIsNative=RegExp("^"+escapeRegExp(objToString).replace(/toString|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");var ArrayBuffer=isNative(ArrayBuffer=context.ArrayBuffer)&&ArrayBuffer,bufferSlice=isNative(bufferSlice=ArrayBuffer&&new ArrayBuffer(0).slice)&&bufferSlice,ceil=Math.ceil,clearTimeout=context.clearTimeout,floor=Math.floor,getOwnPropertySymbols=isNative(getOwnPropertySymbols=Object.getOwnPropertySymbols)&&getOwnPropertySymbols,getPrototypeOf=isNative(getPrototypeOf=Object.getPrototypeOf)&&getPrototypeOf,push=arrayProto.push,preventExtensions=isNative(Object.preventExtensions=Object.preventExtensions)&&preventExtensions,propertyIsEnumerable=objectProto.propertyIsEnumerable,Set=isNative(Set=context.Set)&&Set,setTimeout=context.setTimeout,splice=arrayProto.splice,Uint8Array=isNative(Uint8Array=context.Uint8Array)&&Uint8Array,WeakMap=isNative(WeakMap=context.WeakMap)&&WeakMap;
var Float64Array=function(){try{var func=isNative(func=context.Float64Array)&&func,result=new func(new ArrayBuffer(10),0,1)&&func}catch(e){}return result}();var nativeAssign=function(){var object={1:0},func=preventExtensions&&isNative(func=Object.assign)&&func;try{func(preventExtensions(object),"xo")}catch(e){}return!object[1]&&func}();var nativeIsArray=isNative(nativeIsArray=Array.isArray)&&nativeIsArray,nativeCreate=isNative(nativeCreate=Object.create)&&nativeCreate,nativeIsFinite=context.isFinite,nativeKeys=isNative(nativeKeys=Object.keys)&&nativeKeys,nativeMax=Math.max,nativeMin=Math.min,nativeNow=isNative(nativeNow=Date.now)&&nativeNow,nativeNumIsFinite=isNative(nativeNumIsFinite=Number.isFinite)&&nativeNumIsFinite,nativeParseInt=context.parseInt,nativeRandom=Math.random;var NEGATIVE_INFINITY=Number.NEGATIVE_INFINITY,POSITIVE_INFINITY=Number.POSITIVE_INFINITY;var MAX_ARRAY_LENGTH=Math.pow(2,32)-1,MAX_ARRAY_INDEX=MAX_ARRAY_LENGTH-1,HALF_MAX_ARRAY_LENGTH=MAX_ARRAY_LENGTH>>>1;var FLOAT64_BYTES_PER_ELEMENT=Float64Array?Float64Array.BYTES_PER_ELEMENT:0;var MAX_SAFE_INTEGER=Math.pow(2,53)-1;var metaMap=WeakMap&&new WeakMap;var realNames={};function lodash(value){if(isObjectLike(value)&&!isArray(value)&&!(value instanceof LazyWrapper)){if(value instanceof LodashWrapper){return value}if(hasOwnProperty.call(value,"__chain__")&&hasOwnProperty.call(value,"__wrapped__")){return wrapperClone(value)}}return new LodashWrapper(value)}function baseLodash(){}function LodashWrapper(value,chainAll,actions){this.__wrapped__=value;this.__actions__=actions||[];this.__chain__=!!chainAll}var support=lodash.support={};(function(x){var Ctor=function(){this.x=x},object={0:x,length:x},props=[];Ctor.prototype={valueOf:x,y:x};for(var key in new Ctor){props.push(key)}support.funcDecomp=/\bthis\b/.test(function(){return this});support.funcNames=typeof Function.name=="string";try{support.dom=document.createDocumentFragment().nodeType===11}catch(e){support.dom=false}try{support.nonEnumArgs=!propertyIsEnumerable.call(arguments,1)}catch(e){support.nonEnumArgs=true}})(1,0);lodash.templateSettings={escape:reEscape,evaluate:reEvaluate,interpolate:reInterpolate,variable:"",imports:{_:lodash}};function LazyWrapper(value){this.__wrapped__=value;this.__actions__=null;this.__dir__=1;this.__dropCount__=0;this.__filtered__=false;this.__iteratees__=null;this.__takeCount__=POSITIVE_INFINITY;this.__views__=null}function lazyClone(){var actions=this.__actions__,iteratees=this.__iteratees__,views=this.__views__,result=new LazyWrapper(this.__wrapped__);result.__actions__=actions?arrayCopy(actions):null;result.__dir__=this.__dir__;result.__filtered__=this.__filtered__;result.__iteratees__=iteratees?arrayCopy(iteratees):null;result.__takeCount__=this.__takeCount__;result.__views__=views?arrayCopy(views):null;return result}function lazyReverse(){if(this.__filtered__){var result=new LazyWrapper(this);result.__dir__=-1;result.__filtered__=true}else{result=this.clone();result.__dir__*=-1}return result}function lazyValue(){var array=this.__wrapped__.value();if(!isArray(array)){return baseWrapperValue(array,this.__actions__)}var dir=this.__dir__,isRight=dir<0,view=getView(0,array.length,this.__views__),start=view.start,end=view.end,length=end-start,index=isRight?end:start-1,takeCount=nativeMin(length,this.__takeCount__),iteratees=this.__iteratees__,iterLength=iteratees?iteratees.length:0,resIndex=0,result=[];outer:while(length--&&resIndex<takeCount){index+=dir;var iterIndex=-1,value=array[index];while(++iterIndex<iterLength){var data=iteratees[iterIndex],iteratee=data.iteratee,type=data.type;if(type==LAZY_DROP_WHILE_FLAG){if(data.done&&(isRight?index>data.index:index<data.index)){data.count=0;data.done=false}data.index=index;if(!data.done){var limit=data.limit;if(!(data.done=limit>-1?data.count++>=limit:!iteratee(value))){continue outer}}}else{var computed=iteratee(value);if(type==LAZY_MAP_FLAG){value=computed}else if(!computed){if(type==LAZY_FILTER_FLAG){continue outer}else{break outer}}}}result[resIndex++]=value}return result}function MapCache(){this.__data__={}}function mapDelete(key){return this.has(key)&&delete this.__data__[key]}function mapGet(key){return key=="__proto__"?undefined:this.__data__[key]}function mapHas(key){return key!="__proto__"&&hasOwnProperty.call(this.__data__,key)}function mapSet(key,value){if(key!="__proto__"){this.__data__[key]=value}return this}function SetCache(values){var length=values?values.length:0;this.data={hash:nativeCreate(null),set:new Set};while(length--){this.push(values[length])}}function cacheIndexOf(cache,value){var data=cache.data,result=typeof value=="string"||isObject(value)?data.set.has(value):data.hash[value];return result?0:-1}function cachePush(value){var data=this.data;if(typeof value=="string"||isObject(value)){data.set.add(value)}else{data.hash[value]=true}}function arrayCopy(source,array){var index=-1,length=source.length;array||(array=Array(length));while(++index<length){array[index]=source[index]}return array}function arrayEach(array,iteratee){var index=-1,length=array.length;while(++index<length){if(iteratee(array[index],index,array)===false){break}}return array}function arrayEachRight(array,iteratee){var length=array.length;while(length--){if(iteratee(array[length],length,array)===false){break}}return array}function arrayEvery(array,predicate){var index=-1,length=array.length;while(++index<length){if(!predicate(array[index],index,array)){return false}}return true}function arrayFilter(array,predicate){var index=-1,length=array.length,resIndex=-1,result=[];while(++index<length){var value=array[index];if(predicate(value,index,array)){result[++resIndex]=value}}return result}function arrayMap(array,iteratee){var index=-1,length=array.length,result=Array(length);while(++index<length){result[index]=iteratee(array[index],index,array)}return result}function arrayMax(array){var index=-1,length=array.length,result=NEGATIVE_INFINITY;while(++index<length){var value=array[index];if(value>result){result=value}}return result}function arrayMin(array){var index=-1,length=array.length,result=POSITIVE_INFINITY;while(++index<length){var value=array[index];if(value<result){result=value}}return result}function arrayReduce(array,iteratee,accumulator,initFromArray){var index=-1,length=array.length;if(initFromArray&&length){accumulator=array[++index]}while(++index<length){accumulator=iteratee(accumulator,array[index],index,array)}return accumulator}function arrayReduceRight(array,iteratee,accumulator,initFromArray){var length=array.length;if(initFromArray&&length){accumulator=array[--length]}while(length--){accumulator=iteratee(accumulator,array[length],length,array)}return accumulator}function arraySome(array,predicate){var index=-1,length=array.length;while(++index<length){if(predicate(array[index],index,array)){return true}}return false}function arraySum(array){var length=array.length,result=0;while(length--){result+=+array[length]||0}return result}function assignDefaults(objectValue,sourceValue){return objectValue===undefined?sourceValue:objectValue}function assignOwnDefaults(objectValue,sourceValue,key,object){return objectValue===undefined||!hasOwnProperty.call(object,key)?sourceValue:objectValue}function assignWith(object,source,customizer){var props=keys(source);push.apply(props,getSymbols(source));var index=-1,length=props.length;while(++index<length){var key=props[index],value=object[key],result=customizer(value,source[key],key,object,source);if((result===result?result!==value:value===value)||value===undefined&&!(key in object)){object[key]=result}}return object}var baseAssign=nativeAssign||function(object,source){return source==null?object:baseCopy(source,getSymbols(source),baseCopy(source,keys(source),object))};function baseAt(collection,props){var index=-1,length=collection.length,isArr=isLength(length),propsLength=props.length,result=Array(propsLength);while(++index<propsLength){var key=props[index];if(isArr){result[index]=isIndex(key,length)?collection[key]:undefined}else{result[index]=collection[key]}}return result}function baseCopy(source,props,object){object||(object={});var index=-1,length=props.length;while(++index<length){var key=props[index];object[key]=source[key]}return object}function baseCallback(func,thisArg,argCount){var type=typeof func;if(type=="function"){return thisArg===undefined?func:bindCallback(func,thisArg,argCount)}if(func==null){return identity}if(type=="object"){return baseMatches(func)}return thisArg===undefined?property(func):baseMatchesProperty(func,thisArg)}function baseClone(value,isDeep,customizer,key,object,stackA,stackB){var result;if(customizer){result=object?customizer(value,key,object):customizer(value)}if(result!==undefined){return result}if(!isObject(value)){return value}var isArr=isArray(value);if(isArr){result=initCloneArray(value);if(!isDeep){return arrayCopy(value,result)}}else{var tag=objToString.call(value),isFunc=tag==funcTag;if(tag==objectTag||tag==argsTag||isFunc&&!object){result=initCloneObject(isFunc?{}:value);if(!isDeep){return baseAssign(result,value)}}else{return cloneableTags[tag]?initCloneByTag(value,tag,isDeep):object?value:{}}}stackA||(stackA=[]);stackB||(stackB=[]);var length=stackA.length;while(length--){if(stackA[length]==value){return stackB[length]}}stackA.push(value);stackB.push(result);(isArr?arrayEach:baseForOwn)(value,function(subValue,key){result[key]=baseClone(subValue,isDeep,customizer,key,value,stackA,stackB)});return result}var baseCreate=function(){function Object(){}return function(prototype){if(isObject(prototype)){Object.prototype=prototype;var result=new Object;Object.prototype=null}return result||context.Object()}}();function baseDelay(func,wait,args){if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}return setTimeout(function(){func.apply(undefined,args)},wait)}function baseDifference(array,values){var length=array?array.length:0,result=[];if(!length){return result}var index=-1,indexOf=getIndexOf(),isCommon=indexOf==baseIndexOf,cache=isCommon&&values.length>=200?createCache(values):null,valuesLength=values.length;if(cache){indexOf=cacheIndexOf;isCommon=false;values=cache}outer:while(++index<length){var value=array[index];if(isCommon&&value===value){var valuesIndex=valuesLength;while(valuesIndex--){if(values[valuesIndex]===value){continue outer}}result.push(value)}else if(indexOf(values,value,0)<0){result.push(value)}}return result}var baseEach=createBaseEach(baseForOwn);var baseEachRight=createBaseEach(baseForOwnRight,true);function baseEvery(collection,predicate){var result=true;baseEach(collection,function(value,index,collection){result=!!predicate(value,index,collection);return result});return result}function baseFill(array,value,start,end){var length=array.length;start=start==null?0:+start||0;if(start<0){start=-start>length?0:length+start}end=end===undefined||end>length?length:+end||0;if(end<0){end+=length}length=start>end?0:end>>>0;start>>>=0;while(start<length){array[start++]=value}return array}function baseFilter(collection,predicate){var result=[];baseEach(collection,function(value,index,collection){if(predicate(value,index,collection)){result.push(value)}});return result}function baseFind(collection,predicate,eachFunc,retKey){var result;eachFunc(collection,function(value,key,collection){if(predicate(value,key,collection)){result=retKey?key:value;return false}});return result}function baseFlatten(array,isDeep,isStrict){var index=-1,length=array.length,resIndex=-1,result=[];while(++index<length){var value=array[index];if(isObjectLike(value)&&isLength(value.length)&&(isArray(value)||isArguments(value))){if(isDeep){value=baseFlatten(value,isDeep,isStrict)}var valIndex=-1,valLength=value.length;result.length+=valLength;while(++valIndex<valLength){result[++resIndex]=value[valIndex]}}else if(!isStrict){result[++resIndex]=value}}return result}var baseFor=createBaseFor();var baseForRight=createBaseFor(true);function baseForIn(object,iteratee){return baseFor(object,iteratee,keysIn)}function baseForOwn(object,iteratee){return baseFor(object,iteratee,keys)}function baseForOwnRight(object,iteratee){return baseForRight(object,iteratee,keys)}function baseFunctions(object,props){var index=-1,length=props.length,resIndex=-1,result=[];while(++index<length){var key=props[index];if(isFunction(object[key])){result[++resIndex]=key}}return result}function baseGet(object,path,pathKey){if(object==null){return}if(pathKey!==undefined&&pathKey in toObject(object)){path=[pathKey]}var index=-1,length=path.length;while(object!=null&&++index<length){var result=object=object[path[index]]}return result}function baseIsEqual(value,other,customizer,isLoose,stackA,stackB){if(value===other){return value!==0||1/value==1/other}var valType=typeof value,othType=typeof other;if(valType!="function"&&valType!="object"&&othType!="function"&&othType!="object"||value==null||other==null){return value!==value&&other!==other}return baseIsEqualDeep(value,other,baseIsEqual,customizer,isLoose,stackA,stackB)}function baseIsEqualDeep(object,other,equalFunc,customizer,isLoose,stackA,stackB){var objIsArr=isArray(object),othIsArr=isArray(other),objTag=arrayTag,othTag=arrayTag;if(!objIsArr){objTag=objToString.call(object);if(objTag==argsTag){objTag=objectTag}else if(objTag!=objectTag){objIsArr=isTypedArray(object)}}if(!othIsArr){othTag=objToString.call(other);if(othTag==argsTag){othTag=objectTag}else if(othTag!=objectTag){othIsArr=isTypedArray(other)}}var objIsObj=objTag==objectTag,othIsObj=othTag==objectTag,isSameTag=objTag==othTag;if(isSameTag&&!(objIsArr||objIsObj)){return equalByTag(object,other,objTag)}if(!isLoose){var valWrapped=objIsObj&&hasOwnProperty.call(object,"__wrapped__"),othWrapped=othIsObj&&hasOwnProperty.call(other,"__wrapped__");if(valWrapped||othWrapped){return equalFunc(valWrapped?object.value():object,othWrapped?other.value():other,customizer,isLoose,stackA,stackB)}}if(!isSameTag){return false}stackA||(stackA=[]);stackB||(stackB=[]);var length=stackA.length;while(length--){if(stackA[length]==object){return stackB[length]==other}}stackA.push(object);stackB.push(other);var result=(objIsArr?equalArrays:equalObjects)(object,other,equalFunc,customizer,isLoose,stackA,stackB);stackA.pop();stackB.pop();return result}function baseIsMatch(object,props,values,strictCompareFlags,customizer){var index=-1,length=props.length,noCustomizer=!customizer;while(++index<length){if(noCustomizer&&strictCompareFlags[index]?values[index]!==object[props[index]]:!(props[index]in object)){return false}}index=-1;while(++index<length){var key=props[index],objValue=object[key],srcValue=values[index];if(noCustomizer&&strictCompareFlags[index]){var result=objValue!==undefined||key in object}else{result=customizer?customizer(objValue,srcValue,key):undefined;if(result===undefined){result=baseIsEqual(srcValue,objValue,customizer,true)}}if(!result){return false}}return true}function baseMap(collection,iteratee){var index=-1,length=getLength(collection),result=isLength(length)?Array(length):[];baseEach(collection,function(value,key,collection){result[++index]=iteratee(value,key,collection)});return result}function baseMatches(source){var props=keys(source),length=props.length;if(!length){return constant(true)}if(length==1){var key=props[0],value=source[key];if(isStrictComparable(value)){return function(object){if(object==null){return false}return object[key]===value&&(value!==undefined||key in toObject(object))}}}var values=Array(length),strictCompareFlags=Array(length);while(length--){value=source[props[length]];values[length]=value;strictCompareFlags[length]=isStrictComparable(value)}return function(object){return object!=null&&baseIsMatch(toObject(object),props,values,strictCompareFlags)}}function baseMatchesProperty(path,value){var isArr=isArray(path),isCommon=isKey(path)&&isStrictComparable(value),pathKey=path+"";path=toPath(path);return function(object){if(object==null){return false}var key=pathKey;object=toObject(object);if((isArr||!isCommon)&&!(key in object)){object=path.length==1?object:baseGet(object,baseSlice(path,0,-1));if(object==null){return false}key=last(path);object=toObject(object)}return object[key]===value?value!==undefined||key in object:baseIsEqual(value,object[key],null,true)}}function baseMerge(object,source,customizer,stackA,stackB){if(!isObject(object)){return object}var isSrcArr=isLength(source.length)&&(isArray(source)||isTypedArray(source));if(!isSrcArr){var props=keys(source);push.apply(props,getSymbols(source))}arrayEach(props||source,function(srcValue,key){if(props){key=srcValue;srcValue=source[key]}if(isObjectLike(srcValue)){stackA||(stackA=[]);stackB||(stackB=[]);baseMergeDeep(object,source,key,baseMerge,customizer,stackA,stackB)}else{var value=object[key],result=customizer?customizer(value,srcValue,key,object,source):undefined,isCommon=result===undefined;if(isCommon){result=srcValue}if((isSrcArr||result!==undefined)&&(isCommon||(result===result?result!==value:value===value))){object[key]=result}}});return object}function baseMergeDeep(object,source,key,mergeFunc,customizer,stackA,stackB){var length=stackA.length,srcValue=source[key];while(length--){if(stackA[length]==srcValue){object[key]=stackB[length];return}}var value=object[key],result=customizer?customizer(value,srcValue,key,object,source):undefined,isCommon=result===undefined;if(isCommon){result=srcValue;if(isLength(srcValue.length)&&(isArray(srcValue)||isTypedArray(srcValue))){result=isArray(value)?value:getLength(value)?arrayCopy(value):[]}else if(isPlainObject(srcValue)||isArguments(srcValue)){result=isArguments(value)?toPlainObject(value):isPlainObject(value)?value:{}}else{isCommon=false}}stackA.push(srcValue);stackB.push(result);if(isCommon){object[key]=mergeFunc(result,srcValue,customizer,stackA,stackB)}else if(result===result?result!==value:value===value){object[key]=result}}function baseProperty(key){return function(object){return object==null?undefined:object[key]}}function basePropertyDeep(path){var pathKey=path+"";path=toPath(path);return function(object){return baseGet(object,path,pathKey)}}function basePullAt(array,indexes){var length=indexes.length;while(length--){var index=parseFloat(indexes[length]);if(index!=previous&&isIndex(index)){var previous=index;splice.call(array,index,1)}}return array}function baseRandom(min,max){return min+floor(nativeRandom()*(max-min+1))}function baseReduce(collection,iteratee,accumulator,initFromCollection,eachFunc){eachFunc(collection,function(value,index,collection){accumulator=initFromCollection?(initFromCollection=false,value):iteratee(accumulator,value,index,collection)});return accumulator}var baseSetData=!metaMap?identity:function(func,data){metaMap.set(func,data);return func};function baseSlice(array,start,end){var index=-1,length=array.length;start=start==null?0:+start||0;if(start<0){start=-start>length?0:length+start}end=end===undefined||end>length?length:+end||0;if(end<0){end+=length}length=start>end?0:end-start>>>0;start>>>=0;var result=Array(length);while(++index<length){result[index]=array[index+start]}return result}function baseSome(collection,predicate){var result;baseEach(collection,function(value,index,collection){result=predicate(value,index,collection);return!result});return!!result}function baseSortBy(array,comparer){var length=array.length;array.sort(comparer);while(length--){array[length]=array[length].value}return array}function baseSortByOrder(collection,iteratees,orders){var callback=getCallback(),index=-1;iteratees=arrayMap(iteratees,function(iteratee){return callback(iteratee)});var result=baseMap(collection,function(value){var criteria=arrayMap(iteratees,function(iteratee){return iteratee(value)});return{criteria:criteria,index:++index,value:value}});return baseSortBy(result,function(object,other){return compareMultiple(object,other,orders)})}function baseSum(collection,iteratee){var result=0;baseEach(collection,function(value,index,collection){result+=+iteratee(value,index,collection)||0});return result}function baseUniq(array,iteratee){var index=-1,indexOf=getIndexOf(),length=array.length,isCommon=indexOf==baseIndexOf,isLarge=isCommon&&length>=200,seen=isLarge?createCache():null,result=[];if(seen){indexOf=cacheIndexOf;isCommon=false}else{isLarge=false;seen=iteratee?[]:result}outer:while(++index<length){var value=array[index],computed=iteratee?iteratee(value,index,array):value;if(isCommon&&value===value){var seenIndex=seen.length;while(seenIndex--){if(seen[seenIndex]===computed){continue outer}}if(iteratee){seen.push(computed)}result.push(value)}else if(indexOf(seen,computed,0)<0){if(iteratee||isLarge){seen.push(computed)}result.push(value)}}return result}function baseValues(object,props){var index=-1,length=props.length,result=Array(length);while(++index<length){result[index]=object[props[index]]}return result}function baseWhile(array,predicate,isDrop,fromRight){var length=array.length,index=fromRight?length:-1;while((fromRight?index--:++index<length)&&predicate(array[index],index,array)){}return isDrop?baseSlice(array,fromRight?0:index,fromRight?index+1:length):baseSlice(array,fromRight?index+1:0,fromRight?length:index)}function baseWrapperValue(value,actions){var result=value;if(result instanceof LazyWrapper){result=result.value()}var index=-1,length=actions.length;while(++index<length){var args=[result],action=actions[index];push.apply(args,action.args);result=action.func.apply(action.thisArg,args)}return result}function binaryIndex(array,value,retHighest){var low=0,high=array?array.length:low;if(typeof value=="number"&&value===value&&high<=HALF_MAX_ARRAY_LENGTH){while(low<high){var mid=low+high>>>1,computed=array[mid];if(retHighest?computed<=value:computed<value){low=mid+1}else{high=mid}}return high}return binaryIndexBy(array,value,identity,retHighest)}function binaryIndexBy(array,value,iteratee,retHighest){value=iteratee(value);var low=0,high=array?array.length:0,valIsNaN=value!==value,valIsUndef=value===undefined;while(low<high){var mid=floor((low+high)/2),computed=iteratee(array[mid]),isReflexive=computed===computed;if(valIsNaN){var setLow=isReflexive||retHighest}else if(valIsUndef){setLow=isReflexive&&(retHighest||computed!==undefined)}else{setLow=retHighest?computed<=value:computed<value}if(setLow){low=mid+1}else{high=mid}}return nativeMin(high,MAX_ARRAY_INDEX)}function bindCallback(func,thisArg,argCount){if(typeof func!="function"){return identity}if(thisArg===undefined){return func}switch(argCount){case 1:return function(value){return func.call(thisArg,value)};case 3:return function(value,index,collection){return func.call(thisArg,value,index,collection)};case 4:return function(accumulator,value,index,collection){return func.call(thisArg,accumulator,value,index,collection)};case 5:return function(value,other,key,object,source){return func.call(thisArg,value,other,key,object,source)}}return function(){return func.apply(thisArg,arguments)}}function bufferClone(buffer){return bufferSlice.call(buffer,0)}if(!bufferSlice){bufferClone=!(ArrayBuffer&&Uint8Array)?constant(null):function(buffer){var byteLength=buffer.byteLength,floatLength=Float64Array?floor(byteLength/FLOAT64_BYTES_PER_ELEMENT):0,offset=floatLength*FLOAT64_BYTES_PER_ELEMENT,result=new ArrayBuffer(byteLength);if(floatLength){var view=new Float64Array(result,0,floatLength);view.set(new Float64Array(buffer,0,floatLength))}if(byteLength!=offset){view=new Uint8Array(result,offset);view.set(new Uint8Array(buffer,offset))}return result}}function composeArgs(args,partials,holders){var holdersLength=holders.length,argsIndex=-1,argsLength=nativeMax(args.length-holdersLength,0),leftIndex=-1,leftLength=partials.length,result=Array(argsLength+leftLength);while(++leftIndex<leftLength){result[leftIndex]=partials[leftIndex]}while(++argsIndex<holdersLength){result[holders[argsIndex]]=args[argsIndex]}while(argsLength--){result[leftIndex++]=args[argsIndex++]}return result}function composeArgsRight(args,partials,holders){var holdersIndex=-1,holdersLength=holders.length,argsIndex=-1,argsLength=nativeMax(args.length-holdersLength,0),rightIndex=-1,rightLength=partials.length,result=Array(argsLength+rightLength);while(++argsIndex<argsLength){result[argsIndex]=args[argsIndex]}var pad=argsIndex;while(++rightIndex<rightLength){result[pad+rightIndex]=partials[rightIndex]}while(++holdersIndex<holdersLength){result[pad+holders[holdersIndex]]=args[argsIndex++]}return result}function createAggregator(setter,initializer){return function(collection,iteratee,thisArg){var result=initializer?initializer():{};iteratee=getCallback(iteratee,thisArg,3);if(isArray(collection)){var index=-1,length=collection.length;while(++index<length){var value=collection[index];setter(result,value,iteratee(value,index,collection),collection)}}else{baseEach(collection,function(value,key,collection){setter(result,value,iteratee(value,key,collection),collection)})}return result}}function createAssigner(assigner){return restParam(function(object,sources){var index=-1,length=object==null?0:sources.length,customizer=length>2&&sources[length-2],guard=length>2&&sources[2],thisArg=length>1&&sources[length-1];if(typeof customizer=="function"){customizer=bindCallback(customizer,thisArg,5);length-=2}else{customizer=typeof thisArg=="function"?thisArg:null;length-=customizer?1:0}if(guard&&isIterateeCall(sources[0],sources[1],guard)){customizer=length<3?null:customizer;length=1}while(++index<length){var source=sources[index];if(source){assigner(object,source,customizer)}}return object})}function createBaseEach(eachFunc,fromRight){return function(collection,iteratee){var length=collection?getLength(collection):0;if(!isLength(length)){return eachFunc(collection,iteratee)}var index=fromRight?length:-1,iterable=toObject(collection);while(fromRight?index--:++index<length){if(iteratee(iterable[index],index,iterable)===false){break}}return collection}}function createBaseFor(fromRight){return function(object,iteratee,keysFunc){var iterable=toObject(object),props=keysFunc(object),length=props.length,index=fromRight?length:-1;while(fromRight?index--:++index<length){var key=props[index];if(iteratee(iterable[key],key,iterable)===false){break}}return object}}function createBindWrapper(func,thisArg){var Ctor=createCtorWrapper(func);function wrapper(){var fn=this&&this!==root&&this instanceof wrapper?Ctor:func;return fn.apply(thisArg,arguments)}return wrapper}var createCache=!(nativeCreate&&Set)?constant(null):function(values){return new SetCache(values)};function createCompounder(callback){return function(string){var index=-1,array=words(deburr(string)),length=array.length,result="";while(++index<length){result=callback(result,array[index],index)}return result}}function createCtorWrapper(Ctor){return function(){var thisBinding=baseCreate(Ctor.prototype),result=Ctor.apply(thisBinding,arguments);return isObject(result)?result:thisBinding}}function createCurry(flag){function curryFunc(func,arity,guard){if(guard&&isIterateeCall(func,arity,guard)){arity=null}var result=createWrapper(func,flag,null,null,null,null,null,arity);result.placeholder=curryFunc.placeholder;return result}return curryFunc}function createExtremum(arrayFunc,isMin){return function(collection,iteratee,thisArg){if(thisArg&&isIterateeCall(collection,iteratee,thisArg)){iteratee=null}var func=getCallback(),noIteratee=iteratee==null;if(!(func===baseCallback&&noIteratee)){noIteratee=false;iteratee=func(iteratee,thisArg,3)}if(noIteratee){var isArr=isArray(collection);if(!isArr&&isString(collection)){iteratee=charAtCallback}else{return arrayFunc(isArr?collection:toIterable(collection))}}return extremumBy(collection,iteratee,isMin)}}function createFind(eachFunc,fromRight){return function(collection,predicate,thisArg){predicate=getCallback(predicate,thisArg,3);if(isArray(collection)){var index=baseFindIndex(collection,predicate,fromRight);return index>-1?collection[index]:undefined}return baseFind(collection,predicate,eachFunc)}}function createFindIndex(fromRight){return function(array,predicate,thisArg){if(!(array&&array.length)){return-1}predicate=getCallback(predicate,thisArg,3);return baseFindIndex(array,predicate,fromRight)}}function createFindKey(objectFunc){return function(object,predicate,thisArg){predicate=getCallback(predicate,thisArg,3);return baseFind(object,predicate,objectFunc,true)}}function createFlow(fromRight){return function(){var length=arguments.length;if(!length){return function(){return arguments[0]}}var wrapper,index=fromRight?length:-1,leftIndex=0,funcs=Array(length);while(fromRight?index--:++index<length){var func=funcs[leftIndex++]=arguments[index];if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}var funcName=wrapper?"":getFuncName(func);wrapper=funcName=="wrapper"?new LodashWrapper([]):wrapper}index=wrapper?-1:length;while(++index<length){func=funcs[index];funcName=getFuncName(func);var data=funcName=="wrapper"?getData(func):null;if(data&&isLaziable(data[0])){wrapper=wrapper[getFuncName(data[0])].apply(wrapper,data[3])}else{wrapper=func.length==1&&isLaziable(func)?wrapper[funcName]():wrapper.thru(func)}}return function(){var args=arguments;if(wrapper&&args.length==1&&isArray(args[0])){return wrapper.plant(args[0]).value()}var index=0,result=funcs[index].apply(this,args);while(++index<length){result=funcs[index].call(this,result)}return result}}}function createForEach(arrayFunc,eachFunc){return function(collection,iteratee,thisArg){return typeof iteratee=="function"&&thisArg===undefined&&isArray(collection)?arrayFunc(collection,iteratee):eachFunc(collection,bindCallback(iteratee,thisArg,3))}}function createForIn(objectFunc){return function(object,iteratee,thisArg){if(typeof iteratee!="function"||thisArg!==undefined){iteratee=bindCallback(iteratee,thisArg,3)}return objectFunc(object,iteratee,keysIn)}}function createForOwn(objectFunc){return function(object,iteratee,thisArg){if(typeof iteratee!="function"||thisArg!==undefined){iteratee=bindCallback(iteratee,thisArg,3)}return objectFunc(object,iteratee)}}function createPadDir(fromRight){return function(string,length,chars){string=baseToString(string);return string&&(fromRight?string:"")+createPadding(string,length,chars)+(fromRight?"":string)}}function createPartial(flag){var partialFunc=restParam(function(func,partials){var holders=replaceHolders(partials,partialFunc.placeholder);return createWrapper(func,flag,null,partials,holders)});return partialFunc}function createReduce(arrayFunc,eachFunc){return function(collection,iteratee,accumulator,thisArg){var initFromArray=arguments.length<3;return typeof iteratee=="function"&&thisArg===undefined&&isArray(collection)?arrayFunc(collection,iteratee,accumulator,initFromArray):baseReduce(collection,getCallback(iteratee,thisArg,4),accumulator,initFromArray,eachFunc)}}function createHybridWrapper(func,bitmask,thisArg,partials,holders,partialsRight,holdersRight,argPos,ary,arity){var isAry=bitmask&ARY_FLAG,isBind=bitmask&BIND_FLAG,isBindKey=bitmask&BIND_KEY_FLAG,isCurry=bitmask&CURRY_FLAG,isCurryBound=bitmask&CURRY_BOUND_FLAG,isCurryRight=bitmask&CURRY_RIGHT_FLAG;var Ctor=!isBindKey&&createCtorWrapper(func),key=func;function wrapper(){var length=arguments.length,index=length,args=Array(length);while(index--){args[index]=arguments[index]}if(partials){args=composeArgs(args,partials,holders)}if(partialsRight){args=composeArgsRight(args,partialsRight,holdersRight)}if(isCurry||isCurryRight){var placeholder=wrapper.placeholder,argsHolders=replaceHolders(args,placeholder);length-=argsHolders.length;if(length<arity){var newArgPos=argPos?arrayCopy(argPos):null,newArity=nativeMax(arity-length,0),newsHolders=isCurry?argsHolders:null,newHoldersRight=isCurry?null:argsHolders,newPartials=isCurry?args:null,newPartialsRight=isCurry?null:args;bitmask|=isCurry?PARTIAL_FLAG:PARTIAL_RIGHT_FLAG;bitmask&=~(isCurry?PARTIAL_RIGHT_FLAG:PARTIAL_FLAG);if(!isCurryBound){bitmask&=~(BIND_FLAG|BIND_KEY_FLAG)}var newData=[func,bitmask,thisArg,newPartials,newsHolders,newPartialsRight,newHoldersRight,newArgPos,ary,newArity],result=createHybridWrapper.apply(undefined,newData);
if(isLaziable(func)){setData(result,newData)}result.placeholder=placeholder;return result}}var thisBinding=isBind?thisArg:this;if(isBindKey){func=thisBinding[key]}if(argPos){args=reorder(args,argPos)}if(isAry&&ary<args.length){args.length=ary}var fn=this&&this!==root&&this instanceof wrapper?Ctor||createCtorWrapper(func):func;return fn.apply(thisBinding,args)}return wrapper}function createPadding(string,length,chars){var strLength=string.length;length=+length;if(strLength>=length||!nativeIsFinite(length)){return""}var padLength=length-strLength;chars=chars==null?" ":chars+"";return repeat(chars,ceil(padLength/chars.length)).slice(0,padLength)}function createPartialWrapper(func,bitmask,thisArg,partials){var isBind=bitmask&BIND_FLAG,Ctor=createCtorWrapper(func);function wrapper(){var argsIndex=-1,argsLength=arguments.length,leftIndex=-1,leftLength=partials.length,args=Array(argsLength+leftLength);while(++leftIndex<leftLength){args[leftIndex]=partials[leftIndex]}while(argsLength--){args[leftIndex++]=arguments[++argsIndex]}var fn=this&&this!==root&&this instanceof wrapper?Ctor:func;return fn.apply(isBind?thisArg:this,args)}return wrapper}function createSortedIndex(retHighest){return function(array,value,iteratee,thisArg){var func=getCallback(iteratee);return func===baseCallback&&iteratee==null?binaryIndex(array,value,retHighest):binaryIndexBy(array,value,func(iteratee,thisArg,1),retHighest)}}function createWrapper(func,bitmask,thisArg,partials,holders,argPos,ary,arity){var isBindKey=bitmask&BIND_KEY_FLAG;if(!isBindKey&&typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}var length=partials?partials.length:0;if(!length){bitmask&=~(PARTIAL_FLAG|PARTIAL_RIGHT_FLAG);partials=holders=null}length-=holders?holders.length:0;if(bitmask&PARTIAL_RIGHT_FLAG){var partialsRight=partials,holdersRight=holders;partials=holders=null}var data=isBindKey?null:getData(func),newData=[func,bitmask,thisArg,partials,holders,partialsRight,holdersRight,argPos,ary,arity];if(data){mergeData(newData,data);bitmask=newData[1];arity=newData[9]}newData[9]=arity==null?isBindKey?0:func.length:nativeMax(arity-length,0)||0;if(bitmask==BIND_FLAG){var result=createBindWrapper(newData[0],newData[2])}else if((bitmask==PARTIAL_FLAG||bitmask==(BIND_FLAG|PARTIAL_FLAG))&&!newData[4].length){result=createPartialWrapper.apply(undefined,newData)}else{result=createHybridWrapper.apply(undefined,newData)}var setter=data?baseSetData:setData;return setter(result,newData)}function equalArrays(array,other,equalFunc,customizer,isLoose,stackA,stackB){var index=-1,arrLength=array.length,othLength=other.length,result=true;if(arrLength!=othLength&&!(isLoose&&othLength>arrLength)){return false}while(result&&++index<arrLength){var arrValue=array[index],othValue=other[index];result=undefined;if(customizer){result=isLoose?customizer(othValue,arrValue,index):customizer(arrValue,othValue,index)}if(result===undefined){if(isLoose){var othIndex=othLength;while(othIndex--){othValue=other[othIndex];result=arrValue&&arrValue===othValue||equalFunc(arrValue,othValue,customizer,isLoose,stackA,stackB);if(result){break}}}else{result=arrValue&&arrValue===othValue||equalFunc(arrValue,othValue,customizer,isLoose,stackA,stackB)}}}return!!result}function equalByTag(object,other,tag){switch(tag){case boolTag:case dateTag:return+object==+other;case errorTag:return object.name==other.name&&object.message==other.message;case numberTag:return object!=+object?other!=+other:object==0?1/object==1/other:object==+other;case regexpTag:case stringTag:return object==other+""}return false}function equalObjects(object,other,equalFunc,customizer,isLoose,stackA,stackB){var objProps=keys(object),objLength=objProps.length,othProps=keys(other),othLength=othProps.length;if(objLength!=othLength&&!isLoose){return false}var skipCtor=isLoose,index=-1;while(++index<objLength){var key=objProps[index],result=isLoose?key in other:hasOwnProperty.call(other,key);if(result){var objValue=object[key],othValue=other[key];result=undefined;if(customizer){result=isLoose?customizer(othValue,objValue,key):customizer(objValue,othValue,key)}if(result===undefined){result=objValue&&objValue===othValue||equalFunc(objValue,othValue,customizer,isLoose,stackA,stackB)}}if(!result){return false}skipCtor||(skipCtor=key=="constructor")}if(!skipCtor){var objCtor=object.constructor,othCtor=other.constructor;if(objCtor!=othCtor&&("constructor"in object&&"constructor"in other)&&!(typeof objCtor=="function"&&objCtor instanceof objCtor&&typeof othCtor=="function"&&othCtor instanceof othCtor)){return false}}return true}function extremumBy(collection,iteratee,isMin){var exValue=isMin?POSITIVE_INFINITY:NEGATIVE_INFINITY,computed=exValue,result=computed;baseEach(collection,function(value,index,collection){var current=iteratee(value,index,collection);if((isMin?current<computed:current>computed)||current===exValue&&current===result){computed=current;result=value}});return result}function getCallback(func,thisArg,argCount){var result=lodash.callback||callback;result=result===callback?baseCallback:result;return argCount?result(func,thisArg,argCount):result}var getData=!metaMap?noop:function(func){return metaMap.get(func)};var getFuncName=function(){if(!support.funcNames){return constant("")}if(constant.name=="constant"){return baseProperty("name")}return function(func){var result=func.name,array=realNames[result],length=array?array.length:0;while(length--){var data=array[length],otherFunc=data.func;if(otherFunc==null||otherFunc==func){return data.name}}return result}}();function getIndexOf(collection,target,fromIndex){var result=lodash.indexOf||indexOf;result=result===indexOf?baseIndexOf:result;return collection?result(collection,target,fromIndex):result}var getLength=baseProperty("length");var getSymbols=!getOwnPropertySymbols?constant([]):function(object){return getOwnPropertySymbols(toObject(object))};function getView(start,end,transforms){var index=-1,length=transforms?transforms.length:0;while(++index<length){var data=transforms[index],size=data.size;switch(data.type){case"drop":start+=size;break;case"dropRight":end-=size;break;case"take":end=nativeMin(end,start+size);break;case"takeRight":start=nativeMax(start,end-size);break}}return{start:start,end:end}}function initCloneArray(array){var length=array.length,result=new array.constructor(length);if(length&&typeof array[0]=="string"&&hasOwnProperty.call(array,"index")){result.index=array.index;result.input=array.input}return result}function initCloneObject(object){var Ctor=object.constructor;if(!(typeof Ctor=="function"&&Ctor instanceof Ctor)){Ctor=Object}return new Ctor}function initCloneByTag(object,tag,isDeep){var Ctor=object.constructor;switch(tag){case arrayBufferTag:return bufferClone(object);case boolTag:case dateTag:return new Ctor(+object);case float32Tag:case float64Tag:case int8Tag:case int16Tag:case int32Tag:case uint8Tag:case uint8ClampedTag:case uint16Tag:case uint32Tag:var buffer=object.buffer;return new Ctor(isDeep?bufferClone(buffer):buffer,object.byteOffset,object.length);case numberTag:case stringTag:return new Ctor(object);case regexpTag:var result=new Ctor(object.source,reFlags.exec(object));result.lastIndex=object.lastIndex}return result}function invokePath(object,path,args){if(object!=null&&!isKey(path,object)){path=toPath(path);object=path.length==1?object:baseGet(object,baseSlice(path,0,-1));path=last(path)}var func=object==null?object:object[path];return func==null?undefined:func.apply(object,args)}function isIndex(value,length){value=+value;length=length==null?MAX_SAFE_INTEGER:length;return value>-1&&value%1==0&&value<length}function isIterateeCall(value,index,object){if(!isObject(object)){return false}var type=typeof index;if(type=="number"){var length=getLength(object),prereq=isLength(length)&&isIndex(index,length)}else{prereq=type=="string"&&index in object}if(prereq){var other=object[index];return value===value?value===other:other!==other}return false}function isKey(value,object){var type=typeof value;if(type=="string"&&reIsPlainProp.test(value)||type=="number"){return true}if(isArray(value)){return false}var result=!reIsDeepProp.test(value);return result||object!=null&&value in toObject(object)}function isLaziable(func){var funcName=getFuncName(func);return!!funcName&&func===lodash[funcName]&&funcName in LazyWrapper.prototype}function isLength(value){return typeof value=="number"&&value>-1&&value%1==0&&value<=MAX_SAFE_INTEGER}function isStrictComparable(value){return value===value&&(value===0?1/value>0:!isObject(value))}function mergeData(data,source){var bitmask=data[1],srcBitmask=source[1],newBitmask=bitmask|srcBitmask,isCommon=newBitmask<ARY_FLAG;var isCombo=srcBitmask==ARY_FLAG&&bitmask==CURRY_FLAG||srcBitmask==ARY_FLAG&&bitmask==REARG_FLAG&&data[7].length<=source[8]||srcBitmask==(ARY_FLAG|REARG_FLAG)&&bitmask==CURRY_FLAG;if(!(isCommon||isCombo)){return data}if(srcBitmask&BIND_FLAG){data[2]=source[2];newBitmask|=bitmask&BIND_FLAG?0:CURRY_BOUND_FLAG}var value=source[3];if(value){var partials=data[3];data[3]=partials?composeArgs(partials,value,source[4]):arrayCopy(value);data[4]=partials?replaceHolders(data[3],PLACEHOLDER):arrayCopy(source[4])}value=source[5];if(value){partials=data[5];data[5]=partials?composeArgsRight(partials,value,source[6]):arrayCopy(value);data[6]=partials?replaceHolders(data[5],PLACEHOLDER):arrayCopy(source[6])}value=source[7];if(value){data[7]=arrayCopy(value)}if(srcBitmask&ARY_FLAG){data[8]=data[8]==null?source[8]:nativeMin(data[8],source[8])}if(data[9]==null){data[9]=source[9]}data[0]=source[0];data[1]=newBitmask;return data}function pickByArray(object,props){object=toObject(object);var index=-1,length=props.length,result={};while(++index<length){var key=props[index];if(key in object){result[key]=object[key]}}return result}function pickByCallback(object,predicate){var result={};baseForIn(object,function(value,key,object){if(predicate(value,key,object)){result[key]=value}});return result}function reorder(array,indexes){var arrLength=array.length,length=nativeMin(indexes.length,arrLength),oldArray=arrayCopy(array);while(length--){var index=indexes[length];array[length]=isIndex(index,arrLength)?oldArray[index]:undefined}return array}var setData=function(){var count=0,lastCalled=0;return function(key,value){var stamp=now(),remaining=HOT_SPAN-(stamp-lastCalled);lastCalled=stamp;if(remaining>0){if(++count>=HOT_COUNT){return key}}else{count=0}return baseSetData(key,value)}}();function shimIsPlainObject(value){var Ctor,support=lodash.support;if(!(isObjectLike(value)&&objToString.call(value)==objectTag)||!hasOwnProperty.call(value,"constructor")&&(Ctor=value.constructor,typeof Ctor=="function"&&!(Ctor instanceof Ctor))){return false}var result;baseForIn(value,function(subValue,key){result=key});return result===undefined||hasOwnProperty.call(value,result)}function shimKeys(object){var props=keysIn(object),propsLength=props.length,length=propsLength&&object.length,support=lodash.support;var allowIndexes=length&&isLength(length)&&(isArray(object)||support.nonEnumArgs&&isArguments(object));var index=-1,result=[];while(++index<propsLength){var key=props[index];if(allowIndexes&&isIndex(key,length)||hasOwnProperty.call(object,key)){result.push(key)}}return result}function toIterable(value){if(value==null){return[]}if(!isLength(getLength(value))){return values(value)}return isObject(value)?value:Object(value)}function toObject(value){return isObject(value)?value:Object(value)}function toPath(value){if(isArray(value)){return value}var result=[];baseToString(value).replace(rePropName,function(match,number,quote,string){result.push(quote?string.replace(reEscapeChar,"$1"):number||match)});return result}function wrapperClone(wrapper){return wrapper instanceof LazyWrapper?wrapper.clone():new LodashWrapper(wrapper.__wrapped__,wrapper.__chain__,arrayCopy(wrapper.__actions__))}function chunk(array,size,guard){if(guard?isIterateeCall(array,size,guard):size==null){size=1}else{size=nativeMax(+size||1,1)}var index=0,length=array?array.length:0,resIndex=-1,result=Array(ceil(length/size));while(index<length){result[++resIndex]=baseSlice(array,index,index+=size)}return result}function compact(array){var index=-1,length=array?array.length:0,resIndex=-1,result=[];while(++index<length){var value=array[index];if(value){result[++resIndex]=value}}return result}var difference=restParam(function(array,values){return isArray(array)||isArguments(array)?baseDifference(array,baseFlatten(values,false,true)):[]});function drop(array,n,guard){var length=array?array.length:0;if(!length){return[]}if(guard?isIterateeCall(array,n,guard):n==null){n=1}return baseSlice(array,n<0?0:n)}function dropRight(array,n,guard){var length=array?array.length:0;if(!length){return[]}if(guard?isIterateeCall(array,n,guard):n==null){n=1}n=length-(+n||0);return baseSlice(array,0,n<0?0:n)}function dropRightWhile(array,predicate,thisArg){return array&&array.length?baseWhile(array,getCallback(predicate,thisArg,3),true,true):[]}function dropWhile(array,predicate,thisArg){return array&&array.length?baseWhile(array,getCallback(predicate,thisArg,3),true):[]}function fill(array,value,start,end){var length=array?array.length:0;if(!length){return[]}if(start&&typeof start!="number"&&isIterateeCall(array,value,start)){start=0;end=length}return baseFill(array,value,start,end)}var findIndex=createFindIndex();var findLastIndex=createFindIndex(true);function first(array){return array?array[0]:undefined}function flatten(array,isDeep,guard){var length=array?array.length:0;if(guard&&isIterateeCall(array,isDeep,guard)){isDeep=false}return length?baseFlatten(array,isDeep):[]}function flattenDeep(array){var length=array?array.length:0;return length?baseFlatten(array,true):[]}function indexOf(array,value,fromIndex){var length=array?array.length:0;if(!length){return-1}if(typeof fromIndex=="number"){fromIndex=fromIndex<0?nativeMax(length+fromIndex,0):fromIndex}else if(fromIndex){var index=binaryIndex(array,value),other=array[index];if(value===value?value===other:other!==other){return index}return-1}return baseIndexOf(array,value,fromIndex||0)}function initial(array){return dropRight(array,1)}function intersection(){var args=[],argsIndex=-1,argsLength=arguments.length,caches=[],indexOf=getIndexOf(),isCommon=indexOf==baseIndexOf,result=[];while(++argsIndex<argsLength){var value=arguments[argsIndex];if(isArray(value)||isArguments(value)){args.push(value);caches.push(isCommon&&value.length>=120?createCache(argsIndex&&value):null)}}argsLength=args.length;if(argsLength<2){return result}var array=args[0],index=-1,length=array?array.length:0,seen=caches[0];outer:while(++index<length){value=array[index];if((seen?cacheIndexOf(seen,value):indexOf(result,value,0))<0){argsIndex=argsLength;while(--argsIndex){var cache=caches[argsIndex];if((cache?cacheIndexOf(cache,value):indexOf(args[argsIndex],value,0))<0){continue outer}}if(seen){seen.push(value)}result.push(value)}}return result}function last(array){var length=array?array.length:0;return length?array[length-1]:undefined}function lastIndexOf(array,value,fromIndex){var length=array?array.length:0;if(!length){return-1}var index=length;if(typeof fromIndex=="number"){index=(fromIndex<0?nativeMax(length+fromIndex,0):nativeMin(fromIndex||0,length-1))+1}else if(fromIndex){index=binaryIndex(array,value,true)-1;var other=array[index];if(value===value?value===other:other!==other){return index}return-1}if(value!==value){return indexOfNaN(array,index,true)}while(index--){if(array[index]===value){return index}}return-1}function pull(){var args=arguments,array=args[0];if(!(array&&array.length)){return array}var index=0,indexOf=getIndexOf(),length=args.length;while(++index<length){var fromIndex=0,value=args[index];while((fromIndex=indexOf(array,value,fromIndex))>-1){splice.call(array,fromIndex,1)}}return array}var pullAt=restParam(function(array,indexes){array||(array=[]);indexes=baseFlatten(indexes);var result=baseAt(array,indexes);basePullAt(array,indexes.sort(baseCompareAscending));return result});function remove(array,predicate,thisArg){var result=[];if(!(array&&array.length)){return result}var index=-1,indexes=[],length=array.length;predicate=getCallback(predicate,thisArg,3);while(++index<length){var value=array[index];if(predicate(value,index,array)){result.push(value);indexes.push(index)}}basePullAt(array,indexes);return result}function rest(array){return drop(array,1)}function slice(array,start,end){var length=array?array.length:0;if(!length){return[]}if(end&&typeof end!="number"&&isIterateeCall(array,start,end)){start=0;end=length}return baseSlice(array,start,end)}var sortedIndex=createSortedIndex();var sortedLastIndex=createSortedIndex(true);function take(array,n,guard){var length=array?array.length:0;if(!length){return[]}if(guard?isIterateeCall(array,n,guard):n==null){n=1}return baseSlice(array,0,n<0?0:n)}function takeRight(array,n,guard){var length=array?array.length:0;if(!length){return[]}if(guard?isIterateeCall(array,n,guard):n==null){n=1}n=length-(+n||0);return baseSlice(array,n<0?0:n)}function takeRightWhile(array,predicate,thisArg){return array&&array.length?baseWhile(array,getCallback(predicate,thisArg,3),false,true):[]}function takeWhile(array,predicate,thisArg){return array&&array.length?baseWhile(array,getCallback(predicate,thisArg,3)):[]}var union=restParam(function(arrays){return baseUniq(baseFlatten(arrays,false,true))});function uniq(array,isSorted,iteratee,thisArg){var length=array?array.length:0;if(!length){return[]}if(isSorted!=null&&typeof isSorted!="boolean"){thisArg=iteratee;iteratee=isIterateeCall(array,isSorted,thisArg)?null:isSorted;isSorted=false}var func=getCallback();if(!(func===baseCallback&&iteratee==null)){iteratee=func(iteratee,thisArg,3)}return isSorted&&getIndexOf()==baseIndexOf?sortedUniq(array,iteratee):baseUniq(array,iteratee)}function unzip(array){var index=-1,length=(array&&array.length&&arrayMax(arrayMap(array,getLength)))>>>0,result=Array(length);while(++index<length){result[index]=arrayMap(array,baseProperty(index))}return result}var without=restParam(function(array,values){return isArray(array)||isArguments(array)?baseDifference(array,values):[]});function xor(){var index=-1,length=arguments.length;while(++index<length){var array=arguments[index];if(isArray(array)||isArguments(array)){var result=result?baseDifference(result,array).concat(baseDifference(array,result)):array}}return result?baseUniq(result):[]}var zip=restParam(unzip);function zipObject(props,values){var index=-1,length=props?props.length:0,result={};if(length&&!values&&!isArray(props[0])){values=[]}while(++index<length){var key=props[index];if(values){result[key]=values[index]}else if(key){result[key[0]]=key[1]}}return result}function chain(value){var result=lodash(value);result.__chain__=true;return result}function tap(value,interceptor,thisArg){interceptor.call(thisArg,value);return value}function thru(value,interceptor,thisArg){return interceptor.call(thisArg,value)}function wrapperChain(){return chain(this)}function wrapperCommit(){return new LodashWrapper(this.value(),this.__chain__)}function wrapperPlant(value){var result,parent=this;while(parent instanceof baseLodash){var clone=wrapperClone(parent);if(result){previous.__wrapped__=clone}else{result=clone}var previous=clone;parent=parent.__wrapped__}previous.__wrapped__=value;return result}function wrapperReverse(){var value=this.__wrapped__;if(value instanceof LazyWrapper){if(this.__actions__.length){value=new LazyWrapper(this)}return new LodashWrapper(value.reverse(),this.__chain__)}return this.thru(function(value){return value.reverse()})}function wrapperToString(){return this.value()+""}function wrapperValue(){return baseWrapperValue(this.__wrapped__,this.__actions__)}var at=restParam(function(collection,props){var length=collection?getLength(collection):0;if(isLength(length)){collection=toIterable(collection)}return baseAt(collection,baseFlatten(props))});var countBy=createAggregator(function(result,value,key){hasOwnProperty.call(result,key)?++result[key]:result[key]=1});function every(collection,predicate,thisArg){var func=isArray(collection)?arrayEvery:baseEvery;if(thisArg&&isIterateeCall(collection,predicate,thisArg)){predicate=null}if(typeof predicate!="function"||thisArg!==undefined){predicate=getCallback(predicate,thisArg,3)}return func(collection,predicate)}function filter(collection,predicate,thisArg){var func=isArray(collection)?arrayFilter:baseFilter;predicate=getCallback(predicate,thisArg,3);return func(collection,predicate)}var find=createFind(baseEach);var findLast=createFind(baseEachRight,true);function findWhere(collection,source){return find(collection,baseMatches(source))}var forEach=createForEach(arrayEach,baseEach);var forEachRight=createForEach(arrayEachRight,baseEachRight);var groupBy=createAggregator(function(result,value,key){if(hasOwnProperty.call(result,key)){result[key].push(value)}else{result[key]=[value]}});function includes(collection,target,fromIndex,guard){var length=collection?getLength(collection):0;if(!isLength(length)){collection=values(collection);length=collection.length}if(!length){return false}if(typeof fromIndex!="number"||guard&&isIterateeCall(target,fromIndex,guard)){fromIndex=0}else{fromIndex=fromIndex<0?nativeMax(length+fromIndex,0):fromIndex||0}return typeof collection=="string"||!isArray(collection)&&isString(collection)?fromIndex<length&&collection.indexOf(target,fromIndex)>-1:getIndexOf(collection,target,fromIndex)>-1}var indexBy=createAggregator(function(result,value,key){result[key]=value});var invoke=restParam(function(collection,path,args){var index=-1,isFunc=typeof path=="function",isProp=isKey(path),length=getLength(collection),result=isLength(length)?Array(length):[];baseEach(collection,function(value){var func=isFunc?path:isProp&&value!=null&&value[path];result[++index]=func?func.apply(value,args):invokePath(value,path,args)});return result});function map(collection,iteratee,thisArg){var func=isArray(collection)?arrayMap:baseMap;iteratee=getCallback(iteratee,thisArg,3);return func(collection,iteratee)}var partition=createAggregator(function(result,value,key){result[key?0:1].push(value)},function(){return[[],[]]});function pluck(collection,path){return map(collection,property(path))}var reduce=createReduce(arrayReduce,baseEach);var reduceRight=createReduce(arrayReduceRight,baseEachRight);function reject(collection,predicate,thisArg){var func=isArray(collection)?arrayFilter:baseFilter;predicate=getCallback(predicate,thisArg,3);return func(collection,function(value,index,collection){return!predicate(value,index,collection)})}function sample(collection,n,guard){if(guard?isIterateeCall(collection,n,guard):n==null){collection=toIterable(collection);var length=collection.length;return length>0?collection[baseRandom(0,length-1)]:undefined}var result=shuffle(collection);result.length=nativeMin(n<0?0:+n||0,result.length);return result}function shuffle(collection){collection=toIterable(collection);var index=-1,length=collection.length,result=Array(length);while(++index<length){var rand=baseRandom(0,index);if(index!=rand){result[index]=result[rand]}result[rand]=collection[index]}return result}function size(collection){var length=collection?getLength(collection):0;return isLength(length)?length:keys(collection).length}function some(collection,predicate,thisArg){var func=isArray(collection)?arraySome:baseSome;if(thisArg&&isIterateeCall(collection,predicate,thisArg)){predicate=null}if(typeof predicate!="function"||thisArg!==undefined){predicate=getCallback(predicate,thisArg,3)}return func(collection,predicate)}function sortBy(collection,iteratee,thisArg){if(collection==null){return[]}if(thisArg&&isIterateeCall(collection,iteratee,thisArg)){iteratee=null}var index=-1;iteratee=getCallback(iteratee,thisArg,3);var result=baseMap(collection,function(value,key,collection){return{criteria:iteratee(value,key,collection),index:++index,value:value}});return baseSortBy(result,compareAscending)}var sortByAll=restParam(function(collection,iteratees){if(collection==null){return[]}var guard=iteratees[2];if(guard&&isIterateeCall(iteratees[0],iteratees[1],guard)){iteratees.length=1}return baseSortByOrder(collection,baseFlatten(iteratees),[])});function sortByOrder(collection,iteratees,orders,guard){if(collection==null){return[]}if(guard&&isIterateeCall(iteratees,orders,guard)){orders=null}if(!isArray(iteratees)){iteratees=iteratees==null?[]:[iteratees]}if(!isArray(orders)){orders=orders==null?[]:[orders]}return baseSortByOrder(collection,iteratees,orders)}function where(collection,source){return filter(collection,baseMatches(source))}var now=nativeNow||function(){return(new Date).getTime()};function after(n,func){if(typeof func!="function"){if(typeof n=="function"){var temp=n;n=func;func=temp}else{throw new TypeError(FUNC_ERROR_TEXT)}}n=nativeIsFinite(n=+n)?n:0;return function(){if(--n<1){return func.apply(this,arguments)}}}function ary(func,n,guard){if(guard&&isIterateeCall(func,n,guard)){n=null}n=func&&n==null?func.length:nativeMax(+n||0,0);return createWrapper(func,ARY_FLAG,null,null,null,null,n)}function before(n,func){var result;if(typeof func!="function"){if(typeof n=="function"){var temp=n;n=func;func=temp}else{throw new TypeError(FUNC_ERROR_TEXT)}}return function(){if(--n>0){result=func.apply(this,arguments)}if(n<=1){func=null}return result}}var bind=restParam(function(func,thisArg,partials){var bitmask=BIND_FLAG;if(partials.length){var holders=replaceHolders(partials,bind.placeholder);bitmask|=PARTIAL_FLAG}return createWrapper(func,bitmask,thisArg,partials,holders)});var bindAll=restParam(function(object,methodNames){methodNames=methodNames.length?baseFlatten(methodNames):functions(object);var index=-1,length=methodNames.length;while(++index<length){var key=methodNames[index];object[key]=createWrapper(object[key],BIND_FLAG,object)}return object});var bindKey=restParam(function(object,key,partials){var bitmask=BIND_FLAG|BIND_KEY_FLAG;if(partials.length){var holders=replaceHolders(partials,bindKey.placeholder);bitmask|=PARTIAL_FLAG}return createWrapper(key,bitmask,object,partials,holders)});var curry=createCurry(CURRY_FLAG);var curryRight=createCurry(CURRY_RIGHT_FLAG);function debounce(func,wait,options){var args,maxTimeoutId,result,stamp,thisArg,timeoutId,trailingCall,lastCalled=0,maxWait=false,trailing=true;if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}wait=wait<0?0:+wait||0;if(options===true){var leading=true;trailing=false}else if(isObject(options)){leading=options.leading;maxWait="maxWait"in options&&nativeMax(+options.maxWait||0,wait);trailing="trailing"in options?options.trailing:trailing}function cancel(){if(timeoutId){clearTimeout(timeoutId)}if(maxTimeoutId){clearTimeout(maxTimeoutId)}maxTimeoutId=timeoutId=trailingCall=undefined}function delayed(){var remaining=wait-(now()-stamp);if(remaining<=0||remaining>wait){if(maxTimeoutId){clearTimeout(maxTimeoutId)}var isCalled=trailingCall;maxTimeoutId=timeoutId=trailingCall=undefined;if(isCalled){lastCalled=now();result=func.apply(thisArg,args);if(!timeoutId&&!maxTimeoutId){args=thisArg=null}}}else{timeoutId=setTimeout(delayed,remaining)}}function maxDelayed(){if(timeoutId){clearTimeout(timeoutId)}maxTimeoutId=timeoutId=trailingCall=undefined;if(trailing||maxWait!==wait){lastCalled=now();result=func.apply(thisArg,args);if(!timeoutId&&!maxTimeoutId){args=thisArg=null}}}function debounced(){args=arguments;stamp=now();thisArg=this;trailingCall=trailing&&(timeoutId||!leading);if(maxWait===false){var leadingCall=leading&&!timeoutId}else{if(!maxTimeoutId&&!leading){lastCalled=stamp}var remaining=maxWait-(stamp-lastCalled),isCalled=remaining<=0||remaining>maxWait;if(isCalled){if(maxTimeoutId){maxTimeoutId=clearTimeout(maxTimeoutId)}lastCalled=stamp;result=func.apply(thisArg,args)}else if(!maxTimeoutId){maxTimeoutId=setTimeout(maxDelayed,remaining)}}if(isCalled&&timeoutId){timeoutId=clearTimeout(timeoutId)}else if(!timeoutId&&wait!==maxWait){timeoutId=setTimeout(delayed,wait)}if(leadingCall){isCalled=true;result=func.apply(thisArg,args)}if(isCalled&&!timeoutId&&!maxTimeoutId){args=thisArg=null}return result}debounced.cancel=cancel;return debounced}var defer=restParam(function(func,args){return baseDelay(func,1,args)});var delay=restParam(function(func,wait,args){return baseDelay(func,wait,args)});var flow=createFlow();var flowRight=createFlow(true);function memoize(func,resolver){if(typeof func!="function"||resolver&&typeof resolver!="function"){throw new TypeError(FUNC_ERROR_TEXT)}var memoized=function(){var args=arguments,cache=memoized.cache,key=resolver?resolver.apply(this,args):args[0];if(cache.has(key)){return cache.get(key)}var result=func.apply(this,args);cache.set(key,result);return result};memoized.cache=new memoize.Cache;return memoized}function negate(predicate){if(typeof predicate!="function"){throw new TypeError(FUNC_ERROR_TEXT)}return function(){return!predicate.apply(this,arguments)}}function once(func){return before(2,func)}var partial=createPartial(PARTIAL_FLAG);var partialRight=createPartial(PARTIAL_RIGHT_FLAG);var rearg=restParam(function(func,indexes){return createWrapper(func,REARG_FLAG,null,null,null,baseFlatten(indexes))});function restParam(func,start){if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}start=nativeMax(start===undefined?func.length-1:+start||0,0);return function(){var args=arguments,index=-1,length=nativeMax(args.length-start,0),rest=Array(length);while(++index<length){rest[index]=args[start+index]}switch(start){case 0:return func.call(this,rest);case 1:return func.call(this,args[0],rest);case 2:return func.call(this,args[0],args[1],rest)}var otherArgs=Array(start+1);index=-1;while(++index<start){otherArgs[index]=args[index]}otherArgs[start]=rest;return func.apply(this,otherArgs)}}function spread(func){if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}return function(array){return func.apply(this,array)}}function throttle(func,wait,options){var leading=true,trailing=true;if(typeof func!="function"){throw new TypeError(FUNC_ERROR_TEXT)}if(options===false){leading=false}else if(isObject(options)){leading="leading"in options?!!options.leading:leading;trailing="trailing"in options?!!options.trailing:trailing}debounceOptions.leading=leading;debounceOptions.maxWait=+wait;debounceOptions.trailing=trailing;return debounce(func,wait,debounceOptions)}function wrap(value,wrapper){wrapper=wrapper==null?identity:wrapper;return createWrapper(wrapper,PARTIAL_FLAG,null,[value],[])}function clone(value,isDeep,customizer,thisArg){if(isDeep&&typeof isDeep!="boolean"&&isIterateeCall(value,isDeep,customizer)){isDeep=false}else if(typeof isDeep=="function"){thisArg=customizer;customizer=isDeep;isDeep=false}customizer=typeof customizer=="function"&&bindCallback(customizer,thisArg,1);return baseClone(value,isDeep,customizer)}function cloneDeep(value,customizer,thisArg){customizer=typeof customizer=="function"&&bindCallback(customizer,thisArg,1);return baseClone(value,true,customizer)}function isArguments(value){var length=isObjectLike(value)?value.length:undefined;return isLength(length)&&objToString.call(value)==argsTag}var isArray=nativeIsArray||function(value){return isObjectLike(value)&&isLength(value.length)&&objToString.call(value)==arrayTag};function isBoolean(value){return value===true||value===false||isObjectLike(value)&&objToString.call(value)==boolTag}function isDate(value){return isObjectLike(value)&&objToString.call(value)==dateTag}function isElement(value){return!!value&&value.nodeType===1&&isObjectLike(value)&&objToString.call(value).indexOf("Element")>-1}if(!support.dom){isElement=function(value){return!!value&&value.nodeType===1&&isObjectLike(value)&&!isPlainObject(value)}}function isEmpty(value){if(value==null){return true}var length=getLength(value);if(isLength(length)&&(isArray(value)||isString(value)||isArguments(value)||isObjectLike(value)&&isFunction(value.splice))){return!length}return!keys(value).length}function isEqual(value,other,customizer,thisArg){
customizer=typeof customizer=="function"&&bindCallback(customizer,thisArg,3);if(!customizer&&isStrictComparable(value)&&isStrictComparable(other)){return value===other}var result=customizer?customizer(value,other):undefined;return result===undefined?baseIsEqual(value,other,customizer):!!result}function isError(value){return isObjectLike(value)&&typeof value.message=="string"&&objToString.call(value)==errorTag}var isFinite=nativeNumIsFinite||function(value){return typeof value=="number"&&nativeIsFinite(value)};var isFunction=!(baseIsFunction(/x/)||Uint8Array&&!baseIsFunction(Uint8Array))?baseIsFunction:function(value){return objToString.call(value)==funcTag};function isObject(value){var type=typeof value;return type=="function"||!!value&&type=="object"}function isMatch(object,source,customizer,thisArg){var props=keys(source),length=props.length;if(!length){return true}if(object==null){return false}customizer=typeof customizer=="function"&&bindCallback(customizer,thisArg,3);object=toObject(object);if(!customizer&&length==1){var key=props[0],value=source[key];if(isStrictComparable(value)){return value===object[key]&&(value!==undefined||key in object)}}var values=Array(length),strictCompareFlags=Array(length);while(length--){value=values[length]=source[props[length]];strictCompareFlags[length]=isStrictComparable(value)}return baseIsMatch(object,props,values,strictCompareFlags,customizer)}function isNaN(value){return isNumber(value)&&value!=+value}function isNative(value){if(value==null){return false}if(objToString.call(value)==funcTag){return reIsNative.test(fnToString.call(value))}return isObjectLike(value)&&reIsHostCtor.test(value)}function isNull(value){return value===null}function isNumber(value){return typeof value=="number"||isObjectLike(value)&&objToString.call(value)==numberTag}var isPlainObject=!getPrototypeOf?shimIsPlainObject:function(value){if(!(value&&objToString.call(value)==objectTag)){return false}var valueOf=value.valueOf,objProto=isNative(valueOf)&&(objProto=getPrototypeOf(valueOf))&&getPrototypeOf(objProto);return objProto?value==objProto||getPrototypeOf(value)==objProto:shimIsPlainObject(value)};function isRegExp(value){return isObjectLike(value)&&objToString.call(value)==regexpTag||false}function isString(value){return typeof value=="string"||isObjectLike(value)&&objToString.call(value)==stringTag}function isTypedArray(value){return isObjectLike(value)&&isLength(value.length)&&!!typedArrayTags[objToString.call(value)]}function isUndefined(value){return value===undefined}function toArray(value){var length=value?getLength(value):0;if(!isLength(length)){return values(value)}if(!length){return[]}return arrayCopy(value)}function toPlainObject(value){return baseCopy(value,keysIn(value))}var assign=createAssigner(function(object,source,customizer){return customizer?assignWith(object,source,customizer):baseAssign(object,source)});function create(prototype,properties,guard){var result=baseCreate(prototype);if(guard&&isIterateeCall(prototype,properties,guard)){properties=null}return properties?baseAssign(result,properties):result}var defaults=restParam(function(args){var object=args[0];if(object==null){return object}args.push(assignDefaults);return assign.apply(undefined,args)});var findKey=createFindKey(baseForOwn);var findLastKey=createFindKey(baseForOwnRight);var forIn=createForIn(baseFor);var forInRight=createForIn(baseForRight);var forOwn=createForOwn(baseForOwn);var forOwnRight=createForOwn(baseForOwnRight);function functions(object){return baseFunctions(object,keysIn(object))}function get(object,path,defaultValue){var result=object==null?undefined:baseGet(object,toPath(path),path+"");return result===undefined?defaultValue:result}function has(object,path){if(object==null){return false}var result=hasOwnProperty.call(object,path);if(!result&&!isKey(path)){path=toPath(path);object=path.length==1?object:baseGet(object,baseSlice(path,0,-1));path=last(path);result=object!=null&&hasOwnProperty.call(object,path)}return result}function invert(object,multiValue,guard){if(guard&&isIterateeCall(object,multiValue,guard)){multiValue=null}var index=-1,props=keys(object),length=props.length,result={};while(++index<length){var key=props[index],value=object[key];if(multiValue){if(hasOwnProperty.call(result,value)){result[value].push(key)}else{result[value]=[key]}}else{result[value]=key}}return result}var keys=!nativeKeys?shimKeys:function(object){if(object){var Ctor=object.constructor,length=object.length}if(typeof Ctor=="function"&&Ctor.prototype===object||typeof object!="function"&&isLength(length)){return shimKeys(object)}return isObject(object)?nativeKeys(object):[]};function keysIn(object){if(object==null){return[]}if(!isObject(object)){object=Object(object)}var length=object.length;length=length&&isLength(length)&&(isArray(object)||support.nonEnumArgs&&isArguments(object))&&length||0;var Ctor=object.constructor,index=-1,isProto=typeof Ctor=="function"&&Ctor.prototype===object,result=Array(length),skipIndexes=length>0;while(++index<length){result[index]=index+""}for(var key in object){if(!(skipIndexes&&isIndex(key,length))&&!(key=="constructor"&&(isProto||!hasOwnProperty.call(object,key)))){result.push(key)}}return result}function mapValues(object,iteratee,thisArg){var result={};iteratee=getCallback(iteratee,thisArg,3);baseForOwn(object,function(value,key,object){result[key]=iteratee(value,key,object)});return result}var merge=createAssigner(baseMerge);var omit=restParam(function(object,props){if(object==null){return{}}if(typeof props[0]!="function"){var props=arrayMap(baseFlatten(props),String);return pickByArray(object,baseDifference(keysIn(object),props))}var predicate=bindCallback(props[0],props[1],3);return pickByCallback(object,function(value,key,object){return!predicate(value,key,object)})});function pairs(object){var index=-1,props=keys(object),length=props.length,result=Array(length);while(++index<length){var key=props[index];result[index]=[key,object[key]]}return result}var pick=restParam(function(object,props){if(object==null){return{}}return typeof props[0]=="function"?pickByCallback(object,bindCallback(props[0],props[1],3)):pickByArray(object,baseFlatten(props))});function result(object,path,defaultValue){var result=object==null?undefined:object[path];if(result===undefined){if(object!=null&&!isKey(path,object)){path=toPath(path);object=path.length==1?object:baseGet(object,baseSlice(path,0,-1));result=object==null?undefined:object[last(path)]}result=result===undefined?defaultValue:result}return isFunction(result)?result.call(object):result}function set(object,path,value){if(object==null){return object}var pathKey=path+"";path=object[pathKey]!=null||isKey(path,object)?[pathKey]:toPath(path);var index=-1,length=path.length,endIndex=length-1,nested=object;while(nested!=null&&++index<length){var key=path[index];if(isObject(nested)){if(index==endIndex){nested[key]=value}else if(nested[key]==null){nested[key]=isIndex(path[index+1])?[]:{}}}nested=nested[key]}return object}function transform(object,iteratee,accumulator,thisArg){var isArr=isArray(object)||isTypedArray(object);iteratee=getCallback(iteratee,thisArg,4);if(accumulator==null){if(isArr||isObject(object)){var Ctor=object.constructor;if(isArr){accumulator=isArray(object)?new Ctor:[]}else{accumulator=baseCreate(isFunction(Ctor)&&Ctor.prototype)}}else{accumulator={}}}(isArr?arrayEach:baseForOwn)(object,function(value,index,object){return iteratee(accumulator,value,index,object)});return accumulator}function values(object){return baseValues(object,keys(object))}function valuesIn(object){return baseValues(object,keysIn(object))}function inRange(value,start,end){start=+start||0;if(typeof end==="undefined"){end=start;start=0}else{end=+end||0}return value>=nativeMin(start,end)&&value<nativeMax(start,end)}function random(min,max,floating){if(floating&&isIterateeCall(min,max,floating)){max=floating=null}var noMin=min==null,noMax=max==null;if(floating==null){if(noMax&&typeof min=="boolean"){floating=min;min=1}else if(typeof max=="boolean"){floating=max;noMax=true}}if(noMin&&noMax){max=1;noMax=false}min=+min||0;if(noMax){max=min;min=0}else{max=+max||0}if(floating||min%1||max%1){var rand=nativeRandom();return nativeMin(min+rand*(max-min+parseFloat("1e-"+((rand+"").length-1))),max)}return baseRandom(min,max)}var camelCase=createCompounder(function(result,word,index){word=word.toLowerCase();return result+(index?word.charAt(0).toUpperCase()+word.slice(1):word)});function capitalize(string){string=baseToString(string);return string&&string.charAt(0).toUpperCase()+string.slice(1)}function deburr(string){string=baseToString(string);return string&&string.replace(reLatin1,deburrLetter).replace(reComboMark,"")}function endsWith(string,target,position){string=baseToString(string);target=target+"";var length=string.length;position=position===undefined?length:nativeMin(position<0?0:+position||0,length);position-=target.length;return position>=0&&string.indexOf(target,position)==position}function escape(string){string=baseToString(string);return string&&reHasUnescapedHtml.test(string)?string.replace(reUnescapedHtml,escapeHtmlChar):string}function escapeRegExp(string){string=baseToString(string);return string&&reHasRegExpChars.test(string)?string.replace(reRegExpChars,"\\$&"):string}var kebabCase=createCompounder(function(result,word,index){return result+(index?"-":"")+word.toLowerCase()});function pad(string,length,chars){string=baseToString(string);length=+length;var strLength=string.length;if(strLength>=length||!nativeIsFinite(length)){return string}var mid=(length-strLength)/2,leftLength=floor(mid),rightLength=ceil(mid);chars=createPadding("",rightLength,chars);return chars.slice(0,leftLength)+string+chars}var padLeft=createPadDir();var padRight=createPadDir(true);function parseInt(string,radix,guard){if(guard&&isIterateeCall(string,radix,guard)){radix=0}return nativeParseInt(string,radix)}if(nativeParseInt(whitespace+"08")!=8){parseInt=function(string,radix,guard){if(guard?isIterateeCall(string,radix,guard):radix==null){radix=0}else if(radix){radix=+radix}string=trim(string);return nativeParseInt(string,radix||(reHasHexPrefix.test(string)?16:10))}}function repeat(string,n){var result="";string=baseToString(string);n=+n;if(n<1||!string||!nativeIsFinite(n)){return result}do{if(n%2){result+=string}n=floor(n/2);string+=string}while(n);return result}var snakeCase=createCompounder(function(result,word,index){return result+(index?"_":"")+word.toLowerCase()});var startCase=createCompounder(function(result,word,index){return result+(index?" ":"")+(word.charAt(0).toUpperCase()+word.slice(1))});function startsWith(string,target,position){string=baseToString(string);position=position==null?0:nativeMin(position<0?0:+position||0,string.length);return string.lastIndexOf(target,position)==position}function template(string,options,otherOptions){var settings=lodash.templateSettings;if(otherOptions&&isIterateeCall(string,options,otherOptions)){options=otherOptions=null}string=baseToString(string);options=assignWith(baseAssign({},otherOptions||options),settings,assignOwnDefaults);var imports=assignWith(baseAssign({},options.imports),settings.imports,assignOwnDefaults),importsKeys=keys(imports),importsValues=baseValues(imports,importsKeys);var isEscaping,isEvaluating,index=0,interpolate=options.interpolate||reNoMatch,source="__p += '";var reDelimiters=RegExp((options.escape||reNoMatch).source+"|"+interpolate.source+"|"+(interpolate===reInterpolate?reEsTemplate:reNoMatch).source+"|"+(options.evaluate||reNoMatch).source+"|$","g");var sourceURL="//# sourceURL="+("sourceURL"in options?options.sourceURL:"lodash.templateSources["+ ++templateCounter+"]")+"\n";string.replace(reDelimiters,function(match,escapeValue,interpolateValue,esTemplateValue,evaluateValue,offset){interpolateValue||(interpolateValue=esTemplateValue);source+=string.slice(index,offset).replace(reUnescapedString,escapeStringChar);if(escapeValue){isEscaping=true;source+="' +\n__e("+escapeValue+") +\n'"}if(evaluateValue){isEvaluating=true;source+="';\n"+evaluateValue+";\n__p += '"}if(interpolateValue){source+="' +\n((__t = ("+interpolateValue+")) == null ? '' : __t) +\n'"}index=offset+match.length;return match});source+="';\n";var variable=options.variable;if(!variable){source="with (obj) {\n"+source+"\n}\n"}source=(isEvaluating?source.replace(reEmptyStringLeading,""):source).replace(reEmptyStringMiddle,"$1").replace(reEmptyStringTrailing,"$1;");source="function("+(variable||"obj")+") {\n"+(variable?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(isEscaping?", __e = _.escape":"")+(isEvaluating?", __j = Array.prototype.join;\n"+"function print() { __p += __j.call(arguments, '') }\n":";\n")+source+"return __p\n}";var result=attempt(function(){return Function(importsKeys,sourceURL+"return "+source).apply(undefined,importsValues)});result.source=source;if(isError(result)){throw result}return result}function trim(string,chars,guard){var value=string;string=baseToString(string);if(!string){return string}if(guard?isIterateeCall(value,chars,guard):chars==null){return string.slice(trimmedLeftIndex(string),trimmedRightIndex(string)+1)}chars=chars+"";return string.slice(charsLeftIndex(string,chars),charsRightIndex(string,chars)+1)}function trimLeft(string,chars,guard){var value=string;string=baseToString(string);if(!string){return string}if(guard?isIterateeCall(value,chars,guard):chars==null){return string.slice(trimmedLeftIndex(string))}return string.slice(charsLeftIndex(string,chars+""))}function trimRight(string,chars,guard){var value=string;string=baseToString(string);if(!string){return string}if(guard?isIterateeCall(value,chars,guard):chars==null){return string.slice(0,trimmedRightIndex(string)+1)}return string.slice(0,charsRightIndex(string,chars+"")+1)}function trunc(string,options,guard){if(guard&&isIterateeCall(string,options,guard)){options=null}var length=DEFAULT_TRUNC_LENGTH,omission=DEFAULT_TRUNC_OMISSION;if(options!=null){if(isObject(options)){var separator="separator"in options?options.separator:separator;length="length"in options?+options.length||0:length;omission="omission"in options?baseToString(options.omission):omission}else{length=+options||0}}string=baseToString(string);if(length>=string.length){return string}var end=length-omission.length;if(end<1){return omission}var result=string.slice(0,end);if(separator==null){return result+omission}if(isRegExp(separator)){if(string.slice(end).search(separator)){var match,newEnd,substring=string.slice(0,end);if(!separator.global){separator=RegExp(separator.source,(reFlags.exec(separator)||"")+"g")}separator.lastIndex=0;while(match=separator.exec(substring)){newEnd=match.index}result=result.slice(0,newEnd==null?end:newEnd)}}else if(string.indexOf(separator,end)!=end){var index=result.lastIndexOf(separator);if(index>-1){result=result.slice(0,index)}}return result+omission}function unescape(string){string=baseToString(string);return string&&reHasEscapedHtml.test(string)?string.replace(reEscapedHtml,unescapeHtmlChar):string}function words(string,pattern,guard){if(guard&&isIterateeCall(string,pattern,guard)){pattern=null}string=baseToString(string);return string.match(pattern||reWords)||[]}var attempt=restParam(function(func,args){try{return func.apply(undefined,args)}catch(e){return isError(e)?e:new Error(e)}});function callback(func,thisArg,guard){if(guard&&isIterateeCall(func,thisArg,guard)){thisArg=null}return baseCallback(func,thisArg)}function constant(value){return function(){return value}}function identity(value){return value}function matches(source){return baseMatches(baseClone(source,true))}function matchesProperty(path,value){return baseMatchesProperty(path,baseClone(value,true))}var method=restParam(function(path,args){return function(object){return invokePath(object,path,args)}});var methodOf=restParam(function(object,args){return function(path){return invokePath(object,path,args)}});function mixin(object,source,options){if(options==null){var isObj=isObject(source),props=isObj&&keys(source),methodNames=props&&props.length&&baseFunctions(source,props);if(!(methodNames?methodNames.length:isObj)){methodNames=false;options=source;source=object;object=this}}if(!methodNames){methodNames=baseFunctions(source,keys(source))}var chain=true,index=-1,isFunc=isFunction(object),length=methodNames.length;if(options===false){chain=false}else if(isObject(options)&&"chain"in options){chain=options.chain}while(++index<length){var methodName=methodNames[index],func=source[methodName];object[methodName]=func;if(isFunc){object.prototype[methodName]=function(func){return function(){var chainAll=this.__chain__;if(chain||chainAll){var result=object(this.__wrapped__),actions=result.__actions__=arrayCopy(this.__actions__);actions.push({func:func,args:arguments,thisArg:object});result.__chain__=chainAll;return result}var args=[this.value()];push.apply(args,arguments);return func.apply(object,args)}}(func)}}return object}function noConflict(){context._=oldDash;return this}function noop(){}function property(path){return isKey(path)?baseProperty(path):basePropertyDeep(path)}function propertyOf(object){return function(path){return baseGet(object,toPath(path),path+"")}}function range(start,end,step){if(step&&isIterateeCall(start,end,step)){end=step=null}start=+start||0;step=step==null?1:+step||0;if(end==null){end=start;start=0}else{end=+end||0}var index=-1,length=nativeMax(ceil((end-start)/(step||1)),0),result=Array(length);while(++index<length){result[index]=start;start+=step}return result}function times(n,iteratee,thisArg){n=floor(n);if(n<1||!nativeIsFinite(n)){return[]}var index=-1,result=Array(nativeMin(n,MAX_ARRAY_LENGTH));iteratee=bindCallback(iteratee,thisArg,1);while(++index<n){if(index<MAX_ARRAY_LENGTH){result[index]=iteratee(index)}else{iteratee(index)}}return result}function uniqueId(prefix){var id=++idCounter;return baseToString(prefix)+id}function add(augend,addend){return(+augend||0)+(+addend||0)}var max=createExtremum(arrayMax);var min=createExtremum(arrayMin,true);function sum(collection,iteratee,thisArg){if(thisArg&&isIterateeCall(collection,iteratee,thisArg)){iteratee=null}var func=getCallback(),noIteratee=iteratee==null;if(!(func===baseCallback&&noIteratee)){noIteratee=false;iteratee=func(iteratee,thisArg,3)}return noIteratee?arraySum(isArray(collection)?collection:toIterable(collection)):baseSum(collection,iteratee)}lodash.prototype=baseLodash.prototype;LodashWrapper.prototype=baseCreate(baseLodash.prototype);LodashWrapper.prototype.constructor=LodashWrapper;LazyWrapper.prototype=baseCreate(baseLodash.prototype);LazyWrapper.prototype.constructor=LazyWrapper;MapCache.prototype["delete"]=mapDelete;MapCache.prototype.get=mapGet;MapCache.prototype.has=mapHas;MapCache.prototype.set=mapSet;SetCache.prototype.push=cachePush;memoize.Cache=MapCache;lodash.after=after;lodash.ary=ary;lodash.assign=assign;lodash.at=at;lodash.before=before;lodash.bind=bind;lodash.bindAll=bindAll;lodash.bindKey=bindKey;lodash.callback=callback;lodash.chain=chain;lodash.chunk=chunk;lodash.compact=compact;lodash.constant=constant;lodash.countBy=countBy;lodash.create=create;lodash.curry=curry;lodash.curryRight=curryRight;lodash.debounce=debounce;lodash.defaults=defaults;lodash.defer=defer;lodash.delay=delay;lodash.difference=difference;lodash.drop=drop;lodash.dropRight=dropRight;lodash.dropRightWhile=dropRightWhile;lodash.dropWhile=dropWhile;lodash.fill=fill;lodash.filter=filter;lodash.flatten=flatten;lodash.flattenDeep=flattenDeep;lodash.flow=flow;lodash.flowRight=flowRight;lodash.forEach=forEach;lodash.forEachRight=forEachRight;lodash.forIn=forIn;lodash.forInRight=forInRight;lodash.forOwn=forOwn;lodash.forOwnRight=forOwnRight;lodash.functions=functions;lodash.groupBy=groupBy;lodash.indexBy=indexBy;lodash.initial=initial;lodash.intersection=intersection;lodash.invert=invert;lodash.invoke=invoke;lodash.keys=keys;lodash.keysIn=keysIn;lodash.map=map;lodash.mapValues=mapValues;lodash.matches=matches;lodash.matchesProperty=matchesProperty;lodash.memoize=memoize;lodash.merge=merge;lodash.method=method;lodash.methodOf=methodOf;lodash.mixin=mixin;lodash.negate=negate;lodash.omit=omit;lodash.once=once;lodash.pairs=pairs;lodash.partial=partial;lodash.partialRight=partialRight;lodash.partition=partition;lodash.pick=pick;lodash.pluck=pluck;lodash.property=property;lodash.propertyOf=propertyOf;lodash.pull=pull;lodash.pullAt=pullAt;lodash.range=range;lodash.rearg=rearg;lodash.reject=reject;lodash.remove=remove;lodash.rest=rest;lodash.restParam=restParam;lodash.set=set;lodash.shuffle=shuffle;lodash.slice=slice;lodash.sortBy=sortBy;lodash.sortByAll=sortByAll;lodash.sortByOrder=sortByOrder;lodash.spread=spread;lodash.take=take;lodash.takeRight=takeRight;lodash.takeRightWhile=takeRightWhile;lodash.takeWhile=takeWhile;lodash.tap=tap;lodash.throttle=throttle;lodash.thru=thru;lodash.times=times;lodash.toArray=toArray;lodash.toPlainObject=toPlainObject;lodash.transform=transform;lodash.union=union;lodash.uniq=uniq;lodash.unzip=unzip;lodash.values=values;lodash.valuesIn=valuesIn;lodash.where=where;lodash.without=without;lodash.wrap=wrap;lodash.xor=xor;lodash.zip=zip;lodash.zipObject=zipObject;lodash.backflow=flowRight;lodash.collect=map;lodash.compose=flowRight;lodash.each=forEach;lodash.eachRight=forEachRight;lodash.extend=assign;lodash.iteratee=callback;lodash.methods=functions;lodash.object=zipObject;lodash.select=filter;lodash.tail=rest;lodash.unique=uniq;mixin(lodash,lodash);lodash.add=add;lodash.attempt=attempt;lodash.camelCase=camelCase;lodash.capitalize=capitalize;lodash.clone=clone;lodash.cloneDeep=cloneDeep;lodash.deburr=deburr;lodash.endsWith=endsWith;lodash.escape=escape;lodash.escapeRegExp=escapeRegExp;lodash.every=every;lodash.find=find;lodash.findIndex=findIndex;lodash.findKey=findKey;lodash.findLast=findLast;lodash.findLastIndex=findLastIndex;lodash.findLastKey=findLastKey;lodash.findWhere=findWhere;lodash.first=first;lodash.get=get;lodash.has=has;lodash.identity=identity;lodash.includes=includes;lodash.indexOf=indexOf;lodash.inRange=inRange;lodash.isArguments=isArguments;lodash.isArray=isArray;lodash.isBoolean=isBoolean;lodash.isDate=isDate;lodash.isElement=isElement;lodash.isEmpty=isEmpty;lodash.isEqual=isEqual;lodash.isError=isError;lodash.isFinite=isFinite;lodash.isFunction=isFunction;lodash.isMatch=isMatch;lodash.isNaN=isNaN;lodash.isNative=isNative;lodash.isNull=isNull;lodash.isNumber=isNumber;lodash.isObject=isObject;lodash.isPlainObject=isPlainObject;lodash.isRegExp=isRegExp;lodash.isString=isString;lodash.isTypedArray=isTypedArray;lodash.isUndefined=isUndefined;lodash.kebabCase=kebabCase;lodash.last=last;lodash.lastIndexOf=lastIndexOf;lodash.max=max;lodash.min=min;lodash.noConflict=noConflict;lodash.noop=noop;lodash.now=now;lodash.pad=pad;lodash.padLeft=padLeft;lodash.padRight=padRight;lodash.parseInt=parseInt;lodash.random=random;lodash.reduce=reduce;lodash.reduceRight=reduceRight;lodash.repeat=repeat;lodash.result=result;lodash.runInContext=runInContext;lodash.size=size;lodash.snakeCase=snakeCase;lodash.some=some;lodash.sortedIndex=sortedIndex;lodash.sortedLastIndex=sortedLastIndex;lodash.startCase=startCase;lodash.startsWith=startsWith;lodash.sum=sum;lodash.template=template;lodash.trim=trim;lodash.trimLeft=trimLeft;lodash.trimRight=trimRight;lodash.trunc=trunc;lodash.unescape=unescape;lodash.uniqueId=uniqueId;lodash.words=words;lodash.all=every;lodash.any=some;lodash.contains=includes;lodash.detect=find;lodash.foldl=reduce;lodash.foldr=reduceRight;lodash.head=first;lodash.include=includes;lodash.inject=reduce;mixin(lodash,function(){var source={};baseForOwn(lodash,function(func,methodName){if(!lodash.prototype[methodName]){source[methodName]=func}});return source}(),false);lodash.sample=sample;lodash.prototype.sample=function(n){if(!this.__chain__&&n==null){return sample(this.value())}return this.thru(function(value){return sample(value,n)})};lodash.VERSION=VERSION;arrayEach(["bind","bindKey","curry","curryRight","partial","partialRight"],function(methodName){lodash[methodName].placeholder=lodash});arrayEach(["dropWhile","filter","map","takeWhile"],function(methodName,type){var isFilter=type!=LAZY_MAP_FLAG,isDropWhile=type==LAZY_DROP_WHILE_FLAG;LazyWrapper.prototype[methodName]=function(iteratee,thisArg){var filtered=this.__filtered__,result=filtered&&isDropWhile?new LazyWrapper(this):this.clone(),iteratees=result.__iteratees__||(result.__iteratees__=[]);iteratees.push({done:false,count:0,index:0,iteratee:getCallback(iteratee,thisArg,1),limit:-1,type:type});result.__filtered__=filtered||isFilter;return result}});arrayEach(["drop","take"],function(methodName,index){var whileName=methodName+"While";LazyWrapper.prototype[methodName]=function(n){var filtered=this.__filtered__,result=filtered&&!index?this.dropWhile():this.clone();n=n==null?1:nativeMax(floor(n)||0,0);if(filtered){if(index){result.__takeCount__=nativeMin(result.__takeCount__,n)}else{last(result.__iteratees__).limit=n}}else{var views=result.__views__||(result.__views__=[]);views.push({size:n,type:methodName+(result.__dir__<0?"Right":"")})}return result};LazyWrapper.prototype[methodName+"Right"]=function(n){return this.reverse()[methodName](n).reverse()};LazyWrapper.prototype[methodName+"RightWhile"]=function(predicate,thisArg){return this.reverse()[whileName](predicate,thisArg).reverse()}});arrayEach(["first","last"],function(methodName,index){var takeName="take"+(index?"Right":"");LazyWrapper.prototype[methodName]=function(){return this[takeName](1).value()[0]}});arrayEach(["initial","rest"],function(methodName,index){var dropName="drop"+(index?"":"Right");LazyWrapper.prototype[methodName]=function(){return this[dropName](1)}});arrayEach(["pluck","where"],function(methodName,index){var operationName=index?"filter":"map",createCallback=index?baseMatches:property;LazyWrapper.prototype[methodName]=function(value){return this[operationName](createCallback(value))}});LazyWrapper.prototype.compact=function(){return this.filter(identity)};LazyWrapper.prototype.reject=function(predicate,thisArg){predicate=getCallback(predicate,thisArg,1);return this.filter(function(value){return!predicate(value)})};LazyWrapper.prototype.slice=function(start,end){start=start==null?0:+start||0;var result=start<0?this.takeRight(-start):this.drop(start);if(end!==undefined){end=+end||0;result=end<0?result.dropRight(-end):result.take(end-start)}return result};LazyWrapper.prototype.toArray=function(){return this.drop(0)};baseForOwn(LazyWrapper.prototype,function(func,methodName){var lodashFunc=lodash[methodName];if(!lodashFunc){return}var checkIteratee=/^(?:filter|map|reject)|While$/.test(methodName),retUnwrapped=/^(?:first|last)$/.test(methodName);lodash.prototype[methodName]=function(){var args=arguments,length=args.length,chainAll=this.__chain__,value=this.__wrapped__,isHybrid=!!this.__actions__.length,isLazy=value instanceof LazyWrapper,iteratee=args[0],useLazy=isLazy||isArray(value);if(useLazy&&checkIteratee&&typeof iteratee=="function"&&iteratee.length!=1){isLazy=useLazy=false}var onlyLazy=isLazy&&!isHybrid;if(retUnwrapped&&!chainAll){return onlyLazy?func.call(value):lodashFunc.call(lodash,this.value())}var interceptor=function(value){var otherArgs=[value];push.apply(otherArgs,args);return lodashFunc.apply(lodash,otherArgs)};if(useLazy){var wrapper=onlyLazy?value:new LazyWrapper(this),result=func.apply(wrapper,args);if(!retUnwrapped&&(isHybrid||result.__actions__)){var actions=result.__actions__||(result.__actions__=[]);actions.push({func:thru,args:[interceptor],thisArg:lodash})}return new LodashWrapper(result,chainAll)}return this.thru(interceptor)}});arrayEach(["concat","join","pop","push","replace","shift","sort","splice","split","unshift"],function(methodName){var func=(/^(?:replace|split)$/.test(methodName)?stringProto:arrayProto)[methodName],chainName=/^(?:push|sort|unshift)$/.test(methodName)?"tap":"thru",retUnwrapped=/^(?:join|pop|replace|shift)$/.test(methodName);lodash.prototype[methodName]=function(){var args=arguments;if(retUnwrapped&&!this.__chain__){return func.apply(this.value(),args)}return this[chainName](function(value){return func.apply(value,args)})}});baseForOwn(LazyWrapper.prototype,function(func,methodName){var lodashFunc=lodash[methodName];if(lodashFunc){var key=lodashFunc.name,names=realNames[key]||(realNames[key]=[]);names.push({name:methodName,func:lodashFunc})}});realNames[createHybridWrapper(null,BIND_KEY_FLAG).name]=[{name:"wrapper",func:null}];LazyWrapper.prototype.clone=lazyClone;LazyWrapper.prototype.reverse=lazyReverse;LazyWrapper.prototype.value=lazyValue;lodash.prototype.chain=wrapperChain;lodash.prototype.commit=wrapperCommit;lodash.prototype.plant=wrapperPlant;lodash.prototype.reverse=wrapperReverse;lodash.prototype.toString=wrapperToString;lodash.prototype.run=lodash.prototype.toJSON=lodash.prototype.valueOf=lodash.prototype.value=wrapperValue;lodash.prototype.collect=lodash.prototype.map;lodash.prototype.head=lodash.prototype.first;lodash.prototype.select=lodash.prototype.filter;lodash.prototype.tail=lodash.prototype.rest;return lodash}var _=runInContext();if(typeof define=="function"&&typeof define.amd=="object"&&define.amd){root._=_;define(function(){return _})}else if(freeExports&&freeModule){if(moduleExports){(freeModule.exports=_)._=_}else{freeExports._=_}}else{root._=_}}).call(this)}).call(this,typeof global!=="undefined"?global:typeof self!=="undefined"?self:typeof window!=="undefined"?window:{})},{}],13:[function(require,module,exports){"use strict";var _=require("lodash");var events=require("events");var reg=require("./reg.js");var state=require("./state.js").state;var unicodeData=require("../data/ascii-identifier-data.js");var asciiIdentifierStartTable=unicodeData.asciiIdentifierStartTable;var asciiIdentifierPartTable=unicodeData.asciiIdentifierPartTable;var nonAsciiIdentifierStartTable=require("../data/non-ascii-identifier-start.js");var nonAsciiIdentifierPartTable=require("../data/non-ascii-identifier-part-only.js");var Token={Identifier:1,Punctuator:2,NumericLiteral:3,StringLiteral:4,Comment:5,Keyword:6,NullLiteral:7,BooleanLiteral:8,RegExp:9,TemplateHead:10,TemplateMiddle:11,TemplateTail:12,NoSubstTemplate:13};var Context={Block:1,Template:2};function isHex(str){return/^[0-9a-fA-F]+$/.test(str)}function isHexDigit(str){return str.length===1&&isHex(str)}function asyncTrigger(){var _checks=[];return{push:function(fn){_checks.push(fn)},check:function(){for(var check=0;check<_checks.length;++check){_checks[check]()}_checks.splice(0,_checks.length)}}}function Lexer(source){var lines=source;if(typeof lines==="string"){lines=lines.replace(/\r\n/g,"\n").replace(/\r/g,"\n").split("\n")}if(lines[0]&&lines[0].substr(0,2)==="#!"){if(lines[0].indexOf("node")!==-1){state.option.node=true}lines[0]=""}this.emitter=new events.EventEmitter;this.source=source;this.setLines(lines);this.prereg=true;this.line=0;this.char=1;this.from=1;this.input="";this.inComment=false;this.context=[];this.templateStarts=[];for(var i=0;i<state.option.indent;i+=1){state.tab+=" "}}Lexer.prototype={_lines:[],inContext:function(ctxType){return this.context.length>0&&this.context[this.context.length-1].type===ctxType},pushContext:function(ctxType){this.context.push({type:ctxType})},popContext:function(){return this.context.pop()},isContext:function(context){return this.context.length>0&&this.context[this.context.length-1]===context},currentContext:function(){return this.context.length>0&&this.context[this.context.length-1]},getLines:function(){this._lines=state.lines;return this._lines},setLines:function(val){this._lines=val;state.lines=this._lines},peek:function(i){return this.input.charAt(i||0)},skip:function(i){i=i||1;this.char+=i;this.input=this.input.slice(i)},on:function(names,listener){names.split(" ").forEach(function(name){this.emitter.on(name,listener)}.bind(this))},trigger:function(){this.emitter.emit.apply(this.emitter,Array.prototype.slice.call(arguments))},triggerAsync:function(type,args,checks,fn){checks.push(function(){if(fn()){
this.trigger(type,args)}}.bind(this))},scanPunctuator:function(){var ch1=this.peek();var ch2,ch3,ch4;switch(ch1){case".":if(/^[0-9]$/.test(this.peek(1))){return null}if(this.peek(1)==="."&&this.peek(2)==="."){return{type:Token.Punctuator,value:"..."}}case"(":case")":case";":case",":case"[":case"]":case":":case"~":case"?":return{type:Token.Punctuator,value:ch1};case"{":this.pushContext(Context.Block);return{type:Token.Punctuator,value:ch1};case"}":if(this.inContext(Context.Block)){this.popContext()}return{type:Token.Punctuator,value:ch1};case"#":return{type:Token.Punctuator,value:ch1};case"":return null}ch2=this.peek(1);ch3=this.peek(2);ch4=this.peek(3);if(ch1===">"&&ch2===">"&&ch3===">"&&ch4==="="){return{type:Token.Punctuator,value:">>>="}}if(ch1==="="&&ch2==="="&&ch3==="="){return{type:Token.Punctuator,value:"==="}}if(ch1==="!"&&ch2==="="&&ch3==="="){return{type:Token.Punctuator,value:"!=="}}if(ch1===">"&&ch2===">"&&ch3===">"){return{type:Token.Punctuator,value:">>>"}}if(ch1==="<"&&ch2==="<"&&ch3==="="){return{type:Token.Punctuator,value:"<<="}}if(ch1===">"&&ch2===">"&&ch3==="="){return{type:Token.Punctuator,value:">>="}}if(ch1==="="&&ch2===">"){return{type:Token.Punctuator,value:ch1+ch2}}if(ch1===ch2&&"+-<>&|".indexOf(ch1)>=0){return{type:Token.Punctuator,value:ch1+ch2}}if("<>=!+-*%&|^/".indexOf(ch1)>=0){if(ch2==="="){return{type:Token.Punctuator,value:ch1+ch2}}return{type:Token.Punctuator,value:ch1}}return null},scanComments:function(checks){var ch1=this.peek();var ch2=this.peek(1);var rest=this.input.substr(2);var startLine=this.line;var startChar=this.char;var self=this;function commentToken(label,body,opt){var special=["jshint","jslint","members","member","globals","global","exported"];var isSpecial=false;var value=label+body;var commentType="plain";opt=opt||{};if(opt.isMultiline){value+="*/"}body=body.replace(/\n/g," ");if(label==="/*"&&reg.fallsThrough.test(body)){isSpecial=true;commentType="falls through"}special.forEach(function(str){if(isSpecial){return}if(label==="//"&&str!=="jshint"){return}if(body.charAt(str.length)===" "&&body.substr(0,str.length)===str){isSpecial=true;label=label+str;body=body.substr(str.length)}if(!isSpecial&&body.charAt(0)===" "&&body.charAt(str.length+1)===" "&&body.substr(1,str.length)===str){isSpecial=true;label=label+" "+str;body=body.substr(str.length+1)}if(!isSpecial){return}switch(str){case"member":commentType="members";break;case"global":commentType="globals";break;default:var options=body.split(":").map(function(v){return v.replace(/^\s+/,"").replace(/\s+$/,"")});if(options.length===2){switch(options[0]){case"ignore":switch(options[1]){case"start":self.ignoringLinterErrors=true;isSpecial=false;break;case"end":self.ignoringLinterErrors=false;isSpecial=false;break}}}commentType=str}});return{type:Token.Comment,commentType:commentType,value:value,body:body,isSpecial:isSpecial,isMultiline:opt.isMultiline||false,isMalformed:opt.isMalformed||false}}if(ch1==="*"&&ch2==="/"){this.trigger("error",{code:"E018",line:startLine,character:startChar});this.skip(2);return null}if(ch1!=="/"||ch2!=="*"&&ch2!=="/"){return null}if(ch2==="/"){this.skip(this.input.length);return commentToken("//",rest)}var body="";if(ch2==="*"){this.inComment=true;this.skip(2);while(this.peek()!=="*"||this.peek(1)!=="/"){if(this.peek()===""){body+="\n";if(!this.nextLine(checks)){this.trigger("error",{code:"E017",line:startLine,character:startChar});this.inComment=false;return commentToken("/*",body,{isMultiline:true,isMalformed:true})}}else{body+=this.peek();this.skip()}}this.skip(2);this.inComment=false;return commentToken("/*",body,{isMultiline:true})}},scanKeyword:function(){var result=/^[a-zA-Z_$][a-zA-Z0-9_$]*/.exec(this.input);var keywords=["if","in","do","var","for","new","try","let","this","else","case","void","with","enum","while","break","catch","throw","const","yield","class","super","return","typeof","delete","switch","export","import","default","finally","extends","function","continue","debugger","instanceof"];if(result&&keywords.indexOf(result[0])>=0){return{type:Token.Keyword,value:result[0]}}return null},scanIdentifier:function(){var id="";var index=0;var type,char;function isNonAsciiIdentifierStart(code){return nonAsciiIdentifierStartTable.indexOf(code)>-1}function isNonAsciiIdentifierPart(code){return isNonAsciiIdentifierStart(code)||nonAsciiIdentifierPartTable.indexOf(code)>-1}var readUnicodeEscapeSequence=function(){index+=1;if(this.peek(index)!=="u"){return null}var sequence=this.peek(index+1)+this.peek(index+2)+this.peek(index+3)+this.peek(index+4);var code;if(isHex(sequence)){code=parseInt(sequence,16);if(asciiIdentifierPartTable[code]||isNonAsciiIdentifierPart(code)){index+=5;return"\\u"+sequence}return null}return null}.bind(this);var getIdentifierStart=function(){var chr=this.peek(index);var code=chr.charCodeAt(0);if(code===92){return readUnicodeEscapeSequence()}if(code<128){if(asciiIdentifierStartTable[code]){index+=1;return chr}return null}if(isNonAsciiIdentifierStart(code)){index+=1;return chr}return null}.bind(this);var getIdentifierPart=function(){var chr=this.peek(index);var code=chr.charCodeAt(0);if(code===92){return readUnicodeEscapeSequence()}if(code<128){if(asciiIdentifierPartTable[code]){index+=1;return chr}return null}if(isNonAsciiIdentifierPart(code)){index+=1;return chr}return null}.bind(this);function removeEscapeSequences(id){return id.replace(/\\u([0-9a-fA-F]{4})/g,function(m0,codepoint){return String.fromCharCode(parseInt(codepoint,16))})}char=getIdentifierStart();if(char===null){return null}id=char;for(;;){char=getIdentifierPart();if(char===null){break}id+=char}switch(id){case"true":case"false":type=Token.BooleanLiteral;break;case"null":type=Token.NullLiteral;break;default:type=Token.Identifier}return{type:type,value:removeEscapeSequences(id),text:id,tokenLength:id.length}},scanNumericLiteral:function(checks){var index=0;var value="";var length=this.input.length;var char=this.peek(index);var bad;var isAllowedDigit=isDecimalDigit;var base=10;var isLegacy=false;function isDecimalDigit(str){return/^[0-9]$/.test(str)}function isOctalDigit(str){return/^[0-7]$/.test(str)}function isBinaryDigit(str){return/^[01]$/.test(str)}function isIdentifierStart(ch){return ch==="$"||ch==="_"||ch==="\\"||ch>="a"&&ch<="z"||ch>="A"&&ch<="Z"}if(char!=="."&&!isDecimalDigit(char)){return null}if(char!=="."){value=this.peek(index);index+=1;char=this.peek(index);if(value==="0"){if(char==="x"||char==="X"){isAllowedDigit=isHexDigit;base=16;index+=1;value+=char}if(char==="o"||char==="O"){isAllowedDigit=isOctalDigit;base=8;if(!state.inES6(true)){this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Octal integer literal","6"]},checks,function(){return true})}index+=1;value+=char}if(char==="b"||char==="B"){isAllowedDigit=isBinaryDigit;base=2;if(!state.inES6(true)){this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Binary integer literal","6"]},checks,function(){return true})}index+=1;value+=char}if(isOctalDigit(char)){isAllowedDigit=isOctalDigit;base=8;isLegacy=true;bad=false;index+=1;value+=char}if(!isOctalDigit(char)&&isDecimalDigit(char)){index+=1;value+=char}}while(index<length){char=this.peek(index);if(isLegacy&&isDecimalDigit(char)){bad=true}else if(!isAllowedDigit(char)){break}value+=char;index+=1}if(isAllowedDigit!==isDecimalDigit){if(!isLegacy&&value.length<=2){return{type:Token.NumericLiteral,value:value,isMalformed:true}}if(index<length){char=this.peek(index);if(isIdentifierStart(char)){return null}}return{type:Token.NumericLiteral,value:value,base:base,isLegacy:isLegacy,isMalformed:false}}}if(char==="."){value+=char;index+=1;while(index<length){char=this.peek(index);if(!isDecimalDigit(char)){break}value+=char;index+=1}}if(char==="e"||char==="E"){value+=char;index+=1;char=this.peek(index);if(char==="+"||char==="-"){value+=this.peek(index);index+=1}char=this.peek(index);if(isDecimalDigit(char)){value+=char;index+=1;while(index<length){char=this.peek(index);if(!isDecimalDigit(char)){break}value+=char;index+=1}}else{return null}}if(index<length){char=this.peek(index);if(isIdentifierStart(char)){return null}}return{type:Token.NumericLiteral,value:value,base:base,isMalformed:!isFinite(value)}},scanEscapeSequence:function(checks){var allowNewLine=false;var jump=1;this.skip();var char=this.peek();switch(char){case"'":this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\'"]},checks,function(){return state.jsonMode});break;case"b":char="\\b";break;case"f":char="\\f";break;case"n":char="\\n";break;case"r":char="\\r";break;case"t":char="\\t";break;case"0":char="\\0";var n=parseInt(this.peek(1),10);this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},checks,function(){return n>=0&&n<=7&&state.isStrict()});break;case"1":case"2":case"3":case"4":case"5":case"6":case"7":char="\\"+char;this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},checks,function(){return state.isStrict()});break;case"u":var sequence=this.input.substr(1,4);var code=parseInt(sequence,16);if(!isHex(sequence)){this.trigger("warning",{code:"W052",line:this.line,character:this.char,data:["u"+sequence]})}char=String.fromCharCode(code);jump=5;break;case"v":this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\v"]},checks,function(){return state.jsonMode});char="";break;case"x":var x=parseInt(this.input.substr(1,2),16);this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\x-"]},checks,function(){return state.jsonMode});char=String.fromCharCode(x);jump=3;break;case"\\":char="\\\\";break;case'"':char='\\"';break;case"/":break;case"":allowNewLine=true;char="";break}return{"char":char,jump:jump,allowNewLine:allowNewLine}},scanTemplateLiteral:function(checks){var tokenType;var value="";var ch;var startLine=this.line;var startChar=this.char;var depth=this.templateStarts.length;if(this.peek()==="`"){if(!state.inES6(true)){this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["template literal syntax","6"]},checks,function(){return true})}tokenType=Token.TemplateHead;this.templateStarts.push({line:this.line,"char":this.char});depth=this.templateStarts.length;this.skip(1);this.pushContext(Context.Template)}else if(this.inContext(Context.Template)&&this.peek()==="}"){tokenType=Token.TemplateMiddle}else{return null}while(this.peek()!=="`"){while((ch=this.peek())===""){value+="\n";if(!this.nextLine(checks)){var startPos=this.templateStarts.pop();this.trigger("error",{code:"E052",line:startPos.line,character:startPos.char});return{type:tokenType,value:value,startLine:startLine,startChar:startChar,isUnclosed:true,depth:depth,context:this.popContext()}}}if(ch==="$"&&this.peek(1)==="{"){value+="${";this.skip(2);return{type:tokenType,value:value,startLine:startLine,startChar:startChar,isUnclosed:false,depth:depth,context:this.currentContext()}}else if(ch==="\\"){var escape=this.scanEscapeSequence(checks);value+=escape.char;this.skip(escape.jump)}else if(ch!=="`"){value+=ch;this.skip(1)}}tokenType=tokenType===Token.TemplateHead?Token.NoSubstTemplate:Token.TemplateTail;this.skip(1);this.templateStarts.pop();return{type:tokenType,value:value,startLine:startLine,startChar:startChar,isUnclosed:false,depth:depth,context:this.popContext()}},scanStringLiteral:function(checks){var quote=this.peek();if(quote!=='"'&&quote!=="'"){return null}this.triggerAsync("warning",{code:"W108",line:this.line,character:this.char},checks,function(){return state.jsonMode&&quote!=='"'});var value="";var startLine=this.line;var startChar=this.char;var allowNewLine=false;this.skip();while(this.peek()!==quote){if(this.peek()===""){if(!allowNewLine){this.trigger("warning",{code:"W112",line:this.line,character:this.char})}else{allowNewLine=false;this.triggerAsync("warning",{code:"W043",line:this.line,character:this.char},checks,function(){return!state.option.multistr});this.triggerAsync("warning",{code:"W042",line:this.line,character:this.char},checks,function(){return state.jsonMode&&state.option.multistr})}if(!this.nextLine(checks)){this.trigger("error",{code:"E029",line:startLine,character:startChar});return{type:Token.StringLiteral,value:value,startLine:startLine,startChar:startChar,isUnclosed:true,quote:quote}}}else{allowNewLine=false;var char=this.peek();var jump=1;if(char<" "){this.triggerAsync("warning",{code:"W113",line:this.line,character:this.char,data:["<non-printable>"]},checks,function(){return true})}if(char==="\\"){var parsed=this.scanEscapeSequence(checks);char=parsed.char;jump=parsed.jump;allowNewLine=parsed.allowNewLine}value+=char;this.skip(jump)}}this.skip();return{type:Token.StringLiteral,value:value,startLine:startLine,startChar:startChar,isUnclosed:false,quote:quote}},scanRegExp:function(checks){var index=0;var length=this.input.length;var char=this.peek();var value=char;var body="";var flags=[];var malformed=false;var isCharSet=false;var terminated,malformedDesc;var scanUnexpectedChars=function(){if(char<" "){malformed=true;this.triggerAsync("warning",{code:"W048",line:this.line,character:this.char},checks,function(){return true})}if(char==="<"){malformed=true;this.triggerAsync("warning",{code:"W049",line:this.line,character:this.char,data:[char]},checks,function(){return true})}}.bind(this);if(!this.prereg||char!=="/"){return null}index+=1;terminated=false;while(index<length){char=this.peek(index);value+=char;body+=char;if(isCharSet){if(char==="]"){if(this.peek(index-1)!=="\\"||this.peek(index-2)==="\\"){isCharSet=false}}if(char==="\\"){index+=1;char=this.peek(index);body+=char;value+=char;scanUnexpectedChars()}index+=1;continue}if(char==="\\"){index+=1;char=this.peek(index);body+=char;value+=char;scanUnexpectedChars();if(char==="/"){index+=1;continue}if(char==="["){index+=1;continue}}if(char==="["){isCharSet=true;index+=1;continue}if(char==="/"){body=body.substr(0,body.length-1);terminated=true;index+=1;break}index+=1}if(!terminated){this.trigger("error",{code:"E015",line:this.line,character:this.from});return void this.trigger("fatal",{line:this.line,from:this.from})}while(index<length){char=this.peek(index);if(!/[gimy]/.test(char)){break}if(char==="y"){if(!state.inES6(true)){this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Sticky RegExp flag","6"]},checks,function(){return true})}if(value.indexOf("y")>-1){malformedDesc="Duplicate RegExp flag"}}else{flags.push(char)}value+=char;index+=1}try{new RegExp(body,flags.join(""))}catch(err){malformedDesc=err.message}if(malformedDesc){malformed=true;this.trigger("error",{code:"E016",line:this.line,character:this.char,data:[malformedDesc]})}return{type:Token.RegExp,value:value,flags:flags,isMalformed:malformed}},scanNonBreakingSpaces:function(){return state.option.nonbsp?this.input.search(/(\u00A0)/):-1},scanUnsafeChars:function(){return this.input.search(reg.unsafeChars)},next:function(checks){this.from=this.char;while(/\s/.test(this.peek())){this.from+=1;this.skip()}var match=this.scanComments(checks)||this.scanStringLiteral(checks)||this.scanTemplateLiteral(checks);if(match){return match}match=this.scanRegExp(checks)||this.scanPunctuator()||this.scanKeyword()||this.scanIdentifier()||this.scanNumericLiteral(checks);if(match){this.skip(match.tokenLength||match.value.length);return match}return null},nextLine:function(checks){var char;if(this.line>=this.getLines().length){return false}this.input=this.getLines()[this.line];this.line+=1;this.char=1;this.from=1;var inputTrimmed=this.input.trim();var startsWith=function(){return _.some(arguments,function(prefix){return inputTrimmed.indexOf(prefix)===0})};var endsWith=function(){return _.some(arguments,function(suffix){return inputTrimmed.indexOf(suffix,inputTrimmed.length-suffix.length)!==-1})};if(this.ignoringLinterErrors===true){if(!startsWith("/*","//")&&!(this.inComment&&endsWith("*/"))){this.input=""}}char=this.scanNonBreakingSpaces();if(char>=0){this.triggerAsync("warning",{code:"W125",line:this.line,character:char+1},checks,function(){return true})}this.input=this.input.replace(/\t/g,state.tab);char=this.scanUnsafeChars();if(char>=0){this.triggerAsync("warning",{code:"W100",line:this.line,character:char},checks,function(){return true})}if(!this.ignoringLinterErrors&&state.option.maxlen&&state.option.maxlen<this.input.length){var inComment=this.inComment||startsWith.call(inputTrimmed,"//")||startsWith.call(inputTrimmed,"/*");var shouldTriggerError=!inComment||!reg.maxlenException.test(inputTrimmed);if(shouldTriggerError){this.triggerAsync("warning",{code:"W101",line:this.line,character:this.input.length},checks,function(){return true})}}return true},token:function(){var checks=asyncTrigger();var token;function isReserved(token,isProperty){if(!token.reserved){return false}var meta=token.meta;if(meta&&meta.isFutureReservedWord&&state.inES5()){if(!meta.es5){return false}if(meta.strictOnly){if(!state.option.strict&&!state.isStrict()){return false}}if(isProperty){return false}}return true}var create=function(type,value,isProperty,token){var obj;if(type!=="(endline)"&&type!=="(end)"){this.prereg=false}if(type==="(punctuator)"){switch(value){case".":case")":case"~":case"#":case"]":case"++":case"--":this.prereg=false;break;default:this.prereg=true}obj=Object.create(state.syntax[value]||state.syntax["(error)"])}if(type==="(identifier)"){if(value==="return"||value==="case"||value==="yield"||value==="typeof"||value==="instanceof"){this.prereg=true}if(_.has(state.syntax,value)){obj=Object.create(state.syntax[value]||state.syntax["(error)"]);if(!isReserved(obj,isProperty&&type==="(identifier)")){obj=null}}}if(type==="(template)"||type==="(template middle)"){this.prereg=true}if(!obj){obj=Object.create(state.syntax[type])}obj.identifier=type==="(identifier)";obj.type=obj.type||type;obj.value=value;obj.line=this.line;obj.character=this.char;obj.from=this.from;if(obj.identifier&&token)obj.raw_text=token.text||token.value;if(token&&token.startLine&&token.startLine!==this.line){obj.startLine=token.startLine}if(token&&token.context){obj.context=token.context}if(token&&token.depth){obj.depth=token.depth}if(token&&token.isUnclosed){obj.isUnclosed=token.isUnclosed}if(isProperty&&obj.identifier){obj.isProperty=isProperty}obj.check=checks.check;return obj}.bind(this);for(;;){if(!this.input.length){if(this.nextLine(checks)){return create("(endline)","")}if(this.exhausted){return null}this.exhausted=true;return create("(end)","")}token=this.next(checks);if(!token){if(this.input.length){this.trigger("error",{code:"E024",line:this.line,character:this.char,data:[this.peek()]});this.input=""}continue}switch(token.type){case Token.StringLiteral:this.triggerAsync("String",{line:this.line,"char":this.char,from:this.from,startLine:token.startLine,startChar:token.startChar,value:token.value,quote:token.quote},checks,function(){return true});return create("(string)",token.value,null,token);case Token.TemplateHead:this.trigger("TemplateHead",{line:this.line,"char":this.char,from:this.from,startLine:token.startLine,startChar:token.startChar,value:token.value});return create("(template)",token.value,null,token);case Token.TemplateMiddle:this.trigger("TemplateMiddle",{line:this.line,"char":this.char,from:this.from,startLine:token.startLine,startChar:token.startChar,value:token.value});return create("(template middle)",token.value,null,token);case Token.TemplateTail:this.trigger("TemplateTail",{line:this.line,"char":this.char,from:this.from,startLine:token.startLine,startChar:token.startChar,value:token.value});return create("(template tail)",token.value,null,token);case Token.NoSubstTemplate:this.trigger("NoSubstTemplate",{line:this.line,"char":this.char,from:this.from,startLine:token.startLine,startChar:token.startChar,value:token.value});return create("(no subst template)",token.value,null,token);case Token.Identifier:this.triggerAsync("Identifier",{line:this.line,"char":this.char,from:this.from,name:token.value,raw_name:token.text,isProperty:state.tokens.curr.id==="."},checks,function(){return true});case Token.Keyword:case Token.NullLiteral:case Token.BooleanLiteral:return create("(identifier)",token.value,state.tokens.curr.id===".",token);case Token.NumericLiteral:if(token.isMalformed){this.trigger("warning",{code:"W045",line:this.line,character:this.char,data:[token.value]})}this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["0x-"]},checks,function(){return token.base===16&&state.jsonMode});this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},checks,function(){return state.isStrict()&&token.base===8&&token.isLegacy});this.trigger("Number",{line:this.line,"char":this.char,from:this.from,value:token.value,base:token.base,isMalformed:token.malformed});return create("(number)",token.value);case Token.RegExp:return create("(regexp)",token.value);case Token.Comment:state.tokens.curr.comment=true;if(token.isSpecial){return{id:"(comment)",value:token.value,body:token.body,type:token.commentType,isSpecial:token.isSpecial,line:this.line,character:this.char,from:this.from}}break;case"":break;default:return create("(punctuator)",token.value)}}}};exports.Lexer=Lexer;exports.Context=Context},{"../data/ascii-identifier-data.js":1,"../data/non-ascii-identifier-part-only.js":2,"../data/non-ascii-identifier-start.js":3,"./reg.js":17,"./state.js":19,events:5,lodash:12}],14:[function(require,module,exports){"use strict";var _=require("lodash");var errors={E001:"Bad option: '{a}'.",E002:"Bad option value.",E003:"Expected a JSON value.",E004:"Input is neither a string nor an array of strings.",E005:"Input is empty.",E006:"Unexpected early end of program.",E007:'Missing "use strict" statement.',E008:"Strict violation.",E009:"Option 'validthis' can't be used in a global scope.",E010:"'with' is not allowed in strict mode.",E011:"'{a}' has already been declared.",E012:"const '{a}' is initialized to 'undefined'.",E013:"Attempting to override '{a}' which is a constant.",E014:"A regular expression literal can be confused with '/='.",E015:"Unclosed regular expression.",E016:"Invalid regular expression.",E017:"Unclosed comment.",E018:"Unbegun comment.",E019:"Unmatched '{a}'.",E020:"Expected '{a}' to match '{b}' from line {c} and instead saw '{d}'.",E021:"Expected '{a}' and instead saw '{b}'.",E022:"Line breaking error '{a}'.",E023:"Missing '{a}'.",E024:"Unexpected '{a}'.",E025:"Missing ':' on a case clause.",E026:"Missing '}' to match '{' from line {a}.",E027:"Missing ']' to match '[' from line {a}.",E028:"Illegal comma.",E029:"Unclosed string.",E030:"Expected an identifier and instead saw '{a}'.",E031:"Bad assignment.",E032:"Expected a small integer or 'false' and instead saw '{a}'.",E033:"Expected an operator and instead saw '{a}'.",E034:"get/set are ES5 features.",E035:"Missing property name.",E036:"Expected to see a statement and instead saw a block.",E037:null,E038:null,E039:"Function declarations are not invocable. Wrap the whole function invocation in parens.",E040:"Each value should have its own case label.",E041:"Unrecoverable syntax error.",E042:"Stopping.",E043:"Too many errors.",E044:null,E045:"Invalid for each loop.",E046:"Yield expressions may only occur within generator functions.",E047:null,E048:"{a} declaration not directly within block.",E049:"A {a} cannot be named '{b}'.",E050:"Mozilla requires the yield expression to be parenthesized here.",E051:null,E052:"Unclosed template literal.",E053:"{a} declarations are only allowed at the top level of module scope.",E054:"Class properties must be methods. Expected '(' but instead saw '{a}'.",E055:"The '{a}' option cannot be set after any executable code.",E056:"'{a}' was used before it was declared, which is illegal for '{b}' variables.",E057:"Invalid meta property: '{a}.{b}'.",E058:"Missing semicolon.",E059:"Incompatible values for the '{a}' and '{b}' linting options.",E060:"Non-callable values cannot be used as the second operand to instanceof.",E061:"Invalid position for 'yield' expression (consider wrapping in parenthesis)."};var warnings={W001:"'hasOwnProperty' is a really bad name.",W002:"Value of '{a}' may be overwritten in IE 8 and earlier.",W003:"'{a}' was used before it was defined.",W004:"'{a}' is already defined.",W005:"A dot following a number can be confused with a decimal point.",W006:"Confusing minuses.",W007:"Confusing plusses.",W008:"A leading decimal point can be confused with a dot: '{a}'.",W009:"The array literal notation [] is preferable.",W010:"The object literal notation {} is preferable.",W011:null,W012:null,W013:null,W014:"Misleading line break before '{a}'; readers may interpret this as an expression boundary.",W015:null,W016:"Unexpected use of '{a}'.",W017:"Bad operand.",W018:"Confusing use of '{a}'.",W019:"Use the isNaN function to compare with NaN.",W020:"Read only.",W021:"Reassignment of '{a}', which is is a {b}. "+"Use 'var' or 'let' to declare bindings that may change.",W022:"Do not assign to the exception parameter.",W023:"Expected an identifier in an assignment and instead saw a function invocation.",W024:"Expected an identifier and instead saw '{a}' (a reserved word).",W025:"Missing name in function declaration.",W026:"Inner functions should be listed at the top of the outer function.",W027:"Unreachable '{a}' after '{b}'.",W028:"Label '{a}' on {b} statement.",W030:"Expected an assignment or function call and instead saw an expression.",W031:"Do not use 'new' for side effects.",W032:"Unnecessary semicolon.",W033:"Missing semicolon.",W034:'Unnecessary directive "{a}".',W035:"Empty block.",W036:"Unexpected /*member '{a}'.",W037:"'{a}' is a statement label.",W038:"'{a}' used out of scope.",W039:"'{a}' is not allowed.",W040:"If a strict mode function is executed using function invocation, "+"its 'this' value will be undefined.",W041:null,W042:"Avoid EOL escaping.",W043:"Bad escaping of EOL. Use option multistr if needed.",W044:"Bad or unnecessary escaping.",W045:"Bad number '{a}'.",W046:"Don't use extra leading zeros '{a}'.",W047:"A trailing decimal point can be confused with a dot: '{a}'.",W048:"Unexpected control character in regular expression.",W049:"Unexpected escaped character '{a}' in regular expression.",W050:"JavaScript URL.",W051:"Variables should not be deleted.",W052:"Unexpected '{a}'.",W053:"Do not use {a} as a constructor.",W054:"The Function constructor is a form of eval.",W055:"A constructor name should start with an uppercase letter.",W056:"Bad constructor.",W057:"Weird construction. Is 'new' necessary?",W058:"Missing '()' invoking a constructor.",W059:"Avoid arguments.{a}.",W060:"document.write can be a form of eval.",W061:"eval can be harmful.",W062:"Wrap an immediate function invocation in parens "+"to assist the reader in understanding that the expression "+"is the result of a function, and not the function itself.",W063:"Math is not a function.",W064:"Missing 'new' prefix when invoking a constructor.",W065:"Missing radix parameter.",W066:"Implied eval. Consider passing a function instead of a string.",W067:"Bad invocation.",W068:"Wrapping non-IIFE function literals in parens is unnecessary.",W069:"['{a}'] is better written in dot notation.",W070:"Extra comma. (it breaks older versions of IE)",W071:"This function has too many statements. ({a})",W072:"This function has too many parameters. ({a})",W073:"Blocks are nested too deeply. ({a})",W074:"This function's cyclomatic complexity is too high. ({a})",W075:"Duplicate {a} '{b}'.",W076:"Unexpected parameter '{a}' in get {b} function.",W077:"Expected a single parameter in set {a} function.",W078:"Setter is defined without getter.",W079:"Redefinition of '{a}'.",W080:"It's not necessary to initialize '{a}' to 'undefined'.",W081:null,W082:"Function declarations should not be placed in blocks. "+"Use a function expression or move the statement to the top of "+"the outer function.",W083:"Functions declared within loops referencing an outer scoped "+"variable may lead to confusing semantics.",W084:"Expected a conditional expression and instead saw an assignment.",W085:"Don't use 'with'.",W086:"Expected a 'break' statement before '{a}'.",W087:"Forgotten 'debugger' statement?",W088:"Creating global 'for' variable. Should be 'for (var {a} ...'.",W089:"The body of a for in should be wrapped in an if statement to filter "+"unwanted properties from the prototype.",W090:"'{a}' is not a statement label.",W091:null,W093:"Did you mean to return a conditional instead of an assignment?",W094:"Unexpected comma.",W095:"Expected a string and instead saw {a}.",W096:"The '{a}' key may produce unexpected results.",W097:'Use the function form of "use strict".',W098:"'{a}' is defined but never used.",W099:null,W100:"This character may get silently deleted by one or more browsers.",W101:"Line is too long.",W102:null,W103:"The '{a}' property is deprecated.",W104:"'{a}' is available in ES{b} (use 'esversion: {b}') or Mozilla JS extensions (use moz).",W105:"Unexpected {a} in '{b}'.",W106:"Identifier '{a}' is not in camel case.",W107:"Script URL.",W108:"Strings must use doublequote.",W109:"Strings must use singlequote.",W110:"Mixed double and single quotes.",W112:"Unclosed string.",W113:"Control character in string: {a}.",W114:"Avoid {a}.",W115:"Octal literals are not allowed in strict mode.",W116:"Expected '{a}' and instead saw '{b}'.",W117:"'{a}' is not defined.",W118:"'{a}' is only available in Mozilla JavaScript extensions (use moz option).",W119:"'{a}' is only available in ES{b} (use 'esversion: {b}').",W120:"You might be leaking a variable ({a}) here.",W121:"Extending prototype of native object: '{a}'.",W122:"Invalid typeof value '{a}'",W123:"'{a}' is already defined in outer scope.",W124:"A generator function should contain at least one yield expression.",W125:"This line contains non-breaking spaces: http://jshint.com/docs/options/#nonbsp",W126:"Unnecessary grouping operator.",W127:"Unexpected use of a comma operator.",W128:"Empty array elements require elision=true.",W129:"'{a}' is defined in a future version of JavaScript. Use a "+"different variable name to avoid migration issues.",W130:"Invalid element after rest element.",W131:"Invalid parameter after rest parameter.",W132:"`var` declarations are forbidden. Use `let` or `const` instead.",W133:"Invalid for-{a} loop left-hand-side: {b}.",W134:"The '{a}' option is only available when linting ECMAScript {b} code.",W135:"{a} may not be supported by non-browser environments.",W136:"'{a}' must be in function scope.",W137:"Empty destructuring.",W138:"Regular parameters should not come after default parameters.",W139:"Function expressions should not be used as the second operand to instanceof.",W140:"Missing comma."};var info={I001:"Comma warnings can be turned off with 'laxcomma'.",I002:null,I003:"ES5 option is now set per default"};exports.errors={};exports.warnings={};exports.info={};_.each(errors,function(desc,code){exports.errors[code]={code:code,desc:desc}});_.each(warnings,function(desc,code){exports.warnings[code]={code:code,desc:desc}});_.each(info,function(desc,code){exports.info[code]={code:code,desc:desc}})},{lodash:12}],15:[function(require,module,exports){"use strict";function NameStack(){this._stack=[]}Object.defineProperty(NameStack.prototype,"length",{get:function(){return this._stack.length}});NameStack.prototype.push=function(){this._stack.push(null)};NameStack.prototype.pop=function(){this._stack.pop()};NameStack.prototype.set=function(token){this._stack[this.length-1]=token};NameStack.prototype.infer=function(){var nameToken=this._stack[this.length-1];var prefix="";var type;if(!nameToken||nameToken.type==="class"){nameToken=this._stack[this.length-2]}if(!nameToken){return"(empty)"}type=nameToken.type;if(type!=="(string)"&&type!=="(number)"&&type!=="(identifier)"&&type!=="default"){return"(expression)"}if(nameToken.accessorType){prefix=nameToken.accessorType+" "}return prefix+nameToken.value};module.exports=NameStack},{}],16:[function(require,module,exports){"use strict";exports.bool={
enforcing:{bitwise:true,freeze:true,camelcase:true,curly:true,eqeqeq:true,futurehostile:true,es3:true,es5:true,forin:true,immed:true,newcap:true,noarg:true,nocomma:true,noempty:true,nonbsp:true,nonew:true,undef:true,singleGroups:false,varstmt:false,enforceall:false,trailingcomma:false},relaxing:{asi:true,multistr:true,debug:true,boss:true,evil:true,funcscope:true,globalstrict:true,iterator:true,notypeof:true,plusplus:true,proto:true,scripturl:true,sub:true,supernew:true,laxbreak:true,laxcomma:true,validthis:true,withstmt:true,moz:true,noyield:true,eqnull:true,lastsemic:true,loopfunc:true,expr:true,esnext:true,elision:true},environments:{mootools:true,couch:true,jasmine:true,jquery:true,node:true,qunit:true,rhino:true,shelljs:true,prototypejs:true,yui:true,mocha:true,module:true,wsh:true,worker:true,nonstandard:true,browser:true,browserify:true,devel:true,dojo:true,typed:true,phantom:true},obsolete:{onecase:true,regexp:true,regexdash:true}};exports.val={maxlen:false,indent:false,maxerr:false,predef:false,globals:false,quotmark:false,scope:false,maxstatements:false,maxdepth:false,maxparams:false,maxcomplexity:false,shadow:false,strict:true,unused:true,latedef:false,ignore:false,ignoreDelimiters:false,esversion:5};exports.inverted={bitwise:true,forin:true,newcap:true,plusplus:true,regexp:true,undef:true,eqeqeq:true,strict:true};exports.validNames=Object.keys(exports.val).concat(Object.keys(exports.bool.relaxing)).concat(Object.keys(exports.bool.enforcing)).concat(Object.keys(exports.bool.obsolete)).concat(Object.keys(exports.bool.environments));exports.renamed={eqeq:"eqeqeq",windows:"wsh",sloppy:"strict"};exports.removed={nomen:true,onevar:true,passfail:true,white:true,gcl:true,smarttabs:true,trailing:true};exports.noenforceall={varstmt:true,strict:true}},{}],17:[function(require,module,exports){"use strict";exports.unsafeString=/@cc|<\/?|script|\]\s*\]|<\s*!|&lt/i;exports.unsafeChars=/[\u0000-\u001f\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/;exports.needEsc=/[\u0000-\u001f&<"\/\\\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/;exports.needEscGlobal=/[\u0000-\u001f&<"\/\\\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;exports.starSlash=/\*\//;exports.identifier=/^([a-zA-Z_$][a-zA-Z0-9_$]*)$/;exports.javascriptURL=/^(?:javascript|jscript|ecmascript|vbscript|livescript)\s*:/i;exports.fallsThrough=/^\s*falls?\sthrough\s*$/;exports.maxlenException=/^(?:(?:\/\/|\/\*|\*) ?)?[^ ]+$/},{}],18:[function(require,module,exports){"use strict";var _=require("lodash");var events=require("events");var marker={};var scopeManager=function(state,predefined,exported,declared){var _current;var _scopeStack=[];function _newScope(type){_current={"(labels)":Object.create(null),"(usages)":Object.create(null),"(breakLabels)":Object.create(null),"(parent)":_current,"(type)":type,"(params)":type==="functionparams"||type==="catchparams"?[]:null};_scopeStack.push(_current)}_newScope("global");_current["(predefined)"]=predefined;var _currentFunctBody=_current;var usedPredefinedAndGlobals=Object.create(null);var impliedGlobals=Object.create(null);var unuseds=[];var emitter=new events.EventEmitter;function warning(code,token){emitter.emit("warning",{code:code,token:token,data:_.slice(arguments,2)})}function error(code,token){emitter.emit("warning",{code:code,token:token,data:_.slice(arguments,2)})}function _setupUsages(labelName){if(!_current["(usages)"][labelName]){_current["(usages)"][labelName]={"(modified)":[],"(reassigned)":[],"(tokens)":[]}}}var _getUnusedOption=function(unused_opt){if(unused_opt===undefined){unused_opt=state.option.unused}if(unused_opt===true){unused_opt="last-param"}return unused_opt};var _warnUnused=function(name,tkn,type,unused_opt){var line=tkn.line;var chr=tkn.from;var raw_name=tkn.raw_text||name;unused_opt=_getUnusedOption(unused_opt);var warnable_types={vars:["var"],"last-param":["var","param"],strict:["var","param","last-param"]};if(unused_opt){if(warnable_types[unused_opt]&&warnable_types[unused_opt].indexOf(type)!==-1){warning("W098",{line:line,from:chr},raw_name)}}if(unused_opt||type==="var"){unuseds.push({name:name,line:line,character:chr})}};function _checkForUnused(){if(_current["(type)"]==="functionparams"){_checkParams();return}var curentLabels=_current["(labels)"];for(var labelName in curentLabels){if(curentLabels[labelName]){if(curentLabels[labelName]["(type)"]!=="exception"&&curentLabels[labelName]["(unused)"]){_warnUnused(labelName,curentLabels[labelName]["(token)"],"var")}}}}function _checkParams(){var params=_current["(params)"];if(!params){return}var param=params.pop();var unused_opt;while(param){var label=_current["(labels)"][param];unused_opt=_getUnusedOption(state.funct["(unusedOption)"]);if(param==="undefined")return;if(label["(unused)"]){_warnUnused(param,label["(token)"],"param",state.funct["(unusedOption)"])}else if(unused_opt==="last-param"){return}param=params.pop()}}function _getLabel(labelName){for(var i=_scopeStack.length-1;i>=0;--i){var scopeLabels=_scopeStack[i]["(labels)"];if(scopeLabels[labelName]){return scopeLabels}}}function usedSoFarInCurrentFunction(labelName){for(var i=_scopeStack.length-1;i>=0;i--){var current=_scopeStack[i];if(current["(usages)"][labelName]){return current["(usages)"][labelName]}if(current===_currentFunctBody){break}}return false}function _checkOuterShadow(labelName,token){if(state.option.shadow!=="outer"){return}var isGlobal=_currentFunctBody["(type)"]==="global",isNewFunction=_current["(type)"]==="functionparams";var outsideCurrentFunction=!isGlobal;for(var i=0;i<_scopeStack.length;i++){var stackItem=_scopeStack[i];if(!isNewFunction&&_scopeStack[i+1]===_currentFunctBody){outsideCurrentFunction=false}if(outsideCurrentFunction&&stackItem["(labels)"][labelName]){warning("W123",token,labelName)}if(stackItem["(breakLabels)"][labelName]){warning("W123",token,labelName)}}}function _latedefWarning(type,labelName,token){if(state.option.latedef){if(state.option.latedef===true&&type==="function"||type!=="function"){warning("W003",token,labelName)}}}var scopeManagerInst={on:function(names,listener){names.split(" ").forEach(function(name){emitter.on(name,listener)})},isPredefined:function(labelName){return!this.has(labelName)&&_.has(_scopeStack[0]["(predefined)"],labelName)},stack:function(type){var previousScope=_current;_newScope(type);if(!type&&previousScope["(type)"]==="functionparams"){_current["(isFuncBody)"]=true;_current["(context)"]=_currentFunctBody;_currentFunctBody=_current}},unstack:function(){var subScope=_scopeStack.length>1?_scopeStack[_scopeStack.length-2]:null;var isUnstackingFunctionBody=_current===_currentFunctBody,isUnstackingFunctionParams=_current["(type)"]==="functionparams",isUnstackingFunctionOuter=_current["(type)"]==="functionouter";var i,j;var currentUsages=_current["(usages)"];var currentLabels=_current["(labels)"];var usedLabelNameList=Object.keys(currentUsages);if(currentUsages.__proto__&&usedLabelNameList.indexOf("__proto__")===-1){usedLabelNameList.push("__proto__")}for(i=0;i<usedLabelNameList.length;i++){var usedLabelName=usedLabelNameList[i];var usage=currentUsages[usedLabelName];var usedLabel=currentLabels[usedLabelName];if(usedLabel){var usedLabelType=usedLabel["(type)"];var isImmutable=usedLabelType==="const"||usedLabelType==="import";if(usedLabel["(useOutsideOfScope)"]&&!state.option.funcscope){var usedTokens=usage["(tokens)"];if(usedTokens){for(j=0;j<usedTokens.length;j++){if(usedLabel["(function)"]===usedTokens[j]["(function)"]){error("W038",usedTokens[j],usedLabelName)}}}}_current["(labels)"][usedLabelName]["(unused)"]=false;if(isImmutable&&usage["(modified)"]){for(j=0;j<usage["(modified)"].length;j++){error("E013",usage["(modified)"][j],usedLabelName)}}if((usedLabelType==="function"||usedLabelType==="class")&&usage["(reassigned)"]){for(j=0;j<usage["(reassigned)"].length;j++){if(!usage["(reassigned)"][j].ignoreW021){warning("W021",usage["(reassigned)"][j],usedLabelName,usedLabelType)}}}continue}if(isUnstackingFunctionOuter){state.funct["(isCapturing)"]=true}if(subScope){if(!subScope["(usages)"][usedLabelName]){subScope["(usages)"][usedLabelName]=usage;if(isUnstackingFunctionBody){subScope["(usages)"][usedLabelName]["(onlyUsedSubFunction)"]=true}}else{var subScopeUsage=subScope["(usages)"][usedLabelName];subScopeUsage["(modified)"]=subScopeUsage["(modified)"].concat(usage["(modified)"]);subScopeUsage["(tokens)"]=subScopeUsage["(tokens)"].concat(usage["(tokens)"]);subScopeUsage["(reassigned)"]=subScopeUsage["(reassigned)"].concat(usage["(reassigned)"])}}else{if(typeof _current["(predefined)"][usedLabelName]==="boolean"){delete declared[usedLabelName];usedPredefinedAndGlobals[usedLabelName]=marker;if(_current["(predefined)"][usedLabelName]===false&&usage["(reassigned)"]){for(j=0;j<usage["(reassigned)"].length;j++){if(!usage["(reassigned)"][j].ignoreW020){warning("W020",usage["(reassigned)"][j])}}}}else{if(usage["(tokens)"]){for(j=0;j<usage["(tokens)"].length;j++){var undefinedToken=usage["(tokens)"][j];if(!undefinedToken.forgiveUndef){if(state.option.undef&&!undefinedToken.ignoreUndef){warning("W117",undefinedToken,usedLabelName)}if(impliedGlobals[usedLabelName]){impliedGlobals[usedLabelName].line.push(undefinedToken.line)}else{impliedGlobals[usedLabelName]={name:usedLabelName,line:[undefinedToken.line]}}}}}}}}if(!subScope){Object.keys(declared).forEach(function(labelNotUsed){_warnUnused(labelNotUsed,declared[labelNotUsed],"var")})}if(subScope&&!isUnstackingFunctionBody&&!isUnstackingFunctionParams&&!isUnstackingFunctionOuter){var labelNames=Object.keys(currentLabels);for(i=0;i<labelNames.length;i++){var defLabelName=labelNames[i];var defLabel=currentLabels[defLabelName];if(!defLabel["(blockscoped)"]&&defLabel["(type)"]!=="exception"){var shadowed=subScope["(labels)"][defLabelName];if(shadowed){shadowed["(unused)"]&=defLabel["(unused)"]}else{defLabel["(useOutsideOfScope)"]=_currentFunctBody["(type)"]!=="global"&&!this.funct.has(defLabelName,{excludeCurrent:true});subScope["(labels)"][defLabelName]=defLabel}delete currentLabels[defLabelName]}}}_checkForUnused();_scopeStack.pop();if(isUnstackingFunctionBody){_currentFunctBody=_scopeStack[_.findLastIndex(_scopeStack,function(scope){return scope["(isFuncBody)"]||scope["(type)"]==="global"})]}_current=subScope},addParam:function(labelName,token,type){type=type||"param";if(type==="exception"){var previouslyDefinedLabelType=this.funct.labeltype(labelName);if(previouslyDefinedLabelType&&previouslyDefinedLabelType!=="exception"){if(!state.option.node){warning("W002",state.tokens.next,labelName)}}}if(_.has(_current["(labels)"],labelName)){_current["(labels)"][labelName].duplicated=true}else{_checkOuterShadow(labelName,token,type);_current["(labels)"][labelName]={"(type)":type,"(token)":token,"(unused)":true};_current["(params)"].push(labelName)}if(_.has(_current["(usages)"],labelName)){var usage=_current["(usages)"][labelName];if(usage["(onlyUsedSubFunction)"]){_latedefWarning(type,labelName,token)}else{warning("E056",token,labelName,type)}}},validateParams:function(){if(_currentFunctBody["(type)"]==="global"){return}var isStrict=state.isStrict();var currentFunctParamScope=_currentFunctBody["(parent)"];if(!currentFunctParamScope["(params)"]){return}currentFunctParamScope["(params)"].forEach(function(labelName){var label=currentFunctParamScope["(labels)"][labelName];if(label&&label.duplicated){if(isStrict){warning("E011",label["(token)"],labelName)}else if(state.option.shadow!==true){warning("W004",label["(token)"],labelName)}}})},getUsedOrDefinedGlobals:function(){var list=Object.keys(usedPredefinedAndGlobals);if(usedPredefinedAndGlobals.__proto__===marker&&list.indexOf("__proto__")===-1){list.push("__proto__")}return list},getImpliedGlobals:function(){var values=_.values(impliedGlobals);var hasProto=false;if(impliedGlobals.__proto__){hasProto=values.some(function(value){return value.name==="__proto__"});if(!hasProto){values.push(impliedGlobals.__proto__)}}return values},getUnuseds:function(){return unuseds},has:function(labelName){return Boolean(_getLabel(labelName))},labeltype:function(labelName){var scopeLabels=_getLabel(labelName);if(scopeLabels){return scopeLabels[labelName]["(type)"]}return null},addExported:function(labelName){var globalLabels=_scopeStack[0]["(labels)"];if(_.has(declared,labelName)){delete declared[labelName]}else if(_.has(globalLabels,labelName)){globalLabels[labelName]["(unused)"]=false}else{for(var i=1;i<_scopeStack.length;i++){var scope=_scopeStack[i];if(!scope["(type)"]){if(_.has(scope["(labels)"],labelName)&&!scope["(labels)"][labelName]["(blockscoped)"]){scope["(labels)"][labelName]["(unused)"]=false;return}}else{break}}exported[labelName]=true}},setExported:function(labelName,token){this.block.use(labelName,token)},initialize:function(labelName){if(_current["(labels)"][labelName]){_current["(labels)"][labelName]["(initialized)"]=true}},addlabel:function(labelName,opts){var type=opts.type;var token=opts.token;var isblockscoped=type==="let"||type==="const"||type==="class"||type==="import";var ishoisted=type==="function"||type==="import";var isexported=(isblockscoped?_current:_currentFunctBody)["(type)"]==="global"&&_.has(exported,labelName);_checkOuterShadow(labelName,token,type);if(isblockscoped){var declaredInCurrentScope=_current["(labels)"][labelName];if(!declaredInCurrentScope&&_current===_currentFunctBody&&_current["(type)"]!=="global"){declaredInCurrentScope=!!_currentFunctBody["(parent)"]["(labels)"][labelName]}if(!declaredInCurrentScope&&_current["(usages)"][labelName]){var usage=_current["(usages)"][labelName];if(usage["(onlyUsedSubFunction)"]||ishoisted){_latedefWarning(type,labelName,token)}else if(!ishoisted){warning("E056",token,labelName,type)}}if(declaredInCurrentScope){warning("E011",token,labelName)}else if(state.option.shadow==="outer"){if(scopeManagerInst.funct.has(labelName)){warning("W004",token,labelName)}}scopeManagerInst.block.add(labelName,type,token,!isexported,opts.initialized)}else{var declaredInCurrentFunctionScope=scopeManagerInst.funct.has(labelName);if(!declaredInCurrentFunctionScope&&usedSoFarInCurrentFunction(labelName)){_latedefWarning(type,labelName,token)}if(scopeManagerInst.funct.has(labelName,{onlyBlockscoped:true})){warning("E011",token,labelName)}else if(state.option.shadow!==true){if(declaredInCurrentFunctionScope&&labelName!=="__proto__"){if(_currentFunctBody["(type)"]!=="global"){warning("W004",token,labelName)}}}scopeManagerInst.funct.add(labelName,type,token,!isexported);if(_currentFunctBody["(type)"]==="global"&&!state.impliedClosure()){usedPredefinedAndGlobals[labelName]=marker}}},funct:{labeltype:function(labelName,options){var onlyBlockscoped=options&&options.onlyBlockscoped;var excludeParams=options&&options.excludeParams;var currentScopeIndex=_scopeStack.length-(options&&options.excludeCurrent?2:1);for(var i=currentScopeIndex;i>=0;i--){var current=_scopeStack[i];if(current["(labels)"][labelName]&&(!onlyBlockscoped||current["(labels)"][labelName]["(blockscoped)"])){return current["(labels)"][labelName]["(type)"]}var scopeCheck=excludeParams?_scopeStack[i-1]:current;if(scopeCheck&&scopeCheck["(type)"]==="functionparams"){return null}}return null},hasBreakLabel:function(labelName){for(var i=_scopeStack.length-1;i>=0;i--){var current=_scopeStack[i];if(current["(breakLabels)"][labelName]){return true}if(current["(type)"]==="functionparams"){return false}}return false},has:function(labelName,options){return Boolean(this.labeltype(labelName,options))},add:function(labelName,type,tok,unused){_current["(labels)"][labelName]={"(type)":type,"(token)":tok,"(blockscoped)":false,"(function)":_currentFunctBody,"(unused)":unused}}},block:{isGlobal:function(){return _current["(type)"]==="global"},use:function(labelName,token){var paramScope=_currentFunctBody["(parent)"];if(paramScope&&paramScope["(labels)"][labelName]&&paramScope["(labels)"][labelName]["(type)"]==="param"){if(!scopeManagerInst.funct.has(labelName,{excludeParams:true,onlyBlockscoped:true})){paramScope["(labels)"][labelName]["(unused)"]=false}}if(token&&(state.ignored.W117||state.option.undef===false)){token.ignoreUndef=true}_setupUsages(labelName);_current["(usages)"][labelName]["(onlyUsedSubFunction)"]=false;if(token){token["(function)"]=_currentFunctBody;_current["(usages)"][labelName]["(tokens)"].push(token)}var label=_current["(labels)"][labelName];if(label&&label["(blockscoped)"]&&!label["(initialized)"]){error("E056",token,labelName,label["(type)"])}},reassign:function(labelName,token){token.ignoreW020=state.ignored.W020;token.ignoreW021=state.ignored.W021;this.modify(labelName,token);_current["(usages)"][labelName]["(reassigned)"].push(token)},modify:function(labelName,token){_setupUsages(labelName);_current["(usages)"][labelName]["(onlyUsedSubFunction)"]=false;_current["(usages)"][labelName]["(modified)"].push(token)},add:function(labelName,type,tok,unused,initialized){_current["(labels)"][labelName]={"(type)":type,"(token)":tok,"(initialized)":!!initialized,"(blockscoped)":true,"(unused)":unused}},addBreakLabel:function(labelName,opts){var token=opts.token;if(scopeManagerInst.funct.hasBreakLabel(labelName)){warning("E011",token,labelName)}else if(state.option.shadow==="outer"){if(scopeManagerInst.funct.has(labelName)){warning("W004",token,labelName)}else{_checkOuterShadow(labelName,token)}}_current["(breakLabels)"][labelName]=token}}};return scopeManagerInst};module.exports=scopeManager},{events:5,lodash:12}],19:[function(require,module,exports){"use strict";var NameStack=require("./name-stack.js");var state={syntax:{},isStrict:function(){return this.directive["use strict"]||this.inClassBody||this.option.module||this.option.strict==="implied"},stmtMissingStrict:function(){if(this.option.strict==="global"){return true}if(this.option.strict===false){return false}if(this.option.globalstrict){return true}return false},allowsGlobalUsd:function(){return this.option.strict==="global"||this.option.globalstrict||this.option.module||this.impliedClosure()},impliedClosure:function(){return this.option.node||this.option.phantom||this.option.browserify},inMoz:function(){return this.option.moz},inES6:function(strict){if(strict){return this.esVersion===6}return this.option.moz||this.esVersion>=6},inES5:function(strict){if(strict){return(!this.esVersion||this.esVersion===5)&&!this.option.moz}return!this.esVersion||this.esVersion>=5||this.option.moz},inferEsVersion:function(){var badOpt=null;if(this.option.esversion){if(this.option.es3){badOpt="es3"}else if(this.option.es5){badOpt="es5"}else if(this.option.esnext){badOpt="esnext"}if(badOpt){return badOpt}if(this.option.esversion===2015){this.esVersion=6}else{this.esVersion=this.option.esversion}}else if(this.option.es3){this.esVersion=3}else if(this.option.esnext){this.esVersion=6}return null},reset:function(){this.tokens={prev:null,next:null,curr:null};this.option={};this.esVersion=5;this.funct=null;this.ignored={};this.directive={};this.jsonMode=false;this.jsonWarnings=[];this.lines=[];this.tab="";this.cache={};this.ignoredLines={};this.forinifcheckneeded=false;this.nameStack=new NameStack;this.inClassBody=false}};exports.state=state},{"./name-stack.js":15}],20:[function(require,module,exports){"use strict";exports.register=function(linter){linter.on("Identifier",function style_scanProto(data){if(linter.getOption("proto")){return}if(data.name==="__proto__"){linter.warn("W103",{line:data.line,"char":data.char,data:[data.name,"6"]})}});linter.on("Identifier",function style_scanIterator(data){if(linter.getOption("iterator")){return}if(data.name==="__iterator__"){linter.warn("W103",{line:data.line,"char":data.char,data:[data.name]})}});linter.on("Identifier",function style_scanCamelCase(data){if(!linter.getOption("camelcase")){return}if(data.name.replace(/^_+|_+$/g,"").indexOf("_")>-1&&!data.name.match(/^[A-Z0-9_]*$/)){linter.warn("W106",{line:data.line,"char":data.char,data:[data.name]})}});linter.on("String",function style_scanQuotes(data){var quotmark=linter.getOption("quotmark");var code;if(!quotmark){return}if(quotmark==="single"&&data.quote!=="'"){code="W109"}if(quotmark==="double"&&data.quote!=='"'){code="W108"}if(quotmark===true){if(!linter.getCache("quotmark")){linter.setCache("quotmark",data.quote)}if(linter.getCache("quotmark")!==data.quote){code="W110"}}if(code){linter.warn(code,{line:data.line,"char":data.char})}});linter.on("Number",function style_scanNumbers(data){if(data.value.charAt(0)==="."){linter.warn("W008",{line:data.line,"char":data.char,data:[data.value]})}if(data.value.substr(data.value.length-1)==="."){linter.warn("W047",{line:data.line,"char":data.char,data:[data.value]})}if(/^00+/.test(data.value)){linter.warn("W046",{line:data.line,"char":data.char,data:[data.value]})}});linter.on("String",function style_scanJavaScriptURLs(data){var re=/^(?:javascript|jscript|ecmascript|vbscript|livescript)\s*:/i;if(linter.getOption("scripturl")){return}if(re.test(data.value)){linter.warn("W107",{line:data.line,"char":data.char})}})}},{}],21:[function(require,module,exports){"use strict";exports.reservedVars={arguments:false,NaN:false};exports.ecmaIdentifiers={3:{Array:false,Boolean:false,Date:false,decodeURI:false,decodeURIComponent:false,encodeURI:false,encodeURIComponent:false,Error:false,eval:false,EvalError:false,Function:false,hasOwnProperty:false,isFinite:false,isNaN:false,Math:false,Number:false,Object:false,parseInt:false,parseFloat:false,RangeError:false,ReferenceError:false,RegExp:false,String:false,SyntaxError:false,TypeError:false,URIError:false},5:{JSON:false},6:{ArrayBuffer:false,DataView:false,Float32Array:false,Float64Array:false,Int8Array:false,Int16Array:false,Int32Array:false,Map:false,Promise:false,Proxy:false,Reflect:false,Set:false,Symbol:false,Uint8Array:false,Uint16Array:false,Uint32Array:false,Uint8ClampledArray:false,WeakMap:false,WeakSet:false}};exports.browser={Audio:false,Blob:false,addEventListener:false,applicationCache:false,atob:false,blur:false,btoa:false,cancelAnimationFrame:false,CanvasGradient:false,CanvasPattern:false,CanvasRenderingContext2D:false,CSS:false,clearInterval:false,clearTimeout:false,close:false,closed:false,Comment:false,CompositionEvent:false,CustomEvent:false,DOMParser:false,defaultStatus:false,Document:false,document:false,DocumentFragment:false,Element:false,ElementTimeControl:false,Event:false,event:false,fetch:false,File:false,FileList:false,FileReader:false,FormData:false,focus:false,frames:false,getComputedStyle:false,HTMLElement:false,HTMLAnchorElement:false,HTMLBaseElement:false,HTMLBlockquoteElement:false,HTMLBodyElement:false,HTMLBRElement:false,HTMLButtonElement:false,HTMLCanvasElement:false,HTMLCollection:false,HTMLDirectoryElement:false,HTMLDivElement:false,HTMLDListElement:false,HTMLFieldSetElement:false,HTMLFontElement:false,HTMLFormElement:false,HTMLFrameElement:false,HTMLFrameSetElement:false,HTMLHeadElement:false,HTMLHeadingElement:false,HTMLHRElement:false,HTMLHtmlElement:false,HTMLIFrameElement:false,HTMLImageElement:false,HTMLInputElement:false,HTMLIsIndexElement:false,HTMLLabelElement:false,HTMLLayerElement:false,HTMLLegendElement:false,HTMLLIElement:false,HTMLLinkElement:false,HTMLMapElement:false,HTMLMenuElement:false,HTMLMetaElement:false,HTMLModElement:false,HTMLObjectElement:false,HTMLOListElement:false,HTMLOptGroupElement:false,HTMLOptionElement:false,HTMLParagraphElement:false,HTMLParamElement:false,HTMLPreElement:false,HTMLQuoteElement:false,HTMLScriptElement:false,HTMLSelectElement:false,HTMLStyleElement:false,HTMLTableCaptionElement:false,HTMLTableCellElement:false,HTMLTableColElement:false,HTMLTableElement:false,HTMLTableRowElement:false,HTMLTableSectionElement:false,HTMLTemplateElement:false,HTMLTextAreaElement:false,HTMLTitleElement:false,HTMLUListElement:false,HTMLVideoElement:false,history:false,Image:false,Intl:false,length:false,localStorage:false,location:false,matchMedia:false,MediaRecorder:false,MessageChannel:false,MessageEvent:false,MessagePort:false,MouseEvent:false,moveBy:false,moveTo:false,MutationObserver:false,name:false,Node:false,NodeFilter:false,NodeList:false,Notification:false,navigator:false,onbeforeunload:true,onblur:true,onerror:true,onfocus:true,onload:true,onresize:true,onunload:true,open:false,openDatabase:false,opener:false,Option:false,parent:false,performance:false,print:false,Range:false,requestAnimationFrame:false,removeEventListener:false,resizeBy:false,resizeTo:false,screen:false,scroll:false,scrollBy:false,scrollTo:false,sessionStorage:false,setInterval:false,setTimeout:false,SharedWorker:false,status:false,Storage:false,SVGAElement:false,SVGAltGlyphDefElement:false,SVGAltGlyphElement:false,SVGAltGlyphItemElement:false,SVGAngle:false,SVGAnimateColorElement:false,SVGAnimateElement:false,SVGAnimateMotionElement:false,SVGAnimateTransformElement:false,SVGAnimatedAngle:false,SVGAnimatedBoolean:false,SVGAnimatedEnumeration:false,SVGAnimatedInteger:false,SVGAnimatedLength:false,SVGAnimatedLengthList:false,SVGAnimatedNumber:false,SVGAnimatedNumberList:false,SVGAnimatedPathData:false,SVGAnimatedPoints:false,SVGAnimatedPreserveAspectRatio:false,SVGAnimatedRect:false,SVGAnimatedString:false,SVGAnimatedTransformList:false,SVGAnimationElement:false,SVGCSSRule:false,SVGCircleElement:false,SVGClipPathElement:false,SVGColor:false,SVGColorProfileElement:false,SVGColorProfileRule:false,SVGComponentTransferFunctionElement:false,SVGCursorElement:false,SVGDefsElement:false,SVGDescElement:false,SVGDocument:false,SVGElement:false,SVGElementInstance:false,SVGElementInstanceList:false,SVGEllipseElement:false,SVGExternalResourcesRequired:false,SVGFEBlendElement:false,SVGFEColorMatrixElement:false,SVGFEComponentTransferElement:false,SVGFECompositeElement:false,SVGFEConvolveMatrixElement:false,SVGFEDiffuseLightingElement:false,SVGFEDisplacementMapElement:false,SVGFEDistantLightElement:false,SVGFEFloodElement:false,SVGFEFuncAElement:false,SVGFEFuncBElement:false,SVGFEFuncGElement:false,SVGFEFuncRElement:false,SVGFEGaussianBlurElement:false,SVGFEImageElement:false,SVGFEMergeElement:false,SVGFEMergeNodeElement:false,SVGFEMorphologyElement:false,SVGFEOffsetElement:false,SVGFEPointLightElement:false,SVGFESpecularLightingElement:false,SVGFESpotLightElement:false,SVGFETileElement:false,SVGFETurbulenceElement:false,SVGFilterElement:false,SVGFilterPrimitiveStandardAttributes:false,SVGFitToViewBox:false,SVGFontElement:false,SVGFontFaceElement:false,SVGFontFaceFormatElement:false,SVGFontFaceNameElement:false,SVGFontFaceSrcElement:false,SVGFontFaceUriElement:false,SVGForeignObjectElement:false,SVGGElement:false,SVGGlyphElement:false,SVGGlyphRefElement:false,SVGGradientElement:false,SVGHKernElement:false,SVGICCColor:false,SVGImageElement:false,SVGLangSpace:false,SVGLength:false,SVGLengthList:false,SVGLineElement:false,SVGLinearGradientElement:false,SVGLocatable:false,SVGMPathElement:false,SVGMarkerElement:false,SVGMaskElement:false,SVGMatrix:false,SVGMetadataElement:false,SVGMissingGlyphElement:false,SVGNumber:false,SVGNumberList:false,SVGPaint:false,SVGPathElement:false,SVGPathSeg:false,SVGPathSegArcAbs:false,SVGPathSegArcRel:false,SVGPathSegClosePath:false,SVGPathSegCurvetoCubicAbs:false,SVGPathSegCurvetoCubicRel:false,SVGPathSegCurvetoCubicSmoothAbs:false,SVGPathSegCurvetoCubicSmoothRel:false,SVGPathSegCurvetoQuadraticAbs:false,SVGPathSegCurvetoQuadraticRel:false,SVGPathSegCurvetoQuadraticSmoothAbs:false,SVGPathSegCurvetoQuadraticSmoothRel:false,SVGPathSegLinetoAbs:false,SVGPathSegLinetoHorizontalAbs:false,SVGPathSegLinetoHorizontalRel:false,SVGPathSegLinetoRel:false,SVGPathSegLinetoVerticalAbs:false,SVGPathSegLinetoVerticalRel:false,SVGPathSegList:false,SVGPathSegMovetoAbs:false,SVGPathSegMovetoRel:false,SVGPatternElement:false,SVGPoint:false,SVGPointList:false,SVGPolygonElement:false,SVGPolylineElement:false,SVGPreserveAspectRatio:false,SVGRadialGradientElement:false,SVGRect:false,SVGRectElement:false,SVGRenderingIntent:false,SVGSVGElement:false,SVGScriptElement:false,SVGSetElement:false,SVGStopElement:false,SVGStringList:false,SVGStylable:false,SVGStyleElement:false,SVGSwitchElement:false,SVGSymbolElement:false,SVGTRefElement:false,SVGTSpanElement:false,SVGTests:false,SVGTextContentElement:false,SVGTextElement:false,SVGTextPathElement:false,SVGTextPositioningElement:false,SVGTitleElement:false,SVGTransform:false,SVGTransformList:false,SVGTransformable:false,SVGURIReference:false,SVGUnitTypes:false,SVGUseElement:false,SVGVKernElement:false,SVGViewElement:false,SVGViewSpec:false,SVGZoomAndPan:false,Text:false,TextDecoder:false,TextEncoder:false,TimeEvent:false,top:false,URL:false,WebGLActiveInfo:false,WebGLBuffer:false,WebGLContextEvent:false,WebGLFramebuffer:false,WebGLProgram:false,WebGLRenderbuffer:false,WebGLRenderingContext:false,WebGLShader:false,WebGLShaderPrecisionFormat:false,WebGLTexture:false,WebGLUniformLocation:false,WebSocket:false,window:false,Window:false,Worker:false,XDomainRequest:false,XMLHttpRequest:false,XMLSerializer:false,XPathEvaluator:false,XPathException:false,XPathExpression:false,XPathNamespace:false,XPathNSResolver:false,XPathResult:false};exports.devel={alert:false,confirm:false,console:false,Debug:false,opera:false,prompt:false};exports.worker={importScripts:true,onmessage:true,postMessage:true,self:true,FileReaderSync:true};exports.nonstandard={escape:false,unescape:false};exports.couch={require:false,respond:false,getRow:false,emit:false,send:false,start:false,sum:false,log:false,exports:false,module:false,provides:false};exports.node={__filename:false,__dirname:false,GLOBAL:false,global:false,module:false,require:false,Buffer:true,console:true,exports:true,process:true,setTimeout:true,clearTimeout:true,setInterval:true,clearInterval:true,setImmediate:true,clearImmediate:true};exports.browserify={__filename:false,__dirname:false,global:false,module:false,require:false,Buffer:true,exports:true,process:true};exports.phantom={phantom:true,require:true,WebPage:true,console:true,exports:true};exports.qunit={asyncTest:false,deepEqual:false,equal:false,expect:false,module:false,notDeepEqual:false,notEqual:false,notOk:false,notPropEqual:false,notStrictEqual:false,ok:false,propEqual:false,QUnit:false,raises:false,start:false,stop:false,strictEqual:false,test:false,"throws":false};exports.rhino={defineClass:false,deserialize:false,gc:false,help:false,importClass:false,importPackage:false,java:false,load:false,loadClass:false,Packages:false,print:false,quit:false,readFile:false,readUrl:false,runCommand:false,seal:false,serialize:false,spawn:false,sync:false,toint32:false,version:false};exports.shelljs={target:false,echo:false,exit:false,cd:false,pwd:false,ls:false,find:false,cp:false,rm:false,mv:false,mkdir:false,test:false,cat:false,sed:false,grep:false,which:false,dirs:false,pushd:false,popd:false,env:false,exec:false,chmod:false,config:false,error:false,tempdir:false};exports.typed={ArrayBuffer:false,ArrayBufferView:false,DataView:false,Float32Array:false,Float64Array:false,Int16Array:false,Int32Array:false,Int8Array:false,Uint16Array:false,Uint32Array:false,Uint8Array:false,Uint8ClampedArray:false};exports.wsh={ActiveXObject:true,Enumerator:true,GetObject:true,ScriptEngine:true,ScriptEngineBuildVersion:true,ScriptEngineMajorVersion:true,ScriptEngineMinorVersion:true,VBArray:true,WSH:true,WScript:true,XDomainRequest:true};exports.dojo={dojo:false,dijit:false,dojox:false,define:false,require:false};exports.jquery={$:false,jQuery:false};exports.mootools={$:false,$$:false,Asset:false,Browser:false,Chain:false,Class:false,Color:false,Cookie:false,Core:false,Document:false,DomReady:false,DOMEvent:false,DOMReady:false,Drag:false,Element:false,Elements:false,Event:false,Events:false,Fx:false,Group:false,Hash:false,HtmlTable:false,IFrame:false,IframeShim:false,InputValidator:false,instanceOf:false,Keyboard:false,Locale:false,Mask:false,MooTools:false,Native:false,Options:false,OverText:false,
Request:false,Scroller:false,Slick:false,Slider:false,Sortables:false,Spinner:false,Swiff:false,Tips:false,Type:false,typeOf:false,URI:false,Window:false};exports.prototypejs={$:false,$$:false,$A:false,$F:false,$H:false,$R:false,$break:false,$continue:false,$w:false,Abstract:false,Ajax:false,Class:false,Enumerable:false,Element:false,Event:false,Field:false,Form:false,Hash:false,Insertion:false,ObjectRange:false,PeriodicalExecuter:false,Position:false,Prototype:false,Selector:false,Template:false,Toggle:false,Try:false,Autocompleter:false,Builder:false,Control:false,Draggable:false,Draggables:false,Droppables:false,Effect:false,Sortable:false,SortableObserver:false,Sound:false,Scriptaculous:false};exports.yui={YUI:false,Y:false,YUI_config:false};exports.mocha={mocha:false,describe:false,xdescribe:false,it:false,xit:false,context:false,xcontext:false,before:false,after:false,beforeEach:false,afterEach:false,suite:false,test:false,setup:false,teardown:false,suiteSetup:false,suiteTeardown:false};exports.jasmine={jasmine:false,describe:false,xdescribe:false,it:false,xit:false,beforeEach:false,afterEach:false,setFixtures:false,loadFixtures:false,spyOn:false,expect:false,runs:false,waitsFor:false,waits:false,beforeAll:false,afterAll:false,fail:false,fdescribe:false,fit:false,pending:false}},{}],jshint:[function(require,module,exports){var _=require("lodash");var events=require("events");var vars=require("./vars.js");var messages=require("./messages.js");var Lexer=require("./lex.js").Lexer;var reg=require("./reg.js");var state=require("./state.js").state;var style=require("./style.js");var options=require("./options.js");var scopeManager=require("./scope-manager.js");var console=require("console-browserify");var JSHINT=function(){"use strict";var api,bang={"<":true,"<=":true,"==":true,"===":true,"!==":true,"!=":true,">":true,">=":true,"+":true,"-":true,"*":true,"/":true,"%":true},declared,functionicity=["closure","exception","global","label","outer","unused","var"],functions,inblock,indent,lookahead,lex,member,membersOnly,predefined,stack,urls,extraModules=[],emitter=new events.EventEmitter;function checkOption(name,t){name=name.trim();if(/^[+-]W\d{3}$/g.test(name)){return true}if(options.validNames.indexOf(name)===-1){if(t.type!=="jslint"&&!_.has(options.removed,name)){error("E001",t,name);return false}}return true}function isString(obj){return Object.prototype.toString.call(obj)==="[object String]"}function isIdentifier(tkn,value){if(!tkn)return false;if(!tkn.identifier||tkn.value!==value)return false;return true}function isReserved(token){if(!token.reserved){return false}var meta=token.meta;if(meta&&meta.isFutureReservedWord){if(meta.moduleOnly&&!state.option.module){return false}if(state.inES5()){if(!meta.es5){return false}if(meta.strictOnly){if(!state.option.strict&&!state.isStrict()){return false}}if(token.isProperty){return false}}}return true}function supplant(str,data){return str.replace(/\{([^{}]*)\}/g,function(a,b){var r=data[b];return typeof r==="string"||typeof r==="number"?r:a})}function combine(dest,src){Object.keys(src).forEach(function(name){if(_.has(JSHINT.blacklist,name))return;dest[name]=src[name]})}function processenforceall(){if(state.option.enforceall){for(var enforceopt in options.bool.enforcing){if(state.option[enforceopt]===undefined&&!options.noenforceall[enforceopt]){state.option[enforceopt]=true}}for(var relaxopt in options.bool.relaxing){if(state.option[relaxopt]===undefined){state.option[relaxopt]=false}}}}function applyOptions(){var badESOpt=null;processenforceall();badESOpt=state.inferEsVersion();if(badESOpt){quit("E059",state.tokens.next,"esversion",badESOpt)}if(state.inES5()){combine(predefined,vars.ecmaIdentifiers[5])}if(state.inES6()){combine(predefined,vars.ecmaIdentifiers[6])}if(state.option.strict==="global"&&"globalstrict"in state.option){quit("E059",state.tokens.next,"strict","globalstrict")}if(state.option.module){if(!state.inES6()){warning("W134",state.tokens.next,"module",6)}}if(state.option.couch){combine(predefined,vars.couch)}if(state.option.qunit){combine(predefined,vars.qunit)}if(state.option.rhino){combine(predefined,vars.rhino)}if(state.option.shelljs){combine(predefined,vars.shelljs);combine(predefined,vars.node)}if(state.option.typed){combine(predefined,vars.typed)}if(state.option.phantom){combine(predefined,vars.phantom)}if(state.option.prototypejs){combine(predefined,vars.prototypejs)}if(state.option.node){combine(predefined,vars.node);combine(predefined,vars.typed)}if(state.option.devel){combine(predefined,vars.devel)}if(state.option.dojo){combine(predefined,vars.dojo)}if(state.option.browser){combine(predefined,vars.browser);combine(predefined,vars.typed)}if(state.option.browserify){combine(predefined,vars.browser);combine(predefined,vars.typed);combine(predefined,vars.browserify)}if(state.option.nonstandard){combine(predefined,vars.nonstandard)}if(state.option.jasmine){combine(predefined,vars.jasmine)}if(state.option.jquery){combine(predefined,vars.jquery)}if(state.option.mootools){combine(predefined,vars.mootools)}if(state.option.worker){combine(predefined,vars.worker)}if(state.option.wsh){combine(predefined,vars.wsh)}if(state.option.yui){combine(predefined,vars.yui)}if(state.option.mocha){combine(predefined,vars.mocha)}}function quit(code,token,a,b){var percentage=Math.floor(token.line/state.lines.length*100);var message=messages.errors[code].desc;var exception={name:"JSHintError",line:token.line,character:token.from,message:message+" ("+percentage+"% scanned).",raw:message,code:code,a:a,b:b};exception.reason=supplant(message,exception)+" ("+percentage+"% scanned).";throw exception}function removeIgnoredMessages(){var ignored=state.ignoredLines;if(_.isEmpty(ignored))return;JSHINT.errors=_.reject(JSHINT.errors,function(err){return ignored[err.line]})}function warning(code,t,a,b,c,d){var ch,l,w,msg;if(/^W\d{3}$/.test(code)){if(state.ignored[code])return;msg=messages.warnings[code]}else if(/E\d{3}/.test(code)){msg=messages.errors[code]}else if(/I\d{3}/.test(code)){msg=messages.info[code]}t=t||state.tokens.next||{};if(t.id==="(end)"){t=state.tokens.curr}l=t.line;ch=t.from;w={id:"(error)",raw:msg.desc,code:msg.code,evidence:state.lines[l-1]||"",line:l,character:ch,scope:JSHINT.scope,a:a,b:b,c:c,d:d};w.reason=supplant(msg.desc,w);JSHINT.errors.push(w);removeIgnoredMessages();if(JSHINT.errors.length>=state.option.maxerr)quit("E043",t);return w}function warningAt(m,l,ch,a,b,c,d){return warning(m,{line:l,from:ch},a,b,c,d)}function error(m,t,a,b,c,d){warning(m,t,a,b,c,d)}function errorAt(m,l,ch,a,b,c,d){return error(m,{line:l,from:ch},a,b,c,d)}function addEvalCode(elem,token){JSHINT.internals.push({id:"(internal)",elem:elem,token:token,code:token.value.replace(/([^\\])(\\*)\2\\n/g,"$1\n")})}function lintingDirective(directiveToken,previous){var body=directiveToken.body.split(",").map(function(s){return s.trim()});var predef={};if(directiveToken.type==="falls through"){previous.caseFallsThrough=true;return}if(directiveToken.type==="globals"){body.forEach(function(g,idx){g=g.split(":");var key=(g[0]||"").trim();var val=(g[1]||"").trim();if(key==="-"||!key.length){if(idx>0&&idx===body.length-1){return}error("E002",directiveToken);return}if(key.charAt(0)==="-"){key=key.slice(1);val=false;JSHINT.blacklist[key]=key;delete predefined[key]}else{predef[key]=val==="true"}});combine(predefined,predef);for(var key in predef){if(_.has(predef,key)){declared[key]=directiveToken}}}if(directiveToken.type==="exported"){body.forEach(function(e,idx){if(!e.length){if(idx>0&&idx===body.length-1){return}error("E002",directiveToken);return}state.funct["(scope)"].addExported(e)})}if(directiveToken.type==="members"){membersOnly=membersOnly||{};body.forEach(function(m){var ch1=m.charAt(0);var ch2=m.charAt(m.length-1);if(ch1===ch2&&(ch1==='"'||ch1==="'")){m=m.substr(1,m.length-2).replace('\\"','"')}membersOnly[m]=false})}var numvals=["maxstatements","maxparams","maxdepth","maxcomplexity","maxerr","maxlen","indent"];if(directiveToken.type==="jshint"||directiveToken.type==="jslint"){body.forEach(function(g){g=g.split(":");var key=(g[0]||"").trim();var val=(g[1]||"").trim();if(!checkOption(key,directiveToken)){return}if(numvals.indexOf(key)>=0){if(val!=="false"){val=+val;if(typeof val!=="number"||!isFinite(val)||val<=0||Math.floor(val)!==val){error("E032",directiveToken,g[1].trim());return}state.option[key]=val}else{state.option[key]=key==="indent"?4:false}return}if(key==="validthis"){if(state.funct["(global)"])return void error("E009");if(val!=="true"&&val!=="false")return void error("E002",directiveToken);state.option.validthis=val==="true";return}if(key==="quotmark"){switch(val){case"true":case"false":state.option.quotmark=val==="true";break;case"double":case"single":state.option.quotmark=val;break;default:error("E002",directiveToken)}return}if(key==="shadow"){switch(val){case"true":state.option.shadow=true;break;case"outer":state.option.shadow="outer";break;case"false":case"inner":state.option.shadow="inner";break;default:error("E002",directiveToken)}return}if(key==="unused"){switch(val){case"true":state.option.unused=true;break;case"false":state.option.unused=false;break;case"vars":case"strict":state.option.unused=val;break;default:error("E002",directiveToken)}return}if(key==="latedef"){switch(val){case"true":state.option.latedef=true;break;case"false":state.option.latedef=false;break;case"nofunc":state.option.latedef="nofunc";break;default:error("E002",directiveToken)}return}if(key==="ignore"){switch(val){case"line":state.ignoredLines[directiveToken.line]=true;removeIgnoredMessages();break;default:error("E002",directiveToken)}return}if(key==="strict"){switch(val){case"true":state.option.strict=true;break;case"false":state.option.strict=false;break;case"global":case"implied":state.option.strict=val;break;default:error("E002",directiveToken)}return}if(key==="module"){if(!hasParsedCode(state.funct)){error("E055",directiveToken,"module")}}if(key==="esversion"){switch(val){case"3":case"5":case"6":case"2015":state.option.moz=false;state.option.esversion=+val;break;default:error("E002",directiveToken)}if(!hasParsedCode(state.funct)){error("E055",directiveToken,"esversion")}return}var match=/^([+-])(W\d{3})$/g.exec(key);if(match){state.ignored[match[2]]=match[1]==="-";return}var tn;if(val==="true"||val==="false"){if(directiveToken.type==="jslint"){tn=options.renamed[key]||key;state.option[tn]=val==="true";if(options.inverted[tn]!==undefined){state.option[tn]=!state.option[tn]}}else{state.option[key]=val==="true"}return}error("E002",directiveToken)});applyOptions()}}function peek(p){var i=p||0,j=lookahead.length,t;if(i<j){return lookahead[i]}while(j<=i){t=lex.token();if(!t){if(!lookahead.length){return state.tokens.next}return lookahead[j-1]}lookahead[j]=t;j+=1}return t}function peekIgnoreEOL(){var i=0;var t;do{t=peek(i++)}while(t.id==="(endline)");return t}function advance(id,t){switch(state.tokens.curr.id){case"(number)":if(state.tokens.next.id==="."){warning("W005",state.tokens.curr)}break;case"-":if(state.tokens.next.id==="-"||state.tokens.next.id==="--"){warning("W006")}break;case"+":if(state.tokens.next.id==="+"||state.tokens.next.id==="++"){warning("W007")}break}if(id&&state.tokens.next.id!==id){if(t){if(state.tokens.next.id==="(end)"){error("E019",t,t.id)}else{error("E020",state.tokens.next,id,t.id,t.line,state.tokens.next.value)}}else if(state.tokens.next.type!=="(identifier)"||state.tokens.next.value!==id){warning("W116",state.tokens.next,id,state.tokens.next.value)}}state.tokens.prev=state.tokens.curr;state.tokens.curr=state.tokens.next;for(;;){state.tokens.next=lookahead.shift()||lex.token();if(!state.tokens.next){quit("E041",state.tokens.curr)}if(state.tokens.next.id==="(end)"||state.tokens.next.id==="(error)"){return}if(state.tokens.next.check){state.tokens.next.check()}if(state.tokens.next.isSpecial){lintingDirective(state.tokens.next,state.tokens.curr)}else{if(state.tokens.next.id!=="(endline)"){break}}}}function isOperator(token){return token.first||token.right||token.left||token.id==="yield"}function isEndOfExpr(curr,next){if(arguments.length===0){curr=state.tokens.curr;next=state.tokens.next}if(next.id===";"||next.id==="}"||next.id===":"){return true}if(next.infix===curr.infix||curr.ltBoundary==="after"||next.ltBoundary==="before"){return curr.line!==startLine(next)}return false}function expression(rbp,initial){var left,isArray=false,isObject=false,isLetExpr=false;state.nameStack.push();if(!initial&&state.tokens.next.value==="let"&&peek(0).value==="("){if(!state.inMoz()){warning("W118",state.tokens.next,"let expressions")}isLetExpr=true;state.funct["(scope)"].stack();advance("let");advance("(");state.tokens.prev.fud();advance(")")}if(state.tokens.next.id==="(end)")error("E006",state.tokens.curr);var isDangerous=state.option.asi&&state.tokens.prev.line!==startLine(state.tokens.curr)&&_.contains(["]",")"],state.tokens.prev.id)&&_.contains(["[","("],state.tokens.curr.id);if(isDangerous)warning("W014",state.tokens.curr,state.tokens.curr.id);advance();if(initial){state.funct["(verb)"]=state.tokens.curr.value;state.tokens.curr.beginsStmt=true}if(initial===true&&state.tokens.curr.fud){left=state.tokens.curr.fud()}else{if(state.tokens.curr.nud){left=state.tokens.curr.nud(rbp)}else{error("E030",state.tokens.curr,state.tokens.curr.id)}while(rbp<state.tokens.next.lbp&&!isEndOfExpr()){isArray=state.tokens.curr.value==="Array";isObject=state.tokens.curr.value==="Object";if(left&&(left.value||left.first&&left.first.value)){if(left.value!=="new"||left.first&&left.first.value&&left.first.value==="."){isArray=false;if(left.value!==state.tokens.curr.value){isObject=false}}}advance();if(isArray&&state.tokens.curr.id==="("&&state.tokens.next.id===")"){warning("W009",state.tokens.curr)}if(isObject&&state.tokens.curr.id==="("&&state.tokens.next.id===")"){warning("W010",state.tokens.curr)}if(left&&state.tokens.curr.led){left=state.tokens.curr.led(left)}else{error("E033",state.tokens.curr,state.tokens.curr.id)}}}if(isLetExpr){state.funct["(scope)"].unstack()}state.nameStack.pop();return left}function startLine(token){return token.startLine||token.line}function nobreaknonadjacent(left,right){left=left||state.tokens.curr;right=right||state.tokens.next;if(!state.option.laxbreak&&left.line!==startLine(right)){warning("W014",right,right.value)}}function nolinebreak(t){t=t||state.tokens.curr;if(t.line!==startLine(state.tokens.next)){warning("E022",t,t.value)}}function nobreakcomma(left,right){if(left.line!==startLine(right)){if(!state.option.laxcomma){if(parseComma.first){warning("I001");parseComma.first=false}warning("W014",left,right.value)}}}function parseComma(opts){opts=opts||{};if(!opts.peek){nobreakcomma(state.tokens.curr,state.tokens.next);advance(",")}else{nobreakcomma(state.tokens.prev,state.tokens.curr)}if(state.tokens.next.identifier&&!(opts.property&&state.inES5())){switch(state.tokens.next.value){case"break":case"case":case"catch":case"continue":case"default":case"do":case"else":case"finally":case"for":case"if":case"in":case"instanceof":case"return":case"switch":case"throw":case"try":case"var":case"let":case"while":case"with":error("E024",state.tokens.next,state.tokens.next.value);return false}}if(state.tokens.next.type==="(punctuator)"){switch(state.tokens.next.value){case"}":case"]":case",":if(opts.allowTrailing){return true}case")":error("E024",state.tokens.next,state.tokens.next.value);return false}}return true}function symbol(s,p){var x=state.syntax[s];if(!x||typeof x!=="object"){state.syntax[s]=x={id:s,lbp:p,value:s}}return x}function delim(s){var x=symbol(s,0);x.delim=true;return x}function stmt(s,f){var x=delim(s);x.identifier=x.reserved=true;x.fud=f;return x}function blockstmt(s,f){var x=stmt(s,f);x.block=true;return x}function reserveName(x){var c=x.id.charAt(0);if(c>="a"&&c<="z"||c>="A"&&c<="Z"){x.identifier=x.reserved=true}return x}function prefix(s,f){var x=symbol(s,150);reserveName(x);x.nud=typeof f==="function"?f:function(){this.arity="unary";this.right=expression(150);if(this.id==="++"||this.id==="--"){if(state.option.plusplus){warning("W016",this,this.id)}else if(this.right&&(!this.right.identifier||isReserved(this.right))&&this.right.id!=="."&&this.right.id!=="["){warning("W017",this)}if(this.right&&this.right.isMetaProperty){error("E031",this)}else if(this.right&&this.right.identifier){state.funct["(scope)"].block.modify(this.right.value,this)}}return this};return x}function type(s,f){var x=delim(s);x.type=s;x.nud=f;return x}function reserve(name,func){var x=type(name,func);x.identifier=true;x.reserved=true;return x}function FutureReservedWord(name,meta){var x=type(name,meta&&meta.nud||function(){return this});meta=meta||{};meta.isFutureReservedWord=true;x.value=name;x.identifier=true;x.reserved=true;x.meta=meta;return x}function reservevar(s,v){return reserve(s,function(){if(typeof v==="function"){v(this)}return this})}function infix(s,f,p,w){var x=symbol(s,p);reserveName(x);x.infix=true;x.led=function(left){if(!w){nobreaknonadjacent(state.tokens.prev,state.tokens.curr)}if((s==="in"||s==="instanceof")&&left.id==="!"){warning("W018",left,"!")}if(typeof f==="function"){return f(left,this)}else{this.left=left;this.right=expression(p);return this}};return x}function application(s){var x=symbol(s,42);x.infix=true;x.led=function(left){nobreaknonadjacent(state.tokens.prev,state.tokens.curr);this.left=left;this.right=doFunction({type:"arrow",loneArg:left});return this};return x}function relation(s,f){var x=symbol(s,100);x.infix=true;x.led=function(left){nobreaknonadjacent(state.tokens.prev,state.tokens.curr);this.left=left;var right=this.right=expression(100);if(isIdentifier(left,"NaN")||isIdentifier(right,"NaN")){warning("W019",this)}else if(f){f.apply(this,[left,right])}if(!left||!right){quit("E041",state.tokens.curr)}if(left.id==="!"){warning("W018",left,"!")}if(right.id==="!"){warning("W018",right,"!")}return this};return x}var typeofValues={};typeofValues.legacy=["xml","unknown"];typeofValues.es3=["undefined","boolean","number","string","function","object"];typeofValues.es3=typeofValues.es3.concat(typeofValues.legacy);typeofValues.es6=typeofValues.es3.concat("symbol");function isTypoTypeof(left,right,state){var values;if(state.option.notypeof)return false;if(!left||!right)return false;values=state.inES6()?typeofValues.es6:typeofValues.es3;if(right.type==="(identifier)"&&right.value==="typeof"&&left.type==="(string)")return!_.contains(values,left.value);return false}function isGlobalEval(left,state){var isGlobal=false;if(left.type==="this"&&state.funct["(context)"]===null){isGlobal=true}else if(left.type==="(identifier)"){if(state.option.node&&left.value==="global"){isGlobal=true}else if(state.option.browser&&(left.value==="window"||left.value==="document")){isGlobal=true}}return isGlobal}function findNativePrototype(left){var natives=["Array","ArrayBuffer","Boolean","Collator","DataView","Date","DateTimeFormat","Error","EvalError","Float32Array","Float64Array","Function","Infinity","Intl","Int16Array","Int32Array","Int8Array","Iterator","Number","NumberFormat","Object","RangeError","ReferenceError","RegExp","StopIteration","String","SyntaxError","TypeError","Uint16Array","Uint32Array","Uint8Array","Uint8ClampedArray","URIError"];function walkPrototype(obj){if(typeof obj!=="object")return;return obj.right==="prototype"?obj:walkPrototype(obj.left)}function walkNative(obj){while(!obj.identifier&&typeof obj.left==="object")obj=obj.left;if(obj.identifier&&natives.indexOf(obj.value)>=0&&state.funct["(scope)"].isPredefined(obj.value)){return obj.value}}var prototype=walkPrototype(left);if(prototype)return walkNative(prototype)}function checkLeftSideAssign(left,assignToken,options){var allowDestructuring=options&&options.allowDestructuring;assignToken=assignToken||left;if(state.option.freeze){var nativeObject=findNativePrototype(left);if(nativeObject)warning("W121",left,nativeObject)}if(checkPunctuator(left,"...")){left=left.right}if(left.identifier&&!left.isMetaProperty){state.funct["(scope)"].block.reassign(left.value,left)}if(left.id==="."){if(!left.left||left.left.value==="arguments"&&!state.isStrict()){warning("E031",assignToken)}state.nameStack.set(state.tokens.prev);return true}else if(left.id==="{"||left.id==="["){if(!allowDestructuring||!left.destructAssign){if(left.id==="{"||!left.left){warning("E031",assignToken)}else if(left.left.value==="arguments"&&!state.isStrict()){warning("E031",assignToken)}}if(left.id==="["){state.nameStack.set(left.right)}return true}else if(left.identifier&&!isReserved(left)&&!left.isMetaProperty){if(state.funct["(scope)"].labeltype(left.value)==="exception"){warning("W022",left)}state.nameStack.set(left);return true}if(left===state.syntax["function"]){warning("W023",state.tokens.curr)}else{error("E031",assignToken)}return false}function assignop(s,f,p){var x=infix(s,typeof f==="function"?f:function(left,that){that.left=left;checkLeftSideAssign(left,that,{allowDestructuring:true});that.right=expression(10);return that},p);x.exps=true;x.assign=true;return x}function bitwise(s,f,p){var x=symbol(s,p);reserveName(x);x.infix=true;x.led=typeof f==="function"?f:function(left){if(state.option.bitwise){warning("W016",this,this.id)}this.left=left;this.right=expression(p);return this};return x}function bitwiseassignop(s){return assignop(s,function(left,that){if(state.option.bitwise){warning("W016",that,that.id)}checkLeftSideAssign(left,that);that.right=expression(10);return that},20)}function suffix(s){var x=symbol(s,150);x.led=function(left){if(state.option.plusplus){warning("W016",this,this.id)}else if((!left.identifier||isReserved(left))&&left.id!=="."&&left.id!=="["){warning("W017",this)}if(left.isMetaProperty){error("E031",this)}else if(left&&left.identifier){state.funct["(scope)"].block.modify(left.value,left)}this.left=left;return this};return x}function optionalidentifier(fnparam,prop,preserve){if(!state.tokens.next.identifier){return}if(!preserve){advance()}var curr=state.tokens.curr;var val=state.tokens.curr.value;if(!isReserved(curr)){return val}if(prop){if(state.inES5()){return val}}if(fnparam&&val==="undefined"){return val}warning("W024",state.tokens.curr,state.tokens.curr.id);return val}function identifier(fnparam,prop){var i=optionalidentifier(fnparam,prop,false);if(i){return i}if(state.tokens.next.value==="..."){if(!state.inES6(true)){warning("W119",state.tokens.next,"spread/rest operator","6")}advance();if(checkPunctuator(state.tokens.next,"...")){warning("E024",state.tokens.next,"...");while(checkPunctuator(state.tokens.next,"...")){advance()}}if(!state.tokens.next.identifier){warning("E024",state.tokens.curr,state.tokens.next.id);return}return identifier(fnparam,prop)}else{error("E030",state.tokens.next,state.tokens.next.value);if(state.tokens.next.id!==";"){advance()}}}function reachable(controlToken){var i=0,t;if(state.tokens.next.id!==";"||controlToken.inBracelessBlock){return}for(;;){do{t=peek(i);i+=1}while(t.id!=="(end)"&&t.id==="(comment)");if(t.reach){return}if(t.id!=="(endline)"){if(t.id==="function"){if(state.option.latedef===true){warning("W026",t)}break}warning("W027",t,t.value,controlToken.value);break}}}function parseFinalSemicolon(stmt){if(state.tokens.next.id!==";"){if(state.tokens.next.isUnclosed)return advance();var sameLine=startLine(state.tokens.next)===state.tokens.curr.line&&state.tokens.next.id!=="(end)";var blockEnd=checkPunctuator(state.tokens.next,"}");if(sameLine&&!blockEnd&&!(stmt.id==="do"&&state.inES6(true))){errorAt("E058",state.tokens.curr.line,state.tokens.curr.character)}else if(!state.option.asi){if(!(blockEnd&&sameLine&&state.option.lastsemic)){warningAt("W033",state.tokens.curr.line,state.tokens.curr.character)}}}else{advance(";")}}function statement(){var i=indent,r,t=state.tokens.next,hasOwnScope=false;if(t.id===";"){advance(";");return}var res=isReserved(t);if(res&&t.meta&&t.meta.isFutureReservedWord&&peek().id===":"){warning("W024",t,t.id);res=false}if(t.identifier&&!res&&peek().id===":"){advance();advance(":");hasOwnScope=true;state.funct["(scope)"].stack();state.funct["(scope)"].block.addBreakLabel(t.value,{token:state.tokens.curr});if(!state.tokens.next.labelled&&state.tokens.next.value!=="{"){warning("W028",state.tokens.next,t.value,state.tokens.next.value)}state.tokens.next.label=t.value;t=state.tokens.next}if(t.id==="{"){var iscase=state.funct["(verb)"]==="case"&&state.tokens.curr.value===":";block(true,true,false,false,iscase);if(hasOwnScope){state.funct["(scope)"].unstack()}return}r=expression(0,true);if(r&&!(r.identifier&&r.value==="function")&&!(r.type==="(punctuator)"&&r.left&&r.left.identifier&&r.left.value==="function")){if(!state.isStrict()&&state.stmtMissingStrict()){warning("E007")}}if(!t.block){if(!state.option.expr&&(!r||!r.exps)){warning("W030",state.tokens.curr)}else if(state.option.nonew&&r&&r.left&&r.id==="("&&r.left.id==="new"){warning("W031",t)}parseFinalSemicolon(t)}indent=i;if(hasOwnScope){state.funct["(scope)"].unstack()}return r}function statements(){var a=[],p;while(!state.tokens.next.reach&&state.tokens.next.id!=="(end)"){if(state.tokens.next.id===";"){p=peek();if(!p||p.id!=="("&&p.id!=="["){warning("W032")}advance(";")}else{a.push(statement())}}return a}function directives(){var current=state.tokens.next;while(state.tokens.next.id==="(string)"){var next=peekIgnoreEOL();if(!isEndOfExpr(current,next)){break}current=next;advance();var directive=state.tokens.curr.value;if(state.directive[directive]||directive==="use strict"&&state.option.strict==="implied"){warning("W034",state.tokens.curr,directive)}state.directive[directive]=true;parseFinalSemicolon(current)}if(state.isStrict()){state.option.undef=true}}function block(ordinary,stmt,isfunc,isfatarrow,iscase){var a,b=inblock,old_indent=indent,m,t,line,d;inblock=ordinary;t=state.tokens.next;var metrics=state.funct["(metrics)"];metrics.nestedBlockDepth+=1;metrics.verifyMaxNestedBlockDepthPerFunction();if(state.tokens.next.id==="{"){advance("{");state.funct["(scope)"].stack();state.funct["(noblockscopedvar)"]=false;line=state.tokens.curr.line;if(state.tokens.next.id!=="}"){indent+=state.option.indent;while(!ordinary&&state.tokens.next.from>indent){indent+=state.option.indent}if(isfunc){m={};for(d in state.directive){if(_.has(state.directive,d)){m[d]=state.directive[d]}}directives();if(state.option.strict&&state.funct["(context)"]["(global)"]){if(!m["use strict"]&&!state.isStrict()){warning("E007")}}}a=statements();metrics.statementCount+=a.length;indent-=state.option.indent}advance("}",t);if(isfunc){state.funct["(scope)"].validateParams();if(m){state.directive=m}}state.funct["(scope)"].unstack();indent=old_indent}else if(!ordinary){if(isfunc){state.funct["(scope)"].stack();m={};if(stmt&&!isfatarrow&&!state.inMoz()){error("W118",state.tokens.curr,"function closure expressions")}if(!stmt){for(d in state.directive){if(_.has(state.directive,d)){m[d]=state.directive[d]}}}expression(10);if(state.option.strict&&state.funct["(context)"]["(global)"]){if(!m["use strict"]&&!state.isStrict()){warning("E007")}}state.funct["(scope)"].unstack()}else{error("E021",state.tokens.next,"{",state.tokens.next.value)}}else{state.funct["(noblockscopedvar)"]=state.tokens.next.id!=="for";state.funct["(scope)"].stack();if(!stmt||state.option.curly){warning("W116",state.tokens.next,"{",state.tokens.next.value)}state.tokens.next.inBracelessBlock=true;indent+=state.option.indent;a=[statement()];indent-=state.option.indent;state.funct["(scope)"].unstack();delete state.funct["(noblockscopedvar)"]}switch(state.funct["(verb)"]){case"break":case"continue":case"return":case"throw":if(iscase){break}default:state.funct["(verb)"]=null}inblock=b;if(ordinary&&state.option.noempty&&(!a||a.length===0)){warning("W035",state.tokens.prev)}metrics.nestedBlockDepth-=1;return a}function countMember(m){if(membersOnly&&typeof membersOnly[m]!=="boolean"){warning("W036",state.tokens.curr,m)}if(typeof member[m]==="number"){member[m]+=1}else{member[m]=1}}type("(number)",function(){return this});type("(string)",function(){return this});state.syntax["(identifier)"]={type:"(identifier)",lbp:0,identifier:true,nud:function(){var v=this.value;if(state.tokens.next.id==="=>"){return this}if(!state.funct["(comparray)"].check(v)){state.funct["(scope)"].block.use(v,state.tokens.curr)}return this},led:function(){error("E033",state.tokens.next,state.tokens.next.value)}};var baseTemplateSyntax={identifier:false,template:true};state.syntax["(template)"]=_.extend({lbp:155,type:"(template)",nud:doTemplateLiteral,led:doTemplateLiteral,noSubst:false},baseTemplateSyntax);state.syntax["(template middle)"]=_.extend({lbp:0,type:"(template middle)",middle:true,noSubst:false},baseTemplateSyntax);state.syntax["(template tail)"]=_.extend({lbp:0,type:"(template tail)",tail:true,noSubst:false},baseTemplateSyntax);state.syntax["(no subst template)"]=_.extend({lbp:155,type:"(template)",nud:doTemplateLiteral,led:doTemplateLiteral,noSubst:true,tail:true},baseTemplateSyntax);type("(regexp)",function(){return this});delim("(endline)");(function(x){x.line=x.from=0})(delim("(begin)"));delim("(end)").reach=true;delim("(error)").reach=true;delim("}").reach=true;delim(")");delim("]");delim('"').reach=true;delim("'").reach=true;delim(";");delim(":").reach=true;delim("#");reserve("else");reserve("case").reach=true;reserve("catch");reserve("default").reach=true;reserve("finally");reservevar("arguments",function(x){if(state.isStrict()&&state.funct["(global)"]){warning("E008",x)}});reservevar("eval");reservevar("false");reservevar("Infinity");reservevar("null");reservevar("this",function(x){if(state.isStrict()&&!isMethod()&&!state.option.validthis&&(state.funct["(statement)"]&&state.funct["(name)"].charAt(0)>"Z"||state.funct["(global)"])){warning("W040",x)}});reservevar("true");reservevar("undefined");assignop("=","assign",20);assignop("+=","assignadd",20);assignop("-=","assignsub",20);assignop("*=","assignmult",20);assignop("/=","assigndiv",20).nud=function(){error("E014")};assignop("%=","assignmod",20);bitwiseassignop("&=");bitwiseassignop("|=");bitwiseassignop("^=");bitwiseassignop("<<=");bitwiseassignop(">>=");bitwiseassignop(">>>=");infix(",",function(left,that){var expr;that.exprs=[left];if(state.option.nocomma){warning("W127")}if(!parseComma({peek:true})){return that}while(true){if(!(expr=expression(10))){break}that.exprs.push(expr);if(state.tokens.next.value!==","||!parseComma()){break}}return that},10,true);infix("?",function(left,that){increaseComplexityCount();that.left=left;that.right=expression(10);advance(":");that["else"]=expression(10);return that},30);var orPrecendence=40;infix("||",function(left,that){increaseComplexityCount();that.left=left;that.right=expression(orPrecendence);return that},orPrecendence);infix("&&","and",50);bitwise("|","bitor",70);bitwise("^","bitxor",80);bitwise("&","bitand",90);relation("==",function(left,right){var eqnull=state.option.eqnull&&((left&&left.value)==="null"||(right&&right.value)==="null");switch(true){case!eqnull&&state.option.eqeqeq:this.from=this.character;warning("W116",this,"===","==");break;case isTypoTypeof(right,left,state):warning("W122",this,right.value);break;case isTypoTypeof(left,right,state):warning("W122",this,left.value);break}return this});relation("===",function(left,right){if(isTypoTypeof(right,left,state)){warning("W122",this,right.value)}else if(isTypoTypeof(left,right,state)){warning("W122",this,left.value)}return this});relation("!=",function(left,right){var eqnull=state.option.eqnull&&((left&&left.value)==="null"||(right&&right.value)==="null");if(!eqnull&&state.option.eqeqeq){this.from=this.character;warning("W116",this,"!==","!=")}else if(isTypoTypeof(right,left,state)){warning("W122",this,right.value)}else if(isTypoTypeof(left,right,state)){warning("W122",this,left.value)}return this});relation("!==",function(left,right){if(isTypoTypeof(right,left,state)){warning("W122",this,right.value)}else if(isTypoTypeof(left,right,state)){warning("W122",this,left.value)}return this});relation("<");relation(">");relation("<=");relation(">=");
bitwise("<<","shiftleft",120);bitwise(">>","shiftright",120);bitwise(">>>","shiftrightunsigned",120);infix("in","in",120);infix("instanceof",function(left,token){var right;var scope=state.funct["(scope)"];token.left=left;token.right=right=expression(120);if(!right){return token}if(right.id==="(number)"||right.id==="(string)"||right.value==="null"||right.value==="undefined"&&!scope.has("undefined")||right.arity==="unary"||right.id==="{"||right.id==="["&&!right.right||right.id==="(regexp)"||right.id==="(template)"&&!right.tag){error("E060")}if(right.id==="function"){warning("W139")}return token},120);infix("+",function(left,that){var right;that.left=left;that.right=right=expression(130);if(left&&right&&left.id==="(string)"&&right.id==="(string)"){left.value+=right.value;left.character=right.character;if(!state.option.scripturl&&reg.javascriptURL.test(left.value)){warning("W050",left)}return left}return that},130);prefix("+","num");prefix("+++",function(){warning("W007");this.arity="unary";this.right=expression(150);return this});infix("+++",function(left){warning("W007");this.left=left;this.right=expression(130);return this},130);infix("-","sub",130);prefix("-","neg");prefix("---",function(){warning("W006");this.arity="unary";this.right=expression(150);return this});infix("---",function(left){warning("W006");this.left=left;this.right=expression(130);return this},130);infix("*","mult",140);infix("/","div",140);infix("%","mod",140);suffix("++");prefix("++","preinc");state.syntax["++"].exps=true;state.syntax["++"].ltBoundary="before";suffix("--");prefix("--","predec");state.syntax["--"].exps=true;state.syntax["--"].ltBoundary="before";prefix("delete",function(){var p=expression(10);if(!p){return this}if(p.id!=="."&&p.id!=="["){warning("W051")}this.first=p;if(p.identifier&&!state.isStrict()){p.forgiveUndef=true}return this}).exps=true;prefix("~",function(){if(state.option.bitwise){warning("W016",this,"~")}this.arity="unary";this.right=expression(150);return this});prefix("...",function(){if(!state.inES6(true)){warning("W119",this,"spread/rest operator","6")}if(!state.tokens.next.identifier&&state.tokens.next.type!=="(string)"&&!checkPunctuators(state.tokens.next,["[","("])){error("E030",state.tokens.next,state.tokens.next.value)}this.right=expression(150);return this});prefix("!",function(){this.arity="unary";this.right=expression(150);if(!this.right){quit("E041",this)}if(bang[this.right.id]===true){warning("W018",this,"!")}return this});prefix("typeof",function(){var p=expression(150);this.first=this.right=p;if(!p){quit("E041",this)}if(p.identifier){p.forgiveUndef=true}return this});prefix("new",function(){var mp=metaProperty("target",function(){if(!state.inES6(true)){warning("W119",state.tokens.prev,"new.target","6")}var inFunction,c=state.funct;while(c){inFunction=!c["(global)"];if(!c["(arrow)"]){break}c=c["(context)"]}if(!inFunction){warning("W136",state.tokens.prev,"new.target")}});if(mp){return mp}var c=expression(155),i;if(c&&c.id!=="function"){if(c.identifier){c["new"]=true;switch(c.value){case"Number":case"String":case"Boolean":case"Math":case"JSON":warning("W053",state.tokens.prev,c.value);break;case"Symbol":if(state.inES6()){warning("W053",state.tokens.prev,c.value)}break;case"Function":if(!state.option.evil){warning("W054")}break;case"Date":case"RegExp":case"this":break;default:if(c.id!=="function"){i=c.value.substr(0,1);if(state.option.newcap&&(i<"A"||i>"Z")&&!state.funct["(scope)"].isPredefined(c.value)){warning("W055",state.tokens.curr)}}}}else{if(c.id!=="."&&c.id!=="["&&c.id!=="("){warning("W056",state.tokens.curr)}}}else{if(!state.option.supernew)warning("W057",this)}if(state.tokens.next.id!=="("&&!state.option.supernew){warning("W058",state.tokens.curr,state.tokens.curr.value)}this.first=this.right=c;return this});state.syntax["new"].exps=true;prefix("void").exps=true;infix(".",function(left,that){var m=identifier(false,true);if(typeof m==="string"){countMember(m)}that.left=left;that.right=m;if(m&&m==="hasOwnProperty"&&state.tokens.next.value==="="){warning("W001")}if(left&&left.value==="arguments"&&(m==="callee"||m==="caller")){if(state.option.noarg)warning("W059",left,m);else if(state.isStrict())error("E008")}else if(!state.option.evil&&left&&left.value==="document"&&(m==="write"||m==="writeln")){warning("W060",left)}if(!state.option.evil&&(m==="eval"||m==="execScript")){if(isGlobalEval(left,state)){warning("W061")}}return that},160,true);infix("(",function(left,that){if(state.option.immed&&left&&!left.immed&&left.id==="function"){warning("W062")}var n=0;var p=[];if(left){if(left.type==="(identifier)"){if(left.value.match(/^[A-Z]([A-Z0-9_$]*[a-z][A-Za-z0-9_$]*)?$/)){if("Array Number String Boolean Date Object Error Symbol".indexOf(left.value)===-1){if(left.value==="Math"){warning("W063",left)}else if(state.option.newcap){warning("W064",left)}}}}}if(state.tokens.next.id!==")"){for(;;){p[p.length]=expression(10);n+=1;if(state.tokens.next.id!==","){break}parseComma()}}advance(")");if(typeof left==="object"){if(!state.inES5()&&left.value==="parseInt"&&n===1){warning("W065",state.tokens.curr)}if(!state.option.evil){if(left.value==="eval"||left.value==="Function"||left.value==="execScript"){warning("W061",left)}else if(p[0]&&p[0].id==="(string)"&&(left.value==="setTimeout"||left.value==="setInterval")){warning("W066",left);addEvalCode(left,p[0])}else if(p[0]&&p[0].id==="(string)"&&left.value==="."&&left.left.value==="window"&&(left.right==="setTimeout"||left.right==="setInterval")){warning("W066",left);addEvalCode(left,p[0])}}if(!left.identifier&&left.id!=="."&&left.id!=="["&&left.id!=="=>"&&left.id!=="("&&left.id!=="&&"&&left.id!=="||"&&left.id!=="?"&&!(state.inES6()&&left["(name)"])){warning("W067",that)}}that.left=left;return that},155,true).exps=true;prefix("(",function(rbp){var pn=state.tokens.next,pn1,i=-1;var ret,triggerFnExpr,first,last;var parens=1;var opening=state.tokens.curr;var preceeding=state.tokens.prev;var isNecessary=!state.option.singleGroups;do{if(pn.value==="("){parens+=1}else if(pn.value===")"){parens-=1}i+=1;pn1=pn;pn=peek(i)}while(!(parens===0&&pn1.value===")")&&pn.type!=="(end)");if(state.tokens.next.id==="function"){triggerFnExpr=state.tokens.next.immed=true}if(pn.value==="=>"){return doFunction({type:"arrow",parsedOpening:true})}var exprs=[];if(state.tokens.next.id!==")"){for(;;){exprs.push(expression(10));if(state.tokens.next.id!==","){break}if(state.option.nocomma){warning("W127")}parseComma()}}advance(")",this);if(state.option.immed&&exprs[0]&&exprs[0].id==="function"){if(state.tokens.next.id!=="("&&state.tokens.next.id!=="."&&state.tokens.next.id!=="["){warning("W068",this)}}if(!exprs.length){return}if(exprs.length>1){ret=Object.create(state.syntax[","]);ret.exprs=exprs;first=exprs[0];last=exprs[exprs.length-1]}else{ret=first=last=exprs[0];if(!isNecessary){isNecessary=opening.beginsStmt&&(ret.id==="{"||triggerFnExpr||isFunctor(ret))||triggerFnExpr&&(!isEndOfExpr()||state.tokens.prev.id!=="}")||isFunctor(ret)&&!isEndOfExpr()||ret.id==="{"&&preceeding.id==="=>"||ret.type==="(number)"&&checkPunctuator(pn,".")&&/^\d+$/.test(ret.value)||opening.beginsStmt&&ret.id==="="&&ret.left.id==="{"}}if(ret){if(!isNecessary&&(isOperator(first)||ret.exprs)){isNecessary=rbp>first.lbp||rbp>0&&rbp===first.lbp||!isEndOfExpr()&&last.lbp<state.tokens.next.lbp}if(!isNecessary){warning("W126",opening)}ret.paren=true}return ret});application("=>");infix("[",function(left,that){var e=expression(10),s;if(e&&e.type==="(string)"){if(!state.option.evil&&(e.value==="eval"||e.value==="execScript")){if(isGlobalEval(left,state)){warning("W061")}}countMember(e.value);if(!state.option.sub&&reg.identifier.test(e.value)){s=state.syntax[e.value];if(!s||!isReserved(s)){warning("W069",state.tokens.prev,e.value)}}}advance("]",that);if(e&&e.value==="hasOwnProperty"&&state.tokens.next.value==="="){warning("W001")}that.left=left;that.right=e;return that},160,true);function comprehensiveArrayExpression(){var res={};res.exps=true;state.funct["(comparray)"].stack();var reversed=false;if(state.tokens.next.value!=="for"){reversed=true;if(!state.inMoz()){warning("W116",state.tokens.next,"for",state.tokens.next.value)}state.funct["(comparray)"].setState("use");res.right=expression(10)}advance("for");if(state.tokens.next.value==="each"){advance("each");if(!state.inMoz()){warning("W118",state.tokens.curr,"for each")}}advance("(");state.funct["(comparray)"].setState("define");res.left=expression(130);if(_.contains(["in","of"],state.tokens.next.value)){advance()}else{error("E045",state.tokens.curr)}state.funct["(comparray)"].setState("generate");expression(10);advance(")");if(state.tokens.next.value==="if"){advance("if");advance("(");state.funct["(comparray)"].setState("filter");res.filter=expression(10);advance(")")}if(!reversed){state.funct["(comparray)"].setState("use");res.right=expression(10)}advance("]");state.funct["(comparray)"].unstack();return res}prefix("[",function(){var blocktype=lookupBlockType();if(blocktype.isCompArray){if(!state.option.esnext&&!state.inMoz()){warning("W118",state.tokens.curr,"array comprehension")}return comprehensiveArrayExpression()}else if(blocktype.isDestAssign){this.destructAssign=destructuringPattern({openingParsed:true,assignment:true});return this}var b=state.tokens.curr.line!==startLine(state.tokens.next);this.first=[];if(b){indent+=state.option.indent;if(state.tokens.next.from===indent+state.option.indent){indent+=state.option.indent}}while(state.tokens.next.id!=="(end)"){while(state.tokens.next.id===","){if(!state.option.elision){if(!state.inES5()){warning("W070")}else{warning("W128");do{advance(",")}while(state.tokens.next.id===",");continue}}advance(",")}if(state.tokens.next.id==="]"){break}this.first.push(expression(10));if(state.tokens.next.id===","){parseComma({allowTrailing:true});if(state.tokens.next.id==="]"&&!state.inES5()){warning("W070",state.tokens.curr);break}}else{if(state.option.trailingcomma&&state.inES5()){warningAt("W140",state.tokens.curr.line,state.tokens.curr.character)}break}}if(b){indent-=state.option.indent}advance("]",this);return this});function isMethod(){return state.funct["(statement)"]&&state.funct["(statement)"].type==="class"||state.funct["(context)"]&&state.funct["(context)"]["(verb)"]==="class"}function isPropertyName(token){return token.identifier||token.id==="(string)"||token.id==="(number)"}function propertyName(preserveOrToken){var id;var preserve=true;if(typeof preserveOrToken==="object"){id=preserveOrToken}else{preserve=preserveOrToken;id=optionalidentifier(false,true,preserve)}if(!id){if(state.tokens.next.id==="(string)"){id=state.tokens.next.value;if(!preserve){advance()}}else if(state.tokens.next.id==="(number)"){id=state.tokens.next.value.toString();if(!preserve){advance()}}}else if(typeof id==="object"){if(id.id==="(string)"||id.id==="(identifier)")id=id.value;else if(id.id==="(number)")id=id.value.toString()}if(id==="hasOwnProperty"){warning("W001")}return id}function functionparams(options){var next;var paramsIds=[];var ident;var tokens=[];var t;var pastDefault=false;var pastRest=false;var arity=0;var loneArg=options&&options.loneArg;if(loneArg&&loneArg.identifier===true){state.funct["(scope)"].addParam(loneArg.value,loneArg);return{arity:1,params:[loneArg.value]}}next=state.tokens.next;if(!options||!options.parsedOpening){advance("(")}if(state.tokens.next.id===")"){advance(")");return}function addParam(addParamArgs){state.funct["(scope)"].addParam.apply(state.funct["(scope)"],addParamArgs)}for(;;){arity++;var currentParams=[];if(_.contains(["{","["],state.tokens.next.id)){tokens=destructuringPattern();for(t in tokens){t=tokens[t];if(t.id){paramsIds.push(t.id);currentParams.push([t.id,t.token])}}}else{if(checkPunctuator(state.tokens.next,"..."))pastRest=true;ident=identifier(true);if(ident){paramsIds.push(ident);currentParams.push([ident,state.tokens.curr])}else{while(!checkPunctuators(state.tokens.next,[",",")"]))advance()}}if(pastDefault){if(state.tokens.next.id!=="="){error("W138",state.tokens.current)}}if(state.tokens.next.id==="="){if(!state.inES6()){warning("W119",state.tokens.next,"default parameters","6")}advance("=");pastDefault=true;expression(10)}currentParams.forEach(addParam);if(state.tokens.next.id===","){if(pastRest){warning("W131",state.tokens.next)}parseComma()}else{advance(")",next);return{arity:arity,params:paramsIds}}}}function functor(name,token,overwrites){var funct={"(name)":name,"(breakage)":0,"(loopage)":0,"(tokens)":{},"(properties)":{},"(catch)":false,"(global)":false,"(line)":null,"(character)":null,"(metrics)":null,"(statement)":null,"(context)":null,"(scope)":null,"(comparray)":null,"(generator)":null,"(arrow)":null,"(params)":null};if(token){_.extend(funct,{"(line)":token.line,"(character)":token.character,"(metrics)":createMetrics(token)})}_.extend(funct,overwrites);if(funct["(context)"]){funct["(scope)"]=funct["(context)"]["(scope)"];funct["(comparray)"]=funct["(context)"]["(comparray)"]}return funct}function isFunctor(token){return"(scope)"in token}function hasParsedCode(funct){return funct["(global)"]&&!funct["(verb)"]}function doTemplateLiteral(leftOrRbp){var ctx=this.context;var noSubst=this.noSubst;var depth=this.depth;var left=typeof leftOrRbp==="number"?null:leftOrRbp;if(!noSubst){while(!end()){if(!state.tokens.next.template||state.tokens.next.depth>depth){expression(0)}else{advance()}}}return{id:"(template)",type:"(template)",tag:left};function end(){if(state.tokens.curr.template&&state.tokens.curr.tail&&state.tokens.curr.context===ctx)return true;var complete=state.tokens.next.template&&state.tokens.next.tail&&state.tokens.next.context===ctx;if(complete)advance();return complete||state.tokens.next.isUnclosed}}function doFunction(options){var f,token,name,statement,classExprBinding,isGenerator,isArrow,ignoreLoopFunc;var oldOption=state.option;var oldIgnored=state.ignored;if(options){name=options.name;statement=options.statement;classExprBinding=options.classExprBinding;isGenerator=options.type==="generator";isArrow=options.type==="arrow";ignoreLoopFunc=options.ignoreLoopFunc}state.option=Object.create(state.option);state.ignored=Object.create(state.ignored);state.funct=functor(name||state.nameStack.infer(),state.tokens.next,{"(statement)":statement,"(context)":state.funct,"(arrow)":isArrow,"(generator)":isGenerator});f=state.funct;token=state.tokens.curr;token.funct=state.funct;functions.push(state.funct);state.funct["(scope)"].stack("functionouter");var internallyAccessibleName=name||classExprBinding;if(internallyAccessibleName){state.funct["(scope)"].block.add(internallyAccessibleName,classExprBinding?"class":"function",state.tokens.curr,false)}state.funct["(scope)"].stack("functionparams");var paramsInfo=functionparams(options);if(paramsInfo){state.funct["(params)"]=paramsInfo.params;state.funct["(metrics)"].arity=paramsInfo.arity;state.funct["(metrics)"].verifyMaxParametersPerFunction()}else{state.funct["(metrics)"].arity=0}if(isArrow){if(!state.inES6(true)){warning("W119",state.tokens.curr,"arrow function syntax (=>)","6")}if(!options.loneArg){advance("=>")}}block(false,true,true,isArrow);if(!state.option.noyield&&isGenerator&&state.funct["(generator)"]!=="yielded"){warning("W124",state.tokens.curr)}state.funct["(metrics)"].verifyMaxStatementsPerFunction();state.funct["(metrics)"].verifyMaxComplexityPerFunction();state.funct["(unusedOption)"]=state.option.unused;state.option=oldOption;state.ignored=oldIgnored;state.funct["(last)"]=state.tokens.curr.line;state.funct["(lastcharacter)"]=state.tokens.curr.character;state.funct["(scope)"].unstack();state.funct["(scope)"].unstack();state.funct=state.funct["(context)"];if(!ignoreLoopFunc&&!state.option.loopfunc&&state.funct["(loopage)"]){if(f["(isCapturing)"]){warning("W083",token)}}return f}function createMetrics(functionStartToken){return{statementCount:0,nestedBlockDepth:-1,ComplexityCount:1,arity:0,verifyMaxStatementsPerFunction:function(){if(state.option.maxstatements&&this.statementCount>state.option.maxstatements){warning("W071",functionStartToken,this.statementCount)}},verifyMaxParametersPerFunction:function(){if(_.isNumber(state.option.maxparams)&&this.arity>state.option.maxparams){warning("W072",functionStartToken,this.arity)}},verifyMaxNestedBlockDepthPerFunction:function(){if(state.option.maxdepth&&this.nestedBlockDepth>0&&this.nestedBlockDepth===state.option.maxdepth+1){warning("W073",null,this.nestedBlockDepth)}},verifyMaxComplexityPerFunction:function(){var max=state.option.maxcomplexity;var cc=this.ComplexityCount;if(max&&cc>max){warning("W074",functionStartToken,cc)}}}}function increaseComplexityCount(){state.funct["(metrics)"].ComplexityCount+=1}function checkCondAssignment(expr){var id,paren;if(expr){id=expr.id;paren=expr.paren;if(id===","&&(expr=expr.exprs[expr.exprs.length-1])){id=expr.id;paren=paren||expr.paren}}switch(id){case"=":case"+=":case"-=":case"*=":case"%=":case"&=":case"|=":case"^=":case"/=":if(!paren&&!state.option.boss){warning("W084")}}}function checkProperties(props){if(state.inES5()){for(var name in props){if(props[name]&&props[name].setterToken&&!props[name].getterToken){warning("W078",props[name].setterToken)}}}}function metaProperty(name,c){if(checkPunctuator(state.tokens.next,".")){var left=state.tokens.curr.id;advance(".");var id=identifier();state.tokens.curr.isMetaProperty=true;if(name!==id){error("E057",state.tokens.prev,left,id)}else{c()}return state.tokens.curr}}(function(x){x.nud=function(){var b,f,i,p,t,isGeneratorMethod=false,nextVal;var props=Object.create(null);b=state.tokens.curr.line!==startLine(state.tokens.next);if(b){indent+=state.option.indent;if(state.tokens.next.from===indent+state.option.indent){indent+=state.option.indent}}var blocktype=lookupBlockType();if(blocktype.isDestAssign){this.destructAssign=destructuringPattern({openingParsed:true,assignment:true});return this}for(;;){if(state.tokens.next.id==="}"){break}nextVal=state.tokens.next.value;if(state.tokens.next.identifier&&(peekIgnoreEOL().id===","||peekIgnoreEOL().id==="}")){if(!state.inES6()){warning("W104",state.tokens.next,"object short notation","6")}i=propertyName(true);saveProperty(props,i,state.tokens.next);expression(10)}else if(peek().id!==":"&&(nextVal==="get"||nextVal==="set")){advance(nextVal);if(!state.inES5()){error("E034")}i=propertyName();if(!i&&!state.inES6()){error("E035")}if(i){saveAccessor(nextVal,props,i,state.tokens.curr)}t=state.tokens.next;f=doFunction();p=f["(params)"];if(nextVal==="get"&&i&&p){warning("W076",t,p[0],i)}else if(nextVal==="set"&&i&&f["(metrics)"].arity!==1){warning("W077",t,i)}}else{if(state.tokens.next.value==="*"&&state.tokens.next.type==="(punctuator)"){if(!state.inES6()){warning("W104",state.tokens.next,"generator functions","6")}advance("*");isGeneratorMethod=true}else{isGeneratorMethod=false}if(state.tokens.next.id==="["){i=computedPropertyName();state.nameStack.set(i)}else{state.nameStack.set(state.tokens.next);i=propertyName();saveProperty(props,i,state.tokens.next);if(typeof i!=="string"){break}}if(state.tokens.next.value==="("){if(!state.inES6()){warning("W104",state.tokens.curr,"concise methods","6")}doFunction({type:isGeneratorMethod?"generator":null})}else{advance(":");expression(10)}}countMember(i);if(state.tokens.next.id===","){parseComma({allowTrailing:true,property:true});if(state.tokens.next.id===","){warning("W070",state.tokens.curr)}else if(state.tokens.next.id==="}"&&!state.inES5()){warning("W070",state.tokens.curr)}}else{if(state.option.trailingcomma&&state.inES5()){warningAt("W140",state.tokens.curr.line,state.tokens.curr.character)}break}}if(b){indent-=state.option.indent}advance("}",this);checkProperties(props);return this};x.fud=function(){error("E036",state.tokens.curr)}})(delim("{"));function destructuringPattern(options){var isAssignment=options&&options.assignment;if(!state.inES6()){warning("W104",state.tokens.curr,isAssignment?"destructuring assignment":"destructuring binding","6")}return destructuringPatternRecursive(options)}function destructuringPatternRecursive(options){var ids;var identifiers=[];var openingParsed=options&&options.openingParsed;var isAssignment=options&&options.assignment;var recursiveOptions=isAssignment?{assignment:isAssignment}:null;var firstToken=openingParsed?state.tokens.curr:state.tokens.next;var nextInnerDE=function(){var ident;if(checkPunctuators(state.tokens.next,["[","{"])){ids=destructuringPatternRecursive(recursiveOptions);for(var id in ids){id=ids[id];identifiers.push({id:id.id,token:id.token})}}else if(checkPunctuator(state.tokens.next,",")){identifiers.push({id:null,token:state.tokens.curr})}else if(checkPunctuator(state.tokens.next,"(")){advance("(");nextInnerDE();advance(")")}else{var is_rest=checkPunctuator(state.tokens.next,"...");if(isAssignment){var assignTarget=expression(20);if(assignTarget){checkLeftSideAssign(assignTarget);if(assignTarget.identifier){ident=assignTarget.value}}}else{ident=identifier()}if(ident){identifiers.push({id:ident,token:state.tokens.curr})}return is_rest}return false};var assignmentProperty=function(){var id;if(checkPunctuator(state.tokens.next,"[")){advance("[");expression(10);advance("]");advance(":");nextInnerDE()}else if(state.tokens.next.id==="(string)"||state.tokens.next.id==="(number)"){advance();advance(":");nextInnerDE()}else{id=identifier();if(checkPunctuator(state.tokens.next,":")){advance(":");nextInnerDE()}else if(id){if(isAssignment){checkLeftSideAssign(state.tokens.curr)}identifiers.push({id:id,token:state.tokens.curr})}}};var id,value;if(checkPunctuator(firstToken,"[")){if(!openingParsed){advance("[")}if(checkPunctuator(state.tokens.next,"]")){warning("W137",state.tokens.curr)}var element_after_rest=false;while(!checkPunctuator(state.tokens.next,"]")){if(nextInnerDE()&&!element_after_rest&&checkPunctuator(state.tokens.next,",")){warning("W130",state.tokens.next);element_after_rest=true}if(checkPunctuator(state.tokens.next,"=")){if(checkPunctuator(state.tokens.prev,"...")){advance("]")}else{advance("=")}id=state.tokens.prev;value=expression(10);if(value&&value.type==="undefined"){warning("W080",id,id.value)}}if(!checkPunctuator(state.tokens.next,"]")){advance(",")}}advance("]")}else if(checkPunctuator(firstToken,"{")){if(!openingParsed){advance("{")}if(checkPunctuator(state.tokens.next,"}")){warning("W137",state.tokens.curr)}while(!checkPunctuator(state.tokens.next,"}")){assignmentProperty();if(checkPunctuator(state.tokens.next,"=")){advance("=");id=state.tokens.prev;value=expression(10);if(value&&value.type==="undefined"){warning("W080",id,id.value)}}if(!checkPunctuator(state.tokens.next,"}")){advance(",");if(checkPunctuator(state.tokens.next,"}")){break}}}advance("}")}return identifiers}function destructuringPatternMatch(tokens,value){var first=value.first;if(!first)return;_.zip(tokens,Array.isArray(first)?first:[first]).forEach(function(val){var token=val[0];var value=val[1];if(token&&value)token.first=value;else if(token&&token.first&&!value)warning("W080",token.first,token.first.value)})}function blockVariableStatement(type,statement,context){var prefix=context&&context.prefix;var inexport=context&&context.inexport;var isLet=type==="let";var isConst=type==="const";var tokens,lone,value,letblock;if(!state.inES6()){warning("W104",state.tokens.curr,type,"6")}if(isLet&&state.tokens.next.value==="("){if(!state.inMoz()){warning("W118",state.tokens.next,"let block")}advance("(");state.funct["(scope)"].stack();letblock=true}else if(state.funct["(noblockscopedvar)"]){error("E048",state.tokens.curr,isConst?"Const":"Let")}statement.first=[];for(;;){var names=[];if(_.contains(["{","["],state.tokens.next.value)){tokens=destructuringPattern();lone=false}else{tokens=[{id:identifier(),token:state.tokens.curr}];lone=true}if(!prefix&&isConst&&state.tokens.next.id!=="="){warning("E012",state.tokens.curr,state.tokens.curr.value)}for(var t in tokens){if(tokens.hasOwnProperty(t)){t=tokens[t];if(state.funct["(scope)"].block.isGlobal()){if(predefined[t.id]===false){warning("W079",t.token,t.id)}}if(t.id&&!state.funct["(noblockscopedvar)"]){state.funct["(scope)"].addlabel(t.id,{type:type,token:t.token});names.push(t.token)}}}if(state.tokens.next.id==="="){advance("=");if(!prefix&&peek(0).id==="="&&state.tokens.next.identifier){warning("W120",state.tokens.next,state.tokens.next.value)}var id=state.tokens.prev;value=expression(prefix?120:10);if(!prefix&&value&&value.type==="undefined"){warning("W080",id,id.value)}if(lone){tokens[0].first=value}else{destructuringPatternMatch(names,value)}}if(!prefix){for(t in tokens){if(tokens.hasOwnProperty(t)){t=tokens[t];state.funct["(scope)"].initialize(t.id);if(lone&&inexport){state.funct["(scope)"].setExported(t.token.value,t.token)}}}}statement.first=statement.first.concat(names);if(state.tokens.next.id!==","){break}parseComma()}if(letblock){advance(")");block(true,true);statement.block=true;state.funct["(scope)"].unstack()}return statement}var conststatement=stmt("const",function(context){return blockVariableStatement("const",this,context)});conststatement.exps=true;var letstatement=stmt("let",function(context){return blockVariableStatement("let",this,context)});letstatement.exps=true;var varstatement=stmt("var",function(context){var prefix=context&&context.prefix;var inexport=context&&context.inexport;var tokens,lone,value;this.first=[];for(;;){var names=[];if(_.contains(["{","["],state.tokens.next.value)){tokens=destructuringPattern();lone=false}else{tokens=[{id:identifier(),token:state.tokens.curr}];lone=true}if(state.option.varstmt){warning("W132",this)}this.first=this.first.concat(names);for(var t in tokens){if(tokens.hasOwnProperty(t)){t=tokens[t];if(state.funct["(global)"]&&!state.impliedClosure()){if(predefined[t.id]===false){warning("W079",t.token,t.id)}else if(state.option.futurehostile===false){if(!state.inES5()&&vars.ecmaIdentifiers[5][t.id]===false||!state.inES6()&&vars.ecmaIdentifiers[6][t.id]===false){warning("W129",t.token,t.id)}}}if(t.id){state.funct["(scope)"].addlabel(t.id,{type:"var",token:t.token});if(lone&&inexport){state.funct["(scope)"].setExported(t.id,t.token)}names.push(t.token)}}}if(state.tokens.next.id==="="){state.nameStack.set(state.tokens.curr);advance("=");if(peek(0).id==="="&&state.tokens.next.identifier){if(!prefix&&!state.funct["(params)"]||state.funct["(params)"].indexOf(state.tokens.next.value)===-1){warning("W120",state.tokens.next,state.tokens.next.value)}}var id=state.tokens.prev;value=expression(prefix?120:10);if(value&&!prefix&&!state.funct["(loopage)"]&&value.type==="undefined"){warning("W080",id,id.value)}if(lone){tokens[0].first=value}else{destructuringPatternMatch(names,value)}}if(state.tokens.next.id!==","){break}parseComma()}return this});varstatement.exps=true;blockstmt("class",function(rbp){return classdef.call(this,rbp,true)});function classdef(rbp,isStatement){if(!state.inES6()){warning("W104",state.tokens.curr,"class","6")}if(isStatement){this.name=identifier();state.funct["(scope)"].addlabel(this.name,{type:"class",token:state.tokens.curr})}else if(state.tokens.next.identifier&&state.tokens.next.value!=="extends"){this.name=identifier();this.namedExpr=true}else{this.name=state.nameStack.infer()}classtail(this);if(isStatement){state.funct["(scope)"].initialize(this.name)}return this}function classtail(c){var wasInClassBody=state.inClassBody;if(state.tokens.next.value==="extends"){advance("extends");c.heritage=expression(10)}state.inClassBody=true;advance("{");c.body=classbody(c);advance("}");state.inClassBody=wasInClassBody}function classbody(c){var name;var isStatic;var isGenerator;var getset;var props=Object.create(null);var staticProps=Object.create(null);var computed;for(var i=0;state.tokens.next.id!=="}";++i){name=state.tokens.next;isStatic=false;isGenerator=false;getset=null;if(name.id===";"){warning("W032");advance(";");continue}if(name.id==="*"){isGenerator=true;advance("*");name=state.tokens.next}if(name.id==="["){name=computedPropertyName();computed=true}else if(isPropertyName(name)){advance();computed=false;if(name.identifier&&name.value==="static"){if(checkPunctuator(state.tokens.next,"*")){isGenerator=true;advance("*")}if(isPropertyName(state.tokens.next)||state.tokens.next.id==="["){computed=state.tokens.next.id==="[";isStatic=true;name=state.tokens.next;if(state.tokens.next.id==="["){name=computedPropertyName()}else advance()}}if(name.identifier&&(name.value==="get"||name.value==="set")){if(isPropertyName(state.tokens.next)||state.tokens.next.id==="["){computed=state.tokens.next.id==="[";getset=name;name=state.tokens.next;if(state.tokens.next.id==="["){name=computedPropertyName()}else advance()}}}else{warning("W052",state.tokens.next,state.tokens.next.value||state.tokens.next.type);advance();continue}if(!checkPunctuator(state.tokens.next,"(")){error("E054",state.tokens.next,state.tokens.next.value);while(state.tokens.next.id!=="}"&&!checkPunctuator(state.tokens.next,"(")){advance()}if(state.tokens.next.value!=="("){doFunction({statement:c})}}if(!computed){if(getset){saveAccessor(getset.value,isStatic?staticProps:props,name.value,name,true,isStatic)}else{if(name.value==="constructor"){state.nameStack.set(c)}else{state.nameStack.set(name)}saveProperty(isStatic?staticProps:props,name.value,name,true,isStatic)}}if(getset&&name.value==="constructor"){var propDesc=getset.value==="get"?"class getter method":"class setter method";error("E049",name,propDesc,"constructor")}else if(name.value==="prototype"){error("E049",name,"class method","prototype")}propertyName(name);doFunction({statement:c,type:isGenerator?"generator":null,classExprBinding:c.namedExpr?c.name:null})}checkProperties(props)}blockstmt("function",function(context){var inexport=context&&context.inexport;var generator=false;if(state.tokens.next.value==="*"){advance("*");if(state.inES6(true)){generator=true}else{warning("W119",state.tokens.curr,"function*","6")}}if(inblock){warning("W082",state.tokens.curr)}var i=optionalidentifier();state.funct["(scope)"].addlabel(i,{type:"function",token:state.tokens.curr});if(i===undefined){warning("W025")}else if(inexport){state.funct["(scope)"].setExported(i,state.tokens.prev)}doFunction({name:i,statement:this,type:generator?"generator":null,ignoreLoopFunc:inblock});if(state.tokens.next.id==="("&&state.tokens.next.line===state.tokens.curr.line){error("E039")}return this});prefix("function",function(){var generator=false;if(state.tokens.next.value==="*"){if(!state.inES6()){warning("W119",state.tokens.curr,"function*","6")}advance("*");generator=true}var i=optionalidentifier();doFunction({name:i,type:generator?"generator":null});return this});blockstmt("if",function(){var t=state.tokens.next;increaseComplexityCount();state.condition=true;advance("(");var expr=expression(0);if(!expr){quit("E041",this)}checkCondAssignment(expr);var forinifcheck=null;if(state.option.forin&&state.forinifcheckneeded){state.forinifcheckneeded=false;forinifcheck=state.forinifchecks[state.forinifchecks.length-1];if(expr.type==="(punctuator)"&&expr.value==="!"){forinifcheck.type="(negative)"}else{forinifcheck.type="(positive)"}}advance(")",t);state.condition=false;var s=block(true,true);if(forinifcheck&&forinifcheck.type==="(negative)"){if(s&&s[0]&&s[0].type==="(identifier)"&&s[0].value==="continue"){forinifcheck.type="(negative-with-continue)"}}if(state.tokens.next.id==="else"){advance("else");if(state.tokens.next.id==="if"||state.tokens.next.id==="switch"){statement()}else{block(true,true)}}return this});blockstmt("try",function(){var b;function doCatch(){advance("catch");advance("(");state.funct["(scope)"].stack("catchparams");if(checkPunctuators(state.tokens.next,["[","{"])){var tokens=destructuringPattern();_.each(tokens,function(token){if(token.id){state.funct["(scope)"].addParam(token.id,token,"exception")}})}else if(state.tokens.next.type!=="(identifier)"){warning("E030",state.tokens.next,state.tokens.next.value)}else{state.funct["(scope)"].addParam(identifier(),state.tokens.curr,"exception")}if(state.tokens.next.value==="if"){if(!state.inMoz()){warning("W118",state.tokens.curr,"catch filter");
}advance("if");expression(0)}advance(")");block(false);state.funct["(scope)"].unstack()}block(true);while(state.tokens.next.id==="catch"){increaseComplexityCount();if(b&&!state.inMoz()){warning("W118",state.tokens.next,"multiple catch blocks")}doCatch();b=true}if(state.tokens.next.id==="finally"){advance("finally");block(true);return}if(!b){error("E021",state.tokens.next,"catch",state.tokens.next.value)}return this});blockstmt("while",function(){var t=state.tokens.next;state.funct["(breakage)"]+=1;state.funct["(loopage)"]+=1;increaseComplexityCount();advance("(");checkCondAssignment(expression(0));advance(")",t);block(true,true);state.funct["(breakage)"]-=1;state.funct["(loopage)"]-=1;return this}).labelled=true;blockstmt("with",function(){var t=state.tokens.next;if(state.isStrict()){error("E010",state.tokens.curr)}else if(!state.option.withstmt){warning("W085",state.tokens.curr)}advance("(");expression(0);advance(")",t);block(true,true);return this});blockstmt("switch",function(){var t=state.tokens.next;var g=false;var noindent=false;state.funct["(breakage)"]+=1;advance("(");checkCondAssignment(expression(0));advance(")",t);t=state.tokens.next;advance("{");if(state.tokens.next.from===indent)noindent=true;if(!noindent)indent+=state.option.indent;this.cases=[];for(;;){switch(state.tokens.next.id){case"case":switch(state.funct["(verb)"]){case"yield":case"break":case"case":case"continue":case"return":case"switch":case"throw":break;default:if(!state.tokens.curr.caseFallsThrough){warning("W086",state.tokens.curr,"case")}}advance("case");this.cases.push(expression(0));increaseComplexityCount();g=true;advance(":");state.funct["(verb)"]="case";break;case"default":switch(state.funct["(verb)"]){case"yield":case"break":case"continue":case"return":case"throw":break;default:if(this.cases.length){if(!state.tokens.curr.caseFallsThrough){warning("W086",state.tokens.curr,"default")}}}advance("default");g=true;advance(":");break;case"}":if(!noindent)indent-=state.option.indent;advance("}",t);state.funct["(breakage)"]-=1;state.funct["(verb)"]=undefined;return;case"(end)":error("E023",state.tokens.next,"}");return;default:indent+=state.option.indent;if(g){switch(state.tokens.curr.id){case",":error("E040");return;case":":g=false;statements();break;default:error("E025",state.tokens.curr);return}}else{if(state.tokens.curr.id===":"){advance(":");error("E024",state.tokens.curr,":");statements()}else{error("E021",state.tokens.next,"case",state.tokens.next.value);return}}indent-=state.option.indent}}return this}).labelled=true;stmt("debugger",function(){if(!state.option.debug){warning("W087",this)}return this}).exps=true;(function(){var x=stmt("do",function(){state.funct["(breakage)"]+=1;state.funct["(loopage)"]+=1;increaseComplexityCount();this.first=block(true,true);advance("while");var t=state.tokens.next;advance("(");checkCondAssignment(expression(0));advance(")",t);state.funct["(breakage)"]-=1;state.funct["(loopage)"]-=1;return this});x.labelled=true;x.exps=true})();blockstmt("for",function(){var s,t=state.tokens.next;var letscope=false;var foreachtok=null;if(t.value==="each"){foreachtok=t;advance("each");if(!state.inMoz()){warning("W118",state.tokens.curr,"for each")}}increaseComplexityCount();advance("(");var nextop;var i=0;var inof=["in","of"];var level=0;var comma;var initializer;var bindingPower;var targets;var target;if(checkPunctuators(state.tokens.next,["{","["]))++level;do{nextop=peek(i);++i;if(checkPunctuators(nextop,["{","["]))++level;else if(checkPunctuators(nextop,["}","]"]))--level;if(level<0)break;if(level===0){if(!comma&&checkPunctuator(nextop,","))comma=nextop;else if(!initializer&&checkPunctuator(nextop,"="))initializer=nextop}}while(level>0||!_.contains(inof,nextop.value)&&nextop.value!==";"&&nextop.type!=="(end)");if(_.contains(inof,nextop.value)){if(nextop.value==="of"){bindingPower=20;if(!state.inES6()){warning("W104",nextop,"for of","6")}}else{bindingPower=0}if(initializer){error("W133",comma,nextop.value,"initializer is forbidden")}if(comma){error("W133",comma,nextop.value,"more than one ForBinding")}if(state.tokens.next.id==="var"){advance("var");state.tokens.curr.fud({prefix:true})}else if(state.tokens.next.id==="let"||state.tokens.next.id==="const"){advance(state.tokens.next.id);letscope=true;state.funct["(scope)"].stack();state.tokens.curr.fud({prefix:true})}else{targets=[];do{if(checkPunctuators(state.tokens.next,["{","["])){destructuringPattern({assignment:true}).forEach(function(elem){this.push(elem.token)},targets)}else{target=expression(120);if(target.type==="(identifier)"){targets.push(target)}checkLeftSideAssign(target,nextop)}if(checkPunctuator(state.tokens.next,"=")){advance("=");expression(120)}if(checkPunctuator(state.tokens.next,",")){advance(",")}}while(state.tokens.next!==nextop);if(!initializer&&!comma){targets.forEach(function(token){if(!state.funct["(scope)"].has(token.value)){warning("W088",token,token.value)}})}}advance(nextop.value);expression(bindingPower);advance(")",t);if(nextop.value==="in"&&state.option.forin){state.forinifcheckneeded=true;if(state.forinifchecks===undefined){state.forinifchecks=[]}state.forinifchecks.push({type:"(none)"})}state.funct["(breakage)"]+=1;state.funct["(loopage)"]+=1;s=block(true,true);if(nextop.value==="in"&&state.option.forin){if(state.forinifchecks&&state.forinifchecks.length>0){var check=state.forinifchecks.pop();if(s&&s.length>0&&(typeof s[0]!=="object"||s[0].value!=="if")||check.type==="(positive)"&&s.length>1||check.type==="(negative)"){warning("W089",this)}}state.forinifcheckneeded=false}state.funct["(breakage)"]-=1;state.funct["(loopage)"]-=1}else{if(foreachtok){error("E045",foreachtok)}if(state.tokens.next.id!==";"){if(state.tokens.next.id==="var"){advance("var");state.tokens.curr.fud()}else if(state.tokens.next.id==="let"){advance("let");letscope=true;state.funct["(scope)"].stack();state.tokens.curr.fud()}else{for(;;){expression(0,"for");if(state.tokens.next.id!==","){break}parseComma()}}}nolinebreak(state.tokens.curr);advance(";");state.funct["(loopage)"]+=1;if(state.tokens.next.id!==";"){checkCondAssignment(expression(0))}nolinebreak(state.tokens.curr);advance(";");if(state.tokens.next.id===";"){error("E021",state.tokens.next,")",";")}if(state.tokens.next.id!==")"){for(;;){expression(0,"for");if(state.tokens.next.id!==","){break}parseComma()}}advance(")",t);state.funct["(breakage)"]+=1;block(true,true);state.funct["(breakage)"]-=1;state.funct["(loopage)"]-=1}if(letscope){state.funct["(scope)"].unstack()}return this}).labelled=true;stmt("break",function(){var v=state.tokens.next.value;if(!state.option.asi)nolinebreak(this);if(state.tokens.next.id!==";"&&!state.tokens.next.reach&&state.tokens.curr.line===startLine(state.tokens.next)){if(!state.funct["(scope)"].funct.hasBreakLabel(v)){warning("W090",state.tokens.next,v)}this.first=state.tokens.next;advance()}else{if(state.funct["(breakage)"]===0)warning("W052",state.tokens.next,this.value)}reachable(this);return this}).exps=true;stmt("continue",function(){var v=state.tokens.next.value;if(state.funct["(breakage)"]===0||!state.funct["(loopage)"]){warning("W052",state.tokens.next,this.value)}if(!state.option.asi)nolinebreak(this);if(state.tokens.next.id!==";"&&!state.tokens.next.reach){if(state.tokens.curr.line===startLine(state.tokens.next)){if(!state.funct["(scope)"].funct.hasBreakLabel(v)){warning("W090",state.tokens.next,v)}this.first=state.tokens.next;advance()}}reachable(this);return this}).exps=true;stmt("return",function(){if(this.line===startLine(state.tokens.next)){if(state.tokens.next.id!==";"&&!state.tokens.next.reach){this.first=expression(0);if(this.first&&this.first.type==="(punctuator)"&&this.first.value==="="&&!this.first.paren&&!state.option.boss){warningAt("W093",this.first.line,this.first.character)}}}else{if(state.tokens.next.type==="(punctuator)"&&["[","{","+","-"].indexOf(state.tokens.next.value)>-1){nolinebreak(this)}}reachable(this);return this}).exps=true;(function(x){x.exps=true;x.lbp=25;x.ltBoundary="after"})(prefix("yield",function(){if(state.inMoz()){return mozYield.call(this)}var prev=state.tokens.prev;if(!this.beginsStmt&&prev.lbp>30&&!checkPunctuators(prev,["("])){error("E061",this)}if(state.inES6(true)&&!state.funct["(generator)"]){if(!("(catch)"===state.funct["(name)"]&&state.funct["(context)"]["(generator)"])){error("E046",state.tokens.curr,"yield")}}else if(!state.inES6()){warning("W104",state.tokens.curr,"yield","6")}state.funct["(generator)"]="yielded";var delegatingYield=false;if(state.tokens.next.value==="*"){delegatingYield=true;advance("*")}if(!isEndOfExpr()&&state.tokens.next.id!==","){if(state.tokens.next.nud){nobreaknonadjacent(state.tokens.curr,state.tokens.next);this.first=expression(10);if(this.first.type==="(punctuator)"&&this.first.value==="="&&!this.first.paren&&!state.option.boss){warningAt("W093",this.first.line,this.first.character)}}else if(state.tokens.next.led){if(state.tokens.next.id!==","){error("W017",state.tokens.next)}}}return this}));var mozYield=function(){var prev=state.tokens.prev;if(state.inES6(true)&&!state.funct["(generator)"]){if(!("(catch)"===state.funct["(name)"]&&state.funct["(context)"]["(generator)"])){error("E046",state.tokens.curr,"yield")}}state.funct["(generator)"]="yielded";var delegatingYield=false;if(state.tokens.next.value==="*"){delegatingYield=true;advance("*")}if(this.line===startLine(state.tokens.next)){if(delegatingYield||state.tokens.next.id!==";"&&!state.option.asi&&!state.tokens.next.reach&&state.tokens.next.nud){nobreaknonadjacent(state.tokens.curr,state.tokens.next);this.first=expression(10);if(this.first.type==="(punctuator)"&&this.first.value==="="&&!this.first.paren&&!state.option.boss){warningAt("W093",this.first.line,this.first.character)}}if(state.tokens.next.id!==")"&&(prev.lbp>30||!prev.assign&&!isEndOfExpr()||prev.id==="yield")){error("E050",this)}}else if(!state.option.asi){nolinebreak(this)}return this};stmt("throw",function(){nolinebreak(this);this.first=expression(20);reachable(this);return this}).exps=true;stmt("import",function(){if(!state.funct["(scope)"].block.isGlobal()){error("E053",state.tokens.curr,"Import")}if(!state.inES6()){warning("W119",state.tokens.curr,"import","6")}if(state.tokens.next.type==="(string)"){advance("(string)");return this}if(state.tokens.next.identifier){this.name=identifier();state.funct["(scope)"].addlabel(this.name,{type:"import",initialized:true,token:state.tokens.curr});if(state.tokens.next.value===","){advance(",")}else{advance("from");advance("(string)");return this}}if(state.tokens.next.id==="*"){advance("*");advance("as");if(state.tokens.next.identifier){this.name=identifier();state.funct["(scope)"].addlabel(this.name,{type:"import",initialized:true,token:state.tokens.curr})}}else{advance("{");for(;;){if(state.tokens.next.value==="}"){advance("}");break}var importName;if(state.tokens.next.type==="default"){importName="default";advance("default")}else{importName=identifier()}if(state.tokens.next.value==="as"){advance("as");importName=identifier()}state.funct["(scope)"].addlabel(importName,{type:"import",initialized:true,token:state.tokens.curr});if(state.tokens.next.value===","){advance(",")}else if(state.tokens.next.value==="}"){advance("}");break}else{error("E024",state.tokens.next,state.tokens.next.value);break}}}advance("from");advance("(string)");return this}).exps=true;stmt("export",function(){var ok=true;var token;var identifier;if(!state.inES6()){warning("W119",state.tokens.curr,"export","6");ok=false}if(!state.funct["(scope)"].block.isGlobal()){error("E053",state.tokens.curr,"Export");ok=false}if(state.tokens.next.value==="*"){advance("*");advance("from");advance("(string)");return this}if(state.tokens.next.type==="default"){state.nameStack.set(state.tokens.next);advance("default");var exportType=state.tokens.next.id;if(exportType==="function"||exportType==="class"){this.block=true}token=peek();expression(10);identifier=token.value;if(this.block){state.funct["(scope)"].addlabel(identifier,{type:exportType,initialized:true,token:token});state.funct["(scope)"].setExported(identifier,token)}return this}if(state.tokens.next.value==="{"){advance("{");var exportedTokens=[];for(;;){if(!state.tokens.next.identifier){error("E030",state.tokens.next,state.tokens.next.value)}advance();exportedTokens.push(state.tokens.curr);if(state.tokens.next.value==="as"){advance("as");if(!state.tokens.next.identifier){error("E030",state.tokens.next,state.tokens.next.value)}advance()}if(state.tokens.next.value===","){advance(",")}else if(state.tokens.next.value==="}"){advance("}");break}else{error("E024",state.tokens.next,state.tokens.next.value);break}}if(state.tokens.next.value==="from"){advance("from");advance("(string)")}else if(ok){exportedTokens.forEach(function(token){state.funct["(scope)"].setExported(token.value,token)})}return this}if(state.tokens.next.id==="var"){advance("var");state.tokens.curr.fud({inexport:true})}else if(state.tokens.next.id==="let"){advance("let");state.tokens.curr.fud({inexport:true})}else if(state.tokens.next.id==="const"){advance("const");state.tokens.curr.fud({inexport:true})}else if(state.tokens.next.id==="function"){this.block=true;advance("function");state.syntax["function"].fud({inexport:true})}else if(state.tokens.next.id==="class"){this.block=true;advance("class");var classNameToken=state.tokens.next;state.syntax["class"].fud();state.funct["(scope)"].setExported(classNameToken.value,classNameToken)}else{error("E024",state.tokens.next,state.tokens.next.value)}return this}).exps=true;FutureReservedWord("abstract");FutureReservedWord("await",{es5:true,moduleOnly:true});FutureReservedWord("boolean");FutureReservedWord("byte");FutureReservedWord("char");FutureReservedWord("class",{es5:true,nud:classdef});FutureReservedWord("double");FutureReservedWord("enum",{es5:true});FutureReservedWord("export",{es5:true});FutureReservedWord("extends",{es5:true});FutureReservedWord("final");FutureReservedWord("float");FutureReservedWord("goto");FutureReservedWord("implements",{es5:true,strictOnly:true});FutureReservedWord("import",{es5:true});FutureReservedWord("int");FutureReservedWord("interface",{es5:true,strictOnly:true});FutureReservedWord("long");FutureReservedWord("native");FutureReservedWord("package",{es5:true,strictOnly:true});FutureReservedWord("private",{es5:true,strictOnly:true});FutureReservedWord("protected",{es5:true,strictOnly:true});FutureReservedWord("public",{es5:true,strictOnly:true});FutureReservedWord("short");FutureReservedWord("static",{es5:true,strictOnly:true});FutureReservedWord("super",{es5:true});FutureReservedWord("synchronized");FutureReservedWord("transient");FutureReservedWord("volatile");var lookupBlockType=function(){var pn,pn1,prev;var i=-1;var bracketStack=0;var ret={};if(checkPunctuators(state.tokens.curr,["[","{"])){bracketStack+=1}do{prev=i===-1?state.tokens.curr:pn;pn=i===-1?state.tokens.next:peek(i);pn1=peek(i+1);i=i+1;if(checkPunctuators(pn,["[","{"])){bracketStack+=1}else if(checkPunctuators(pn,["]","}"])){bracketStack-=1}if(bracketStack===1&&pn.identifier&&pn.value==="for"&&!checkPunctuator(prev,".")){ret.isCompArray=true;ret.notJson=true;break}if(bracketStack===0&&checkPunctuators(pn,["}","]"])){if(pn1.value==="="){ret.isDestAssign=true;ret.notJson=true;break}else if(pn1.value==="."){ret.notJson=true;break}}if(checkPunctuator(pn,";")){ret.isBlock=true;ret.notJson=true}}while(bracketStack>0&&pn.id!=="(end)");return ret};function saveProperty(props,name,tkn,isClass,isStatic){var msg=["key","class method","static class method"];msg=msg[(isClass||false)+(isStatic||false)];if(tkn.identifier){name=tkn.value}if(props[name]&&name!=="__proto__"){warning("W075",state.tokens.next,msg,name)}else{props[name]=Object.create(null)}props[name].basic=true;props[name].basictkn=tkn}function saveAccessor(accessorType,props,name,tkn,isClass,isStatic){var flagName=accessorType==="get"?"getterToken":"setterToken";var msg="";if(isClass){if(isStatic){msg+="static "}msg+=accessorType+"ter method"}else{msg="key"}state.tokens.curr.accessorType=accessorType;state.nameStack.set(tkn);if(props[name]){if((props[name].basic||props[name][flagName])&&name!=="__proto__"){warning("W075",state.tokens.next,msg,name)}}else{props[name]=Object.create(null)}props[name][flagName]=tkn}function computedPropertyName(){advance("[");if(!state.inES6()){warning("W119",state.tokens.curr,"computed property names","6")}var value=expression(10);advance("]");return value}function checkPunctuators(token,values){if(token.type==="(punctuator)"){return _.contains(values,token.value)}return false}function checkPunctuator(token,value){return token.type==="(punctuator)"&&token.value===value}function destructuringAssignOrJsonValue(){var block=lookupBlockType();if(block.notJson){if(!state.inES6()&&block.isDestAssign){warning("W104",state.tokens.curr,"destructuring assignment","6")}statements()}else{state.option.laxbreak=true;state.jsonMode=true;jsonValue()}}var arrayComprehension=function(){var CompArray=function(){this.mode="use";this.variables=[]};var _carrays=[];var _current;function declare(v){var l=_current.variables.filter(function(elt){if(elt.value===v){elt.undef=false;return v}}).length;return l!==0}function use(v){var l=_current.variables.filter(function(elt){if(elt.value===v&&!elt.undef){if(elt.unused===true){elt.unused=false}return v}}).length;return l===0}return{stack:function(){_current=new CompArray;_carrays.push(_current)},unstack:function(){_current.variables.filter(function(v){if(v.unused)warning("W098",v.token,v.raw_text||v.value);if(v.undef)state.funct["(scope)"].block.use(v.value,v.token)});_carrays.splice(-1,1);_current=_carrays[_carrays.length-1]},setState:function(s){if(_.contains(["use","define","generate","filter"],s))_current.mode=s},check:function(v){if(!_current){return}if(_current&&_current.mode==="use"){if(use(v)){_current.variables.push({funct:state.funct,token:state.tokens.curr,value:v,undef:true,unused:false})}return true}else if(_current&&_current.mode==="define"){if(!declare(v)){_current.variables.push({funct:state.funct,token:state.tokens.curr,value:v,undef:false,unused:true})}return true}else if(_current&&_current.mode==="generate"){state.funct["(scope)"].block.use(v,state.tokens.curr);return true}else if(_current&&_current.mode==="filter"){if(use(v)){state.funct["(scope)"].block.use(v,state.tokens.curr)}return true}return false}}};function jsonValue(){function jsonObject(){var o={},t=state.tokens.next;advance("{");if(state.tokens.next.id!=="}"){for(;;){if(state.tokens.next.id==="(end)"){error("E026",state.tokens.next,t.line)}else if(state.tokens.next.id==="}"){warning("W094",state.tokens.curr);break}else if(state.tokens.next.id===","){error("E028",state.tokens.next)}else if(state.tokens.next.id!=="(string)"){warning("W095",state.tokens.next,state.tokens.next.value)}if(o[state.tokens.next.value]===true){warning("W075",state.tokens.next,"key",state.tokens.next.value)}else if(state.tokens.next.value==="__proto__"&&!state.option.proto||state.tokens.next.value==="__iterator__"&&!state.option.iterator){warning("W096",state.tokens.next,state.tokens.next.value)}else{o[state.tokens.next.value]=true}advance();advance(":");jsonValue();if(state.tokens.next.id!==","){break}advance(",")}}advance("}")}function jsonArray(){var t=state.tokens.next;advance("[");if(state.tokens.next.id!=="]"){for(;;){if(state.tokens.next.id==="(end)"){error("E027",state.tokens.next,t.line)}else if(state.tokens.next.id==="]"){warning("W094",state.tokens.curr);break}else if(state.tokens.next.id===","){error("E028",state.tokens.next)}jsonValue();if(state.tokens.next.id!==","){break}advance(",")}}advance("]")}switch(state.tokens.next.id){case"{":jsonObject();break;case"[":jsonArray();break;case"true":case"false":case"null":case"(number)":case"(string)":advance();break;case"-":advance("-");advance("(number)");break;default:error("E003",state.tokens.next)}}function lintEvalCode(internals,options,globals){var priorErrorCount,idx,jdx,internal;for(idx=0;idx<internals.length;idx+=1){internal=internals[idx];options.scope=internal.elem;priorErrorCount=JSHINT.errors.length;itself(internal.code,options,globals);for(jdx=priorErrorCount;jdx<JSHINT.errors.length;jdx+=1){JSHINT.errors[jdx].line+=internal.token.line-1}}}var escapeRegex=function(str){return str.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")};var itself=function(s,o,g){var x,reIgnoreStr,reIgnore;var optionKeys;var newOptionObj={};var newIgnoredObj={};o=_.clone(o);state.reset();if(o&&o.scope){JSHINT.scope=o.scope}else{JSHINT.errors=[];JSHINT.undefs=[];JSHINT.internals=[];JSHINT.blacklist={};JSHINT.scope="(main)"}predefined=Object.create(null);combine(predefined,vars.ecmaIdentifiers[3]);combine(predefined,vars.reservedVars);combine(predefined,g||{});declared=Object.create(null);var exported=Object.create(null);function each(obj,cb){if(!obj)return;if(!Array.isArray(obj)&&typeof obj==="object")obj=Object.keys(obj);obj.forEach(cb)}if(o){each(o.predef||null,function(item){var slice,prop;if(item[0]==="-"){slice=item.slice(1);JSHINT.blacklist[slice]=slice;delete predefined[slice]}else{prop=Object.getOwnPropertyDescriptor(o.predef,item);predefined[item]=prop?prop.value:false}});each(o.exported||null,function(item){exported[item]=true});delete o.predef;delete o.exported;optionKeys=Object.keys(o);for(x=0;x<optionKeys.length;x++){if(/^-W\d{3}$/g.test(optionKeys[x])){newIgnoredObj[optionKeys[x].slice(1)]=true}else{var optionKey=optionKeys[x];newOptionObj[optionKey]=o[optionKey]}}}state.option=newOptionObj;state.ignored=newIgnoredObj;state.option.indent=state.option.indent||4;state.option.maxerr=state.option.maxerr||50;indent=1;var scopeManagerInst=scopeManager(state,predefined,exported,declared);scopeManagerInst.on("warning",function(ev){warning.apply(null,[ev.code,ev.token].concat(ev.data))});scopeManagerInst.on("error",function(ev){error.apply(null,[ev.code,ev.token].concat(ev.data))});state.funct=functor("(global)",null,{"(global)":true,"(scope)":scopeManagerInst,"(comparray)":arrayComprehension(),"(metrics)":createMetrics(state.tokens.next)});functions=[state.funct];urls=[];stack=null;member={};membersOnly=null;inblock=false;lookahead=[];if(!isString(s)&&!Array.isArray(s)){errorAt("E004",0);return false}api={get isJSON(){return state.jsonMode},getOption:function(name){return state.option[name]||null},getCache:function(name){return state.cache[name]},setCache:function(name,value){state.cache[name]=value},warn:function(code,data){warningAt.apply(null,[code,data.line,data.char].concat(data.data))},on:function(names,listener){names.split(" ").forEach(function(name){emitter.on(name,listener)}.bind(this))}};emitter.removeAllListeners();(extraModules||[]).forEach(function(func){func(api)});state.tokens.prev=state.tokens.curr=state.tokens.next=state.syntax["(begin)"];if(o&&o.ignoreDelimiters){if(!Array.isArray(o.ignoreDelimiters)){o.ignoreDelimiters=[o.ignoreDelimiters]}o.ignoreDelimiters.forEach(function(delimiterPair){if(!delimiterPair.start||!delimiterPair.end)return;reIgnoreStr=escapeRegex(delimiterPair.start)+"[\\s\\S]*?"+escapeRegex(delimiterPair.end);reIgnore=new RegExp(reIgnoreStr,"ig");s=s.replace(reIgnore,function(match){return match.replace(/./g," ")})})}lex=new Lexer(s);lex.on("warning",function(ev){warningAt.apply(null,[ev.code,ev.line,ev.character].concat(ev.data))});lex.on("error",function(ev){errorAt.apply(null,[ev.code,ev.line,ev.character].concat(ev.data))});lex.on("fatal",function(ev){quit("E041",ev)});lex.on("Identifier",function(ev){emitter.emit("Identifier",ev)});lex.on("String",function(ev){emitter.emit("String",ev)});lex.on("Number",function(ev){emitter.emit("Number",ev)});for(var name in o){if(_.has(o,name)){checkOption(name,state.tokens.curr)}}try{applyOptions();combine(predefined,g||{});parseComma.first=true;advance();switch(state.tokens.next.id){case"{":case"[":destructuringAssignOrJsonValue();break;default:directives();if(state.directive["use strict"]){if(!state.allowsGlobalUsd()){warning("W097",state.tokens.prev)}}statements()}if(state.tokens.next.id!=="(end)"){quit("E041",state.tokens.curr)}state.funct["(scope)"].unstack()}catch(err){if(err&&err.name==="JSHintError"){var nt=state.tokens.next||{};JSHINT.errors.push({scope:"(main)",raw:err.raw,code:err.code,reason:err.reason,line:err.line||nt.line,character:err.character||nt.from})}else{throw err}}if(JSHINT.scope==="(main)"){lintEvalCode(JSHINT.internals,o||{},g)}return JSHINT.errors.length===0};itself.addModule=function(func){extraModules.push(func)};itself.addModule(style.register);itself.data=function(){var data={functions:[],options:state.option};var fu,f,i,j,n,globals;if(itself.errors.length){data.errors=itself.errors}if(state.jsonMode){data.json=true}var impliedGlobals=state.funct["(scope)"].getImpliedGlobals();if(impliedGlobals.length>0){data.implieds=impliedGlobals}if(urls.length>0){data.urls=urls}globals=state.funct["(scope)"].getUsedOrDefinedGlobals();if(globals.length>0){data.globals=globals}for(i=1;i<functions.length;i+=1){f=functions[i];fu={};for(j=0;j<functionicity.length;j+=1){fu[functionicity[j]]=[]}for(j=0;j<functionicity.length;j+=1){if(fu[functionicity[j]].length===0){delete fu[functionicity[j]]}}fu.name=f["(name)"];fu.param=f["(params)"];fu.line=f["(line)"];fu.character=f["(character)"];fu.last=f["(last)"];fu.lastcharacter=f["(lastcharacter)"];fu.metrics={complexity:f["(metrics)"].ComplexityCount,parameters:f["(metrics)"].arity,statements:f["(metrics)"].statementCount};data.functions.push(fu)}var unuseds=state.funct["(scope)"].getUnuseds();if(unuseds.length>0){data.unused=unuseds}for(n in member){if(typeof member[n]==="number"){data.member=member;break}}return data};itself.jshint=itself;return itself}();if(typeof exports==="object"&&exports){exports.JSHINT=JSHINT}},{"./lex.js":13,"./messages.js":14,"./options.js":16,"./reg.js":17,"./scope-manager.js":18,"./state.js":19,"./style.js":20,"./vars.js":21,"console-browserify":10,events:5,lodash:12}]},{},[]);JSHINT=require("jshint").JSHINT;if(typeof exports==="object"&&exports)exports.JSHINT=JSHINT})();

// FILE: bower_components/zh-cache-bypass/src/CacheBypass.js
(function() {
    angular.module("ZeedhiApp").config(function($httpProvider) {
        if (Boolean(window.applicationVersion)) {
            $httpProvider.interceptors.push(function() {
                return {
                    'request': function cacheBypass(config) {
                        if (config && config.url && shouldApplyBypass(config.url)) {
                            config.url += '?v='+window.applicationVersion;
                        }
                        return config;
                    }
                };
            });
        } else {
            console.warn('Missing "applicationVersion" global property for CacheBypass.');
        }
    });

    function shouldApplyBypass(url) {
        return (
            Util.stringEndsWith(url, '.json') ||
            Util.stringEndsWith(url, '.html') ||
            Util.stringEndsWith(url, '.svg')
        );
    }
})();

// FILE: bower_components/zh-charts/dist/zh-charts.js

// FILE: src/charts/assets/js/libs/FileSaver/Blob.js
/* Blob.js
 * A Blob implementation.
 * 2014-07-24
 *
 * By Eli Grey, http://eligrey.com
 * By Devin Samarin, https://github.com/dsamarin
 * License: MIT
 *   See https://github.com/eligrey/Blob.js/blob/master/LICENSE.md
 */

/*global self, unescape */
/*jslint bitwise: true, regexp: true, confusion: true, es5: true, vars: true, white: true,
  plusplus: true */

/*! @source http://purl.eligrey.com/github/Blob.js/blob/master/Blob.js */

(function (view) {
	"use strict";

	view.URL = view.URL || view.webkitURL;

	if (view.Blob && view.URL) {
		try {
			new Blob;
			return;
		} catch (e) {}
	}

	// Internally we use a BlobBuilder implementation to base Blob off of
	// in order to support older browsers that only have BlobBuilder
	var BlobBuilder = view.BlobBuilder || view.WebKitBlobBuilder || view.MozBlobBuilder || (function(view) {
		var
			  get_class = function(object) {
				return Object.prototype.toString.call(object).match(/^\[object\s(.*)\]$/)[1];
			}
			, FakeBlobBuilder = function BlobBuilder() {
				this.data = [];
			}
			, FakeBlob = function Blob(data, type, encoding) {
				this.data = data;
				this.size = data.length;
				this.type = type;
				this.encoding = encoding;
			}
			, FBB_proto = FakeBlobBuilder.prototype
			, FB_proto = FakeBlob.prototype
			, FileReaderSync = view.FileReaderSync
			, FileException = function(type) {
				this.code = this[this.name = type];
			}
			, file_ex_codes = (
				  "NOT_FOUND_ERR SECURITY_ERR ABORT_ERR NOT_READABLE_ERR ENCODING_ERR "
				+ "NO_MODIFICATION_ALLOWED_ERR INVALID_STATE_ERR SYNTAX_ERR"
			).split(" ")
			, file_ex_code = file_ex_codes.length
			, real_URL = view.URL || view.webkitURL || view
			, real_create_object_URL = real_URL.createObjectURL
			, real_revoke_object_URL = real_URL.revokeObjectURL
			, URL = real_URL
			, btoa = view.btoa
			, atob = view.atob

			, ArrayBuffer = view.ArrayBuffer
			, Uint8Array = view.Uint8Array

			, origin = /^[\w-]+:\/*\[?[\w\.:-]+\]?(?::[0-9]+)?/
		;
		FakeBlob.fake = FB_proto.fake = true;
		while (file_ex_code--) {
			FileException.prototype[file_ex_codes[file_ex_code]] = file_ex_code + 1;
		}
		// Polyfill URL
		if (!real_URL.createObjectURL) {
			URL = view.URL = function(uri) {
				var
					  uri_info = document.createElementNS("http://www.w3.org/1999/xhtml", "a")
					, uri_origin
				;
				uri_info.href = uri;
				if (!("origin" in uri_info)) {
					if (uri_info.protocol.toLowerCase() === "data:") {
						uri_info.origin = null;
					} else {
						uri_origin = uri.match(origin);
						uri_info.origin = uri_origin && uri_origin[1];
					}
				}
				return uri_info;
			};
		}
		URL.createObjectURL = function(blob) {
			var
				  type = blob.type
				, data_URI_header
			;
			if (type === null) {
				type = "application/octet-stream";
			}
			if (blob instanceof FakeBlob) {
				data_URI_header = "data:" + type;
				if (blob.encoding === "base64") {
					return data_URI_header + ";base64," + blob.data;
				} else if (blob.encoding === "URI") {
					return data_URI_header + "," + decodeURIComponent(blob.data);
				} if (btoa) {
					return data_URI_header + ";base64," + btoa(blob.data);
				} else {
					return data_URI_header + "," + encodeURIComponent(blob.data);
				}
			} else if (real_create_object_URL) {
				return real_create_object_URL.call(real_URL, blob);
			}
		};
		URL.revokeObjectURL = function(object_URL) {
			if (object_URL.substring(0, 5) !== "data:" && real_revoke_object_URL) {
				real_revoke_object_URL.call(real_URL, object_URL);
			}
		};
		FBB_proto.append = function(data/*, endings*/) {
			var bb = this.data;
			// decode data to a binary string
			if (Uint8Array && (data instanceof ArrayBuffer || data instanceof Uint8Array)) {
				var
					  str = ""
					, buf = new Uint8Array(data)
					, i = 0
					, buf_len = buf.length
				;
				for (; i < buf_len; i++) {
					str += String.fromCharCode(buf[i]);
				}
				bb.push(str);
			} else if (get_class(data) === "Blob" || get_class(data) === "File") {
				if (FileReaderSync) {
					var fr = new FileReaderSync;
					bb.push(fr.readAsBinaryString(data));
				} else {
					// async FileReader won't work as BlobBuilder is sync
					throw new FileException("NOT_READABLE_ERR");
				}
			} else if (data instanceof FakeBlob) {
				if (data.encoding === "base64" && atob) {
					bb.push(atob(data.data));
				} else if (data.encoding === "URI") {
					bb.push(decodeURIComponent(data.data));
				} else if (data.encoding === "raw") {
					bb.push(data.data);
				}
			} else {
				if (typeof data !== "string") {
					data += ""; // convert unsupported types to strings
				}
				// decode UTF-16 to binary string
				bb.push(unescape(encodeURIComponent(data)));
			}
		};
		FBB_proto.getBlob = function(type) {
			if (!arguments.length) {
				type = null;
			}
			return new FakeBlob(this.data.join(""), type, "raw");
		};
		FBB_proto.toString = function() {
			return "[object BlobBuilder]";
		};
		FB_proto.slice = function(start, end, type) {
			var args = arguments.length;
			if (args < 3) {
				type = null;
			}
			return new FakeBlob(
				  this.data.slice(start, args > 1 ? end : this.data.length)
				, type
				, this.encoding
			);
		};
		FB_proto.toString = function() {
			return "[object Blob]";
		};
		FB_proto.close = function() {
			this.size = 0;
			delete this.data;
		};
		return FakeBlobBuilder;
	}(view));

	view.Blob = function(blobParts, options) {
		var type = options ? (options.type || "") : "";
		var builder = new BlobBuilder();
		if (blobParts) {
			for (var i = 0, len = blobParts.length; i < len; i++) {
				if (Uint8Array && blobParts[i] instanceof Uint8Array) {
					builder.append(blobParts[i].buffer);
				}
				else {
					builder.append(blobParts[i]);
				}
			}
		}
		var blob = builder.getBlob(type);
		if (!blob.slice && blob.webkitSlice) {
			blob.slice = blob.webkitSlice;
		}
		return blob;
	};

	var getPrototypeOf = Object.getPrototypeOf || function(object) {
		return object.__proto__;
	};
	view.Blob.prototype = getPrototypeOf(new view.Blob());
}(typeof self !== "undefined" && self || typeof window !== "undefined" && window || this.content || this));


// FILE: src/charts/assets/js/libs/FileSaver/FileSaver.js
/* FileSaver.js
 * A saveAs() FileSaver implementation.
 * 1.3.0
 *
 * By Eli Grey, http://eligrey.com
 * License: MIT
 *   See https://github.com/eligrey/FileSaver.js/blob/master/LICENSE.md
 */

/*global self */
/*jslint bitwise: true, indent: 4, laxbreak: true, laxcomma: true, smarttabs: true, plusplus: true */

/*! @source http://purl.eligrey.com/github/FileSaver.js/blob/master/FileSaver.js */

var saveAs = saveAs || (function(view) {
	"use strict";
	// IE <10 is explicitly unsupported
	if (typeof view === "undefined" || typeof navigator !== "undefined" && /MSIE [1-9]\./.test(navigator.userAgent)) {
		return;
	}
	var
		  doc = view.document
		  // only get URL when necessary in case Blob.js hasn't overridden it yet
		, get_URL = function() {
			return view.URL || view.webkitURL || view;
		}
		, save_link = doc.createElementNS("http://www.w3.org/1999/xhtml", "a")
		, can_use_save_link = "download" in save_link
		, click = function(node) {
			var event = new MouseEvent("click");
			node.dispatchEvent(event);
		}
		, is_safari = /constructor/i.test(view.HTMLElement)
		, throw_outside = function(ex) {
			(view.setImmediate || view.setTimeout)(function() {
				throw ex;
			}, 0);
		}
		, force_saveable_type = "application/octet-stream"
		// the Blob API is fundamentally broken as there is no "downloadfinished" event to subscribe to
		, arbitrary_revoke_timeout = 1000 * 40 // in ms
		, revoke = function(file) {
			var revoker = function() {
				if (typeof file === "string") { // file is an object URL
					get_URL().revokeObjectURL(file);
				} else { // file is a File
					file.remove();
				}
			};
			setTimeout(revoker, arbitrary_revoke_timeout);
		}
		, dispatch = function(filesaver, event_types, event) {
			event_types = [].concat(event_types);
			var i = event_types.length;
			while (i--) {
				var listener = filesaver["on" + event_types[i]];
				if (typeof listener === "function") {
					try {
						listener.call(filesaver, event || filesaver);
					} catch (ex) {
						throw_outside(ex);
					}
				}
			}
		}
		, auto_bom = function(blob) {
			// prepend BOM for UTF-8 XML and text/* types (including HTML)
			// note: your browser will automatically convert UTF-16 U+FEFF to EF BB BF
			if (/^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(blob.type)) {
				return new Blob([String.fromCharCode(0xFEFF), blob], {type: blob.type});
			}
			return blob;
		}
		, FileSaver = function(blob, name, no_auto_bom) {
			if (!no_auto_bom) {
				blob = auto_bom(blob);
			}
			// First try a.download, then web filesystem, then object URLs
			var
				  filesaver = this
				, type = blob.type
				, force = type === force_saveable_type
				, object_url
				, dispatch_all = function() {
					dispatch(filesaver, "writestart progress write writeend".split(" "));
				}
				// on any filesys errors revert to saving with object URLs
				, fs_error = function() {
					if (force && is_safari && view.FileReader) {
						// Safari doesn't allow downloading of blob urls
						var reader = new FileReader();
						reader.onloadend = function() {
							var base64Data = reader.result;
							view.location.href = "data:attachment/file" + base64Data.slice(base64Data.search(/[,;]/));
							filesaver.readyState = filesaver.DONE;
							dispatch_all();
						};
						reader.readAsDataURL(blob);
						filesaver.readyState = filesaver.INIT;
						return;
					}
					// don't create more object URLs than needed
					if (!object_url) {
						object_url = get_URL().createObjectURL(blob);
					}
					if (force) {
						view.location.href = object_url;
					} else {
						var opened = view.open(object_url, "_blank");
						if (!opened) {
							// Apple does not allow window.open, see https://developer.apple.com/library/safari/documentation/Tools/Conceptual/SafariExtensionGuide/WorkingwithWindowsandTabs/WorkingwithWindowsandTabs.html
							view.location.href = object_url;
						}
					}
					filesaver.readyState = filesaver.DONE;
					dispatch_all();
					revoke(object_url);
				}
			;
			filesaver.readyState = filesaver.INIT;

			if (can_use_save_link) {
				object_url = get_URL().createObjectURL(blob);
				setTimeout(function() {
					save_link.href = object_url;
					save_link.download = name;
					click(save_link);
					dispatch_all();
					revoke(object_url);
					filesaver.readyState = filesaver.DONE;
				});
				return;
			}

			fs_error();
		}
		, FS_proto = FileSaver.prototype
		, saveAs = function(blob, name, no_auto_bom) {
			return new FileSaver(blob, name || blob.name || "download", no_auto_bom);
		}
	;
	// IE 10+ (native saveAs)
	if (typeof navigator !== "undefined" && navigator.msSaveOrOpenBlob) {
		return function(blob, name, no_auto_bom) {
			name = name || blob.name || "download";

			if (!no_auto_bom) {
				blob = auto_bom(blob);
			}
			return navigator.msSaveOrOpenBlob(blob, name);
		};
	}

	FS_proto.abort = function(){};
	FS_proto.readyState = FS_proto.INIT = 0;
	FS_proto.WRITING = 1;
	FS_proto.DONE = 2;

	FS_proto.error =
	FS_proto.onwritestart =
	FS_proto.onprogress =
	FS_proto.onwrite =
	FS_proto.onabort =
	FS_proto.onerror =
	FS_proto.onwriteend =
		null;

	return saveAs;
}(
	   typeof self !== "undefined" && self
	|| typeof window !== "undefined" && window
	|| this.content
));
// `self` is undefined in Firefox for Android content script context
// while `this` is nsIContentFrameMessageManager
// with an attribute `content` that corresponds to the window

// removed by ZED 01/07/2016
// if (typeof module !== "undefined" && module.exports) {
//   module.exports.saveAs = saveAs;
// } else if ((typeof define !== "undefined" && define !== null) && (define.amd !== null)) {
//   define([], function() {
//     return saveAs;
//   });
// }


// FILE: src/charts/assets/js/libs/FileSaver/canvas-toBlob.js
/* canvas-toBlob.js
 * A canvas.toBlob() implementation.
 * 2016-05-26
 * 
 * By Eli Grey, http://eligrey.com and Devin Samarin, https://github.com/eboyjr
 * License: MIT
 *   See https://github.com/eligrey/canvas-toBlob.js/blob/master/LICENSE.md
 */

/*global self */
/*jslint bitwise: true, regexp: true, confusion: true, es5: true, vars: true, white: true,
  plusplus: true */

/*! @source http://purl.eligrey.com/github/canvas-toBlob.js/blob/master/canvas-toBlob.js */

(function(view) {
"use strict";
var
	  Uint8Array = view.Uint8Array
	, HTMLCanvasElement = view.HTMLCanvasElement
	, canvas_proto = HTMLCanvasElement && HTMLCanvasElement.prototype
	, is_base64_regex = /\s*;\s*base64\s*(?:;|$)/i
	, to_data_url = "toDataURL"
	, base64_ranks
	, decode_base64 = function(base64) {
		var
			  len = base64.length
			, buffer = new Uint8Array(len / 4 * 3 | 0)
			, i = 0
			, outptr = 0
			, last = [0, 0]
			, state = 0
			, save = 0
			, rank
			, code
			, undef
		;
		while (len--) {
			code = base64.charCodeAt(i++);
			rank = base64_ranks[code-43];
			if (rank !== 255 && rank !== undef) {
				last[1] = last[0];
				last[0] = code;
				save = (save << 6) | rank;
				state++;
				if (state === 4) {
					buffer[outptr++] = save >>> 16;
					if (last[1] !== 61 /* padding character */) {
						buffer[outptr++] = save >>> 8;
					}
					if (last[0] !== 61 /* padding character */) {
						buffer[outptr++] = save;
					}
					state = 0;
				}
			}
		}
		// 2/3 chance there's going to be some null bytes at the end, but that
		// doesn't really matter with most image formats.
		// If it somehow matters for you, truncate the buffer up outptr.
		return buffer;
	}
;
if (Uint8Array) {
	base64_ranks = new Uint8Array([
		  62, -1, -1, -1, 63, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1
		, -1, -1,  0, -1, -1, -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9
		, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25
		, -1, -1, -1, -1, -1, -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35
		, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51
	]);
}
if (HTMLCanvasElement && (!canvas_proto.toBlob || !canvas_proto.toBlobHD)) {
	if (!canvas_proto.toBlob)
	canvas_proto.toBlob = function(callback, type /*, ...args*/) {
		  if (!type) {
			type = "image/png";
		} if (this.mozGetAsFile) {
			callback(this.mozGetAsFile("canvas", type));
			return;
		} if (this.msToBlob && /^\s*image\/png\s*(?:$|;)/i.test(type)) {
			callback(this.msToBlob());
			return;
		}

		var
			  args = Array.prototype.slice.call(arguments, 1)
			, dataURI = this[to_data_url].apply(this, args)
			, header_end = dataURI.indexOf(",")
			, data = dataURI.substring(header_end + 1)
			, is_base64 = is_base64_regex.test(dataURI.substring(0, header_end))
			, blob
		;
		if (Blob.fake) {
			// no reason to decode a data: URI that's just going to become a data URI again
			blob = new Blob
			if (is_base64) {
				blob.encoding = "base64";
			} else {
				blob.encoding = "URI";
			}
			blob.data = data;
			blob.size = data.length;
		} else if (Uint8Array) {
			if (is_base64) {
				blob = new Blob([decode_base64(data)], {type: type});
			} else {
				blob = new Blob([decodeURIComponent(data)], {type: type});
			}
		}
		callback(blob);
	};

	if (!canvas_proto.toBlobHD && canvas_proto.toDataURLHD) {
		canvas_proto.toBlobHD = function() {
			to_data_url = "toDataURLHD";
			var blob = this.toBlob();
			to_data_url = "toDataURL";
			return blob;
		}
	} else {
		canvas_proto.toBlobHD = canvas_proto.toBlob;
	}
}
}(typeof self !== "undefined" && self || typeof window !== "undefined" && window || this.content || this));


// FILE: src/charts/assets/js/libs/d3/d3.3.5.17.js
!function() {
  var d3 = {
    version: "3.5.17"
  };
  var d3_arraySlice = [].slice, d3_array = function(list) {
    return d3_arraySlice.call(list);
  };
  var d3_document = this.document;
  function d3_documentElement(node) {
    return node && (node.ownerDocument || node.document || node).documentElement;
  }
  function d3_window(node) {
    return node && (node.ownerDocument && node.ownerDocument.defaultView || node.document && node || node.defaultView);
  }
  if (d3_document) {
    try {
      d3_array(d3_document.documentElement.childNodes)[0].nodeType;
    } catch (e) {
      d3_array = function(list) {
        var i = list.length, array = new Array(i);
        while (i--) array[i] = list[i];
        return array;
      };
    }
  }
  if (!Date.now) Date.now = function() {
    return +new Date();
  };
  if (d3_document) {
    try {
      d3_document.createElement("DIV").style.setProperty("opacity", 0, "");
    } catch (error) {
      var d3_element_prototype = this.Element.prototype, d3_element_setAttribute = d3_element_prototype.setAttribute, d3_element_setAttributeNS = d3_element_prototype.setAttributeNS, d3_style_prototype = this.CSSStyleDeclaration.prototype, d3_style_setProperty = d3_style_prototype.setProperty;
      d3_element_prototype.setAttribute = function(name, value) {
        d3_element_setAttribute.call(this, name, value + "");
      };
      d3_element_prototype.setAttributeNS = function(space, local, value) {
        d3_element_setAttributeNS.call(this, space, local, value + "");
      };
      d3_style_prototype.setProperty = function(name, value, priority) {
        d3_style_setProperty.call(this, name, value + "", priority);
      };
    }
  }
  d3.ascending = d3_ascending;
  function d3_ascending(a, b) {
    return a < b ? -1 : a > b ? 1 : a >= b ? 0 : NaN;
  }
  d3.descending = function(a, b) {
    return b < a ? -1 : b > a ? 1 : b >= a ? 0 : NaN;
  };
  d3.min = function(array, f) {
    var i = -1, n = array.length, a, b;
    if (arguments.length === 1) {
      while (++i < n) if ((b = array[i]) != null && b >= b) {
        a = b;
        break;
      }
      while (++i < n) if ((b = array[i]) != null && a > b) a = b;
    } else {
      while (++i < n) if ((b = f.call(array, array[i], i)) != null && b >= b) {
        a = b;
        break;
      }
      while (++i < n) if ((b = f.call(array, array[i], i)) != null && a > b) a = b;
    }
    return a;
  };
  d3.max = function(array, f) {
    var i = -1, n = array.length, a, b;
    if (arguments.length === 1) {
      while (++i < n) if ((b = array[i]) != null && b >= b) {
        a = b;
        break;
      }
      while (++i < n) if ((b = array[i]) != null && b > a) a = b;
    } else {
      while (++i < n) if ((b = f.call(array, array[i], i)) != null && b >= b) {
        a = b;
        break;
      }
      while (++i < n) if ((b = f.call(array, array[i], i)) != null && b > a) a = b;
    }
    return a;
  };
  d3.extent = function(array, f) {
    var i = -1, n = array.length, a, b, c;
    if (arguments.length === 1) {
      while (++i < n) if ((b = array[i]) != null && b >= b) {
        a = c = b;
        break;
      }
      while (++i < n) if ((b = array[i]) != null) {
        if (a > b) a = b;
        if (c < b) c = b;
      }
    } else {
      while (++i < n) if ((b = f.call(array, array[i], i)) != null && b >= b) {
        a = c = b;
        break;
      }
      while (++i < n) if ((b = f.call(array, array[i], i)) != null) {
        if (a > b) a = b;
        if (c < b) c = b;
      }
    }
    return [ a, c ];
  };
  function d3_number(x) {
    return x === null ? NaN : +x;
  }
  function d3_numeric(x) {
    return !isNaN(x);
  }
  d3.sum = function(array, f) {
    var s = 0, n = array.length, a, i = -1;
    if (arguments.length === 1) {
      while (++i < n) if (d3_numeric(a = +array[i])) s += a;
    } else {
      while (++i < n) if (d3_numeric(a = +f.call(array, array[i], i))) s += a;
    }
    return s;
  };
  d3.mean = function(array, f) {
    var s = 0, n = array.length, a, i = -1, j = n;
    if (arguments.length === 1) {
      while (++i < n) if (d3_numeric(a = d3_number(array[i]))) s += a; else --j;
    } else {
      while (++i < n) if (d3_numeric(a = d3_number(f.call(array, array[i], i)))) s += a; else --j;
    }
    if (j) return s / j;
  };
  d3.quantile = function(values, p) {
    var H = (values.length - 1) * p + 1, h = Math.floor(H), v = +values[h - 1], e = H - h;
    return e ? v + e * (values[h] - v) : v;
  };
  d3.median = function(array, f) {
    var numbers = [], n = array.length, a, i = -1;
    if (arguments.length === 1) {
      while (++i < n) if (d3_numeric(a = d3_number(array[i]))) numbers.push(a);
    } else {
      while (++i < n) if (d3_numeric(a = d3_number(f.call(array, array[i], i)))) numbers.push(a);
    }
    if (numbers.length) return d3.quantile(numbers.sort(d3_ascending), .5);
  };
  d3.variance = function(array, f) {
    var n = array.length, m = 0, a, d, s = 0, i = -1, j = 0;
    if (arguments.length === 1) {
      while (++i < n) {
        if (d3_numeric(a = d3_number(array[i]))) {
          d = a - m;
          m += d / ++j;
          s += d * (a - m);
        }
      }
    } else {
      while (++i < n) {
        if (d3_numeric(a = d3_number(f.call(array, array[i], i)))) {
          d = a - m;
          m += d / ++j;
          s += d * (a - m);
        }
      }
    }
    if (j > 1) return s / (j - 1);
  };
  d3.deviation = function() {
    var v = d3.variance.apply(this, arguments);
    return v ? Math.sqrt(v) : v;
  };
  function d3_bisector(compare) {
    return {
      left: function(a, x, lo, hi) {
        if (arguments.length < 3) lo = 0;
        if (arguments.length < 4) hi = a.length;
        while (lo < hi) {
          var mid = lo + hi >>> 1;
          if (compare(a[mid], x) < 0) lo = mid + 1; else hi = mid;
        }
        return lo;
      },
      right: function(a, x, lo, hi) {
        if (arguments.length < 3) lo = 0;
        if (arguments.length < 4) hi = a.length;
        while (lo < hi) {
          var mid = lo + hi >>> 1;
          if (compare(a[mid], x) > 0) hi = mid; else lo = mid + 1;
        }
        return lo;
      }
    };
  }
  var d3_bisect = d3_bisector(d3_ascending);
  d3.bisectLeft = d3_bisect.left;
  d3.bisect = d3.bisectRight = d3_bisect.right;
  d3.bisector = function(f) {
    return d3_bisector(f.length === 1 ? function(d, x) {
      return d3_ascending(f(d), x);
    } : f);
  };
  d3.shuffle = function(array, i0, i1) {
    if ((m = arguments.length) < 3) {
      i1 = array.length;
      if (m < 2) i0 = 0;
    }
    var m = i1 - i0, t, i;
    while (m) {
      i = Math.random() * m-- | 0;
      t = array[m + i0], array[m + i0] = array[i + i0], array[i + i0] = t;
    }
    return array;
  };
  d3.permute = function(array, indexes) {
    var i = indexes.length, permutes = new Array(i);
    while (i--) permutes[i] = array[indexes[i]];
    return permutes;
  };
  d3.pairs = function(array) {
    var i = 0, n = array.length - 1, p0, p1 = array[0], pairs = new Array(n < 0 ? 0 : n);
    while (i < n) pairs[i] = [ p0 = p1, p1 = array[++i] ];
    return pairs;
  };
  d3.transpose = function(matrix) {
    if (!(n = matrix.length)) return [];
    for (var i = -1, m = d3.min(matrix, d3_transposeLength), transpose = new Array(m); ++i < m; ) {
      for (var j = -1, n, row = transpose[i] = new Array(n); ++j < n; ) {
        row[j] = matrix[j][i];
      }
    }
    return transpose;
  };
  function d3_transposeLength(d) {
    return d.length;
  }
  d3.zip = function() {
    return d3.transpose(arguments);
  };
  d3.keys = function(map) {
    var keys = [];
    for (var key in map) keys.push(key);
    return keys;
  };
  d3.values = function(map) {
    var values = [];
    for (var key in map) values.push(map[key]);
    return values;
  };
  d3.entries = function(map) {
    var entries = [];
    for (var key in map) entries.push({
      key: key,
      value: map[key]
    });
    return entries;
  };
  d3.merge = function(arrays) {
    var n = arrays.length, m, i = -1, j = 0, merged, array;
    while (++i < n) j += arrays[i].length;
    merged = new Array(j);
    while (--n >= 0) {
      array = arrays[n];
      m = array.length;
      while (--m >= 0) {
        merged[--j] = array[m];
      }
    }
    return merged;
  };
  var abs = Math.abs;
  d3.range = function(start, stop, step) {
    if (arguments.length < 3) {
      step = 1;
      if (arguments.length < 2) {
        stop = start;
        start = 0;
      }
    }
    if ((stop - start) / step === Infinity) throw new Error("infinite range");
    var range = [], k = d3_range_integerScale(abs(step)), i = -1, j;
    start *= k, stop *= k, step *= k;
    if (step < 0) while ((j = start + step * ++i) > stop) range.push(j / k); else while ((j = start + step * ++i) < stop) range.push(j / k);
    return range;
  };
  function d3_range_integerScale(x) {
    var k = 1;
    while (x * k % 1) k *= 10;
    return k;
  }
  function d3_class(ctor, properties) {
    for (var key in properties) {
      Object.defineProperty(ctor.prototype, key, {
        value: properties[key],
        enumerable: false
      });
    }
  }
  d3.map = function(object, f) {
    var map = new d3_Map();
    if (object instanceof d3_Map) {
      object.forEach(function(key, value) {
        map.set(key, value);
      });
    } else if (Array.isArray(object)) {
      var i = -1, n = object.length, o;
      if (arguments.length === 1) while (++i < n) map.set(i, object[i]); else while (++i < n) map.set(f.call(object, o = object[i], i), o);
    } else {
      for (var key in object) map.set(key, object[key]);
    }
    return map;
  };
  function d3_Map() {
    this._ = Object.create(null);
  }
  var d3_map_proto = "__proto__", d3_map_zero = "\x00";
  d3_class(d3_Map, {
    has: d3_map_has,
    get: function(key) {
      return this._[d3_map_escape(key)];
    },
    set: function(key, value) {
      return this._[d3_map_escape(key)] = value;
    },
    remove: d3_map_remove,
    keys: d3_map_keys,
    values: function() {
      var values = [];
      for (var key in this._) values.push(this._[key]);
      return values;
    },
    entries: function() {
      var entries = [];
      for (var key in this._) entries.push({
        key: d3_map_unescape(key),
        value: this._[key]
      });
      return entries;
    },
    size: d3_map_size,
    empty: d3_map_empty,
    forEach: function(f) {
      for (var key in this._) f.call(this, d3_map_unescape(key), this._[key]);
    }
  });
  function d3_map_escape(key) {
    return (key += "") === d3_map_proto || key[0] === d3_map_zero ? d3_map_zero + key : key;
  }
  function d3_map_unescape(key) {
    return (key += "")[0] === d3_map_zero ? key.slice(1) : key;
  }
  function d3_map_has(key) {
    return d3_map_escape(key) in this._;
  }
  function d3_map_remove(key) {
    return (key = d3_map_escape(key)) in this._ && delete this._[key];
  }
  function d3_map_keys() {
    var keys = [];
    for (var key in this._) keys.push(d3_map_unescape(key));
    return keys;
  }
  function d3_map_size() {
    var size = 0;
    for (var key in this._) ++size;
    return size;
  }
  function d3_map_empty() {
    for (var key in this._) return false;
    return true;
  }
  d3.nest = function() {
    var nest = {}, keys = [], sortKeys = [], sortValues, rollup;
    function map(mapType, array, depth) {
      if (depth >= keys.length) return rollup ? rollup.call(nest, array) : sortValues ? array.sort(sortValues) : array;
      var i = -1, n = array.length, key = keys[depth++], keyValue, object, setter, valuesByKey = new d3_Map(), values;
      while (++i < n) {
        if (values = valuesByKey.get(keyValue = key(object = array[i]))) {
          values.push(object);
        } else {
          valuesByKey.set(keyValue, [ object ]);
        }
      }
      if (mapType) {
        object = mapType();
        setter = function(keyValue, values) {
          object.set(keyValue, map(mapType, values, depth));
        };
      } else {
        object = {};
        setter = function(keyValue, values) {
          object[keyValue] = map(mapType, values, depth);
        };
      }
      valuesByKey.forEach(setter);
      return object;
    }
    function entries(map, depth) {
      if (depth >= keys.length) return map;
      var array = [], sortKey = sortKeys[depth++];
      map.forEach(function(key, keyMap) {
        array.push({
          key: key,
          values: entries(keyMap, depth)
        });
      });
      return sortKey ? array.sort(function(a, b) {
        return sortKey(a.key, b.key);
      }) : array;
    }
    nest.map = function(array, mapType) {
      return map(mapType, array, 0);
    };
    nest.entries = function(array) {
      return entries(map(d3.map, array, 0), 0);
    };
    nest.key = function(d) {
      keys.push(d);
      return nest;
    };
    nest.sortKeys = function(order) {
      sortKeys[keys.length - 1] = order;
      return nest;
    };
    nest.sortValues = function(order) {
      sortValues = order;
      return nest;
    };
    nest.rollup = function(f) {
      rollup = f;
      return nest;
    };
    return nest;
  };
  d3.set = function(array) {
    var set = new d3_Set();
    if (array) for (var i = 0, n = array.length; i < n; ++i) set.add(array[i]);
    return set;
  };
  function d3_Set() {
    this._ = Object.create(null);
  }
  d3_class(d3_Set, {
    has: d3_map_has,
    add: function(key) {
      this._[d3_map_escape(key += "")] = true;
      return key;
    },
    remove: d3_map_remove,
    values: d3_map_keys,
    size: d3_map_size,
    empty: d3_map_empty,
    forEach: function(f) {
      for (var key in this._) f.call(this, d3_map_unescape(key));
    }
  });
  d3.behavior = {};
  function d3_identity(d) {
    return d;
  }
  d3.rebind = function(target, source) {
    var i = 1, n = arguments.length, method;
    while (++i < n) target[method = arguments[i]] = d3_rebind(target, source, source[method]);
    return target;
  };
  function d3_rebind(target, source, method) {
    return function() {
      var value = method.apply(source, arguments);
      return value === source ? target : value;
    };
  }
  function d3_vendorSymbol(object, name) {
    if (name in object) return name;
    name = name.charAt(0).toUpperCase() + name.slice(1);
    for (var i = 0, n = d3_vendorPrefixes.length; i < n; ++i) {
      var prefixName = d3_vendorPrefixes[i] + name;
      if (prefixName in object) return prefixName;
    }
  }
  var d3_vendorPrefixes = [ "webkit", "ms", "moz", "Moz", "o", "O" ];
  function d3_noop() {}
  d3.dispatch = function() {
    var dispatch = new d3_dispatch(), i = -1, n = arguments.length;
    while (++i < n) dispatch[arguments[i]] = d3_dispatch_event(dispatch);
    return dispatch;
  };
  function d3_dispatch() {}
  d3_dispatch.prototype.on = function(type, listener) {
    var i = type.indexOf("."), name = "";
    if (i >= 0) {
      name = type.slice(i + 1);
      type = type.slice(0, i);
    }
    if (type) return arguments.length < 2 ? this[type].on(name) : this[type].on(name, listener);
    if (arguments.length === 2) {
      if (listener == null) for (type in this) {
        if (this.hasOwnProperty(type)) this[type].on(name, null);
      }
      return this;
    }
  };
  function d3_dispatch_event(dispatch) {
    var listeners = [], listenerByName = new d3_Map();
    function event() {
      var z = listeners, i = -1, n = z.length, l;
      while (++i < n) if (l = z[i].on) l.apply(this, arguments);
      return dispatch;
    }
    event.on = function(name, listener) {
      var l = listenerByName.get(name), i;
      if (arguments.length < 2) return l && l.on;
      if (l) {
        l.on = null;
        listeners = listeners.slice(0, i = listeners.indexOf(l)).concat(listeners.slice(i + 1));
        listenerByName.remove(name);
      }
      if (listener) listeners.push(listenerByName.set(name, {
        on: listener
      }));
      return dispatch;
    };
    return event;
  }
  d3.event = null;
  function d3_eventPreventDefault() {
    d3.event.preventDefault();
  }
  function d3_eventSource() {
    var e = d3.event, s;
    while (s = e.sourceEvent) e = s;
    return e;
  }
  function d3_eventDispatch(target) {
    var dispatch = new d3_dispatch(), i = 0, n = arguments.length;
    while (++i < n) dispatch[arguments[i]] = d3_dispatch_event(dispatch);
    dispatch.of = function(thiz, argumentz) {
      return function(e1) {
        try {
          var e0 = e1.sourceEvent = d3.event;
          e1.target = target;
          d3.event = e1;
          dispatch[e1.type].apply(thiz, argumentz);
        } finally {
          d3.event = e0;
        }
      };
    };
    return dispatch;
  }
  d3.requote = function(s) {
    return s.replace(d3_requote_re, "\\$&");
  };
  var d3_requote_re = /[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g;
  var d3_subclass = {}.__proto__ ? function(object, prototype) {
    object.__proto__ = prototype;
  } : function(object, prototype) {
    for (var property in prototype) object[property] = prototype[property];
  };
  function d3_selection(groups) {
    d3_subclass(groups, d3_selectionPrototype);
    return groups;
  }
  var d3_select = function(s, n) {
    return n.querySelector(s);
  }, d3_selectAll = function(s, n) {
    return n.querySelectorAll(s);
  }, d3_selectMatches = function(n, s) {
    var d3_selectMatcher = n.matches || n[d3_vendorSymbol(n, "matchesSelector")];
    d3_selectMatches = function(n, s) {
      return d3_selectMatcher.call(n, s);
    };
    return d3_selectMatches(n, s);
  };
  if (typeof Sizzle === "function") {
    d3_select = function(s, n) {
      return Sizzle(s, n)[0] || null;
    };
    d3_selectAll = Sizzle;
    d3_selectMatches = Sizzle.matchesSelector;
  }
  d3.selection = function() {
    return d3.select(d3_document.documentElement);
  };
  var d3_selectionPrototype = d3.selection.prototype = [];
  d3_selectionPrototype.select = function(selector) {
    var subgroups = [], subgroup, subnode, group, node;
    selector = d3_selection_selector(selector);
    for (var j = -1, m = this.length; ++j < m; ) {
      subgroups.push(subgroup = []);
      subgroup.parentNode = (group = this[j]).parentNode;
      for (var i = -1, n = group.length; ++i < n; ) {
        if (node = group[i]) {
          subgroup.push(subnode = selector.call(node, node.__data__, i, j));
          if (subnode && "__data__" in node) subnode.__data__ = node.__data__;
        } else {
          subgroup.push(null);
        }
      }
    }
    return d3_selection(subgroups);
  };
  function d3_selection_selector(selector) {
    return typeof selector === "function" ? selector : function() {
      return d3_select(selector, this);
    };
  }
  d3_selectionPrototype.selectAll = function(selector) {
    var subgroups = [], subgroup, node;
    selector = d3_selection_selectorAll(selector);
    for (var j = -1, m = this.length; ++j < m; ) {
      for (var group = this[j], i = -1, n = group.length; ++i < n; ) {
        if (node = group[i]) {
          subgroups.push(subgroup = d3_array(selector.call(node, node.__data__, i, j)));
          subgroup.parentNode = node;
        }
      }
    }
    return d3_selection(subgroups);
  };
  function d3_selection_selectorAll(selector) {
    return typeof selector === "function" ? selector : function() {
      return d3_selectAll(selector, this);
    };
  }
  var d3_nsXhtml = "http://www.w3.org/1999/xhtml";
  var d3_nsPrefix = {
    svg: "http://www.w3.org/2000/svg",
    xhtml: d3_nsXhtml,
    xlink: "http://www.w3.org/1999/xlink",
    xml: "http://www.w3.org/XML/1998/namespace",
    xmlns: "http://www.w3.org/2000/xmlns/"
  };
  d3.ns = {
    prefix: d3_nsPrefix,
    qualify: function(name) {
      var i = name.indexOf(":"), prefix = name;
      if (i >= 0 && (prefix = name.slice(0, i)) !== "xmlns") name = name.slice(i + 1);
      return d3_nsPrefix.hasOwnProperty(prefix) ? {
        space: d3_nsPrefix[prefix],
        local: name
      } : name;
    }
  };
  d3_selectionPrototype.attr = function(name, value) {
    if (arguments.length < 2) {
      if (typeof name === "string") {
        var node = this.node();
        name = d3.ns.qualify(name);
        return name.local ? node.getAttributeNS(name.space, name.local) : node.getAttribute(name);
      }
      for (value in name) this.each(d3_selection_attr(value, name[value]));
      return this;
    }
    return this.each(d3_selection_attr(name, value));
  };
  function d3_selection_attr(name, value) {
    name = d3.ns.qualify(name);
    function attrNull() {
      this.removeAttribute(name);
    }
    function attrNullNS() {
      this.removeAttributeNS(name.space, name.local);
    }
    function attrConstant() {
      this.setAttribute(name, value);
    }
    function attrConstantNS() {
      this.setAttributeNS(name.space, name.local, value);
    }
    function attrFunction() {
      var x = value.apply(this, arguments);
      if (x == null) this.removeAttribute(name); else this.setAttribute(name, x);
    }
    function attrFunctionNS() {
      var x = value.apply(this, arguments);
      if (x == null) this.removeAttributeNS(name.space, name.local); else this.setAttributeNS(name.space, name.local, x);
    }
    return value == null ? name.local ? attrNullNS : attrNull : typeof value === "function" ? name.local ? attrFunctionNS : attrFunction : name.local ? attrConstantNS : attrConstant;
  }
  function d3_collapse(s) {
    return s.trim().replace(/\s+/g, " ");
  }
  d3_selectionPrototype.classed = function(name, value) {
    if (arguments.length < 2) {
      if (typeof name === "string") {
        var node = this.node(), n = (name = d3_selection_classes(name)).length, i = -1;
        if (value = node.classList) {
          while (++i < n) if (!value.contains(name[i])) return false;
        } else {
          value = node.getAttribute("class");
          while (++i < n) if (!d3_selection_classedRe(name[i]).test(value)) return false;
        }
        return true;
      }
      for (value in name) this.each(d3_selection_classed(value, name[value]));
      return this;
    }
    return this.each(d3_selection_classed(name, value));
  };
  function d3_selection_classedRe(name) {
    return new RegExp("(?:^|\\s+)" + d3.requote(name) + "(?:\\s+|$)", "g");
  }
  function d3_selection_classes(name) {
    return (name + "").trim().split(/^|\s+/);
  }
  function d3_selection_classed(name, value) {
    name = d3_selection_classes(name).map(d3_selection_classedName);
    var n = name.length;
    function classedConstant() {
      var i = -1;
      while (++i < n) name[i](this, value);
    }
    function classedFunction() {
      var i = -1, x = value.apply(this, arguments);
      while (++i < n) name[i](this, x);
    }
    return typeof value === "function" ? classedFunction : classedConstant;
  }
  function d3_selection_classedName(name) {
    var re = d3_selection_classedRe(name);
    return function(node, value) {
      if (c = node.classList) return value ? c.add(name) : c.remove(name);
      var c = node.getAttribute("class") || "";
      if (value) {
        re.lastIndex = 0;
        if (!re.test(c)) node.setAttribute("class", d3_collapse(c + " " + name));
      } else {
        node.setAttribute("class", d3_collapse(c.replace(re, " ")));
      }
    };
  }
  d3_selectionPrototype.style = function(name, value, priority) {
    var n = arguments.length;
    if (n < 3) {
      if (typeof name !== "string") {
        if (n < 2) value = "";
        for (priority in name) this.each(d3_selection_style(priority, name[priority], value));
        return this;
      }
      if (n < 2) {
        var node = this.node();
        return d3_window(node).getComputedStyle(node, null).getPropertyValue(name);
      }
      priority = "";
    }
    return this.each(d3_selection_style(name, value, priority));
  };
  function d3_selection_style(name, value, priority) {
    function styleNull() {
      this.style.removeProperty(name);
    }
    function styleConstant() {
      this.style.setProperty(name, value, priority);
    }
    function styleFunction() {
      var x = value.apply(this, arguments);
      if (x == null) this.style.removeProperty(name); else this.style.setProperty(name, x, priority);
    }
    return value == null ? styleNull : typeof value === "function" ? styleFunction : styleConstant;
  }
  d3_selectionPrototype.property = function(name, value) {
    if (arguments.length < 2) {
      if (typeof name === "string") return this.node()[name];
      for (value in name) this.each(d3_selection_property(value, name[value]));
      return this;
    }
    return this.each(d3_selection_property(name, value));
  };
  function d3_selection_property(name, value) {
    function propertyNull() {
      delete this[name];
    }
    function propertyConstant() {
      this[name] = value;
    }
    function propertyFunction() {
      var x = value.apply(this, arguments);
      if (x == null) delete this[name]; else this[name] = x;
    }
    return value == null ? propertyNull : typeof value === "function" ? propertyFunction : propertyConstant;
  }
  d3_selectionPrototype.text = function(value) {
    return arguments.length ? this.each(typeof value === "function" ? function() {
      var v = value.apply(this, arguments);
      this.textContent = v == null ? "" : v;
    } : value == null ? function() {
      this.textContent = "";
    } : function() {
      this.textContent = value;
    }) : this.node().textContent;
  };
  d3_selectionPrototype.html = function(value) {
    return arguments.length ? this.each(typeof value === "function" ? function() {
      var v = value.apply(this, arguments);
      this.innerHTML = v == null ? "" : v;
    } : value == null ? function() {
      this.innerHTML = "";
    } : function() {
      this.innerHTML = value;
    }) : this.node().innerHTML;
  };
  d3_selectionPrototype.append = function(name) {
    name = d3_selection_creator(name);
    return this.select(function() {
      return this.appendChild(name.apply(this, arguments));
    });
  };
  function d3_selection_creator(name) {
    function create() {
      var document = this.ownerDocument, namespace = this.namespaceURI;
      return namespace === d3_nsXhtml && document.documentElement.namespaceURI === d3_nsXhtml ? document.createElement(name) : document.createElementNS(namespace, name);
    }
    function createNS() {
      return this.ownerDocument.createElementNS(name.space, name.local);
    }
    return typeof name === "function" ? name : (name = d3.ns.qualify(name)).local ? createNS : create;
  }
  d3_selectionPrototype.insert = function(name, before) {
    name = d3_selection_creator(name);
    before = d3_selection_selector(before);
    return this.select(function() {
      return this.insertBefore(name.apply(this, arguments), before.apply(this, arguments) || null);
    });
  };
  d3_selectionPrototype.remove = function() {
    return this.each(d3_selectionRemove);
  };
  function d3_selectionRemove() {
    var parent = this.parentNode;
    if (parent) parent.removeChild(this);
  }
  d3_selectionPrototype.data = function(value, key) {
    var i = -1, n = this.length, group, node;
    if (!arguments.length) {
      value = new Array(n = (group = this[0]).length);
      while (++i < n) {
        if (node = group[i]) {
          value[i] = node.__data__;
        }
      }
      return value;
    }
    function bind(group, groupData) {
      var i, n = group.length, m = groupData.length, n0 = Math.min(n, m), updateNodes = new Array(m), enterNodes = new Array(m), exitNodes = new Array(n), node, nodeData;
      if (key) {
        var nodeByKeyValue = new d3_Map(), keyValues = new Array(n), keyValue;
        for (i = -1; ++i < n; ) {
          if (node = group[i]) {
            if (nodeByKeyValue.has(keyValue = key.call(node, node.__data__, i))) {
              exitNodes[i] = node;
            } else {
              nodeByKeyValue.set(keyValue, node);
            }
            keyValues[i] = keyValue;
          }
        }
        for (i = -1; ++i < m; ) {
          if (!(node = nodeByKeyValue.get(keyValue = key.call(groupData, nodeData = groupData[i], i)))) {
            enterNodes[i] = d3_selection_dataNode(nodeData);
          } else if (node !== true) {
            updateNodes[i] = node;
            node.__data__ = nodeData;
          }
          nodeByKeyValue.set(keyValue, true);
        }
        for (i = -1; ++i < n; ) {
          if (i in keyValues && nodeByKeyValue.get(keyValues[i]) !== true) {
            exitNodes[i] = group[i];
          }
        }
      } else {
        for (i = -1; ++i < n0; ) {
          node = group[i];
          nodeData = groupData[i];
          if (node) {
            node.__data__ = nodeData;
            updateNodes[i] = node;
          } else {
            enterNodes[i] = d3_selection_dataNode(nodeData);
          }
        }
        for (;i < m; ++i) {
          enterNodes[i] = d3_selection_dataNode(groupData[i]);
        }
        for (;i < n; ++i) {
          exitNodes[i] = group[i];
        }
      }
      enterNodes.update = updateNodes;
      enterNodes.parentNode = updateNodes.parentNode = exitNodes.parentNode = group.parentNode;
      enter.push(enterNodes);
      update.push(updateNodes);
      exit.push(exitNodes);
    }
    var enter = d3_selection_enter([]), update = d3_selection([]), exit = d3_selection([]);
    if (typeof value === "function") {
      while (++i < n) {
        bind(group = this[i], value.call(group, group.parentNode.__data__, i));
      }
    } else {
      while (++i < n) {
        bind(group = this[i], value);
      }
    }
    update.enter = function() {
      return enter;
    };
    update.exit = function() {
      return exit;
    };
    return update;
  };
  function d3_selection_dataNode(data) {
    return {
      __data__: data
    };
  }
  d3_selectionPrototype.datum = function(value) {
    return arguments.length ? this.property("__data__", value) : this.property("__data__");
  };
  d3_selectionPrototype.filter = function(filter) {
    var subgroups = [], subgroup, group, node;
    if (typeof filter !== "function") filter = d3_selection_filter(filter);
    for (var j = 0, m = this.length; j < m; j++) {
      subgroups.push(subgroup = []);
      subgroup.parentNode = (group = this[j]).parentNode;
      for (var i = 0, n = group.length; i < n; i++) {
        if ((node = group[i]) && filter.call(node, node.__data__, i, j)) {
          subgroup.push(node);
        }
      }
    }
    return d3_selection(subgroups);
  };
  function d3_selection_filter(selector) {
    return function() {
      return d3_selectMatches(this, selector);
    };
  }
  d3_selectionPrototype.order = function() {
    for (var j = -1, m = this.length; ++j < m; ) {
      for (var group = this[j], i = group.length - 1, next = group[i], node; --i >= 0; ) {
        if (node = group[i]) {
          if (next && next !== node.nextSibling) next.parentNode.insertBefore(node, next);
          next = node;
        }
      }
    }
    return this;
  };
  d3_selectionPrototype.sort = function(comparator) {
    comparator = d3_selection_sortComparator.apply(this, arguments);
    for (var j = -1, m = this.length; ++j < m; ) this[j].sort(comparator);
    return this.order();
  };
  function d3_selection_sortComparator(comparator) {
    if (!arguments.length) comparator = d3_ascending;
    return function(a, b) {
      return a && b ? comparator(a.__data__, b.__data__) : !a - !b;
    };
  }
  d3_selectionPrototype.each = function(callback) {
    return d3_selection_each(this, function(node, i, j) {
      callback.call(node, node.__data__, i, j);
    });
  };
  function d3_selection_each(groups, callback) {
    for (var j = 0, m = groups.length; j < m; j++) {
      for (var group = groups[j], i = 0, n = group.length, node; i < n; i++) {
        if (node = group[i]) callback(node, i, j);
      }
    }
    return groups;
  }
  d3_selectionPrototype.call = function(callback) {
    var args = d3_array(arguments);
    callback.apply(args[0] = this, args);
    return this;
  };
  d3_selectionPrototype.empty = function() {
    return !this.node();
  };
  d3_selectionPrototype.node = function() {
    for (var j = 0, m = this.length; j < m; j++) {
      for (var group = this[j], i = 0, n = group.length; i < n; i++) {
        var node = group[i];
        if (node) return node;
      }
    }
    return null;
  };
  d3_selectionPrototype.size = function() {
    var n = 0;
    d3_selection_each(this, function() {
      ++n;
    });
    return n;
  };
  function d3_selection_enter(selection) {
    d3_subclass(selection, d3_selection_enterPrototype);
    return selection;
  }
  var d3_selection_enterPrototype = [];
  d3.selection.enter = d3_selection_enter;
  d3.selection.enter.prototype = d3_selection_enterPrototype;
  d3_selection_enterPrototype.append = d3_selectionPrototype.append;
  d3_selection_enterPrototype.empty = d3_selectionPrototype.empty;
  d3_selection_enterPrototype.node = d3_selectionPrototype.node;
  d3_selection_enterPrototype.call = d3_selectionPrototype.call;
  d3_selection_enterPrototype.size = d3_selectionPrototype.size;
  d3_selection_enterPrototype.select = function(selector) {
    var subgroups = [], subgroup, subnode, upgroup, group, node;
    for (var j = -1, m = this.length; ++j < m; ) {
      upgroup = (group = this[j]).update;
      subgroups.push(subgroup = []);
      subgroup.parentNode = group.parentNode;
      for (var i = -1, n = group.length; ++i < n; ) {
        if (node = group[i]) {
          subgroup.push(upgroup[i] = subnode = selector.call(group.parentNode, node.__data__, i, j));
          subnode.__data__ = node.__data__;
        } else {
          subgroup.push(null);
        }
      }
    }
    return d3_selection(subgroups);
  };
  d3_selection_enterPrototype.insert = function(name, before) {
    if (arguments.length < 2) before = d3_selection_enterInsertBefore(this);
    return d3_selectionPrototype.insert.call(this, name, before);
  };
  function d3_selection_enterInsertBefore(enter) {
    var i0, j0;
    return function(d, i, j) {
      var group = enter[j].update, n = group.length, node;
      if (j != j0) j0 = j, i0 = 0;
      if (i >= i0) i0 = i + 1;
      while (!(node = group[i0]) && ++i0 < n) ;
      return node;
    };
  }
  d3.select = function(node) {
    var group;
    if (typeof node === "string") {
      group = [ d3_select(node, d3_document) ];
      group.parentNode = d3_document.documentElement;
    } else {
      group = [ node ];
      group.parentNode = d3_documentElement(node);
    }
    return d3_selection([ group ]);
  };
  d3.selectAll = function(nodes) {
    var group;
    if (typeof nodes === "string") {
      group = d3_array(d3_selectAll(nodes, d3_document));
      group.parentNode = d3_document.documentElement;
    } else {
      group = d3_array(nodes);
      group.parentNode = null;
    }
    return d3_selection([ group ]);
  };
  d3_selectionPrototype.on = function(type, listener, capture) {
    var n = arguments.length;
    if (n < 3) {
      if (typeof type !== "string") {
        if (n < 2) listener = false;
        for (capture in type) this.each(d3_selection_on(capture, type[capture], listener));
        return this;
      }
      if (n < 2) return (n = this.node()["__on" + type]) && n._;
      capture = false;
    }
    return this.each(d3_selection_on(type, listener, capture));
  };
  function d3_selection_on(type, listener, capture) {
    var name = "__on" + type, i = type.indexOf("."), wrap = d3_selection_onListener;
    if (i > 0) type = type.slice(0, i);
    var filter = d3_selection_onFilters.get(type);
    if (filter) type = filter, wrap = d3_selection_onFilter;
    function onRemove() {
      var l = this[name];
      if (l) {
        this.removeEventListener(type, l, l.$);
        delete this[name];
      }
    }
    function onAdd() {
      var l = wrap(listener, d3_array(arguments));
      onRemove.call(this);
      this.addEventListener(type, this[name] = l, l.$ = capture);
      l._ = listener;
    }
    function removeAll() {
      var re = new RegExp("^__on([^.]+)" + d3.requote(type) + "$"), match;
      for (var name in this) {
        if (match = name.match(re)) {
          var l = this[name];
          this.removeEventListener(match[1], l, l.$);
          delete this[name];
        }
      }
    }
    return i ? listener ? onAdd : onRemove : listener ? d3_noop : removeAll;
  }
  var d3_selection_onFilters = d3.map({
    mouseenter: "mouseover",
    mouseleave: "mouseout"
  });
  if (d3_document) {
    d3_selection_onFilters.forEach(function(k) {
      if ("on" + k in d3_document) d3_selection_onFilters.remove(k);
    });
  }
  function d3_selection_onListener(listener, argumentz) {
    return function(e) {
      var o = d3.event;
      d3.event = e;
      argumentz[0] = this.__data__;
      try {
        listener.apply(this, argumentz);
      } finally {
        d3.event = o;
      }
    };
  }
  function d3_selection_onFilter(listener, argumentz) {
    var l = d3_selection_onListener(listener, argumentz);
    return function(e) {
      var target = this, related = e.relatedTarget;
      if (!related || related !== target && !(related.compareDocumentPosition(target) & 8)) {
        l.call(target, e);
      }
    };
  }
  var d3_event_dragSelect, d3_event_dragId = 0;
  function d3_event_dragSuppress(node) {
    var name = ".dragsuppress-" + ++d3_event_dragId, click = "click" + name, w = d3.select(d3_window(node)).on("touchmove" + name, d3_eventPreventDefault).on("dragstart" + name, d3_eventPreventDefault).on("selectstart" + name, d3_eventPreventDefault);
    if (d3_event_dragSelect == null) {
      d3_event_dragSelect = "onselectstart" in node ? false : d3_vendorSymbol(node.style, "userSelect");
    }
    if (d3_event_dragSelect) {
      var style = d3_documentElement(node).style, select = style[d3_event_dragSelect];
      style[d3_event_dragSelect] = "none";
    }
    return function(suppressClick) {
      w.on(name, null);
      if (d3_event_dragSelect) style[d3_event_dragSelect] = select;
      if (suppressClick) {
        var off = function() {
          w.on(click, null);
        };
        w.on(click, function() {
          d3_eventPreventDefault();
          off();
        }, true);
        setTimeout(off, 0);
      }
    };
  }
  d3.mouse = function(container) {
    return d3_mousePoint(container, d3_eventSource());
  };
  var d3_mouse_bug44083 = this.navigator && /WebKit/.test(this.navigator.userAgent) ? -1 : 0;
  function d3_mousePoint(container, e) {
    if (e.changedTouches) e = e.changedTouches[0];
    var svg = container.ownerSVGElement || container;
    if (svg.createSVGPoint) {
      var point = svg.createSVGPoint();
      if (d3_mouse_bug44083 < 0) {
        var window = d3_window(container);
        if (window.scrollX || window.scrollY) {
          svg = d3.select("body").append("svg").style({
            position: "absolute",
            top: 0,
            left: 0,
            margin: 0,
            padding: 0,
            border: "none"
          }, "important");
          var ctm = svg[0][0].getScreenCTM();
          d3_mouse_bug44083 = !(ctm.f || ctm.e);
          svg.remove();
        }
      }
      if (d3_mouse_bug44083) point.x = e.pageX, point.y = e.pageY; else point.x = e.clientX, 
      point.y = e.clientY;
      point = point.matrixTransform(container.getScreenCTM().inverse());
      return [ point.x, point.y ];
    }
    var rect = container.getBoundingClientRect();
    return [ e.clientX - rect.left - container.clientLeft, e.clientY - rect.top - container.clientTop ];
  }
  d3.touch = function(container, touches, identifier) {
    if (arguments.length < 3) identifier = touches, touches = d3_eventSource().changedTouches;
    if (touches) for (var i = 0, n = touches.length, touch; i < n; ++i) {
      if ((touch = touches[i]).identifier === identifier) {
        return d3_mousePoint(container, touch);
      }
    }
  };
  d3.behavior.drag = function() {
    var event = d3_eventDispatch(drag, "drag", "dragstart", "dragend"), origin = null, mousedown = dragstart(d3_noop, d3.mouse, d3_window, "mousemove", "mouseup"), touchstart = dragstart(d3_behavior_dragTouchId, d3.touch, d3_identity, "touchmove", "touchend");
    function drag() {
      this.on("mousedown.drag", mousedown).on("touchstart.drag", touchstart);
    }
    function dragstart(id, position, subject, move, end) {
      return function() {
        var that = this, target = d3.event.target.correspondingElement || d3.event.target, parent = that.parentNode, dispatch = event.of(that, arguments), dragged = 0, dragId = id(), dragName = ".drag" + (dragId == null ? "" : "-" + dragId), dragOffset, dragSubject = d3.select(subject(target)).on(move + dragName, moved).on(end + dragName, ended), dragRestore = d3_event_dragSuppress(target), position0 = position(parent, dragId);
        if (origin) {
          dragOffset = origin.apply(that, arguments);
          dragOffset = [ dragOffset.x - position0[0], dragOffset.y - position0[1] ];
        } else {
          dragOffset = [ 0, 0 ];
        }
        dispatch({
          type: "dragstart"
        });
        function moved() {
          var position1 = position(parent, dragId), dx, dy;
          if (!position1) return;
          dx = position1[0] - position0[0];
          dy = position1[1] - position0[1];
          dragged |= dx | dy;
          position0 = position1;
          dispatch({
            type: "drag",
            x: position1[0] + dragOffset[0],
            y: position1[1] + dragOffset[1],
            dx: dx,
            dy: dy
          });
        }
        function ended() {
          if (!position(parent, dragId)) return;
          dragSubject.on(move + dragName, null).on(end + dragName, null);
          dragRestore(dragged);
          dispatch({
            type: "dragend"
          });
        }
      };
    }
    drag.origin = function(x) {
      if (!arguments.length) return origin;
      origin = x;
      return drag;
    };
    return d3.rebind(drag, event, "on");
  };
  function d3_behavior_dragTouchId() {
    return d3.event.changedTouches[0].identifier;
  }
  d3.touches = function(container, touches) {
    if (arguments.length < 2) touches = d3_eventSource().touches;
    return touches ? d3_array(touches).map(function(touch) {
      var point = d3_mousePoint(container, touch);
      point.identifier = touch.identifier;
      return point;
    }) : [];
  };
  var ε = 1e-6, ε2 = ε * ε, π = Math.PI, τ = 2 * π, τε = τ - ε, halfπ = π / 2, d3_radians = π / 180, d3_degrees = 180 / π;
  function d3_sgn(x) {
    return x > 0 ? 1 : x < 0 ? -1 : 0;
  }
  function d3_cross2d(a, b, c) {
    return (b[0] - a[0]) * (c[1] - a[1]) - (b[1] - a[1]) * (c[0] - a[0]);
  }
  function d3_acos(x) {
    return x > 1 ? 0 : x < -1 ? π : Math.acos(x);
  }
  function d3_asin(x) {
    return x > 1 ? halfπ : x < -1 ? -halfπ : Math.asin(x);
  }
  function d3_sinh(x) {
    return ((x = Math.exp(x)) - 1 / x) / 2;
  }
  function d3_cosh(x) {
    return ((x = Math.exp(x)) + 1 / x) / 2;
  }
  function d3_tanh(x) {
    return ((x = Math.exp(2 * x)) - 1) / (x + 1);
  }
  function d3_haversin(x) {
    return (x = Math.sin(x / 2)) * x;
  }
  var ρ = Math.SQRT2, ρ2 = 2, ρ4 = 4;
  d3.interpolateZoom = function(p0, p1) {
    var ux0 = p0[0], uy0 = p0[1], w0 = p0[2], ux1 = p1[0], uy1 = p1[1], w1 = p1[2], dx = ux1 - ux0, dy = uy1 - uy0, d2 = dx * dx + dy * dy, i, S;
    if (d2 < ε2) {
      S = Math.log(w1 / w0) / ρ;
      i = function(t) {
        return [ ux0 + t * dx, uy0 + t * dy, w0 * Math.exp(ρ * t * S) ];
      };
    } else {
      var d1 = Math.sqrt(d2), b0 = (w1 * w1 - w0 * w0 + ρ4 * d2) / (2 * w0 * ρ2 * d1), b1 = (w1 * w1 - w0 * w0 - ρ4 * d2) / (2 * w1 * ρ2 * d1), r0 = Math.log(Math.sqrt(b0 * b0 + 1) - b0), r1 = Math.log(Math.sqrt(b1 * b1 + 1) - b1);
      S = (r1 - r0) / ρ;
      i = function(t) {
        var s = t * S, coshr0 = d3_cosh(r0), u = w0 / (ρ2 * d1) * (coshr0 * d3_tanh(ρ * s + r0) - d3_sinh(r0));
        return [ ux0 + u * dx, uy0 + u * dy, w0 * coshr0 / d3_cosh(ρ * s + r0) ];
      };
    }
    i.duration = S * 1e3;
    return i;
  };
  d3.behavior.zoom = function() {
    var view = {
      x: 0,
      y: 0,
      k: 1
    }, translate0, center0, center, size = [ 960, 500 ], scaleExtent = d3_behavior_zoomInfinity, duration = 250, zooming = 0, mousedown = "mousedown.zoom", mousemove = "mousemove.zoom", mouseup = "mouseup.zoom", mousewheelTimer, touchstart = "touchstart.zoom", touchtime, event = d3_eventDispatch(zoom, "zoomstart", "zoom", "zoomend"), x0, x1, y0, y1;
    if (!d3_behavior_zoomWheel) {
      d3_behavior_zoomWheel = "onwheel" in d3_document ? (d3_behavior_zoomDelta = function() {
        return -d3.event.deltaY * (d3.event.deltaMode ? 120 : 1);
      }, "wheel") : "onmousewheel" in d3_document ? (d3_behavior_zoomDelta = function() {
        return d3.event.wheelDelta;
      }, "mousewheel") : (d3_behavior_zoomDelta = function() {
        return -d3.event.detail;
      }, "MozMousePixelScroll");
    }
    function zoom(g) {
      g.on(mousedown, mousedowned).on(d3_behavior_zoomWheel + ".zoom", mousewheeled).on("dblclick.zoom", dblclicked).on(touchstart, touchstarted);
    }
    zoom.event = function(g) {
      g.each(function() {
        var dispatch = event.of(this, arguments), view1 = view;
        if (d3_transitionInheritId) {
          d3.select(this).transition().each("start.zoom", function() {
            view = this.__chart__ || {
              x: 0,
              y: 0,
              k: 1
            };
            zoomstarted(dispatch);
          }).tween("zoom:zoom", function() {
            var dx = size[0], dy = size[1], cx = center0 ? center0[0] : dx / 2, cy = center0 ? center0[1] : dy / 2, i = d3.interpolateZoom([ (cx - view.x) / view.k, (cy - view.y) / view.k, dx / view.k ], [ (cx - view1.x) / view1.k, (cy - view1.y) / view1.k, dx / view1.k ]);
            return function(t) {
              var l = i(t), k = dx / l[2];
              this.__chart__ = view = {
                x: cx - l[0] * k,
                y: cy - l[1] * k,
                k: k
              };
              zoomed(dispatch);
            };
          }).each("interrupt.zoom", function() {
            zoomended(dispatch);
          }).each("end.zoom", function() {
            zoomended(dispatch);
          });
        } else {
          this.__chart__ = view;
          zoomstarted(dispatch);
          zoomed(dispatch);
          zoomended(dispatch);
        }
      });
    };
    zoom.translate = function(_) {
      if (!arguments.length) return [ view.x, view.y ];
      view = {
        x: +_[0],
        y: +_[1],
        k: view.k
      };
      rescale();
      return zoom;
    };
    zoom.scale = function(_) {
      if (!arguments.length) return view.k;
      view = {
        x: view.x,
        y: view.y,
        k: null
      };
      scaleTo(+_);
      rescale();
      return zoom;
    };
    zoom.scaleExtent = function(_) {
      if (!arguments.length) return scaleExtent;
      scaleExtent = _ == null ? d3_behavior_zoomInfinity : [ +_[0], +_[1] ];
      return zoom;
    };
    zoom.center = function(_) {
      if (!arguments.length) return center;
      center = _ && [ +_[0], +_[1] ];
      return zoom;
    };
    zoom.size = function(_) {
      if (!arguments.length) return size;
      size = _ && [ +_[0], +_[1] ];
      return zoom;
    };
    zoom.duration = function(_) {
      if (!arguments.length) return duration;
      duration = +_;
      return zoom;
    };
    zoom.x = function(z) {
      if (!arguments.length) return x1;
      x1 = z;
      x0 = z.copy();
      view = {
        x: 0,
        y: 0,
        k: 1
      };
      return zoom;
    };
    zoom.y = function(z) {
      if (!arguments.length) return y1;
      y1 = z;
      y0 = z.copy();
      view = {
        x: 0,
        y: 0,
        k: 1
      };
      return zoom;
    };
    function location(p) {
      return [ (p[0] - view.x) / view.k, (p[1] - view.y) / view.k ];
    }
    function point(l) {
      return [ l[0] * view.k + view.x, l[1] * view.k + view.y ];
    }
    function scaleTo(s) {
      view.k = Math.max(scaleExtent[0], Math.min(scaleExtent[1], s));
    }
    function translateTo(p, l) {
      l = point(l);
      view.x += p[0] - l[0];
      view.y += p[1] - l[1];
    }
    function zoomTo(that, p, l, k) {
      that.__chart__ = {
        x: view.x,
        y: view.y,
        k: view.k
      };
      scaleTo(Math.pow(2, k));
      translateTo(center0 = p, l);
      that = d3.select(that);
      if (duration > 0) that = that.transition().duration(duration);
      that.call(zoom.event);
    }
    function rescale() {
      if (x1) x1.domain(x0.range().map(function(x) {
        return (x - view.x) / view.k;
      }).map(x0.invert));
      if (y1) y1.domain(y0.range().map(function(y) {
        return (y - view.y) / view.k;
      }).map(y0.invert));
    }
    function zoomstarted(dispatch) {
      if (!zooming++) dispatch({
        type: "zoomstart"
      });
    }
    function zoomed(dispatch) {
      rescale();
      dispatch({
        type: "zoom",
        scale: view.k,
        translate: [ view.x, view.y ]
      });
    }
    function zoomended(dispatch) {
      if (!--zooming) dispatch({
        type: "zoomend"
      }), center0 = null;
    }
    function mousedowned() {
      var that = this, dispatch = event.of(that, arguments), dragged = 0, subject = d3.select(d3_window(that)).on(mousemove, moved).on(mouseup, ended), location0 = location(d3.mouse(that)), dragRestore = d3_event_dragSuppress(that);
      d3_selection_interrupt.call(that);
      zoomstarted(dispatch);
      function moved() {
        dragged = 1;
        translateTo(d3.mouse(that), location0);
        zoomed(dispatch);
      }
      function ended() {
        subject.on(mousemove, null).on(mouseup, null);
        dragRestore(dragged);
        zoomended(dispatch);
      }
    }
    function touchstarted() {
      var that = this, dispatch = event.of(that, arguments), locations0 = {}, distance0 = 0, scale0, zoomName = ".zoom-" + d3.event.changedTouches[0].identifier, touchmove = "touchmove" + zoomName, touchend = "touchend" + zoomName, targets = [], subject = d3.select(that), dragRestore = d3_event_dragSuppress(that);
      started();
      zoomstarted(dispatch);
      subject.on(mousedown, null).on(touchstart, started);
      function relocate() {
        var touches = d3.touches(that);
        scale0 = view.k;
        touches.forEach(function(t) {
          if (t.identifier in locations0) locations0[t.identifier] = location(t);
        });
        return touches;
      }
      function started() {
        var target = d3.event.target;
        d3.select(target).on(touchmove, moved).on(touchend, ended);
        targets.push(target);
        var changed = d3.event.changedTouches;
        for (var i = 0, n = changed.length; i < n; ++i) {
          locations0[changed[i].identifier] = null;
        }
        var touches = relocate(), now = Date.now();
        if (touches.length === 1) {
          if (now - touchtime < 500) {
            var p = touches[0];
            zoomTo(that, p, locations0[p.identifier], Math.floor(Math.log(view.k) / Math.LN2) + 1);
            d3_eventPreventDefault();
          }
          touchtime = now;
        } else if (touches.length > 1) {
          var p = touches[0], q = touches[1], dx = p[0] - q[0], dy = p[1] - q[1];
          distance0 = dx * dx + dy * dy;
        }
      }
      function moved() {
        var touches = d3.touches(that), p0, l0, p1, l1;
        d3_selection_interrupt.call(that);
        for (var i = 0, n = touches.length; i < n; ++i, l1 = null) {
          p1 = touches[i];
          if (l1 = locations0[p1.identifier]) {
            if (l0) break;
            p0 = p1, l0 = l1;
          }
        }
        if (l1) {
          var distance1 = (distance1 = p1[0] - p0[0]) * distance1 + (distance1 = p1[1] - p0[1]) * distance1, scale1 = distance0 && Math.sqrt(distance1 / distance0);
          p0 = [ (p0[0] + p1[0]) / 2, (p0[1] + p1[1]) / 2 ];
          l0 = [ (l0[0] + l1[0]) / 2, (l0[1] + l1[1]) / 2 ];
          scaleTo(scale1 * scale0);
        }
        touchtime = null;
        translateTo(p0, l0);
        zoomed(dispatch);
      }
      function ended() {
        if (d3.event.touches.length) {
          var changed = d3.event.changedTouches;
          for (var i = 0, n = changed.length; i < n; ++i) {
            delete locations0[changed[i].identifier];
          }
          for (var identifier in locations0) {
            return void relocate();
          }
        }
        d3.selectAll(targets).on(zoomName, null);
        subject.on(mousedown, mousedowned).on(touchstart, touchstarted);
        dragRestore();
        zoomended(dispatch);
      }
    }
    function mousewheeled() {
      var dispatch = event.of(this, arguments);
      if (mousewheelTimer) clearTimeout(mousewheelTimer); else d3_selection_interrupt.call(this), 
      translate0 = location(center0 = center || d3.mouse(this)), zoomstarted(dispatch);
      mousewheelTimer = setTimeout(function() {
        mousewheelTimer = null;
        zoomended(dispatch);
      }, 50);
      d3_eventPreventDefault();
      scaleTo(Math.pow(2, d3_behavior_zoomDelta() * .002) * view.k);
      translateTo(center0, translate0);
      zoomed(dispatch);
    }
    function dblclicked() {
      var p = d3.mouse(this), k = Math.log(view.k) / Math.LN2;
      zoomTo(this, p, location(p), d3.event.shiftKey ? Math.ceil(k) - 1 : Math.floor(k) + 1);
    }
    return d3.rebind(zoom, event, "on");
  };
  var d3_behavior_zoomInfinity = [ 0, Infinity ], d3_behavior_zoomDelta, d3_behavior_zoomWheel;
  d3.color = d3_color;
  function d3_color() {}
  d3_color.prototype.toString = function() {
    return this.rgb() + "";
  };
  d3.hsl = d3_hsl;
  function d3_hsl(h, s, l) {
    return this instanceof d3_hsl ? void (this.h = +h, this.s = +s, this.l = +l) : arguments.length < 2 ? h instanceof d3_hsl ? new d3_hsl(h.h, h.s, h.l) : d3_rgb_parse("" + h, d3_rgb_hsl, d3_hsl) : new d3_hsl(h, s, l);
  }
  var d3_hslPrototype = d3_hsl.prototype = new d3_color();
  d3_hslPrototype.brighter = function(k) {
    k = Math.pow(.7, arguments.length ? k : 1);
    return new d3_hsl(this.h, this.s, this.l / k);
  };
  d3_hslPrototype.darker = function(k) {
    k = Math.pow(.7, arguments.length ? k : 1);
    return new d3_hsl(this.h, this.s, k * this.l);
  };
  d3_hslPrototype.rgb = function() {
    return d3_hsl_rgb(this.h, this.s, this.l);
  };
  function d3_hsl_rgb(h, s, l) {
    var m1, m2;
    h = isNaN(h) ? 0 : (h %= 360) < 0 ? h + 360 : h;
    s = isNaN(s) ? 0 : s < 0 ? 0 : s > 1 ? 1 : s;
    l = l < 0 ? 0 : l > 1 ? 1 : l;
    m2 = l <= .5 ? l * (1 + s) : l + s - l * s;
    m1 = 2 * l - m2;
    function v(h) {
      if (h > 360) h -= 360; else if (h < 0) h += 360;
      if (h < 60) return m1 + (m2 - m1) * h / 60;
      if (h < 180) return m2;
      if (h < 240) return m1 + (m2 - m1) * (240 - h) / 60;
      return m1;
    }
    function vv(h) {
      return Math.round(v(h) * 255);
    }
    return new d3_rgb(vv(h + 120), vv(h), vv(h - 120));
  }
  d3.hcl = d3_hcl;
  function d3_hcl(h, c, l) {
    return this instanceof d3_hcl ? void (this.h = +h, this.c = +c, this.l = +l) : arguments.length < 2 ? h instanceof d3_hcl ? new d3_hcl(h.h, h.c, h.l) : h instanceof d3_lab ? d3_lab_hcl(h.l, h.a, h.b) : d3_lab_hcl((h = d3_rgb_lab((h = d3.rgb(h)).r, h.g, h.b)).l, h.a, h.b) : new d3_hcl(h, c, l);
  }
  var d3_hclPrototype = d3_hcl.prototype = new d3_color();
  d3_hclPrototype.brighter = function(k) {
    return new d3_hcl(this.h, this.c, Math.min(100, this.l + d3_lab_K * (arguments.length ? k : 1)));
  };
  d3_hclPrototype.darker = function(k) {
    return new d3_hcl(this.h, this.c, Math.max(0, this.l - d3_lab_K * (arguments.length ? k : 1)));
  };
  d3_hclPrototype.rgb = function() {
    return d3_hcl_lab(this.h, this.c, this.l).rgb();
  };
  function d3_hcl_lab(h, c, l) {
    if (isNaN(h)) h = 0;
    if (isNaN(c)) c = 0;
    return new d3_lab(l, Math.cos(h *= d3_radians) * c, Math.sin(h) * c);
  }
  d3.lab = d3_lab;
  function d3_lab(l, a, b) {
    return this instanceof d3_lab ? void (this.l = +l, this.a = +a, this.b = +b) : arguments.length < 2 ? l instanceof d3_lab ? new d3_lab(l.l, l.a, l.b) : l instanceof d3_hcl ? d3_hcl_lab(l.h, l.c, l.l) : d3_rgb_lab((l = d3_rgb(l)).r, l.g, l.b) : new d3_lab(l, a, b);
  }
  var d3_lab_K = 18;
  var d3_lab_X = .95047, d3_lab_Y = 1, d3_lab_Z = 1.08883;
  var d3_labPrototype = d3_lab.prototype = new d3_color();
  d3_labPrototype.brighter = function(k) {
    return new d3_lab(Math.min(100, this.l + d3_lab_K * (arguments.length ? k : 1)), this.a, this.b);
  };
  d3_labPrototype.darker = function(k) {
    return new d3_lab(Math.max(0, this.l - d3_lab_K * (arguments.length ? k : 1)), this.a, this.b);
  };
  d3_labPrototype.rgb = function() {
    return d3_lab_rgb(this.l, this.a, this.b);
  };
  function d3_lab_rgb(l, a, b) {
    var y = (l + 16) / 116, x = y + a / 500, z = y - b / 200;
    x = d3_lab_xyz(x) * d3_lab_X;
    y = d3_lab_xyz(y) * d3_lab_Y;
    z = d3_lab_xyz(z) * d3_lab_Z;
    return new d3_rgb(d3_xyz_rgb(3.2404542 * x - 1.5371385 * y - .4985314 * z), d3_xyz_rgb(-.969266 * x + 1.8760108 * y + .041556 * z), d3_xyz_rgb(.0556434 * x - .2040259 * y + 1.0572252 * z));
  }
  function d3_lab_hcl(l, a, b) {
    return l > 0 ? new d3_hcl(Math.atan2(b, a) * d3_degrees, Math.sqrt(a * a + b * b), l) : new d3_hcl(NaN, NaN, l);
  }
  function d3_lab_xyz(x) {
    return x > .206893034 ? x * x * x : (x - 4 / 29) / 7.787037;
  }
  function d3_xyz_lab(x) {
    return x > .008856 ? Math.pow(x, 1 / 3) : 7.787037 * x + 4 / 29;
  }
  function d3_xyz_rgb(r) {
    return Math.round(255 * (r <= .00304 ? 12.92 * r : 1.055 * Math.pow(r, 1 / 2.4) - .055));
  }
  d3.rgb = d3_rgb;
  function d3_rgb(r, g, b) {
    return this instanceof d3_rgb ? void (this.r = ~~r, this.g = ~~g, this.b = ~~b) : arguments.length < 2 ? r instanceof d3_rgb ? new d3_rgb(r.r, r.g, r.b) : d3_rgb_parse("" + r, d3_rgb, d3_hsl_rgb) : new d3_rgb(r, g, b);
  }
  function d3_rgbNumber(value) {
    return new d3_rgb(value >> 16, value >> 8 & 255, value & 255);
  }
  function d3_rgbString(value) {
    return d3_rgbNumber(value) + "";
  }
  var d3_rgbPrototype = d3_rgb.prototype = new d3_color();
  d3_rgbPrototype.brighter = function(k) {
    k = Math.pow(.7, arguments.length ? k : 1);
    var r = this.r, g = this.g, b = this.b, i = 30;
    if (!r && !g && !b) return new d3_rgb(i, i, i);
    if (r && r < i) r = i;
    if (g && g < i) g = i;
    if (b && b < i) b = i;
    return new d3_rgb(Math.min(255, r / k), Math.min(255, g / k), Math.min(255, b / k));
  };
  d3_rgbPrototype.darker = function(k) {
    k = Math.pow(.7, arguments.length ? k : 1);
    return new d3_rgb(k * this.r, k * this.g, k * this.b);
  };
  d3_rgbPrototype.hsl = function() {
    return d3_rgb_hsl(this.r, this.g, this.b);
  };
  d3_rgbPrototype.toString = function() {
    return "#" + d3_rgb_hex(this.r) + d3_rgb_hex(this.g) + d3_rgb_hex(this.b);
  };
  function d3_rgb_hex(v) {
    return v < 16 ? "0" + Math.max(0, v).toString(16) : Math.min(255, v).toString(16);
  }
  function d3_rgb_parse(format, rgb, hsl) {
    var r = 0, g = 0, b = 0, m1, m2, color;
    m1 = /([a-z]+)\((.*)\)/.exec(format = format.toLowerCase());
    if (m1) {
      m2 = m1[2].split(",");
      switch (m1[1]) {
       case "hsl":
        {
          return hsl(parseFloat(m2[0]), parseFloat(m2[1]) / 100, parseFloat(m2[2]) / 100);
        }

       case "rgb":
        {
          return rgb(d3_rgb_parseNumber(m2[0]), d3_rgb_parseNumber(m2[1]), d3_rgb_parseNumber(m2[2]));
        }
      }
    }
    if (color = d3_rgb_names.get(format)) {
      return rgb(color.r, color.g, color.b);
    }
    if (format != null && format.charAt(0) === "#" && !isNaN(color = parseInt(format.slice(1), 16))) {
      if (format.length === 4) {
        r = (color & 3840) >> 4;
        r = r >> 4 | r;
        g = color & 240;
        g = g >> 4 | g;
        b = color & 15;
        b = b << 4 | b;
      } else if (format.length === 7) {
        r = (color & 16711680) >> 16;
        g = (color & 65280) >> 8;
        b = color & 255;
      }
    }
    return rgb(r, g, b);
  }
  function d3_rgb_hsl(r, g, b) {
    var min = Math.min(r /= 255, g /= 255, b /= 255), max = Math.max(r, g, b), d = max - min, h, s, l = (max + min) / 2;
    if (d) {
      s = l < .5 ? d / (max + min) : d / (2 - max - min);
      if (r == max) h = (g - b) / d + (g < b ? 6 : 0); else if (g == max) h = (b - r) / d + 2; else h = (r - g) / d + 4;
      h *= 60;
    } else {
      h = NaN;
      s = l > 0 && l < 1 ? 0 : h;
    }
    return new d3_hsl(h, s, l);
  }
  function d3_rgb_lab(r, g, b) {
    r = d3_rgb_xyz(r);
    g = d3_rgb_xyz(g);
    b = d3_rgb_xyz(b);
    var x = d3_xyz_lab((.4124564 * r + .3575761 * g + .1804375 * b) / d3_lab_X), y = d3_xyz_lab((.2126729 * r + .7151522 * g + .072175 * b) / d3_lab_Y), z = d3_xyz_lab((.0193339 * r + .119192 * g + .9503041 * b) / d3_lab_Z);
    return d3_lab(116 * y - 16, 500 * (x - y), 200 * (y - z));
  }
  function d3_rgb_xyz(r) {
    return (r /= 255) <= .04045 ? r / 12.92 : Math.pow((r + .055) / 1.055, 2.4);
  }
  function d3_rgb_parseNumber(c) {
    var f = parseFloat(c);
    return c.charAt(c.length - 1) === "%" ? Math.round(f * 2.55) : f;
  }
  var d3_rgb_names = d3.map({
    aliceblue: 15792383,
    antiquewhite: 16444375,
    aqua: 65535,
    aquamarine: 8388564,
    azure: 15794175,
    beige: 16119260,
    bisque: 16770244,
    black: 0,
    blanchedalmond: 16772045,
    blue: 255,
    blueviolet: 9055202,
    brown: 10824234,
    burlywood: 14596231,
    cadetblue: 6266528,
    chartreuse: 8388352,
    chocolate: 13789470,
    coral: 16744272,
    cornflowerblue: 6591981,
    cornsilk: 16775388,
    crimson: 14423100,
    cyan: 65535,
    darkblue: 139,
    darkcyan: 35723,
    darkgoldenrod: 12092939,
    darkgray: 11119017,
    darkgreen: 25600,
    darkgrey: 11119017,
    darkkhaki: 12433259,
    darkmagenta: 9109643,
    darkolivegreen: 5597999,
    darkorange: 16747520,
    darkorchid: 10040012,
    darkred: 9109504,
    darksalmon: 15308410,
    darkseagreen: 9419919,
    darkslateblue: 4734347,
    darkslategray: 3100495,
    darkslategrey: 3100495,
    darkturquoise: 52945,
    darkviolet: 9699539,
    deeppink: 16716947,
    deepskyblue: 49151,
    dimgray: 6908265,
    dimgrey: 6908265,
    dodgerblue: 2003199,
    firebrick: 11674146,
    floralwhite: 16775920,
    forestgreen: 2263842,
    fuchsia: 16711935,
    gainsboro: 14474460,
    ghostwhite: 16316671,
    gold: 16766720,
    goldenrod: 14329120,
    gray: 8421504,
    green: 32768,
    greenyellow: 11403055,
    grey: 8421504,
    honeydew: 15794160,
    hotpink: 16738740,
    indianred: 13458524,
    indigo: 4915330,
    ivory: 16777200,
    khaki: 15787660,
    lavender: 15132410,
    lavenderblush: 16773365,
    lawngreen: 8190976,
    lemonchiffon: 16775885,
    lightblue: 11393254,
    lightcoral: 15761536,
    lightcyan: 14745599,
    lightgoldenrodyellow: 16448210,
    lightgray: 13882323,
    lightgreen: 9498256,
    lightgrey: 13882323,
    lightpink: 16758465,
    lightsalmon: 16752762,
    lightseagreen: 2142890,
    lightskyblue: 8900346,
    lightslategray: 7833753,
    lightslategrey: 7833753,
    lightsteelblue: 11584734,
    lightyellow: 16777184,
    lime: 65280,
    limegreen: 3329330,
    linen: 16445670,
    magenta: 16711935,
    maroon: 8388608,
    mediumaquamarine: 6737322,
    mediumblue: 205,
    mediumorchid: 12211667,
    mediumpurple: 9662683,
    mediumseagreen: 3978097,
    mediumslateblue: 8087790,
    mediumspringgreen: 64154,
    mediumturquoise: 4772300,
    mediumvioletred: 13047173,
    midnightblue: 1644912,
    mintcream: 16121850,
    mistyrose: 16770273,
    moccasin: 16770229,
    navajowhite: 16768685,
    navy: 128,
    oldlace: 16643558,
    olive: 8421376,
    olivedrab: 7048739,
    orange: 16753920,
    orangered: 16729344,
    orchid: 14315734,
    palegoldenrod: 15657130,
    palegreen: 10025880,
    paleturquoise: 11529966,
    palevioletred: 14381203,
    papayawhip: 16773077,
    peachpuff: 16767673,
    peru: 13468991,
    pink: 16761035,
    plum: 14524637,
    powderblue: 11591910,
    purple: 8388736,
    rebeccapurple: 6697881,
    red: 16711680,
    rosybrown: 12357519,
    royalblue: 4286945,
    saddlebrown: 9127187,
    salmon: 16416882,
    sandybrown: 16032864,
    seagreen: 3050327,
    seashell: 16774638,
    sienna: 10506797,
    silver: 12632256,
    skyblue: 8900331,
    slateblue: 6970061,
    slategray: 7372944,
    slategrey: 7372944,
    snow: 16775930,
    springgreen: 65407,
    steelblue: 4620980,
    tan: 13808780,
    teal: 32896,
    thistle: 14204888,
    tomato: 16737095,
    turquoise: 4251856,
    violet: 15631086,
    wheat: 16113331,
    white: 16777215,
    whitesmoke: 16119285,
    yellow: 16776960,
    yellowgreen: 10145074
  });
  d3_rgb_names.forEach(function(key, value) {
    d3_rgb_names.set(key, d3_rgbNumber(value));
  });
  function d3_functor(v) {
    return typeof v === "function" ? v : function() {
      return v;
    };
  }
  d3.functor = d3_functor;
  d3.xhr = d3_xhrType(d3_identity);
  function d3_xhrType(response) {
    return function(url, mimeType, callback) {
      if (arguments.length === 2 && typeof mimeType === "function") callback = mimeType, 
      mimeType = null;
      return d3_xhr(url, mimeType, response, callback);
    };
  }
  function d3_xhr(url, mimeType, response, callback) {
    var xhr = {}, dispatch = d3.dispatch("beforesend", "progress", "load", "error"), headers = {}, request = new XMLHttpRequest(), responseType = null;
    if (this.XDomainRequest && !("withCredentials" in request) && /^(http(s)?:)?\/\//.test(url)) request = new XDomainRequest();
    "onload" in request ? request.onload = request.onerror = respond : request.onreadystatechange = function() {
      request.readyState > 3 && respond();
    };
    function respond() {
      var status = request.status, result;
      if (!status && d3_xhrHasResponse(request) || status >= 200 && status < 300 || status === 304) {
        try {
          result = response.call(xhr, request);
        } catch (e) {
          dispatch.error.call(xhr, e);
          return;
        }
        dispatch.load.call(xhr, result);
      } else {
        dispatch.error.call(xhr, request);
      }
    }
    request.onprogress = function(event) {
      var o = d3.event;
      d3.event = event;
      try {
        dispatch.progress.call(xhr, request);
      } finally {
        d3.event = o;
      }
    };
    xhr.header = function(name, value) {
      name = (name + "").toLowerCase();
      if (arguments.length < 2) return headers[name];
      if (value == null) delete headers[name]; else headers[name] = value + "";
      return xhr;
    };
    xhr.mimeType = function(value) {
      if (!arguments.length) return mimeType;
      mimeType = value == null ? null : value + "";
      return xhr;
    };
    xhr.responseType = function(value) {
      if (!arguments.length) return responseType;
      responseType = value;
      return xhr;
    };
    xhr.response = function(value) {
      response = value;
      return xhr;
    };
    [ "get", "post" ].forEach(function(method) {
      xhr[method] = function() {
        return xhr.send.apply(xhr, [ method ].concat(d3_array(arguments)));
      };
    });
    xhr.send = function(method, data, callback) {
      if (arguments.length === 2 && typeof data === "function") callback = data, data = null;
      request.open(method, url, true);
      if (mimeType != null && !("accept" in headers)) headers["accept"] = mimeType + ",*/*";
      if (request.setRequestHeader) for (var name in headers) request.setRequestHeader(name, headers[name]);
      if (mimeType != null && request.overrideMimeType) request.overrideMimeType(mimeType);
      if (responseType != null) request.responseType = responseType;
      if (callback != null) xhr.on("error", callback).on("load", function(request) {
        callback(null, request);
      });
      dispatch.beforesend.call(xhr, request);
      request.send(data == null ? null : data);
      return xhr;
    };
    xhr.abort = function() {
      request.abort();
      return xhr;
    };
    d3.rebind(xhr, dispatch, "on");
    return callback == null ? xhr : xhr.get(d3_xhr_fixCallback(callback));
  }
  function d3_xhr_fixCallback(callback) {
    return callback.length === 1 ? function(error, request) {
      callback(error == null ? request : null);
    } : callback;
  }
  function d3_xhrHasResponse(request) {
    var type = request.responseType;
    return type && type !== "text" ? request.response : request.responseText;
  }
  d3.dsv = function(delimiter, mimeType) {
    var reFormat = new RegExp('["' + delimiter + "\n]"), delimiterCode = delimiter.charCodeAt(0);
    function dsv(url, row, callback) {
      if (arguments.length < 3) callback = row, row = null;
      var xhr = d3_xhr(url, mimeType, row == null ? response : typedResponse(row), callback);
      xhr.row = function(_) {
        return arguments.length ? xhr.response((row = _) == null ? response : typedResponse(_)) : row;
      };
      return xhr;
    }
    function response(request) {
      return dsv.parse(request.responseText);
    }
    function typedResponse(f) {
      return function(request) {
        return dsv.parse(request.responseText, f);
      };
    }
    dsv.parse = function(text, f) {
      var o;
      return dsv.parseRows(text, function(row, i) {
        if (o) return o(row, i - 1);
        var a = new Function("d", "return {" + row.map(function(name, i) {
          return JSON.stringify(name) + ": d[" + i + "]";
        }).join(",") + "}");
        o = f ? function(row, i) {
          return f(a(row), i);
        } : a;
      });
    };
    dsv.parseRows = function(text, f) {
      var EOL = {}, EOF = {}, rows = [], N = text.length, I = 0, n = 0, t, eol;
      function token() {
        if (I >= N) return EOF;
        if (eol) return eol = false, EOL;
        var j = I;
        if (text.charCodeAt(j) === 34) {
          var i = j;
          while (i++ < N) {
            if (text.charCodeAt(i) === 34) {
              if (text.charCodeAt(i + 1) !== 34) break;
              ++i;
            }
          }
          I = i + 2;
          var c = text.charCodeAt(i + 1);
          if (c === 13) {
            eol = true;
            if (text.charCodeAt(i + 2) === 10) ++I;
          } else if (c === 10) {
            eol = true;
          }
          return text.slice(j + 1, i).replace(/""/g, '"');
        }
        while (I < N) {
          var c = text.charCodeAt(I++), k = 1;
          if (c === 10) eol = true; else if (c === 13) {
            eol = true;
            if (text.charCodeAt(I) === 10) ++I, ++k;
          } else if (c !== delimiterCode) continue;
          return text.slice(j, I - k);
        }
        return text.slice(j);
      }
      while ((t = token()) !== EOF) {
        var a = [];
        while (t !== EOL && t !== EOF) {
          a.push(t);
          t = token();
        }
        if (f && (a = f(a, n++)) == null) continue;
        rows.push(a);
      }
      return rows;
    };
    dsv.format = function(rows) {
      if (Array.isArray(rows[0])) return dsv.formatRows(rows);
      var fieldSet = new d3_Set(), fields = [];
      rows.forEach(function(row) {
        for (var field in row) {
          if (!fieldSet.has(field)) {
            fields.push(fieldSet.add(field));
          }
        }
      });
      return [ fields.map(formatValue).join(delimiter) ].concat(rows.map(function(row) {
        return fields.map(function(field) {
          return formatValue(row[field]);
        }).join(delimiter);
      })).join("\n");
    };
    dsv.formatRows = function(rows) {
      return rows.map(formatRow).join("\n");
    };
    function formatRow(row) {
      return row.map(formatValue).join(delimiter);
    }
    function formatValue(text) {
      return reFormat.test(text) ? '"' + text.replace(/\"/g, '""') + '"' : text;
    }
    return dsv;
  };
  d3.csv = d3.dsv(",", "text/csv");
  d3.tsv = d3.dsv("	", "text/tab-separated-values");
  var d3_timer_queueHead, d3_timer_queueTail, d3_timer_interval, d3_timer_timeout, d3_timer_frame = this[d3_vendorSymbol(this, "requestAnimationFrame")] || function(callback) {
    setTimeout(callback, 17);
  };
  d3.timer = function() {
    d3_timer.apply(this, arguments);
  };
  function d3_timer(callback, delay, then) {
    var n = arguments.length;
    if (n < 2) delay = 0;
    if (n < 3) then = Date.now();
    var time = then + delay, timer = {
      c: callback,
      t: time,
      n: null
    };
    if (d3_timer_queueTail) d3_timer_queueTail.n = timer; else d3_timer_queueHead = timer;
    d3_timer_queueTail = timer;
    if (!d3_timer_interval) {
      d3_timer_timeout = clearTimeout(d3_timer_timeout);
      d3_timer_interval = 1;
      d3_timer_frame(d3_timer_step);
    }
    return timer;
  }
  function d3_timer_step() {
    var now = d3_timer_mark(), delay = d3_timer_sweep() - now;
    if (delay > 24) {
      if (isFinite(delay)) {
        clearTimeout(d3_timer_timeout);
        d3_timer_timeout = setTimeout(d3_timer_step, delay);
      }
      d3_timer_interval = 0;
    } else {
      d3_timer_interval = 1;
      d3_timer_frame(d3_timer_step);
    }
  }
  d3.timer.flush = function() {
    d3_timer_mark();
    d3_timer_sweep();
  };
  function d3_timer_mark() {
    var now = Date.now(), timer = d3_timer_queueHead;
    while (timer) {
      if (now >= timer.t && timer.c(now - timer.t)) timer.c = null;
      timer = timer.n;
    }
    return now;
  }
  function d3_timer_sweep() {
    var t0, t1 = d3_timer_queueHead, time = Infinity;
    while (t1) {
      if (t1.c) {
        if (t1.t < time) time = t1.t;
        t1 = (t0 = t1).n;
      } else {
        t1 = t0 ? t0.n = t1.n : d3_timer_queueHead = t1.n;
      }
    }
    d3_timer_queueTail = t0;
    return time;
  }
  function d3_format_precision(x, p) {
    return p - (x ? Math.ceil(Math.log(x) / Math.LN10) : 1);
  }
  d3.round = function(x, n) {
    return n ? Math.round(x * (n = Math.pow(10, n))) / n : Math.round(x);
  };
  var d3_formatPrefixes = [ "y", "z", "a", "f", "p", "n", "µ", "m", "", "k", "M", "G", "T", "P", "E", "Z", "Y" ].map(d3_formatPrefix);
  d3.formatPrefix = function(value, precision) {
    var i = 0;
    if (value = +value) {
      if (value < 0) value *= -1;
      if (precision) value = d3.round(value, d3_format_precision(value, precision));
      i = 1 + Math.floor(1e-12 + Math.log(value) / Math.LN10);
      i = Math.max(-24, Math.min(24, Math.floor((i - 1) / 3) * 3));
    }
    return d3_formatPrefixes[8 + i / 3];
  };
  function d3_formatPrefix(d, i) {
    var k = Math.pow(10, abs(8 - i) * 3);
    return {
      scale: i > 8 ? function(d) {
        return d / k;
      } : function(d) {
        return d * k;
      },
      symbol: d
    };
  }
  function d3_locale_numberFormat(locale) {
    var locale_decimal = locale.decimal, locale_thousands = locale.thousands, locale_grouping = locale.grouping, locale_currency = locale.currency, formatGroup = locale_grouping && locale_thousands ? function(value, width) {
      var i = value.length, t = [], j = 0, g = locale_grouping[0], length = 0;
      while (i > 0 && g > 0) {
        if (length + g + 1 > width) g = Math.max(1, width - length);
        t.push(value.substring(i -= g, i + g));
        if ((length += g + 1) > width) break;
        g = locale_grouping[j = (j + 1) % locale_grouping.length];
      }
      return t.reverse().join(locale_thousands);
    } : d3_identity;
    return function(specifier) {
      var match = d3_format_re.exec(specifier), fill = match[1] || " ", align = match[2] || ">", sign = match[3] || "-", symbol = match[4] || "", zfill = match[5], width = +match[6], comma = match[7], precision = match[8], type = match[9], scale = 1, prefix = "", suffix = "", integer = false, exponent = true;
      if (precision) precision = +precision.substring(1);
      if (zfill || fill === "0" && align === "=") {
        zfill = fill = "0";
        align = "=";
      }
      switch (type) {
       case "n":
        comma = true;
        type = "g";
        break;

       case "%":
        scale = 100;
        suffix = "%";
        type = "f";
        break;

       case "p":
        scale = 100;
        suffix = "%";
        type = "r";
        break;

       case "b":
       case "o":
       case "x":
       case "X":
        if (symbol === "#") prefix = "0" + type.toLowerCase();

       case "c":
        exponent = false;

       case "d":
        integer = true;
        precision = 0;
        break;

       case "s":
        scale = -1;
        type = "r";
        break;
      }
      if (symbol === "$") prefix = locale_currency[0], suffix = locale_currency[1];
      if (type == "r" && !precision) type = "g";
      if (precision != null) {
        if (type == "g") precision = Math.max(1, Math.min(21, precision)); else if (type == "e" || type == "f") precision = Math.max(0, Math.min(20, precision));
      }
      type = d3_format_types.get(type) || d3_format_typeDefault;
      var zcomma = zfill && comma;
      return function(value) {
        var fullSuffix = suffix;
        if (integer && value % 1) return "";
        var negative = value < 0 || value === 0 && 1 / value < 0 ? (value = -value, "-") : sign === "-" ? "" : sign;
        if (scale < 0) {
          var unit = d3.formatPrefix(value, precision);
          value = unit.scale(value);
          fullSuffix = unit.symbol + suffix;
        } else {
          value *= scale;
        }
        value = type(value, precision);
        var i = value.lastIndexOf("."), before, after;
        if (i < 0) {
          var j = exponent ? value.lastIndexOf("e") : -1;
          if (j < 0) before = value, after = ""; else before = value.substring(0, j), after = value.substring(j);
        } else {
          before = value.substring(0, i);
          after = locale_decimal + value.substring(i + 1);
        }
        if (!zfill && comma) before = formatGroup(before, Infinity);
        var length = prefix.length + before.length + after.length + (zcomma ? 0 : negative.length), padding = length < width ? new Array(length = width - length + 1).join(fill) : "";
        if (zcomma) before = formatGroup(padding + before, padding.length ? width - after.length : Infinity);
        negative += prefix;
        value = before + after;
        return (align === "<" ? negative + value + padding : align === ">" ? padding + negative + value : align === "^" ? padding.substring(0, length >>= 1) + negative + value + padding.substring(length) : negative + (zcomma ? value : padding + value)) + fullSuffix;
      };
    };
  }
  var d3_format_re = /(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i;
  var d3_format_types = d3.map({
    b: function(x) {
      return x.toString(2);
    },
    c: function(x) {
      return String.fromCharCode(x);
    },
    o: function(x) {
      return x.toString(8);
    },
    x: function(x) {
      return x.toString(16);
    },
    X: function(x) {
      return x.toString(16).toUpperCase();
    },
    g: function(x, p) {
      return x.toPrecision(p);
    },
    e: function(x, p) {
      return x.toExponential(p);
    },
    f: function(x, p) {
      return x.toFixed(p);
    },
    r: function(x, p) {
      return (x = d3.round(x, d3_format_precision(x, p))).toFixed(Math.max(0, Math.min(20, d3_format_precision(x * (1 + 1e-15), p))));
    }
  });
  function d3_format_typeDefault(x) {
    return x + "";
  }
  var d3_time = d3.time = {}, d3_date = Date;
  function d3_date_utc() {
    this._ = new Date(arguments.length > 1 ? Date.UTC.apply(this, arguments) : arguments[0]);
  }
  d3_date_utc.prototype = {
    getDate: function() {
      return this._.getUTCDate();
    },
    getDay: function() {
      return this._.getUTCDay();
    },
    getFullYear: function() {
      return this._.getUTCFullYear();
    },
    getHours: function() {
      return this._.getUTCHours();
    },
    getMilliseconds: function() {
      return this._.getUTCMilliseconds();
    },
    getMinutes: function() {
      return this._.getUTCMinutes();
    },
    getMonth: function() {
      return this._.getUTCMonth();
    },
    getSeconds: function() {
      return this._.getUTCSeconds();
    },
    getTime: function() {
      return this._.getTime();
    },
    getTimezoneOffset: function() {
      return 0;
    },
    valueOf: function() {
      return this._.valueOf();
    },
    setDate: function() {
      d3_time_prototype.setUTCDate.apply(this._, arguments);
    },
    setDay: function() {
      d3_time_prototype.setUTCDay.apply(this._, arguments);
    },
    setFullYear: function() {
      d3_time_prototype.setUTCFullYear.apply(this._, arguments);
    },
    setHours: function() {
      d3_time_prototype.setUTCHours.apply(this._, arguments);
    },
    setMilliseconds: function() {
      d3_time_prototype.setUTCMilliseconds.apply(this._, arguments);
    },
    setMinutes: function() {
      d3_time_prototype.setUTCMinutes.apply(this._, arguments);
    },
    setMonth: function() {
      d3_time_prototype.setUTCMonth.apply(this._, arguments);
    },
    setSeconds: function() {
      d3_time_prototype.setUTCSeconds.apply(this._, arguments);
    },
    setTime: function() {
      d3_time_prototype.setTime.apply(this._, arguments);
    }
  };
  var d3_time_prototype = Date.prototype;
  function d3_time_interval(local, step, number) {
    function round(date) {
      var d0 = local(date), d1 = offset(d0, 1);
      return date - d0 < d1 - date ? d0 : d1;
    }
    function ceil(date) {
      step(date = local(new d3_date(date - 1)), 1);
      return date;
    }
    function offset(date, k) {
      step(date = new d3_date(+date), k);
      return date;
    }
    function range(t0, t1, dt) {
      var time = ceil(t0), times = [];
      if (dt > 1) {
        while (time < t1) {
          if (!(number(time) % dt)) times.push(new Date(+time));
          step(time, 1);
        }
      } else {
        while (time < t1) times.push(new Date(+time)), step(time, 1);
      }
      return times;
    }
    function range_utc(t0, t1, dt) {
      try {
        d3_date = d3_date_utc;
        var utc = new d3_date_utc();
        utc._ = t0;
        return range(utc, t1, dt);
      } finally {
        d3_date = Date;
      }
    }
    local.floor = local;
    local.round = round;
    local.ceil = ceil;
    local.offset = offset;
    local.range = range;
    var utc = local.utc = d3_time_interval_utc(local);
    utc.floor = utc;
    utc.round = d3_time_interval_utc(round);
    utc.ceil = d3_time_interval_utc(ceil);
    utc.offset = d3_time_interval_utc(offset);
    utc.range = range_utc;
    return local;
  }
  function d3_time_interval_utc(method) {
    return function(date, k) {
      try {
        d3_date = d3_date_utc;
        var utc = new d3_date_utc();
        utc._ = date;
        return method(utc, k)._;
      } finally {
        d3_date = Date;
      }
    };
  }
  d3_time.year = d3_time_interval(function(date) {
    date = d3_time.day(date);
    date.setMonth(0, 1);
    return date;
  }, function(date, offset) {
    date.setFullYear(date.getFullYear() + offset);
  }, function(date) {
    return date.getFullYear();
  });
  d3_time.years = d3_time.year.range;
  d3_time.years.utc = d3_time.year.utc.range;
  d3_time.day = d3_time_interval(function(date) {
    var day = new d3_date(2e3, 0);
    day.setFullYear(date.getFullYear(), date.getMonth(), date.getDate());
    return day;
  }, function(date, offset) {
    date.setDate(date.getDate() + offset);
  }, function(date) {
    return date.getDate() - 1;
  });
  d3_time.days = d3_time.day.range;
  d3_time.days.utc = d3_time.day.utc.range;
  d3_time.dayOfYear = function(date) {
    var year = d3_time.year(date);
    return Math.floor((date - year - (date.getTimezoneOffset() - year.getTimezoneOffset()) * 6e4) / 864e5);
  };
  [ "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday" ].forEach(function(day, i) {
    i = 7 - i;
    var interval = d3_time[day] = d3_time_interval(function(date) {
      (date = d3_time.day(date)).setDate(date.getDate() - (date.getDay() + i) % 7);
      return date;
    }, function(date, offset) {
      date.setDate(date.getDate() + Math.floor(offset) * 7);
    }, function(date) {
      var day = d3_time.year(date).getDay();
      return Math.floor((d3_time.dayOfYear(date) + (day + i) % 7) / 7) - (day !== i);
    });
    d3_time[day + "s"] = interval.range;
    d3_time[day + "s"].utc = interval.utc.range;
    d3_time[day + "OfYear"] = function(date) {
      var day = d3_time.year(date).getDay();
      return Math.floor((d3_time.dayOfYear(date) + (day + i) % 7) / 7);
    };
  });
  d3_time.week = d3_time.sunday;
  d3_time.weeks = d3_time.sunday.range;
  d3_time.weeks.utc = d3_time.sunday.utc.range;
  d3_time.weekOfYear = d3_time.sundayOfYear;
  function d3_locale_timeFormat(locale) {
    var locale_dateTime = locale.dateTime, locale_date = locale.date, locale_time = locale.time, locale_periods = locale.periods, locale_days = locale.days, locale_shortDays = locale.shortDays, locale_months = locale.months, locale_shortMonths = locale.shortMonths;
    function d3_time_format(template) {
      var n = template.length;
      function format(date) {
        var string = [], i = -1, j = 0, c, p, f;
        while (++i < n) {
          if (template.charCodeAt(i) === 37) {
            string.push(template.slice(j, i));
            if ((p = d3_time_formatPads[c = template.charAt(++i)]) != null) c = template.charAt(++i);
            if (f = d3_time_formats[c]) c = f(date, p == null ? c === "e" ? " " : "0" : p);
            string.push(c);
            j = i + 1;
          }
        }
        string.push(template.slice(j, i));
        return string.join("");
      }
      format.parse = function(string) {
        var d = {
          y: 1900,
          m: 0,
          d: 1,
          H: 0,
          M: 0,
          S: 0,
          L: 0,
          Z: null
        }, i = d3_time_parse(d, template, string, 0);
        if (i != string.length) return null;
        if ("p" in d) d.H = d.H % 12 + d.p * 12;
        var localZ = d.Z != null && d3_date !== d3_date_utc, date = new (localZ ? d3_date_utc : d3_date)();
        if ("j" in d) date.setFullYear(d.y, 0, d.j); else if ("W" in d || "U" in d) {
          if (!("w" in d)) d.w = "W" in d ? 1 : 0;
          date.setFullYear(d.y, 0, 1);
          date.setFullYear(d.y, 0, "W" in d ? (d.w + 6) % 7 + d.W * 7 - (date.getDay() + 5) % 7 : d.w + d.U * 7 - (date.getDay() + 6) % 7);
        } else date.setFullYear(d.y, d.m, d.d);
        date.setHours(d.H + (d.Z / 100 | 0), d.M + d.Z % 100, d.S, d.L);
        return localZ ? date._ : date;
      };
      format.toString = function() {
        return template;
      };
      return format;
    }
    function d3_time_parse(date, template, string, j) {
      var c, p, t, i = 0, n = template.length, m = string.length;
      while (i < n) {
        if (j >= m) return -1;
        c = template.charCodeAt(i++);
        if (c === 37) {
          t = template.charAt(i++);
          p = d3_time_parsers[t in d3_time_formatPads ? template.charAt(i++) : t];
          if (!p || (j = p(date, string, j)) < 0) return -1;
        } else if (c != string.charCodeAt(j++)) {
          return -1;
        }
      }
      return j;
    }
    d3_time_format.utc = function(template) {
      var local = d3_time_format(template);
      function format(date) {
        try {
          d3_date = d3_date_utc;
          var utc = new d3_date();
          utc._ = date;
          return local(utc);
        } finally {
          d3_date = Date;
        }
      }
      format.parse = function(string) {
        try {
          d3_date = d3_date_utc;
          var date = local.parse(string);
          return date && date._;
        } finally {
          d3_date = Date;
        }
      };
      format.toString = local.toString;
      return format;
    };
    d3_time_format.multi = d3_time_format.utc.multi = d3_time_formatMulti;
    var d3_time_periodLookup = d3.map(), d3_time_dayRe = d3_time_formatRe(locale_days), d3_time_dayLookup = d3_time_formatLookup(locale_days), d3_time_dayAbbrevRe = d3_time_formatRe(locale_shortDays), d3_time_dayAbbrevLookup = d3_time_formatLookup(locale_shortDays), d3_time_monthRe = d3_time_formatRe(locale_months), d3_time_monthLookup = d3_time_formatLookup(locale_months), d3_time_monthAbbrevRe = d3_time_formatRe(locale_shortMonths), d3_time_monthAbbrevLookup = d3_time_formatLookup(locale_shortMonths);
    locale_periods.forEach(function(p, i) {
      d3_time_periodLookup.set(p.toLowerCase(), i);
    });
    var d3_time_formats = {
      a: function(d) {
        return locale_shortDays[d.getDay()];
      },
      A: function(d) {
        return locale_days[d.getDay()];
      },
      b: function(d) {
        return locale_shortMonths[d.getMonth()];
      },
      B: function(d) {
        return locale_months[d.getMonth()];
      },
      c: d3_time_format(locale_dateTime),
      d: function(d, p) {
        return d3_time_formatPad(d.getDate(), p, 2);
      },
      e: function(d, p) {
        return d3_time_formatPad(d.getDate(), p, 2);
      },
      H: function(d, p) {
        return d3_time_formatPad(d.getHours(), p, 2);
      },
      I: function(d, p) {
        return d3_time_formatPad(d.getHours() % 12 || 12, p, 2);
      },
      j: function(d, p) {
        return d3_time_formatPad(1 + d3_time.dayOfYear(d), p, 3);
      },
      L: function(d, p) {
        return d3_time_formatPad(d.getMilliseconds(), p, 3);
      },
      m: function(d, p) {
        return d3_time_formatPad(d.getMonth() + 1, p, 2);
      },
      M: function(d, p) {
        return d3_time_formatPad(d.getMinutes(), p, 2);
      },
      p: function(d) {
        return locale_periods[+(d.getHours() >= 12)];
      },
      S: function(d, p) {
        return d3_time_formatPad(d.getSeconds(), p, 2);
      },
      U: function(d, p) {
        return d3_time_formatPad(d3_time.sundayOfYear(d), p, 2);
      },
      w: function(d) {
        return d.getDay();
      },
      W: function(d, p) {
        return d3_time_formatPad(d3_time.mondayOfYear(d), p, 2);
      },
      x: d3_time_format(locale_date),
      X: d3_time_format(locale_time),
      y: function(d, p) {
        return d3_time_formatPad(d.getFullYear() % 100, p, 2);
      },
      Y: function(d, p) {
        return d3_time_formatPad(d.getFullYear() % 1e4, p, 4);
      },
      Z: d3_time_zone,
      "%": function() {
        return "%";
      }
    };
    var d3_time_parsers = {
      a: d3_time_parseWeekdayAbbrev,
      A: d3_time_parseWeekday,
      b: d3_time_parseMonthAbbrev,
      B: d3_time_parseMonth,
      c: d3_time_parseLocaleFull,
      d: d3_time_parseDay,
      e: d3_time_parseDay,
      H: d3_time_parseHour24,
      I: d3_time_parseHour24,
      j: d3_time_parseDayOfYear,
      L: d3_time_parseMilliseconds,
      m: d3_time_parseMonthNumber,
      M: d3_time_parseMinutes,
      p: d3_time_parseAmPm,
      S: d3_time_parseSeconds,
      U: d3_time_parseWeekNumberSunday,
      w: d3_time_parseWeekdayNumber,
      W: d3_time_parseWeekNumberMonday,
      x: d3_time_parseLocaleDate,
      X: d3_time_parseLocaleTime,
      y: d3_time_parseYear,
      Y: d3_time_parseFullYear,
      Z: d3_time_parseZone,
      "%": d3_time_parseLiteralPercent
    };
    function d3_time_parseWeekdayAbbrev(date, string, i) {
      d3_time_dayAbbrevRe.lastIndex = 0;
      var n = d3_time_dayAbbrevRe.exec(string.slice(i));
      return n ? (date.w = d3_time_dayAbbrevLookup.get(n[0].toLowerCase()), i + n[0].length) : -1;
    }
    function d3_time_parseWeekday(date, string, i) {
      d3_time_dayRe.lastIndex = 0;
      var n = d3_time_dayRe.exec(string.slice(i));
      return n ? (date.w = d3_time_dayLookup.get(n[0].toLowerCase()), i + n[0].length) : -1;
    }
    function d3_time_parseMonthAbbrev(date, string, i) {
      d3_time_monthAbbrevRe.lastIndex = 0;
      var n = d3_time_monthAbbrevRe.exec(string.slice(i));
      return n ? (date.m = d3_time_monthAbbrevLookup.get(n[0].toLowerCase()), i + n[0].length) : -1;
    }
    function d3_time_parseMonth(date, string, i) {
      d3_time_monthRe.lastIndex = 0;
      var n = d3_time_monthRe.exec(string.slice(i));
      return n ? (date.m = d3_time_monthLookup.get(n[0].toLowerCase()), i + n[0].length) : -1;
    }
    function d3_time_parseLocaleFull(date, string, i) {
      return d3_time_parse(date, d3_time_formats.c.toString(), string, i);
    }
    function d3_time_parseLocaleDate(date, string, i) {
      return d3_time_parse(date, d3_time_formats.x.toString(), string, i);
    }
    function d3_time_parseLocaleTime(date, string, i) {
      return d3_time_parse(date, d3_time_formats.X.toString(), string, i);
    }
    function d3_time_parseAmPm(date, string, i) {
      var n = d3_time_periodLookup.get(string.slice(i, i += 2).toLowerCase());
      return n == null ? -1 : (date.p = n, i);
    }
    return d3_time_format;
  }
  var d3_time_formatPads = {
    "-": "",
    _: " ",
    "0": "0"
  }, d3_time_numberRe = /^\s*\d+/, d3_time_percentRe = /^%/;
  function d3_time_formatPad(value, fill, width) {
    var sign = value < 0 ? "-" : "", string = (sign ? -value : value) + "", length = string.length;
    return sign + (length < width ? new Array(width - length + 1).join(fill) + string : string);
  }
  function d3_time_formatRe(names) {
    return new RegExp("^(?:" + names.map(d3.requote).join("|") + ")", "i");
  }
  function d3_time_formatLookup(names) {
    var map = new d3_Map(), i = -1, n = names.length;
    while (++i < n) map.set(names[i].toLowerCase(), i);
    return map;
  }
  function d3_time_parseWeekdayNumber(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 1));
    return n ? (date.w = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseWeekNumberSunday(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i));
    return n ? (date.U = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseWeekNumberMonday(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i));
    return n ? (date.W = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseFullYear(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 4));
    return n ? (date.y = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseYear(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.y = d3_time_expandYear(+n[0]), i + n[0].length) : -1;
  }
  function d3_time_parseZone(date, string, i) {
    return /^[+-]\d{4}$/.test(string = string.slice(i, i + 5)) ? (date.Z = -string, 
    i + 5) : -1;
  }
  function d3_time_expandYear(d) {
    return d + (d > 68 ? 1900 : 2e3);
  }
  function d3_time_parseMonthNumber(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.m = n[0] - 1, i + n[0].length) : -1;
  }
  function d3_time_parseDay(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.d = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseDayOfYear(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 3));
    return n ? (date.j = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseHour24(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.H = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseMinutes(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.M = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseSeconds(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 2));
    return n ? (date.S = +n[0], i + n[0].length) : -1;
  }
  function d3_time_parseMilliseconds(date, string, i) {
    d3_time_numberRe.lastIndex = 0;
    var n = d3_time_numberRe.exec(string.slice(i, i + 3));
    return n ? (date.L = +n[0], i + n[0].length) : -1;
  }
  function d3_time_zone(d) {
    var z = d.getTimezoneOffset(), zs = z > 0 ? "-" : "+", zh = abs(z) / 60 | 0, zm = abs(z) % 60;
    return zs + d3_time_formatPad(zh, "0", 2) + d3_time_formatPad(zm, "0", 2);
  }
  function d3_time_parseLiteralPercent(date, string, i) {
    d3_time_percentRe.lastIndex = 0;
    var n = d3_time_percentRe.exec(string.slice(i, i + 1));
    return n ? i + n[0].length : -1;
  }
  function d3_time_formatMulti(formats) {
    var n = formats.length, i = -1;
    while (++i < n) formats[i][0] = this(formats[i][0]);
    return function(date) {
      var i = 0, f = formats[i];
      while (!f[1](date)) f = formats[++i];
      return f[0](date);
    };
  }
  d3.locale = function(locale) {
    return {
      numberFormat: d3_locale_numberFormat(locale),
      timeFormat: d3_locale_timeFormat(locale)
    };
  };
  var d3_locale_enUS = d3.locale({
    decimal: ".",
    thousands: ",",
    grouping: [ 3 ],
    currency: [ "$", "" ],
    dateTime: "%a %b %e %X %Y",
    date: "%m/%d/%Y",
    time: "%H:%M:%S",
    periods: [ "AM", "PM" ],
    days: [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ],
    shortDays: [ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" ],
    months: [ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ],
    shortMonths: [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" ]
  });
  d3.format = d3_locale_enUS.numberFormat;
  d3.geo = {};
  function d3_adder() {}
  d3_adder.prototype = {
    s: 0,
    t: 0,
    add: function(y) {
      d3_adderSum(y, this.t, d3_adderTemp);
      d3_adderSum(d3_adderTemp.s, this.s, this);
      if (this.s) this.t += d3_adderTemp.t; else this.s = d3_adderTemp.t;
    },
    reset: function() {
      this.s = this.t = 0;
    },
    valueOf: function() {
      return this.s;
    }
  };
  var d3_adderTemp = new d3_adder();
  function d3_adderSum(a, b, o) {
    var x = o.s = a + b, bv = x - a, av = x - bv;
    o.t = a - av + (b - bv);
  }
  d3.geo.stream = function(object, listener) {
    if (object && d3_geo_streamObjectType.hasOwnProperty(object.type)) {
      d3_geo_streamObjectType[object.type](object, listener);
    } else {
      d3_geo_streamGeometry(object, listener);
    }
  };
  function d3_geo_streamGeometry(geometry, listener) {
    if (geometry && d3_geo_streamGeometryType.hasOwnProperty(geometry.type)) {
      d3_geo_streamGeometryType[geometry.type](geometry, listener);
    }
  }
  var d3_geo_streamObjectType = {
    Feature: function(feature, listener) {
      d3_geo_streamGeometry(feature.geometry, listener);
    },
    FeatureCollection: function(object, listener) {
      var features = object.features, i = -1, n = features.length;
      while (++i < n) d3_geo_streamGeometry(features[i].geometry, listener);
    }
  };
  var d3_geo_streamGeometryType = {
    Sphere: function(object, listener) {
      listener.sphere();
    },
    Point: function(object, listener) {
      object = object.coordinates;
      listener.point(object[0], object[1], object[2]);
    },
    MultiPoint: function(object, listener) {
      var coordinates = object.coordinates, i = -1, n = coordinates.length;
      while (++i < n) object = coordinates[i], listener.point(object[0], object[1], object[2]);
    },
    LineString: function(object, listener) {
      d3_geo_streamLine(object.coordinates, listener, 0);
    },
    MultiLineString: function(object, listener) {
      var coordinates = object.coordinates, i = -1, n = coordinates.length;
      while (++i < n) d3_geo_streamLine(coordinates[i], listener, 0);
    },
    Polygon: function(object, listener) {
      d3_geo_streamPolygon(object.coordinates, listener);
    },
    MultiPolygon: function(object, listener) {
      var coordinates = object.coordinates, i = -1, n = coordinates.length;
      while (++i < n) d3_geo_streamPolygon(coordinates[i], listener);
    },
    GeometryCollection: function(object, listener) {
      var geometries = object.geometries, i = -1, n = geometries.length;
      while (++i < n) d3_geo_streamGeometry(geometries[i], listener);
    }
  };
  function d3_geo_streamLine(coordinates, listener, closed) {
    var i = -1, n = coordinates.length - closed, coordinate;
    listener.lineStart();
    while (++i < n) coordinate = coordinates[i], listener.point(coordinate[0], coordinate[1], coordinate[2]);
    listener.lineEnd();
  }
  function d3_geo_streamPolygon(coordinates, listener) {
    var i = -1, n = coordinates.length;
    listener.polygonStart();
    while (++i < n) d3_geo_streamLine(coordinates[i], listener, 1);
    listener.polygonEnd();
  }
  d3.geo.area = function(object) {
    d3_geo_areaSum = 0;
    d3.geo.stream(object, d3_geo_area);
    return d3_geo_areaSum;
  };
  var d3_geo_areaSum, d3_geo_areaRingSum = new d3_adder();
  var d3_geo_area = {
    sphere: function() {
      d3_geo_areaSum += 4 * π;
    },
    point: d3_noop,
    lineStart: d3_noop,
    lineEnd: d3_noop,
    polygonStart: function() {
      d3_geo_areaRingSum.reset();
      d3_geo_area.lineStart = d3_geo_areaRingStart;
    },
    polygonEnd: function() {
      var area = 2 * d3_geo_areaRingSum;
      d3_geo_areaSum += area < 0 ? 4 * π + area : area;
      d3_geo_area.lineStart = d3_geo_area.lineEnd = d3_geo_area.point = d3_noop;
    }
  };
  function d3_geo_areaRingStart() {
    var λ00, φ00, λ0, cosφ0, sinφ0;
    d3_geo_area.point = function(λ, φ) {
      d3_geo_area.point = nextPoint;
      λ0 = (λ00 = λ) * d3_radians, cosφ0 = Math.cos(φ = (φ00 = φ) * d3_radians / 2 + π / 4), 
      sinφ0 = Math.sin(φ);
    };
    function nextPoint(λ, φ) {
      λ *= d3_radians;
      φ = φ * d3_radians / 2 + π / 4;
      var dλ = λ - λ0, sdλ = dλ >= 0 ? 1 : -1, adλ = sdλ * dλ, cosφ = Math.cos(φ), sinφ = Math.sin(φ), k = sinφ0 * sinφ, u = cosφ0 * cosφ + k * Math.cos(adλ), v = k * sdλ * Math.sin(adλ);
      d3_geo_areaRingSum.add(Math.atan2(v, u));
      λ0 = λ, cosφ0 = cosφ, sinφ0 = sinφ;
    }
    d3_geo_area.lineEnd = function() {
      nextPoint(λ00, φ00);
    };
  }
  function d3_geo_cartesian(spherical) {
    var λ = spherical[0], φ = spherical[1], cosφ = Math.cos(φ);
    return [ cosφ * Math.cos(λ), cosφ * Math.sin(λ), Math.sin(φ) ];
  }
  function d3_geo_cartesianDot(a, b) {
    return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
  }
  function d3_geo_cartesianCross(a, b) {
    return [ a[1] * b[2] - a[2] * b[1], a[2] * b[0] - a[0] * b[2], a[0] * b[1] - a[1] * b[0] ];
  }
  function d3_geo_cartesianAdd(a, b) {
    a[0] += b[0];
    a[1] += b[1];
    a[2] += b[2];
  }
  function d3_geo_cartesianScale(vector, k) {
    return [ vector[0] * k, vector[1] * k, vector[2] * k ];
  }
  function d3_geo_cartesianNormalize(d) {
    var l = Math.sqrt(d[0] * d[0] + d[1] * d[1] + d[2] * d[2]);
    d[0] /= l;
    d[1] /= l;
    d[2] /= l;
  }
  function d3_geo_spherical(cartesian) {
    return [ Math.atan2(cartesian[1], cartesian[0]), d3_asin(cartesian[2]) ];
  }
  function d3_geo_sphericalEqual(a, b) {
    return abs(a[0] - b[0]) < ε && abs(a[1] - b[1]) < ε;
  }
  d3.geo.bounds = function() {
    var λ0, φ0, λ1, φ1, λ_, λ__, φ__, p0, dλSum, ranges, range;
    var bound = {
      point: point,
      lineStart: lineStart,
      lineEnd: lineEnd,
      polygonStart: function() {
        bound.point = ringPoint;
        bound.lineStart = ringStart;
        bound.lineEnd = ringEnd;
        dλSum = 0;
        d3_geo_area.polygonStart();
      },
      polygonEnd: function() {
        d3_geo_area.polygonEnd();
        bound.point = point;
        bound.lineStart = lineStart;
        bound.lineEnd = lineEnd;
        if (d3_geo_areaRingSum < 0) λ0 = -(λ1 = 180), φ0 = -(φ1 = 90); else if (dλSum > ε) φ1 = 90; else if (dλSum < -ε) φ0 = -90;
        range[0] = λ0, range[1] = λ1;
      }
    };
    function point(λ, φ) {
      ranges.push(range = [ λ0 = λ, λ1 = λ ]);
      if (φ < φ0) φ0 = φ;
      if (φ > φ1) φ1 = φ;
    }
    function linePoint(λ, φ) {
      var p = d3_geo_cartesian([ λ * d3_radians, φ * d3_radians ]);
      if (p0) {
        var normal = d3_geo_cartesianCross(p0, p), equatorial = [ normal[1], -normal[0], 0 ], inflection = d3_geo_cartesianCross(equatorial, normal);
        d3_geo_cartesianNormalize(inflection);
        inflection = d3_geo_spherical(inflection);
        var dλ = λ - λ_, s = dλ > 0 ? 1 : -1, λi = inflection[0] * d3_degrees * s, antimeridian = abs(dλ) > 180;
        if (antimeridian ^ (s * λ_ < λi && λi < s * λ)) {
          var φi = inflection[1] * d3_degrees;
          if (φi > φ1) φ1 = φi;
        } else if (λi = (λi + 360) % 360 - 180, antimeridian ^ (s * λ_ < λi && λi < s * λ)) {
          var φi = -inflection[1] * d3_degrees;
          if (φi < φ0) φ0 = φi;
        } else {
          if (φ < φ0) φ0 = φ;
          if (φ > φ1) φ1 = φ;
        }
        if (antimeridian) {
          if (λ < λ_) {
            if (angle(λ0, λ) > angle(λ0, λ1)) λ1 = λ;
          } else {
            if (angle(λ, λ1) > angle(λ0, λ1)) λ0 = λ;
          }
        } else {
          if (λ1 >= λ0) {
            if (λ < λ0) λ0 = λ;
            if (λ > λ1) λ1 = λ;
          } else {
            if (λ > λ_) {
              if (angle(λ0, λ) > angle(λ0, λ1)) λ1 = λ;
            } else {
              if (angle(λ, λ1) > angle(λ0, λ1)) λ0 = λ;
            }
          }
        }
      } else {
        point(λ, φ);
      }
      p0 = p, λ_ = λ;
    }
    function lineStart() {
      bound.point = linePoint;
    }
    function lineEnd() {
      range[0] = λ0, range[1] = λ1;
      bound.point = point;
      p0 = null;
    }
    function ringPoint(λ, φ) {
      if (p0) {
        var dλ = λ - λ_;
        dλSum += abs(dλ) > 180 ? dλ + (dλ > 0 ? 360 : -360) : dλ;
      } else λ__ = λ, φ__ = φ;
      d3_geo_area.point(λ, φ);
      linePoint(λ, φ);
    }
    function ringStart() {
      d3_geo_area.lineStart();
    }
    function ringEnd() {
      ringPoint(λ__, φ__);
      d3_geo_area.lineEnd();
      if (abs(dλSum) > ε) λ0 = -(λ1 = 180);
      range[0] = λ0, range[1] = λ1;
      p0 = null;
    }
    function angle(λ0, λ1) {
      return (λ1 -= λ0) < 0 ? λ1 + 360 : λ1;
    }
    function compareRanges(a, b) {
      return a[0] - b[0];
    }
    function withinRange(x, range) {
      return range[0] <= range[1] ? range[0] <= x && x <= range[1] : x < range[0] || range[1] < x;
    }
    return function(feature) {
      φ1 = λ1 = -(λ0 = φ0 = Infinity);
      ranges = [];
      d3.geo.stream(feature, bound);
      var n = ranges.length;
      if (n) {
        ranges.sort(compareRanges);
        for (var i = 1, a = ranges[0], b, merged = [ a ]; i < n; ++i) {
          b = ranges[i];
          if (withinRange(b[0], a) || withinRange(b[1], a)) {
            if (angle(a[0], b[1]) > angle(a[0], a[1])) a[1] = b[1];
            if (angle(b[0], a[1]) > angle(a[0], a[1])) a[0] = b[0];
          } else {
            merged.push(a = b);
          }
        }
        var best = -Infinity, dλ;
        for (var n = merged.length - 1, i = 0, a = merged[n], b; i <= n; a = b, ++i) {
          b = merged[i];
          if ((dλ = angle(a[1], b[0])) > best) best = dλ, λ0 = b[0], λ1 = a[1];
        }
      }
      ranges = range = null;
      return λ0 === Infinity || φ0 === Infinity ? [ [ NaN, NaN ], [ NaN, NaN ] ] : [ [ λ0, φ0 ], [ λ1, φ1 ] ];
    };
  }();
  d3.geo.centroid = function(object) {
    d3_geo_centroidW0 = d3_geo_centroidW1 = d3_geo_centroidX0 = d3_geo_centroidY0 = d3_geo_centroidZ0 = d3_geo_centroidX1 = d3_geo_centroidY1 = d3_geo_centroidZ1 = d3_geo_centroidX2 = d3_geo_centroidY2 = d3_geo_centroidZ2 = 0;
    d3.geo.stream(object, d3_geo_centroid);
    var x = d3_geo_centroidX2, y = d3_geo_centroidY2, z = d3_geo_centroidZ2, m = x * x + y * y + z * z;
    if (m < ε2) {
      x = d3_geo_centroidX1, y = d3_geo_centroidY1, z = d3_geo_centroidZ1;
      if (d3_geo_centroidW1 < ε) x = d3_geo_centroidX0, y = d3_geo_centroidY0, z = d3_geo_centroidZ0;
      m = x * x + y * y + z * z;
      if (m < ε2) return [ NaN, NaN ];
    }
    return [ Math.atan2(y, x) * d3_degrees, d3_asin(z / Math.sqrt(m)) * d3_degrees ];
  };
  var d3_geo_centroidW0, d3_geo_centroidW1, d3_geo_centroidX0, d3_geo_centroidY0, d3_geo_centroidZ0, d3_geo_centroidX1, d3_geo_centroidY1, d3_geo_centroidZ1, d3_geo_centroidX2, d3_geo_centroidY2, d3_geo_centroidZ2;
  var d3_geo_centroid = {
    sphere: d3_noop,
    point: d3_geo_centroidPoint,
    lineStart: d3_geo_centroidLineStart,
    lineEnd: d3_geo_centroidLineEnd,
    polygonStart: function() {
      d3_geo_centroid.lineStart = d3_geo_centroidRingStart;
    },
    polygonEnd: function() {
      d3_geo_centroid.lineStart = d3_geo_centroidLineStart;
    }
  };
  function d3_geo_centroidPoint(λ, φ) {
    λ *= d3_radians;
    var cosφ = Math.cos(φ *= d3_radians);
    d3_geo_centroidPointXYZ(cosφ * Math.cos(λ), cosφ * Math.sin(λ), Math.sin(φ));
  }
  function d3_geo_centroidPointXYZ(x, y, z) {
    ++d3_geo_centroidW0;
    d3_geo_centroidX0 += (x - d3_geo_centroidX0) / d3_geo_centroidW0;
    d3_geo_centroidY0 += (y - d3_geo_centroidY0) / d3_geo_centroidW0;
    d3_geo_centroidZ0 += (z - d3_geo_centroidZ0) / d3_geo_centroidW0;
  }
  function d3_geo_centroidLineStart() {
    var x0, y0, z0;
    d3_geo_centroid.point = function(λ, φ) {
      λ *= d3_radians;
      var cosφ = Math.cos(φ *= d3_radians);
      x0 = cosφ * Math.cos(λ);
      y0 = cosφ * Math.sin(λ);
      z0 = Math.sin(φ);
      d3_geo_centroid.point = nextPoint;
      d3_geo_centroidPointXYZ(x0, y0, z0);
    };
    function nextPoint(λ, φ) {
      λ *= d3_radians;
      var cosφ = Math.cos(φ *= d3_radians), x = cosφ * Math.cos(λ), y = cosφ * Math.sin(λ), z = Math.sin(φ), w = Math.atan2(Math.sqrt((w = y0 * z - z0 * y) * w + (w = z0 * x - x0 * z) * w + (w = x0 * y - y0 * x) * w), x0 * x + y0 * y + z0 * z);
      d3_geo_centroidW1 += w;
      d3_geo_centroidX1 += w * (x0 + (x0 = x));
      d3_geo_centroidY1 += w * (y0 + (y0 = y));
      d3_geo_centroidZ1 += w * (z0 + (z0 = z));
      d3_geo_centroidPointXYZ(x0, y0, z0);
    }
  }
  function d3_geo_centroidLineEnd() {
    d3_geo_centroid.point = d3_geo_centroidPoint;
  }
  function d3_geo_centroidRingStart() {
    var λ00, φ00, x0, y0, z0;
    d3_geo_centroid.point = function(λ, φ) {
      λ00 = λ, φ00 = φ;
      d3_geo_centroid.point = nextPoint;
      λ *= d3_radians;
      var cosφ = Math.cos(φ *= d3_radians);
      x0 = cosφ * Math.cos(λ);
      y0 = cosφ * Math.sin(λ);
      z0 = Math.sin(φ);
      d3_geo_centroidPointXYZ(x0, y0, z0);
    };
    d3_geo_centroid.lineEnd = function() {
      nextPoint(λ00, φ00);
      d3_geo_centroid.lineEnd = d3_geo_centroidLineEnd;
      d3_geo_centroid.point = d3_geo_centroidPoint;
    };
    function nextPoint(λ, φ) {
      λ *= d3_radians;
      var cosφ = Math.cos(φ *= d3_radians), x = cosφ * Math.cos(λ), y = cosφ * Math.sin(λ), z = Math.sin(φ), cx = y0 * z - z0 * y, cy = z0 * x - x0 * z, cz = x0 * y - y0 * x, m = Math.sqrt(cx * cx + cy * cy + cz * cz), u = x0 * x + y0 * y + z0 * z, v = m && -d3_acos(u) / m, w = Math.atan2(m, u);
      d3_geo_centroidX2 += v * cx;
      d3_geo_centroidY2 += v * cy;
      d3_geo_centroidZ2 += v * cz;
      d3_geo_centroidW1 += w;
      d3_geo_centroidX1 += w * (x0 + (x0 = x));
      d3_geo_centroidY1 += w * (y0 + (y0 = y));
      d3_geo_centroidZ1 += w * (z0 + (z0 = z));
      d3_geo_centroidPointXYZ(x0, y0, z0);
    }
  }
  function d3_geo_compose(a, b) {
    function compose(x, y) {
      return x = a(x, y), b(x[0], x[1]);
    }
    if (a.invert && b.invert) compose.invert = function(x, y) {
      return x = b.invert(x, y), x && a.invert(x[0], x[1]);
    };
    return compose;
  }
  function d3_true() {
    return true;
  }
  function d3_geo_clipPolygon(segments, compare, clipStartInside, interpolate, listener) {
    var subject = [], clip = [];
    segments.forEach(function(segment) {
      if ((n = segment.length - 1) <= 0) return;
      var n, p0 = segment[0], p1 = segment[n];
      if (d3_geo_sphericalEqual(p0, p1)) {
        listener.lineStart();
        for (var i = 0; i < n; ++i) listener.point((p0 = segment[i])[0], p0[1]);
        listener.lineEnd();
        return;
      }
      var a = new d3_geo_clipPolygonIntersection(p0, segment, null, true), b = new d3_geo_clipPolygonIntersection(p0, null, a, false);
      a.o = b;
      subject.push(a);
      clip.push(b);
      a = new d3_geo_clipPolygonIntersection(p1, segment, null, false);
      b = new d3_geo_clipPolygonIntersection(p1, null, a, true);
      a.o = b;
      subject.push(a);
      clip.push(b);
    });
    clip.sort(compare);
    d3_geo_clipPolygonLinkCircular(subject);
    d3_geo_clipPolygonLinkCircular(clip);
    if (!subject.length) return;
    for (var i = 0, entry = clipStartInside, n = clip.length; i < n; ++i) {
      clip[i].e = entry = !entry;
    }
    var start = subject[0], points, point;
    while (1) {
      var current = start, isSubject = true;
      while (current.v) if ((current = current.n) === start) return;
      points = current.z;
      listener.lineStart();
      do {
        current.v = current.o.v = true;
        if (current.e) {
          if (isSubject) {
            for (var i = 0, n = points.length; i < n; ++i) listener.point((point = points[i])[0], point[1]);
          } else {
            interpolate(current.x, current.n.x, 1, listener);
          }
          current = current.n;
        } else {
          if (isSubject) {
            points = current.p.z;
            for (var i = points.length - 1; i >= 0; --i) listener.point((point = points[i])[0], point[1]);
          } else {
            interpolate(current.x, current.p.x, -1, listener);
          }
          current = current.p;
        }
        current = current.o;
        points = current.z;
        isSubject = !isSubject;
      } while (!current.v);
      listener.lineEnd();
    }
  }
  function d3_geo_clipPolygonLinkCircular(array) {
    if (!(n = array.length)) return;
    var n, i = 0, a = array[0], b;
    while (++i < n) {
      a.n = b = array[i];
      b.p = a;
      a = b;
    }
    a.n = b = array[0];
    b.p = a;
  }
  function d3_geo_clipPolygonIntersection(point, points, other, entry) {
    this.x = point;
    this.z = points;
    this.o = other;
    this.e = entry;
    this.v = false;
    this.n = this.p = null;
  }
  function d3_geo_clip(pointVisible, clipLine, interpolate, clipStart) {
    return function(rotate, listener) {
      var line = clipLine(listener), rotatedClipStart = rotate.invert(clipStart[0], clipStart[1]);
      var clip = {
        point: point,
        lineStart: lineStart,
        lineEnd: lineEnd,
        polygonStart: function() {
          clip.point = pointRing;
          clip.lineStart = ringStart;
          clip.lineEnd = ringEnd;
          segments = [];
          polygon = [];
        },
        polygonEnd: function() {
          clip.point = point;
          clip.lineStart = lineStart;
          clip.lineEnd = lineEnd;
          segments = d3.merge(segments);
          var clipStartInside = d3_geo_pointInPolygon(rotatedClipStart, polygon);
          if (segments.length) {
            if (!polygonStarted) listener.polygonStart(), polygonStarted = true;
            d3_geo_clipPolygon(segments, d3_geo_clipSort, clipStartInside, interpolate, listener);
          } else if (clipStartInside) {
            if (!polygonStarted) listener.polygonStart(), polygonStarted = true;
            listener.lineStart();
            interpolate(null, null, 1, listener);
            listener.lineEnd();
          }
          if (polygonStarted) listener.polygonEnd(), polygonStarted = false;
          segments = polygon = null;
        },
        sphere: function() {
          listener.polygonStart();
          listener.lineStart();
          interpolate(null, null, 1, listener);
          listener.lineEnd();
          listener.polygonEnd();
        }
      };
      function point(λ, φ) {
        var point = rotate(λ, φ);
        if (pointVisible(λ = point[0], φ = point[1])) listener.point(λ, φ);
      }
      function pointLine(λ, φ) {
        var point = rotate(λ, φ);
        line.point(point[0], point[1]);
      }
      function lineStart() {
        clip.point = pointLine;
        line.lineStart();
      }
      function lineEnd() {
        clip.point = point;
        line.lineEnd();
      }
      var segments;
      var buffer = d3_geo_clipBufferListener(), ringListener = clipLine(buffer), polygonStarted = false, polygon, ring;
      function pointRing(λ, φ) {
        ring.push([ λ, φ ]);
        var point = rotate(λ, φ);
        ringListener.point(point[0], point[1]);
      }
      function ringStart() {
        ringListener.lineStart();
        ring = [];
      }
      function ringEnd() {
        pointRing(ring[0][0], ring[0][1]);
        ringListener.lineEnd();
        var clean = ringListener.clean(), ringSegments = buffer.buffer(), segment, n = ringSegments.length;
        ring.pop();
        polygon.push(ring);
        ring = null;
        if (!n) return;
        if (clean & 1) {
          segment = ringSegments[0];
          var n = segment.length - 1, i = -1, point;
          if (n > 0) {
            if (!polygonStarted) listener.polygonStart(), polygonStarted = true;
            listener.lineStart();
            while (++i < n) listener.point((point = segment[i])[0], point[1]);
            listener.lineEnd();
          }
          return;
        }
        if (n > 1 && clean & 2) ringSegments.push(ringSegments.pop().concat(ringSegments.shift()));
        segments.push(ringSegments.filter(d3_geo_clipSegmentLength1));
      }
      return clip;
    };
  }
  function d3_geo_clipSegmentLength1(segment) {
    return segment.length > 1;
  }
  function d3_geo_clipBufferListener() {
    var lines = [], line;
    return {
      lineStart: function() {
        lines.push(line = []);
      },
      point: function(λ, φ) {
        line.push([ λ, φ ]);
      },
      lineEnd: d3_noop,
      buffer: function() {
        var buffer = lines;
        lines = [];
        line = null;
        return buffer;
      },
      rejoin: function() {
        if (lines.length > 1) lines.push(lines.pop().concat(lines.shift()));
      }
    };
  }
  function d3_geo_clipSort(a, b) {
    return ((a = a.x)[0] < 0 ? a[1] - halfπ - ε : halfπ - a[1]) - ((b = b.x)[0] < 0 ? b[1] - halfπ - ε : halfπ - b[1]);
  }
  var d3_geo_clipAntimeridian = d3_geo_clip(d3_true, d3_geo_clipAntimeridianLine, d3_geo_clipAntimeridianInterpolate, [ -π, -π / 2 ]);
  function d3_geo_clipAntimeridianLine(listener) {
    var λ0 = NaN, φ0 = NaN, sλ0 = NaN, clean;
    return {
      lineStart: function() {
        listener.lineStart();
        clean = 1;
      },
      point: function(λ1, φ1) {
        var sλ1 = λ1 > 0 ? π : -π, dλ = abs(λ1 - λ0);
        if (abs(dλ - π) < ε) {
          listener.point(λ0, φ0 = (φ0 + φ1) / 2 > 0 ? halfπ : -halfπ);
          listener.point(sλ0, φ0);
          listener.lineEnd();
          listener.lineStart();
          listener.point(sλ1, φ0);
          listener.point(λ1, φ0);
          clean = 0;
        } else if (sλ0 !== sλ1 && dλ >= π) {
          if (abs(λ0 - sλ0) < ε) λ0 -= sλ0 * ε;
          if (abs(λ1 - sλ1) < ε) λ1 -= sλ1 * ε;
          φ0 = d3_geo_clipAntimeridianIntersect(λ0, φ0, λ1, φ1);
          listener.point(sλ0, φ0);
          listener.lineEnd();
          listener.lineStart();
          listener.point(sλ1, φ0);
          clean = 0;
        }
        listener.point(λ0 = λ1, φ0 = φ1);
        sλ0 = sλ1;
      },
      lineEnd: function() {
        listener.lineEnd();
        λ0 = φ0 = NaN;
      },
      clean: function() {
        return 2 - clean;
      }
    };
  }
  function d3_geo_clipAntimeridianIntersect(λ0, φ0, λ1, φ1) {
    var cosφ0, cosφ1, sinλ0_λ1 = Math.sin(λ0 - λ1);
    return abs(sinλ0_λ1) > ε ? Math.atan((Math.sin(φ0) * (cosφ1 = Math.cos(φ1)) * Math.sin(λ1) - Math.sin(φ1) * (cosφ0 = Math.cos(φ0)) * Math.sin(λ0)) / (cosφ0 * cosφ1 * sinλ0_λ1)) : (φ0 + φ1) / 2;
  }
  function d3_geo_clipAntimeridianInterpolate(from, to, direction, listener) {
    var φ;
    if (from == null) {
      φ = direction * halfπ;
      listener.point(-π, φ);
      listener.point(0, φ);
      listener.point(π, φ);
      listener.point(π, 0);
      listener.point(π, -φ);
      listener.point(0, -φ);
      listener.point(-π, -φ);
      listener.point(-π, 0);
      listener.point(-π, φ);
    } else if (abs(from[0] - to[0]) > ε) {
      var s = from[0] < to[0] ? π : -π;
      φ = direction * s / 2;
      listener.point(-s, φ);
      listener.point(0, φ);
      listener.point(s, φ);
    } else {
      listener.point(to[0], to[1]);
    }
  }
  function d3_geo_pointInPolygon(point, polygon) {
    var meridian = point[0], parallel = point[1], meridianNormal = [ Math.sin(meridian), -Math.cos(meridian), 0 ], polarAngle = 0, winding = 0;
    d3_geo_areaRingSum.reset();
    for (var i = 0, n = polygon.length; i < n; ++i) {
      var ring = polygon[i], m = ring.length;
      if (!m) continue;
      var point0 = ring[0], λ0 = point0[0], φ0 = point0[1] / 2 + π / 4, sinφ0 = Math.sin(φ0), cosφ0 = Math.cos(φ0), j = 1;
      while (true) {
        if (j === m) j = 0;
        point = ring[j];
        var λ = point[0], φ = point[1] / 2 + π / 4, sinφ = Math.sin(φ), cosφ = Math.cos(φ), dλ = λ - λ0, sdλ = dλ >= 0 ? 1 : -1, adλ = sdλ * dλ, antimeridian = adλ > π, k = sinφ0 * sinφ;
        d3_geo_areaRingSum.add(Math.atan2(k * sdλ * Math.sin(adλ), cosφ0 * cosφ + k * Math.cos(adλ)));
        polarAngle += antimeridian ? dλ + sdλ * τ : dλ;
        if (antimeridian ^ λ0 >= meridian ^ λ >= meridian) {
          var arc = d3_geo_cartesianCross(d3_geo_cartesian(point0), d3_geo_cartesian(point));
          d3_geo_cartesianNormalize(arc);
          var intersection = d3_geo_cartesianCross(meridianNormal, arc);
          d3_geo_cartesianNormalize(intersection);
          var φarc = (antimeridian ^ dλ >= 0 ? -1 : 1) * d3_asin(intersection[2]);
          if (parallel > φarc || parallel === φarc && (arc[0] || arc[1])) {
            winding += antimeridian ^ dλ >= 0 ? 1 : -1;
          }
        }
        if (!j++) break;
        λ0 = λ, sinφ0 = sinφ, cosφ0 = cosφ, point0 = point;
      }
    }
    return (polarAngle < -ε || polarAngle < ε && d3_geo_areaRingSum < -ε) ^ winding & 1;
  }
  function d3_geo_clipCircle(radius) {
    var cr = Math.cos(radius), smallRadius = cr > 0, notHemisphere = abs(cr) > ε, interpolate = d3_geo_circleInterpolate(radius, 6 * d3_radians);
    return d3_geo_clip(visible, clipLine, interpolate, smallRadius ? [ 0, -radius ] : [ -π, radius - π ]);
    function visible(λ, φ) {
      return Math.cos(λ) * Math.cos(φ) > cr;
    }
    function clipLine(listener) {
      var point0, c0, v0, v00, clean;
      return {
        lineStart: function() {
          v00 = v0 = false;
          clean = 1;
        },
        point: function(λ, φ) {
          var point1 = [ λ, φ ], point2, v = visible(λ, φ), c = smallRadius ? v ? 0 : code(λ, φ) : v ? code(λ + (λ < 0 ? π : -π), φ) : 0;
          if (!point0 && (v00 = v0 = v)) listener.lineStart();
          if (v !== v0) {
            point2 = intersect(point0, point1);
            if (d3_geo_sphericalEqual(point0, point2) || d3_geo_sphericalEqual(point1, point2)) {
              point1[0] += ε;
              point1[1] += ε;
              v = visible(point1[0], point1[1]);
            }
          }
          if (v !== v0) {
            clean = 0;
            if (v) {
              listener.lineStart();
              point2 = intersect(point1, point0);
              listener.point(point2[0], point2[1]);
            } else {
              point2 = intersect(point0, point1);
              listener.point(point2[0], point2[1]);
              listener.lineEnd();
            }
            point0 = point2;
          } else if (notHemisphere && point0 && smallRadius ^ v) {
            var t;
            if (!(c & c0) && (t = intersect(point1, point0, true))) {
              clean = 0;
              if (smallRadius) {
                listener.lineStart();
                listener.point(t[0][0], t[0][1]);
                listener.point(t[1][0], t[1][1]);
                listener.lineEnd();
              } else {
                listener.point(t[1][0], t[1][1]);
                listener.lineEnd();
                listener.lineStart();
                listener.point(t[0][0], t[0][1]);
              }
            }
          }
          if (v && (!point0 || !d3_geo_sphericalEqual(point0, point1))) {
            listener.point(point1[0], point1[1]);
          }
          point0 = point1, v0 = v, c0 = c;
        },
        lineEnd: function() {
          if (v0) listener.lineEnd();
          point0 = null;
        },
        clean: function() {
          return clean | (v00 && v0) << 1;
        }
      };
    }
    function intersect(a, b, two) {
      var pa = d3_geo_cartesian(a), pb = d3_geo_cartesian(b);
      var n1 = [ 1, 0, 0 ], n2 = d3_geo_cartesianCross(pa, pb), n2n2 = d3_geo_cartesianDot(n2, n2), n1n2 = n2[0], determinant = n2n2 - n1n2 * n1n2;
      if (!determinant) return !two && a;
      var c1 = cr * n2n2 / determinant, c2 = -cr * n1n2 / determinant, n1xn2 = d3_geo_cartesianCross(n1, n2), A = d3_geo_cartesianScale(n1, c1), B = d3_geo_cartesianScale(n2, c2);
      d3_geo_cartesianAdd(A, B);
      var u = n1xn2, w = d3_geo_cartesianDot(A, u), uu = d3_geo_cartesianDot(u, u), t2 = w * w - uu * (d3_geo_cartesianDot(A, A) - 1);
      if (t2 < 0) return;
      var t = Math.sqrt(t2), q = d3_geo_cartesianScale(u, (-w - t) / uu);
      d3_geo_cartesianAdd(q, A);
      q = d3_geo_spherical(q);
      if (!two) return q;
      var λ0 = a[0], λ1 = b[0], φ0 = a[1], φ1 = b[1], z;
      if (λ1 < λ0) z = λ0, λ0 = λ1, λ1 = z;
      var δλ = λ1 - λ0, polar = abs(δλ - π) < ε, meridian = polar || δλ < ε;
      if (!polar && φ1 < φ0) z = φ0, φ0 = φ1, φ1 = z;
      if (meridian ? polar ? φ0 + φ1 > 0 ^ q[1] < (abs(q[0] - λ0) < ε ? φ0 : φ1) : φ0 <= q[1] && q[1] <= φ1 : δλ > π ^ (λ0 <= q[0] && q[0] <= λ1)) {
        var q1 = d3_geo_cartesianScale(u, (-w + t) / uu);
        d3_geo_cartesianAdd(q1, A);
        return [ q, d3_geo_spherical(q1) ];
      }
    }
    function code(λ, φ) {
      var r = smallRadius ? radius : π - radius, code = 0;
      if (λ < -r) code |= 1; else if (λ > r) code |= 2;
      if (φ < -r) code |= 4; else if (φ > r) code |= 8;
      return code;
    }
  }
  function d3_geom_clipLine(x0, y0, x1, y1) {
    return function(line) {
      var a = line.a, b = line.b, ax = a.x, ay = a.y, bx = b.x, by = b.y, t0 = 0, t1 = 1, dx = bx - ax, dy = by - ay, r;
      r = x0 - ax;
      if (!dx && r > 0) return;
      r /= dx;
      if (dx < 0) {
        if (r < t0) return;
        if (r < t1) t1 = r;
      } else if (dx > 0) {
        if (r > t1) return;
        if (r > t0) t0 = r;
      }
      r = x1 - ax;
      if (!dx && r < 0) return;
      r /= dx;
      if (dx < 0) {
        if (r > t1) return;
        if (r > t0) t0 = r;
      } else if (dx > 0) {
        if (r < t0) return;
        if (r < t1) t1 = r;
      }
      r = y0 - ay;
      if (!dy && r > 0) return;
      r /= dy;
      if (dy < 0) {
        if (r < t0) return;
        if (r < t1) t1 = r;
      } else if (dy > 0) {
        if (r > t1) return;
        if (r > t0) t0 = r;
      }
      r = y1 - ay;
      if (!dy && r < 0) return;
      r /= dy;
      if (dy < 0) {
        if (r > t1) return;
        if (r > t0) t0 = r;
      } else if (dy > 0) {
        if (r < t0) return;
        if (r < t1) t1 = r;
      }
      if (t0 > 0) line.a = {
        x: ax + t0 * dx,
        y: ay + t0 * dy
      };
      if (t1 < 1) line.b = {
        x: ax + t1 * dx,
        y: ay + t1 * dy
      };
      return line;
    };
  }
  var d3_geo_clipExtentMAX = 1e9;
  d3.geo.clipExtent = function() {
    var x0, y0, x1, y1, stream, clip, clipExtent = {
      stream: function(output) {
        if (stream) stream.valid = false;
        stream = clip(output);
        stream.valid = true;
        return stream;
      },
      extent: function(_) {
        if (!arguments.length) return [ [ x0, y0 ], [ x1, y1 ] ];
        clip = d3_geo_clipExtent(x0 = +_[0][0], y0 = +_[0][1], x1 = +_[1][0], y1 = +_[1][1]);
        if (stream) stream.valid = false, stream = null;
        return clipExtent;
      }
    };
    return clipExtent.extent([ [ 0, 0 ], [ 960, 500 ] ]);
  };
  function d3_geo_clipExtent(x0, y0, x1, y1) {
    return function(listener) {
      var listener_ = listener, bufferListener = d3_geo_clipBufferListener(), clipLine = d3_geom_clipLine(x0, y0, x1, y1), segments, polygon, ring;
      var clip = {
        point: point,
        lineStart: lineStart,
        lineEnd: lineEnd,
        polygonStart: function() {
          listener = bufferListener;
          segments = [];
          polygon = [];
          clean = true;
        },
        polygonEnd: function() {
          listener = listener_;
          segments = d3.merge(segments);
          var clipStartInside = insidePolygon([ x0, y1 ]), inside = clean && clipStartInside, visible = segments.length;
          if (inside || visible) {
            listener.polygonStart();
            if (inside) {
              listener.lineStart();
              interpolate(null, null, 1, listener);
              listener.lineEnd();
            }
            if (visible) {
              d3_geo_clipPolygon(segments, compare, clipStartInside, interpolate, listener);
            }
            listener.polygonEnd();
          }
          segments = polygon = ring = null;
        }
      };
      function insidePolygon(p) {
        var wn = 0, n = polygon.length, y = p[1];
        for (var i = 0; i < n; ++i) {
          for (var j = 1, v = polygon[i], m = v.length, a = v[0], b; j < m; ++j) {
            b = v[j];
            if (a[1] <= y) {
              if (b[1] > y && d3_cross2d(a, b, p) > 0) ++wn;
            } else {
              if (b[1] <= y && d3_cross2d(a, b, p) < 0) --wn;
            }
            a = b;
          }
        }
        return wn !== 0;
      }
      function interpolate(from, to, direction, listener) {
        var a = 0, a1 = 0;
        if (from == null || (a = corner(from, direction)) !== (a1 = corner(to, direction)) || comparePoints(from, to) < 0 ^ direction > 0) {
          do {
            listener.point(a === 0 || a === 3 ? x0 : x1, a > 1 ? y1 : y0);
          } while ((a = (a + direction + 4) % 4) !== a1);
        } else {
          listener.point(to[0], to[1]);
        }
      }
      function pointVisible(x, y) {
        return x0 <= x && x <= x1 && y0 <= y && y <= y1;
      }
      function point(x, y) {
        if (pointVisible(x, y)) listener.point(x, y);
      }
      var x__, y__, v__, x_, y_, v_, first, clean;
      function lineStart() {
        clip.point = linePoint;
        if (polygon) polygon.push(ring = []);
        first = true;
        v_ = false;
        x_ = y_ = NaN;
      }
      function lineEnd() {
        if (segments) {
          linePoint(x__, y__);
          if (v__ && v_) bufferListener.rejoin();
          segments.push(bufferListener.buffer());
        }
        clip.point = point;
        if (v_) listener.lineEnd();
      }
      function linePoint(x, y) {
        x = Math.max(-d3_geo_clipExtentMAX, Math.min(d3_geo_clipExtentMAX, x));
        y = Math.max(-d3_geo_clipExtentMAX, Math.min(d3_geo_clipExtentMAX, y));
        var v = pointVisible(x, y);
        if (polygon) ring.push([ x, y ]);
        if (first) {
          x__ = x, y__ = y, v__ = v;
          first = false;
          if (v) {
            listener.lineStart();
            listener.point(x, y);
          }
        } else {
          if (v && v_) listener.point(x, y); else {
            var l = {
              a: {
                x: x_,
                y: y_
              },
              b: {
                x: x,
                y: y
              }
            };
            if (clipLine(l)) {
              if (!v_) {
                listener.lineStart();
                listener.point(l.a.x, l.a.y);
              }
              listener.point(l.b.x, l.b.y);
              if (!v) listener.lineEnd();
              clean = false;
            } else if (v) {
              listener.lineStart();
              listener.point(x, y);
              clean = false;
            }
          }
        }
        x_ = x, y_ = y, v_ = v;
      }
      return clip;
    };
    function corner(p, direction) {
      return abs(p[0] - x0) < ε ? direction > 0 ? 0 : 3 : abs(p[0] - x1) < ε ? direction > 0 ? 2 : 1 : abs(p[1] - y0) < ε ? direction > 0 ? 1 : 0 : direction > 0 ? 3 : 2;
    }
    function compare(a, b) {
      return comparePoints(a.x, b.x);
    }
    function comparePoints(a, b) {
      var ca = corner(a, 1), cb = corner(b, 1);
      return ca !== cb ? ca - cb : ca === 0 ? b[1] - a[1] : ca === 1 ? a[0] - b[0] : ca === 2 ? a[1] - b[1] : b[0] - a[0];
    }
  }
  function d3_geo_conic(projectAt) {
    var φ0 = 0, φ1 = π / 3, m = d3_geo_projectionMutator(projectAt), p = m(φ0, φ1);
    p.parallels = function(_) {
      if (!arguments.length) return [ φ0 / π * 180, φ1 / π * 180 ];
      return m(φ0 = _[0] * π / 180, φ1 = _[1] * π / 180);
    };
    return p;
  }
  function d3_geo_conicEqualArea(φ0, φ1) {
    var sinφ0 = Math.sin(φ0), n = (sinφ0 + Math.sin(φ1)) / 2, C = 1 + sinφ0 * (2 * n - sinφ0), ρ0 = Math.sqrt(C) / n;
    function forward(λ, φ) {
      var ρ = Math.sqrt(C - 2 * n * Math.sin(φ)) / n;
      return [ ρ * Math.sin(λ *= n), ρ0 - ρ * Math.cos(λ) ];
    }
    forward.invert = function(x, y) {
      var ρ0_y = ρ0 - y;
      return [ Math.atan2(x, ρ0_y) / n, d3_asin((C - (x * x + ρ0_y * ρ0_y) * n * n) / (2 * n)) ];
    };
    return forward;
  }
  (d3.geo.conicEqualArea = function() {
    return d3_geo_conic(d3_geo_conicEqualArea);
  }).raw = d3_geo_conicEqualArea;
  d3.geo.albers = function() {
    return d3.geo.conicEqualArea().rotate([ 96, 0 ]).center([ -.6, 38.7 ]).parallels([ 29.5, 45.5 ]).scale(1070);
  };
  d3.geo.albersUsa = function() {
    var lower48 = d3.geo.albers();
    var alaska = d3.geo.conicEqualArea().rotate([ 154, 0 ]).center([ -2, 58.5 ]).parallels([ 55, 65 ]);
    var hawaii = d3.geo.conicEqualArea().rotate([ 157, 0 ]).center([ -3, 19.9 ]).parallels([ 8, 18 ]);
    var point, pointStream = {
      point: function(x, y) {
        point = [ x, y ];
      }
    }, lower48Point, alaskaPoint, hawaiiPoint;
    function albersUsa(coordinates) {
      var x = coordinates[0], y = coordinates[1];
      point = null;
      (lower48Point(x, y), point) || (alaskaPoint(x, y), point) || hawaiiPoint(x, y);
      return point;
    }
    albersUsa.invert = function(coordinates) {
      var k = lower48.scale(), t = lower48.translate(), x = (coordinates[0] - t[0]) / k, y = (coordinates[1] - t[1]) / k;
      return (y >= .12 && y < .234 && x >= -.425 && x < -.214 ? alaska : y >= .166 && y < .234 && x >= -.214 && x < -.115 ? hawaii : lower48).invert(coordinates);
    };
    albersUsa.stream = function(stream) {
      var lower48Stream = lower48.stream(stream), alaskaStream = alaska.stream(stream), hawaiiStream = hawaii.stream(stream);
      return {
        point: function(x, y) {
          lower48Stream.point(x, y);
          alaskaStream.point(x, y);
          hawaiiStream.point(x, y);
        },
        sphere: function() {
          lower48Stream.sphere();
          alaskaStream.sphere();
          hawaiiStream.sphere();
        },
        lineStart: function() {
          lower48Stream.lineStart();
          alaskaStream.lineStart();
          hawaiiStream.lineStart();
        },
        lineEnd: function() {
          lower48Stream.lineEnd();
          alaskaStream.lineEnd();
          hawaiiStream.lineEnd();
        },
        polygonStart: function() {
          lower48Stream.polygonStart();
          alaskaStream.polygonStart();
          hawaiiStream.polygonStart();
        },
        polygonEnd: function() {
          lower48Stream.polygonEnd();
          alaskaStream.polygonEnd();
          hawaiiStream.polygonEnd();
        }
      };
    };
    albersUsa.precision = function(_) {
      if (!arguments.length) return lower48.precision();
      lower48.precision(_);
      alaska.precision(_);
      hawaii.precision(_);
      return albersUsa;
    };
    albersUsa.scale = function(_) {
      if (!arguments.length) return lower48.scale();
      lower48.scale(_);
      alaska.scale(_ * .35);
      hawaii.scale(_);
      return albersUsa.translate(lower48.translate());
    };
    albersUsa.translate = function(_) {
      if (!arguments.length) return lower48.translate();
      var k = lower48.scale(), x = +_[0], y = +_[1];
      lower48Point = lower48.translate(_).clipExtent([ [ x - .455 * k, y - .238 * k ], [ x + .455 * k, y + .238 * k ] ]).stream(pointStream).point;
      alaskaPoint = alaska.translate([ x - .307 * k, y + .201 * k ]).clipExtent([ [ x - .425 * k + ε, y + .12 * k + ε ], [ x - .214 * k - ε, y + .234 * k - ε ] ]).stream(pointStream).point;
      hawaiiPoint = hawaii.translate([ x - .205 * k, y + .212 * k ]).clipExtent([ [ x - .214 * k + ε, y + .166 * k + ε ], [ x - .115 * k - ε, y + .234 * k - ε ] ]).stream(pointStream).point;
      return albersUsa;
    };
    return albersUsa.scale(1070);
  };
  var d3_geo_pathAreaSum, d3_geo_pathAreaPolygon, d3_geo_pathArea = {
    point: d3_noop,
    lineStart: d3_noop,
    lineEnd: d3_noop,
    polygonStart: function() {
      d3_geo_pathAreaPolygon = 0;
      d3_geo_pathArea.lineStart = d3_geo_pathAreaRingStart;
    },
    polygonEnd: function() {
      d3_geo_pathArea.lineStart = d3_geo_pathArea.lineEnd = d3_geo_pathArea.point = d3_noop;
      d3_geo_pathAreaSum += abs(d3_geo_pathAreaPolygon / 2);
    }
  };
  function d3_geo_pathAreaRingStart() {
    var x00, y00, x0, y0;
    d3_geo_pathArea.point = function(x, y) {
      d3_geo_pathArea.point = nextPoint;
      x00 = x0 = x, y00 = y0 = y;
    };
    function nextPoint(x, y) {
      d3_geo_pathAreaPolygon += y0 * x - x0 * y;
      x0 = x, y0 = y;
    }
    d3_geo_pathArea.lineEnd = function() {
      nextPoint(x00, y00);
    };
  }
  var d3_geo_pathBoundsX0, d3_geo_pathBoundsY0, d3_geo_pathBoundsX1, d3_geo_pathBoundsY1;
  var d3_geo_pathBounds = {
    point: d3_geo_pathBoundsPoint,
    lineStart: d3_noop,
    lineEnd: d3_noop,
    polygonStart: d3_noop,
    polygonEnd: d3_noop
  };
  function d3_geo_pathBoundsPoint(x, y) {
    if (x < d3_geo_pathBoundsX0) d3_geo_pathBoundsX0 = x;
    if (x > d3_geo_pathBoundsX1) d3_geo_pathBoundsX1 = x;
    if (y < d3_geo_pathBoundsY0) d3_geo_pathBoundsY0 = y;
    if (y > d3_geo_pathBoundsY1) d3_geo_pathBoundsY1 = y;
  }
  function d3_geo_pathBuffer() {
    var pointCircle = d3_geo_pathBufferCircle(4.5), buffer = [];
    var stream = {
      point: point,
      lineStart: function() {
        stream.point = pointLineStart;
      },
      lineEnd: lineEnd,
      polygonStart: function() {
        stream.lineEnd = lineEndPolygon;
      },
      polygonEnd: function() {
        stream.lineEnd = lineEnd;
        stream.point = point;
      },
      pointRadius: function(_) {
        pointCircle = d3_geo_pathBufferCircle(_);
        return stream;
      },
      result: function() {
        if (buffer.length) {
          var result = buffer.join("");
          buffer = [];
          return result;
        }
      }
    };
    function point(x, y) {
      buffer.push("M", x, ",", y, pointCircle);
    }
    function pointLineStart(x, y) {
      buffer.push("M", x, ",", y);
      stream.point = pointLine;
    }
    function pointLine(x, y) {
      buffer.push("L", x, ",", y);
    }
    function lineEnd() {
      stream.point = point;
    }
    function lineEndPolygon() {
      buffer.push("Z");
    }
    return stream;
  }
  function d3_geo_pathBufferCircle(radius) {
    return "m0," + radius + "a" + radius + "," + radius + " 0 1,1 0," + -2 * radius + "a" + radius + "," + radius + " 0 1,1 0," + 2 * radius + "z";
  }
  var d3_geo_pathCentroid = {
    point: d3_geo_pathCentroidPoint,
    lineStart: d3_geo_pathCentroidLineStart,
    lineEnd: d3_geo_pathCentroidLineEnd,
    polygonStart: function() {
      d3_geo_pathCentroid.lineStart = d3_geo_pathCentroidRingStart;
    },
    polygonEnd: function() {
      d3_geo_pathCentroid.point = d3_geo_pathCentroidPoint;
      d3_geo_pathCentroid.lineStart = d3_geo_pathCentroidLineStart;
      d3_geo_pathCentroid.lineEnd = d3_geo_pathCentroidLineEnd;
    }
  };
  function d3_geo_pathCentroidPoint(x, y) {
    d3_geo_centroidX0 += x;
    d3_geo_centroidY0 += y;
    ++d3_geo_centroidZ0;
  }
  function d3_geo_pathCentroidLineStart() {
    var x0, y0;
    d3_geo_pathCentroid.point = function(x, y) {
      d3_geo_pathCentroid.point = nextPoint;
      d3_geo_pathCentroidPoint(x0 = x, y0 = y);
    };
    function nextPoint(x, y) {
      var dx = x - x0, dy = y - y0, z = Math.sqrt(dx * dx + dy * dy);
      d3_geo_centroidX1 += z * (x0 + x) / 2;
      d3_geo_centroidY1 += z * (y0 + y) / 2;
      d3_geo_centroidZ1 += z;
      d3_geo_pathCentroidPoint(x0 = x, y0 = y);
    }
  }
  function d3_geo_pathCentroidLineEnd() {
    d3_geo_pathCentroid.point = d3_geo_pathCentroidPoint;
  }
  function d3_geo_pathCentroidRingStart() {
    var x00, y00, x0, y0;
    d3_geo_pathCentroid.point = function(x, y) {
      d3_geo_pathCentroid.point = nextPoint;
      d3_geo_pathCentroidPoint(x00 = x0 = x, y00 = y0 = y);
    };
    function nextPoint(x, y) {
      var dx = x - x0, dy = y - y0, z = Math.sqrt(dx * dx + dy * dy);
      d3_geo_centroidX1 += z * (x0 + x) / 2;
      d3_geo_centroidY1 += z * (y0 + y) / 2;
      d3_geo_centroidZ1 += z;
      z = y0 * x - x0 * y;
      d3_geo_centroidX2 += z * (x0 + x);
      d3_geo_centroidY2 += z * (y0 + y);
      d3_geo_centroidZ2 += z * 3;
      d3_geo_pathCentroidPoint(x0 = x, y0 = y);
    }
    d3_geo_pathCentroid.lineEnd = function() {
      nextPoint(x00, y00);
    };
  }
  function d3_geo_pathContext(context) {
    var pointRadius = 4.5;
    var stream = {
      point: point,
      lineStart: function() {
        stream.point = pointLineStart;
      },
      lineEnd: lineEnd,
      polygonStart: function() {
        stream.lineEnd = lineEndPolygon;
      },
      polygonEnd: function() {
        stream.lineEnd = lineEnd;
        stream.point = point;
      },
      pointRadius: function(_) {
        pointRadius = _;
        return stream;
      },
      result: d3_noop
    };
    function point(x, y) {
      context.moveTo(x + pointRadius, y);
      context.arc(x, y, pointRadius, 0, τ);
    }
    function pointLineStart(x, y) {
      context.moveTo(x, y);
      stream.point = pointLine;
    }
    function pointLine(x, y) {
      context.lineTo(x, y);
    }
    function lineEnd() {
      stream.point = point;
    }
    function lineEndPolygon() {
      context.closePath();
    }
    return stream;
  }
  function d3_geo_resample(project) {
    var δ2 = .5, cosMinDistance = Math.cos(30 * d3_radians), maxDepth = 16;
    function resample(stream) {
      return (maxDepth ? resampleRecursive : resampleNone)(stream);
    }
    function resampleNone(stream) {
      return d3_geo_transformPoint(stream, function(x, y) {
        x = project(x, y);
        stream.point(x[0], x[1]);
      });
    }
    function resampleRecursive(stream) {
      var λ00, φ00, x00, y00, a00, b00, c00, λ0, x0, y0, a0, b0, c0;
      var resample = {
        point: point,
        lineStart: lineStart,
        lineEnd: lineEnd,
        polygonStart: function() {
          stream.polygonStart();
          resample.lineStart = ringStart;
        },
        polygonEnd: function() {
          stream.polygonEnd();
          resample.lineStart = lineStart;
        }
      };
      function point(x, y) {
        x = project(x, y);
        stream.point(x[0], x[1]);
      }
      function lineStart() {
        x0 = NaN;
        resample.point = linePoint;
        stream.lineStart();
      }
      function linePoint(λ, φ) {
        var c = d3_geo_cartesian([ λ, φ ]), p = project(λ, φ);
        resampleLineTo(x0, y0, λ0, a0, b0, c0, x0 = p[0], y0 = p[1], λ0 = λ, a0 = c[0], b0 = c[1], c0 = c[2], maxDepth, stream);
        stream.point(x0, y0);
      }
      function lineEnd() {
        resample.point = point;
        stream.lineEnd();
      }
      function ringStart() {
        lineStart();
        resample.point = ringPoint;
        resample.lineEnd = ringEnd;
      }
      function ringPoint(λ, φ) {
        linePoint(λ00 = λ, φ00 = φ), x00 = x0, y00 = y0, a00 = a0, b00 = b0, c00 = c0;
        resample.point = linePoint;
      }
      function ringEnd() {
        resampleLineTo(x0, y0, λ0, a0, b0, c0, x00, y00, λ00, a00, b00, c00, maxDepth, stream);
        resample.lineEnd = lineEnd;
        lineEnd();
      }
      return resample;
    }
    function resampleLineTo(x0, y0, λ0, a0, b0, c0, x1, y1, λ1, a1, b1, c1, depth, stream) {
      var dx = x1 - x0, dy = y1 - y0, d2 = dx * dx + dy * dy;
      if (d2 > 4 * δ2 && depth--) {
        var a = a0 + a1, b = b0 + b1, c = c0 + c1, m = Math.sqrt(a * a + b * b + c * c), φ2 = Math.asin(c /= m), λ2 = abs(abs(c) - 1) < ε || abs(λ0 - λ1) < ε ? (λ0 + λ1) / 2 : Math.atan2(b, a), p = project(λ2, φ2), x2 = p[0], y2 = p[1], dx2 = x2 - x0, dy2 = y2 - y0, dz = dy * dx2 - dx * dy2;
        if (dz * dz / d2 > δ2 || abs((dx * dx2 + dy * dy2) / d2 - .5) > .3 || a0 * a1 + b0 * b1 + c0 * c1 < cosMinDistance) {
          resampleLineTo(x0, y0, λ0, a0, b0, c0, x2, y2, λ2, a /= m, b /= m, c, depth, stream);
          stream.point(x2, y2);
          resampleLineTo(x2, y2, λ2, a, b, c, x1, y1, λ1, a1, b1, c1, depth, stream);
        }
      }
    }
    resample.precision = function(_) {
      if (!arguments.length) return Math.sqrt(δ2);
      maxDepth = (δ2 = _ * _) > 0 && 16;
      return resample;
    };
    return resample;
  }
  d3.geo.path = function() {
    var pointRadius = 4.5, projection, context, projectStream, contextStream, cacheStream;
    function path(object) {
      if (object) {
        if (typeof pointRadius === "function") contextStream.pointRadius(+pointRadius.apply(this, arguments));
        if (!cacheStream || !cacheStream.valid) cacheStream = projectStream(contextStream);
        d3.geo.stream(object, cacheStream);
      }
      return contextStream.result();
    }
    path.area = function(object) {
      d3_geo_pathAreaSum = 0;
      d3.geo.stream(object, projectStream(d3_geo_pathArea));
      return d3_geo_pathAreaSum;
    };
    path.centroid = function(object) {
      d3_geo_centroidX0 = d3_geo_centroidY0 = d3_geo_centroidZ0 = d3_geo_centroidX1 = d3_geo_centroidY1 = d3_geo_centroidZ1 = d3_geo_centroidX2 = d3_geo_centroidY2 = d3_geo_centroidZ2 = 0;
      d3.geo.stream(object, projectStream(d3_geo_pathCentroid));
      return d3_geo_centroidZ2 ? [ d3_geo_centroidX2 / d3_geo_centroidZ2, d3_geo_centroidY2 / d3_geo_centroidZ2 ] : d3_geo_centroidZ1 ? [ d3_geo_centroidX1 / d3_geo_centroidZ1, d3_geo_centroidY1 / d3_geo_centroidZ1 ] : d3_geo_centroidZ0 ? [ d3_geo_centroidX0 / d3_geo_centroidZ0, d3_geo_centroidY0 / d3_geo_centroidZ0 ] : [ NaN, NaN ];
    };
    path.bounds = function(object) {
      d3_geo_pathBoundsX1 = d3_geo_pathBoundsY1 = -(d3_geo_pathBoundsX0 = d3_geo_pathBoundsY0 = Infinity);
      d3.geo.stream(object, projectStream(d3_geo_pathBounds));
      return [ [ d3_geo_pathBoundsX0, d3_geo_pathBoundsY0 ], [ d3_geo_pathBoundsX1, d3_geo_pathBoundsY1 ] ];
    };
    path.projection = function(_) {
      if (!arguments.length) return projection;
      projectStream = (projection = _) ? _.stream || d3_geo_pathProjectStream(_) : d3_identity;
      return reset();
    };
    path.context = function(_) {
      if (!arguments.length) return context;
      contextStream = (context = _) == null ? new d3_geo_pathBuffer() : new d3_geo_pathContext(_);
      if (typeof pointRadius !== "function") contextStream.pointRadius(pointRadius);
      return reset();
    };
    path.pointRadius = function(_) {
      if (!arguments.length) return pointRadius;
      pointRadius = typeof _ === "function" ? _ : (contextStream.pointRadius(+_), +_);
      return path;
    };
    function reset() {
      cacheStream = null;
      return path;
    }
    return path.projection(d3.geo.albersUsa()).context(null);
  };
  function d3_geo_pathProjectStream(project) {
    var resample = d3_geo_resample(function(x, y) {
      return project([ x * d3_degrees, y * d3_degrees ]);
    });
    return function(stream) {
      return d3_geo_projectionRadians(resample(stream));
    };
  }
  d3.geo.transform = function(methods) {
    return {
      stream: function(stream) {
        var transform = new d3_geo_transform(stream);
        for (var k in methods) transform[k] = methods[k];
        return transform;
      }
    };
  };
  function d3_geo_transform(stream) {
    this.stream = stream;
  }
  d3_geo_transform.prototype = {
    point: function(x, y) {
      this.stream.point(x, y);
    },
    sphere: function() {
      this.stream.sphere();
    },
    lineStart: function() {
      this.stream.lineStart();
    },
    lineEnd: function() {
      this.stream.lineEnd();
    },
    polygonStart: function() {
      this.stream.polygonStart();
    },
    polygonEnd: function() {
      this.stream.polygonEnd();
    }
  };
  function d3_geo_transformPoint(stream, point) {
    return {
      point: point,
      sphere: function() {
        stream.sphere();
      },
      lineStart: function() {
        stream.lineStart();
      },
      lineEnd: function() {
        stream.lineEnd();
      },
      polygonStart: function() {
        stream.polygonStart();
      },
      polygonEnd: function() {
        stream.polygonEnd();
      }
    };
  }
  d3.geo.projection = d3_geo_projection;
  d3.geo.projectionMutator = d3_geo_projectionMutator;
  function d3_geo_projection(project) {
    return d3_geo_projectionMutator(function() {
      return project;
    })();
  }
  function d3_geo_projectionMutator(projectAt) {
    var project, rotate, projectRotate, projectResample = d3_geo_resample(function(x, y) {
      x = project(x, y);
      return [ x[0] * k + δx, δy - x[1] * k ];
    }), k = 150, x = 480, y = 250, λ = 0, φ = 0, δλ = 0, δφ = 0, δγ = 0, δx, δy, preclip = d3_geo_clipAntimeridian, postclip = d3_identity, clipAngle = null, clipExtent = null, stream;
    function projection(point) {
      point = projectRotate(point[0] * d3_radians, point[1] * d3_radians);
      return [ point[0] * k + δx, δy - point[1] * k ];
    }
    function invert(point) {
      point = projectRotate.invert((point[0] - δx) / k, (δy - point[1]) / k);
      return point && [ point[0] * d3_degrees, point[1] * d3_degrees ];
    }
    projection.stream = function(output) {
      if (stream) stream.valid = false;
      stream = d3_geo_projectionRadians(preclip(rotate, projectResample(postclip(output))));
      stream.valid = true;
      return stream;
    };
    projection.clipAngle = function(_) {
      if (!arguments.length) return clipAngle;
      preclip = _ == null ? (clipAngle = _, d3_geo_clipAntimeridian) : d3_geo_clipCircle((clipAngle = +_) * d3_radians);
      return invalidate();
    };
    projection.clipExtent = function(_) {
      if (!arguments.length) return clipExtent;
      clipExtent = _;
      postclip = _ ? d3_geo_clipExtent(_[0][0], _[0][1], _[1][0], _[1][1]) : d3_identity;
      return invalidate();
    };
    projection.scale = function(_) {
      if (!arguments.length) return k;
      k = +_;
      return reset();
    };
    projection.translate = function(_) {
      if (!arguments.length) return [ x, y ];
      x = +_[0];
      y = +_[1];
      return reset();
    };
    projection.center = function(_) {
      if (!arguments.length) return [ λ * d3_degrees, φ * d3_degrees ];
      λ = _[0] % 360 * d3_radians;
      φ = _[1] % 360 * d3_radians;
      return reset();
    };
    projection.rotate = function(_) {
      if (!arguments.length) return [ δλ * d3_degrees, δφ * d3_degrees, δγ * d3_degrees ];
      δλ = _[0] % 360 * d3_radians;
      δφ = _[1] % 360 * d3_radians;
      δγ = _.length > 2 ? _[2] % 360 * d3_radians : 0;
      return reset();
    };
    d3.rebind(projection, projectResample, "precision");
    function reset() {
      projectRotate = d3_geo_compose(rotate = d3_geo_rotation(δλ, δφ, δγ), project);
      var center = project(λ, φ);
      δx = x - center[0] * k;
      δy = y + center[1] * k;
      return invalidate();
    }
    function invalidate() {
      if (stream) stream.valid = false, stream = null;
      return projection;
    }
    return function() {
      project = projectAt.apply(this, arguments);
      projection.invert = project.invert && invert;
      return reset();
    };
  }
  function d3_geo_projectionRadians(stream) {
    return d3_geo_transformPoint(stream, function(x, y) {
      stream.point(x * d3_radians, y * d3_radians);
    });
  }
  function d3_geo_equirectangular(λ, φ) {
    return [ λ, φ ];
  }
  (d3.geo.equirectangular = function() {
    return d3_geo_projection(d3_geo_equirectangular);
  }).raw = d3_geo_equirectangular.invert = d3_geo_equirectangular;
  d3.geo.rotation = function(rotate) {
    rotate = d3_geo_rotation(rotate[0] % 360 * d3_radians, rotate[1] * d3_radians, rotate.length > 2 ? rotate[2] * d3_radians : 0);
    function forward(coordinates) {
      coordinates = rotate(coordinates[0] * d3_radians, coordinates[1] * d3_radians);
      return coordinates[0] *= d3_degrees, coordinates[1] *= d3_degrees, coordinates;
    }
    forward.invert = function(coordinates) {
      coordinates = rotate.invert(coordinates[0] * d3_radians, coordinates[1] * d3_radians);
      return coordinates[0] *= d3_degrees, coordinates[1] *= d3_degrees, coordinates;
    };
    return forward;
  };
  function d3_geo_identityRotation(λ, φ) {
    return [ λ > π ? λ - τ : λ < -π ? λ + τ : λ, φ ];
  }
  d3_geo_identityRotation.invert = d3_geo_equirectangular;
  function d3_geo_rotation(δλ, δφ, δγ) {
    return δλ ? δφ || δγ ? d3_geo_compose(d3_geo_rotationλ(δλ), d3_geo_rotationφγ(δφ, δγ)) : d3_geo_rotationλ(δλ) : δφ || δγ ? d3_geo_rotationφγ(δφ, δγ) : d3_geo_identityRotation;
  }
  function d3_geo_forwardRotationλ(δλ) {
    return function(λ, φ) {
      return λ += δλ, [ λ > π ? λ - τ : λ < -π ? λ + τ : λ, φ ];
    };
  }
  function d3_geo_rotationλ(δλ) {
    var rotation = d3_geo_forwardRotationλ(δλ);
    rotation.invert = d3_geo_forwardRotationλ(-δλ);
    return rotation;
  }
  function d3_geo_rotationφγ(δφ, δγ) {
    var cosδφ = Math.cos(δφ), sinδφ = Math.sin(δφ), cosδγ = Math.cos(δγ), sinδγ = Math.sin(δγ);
    function rotation(λ, φ) {
      var cosφ = Math.cos(φ), x = Math.cos(λ) * cosφ, y = Math.sin(λ) * cosφ, z = Math.sin(φ), k = z * cosδφ + x * sinδφ;
      return [ Math.atan2(y * cosδγ - k * sinδγ, x * cosδφ - z * sinδφ), d3_asin(k * cosδγ + y * sinδγ) ];
    }
    rotation.invert = function(λ, φ) {
      var cosφ = Math.cos(φ), x = Math.cos(λ) * cosφ, y = Math.sin(λ) * cosφ, z = Math.sin(φ), k = z * cosδγ - y * sinδγ;
      return [ Math.atan2(y * cosδγ + z * sinδγ, x * cosδφ + k * sinδφ), d3_asin(k * cosδφ - x * sinδφ) ];
    };
    return rotation;
  }
  d3.geo.circle = function() {
    var origin = [ 0, 0 ], angle, precision = 6, interpolate;
    function circle() {
      var center = typeof origin === "function" ? origin.apply(this, arguments) : origin, rotate = d3_geo_rotation(-center[0] * d3_radians, -center[1] * d3_radians, 0).invert, ring = [];
      interpolate(null, null, 1, {
        point: function(x, y) {
          ring.push(x = rotate(x, y));
          x[0] *= d3_degrees, x[1] *= d3_degrees;
        }
      });
      return {
        type: "Polygon",
        coordinates: [ ring ]
      };
    }
    circle.origin = function(x) {
      if (!arguments.length) return origin;
      origin = x;
      return circle;
    };
    circle.angle = function(x) {
      if (!arguments.length) return angle;
      interpolate = d3_geo_circleInterpolate((angle = +x) * d3_radians, precision * d3_radians);
      return circle;
    };
    circle.precision = function(_) {
      if (!arguments.length) return precision;
      interpolate = d3_geo_circleInterpolate(angle * d3_radians, (precision = +_) * d3_radians);
      return circle;
    };
    return circle.angle(90);
  };
  function d3_geo_circleInterpolate(radius, precision) {
    var cr = Math.cos(radius), sr = Math.sin(radius);
    return function(from, to, direction, listener) {
      var step = direction * precision;
      if (from != null) {
        from = d3_geo_circleAngle(cr, from);
        to = d3_geo_circleAngle(cr, to);
        if (direction > 0 ? from < to : from > to) from += direction * τ;
      } else {
        from = radius + direction * τ;
        to = radius - .5 * step;
      }
      for (var point, t = from; direction > 0 ? t > to : t < to; t -= step) {
        listener.point((point = d3_geo_spherical([ cr, -sr * Math.cos(t), -sr * Math.sin(t) ]))[0], point[1]);
      }
    };
  }
  function d3_geo_circleAngle(cr, point) {
    var a = d3_geo_cartesian(point);
    a[0] -= cr;
    d3_geo_cartesianNormalize(a);
    var angle = d3_acos(-a[1]);
    return ((-a[2] < 0 ? -angle : angle) + 2 * Math.PI - ε) % (2 * Math.PI);
  }
  d3.geo.distance = function(a, b) {
    var Δλ = (b[0] - a[0]) * d3_radians, φ0 = a[1] * d3_radians, φ1 = b[1] * d3_radians, sinΔλ = Math.sin(Δλ), cosΔλ = Math.cos(Δλ), sinφ0 = Math.sin(φ0), cosφ0 = Math.cos(φ0), sinφ1 = Math.sin(φ1), cosφ1 = Math.cos(φ1), t;
    return Math.atan2(Math.sqrt((t = cosφ1 * sinΔλ) * t + (t = cosφ0 * sinφ1 - sinφ0 * cosφ1 * cosΔλ) * t), sinφ0 * sinφ1 + cosφ0 * cosφ1 * cosΔλ);
  };
  d3.geo.graticule = function() {
    var x1, x0, X1, X0, y1, y0, Y1, Y0, dx = 10, dy = dx, DX = 90, DY = 360, x, y, X, Y, precision = 2.5;
    function graticule() {
      return {
        type: "MultiLineString",
        coordinates: lines()
      };
    }
    function lines() {
      return d3.range(Math.ceil(X0 / DX) * DX, X1, DX).map(X).concat(d3.range(Math.ceil(Y0 / DY) * DY, Y1, DY).map(Y)).concat(d3.range(Math.ceil(x0 / dx) * dx, x1, dx).filter(function(x) {
        return abs(x % DX) > ε;
      }).map(x)).concat(d3.range(Math.ceil(y0 / dy) * dy, y1, dy).filter(function(y) {
        return abs(y % DY) > ε;
      }).map(y));
    }
    graticule.lines = function() {
      return lines().map(function(coordinates) {
        return {
          type: "LineString",
          coordinates: coordinates
        };
      });
    };
    graticule.outline = function() {
      return {
        type: "Polygon",
        coordinates: [ X(X0).concat(Y(Y1).slice(1), X(X1).reverse().slice(1), Y(Y0).reverse().slice(1)) ]
      };
    };
    graticule.extent = function(_) {
      if (!arguments.length) return graticule.minorExtent();
      return graticule.majorExtent(_).minorExtent(_);
    };
    graticule.majorExtent = function(_) {
      if (!arguments.length) return [ [ X0, Y0 ], [ X1, Y1 ] ];
      X0 = +_[0][0], X1 = +_[1][0];
      Y0 = +_[0][1], Y1 = +_[1][1];
      if (X0 > X1) _ = X0, X0 = X1, X1 = _;
      if (Y0 > Y1) _ = Y0, Y0 = Y1, Y1 = _;
      return graticule.precision(precision);
    };
    graticule.minorExtent = function(_) {
      if (!arguments.length) return [ [ x0, y0 ], [ x1, y1 ] ];
      x0 = +_[0][0], x1 = +_[1][0];
      y0 = +_[0][1], y1 = +_[1][1];
      if (x0 > x1) _ = x0, x0 = x1, x1 = _;
      if (y0 > y1) _ = y0, y0 = y1, y1 = _;
      return graticule.precision(precision);
    };
    graticule.step = function(_) {
      if (!arguments.length) return graticule.minorStep();
      return graticule.majorStep(_).minorStep(_);
    };
    graticule.majorStep = function(_) {
      if (!arguments.length) return [ DX, DY ];
      DX = +_[0], DY = +_[1];
      return graticule;
    };
    graticule.minorStep = function(_) {
      if (!arguments.length) return [ dx, dy ];
      dx = +_[0], dy = +_[1];
      return graticule;
    };
    graticule.precision = function(_) {
      if (!arguments.length) return precision;
      precision = +_;
      x = d3_geo_graticuleX(y0, y1, 90);
      y = d3_geo_graticuleY(x0, x1, precision);
      X = d3_geo_graticuleX(Y0, Y1, 90);
      Y = d3_geo_graticuleY(X0, X1, precision);
      return graticule;
    };
    return graticule.majorExtent([ [ -180, -90 + ε ], [ 180, 90 - ε ] ]).minorExtent([ [ -180, -80 - ε ], [ 180, 80 + ε ] ]);
  };
  function d3_geo_graticuleX(y0, y1, dy) {
    var y = d3.range(y0, y1 - ε, dy).concat(y1);
    return function(x) {
      return y.map(function(y) {
        return [ x, y ];
      });
    };
  }
  function d3_geo_graticuleY(x0, x1, dx) {
    var x = d3.range(x0, x1 - ε, dx).concat(x1);
    return function(y) {
      return x.map(function(x) {
        return [ x, y ];
      });
    };
  }
  function d3_source(d) {
    return d.source;
  }
  function d3_target(d) {
    return d.target;
  }
  d3.geo.greatArc = function() {
    var source = d3_source, source_, target = d3_target, target_;
    function greatArc() {
      return {
        type: "LineString",
        coordinates: [ source_ || source.apply(this, arguments), target_ || target.apply(this, arguments) ]
      };
    }
    greatArc.distance = function() {
      return d3.geo.distance(source_ || source.apply(this, arguments), target_ || target.apply(this, arguments));
    };
    greatArc.source = function(_) {
      if (!arguments.length) return source;
      source = _, source_ = typeof _ === "function" ? null : _;
      return greatArc;
    };
    greatArc.target = function(_) {
      if (!arguments.length) return target;
      target = _, target_ = typeof _ === "function" ? null : _;
      return greatArc;
    };
    greatArc.precision = function() {
      return arguments.length ? greatArc : 0;
    };
    return greatArc;
  };
  d3.geo.interpolate = function(source, target) {
    return d3_geo_interpolate(source[0] * d3_radians, source[1] * d3_radians, target[0] * d3_radians, target[1] * d3_radians);
  };
  function d3_geo_interpolate(x0, y0, x1, y1) {
    var cy0 = Math.cos(y0), sy0 = Math.sin(y0), cy1 = Math.cos(y1), sy1 = Math.sin(y1), kx0 = cy0 * Math.cos(x0), ky0 = cy0 * Math.sin(x0), kx1 = cy1 * Math.cos(x1), ky1 = cy1 * Math.sin(x1), d = 2 * Math.asin(Math.sqrt(d3_haversin(y1 - y0) + cy0 * cy1 * d3_haversin(x1 - x0))), k = 1 / Math.sin(d);
    var interpolate = d ? function(t) {
      var B = Math.sin(t *= d) * k, A = Math.sin(d - t) * k, x = A * kx0 + B * kx1, y = A * ky0 + B * ky1, z = A * sy0 + B * sy1;
      return [ Math.atan2(y, x) * d3_degrees, Math.atan2(z, Math.sqrt(x * x + y * y)) * d3_degrees ];
    } : function() {
      return [ x0 * d3_degrees, y0 * d3_degrees ];
    };
    interpolate.distance = d;
    return interpolate;
  }
  d3.geo.length = function(object) {
    d3_geo_lengthSum = 0;
    d3.geo.stream(object, d3_geo_length);
    return d3_geo_lengthSum;
  };
  var d3_geo_lengthSum;
  var d3_geo_length = {
    sphere: d3_noop,
    point: d3_noop,
    lineStart: d3_geo_lengthLineStart,
    lineEnd: d3_noop,
    polygonStart: d3_noop,
    polygonEnd: d3_noop
  };
  function d3_geo_lengthLineStart() {
    var λ0, sinφ0, cosφ0;
    d3_geo_length.point = function(λ, φ) {
      λ0 = λ * d3_radians, sinφ0 = Math.sin(φ *= d3_radians), cosφ0 = Math.cos(φ);
      d3_geo_length.point = nextPoint;
    };
    d3_geo_length.lineEnd = function() {
      d3_geo_length.point = d3_geo_length.lineEnd = d3_noop;
    };
    function nextPoint(λ, φ) {
      var sinφ = Math.sin(φ *= d3_radians), cosφ = Math.cos(φ), t = abs((λ *= d3_radians) - λ0), cosΔλ = Math.cos(t);
      d3_geo_lengthSum += Math.atan2(Math.sqrt((t = cosφ * Math.sin(t)) * t + (t = cosφ0 * sinφ - sinφ0 * cosφ * cosΔλ) * t), sinφ0 * sinφ + cosφ0 * cosφ * cosΔλ);
      λ0 = λ, sinφ0 = sinφ, cosφ0 = cosφ;
    }
  }
  function d3_geo_azimuthal(scale, angle) {
    function azimuthal(λ, φ) {
      var cosλ = Math.cos(λ), cosφ = Math.cos(φ), k = scale(cosλ * cosφ);
      return [ k * cosφ * Math.sin(λ), k * Math.sin(φ) ];
    }
    azimuthal.invert = function(x, y) {
      var ρ = Math.sqrt(x * x + y * y), c = angle(ρ), sinc = Math.sin(c), cosc = Math.cos(c);
      return [ Math.atan2(x * sinc, ρ * cosc), Math.asin(ρ && y * sinc / ρ) ];
    };
    return azimuthal;
  }
  var d3_geo_azimuthalEqualArea = d3_geo_azimuthal(function(cosλcosφ) {
    return Math.sqrt(2 / (1 + cosλcosφ));
  }, function(ρ) {
    return 2 * Math.asin(ρ / 2);
  });
  (d3.geo.azimuthalEqualArea = function() {
    return d3_geo_projection(d3_geo_azimuthalEqualArea);
  }).raw = d3_geo_azimuthalEqualArea;
  var d3_geo_azimuthalEquidistant = d3_geo_azimuthal(function(cosλcosφ) {
    var c = Math.acos(cosλcosφ);
    return c && c / Math.sin(c);
  }, d3_identity);
  (d3.geo.azimuthalEquidistant = function() {
    return d3_geo_projection(d3_geo_azimuthalEquidistant);
  }).raw = d3_geo_azimuthalEquidistant;
  function d3_geo_conicConformal(φ0, φ1) {
    var cosφ0 = Math.cos(φ0), t = function(φ) {
      return Math.tan(π / 4 + φ / 2);
    }, n = φ0 === φ1 ? Math.sin(φ0) : Math.log(cosφ0 / Math.cos(φ1)) / Math.log(t(φ1) / t(φ0)), F = cosφ0 * Math.pow(t(φ0), n) / n;
    if (!n) return d3_geo_mercator;
    function forward(λ, φ) {
      if (F > 0) {
        if (φ < -halfπ + ε) φ = -halfπ + ε;
      } else {
        if (φ > halfπ - ε) φ = halfπ - ε;
      }
      var ρ = F / Math.pow(t(φ), n);
      return [ ρ * Math.sin(n * λ), F - ρ * Math.cos(n * λ) ];
    }
    forward.invert = function(x, y) {
      var ρ0_y = F - y, ρ = d3_sgn(n) * Math.sqrt(x * x + ρ0_y * ρ0_y);
      return [ Math.atan2(x, ρ0_y) / n, 2 * Math.atan(Math.pow(F / ρ, 1 / n)) - halfπ ];
    };
    return forward;
  }
  (d3.geo.conicConformal = function() {
    return d3_geo_conic(d3_geo_conicConformal);
  }).raw = d3_geo_conicConformal;
  function d3_geo_conicEquidistant(φ0, φ1) {
    var cosφ0 = Math.cos(φ0), n = φ0 === φ1 ? Math.sin(φ0) : (cosφ0 - Math.cos(φ1)) / (φ1 - φ0), G = cosφ0 / n + φ0;
    if (abs(n) < ε) return d3_geo_equirectangular;
    function forward(λ, φ) {
      var ρ = G - φ;
      return [ ρ * Math.sin(n * λ), G - ρ * Math.cos(n * λ) ];
    }
    forward.invert = function(x, y) {
      var ρ0_y = G - y;
      return [ Math.atan2(x, ρ0_y) / n, G - d3_sgn(n) * Math.sqrt(x * x + ρ0_y * ρ0_y) ];
    };
    return forward;
  }
  (d3.geo.conicEquidistant = function() {
    return d3_geo_conic(d3_geo_conicEquidistant);
  }).raw = d3_geo_conicEquidistant;
  var d3_geo_gnomonic = d3_geo_azimuthal(function(cosλcosφ) {
    return 1 / cosλcosφ;
  }, Math.atan);
  (d3.geo.gnomonic = function() {
    return d3_geo_projection(d3_geo_gnomonic);
  }).raw = d3_geo_gnomonic;
  function d3_geo_mercator(λ, φ) {
    return [ λ, Math.log(Math.tan(π / 4 + φ / 2)) ];
  }
  d3_geo_mercator.invert = function(x, y) {
    return [ x, 2 * Math.atan(Math.exp(y)) - halfπ ];
  };
  function d3_geo_mercatorProjection(project) {
    var m = d3_geo_projection(project), scale = m.scale, translate = m.translate, clipExtent = m.clipExtent, clipAuto;
    m.scale = function() {
      var v = scale.apply(m, arguments);
      return v === m ? clipAuto ? m.clipExtent(null) : m : v;
    };
    m.translate = function() {
      var v = translate.apply(m, arguments);
      return v === m ? clipAuto ? m.clipExtent(null) : m : v;
    };
    m.clipExtent = function(_) {
      var v = clipExtent.apply(m, arguments);
      if (v === m) {
        if (clipAuto = _ == null) {
          var k = π * scale(), t = translate();
          clipExtent([ [ t[0] - k, t[1] - k ], [ t[0] + k, t[1] + k ] ]);
        }
      } else if (clipAuto) {
        v = null;
      }
      return v;
    };
    return m.clipExtent(null);
  }
  (d3.geo.mercator = function() {
    return d3_geo_mercatorProjection(d3_geo_mercator);
  }).raw = d3_geo_mercator;
  var d3_geo_orthographic = d3_geo_azimuthal(function() {
    return 1;
  }, Math.asin);
  (d3.geo.orthographic = function() {
    return d3_geo_projection(d3_geo_orthographic);
  }).raw = d3_geo_orthographic;
  var d3_geo_stereographic = d3_geo_azimuthal(function(cosλcosφ) {
    return 1 / (1 + cosλcosφ);
  }, function(ρ) {
    return 2 * Math.atan(ρ);
  });
  (d3.geo.stereographic = function() {
    return d3_geo_projection(d3_geo_stereographic);
  }).raw = d3_geo_stereographic;
  function d3_geo_transverseMercator(λ, φ) {
    return [ Math.log(Math.tan(π / 4 + φ / 2)), -λ ];
  }
  d3_geo_transverseMercator.invert = function(x, y) {
    return [ -y, 2 * Math.atan(Math.exp(x)) - halfπ ];
  };
  (d3.geo.transverseMercator = function() {
    var projection = d3_geo_mercatorProjection(d3_geo_transverseMercator), center = projection.center, rotate = projection.rotate;
    projection.center = function(_) {
      return _ ? center([ -_[1], _[0] ]) : (_ = center(), [ _[1], -_[0] ]);
    };
    projection.rotate = function(_) {
      return _ ? rotate([ _[0], _[1], _.length > 2 ? _[2] + 90 : 90 ]) : (_ = rotate(), 
      [ _[0], _[1], _[2] - 90 ]);
    };
    return rotate([ 0, 0, 90 ]);
  }).raw = d3_geo_transverseMercator;
  d3.geom = {};
  function d3_geom_pointX(d) {
    return d[0];
  }
  function d3_geom_pointY(d) {
    return d[1];
  }
  d3.geom.hull = function(vertices) {
    var x = d3_geom_pointX, y = d3_geom_pointY;
    if (arguments.length) return hull(vertices);
    function hull(data) {
      if (data.length < 3) return [];
      var fx = d3_functor(x), fy = d3_functor(y), i, n = data.length, points = [], flippedPoints = [];
      for (i = 0; i < n; i++) {
        points.push([ +fx.call(this, data[i], i), +fy.call(this, data[i], i), i ]);
      }
      points.sort(d3_geom_hullOrder);
      for (i = 0; i < n; i++) flippedPoints.push([ points[i][0], -points[i][1] ]);
      var upper = d3_geom_hullUpper(points), lower = d3_geom_hullUpper(flippedPoints);
      var skipLeft = lower[0] === upper[0], skipRight = lower[lower.length - 1] === upper[upper.length - 1], polygon = [];
      for (i = upper.length - 1; i >= 0; --i) polygon.push(data[points[upper[i]][2]]);
      for (i = +skipLeft; i < lower.length - skipRight; ++i) polygon.push(data[points[lower[i]][2]]);
      return polygon;
    }
    hull.x = function(_) {
      return arguments.length ? (x = _, hull) : x;
    };
    hull.y = function(_) {
      return arguments.length ? (y = _, hull) : y;
    };
    return hull;
  };
  function d3_geom_hullUpper(points) {
    var n = points.length, hull = [ 0, 1 ], hs = 2;
    for (var i = 2; i < n; i++) {
      while (hs > 1 && d3_cross2d(points[hull[hs - 2]], points[hull[hs - 1]], points[i]) <= 0) --hs;
      hull[hs++] = i;
    }
    return hull.slice(0, hs);
  }
  function d3_geom_hullOrder(a, b) {
    return a[0] - b[0] || a[1] - b[1];
  }
  d3.geom.polygon = function(coordinates) {
    d3_subclass(coordinates, d3_geom_polygonPrototype);
    return coordinates;
  };
  var d3_geom_polygonPrototype = d3.geom.polygon.prototype = [];
  d3_geom_polygonPrototype.area = function() {
    var i = -1, n = this.length, a, b = this[n - 1], area = 0;
    while (++i < n) {
      a = b;
      b = this[i];
      area += a[1] * b[0] - a[0] * b[1];
    }
    return area * .5;
  };
  d3_geom_polygonPrototype.centroid = function(k) {
    var i = -1, n = this.length, x = 0, y = 0, a, b = this[n - 1], c;
    if (!arguments.length) k = -1 / (6 * this.area());
    while (++i < n) {
      a = b;
      b = this[i];
      c = a[0] * b[1] - b[0] * a[1];
      x += (a[0] + b[0]) * c;
      y += (a[1] + b[1]) * c;
    }
    return [ x * k, y * k ];
  };
  d3_geom_polygonPrototype.clip = function(subject) {
    var input, closed = d3_geom_polygonClosed(subject), i = -1, n = this.length - d3_geom_polygonClosed(this), j, m, a = this[n - 1], b, c, d;
    while (++i < n) {
      input = subject.slice();
      subject.length = 0;
      b = this[i];
      c = input[(m = input.length - closed) - 1];
      j = -1;
      while (++j < m) {
        d = input[j];
        if (d3_geom_polygonInside(d, a, b)) {
          if (!d3_geom_polygonInside(c, a, b)) {
            subject.push(d3_geom_polygonIntersect(c, d, a, b));
          }
          subject.push(d);
        } else if (d3_geom_polygonInside(c, a, b)) {
          subject.push(d3_geom_polygonIntersect(c, d, a, b));
        }
        c = d;
      }
      if (closed) subject.push(subject[0]);
      a = b;
    }
    return subject;
  };
  function d3_geom_polygonInside(p, a, b) {
    return (b[0] - a[0]) * (p[1] - a[1]) < (b[1] - a[1]) * (p[0] - a[0]);
  }
  function d3_geom_polygonIntersect(c, d, a, b) {
    var x1 = c[0], x3 = a[0], x21 = d[0] - x1, x43 = b[0] - x3, y1 = c[1], y3 = a[1], y21 = d[1] - y1, y43 = b[1] - y3, ua = (x43 * (y1 - y3) - y43 * (x1 - x3)) / (y43 * x21 - x43 * y21);
    return [ x1 + ua * x21, y1 + ua * y21 ];
  }
  function d3_geom_polygonClosed(coordinates) {
    var a = coordinates[0], b = coordinates[coordinates.length - 1];
    return !(a[0] - b[0] || a[1] - b[1]);
  }
  var d3_geom_voronoiEdges, d3_geom_voronoiCells, d3_geom_voronoiBeaches, d3_geom_voronoiBeachPool = [], d3_geom_voronoiFirstCircle, d3_geom_voronoiCircles, d3_geom_voronoiCirclePool = [];
  function d3_geom_voronoiBeach() {
    d3_geom_voronoiRedBlackNode(this);
    this.edge = this.site = this.circle = null;
  }
  function d3_geom_voronoiCreateBeach(site) {
    var beach = d3_geom_voronoiBeachPool.pop() || new d3_geom_voronoiBeach();
    beach.site = site;
    return beach;
  }
  function d3_geom_voronoiDetachBeach(beach) {
    d3_geom_voronoiDetachCircle(beach);
    d3_geom_voronoiBeaches.remove(beach);
    d3_geom_voronoiBeachPool.push(beach);
    d3_geom_voronoiRedBlackNode(beach);
  }
  function d3_geom_voronoiRemoveBeach(beach) {
    var circle = beach.circle, x = circle.x, y = circle.cy, vertex = {
      x: x,
      y: y
    }, previous = beach.P, next = beach.N, disappearing = [ beach ];
    d3_geom_voronoiDetachBeach(beach);
    var lArc = previous;
    while (lArc.circle && abs(x - lArc.circle.x) < ε && abs(y - lArc.circle.cy) < ε) {
      previous = lArc.P;
      disappearing.unshift(lArc);
      d3_geom_voronoiDetachBeach(lArc);
      lArc = previous;
    }
    disappearing.unshift(lArc);
    d3_geom_voronoiDetachCircle(lArc);
    var rArc = next;
    while (rArc.circle && abs(x - rArc.circle.x) < ε && abs(y - rArc.circle.cy) < ε) {
      next = rArc.N;
      disappearing.push(rArc);
      d3_geom_voronoiDetachBeach(rArc);
      rArc = next;
    }
    disappearing.push(rArc);
    d3_geom_voronoiDetachCircle(rArc);
    var nArcs = disappearing.length, iArc;
    for (iArc = 1; iArc < nArcs; ++iArc) {
      rArc = disappearing[iArc];
      lArc = disappearing[iArc - 1];
      d3_geom_voronoiSetEdgeEnd(rArc.edge, lArc.site, rArc.site, vertex);
    }
    lArc = disappearing[0];
    rArc = disappearing[nArcs - 1];
    rArc.edge = d3_geom_voronoiCreateEdge(lArc.site, rArc.site, null, vertex);
    d3_geom_voronoiAttachCircle(lArc);
    d3_geom_voronoiAttachCircle(rArc);
  }
  function d3_geom_voronoiAddBeach(site) {
    var x = site.x, directrix = site.y, lArc, rArc, dxl, dxr, node = d3_geom_voronoiBeaches._;
    while (node) {
      dxl = d3_geom_voronoiLeftBreakPoint(node, directrix) - x;
      if (dxl > ε) node = node.L; else {
        dxr = x - d3_geom_voronoiRightBreakPoint(node, directrix);
        if (dxr > ε) {
          if (!node.R) {
            lArc = node;
            break;
          }
          node = node.R;
        } else {
          if (dxl > -ε) {
            lArc = node.P;
            rArc = node;
          } else if (dxr > -ε) {
            lArc = node;
            rArc = node.N;
          } else {
            lArc = rArc = node;
          }
          break;
        }
      }
    }
    var newArc = d3_geom_voronoiCreateBeach(site);
    d3_geom_voronoiBeaches.insert(lArc, newArc);
    if (!lArc && !rArc) return;
    if (lArc === rArc) {
      d3_geom_voronoiDetachCircle(lArc);
      rArc = d3_geom_voronoiCreateBeach(lArc.site);
      d3_geom_voronoiBeaches.insert(newArc, rArc);
      newArc.edge = rArc.edge = d3_geom_voronoiCreateEdge(lArc.site, newArc.site);
      d3_geom_voronoiAttachCircle(lArc);
      d3_geom_voronoiAttachCircle(rArc);
      return;
    }
    if (!rArc) {
      newArc.edge = d3_geom_voronoiCreateEdge(lArc.site, newArc.site);
      return;
    }
    d3_geom_voronoiDetachCircle(lArc);
    d3_geom_voronoiDetachCircle(rArc);
    var lSite = lArc.site, ax = lSite.x, ay = lSite.y, bx = site.x - ax, by = site.y - ay, rSite = rArc.site, cx = rSite.x - ax, cy = rSite.y - ay, d = 2 * (bx * cy - by * cx), hb = bx * bx + by * by, hc = cx * cx + cy * cy, vertex = {
      x: (cy * hb - by * hc) / d + ax,
      y: (bx * hc - cx * hb) / d + ay
    };
    d3_geom_voronoiSetEdgeEnd(rArc.edge, lSite, rSite, vertex);
    newArc.edge = d3_geom_voronoiCreateEdge(lSite, site, null, vertex);
    rArc.edge = d3_geom_voronoiCreateEdge(site, rSite, null, vertex);
    d3_geom_voronoiAttachCircle(lArc);
    d3_geom_voronoiAttachCircle(rArc);
  }
  function d3_geom_voronoiLeftBreakPoint(arc, directrix) {
    var site = arc.site, rfocx = site.x, rfocy = site.y, pby2 = rfocy - directrix;
    if (!pby2) return rfocx;
    var lArc = arc.P;
    if (!lArc) return -Infinity;
    site = lArc.site;
    var lfocx = site.x, lfocy = site.y, plby2 = lfocy - directrix;
    if (!plby2) return lfocx;
    var hl = lfocx - rfocx, aby2 = 1 / pby2 - 1 / plby2, b = hl / plby2;
    if (aby2) return (-b + Math.sqrt(b * b - 2 * aby2 * (hl * hl / (-2 * plby2) - lfocy + plby2 / 2 + rfocy - pby2 / 2))) / aby2 + rfocx;
    return (rfocx + lfocx) / 2;
  }
  function d3_geom_voronoiRightBreakPoint(arc, directrix) {
    var rArc = arc.N;
    if (rArc) return d3_geom_voronoiLeftBreakPoint(rArc, directrix);
    var site = arc.site;
    return site.y === directrix ? site.x : Infinity;
  }
  function d3_geom_voronoiCell(site) {
    this.site = site;
    this.edges = [];
  }
  d3_geom_voronoiCell.prototype.prepare = function() {
    var halfEdges = this.edges, iHalfEdge = halfEdges.length, edge;
    while (iHalfEdge--) {
      edge = halfEdges[iHalfEdge].edge;
      if (!edge.b || !edge.a) halfEdges.splice(iHalfEdge, 1);
    }
    halfEdges.sort(d3_geom_voronoiHalfEdgeOrder);
    return halfEdges.length;
  };
  function d3_geom_voronoiCloseCells(extent) {
    var x0 = extent[0][0], x1 = extent[1][0], y0 = extent[0][1], y1 = extent[1][1], x2, y2, x3, y3, cells = d3_geom_voronoiCells, iCell = cells.length, cell, iHalfEdge, halfEdges, nHalfEdges, start, end;
    while (iCell--) {
      cell = cells[iCell];
      if (!cell || !cell.prepare()) continue;
      halfEdges = cell.edges;
      nHalfEdges = halfEdges.length;
      iHalfEdge = 0;
      while (iHalfEdge < nHalfEdges) {
        end = halfEdges[iHalfEdge].end(), x3 = end.x, y3 = end.y;
        start = halfEdges[++iHalfEdge % nHalfEdges].start(), x2 = start.x, y2 = start.y;
        if (abs(x3 - x2) > ε || abs(y3 - y2) > ε) {
          halfEdges.splice(iHalfEdge, 0, new d3_geom_voronoiHalfEdge(d3_geom_voronoiCreateBorderEdge(cell.site, end, abs(x3 - x0) < ε && y1 - y3 > ε ? {
            x: x0,
            y: abs(x2 - x0) < ε ? y2 : y1
          } : abs(y3 - y1) < ε && x1 - x3 > ε ? {
            x: abs(y2 - y1) < ε ? x2 : x1,
            y: y1
          } : abs(x3 - x1) < ε && y3 - y0 > ε ? {
            x: x1,
            y: abs(x2 - x1) < ε ? y2 : y0
          } : abs(y3 - y0) < ε && x3 - x0 > ε ? {
            x: abs(y2 - y0) < ε ? x2 : x0,
            y: y0
          } : null), cell.site, null));
          ++nHalfEdges;
        }
      }
    }
  }
  function d3_geom_voronoiHalfEdgeOrder(a, b) {
    return b.angle - a.angle;
  }
  function d3_geom_voronoiCircle() {
    d3_geom_voronoiRedBlackNode(this);
    this.x = this.y = this.arc = this.site = this.cy = null;
  }
  function d3_geom_voronoiAttachCircle(arc) {
    var lArc = arc.P, rArc = arc.N;
    if (!lArc || !rArc) return;
    var lSite = lArc.site, cSite = arc.site, rSite = rArc.site;
    if (lSite === rSite) return;
    var bx = cSite.x, by = cSite.y, ax = lSite.x - bx, ay = lSite.y - by, cx = rSite.x - bx, cy = rSite.y - by;
    var d = 2 * (ax * cy - ay * cx);
    if (d >= -ε2) return;
    var ha = ax * ax + ay * ay, hc = cx * cx + cy * cy, x = (cy * ha - ay * hc) / d, y = (ax * hc - cx * ha) / d, cy = y + by;
    var circle = d3_geom_voronoiCirclePool.pop() || new d3_geom_voronoiCircle();
    circle.arc = arc;
    circle.site = cSite;
    circle.x = x + bx;
    circle.y = cy + Math.sqrt(x * x + y * y);
    circle.cy = cy;
    arc.circle = circle;
    var before = null, node = d3_geom_voronoiCircles._;
    while (node) {
      if (circle.y < node.y || circle.y === node.y && circle.x <= node.x) {
        if (node.L) node = node.L; else {
          before = node.P;
          break;
        }
      } else {
        if (node.R) node = node.R; else {
          before = node;
          break;
        }
      }
    }
    d3_geom_voronoiCircles.insert(before, circle);
    if (!before) d3_geom_voronoiFirstCircle = circle;
  }
  function d3_geom_voronoiDetachCircle(arc) {
    var circle = arc.circle;
    if (circle) {
      if (!circle.P) d3_geom_voronoiFirstCircle = circle.N;
      d3_geom_voronoiCircles.remove(circle);
      d3_geom_voronoiCirclePool.push(circle);
      d3_geom_voronoiRedBlackNode(circle);
      arc.circle = null;
    }
  }
  function d3_geom_voronoiClipEdges(extent) {
    var edges = d3_geom_voronoiEdges, clip = d3_geom_clipLine(extent[0][0], extent[0][1], extent[1][0], extent[1][1]), i = edges.length, e;
    while (i--) {
      e = edges[i];
      if (!d3_geom_voronoiConnectEdge(e, extent) || !clip(e) || abs(e.a.x - e.b.x) < ε && abs(e.a.y - e.b.y) < ε) {
        e.a = e.b = null;
        edges.splice(i, 1);
      }
    }
  }
  function d3_geom_voronoiConnectEdge(edge, extent) {
    var vb = edge.b;
    if (vb) return true;
    var va = edge.a, x0 = extent[0][0], x1 = extent[1][0], y0 = extent[0][1], y1 = extent[1][1], lSite = edge.l, rSite = edge.r, lx = lSite.x, ly = lSite.y, rx = rSite.x, ry = rSite.y, fx = (lx + rx) / 2, fy = (ly + ry) / 2, fm, fb;
    if (ry === ly) {
      if (fx < x0 || fx >= x1) return;
      if (lx > rx) {
        if (!va) va = {
          x: fx,
          y: y0
        }; else if (va.y >= y1) return;
        vb = {
          x: fx,
          y: y1
        };
      } else {
        if (!va) va = {
          x: fx,
          y: y1
        }; else if (va.y < y0) return;
        vb = {
          x: fx,
          y: y0
        };
      }
    } else {
      fm = (lx - rx) / (ry - ly);
      fb = fy - fm * fx;
      if (fm < -1 || fm > 1) {
        if (lx > rx) {
          if (!va) va = {
            x: (y0 - fb) / fm,
            y: y0
          }; else if (va.y >= y1) return;
          vb = {
            x: (y1 - fb) / fm,
            y: y1
          };
        } else {
          if (!va) va = {
            x: (y1 - fb) / fm,
            y: y1
          }; else if (va.y < y0) return;
          vb = {
            x: (y0 - fb) / fm,
            y: y0
          };
        }
      } else {
        if (ly < ry) {
          if (!va) va = {
            x: x0,
            y: fm * x0 + fb
          }; else if (va.x >= x1) return;
          vb = {
            x: x1,
            y: fm * x1 + fb
          };
        } else {
          if (!va) va = {
            x: x1,
            y: fm * x1 + fb
          }; else if (va.x < x0) return;
          vb = {
            x: x0,
            y: fm * x0 + fb
          };
        }
      }
    }
    edge.a = va;
    edge.b = vb;
    return true;
  }
  function d3_geom_voronoiEdge(lSite, rSite) {
    this.l = lSite;
    this.r = rSite;
    this.a = this.b = null;
  }
  function d3_geom_voronoiCreateEdge(lSite, rSite, va, vb) {
    var edge = new d3_geom_voronoiEdge(lSite, rSite);
    d3_geom_voronoiEdges.push(edge);
    if (va) d3_geom_voronoiSetEdgeEnd(edge, lSite, rSite, va);
    if (vb) d3_geom_voronoiSetEdgeEnd(edge, rSite, lSite, vb);
    d3_geom_voronoiCells[lSite.i].edges.push(new d3_geom_voronoiHalfEdge(edge, lSite, rSite));
    d3_geom_voronoiCells[rSite.i].edges.push(new d3_geom_voronoiHalfEdge(edge, rSite, lSite));
    return edge;
  }
  function d3_geom_voronoiCreateBorderEdge(lSite, va, vb) {
    var edge = new d3_geom_voronoiEdge(lSite, null);
    edge.a = va;
    edge.b = vb;
    d3_geom_voronoiEdges.push(edge);
    return edge;
  }
  function d3_geom_voronoiSetEdgeEnd(edge, lSite, rSite, vertex) {
    if (!edge.a && !edge.b) {
      edge.a = vertex;
      edge.l = lSite;
      edge.r = rSite;
    } else if (edge.l === rSite) {
      edge.b = vertex;
    } else {
      edge.a = vertex;
    }
  }
  function d3_geom_voronoiHalfEdge(edge, lSite, rSite) {
    var va = edge.a, vb = edge.b;
    this.edge = edge;
    this.site = lSite;
    this.angle = rSite ? Math.atan2(rSite.y - lSite.y, rSite.x - lSite.x) : edge.l === lSite ? Math.atan2(vb.x - va.x, va.y - vb.y) : Math.atan2(va.x - vb.x, vb.y - va.y);
  }
  d3_geom_voronoiHalfEdge.prototype = {
    start: function() {
      return this.edge.l === this.site ? this.edge.a : this.edge.b;
    },
    end: function() {
      return this.edge.l === this.site ? this.edge.b : this.edge.a;
    }
  };
  function d3_geom_voronoiRedBlackTree() {
    this._ = null;
  }
  function d3_geom_voronoiRedBlackNode(node) {
    node.U = node.C = node.L = node.R = node.P = node.N = null;
  }
  d3_geom_voronoiRedBlackTree.prototype = {
    insert: function(after, node) {
      var parent, grandpa, uncle;
      if (after) {
        node.P = after;
        node.N = after.N;
        if (after.N) after.N.P = node;
        after.N = node;
        if (after.R) {
          after = after.R;
          while (after.L) after = after.L;
          after.L = node;
        } else {
          after.R = node;
        }
        parent = after;
      } else if (this._) {
        after = d3_geom_voronoiRedBlackFirst(this._);
        node.P = null;
        node.N = after;
        after.P = after.L = node;
        parent = after;
      } else {
        node.P = node.N = null;
        this._ = node;
        parent = null;
      }
      node.L = node.R = null;
      node.U = parent;
      node.C = true;
      after = node;
      while (parent && parent.C) {
        grandpa = parent.U;
        if (parent === grandpa.L) {
          uncle = grandpa.R;
          if (uncle && uncle.C) {
            parent.C = uncle.C = false;
            grandpa.C = true;
            after = grandpa;
          } else {
            if (after === parent.R) {
              d3_geom_voronoiRedBlackRotateLeft(this, parent);
              after = parent;
              parent = after.U;
            }
            parent.C = false;
            grandpa.C = true;
            d3_geom_voronoiRedBlackRotateRight(this, grandpa);
          }
        } else {
          uncle = grandpa.L;
          if (uncle && uncle.C) {
            parent.C = uncle.C = false;
            grandpa.C = true;
            after = grandpa;
          } else {
            if (after === parent.L) {
              d3_geom_voronoiRedBlackRotateRight(this, parent);
              after = parent;
              parent = after.U;
            }
            parent.C = false;
            grandpa.C = true;
            d3_geom_voronoiRedBlackRotateLeft(this, grandpa);
          }
        }
        parent = after.U;
      }
      this._.C = false;
    },
    remove: function(node) {
      if (node.N) node.N.P = node.P;
      if (node.P) node.P.N = node.N;
      node.N = node.P = null;
      var parent = node.U, sibling, left = node.L, right = node.R, next, red;
      if (!left) next = right; else if (!right) next = left; else next = d3_geom_voronoiRedBlackFirst(right);
      if (parent) {
        if (parent.L === node) parent.L = next; else parent.R = next;
      } else {
        this._ = next;
      }
      if (left && right) {
        red = next.C;
        next.C = node.C;
        next.L = left;
        left.U = next;
        if (next !== right) {
          parent = next.U;
          next.U = node.U;
          node = next.R;
          parent.L = node;
          next.R = right;
          right.U = next;
        } else {
          next.U = parent;
          parent = next;
          node = next.R;
        }
      } else {
        red = node.C;
        node = next;
      }
      if (node) node.U = parent;
      if (red) return;
      if (node && node.C) {
        node.C = false;
        return;
      }
      do {
        if (node === this._) break;
        if (node === parent.L) {
          sibling = parent.R;
          if (sibling.C) {
            sibling.C = false;
            parent.C = true;
            d3_geom_voronoiRedBlackRotateLeft(this, parent);
            sibling = parent.R;
          }
          if (sibling.L && sibling.L.C || sibling.R && sibling.R.C) {
            if (!sibling.R || !sibling.R.C) {
              sibling.L.C = false;
              sibling.C = true;
              d3_geom_voronoiRedBlackRotateRight(this, sibling);
              sibling = parent.R;
            }
            sibling.C = parent.C;
            parent.C = sibling.R.C = false;
            d3_geom_voronoiRedBlackRotateLeft(this, parent);
            node = this._;
            break;
          }
        } else {
          sibling = parent.L;
          if (sibling.C) {
            sibling.C = false;
            parent.C = true;
            d3_geom_voronoiRedBlackRotateRight(this, parent);
            sibling = parent.L;
          }
          if (sibling.L && sibling.L.C || sibling.R && sibling.R.C) {
            if (!sibling.L || !sibling.L.C) {
              sibling.R.C = false;
              sibling.C = true;
              d3_geom_voronoiRedBlackRotateLeft(this, sibling);
              sibling = parent.L;
            }
            sibling.C = parent.C;
            parent.C = sibling.L.C = false;
            d3_geom_voronoiRedBlackRotateRight(this, parent);
            node = this._;
            break;
          }
        }
        sibling.C = true;
        node = parent;
        parent = parent.U;
      } while (!node.C);
      if (node) node.C = false;
    }
  };
  function d3_geom_voronoiRedBlackRotateLeft(tree, node) {
    var p = node, q = node.R, parent = p.U;
    if (parent) {
      if (parent.L === p) parent.L = q; else parent.R = q;
    } else {
      tree._ = q;
    }
    q.U = parent;
    p.U = q;
    p.R = q.L;
    if (p.R) p.R.U = p;
    q.L = p;
  }
  function d3_geom_voronoiRedBlackRotateRight(tree, node) {
    var p = node, q = node.L, parent = p.U;
    if (parent) {
      if (parent.L === p) parent.L = q; else parent.R = q;
    } else {
      tree._ = q;
    }
    q.U = parent;
    p.U = q;
    p.L = q.R;
    if (p.L) p.L.U = p;
    q.R = p;
  }
  function d3_geom_voronoiRedBlackFirst(node) {
    while (node.L) node = node.L;
    return node;
  }
  function d3_geom_voronoi(sites, bbox) {
    var site = sites.sort(d3_geom_voronoiVertexOrder).pop(), x0, y0, circle;
    d3_geom_voronoiEdges = [];
    d3_geom_voronoiCells = new Array(sites.length);
    d3_geom_voronoiBeaches = new d3_geom_voronoiRedBlackTree();
    d3_geom_voronoiCircles = new d3_geom_voronoiRedBlackTree();
    while (true) {
      circle = d3_geom_voronoiFirstCircle;
      if (site && (!circle || site.y < circle.y || site.y === circle.y && site.x < circle.x)) {
        if (site.x !== x0 || site.y !== y0) {
          d3_geom_voronoiCells[site.i] = new d3_geom_voronoiCell(site);
          d3_geom_voronoiAddBeach(site);
          x0 = site.x, y0 = site.y;
        }
        site = sites.pop();
      } else if (circle) {
        d3_geom_voronoiRemoveBeach(circle.arc);
      } else {
        break;
      }
    }
    if (bbox) d3_geom_voronoiClipEdges(bbox), d3_geom_voronoiCloseCells(bbox);
    var diagram = {
      cells: d3_geom_voronoiCells,
      edges: d3_geom_voronoiEdges
    };
    d3_geom_voronoiBeaches = d3_geom_voronoiCircles = d3_geom_voronoiEdges = d3_geom_voronoiCells = null;
    return diagram;
  }
  function d3_geom_voronoiVertexOrder(a, b) {
    return b.y - a.y || b.x - a.x;
  }
  d3.geom.voronoi = function(points) {
    var x = d3_geom_pointX, y = d3_geom_pointY, fx = x, fy = y, clipExtent = d3_geom_voronoiClipExtent;
    if (points) return voronoi(points);
    function voronoi(data) {
      var polygons = new Array(data.length), x0 = clipExtent[0][0], y0 = clipExtent[0][1], x1 = clipExtent[1][0], y1 = clipExtent[1][1];
      d3_geom_voronoi(sites(data), clipExtent).cells.forEach(function(cell, i) {
        var edges = cell.edges, site = cell.site, polygon = polygons[i] = edges.length ? edges.map(function(e) {
          var s = e.start();
          return [ s.x, s.y ];
        }) : site.x >= x0 && site.x <= x1 && site.y >= y0 && site.y <= y1 ? [ [ x0, y1 ], [ x1, y1 ], [ x1, y0 ], [ x0, y0 ] ] : [];
        polygon.point = data[i];
      });
      return polygons;
    }
    function sites(data) {
      return data.map(function(d, i) {
        return {
          x: Math.round(fx(d, i) / ε) * ε,
          y: Math.round(fy(d, i) / ε) * ε,
          i: i
        };
      });
    }
    voronoi.links = function(data) {
      return d3_geom_voronoi(sites(data)).edges.filter(function(edge) {
        return edge.l && edge.r;
      }).map(function(edge) {
        return {
          source: data[edge.l.i],
          target: data[edge.r.i]
        };
      });
    };
    voronoi.triangles = function(data) {
      var triangles = [];
      d3_geom_voronoi(sites(data)).cells.forEach(function(cell, i) {
        var site = cell.site, edges = cell.edges.sort(d3_geom_voronoiHalfEdgeOrder), j = -1, m = edges.length, e0, s0, e1 = edges[m - 1].edge, s1 = e1.l === site ? e1.r : e1.l;
        while (++j < m) {
          e0 = e1;
          s0 = s1;
          e1 = edges[j].edge;
          s1 = e1.l === site ? e1.r : e1.l;
          if (i < s0.i && i < s1.i && d3_geom_voronoiTriangleArea(site, s0, s1) < 0) {
            triangles.push([ data[i], data[s0.i], data[s1.i] ]);
          }
        }
      });
      return triangles;
    };
    voronoi.x = function(_) {
      return arguments.length ? (fx = d3_functor(x = _), voronoi) : x;
    };
    voronoi.y = function(_) {
      return arguments.length ? (fy = d3_functor(y = _), voronoi) : y;
    };
    voronoi.clipExtent = function(_) {
      if (!arguments.length) return clipExtent === d3_geom_voronoiClipExtent ? null : clipExtent;
      clipExtent = _ == null ? d3_geom_voronoiClipExtent : _;
      return voronoi;
    };
    voronoi.size = function(_) {
      if (!arguments.length) return clipExtent === d3_geom_voronoiClipExtent ? null : clipExtent && clipExtent[1];
      return voronoi.clipExtent(_ && [ [ 0, 0 ], _ ]);
    };
    return voronoi;
  };
  var d3_geom_voronoiClipExtent = [ [ -1e6, -1e6 ], [ 1e6, 1e6 ] ];
  function d3_geom_voronoiTriangleArea(a, b, c) {
    return (a.x - c.x) * (b.y - a.y) - (a.x - b.x) * (c.y - a.y);
  }
  d3.geom.delaunay = function(vertices) {
    return d3.geom.voronoi().triangles(vertices);
  };
  d3.geom.quadtree = function(points, x1, y1, x2, y2) {
    var x = d3_geom_pointX, y = d3_geom_pointY, compat;
    if (compat = arguments.length) {
      x = d3_geom_quadtreeCompatX;
      y = d3_geom_quadtreeCompatY;
      if (compat === 3) {
        y2 = y1;
        x2 = x1;
        y1 = x1 = 0;
      }
      return quadtree(points);
    }
    function quadtree(data) {
      var d, fx = d3_functor(x), fy = d3_functor(y), xs, ys, i, n, x1_, y1_, x2_, y2_;
      if (x1 != null) {
        x1_ = x1, y1_ = y1, x2_ = x2, y2_ = y2;
      } else {
        x2_ = y2_ = -(x1_ = y1_ = Infinity);
        xs = [], ys = [];
        n = data.length;
        if (compat) for (i = 0; i < n; ++i) {
          d = data[i];
          if (d.x < x1_) x1_ = d.x;
          if (d.y < y1_) y1_ = d.y;
          if (d.x > x2_) x2_ = d.x;
          if (d.y > y2_) y2_ = d.y;
          xs.push(d.x);
          ys.push(d.y);
        } else for (i = 0; i < n; ++i) {
          var x_ = +fx(d = data[i], i), y_ = +fy(d, i);
          if (x_ < x1_) x1_ = x_;
          if (y_ < y1_) y1_ = y_;
          if (x_ > x2_) x2_ = x_;
          if (y_ > y2_) y2_ = y_;
          xs.push(x_);
          ys.push(y_);
        }
      }
      var dx = x2_ - x1_, dy = y2_ - y1_;
      if (dx > dy) y2_ = y1_ + dx; else x2_ = x1_ + dy;
      function insert(n, d, x, y, x1, y1, x2, y2) {
        if (isNaN(x) || isNaN(y)) return;
        if (n.leaf) {
          var nx = n.x, ny = n.y;
          if (nx != null) {
            if (abs(nx - x) + abs(ny - y) < .01) {
              insertChild(n, d, x, y, x1, y1, x2, y2);
            } else {
              var nPoint = n.point;
              n.x = n.y = n.point = null;
              insertChild(n, nPoint, nx, ny, x1, y1, x2, y2);
              insertChild(n, d, x, y, x1, y1, x2, y2);
            }
          } else {
            n.x = x, n.y = y, n.point = d;
          }
        } else {
          insertChild(n, d, x, y, x1, y1, x2, y2);
        }
      }
      function insertChild(n, d, x, y, x1, y1, x2, y2) {
        var xm = (x1 + x2) * .5, ym = (y1 + y2) * .5, right = x >= xm, below = y >= ym, i = below << 1 | right;
        n.leaf = false;
        n = n.nodes[i] || (n.nodes[i] = d3_geom_quadtreeNode());
        if (right) x1 = xm; else x2 = xm;
        if (below) y1 = ym; else y2 = ym;
        insert(n, d, x, y, x1, y1, x2, y2);
      }
      var root = d3_geom_quadtreeNode();
      root.add = function(d) {
        insert(root, d, +fx(d, ++i), +fy(d, i), x1_, y1_, x2_, y2_);
      };
      root.visit = function(f) {
        d3_geom_quadtreeVisit(f, root, x1_, y1_, x2_, y2_);
      };
      root.find = function(point) {
        return d3_geom_quadtreeFind(root, point[0], point[1], x1_, y1_, x2_, y2_);
      };
      i = -1;
      if (x1 == null) {
        while (++i < n) {
          insert(root, data[i], xs[i], ys[i], x1_, y1_, x2_, y2_);
        }
        --i;
      } else data.forEach(root.add);
      xs = ys = data = d = null;
      return root;
    }
    quadtree.x = function(_) {
      return arguments.length ? (x = _, quadtree) : x;
    };
    quadtree.y = function(_) {
      return arguments.length ? (y = _, quadtree) : y;
    };
    quadtree.extent = function(_) {
      if (!arguments.length) return x1 == null ? null : [ [ x1, y1 ], [ x2, y2 ] ];
      if (_ == null) x1 = y1 = x2 = y2 = null; else x1 = +_[0][0], y1 = +_[0][1], x2 = +_[1][0], 
      y2 = +_[1][1];
      return quadtree;
    };
    quadtree.size = function(_) {
      if (!arguments.length) return x1 == null ? null : [ x2 - x1, y2 - y1 ];
      if (_ == null) x1 = y1 = x2 = y2 = null; else x1 = y1 = 0, x2 = +_[0], y2 = +_[1];
      return quadtree;
    };
    return quadtree;
  };
  function d3_geom_quadtreeCompatX(d) {
    return d.x;
  }
  function d3_geom_quadtreeCompatY(d) {
    return d.y;
  }
  function d3_geom_quadtreeNode() {
    return {
      leaf: true,
      nodes: [],
      point: null,
      x: null,
      y: null
    };
  }
  function d3_geom_quadtreeVisit(f, node, x1, y1, x2, y2) {
    if (!f(node, x1, y1, x2, y2)) {
      var sx = (x1 + x2) * .5, sy = (y1 + y2) * .5, children = node.nodes;
      if (children[0]) d3_geom_quadtreeVisit(f, children[0], x1, y1, sx, sy);
      if (children[1]) d3_geom_quadtreeVisit(f, children[1], sx, y1, x2, sy);
      if (children[2]) d3_geom_quadtreeVisit(f, children[2], x1, sy, sx, y2);
      if (children[3]) d3_geom_quadtreeVisit(f, children[3], sx, sy, x2, y2);
    }
  }
  function d3_geom_quadtreeFind(root, x, y, x0, y0, x3, y3) {
    var minDistance2 = Infinity, closestPoint;
    (function find(node, x1, y1, x2, y2) {
      if (x1 > x3 || y1 > y3 || x2 < x0 || y2 < y0) return;
      if (point = node.point) {
        var point, dx = x - node.x, dy = y - node.y, distance2 = dx * dx + dy * dy;
        if (distance2 < minDistance2) {
          var distance = Math.sqrt(minDistance2 = distance2);
          x0 = x - distance, y0 = y - distance;
          x3 = x + distance, y3 = y + distance;
          closestPoint = point;
        }
      }
      var children = node.nodes, xm = (x1 + x2) * .5, ym = (y1 + y2) * .5, right = x >= xm, below = y >= ym;
      for (var i = below << 1 | right, j = i + 4; i < j; ++i) {
        if (node = children[i & 3]) switch (i & 3) {
         case 0:
          find(node, x1, y1, xm, ym);
          break;

         case 1:
          find(node, xm, y1, x2, ym);
          break;

         case 2:
          find(node, x1, ym, xm, y2);
          break;

         case 3:
          find(node, xm, ym, x2, y2);
          break;
        }
      }
    })(root, x0, y0, x3, y3);
    return closestPoint;
  }
  d3.interpolateRgb = d3_interpolateRgb;
  function d3_interpolateRgb(a, b) {
    a = d3.rgb(a);
    b = d3.rgb(b);
    var ar = a.r, ag = a.g, ab = a.b, br = b.r - ar, bg = b.g - ag, bb = b.b - ab;
    return function(t) {
      return "#" + d3_rgb_hex(Math.round(ar + br * t)) + d3_rgb_hex(Math.round(ag + bg * t)) + d3_rgb_hex(Math.round(ab + bb * t));
    };
  }
  d3.interpolateObject = d3_interpolateObject;
  function d3_interpolateObject(a, b) {
    var i = {}, c = {}, k;
    for (k in a) {
      if (k in b) {
        i[k] = d3_interpolate(a[k], b[k]);
      } else {
        c[k] = a[k];
      }
    }
    for (k in b) {
      if (!(k in a)) {
        c[k] = b[k];
      }
    }
    return function(t) {
      for (k in i) c[k] = i[k](t);
      return c;
    };
  }
  d3.interpolateNumber = d3_interpolateNumber;
  function d3_interpolateNumber(a, b) {
    a = +a, b = +b;
    return function(t) {
      return a * (1 - t) + b * t;
    };
  }
  d3.interpolateString = d3_interpolateString;
  function d3_interpolateString(a, b) {
    var bi = d3_interpolate_numberA.lastIndex = d3_interpolate_numberB.lastIndex = 0, am, bm, bs, i = -1, s = [], q = [];
    a = a + "", b = b + "";
    while ((am = d3_interpolate_numberA.exec(a)) && (bm = d3_interpolate_numberB.exec(b))) {
      if ((bs = bm.index) > bi) {
        bs = b.slice(bi, bs);
        if (s[i]) s[i] += bs; else s[++i] = bs;
      }
      if ((am = am[0]) === (bm = bm[0])) {
        if (s[i]) s[i] += bm; else s[++i] = bm;
      } else {
        s[++i] = null;
        q.push({
          i: i,
          x: d3_interpolateNumber(am, bm)
        });
      }
      bi = d3_interpolate_numberB.lastIndex;
    }
    if (bi < b.length) {
      bs = b.slice(bi);
      if (s[i]) s[i] += bs; else s[++i] = bs;
    }
    return s.length < 2 ? q[0] ? (b = q[0].x, function(t) {
      return b(t) + "";
    }) : function() {
      return b;
    } : (b = q.length, function(t) {
      for (var i = 0, o; i < b; ++i) s[(o = q[i]).i] = o.x(t);
      return s.join("");
    });
  }
  var d3_interpolate_numberA = /[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g, d3_interpolate_numberB = new RegExp(d3_interpolate_numberA.source, "g");
  d3.interpolate = d3_interpolate;
  function d3_interpolate(a, b) {
    var i = d3.interpolators.length, f;
    while (--i >= 0 && !(f = d3.interpolators[i](a, b))) ;
    return f;
  }
  d3.interpolators = [ function(a, b) {
    var t = typeof b;
    return (t === "string" ? d3_rgb_names.has(b.toLowerCase()) || /^(#|rgb\(|hsl\()/i.test(b) ? d3_interpolateRgb : d3_interpolateString : b instanceof d3_color ? d3_interpolateRgb : Array.isArray(b) ? d3_interpolateArray : t === "object" && isNaN(b) ? d3_interpolateObject : d3_interpolateNumber)(a, b);
  } ];
  d3.interpolateArray = d3_interpolateArray;
  function d3_interpolateArray(a, b) {
    var x = [], c = [], na = a.length, nb = b.length, n0 = Math.min(a.length, b.length), i;
    for (i = 0; i < n0; ++i) x.push(d3_interpolate(a[i], b[i]));
    for (;i < na; ++i) c[i] = a[i];
    for (;i < nb; ++i) c[i] = b[i];
    return function(t) {
      for (i = 0; i < n0; ++i) c[i] = x[i](t);
      return c;
    };
  }
  var d3_ease_default = function() {
    return d3_identity;
  };
  var d3_ease = d3.map({
    linear: d3_ease_default,
    poly: d3_ease_poly,
    quad: function() {
      return d3_ease_quad;
    },
    cubic: function() {
      return d3_ease_cubic;
    },
    sin: function() {
      return d3_ease_sin;
    },
    exp: function() {
      return d3_ease_exp;
    },
    circle: function() {
      return d3_ease_circle;
    },
    elastic: d3_ease_elastic,
    back: d3_ease_back,
    bounce: function() {
      return d3_ease_bounce;
    }
  });
  var d3_ease_mode = d3.map({
    "in": d3_identity,
    out: d3_ease_reverse,
    "in-out": d3_ease_reflect,
    "out-in": function(f) {
      return d3_ease_reflect(d3_ease_reverse(f));
    }
  });
  d3.ease = function(name) {
    var i = name.indexOf("-"), t = i >= 0 ? name.slice(0, i) : name, m = i >= 0 ? name.slice(i + 1) : "in";
    t = d3_ease.get(t) || d3_ease_default;
    m = d3_ease_mode.get(m) || d3_identity;
    return d3_ease_clamp(m(t.apply(null, d3_arraySlice.call(arguments, 1))));
  };
  function d3_ease_clamp(f) {
    return function(t) {
      return t <= 0 ? 0 : t >= 1 ? 1 : f(t);
    };
  }
  function d3_ease_reverse(f) {
    return function(t) {
      return 1 - f(1 - t);
    };
  }
  function d3_ease_reflect(f) {
    return function(t) {
      return .5 * (t < .5 ? f(2 * t) : 2 - f(2 - 2 * t));
    };
  }
  function d3_ease_quad(t) {
    return t * t;
  }
  function d3_ease_cubic(t) {
    return t * t * t;
  }
  function d3_ease_cubicInOut(t) {
    if (t <= 0) return 0;
    if (t >= 1) return 1;
    var t2 = t * t, t3 = t2 * t;
    return 4 * (t < .5 ? t3 : 3 * (t - t2) + t3 - .75);
  }
  function d3_ease_poly(e) {
    return function(t) {
      return Math.pow(t, e);
    };
  }
  function d3_ease_sin(t) {
    return 1 - Math.cos(t * halfπ);
  }
  function d3_ease_exp(t) {
    return Math.pow(2, 10 * (t - 1));
  }
  function d3_ease_circle(t) {
    return 1 - Math.sqrt(1 - t * t);
  }
  function d3_ease_elastic(a, p) {
    var s;
    if (arguments.length < 2) p = .45;
    if (arguments.length) s = p / τ * Math.asin(1 / a); else a = 1, s = p / 4;
    return function(t) {
      return 1 + a * Math.pow(2, -10 * t) * Math.sin((t - s) * τ / p);
    };
  }
  function d3_ease_back(s) {
    if (!s) s = 1.70158;
    return function(t) {
      return t * t * ((s + 1) * t - s);
    };
  }
  function d3_ease_bounce(t) {
    return t < 1 / 2.75 ? 7.5625 * t * t : t < 2 / 2.75 ? 7.5625 * (t -= 1.5 / 2.75) * t + .75 : t < 2.5 / 2.75 ? 7.5625 * (t -= 2.25 / 2.75) * t + .9375 : 7.5625 * (t -= 2.625 / 2.75) * t + .984375;
  }
  d3.interpolateHcl = d3_interpolateHcl;
  function d3_interpolateHcl(a, b) {
    a = d3.hcl(a);
    b = d3.hcl(b);
    var ah = a.h, ac = a.c, al = a.l, bh = b.h - ah, bc = b.c - ac, bl = b.l - al;
    if (isNaN(bc)) bc = 0, ac = isNaN(ac) ? b.c : ac;
    if (isNaN(bh)) bh = 0, ah = isNaN(ah) ? b.h : ah; else if (bh > 180) bh -= 360; else if (bh < -180) bh += 360;
    return function(t) {
      return d3_hcl_lab(ah + bh * t, ac + bc * t, al + bl * t) + "";
    };
  }
  d3.interpolateHsl = d3_interpolateHsl;
  function d3_interpolateHsl(a, b) {
    a = d3.hsl(a);
    b = d3.hsl(b);
    var ah = a.h, as = a.s, al = a.l, bh = b.h - ah, bs = b.s - as, bl = b.l - al;
    if (isNaN(bs)) bs = 0, as = isNaN(as) ? b.s : as;
    if (isNaN(bh)) bh = 0, ah = isNaN(ah) ? b.h : ah; else if (bh > 180) bh -= 360; else if (bh < -180) bh += 360;
    return function(t) {
      return d3_hsl_rgb(ah + bh * t, as + bs * t, al + bl * t) + "";
    };
  }
  d3.interpolateLab = d3_interpolateLab;
  function d3_interpolateLab(a, b) {
    a = d3.lab(a);
    b = d3.lab(b);
    var al = a.l, aa = a.a, ab = a.b, bl = b.l - al, ba = b.a - aa, bb = b.b - ab;
    return function(t) {
      return d3_lab_rgb(al + bl * t, aa + ba * t, ab + bb * t) + "";
    };
  }
  d3.interpolateRound = d3_interpolateRound;
  function d3_interpolateRound(a, b) {
    b -= a;
    return function(t) {
      return Math.round(a + b * t);
    };
  }
  d3.transform = function(string) {
    var g = d3_document.createElementNS(d3.ns.prefix.svg, "g");
    return (d3.transform = function(string) {
      if (string != null) {
        g.setAttribute("transform", string);
        var t = g.transform.baseVal.consolidate();
      }
      return new d3_transform(t ? t.matrix : d3_transformIdentity);
    })(string);
  };
  function d3_transform(m) {
    var r0 = [ m.a, m.b ], r1 = [ m.c, m.d ], kx = d3_transformNormalize(r0), kz = d3_transformDot(r0, r1), ky = d3_transformNormalize(d3_transformCombine(r1, r0, -kz)) || 0;
    if (r0[0] * r1[1] < r1[0] * r0[1]) {
      r0[0] *= -1;
      r0[1] *= -1;
      kx *= -1;
      kz *= -1;
    }
    this.rotate = (kx ? Math.atan2(r0[1], r0[0]) : Math.atan2(-r1[0], r1[1])) * d3_degrees;
    this.translate = [ m.e, m.f ];
    this.scale = [ kx, ky ];
    this.skew = ky ? Math.atan2(kz, ky) * d3_degrees : 0;
  }
  d3_transform.prototype.toString = function() {
    return "translate(" + this.translate + ")rotate(" + this.rotate + ")skewX(" + this.skew + ")scale(" + this.scale + ")";
  };
  function d3_transformDot(a, b) {
    return a[0] * b[0] + a[1] * b[1];
  }
  function d3_transformNormalize(a) {
    var k = Math.sqrt(d3_transformDot(a, a));
    if (k) {
      a[0] /= k;
      a[1] /= k;
    }
    return k;
  }
  function d3_transformCombine(a, b, k) {
    a[0] += k * b[0];
    a[1] += k * b[1];
    return a;
  }
  var d3_transformIdentity = {
    a: 1,
    b: 0,
    c: 0,
    d: 1,
    e: 0,
    f: 0
  };
  d3.interpolateTransform = d3_interpolateTransform;
  function d3_interpolateTransformPop(s) {
    return s.length ? s.pop() + "," : "";
  }
  function d3_interpolateTranslate(ta, tb, s, q) {
    if (ta[0] !== tb[0] || ta[1] !== tb[1]) {
      var i = s.push("translate(", null, ",", null, ")");
      q.push({
        i: i - 4,
        x: d3_interpolateNumber(ta[0], tb[0])
      }, {
        i: i - 2,
        x: d3_interpolateNumber(ta[1], tb[1])
      });
    } else if (tb[0] || tb[1]) {
      s.push("translate(" + tb + ")");
    }
  }
  function d3_interpolateRotate(ra, rb, s, q) {
    if (ra !== rb) {
      if (ra - rb > 180) rb += 360; else if (rb - ra > 180) ra += 360;
      q.push({
        i: s.push(d3_interpolateTransformPop(s) + "rotate(", null, ")") - 2,
        x: d3_interpolateNumber(ra, rb)
      });
    } else if (rb) {
      s.push(d3_interpolateTransformPop(s) + "rotate(" + rb + ")");
    }
  }
  function d3_interpolateSkew(wa, wb, s, q) {
    if (wa !== wb) {
      q.push({
        i: s.push(d3_interpolateTransformPop(s) + "skewX(", null, ")") - 2,
        x: d3_interpolateNumber(wa, wb)
      });
    } else if (wb) {
      s.push(d3_interpolateTransformPop(s) + "skewX(" + wb + ")");
    }
  }
  function d3_interpolateScale(ka, kb, s, q) {
    if (ka[0] !== kb[0] || ka[1] !== kb[1]) {
      var i = s.push(d3_interpolateTransformPop(s) + "scale(", null, ",", null, ")");
      q.push({
        i: i - 4,
        x: d3_interpolateNumber(ka[0], kb[0])
      }, {
        i: i - 2,
        x: d3_interpolateNumber(ka[1], kb[1])
      });
    } else if (kb[0] !== 1 || kb[1] !== 1) {
      s.push(d3_interpolateTransformPop(s) + "scale(" + kb + ")");
    }
  }
  function d3_interpolateTransform(a, b) {
    var s = [], q = [];
    a = d3.transform(a), b = d3.transform(b);
    d3_interpolateTranslate(a.translate, b.translate, s, q);
    d3_interpolateRotate(a.rotate, b.rotate, s, q);
    d3_interpolateSkew(a.skew, b.skew, s, q);
    d3_interpolateScale(a.scale, b.scale, s, q);
    a = b = null;
    return function(t) {
      var i = -1, n = q.length, o;
      while (++i < n) s[(o = q[i]).i] = o.x(t);
      return s.join("");
    };
  }
  function d3_uninterpolateNumber(a, b) {
    b = (b -= a = +a) || 1 / b;
    return function(x) {
      return (x - a) / b;
    };
  }
  function d3_uninterpolateClamp(a, b) {
    b = (b -= a = +a) || 1 / b;
    return function(x) {
      return Math.max(0, Math.min(1, (x - a) / b));
    };
  }
  d3.layout = {};
  d3.layout.bundle = function() {
    return function(links) {
      var paths = [], i = -1, n = links.length;
      while (++i < n) paths.push(d3_layout_bundlePath(links[i]));
      return paths;
    };
  };
  function d3_layout_bundlePath(link) {
    var start = link.source, end = link.target, lca = d3_layout_bundleLeastCommonAncestor(start, end), points = [ start ];
    while (start !== lca) {
      start = start.parent;
      points.push(start);
    }
    var k = points.length;
    while (end !== lca) {
      points.splice(k, 0, end);
      end = end.parent;
    }
    return points;
  }
  function d3_layout_bundleAncestors(node) {
    var ancestors = [], parent = node.parent;
    while (parent != null) {
      ancestors.push(node);
      node = parent;
      parent = parent.parent;
    }
    ancestors.push(node);
    return ancestors;
  }
  function d3_layout_bundleLeastCommonAncestor(a, b) {
    if (a === b) return a;
    var aNodes = d3_layout_bundleAncestors(a), bNodes = d3_layout_bundleAncestors(b), aNode = aNodes.pop(), bNode = bNodes.pop(), sharedNode = null;
    while (aNode === bNode) {
      sharedNode = aNode;
      aNode = aNodes.pop();
      bNode = bNodes.pop();
    }
    return sharedNode;
  }
  d3.layout.chord = function() {
    var chord = {}, chords, groups, matrix, n, padding = 0, sortGroups, sortSubgroups, sortChords;
    function relayout() {
      var subgroups = {}, groupSums = [], groupIndex = d3.range(n), subgroupIndex = [], k, x, x0, i, j;
      chords = [];
      groups = [];
      k = 0, i = -1;
      while (++i < n) {
        x = 0, j = -1;
        while (++j < n) {
          x += matrix[i][j];
        }
        groupSums.push(x);
        subgroupIndex.push(d3.range(n));
        k += x;
      }
      if (sortGroups) {
        groupIndex.sort(function(a, b) {
          return sortGroups(groupSums[a], groupSums[b]);
        });
      }
      if (sortSubgroups) {
        subgroupIndex.forEach(function(d, i) {
          d.sort(function(a, b) {
            return sortSubgroups(matrix[i][a], matrix[i][b]);
          });
        });
      }
      k = (τ - padding * n) / k;
      x = 0, i = -1;
      while (++i < n) {
        x0 = x, j = -1;
        while (++j < n) {
          var di = groupIndex[i], dj = subgroupIndex[di][j], v = matrix[di][dj], a0 = x, a1 = x += v * k;
          subgroups[di + "-" + dj] = {
            index: di,
            subindex: dj,
            startAngle: a0,
            endAngle: a1,
            value: v
          };
        }
        groups[di] = {
          index: di,
          startAngle: x0,
          endAngle: x,
          value: groupSums[di]
        };
        x += padding;
      }
      i = -1;
      while (++i < n) {
        j = i - 1;
        while (++j < n) {
          var source = subgroups[i + "-" + j], target = subgroups[j + "-" + i];
          if (source.value || target.value) {
            chords.push(source.value < target.value ? {
              source: target,
              target: source
            } : {
              source: source,
              target: target
            });
          }
        }
      }
      if (sortChords) resort();
    }
    function resort() {
      chords.sort(function(a, b) {
        return sortChords((a.source.value + a.target.value) / 2, (b.source.value + b.target.value) / 2);
      });
    }
    chord.matrix = function(x) {
      if (!arguments.length) return matrix;
      n = (matrix = x) && matrix.length;
      chords = groups = null;
      return chord;
    };
    chord.padding = function(x) {
      if (!arguments.length) return padding;
      padding = x;
      chords = groups = null;
      return chord;
    };
    chord.sortGroups = function(x) {
      if (!arguments.length) return sortGroups;
      sortGroups = x;
      chords = groups = null;
      return chord;
    };
    chord.sortSubgroups = function(x) {
      if (!arguments.length) return sortSubgroups;
      sortSubgroups = x;
      chords = null;
      return chord;
    };
    chord.sortChords = function(x) {
      if (!arguments.length) return sortChords;
      sortChords = x;
      if (chords) resort();
      return chord;
    };
    chord.chords = function() {
      if (!chords) relayout();
      return chords;
    };
    chord.groups = function() {
      if (!groups) relayout();
      return groups;
    };
    return chord;
  };
  d3.layout.force = function() {
    var force = {}, event = d3.dispatch("start", "tick", "end"), timer, size = [ 1, 1 ], drag, alpha, friction = .9, linkDistance = d3_layout_forceLinkDistance, linkStrength = d3_layout_forceLinkStrength, charge = -30, chargeDistance2 = d3_layout_forceChargeDistance2, gravity = .1, theta2 = .64, nodes = [], links = [], distances, strengths, charges;
    function repulse(node) {
      return function(quad, x1, _, x2) {
        if (quad.point !== node) {
          var dx = quad.cx - node.x, dy = quad.cy - node.y, dw = x2 - x1, dn = dx * dx + dy * dy;
          if (dw * dw / theta2 < dn) {
            if (dn < chargeDistance2) {
              var k = quad.charge / dn;
              node.px -= dx * k;
              node.py -= dy * k;
            }
            return true;
          }
          if (quad.point && dn && dn < chargeDistance2) {
            var k = quad.pointCharge / dn;
            node.px -= dx * k;
            node.py -= dy * k;
          }
        }
        return !quad.charge;
      };
    }
    force.tick = function() {
      if ((alpha *= .99) < .005) {
        timer = null;
        event.end({
          type: "end",
          alpha: alpha = 0
        });
        return true;
      }
      var n = nodes.length, m = links.length, q, i, o, s, t, l, k, x, y;
      for (i = 0; i < m; ++i) {
        o = links[i];
        s = o.source;
        t = o.target;
        x = t.x - s.x;
        y = t.y - s.y;
        if (l = x * x + y * y) {
          l = alpha * strengths[i] * ((l = Math.sqrt(l)) - distances[i]) / l;
          x *= l;
          y *= l;
          t.x -= x * (k = s.weight + t.weight ? s.weight / (s.weight + t.weight) : .5);
          t.y -= y * k;
          s.x += x * (k = 1 - k);
          s.y += y * k;
        }
      }
      if (k = alpha * gravity) {
        x = size[0] / 2;
        y = size[1] / 2;
        i = -1;
        if (k) while (++i < n) {
          o = nodes[i];
          o.x += (x - o.x) * k;
          o.y += (y - o.y) * k;
        }
      }
      if (charge) {
        d3_layout_forceAccumulate(q = d3.geom.quadtree(nodes), alpha, charges);
        i = -1;
        while (++i < n) {
          if (!(o = nodes[i]).fixed) {
            q.visit(repulse(o));
          }
        }
      }
      i = -1;
      while (++i < n) {
        o = nodes[i];
        if (o.fixed) {
          o.x = o.px;
          o.y = o.py;
        } else {
          o.x -= (o.px - (o.px = o.x)) * friction;
          o.y -= (o.py - (o.py = o.y)) * friction;
        }
      }
      event.tick({
        type: "tick",
        alpha: alpha
      });
    };
    force.nodes = function(x) {
      if (!arguments.length) return nodes;
      nodes = x;
      return force;
    };
    force.links = function(x) {
      if (!arguments.length) return links;
      links = x;
      return force;
    };
    force.size = function(x) {
      if (!arguments.length) return size;
      size = x;
      return force;
    };
    force.linkDistance = function(x) {
      if (!arguments.length) return linkDistance;
      linkDistance = typeof x === "function" ? x : +x;
      return force;
    };
    force.distance = force.linkDistance;
    force.linkStrength = function(x) {
      if (!arguments.length) return linkStrength;
      linkStrength = typeof x === "function" ? x : +x;
      return force;
    };
    force.friction = function(x) {
      if (!arguments.length) return friction;
      friction = +x;
      return force;
    };
    force.charge = function(x) {
      if (!arguments.length) return charge;
      charge = typeof x === "function" ? x : +x;
      return force;
    };
    force.chargeDistance = function(x) {
      if (!arguments.length) return Math.sqrt(chargeDistance2);
      chargeDistance2 = x * x;
      return force;
    };
    force.gravity = function(x) {
      if (!arguments.length) return gravity;
      gravity = +x;
      return force;
    };
    force.theta = function(x) {
      if (!arguments.length) return Math.sqrt(theta2);
      theta2 = x * x;
      return force;
    };
    force.alpha = function(x) {
      if (!arguments.length) return alpha;
      x = +x;
      if (alpha) {
        if (x > 0) {
          alpha = x;
        } else {
          timer.c = null, timer.t = NaN, timer = null;
          event.end({
            type: "end",
            alpha: alpha = 0
          });
        }
      } else if (x > 0) {
        event.start({
          type: "start",
          alpha: alpha = x
        });
        timer = d3_timer(force.tick);
      }
      return force;
    };
    force.start = function() {
      var i, n = nodes.length, m = links.length, w = size[0], h = size[1], neighbors, o;
      for (i = 0; i < n; ++i) {
        (o = nodes[i]).index = i;
        o.weight = 0;
      }
      for (i = 0; i < m; ++i) {
        o = links[i];
        if (typeof o.source == "number") o.source = nodes[o.source];
        if (typeof o.target == "number") o.target = nodes[o.target];
        ++o.source.weight;
        ++o.target.weight;
      }
      for (i = 0; i < n; ++i) {
        o = nodes[i];
        if (isNaN(o.x)) o.x = position("x", w);
        if (isNaN(o.y)) o.y = position("y", h);
        if (isNaN(o.px)) o.px = o.x;
        if (isNaN(o.py)) o.py = o.y;
      }
      distances = [];
      if (typeof linkDistance === "function") for (i = 0; i < m; ++i) distances[i] = +linkDistance.call(this, links[i], i); else for (i = 0; i < m; ++i) distances[i] = linkDistance;
      strengths = [];
      if (typeof linkStrength === "function") for (i = 0; i < m; ++i) strengths[i] = +linkStrength.call(this, links[i], i); else for (i = 0; i < m; ++i) strengths[i] = linkStrength;
      charges = [];
      if (typeof charge === "function") for (i = 0; i < n; ++i) charges[i] = +charge.call(this, nodes[i], i); else for (i = 0; i < n; ++i) charges[i] = charge;
      function position(dimension, size) {
        if (!neighbors) {
          neighbors = new Array(n);
          for (j = 0; j < n; ++j) {
            neighbors[j] = [];
          }
          for (j = 0; j < m; ++j) {
            var o = links[j];
            neighbors[o.source.index].push(o.target);
            neighbors[o.target.index].push(o.source);
          }
        }
        var candidates = neighbors[i], j = -1, l = candidates.length, x;
        while (++j < l) if (!isNaN(x = candidates[j][dimension])) return x;
        return Math.random() * size;
      }
      return force.resume();
    };
    force.resume = function() {
      return force.alpha(.1);
    };
    force.stop = function() {
      return force.alpha(0);
    };
    force.drag = function() {
      if (!drag) drag = d3.behavior.drag().origin(d3_identity).on("dragstart.force", d3_layout_forceDragstart).on("drag.force", dragmove).on("dragend.force", d3_layout_forceDragend);
      if (!arguments.length) return drag;
      this.on("mouseover.force", d3_layout_forceMouseover).on("mouseout.force", d3_layout_forceMouseout).call(drag);
    };
    function dragmove(d) {
      d.px = d3.event.x, d.py = d3.event.y;
      force.resume();
    }
    return d3.rebind(force, event, "on");
  };
  function d3_layout_forceDragstart(d) {
    d.fixed |= 2;
  }
  function d3_layout_forceDragend(d) {
    d.fixed &= ~6;
  }
  function d3_layout_forceMouseover(d) {
    d.fixed |= 4;
    d.px = d.x, d.py = d.y;
  }
  function d3_layout_forceMouseout(d) {
    d.fixed &= ~4;
  }
  function d3_layout_forceAccumulate(quad, alpha, charges) {
    var cx = 0, cy = 0;
    quad.charge = 0;
    if (!quad.leaf) {
      var nodes = quad.nodes, n = nodes.length, i = -1, c;
      while (++i < n) {
        c = nodes[i];
        if (c == null) continue;
        d3_layout_forceAccumulate(c, alpha, charges);
        quad.charge += c.charge;
        cx += c.charge * c.cx;
        cy += c.charge * c.cy;
      }
    }
    if (quad.point) {
      if (!quad.leaf) {
        quad.point.x += Math.random() - .5;
        quad.point.y += Math.random() - .5;
      }
      var k = alpha * charges[quad.point.index];
      quad.charge += quad.pointCharge = k;
      cx += k * quad.point.x;
      cy += k * quad.point.y;
    }
    quad.cx = cx / quad.charge;
    quad.cy = cy / quad.charge;
  }
  var d3_layout_forceLinkDistance = 20, d3_layout_forceLinkStrength = 1, d3_layout_forceChargeDistance2 = Infinity;
  d3.layout.hierarchy = function() {
    var sort = d3_layout_hierarchySort, children = d3_layout_hierarchyChildren, value = d3_layout_hierarchyValue;
    function hierarchy(root) {
      var stack = [ root ], nodes = [], node;
      root.depth = 0;
      while ((node = stack.pop()) != null) {
        nodes.push(node);
        if ((childs = children.call(hierarchy, node, node.depth)) && (n = childs.length)) {
          var n, childs, child;
          while (--n >= 0) {
            stack.push(child = childs[n]);
            child.parent = node;
            child.depth = node.depth + 1;
          }
          if (value) node.value = 0;
          node.children = childs;
        } else {
          if (value) node.value = +value.call(hierarchy, node, node.depth) || 0;
          delete node.children;
        }
      }
      d3_layout_hierarchyVisitAfter(root, function(node) {
        var childs, parent;
        if (sort && (childs = node.children)) childs.sort(sort);
        if (value && (parent = node.parent)) parent.value += node.value;
      });
      return nodes;
    }
    hierarchy.sort = function(x) {
      if (!arguments.length) return sort;
      sort = x;
      return hierarchy;
    };
    hierarchy.children = function(x) {
      if (!arguments.length) return children;
      children = x;
      return hierarchy;
    };
    hierarchy.value = function(x) {
      if (!arguments.length) return value;
      value = x;
      return hierarchy;
    };
    hierarchy.revalue = function(root) {
      if (value) {
        d3_layout_hierarchyVisitBefore(root, function(node) {
          if (node.children) node.value = 0;
        });
        d3_layout_hierarchyVisitAfter(root, function(node) {
          var parent;
          if (!node.children) node.value = +value.call(hierarchy, node, node.depth) || 0;
          if (parent = node.parent) parent.value += node.value;
        });
      }
      return root;
    };
    return hierarchy;
  };
  function d3_layout_hierarchyRebind(object, hierarchy) {
    d3.rebind(object, hierarchy, "sort", "children", "value");
    object.nodes = object;
    object.links = d3_layout_hierarchyLinks;
    return object;
  }
  function d3_layout_hierarchyVisitBefore(node, callback) {
    var nodes = [ node ];
    while ((node = nodes.pop()) != null) {
      callback(node);
      if ((children = node.children) && (n = children.length)) {
        var n, children;
        while (--n >= 0) nodes.push(children[n]);
      }
    }
  }
  function d3_layout_hierarchyVisitAfter(node, callback) {
    var nodes = [ node ], nodes2 = [];
    while ((node = nodes.pop()) != null) {
      nodes2.push(node);
      if ((children = node.children) && (n = children.length)) {
        var i = -1, n, children;
        while (++i < n) nodes.push(children[i]);
      }
    }
    while ((node = nodes2.pop()) != null) {
      callback(node);
    }
  }
  function d3_layout_hierarchyChildren(d) {
    return d.children;
  }
  function d3_layout_hierarchyValue(d) {
    return d.value;
  }
  function d3_layout_hierarchySort(a, b) {
    return b.value - a.value;
  }
  function d3_layout_hierarchyLinks(nodes) {
    return d3.merge(nodes.map(function(parent) {
      return (parent.children || []).map(function(child) {
        return {
          source: parent,
          target: child
        };
      });
    }));
  }
  d3.layout.partition = function() {
    var hierarchy = d3.layout.hierarchy(), size = [ 1, 1 ];
    function position(node, x, dx, dy) {
      var children = node.children;
      node.x = x;
      node.y = node.depth * dy;
      node.dx = dx;
      node.dy = dy;
      if (children && (n = children.length)) {
        var i = -1, n, c, d;
        dx = node.value ? dx / node.value : 0;
        while (++i < n) {
          position(c = children[i], x, d = c.value * dx, dy);
          x += d;
        }
      }
    }
    function depth(node) {
      var children = node.children, d = 0;
      if (children && (n = children.length)) {
        var i = -1, n;
        while (++i < n) d = Math.max(d, depth(children[i]));
      }
      return 1 + d;
    }
    function partition(d, i) {
      var nodes = hierarchy.call(this, d, i);
      position(nodes[0], 0, size[0], size[1] / depth(nodes[0]));
      return nodes;
    }
    partition.size = function(x) {
      if (!arguments.length) return size;
      size = x;
      return partition;
    };
    return d3_layout_hierarchyRebind(partition, hierarchy);
  };
  d3.layout.pie = function() {
    var value = Number, sort = d3_layout_pieSortByValue, startAngle = 0, endAngle = τ, padAngle = 0;
    function pie(data) {
      var n = data.length, values = data.map(function(d, i) {
        return +value.call(pie, d, i);
      }), a = +(typeof startAngle === "function" ? startAngle.apply(this, arguments) : startAngle), da = (typeof endAngle === "function" ? endAngle.apply(this, arguments) : endAngle) - a, p = Math.min(Math.abs(da) / n, +(typeof padAngle === "function" ? padAngle.apply(this, arguments) : padAngle)), pa = p * (da < 0 ? -1 : 1), sum = d3.sum(values), k = sum ? (da - n * pa) / sum : 0, index = d3.range(n), arcs = [], v;
      if (sort != null) index.sort(sort === d3_layout_pieSortByValue ? function(i, j) {
        return values[j] - values[i];
      } : function(i, j) {
        return sort(data[i], data[j]);
      });
      index.forEach(function(i) {
        arcs[i] = {
          data: data[i],
          value: v = values[i],
          startAngle: a,
          endAngle: a += v * k + pa,
          padAngle: p
        };
      });
      return arcs;
    }
    pie.value = function(_) {
      if (!arguments.length) return value;
      value = _;
      return pie;
    };
    pie.sort = function(_) {
      if (!arguments.length) return sort;
      sort = _;
      return pie;
    };
    pie.startAngle = function(_) {
      if (!arguments.length) return startAngle;
      startAngle = _;
      return pie;
    };
    pie.endAngle = function(_) {
      if (!arguments.length) return endAngle;
      endAngle = _;
      return pie;
    };
    pie.padAngle = function(_) {
      if (!arguments.length) return padAngle;
      padAngle = _;
      return pie;
    };
    return pie;
  };
  var d3_layout_pieSortByValue = {};
  d3.layout.stack = function() {
    var values = d3_identity, order = d3_layout_stackOrderDefault, offset = d3_layout_stackOffsetZero, out = d3_layout_stackOut, x = d3_layout_stackX, y = d3_layout_stackY;
    function stack(data, index) {
      if (!(n = data.length)) return data;
      var series = data.map(function(d, i) {
        return values.call(stack, d, i);
      });
      var points = series.map(function(d) {
        return d.map(function(v, i) {
          return [ x.call(stack, v, i), y.call(stack, v, i) ];
        });
      });
      var orders = order.call(stack, points, index);
      series = d3.permute(series, orders);
      points = d3.permute(points, orders);
      var offsets = offset.call(stack, points, index);
      var m = series[0].length, n, i, j, o;
      for (j = 0; j < m; ++j) {
        out.call(stack, series[0][j], o = offsets[j], points[0][j][1]);
        for (i = 1; i < n; ++i) {
          out.call(stack, series[i][j], o += points[i - 1][j][1], points[i][j][1]);
        }
      }
      return data;
    }
    stack.values = function(x) {
      if (!arguments.length) return values;
      values = x;
      return stack;
    };
    stack.order = function(x) {
      if (!arguments.length) return order;
      order = typeof x === "function" ? x : d3_layout_stackOrders.get(x) || d3_layout_stackOrderDefault;
      return stack;
    };
    stack.offset = function(x) {
      if (!arguments.length) return offset;
      offset = typeof x === "function" ? x : d3_layout_stackOffsets.get(x) || d3_layout_stackOffsetZero;
      return stack;
    };
    stack.x = function(z) {
      if (!arguments.length) return x;
      x = z;
      return stack;
    };
    stack.y = function(z) {
      if (!arguments.length) return y;
      y = z;
      return stack;
    };
    stack.out = function(z) {
      if (!arguments.length) return out;
      out = z;
      return stack;
    };
    return stack;
  };
  function d3_layout_stackX(d) {
    return d.x;
  }
  function d3_layout_stackY(d) {
    return d.y;
  }
  function d3_layout_stackOut(d, y0, y) {
    d.y0 = y0;
    d.y = y;
  }
  var d3_layout_stackOrders = d3.map({
    "inside-out": function(data) {
      var n = data.length, i, j, max = data.map(d3_layout_stackMaxIndex), sums = data.map(d3_layout_stackReduceSum), index = d3.range(n).sort(function(a, b) {
        return max[a] - max[b];
      }), top = 0, bottom = 0, tops = [], bottoms = [];
      for (i = 0; i < n; ++i) {
        j = index[i];
        if (top < bottom) {
          top += sums[j];
          tops.push(j);
        } else {
          bottom += sums[j];
          bottoms.push(j);
        }
      }
      return bottoms.reverse().concat(tops);
    },
    reverse: function(data) {
      return d3.range(data.length).reverse();
    },
    "default": d3_layout_stackOrderDefault
  });
  var d3_layout_stackOffsets = d3.map({
    silhouette: function(data) {
      var n = data.length, m = data[0].length, sums = [], max = 0, i, j, o, y0 = [];
      for (j = 0; j < m; ++j) {
        for (i = 0, o = 0; i < n; i++) o += data[i][j][1];
        if (o > max) max = o;
        sums.push(o);
      }
      for (j = 0; j < m; ++j) {
        y0[j] = (max - sums[j]) / 2;
      }
      return y0;
    },
    wiggle: function(data) {
      var n = data.length, x = data[0], m = x.length, i, j, k, s1, s2, s3, dx, o, o0, y0 = [];
      y0[0] = o = o0 = 0;
      for (j = 1; j < m; ++j) {
        for (i = 0, s1 = 0; i < n; ++i) s1 += data[i][j][1];
        for (i = 0, s2 = 0, dx = x[j][0] - x[j - 1][0]; i < n; ++i) {
          for (k = 0, s3 = (data[i][j][1] - data[i][j - 1][1]) / (2 * dx); k < i; ++k) {
            s3 += (data[k][j][1] - data[k][j - 1][1]) / dx;
          }
          s2 += s3 * data[i][j][1];
        }
        y0[j] = o -= s1 ? s2 / s1 * dx : 0;
        if (o < o0) o0 = o;
      }
      for (j = 0; j < m; ++j) y0[j] -= o0;
      return y0;
    },
    expand: function(data) {
      var n = data.length, m = data[0].length, k = 1 / n, i, j, o, y0 = [];
      for (j = 0; j < m; ++j) {
        for (i = 0, o = 0; i < n; i++) o += data[i][j][1];
        if (o) for (i = 0; i < n; i++) data[i][j][1] /= o; else for (i = 0; i < n; i++) data[i][j][1] = k;
      }
      for (j = 0; j < m; ++j) y0[j] = 0;
      return y0;
    },
    zero: d3_layout_stackOffsetZero
  });
  function d3_layout_stackOrderDefault(data) {
    return d3.range(data.length);
  }
  function d3_layout_stackOffsetZero(data) {
    var j = -1, m = data[0].length, y0 = [];
    while (++j < m) y0[j] = 0;
    return y0;
  }
  function d3_layout_stackMaxIndex(array) {
    var i = 1, j = 0, v = array[0][1], k, n = array.length;
    for (;i < n; ++i) {
      if ((k = array[i][1]) > v) {
        j = i;
        v = k;
      }
    }
    return j;
  }
  function d3_layout_stackReduceSum(d) {
    return d.reduce(d3_layout_stackSum, 0);
  }
  function d3_layout_stackSum(p, d) {
    return p + d[1];
  }
  d3.layout.histogram = function() {
    var frequency = true, valuer = Number, ranger = d3_layout_histogramRange, binner = d3_layout_histogramBinSturges;
    function histogram(data, i) {
      var bins = [], values = data.map(valuer, this), range = ranger.call(this, values, i), thresholds = binner.call(this, range, values, i), bin, i = -1, n = values.length, m = thresholds.length - 1, k = frequency ? 1 : 1 / n, x;
      while (++i < m) {
        bin = bins[i] = [];
        bin.dx = thresholds[i + 1] - (bin.x = thresholds[i]);
        bin.y = 0;
      }
      if (m > 0) {
        i = -1;
        while (++i < n) {
          x = values[i];
          if (x >= range[0] && x <= range[1]) {
            bin = bins[d3.bisect(thresholds, x, 1, m) - 1];
            bin.y += k;
            bin.push(data[i]);
          }
        }
      }
      return bins;
    }
    histogram.value = function(x) {
      if (!arguments.length) return valuer;
      valuer = x;
      return histogram;
    };
    histogram.range = function(x) {
      if (!arguments.length) return ranger;
      ranger = d3_functor(x);
      return histogram;
    };
    histogram.bins = function(x) {
      if (!arguments.length) return binner;
      binner = typeof x === "number" ? function(range) {
        return d3_layout_histogramBinFixed(range, x);
      } : d3_functor(x);
      return histogram;
    };
    histogram.frequency = function(x) {
      if (!arguments.length) return frequency;
      frequency = !!x;
      return histogram;
    };
    return histogram;
  };
  function d3_layout_histogramBinSturges(range, values) {
    return d3_layout_histogramBinFixed(range, Math.ceil(Math.log(values.length) / Math.LN2 + 1));
  }
  function d3_layout_histogramBinFixed(range, n) {
    var x = -1, b = +range[0], m = (range[1] - b) / n, f = [];
    while (++x <= n) f[x] = m * x + b;
    return f;
  }
  function d3_layout_histogramRange(values) {
    return [ d3.min(values), d3.max(values) ];
  }
  d3.layout.pack = function() {
    var hierarchy = d3.layout.hierarchy().sort(d3_layout_packSort), padding = 0, size = [ 1, 1 ], radius;
    function pack(d, i) {
      var nodes = hierarchy.call(this, d, i), root = nodes[0], w = size[0], h = size[1], r = radius == null ? Math.sqrt : typeof radius === "function" ? radius : function() {
        return radius;
      };
      root.x = root.y = 0;
      d3_layout_hierarchyVisitAfter(root, function(d) {
        d.r = +r(d.value);
      });
      d3_layout_hierarchyVisitAfter(root, d3_layout_packSiblings);
      if (padding) {
        var dr = padding * (radius ? 1 : Math.max(2 * root.r / w, 2 * root.r / h)) / 2;
        d3_layout_hierarchyVisitAfter(root, function(d) {
          d.r += dr;
        });
        d3_layout_hierarchyVisitAfter(root, d3_layout_packSiblings);
        d3_layout_hierarchyVisitAfter(root, function(d) {
          d.r -= dr;
        });
      }
      d3_layout_packTransform(root, w / 2, h / 2, radius ? 1 : 1 / Math.max(2 * root.r / w, 2 * root.r / h));
      return nodes;
    }
    pack.size = function(_) {
      if (!arguments.length) return size;
      size = _;
      return pack;
    };
    pack.radius = function(_) {
      if (!arguments.length) return radius;
      radius = _ == null || typeof _ === "function" ? _ : +_;
      return pack;
    };
    pack.padding = function(_) {
      if (!arguments.length) return padding;
      padding = +_;
      return pack;
    };
    return d3_layout_hierarchyRebind(pack, hierarchy);
  };
  function d3_layout_packSort(a, b) {
    return a.value - b.value;
  }
  function d3_layout_packInsert(a, b) {
    var c = a._pack_next;
    a._pack_next = b;
    b._pack_prev = a;
    b._pack_next = c;
    c._pack_prev = b;
  }
  function d3_layout_packSplice(a, b) {
    a._pack_next = b;
    b._pack_prev = a;
  }
  function d3_layout_packIntersects(a, b) {
    var dx = b.x - a.x, dy = b.y - a.y, dr = a.r + b.r;
    return .999 * dr * dr > dx * dx + dy * dy;
  }
  function d3_layout_packSiblings(node) {
    if (!(nodes = node.children) || !(n = nodes.length)) return;
    var nodes, xMin = Infinity, xMax = -Infinity, yMin = Infinity, yMax = -Infinity, a, b, c, i, j, k, n;
    function bound(node) {
      xMin = Math.min(node.x - node.r, xMin);
      xMax = Math.max(node.x + node.r, xMax);
      yMin = Math.min(node.y - node.r, yMin);
      yMax = Math.max(node.y + node.r, yMax);
    }
    nodes.forEach(d3_layout_packLink);
    a = nodes[0];
    a.x = -a.r;
    a.y = 0;
    bound(a);
    if (n > 1) {
      b = nodes[1];
      b.x = b.r;
      b.y = 0;
      bound(b);
      if (n > 2) {
        c = nodes[2];
        d3_layout_packPlace(a, b, c);
        bound(c);
        d3_layout_packInsert(a, c);
        a._pack_prev = c;
        d3_layout_packInsert(c, b);
        b = a._pack_next;
        for (i = 3; i < n; i++) {
          d3_layout_packPlace(a, b, c = nodes[i]);
          var isect = 0, s1 = 1, s2 = 1;
          for (j = b._pack_next; j !== b; j = j._pack_next, s1++) {
            if (d3_layout_packIntersects(j, c)) {
              isect = 1;
              break;
            }
          }
          if (isect == 1) {
            for (k = a._pack_prev; k !== j._pack_prev; k = k._pack_prev, s2++) {
              if (d3_layout_packIntersects(k, c)) {
                break;
              }
            }
          }
          if (isect) {
            if (s1 < s2 || s1 == s2 && b.r < a.r) d3_layout_packSplice(a, b = j); else d3_layout_packSplice(a = k, b);
            i--;
          } else {
            d3_layout_packInsert(a, c);
            b = c;
            bound(c);
          }
        }
      }
    }
    var cx = (xMin + xMax) / 2, cy = (yMin + yMax) / 2, cr = 0;
    for (i = 0; i < n; i++) {
      c = nodes[i];
      c.x -= cx;
      c.y -= cy;
      cr = Math.max(cr, c.r + Math.sqrt(c.x * c.x + c.y * c.y));
    }
    node.r = cr;
    nodes.forEach(d3_layout_packUnlink);
  }
  function d3_layout_packLink(node) {
    node._pack_next = node._pack_prev = node;
  }
  function d3_layout_packUnlink(node) {
    delete node._pack_next;
    delete node._pack_prev;
  }
  function d3_layout_packTransform(node, x, y, k) {
    var children = node.children;
    node.x = x += k * node.x;
    node.y = y += k * node.y;
    node.r *= k;
    if (children) {
      var i = -1, n = children.length;
      while (++i < n) d3_layout_packTransform(children[i], x, y, k);
    }
  }
  function d3_layout_packPlace(a, b, c) {
    var db = a.r + c.r, dx = b.x - a.x, dy = b.y - a.y;
    if (db && (dx || dy)) {
      var da = b.r + c.r, dc = dx * dx + dy * dy;
      da *= da;
      db *= db;
      var x = .5 + (db - da) / (2 * dc), y = Math.sqrt(Math.max(0, 2 * da * (db + dc) - (db -= dc) * db - da * da)) / (2 * dc);
      c.x = a.x + x * dx + y * dy;
      c.y = a.y + x * dy - y * dx;
    } else {
      c.x = a.x + db;
      c.y = a.y;
    }
  }
  d3.layout.tree = function() {
    var hierarchy = d3.layout.hierarchy().sort(null).value(null), separation = d3_layout_treeSeparation, size = [ 1, 1 ], nodeSize = null;
    function tree(d, i) {
      var nodes = hierarchy.call(this, d, i), root0 = nodes[0], root1 = wrapTree(root0);
      d3_layout_hierarchyVisitAfter(root1, firstWalk), root1.parent.m = -root1.z;
      d3_layout_hierarchyVisitBefore(root1, secondWalk);
      if (nodeSize) d3_layout_hierarchyVisitBefore(root0, sizeNode); else {
        var left = root0, right = root0, bottom = root0;
        d3_layout_hierarchyVisitBefore(root0, function(node) {
          if (node.x < left.x) left = node;
          if (node.x > right.x) right = node;
          if (node.depth > bottom.depth) bottom = node;
        });
        var tx = separation(left, right) / 2 - left.x, kx = size[0] / (right.x + separation(right, left) / 2 + tx), ky = size[1] / (bottom.depth || 1);
        d3_layout_hierarchyVisitBefore(root0, function(node) {
          node.x = (node.x + tx) * kx;
          node.y = node.depth * ky;
        });
      }
      return nodes;
    }
    function wrapTree(root0) {
      var root1 = {
        A: null,
        children: [ root0 ]
      }, queue = [ root1 ], node1;
      while ((node1 = queue.pop()) != null) {
        for (var children = node1.children, child, i = 0, n = children.length; i < n; ++i) {
          queue.push((children[i] = child = {
            _: children[i],
            parent: node1,
            children: (child = children[i].children) && child.slice() || [],
            A: null,
            a: null,
            z: 0,
            m: 0,
            c: 0,
            s: 0,
            t: null,
            i: i
          }).a = child);
        }
      }
      return root1.children[0];
    }
    function firstWalk(v) {
      var children = v.children, siblings = v.parent.children, w = v.i ? siblings[v.i - 1] : null;
      if (children.length) {
        d3_layout_treeShift(v);
        var midpoint = (children[0].z + children[children.length - 1].z) / 2;
        if (w) {
          v.z = w.z + separation(v._, w._);
          v.m = v.z - midpoint;
        } else {
          v.z = midpoint;
        }
      } else if (w) {
        v.z = w.z + separation(v._, w._);
      }
      v.parent.A = apportion(v, w, v.parent.A || siblings[0]);
    }
    function secondWalk(v) {
      v._.x = v.z + v.parent.m;
      v.m += v.parent.m;
    }
    function apportion(v, w, ancestor) {
      if (w) {
        var vip = v, vop = v, vim = w, vom = vip.parent.children[0], sip = vip.m, sop = vop.m, sim = vim.m, som = vom.m, shift;
        while (vim = d3_layout_treeRight(vim), vip = d3_layout_treeLeft(vip), vim && vip) {
          vom = d3_layout_treeLeft(vom);
          vop = d3_layout_treeRight(vop);
          vop.a = v;
          shift = vim.z + sim - vip.z - sip + separation(vim._, vip._);
          if (shift > 0) {
            d3_layout_treeMove(d3_layout_treeAncestor(vim, v, ancestor), v, shift);
            sip += shift;
            sop += shift;
          }
          sim += vim.m;
          sip += vip.m;
          som += vom.m;
          sop += vop.m;
        }
        if (vim && !d3_layout_treeRight(vop)) {
          vop.t = vim;
          vop.m += sim - sop;
        }
        if (vip && !d3_layout_treeLeft(vom)) {
          vom.t = vip;
          vom.m += sip - som;
          ancestor = v;
        }
      }
      return ancestor;
    }
    function sizeNode(node) {
      node.x *= size[0];
      node.y = node.depth * size[1];
    }
    tree.separation = function(x) {
      if (!arguments.length) return separation;
      separation = x;
      return tree;
    };
    tree.size = function(x) {
      if (!arguments.length) return nodeSize ? null : size;
      nodeSize = (size = x) == null ? sizeNode : null;
      return tree;
    };
    tree.nodeSize = function(x) {
      if (!arguments.length) return nodeSize ? size : null;
      nodeSize = (size = x) == null ? null : sizeNode;
      return tree;
    };
    return d3_layout_hierarchyRebind(tree, hierarchy);
  };
  function d3_layout_treeSeparation(a, b) {
    return a.parent == b.parent ? 1 : 2;
  }
  function d3_layout_treeLeft(v) {
    var children = v.children;
    return children.length ? children[0] : v.t;
  }
  function d3_layout_treeRight(v) {
    var children = v.children, n;
    return (n = children.length) ? children[n - 1] : v.t;
  }
  function d3_layout_treeMove(wm, wp, shift) {
    var change = shift / (wp.i - wm.i);
    wp.c -= change;
    wp.s += shift;
    wm.c += change;
    wp.z += shift;
    wp.m += shift;
  }
  function d3_layout_treeShift(v) {
    var shift = 0, change = 0, children = v.children, i = children.length, w;
    while (--i >= 0) {
      w = children[i];
      w.z += shift;
      w.m += shift;
      shift += w.s + (change += w.c);
    }
  }
  function d3_layout_treeAncestor(vim, v, ancestor) {
    return vim.a.parent === v.parent ? vim.a : ancestor;
  }
  d3.layout.cluster = function() {
    var hierarchy = d3.layout.hierarchy().sort(null).value(null), separation = d3_layout_treeSeparation, size = [ 1, 1 ], nodeSize = false;
    function cluster(d, i) {
      var nodes = hierarchy.call(this, d, i), root = nodes[0], previousNode, x = 0;
      d3_layout_hierarchyVisitAfter(root, function(node) {
        var children = node.children;
        if (children && children.length) {
          node.x = d3_layout_clusterX(children);
          node.y = d3_layout_clusterY(children);
        } else {
          node.x = previousNode ? x += separation(node, previousNode) : 0;
          node.y = 0;
          previousNode = node;
        }
      });
      var left = d3_layout_clusterLeft(root), right = d3_layout_clusterRight(root), x0 = left.x - separation(left, right) / 2, x1 = right.x + separation(right, left) / 2;
      d3_layout_hierarchyVisitAfter(root, nodeSize ? function(node) {
        node.x = (node.x - root.x) * size[0];
        node.y = (root.y - node.y) * size[1];
      } : function(node) {
        node.x = (node.x - x0) / (x1 - x0) * size[0];
        node.y = (1 - (root.y ? node.y / root.y : 1)) * size[1];
      });
      return nodes;
    }
    cluster.separation = function(x) {
      if (!arguments.length) return separation;
      separation = x;
      return cluster;
    };
    cluster.size = function(x) {
      if (!arguments.length) return nodeSize ? null : size;
      nodeSize = (size = x) == null;
      return cluster;
    };
    cluster.nodeSize = function(x) {
      if (!arguments.length) return nodeSize ? size : null;
      nodeSize = (size = x) != null;
      return cluster;
    };
    return d3_layout_hierarchyRebind(cluster, hierarchy);
  };
  function d3_layout_clusterY(children) {
    return 1 + d3.max(children, function(child) {
      return child.y;
    });
  }
  function d3_layout_clusterX(children) {
    return children.reduce(function(x, child) {
      return x + child.x;
    }, 0) / children.length;
  }
  function d3_layout_clusterLeft(node) {
    var children = node.children;
    return children && children.length ? d3_layout_clusterLeft(children[0]) : node;
  }
  function d3_layout_clusterRight(node) {
    var children = node.children, n;
    return children && (n = children.length) ? d3_layout_clusterRight(children[n - 1]) : node;
  }
  d3.layout.treemap = function() {
    var hierarchy = d3.layout.hierarchy(), round = Math.round, size = [ 1, 1 ], padding = null, pad = d3_layout_treemapPadNull, sticky = false, stickies, mode = "squarify", ratio = .5 * (1 + Math.sqrt(5));
    function scale(children, k) {
      var i = -1, n = children.length, child, area;
      while (++i < n) {
        area = (child = children[i]).value * (k < 0 ? 0 : k);
        child.area = isNaN(area) || area <= 0 ? 0 : area;
      }
    }
    function squarify(node) {
      var children = node.children;
      if (children && children.length) {
        var rect = pad(node), row = [], remaining = children.slice(), child, best = Infinity, score, u = mode === "slice" ? rect.dx : mode === "dice" ? rect.dy : mode === "slice-dice" ? node.depth & 1 ? rect.dy : rect.dx : Math.min(rect.dx, rect.dy), n;
        scale(remaining, rect.dx * rect.dy / node.value);
        row.area = 0;
        while ((n = remaining.length) > 0) {
          row.push(child = remaining[n - 1]);
          row.area += child.area;
          if (mode !== "squarify" || (score = worst(row, u)) <= best) {
            remaining.pop();
            best = score;
          } else {
            row.area -= row.pop().area;
            position(row, u, rect, false);
            u = Math.min(rect.dx, rect.dy);
            row.length = row.area = 0;
            best = Infinity;
          }
        }
        if (row.length) {
          position(row, u, rect, true);
          row.length = row.area = 0;
        }
        children.forEach(squarify);
      }
    }
    function stickify(node) {
      var children = node.children;
      if (children && children.length) {
        var rect = pad(node), remaining = children.slice(), child, row = [];
        scale(remaining, rect.dx * rect.dy / node.value);
        row.area = 0;
        while (child = remaining.pop()) {
          row.push(child);
          row.area += child.area;
          if (child.z != null) {
            position(row, child.z ? rect.dx : rect.dy, rect, !remaining.length);
            row.length = row.area = 0;
          }
        }
        children.forEach(stickify);
      }
    }
    function worst(row, u) {
      var s = row.area, r, rmax = 0, rmin = Infinity, i = -1, n = row.length;
      while (++i < n) {
        if (!(r = row[i].area)) continue;
        if (r < rmin) rmin = r;
        if (r > rmax) rmax = r;
      }
      s *= s;
      u *= u;
      return s ? Math.max(u * rmax * ratio / s, s / (u * rmin * ratio)) : Infinity;
    }
    function position(row, u, rect, flush) {
      var i = -1, n = row.length, x = rect.x, y = rect.y, v = u ? round(row.area / u) : 0, o;
      if (u == rect.dx) {
        if (flush || v > rect.dy) v = rect.dy;
        while (++i < n) {
          o = row[i];
          o.x = x;
          o.y = y;
          o.dy = v;
          x += o.dx = Math.min(rect.x + rect.dx - x, v ? round(o.area / v) : 0);
        }
        o.z = true;
        o.dx += rect.x + rect.dx - x;
        rect.y += v;
        rect.dy -= v;
      } else {
        if (flush || v > rect.dx) v = rect.dx;
        while (++i < n) {
          o = row[i];
          o.x = x;
          o.y = y;
          o.dx = v;
          y += o.dy = Math.min(rect.y + rect.dy - y, v ? round(o.area / v) : 0);
        }
        o.z = false;
        o.dy += rect.y + rect.dy - y;
        rect.x += v;
        rect.dx -= v;
      }
    }
    function treemap(d) {
      var nodes = stickies || hierarchy(d), root = nodes[0];
      root.x = root.y = 0;
      if (root.value) root.dx = size[0], root.dy = size[1]; else root.dx = root.dy = 0;
      if (stickies) hierarchy.revalue(root);
      scale([ root ], root.dx * root.dy / root.value);
      (stickies ? stickify : squarify)(root);
      if (sticky) stickies = nodes;
      return nodes;
    }
    treemap.size = function(x) {
      if (!arguments.length) return size;
      size = x;
      return treemap;
    };
    treemap.padding = function(x) {
      if (!arguments.length) return padding;
      function padFunction(node) {
        var p = x.call(treemap, node, node.depth);
        return p == null ? d3_layout_treemapPadNull(node) : d3_layout_treemapPad(node, typeof p === "number" ? [ p, p, p, p ] : p);
      }
      function padConstant(node) {
        return d3_layout_treemapPad(node, x);
      }
      var type;
      pad = (padding = x) == null ? d3_layout_treemapPadNull : (type = typeof x) === "function" ? padFunction : type === "number" ? (x = [ x, x, x, x ], 
      padConstant) : padConstant;
      return treemap;
    };
    treemap.round = function(x) {
      if (!arguments.length) return round != Number;
      round = x ? Math.round : Number;
      return treemap;
    };
    treemap.sticky = function(x) {
      if (!arguments.length) return sticky;
      sticky = x;
      stickies = null;
      return treemap;
    };
    treemap.ratio = function(x) {
      if (!arguments.length) return ratio;
      ratio = x;
      return treemap;
    };
    treemap.mode = function(x) {
      if (!arguments.length) return mode;
      mode = x + "";
      return treemap;
    };
    return d3_layout_hierarchyRebind(treemap, hierarchy);
  };
  function d3_layout_treemapPadNull(node) {
    return {
      x: node.x,
      y: node.y,
      dx: node.dx,
      dy: node.dy
    };
  }
  function d3_layout_treemapPad(node, padding) {
    var x = node.x + padding[3], y = node.y + padding[0], dx = node.dx - padding[1] - padding[3], dy = node.dy - padding[0] - padding[2];
    if (dx < 0) {
      x += dx / 2;
      dx = 0;
    }
    if (dy < 0) {
      y += dy / 2;
      dy = 0;
    }
    return {
      x: x,
      y: y,
      dx: dx,
      dy: dy
    };
  }
  d3.random = {
    normal: function(µ, σ) {
      var n = arguments.length;
      if (n < 2) σ = 1;
      if (n < 1) µ = 0;
      return function() {
        var x, y, r;
        do {
          x = Math.random() * 2 - 1;
          y = Math.random() * 2 - 1;
          r = x * x + y * y;
        } while (!r || r > 1);
        return µ + σ * x * Math.sqrt(-2 * Math.log(r) / r);
      };
    },
    logNormal: function() {
      var random = d3.random.normal.apply(d3, arguments);
      return function() {
        return Math.exp(random());
      };
    },
    bates: function(m) {
      var random = d3.random.irwinHall(m);
      return function() {
        return random() / m;
      };
    },
    irwinHall: function(m) {
      return function() {
        for (var s = 0, j = 0; j < m; j++) s += Math.random();
        return s;
      };
    }
  };
  d3.scale = {};
  function d3_scaleExtent(domain) {
    var start = domain[0], stop = domain[domain.length - 1];
    return start < stop ? [ start, stop ] : [ stop, start ];
  }
  function d3_scaleRange(scale) {
    return scale.rangeExtent ? scale.rangeExtent() : d3_scaleExtent(scale.range());
  }
  function d3_scale_bilinear(domain, range, uninterpolate, interpolate) {
    var u = uninterpolate(domain[0], domain[1]), i = interpolate(range[0], range[1]);
    return function(x) {
      return i(u(x));
    };
  }
  function d3_scale_nice(domain, nice) {
    var i0 = 0, i1 = domain.length - 1, x0 = domain[i0], x1 = domain[i1], dx;
    if (x1 < x0) {
      dx = i0, i0 = i1, i1 = dx;
      dx = x0, x0 = x1, x1 = dx;
    }
    domain[i0] = nice.floor(x0);
    domain[i1] = nice.ceil(x1);
    return domain;
  }
  function d3_scale_niceStep(step) {
    return step ? {
      floor: function(x) {
        return Math.floor(x / step) * step;
      },
      ceil: function(x) {
        return Math.ceil(x / step) * step;
      }
    } : d3_scale_niceIdentity;
  }
  var d3_scale_niceIdentity = {
    floor: d3_identity,
    ceil: d3_identity
  };
  function d3_scale_polylinear(domain, range, uninterpolate, interpolate) {
    var u = [], i = [], j = 0, k = Math.min(domain.length, range.length) - 1;
    if (domain[k] < domain[0]) {
      domain = domain.slice().reverse();
      range = range.slice().reverse();
    }
    while (++j <= k) {
      u.push(uninterpolate(domain[j - 1], domain[j]));
      i.push(interpolate(range[j - 1], range[j]));
    }
    return function(x) {
      var j = d3.bisect(domain, x, 1, k) - 1;
      return i[j](u[j](x));
    };
  }
  d3.scale.linear = function() {
    return d3_scale_linear([ 0, 1 ], [ 0, 1 ], d3_interpolate, false);
  };
  function d3_scale_linear(domain, range, interpolate, clamp) {
    var output, input;
    function rescale() {
      var linear = Math.min(domain.length, range.length) > 2 ? d3_scale_polylinear : d3_scale_bilinear, uninterpolate = clamp ? d3_uninterpolateClamp : d3_uninterpolateNumber;
      output = linear(domain, range, uninterpolate, interpolate);
      input = linear(range, domain, uninterpolate, d3_interpolate);
      return scale;
    }
    function scale(x) {
      return output(x);
    }
    scale.invert = function(y) {
      return input(y);
    };
    scale.domain = function(x) {
      if (!arguments.length) return domain;
      domain = x.map(Number);
      return rescale();
    };
    scale.range = function(x) {
      if (!arguments.length) return range;
      range = x;
      return rescale();
    };
    scale.rangeRound = function(x) {
      return scale.range(x).interpolate(d3_interpolateRound);
    };
    scale.clamp = function(x) {
      if (!arguments.length) return clamp;
      clamp = x;
      return rescale();
    };
    scale.interpolate = function(x) {
      if (!arguments.length) return interpolate;
      interpolate = x;
      return rescale();
    };
    scale.ticks = function(m) {
      return d3_scale_linearTicks(domain, m);
    };
    scale.tickFormat = function(m, format) {
      return d3_scale_linearTickFormat(domain, m, format);
    };
    scale.nice = function(m) {
      d3_scale_linearNice(domain, m);
      return rescale();
    };
    scale.copy = function() {
      return d3_scale_linear(domain, range, interpolate, clamp);
    };
    return rescale();
  }
  function d3_scale_linearRebind(scale, linear) {
    return d3.rebind(scale, linear, "range", "rangeRound", "interpolate", "clamp");
  }
  function d3_scale_linearNice(domain, m) {
    d3_scale_nice(domain, d3_scale_niceStep(d3_scale_linearTickRange(domain, m)[2]));
    d3_scale_nice(domain, d3_scale_niceStep(d3_scale_linearTickRange(domain, m)[2]));
    return domain;
  }
  function d3_scale_linearTickRange(domain, m) {
    if (m == null) m = 10;
    var extent = d3_scaleExtent(domain), span = extent[1] - extent[0], step = Math.pow(10, Math.floor(Math.log(span / m) / Math.LN10)), err = m / span * step;
    if (err <= .15) step *= 10; else if (err <= .35) step *= 5; else if (err <= .75) step *= 2;
    extent[0] = Math.ceil(extent[0] / step) * step;
    extent[1] = Math.floor(extent[1] / step) * step + step * .5;
    extent[2] = step;
    return extent;
  }
  function d3_scale_linearTicks(domain, m) {
    return d3.range.apply(d3, d3_scale_linearTickRange(domain, m));
  }
  function d3_scale_linearTickFormat(domain, m, format) {
    var range = d3_scale_linearTickRange(domain, m);
    if (format) {
      var match = d3_format_re.exec(format);
      match.shift();
      if (match[8] === "s") {
        var prefix = d3.formatPrefix(Math.max(abs(range[0]), abs(range[1])));
        if (!match[7]) match[7] = "." + d3_scale_linearPrecision(prefix.scale(range[2]));
        match[8] = "f";
        format = d3.format(match.join(""));
        return function(d) {
          return format(prefix.scale(d)) + prefix.symbol;
        };
      }
      if (!match[7]) match[7] = "." + d3_scale_linearFormatPrecision(match[8], range);
      format = match.join("");
    } else {
      format = ",." + d3_scale_linearPrecision(range[2]) + "f";
    }
    return d3.format(format);
  }
  var d3_scale_linearFormatSignificant = {
    s: 1,
    g: 1,
    p: 1,
    r: 1,
    e: 1
  };
  function d3_scale_linearPrecision(value) {
    return -Math.floor(Math.log(value) / Math.LN10 + .01);
  }
  function d3_scale_linearFormatPrecision(type, range) {
    var p = d3_scale_linearPrecision(range[2]);
    return type in d3_scale_linearFormatSignificant ? Math.abs(p - d3_scale_linearPrecision(Math.max(abs(range[0]), abs(range[1])))) + +(type !== "e") : p - (type === "%") * 2;
  }
  d3.scale.log = function() {
    return d3_scale_log(d3.scale.linear().domain([ 0, 1 ]), 10, true, [ 1, 10 ]);
  };
  function d3_scale_log(linear, base, positive, domain) {
    function log(x) {
      return (positive ? Math.log(x < 0 ? 0 : x) : -Math.log(x > 0 ? 0 : -x)) / Math.log(base);
    }
    function pow(x) {
      return positive ? Math.pow(base, x) : -Math.pow(base, -x);
    }
    function scale(x) {
      return linear(log(x));
    }
    scale.invert = function(x) {
      return pow(linear.invert(x));
    };
    scale.domain = function(x) {
      if (!arguments.length) return domain;
      positive = x[0] >= 0;
      linear.domain((domain = x.map(Number)).map(log));
      return scale;
    };
    scale.base = function(_) {
      if (!arguments.length) return base;
      base = +_;
      linear.domain(domain.map(log));
      return scale;
    };
    scale.nice = function() {
      var niced = d3_scale_nice(domain.map(log), positive ? Math : d3_scale_logNiceNegative);
      linear.domain(niced);
      domain = niced.map(pow);
      return scale;
    };
    scale.ticks = function() {
      var extent = d3_scaleExtent(domain), ticks = [], u = extent[0], v = extent[1], i = Math.floor(log(u)), j = Math.ceil(log(v)), n = base % 1 ? 2 : base;
      if (isFinite(j - i)) {
        if (positive) {
          for (;i < j; i++) for (var k = 1; k < n; k++) ticks.push(pow(i) * k);
          ticks.push(pow(i));
        } else {
          ticks.push(pow(i));
          for (;i++ < j; ) for (var k = n - 1; k > 0; k--) ticks.push(pow(i) * k);
        }
        for (i = 0; ticks[i] < u; i++) {}
        for (j = ticks.length; ticks[j - 1] > v; j--) {}
        ticks = ticks.slice(i, j);
      }
      return ticks;
    };
    scale.tickFormat = function(n, format) {
      if (!arguments.length) return d3_scale_logFormat;
      if (arguments.length < 2) format = d3_scale_logFormat; else if (typeof format !== "function") format = d3.format(format);
      var k = Math.max(1, base * n / scale.ticks().length);
      return function(d) {
        var i = d / pow(Math.round(log(d)));
        if (i * base < base - .5) i *= base;
        return i <= k ? format(d) : "";
      };
    };
    scale.copy = function() {
      return d3_scale_log(linear.copy(), base, positive, domain);
    };
    return d3_scale_linearRebind(scale, linear);
  }
  var d3_scale_logFormat = d3.format(".0e"), d3_scale_logNiceNegative = {
    floor: function(x) {
      return -Math.ceil(-x);
    },
    ceil: function(x) {
      return -Math.floor(-x);
    }
  };
  d3.scale.pow = function() {
    return d3_scale_pow(d3.scale.linear(), 1, [ 0, 1 ]);
  };
  function d3_scale_pow(linear, exponent, domain) {
    var powp = d3_scale_powPow(exponent), powb = d3_scale_powPow(1 / exponent);
    function scale(x) {
      return linear(powp(x));
    }
    scale.invert = function(x) {
      return powb(linear.invert(x));
    };
    scale.domain = function(x) {
      if (!arguments.length) return domain;
      linear.domain((domain = x.map(Number)).map(powp));
      return scale;
    };
    scale.ticks = function(m) {
      return d3_scale_linearTicks(domain, m);
    };
    scale.tickFormat = function(m, format) {
      return d3_scale_linearTickFormat(domain, m, format);
    };
    scale.nice = function(m) {
      return scale.domain(d3_scale_linearNice(domain, m));
    };
    scale.exponent = function(x) {
      if (!arguments.length) return exponent;
      powp = d3_scale_powPow(exponent = x);
      powb = d3_scale_powPow(1 / exponent);
      linear.domain(domain.map(powp));
      return scale;
    };
    scale.copy = function() {
      return d3_scale_pow(linear.copy(), exponent, domain);
    };
    return d3_scale_linearRebind(scale, linear);
  }
  function d3_scale_powPow(e) {
    return function(x) {
      return x < 0 ? -Math.pow(-x, e) : Math.pow(x, e);
    };
  }
  d3.scale.sqrt = function() {
    return d3.scale.pow().exponent(.5);
  };
  d3.scale.ordinal = function() {
    return d3_scale_ordinal([], {
      t: "range",
      a: [ [] ]
    });
  };
  function d3_scale_ordinal(domain, ranger) {
    var index, range, rangeBand;
    function scale(x) {
      return range[((index.get(x) || (ranger.t === "range" ? index.set(x, domain.push(x)) : NaN)) - 1) % range.length];
    }
    function steps(start, step) {
      return d3.range(domain.length).map(function(i) {
        return start + step * i;
      });
    }
    scale.domain = function(x) {
      if (!arguments.length) return domain;
      domain = [];
      index = new d3_Map();
      var i = -1, n = x.length, xi;
      while (++i < n) if (!index.has(xi = x[i])) index.set(xi, domain.push(xi));
      return scale[ranger.t].apply(scale, ranger.a);
    };
    scale.range = function(x) {
      if (!arguments.length) return range;
      range = x;
      rangeBand = 0;
      ranger = {
        t: "range",
        a: arguments
      };
      return scale;
    };
    scale.rangePoints = function(x, padding) {
      if (arguments.length < 2) padding = 0;
      var start = x[0], stop = x[1], step = domain.length < 2 ? (start = (start + stop) / 2, 
      0) : (stop - start) / (domain.length - 1 + padding);
      range = steps(start + step * padding / 2, step);
      rangeBand = 0;
      ranger = {
        t: "rangePoints",
        a: arguments
      };
      return scale;
    };
    scale.rangeRoundPoints = function(x, padding) {
      if (arguments.length < 2) padding = 0;
      var start = x[0], stop = x[1], step = domain.length < 2 ? (start = stop = Math.round((start + stop) / 2), 
      0) : (stop - start) / (domain.length - 1 + padding) | 0;
      range = steps(start + Math.round(step * padding / 2 + (stop - start - (domain.length - 1 + padding) * step) / 2), step);
      rangeBand = 0;
      ranger = {
        t: "rangeRoundPoints",
        a: arguments
      };
      return scale;
    };
    scale.rangeBands = function(x, padding, outerPadding) {
      if (arguments.length < 2) padding = 0;
      if (arguments.length < 3) outerPadding = padding;
      var reverse = x[1] < x[0], start = x[reverse - 0], stop = x[1 - reverse], step = (stop - start) / (domain.length - padding + 2 * outerPadding);
      range = steps(start + step * outerPadding, step);
      if (reverse) range.reverse();
      rangeBand = step * (1 - padding);
      ranger = {
        t: "rangeBands",
        a: arguments
      };
      return scale;
    };
    scale.rangeRoundBands = function(x, padding, outerPadding) {
      if (arguments.length < 2) padding = 0;
      if (arguments.length < 3) outerPadding = padding;
      var reverse = x[1] < x[0], start = x[reverse - 0], stop = x[1 - reverse], step = Math.floor((stop - start) / (domain.length - padding + 2 * outerPadding));
      range = steps(start + Math.round((stop - start - (domain.length - padding) * step) / 2), step);
      if (reverse) range.reverse();
      rangeBand = Math.round(step * (1 - padding));
      ranger = {
        t: "rangeRoundBands",
        a: arguments
      };
      return scale;
    };
    scale.rangeBand = function() {
      return rangeBand;
    };
    scale.rangeExtent = function() {
      return d3_scaleExtent(ranger.a[0]);
    };
    scale.copy = function() {
      return d3_scale_ordinal(domain, ranger);
    };
    return scale.domain(domain);
  }
  d3.scale.category10 = function() {
    return d3.scale.ordinal().range(d3_category10);
  };
  d3.scale.category20 = function() {
    return d3.scale.ordinal().range(d3_category20);
  };
  d3.scale.category20b = function() {
    return d3.scale.ordinal().range(d3_category20b);
  };
  d3.scale.category20c = function() {
    return d3.scale.ordinal().range(d3_category20c);
  };
  var d3_category10 = [ 2062260, 16744206, 2924588, 14034728, 9725885, 9197131, 14907330, 8355711, 12369186, 1556175 ].map(d3_rgbString);
  var d3_category20 = [ 2062260, 11454440, 16744206, 16759672, 2924588, 10018698, 14034728, 16750742, 9725885, 12955861, 9197131, 12885140, 14907330, 16234194, 8355711, 13092807, 12369186, 14408589, 1556175, 10410725 ].map(d3_rgbString);
  var d3_category20b = [ 3750777, 5395619, 7040719, 10264286, 6519097, 9216594, 11915115, 13556636, 9202993, 12426809, 15186514, 15190932, 8666169, 11356490, 14049643, 15177372, 8077683, 10834324, 13528509, 14589654 ].map(d3_rgbString);
  var d3_category20c = [ 3244733, 7057110, 10406625, 13032431, 15095053, 16616764, 16625259, 16634018, 3253076, 7652470, 10607003, 13101504, 7695281, 10394312, 12369372, 14342891, 6513507, 9868950, 12434877, 14277081 ].map(d3_rgbString);
  d3.scale.quantile = function() {
    return d3_scale_quantile([], []);
  };
  function d3_scale_quantile(domain, range) {
    var thresholds;
    function rescale() {
      var k = 0, q = range.length;
      thresholds = [];
      while (++k < q) thresholds[k - 1] = d3.quantile(domain, k / q);
      return scale;
    }
    function scale(x) {
      if (!isNaN(x = +x)) return range[d3.bisect(thresholds, x)];
    }
    scale.domain = function(x) {
      if (!arguments.length) return domain;
      domain = x.map(d3_number).filter(d3_numeric).sort(d3_ascending);
      return rescale();
    };
    scale.range = function(x) {
      if (!arguments.length) return range;
      range = x;
      return rescale();
    };
    scale.quantiles = function() {
      return thresholds;
    };
    scale.invertExtent = function(y) {
      y = range.indexOf(y);
      return y < 0 ? [ NaN, NaN ] : [ y > 0 ? thresholds[y - 1] : domain[0], y < thresholds.length ? thresholds[y] : domain[domain.length - 1] ];
    };
    scale.copy = function() {
      return d3_scale_quantile(domain, range);
    };
    return rescale();
  }
  d3.scale.quantize = function() {
    return d3_scale_quantize(0, 1, [ 0, 1 ]);
  };
  function d3_scale_quantize(x0, x1, range) {
    var kx, i;
    function scale(x) {
      return range[Math.max(0, Math.min(i, Math.floor(kx * (x - x0))))];
    }
    function rescale() {
      kx = range.length / (x1 - x0);
      i = range.length - 1;
      return scale;
    }
    scale.domain = function(x) {
      if (!arguments.length) return [ x0, x1 ];
      x0 = +x[0];
      x1 = +x[x.length - 1];
      return rescale();
    };
    scale.range = function(x) {
      if (!arguments.length) return range;
      range = x;
      return rescale();
    };
    scale.invertExtent = function(y) {
      y = range.indexOf(y);
      y = y < 0 ? NaN : y / kx + x0;
      return [ y, y + 1 / kx ];
    };
    scale.copy = function() {
      return d3_scale_quantize(x0, x1, range);
    };
    return rescale();
  }
  d3.scale.threshold = function() {
    return d3_scale_threshold([ .5 ], [ 0, 1 ]);
  };
  function d3_scale_threshold(domain, range) {
    function scale(x) {
      if (x <= x) return range[d3.bisect(domain, x)];
    }
    scale.domain = function(_) {
      if (!arguments.length) return domain;
      domain = _;
      return scale;
    };
    scale.range = function(_) {
      if (!arguments.length) return range;
      range = _;
      return scale;
    };
    scale.invertExtent = function(y) {
      y = range.indexOf(y);
      return [ domain[y - 1], domain[y] ];
    };
    scale.copy = function() {
      return d3_scale_threshold(domain, range);
    };
    return scale;
  }
  d3.scale.identity = function() {
    return d3_scale_identity([ 0, 1 ]);
  };
  function d3_scale_identity(domain) {
    function identity(x) {
      return +x;
    }
    identity.invert = identity;
    identity.domain = identity.range = function(x) {
      if (!arguments.length) return domain;
      domain = x.map(identity);
      return identity;
    };
    identity.ticks = function(m) {
      return d3_scale_linearTicks(domain, m);
    };
    identity.tickFormat = function(m, format) {
      return d3_scale_linearTickFormat(domain, m, format);
    };
    identity.copy = function() {
      return d3_scale_identity(domain);
    };
    return identity;
  }
  d3.svg = {};
  function d3_zero() {
    return 0;
  }
  d3.svg.arc = function() {
    var innerRadius = d3_svg_arcInnerRadius, outerRadius = d3_svg_arcOuterRadius, cornerRadius = d3_zero, padRadius = d3_svg_arcAuto, startAngle = d3_svg_arcStartAngle, endAngle = d3_svg_arcEndAngle, padAngle = d3_svg_arcPadAngle;
    function arc() {
      var r0 = Math.max(0, +innerRadius.apply(this, arguments)), r1 = Math.max(0, +outerRadius.apply(this, arguments)), a0 = startAngle.apply(this, arguments) - halfπ, a1 = endAngle.apply(this, arguments) - halfπ, da = Math.abs(a1 - a0), cw = a0 > a1 ? 0 : 1;
      if (r1 < r0) rc = r1, r1 = r0, r0 = rc;
      if (da >= τε) return circleSegment(r1, cw) + (r0 ? circleSegment(r0, 1 - cw) : "") + "Z";
      var rc, cr, rp, ap, p0 = 0, p1 = 0, x0, y0, x1, y1, x2, y2, x3, y3, path = [];
      if (ap = (+padAngle.apply(this, arguments) || 0) / 2) {
        rp = padRadius === d3_svg_arcAuto ? Math.sqrt(r0 * r0 + r1 * r1) : +padRadius.apply(this, arguments);
        if (!cw) p1 *= -1;
        if (r1) p1 = d3_asin(rp / r1 * Math.sin(ap));
        if (r0) p0 = d3_asin(rp / r0 * Math.sin(ap));
      }
      if (r1) {
        x0 = r1 * Math.cos(a0 + p1);
        y0 = r1 * Math.sin(a0 + p1);
        x1 = r1 * Math.cos(a1 - p1);
        y1 = r1 * Math.sin(a1 - p1);
        var l1 = Math.abs(a1 - a0 - 2 * p1) <= π ? 0 : 1;
        if (p1 && d3_svg_arcSweep(x0, y0, x1, y1) === cw ^ l1) {
          var h1 = (a0 + a1) / 2;
          x0 = r1 * Math.cos(h1);
          y0 = r1 * Math.sin(h1);
          x1 = y1 = null;
        }
      } else {
        x0 = y0 = 0;
      }
      if (r0) {
        x2 = r0 * Math.cos(a1 - p0);
        y2 = r0 * Math.sin(a1 - p0);
        x3 = r0 * Math.cos(a0 + p0);
        y3 = r0 * Math.sin(a0 + p0);
        var l0 = Math.abs(a0 - a1 + 2 * p0) <= π ? 0 : 1;
        if (p0 && d3_svg_arcSweep(x2, y2, x3, y3) === 1 - cw ^ l0) {
          var h0 = (a0 + a1) / 2;
          x2 = r0 * Math.cos(h0);
          y2 = r0 * Math.sin(h0);
          x3 = y3 = null;
        }
      } else {
        x2 = y2 = 0;
      }
      if (da > ε && (rc = Math.min(Math.abs(r1 - r0) / 2, +cornerRadius.apply(this, arguments))) > .001) {
        cr = r0 < r1 ^ cw ? 0 : 1;
        var rc1 = rc, rc0 = rc;
        if (da < π) {
          var oc = x3 == null ? [ x2, y2 ] : x1 == null ? [ x0, y0 ] : d3_geom_polygonIntersect([ x0, y0 ], [ x3, y3 ], [ x1, y1 ], [ x2, y2 ]), ax = x0 - oc[0], ay = y0 - oc[1], bx = x1 - oc[0], by = y1 - oc[1], kc = 1 / Math.sin(Math.acos((ax * bx + ay * by) / (Math.sqrt(ax * ax + ay * ay) * Math.sqrt(bx * bx + by * by))) / 2), lc = Math.sqrt(oc[0] * oc[0] + oc[1] * oc[1]);
          rc0 = Math.min(rc, (r0 - lc) / (kc - 1));
          rc1 = Math.min(rc, (r1 - lc) / (kc + 1));
        }
        if (x1 != null) {
          var t30 = d3_svg_arcCornerTangents(x3 == null ? [ x2, y2 ] : [ x3, y3 ], [ x0, y0 ], r1, rc1, cw), t12 = d3_svg_arcCornerTangents([ x1, y1 ], [ x2, y2 ], r1, rc1, cw);
          if (rc === rc1) {
            path.push("M", t30[0], "A", rc1, ",", rc1, " 0 0,", cr, " ", t30[1], "A", r1, ",", r1, " 0 ", 1 - cw ^ d3_svg_arcSweep(t30[1][0], t30[1][1], t12[1][0], t12[1][1]), ",", cw, " ", t12[1], "A", rc1, ",", rc1, " 0 0,", cr, " ", t12[0]);
          } else {
            path.push("M", t30[0], "A", rc1, ",", rc1, " 0 1,", cr, " ", t12[0]);
          }
        } else {
          path.push("M", x0, ",", y0);
        }
        if (x3 != null) {
          var t03 = d3_svg_arcCornerTangents([ x0, y0 ], [ x3, y3 ], r0, -rc0, cw), t21 = d3_svg_arcCornerTangents([ x2, y2 ], x1 == null ? [ x0, y0 ] : [ x1, y1 ], r0, -rc0, cw);
          if (rc === rc0) {
            path.push("L", t21[0], "A", rc0, ",", rc0, " 0 0,", cr, " ", t21[1], "A", r0, ",", r0, " 0 ", cw ^ d3_svg_arcSweep(t21[1][0], t21[1][1], t03[1][0], t03[1][1]), ",", 1 - cw, " ", t03[1], "A", rc0, ",", rc0, " 0 0,", cr, " ", t03[0]);
          } else {
            path.push("L", t21[0], "A", rc0, ",", rc0, " 0 0,", cr, " ", t03[0]);
          }
        } else {
          path.push("L", x2, ",", y2);
        }
      } else {
        path.push("M", x0, ",", y0);
        if (x1 != null) path.push("A", r1, ",", r1, " 0 ", l1, ",", cw, " ", x1, ",", y1);
        path.push("L", x2, ",", y2);
        if (x3 != null) path.push("A", r0, ",", r0, " 0 ", l0, ",", 1 - cw, " ", x3, ",", y3);
      }
      path.push("Z");
      return path.join("");
    }
    function circleSegment(r1, cw) {
      return "M0," + r1 + "A" + r1 + "," + r1 + " 0 1," + cw + " 0," + -r1 + "A" + r1 + "," + r1 + " 0 1," + cw + " 0," + r1;
    }
    arc.innerRadius = function(v) {
      if (!arguments.length) return innerRadius;
      innerRadius = d3_functor(v);
      return arc;
    };
    arc.outerRadius = function(v) {
      if (!arguments.length) return outerRadius;
      outerRadius = d3_functor(v);
      return arc;
    };
    arc.cornerRadius = function(v) {
      if (!arguments.length) return cornerRadius;
      cornerRadius = d3_functor(v);
      return arc;
    };
    arc.padRadius = function(v) {
      if (!arguments.length) return padRadius;
      padRadius = v == d3_svg_arcAuto ? d3_svg_arcAuto : d3_functor(v);
      return arc;
    };
    arc.startAngle = function(v) {
      if (!arguments.length) return startAngle;
      startAngle = d3_functor(v);
      return arc;
    };
    arc.endAngle = function(v) {
      if (!arguments.length) return endAngle;
      endAngle = d3_functor(v);
      return arc;
    };
    arc.padAngle = function(v) {
      if (!arguments.length) return padAngle;
      padAngle = d3_functor(v);
      return arc;
    };
    arc.centroid = function() {
      var r = (+innerRadius.apply(this, arguments) + +outerRadius.apply(this, arguments)) / 2, a = (+startAngle.apply(this, arguments) + +endAngle.apply(this, arguments)) / 2 - halfπ;
      return [ Math.cos(a) * r, Math.sin(a) * r ];
    };
    return arc;
  };
  var d3_svg_arcAuto = "auto";
  function d3_svg_arcInnerRadius(d) {
    return d.innerRadius;
  }
  function d3_svg_arcOuterRadius(d) {
    return d.outerRadius;
  }
  function d3_svg_arcStartAngle(d) {
    return d.startAngle;
  }
  function d3_svg_arcEndAngle(d) {
    return d.endAngle;
  }
  function d3_svg_arcPadAngle(d) {
    return d && d.padAngle;
  }
  function d3_svg_arcSweep(x0, y0, x1, y1) {
    return (x0 - x1) * y0 - (y0 - y1) * x0 > 0 ? 0 : 1;
  }
  function d3_svg_arcCornerTangents(p0, p1, r1, rc, cw) {
    var x01 = p0[0] - p1[0], y01 = p0[1] - p1[1], lo = (cw ? rc : -rc) / Math.sqrt(x01 * x01 + y01 * y01), ox = lo * y01, oy = -lo * x01, x1 = p0[0] + ox, y1 = p0[1] + oy, x2 = p1[0] + ox, y2 = p1[1] + oy, x3 = (x1 + x2) / 2, y3 = (y1 + y2) / 2, dx = x2 - x1, dy = y2 - y1, d2 = dx * dx + dy * dy, r = r1 - rc, D = x1 * y2 - x2 * y1, d = (dy < 0 ? -1 : 1) * Math.sqrt(Math.max(0, r * r * d2 - D * D)), cx0 = (D * dy - dx * d) / d2, cy0 = (-D * dx - dy * d) / d2, cx1 = (D * dy + dx * d) / d2, cy1 = (-D * dx + dy * d) / d2, dx0 = cx0 - x3, dy0 = cy0 - y3, dx1 = cx1 - x3, dy1 = cy1 - y3;
    if (dx0 * dx0 + dy0 * dy0 > dx1 * dx1 + dy1 * dy1) cx0 = cx1, cy0 = cy1;
    return [ [ cx0 - ox, cy0 - oy ], [ cx0 * r1 / r, cy0 * r1 / r ] ];
  }
  function d3_svg_line(projection) {
    var x = d3_geom_pointX, y = d3_geom_pointY, defined = d3_true, interpolate = d3_svg_lineLinear, interpolateKey = interpolate.key, tension = .7;
    function line(data) {
      var segments = [], points = [], i = -1, n = data.length, d, fx = d3_functor(x), fy = d3_functor(y);
      function segment() {
        segments.push("M", interpolate(projection(points), tension));
      }
      while (++i < n) {
        if (defined.call(this, d = data[i], i)) {
          points.push([ +fx.call(this, d, i), +fy.call(this, d, i) ]);
        } else if (points.length) {
          segment();
          points = [];
        }
      }
      if (points.length) segment();
      return segments.length ? segments.join("") : null;
    }
    line.x = function(_) {
      if (!arguments.length) return x;
      x = _;
      return line;
    };
    line.y = function(_) {
      if (!arguments.length) return y;
      y = _;
      return line;
    };
    line.defined = function(_) {
      if (!arguments.length) return defined;
      defined = _;
      return line;
    };
    line.interpolate = function(_) {
      if (!arguments.length) return interpolateKey;
      if (typeof _ === "function") interpolateKey = interpolate = _; else interpolateKey = (interpolate = d3_svg_lineInterpolators.get(_) || d3_svg_lineLinear).key;
      return line;
    };
    line.tension = function(_) {
      if (!arguments.length) return tension;
      tension = _;
      return line;
    };
    return line;
  }
  d3.svg.line = function() {
    return d3_svg_line(d3_identity);
  };
  var d3_svg_lineInterpolators = d3.map({
    linear: d3_svg_lineLinear,
    "linear-closed": d3_svg_lineLinearClosed,
    step: d3_svg_lineStep,
    "step-before": d3_svg_lineStepBefore,
    "step-after": d3_svg_lineStepAfter,
    basis: d3_svg_lineBasis,
    "basis-open": d3_svg_lineBasisOpen,
    "basis-closed": d3_svg_lineBasisClosed,
    bundle: d3_svg_lineBundle,
    cardinal: d3_svg_lineCardinal,
    "cardinal-open": d3_svg_lineCardinalOpen,
    "cardinal-closed": d3_svg_lineCardinalClosed,
    monotone: d3_svg_lineMonotone
  });
  d3_svg_lineInterpolators.forEach(function(key, value) {
    value.key = key;
    value.closed = /-closed$/.test(key);
  });
  function d3_svg_lineLinear(points) {
    return points.length > 1 ? points.join("L") : points + "Z";
  }
  function d3_svg_lineLinearClosed(points) {
    return points.join("L") + "Z";
  }
  function d3_svg_lineStep(points) {
    var i = 0, n = points.length, p = points[0], path = [ p[0], ",", p[1] ];
    while (++i < n) path.push("H", (p[0] + (p = points[i])[0]) / 2, "V", p[1]);
    if (n > 1) path.push("H", p[0]);
    return path.join("");
  }
  function d3_svg_lineStepBefore(points) {
    var i = 0, n = points.length, p = points[0], path = [ p[0], ",", p[1] ];
    while (++i < n) path.push("V", (p = points[i])[1], "H", p[0]);
    return path.join("");
  }
  function d3_svg_lineStepAfter(points) {
    var i = 0, n = points.length, p = points[0], path = [ p[0], ",", p[1] ];
    while (++i < n) path.push("H", (p = points[i])[0], "V", p[1]);
    return path.join("");
  }
  function d3_svg_lineCardinalOpen(points, tension) {
    return points.length < 4 ? d3_svg_lineLinear(points) : points[1] + d3_svg_lineHermite(points.slice(1, -1), d3_svg_lineCardinalTangents(points, tension));
  }
  function d3_svg_lineCardinalClosed(points, tension) {
    return points.length < 3 ? d3_svg_lineLinearClosed(points) : points[0] + d3_svg_lineHermite((points.push(points[0]), 
    points), d3_svg_lineCardinalTangents([ points[points.length - 2] ].concat(points, [ points[1] ]), tension));
  }
  function d3_svg_lineCardinal(points, tension) {
    return points.length < 3 ? d3_svg_lineLinear(points) : points[0] + d3_svg_lineHermite(points, d3_svg_lineCardinalTangents(points, tension));
  }
  function d3_svg_lineHermite(points, tangents) {
    if (tangents.length < 1 || points.length != tangents.length && points.length != tangents.length + 2) {
      return d3_svg_lineLinear(points);
    }
    var quad = points.length != tangents.length, path = "", p0 = points[0], p = points[1], t0 = tangents[0], t = t0, pi = 1;
    if (quad) {
      path += "Q" + (p[0] - t0[0] * 2 / 3) + "," + (p[1] - t0[1] * 2 / 3) + "," + p[0] + "," + p[1];
      p0 = points[1];
      pi = 2;
    }
    if (tangents.length > 1) {
      t = tangents[1];
      p = points[pi];
      pi++;
      path += "C" + (p0[0] + t0[0]) + "," + (p0[1] + t0[1]) + "," + (p[0] - t[0]) + "," + (p[1] - t[1]) + "," + p[0] + "," + p[1];
      for (var i = 2; i < tangents.length; i++, pi++) {
        p = points[pi];
        t = tangents[i];
        path += "S" + (p[0] - t[0]) + "," + (p[1] - t[1]) + "," + p[0] + "," + p[1];
      }
    }
    if (quad) {
      var lp = points[pi];
      path += "Q" + (p[0] + t[0] * 2 / 3) + "," + (p[1] + t[1] * 2 / 3) + "," + lp[0] + "," + lp[1];
    }
    return path;
  }
  function d3_svg_lineCardinalTangents(points, tension) {
    var tangents = [], a = (1 - tension) / 2, p0, p1 = points[0], p2 = points[1], i = 1, n = points.length;
    while (++i < n) {
      p0 = p1;
      p1 = p2;
      p2 = points[i];
      tangents.push([ a * (p2[0] - p0[0]), a * (p2[1] - p0[1]) ]);
    }
    return tangents;
  }
  function d3_svg_lineBasis(points) {
    if (points.length < 3) return d3_svg_lineLinear(points);
    var i = 1, n = points.length, pi = points[0], x0 = pi[0], y0 = pi[1], px = [ x0, x0, x0, (pi = points[1])[0] ], py = [ y0, y0, y0, pi[1] ], path = [ x0, ",", y0, "L", d3_svg_lineDot4(d3_svg_lineBasisBezier3, px), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier3, py) ];
    points.push(points[n - 1]);
    while (++i <= n) {
      pi = points[i];
      px.shift();
      px.push(pi[0]);
      py.shift();
      py.push(pi[1]);
      d3_svg_lineBasisBezier(path, px, py);
    }
    points.pop();
    path.push("L", pi);
    return path.join("");
  }
  function d3_svg_lineBasisOpen(points) {
    if (points.length < 4) return d3_svg_lineLinear(points);
    var path = [], i = -1, n = points.length, pi, px = [ 0 ], py = [ 0 ];
    while (++i < 3) {
      pi = points[i];
      px.push(pi[0]);
      py.push(pi[1]);
    }
    path.push(d3_svg_lineDot4(d3_svg_lineBasisBezier3, px) + "," + d3_svg_lineDot4(d3_svg_lineBasisBezier3, py));
    --i;
    while (++i < n) {
      pi = points[i];
      px.shift();
      px.push(pi[0]);
      py.shift();
      py.push(pi[1]);
      d3_svg_lineBasisBezier(path, px, py);
    }
    return path.join("");
  }
  function d3_svg_lineBasisClosed(points) {
    var path, i = -1, n = points.length, m = n + 4, pi, px = [], py = [];
    while (++i < 4) {
      pi = points[i % n];
      px.push(pi[0]);
      py.push(pi[1]);
    }
    path = [ d3_svg_lineDot4(d3_svg_lineBasisBezier3, px), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier3, py) ];
    --i;
    while (++i < m) {
      pi = points[i % n];
      px.shift();
      px.push(pi[0]);
      py.shift();
      py.push(pi[1]);
      d3_svg_lineBasisBezier(path, px, py);
    }
    return path.join("");
  }
  function d3_svg_lineBundle(points, tension) {
    var n = points.length - 1;
    if (n) {
      var x0 = points[0][0], y0 = points[0][1], dx = points[n][0] - x0, dy = points[n][1] - y0, i = -1, p, t;
      while (++i <= n) {
        p = points[i];
        t = i / n;
        p[0] = tension * p[0] + (1 - tension) * (x0 + t * dx);
        p[1] = tension * p[1] + (1 - tension) * (y0 + t * dy);
      }
    }
    return d3_svg_lineBasis(points);
  }
  function d3_svg_lineDot4(a, b) {
    return a[0] * b[0] + a[1] * b[1] + a[2] * b[2] + a[3] * b[3];
  }
  var d3_svg_lineBasisBezier1 = [ 0, 2 / 3, 1 / 3, 0 ], d3_svg_lineBasisBezier2 = [ 0, 1 / 3, 2 / 3, 0 ], d3_svg_lineBasisBezier3 = [ 0, 1 / 6, 2 / 3, 1 / 6 ];
  function d3_svg_lineBasisBezier(path, x, y) {
    path.push("C", d3_svg_lineDot4(d3_svg_lineBasisBezier1, x), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier1, y), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier2, x), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier2, y), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier3, x), ",", d3_svg_lineDot4(d3_svg_lineBasisBezier3, y));
  }
  function d3_svg_lineSlope(p0, p1) {
    return (p1[1] - p0[1]) / (p1[0] - p0[0]);
  }
  function d3_svg_lineFiniteDifferences(points) {
    var i = 0, j = points.length - 1, m = [], p0 = points[0], p1 = points[1], d = m[0] = d3_svg_lineSlope(p0, p1);
    while (++i < j) {
      m[i] = (d + (d = d3_svg_lineSlope(p0 = p1, p1 = points[i + 1]))) / 2;
    }
    m[i] = d;
    return m;
  }
  function d3_svg_lineMonotoneTangents(points) {
    var tangents = [], d, a, b, s, m = d3_svg_lineFiniteDifferences(points), i = -1, j = points.length - 1;
    while (++i < j) {
      d = d3_svg_lineSlope(points[i], points[i + 1]);
      if (abs(d) < ε) {
        m[i] = m[i + 1] = 0;
      } else {
        a = m[i] / d;
        b = m[i + 1] / d;
        s = a * a + b * b;
        if (s > 9) {
          s = d * 3 / Math.sqrt(s);
          m[i] = s * a;
          m[i + 1] = s * b;
        }
      }
    }
    i = -1;
    while (++i <= j) {
      s = (points[Math.min(j, i + 1)][0] - points[Math.max(0, i - 1)][0]) / (6 * (1 + m[i] * m[i]));
      tangents.push([ s || 0, m[i] * s || 0 ]);
    }
    return tangents;
  }
  function d3_svg_lineMonotone(points) {
    return points.length < 3 ? d3_svg_lineLinear(points) : points[0] + d3_svg_lineHermite(points, d3_svg_lineMonotoneTangents(points));
  }
  d3.svg.line.radial = function() {
    var line = d3_svg_line(d3_svg_lineRadial);
    line.radius = line.x, delete line.x;
    line.angle = line.y, delete line.y;
    return line;
  };
  function d3_svg_lineRadial(points) {
    var point, i = -1, n = points.length, r, a;
    while (++i < n) {
      point = points[i];
      r = point[0];
      a = point[1] - halfπ;
      point[0] = r * Math.cos(a);
      point[1] = r * Math.sin(a);
    }
    return points;
  }
  function d3_svg_area(projection) {
    var x0 = d3_geom_pointX, x1 = d3_geom_pointX, y0 = 0, y1 = d3_geom_pointY, defined = d3_true, interpolate = d3_svg_lineLinear, interpolateKey = interpolate.key, interpolateReverse = interpolate, L = "L", tension = .7;
    function area(data) {
      var segments = [], points0 = [], points1 = [], i = -1, n = data.length, d, fx0 = d3_functor(x0), fy0 = d3_functor(y0), fx1 = x0 === x1 ? function() {
        return x;
      } : d3_functor(x1), fy1 = y0 === y1 ? function() {
        return y;
      } : d3_functor(y1), x, y;
      function segment() {
        segments.push("M", interpolate(projection(points1), tension), L, interpolateReverse(projection(points0.reverse()), tension), "Z");
      }
      while (++i < n) {
        if (defined.call(this, d = data[i], i)) {
          points0.push([ x = +fx0.call(this, d, i), y = +fy0.call(this, d, i) ]);
          points1.push([ +fx1.call(this, d, i), +fy1.call(this, d, i) ]);
        } else if (points0.length) {
          segment();
          points0 = [];
          points1 = [];
        }
      }
      if (points0.length) segment();
      return segments.length ? segments.join("") : null;
    }
    area.x = function(_) {
      if (!arguments.length) return x1;
      x0 = x1 = _;
      return area;
    };
    area.x0 = function(_) {
      if (!arguments.length) return x0;
      x0 = _;
      return area;
    };
    area.x1 = function(_) {
      if (!arguments.length) return x1;
      x1 = _;
      return area;
    };
    area.y = function(_) {
      if (!arguments.length) return y1;
      y0 = y1 = _;
      return area;
    };
    area.y0 = function(_) {
      if (!arguments.length) return y0;
      y0 = _;
      return area;
    };
    area.y1 = function(_) {
      if (!arguments.length) return y1;
      y1 = _;
      return area;
    };
    area.defined = function(_) {
      if (!arguments.length) return defined;
      defined = _;
      return area;
    };
    area.interpolate = function(_) {
      if (!arguments.length) return interpolateKey;
      if (typeof _ === "function") interpolateKey = interpolate = _; else interpolateKey = (interpolate = d3_svg_lineInterpolators.get(_) || d3_svg_lineLinear).key;
      interpolateReverse = interpolate.reverse || interpolate;
      L = interpolate.closed ? "M" : "L";
      return area;
    };
    area.tension = function(_) {
      if (!arguments.length) return tension;
      tension = _;
      return area;
    };
    return area;
  }
  d3_svg_lineStepBefore.reverse = d3_svg_lineStepAfter;
  d3_svg_lineStepAfter.reverse = d3_svg_lineStepBefore;
  d3.svg.area = function() {
    return d3_svg_area(d3_identity);
  };
  d3.svg.area.radial = function() {
    var area = d3_svg_area(d3_svg_lineRadial);
    area.radius = area.x, delete area.x;
    area.innerRadius = area.x0, delete area.x0;
    area.outerRadius = area.x1, delete area.x1;
    area.angle = area.y, delete area.y;
    area.startAngle = area.y0, delete area.y0;
    area.endAngle = area.y1, delete area.y1;
    return area;
  };
  d3.svg.chord = function() {
    var source = d3_source, target = d3_target, radius = d3_svg_chordRadius, startAngle = d3_svg_arcStartAngle, endAngle = d3_svg_arcEndAngle;
    function chord(d, i) {
      var s = subgroup(this, source, d, i), t = subgroup(this, target, d, i);
      return "M" + s.p0 + arc(s.r, s.p1, s.a1 - s.a0) + (equals(s, t) ? curve(s.r, s.p1, s.r, s.p0) : curve(s.r, s.p1, t.r, t.p0) + arc(t.r, t.p1, t.a1 - t.a0) + curve(t.r, t.p1, s.r, s.p0)) + "Z";
    }
    function subgroup(self, f, d, i) {
      var subgroup = f.call(self, d, i), r = radius.call(self, subgroup, i), a0 = startAngle.call(self, subgroup, i) - halfπ, a1 = endAngle.call(self, subgroup, i) - halfπ;
      return {
        r: r,
        a0: a0,
        a1: a1,
        p0: [ r * Math.cos(a0), r * Math.sin(a0) ],
        p1: [ r * Math.cos(a1), r * Math.sin(a1) ]
      };
    }
    function equals(a, b) {
      return a.a0 == b.a0 && a.a1 == b.a1;
    }
    function arc(r, p, a) {
      return "A" + r + "," + r + " 0 " + +(a > π) + ",1 " + p;
    }
    function curve(r0, p0, r1, p1) {
      return "Q 0,0 " + p1;
    }
    chord.radius = function(v) {
      if (!arguments.length) return radius;
      radius = d3_functor(v);
      return chord;
    };
    chord.source = function(v) {
      if (!arguments.length) return source;
      source = d3_functor(v);
      return chord;
    };
    chord.target = function(v) {
      if (!arguments.length) return target;
      target = d3_functor(v);
      return chord;
    };
    chord.startAngle = function(v) {
      if (!arguments.length) return startAngle;
      startAngle = d3_functor(v);
      return chord;
    };
    chord.endAngle = function(v) {
      if (!arguments.length) return endAngle;
      endAngle = d3_functor(v);
      return chord;
    };
    return chord;
  };
  function d3_svg_chordRadius(d) {
    return d.radius;
  }
  d3.svg.diagonal = function() {
    var source = d3_source, target = d3_target, projection = d3_svg_diagonalProjection;
    function diagonal(d, i) {
      var p0 = source.call(this, d, i), p3 = target.call(this, d, i), m = (p0.y + p3.y) / 2, p = [ p0, {
        x: p0.x,
        y: m
      }, {
        x: p3.x,
        y: m
      }, p3 ];
      p = p.map(projection);
      return "M" + p[0] + "C" + p[1] + " " + p[2] + " " + p[3];
    }
    diagonal.source = function(x) {
      if (!arguments.length) return source;
      source = d3_functor(x);
      return diagonal;
    };
    diagonal.target = function(x) {
      if (!arguments.length) return target;
      target = d3_functor(x);
      return diagonal;
    };
    diagonal.projection = function(x) {
      if (!arguments.length) return projection;
      projection = x;
      return diagonal;
    };
    return diagonal;
  };
  function d3_svg_diagonalProjection(d) {
    return [ d.x, d.y ];
  }
  d3.svg.diagonal.radial = function() {
    var diagonal = d3.svg.diagonal(), projection = d3_svg_diagonalProjection, projection_ = diagonal.projection;
    diagonal.projection = function(x) {
      return arguments.length ? projection_(d3_svg_diagonalRadialProjection(projection = x)) : projection;
    };
    return diagonal;
  };
  function d3_svg_diagonalRadialProjection(projection) {
    return function() {
      var d = projection.apply(this, arguments), r = d[0], a = d[1] - halfπ;
      return [ r * Math.cos(a), r * Math.sin(a) ];
    };
  }
  d3.svg.symbol = function() {
    var type = d3_svg_symbolType, size = d3_svg_symbolSize;
    function symbol(d, i) {
      return (d3_svg_symbols.get(type.call(this, d, i)) || d3_svg_symbolCircle)(size.call(this, d, i));
    }
    symbol.type = function(x) {
      if (!arguments.length) return type;
      type = d3_functor(x);
      return symbol;
    };
    symbol.size = function(x) {
      if (!arguments.length) return size;
      size = d3_functor(x);
      return symbol;
    };
    return symbol;
  };
  function d3_svg_symbolSize() {
    return 64;
  }
  function d3_svg_symbolType() {
    return "circle";
  }
  function d3_svg_symbolCircle(size) {
    var r = Math.sqrt(size / π);
    return "M0," + r + "A" + r + "," + r + " 0 1,1 0," + -r + "A" + r + "," + r + " 0 1,1 0," + r + "Z";
  }
  var d3_svg_symbols = d3.map({
    circle: d3_svg_symbolCircle,
    cross: function(size) {
      var r = Math.sqrt(size / 5) / 2;
      return "M" + -3 * r + "," + -r + "H" + -r + "V" + -3 * r + "H" + r + "V" + -r + "H" + 3 * r + "V" + r + "H" + r + "V" + 3 * r + "H" + -r + "V" + r + "H" + -3 * r + "Z";
    },
    diamond: function(size) {
      var ry = Math.sqrt(size / (2 * d3_svg_symbolTan30)), rx = ry * d3_svg_symbolTan30;
      return "M0," + -ry + "L" + rx + ",0" + " 0," + ry + " " + -rx + ",0" + "Z";
    },
    square: function(size) {
      var r = Math.sqrt(size) / 2;
      return "M" + -r + "," + -r + "L" + r + "," + -r + " " + r + "," + r + " " + -r + "," + r + "Z";
    },
    "triangle-down": function(size) {
      var rx = Math.sqrt(size / d3_svg_symbolSqrt3), ry = rx * d3_svg_symbolSqrt3 / 2;
      return "M0," + ry + "L" + rx + "," + -ry + " " + -rx + "," + -ry + "Z";
    },
    "triangle-up": function(size) {
      var rx = Math.sqrt(size / d3_svg_symbolSqrt3), ry = rx * d3_svg_symbolSqrt3 / 2;
      return "M0," + -ry + "L" + rx + "," + ry + " " + -rx + "," + ry + "Z";
    }
  });
  d3.svg.symbolTypes = d3_svg_symbols.keys();
  var d3_svg_symbolSqrt3 = Math.sqrt(3), d3_svg_symbolTan30 = Math.tan(30 * d3_radians);
  d3_selectionPrototype.transition = function(name) {
    var id = d3_transitionInheritId || ++d3_transitionId, ns = d3_transitionNamespace(name), subgroups = [], subgroup, node, transition = d3_transitionInherit || {
      time: Date.now(),
      ease: d3_ease_cubicInOut,
      delay: 0,
      duration: 250
    };
    for (var j = -1, m = this.length; ++j < m; ) {
      subgroups.push(subgroup = []);
      for (var group = this[j], i = -1, n = group.length; ++i < n; ) {
        if (node = group[i]) d3_transitionNode(node, i, ns, id, transition);
        subgroup.push(node);
      }
    }
    return d3_transition(subgroups, ns, id);
  };
  d3_selectionPrototype.interrupt = function(name) {
    return this.each(name == null ? d3_selection_interrupt : d3_selection_interruptNS(d3_transitionNamespace(name)));
  };
  var d3_selection_interrupt = d3_selection_interruptNS(d3_transitionNamespace());
  function d3_selection_interruptNS(ns) {
    return function() {
      var lock, activeId, active;
      if ((lock = this[ns]) && (active = lock[activeId = lock.active])) {
        active.timer.c = null;
        active.timer.t = NaN;
        if (--lock.count) delete lock[activeId]; else delete this[ns];
        lock.active += .5;
        active.event && active.event.interrupt.call(this, this.__data__, active.index);
      }
    };
  }
  function d3_transition(groups, ns, id) {
    d3_subclass(groups, d3_transitionPrototype);
    groups.namespace = ns;
    groups.id = id;
    return groups;
  }
  var d3_transitionPrototype = [], d3_transitionId = 0, d3_transitionInheritId, d3_transitionInherit;
  d3_transitionPrototype.call = d3_selectionPrototype.call;
  d3_transitionPrototype.empty = d3_selectionPrototype.empty;
  d3_transitionPrototype.node = d3_selectionPrototype.node;
  d3_transitionPrototype.size = d3_selectionPrototype.size;
  d3.transition = function(selection, name) {
    return selection && selection.transition ? d3_transitionInheritId ? selection.transition(name) : selection : d3.selection().transition(selection);
  };
  d3.transition.prototype = d3_transitionPrototype;
  d3_transitionPrototype.select = function(selector) {
    var id = this.id, ns = this.namespace, subgroups = [], subgroup, subnode, node;
    selector = d3_selection_selector(selector);
    for (var j = -1, m = this.length; ++j < m; ) {
      subgroups.push(subgroup = []);
      for (var group = this[j], i = -1, n = group.length; ++i < n; ) {
        if ((node = group[i]) && (subnode = selector.call(node, node.__data__, i, j))) {
          if ("__data__" in node) subnode.__data__ = node.__data__;
          d3_transitionNode(subnode, i, ns, id, node[ns][id]);
          subgroup.push(subnode);
        } else {
          subgroup.push(null);
        }
      }
    }
    return d3_transition(subgroups, ns, id);
  };
  d3_transitionPrototype.selectAll = function(selector) {
    var id = this.id, ns = this.namespace, subgroups = [], subgroup, subnodes, node, subnode, transition;
    selector = d3_selection_selectorAll(selector);
    for (var j = -1, m = this.length; ++j < m; ) {
      for (var group = this[j], i = -1, n = group.length; ++i < n; ) {
        if (node = group[i]) {
          transition = node[ns][id];
          subnodes = selector.call(node, node.__data__, i, j);
          subgroups.push(subgroup = []);
          for (var k = -1, o = subnodes.length; ++k < o; ) {
            if (subnode = subnodes[k]) d3_transitionNode(subnode, k, ns, id, transition);
            subgroup.push(subnode);
          }
        }
      }
    }
    return d3_transition(subgroups, ns, id);
  };
  d3_transitionPrototype.filter = function(filter) {
    var subgroups = [], subgroup, group, node;
    if (typeof filter !== "function") filter = d3_selection_filter(filter);
    for (var j = 0, m = this.length; j < m; j++) {
      subgroups.push(subgroup = []);
      for (var group = this[j], i = 0, n = group.length; i < n; i++) {
        if ((node = group[i]) && filter.call(node, node.__data__, i, j)) {
          subgroup.push(node);
        }
      }
    }
    return d3_transition(subgroups, this.namespace, this.id);
  };
  d3_transitionPrototype.tween = function(name, tween) {
    var id = this.id, ns = this.namespace;
    if (arguments.length < 2) return this.node()[ns][id].tween.get(name);
    return d3_selection_each(this, tween == null ? function(node) {
      node[ns][id].tween.remove(name);
    } : function(node) {
      node[ns][id].tween.set(name, tween);
    });
  };
  function d3_transition_tween(groups, name, value, tween) {
    var id = groups.id, ns = groups.namespace;
    return d3_selection_each(groups, typeof value === "function" ? function(node, i, j) {
      node[ns][id].tween.set(name, tween(value.call(node, node.__data__, i, j)));
    } : (value = tween(value), function(node) {
      node[ns][id].tween.set(name, value);
    }));
  }
  d3_transitionPrototype.attr = function(nameNS, value) {
    if (arguments.length < 2) {
      for (value in nameNS) this.attr(value, nameNS[value]);
      return this;
    }
    var interpolate = nameNS == "transform" ? d3_interpolateTransform : d3_interpolate, name = d3.ns.qualify(nameNS);
    function attrNull() {
      this.removeAttribute(name);
    }
    function attrNullNS() {
      this.removeAttributeNS(name.space, name.local);
    }
    function attrTween(b) {
      return b == null ? attrNull : (b += "", function() {
        var a = this.getAttribute(name), i;
        return a !== b && (i = interpolate(a, b), function(t) {
          this.setAttribute(name, i(t));
        });
      });
    }
    function attrTweenNS(b) {
      return b == null ? attrNullNS : (b += "", function() {
        var a = this.getAttributeNS(name.space, name.local), i;
        return a !== b && (i = interpolate(a, b), function(t) {
          this.setAttributeNS(name.space, name.local, i(t));
        });
      });
    }
    return d3_transition_tween(this, "attr." + nameNS, value, name.local ? attrTweenNS : attrTween);
  };
  d3_transitionPrototype.attrTween = function(nameNS, tween) {
    var name = d3.ns.qualify(nameNS);
    function attrTween(d, i) {
      var f = tween.call(this, d, i, this.getAttribute(name));
      return f && function(t) {
        this.setAttribute(name, f(t));
      };
    }
    function attrTweenNS(d, i) {
      var f = tween.call(this, d, i, this.getAttributeNS(name.space, name.local));
      return f && function(t) {
        this.setAttributeNS(name.space, name.local, f(t));
      };
    }
    return this.tween("attr." + nameNS, name.local ? attrTweenNS : attrTween);
  };
  d3_transitionPrototype.style = function(name, value, priority) {
    var n = arguments.length;
    if (n < 3) {
      if (typeof name !== "string") {
        if (n < 2) value = "";
        for (priority in name) this.style(priority, name[priority], value);
        return this;
      }
      priority = "";
    }
    function styleNull() {
      this.style.removeProperty(name);
    }
    function styleString(b) {
      return b == null ? styleNull : (b += "", function() {
        var a = d3_window(this).getComputedStyle(this, null).getPropertyValue(name), i;
        return a !== b && (i = d3_interpolate(a, b), function(t) {
          this.style.setProperty(name, i(t), priority);
        });
      });
    }
    return d3_transition_tween(this, "style." + name, value, styleString);
  };
  d3_transitionPrototype.styleTween = function(name, tween, priority) {
    if (arguments.length < 3) priority = "";
    function styleTween(d, i) {
      var f = tween.call(this, d, i, d3_window(this).getComputedStyle(this, null).getPropertyValue(name));
      return f && function(t) {
        this.style.setProperty(name, f(t), priority);
      };
    }
    return this.tween("style." + name, styleTween);
  };
  d3_transitionPrototype.text = function(value) {
    return d3_transition_tween(this, "text", value, d3_transition_text);
  };
  function d3_transition_text(b) {
    if (b == null) b = "";
    return function() {
      this.textContent = b;
    };
  }
  d3_transitionPrototype.remove = function() {
    var ns = this.namespace;
    return this.each("end.transition", function() {
      var p;
      if (this[ns].count < 2 && (p = this.parentNode)) p.removeChild(this);
    });
  };
  d3_transitionPrototype.ease = function(value) {
    var id = this.id, ns = this.namespace;
    if (arguments.length < 1) return this.node()[ns][id].ease;
    if (typeof value !== "function") value = d3.ease.apply(d3, arguments);
    return d3_selection_each(this, function(node) {
      node[ns][id].ease = value;
    });
  };
  d3_transitionPrototype.delay = function(value) {
    var id = this.id, ns = this.namespace;
    if (arguments.length < 1) return this.node()[ns][id].delay;
    return d3_selection_each(this, typeof value === "function" ? function(node, i, j) {
      node[ns][id].delay = +value.call(node, node.__data__, i, j);
    } : (value = +value, function(node) {
      node[ns][id].delay = value;
    }));
  };
  d3_transitionPrototype.duration = function(value) {
    var id = this.id, ns = this.namespace;
    if (arguments.length < 1) return this.node()[ns][id].duration;
    return d3_selection_each(this, typeof value === "function" ? function(node, i, j) {
      node[ns][id].duration = Math.max(1, value.call(node, node.__data__, i, j));
    } : (value = Math.max(1, value), function(node) {
      node[ns][id].duration = value;
    }));
  };
  d3_transitionPrototype.each = function(type, listener) {
    var id = this.id, ns = this.namespace;
    if (arguments.length < 2) {
      var inherit = d3_transitionInherit, inheritId = d3_transitionInheritId;
      try {
        d3_transitionInheritId = id;
        d3_selection_each(this, function(node, i, j) {
          d3_transitionInherit = node[ns][id];
          type.call(node, node.__data__, i, j);
        });
      } finally {
        d3_transitionInherit = inherit;
        d3_transitionInheritId = inheritId;
      }
    } else {
      d3_selection_each(this, function(node) {
        var transition = node[ns][id];
        (transition.event || (transition.event = d3.dispatch("start", "end", "interrupt"))).on(type, listener);
      });
    }
    return this;
  };
  d3_transitionPrototype.transition = function() {
    var id0 = this.id, id1 = ++d3_transitionId, ns = this.namespace, subgroups = [], subgroup, group, node, transition;
    for (var j = 0, m = this.length; j < m; j++) {
      subgroups.push(subgroup = []);
      for (var group = this[j], i = 0, n = group.length; i < n; i++) {
        if (node = group[i]) {
          transition = node[ns][id0];
          d3_transitionNode(node, i, ns, id1, {
            time: transition.time,
            ease: transition.ease,
            delay: transition.delay + transition.duration,
            duration: transition.duration
          });
        }
        subgroup.push(node);
      }
    }
    return d3_transition(subgroups, ns, id1);
  };
  function d3_transitionNamespace(name) {
    return name == null ? "__transition__" : "__transition_" + name + "__";
  }
  function d3_transitionNode(node, i, ns, id, inherit) {
    var lock = node[ns] || (node[ns] = {
      active: 0,
      count: 0
    }), transition = lock[id], time, timer, duration, ease, tweens;
    function schedule(elapsed) {
      var delay = transition.delay;
      timer.t = delay + time;
      if (delay <= elapsed) return start(elapsed - delay);
      timer.c = start;
    }
    function start(elapsed) {
      var activeId = lock.active, active = lock[activeId];
      if (active) {
        active.timer.c = null;
        active.timer.t = NaN;
        --lock.count;
        delete lock[activeId];
        active.event && active.event.interrupt.call(node, node.__data__, active.index);
      }
      for (var cancelId in lock) {
        if (+cancelId < id) {
          var cancel = lock[cancelId];
          cancel.timer.c = null;
          cancel.timer.t = NaN;
          --lock.count;
          delete lock[cancelId];
        }
      }
      timer.c = tick;
      d3_timer(function() {
        if (timer.c && tick(elapsed || 1)) {
          timer.c = null;
          timer.t = NaN;
        }
        return 1;
      }, 0, time);
      lock.active = id;
      transition.event && transition.event.start.call(node, node.__data__, i);
      tweens = [];
      transition.tween.forEach(function(key, value) {
        if (value = value.call(node, node.__data__, i)) {
          tweens.push(value);
        }
      });
      ease = transition.ease;
      duration = transition.duration;
    }
    function tick(elapsed) {
      var t = elapsed / duration, e = ease(t), n = tweens.length;
      while (n > 0) {
        tweens[--n].call(node, e);
      }
      if (t >= 1) {
        transition.event && transition.event.end.call(node, node.__data__, i);
        if (--lock.count) delete lock[id]; else delete node[ns];
        return 1;
      }
    }
    if (!transition) {
      time = inherit.time;
      timer = d3_timer(schedule, 0, time);
      transition = lock[id] = {
        tween: new d3_Map(),
        time: time,
        timer: timer,
        delay: inherit.delay,
        duration: inherit.duration,
        ease: inherit.ease,
        index: i
      };
      inherit = null;
      ++lock.count;
    }
  }
  d3.svg.axis = function() {
    var scale = d3.scale.linear(), orient = d3_svg_axisDefaultOrient, innerTickSize = 6, outerTickSize = 6, tickPadding = 3, tickArguments_ = [ 10 ], tickValues = null, tickFormat_;
    function axis(g) {
      g.each(function() {
        var g = d3.select(this);
        var scale0 = this.__chart__ || scale, scale1 = this.__chart__ = scale.copy();
        var ticks = tickValues == null ? scale1.ticks ? scale1.ticks.apply(scale1, tickArguments_) : scale1.domain() : tickValues, tickFormat = tickFormat_ == null ? scale1.tickFormat ? scale1.tickFormat.apply(scale1, tickArguments_) : d3_identity : tickFormat_, tick = g.selectAll(".tick").data(ticks, scale1), tickEnter = tick.enter().insert("g", ".domain").attr("class", "tick").style("opacity", ε), tickExit = d3.transition(tick.exit()).style("opacity", ε).remove(), tickUpdate = d3.transition(tick.order()).style("opacity", 1), tickSpacing = Math.max(innerTickSize, 0) + tickPadding, tickTransform;
        var range = d3_scaleRange(scale1), path = g.selectAll(".domain").data([ 0 ]), pathUpdate = (path.enter().append("path").attr("class", "domain"), 
        d3.transition(path));
        tickEnter.append("line");
        tickEnter.append("text");
        var lineEnter = tickEnter.select("line"), lineUpdate = tickUpdate.select("line"), text = tick.select("text").text(tickFormat), textEnter = tickEnter.select("text"), textUpdate = tickUpdate.select("text"), sign = orient === "top" || orient === "left" ? -1 : 1, x1, x2, y1, y2;
        if (orient === "bottom" || orient === "top") {
          tickTransform = d3_svg_axisX, x1 = "x", y1 = "y", x2 = "x2", y2 = "y2";
          text.attr("dy", sign < 0 ? "0em" : ".71em").style("text-anchor", "middle");
          pathUpdate.attr("d", "M" + range[0] + "," + sign * outerTickSize + "V0H" + range[1] + "V" + sign * outerTickSize);
        } else {
          tickTransform = d3_svg_axisY, x1 = "y", y1 = "x", x2 = "y2", y2 = "x2";
          text.attr("dy", ".32em").style("text-anchor", sign < 0 ? "end" : "start");
          pathUpdate.attr("d", "M" + sign * outerTickSize + "," + range[0] + "H0V" + range[1] + "H" + sign * outerTickSize);
        }
        lineEnter.attr(y2, sign * innerTickSize);
        textEnter.attr(y1, sign * tickSpacing);
        lineUpdate.attr(x2, 0).attr(y2, sign * innerTickSize);
        textUpdate.attr(x1, 0).attr(y1, sign * tickSpacing);
        if (scale1.rangeBand) {
          var x = scale1, dx = x.rangeBand() / 2;
          scale0 = scale1 = function(d) {
            return x(d) + dx;
          };
        } else if (scale0.rangeBand) {
          scale0 = scale1;
        } else {
          tickExit.call(tickTransform, scale1, scale0);
        }
        tickEnter.call(tickTransform, scale0, scale1);
        tickUpdate.call(tickTransform, scale1, scale1);
      });
    }
    axis.scale = function(x) {
      if (!arguments.length) return scale;
      scale = x;
      return axis;
    };
    axis.orient = function(x) {
      if (!arguments.length) return orient;
      orient = x in d3_svg_axisOrients ? x + "" : d3_svg_axisDefaultOrient;
      return axis;
    };
    axis.ticks = function() {
      if (!arguments.length) return tickArguments_;
      tickArguments_ = d3_array(arguments);
      return axis;
    };
    axis.tickValues = function(x) {
      if (!arguments.length) return tickValues;
      tickValues = x;
      return axis;
    };
    axis.tickFormat = function(x) {
      if (!arguments.length) return tickFormat_;
      tickFormat_ = x;
      return axis;
    };
    axis.tickSize = function(x) {
      var n = arguments.length;
      if (!n) return innerTickSize;
      innerTickSize = +x;
      outerTickSize = +arguments[n - 1];
      return axis;
    };
    axis.innerTickSize = function(x) {
      if (!arguments.length) return innerTickSize;
      innerTickSize = +x;
      return axis;
    };
    axis.outerTickSize = function(x) {
      if (!arguments.length) return outerTickSize;
      outerTickSize = +x;
      return axis;
    };
    axis.tickPadding = function(x) {
      if (!arguments.length) return tickPadding;
      tickPadding = +x;
      return axis;
    };
    axis.tickSubdivide = function() {
      return arguments.length && axis;
    };
    return axis;
  };
  var d3_svg_axisDefaultOrient = "bottom", d3_svg_axisOrients = {
    top: 1,
    right: 1,
    bottom: 1,
    left: 1
  };
  function d3_svg_axisX(selection, x0, x1) {
    selection.attr("transform", function(d) {
      var v0 = x0(d);
      return "translate(" + (isFinite(v0) ? v0 : x1(d)) + ",0)";
    });
  }
  function d3_svg_axisY(selection, y0, y1) {
    selection.attr("transform", function(d) {
      var v0 = y0(d);
      return "translate(0," + (isFinite(v0) ? v0 : y1(d)) + ")";
    });
  }
  d3.svg.brush = function() {
    var event = d3_eventDispatch(brush, "brushstart", "brush", "brushend"), x = null, y = null, xExtent = [ 0, 0 ], yExtent = [ 0, 0 ], xExtentDomain, yExtentDomain, xClamp = true, yClamp = true, resizes = d3_svg_brushResizes[0];
    function brush(g) {
      g.each(function() {
        var g = d3.select(this).style("pointer-events", "all").style("-webkit-tap-highlight-color", "rgba(0,0,0,0)").on("mousedown.brush", brushstart).on("touchstart.brush", brushstart);
        var background = g.selectAll(".background").data([ 0 ]);
        background.enter().append("rect").attr("class", "background").style("visibility", "hidden").style("cursor", "crosshair");
        g.selectAll(".extent").data([ 0 ]).enter().append("rect").attr("class", "extent").style("cursor", "move");
        var resize = g.selectAll(".resize").data(resizes, d3_identity);
        resize.exit().remove();
        resize.enter().append("g").attr("class", function(d) {
          return "resize " + d;
        }).style("cursor", function(d) {
          return d3_svg_brushCursor[d];
        }).append("rect").attr("x", function(d) {
          return /[ew]$/.test(d) ? -3 : null;
        }).attr("y", function(d) {
          return /^[ns]/.test(d) ? -3 : null;
        }).attr("width", 6).attr("height", 6).style("visibility", "hidden");
        resize.style("display", brush.empty() ? "none" : null);
        var gUpdate = d3.transition(g), backgroundUpdate = d3.transition(background), range;
        if (x) {
          range = d3_scaleRange(x);
          backgroundUpdate.attr("x", range[0]).attr("width", range[1] - range[0]);
          redrawX(gUpdate);
        }
        if (y) {
          range = d3_scaleRange(y);
          backgroundUpdate.attr("y", range[0]).attr("height", range[1] - range[0]);
          redrawY(gUpdate);
        }
        redraw(gUpdate);
      });
    }
    brush.event = function(g) {
      g.each(function() {
        var event_ = event.of(this, arguments), extent1 = {
          x: xExtent,
          y: yExtent,
          i: xExtentDomain,
          j: yExtentDomain
        }, extent0 = this.__chart__ || extent1;
        this.__chart__ = extent1;
        if (d3_transitionInheritId) {
          d3.select(this).transition().each("start.brush", function() {
            xExtentDomain = extent0.i;
            yExtentDomain = extent0.j;
            xExtent = extent0.x;
            yExtent = extent0.y;
            event_({
              type: "brushstart"
            });
          }).tween("brush:brush", function() {
            var xi = d3_interpolateArray(xExtent, extent1.x), yi = d3_interpolateArray(yExtent, extent1.y);
            xExtentDomain = yExtentDomain = null;
            return function(t) {
              xExtent = extent1.x = xi(t);
              yExtent = extent1.y = yi(t);
              event_({
                type: "brush",
                mode: "resize"
              });
            };
          }).each("end.brush", function() {
            xExtentDomain = extent1.i;
            yExtentDomain = extent1.j;
            event_({
              type: "brush",
              mode: "resize"
            });
            event_({
              type: "brushend"
            });
          });
        } else {
          event_({
            type: "brushstart"
          });
          event_({
            type: "brush",
            mode: "resize"
          });
          event_({
            type: "brushend"
          });
        }
      });
    };
    function redraw(g) {
      g.selectAll(".resize").attr("transform", function(d) {
        return "translate(" + xExtent[+/e$/.test(d)] + "," + yExtent[+/^s/.test(d)] + ")";
      });
    }
    function redrawX(g) {
      g.select(".extent").attr("x", xExtent[0]);
      g.selectAll(".extent,.n>rect,.s>rect").attr("width", xExtent[1] - xExtent[0]);
    }
    function redrawY(g) {
      g.select(".extent").attr("y", yExtent[0]);
      g.selectAll(".extent,.e>rect,.w>rect").attr("height", yExtent[1] - yExtent[0]);
    }
    function brushstart() {
      var target = this, eventTarget = d3.select(d3.event.target), event_ = event.of(target, arguments), g = d3.select(target), resizing = eventTarget.datum(), resizingX = !/^(n|s)$/.test(resizing) && x, resizingY = !/^(e|w)$/.test(resizing) && y, dragging = eventTarget.classed("extent"), dragRestore = d3_event_dragSuppress(target), center, origin = d3.mouse(target), offset;
      var w = d3.select(d3_window(target)).on("keydown.brush", keydown).on("keyup.brush", keyup);
      if (d3.event.changedTouches) {
        w.on("touchmove.brush", brushmove).on("touchend.brush", brushend);
      } else {
        w.on("mousemove.brush", brushmove).on("mouseup.brush", brushend);
      }
      g.interrupt().selectAll("*").interrupt();
      if (dragging) {
        origin[0] = xExtent[0] - origin[0];
        origin[1] = yExtent[0] - origin[1];
      } else if (resizing) {
        var ex = +/w$/.test(resizing), ey = +/^n/.test(resizing);
        offset = [ xExtent[1 - ex] - origin[0], yExtent[1 - ey] - origin[1] ];
        origin[0] = xExtent[ex];
        origin[1] = yExtent[ey];
      } else if (d3.event.altKey) center = origin.slice();
      g.style("pointer-events", "none").selectAll(".resize").style("display", null);
      d3.select("body").style("cursor", eventTarget.style("cursor"));
      event_({
        type: "brushstart"
      });
      brushmove();
      function keydown() {
        if (d3.event.keyCode == 32) {
          if (!dragging) {
            center = null;
            origin[0] -= xExtent[1];
            origin[1] -= yExtent[1];
            dragging = 2;
          }
          d3_eventPreventDefault();
        }
      }
      function keyup() {
        if (d3.event.keyCode == 32 && dragging == 2) {
          origin[0] += xExtent[1];
          origin[1] += yExtent[1];
          dragging = 0;
          d3_eventPreventDefault();
        }
      }
      function brushmove() {
        var point = d3.mouse(target), moved = false;
        if (offset) {
          point[0] += offset[0];
          point[1] += offset[1];
        }
        if (!dragging) {
          if (d3.event.altKey) {
            if (!center) center = [ (xExtent[0] + xExtent[1]) / 2, (yExtent[0] + yExtent[1]) / 2 ];
            origin[0] = xExtent[+(point[0] < center[0])];
            origin[1] = yExtent[+(point[1] < center[1])];
          } else center = null;
        }
        if (resizingX && move1(point, x, 0)) {
          redrawX(g);
          moved = true;
        }
        if (resizingY && move1(point, y, 1)) {
          redrawY(g);
          moved = true;
        }
        if (moved) {
          redraw(g);
          event_({
            type: "brush",
            mode: dragging ? "move" : "resize"
          });
        }
      }
      function move1(point, scale, i) {
        var range = d3_scaleRange(scale), r0 = range[0], r1 = range[1], position = origin[i], extent = i ? yExtent : xExtent, size = extent[1] - extent[0], min, max;
        if (dragging) {
          r0 -= position;
          r1 -= size + position;
        }
        min = (i ? yClamp : xClamp) ? Math.max(r0, Math.min(r1, point[i])) : point[i];
        if (dragging) {
          max = (min += position) + size;
        } else {
          if (center) position = Math.max(r0, Math.min(r1, 2 * center[i] - min));
          if (position < min) {
            max = min;
            min = position;
          } else {
            max = position;
          }
        }
        if (extent[0] != min || extent[1] != max) {
          if (i) yExtentDomain = null; else xExtentDomain = null;
          extent[0] = min;
          extent[1] = max;
          return true;
        }
      }
      function brushend() {
        brushmove();
        g.style("pointer-events", "all").selectAll(".resize").style("display", brush.empty() ? "none" : null);
        d3.select("body").style("cursor", null);
        w.on("mousemove.brush", null).on("mouseup.brush", null).on("touchmove.brush", null).on("touchend.brush", null).on("keydown.brush", null).on("keyup.brush", null);
        dragRestore();
        event_({
          type: "brushend"
        });
      }
    }
    brush.x = function(z) {
      if (!arguments.length) return x;
      x = z;
      resizes = d3_svg_brushResizes[!x << 1 | !y];
      return brush;
    };
    brush.y = function(z) {
      if (!arguments.length) return y;
      y = z;
      resizes = d3_svg_brushResizes[!x << 1 | !y];
      return brush;
    };
    brush.clamp = function(z) {
      if (!arguments.length) return x && y ? [ xClamp, yClamp ] : x ? xClamp : y ? yClamp : null;
      if (x && y) xClamp = !!z[0], yClamp = !!z[1]; else if (x) xClamp = !!z; else if (y) yClamp = !!z;
      return brush;
    };
    brush.extent = function(z) {
      var x0, x1, y0, y1, t;
      if (!arguments.length) {
        if (x) {
          if (xExtentDomain) {
            x0 = xExtentDomain[0], x1 = xExtentDomain[1];
          } else {
            x0 = xExtent[0], x1 = xExtent[1];
            if (x.invert) x0 = x.invert(x0), x1 = x.invert(x1);
            if (x1 < x0) t = x0, x0 = x1, x1 = t;
          }
        }
        if (y) {
          if (yExtentDomain) {
            y0 = yExtentDomain[0], y1 = yExtentDomain[1];
          } else {
            y0 = yExtent[0], y1 = yExtent[1];
            if (y.invert) y0 = y.invert(y0), y1 = y.invert(y1);
            if (y1 < y0) t = y0, y0 = y1, y1 = t;
          }
        }
        return x && y ? [ [ x0, y0 ], [ x1, y1 ] ] : x ? [ x0, x1 ] : y && [ y0, y1 ];
      }
      if (x) {
        x0 = z[0], x1 = z[1];
        if (y) x0 = x0[0], x1 = x1[0];
        xExtentDomain = [ x0, x1 ];
        if (x.invert) x0 = x(x0), x1 = x(x1);
        if (x1 < x0) t = x0, x0 = x1, x1 = t;
        if (x0 != xExtent[0] || x1 != xExtent[1]) xExtent = [ x0, x1 ];
      }
      if (y) {
        y0 = z[0], y1 = z[1];
        if (x) y0 = y0[1], y1 = y1[1];
        yExtentDomain = [ y0, y1 ];
        if (y.invert) y0 = y(y0), y1 = y(y1);
        if (y1 < y0) t = y0, y0 = y1, y1 = t;
        if (y0 != yExtent[0] || y1 != yExtent[1]) yExtent = [ y0, y1 ];
      }
      return brush;
    };
    brush.clear = function() {
      if (!brush.empty()) {
        xExtent = [ 0, 0 ], yExtent = [ 0, 0 ];
        xExtentDomain = yExtentDomain = null;
      }
      return brush;
    };
    brush.empty = function() {
      return !!x && xExtent[0] == xExtent[1] || !!y && yExtent[0] == yExtent[1];
    };
    return d3.rebind(brush, event, "on");
  };
  var d3_svg_brushCursor = {
    n: "ns-resize",
    e: "ew-resize",
    s: "ns-resize",
    w: "ew-resize",
    nw: "nwse-resize",
    ne: "nesw-resize",
    se: "nwse-resize",
    sw: "nesw-resize"
  };
  var d3_svg_brushResizes = [ [ "n", "e", "s", "w", "nw", "ne", "se", "sw" ], [ "e", "w" ], [ "n", "s" ], [] ];
  var d3_time_format = d3_time.format = d3_locale_enUS.timeFormat;
  var d3_time_formatUtc = d3_time_format.utc;
  var d3_time_formatIso = d3_time_formatUtc("%Y-%m-%dT%H:%M:%S.%LZ");
  d3_time_format.iso = Date.prototype.toISOString && +new Date("2000-01-01T00:00:00.000Z") ? d3_time_formatIsoNative : d3_time_formatIso;
  function d3_time_formatIsoNative(date) {
    return date.toISOString();
  }
  d3_time_formatIsoNative.parse = function(string) {
    var date = new Date(string);
    return isNaN(date) ? null : date;
  };
  d3_time_formatIsoNative.toString = d3_time_formatIso.toString;
  d3_time.second = d3_time_interval(function(date) {
    return new d3_date(Math.floor(date / 1e3) * 1e3);
  }, function(date, offset) {
    date.setTime(date.getTime() + Math.floor(offset) * 1e3);
  }, function(date) {
    return date.getSeconds();
  });
  d3_time.seconds = d3_time.second.range;
  d3_time.seconds.utc = d3_time.second.utc.range;
  d3_time.minute = d3_time_interval(function(date) {
    return new d3_date(Math.floor(date / 6e4) * 6e4);
  }, function(date, offset) {
    date.setTime(date.getTime() + Math.floor(offset) * 6e4);
  }, function(date) {
    return date.getMinutes();
  });
  d3_time.minutes = d3_time.minute.range;
  d3_time.minutes.utc = d3_time.minute.utc.range;
  d3_time.hour = d3_time_interval(function(date) {
    var timezone = date.getTimezoneOffset() / 60;
    return new d3_date((Math.floor(date / 36e5 - timezone) + timezone) * 36e5);
  }, function(date, offset) {
    date.setTime(date.getTime() + Math.floor(offset) * 36e5);
  }, function(date) {
    return date.getHours();
  });
  d3_time.hours = d3_time.hour.range;
  d3_time.hours.utc = d3_time.hour.utc.range;
  d3_time.month = d3_time_interval(function(date) {
    date = d3_time.day(date);
    date.setDate(1);
    return date;
  }, function(date, offset) {
    date.setMonth(date.getMonth() + offset);
  }, function(date) {
    return date.getMonth();
  });
  d3_time.months = d3_time.month.range;
  d3_time.months.utc = d3_time.month.utc.range;
  function d3_time_scale(linear, methods, format) {
    function scale(x) {
      return linear(x);
    }
    scale.invert = function(x) {
      return d3_time_scaleDate(linear.invert(x));
    };
    scale.domain = function(x) {
      if (!arguments.length) return linear.domain().map(d3_time_scaleDate);
      linear.domain(x);
      return scale;
    };
    function tickMethod(extent, count) {
      var span = extent[1] - extent[0], target = span / count, i = d3.bisect(d3_time_scaleSteps, target);
      return i == d3_time_scaleSteps.length ? [ methods.year, d3_scale_linearTickRange(extent.map(function(d) {
        return d / 31536e6;
      }), count)[2] ] : !i ? [ d3_time_scaleMilliseconds, d3_scale_linearTickRange(extent, count)[2] ] : methods[target / d3_time_scaleSteps[i - 1] < d3_time_scaleSteps[i] / target ? i - 1 : i];
    }
    scale.nice = function(interval, skip) {
      var domain = scale.domain(), extent = d3_scaleExtent(domain), method = interval == null ? tickMethod(extent, 10) : typeof interval === "number" && tickMethod(extent, interval);
      if (method) interval = method[0], skip = method[1];
      function skipped(date) {
        return !isNaN(date) && !interval.range(date, d3_time_scaleDate(+date + 1), skip).length;
      }
      return scale.domain(d3_scale_nice(domain, skip > 1 ? {
        floor: function(date) {
          while (skipped(date = interval.floor(date))) date = d3_time_scaleDate(date - 1);
          return date;
        },
        ceil: function(date) {
          while (skipped(date = interval.ceil(date))) date = d3_time_scaleDate(+date + 1);
          return date;
        }
      } : interval));
    };
    scale.ticks = function(interval, skip) {
      var extent = d3_scaleExtent(scale.domain()), method = interval == null ? tickMethod(extent, 10) : typeof interval === "number" ? tickMethod(extent, interval) : !interval.range && [ {
        range: interval
      }, skip ];
      if (method) interval = method[0], skip = method[1];
      return interval.range(extent[0], d3_time_scaleDate(+extent[1] + 1), skip < 1 ? 1 : skip);
    };
    scale.tickFormat = function() {
      return format;
    };
    scale.copy = function() {
      return d3_time_scale(linear.copy(), methods, format);
    };
    return d3_scale_linearRebind(scale, linear);
  }
  function d3_time_scaleDate(t) {
    return new Date(t);
  }
  var d3_time_scaleSteps = [ 1e3, 5e3, 15e3, 3e4, 6e4, 3e5, 9e5, 18e5, 36e5, 108e5, 216e5, 432e5, 864e5, 1728e5, 6048e5, 2592e6, 7776e6, 31536e6 ];
  var d3_time_scaleLocalMethods = [ [ d3_time.second, 1 ], [ d3_time.second, 5 ], [ d3_time.second, 15 ], [ d3_time.second, 30 ], [ d3_time.minute, 1 ], [ d3_time.minute, 5 ], [ d3_time.minute, 15 ], [ d3_time.minute, 30 ], [ d3_time.hour, 1 ], [ d3_time.hour, 3 ], [ d3_time.hour, 6 ], [ d3_time.hour, 12 ], [ d3_time.day, 1 ], [ d3_time.day, 2 ], [ d3_time.week, 1 ], [ d3_time.month, 1 ], [ d3_time.month, 3 ], [ d3_time.year, 1 ] ];
  var d3_time_scaleLocalFormat = d3_time_format.multi([ [ ".%L", function(d) {
    return d.getMilliseconds();
  } ], [ ":%S", function(d) {
    return d.getSeconds();
  } ], [ "%I:%M", function(d) {
    return d.getMinutes();
  } ], [ "%I %p", function(d) {
    return d.getHours();
  } ], [ "%a %d", function(d) {
    return d.getDay() && d.getDate() != 1;
  } ], [ "%b %d", function(d) {
    return d.getDate() != 1;
  } ], [ "%B", function(d) {
    return d.getMonth();
  } ], [ "%Y", d3_true ] ]);
  var d3_time_scaleMilliseconds = {
    range: function(start, stop, step) {
      return d3.range(Math.ceil(start / step) * step, +stop, step).map(d3_time_scaleDate);
    },
    floor: d3_identity,
    ceil: d3_identity
  };
  d3_time_scaleLocalMethods.year = d3_time.year;
  d3_time.scale = function() {
    return d3_time_scale(d3.scale.linear(), d3_time_scaleLocalMethods, d3_time_scaleLocalFormat);
  };
  var d3_time_scaleUtcMethods = d3_time_scaleLocalMethods.map(function(m) {
    return [ m[0].utc, m[1] ];
  });
  var d3_time_scaleUtcFormat = d3_time_formatUtc.multi([ [ ".%L", function(d) {
    return d.getUTCMilliseconds();
  } ], [ ":%S", function(d) {
    return d.getUTCSeconds();
  } ], [ "%I:%M", function(d) {
    return d.getUTCMinutes();
  } ], [ "%I %p", function(d) {
    return d.getUTCHours();
  } ], [ "%a %d", function(d) {
    return d.getUTCDay() && d.getUTCDate() != 1;
  } ], [ "%b %d", function(d) {
    return d.getUTCDate() != 1;
  } ], [ "%B", function(d) {
    return d.getUTCMonth();
  } ], [ "%Y", d3_true ] ]);
  d3_time_scaleUtcMethods.year = d3_time.year.utc;
  d3_time.scale.utc = function() {
    return d3_time_scale(d3.scale.linear(), d3_time_scaleUtcMethods, d3_time_scaleUtcFormat);
  };
  d3.text = d3_xhrType(function(request) {
    return request.responseText;
  });
  d3.json = function(url, callback) {
    return d3_xhr(url, "application/json", d3_json, callback);
  };
  function d3_json(request) {
    return JSON.parse(request.responseText);
  }
  d3.html = function(url, callback) {
    return d3_xhr(url, "text/html", d3_html, callback);
  };
  function d3_html(request) {
    var range = d3_document.createRange();
    range.selectNode(d3_document.body);
    return range.createContextualFragment(request.responseText);
  }
  d3.xml = d3_xhrType(function(request) {
    return request.responseXML;
  });
  if (typeof define === "function" && define.amd) this.d3 = d3, define(d3); else if (typeof module === "object" && module.exports) module.exports = d3; else this.d3 = d3;
}();

// FILE: src/charts/assets/js/libs/zhCharts/zhCharts.js
function zhChart(options, ScreenService) {
	var defaultOptions = {
			messageNoData: ScreenService.i18n("No data."),
			width: "100%",
			height: "100%",
			fontName: "'Droid Sans', Helvetica, Arial, sans-serif",
			padding: {
				left: 20,
				top: 10,
				right: 20,
				bottom: 10,
				title: 15
			},
			dataSource: {
				data: []
			},
			showTooltip: true,
			showValueLabels: true,
			showDrillUpButton: true,
			xAxis: {
				type: "points",
				bandDimensions: {
					type: "auto",
					gap: 0.1
				},
				ticks: {
					maxWidth: undefined,
					maxLines: 2,
				}
			},
			legend: {
				position: "top right",
				margin: 10,
				visible: true,
				closed: false
			},
			invertedAxes: false,
			radialAxis: false,
			scrollable: true,
			events: {}
		};

	var chart, chartElement;
	var chartContainer, svgContainer;
	var svgContainerWidth, svgContainerHeight;
	var titlePadding = 5;
	var _duration = 1000;
	var drillPath = [];
	var axisDimensions = {
		left: 0,
		top: 0,
		right: 0,
		bottom: 0
	};
	var chartPanning = false;

	/* UTIL */
	function isUndefined(value) {
		return typeof(value) === "undefined";
	}

	function isUndefinedOrNull(value) {
		return isUndefined(value) || isNull(value);
	}

	function isNull(value) {
		return value === null;
	}

	function isFunction(value) {
		return Object.prototype.toString.call(value) === '[object Function]';
	}

	function isMobile() {
		return ( /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) );
	}

	function formatValue(value, format) {
		/* jslint evil: true */
		return (new Function("value", "return "+format))(value);
	}

	function colorExpression(value, defaultColor, expression) {
		/* jslint evil: true */
		return (new Function("value, defaultColor", "return "+expression))(value, defaultColor);
	}

	function buildDebounceMethod(method, milisecondsToDelay, firstCall) {
		var timeout;
		firstCall = typeof firstCall != "boolean" ? false : firstCall;
		return function() {
			var obj = this,
				args = arguments;

			function delayed() {
				if (!firstCall) {
					method.apply(obj, args);
				}
				timeout = null;
			}
			if (timeout) {
				clearTimeout(timeout);
			} else if (firstCall) {
				method.apply(obj, args);
			}
			timeout = setTimeout(delayed, milisecondsToDelay || 100);
		};
	}

	function initChart(element) {
		chart = jQuery.extend(true, {}, defaultOptions, options || {});
		chart.id = Math.round(Math.random() * 1000000);
		chartElement = element || chartElement;

		chartContainer = d3.select(chartElement || "body");
		chartContainer.classed("zh-chart-container", true);
		chartContainer.node()._chart = _chart;
		d3.select(chartContainer.node().parentNode).style("overflow", "hidden");

		svgContainer = chartContainer.select("svg");
		if (!svgContainer.empty()) {
			svgContainer.remove();
		}

		svgContainer = chartContainer.append("svg")
			.attr("id", chart.id)
			.style("font-family", chart.fontName);

		chart.clippingPath = svgContainer.append("defs")
			.append("clipPath")
			.attr("id", "clip-path-"+chart.id)
			.append("rect")
			.attr("fill", "none");

		chart.clippingPathAxis = svgContainer.select("defs")
			.append("clipPath")
			.attr("id", "clip-path-axis-"+chart.id)
			.append("rect")
			.attr("fill", "none");

		svgContainer = svgContainer.append("g");

		if (chart.scrollable) {
			svgContainer.append("rect")
				.attr("class","panning-controller")
				.attr("fill","transparent");
		}

		initTooltip();
	}

	function resizeChart() {
		var chartContainerWidth, chartContainerHeight;

		chartContainerWidth = chartContainer.node().clientWidth - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-left")) - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-right"));
		chartContainerHeight = chartContainer.node().clientHeight - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-top")) - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-bottom"));
		if (chart.width.indexOf('%') !== -1) {
			svgContainerWidth = chartContainerWidth * parseFloat(chart.width)/100;
		} else {
			svgContainerWidth = chart.width;
		}
		if (chart.height.indexOf('%') !== -1) {
			svgContainerHeight = chartContainerHeight * parseFloat(chart.height)/100;
		} else {
			svgContainerHeight = chart.width;
		}

		chartContainer.select("svg")
			.attr("width", svgContainerWidth)
			.attr("height", svgContainerHeight);

		axisDimensions = {
			left: 0,
			top: 0,
			right: 0,
			bottom: 0
		};
	}

	function drawTitle() {
		if (chart.title) {
			var drillUpButton = svgContainer.select(".drill-up-button");
			if (!drillUpButton.empty() && drillUpButton.style("display") !== "none") {
				drillUpButton = drillUpButton.select("rect").node().getBBox().width + 10;
			} else {
				drillUpButton = 0;
			}

			var titleContainer = svgContainer.select("g.title");
			if (titleContainer.empty()) {
				titleContainer = svgContainer.append("g")
					.attr("class", "title");
			}

			var title = titleContainer.select("text");
			if (title.empty()) {
				title = titleContainer.append("text")
					.attr("text-anchor","start")
					.style("font-size", chart.title.fontSize || "25px")
					.style("fill", chart.title.color || "#000");
			}
			title.text(chart.title.text || chart.title)
				.attr("x", chart.padding.left + drillUpButton);

			var titleHeight = title.node().getBBox().height;
			title.attr("y", chart.padding.top + titleHeight);

			titlePadding = titleHeight + chart.padding.title;
		}
	}

	function drawSubTitle() {
		if (chart.subtitle) {
			var drillUpButton = svgContainer.select(".drill-up-button");
			if (!drillUpButton.empty() && drillUpButton.style("display") !== "none") {
				drillUpButton = drillUpButton.select("rect").node().getBBox().width + 10;
			} else {
				drillUpButton = 0;
			}

			var titleContainer = svgContainer.select("g.sub-title");
			if (titleContainer.empty()) {
				titleContainer = svgContainer.append("g")
					.attr("class", "sub-title");
			}

			var subtitle = titleContainer.select("text");
			if (subtitle.empty()) {
				subtitle = titleContainer.append("text")
					.attr("text-anchor","start")
					.style("font-size", chart.subtitle.fontSize || "15px")
					.style("fill", chart.subtitle.color || "#999");
			}
			subtitle.text(chart.subtitle.text || chart.subtitle)
				.attr("x", chart.padding.left + drillUpButton);

			var titleHeight = subtitle.node().getBBox().height;
			subtitle.attr("y", chart.padding.top + titlePadding - chart.padding.title + titleHeight + 2);

			if (titlePadding === 0) {
				titlePadding += chart.padding.title;
			}
			titlePadding += titleHeight;
		}
	}

	function checkTitleOverflow() {
		var title = svgContainer.select("g.title text");
		var _text;
		if (!title.empty()) {
			_text = chart.title.text || chart.title;

			while (title.node().getBBox().width + parseFloat(title.attr("x")) > chart._chartWidth) {
				_text = _text.slice(0, -1);
				title.text(_text+"...");
			}
		}

		var subtitle = svgContainer.select("g.sub-title text");
		if (!subtitle.empty()){
			_text = chart.subtitle.text || chart.subtitle;

			while (subtitle.node().getBBox().width + parseFloat(subtitle.attr("x")) > chart._chartWidth) {
				_text = _text.slice(0, -1);
				subtitle.text(_text+"...");
			}
		}
	}

	var defaultSerieOptions = {
 			colors: ["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"],
			yAxis: {
				position: "left",
				showGridLines: true,
				range: {}
			},
			showValueLabels: true
		};
	var defaultBarOptions = {
			stacked: false,
			normalized: false,
			horizontal: false,
			floating: false,
			maxBarWidth: false,
			floatingGap: 0.1,
			centeredBars: false,
			showCenterLine: false
		};
	var defaultLineOptions = {
			stacked: false,
			normalized: false,
			horizontal: false,
			lineType: "linear",
			lineExpanded: false,
			lineWidth: 5,
			marks: {
				type: "circle",
				size: 10
			}
		};
	var defaultAreaOptions = jQuery.extend(true, {}, defaultLineOptions, {
			areaOpacity: 0.3,
			lineWidth: 0
		});
	var defaultPieOptions = {
			showValueLabels: "%",
			sort: false,
			strokeWidth: 0,
			strokeColor: "#FFFFFF",
			arcsGap: 0,
			maxSlices: 10,
			maxSlicesText: "Others"
		};
	var defaultDonutOptions = jQuery.extend(true, {}, defaultPieOptions, {
			donutWidth: 0.25
		});
	var defaultGaugeOptions = jQuery.extend(true, {}, defaultDonutOptions, {
			rangeType: "%",
			colors: ["#2ecc71","#f1c40f","#e67e22","#d35400"],
			gaugeWidth: 0.5,
			gaugeShowLegend: false,
			horizontal: false,
			pointerColor: "#000000",
			showGaugeValue: "value",
			ranges: []
		});
	var defaultProgressOptions = {
			showValueLabels: false,
			showProgressValue: "%",
			backgroundColor: "#EEEEEE",
			progressWidth: 0.1,
			horizontal: false
		};
	var defaultGanttOptions = {
			yAxis: false,
			ganttTimeScale: "day", // day, week, month, quarter, year
			dateFormat: "MM/DD/YYYY",
			itemHeight: 30,
			itemBarHeight: 10,
			showTaskCompletionLabel: "below",
			showCompletionLabelOnParentTasks: false,
			realBarType: "percentual",
			showCompletedTasks: true,
			showTaskName: true,
			showPredecessors: true
		};
	var defaultSeriesOptions = {
			bar: defaultBarOptions,
			line: defaultLineOptions,
			area: defaultAreaOptions,
			pie: defaultPieOptions,
			donut: defaultDonutOptions,
			gauge: defaultGaugeOptions,
			progress: defaultProgressOptions,
			gantt: defaultGanttOptions
		};

	function drawBarSerie(serie) {
		var bars = serie._serieContainer._dataContainer.selectAll("g.bars").data(chart.dataSource.data);

		bars.transition().duration(_duration)
			.attr("transform", function(d) {
				var translate;
				if (!chart.invertedAxes) {
					translate = "translate(" + chart._bandLeft(d) + ",0)";
				} else {
					translate = "translate(0," + serie._bandTop(d) + ")";
				}

				return translate;
			});

		bars.enter().append("g")
			.attr("class","bars")
			.attr("transform", function(data) {
				var translate;
				if (!chart.invertedAxes) {
					translate = "translate(" + chart._bandLeft(data) + ",0)";
				} else {
					translate = "translate(0," + serie._bandTop(data) + ")";
				}

				return translate;
			})
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip);

		setupShowTooltipOnLongTap(bars);

		bars.exit().remove();

		var floatingScale;
		var floatingGap;
		if (serie.stacked && serie.floating) {
			if (!chart.invertedAxes) {
				floatingGap = serie.floatingGap * chart._chartHeight;
				floatingScale = d3.scale.linear().range([0, chart._chartHeight - (floatingGap * (serie.fields.length - 1))]);
			} else {
				floatingGap = serie.floatingGap * chart._chartWidth;
				floatingScale = d3.scale.linear().range([0, chart._chartWidth - (floatingGap * (serie.fields.length - 1))]);
			}
			floatingScale.domain([d3.sum(serie.fields, function(field) {
				return serie._fieldsMinValue[field] || 0;
			}), d3.sum(serie.fields, function(field) {
				return serie._fieldsMaxValue[field] || 0;
			})]);
		}

		var barWidth, barHeight;
		if (!chart.invertedAxes) {
			if (serie.stacked) {
				barWidth = chart._bandWidth();
			} else {
				barWidth = chart._bandWidth()/serie.fields.length;
			}
		} else {
			if (serie.stacked) {
				barHeight = serie._bandHeight();
			} else {
				barHeight = serie._bandHeight()/serie.fields.length;
			}
		}

		function getBarColor(d, i) {
			var defaultColor = serie.colors[i % serie.colors.length];
			var value = d[serie.fields[i]];
			var color;
			if (!isUndefinedOrNull(serie.colorExpression)) {
				color = colorExpression(value, defaultColor, serie.colorExpression);
			} else {
				color = defaultColor;
			}

			return color;
		}

		function drawLabel(barGroup, fieldIndex, fieldName) {
			var label = barGroup.select("text.field-"+fieldIndex);

			if (!chart.invertedAxes) {
				var _barLeft = serie.stacked ? 0 : barWidth * fieldIndex;
				if (label.empty()) {
					label = barGroup.append("text")
						.attr("class", "label-value field-"+fieldIndex)
						.attr("field-name", fieldName)
						.style("font-size", "11px")
						.attr("x", _barLeft + barWidth / 2)
						.attr("y", function(d) { return getSerieStartPosition(fieldIndex); })
						.attr("text-anchor","middle");
				}

				label.attr("dy", function(d) {
						var _dy;
						if (d[fieldName] < 0) {
							_dy = "1em";
						} else {
							_dy = "-0.3em";
						}
						return _dy;
					})
					.attr("fill", function(d) { return serie.stacked ? "#999" : getBarColor(d, fieldIndex); });

				label.transition().duration(_duration)
					.attr("x", _barLeft + barWidth / 2)
					.attr("y", function(d) {
						var _y;
						if (d[fieldName] < 0) {
							_y = serie._yScale(d[fieldName]);
						} else {
							_y = getSerieBarTop(d, fieldIndex);
						}
						return _y;
					})
					.each("end", function(d, i) {
						var _label = d3.select(this);
						var labelBox = _label.node().getBBox();
						if (labelBox.y + labelBox.height > chart._chartHeight) {
							_label.attr("dy", "-0.3em");
						}
					});
			} else {
				var _barTop = serie.stacked ? 0 : barHeight * fieldIndex;
				if (label.empty()) {
					label = barGroup.append("text")
						.attr("class", "label-value field-"+fieldIndex)
						.attr("field-name", fieldName)
						.style("font-size", "11px")
						.attr("x", function(d) { return getSerieStartPosition(fieldIndex); })
						.attr("y", _barTop + barHeight / 2)
						.attr("dy","0.35em");
				}

				label.attr("dx", function(d) {
						var _dx;
						if (d[fieldName] < 0) {
							_dx = "-0.3em";
						} else {
							_dx = "0.3em";
						}
						return _dx;
					})
					.attr("text-anchor", function(d) {
						var _ta;
						if (d[fieldName] < 0) {
							_ta = "end";
						} else {
							_ta = "start";
						}
						return _ta;
					})
					.attr("fill", function(d) { return serie.stacked ? "#999" : getBarColor(d, fieldIndex); });

				label.transition().duration(_duration)
					.attr("x", function(d) {
						var _x;
						if (serie.stacked && serie.floating) {
							_x = getSerieBarLeft(d, fieldIndex) + getSerieBarWidth(d, fieldIndex);
						} else if (serie.centeredBars) {
							_x = (chart._chartWidth/2) + (chart._xScale(d[fieldName]) - chart._startPosition)/2;
						} else {
							_x = chart._xScale(d[fieldName]);
						}
						return _x;
					})
					.attr("y", _barTop + barHeight / 2)
					.each("end", function(d) {
						var _label = d3.select(this);
						var labelBox = _label.node().getBBox();

						if (serie.stacked && serie.floating && fieldIndex < serie.fields.length - 1) {
							if (labelBox.x + labelBox.width > getSerieStartPosition(fieldIndex+1)) {
								_label.attr("dx", "-0.71em")
									.attr("text-anchor", "end");
							}
						} else {
							if (labelBox.x + labelBox.width > chart._chartWidth) {
								_label.attr("dx", "-0.71em")
									.attr("text-anchor", "end");
							}
						}
						if (labelBox.x < 0) {
							_label.attr("dx", "0.71em")
								.attr("text-anchor", "start");
						}
					});

			}

			label.text(function(d) {
				var value = d[fieldName];
				if (serie.yAxis.format) {
					if (isFunction(serie.yAxis.format)) {
						value = serie.yAxis.format(value);
					} else {
						value = formatValue(value, serie.yAxis.format);
					}
				}
				return value;
			});
		}

		function getSerieBarTop(d, i) {
			var barY, j;

			if (d[serie.fields[i]] < 0) {
				barY = chart._startPosition;
			} else if (serie.stacked && serie.floating) {
				barY = chart._startPosition - floatingScale(d[serie.fields[i]]);
			} else if (serie.centeredBars) {
			 	barY = serie._yScale(d[serie.fields[i]]) - chart._chartHeight + (chart._startPosition - (serie._yScale(d[serie.fields[i]])) / 2);
			} else {
				barY = serie._yScale(d[serie.fields[i]]);
			}

			if (!serie.centeredBars) {
				if (serie.stacked && i > 0) {
					for (j = 0; j < i; j++) {
						if (d[serie.fields[i]] >= 0 && d[serie.fields[j]] >= 0) {
							if (!serie.floating) {
								barY = barY - (chart._startPosition - serie._yScale(d[serie.fields[j]]));
							} else {
								barY = barY - floatingScale(serie._fieldsMaxValue[serie.fields[j]]) - floatingGap;
							}
						} else if (d[serie.fields[i]] < 0 && d[serie.fields[j]] < 0) {
							if (!serie.floating) {
								barY = barY + (serie._yScale(d[serie.fields[j]]) - chart._startPosition);
							} else {
								barY = barY + floatingScale(serie._fieldsMaxValue[serie.fields[j]]) + floatingGap;
							}
						}
					}
				}
			} else {
				if (serie.stacked) {
					barY = barY + (serie._yScale(d[serie.fields[i]]) - serie._yScale(d._stackedMaxValue)) / 2;
					for (j = 0; j < i; j++) {
						if (!isUndefined(d[serie.fields[j]])) {
							barY = barY - (chart._startPosition - serie._yScale(d[serie.fields[j]]));
						}
					}
				}
			}
			return barY;
		}

		function getSerieBarLeft(d, i) {
			var barX, j;

			if (d[serie.fields[i]] < 0) {
				barX = chart._xScale(d[serie.fields[i]]);
			} else if (serie.centeredBars) {
			 	barX = chart._startPosition + chart._chartWidth/2 - (chart._xScale(d[serie.fields[i]]) - chart._startPosition) / 2;
			} else {
				barX = chart._startPosition;
			}

			if (!serie.centeredBars) {
				if (serie.stacked && i > 0) {
					for (j = 0; j < i; j++) {
						if (!serie.floating) {
							barX = barX + chart._xScale(d[serie.fields[j]]);
						} else {
							barX = barX + floatingScale(serie._fieldsMaxValue[serie.fields[j]]) + floatingGap;
						}
					}
				}
			} else {
				if (serie.stacked) {
					barX = barX + (chart._xScale(d[serie.fields[i]]) - chart._xScale(d._stackedMaxValue))/2;
					for (j = 0; j < i; j++) {
						if (!isUndefined(d[serie.fields[j]])) {
							barX = barX + (chart._xScale(d[serie.fields[j]]) - chart._startPosition);
						}
					}
				}
			}
			return barX;
		}

		function getSerieStartPosition(i) {
			var pos = chart._startPosition;
			if (serie.centeredBars) {
				if (!chart.invertedAxes) {
					pos -= chart._chartHeight / 2;
				} else {
					pos += chart._chartWidth / 2;
				}
			} else {
				if (serie.stacked && serie.floating && i > 0) {
					for (var j = 0; j < i; j++) {
						if (!chart.invertedAxes) {
							pos = pos - floatingScale(serie._fieldsMaxValue[serie.fields[j]]) - floatingGap;
						} else {
							pos = pos + floatingScale(serie._fieldsMaxValue[serie.fields[j]]) + floatingGap;
						}
					}
				}
			}

			return pos;
		}

		function getSerieBarHeight(d, i) {
			var height;
			if (serie.stacked && serie.floating) {
				height = floatingScale(d[serie.fields[i]]);
			} else {
				height = Math.abs(chart._startPosition - serie._yScale(d[serie.fields[i]]));
			}

			return height;
		}

		function getSerieBarWidth(d, i) {
			var width;
			if (serie.stacked && serie.floating) {
				width = floatingScale(d[serie.fields[i]]);
			} else {
				width = Math.abs(chart._xScale(d[serie.fields[i]]) - chart._startPosition);
			}
			return width;
		}

		function drawCenterLine() {
			var centerLine = serie._serieContainer._dataContainer.select(".center-line");

			if (centerLine.empty()) {
				centerLine = serie._serieContainer._dataContainer.append("line")
					.attr("class", "center-line")
					.attr("stroke","#ccc")
					.attr("stroke-width", 1)
					.attr("shape-rendering", "crispEdges")
					.attr("stroke-dasharray", "5 5");
			}

			if (!chart.invertedAxes) {
				centerLine.transition().duration(_duration)
					.attr("x1", 0).attr("x2", chart._chartWidth)
					.attr("y1", chart._startPosition - chart._chartHeight/2).attr("y2", chart._startPosition - chart._chartHeight/2);
			} else {
				centerLine.transition().duration(_duration)
					.attr("x1", chart._startPosition + chart._chartWidth/2).attr("x2", chart._startPosition + chart._chartWidth/2)
					.attr("y1", 0).attr("y2", chart._chartHeight);
			}
		}

		serie.fields.forEach(function(field, i) {
			if (!chart.invertedAxes) {
				var _barLeft = serie.stacked ? 0 : barWidth * i;

				if (serie.stacked && serie.floating) {
					serie._yAxisContainer.select(".tick-"+i)
						.transition().duration(_duration)
						.attr("transform", "translate(0, " + getSerieStartPosition(i) + ")");
				}

				bars.each(function(d) {
					if (!isUndefinedOrNull(d[field])) {
						var barGroup = d3.select(this);
						var bar = barGroup.select("rect.field-"+i);

						var barOffset = 0;
						var _barWidth = barWidth;
						if (serie.maxBarWidth && barWidth > serie.maxBarWidth) {
							barOffset = (barWidth - serie.maxBarWidth) / 2;
							_barWidth = serie.maxBarWidth;
						}

						if (bar.empty()) {
							bar = barGroup.append("rect")
								.attr("class", "field-"+i)
								.attr("field-name", field)
								.attr("x", _barLeft + barOffset)
								.attr("y", function(d) { return getSerieStartPosition(i); })
								.attr("height", 0)
								.attr("width", _barWidth)
								.attr("fill", function(d) { return getBarColor(d, i); })
								.on("click", dataClick);
						}

						bar.transition().duration(_duration)
							.attr("x", _barLeft + barOffset)
							.attr("y", function(d) { return getSerieBarTop(d, i); })
							.attr("width", _barWidth)
							.attr("height", function(d) { return getSerieBarHeight(d, i); });

						if (serie.showValueLabels && (!serie.stacked || serie.floating)) {
							drawLabel(barGroup, i, serie.fields[i]);
						}
					}
				});
			} else {
				var _barTop = serie.stacked ? 0 : barHeight * i;

				if (serie.stacked && serie.floating) {
					chart._xAxisContainer.select(".tick-"+i)
						.transition().duration(_duration)
						.attr("transform", "translate(" + getSerieStartPosition(i) + ", 0)");
				}

				bars.each(function(d) {
					if (!isUndefinedOrNull(d[field])) {
						var barGroup = d3.select(this);
						var bar = barGroup.select("rect.field-"+i);

						var barOffset = 0;
						var _barHeight = barHeight;
						if (serie.maxBarWidth && barHeight > serie.maxBarWidth) {
							barOffset = (barHeight - serie.maxBarWidth) / 2;
							_barHeight = serie.maxBarWidth;
						}

						if (bar.empty()) {
							bar = barGroup.append("rect")
								.attr("class", "field-"+i)
								.attr("field-name", field)
								.attr("x", function(d) { return getSerieStartPosition(i); })
								.attr("y", _barTop + barOffset)
								.attr("height", _barHeight)
								.attr("width", 0)
								.attr("fill", function(d) { return getBarColor(d, i); })
								.on("click", dataClick);
						}

						bar.transition().duration(_duration)
							.attr("y", _barTop + barOffset)
							.attr("x", function(d) { return getSerieBarLeft(d, i); })
							.attr("height", _barHeight)
							.attr("width", function(d) { return getSerieBarWidth(d, i); });

						if (serie.showValueLabels && (!serie.stacked || serie.floating)) {
							drawLabel(barGroup, i, serie.fields[i]);
						}
					}
				});
			}
		});

		if (serie.stacked && !serie.floating && serie.showValueLabels) {
			bars.each(function(d) {
				var barGroup = d3.select(this);
				var fieldIdx;
				if (d._stackedMaxValue > 0) {
					serie.fields.some(function(field, idx) {
						if (d["_stackedValue_"+field] == d._stackedMaxValue) {
							fieldIdx = idx;
							return true;
						}
					});
					drawLabel(barGroup, fieldIdx, "_stackedMaxValue");
				}
				if (d._stackedMinValue < 0) {
					serie.fields.some(function(field, idx) {
						if (d["_stackedValue_"+field] == d._stackedMinValue) {
							fieldIdx = idx;
							return true;
						}
					});
					drawLabel(barGroup, fieldIdx, "_stackedMinValue");
				}
			});
		}

		if (serie.centeredBars && serie.showCenterLine) {
			drawCenterLine();
		}
	}

	function drawArea(serie) {
		serie.fields.forEach(function(field, i) {
			var initialArea = d3.svg.area()
				.interpolate(serie.lineType)
				.defined(function(d) { return !isUndefinedOrNull(d[field]); })
				.x0(function(d) {
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						x = chart._startPosition;
					}
					return x;
				})
				.x1(function(d) {
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						x = chart._startPosition;
					}
					return x;
				})
				.y0(function(d) {
					var y;
					if (!chart.invertedAxes) {
						y = chart._startPosition;
					} else {
						y = serie._pointTop(d);
					}
					return y;
				})
				.y1(function(d) {
					var y;
					if (!chart.invertedAxes) {
						y = chart._startPosition;
					} else {
						y = serie._pointTop(d);
					}
					return y;
				});

			var area = d3.svg.area()
				.interpolate(serie.lineType)
				.defined(function(d) { return !isUndefinedOrNull(d[field]); })
				.x0(function(d) {
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						if (serie.stacked && i > 0) {
							x = chart._xScale(d["_stackedValue_"+serie.fields[i-1]]);
						} else {
							x = chart._startPosition;
						}
					}
					return x;
				})
				.x1(function(d) {
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						if (serie.stacked) {
							x = chart._xScale(d["_stackedValue_"+field]);
						} else {
							x = chart._xScale(d[field]);
						}
					}
					return x;
				})
				.y0(function(d) {
					var y;
					if (!chart.invertedAxes) {
						if (serie.stacked && i > 0) {
							y = serie._yScale(d["_stackedValue_"+serie.fields[i-1]]);
						} else {
							y = chart._startPosition;
						}
					} else {
						y = serie._pointTop(d);
					}
					return y;
				})
				.y1(function(d) {
					var y;
					if (!chart.invertedAxes) {
						if (serie.stacked) {
							y = serie._yScale(d["_stackedValue_"+field]);
						} else {
							y = serie._yScale(d[field]);
						}
					} else {
						y = serie._pointTop(d);
					}
					return y;
				});

			var path = serie._serieContainer._dataContainer.select("path.area.field-"+i);
			if (path.empty()) {
				path = serie._serieContainer._dataContainer.append("path")
					.attr("class", "area field-"+i)
					.attr("fill", serie.colors[i % serie.colors.length])
					.attr("opacity", serie.areaOpacity)
					.attr("stroke-width", 0)
					.attr("d", initialArea(chart.dataSource.data))
					.on("mouseup", function(){
						chartAreaClick(serie);
					});

			}

			path.transition().duration(_duration)
				.attr("d", area(chart.dataSource.data));
		});
	}

	function drawLine(serie) {
		if (serie.lineType !== false) {
			var clonedDataSource = Util.clone(chart.dataSource.data);

			if (chart.dataSource.data.length > 0 && serie.lineExpanded) {
				clonedDataSource.push(clonedDataSource[clonedDataSource.length -1]);
				clonedDataSource.unshift(clonedDataSource[0]);
			}

			serie.fields.forEach(function(field, i) {
				var initialLine = d3.svg.line()
					.interpolate(serie.lineType)
					.defined(function(d) { return !isUndefinedOrNull(d[field]); })
					.x(function(d, i) {
						var _x;
						if (!chart.invertedAxes) {
							_x = chart._pointLeft(d);
							// Check if not inverted
							if (serie.lineExpanded && i === 0) {
								_x = 0;
							} else if (serie.lineExpanded && i === clonedDataSource.length -1) {
								_x = chart._xAxisContainer.select("path.domain").node().getBBox().width;
							}

						} else {
							_x = chart._startPosition;
						}
						return _x;
					})
					.y(function(d, i) {
						var _y;
						if (!chart.invertedAxes) {
							_y = chart._startPosition;
						} else {
							_y = serie._pointTop(d);
							if (serie.lineExpanded && i === 0) {
							    _y = chart.series[0]._serieContainer.node().getBBox().height;
							} else if (serie.lineExpanded && i === clonedDataSource.length -1) {
							    _y = 0;
							}
						}
						return _y;
					});

				var line = d3.svg.line()
					.interpolate(serie.lineType)
					.defined(function(d) { return !isUndefinedOrNull(d[field]); })
					.x(function(d, i) {
						var _x;
						if (!chart.invertedAxes) {
							_x = chart._pointLeft(d);
							// Check if not inverted
							if(serie.lineExpanded && i === 0) {
								_x = 0;
							} else if (serie.lineExpanded && i === clonedDataSource.length -1) {
								_x = chart._xAxisContainer.select("path.domain").node().getBBox().width;
							}

						} else {
							if (serie.stacked) {
								_x = chart._xScale(d["_stackedValue_"+field]);
							} else {
								_x = chart._xScale(d[field]);
							}
						}
						return _x;
					})
					.y(function(d, i) {
						var _y;
						if (!chart.invertedAxes) {
							if (serie.stacked) {
								_y = serie._yScale(d["_stackedValue_"+field]);
							} else {
								_y = serie._yScale(d[field]);
							}
						} else {
							_y = serie._pointTop(d);
							// Check if is inverted
							if (serie.lineExpanded && i === 0) {
							    _y = chart.series[0]._serieContainer.node().getBBox().height;
							} else if (serie.lineExpanded && i === clonedDataSource.length -1) {
							    _y = 0;
							}
						}

						return _y;
					});

				var path = serie._serieContainer._dataContainer.select("path.line.field-"+i);
				if (path.empty()) {
					path = serie._serieContainer._dataContainer.append("path")
						.attr("class", "line field-"+i)
						.attr("fill", "none")
						.attr("stroke-width", serie.lineWidth)
						.attr("stroke-linecap", "round")
						.attr("stroke-linejoin", "round")
						.attr("d", initialLine(clonedDataSource))
						.attr("stroke", serie.colors[i % serie.colors.length]);
				}

				path.transition().duration(_duration)
					.attr("d", line(clonedDataSource));
			});
		}
	}

	function drawMarks(serie) {
		var getMarkY = function(field, d) {
			var y;
			if (serie.stacked) {
				y = serie._yScale(d["_stackedValue_"+field]);
			} else {
				y = serie._yScale(d[field]);
			}
			return y;
		};

		var getMarkX = function(field, d) {
			var x;
			if (serie.stacked) {
				x = chart._xScale(d["_stackedValue_"+field]);
			} else {
				x = chart._xScale(d[field]);
			}
			return x;
		};

		var getTextAnchor = function(d, i, field) {
			var leftDir, rightDir;
			var leftValue, rightValue, value;
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;

			var textAnchor;
			if (i === 0) {
				textAnchor = "start";
			} else {
				value = chart.dataSource.data[i][fieldName];
				leftValue = chart.dataSource.data[i - 1][fieldName];

				leftDir = leftValue > value ? "up" : "down";
			}

			if (i == chart.dataSource.data.length - 1) {
				textAnchor = "end";
			} else {
				value = chart.dataSource.data[i][fieldName];
				rightValue = chart.dataSource.data[i + 1][fieldName];

				rightDir = rightValue > value ? "up" : "down";
			}

			return textAnchor || (leftDir != rightDir ? "start" : "middle");
		};

		var getTextAnchorInverted = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textAnchor;
			if (i === 0) {
				textAnchor = "start";
			} else {
				textAnchor = chart.dataSource.data[i - 1][fieldName] <= chart.dataSource.data[i][fieldName] ? "start" : "end";
			}

			return textAnchor;
		};

		var getTextRelativeY = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textRelativeY;
			if (chart.dataSource.data.length === 1) {
				textRelativeY = "-1em";
			} else {
				if (i == chart.dataSource.data.length - 1) {
					textRelativeY = chart.dataSource.data[i - 1][fieldName] > chart.dataSource.data[i][fieldName] ? "1.7em" : "-1em";
				} else {
					textRelativeY = chart.dataSource.data[i + 1][fieldName] > chart.dataSource.data[i][fieldName] ? "1.7em" : "-1em";
				}
			}

			return textRelativeY;
		};

		var getTextRelativeX = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textRelativeY;
			if (i === 0) {
				textRelativeY = "0.6em";
			} else {
				textRelativeY = chart.dataSource.data[i - 1][fieldName] <= chart.dataSource.data[i][fieldName] ? "0.6em" : "-0.6em";
			}

			return textRelativeY;
		};

		if (serie.marks) {
			var marks = serie._serieContainer._dataContainer.selectAll("g.marks").data(chart.dataSource.data);

			marks.enter().append("g")
				.attr("class","marks");

			marks.exit().remove();

			serie.fields.forEach(function(field, i) {
				var symbol;
				marks.each(function(markData, markIndex) {
					if (!isUndefinedOrNull(markData[field])) {
						var markGroup = d3.select(this);
						var markY = getMarkY(field, markData);
						var markX = getMarkX(field, markData);

						if (serie.marks.type !== false) {
							var mark = markGroup.select("path.field-"+i);
							if (mark.empty()) {
								symbol = d3.svg.symbol().type(serie.marks.type).size(0);
								mark = markGroup.append("path")
									.attr("class","field-"+i)
									.attr("transform", function(d) {
										if (!chart.invertedAxes) {
											this._translate = "translate(" + chart._pointLeft(d) + ", " + chart._startPosition + ")";
										} else {
											this._translate = "translate(" + chart._startPosition + ", " + serie._pointTop(d) + ")";
										}
										return this._translate;
									})
									.attr("d", symbol)
									.attr("fill", serie.colors[i % serie.colors.length])
									.attr("field-name", field)
									.on("mouseover", function(d) {
										d3.select(this).transition().duration(100).attr("transform", this._translate + " scale(1.5)");
										showTooltip(d);
									})
									.on("mousemove", showTooltip)
									.on("mouseout", function(d) {
										d3.select(this).transition().duration(100).attr("transform", this._translate + " scale(1)");
										hideTooltip();
									})
									.on("mouseup", dataClick);
							}
							symbol = d3.svg.symbol().type(serie.marks.type).size(Math.pow(serie.marks.size,2));

							if (!chart.invertedAxes) {
								mark.transition().duration(_duration)
									.attr("transform", function(d, i) {
										this._translate = "translate(" + chart._pointLeft(d) + ", " + markY + ")";
										return this._translate;
									})
									.attr("d", symbol);
							} else {
								mark.transition().duration(_duration)
									.attr("transform", function(d, i) {
										this._translate = "translate(" + markX + ", " + serie._pointTop(d) + ")";
										return this._translate;
									})
									.attr("d", symbol);
							}
						}

						setupShowTooltipOnLongTap(markGroup.selectAll("path"));

						if (serie.showValueLabels) {
							var label = markGroup.select("text.field-"+i);

							if (!chart.invertedAxes) {
								if (label.empty()) {
									label = markGroup.append("text")
										.attr("class", "label-value field-"+i)
										.style("font-size", "11px")
										.attr("x", function(d) { return chart._pointLeft(d); })
										.attr("y", chart._startPosition)
										.attr("dy", function(d) { return getTextRelativeY(d, markIndex, field); })
										.attr("text-anchor", function(d) { return getTextAnchor(d, markIndex, field); })
										.attr("field-name", field)
										.attr("fill", serie.colors[i % serie.colors.length])
										.on("mouseover", showTooltip)
										.on("mousemove", showTooltip)
										.on("mouseout", hideTooltip)
										.on("mouseup", dataClick);
								}

								label.transition().duration(_duration)
									.attr("x", function(d) { return chart._pointLeft(d); })
									.attr("y", function(d) { return markY; });
							} else {
								if (label.empty()) {
									label = markGroup.append("text")
										.attr("class", "label-value field-"+i)
										.style("font-size", "11px")
										.attr("x", chart._startPosition)
										.attr("y", function(d) { return serie._pointTop(d); })
										.attr("dy", "-0.6em")
										.attr("dx", function(d) { return getTextRelativeX(d, markIndex, field); })
										.attr("text-anchor", function(d) { return getTextAnchorInverted(d, markIndex, field); })
										.attr("field-name", field)
										.attr("fill", serie.colors[i % serie.colors.length])
										.on("mouseover", showTooltip)
										.on("mousemove", showTooltip)
										.on("mouseout", hideTooltip)
										.on("mouseup", dataClick);
								}

								label.transition().duration(_duration)
									.attr("x", function(d) { return markX; })
									.attr("y", function(d) { return serie._pointTop(d); })
									.each("end", function(d, i) {
										var _label = d3.select(this);
										var labelBox = _label.node().getBBox();
										if (labelBox.x + labelBox.width > chart._chartWidth) {
											_label.attr("dx", "-0.6em")
												.attr("text-anchor", "end");
										}
										if (labelBox.x < 0) {
											_label.attr("dx", "0.6em")
												.attr("text-anchor", "start");
										}
									});
							}

							label.text(function(d) {
								var value = d[field];
								if (serie.yAxis.format) {
									if (isFunction(serie.yAxis.format)) {
										value = serie.yAxis.format(value);
									} else {
										value = formatValue(value, serie.yAxis.format);
									}
								}
								return value;
							});

							setupShowTooltipOnLongTap(markGroup.selectAll("text"));
						}
					}
				});
			});
		}
	}

	function drawLineSerie(serie) {
		drawLine(serie);
		drawMarks(serie);
	}

	function drawAreaSerie(serie) {
		drawArea(serie);
		drawLine(serie);
		drawMarks(serie);
	}

	function checkLabelOverlap(labels) {
		var again = false;
		var a, b;
		var minSpace = 15;

		labels.each(function(aData, i) {
			a = this;

			labels.each(function(bData, j) {
				b = this;

				if (a == b || i > j) return; // same object

				var aObj = d3.select(a);
				var bObj = d3.select(b);

				if (aObj.attr("text-anchor") != bObj.attr("text-anchor")) return; // diferent text-anchor don't collide

				var aPosY = parseFloat(aObj.attr("new-y") || aObj.attr("transform").substring(10).slice(0,-1).split(", ")[1]);
				var bPosY = parseFloat(bObj.attr("new-y") || bObj.attr("transform").substring(10).slice(0,-1).split(", ")[1]);
				var delta = aPosY - bPosY;

				if (Math.round(Math.abs(delta)) >= minSpace) return; // no collision

				again = true;

				var aAngle = (aData.startAngle + aData.endAngle) / 2;
				var signY = aObj.attr("text-anchor") == "end" ? 1 : -1;
				var alpha = (minSpace - Math.abs(delta)) * signY;

				if (Math.tan(aAngle) > 0) { // first and third quadrant
					aObj.attr("new-y", aPosY + alpha);
				} else {
					bObj.attr("new-y", bPosY - alpha);
				}
			});
		});

		if (again) {
			checkLabelOverlap(labels);
		} else {
			labels.each(function(data) {
				a = d3.select(this);

				if (a.attr("new-y")) {
					var posX = a.attr("transform").substring(10).slice(0,-1).split(", ")[0];
					var posY = a.attr("transform").substring(10).slice(0,-1).split(", ")[1];

					a.transition().duration(250).attr("transform", "translate(" + posX + ", " + a.attr("new-y") + ")");
					a.attr("new-y", null);
				}
			});
		}
	}

	function getLabelValue(serie, d) {
		var value;

		if (serie.showValueLabels == "value") {
			value = d.data[serie.fieldValue];
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
		} else if (serie.showValueLabels == "%") {
			value = d.data[serie.fieldValue];
			value = (value * 100 / serie._sumData).formatFloat(2)+"%";
		} else if (serie.showValueLabels == "label") {
			value = d.data[serie.fieldLabel];
		}

		return value;
	}

	var _labelRadius = 20;
	function drawSlices(serie, radius) {
		var pie, pieData, dataLength, slicesY;
		if (serie.type == "gauge") {
			pie = d3.layout.pie()
				.sort(null)
				.value(function(d) { return d.rangeSize; })
				.startAngle(-Math.PI/2)
				.endAngle(Math.PI/2)
				.padAngle(serie.arcsGap);

			pieData = pie(serie._gaugeData);
			dataLength = serie._gaugeData.length;
			slicesY = (chart._chartHeight + radius) / 2;
			if (serie.showGaugeValue) {
				slicesY -= 20;
			}
		} else {
			pie = d3.layout.pie()
				.sort(null)
				.value(function(d) { return d[serie.fieldValue]; })
				.padAngle(serie.arcsGap);

			pieData = pie(serie.pieData);
			dataLength = serie.pieData.length;
			slicesY = chart._chartHeight/2;
		}

		var slices = serie._serieContainer._dataContainer.selectAll("g.slices");
		if (slices.empty()) {
			slices = serie._serieContainer._dataContainer.append("g")
				.attr("class", "slices")
				.attr("transform", "translate(" + chart._chartWidth/2 + ", " + slicesY + ")");
		}

		slices.transition().duration(_duration)
			.attr("transform", "translate(" + chart._chartWidth/2 + ", " + slicesY + ")");

		var arcs = slices.selectAll("path").data(pieData);
		var arcsLength = arcs.size();

		arcs.enter().append("path")
			.attr("class","slice")
			.attr("fill", function(d, i) {
				var color = serie.colors[i % serie.colors.length];
				return (serie.fieldColor && d.data[serie.fieldColor]) || color;
			})
			.attr("stroke", serie.strokeColor)
			.attr("stroke-width", serie.strokeWidth)
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip)
			.on("mouseup", function(d) {
				dataClick(d.data);
			});

		arcs.exit().remove();

		setupShowTooltipOnLongTap(arcs);

		arcs.transition().duration(_duration)
			.attrTween("d", function(d) {
				if (isUndefinedOrNull(this._current)) {
					this._current = jQuery.extend(true, {}, d);
					if (serie.type == "gauge") {
						this._current.startAngle = -Math.PI/2;
						this._current.endAngle = -Math.PI/2;
						this._current.radius = radius;
					} else {
						if (arcsLength > 0) {
							this._current.startAngle = Math.PI*2;
							this._current.endAngle = Math.PI*2;
							this._current.radius = radius;
						} else {
							this._current.startAngle = 0;
							this._current.endAngle = 0;
							this._current.radius = 0;
						}
					}
				}
				var dataInterpolation = d3.interpolate(this._current, d);
				var radiusInterpolation = d3.interpolate(this._current.radius, radius);
				this._current = d;
				this._current.radius = radius;

				return function(t) {
					d = dataInterpolation(t);

					var r = radiusInterpolation(t);
					var arc = d3.svg.arc().outerRadius(r);

					if (serie.type == "donut" || serie.type == "gauge") {
						arc.innerRadius(r - r * serie.donutWidth);
					} else {
						arc.innerRadius(0);
					}

					return arc(d);
				};
			});

		if (serie.showValueLabels !== false && serie.type != "gauge") {
			var labels = slices.selectAll("text.label-value").data(pieData);

			labels.enter().append("text")
				.attr("class", "label-value")
				.attr("fill", function(d, i) {
					var color = serie.colors[i % serie.colors.length];
					return (serie.fieldColor && d.data[serie.fieldColor]) || color;
				})
				.style("font-size", "11px")
				.attr("dy", "0.35em")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip)
				.attr("text-anchor", function(d, i) {
					var arc = d3.svg.arc()
							.outerRadius(radius)
							.innerRadius(radius);

					var centroid = arc.centroid(d);
					var midAngle = Math.atan2(centroid[1], centroid[0]);
					var x = Math.cos(midAngle) * _labelRadius;
					return x > 0 ? "start" : "end";
				});

			labels.exit().remove();

			setupShowTooltipOnLongTap(labels);

			labels.text(function(d) { return getLabelValue(serie, d); });

			labels.transition().duration(_duration)
				.each("end", function(d, i) {
					if (i == dataLength - 1) {
						checkLabelOverlap(labels);
					}
				})
				.attrTween("transform", function(d) {
					if (isUndefinedOrNull(this._current)) {
						this._current = jQuery.extend(true, {}, d);

						if (arcsLength > 0) {
							this._current.startAngle = Math.PI*2;
							this._current.endAngle = Math.PI*2;
							this._current.radius = radius;
						} else {
							this._current.startAngle = 0;
							this._current.endAngle = 0;
							this._current.radius = 0;
						}
					}
					var dataInterpolation = d3.interpolate(this._current, d);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					this._current = d;
					this._current.radius = radius;

					return function(t) {
						d = dataInterpolation(t);

						var r = radiusInterpolation(t);
						var arc = d3.svg.arc()
							.outerRadius(r)
							.innerRadius(r);

						var centroid = arc.centroid(d);
						var x = centroid[0], y = centroid[1];
						var midAngle = Math.atan2(y, x);

						var _x = Math.cos(midAngle) * _labelRadius/2;
						var sign = (_x > 0) ? 1 : -1;
						x += _x + (5 * sign);

						var _y = Math.sin(midAngle) * _labelRadius/2;
						y += _y;

						return "translate(" + x + ", " + y + ")";
					};
				});
		}
	}

	function getPieLabelsMaxDim(serie) {
		var dim = {
			width: 0,
			height: 0
		};

		var text, value;
		for (var i = 0; i < serie.pieData.length; i++) {
			value = getLabelValue(serie, { data: serie.pieData[i] });

			text = serie._serieContainer.append("text")
				.attr("font-size", "11px")
				.text(value);

			dim.width = Math.max(dim.width, text.node().getBBox().width);
			dim.height = Math.max(dim.height, text.node().getBBox().height);

			text.remove();
		}

		return dim;
	}

	function drawPieDonutSerie(serie) {
		var radius = Math.min(chart._chartWidth, chart._chartHeight) / 2;
		if (serie.showValueLabels !== false) {
			var labelsMaxDim = getPieLabelsMaxDim(serie);
			radius = Math.min(chart._chartWidth - labelsMaxDim.width * 2, chart._chartHeight - labelsMaxDim.height * 2) / 2 - _labelRadius;
		}

		drawSlices(serie, radius);
	}

	function drawGaugePointer(serie, radius) {
		var config, pointerX, pointerY;
		var startX, maxWidth;

		if (serie.horizontal) {
			config = {
				pointerWidth: Math.round(radius * 0.03),
				pointerTailLength: Math.round(radius * 0.02),
				pointerHeadLength: Math.round(radius * serie.gaugeWidth * 0.95)
			};
			pointerX = chart._chartWidth * 0.1;
			pointerY = radius * (1 + serie.gaugeWidth) / 2;
		} else {
			config = {
				pointerWidth: Math.round(radius * 0.05),
				pointerTailLength: Math.round(radius * 0.03),
				pointerHeadLength: Math.round(radius * 0.95)
			};
			pointerX = chart._chartWidth/2;
			pointerY = (chart._chartHeight + radius) / 2;

			if (serie.showGaugeValue) {
				pointerY -= 20;
			}
		}

		var lineData = [
			[config.pointerWidth / 2, 0],
			[0, -config.pointerHeadLength],
			[-(config.pointerWidth / 2), 0],
			[0, config.pointerTailLength]
		];
		var pointerLine = d3.svg.line().interpolate('monotone');

		var pointer = serie._serieContainer._dataContainer.selectAll("g.pointer");
		if (pointer.empty()) {
			pointer = serie._serieContainer._dataContainer.append("g").data([lineData])
				.attr("class", "pointer")
				.attr("fill", serie.pointerColor)
				.attr("transform", "translate(" + pointerX + ", " + pointerY + ")");

			pointer.append("path");

			if (!serie.horizontal) {
				pointer.select("path").attr("transform", "rotate(-90)");
			}
		}

		var angle;
		if (serie.horizontal) {
			startX = chart._chartWidth * 0.1;
			maxWidth = chart._chartWidth * 0.8;
			angle = 0;
			pointerX = startX + maxWidth * serie.value / (serie._maxValue - serie._minValue);
		} else {
			angle = Math.min(-90 + (180 * serie.value / (serie._maxValue - serie._minValue)), 90);
		}

		pointer.transition().duration(_duration)
			.attr("transform", "translate(" + pointerX + ", " + pointerY + ")")
			.select("path")
			.attr("d", function(d) { return pointerLine(d) + "Z"; })
			.attr("transform", "rotate(" + angle + ")");

		if (serie.showValueLabels) {
			var label = pointer.selectAll("text.label-value");

			if (label.empty()) {
				label = pointer.append("text")
					.attr("class", "label-value")
					.style("font-size", "11px")
					.attr("dy", "0.35em");
			}

			if (serie.horizontal) {
				label.attr("text-anchor", "middle");
			}

			label.transition().duration(_duration)
				.each("end", function(d) {
					var obj = d3.select(this);
					if (serie.showValueLabels == "%"){
						obj.text((serie.value * 100 / (serie._maxValue - serie._minValue)).formatFloat(2)+"%");
					} else {
						var value = serie.value;
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								value = serie.valueFormat(value);
							} else {
								value = formatValue(value, serie.valueFormat);
							}
						}
						obj.text(value);
					}
				})
				.attrTween("transform", function(d) {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { angle: -90, radius: radius, value: serie._minValue };
					}
					var angleInterpolation = d3.interpolate(this._current.angle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					this._current.angle = angle;
					this._current.radius = radius;
					this._current.value = serie.value;

					return function(t) {
						var x, y, percent;
						var aRad;
						var r = radiusInterpolation(t);

						if (serie.horizontal) {
							var v = valueInterpolation(t);
							percent = v * 100 / (serie._maxValue - serie._minValue);

							x = 0;
							y = (-r * serie.gaugeWidth) - (_labelRadius / 2);
						} else {
							var a = angleInterpolation(t);
							aRad = a * Math.PI / 180;

							percent = (a + 90) * 100 / 180;

							x = Math.sin(aRad) * (r + _labelRadius / 2);
							y = Math.cos(aRad) * (r + _labelRadius / 2) * -1;
						}

						var anchor;
						if (serie.horizontal) {
							anchor = "middle";
						} else {
							if (Math.tan(aRad) < 0) {
								anchor = "end";
							} else if (Math.tan(aRad) === 0) {
								anchor = "middle";
							} else {
								anchor = "start";
							}
						}
						obj.attr("text-anchor", anchor);

						if (serie.showValueLabels == "%") {
							obj.text(percent.formatFloat(2)+"%");
						} else {
							var value = (serie._maxValue - serie._minValue) * percent / 100;
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									value = serie.valueFormat(value);
								} else {
									value = formatValue(value, serie.valueFormat);
								}
							}
							obj.text(value);
						}

						return "translate(" + x + ", " + y + ")";
					};
				});
		}

		if (serie.showGaugeValue) {
			var valueLabel = pointer.selectAll("text.value");

			if (valueLabel.empty()) {
				valueLabel = pointer.append("text")
					.attr("class", "value")
					.attr("text-anchor", "middle")
					.attr("font-size", 21)
					.attr("dy", "2em");

				if (serie.horizontal) {
					valueLabel.attr("dy", "2.5em");
				}
			}

			valueLabel.transition().duration(_duration)
				.tween("text", function() {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { value: 0 };
					}
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					this._current.value = serie.value;

					return function(t) {
						var v = valueInterpolation(t);

						if (serie.showGaugeValue == "%") {
							obj.text((v * 100 / (serie.maxValue || 1)).formatFloat(2)+"%");
						} else {
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									v = serie.valueFormat(v);
								} else {
									v = formatValue(v, serie.valueFormat);
								}
							}
							obj.text(v);
						}
					};
				});
		}
	}

	function drawGaugeLimits(serie, radius) {
		var limitsX, limitsY;

		if (serie.horizontal) {
			limitsX = chart._chartWidth * 0.1;
			limitsY = chart._chartHeight * (1 + serie.gaugeWidth) / 2;
		} else {
			limitsX = chart._chartWidth / 2;
			limitsY = (chart._chartHeight + radius) / 2;

			if (serie.showGaugeValue) {
				limitsY -= 20;
			}
		}

		var limits = serie._serieContainer._dataContainer.selectAll("g.limits");
		if (limits.empty()) {
			limits = serie._serieContainer._dataContainer.append("g")
				.attr("class", "limits")
				.attr("transform", "translate(" + limitsX + ", " + limitsY + ")");
		}

		limits.transition().duration(_duration)
			.attr("transform", "translate(" + limitsX + ", " + limitsY + ")");

		var minLimit = limits.select(".min-limit");
		var maxLimit = limits.select(".max-limit");

		var minLimitX, maxLimitX;

		if (serie.horizontal) {
			minLimitX = 0;
			maxLimitX = chart._chartWidth * 0.8;
		} else {
			minLimitX = radius*(-1+serie.donutWidth/2);
			maxLimitX = radius*(1-serie.donutWidth/2);
		}

		var fontSize = 14;
		if (minLimit.empty()) { // && maxLimit.empty()
			minLimit = limits.append("text")
				.attr("class", "min-limit")
				.attr("font-size", fontSize)
				.attr("text-anchor", "middle")
				.attr("dy", "1.3em")
				.attr("transform", "translate(" + minLimitX + ", 0)");

			maxLimit = limits.append("text")
				.attr("class", "max-limit")
				.attr("font-size", fontSize)
				.attr("text-anchor", "middle")
				.attr("dy", "1.3em")
				.attr("transform", "translate(" + maxLimitX + ", 0)");
		}

		minLimit.text(function() {
			var value = serie._minValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			return value;
		});

		maxLimit.text(function() {
			var value = serie._maxValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			return value;
		});

		if (serie.horizontal) {
			while (Math.max(maxLimit.node().getBBox().width, minLimit.node().getBBox().width) >= (chart._chartWidth * 0.2) && fontSize > 9) {
				fontSize--;
				limits.selectAll("text").attr("font-size", fontSize);
			}
		} else {
			while (Math.max(maxLimit.node().getBBox().width, minLimit.node().getBBox().width) >= (radius * serie.donutWidth) && fontSize > 9) {
				fontSize--;
				limits.selectAll("text").attr("font-size", fontSize);
			}

			if (minLimit.node().getBBox().width - minLimitX > chart._chartWidth / 2) {
				minLimit.attr("text-anchor", "start");
			}

			if (minLimit.node().getBBox().width + maxLimitX > chart._chartWidth / 2) {
				maxLimit.attr("text-anchor", "end");
			}
		}

		minLimit.transition().duration(_duration)
			.attr("transform", "translate(" + minLimitX + ", 0)");

		maxLimit.transition().duration(_duration)
			.attr("transform", "translate(" + maxLimitX + ", 0)");
	}

	function drawGaugeBands(serie) {
		var bands = serie._serieContainer._dataContainer.selectAll("g.bands");
		if (bands.empty()) {
			bands = serie._serieContainer._dataContainer.append("g")
				.attr("class", "bands")
				.attr("transform", "translate(" + chart._chartWidth * 0.1 + ", " + chart._chartHeight/2 + ")");
		}

		bands.transition().duration(_duration)
			.attr("transform", "translate(" + chart._chartWidth * 0.1 + ", " + chart._chartHeight/2 + ")");

		var rects = bands.selectAll("rect").data(serie._gaugeData);

		var maxWidth = chart._chartWidth * 0.8;
		rects.enter().append("rect")
			.attr("class","band")
			.attr("fill", function(d, i) { return serie.colors[i % serie.colors.length]; })
			.attr("stroke", serie.strokeColor)
			.attr("stroke-width", serie.strokeWidth)
			.attr("width", 0)
			.attr("height", chart._chartHeight * serie.gaugeWidth)
			.attr("x", 0)
			.attr("y", chart._chartHeight * -serie.gaugeWidth / 2)
			.on("mouseover", function(d) { showTooltip({data: d}); })
			.on("mousemove", function(d) { showTooltip({data: d}); })
			.on("mouseout", hideTooltip)
			.on("mouseup", dataClick);

		rects.exit().remove();

		function getBandWidth(value) {
			return value * maxWidth / serie._maxValue;
		}

		rects.transition().duration(_duration)
			.attr("height", chart._chartHeight * serie.gaugeWidth)
			.attr("width", function(d, i) { return getBandWidth(d.rangeSize); })
			.attr("x", function(d, i) { return i === 0 ? 0 : getBandWidth(serie.ranges[i-1]) + getBandWidth(serie._gaugeData[i-1].rangeSize); })
			.attr("y", chart._chartHeight * -serie.gaugeWidth / 2);
	}

	function getGaugeLabelsMaxDim(serie) {
		var dim = {
			width: 0,
			height: 0
		};

		var text = serie._serieContainer.append("text")
			.attr("font-size", "11px");

		if (serie.showValueLabels == "%") {
			text.text((100).formatFloat(2)+"%");
		} else {
			var value = serie._maxValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			text.text(value);
		}

		dim.width = Math.max(dim.width, text.node().getBBox().width);
		dim.height = Math.max(dim.height, text.node().getBBox().height);

		text.remove();

		return dim;
	}

	function drawGaugeSerie(serie) {
		var valueHeight = serie.showGaugeValue ? 40 : 0;
		var radius = Math.min(chart._chartWidth / 2, chart._chartHeight - valueHeight);
		if (serie.showValueLabels !== false) {
			var labelsMaxDim = getGaugeLabelsMaxDim(serie);
			radius = Math.min(chart._chartWidth / 2 - labelsMaxDim.width, chart._chartHeight - valueHeight - labelsMaxDim.height) - _labelRadius;
		}

		if (serie.horizontal) {
			radius = chart._chartHeight;
			drawGaugeBands(serie);
		} else {
			serie.donutWidth = serie.gaugeWidth;
			drawSlices(serie, radius);
		}

		drawGaugeLimits(serie, radius);
		drawGaugePointer(serie, radius);
	}

	function drawProgressSerie(serie) {
		var radius;
		var progressX, progressY, progressWidth, maxWidth;

		if (serie.horizontal) {
			radius = chart._chartHeight;
			progressX = chart._chartWidth * 0.1;
		} else {
			radius = Math.min(chart._chartWidth, chart._chartHeight) / 2;
			if (serie.showValueLabels !== false) {
				radius -= _labelRadius;
			}

			progressX = chart._chartWidth / 2;
		}
		progressY = chart._chartHeight / 2;
		progressWidth = chart._chartHeight * serie.progressWidth;
		maxWidth = chart._chartWidth * 0.8;

		var progressContainer = serie._serieContainer._dataContainer.selectAll("g.progress-container");
		if (progressContainer.empty()) {
			progressContainer = serie._serieContainer._dataContainer.append("g")
				.attr("class", "progress-container")
				.attr("transform", "translate(" + progressX + ", " + progressY + ")");
		}
		progressContainer.transition().duration(_duration)
			.attr("transform", "translate(" + progressX + ", " + progressY + ")");

		progressBackground = progressContainer.select(".progress-background");
		if (progressBackground.empty()) {
			if (serie.horizontal) {
				progressBackground = progressContainer.append("rect")
					.attr("x", 0)
					.attr("y", -progressWidth / 2)
					.attr("rx", progressWidth / 2)
					.attr("ry", progressWidth / 2)
					.attr("width", maxWidth)
					.attr("height", progressWidth);

			} else {
				progressBackground = progressContainer.append("path")
					.attr("d", function(d) {
						var arc = d3.svg.arc()
							.innerRadius(radius * (1 - serie.progressWidth))
							.outerRadius(radius)
							.startAngle(0)
							.endAngle(Math.PI * 2);
						return arc();
					});
			}

			progressBackground.attr("class","progress-background")
				.attr("stroke-width",0)
				.attr("fill", serie.backgroundColor);
		}


		if (serie.horizontal) {
			progressBackground.transition().duration(_duration)
				.attr("x", 0)
				.attr("y", -progressWidth / 2)
				.attr("rx", progressWidth / 2)
				.attr("ry", progressWidth / 2)
				.attr("width", maxWidth)
				.attr("height", progressWidth);
		} else {
			progressBackground.transition().duration(_duration)
				.attr("d", function(d) {
					var arc = d3.svg.arc()
						.innerRadius(radius * (1 - serie.progressWidth))
						.outerRadius(radius)
						.startAngle(0)
						.endAngle(Math.PI * 2);
					return arc();
				});
		}

		progress = progressContainer.select(".progress-indicator");
		if (progress.empty()) {
			if (serie.horizontal) {
				progress = progressContainer.append("rect")
					.attr("x", 0)
					.attr("y", -progressWidth / 2)
					.attr("rx", progressWidth / 2)
					.attr("ry", progressWidth / 2)
					.attr("width", 0)
					.attr("height", progressWidth);
			} else {
				progress = progressContainer.append("path");
			}

			progress.attr("class","progress-indicator")
				.attr("stroke-width",0)
				.attr("fill", serie.colors[0]);
		}

		if (serie.horizontal) {
			progress.transition().duration(_duration)
				.attr("y", -progressWidth / 2)
				.attr("rx", progressWidth / 2)
				.attr("ry", progressWidth / 2)
				.attr("width", serie.value * maxWidth / (serie.maxValue || 1))
				.attr("height", progressWidth);
		} else {
			progress.transition().duration(_duration)
				.attrTween("d", function() {
					if (isUndefinedOrNull(this._current)) {
						this._current = { endAngle: 0, radius: radius };
					}
					var angle = Math.PI * 2 * serie.value / (serie.maxValue || 1);
					var angleInterpolation = d3.interpolate(this._current.endAngle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					this._current.endAngle = angle;
					this._current.radius = radius;

					return function(t) {
						var a = angleInterpolation(t);
						var r = radiusInterpolation(t);

						var arc = d3.svg.arc()
							.innerRadius(r * (1 - serie.progressWidth))
							.outerRadius(r)
							.cornerRadius(r * serie.progressWidth)
							.startAngle(0)
							.endAngle(a);

						return arc();
					};
				});
		}

		if (serie.showProgressValue) {
			var valueLabel = progressContainer.selectAll("text.value");

			if (valueLabel.empty()) {
				valueLabel = progressContainer.append("text")
					.attr("class", "value")
					.attr("text-anchor", "middle")
					.attr("dy", "0.35em");

				if (serie.horizontal) {
					valueLabel.attr("dy", "1em");
				}
			}

			valueLabel.transition().duration(_duration)
				.attrTween("font-size", function() {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { radius: radius, value: 0, maxWidth: maxWidth };
					}
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					var maxWidthInterpolation = d3.interpolate(this._current.maxWidth, maxWidth);
					this._current.radius = radius;
					this._current.value = serie.value;
					this._current.maxWidth = maxWidth;

					return function(t) {
						var objWidth = obj.node().getBBox().width;
						var fontSize = parseInt(obj.attr("font-size") || 45);

						var r = radiusInterpolation(t);
						var v = valueInterpolation(t);

						if (serie.horizontal) {
							fontSize = 30;

							var w = maxWidthInterpolation(t);

							obj.attr("x", w / 2)
								.attr("y", r * serie.progressWidth / 2 + _labelRadius / 2);
						} else {
							if (objWidth > r * 0.9) {
								fontSize--;
							} else if (objWidth < r * 0.9) {
								fontSize++;
							}
						}

						if (serie.showProgressValue == "%") {
							obj.text((v * 100 / (serie.maxValue || 1)).formatFloat(2)+"%");
						} else {
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									v = serie.valueFormat(v);
								} else {
									v = formatValue(v, serie.valueFormat);
								}
							}
							obj.text(v);
						}

						return fontSize;
					};
				});
		}

		if (serie.showValueLabels) {
			var label = progressContainer.selectAll("text.label-value");

			if (label.empty()) {
				label = progressContainer.append("text")
					.attr("class", "label-value")
					.style("font-size", "11px")
					.attr("dy", "0.35em");
			}

			label.transition().duration(_duration)
				.each("end", function(d) {
					var obj = d3.select(this);
					if (serie.showValueLabels == "%"){
						obj.text((serie.value * 100 / (serie.maxValue || 1)).formatFloat(2)+"%");
					} else {
						var value = serie.value;
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								value = serie.valueFormat(value);
							} else {
								value = formatValue(value, serie.valueFormat);
							}
						}
						obj.text(value);
					}
				})
				.attrTween("transform", function(d) {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { angle: 0, radius: radius, value: 0, maxWidth: maxWidth };
					}
					var angle = Math.PI * 2 * Math.min(1, serie.value / (serie.maxValue || 1));
					var angleInterpolation = d3.interpolate(this._current.angle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					var maxWidthInterpolation = d3.interpolate(this._current.maxWidth, maxWidth);
					this._current.angle = angle;
					this._current.radius = radius;
					this._current.value = serie.value;
					this._current.maxWidth = maxWidth;

					return function(t) {
						var x, y, percent, a;
						var r = radiusInterpolation(t);

						if (serie.horizontal) {
							var v = valueInterpolation(t);
							var w = maxWidthInterpolation(t);
							percent = v * 100 / (serie.maxValue || 1);

							x = w * percent / 100;
							y = (-r * serie.progressWidth / 2) - (_labelRadius / 2);
						} else {
							a = angleInterpolation(t);

							percent = a * 100 / (Math.PI * 2);

							x = Math.sin(a) * (r + _labelRadius / 2);
							y = Math.cos(a) * (r + _labelRadius / 2) * -1;
						}

						var anchor;
						if (serie.horizontal) {
							anchor = "middle";
						} else {
							if (Math.round(Math.sin(a)) < 0) {
								anchor = "end";
							} else if (Math.round(Math.sin(a)) === 0) {
								anchor = "middle";
							} else {
								anchor = "start";
							}
						}
						obj.attr("text-anchor", anchor);

						if (serie.showValueLabels == "%") {
							obj.text(percent.formatFloat(2)+"%");
						} else {
							var value = serie.maxValue * percent / 100;
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									value = serie.valueFormat(value);
								} else {
									value = formatValue(value, serie.valueFormat);
								}
							}
							obj.text(value);
						}

						return "translate(" + x + ", " + y + ")";
					};
				});
		}
	}

	function drawGanttSerie(serie) {
		var bars = serie._serieContainer._dataContainer.selectAll("g.bars").data(chart.dataSource.data);

		var clippingPathX = chart.clippingPath.attr("transform").split("(")[1].split(",")[0];
		chart.clippingPath
			.attr("height", chart._chartHeight)
			.attr("transform", "translate(" + clippingPathX + ", 0)");

		var itemHeight = serie.itemHeight; // + (serie.showTaskCompletionLabel === true || serie.showTaskCompletionLabel == "below" ? 15 : 0);

		function getBarTransformY(d, i) {
			return (itemHeight*i);
		}

		function transformBar(d, i) {
			var translate = "translate(0, " + getBarTransformY(d, i) + ")";

			return translate;
		}

		bars.attr("transform", transformBar);

		bars.attr("data-transform-y", getBarTransformY);

		bars.enter().append("g")
			.attr("class","bars")
			.attr("transform", transformBar)
			.attr("data-transform-y", getBarTransformY)
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip);

		setupShowTooltipOnLongTap(bars);

		bars.exit().remove();

		// add 'hidden element' to fix seriecontainer.getBBox.y
		var rectBogus = serie._serieContainer._dataContainer.select("rect.bogus");
		if (rectBogus.empty()) {
			rectBogus = serie._serieContainer._dataContainer
				.append("rect")
				.attr("class", "bogus")
				.attr("height", 10)
				.attr("width", 10)
				.attr("fill", "transparent");
		}

		function getBarColorEstimated(d, i) {
			var color = serie.colors[i % serie.colors.length];
			if (serie.overrideTaskColorEstimated) {
				color = serie.overrideTaskColorEstimated;
			} else if (serie.ganttFields.taskColorEstimated) {
				color = d[serie.ganttFields.taskColorEstimated];
			}

			return color;
		}

		function getBarColorReal(d, i) {
			var color = getBarColorEstimated(d,i);
			if (serie.overrideTaskColorReal) {
				color = serie.overrideTaskColorReal;
			} else if (serie.ganttFields.taskColorReal) {
				color = d[serie.ganttFields.taskColorReal];
			} else {
				color = d3.rgb(color).darker(2).toString();
			}

			return color;
		}

		function drawCompletionLabel(barGroup, barData, barIndex) {
			var label = barGroup.select("text.field");
			var _barLeft;
			var textTop = serie.itemBarHeight;

			var estimatedBarWidth = chart._bandLeft(barData, serie.ganttFields.taskEstimatedEnd, 1) - chart._bandLeft(barData, serie.ganttFields.taskEstimatedBegin);

			if (serie.realBarType == "percentual" || isUndefinedOrNull(barData[serie.ganttFields.taskRealBegin])){
				_barLeft = chart._bandLeft(barData, serie.ganttFields.taskEstimatedBegin);
			} else {
				_barLeft = chart._bandLeft(barData, serie.ganttFields.taskRealBegin);
			}

			if (label.empty()) {
				label = barGroup.append("text")
					.attr("class", "label-value field")
					.style("font-size", "11px")
					.attr("x", _barLeft)
					.attr("y", textTop)
					.attr("dy", "1.1em")
					.attr("fill", function() { return getBarColorReal(barData, barIndex); })
					.attr("text-anchor","start");
			}

			label.transition().duration(_duration)
				.attr("x", function() {
					if (serie.realBarType == "percentual") {
						return _barLeft + estimatedBarWidth * barData[serie.ganttFields.taskCompleted] / 100;
					} else {
						return isUndefinedOrNull(barData[serie.ganttFields.taskRealEnd]) ? _barLeft : chart._bandLeft(barData, serie.ganttFields.taskRealEnd, 1);
					}
				})
				.tween("text", function() {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { value: 0 };
					}
					var valueInterpolation = d3.interpolate(this._current.value, barData[serie.ganttFields.taskCompleted]);
					this._current.value = barData[serie.ganttFields.taskCompleted];

					return function(t) {
						var v = valueInterpolation(t);

						v = Math.round(v) + '%';
						obj.text(v);
					};
				});

			label.attr("y", textTop)
				 .attr("fill", function() { return getBarColorReal(barData, barIndex); });
		}

		var estimatedBarTop = 2;
		var projectBarTop = estimatedBarTop;

		function drawTaskName(barGroup, barData, barIndex, taskProject) {
			var taskName = "";
			var label = barGroup.select("text.field-task-name");
			var _barLeft = chart._bandLeft(barData, serie.ganttFields.taskEstimatedBegin);
			var estimatedBarWidth = chart._bandLeft(barData, serie.ganttFields.taskEstimatedEnd, 1) - chart._bandLeft(barData, serie.ganttFields.taskEstimatedBegin);
			var labelX = chart._bandLeft(barData, serie.ganttFields.taskEstimatedEnd, 1);
			var labelRealX;

			if (serie.realBarType == "percentual") {
				labelRealX = _barLeft + (estimatedBarWidth * barData[serie.ganttFields.taskCompleted] / 100);
			} else {
				labelRealX = isUndefinedOrNull(barData[serie.ganttFields.taskRealEnd]) ? _barLeft : chart._bandLeft(barData, serie.ganttFields.taskRealEnd, 1);
			}

			labelX = Math.max(labelX, labelRealX);

			if (serie.showTaskName) {
				taskName = barData[serie.ganttFields.taskName];
			}

			if (serie.showTaskCompletionLabel == "front") {
				if (!isProject(barData) || serie.showCompletionLabelOnParentTasks) {
					taskName = Math.round(barData[serie.ganttFields.taskCompleted])+'% '+taskName;
				}
			}

			if (label.empty()) {
				label = barGroup.append("text")
					.attr("class", "label-value field-task-name")
					.style("font-size", "9px")
					.attr("x", _barLeft)
					.attr("y", estimatedBarTop)
					.attr("dy", "0.9em")
					.attr("dx", (taskProject ? 10 : 5) + "px")
					.attr("fill", function() { return getBarColorReal(barData, barIndex); })
					.attr("text-anchor","start")
					.text(taskName);
			}

			label.transition().duration(_duration)
				.attr("x", labelX)
				.text(taskName);

			label.attr("y", estimatedBarTop)
				 .attr("fill", function() { return getBarColorReal(barData, barIndex); });
		}

		function checkArrowMarker() {
			var defs = d3.select(svgContainer.node().parentNode).select("defs");
			if (defs.select("#marker-arrow").empty()) {
				defs.append("marker")
					.attr({
						"id":"marker-arrow",
						"viewBox":"0 -5 10 10",
						"refX":10,
						"refY":0,
						"markerWidth":8,
						"markerHeight":8,
						"orient":"auto"
					})
					.append("path")
						.attr("d", "M0,-4L10,0L0,4")
						.attr("class","arrowHead");
			}
		}

		function drawPredecessorArrows() {
			var predecessorData = {};

			checkArrowMarker();

			chart.dataSource.data.forEach(function(row) {
				if (!isUndefinedOrNull(row[serie.ganttFields.taskPredecessor])) {
					if (isUndefined(predecessorData[row[serie.ganttFields.taskId]])) {
						if (row[serie.ganttFields.taskPredecessor] instanceof Array) {
							predecessorData[row[serie.ganttFields.taskId]] = row[serie.ganttFields.taskPredecessor];
						} else {
							predecessorData[row[serie.ganttFields.taskId]] = [row[serie.ganttFields.taskPredecessor]];
						}
					} else {
						if (row[serie.ganttFields.taskPredecessor] instanceof Array) {
							predecessorData[row[serie.ganttFields.taskId]].concat(row[serie.ganttFields.taskPredecessor]);
						} else {
							predecessorData[row[serie.ganttFields.taskId]].push(row[serie.ganttFields.taskPredecessor]);
						}
					}
				}
			});

			bars.each(function(d, i) {
				var barGroup = d3.select(this);
				var predecessors = predecessorData[d[serie.ganttFields.taskId]];

				if (!isUndefined(predecessors)) {
					predecessors.forEach(function(predecessorId, idx) {
						var barGroupTranslateY = +barGroup.attr("data-transform-y");

						var predecessorBarGroup = serie._serieContainer._dataContainer.select("g#bar-"+predecessorId);
						if (!predecessorBarGroup.empty() && isParentOpened(d)) {
							var predecessorBarGroupTranslateY = +predecessorBarGroup.attr("data-transform-y");
							var predecessorLineHeight = barGroupTranslateY - predecessorBarGroupTranslateY;

							var predecessorLine = barGroup.select("path.predecessor-line-"+predecessorId);
							var lineData;
							var pointerLine = d3.svg.line().interpolate('linear');

							if (predecessorLine.empty()) {
								lineData = [
									[chart._bandLeft(predecessorBarGroup.datum(), serie.ganttFields.taskEstimatedBegin), estimatedBarTop - predecessorLineHeight + serie.itemBarHeight/2],
									[chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin)+7+(idx*3), estimatedBarTop - predecessorLineHeight + serie.itemBarHeight/2],
									[chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin)+7+(idx*3), estimatedBarTop]
								];
								predecessorLine = barGroup.append("path")
									.data([lineData])
									.attr("class", "predecessor-line-"+predecessorId)
									.attr("stroke", "black")
									.attr("stroke-width", 1)
									.attr("fill", "none")
									.attr("shape-rendering", "crispEdges")
									.attr("marker-end", "url(#marker-arrow)")
									.attr("d", function(d) { return pointerLine(d); });
							}

							lineData = [
								[chart._bandLeft(predecessorBarGroup.datum(), serie.ganttFields.taskEstimatedEnd, 1), estimatedBarTop - predecessorLineHeight + serie.itemBarHeight/2],
								[chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin)+7+(idx*3), estimatedBarTop - predecessorLineHeight + serie.itemBarHeight/2],
								[chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin)+7+(idx*3), estimatedBarTop]
							];

							predecessorLine.data([lineData]) //.transition().duration(_duration)
									.attr("d", function(d) { return pointerLine(d); });

							if (isParentOpened(predecessorBarGroup.data()[0])) {
								predecessorLine.style('display', 'block');
							} else {
								predecessorLine.style('display','none');
							}
						}
					});
				}
			});
		}

		function isProject(d) {
			return d[serie.ganttFields.taskIsParent] === true || d[serie.ganttFields.taskIsParent] == "Y" || chart.dataSource.data.some(function(row) {
				return row[serie.ganttFields.taskParent] == d[serie.ganttFields.taskId];
			});
		}

		function isParentOpened(d) {
			var opened = true;
			var taskParentId = d[serie.ganttFields.taskParent];

			if (taskParentId) {
				var taskParent = serie._serieContainer._dataContainer.select("g#bar-"+taskParentId);
				var taskParentData = taskParent.data()[0];
				if (!taskParentData || !taskParentData._opened) {
					opened = false;
				} else {
					opened = isParentOpened(taskParentData);
				}
			}

			return opened;
		}

		var hiddenBarsOffset = 0;
		bars.each(function(d, i) {
			if (!isUndefinedOrNull(d[serie.ganttFields.taskEstimatedBegin]) && !isUndefinedOrNull(d[serie.ganttFields.taskEstimatedEnd])) {
				var isTaskProject = isProject(d);

				var barGroup = d3.select(this);
				barGroup.attr("id", function() { return "bar-"+d[serie.ganttFields.taskId]; });

				if (isParentOpened(d)) {
					barGroup.style('display', 'block');
					barGroup.attr('data-transform-y', function() {
						return getBarTransformY(d, i) - hiddenBarsOffset;
					});
					barGroup.attr('transform', 'translate(0,' + barGroup.attr('data-transform-y') + ')');

					var currentRect = barGroup.select("rect.current");
					if (_chart.currentRow == d) {
						if (currentRect.empty()) {
							currentRect = barGroup.append("rect")
								.attr("class", "current")
								.attr("x", 0)
								.attr("y", estimatedBarTop)
								.attr("height", itemHeight - 5)
								.attr("width", chart._xAxisContainer.select("path.domain").node().getBBox().width)
								.attr("fill", "#00000016");
						}

						currentRect.transition().duration(_duration)
							.attr("height", itemHeight - 5);
					} else {
						currentRect.remove();
					}

					var backgroundRect = barGroup.select('rect.bkg');
					if (backgroundRect.empty()) {
						backgroundRect = barGroup.append('rect')
							.attr('class', 'rect')
							.attr('x', 0)
							.attr("y", estimatedBarTop)
							.attr("height", itemHeight)
							.attr("width", chart._xAxisContainer.select("path.domain").node().getBBox().width)
							.attr("fill", 'transparent');
					}

					if (isTaskProject) {
						barGroup.select('rect.estimated').remove();
						barGroup.select('rect.real').remove();
						barGroup.select('text.field').remove();

						var projectBarHeight = Math.max(20, serie.itemBarHeight);
						var projectBar = barGroup.select("rect.project");
						var projectBarStart = barGroup.select("path.start");
						var projectBarEnd = barGroup.select("path.end");
						var projectBarWidth = chart._bandLeft(d, serie.ganttFields.taskEstimatedEnd, 1) - chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin);
						var projectBarEdgeData = [
							[-projectBarHeight/4, projectBarTop + projectBarHeight/2 - 1],
							[projectBarHeight/4, projectBarTop + projectBarHeight/2 - 1],
							[0, projectBarTop + projectBarHeight]
						];
						var projectBarEdgeLine = d3.svg.line().interpolate('linear');

						if (projectBar.empty()) {
							projectBar = barGroup.append("rect")
								.attr("class", "project")
								.attr("x", chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) - projectBarHeight/4)
								.attr("y", projectBarTop)
								.attr("height", projectBarHeight/2)
								.attr("width", projectBarHeight/2)
								.attr("fill", "#000")
								.on("click", dataClick);

							projectBarStart = barGroup.append("path")
								.attr("class", "start")
								.attr("transform", "translate(" + chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) + ",0)");

							projectBarEnd = barGroup.append("path")
								.attr("class", "end")
								.attr("transform", "translate(" + chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) + ",0)");
						}

						projectBar.transition().duration(_duration)
							.attr("x", chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) - projectBarHeight/4)
							.attr("height", projectBarHeight/2)
							.attr("width", projectBarWidth + projectBarHeight/2);

						projectBar.attr("y", projectBarTop);

						projectBarStart.data([projectBarEdgeData]).transition().duration(_duration)
							.attr("transform", "translate(" + chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) + ",0)");
						projectBarStart.attr("d", function(d) { return projectBarEdgeLine(d) + "Z"; });

						projectBarEnd.data([projectBarEdgeData]).transition().duration(_duration)
							.attr("transform", "translate(" + (chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin) + projectBarWidth) + ",0)");
						projectBarEnd.attr("d", function(d) { return projectBarEdgeLine(d) + "Z"; });
					} else {
						barGroup.select('rect.project').remove();
						barGroup.select('path.start').remove();
						barGroup.select('path.end').remove();

						var estimatedBar = barGroup.select("rect.estimated");
						var estimatedBarWidth = chart._bandLeft(d, serie.ganttFields.taskEstimatedEnd, 1) - chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin);

						if (estimatedBar.empty()) {
							estimatedBar = barGroup.append("rect")
								.attr("class", "estimated")
								.attr("x", chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin))
								.attr("y", estimatedBarTop)
								.attr("height", serie.itemBarHeight)
								.attr("width", 0)
								.attr("fill", function(d) { return getBarColorEstimated(d, i); })
								.on("click", dataClick);
						}

						estimatedBar.transition().duration(_duration)
							.attr("x", chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin))
							.attr("height", serie.itemBarHeight)
							.attr("width", estimatedBarWidth);

						estimatedBar.attr("y", estimatedBarTop)
									.attr("fill", function(d) { return getBarColorEstimated(d, i); });

						var realBar = barGroup.select("rect.real");
						var realBarTop = estimatedBarTop + (serie.itemBarHeight / 2);
						var realBarHeight = serie.itemBarHeight / 2;
						var realBarLeft;
						if (serie.realBarType == "percentual" || isUndefinedOrNull(d[serie.ganttFields.taskRealBegin])) {
							realBarLeft = chart._bandLeft(d, serie.ganttFields.taskEstimatedBegin);
						} else {
							realBarLeft = chart._bandLeft(d, serie.ganttFields.taskRealBegin);
						}

						if (realBar.empty()) {
							realBar = barGroup.append("rect")
								.attr("class", "real")
								.attr("x", realBarLeft)
								.attr("y", realBarTop)
								.attr("height", realBarHeight)
								.attr("width", 0)
								.attr("fill", function() { return getBarColorReal(d, i); })
								.on("click", dataClick);
						}

						realBar.transition().duration(_duration)
							.attr("x", realBarLeft)
							.attr("height", realBarHeight)
							.attr("width", function() {
								if (serie.realBarType == "percentual") {
									return estimatedBarWidth * d[serie.ganttFields.taskCompleted] / 100;
								} else {
									return isUndefinedOrNull(d[serie.ganttFields.taskRealEnd]) ? 0 : chart._bandLeft(d, serie.ganttFields.taskRealEnd, 1) - realBarLeft;
								}
							});

						realBar.attr("y", realBarTop)
							   .attr("fill", function() { return getBarColorReal(d, i); });

						if (serie.showTaskCompletionLabel === true || serie.showTaskCompletionLabel == "below") {
							drawCompletionLabel(barGroup, d, i);
						}
					}

					if (serie.showTaskName || serie.showTaskCompletionLabel == "front") {
						drawTaskName(barGroup, d, i, isTaskProject);
					}
				} else {
					barGroup.style('display', 'none');

					hiddenBarsOffset += itemHeight;
				}
			}
		});

		if (serie.showPredecessors) {
			drawPredecessorArrows();
		}

		setTimeout(function() {
			setupPanning(true);
		}, _duration);
	}

	function ganttScrollToView(row) {
		var serie  = chart.series[0];
		var taskId = row[serie.ganttFields.taskId];
		var bar = serie._serieContainer._dataContainer.select('#bar-'+taskId);
		var rect = bar.select('rect.estimated');
		if (rect.empty()) {
			rect = bar.select('rect.project');
		}
		var rectBox = rect.node().getBBox();

		if (rectBox.x + rectBox.width + panningX < 0) {
			panChart(-panningX - rectBox.x + 10);
		} else if (rectBox.x + panningX > chart._chartWidth) {
			panChart(-panningX - rectBox.x + 10);
		}
	}

	function hasAxis(serie) {
		return serie.type == "bar" || serie.type == "line" || serie.type == "area";
	}

	function normalizeSerie(serie) {
		chart.dataSource.data.forEach(function(d) {
			var total = d3.sum(serie.fields, function(field) {
				return Math.max(0, +d[field]);
			});
			serie.fields.forEach(function(field) {
				d[field+"_NORMALIZED"] = Math.max(0, +d[field]*100/total);
			});
		});

		serie.fields = serie.fields.map(function(field) {
			return field + "_NORMALIZED";
		});
	}

	function initSeries() {
		var barSeries = false;
		var lineAreaSeries = false;
		var circularSeries = false; // pie, donut, gauge
		var stackedFloatingSeries = false;

		chart.series.forEach(function(serie, i) {
			serie = chart.series[i] = jQuery.extend(true, {}, defaultSerieOptions, defaultSeriesOptions[serie.type], serie);
			if (isUndefinedOrNull(serie._serieContainer)) {
				serie._serieContainer = svgContainer.append("g")
					.attr("id","serie-"+i)
					.attr("class", "serie-container");

				if (hasAxis(serie)) {
					serie._serieContainer.append("g").attr("class","grid");
				}

				serie._serieContainer._dataContainer = serie._serieContainer.append("g")
					.attr("clip-path", "url(#clip-path-" + chart.id + ")")
					.append("g")
					.attr("class","data-container");

				if (hasAxis(serie)) {
					serie._serieContainer.append("g").attr("class","axis y-axis");
				}
			}

			barSeries = barSeries || serie.type == "bar";
			lineAreaSeries = lineAreaSeries || serie.type == "line" || serie.type == "area";
			circularSeries = circularSeries || serie.type == "pie" || serie.type == "donut" || serie.type == "gauge" || serie.type == "progress";
			stackedFloatingSeries = stackedFloatingSeries || (serie.stacked && serie.floating);

			if (serie.colors && typeof serie.colors == "string") {
				serie.colors = serie.colors.split("_");
			}
			if (serie.fields && typeof serie.fields == "string") {
				serie.fields = serie.fields.split("_");
			}
			if (serie.labels && typeof serie.labels == "string") {
				serie.labels = serie.labels.split("_");
			}

			if (hasAxis(serie)) {
				var dataMaxValue;
				var dataMinValue;

				if (chart.dataSource.data.length === 0) {
					dataMaxValue = 0;
					dataMinValue = 0;
				} else {
					var fieldsMaxValue = {};
					var fieldsMinValue = {};
					if (serie.stacked) {

						if (serie.normalized) {
							normalizeSerie(serie);
						}

						dataMaxValue = d3.max(chart.dataSource.data, function(d) {
							var stackedValue = 0;
							serie.fields.forEach(function(field) {
								if (d[field] >= 0) {
									stackedValue += d[field];
									d["_stackedValue_"+field] = stackedValue;

									if (serie.floating) {
										fieldsMaxValue[field] = Math.max(fieldsMaxValue[field] || Number.MIN_SAFE_INTEGER, d[field]);
									}
								}
							});
							d._stackedMaxValue = stackedValue;
							return stackedValue;
						});
						dataMinValue = d3.min(chart.dataSource.data, function(d) {
							var stackedValue = 0;
							serie.fields.forEach(function(field) {
								if (d[field] < 0) {
									stackedValue += d[field];
									d["_stackedValue_"+field] = stackedValue;

									if (serie.floating) {
										fieldsMinValue[field] = Math.min(fieldsMinValue[field] || Number.MAX_SAFE_INTEGER, d[field]);
									}
								}
							});
							d._stackedMinValue = stackedValue;
							return stackedValue;
						});

						if (serie.floating) {
							serie._fieldsMaxValue = fieldsMaxValue;
							serie._fieldsMinValue = fieldsMinValue;
						}
					} else {
						dataMaxValue = Math.max(0,d3.max(chart.dataSource.data, function(d) {
							return d3.max(serie.fields, function(f) { return d[f]; });
						}));
						dataMinValue = Math.min(0, d3.min(chart.dataSource.data, function(d) {
							return d3.min(serie.fields, function(f) { return d[f]; });
						}));
					}
				}

				serie._minValue = serie.yAxis.range.min || dataMinValue;
				serie._maxValue = serie.yAxis.range.max || dataMaxValue;

				if (!isUndefinedOrNull(serie.yAxis.linkedSerie)) {
					var linkedSerie = chart.series[serie.yAxis.linkedSerie];
					linkedSerie._minValue = linkedSerie.yAxis.range.min || Math.min(dataMinValue, linkedSerie._minValue);
					linkedSerie._maxValue = linkedSerie.yAxis.range.max || Math.max(dataMaxValue, linkedSerie._maxValue);
				}

				if (chart.invertedAxes) {
					if (i > 0) {
						serie.linkedSerie = 0;
					}
				}
			}

			if (serie.type == "bar" && serie.centeredBars) {
				if (!chart.invertedAxes) {
					serie.yAxis.visible = false;
				} else {
					chart.xAxis.visible = false;
				}
			}

			if (serie.type == "pie" || serie.type == "donut") {
				serie.pieData = [];
				if (serie.pivoted && chart.dataSource.data.length > 0) {
					serie.pivotedFields.forEach(function(field, i) {
						serie.pieData.push({
							label: serie.pivotedLabels[i],
							value: chart.dataSource.data[0][field] || 0
						});
					});

					serie.fieldValue = "value";
					serie.fieldLabel = "label";
				} else {
					serie.pieData = chart.dataSource.data;
				}

				serie._sumData = d3.sum(serie.pieData, function(d) {
					if (isUndefinedOrNull(d[serie.fieldValue])) {
						d[serie.fieldValue] = 0;
					}
					return d[serie.fieldValue];
				});

				if (serie.sort !== false) {
					serie.pieData.sort(function(a, b) {
						var field = serie.sort == "label" ? serie.fieldLabel : serie.fieldValue;
						return b[field] - a[field];
					});
				}

				if (serie.pieData.length > serie.maxSlices) {
					var otherSlice = {};
					otherSlice[serie.fieldLabel] = serie.maxSlicesText;
					otherSlice[serie.fieldValue] = 0;
					otherSlice.originalSlices = [];
					for (var slice = serie.maxSlices - 1; slice < serie.pieData.length; slice++) {
						otherSlice[serie.fieldValue] += serie.pieData[slice][serie.fieldValue];
						otherSlice.originalSlices.push(serie.pieData[slice]);
					}
					serie.pieData = serie.pieData.slice(0, serie.maxSlices - 1);
					serie.pieData.push(otherSlice);
				}
			}

			if (serie.type == "gauge") {
				if (chart.legend) {
					chart.legend.visible = serie.gaugeShowLegend;
				}

				if (serie.rangeType == "%" && isUndefinedOrNull(serie.maxValue)) {
					throw new Error("Gauge with percentual ranges must have a maxValue set.");
				}

				serie._gaugeData = [];
				var rangeValue;
				for (var idx = 1; idx < serie.ranges.length; idx++) {
					rangeValue = serie.ranges[idx] - serie.ranges[idx - 1];
					if (serie.rangeType == "%") {
						rangeValue = rangeValue * serie.maxValue / 100;
					}
					serie._gaugeData.push({
						rangeSize: rangeValue,
						rangeMinValue: serie.ranges[idx - 1],
						rangeMaxValue: serie.ranges[idx]
					});
				}

				serie._minValue = serie.ranges[0];
				serie._maxValue = serie.ranges[serie.ranges.length - 1];
				if (serie.rangeType == "%") {
					serie._minValue = serie._minValue * serie.maxValue / 100;
					serie._maxValue = serie._maxValue * serie.maxValue / 100;
				}

				if (serie.maxValue && serie._maxValue && serie._maxValue !== serie.maxValue) {
					serie._gaugeData.push({
						rangeSize: serie.maxValue - serie._maxValue,
						rangeMinValue: serie._maxValue,
						rangeMaxValue: serie.maxValue
					});
					serie._maxValue = serie.maxValue;
				}

				if (serie._gaugeData.length === 0) {
					chart.legend.visible = false;
				}
			}

			if (serie.type == "progress") {
				chart.showTooltip = false;
				chart.legend.visible = false;
				if (serie.maxValue < serie.value) {
					serie.maxValue = serie.value;
				}
			}

			if (serie.type == "gantt") {
				chart.legend.visible = false;

				var tomorrow = moment().startOf("day").add(1, "d");
				chart.dataSource.data.forEach(function(row) {
					if (isUndefinedOrNull(row[serie.ganttFields.taskRealBegin]) || moment(row[serie.ganttFields.taskRealBegin], serie.dateFormat) >= tomorrow) {
						row[serie.ganttFields.taskRealBegin] = null;
						row[serie.ganttFields.taskRealEnd] = null;
						row[serie.ganttFields.taskCompleted] = 0;
					}
					if (!isUndefinedOrNull(row[serie.ganttFields.taskRealBegin]) &&
						(isUndefinedOrNull(row[serie.ganttFields.taskRealEnd]) || moment(row[serie.ganttFields.taskRealEnd], serie.dateFormat) >= tomorrow)) {
						row[serie.ganttFields.taskRealEnd] = moment().format(serie.dateFormat);
						row._noRealEndDate = true;
					}
				});

				serie._maxDate = moment(d3.max(chart.dataSource.data, function(d) {
					var taskEstimatedEnd = moment(d[serie.ganttFields.taskEstimatedEnd], serie.dateFormat);
					var taskRealEnd = moment(d[serie.ganttFields.taskRealEnd], serie.dateFormat);
					if (!taskRealEnd.isValid()) {
						taskRealEnd = taskEstimatedEnd;
					}
					return moment.max(taskEstimatedEnd, taskRealEnd)._d;
				})).add(2,'d');
				serie._minDate = moment(d3.min(chart.dataSource.data, function(d) {
					var taskEstimatedBegin = moment(d[serie.ganttFields.taskEstimatedBegin], serie.dateFormat);
					var taskRealBegin = moment(d[serie.ganttFields.taskRealBegin], serie.dateFormat);
					if (!taskRealBegin.isValid()) {
						taskRealBegin = taskEstimatedBegin;
					}
					return moment.min(taskEstimatedBegin, taskRealBegin)._d;
				})).add(-1,'d');

				if (serie.ganttTimeScale != "day") {
					serie._minDate = serie._minDate.startOf(serie.ganttTimeScale).startOf("day");
					serie._maxDate = serie._maxDate.endOf(serie.ganttTimeScale).startOf("day");

					if (serie.ganttTimeScale == "week") {
						serie._maxDate.add(1,"d");
					}
				}
			}
		});

		if (barSeries) {
			chart.xAxis.type = "bands";
		} else {
			chart.xAxis.type = "points";
		}

		if (circularSeries) {
			if (chart.series.length > 1) {
				throw new Error("Pie, donut, gauge and progress series cannot be associated with other series");
			}

			chart.xAxis.visible = false;
		}

		if (stackedFloatingSeries) {
			// stacked floating series cannot show legend
			chart.legend.visible = false;
		}
	}

	function draw() {
		resizeChart();

		drawTitle();
		drawSubTitle();

		preRenderAxes();		// prerender axes to calc axes dimensions

		calcChartDimensions();	// calc chart dimensions considering axes dimensions
		drawAxes();				// draw axes

		checkTitleOverflow();

		drawSeries();
		drawLegend();

		if (chart.events.onFinishDrawing) {
			chart.events.onFinishDrawing();
		}
	}

	function needData() {
		var result = false;
		chart.series.forEach(function(serie, i) {
			result = result || hasAxis(serie) || serie.type == "pie" || serie.type == "donut";
		});

		return result;
	}

	function isGauge() {
		return chart.series.length === 1 && chart.series[0].type == 'gauge';
	}

	function gaugeNoData() {
		var serie = chart.series[0];
		return (!serie.ranges || serie.ranges.length === 0) || (serie.rangeType == "%" && isUndefinedOrNull(serie.maxValue));
	}

	function drawSeries() {
		var drawFunction = {
			bar: drawBarSerie,
			line: drawLineSerie,
			area: drawAreaSerie,
			pie: drawPieDonutSerie,
			donut: drawPieDonutSerie,
			gauge: drawGaugeSerie,
			progress: drawProgressSerie,
			gantt: drawGanttSerie
		};

		svgContainer.select("text.empty-text").remove();

		if ((needData() && chart.dataSource.data.length === 0) || (isGauge() && gaugeNoData())) {
			svgContainer.append("text")
				.attr("class","empty-text")
				.attr("x", chart._chartWidth / 2)
				.attr("y", chart._chartHeight / 2)
				.attr("text-anchor", "middle")
				.attr("font-size", 12)
				.text(chart.messageNoData);

			chart.series.forEach(function(serie) {
				serie._serieContainer._dataContainer.selectAll("*").remove();
			});
		} else {
			chart.series.forEach(function(serie, i) {
				drawFunction[serie.type](serie);
			});
		}
	}

	function calcChartDimensions() {
		chart._chartHeight = svgContainerHeight - chart.padding.top - chart.padding.bottom - titlePadding - axisDimensions.top - axisDimensions.bottom;
		chart._chartWidth = svgContainerWidth - chart.padding.left - chart.padding.right - axisDimensions.left - axisDimensions.right;
		chart._chartTop = chart.padding.top + titlePadding + axisDimensions.top;
		chart._chartLeft = chart.padding.left + axisDimensions.left;

		chart.series.forEach(function(serie, i) {
			serie._serieContainer.attr("transform", "translate("+chart._chartLeft + ", " + chart._chartTop +")");
		});

		if (chart._xAxisContainer && ((chart._chartWidth + axisDimensions.left + axisDimensions.right) < chart._xAxisContainer.select("path.domain").node().getBBox().width)) {
			chart.clippingPath //.transition().duration(_duration)
				.attr("width", chart._chartWidth + 2)
				.attr("height", svgContainerHeight)
				.attr("transform", "translate(-1, " + -titlePadding + ")");
		} else {
			chart.clippingPath //.transition().duration(_duration)
				.attr("width", chart._chartWidth + 20)
				.attr("height", svgContainerHeight)
				.attr("transform", "translate(-10, " + -titlePadding + ")");
		}

		chart.clippingPathAxis //.transition().duration(_duration)
			.attr("width", svgContainerWidth - chart.padding.left - chart.padding.right + 2)
			.attr("height", svgContainerHeight)
			.attr("transform", "translate(-"+(axisDimensions.left+1)+", -"+(axisDimensions.top || 1)+")");

		svgContainer.select("rect.panning-controller")
			.attr("width", chart._chartWidth)
			.attr("height", chart._chartHeight)
			.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");
	}

	function preRenderAxes() {
		var renderOrder = chart.invertedAxes ? ["x", "y"] : ["y", "x"];

		fixTicks(false);

		renderOrder.forEach(function(axis) {
			calcChartDimensions();	// calc chart dimensions without axes dimensions
			if (axis == "y") {
				chart.series.forEach(function(serie, i) {
					if ((serie.yAxis && hasAxis(serie)) || chart.invertedAxes) {
						initSerieYScale(serie);
						if (serie.yAxis.visible !== false) {
							drawSerieYAxis(serie);
							axisDimensions[serie.yAxis.position] = Math.max(axisDimensions[serie.yAxis.position], serie._yAxisContainer.node().getBBox().width);
						}
					}
				});
			} else {
				if (chart.xAxis || chart.invertedAxes) {
					if (chart.series[0].type != "gantt") {
						initChartXScale();

						if (chart.xAxis.visible !== false) {
							drawChartXAxis();
							axisDimensions.bottom = Math.max(axisDimensions.bottom, chart._xAxisContainer.node().getBBox().height);
						}
					} else {
						initChartGanttScale();

						if (chart.xAxis.visible !== false) {
							drawChartGanttAxis();
							axisDimensions.top = Math.max(axisDimensions.top, chart._xAxisContainer.node().getBBox().height);
						}
					}
				}
			}
		});
	}

	function drawAxes() {
		var renderOrder = chart.invertedAxes ? ["x", "y"] : ["y", "x"];
		renderOrder.forEach(function(axis) {
			if (axis == "y") {
				chart.series.forEach(function(serie, i) {
					if (serie.yAxis && hasAxis(serie)) {
						initSerieYScale(serie);
						if (serie.yAxis.visible !== false || !isUndefinedOrNull(serie.yAxis.linkedSerie)) {
							drawSerieYAxis(serie);
						}
					}
				});
			} else {
				if (chart.xAxis) {
					if (chart.series[0].type != "gantt") {
						initChartXScale();
						if (chart.xAxis.visible !== false) {
							drawChartXAxis();
						}
					} else {
						initChartGanttScale();
						if (chart.xAxis.visible !== false) {
							drawChartGanttAxis();
						}
					}
				}
			}
		});

		fixTicks(chart.series[0].type != "gantt");
	}

	function fixTicks(fix) {
		// fix axis ticks on the bottom
		if (chart.xAxis && chart.xAxis.visible !== false) {
			var xTransform, yTransform;
			if (!chart.invertedAxes && chart._xAxisContainer) {
				yTransform = fix ? chart._chartHeight - chart._startPosition : 0;
				chart._xAxisContainer.select("g.ticks").selectAll("g.tick").each(function (){
					var tick = d3.select(this);
					xTransform = +tick.attr("transform").split("(")[1].split(",")[0];
					if (tick.attr("original-transform-x")) {
						xTransform = +tick.attr("original-transform-x");
					}

					tick.attr("transform", "translate("+(xTransform+panningX)+","+yTransform+")");
				});
			} else if (chart.series.length > 0 && chart.series[0]._yAxisContainer) {
				xTransform = fix ? -chart._startPosition : 0;
				chart.series[0]._yAxisContainer.select("g.ticks").selectAll("g.tick").each(function (){
					var tick = d3.select(this);
					yTransform = +tick.attr("transform").split(",")[1].split(")")[0];

					tick.attr("transform", "translate("+xTransform+","+yTransform+")");
				});
			}
		}
	}

	function initChartXScale() {
		var bandWidth;

		if (!chart.invertedAxes) {
			if (chart.xAxis.type == "bands") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					chart._xScale = d3.scale.ordinal()
						.rangeRoundBands([0, chart._chartWidth], chart.xAxis.bandDimensions.gap, 0);
				} else {
					if (chart.dataSource.data.length !== 0) {
						bandWidth = Math.max(chart._chartWidth / chart.dataSource.data.length, chart.xAxis.bandDimensions.minWidth);

						chart._xScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandWidth * chart.dataSource.data.length], chart.xAxis.bandDimensions.gap, 0);
					} else {
						bandWidth = chart._chartWidth;

						chart._xScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandWidth], chart.xAxis.bandDimensions.gap, 0);
					}

				}
			} else if (chart.xAxis.type == "points") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					chart._xScale = d3.scale.ordinal()
						.rangeRoundPoints([0, chart._chartWidth], chart.xAxis.bandDimensions.gap);
				} else {
					if (chart.dataSource.data.length > 1) {
						bandWidth = Math.max(chart._chartWidth / (chart.dataSource.data.length - 1), chart.xAxis.bandDimensions.minWidth);

						chart._xScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandWidth * (chart.dataSource.data.length - 1)], chart.xAxis.bandDimensions.gap);
					} else {
						bandWidth = chart._chartWidth;

						chart._xScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandWidth], chart.xAxis.bandDimensions.gap);
					}
				}
			}

			chart._xScale.domain(chart.dataSource.data.map(function(d) { return d[chart.xAxis.field]; }));

			chart._bandLeft = function(data) {
				return chart._xScale(data[chart.xAxis.field]);
			};
			chart._bandWidth = function() {
				return chart._xScale.rangeBand();
			};
			chart._pointLeft = function(data) {
				return chart._bandLeft(data) + chart._bandWidth() / 2;
			};

			// init chart start position to draw elements (bars, lines)
			chart._startPosition = chart.series[0]._yScale ? chart.series[0]._yScale(chart.series[0]._startValue) : chart._chartHeight;
		} else {
			var _minValue, _maxValue;

			_minValue = d3.min(chart.series, function(serie) { return serie._minValue; });
			_maxValue = d3.max(chart.series, function(serie) { return serie._maxValue; });

			chart._xScale = d3.scale.linear()
				.range([0, chart._chartWidth])
				.domain([_minValue, _maxValue]);

			chart._startValue = _minValue < 0 ? 0 : _minValue;

			// init chart start position to draw elements (bars, lines)
			chart._startPosition = chart._xScale(chart._startValue);
		}
	}

	function xAxisOverflow() {
		var textSpace;
		var textWidths = [];

		if (chart.xAxis.type == "bands") {
			textSpace = chart._bandWidth();
		} else {
			if (chart.dataSource.data.length > 1) {
				textSpace = chart._pointLeft(chart.dataSource.data[1]) - chart._pointLeft(chart.dataSource.data[0]);
			} else {
				textSpace = chart._chartWidth;
			}
		}

		chart._xAxisContainer.selectAll("text").each(function() { textWidths.push(this.getBBox().width); });

		return d3.max(textWidths) > textSpace;
	}

	function wrapTicks(text) {
		var maxWidth;

		if (!isUndefinedOrNull(chart.xAxis.ticks.maxWidth) && !isUndefinedOrNull(chart.xAxis.ticks.maxLines)) {
			maxWidth = chart.xAxis.ticks.maxWidth;

			text.each(function() {
				var text = d3.select(this),
					words = text.text().split(/\s+/).reverse(),
					word, splitWord,
					line = [],
					lineNumber = 0,
					lineHeight = 1.1,  //ems
					y = text.attr("y"),
					dy = parseFloat(text.attr("dy")),
					tspan = text.text(null).append("tspan")
						.attr("x", 0)
						.attr("y", y)
						.attr("dy", dy + "em")
						.attr("dx",text.attr("dx"));

				word = words.pop();
				while (!isUndefinedOrNull(word)) {
					line.push(word);
					tspan.text(line.join(" "));

					if (tspan.node().getComputedTextLength() > maxWidth) {
						if (lineNumber === chart.xAxis.ticks.maxLines - 1) {
							break;
						}

						line.pop();

						if (line.length > 0) {
							tspan.text(line.join(" "));
							line = [word];
							tspan = text.append("tspan")
								.attr("x", 0)
								.attr("y", y)
								.attr("dy", ++lineNumber * lineHeight + dy + "em")
								.attr("dx",text.attr("dx"))
								.text(word);
						} else {
							if (words.length > 0) {
								splitWord = "";
								while (tspan.node().getComputedTextLength() > maxWidth) {
									splitWord = word.substr(-1) + splitWord;
									word = word.slice(0, -1);
									tspan.text(word);
								}
								word = splitWord;
								line = [word];

								tspan = text.append("tspan")
									.attr("x", 0)
									.attr("y", y)
									.attr("dy", ++lineNumber * lineHeight + dy + "em")
									.attr("dx",text.attr("dx"))
									.text(word);
							}
						}
					}

					word = words.pop();
				}

				// adjust last line
				word = tspan.text();
				while (word.length > 0 && tspan.node().getComputedTextLength() > maxWidth) {
					word = word.slice(0, -1);
					tspan.text(word+"...");
				}
			});
		}
	}

	function drawXAxis(xAxis, xAxisYPosition, mouseover) {
		if (isUndefined(mouseover)) {
			mouseover = true;
		}

		var axisSelection;
		chart._xAxisContainer = svgContainer.select("g.x-axis");
		if (chart._xAxisContainer.empty()) {
			chart._xAxisContainer = svgContainer.append("g")
				.style('opacity', 0)
				.attr("class", "axis x-axis")
				.attr("transform", "translate(0, " + xAxisYPosition + ")");

			axisSelection = chart._xAxisContainer.append("g")
				.attr("class","ticks-container")
				.attr("clip-path", "url(#clip-path-axis-" + chart.id + ")")
				.append("g")
				.attr("class","ticks");
		} else {
			var xAxisSelection = chart._xAxisContainer;
			chart._xAxisInitialized = true;

			axisSelection = xAxisSelection
				.style('opacity', 1)
				.attr("transform", "translate(" + chart._chartLeft + ", " + xAxisYPosition + ")");
		}

		chart._xAxisContainer.select("g.ticks").call(xAxis);

		chart._xAxisContainer.selectAll("path, line")
			.style("fill","none")
			.style("stroke","#999")
			.style("shape-rendering", "crispEdges");

		if (chart.invertedAxes) {
			chart._xAxisContainer.selectAll("path.domain")
				.style("display", "none");
		}

		chart._xAxisContainer.selectAll("text")
			.style("font-size", "10px")
			.call(wrapTicks);

		if (!chart.invertedAxes && mouseover) {
			chart._xAxisContainer.selectAll("text")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip);

			setupShowTooltipOnLongTap(chart._xAxisContainer.selectAll("text"));
		}
	}

	function getGanttTimeScalesWidthConstraints(serie) {
		var timeScalesWidth = {
			day: 55,
			week: 60,
			month: 50,
			quarter: 50,
			year: 30
		};

		return timeScalesWidth[serie.ganttTimeScale];
	}

	function getGanttTimeScalesFormat(serie) {
		var timeScalesFormat = {
			day: 'DD/MM',
			week: 'DD/MM/YYYY',
			month: 'MMM/YYYY',
			quarter: 'Qo[Q]/YYYY',
			year: 'YYYY'
		};

		return timeScalesFormat[serie.ganttTimeScale];
	}

	function getGanttTimeScalesCount(serie) {
		var timeScalesString = {
			day: 'days',
			week: 'weeks',
			month: 'months',
			quarter: 'quarters',
			year: 'years'
		};
		return serie._maxDate.diff(serie._minDate, timeScalesString[serie.ganttTimeScale])+1;
	}

	function initChartGanttScale() {
		var bandWidthMin = 15;
		var timeScalesCount = getGanttTimeScalesCount(chart.series[0]);
		var bandWidth = Math.max(chart._chartWidth / timeScalesCount, bandWidthMin);

		chart._xScale = d3.time.scale()
			.range([0, bandWidth * timeScalesCount])
			.domain([chart.series[0]._minDate._d, chart.series[0]._maxDate._d]);

		chart._bandLeft = function(data, field, delta) {
			delta = delta || 0;
			return chart._xScale(moment(data[field], chart.series[0].dateFormat).add(delta, 'days')._d);
		};
		chart._bandWidth = function() {
			return bandWidth;
		};
		chart._pointLeft = function(data, field) {
			return chart._bandLeft(data, field) + chart._bandWidth() / 2;
		};

		chart._startPosition = 0;
	}

	function drawGanttGrid() {
		var xGrid = d3.svg.axis()
			.scale(chart._xScale)
			.orient("bottom")
			.tickSize(chart._chartHeight, 0)
			.tickFormat("")
			.ticks(getGanttTimeScalesCount(chart.series[0]));
		var xAxisYPosition = chart._startPosition + chart._chartTop;

		if (isUndefinedOrNull(chart._xGridContainer)) {
			chart._xGridContainer = svgContainer.select("g.x-grid");

			if (chart._xGridContainer.empty()) {
				chart._xGridContainer = svgContainer.insert("g", "g.serie-container")
					.attr("class","x-grid");

				chart._xGridContainer._dataContainer = chart._xGridContainer.append("g")
					.attr("clip-path", "url(#clip-path-" + chart.id + ")")
					.append("g");
			}

			chart._xGridContainer.style('opacity', 0)
				.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");
		} else {
			var gridSelection;
			if (chart._xGridInitialized) {
				gridSelection = chart._xGridContainer.transition().duration(_duration);
			} else {
				gridSelection = chart._xGridContainer;
				chart._xGridInitialized = true;
			}
			gridSelection
				.style("opacity", 1)
				.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");
		}

		chart._xGridContainer._dataContainer.call(xGrid);

		chart._xGridContainer._dataContainer.selectAll(".tick")
			.style("stroke", "#ccc")
			.style("stroke-opacity", 0.7)
			.style("shape-rendering", "crispEdges");

		drawWeekendDays(xGrid);

		chart._xGridContainer.selectAll("path")
			.style("display", "none");
	}

	function drawWeekendDays(xGrid) {
		chart._xGridContainer._dataContainer.selectAll("g.tick line.saturday").remove();
		chart._xGridContainer._dataContainer.selectAll("g.tick line").attr("stroke-width", null).attr("stroke", null).attr("x1", null).attr("x2", 0);

		var todayLine = chart._xGridContainer._dataContainer.selectAll("line.today");
		if (todayLine.empty()) {
			todayLine = chart._xGridContainer._dataContainer.append("line")
				.attr("class", "today")
				.attr("stroke-width", 1)
				.attr("stroke", "darkred");
		}

		todayLine.transition().duration(_duration)
			.attr("x1", chart._xScale(moment().startOf("day")._d))
			.attr("x2", chart._xScale(moment().startOf("day")._d))
			.attr("y1", 0)
			.attr("y2", chart._chartHeight);

		if (chart.series[0].ganttTimeScale == "day" || chart.series[0].ganttTimeScale == "week") {
			var tickCount = xGrid.ticks()[0];
			var tickArrValues = chart._xScale.ticks(tickCount);
			var tickDistance = tickCount > 1 ? chart._xScale(tickArrValues[1]) - chart._xScale(tickArrValues[0]) : chart._chartWidth;

			chart._xGridContainer._dataContainer.selectAll(".tick").each(function(d, i) {
				var tick = d3.select(this);
				var line = tick.select("line");

				if (d) {
					if (chart.series[0].ganttTimeScale == "day") {
						if (d.getDay() === 0 || d.getDay() === 6) {
							line.attr("stroke-width", tickDistance-1)
								.attr("x1", tickDistance/2)
								.attr("x2", tickDistance/2);
						}
					} else if (chart.series[0].ganttTimeScale == "week") {
						var dayDistance = tickDistance/7;
						line.attr("stroke-width", dayDistance-1)
							.attr("x1", dayDistance/2)
							.attr("x2", dayDistance/2);

						var saturdayLine = tick.select("line.saturday");
						if (saturdayLine.empty()) {
							saturdayLine = tick.append("line")
								.attr("class", "saturday");
						}

						saturdayLine.attr("y2", line.attr("y2"))
							.attr("stroke-width", line.attr("stroke-width"))
							.attr("x1", dayDistance*13/2)
							.attr("x2", dayDistance*13/2);
					}
				}
			});
		}
	}

	function drawChartGanttAxis() {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			var xAxis = d3.svg.axis()
				.scale(chart._xScale)
				.orient("top");
			var xAxisYPosition = chart._startPosition + chart._chartTop;

			xAxis.tickFormat(function(value) {
				var format = chart.series[0].ganttTimeScale == "day" && value.getDate() === 1 ? "MMMM" : getGanttTimeScalesFormat(chart.series[0]);
				return moment(value).format(format).toUpperCase();
			});

			xAxis.ticks(getGanttTimeScalesCount(chart.series[0]));

			drawXAxis(xAxis, xAxisYPosition, false);

			// rotation must be between [0..90] degrees
			chart.xAxis.ticks.rotation = chart.xAxis.ticks.rotation > 90 ? chart.xAxis.ticks.rotation % 90 : chart.xAxis.ticks.rotation;
			if (chart.xAxis.ticks.rotation || chart._bandWidth() < getGanttTimeScalesWidthConstraints(chart.series[0])) {
				chart._xAxisContainer.selectAll('text')
					.attr("transform", "rotate(-" + (chart.xAxis.ticks.rotation || 60) + ")")
					.attr("dx", "0.8em")
					.attr("dy", "1em")
					.style("text-anchor", "start")
					.selectAll("tspan")
					.attr("y", 0)
					.attr("x", -2);
			} else {
				chart._xAxisContainer.selectAll('text')
					.attr("transform", null)
					.attr("dx", null)
					.style("text-anchor", "start");
			}

			drawGanttGrid();
			setupPanning(true);
		}
	}

	function addFloatingChartAxis(serie) {
		var _scale = d3.scale.ordinal()
			.rangeRoundBands([0, chart._chartWidth])
			.domain(serie.labels);

		var xAxis = d3.svg.axis()
			.scale(_scale)
			.orient("bottom");

		var xAxisYPosition = chart._chartHeight + chart._chartTop;
		drawXAxis(xAxis, xAxisYPosition);

		chart._xAxisContainer.selectAll("path.domain")
			.style("display", "block");

		chart._xAxisContainer.selectAll("text")
			.style("text-anchor", "start");

		chart._xAxisContainer.selectAll("g.tick").each(function(d, i) {
			var tick = d3.select(this);
			tick.classed("tick-"+i, true);
		});
	}

	function drawChartXAxis() {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			if (chart.series[0].stacked && chart.series[0].floating && chart.invertedAxes) {
				addFloatingChartAxis(chart.series[0]);
			} else {
				var xAxis = d3.svg.axis()
					.scale(chart._xScale)
					.orient("bottom");

				var xAxisYPosition;
				if (!chart.invertedAxes) {
					xAxisYPosition = chart._startPosition + chart._chartTop;

					if (chart.xAxis.format) {
						if (isFunction(serie.xAxis.format)) {
							xAxis.tickFormat(chart.xAxis.format);
						} else {
							xAxis.tickFormat(function(value) { return formatValue(value, chart.xAxis.format); });
						}
					}
				} else {
					xAxisYPosition = chart._chartHeight + chart._chartTop;

					if (chart.series[0].yAxis.format) {
						if (isFunction(chart.series[0].yAxis.format)) {
							xAxis.tickFormat(chart.series[0].yAxis.format);
						} else {
							xAxis.tickFormat(function(value) { return formatValue(value, chart.series[0].yAxis.format); });
						}
					}
				}

				xAxis.ticks(Math.max(chart._chartWidth/50, 2));
				if (chart.invertedAxes && chart.series[0].yAxis.formatName == "number" && (chart._xScale.domain()[1] - chart._xScale.domain()[0] < xAxis.ticks()[0])) {
					xAxis.tickValues(d3.range(chart._xScale.domain()[0], chart._xScale.domain()[1]+1, 1));
				}

				drawXAxis(xAxis, xAxisYPosition);

				// rotation must be between [0..90] degrees
				chart.xAxis.ticks.rotation = chart.xAxis.ticks.rotation > 90 ? chart.xAxis.ticks.rotation % 90 : chart.xAxis.ticks.rotation;
				if (chart.xAxis.ticks.rotation || chart.invertedAxes || xAxisOverflow()) {
					chart._xAxisContainer.selectAll('text')
						.attr("transform", "rotate(-" + (chart.xAxis.ticks.rotation || 60) + ")")
						.attr("dx", "-.8em")
						.attr("dy", ".15em")
						.style("text-anchor", "end")
						.selectAll("tspan")
						.attr("y", 0)
						.attr("x", -2);
				} else {
					chart._xAxisContainer.selectAll('text')
						.attr("transform", null)
						.attr("dx", null)
						.attr("dy", ".71em")
						.style("text-anchor", "middle");
				}

				if (chart.xAxis.label) {
					var label = chart._xAxisContainer.select("text.label");
					if (label.empty()) {
						label = chart._xAxisContainer.append("text")
							.style('opacity', 0)
							.attr("class","label")
							.attr("text-anchor","middle")
							.attr("x",chart._chartWidth/2)
							.attr("y",(chart._xAxisContainer.select("g.ticks").node().getBBox().height + 20))
							.text(chart.xAxis.label);
					} else {
						var xLabelSelection;
						if (chart._xLabelInitialized) {
							xLabelSelection = label.transition().duration(_duration);
						} else {
							xLabelSelection = label;
							chart._xLabelInitialized = true;
						}
						xLabelSelection
							.style('opacity', 1)
							.attr("x",chart._serieWidth/2)
							.attr("y",(chart._xAxisContainer.select("g.ticks").node().getBBox().height + 20));
					}
				}

				if (chart.invertedAxes) {
					if (chart.series[0].yAxis.showGridLines) {
						var xGrid = d3.svg.axis()
							.scale(chart._xScale)
							.orient("bottom")
							.tickSize(chart._chartHeight, 0)
							.tickFormat("")
							.ticks(Math.max(chart._chartWidth/50, 2));

						if (isUndefinedOrNull(chart._xGridContainer)) {
							chart._xGridContainer = svgContainer.insert("g", "g.serie-container")
								.style('opacity', 0)
								.attr("class", "x-grid")
								.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");

							chart._xGridContainer._dataContainer = chart._xGridContainer.append("g")
								.attr("clip-path", "url(#clip-path-" + chart.id + ")")
								.append("g");
						} else {
							var gridSelection;
							if (chart._xGridInitialized) {
								gridSelection = chart._xGridContainer.transition().duration(_duration);
							} else {
								gridSelection = chart._xGridContainer;
								chart._xGridInitialized = true;
							}
							gridSelection
								.style("opacity", 1)
								.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");
						}

						chart._xGridContainer._dataContainer.call(xGrid);

						chart._xGridContainer._dataContainer.selectAll(".tick")
							.style("stroke", "#ccc")
							.style("stroke-opacity", 0.7)
							.style("shape-rendering", "crispEdges");

						chart._xGridContainer._dataContainer.selectAll("path")
							.style("display", "none");
					}
				}
			}

			setupPanning(true);
		}
	}

	function mouseDownPanning() {
		if ($(d3.event.target).parents('g.legend').length === 0) {
			chartPanning = chartPanning || {};
			if (d3.event.type == "mousedown") {
				chartPanning.x = d3.event.clientX;
				chartPanning.y = d3.event.clientY;
			} else {
				chartPanning.x = d3.event.touches[0].clientX;
				chartPanning.y = d3.event.touches[0].clientY;
			}
		}
	}

	function mouseMovePanning() {
		var _x, deltaX;
		var _y, deltaY;
		if (chartPanning && !legendDragging) {
			if (d3.event.type == "mousemove") {
				_x = d3.event.clientX;
				_y = d3.event.clientY;
			} else {
				_x = d3.event.touches[0].clientX;
				_y = d3.event.touches[0].clientY;
			}
			deltaX = _x - chartPanning.x;
			deltaY = _y - chartPanning.y;
			if (Math.abs(deltaX) > 0 || Math.abs(deltaY) > 0) {
				panChart(deltaX);
				panChartY(deltaY);

				chartPanning = chartPanning || {};
				chartPanning.x = _x;
				chartPanning.y = _y;
				chartPanning.panned = true;
			}
		}
	}

	function mouseUpPanning() {
		if (chartPanning.panned) {
			d3.event.stopPropagation();
		}
		chartPanning = false;
	}

	function windowMouseMovePanning(e) {
		if (chartPanning) {
			var panningClientRect = svgContainer.select(".panning-controller")[0][0].getBoundingClientRect();
			if (e.clientX < panningClientRect.x ||
				e.clientY < panningClientRect.y ||
				e.clientX > panningClientRect.x + panningClientRect.width ||
				e.clientY > panningClientRect.y + panningClientRect.height) {

				if (chartPanning.panned) {
					e.stopPropagation();
				}
				chartPanning = false;
			}
		}
	}

	function mouseWheelPanning() {
		var deltaY;
		if (!legendDragging) {
			deltaY = d3.event.wheelDelta/2;

			if (Math.abs(deltaY) > 0) {
				panChartY(deltaY);

				chartPanning = chartPanning || {};
				chartPanning.panned = true;
			}
		}
	}

	function setupPanning(enabled) {
		var enabledX, enabledY;

		if (chart.scrollable) {
			if (chart._xAxisContainer) {
				enabledX = enabled && chart._chartWidth < chart._xAxisContainer.select("path.domain").node().getBBox().width;
			}

			if (chart.series[0]._serieContainer && chart.scrollable) {
				enabledY = enabled && chart._chartHeight < chart.series[0]._serieContainer.node().getBBox().height;
			}
		}

		if (enabledX || enabledY) {
			if (!svgContainer._panningEvents) {
				svgContainer._panningEvents = true;
				svgContainer.on("mousedown.panning", mouseDownPanning);
				svgContainer.on("mousemove.panning", mouseMovePanning);
				svgContainer.on("mouseup.panning", mouseUpPanning);
				svgContainer.on("touchstart.panning", mouseDownPanning);
				svgContainer.on("touchmove.panning", mouseMovePanning);
				svgContainer.on("touchend.panning", mouseUpPanning);
				svgContainer.on("wheel.panning mousewheel.panning DOMMouseScroll.panning", mouseWheelPanning);
				$(window).off("mousemove.panning touchmove.panning");
				$(window).on("mousemove.panning touchmove.panning", windowMouseMovePanning);
			}
		} else {
			svgContainer._panningEvents = false;
			svgContainer.on("mousedown.panning", null);
			svgContainer.on("mousemove.panning", null);
			svgContainer.on("mouseup.panning", null);
			svgContainer.on("mouseout.panning", null);
			svgContainer.on("touchstart.panning", null);
			svgContainer.on("touchmove.panning", null);
			svgContainer.on("touchend.panning", null);
			svgContainer.on("wheel.panning mousewheel.panning DOMMouseScroll.panning", null);
			$(window).off("mousemove.panning touchmove.panning");
		}

		if (enabledX) {
			drawScrollBar();
		} else {
			hideScrollBar();
		}

		if (enabledY) {
			drawScrollBarY();
		} else {
			hideScrollBarY();
		}
	}

	function hideScrollBar() {
		svgContainer.select("g.scroll-bar").style("display", "none");
	}

	function hideScrollBarY() {
		svgContainer.select("g.scroll-bar-y").style("display", "none");
	}

	function resizeScrollBar(bar, increase) {
		bar = d3.select(bar);
		var size = increase ? 10 : 3;
		var delta = increase ? 22 : 15;

		var transitionTime = 300;

		if (bar.classed('scroll-bar')) {
			bar.transition().duration(transitionTime).attr("transform", "translate(0, " + (svgContainerHeight - delta) + ")");
			bar.select('rect.scroll-bar-track').transition().duration(transitionTime).attr('height', size);
			bar.select('rect.scroll-bar-thumb').transition().duration(transitionTime).attr('height', size);
		} else {
			bar.transition().duration(transitionTime).attr("transform", "translate(" + (svgContainerWidth - delta) + ", 0)");
			bar.select('rect.scroll-bar-track').transition().duration(transitionTime).attr('width', size);
			bar.select('rect.scroll-bar-thumb').transition().duration(transitionTime).attr('width', size);
		}
	}

	function drawScrollBar() {
		var scrollbarContainer = svgContainer.select("g.scroll-bar");

		if (scrollbarContainer.empty()) {
			scrollbarContainer = svgContainer.append("g")
				.attr("class", "scroll-bar")
				.attr("transform", "translate(0, " + (svgContainerHeight - 15) + ")")
				.on("mouseover", function() { resizeScrollBar(this, true); })
				.on("mouseout", function() { resizeScrollBar(this, false); });

			scrollbarContainer.append("rect")
				.attr("class", "scroll-bar-box")
				.attr("fill", "transparent")
				.attr("height", 15);

			scrollbarContainer.append("rect")
				.attr("fill", "#eee")
				.attr("class", "scroll-bar-track")
				.attr("height", 3);

			var thumb = scrollbarContainer.append("rect")
				.attr("fill", "#aaa")
				.attr("class", "scroll-bar-thumb")
				.attr("height", 3);

			var thumbDragging = false;
			var drag = d3.behavior.drag()
				.origin(Object)
				.on('dragstart', function() {
					if (!thumbDragging) {
						thumbDragging = true;
					}
				})
				.on('drag', function() {
					if (thumbDragging) {
						panChart(-d3.event.dx);
					}
				})
				.on('dragend', function() {
					if (thumbDragging) {
						thumbDragging = false;
					}
				});

			thumb.call(drag);
		}

		scrollbarContainer.style("display", "block")
			.attr("transform", "translate(0, " + (svgContainerHeight - 15) + ")");

		scrollbarContainer.select("rect.scroll-bar-box")
			.attr("width", chart._chartWidth);

		scrollbarContainer.select("rect.scroll-bar-track")
			.attr("width", chart._chartWidth)
			.attr("transform", "translate(" + chart._chartLeft + ", 12)");

		var scrollbarDelta = panningX * chart._chartWidth / chart._xAxisContainer.select("path.domain").node().getBBox().width;
		scrollbarContainer.select("rect.scroll-bar-thumb")
			.attr("width", chart._chartWidth * chart._chartWidth / chart._xAxisContainer.select("path.domain").node().getBBox().width)
			.attr("transform", "translate(" + (chart._chartLeft - scrollbarDelta) + ", 12)");
	}

	function drawScrollBarY() {
		var scrollbarContainer = svgContainer.select("g.scroll-bar-y");

		if (scrollbarContainer.empty()) {
			scrollbarContainer = svgContainer.append("g")
				.attr("class", "scroll-bar-y")
				.attr("transform", "translate(" + (svgContainerWidth - 15) + ", 0)")
				.on("mouseover", function() { resizeScrollBar(this, true); })
				.on("mouseout", function() { resizeScrollBar(this, false); });

			scrollbarContainer.append("rect")
				.attr("class", "scroll-bar-box")
				.attr("fill", "transparent")
				.attr("width", 15);

			scrollbarContainer.append("rect")
				.attr("fill", "#eee")
				.attr("class", "scroll-bar-track")
				.attr("width", 3);

			var thumb = scrollbarContainer.append("rect")
				.attr("fill", "#aaa")
				.attr("class", "scroll-bar-thumb")
				.attr("width", 3);

			var thumbDragging = false;
			var drag = d3.behavior.drag()
				.origin(Object)
				.on('dragstart', function() {
					if (!thumbDragging) {
						thumbDragging = true;
					}
				})
				.on('drag', function() {
					if (thumbDragging) {
						panChartY(-d3.event.dy);
					}
				})
				.on('dragend', function() {
					if (thumbDragging) {
						thumbDragging = false;
					}
				});

			thumb.call(drag);
		}

		scrollbarContainer.style("display", "block")
			.attr("transform", "translate(" + (svgContainerWidth - 15) + ", 0)");

		scrollbarContainer.select("rect.scroll-bar-box")
			.attr("height", chart._chartHeight);

		scrollbarContainer.select("rect.scroll-bar-track")
			.attr("height", chart._chartHeight)
			.attr("transform", "translate(12, " + chart._chartTop + ")");

		var scrollbarDelta = panningY * chart._chartHeight / chart.series[0]._serieContainer.node().getBBox().height;
		scrollbarContainer.select("rect.scroll-bar-thumb")
			.attr("height", chart._chartHeight * chart._chartHeight / chart.series[0]._serieContainer.node().getBBox().height)
			.attr("transform", "translate(12, " + (chart._chartTop - scrollbarDelta) + ")");
	}

	function initSerieYScale(serie) {
		if (!chart.invertedAxes) {
			if (isUndefinedOrNull(serie.yAxis.linkedSerie)) {
				serie._yScale = d3.scale.linear()
					.range([chart._chartHeight, 15]) // 15px is the space needed for value labels
					.domain([serie._minValue, serie._maxValue]);

				serie._startValue = serie._minValue < 0 ? 0 : serie._minValue;
			} else {
				var linkedSerie = chart.series[serie.yAxis.linkedSerie];

				serie.yAxis.visible = false;
				serie.yAxis.format = linkedSerie.yAxis.format;

				serie._minValue = linkedSerie._minValue;
				serie._maxValue = linkedSerie._maxValue;
				serie._startValue = linkedSerie._startValue;
				serie._yScale = linkedSerie._yScale;
			}
		} else {
			var bandHeight;

			if (chart.xAxis.type == "bands") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					serie._yScale = d3.scale.ordinal()
						.rangeRoundBands([0, chart._chartHeight], chart.xAxis.bandDimensions.gap, 0);
				} else {
					if (chart.dataSource.data.length !== 0) {
						bandHeight = chart._chartHeight / chart.dataSource.data.length;

						serie._yScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandHeight * chart.dataSource.data.length], chart.xAxis.bandDimensions.gap, 0);
					} else {
						bandHeight = chart._chartHeight;

						serie._yScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandHeight], chart.xAxis.bandDimensions.gap, 0);
					}
				}
			} else if (chart.xAxis.type == "points") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					serie._yScale = d3.scale.ordinal()
						.rangeRoundPoints([0, chart._chartHeight], chart.xAxis.bandDimensions.gap);
				} else {
					if (chart.dataSource.data.length > 1) {
						bandHeight = chart._chartHeight / (chart.dataSource.data.length - 1);

						serie._yScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandHeight * (chart.dataSource.data.length - 1)], chart.xAxis.bandDimensions.gap);
					} else {
						bandHeight = chart._chartHeight;

						serie._yScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandHeight], chart.xAxis.bandDimensions.gap);
					}
				}
			}

			serie._yScale.domain(chart.dataSource.data.map(function(d) { return d[chart.xAxis.field]; }));

			serie._bandTop = function(data) {
				return serie._yScale(data[chart.xAxis.field]);
			};
			serie._bandHeight = function() {
				return serie._yScale.rangeBand();
			};
			serie._pointTop = function(d) {
				return serie._bandTop(d) + serie._bandHeight() / 2;
			};
		}
	}

	function drawYAxis(serie, yAxis, yAxisPosition) {
		var axisSelection;
		if (isUndefinedOrNull(serie._yAxisContainer)) {
			serie._yAxisContainer = serie._serieContainer.select("g.y-axis")
				.style('opacity', 0)
				.attr("class", "axis y-axis")
				.attr("transform", "translate(" + yAxisPosition + ",0)");

			axisSelection = serie._yAxisContainer.append("g")
				.attr("class","ticks")
				.call(yAxis);

		} else {
			var yAxisSelection = serie._yAxisContainer;
			serie._yAxisInitialized = true;

			axisSelection = yAxisSelection
				.attr("transform", "translate(" + yAxisPosition + ",0)")
				.style('opacity', 1)
				.select("g.ticks")
				.call(yAxis);
		}

		serie._yAxisContainer.selectAll("path, line")
			.style("fill","none")
			.style("stroke","#999")
			.style("shape-rendering", "crispEdges");

		if (!chart.invertedAxes) {
			serie._yAxisContainer.selectAll("path.domain")
				.style("display", "none");
		}

		serie._yAxisContainer.selectAll("text")
			.style("font-size", "10px");

		if (chart.invertedAxes) {
			serie._yAxisContainer.selectAll("text")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip);

			setupShowTooltipOnLongTap(serie._yAxisContainer.selectAll("text"));
		}
	}

	function addFloatingSerieAxis(serie) {
		var _scale = d3.scale.ordinal()
			.rangeRoundBands([chart._chartHeight, 0])
			.domain(serie.labels);

		var yAxis = d3.svg.axis()
			.scale(_scale)
			.orient(serie.yAxis.position);

		var yAxisPosition;
		if (serie.yAxis.position == "left") {
			yAxisPosition = 0;
		} else {
			yAxisPosition = chart._chartWidth;
		}

		drawYAxis(serie, yAxis, yAxisPosition);

		serie._yAxisContainer.selectAll("path.domain")
			.style("display", "block");

		serie._yAxisContainer.selectAll("g.tick").each(function(d, i) {
			var tick = d3.select(this);
			tick.classed("tick-"+i, true);
		});
	}

	function drawSerieYAxis(serie) {
		if (serie.yAxis && serie.yAxis.visible !== false) {
			if (serie.stacked && serie.floating && !chart.invertedAxes) {
				addFloatingSerieAxis(serie);
			} else {
				var yAxis = d3.svg.axis()
					.scale(serie._yScale)
					.orient(chart.invertedAxes ? "left" : serie.yAxis.position);

				var yAxisPosition;
				if (!chart.invertedAxes) {
					yAxis.ticks(Math.max(chart._chartHeight/40, 2));

					if (serie.yAxis.formatName == "number" && (serie._maxValue - serie._minValue < yAxis.ticks()[0])) {
						yAxis.tickValues(d3.range(serie._minValue, serie._maxValue+1, 1));
					}

					if (serie.yAxis.position == "left") {
						yAxisPosition = 0;
					} else {
						yAxisPosition = chart._chartWidth;
					}

					if (serie.yAxis.format) {
						if (isFunction(serie.yAxis.format)) {
							yAxis.tickFormat(serie.yAxis.format);
						} else {
							yAxis.tickFormat(function(value) { return formatValue(value, serie.yAxis.format); });
						}
					}
				} else {
					yAxisPosition = chart._xScale(chart._startValue);
					if (chart.xAxis.format) {
						if (isFunction(chart.xAxis.format)) {
							yAxis.tickFormat(chart.xAxis.format);
						} else {
							yAxis.tickFormat(function(value) { return formatValue(value, chart.xAxis.format); });
						}
					}
				}

				drawYAxis(serie, yAxis, yAxisPosition);

				if (!chart.invertedAxes) {
					if (serie.yAxis.showGridLines && serie == chart.series[0]) {
						var yGrid = d3.svg.axis()
							.scale(serie._yScale)
							.orient(serie.yAxis.position)
							.tickSize(-chart._chartWidth, 0)
							.tickFormat("")
							.ticks(Math.max(chart._chartHeight/40, 2));

						if (isUndefinedOrNull(serie._gridContainer)) {
							serie._gridContainer = serie._serieContainer.select("g.grid")
								.style('opacity', 0)
								.attr("class", "grid")
								.attr("transform", "translate(" + yAxisPosition + ",0)")
								.call(yGrid);

							serie._gridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							serie._gridContainer.selectAll("path")
								.style("display", "none");
						} else {
							var gridSelection;
							if (serie._gridInitialized) {
								gridSelection = serie._gridContainer.transition().duration(_duration);
							} else {
								gridSelection = serie._gridContainer;
								serie._gridInitialized = true;
							}
							gridSelection
								.style("opacity", 1)
								.attr("transform", "translate(" + yAxisPosition + ",0)")
								.call(yGrid);

							serie._gridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							serie._gridContainer.selectAll("path")
								.style("display", "none");
						}
					}

					if (serie.yAxis.label) {
						var deltaX = { left: -1, right: 1};
						var rotation = { left: -90, right: 90};

						var label = serie._yAxisContainer.select("text.label");
						if (label.empty()) {
							label = serie._yAxisContainer.append("text")
								.style('opacity', 0)
								.attr("class","label")
								.attr("text-anchor","middle")
								.attr("transform","rotate(" + rotation[serie.yAxis.position] + ")")
								.attr("x",chart._chartHeight/2 * deltaX[serie.yAxis.position])
								.attr("y",(serie._yAxisContainer.select("g.ticks").node().getBBox().width + 10) * -1)
								.style("font-weight", 700)
								.style("font-size", "12px")
								.text(serie.yAxis.label);

								if (serie.fields.length === 1) {
									label.attr("fill", serie.colors[0 % serie.colors.length]);
								}
						} else {
							var yLabelSelection;
							if (serie._yLabelInitialized) {
								yLabelSelection = label.transition().duration(_duration);
							} else {
								yLabelSelection = label;
								serie._yLabelInitialized = true;
							}
							yLabelSelection
								.style('opacity', 1)
								.attr("x",chart._chartHeight/2 * deltaX[serie.yAxis.position])
								.attr("y",(serie._yAxisContainer.select("g.ticks").node().getBBox().width + 10) * -1);
						}
					}
				}
			}
		}
	}

	var legendDragging = false;
	var legendTop = 0;
	var legendLeft = 0;

	function drawLegend() {
		var legendContainer;
		var handlerRadius = 12;
		var labelHeight = 10;
		var label;
		var labelSymbolWidth = 20;
		var labelMargin = 5;
		var boxMargin = 10;
		var boxWidth = 0;
		var boxHeight = boxMargin;

		var drawLegendItem = function(serie, label, serieIdx, labelIdx) {
			var legendLabel = legendContainer.select("g.legend-label-"+serieIdx+"-"+labelIdx);
			if (legendLabel.empty()) {
				legendLabel = legendContainer.append("g")
					.attr("class","legend-label legend-label-"+serieIdx+"-"+labelIdx)
					.attr("transform", "translate(" + (boxMargin + handlerRadius) + ", " + boxHeight + ")");

				if (serie.type == "bar") {
					legendLabel.append("rect")
						.attr("width", labelSymbolWidth)
						.attr("height", labelHeight)
						.attr("fill", serie.colors[labelIdx % serie.colors.length]);
				}

				if (serie.type == "line" || serie.type == "area") {
					var	symbol = d3.svg.symbol().type(serie.marks.type).size(Math.pow(labelHeight-2,2));

					if (serie.type == "area") {
						legendLabel.append("rect")
							.attr("width", labelSymbolWidth)
							.attr("height", labelHeight / 2)
							.attr("y", labelHeight / 2)
							.attr("opacity", serie.areaOpacity)
							.attr("fill", serie.colors[labelIdx % serie.colors.length]);
					}

					legendLabel.append("path")
						.attr("transform", "translate(" + labelSymbolWidth/2 + ", " + labelHeight/2 + ")")
						.attr("d", symbol)
						.attr("fill", serie.colors[labelIdx % serie.colors.length]);

					legendLabel.append("line")
						.attr("stroke-width", Math.min(serie.lineWidth, 3))
						.attr("stroke", serie.colors[labelIdx % serie.colors.length])
						.attr("x1", 0).attr("y1", labelHeight/2)
						.attr("x2", labelSymbolWidth).attr("y2", labelHeight/2);
				}

				if (serie.type == "pie" || serie.type == "donut") {
					legendLabel.append("path")
						.attr("fill", function() {
							var color = serie.colors[labelIdx % serie.colors.length];
							return (serie.fieldColor && serie.pieData && serie.pieData[labelIdx][serie.fieldColor]) || color;
						})
						.attr("stroke-width", 0)
						.attr("transform", "translate(" + labelSymbolWidth / 2 + ", " + labelHeight + ")")
						.attr("d", function() {
							var arc = d3.svg.arc()
								.innerRadius(labelHeight / 2.5)
								.outerRadius(labelHeight)
								.startAngle(-60 * (Math.PI/180))
								.endAngle(60 * (Math.PI/180));
							return arc();
						});
				}

				if (serie.type == "gauge") {
					if (serie.horizontal) {
						legendLabel.append("rect")
							.attr("width", labelSymbolWidth)
							.attr("height", labelHeight)
							.attr("fill", serie.colors[labelIdx % serie.colors.length]);
					} else {
						legendLabel.append("path")
							.attr("fill", serie.colors[labelIdx % serie.colors.length])
							.attr("stroke-width", 0)
							.attr("transform", "translate(" + labelSymbolWidth / 2 + ", " + labelHeight + ")")
							.attr("d", function() {
								var arc = d3.svg.arc()
									.innerRadius(labelHeight / 2.5)
									.outerRadius(labelHeight)
									.startAngle(-60 * (Math.PI/180))
									.endAngle(60 * (Math.PI/180));
								return arc();
							});
					}
				}

				if (serie.type == "pie" || serie.type == "donut") {
					legendLabel.attr("field-idx",labelIdx)
						.on("mouseup", function(d) {
							if (!legendDragging) {
								var idx = d3.select(this).attr("field-idx");
								dataClick(serie.pieData[idx]);
							}
						});
				}

				legendLabel.append("text")
					.attr("dx", labelSymbolWidth + labelMargin)
					.attr("y", labelHeight / 2)
					.attr("dy", "0.35em")
					.style("font-size", "10px")
					.text(label);
			}

			boxWidth = Math.max(boxWidth, Math.round(legendLabel.node().getBBox().width) + boxMargin * 2);
			boxHeight += Math.round(legendLabel.node().getBBox().height) + labelMargin;
		};

		function calcLegendInitialPosition() {
			var legendPos = {};
			var boxPosition = chart.legend.position.split(" ");

			if (boxPosition[0] == "top") {
				legendPos.top = chart._chartTop + chart.legend.margin;
			} else if (boxPosition[0] == "bottom") {
				legendPos.top = Math.max(chart._chartTop + chart.legend.margin, chart._chartTop + chart._chartHeight - chart.legend.margin - legendContainer.node().getBBox().height - 1);
			} else if (boxPosition[0] == "center") {
				legendPos.top = Math.max(chart._chartTop + chart.legend.margin, (chart._chartTop + chart._chartHeight - boxHeight - handlerRadius) / 2);
			}
			if (boxPosition[1] == "left") {
				legendPos.left = chart._chartLeft + chart.legend.margin;
			} else if (boxPosition[1] == "right") {
				legendPos.left = Math.max(chart._chartLeft + chart.legend.margin, chart._chartLeft + chart._chartWidth - chart.legend.margin - legendContainer.node().getBBox().width - 1);
			} else if (boxPosition[1] == "center") {
				legendPos.left = Math.max(chart._chartLeft + chart.legend.margin, (chart._chartLeft + chart._chartWidth - boxWidth - handlerRadius) / 2);
			}

			return legendPos;
		}

		function moveLegend(animate, initial) {
			if (initial) {
				var legendPos = calcLegendInitialPosition();
				legendLeft = legendPos.left;
				legendTop = legendPos.top;
			} else {
				legendLeft = Math.min(chart._chartLeft + chart._chartWidth - chart.legend.margin - legendContainer.node().getBBox().width - 1, legendLeft);
				legendLeft = Math.max(chart._chartLeft + chart.legend.margin, legendLeft);

				legendTop = Math.min(chart._chartTop + chart._chartHeight - chart.legend.margin - legendContainer.node().getBBox().height - 1, legendTop);
				legendTop = Math.max(chart._chartTop + chart.legend.margin, legendTop);
			}

			if (animate) {
				legendContainer.transition().duration(_duration)
					.attr("transform", "translate(" + legendLeft + ", " + legendTop + ")");
			} else {
				legendContainer.attr("transform", "translate(" + legendLeft + ", " + legendTop + ")");
			}
		}

		if (chart.legend) {
			legendContainer = svgContainer.select("g.legend");
			if (legendContainer.empty()) {
				legendContainer = svgContainer.append("g")
					.attr("class","legend")
					.style("cursor","move");

				var drag = d3.behavior.drag()
					.origin(Object)
					.on('dragstart', function() {
						if ($(d3.event.sourceEvent.target).parents('g.legend-handler').length === 0) {
							legendContainer.style("opacity", 0.5);

							if (!legendDragging) {
								legendDragging = true;
								setupPanning(false);
							}
						}
					})
					.on('drag', function() {
						if (legendDragging) {
							legendLeft += d3.event.dx;
							legendTop += d3.event.dy;

							moveLegend(false);
						}
					})
					.on('dragend', function() {
						if (legendDragging) {
							legendContainer.style("opacity", 1);
							legendDragging = false;
							setupPanning(true);
						}
					});

				legendContainer.call(drag);
			} else {
				legendContainer.html("");
			}

			var legendBox = legendContainer.select("rect.legend-box");
			if (legendBox.empty()) {
				legendBox = legendContainer.append("rect")
					.attr("class","legend-box")
					.attr("y", handlerRadius)
					.attr("x", handlerRadius)
					.attr("fill", "white")
					.attr("fill-opacity", 0.7)
					.attr("stroke", "#ccc")
					.attr("stroke-width", 1)
					.attr("shape-rendering", "crispEdges");

				boxHeight += 12;
			}

			var legendHandler = legendContainer.select("g.legent-handler");
			var legendCloseTimeout;
			if (legendHandler.empty()) {
				legendHandler = legendContainer.append("g")
					.attr("class", "legend-handler")
					.style("cursor", "default")
					.on("mouseup", function() {
						if (!legendDragging) {
							clearTimeout(legendCloseTimeout);
							chart.legend.closed = !chart.legend.closed;
							drawLegend();
							moveLegend(true); // prevent legend to be braw over the limits of svg
						}
					});

				legendHandler.append("circle")
					.attr("cx", handlerRadius)
					.attr("cy", handlerRadius)
					.attr("r", handlerRadius)
					.attr("fill", "#ccc");

				legendHandler.append("line")
					.attr("stroke-width", 2)
					.attr("stroke", "white")
					.attr("x1", handlerRadius/2)
					.attr("y1", handlerRadius)
					.attr("x2", handlerRadius + handlerRadius/2)
					.attr("y2", handlerRadius);

				if (chart.legend.closed == "delay") {
					chart.legend.closed = false;
					legendCloseTimeout = setTimeout(function() {
						chart.legend.closed = true;
						drawLegend();
						moveLegend(true, true);
					}, 5000);
				}

				if (chart.legend.closed) {
					legendHandler.append("line")
						.attr("stroke-width", 2)
						.attr("stroke", "white")
						.attr("x1", handlerRadius)
						.attr("y1", handlerRadius/2)
						.attr("x2", handlerRadius)
						.attr("y2", handlerRadius + handlerRadius/2);

					var closedText = legendContainer.append("text")
						.attr("font-size", "12px")
						.attr("y", boxHeight)
						.attr("dy", "0.7em")
						.attr("x", boxMargin + handlerRadius)
						.text(ScreenService.i18n("LEGEND"));

					boxWidth = Math.max(boxWidth, Math.round(closedText.node().getBBox().width) + boxMargin * 2);
					boxHeight += Math.round(closedText.node().getBBox().height - 3);
				}
			}

			if (!chart.legend.closed) {
				chart.series.forEach(function(serie, serieIdx) {
					if (hasAxis(serie)) {
						var _labels = serie.labels.slice(0);
						if (serie.stacked && !chart.invertedAxes) {
							_labels.reverse();
						}

						_labels.forEach(function(label, labelIdx) {
							if (serie.stacked && !chart.invertedAxes) {
								labelIdx = _labels.length - 1 - labelIdx;
							}
							drawLegendItem(serie, label, serieIdx, labelIdx);
						});
					} else {
						if (serie.type == "pie" || serie.type == "donut") {
							serie.pieData.forEach(function(d, labelIdx) {
								label = d[serie.fieldLabel];
								drawLegendItem(serie, label, serieIdx, labelIdx);
							});
						} else if (serie.type == "gauge") {
							serie._gaugeData.forEach(function(d, labelIdx) {
								var minValue = d.rangeMinValue;
								var maxValue = d.rangeMaxValue;
								label = minValue + ' - ' + maxValue;

								if (serie.valueFormat) {
									if (isFunction(serie.valueFormat)) {
										label = serie.valueFormat(minValue) + ' - ' + serie.valueFormat(maxValue);
									} else {
										label = formatValue(minValue, serie.valueFormat) + ' - ' + formatValue(maxValue, serie.valueFormat);
									}
								}

								drawLegendItem(serie, label, serieIdx, labelIdx);
							});
						} else {
							chart.dataSource.data.forEach(function(d, labelIdx) {
								label = d[serie.fieldLabel];
								drawLegendItem(serie, label, serieIdx, labelIdx);
							});
						}
					}
				});
			}

			boxHeight += boxMargin - labelMargin;

			legendBox.attr("height", boxHeight - handlerRadius)
				.attr("width", boxWidth);

			if (!chart._legendInitialized) {
				var legendPos = calcLegendInitialPosition();
				legendLeft = legendPos.left;
				legendTop = legendPos.top;
				legendContainer.attr("transform", "translate(" + legendLeft +  ", " + legendTop + ")");

				chart._legendInitialized = true;
			} else {
				moveLegend(true); // prevent legend to be braw over the limits of svg
			}

			legendContainer.style("display", chart.legend.visible !== false ? "block" : "none");
		}
	}

	var panningX = 0;
	var panningY = 0;
	function panChart(deltaX) {
		if (chart.xAxis && chart.xAxis.visible !== false && Math.abs(deltaX) > 0) {
			var scrollbarContainer = svgContainer.select("g.scroll-bar");
			var scrollbarThumb = scrollbarContainer.select("rect.scroll-bar-thumb");

			if (!scrollbarContainer.empty()) {
				panningX += deltaX;
				panningX = Math.max(Math.min(panningX, 0), chart._chartWidth - chart._xAxisContainer.select("path.domain").node().getBBox().width);

				if (chart._chartWidth >= chart._xAxisContainer.select("path.domain").node().getBBox().width) {
					panningX = 0;
				}

				if (chart.events.onPanning) {
					chart.events.onPanning({x: Math.abs(panningX), y: Math.abs(panningY)});
				}

				var transform, transformX, transformY;

				chart.series.forEach(function(serie) {
					transform = serie._serieContainer._dataContainer.attr("transform");
					transformY = transform ? +transform.split(",")[1].split(")")[0] : 0;
					serie._serieContainer._dataContainer.attr("transform", "translate(" + panningX + ", " + transformY + ")");
				});

				chart._xAxisContainer.select("g.ticks").selectAll("g.tick").each(function() {
					var tick = d3.select(this);
					transform = tick.attr("transform");
					transformX = transform ? +transform.split("(")[1].split(",")[0] : 0;
					transformY = transform ? +transform.split(",")[1].split(")")[0] : 0;
					var originalTransformX = transformX;
					if (!tick.attr("original-transform-x")) {
						tick.attr("original-transform-x", originalTransformX);
					} else {
						originalTransformX = +tick.attr("original-transform-x");
					}

					tick.attr("transform", "translate(" + (originalTransformX + panningX) + ", " + transformY+ ")");
				});

				if (!isUndefinedOrNull(chart._xGridContainer)) {
					chart._xGridContainer._dataContainer.attr("transform", "translate(" + panningX + ", 0)");
				}

				var scrollbarDelta = panningX * chart._chartWidth / chart._xAxisContainer.select("path.domain").node().getBBox().width;
				var thumbY = scrollbarThumb.attr("transform").split(', ')[1].split(')')[0];

				scrollbarThumb.attr("transform", "translate(" + (chart._chartLeft - scrollbarDelta) + ", " + thumbY + ")");
			}
		}
	}

	function panChartY(deltaY) {
		if (chart.xAxis && chart.xAxis.visible !== false && chart.series[0].type == "gantt" && Math.abs(deltaY) > 0) {
			var scrollbarContainer = svgContainer.select("g.scroll-bar-y");
			var scrollbarThumb = scrollbarContainer.select("rect.scroll-bar-thumb");

			if (!scrollbarContainer.empty()) {
				panningY += deltaY;
				panningY = Math.max(Math.min(panningY, 0), chart._chartHeight - chart.series[0]._serieContainer.node().getBBox().height);

				if (chart._chartHeight >= chart.series[0]._serieContainer.node().getBBox().height) {
					panningY = 0;
				}

				if (chart.events.onPanning) {
					chart.events.onPanning({x: Math.abs(panningX), y: Math.abs(panningY)});
				}

				var transform, transformX;

				chart.series.forEach(function(serie) {
					transform = serie._serieContainer._dataContainer.attr("transform");
					transformX = transform ? +transform.split("(")[1].split(",")[0] : 0;
					serie._serieContainer._dataContainer.attr("transform", "translate(" + transformX + ", " + panningY + ")");
				});

				var scrollbarDelta = panningY * chart._chartHeight / chart.series[0]._serieContainer.node().getBBox().height;
				var thumbX = scrollbarThumb.attr("transform").split('(')[1].split(', ')[0];

				scrollbarThumb.attr("transform", "translate(" + thumbX + ", " + (chart._chartTop - scrollbarDelta) + ")");
			}
		}
	}

	function resetPanning() {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			panningX = 0;

			chart.series.forEach(function(serie) {
				serie._serieContainer._dataContainer.transition().duration(_duration).attr("transform", "translate(" + panningX + ", 0)");
			});

			chartContainer.select("g.x-axis").select("g.ticks").transition().duration(_duration).attr("transform", "translate(" + panningX + ", 0)");

			if (!isUndefinedOrNull(chart._xGridContainer)) {
				chart._xGridContainer._dataContainer.attr("transform", "translate(0, 0)");
			}
		}
	}

	function getPanningData() {
		return {x: panningX, y: panningY};
	}

	function setPanningData(panningData) {
		panChart(-panningX - panningData.x);
		panChartY(-panningY - panningData.y);
	}

	function dataClick(data) {
		if (!(chartPanning && chartPanning.panned) && chart.events.onDataClick) {
			chart.events.onDataClick(d3.event, data);
		}
	}

	function chartAreaClick(serie) {
		if (!(chartPanning && chartPanning.panned) && chart.events.onDataClick) {
			chart.events.onDataClick(d3.event, serie);
		}
	}

	var lastTooltipData;
	var tooltipOffset = 5;

	var longPressInterval = 750;
	function setupShowTooltipOnLongTap(collection) {
		collection.each(function(d, dataIdx) {
			var item = this;
			d3.select(item)
				.on("touchstart", function() {
					var touchEvent = d3.event.touches[0];
					item._showTooltipTimeout = setTimeout(function() {
						showTooltip(d, dataIdx, touchEvent);
					}, longPressInterval);

					hideTooltip();
				})
				.on("touchmove", function() {
					clearTimeout(item._showTooltipTimeout);
				})
				.on("touchend", function() {
					clearTimeout(item._showTooltipTimeout);
				});
		});
	}

	function showTooltip(d, dataIdx, touchEvent) {
		if (chart.showTooltip && !legendDragging && (!isMobile() || touchEvent)) {
			if (typeof d == "string") {
				d = chart.dataSource.data[dataIdx];
			}

			if (d !== lastTooltipData) {
				lastTooltipData = d;

				var html = "";
				var serieData = "";

				chart.series.forEach(function(serie, idx) {
					if (hasAxis(serie)) {
						if (idx === 0) {
							html += "<span class=\"x-label\">" + d[chart.xAxis.field] + "</span>";
						}

						serie.fields.forEach(function(field, i) {
							if (!isUndefinedOrNull(d[field])) {
								var value = d[field];
								if (serie.yAxis.format) {
									if (isFunction(serie.yAxis.format)) {
										value = serie.yAxis.format(value);
									} else {
										value = formatValue(value, serie.yAxis.format);
									}
								}
								if (serie.yAxis.label) {
									value = value + " " + serie.yAxis.label;
								}
								if (serie.stacked && !chart.invertedAxes) {
									serieData = "<tr><td><span class=\"x-value\">" + serie.labels[i] + "</span></td><td>" + value + "</td></tr>" + serieData;
								} else {
									serieData = serieData + "<tr><td><span class=\"x-value\">" + serie.labels[i] + "</span></td><td>" + " " + value + "</td></tr>";
								}
							}
						});
					} else if (serie.type == "pie" || serie.type == "donut") {
						if (!d.data.originalSlices) {
							serieData += "<tr><td><span class=\"x-value\">" + d.data[serie.fieldLabel] + "</span></td><td>";
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									serieData += serie.valueFormat(d.data[serie.fieldValue]);
								} else {
									serieData += formatValue(d.data[serie.fieldValue], serie.valueFormat);
								}
							} else {
								serieData += d.data[serie.fieldValue];
							}
							serieData += "</td></tr>";
						} else {
							d.data.originalSlices.forEach(function(slice) {
								serieData += "<tr><td><span class=\"x-value\">" + slice[serie.fieldLabel] + "</span></td><td>";
								if (serie.valueFormat) {
									if (isFunction(serie.valueFormat)) {
										serieData += serie.valueFormat(slice[serie.fieldValue]);
									} else {
										serieData += formatValue(slice[serie.fieldValue], serie.valueFormat);
									}
								} else {
									serieData += slice[serie.fieldValue];
								}
								serieData += "<br />";
							});
							serieData += "</td></tr>";
						}
					} else if (serie.type == "gauge") {
						serieData += "<tr><td>";
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								serieData += serie.valueFormat(d.data.rangeMinValue) + ' - ' + serie.valueFormat(d.data.rangeMaxValue);
							} else {
								serieData += formatValue(d.data.rangeMinValue, serie.valueFormat) + ' - ' + formatValue(d.data.rangeMaxValue, serie.valueFormat);
							}
						} else {
							serieData += d.data.rangeMinValue + ' - ' + d.data.rangeMaxValue;
						}
						serieData += "</td></tr>";
					} else if (serie.type == "gantt") {
						html += "<span class=\"x-label\">" + d[serie.ganttFields.taskName] + "</span>";

						serieData = serieData + "<tr><td><span class=\"x-value\">Estimated begin</span></td><td>" + " " + d[serie.ganttFields.taskEstimatedBegin] + "</td></tr>";
						serieData = serieData + "<tr><td><span class=\"x-value\">Estimated end</span></td><td>" + " " + d[serie.ganttFields.taskEstimatedEnd] + "</td></tr>";
						if (!isUndefinedOrNull(d[serie.ganttFields.taskRealBegin])) {
							serieData = serieData + "<tr><td><span class=\"x-value\">Real begin</span></td><td>" + " " + d[serie.ganttFields.taskRealBegin] + "</td></tr>";
						}
						if (!isUndefinedOrNull(d[serie.ganttFields.taskRealEnd]) && !d._noRealEndDate) {
							serieData = serieData + "<tr><td><span class=\"x-value\">Real end</span></td><td>" + " " + d[serie.ganttFields.taskRealEnd] + "</td></tr>";
						}
						serieData = serieData + "<tr><td><span class=\"x-value\">Completed</span></td><td>" + " " + Math.round(d[serie.ganttFields.taskCompleted]) + "%</td></tr>";
					}
				});
				html += "<table>"+ serieData + "</table>";
				chart._tooltipContainer.html(html);
			}

			var eventSource = touchEvent ? touchEvent : d3.event;

			var left = eventSource.clientX + tooltipOffset - $(chartContainer.node()).offsetParent().offset().left;
			if (left + chart._tooltipContainer.node().clientWidth > chartContainer.node().clientWidth) {
				left = eventSource.clientX - tooltipOffset - $(chartContainer.node()).offsetParent().offset().left - chart._tooltipContainer.node().clientWidth;
				if (left < 0) {
					left = eventSource.clientX + tooltipOffset - $(chartContainer.node()).offsetParent().offset().left;
				}
			}
			var top = eventSource.clientY + tooltipOffset - $(chartContainer.node()).offsetParent().offset().top;
			if (top + chart._tooltipContainer.node().clientHeight > chartContainer.node().clientHeight) {
				top = eventSource.clientY - tooltipOffset - $(chartContainer.node()).offsetParent().offset().top - chart._tooltipContainer.node().clientHeight;
				if (top < 0) {
					top = eventSource.clientY + tooltipOffset - $(chartContainer.node()).offsetParent().offset().top;
				}
			}
			chart._tooltipContainer
				.style("opacity", 1)
				.style("left", left + "px")
				.style("top", top + "px");
		}
	}

	function hideTooltip() {
		chart._tooltipContainer.style("opacity", 0)
			.style("left", -1000 + "px")
			.style("top", -1000 + "px");
	}

	function initTooltip() {
		chart._tooltipContainer = d3.select(chartContainer.node().parentNode).select(".zh-chart-tooltip");
	}

	function _chart(element) {
		initChart(element);

		initSeries();
		draw();
	}

	_chart.redraw = buildDebounceMethod(function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			if (chart.scrollable) {
				resetPanning();
			}
			draw();
		}
	}, 100);

	_chart.savePNG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var uri = this.getBase64();

			var image = new Image();
			image.onload = function() {
				var canvas = document.createElement('canvas');
				canvas.width = image.width;
				canvas.height = image.height;
				var context = canvas.getContext('2d');
				context.drawImage(image, 0, 0);

				canvas.toBlobHD(function(blob) {
					saveAs(blob, "chart_"+(new Date().getTime())+".png");
				}, "image/png");
			};
			image.onerror = function() {
				console.error(
					'There was an error loading the data URI as an image on the following SVG\n',
					window.atob(uri.slice(26)), '\n',
					"Open the following link to see browser's diagnosis\n",
					uri);
			};
			image.src = uri;
		}
	};

	_chart.getBase64 = function() {
		function reEncode(data) {
			data = encodeURIComponent(data);
			data = data.replace(/%([0-9A-F]{2})/g, function(match, p1) {
				var c = String.fromCharCode('0x'+p1);
				return c === '%' ? '%25' : c;
			});
			return decodeURIComponent(data);
		}

		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var html = this.getSVG();
			var uri = 'data:image/svg+xml;base64,' + window.btoa(reEncode(html));

			return uri;
		}
	};

	_chart.saveSVG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			try {
				var isFileSaverSupported = !!new Blob();
			} catch (e) {
				alert("blob not supported");
			}

			var html = this.getSVG();

			var blob = new Blob([html], {type: "image/svg+xml"});
			saveAs(blob, "chart_"+(new Date().getTime())+".svg");
		}
	};

	_chart.getSVG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			svgContainer.selectAll("g.scroll-bar").style("display", "none");
			svgContainer.selectAll("g.scroll-bar-y").style("display", "none");

			var seriesWidth = svgContainer.node().getBBox().width + 20;
			var seriesHeight = svgContainer.node().getBBox().height + 20;

			var widthDiff = seriesWidth - chart.clippingPath.node().getBBox().width;
			var heightDiff = seriesHeight - chart.clippingPath.node().getBBox().height;

			chart.clippingPath.attr("width", seriesWidth).attr("height", seriesHeight);
			chart.clippingPathAxis.attr("width", seriesWidth).attr("height", seriesHeight);

			chartContainer.select("svg").attr("width", seriesWidth).attr("height", seriesHeight);

			svgContainer.selectAll('.serie-container g.grid g.tick line').attr('x2', seriesWidth);

			var html = chartContainer.select("svg")
				.attr("title", chart.title)
				.attr("version", 1.1)
				.attr("xmlns", "http://www.w3.org/2000/svg")
				.node().parentNode.innerHTML;

			svgContainer.selectAll("g.scroll-bar").style("display", "block");
			svgContainer.selectAll("g.scroll-bar-y").style("display", "block");

			chart.clippingPath.attr("width", seriesWidth-widthDiff).attr("height", seriesHeight-heightDiff);
			chart.clippingPathAxis.attr("width", seriesWidth-widthDiff).attr("height", seriesHeight-heightDiff);

			chartContainer.select("svg").attr("width", svgContainerWidth).attr("height", svgContainerHeight);

			svgContainer.selectAll(".x-grid line").attr("y2", chart._chartHeight);
			return html;
		}
	};

	_chart.toggleTooltip = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.showTooltip = !isUndefinedOrNull(show) ? show : !chart.showTooltip;
		}
	};

	_chart.toggleLegend = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.legend.visible = !isUndefinedOrNull(show) ? show : !chart.legend.visible;
			svgContainer.select("g.legend").style("display", chart.legend.visible ? "block" : "none");
		}
	};

	_chart.isLegendVisible = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			return chart.legend && chart.legend.visible;
		}
	};

	_chart.toggleValueLabels = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.showValueLabels = !isUndefinedOrNull(show) ? show : !chart.showValueLabels;
			svgContainer.selectAll(".label-value").style("display", chart.showValueLabels ? "block" : "none");
		}
	};

	function showDrillUpButton() {
		if (chart.showDrillUpButton) {
			var button = svgContainer.select(".drill-up-button");
			if (button.empty()) {
				button = svgContainer.append("g")
					.attr("class","drill-up-button")
					.attr("transform", "translate(" + chart.padding.left + ", " + chart.padding.left + ")")
					.style("cursor", "pointer")
					.on("click", function() {
						_chart.drillUp();
					});

				var rect = button.append("rect")
					.attr("fill","#333")
					.attr("x", 0)
					.attr("y", 0);

				var text = button.append("text")
					.attr("x", 10)
					.attr("y", 10)
					.attr("dy", "1em")
					.attr("font-size", 12)
					.attr("fill", "white")
					.text("Drill Up");

				rect.attr("width", text.node().getBBox().width + 20)
					.attr("height", text.node().getBBox().height + 20);
			}

			button.style("display", "block");
		}
	}

	function hideDrillUpButton() {
		svgContainer.select(".drill-up-button").style("display", "none");
	}

	_chart.drillDown = function(options) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var currentChart = jQuery.extend(true, {}, chart);
			drillPath.push(currentChart);
			showDrillUpButton();

			chart.dataSource.data = [];
			chart = jQuery.extend(true, chart, options);

			initSeries();
			draw();
		}
	};

	_chart.drilledDown = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			return drillPath.length > 0;
		}
	};

	_chart.drillUp = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			if (drillPath.length > 0) {
				chart = drillPath.splice(-1,1)[0];
				if (drillPath.length === 0) {
					hideDrillUpButton();
				}

				initSeries();
				draw();
			}
		}
	};

	_chart.updateChart = function(newOptions) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.dataSource.data = [];
			options = newOptions;

			initChart();

			initSeries();
			draw();
		}
	};

	_chart.value = function(value) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(value)) {
				options.series[0].value = value;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge and progress are the first and only serie
			if (isUndefinedOrNull(value)) {
				return serie.value;
			} else {
				if (serie.type == "gauge") {
					serie.value = Math.max(value, serie._minValue);
				} else if (serie.type == "progress") {
					serie.value = Math.min(Math.max(value, 0), serie.maxValue);
				}
				return _chart;
			}
		}
	};

	_chart.maxValue = function(value) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(value)) {
				options.series[0].maxValue = value;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge and progress are the first and only serie
			if (isUndefinedOrNull(value)) {
				return serie.maxValue;
			} else {
				serie.maxValue = value;
				return _chart;
			}
		}
	};

	_chart.gaugeRanges = function(ranges) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(ranges)) {
				options.series[0].ranges = ranges;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge is the first and only serie
			if (isUndefinedOrNull(ranges)) {
				return serie.ranges;
			} else {
				serie.ranges = ranges;
				return _chart;
			}
		}
	};

	_chart.dataSourceData = function(data) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(data)) {
				options.dataSource.data = data;
			}
			return _chart;
		} else {
			if (isUndefinedOrNull(data)) {
				return chart.dataSource.data;
			} else {
				chart.dataSource.data = data;
				return _chart;
			}
		}
	};

	_chart.updateSeries = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			initSeries();
			draw();
		}
	};


	_chart.clear = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			this.dataSourceData([]).updateSeries();
		}
	};

	_chart.ganttTimeScale = function(scale) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(scale)) {
				options.series[0].ganttTimeScale = scale;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gantt is the first and only serie
			if (isUndefinedOrNull(scale)) {
				return serie.ganttTimeScale;
			} else {
				serie.ganttTimeScale = scale;
				return _chart;
			}
		}
	};

	_chart.ganttScrollToView = function(row) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			ganttScrollToView(row);
		}
	};

	_chart.panning = function(panningData) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			if (isUndefinedOrNull(panningData)) {
				return getPanningData();
			} else {
				setPanningData(panningData);
				return _chart;
			}
		}
	};

	_chart.updateSerieOptions = function(serieIndex, newOptions) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(newOptions)) {
				Object.keys(newOptions).forEach(function(key) {
					options.series[serieIndex][key] = newOptions[key];
				});
			}
			return _chart;
		} else {
			var serie = chart.series[serieIndex];
			if (isUndefinedOrNull(newOptions)) {
				return serie;
			} else {
				Object.keys(newOptions).forEach(function(key) {
					serie[key] = newOptions[key];
				});
				return _chart;
			}
		}
	};

	return _chart;
}

$(window).on("resize.chart", function() {
	$(".zh-chart-container:visible").each(function(idx, element) {
		if (element._chart) {
			element._chart.redraw();
		}
	});
});

// FILE: src/charts/assets/js/directives/zhChart.js
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhChart', function(templateManager, ScreenService, $timeout, eventEngine, ApplicationContext, $window, $rootScope, maskEngine) {
	return {
		link: function($scope, element, attrs, controller) {
			function getFormat(format, params) {
				params = params || {};
				var result;
				switch (format) {
					case "currency":
						result = (function(value) { return maskEngine.currencyFormat(value, params); });
						break;
					case "float":
						result = (function(value) { return maskEngine.floatFormat(value, params); });
						break;
					case "percent":
						result = (function(value) { return maskEngine.percentFormat(value, params); });
						break;
					case "number":
						result = (function(value) { return maskEngine.numberFormat(value, params); });
						break;
					case "metric":
						result = (function(value) { return maskEngine.metricFormat(value, params); });
						break;
					case "date":
						result = (function(value) { return maskEngine.dateFormat(value, params); });
						break;
					case "time":
						result = (function(value) { return maskEngine.secondsToTimeFormat(value, params); });
						break;
					default:
						result = format;
				}

				return result;
			}

			var widget = $scope.widget;

			// Floating Control defaults
			var floatingControlDefaults = {
				position: "top right"
			};
			widget.floatingControl = widget.floatingControl !== false && Util.extend(floatingControlDefaults, widget.floatingControl || {});

			var options;
			function buildOptions(widget) {
				var opt = {
					title: widget.chartTitle || false,
					subtitle: widget.chartSubTitle || false,
					messageNoData: ScreenService.i18n(widget.chartMessageNoData),
					fontName: "open_sanslight, Helvetica, Arial, sans-serif",
					dataSource: {
						data: widget.dataSource.data
					},
					showTooltip: widget.chartShowTooltip,
					showValueLabels: widget.chartShowValueLabels,
					showDrillUpButton: widget.chartShowDrillUpButton,
					xAxis: {
						field: widget.chartXAxisField,
						bandDimensions: {
							type: widget.chartXAxisBandDimensionType,
							minWidth: widget.chartXAxisBandDimensionMinWidth,
							gap: widget.chartXAxisBandDimensionGap
						},
						ticks: {
							maxWidth: widget.chartXAxisTicksMaxWidth,
							maxLines: widget.chartXAxisTicksMaxLines,
							rotation: widget.chartXAxisTicksRotation
						},
						format: getFormat(widget.chartXAxisFormat, widget.chartXAxisFormatParams)
					},
					legend: {
						position: widget.chartLegendPosition,
						margin: widget.chartLegendMargin,
						visible: widget.chartLegendVisible,
						closed: widget.chartLegendClosed
					},
					series: [],
					invertedAxes: widget.chartInvertedAxes,
					radialAxis: widget.chartRadialAxis,
					events: {
						onDataClick: dataClick,
						onFinishDrawing: finishDrawing,
						onPanning: chartPanning
					}
				};

				if (element.hasClass('zh-dashboard-chart')) {
					opt.padding = {
						left: 0,
						top: 0,
						right: 0,
						bottom: 0,
						title: 15
					};
				} else {
					opt.padding = widget.chartPadding;
				}

				var serie;
				for (var i = 0; i < widget.chartSeries.length; i++) {
					serie = {
						// all series
						type: widget.chartSeries[i].serieType,
						colors: widget.chartSeries[i].serieColors,
						showValueLabels: widget.chartSeries[i].serieShowValueLabels,
						// bar, line and area
						fields: widget.chartSeries[i].serieFields,
						labels: widget.chartSeries[i].serieLabels,
						yAxis: {
							label: widget.chartSeries[i].serieYAxisLabel,
							position: widget.chartSeries[i].serieYAxisPosition,
							showGridLines: widget.chartSeries[i].serieYAxisShowGridLines,
							range: widget.chartSeries[i].serieYAxisRange,
							formatName: widget.chartSeries[i].serieYAxisFormat,
							format: getFormat(widget.chartSeries[i].serieYAxisFormat, widget.chartSeries[i].serieYAxisFormatParams),
							linkedSerie: widget.chartSeries[i].serieYAxisLinkedSerie
						},
						stacked: widget.chartSeries[i].serieStacked,
						normalized: widget.chartSeries[i].serieStackedNormalized,
						// bar
						floating: widget.chartSeries[i].serieFloating,
						floatingGap: widget.chartSeries[i].serieFloatingGap,
						centeredBars: widget.chartSeries[i].serieCenteredBars,
						showCenterLine: widget.chartSeries[i].serieShowCenterLine,
						maxBarWidth: widget.chartSeries[i].serieMaxBarWidth,
						colorExpression: widget.chartSeries[i].serieColorExpression,
						// line and area
						lineType: widget.chartSeries[i].serieLineType,
						lineWidth: widget.chartSeries[i].serieLineWidth,
						lineExpanded: widget.chartSeries[i].serieLineExpanded,
						marks: {
							type: widget.chartSeries[i].serieLineMarkType,
							size: widget.chartSeries[i].serieLineMarkSize
						},
						// area
						areaOpacity: widget.chartSeries[i].serieAreaOpacity,
						// pie, donut
						sort: widget.chartSeries[i].serieSort,
						fieldLabel: widget.chartSeries[i].serieFieldLabel,
						fieldValue: widget.chartSeries[i].serieFieldValue,
						fieldColor: widget.chartSeries[i].serieFieldColor,
						pivoted: widget.chartSeries[i].seriePivoted,
						pivotedFields: widget.chartSeries[i].seriePivotedFields,
						pivotedLabels: widget.chartSeries[i].seriePivotedLabels,
						maxSlices: widget.chartSeries[i].serieMaxSlices,
						maxSlicesText: widget.chartSeries[i].serieMaxSlicesText,
						// pie, donut and gauge
						strokeWidth: widget.chartSeries[i].serieStrokeWidth,
						strokeColor: widget.chartSeries[i].serieStrokeColor,
						arcsGap: widget.chartSeries[i].serieArcsGap,
						// pie, donut, gauge and progress
						valueFormat: getFormat(widget.chartSeries[i].serieValueFormat, widget.chartSeries[i].serieValueFormatParams),
						// donut
						donutWidth: widget.chartSeries[i].serieDonutWidth,
						// gauge
						rangeType: widget.chartSeries[i].serieRangeType,
						ranges: widget.chartSeries[i].serieRanges,
						gaugeWidth: widget.chartSeries[i].serieGaugeWidth,
						gaugeShowLegend: widget.chartSeries[i].serieGaugeShowLegend,
						pointerColor: widget.chartSeries[i].seriePointerColor,
						showGaugeValue: widget.chartSeries[i].serieShowGaugeValue,
						// gauge and progress
						horizontal: widget.chartSeries[i].serieHorizontal,
						value: widget.chartSeries[i].serieValue,
						maxValue: widget.chartSeries[i].serieMaxValue,
						// progress
						showProgressValue: widget.chartSeries[i].serieShowProgressValue,
						backgroundColor: widget.chartSeries[i].serieProgressBackgroundColor,
						progressWidth: widget.chartSeries[i].serieProgressWidth,
						// gantt
						ganttFields: widget.chartSeries[i].serieGanttFields,
						ganttTimeScale: widget.chartSeries[i].serieGanttTimeScale,
						dateFormat: widget.chartSeries[i].serieGanttDateFormat,
						itemHeight: widget.chartSeries[i].serieGanttItemHeight,
						itemBarHeight: widget.chartSeries[i].serieGanttItemBarHeight,
						showTaskCompletionLabel: widget.chartSeries[i].serieGanttShowTaskCompletionLabel,
						realBarType: widget.chartSeries[i].serieGanttRealBarType,
						showTaskName: widget.chartSeries[i].serieGanttShowTaskName,
						showPredecessors: widget.chartSeries[i].serieGanttShowPredecessors,
						showCompletionLabelOnParentTasks: widget.chartSeries[i].serieGanttShowCompletionLabelOnParentTasks,
						overrideTaskColorEstimated: widget.chartSeries[i].serieGanttOverrideTaskColorEstimated,
						overrideTaskColorReal: widget.chartSeries[i].serieGanttOverrideTaskColorReal
					};

					opt.series.push(serie);
				}

				return opt;
			}
			options = buildOptions(widget);

			var chart = widget.chart = zhChart(options, ScreenService);
			if (element.hasClass('zh-dashboard-chart')) {
				$scope.$on("zh-masonry-grid-finished", function(events, el) {
					if (el[0] == element.parents('[data-zh-masonry-grid]')[0] && element.is(":visible")) {
						$timeout(function() {
							if (Util.isUndefined(element[0]._chart)) {
								chart.dataSourceData(widget.dataSource.data)(element[0]);
							} else {
								element[0]._chart.redraw();
							}

							if (chart.isLegendVisible() === false && floatingControl) {
								button = floatingControl.find("li.show-legend");
								button.addClass("disabled");
							}
						});
					}
				});
			} else {
				$scope.$on("zh-scrolled-finished", function(events, el) {
					if (el[0] == element.parent()[0] && element.is(":visible")) {
						$timeout(function() {
							chart.dataSourceData(widget.dataSource.data)(element[0]);

							if (chart.isLegendVisible() === false && floatingControl) {
								button = floatingControl.find("li.show-legend");
								button.addClass("disabled");
							}
						});
					}
				});
			}

			$scope.$watch("widget.dataSource.data", function(newValue) {
				if (element.is(":visible")) {
					chart.dataSourceData(newValue).updateSeries();
				}
			}, true);

			if (widget) {
				var events = widget.events;
				if (events) {
					eventEngine.bindEvents(
						element,
						events,
						function() {
							if (widget.dataSource) {
								return widget.dataSource.data;
							}
						},
						function() {
							return widget.currentRow;
						},
						function() {
							return widget.selectedRow;
						}
					);
				}
			}

			function dataClick(event, data) {
				if (widget) {
					widget.setCurrentRow(data);
					if ($(event.target).attr('field-name') && widget.chartFieldClick) {
						var fieldName = $(event.target).attr('field-name');
						widget.chartFieldClick(null, {data: {fieldName: fieldName}});
					}

					if (data.type === 'area' && widget.chartAreaClick) {
						widget.chartAreaClick(null, data);
					}
				}
			}

			function finishDrawing() {
				if (widget && widget.chartFinishDrawing) {
					widget.chartFinishDrawing(null, {data: {element: element}});
				}
			}

			function chartPanning(panningData) {
				if (widget && widget.chartPanning) {
					widget.chartPanning(null, {data: panningData});
				}
			}

			$scope.saveChartAsSVG = function() {
				chart.saveSVG();
			};

			$scope.saveChartAsPNG = function() {
				chart.savePNG();
			};

			var floatingControl = element.parent().find(".zh-floating-control");
			var button;

			if (widget.chartShowValueLabels === false) {
				button = floatingControl.find("li.show-labels-action");
				button.addClass("disabled");
			}

			$scope.toggleValueLabels = function() {
				var li = floatingControl.find("li.show-labels-action");
				li.toggleClass("disabled");
				chart.toggleValueLabels(!li.hasClass("disabled"));
			};

			if (widget.chartShowTooltip === false) {
				button = floatingControl.find("li.show-tooltip");
				button.addClass("disabled");
			}

			$scope.toggleTooltip = function() {
				var li = floatingControl.find("li.show-tooltip");
				li.toggleClass("disabled");
				chart.toggleTooltip(!li.hasClass("disabled"));
			};

			if (widget.chartLegendVisible === false) {
				button = floatingControl.find("li.show-legend");
				button.addClass("disabled");
			}

			$scope.toggleLegend = function() {
				var li = floatingControl.find("li.show-legend");
				li.toggleClass("disabled");
				chart.toggleLegend(!li.hasClass("disabled"));
			};

			widget.updateChart = function() {
				var options = buildOptions(this);
				if (widget.chart) {
					widget.chart.updateChart(options);
				}
			};
		}
	};
});

Configuration(function (ContextRegister, eventEngine) {
	// register the event finishDrawingEvent
	eventEngine.registerEvent('WidgetOnChartFinishDrawing', 'chartFinishDrawing');
	eventEngine.registerEvent('WidgetOnChartFieldClick', 'chartFieldClick');
	eventEngine.registerEvent('WidgetOnChartAreaClick', 'chartAreaClick');
	eventEngine.registerEvent('WidgetOnChartPanning', 'chartPanning');
});

// FILE: src/charts/assets/js/directives/zhChartsFloatingControl.js
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhChartsFloatingControl', function() {
	return {
		link: function($scope) {
			$scope.getFloatingControlActions = function(){
				var actions = $scope.getCustomActions() || [];
				var customizationActions = $scope.widget.floatingControl.customizationAction || {};
				return actions.filter(function(action){
					return customizationActions[action.type];
				});
			};
		}
	};
});

// FILE: bower_components/tek-libraries/dist/main.js

// FILE: js/Util/Colors.js
function Colors() {

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (rgb) {
	    return "#" + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1);
	};
	
	this.RGBToTColor = function(r, g, b){
		var Tcolor = r | (g << 8) | (b << 16);
		return Tcolor;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Colors', Colors);
});

// FILE: js/Util/Endereco.js
var Endereco = function (restEngine, ScreenService, MessageService, ZHPromise) {
    var self = this;

    //Busca pelo CEP na API dos correios e retorna o resultado por promise
    self.buscarCep = function(CEP) {
        CEP = CEP.replace(/[\D]+/g,'');

        if(CEP.length == 8) {
            return restEngine.post('/lib_findAddress', {
                    'requestType':'Row',
                    'row': {'CEP': CEP}
            });
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    /* Busca pelo CEP na API dos correios e trata o retorno.
     * Usar apenas de exemplo para fazer um método próprio!
     */
    self.validarCep = function(widget, CEP) {
        self.buscarCep(CEP).then(function(response) {
            var endereco = response.dataset.ENDERECO;

            if(endereco) {
                widget.currentRow.CEP = CEP;
                widget.currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                widget.currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                widget.currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                widget.currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                widget.currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                widget.currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                widget.currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                widget.currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                widget.currentRow.CDLOGRADOURO = (endereco.CDLOGRADOURO)? endereco.CDLOGRADOURO : '';
                widget.currentRow.DSLOGRADOURO = (endereco.DSLOGRADOURO)? endereco.DSLOGRADOURO : '';
                widget.currentRow.DSENDERECO = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
            } else {
                widget.currentRow.CEP = '';
            }
        }, function(error) {            
            MessageService.getMessageByCode("PROXY_CONNECTION_FAIL").then(function(errorMessage) {
                var row = {
                    "CEP": CEP, 
                    "errorMessage": errorMessage
                };
                widget.currentRow.CEP = '';

                if(error.data)
                    errorMessage = error.data.error;
            
                ScreenService.showMessageByCode("REQUEST_CEP_ERROR", row, Constants.errorMessage);
            });
        });
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('Endereco', Endereco);
});

// FILE: js/Util/General.js
var General = function (ScreenService, templateManager, localStorageEngine, restEngine, maskEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory, MessageService) {
	var searchListDeleted;
	var parent = this;

	//@TODO Criar objeto de mensagens para tradução.

	/*
	 * Função que atualiza o dataSourceFilter de um field que depende do resultado de
	 * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
	 * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
	 * ao alterar o valor do field "pai".
	 *
	 * @param field dependentField
	 * @param array parameters
	 *
	 * Exemplo:
	 * General.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
	 */
	this.onClickDependentField = function (dependentField, parameters) {

		var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

		var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

		if (changedDataSourceFilter) {
			changeDependetFieldFilter(dependentField, dataSourceFilterNew);
		}

		if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
			dependentField.reload();
		}
	};

	this.buildDataSourceFilter = function (dataSourceFilter, parameters) {
		if (!dataSourceFilter) {
			throw "DataSourceFilter is not defined!";
		}

		for (var position in dataSourceFilter) {
			if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
				dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
			}
		}

		return dataSourceFilter;
	};

	var changeDependetFieldFilter = function (dependentField, dataSourceFilterNew) {

		dependentField.dataSourceFilter = dataSourceFilterNew;
		dependentField.dataSource.lazyLoad = false;
		if (dependentField.selectWidget) {
			dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
		}

		if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

			searchListDeleted = true;
			delete dependentField.selectWidget.searchList;

		} else if (dependentField.selectWidget) {

			searchListDeleted = false;

		}

	};

	var compareDataSourceFilters = function (selectWidget, dataSourceFilterNew, parameters) {
		var changedDataSourceFilter = false;
		if (selectWidget) {
			var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
			for (var position = 0; position < dataSourceFilterNew.length; position++) {
				if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
					if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
						changedDataSourceFilter = true;
					}
				}
			}
		} else {
			changedDataSourceFilter = true;
		}
		return changedDataSourceFilter;
	};

	/*
	 * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
	 * ou produtos correspondentes à máscara utilizada. Através de cada nível
	 * da máscara, é gerado o número correspondente à ele.
	 *
	 * @param string maskName
	 * @param int firstLevel
	 *
	 * @return array
	 *
	 * Exemplo:
	 * General.getLevelByMask('CDPICTPRAT', 1);
	 */
	this.getLevelByMask = function (maskName, firstLevel) {

		firstLevel = toNumber(firstLevel);

		var mask = getLocalVar(maskName);
		var splitMask = mask.split('.');
		var biggestLevel = splitMask.length;

		var level = [];
		var objLevel = {};

		for (var i = 0; i < biggestLevel; i++) {

			objLevel = {
				nivel: i + firstLevel,
				caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
			};

			objLevel.nivel = objLevel.nivel.toString();
			objLevel.caracteres = objLevel.caracteres.toString();

			level.push(objLevel);
		}

		return level;
	};

	var toNumber = function (value) {

		if (!value && value !== 0) {

			throw 'firstLevel is not defined';

		} else if (typeof value !== 'number' && value) {

			value = parseInt(value);

			if (isNaN(parseInt(value))) {

				throw 'Invalid firstLevel';

			} else {

				return value;
			}
		}
		return value;
	};

	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
		templateManager.updateTemplate();
	};

	this.setLoggedUnit = function (widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
	};

	var currentIdgrupoper;
	var currentCallback;
	var currentParameters;
	var gruposDeOperador = {
		LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
	};
	var fields = [
		{
			"name": "email",
			"label": "E-mail",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"template": "field/text-edit.html",
			"events": [],
			"fieldGroup": "AUTH",
			"validations": {
				"required": true,
				"mail": true
			}
        },
		{
			"name": "senha",
			"label": "Senha",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"fieldGroup": "AUTH",
			"template": "field/password-edit.html",
			"events": [],
			"validations": {
				"required": true
			}
        }
    ];
	var actions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.validateUser(args.owner.widget);"
                }
            ]
        }
    ];

	this.openAuthPopup = function (idgrupoper, message, callback, parameters) {

		if (!idgrupoper) {
			throw 'idgrupoper is not defined';
		}

		initAuthVariables(idgrupoper, callback, parameters);

		if (!message) {
			message = gruposDeOperador[currentIdgrupoper] || '';
		}

		var widgetAuth = metaDataFactory.widgetFactory({
			"id": 0,
			"name": "authWidget",
			"label": "Autenticação",
			"template": "widget/form.html",
			"container": templateManager.container,
			"isVisible": true,
			"popupNoBlock": false,
			"fieldGroups": [
				{
					"id": 100000,
					"name": "AUTH",
					"opened": true,
					"collapsible": false,
					"label": message
                }
            ],
			"fields": fields,
			"actions": actions,
			"events": []
		});

		ScreenService.openPopup(widgetAuth);

	};

	var initAuthVariables = function (idgrupoper, callback, parameters) {
		currentIdgrupoper = idgrupoper;
		currentCallback = callback || null;
		currentParameters = parameters || [];
	};

	this.validateUser = function (widget) {

		var self = this;

		if (widget.isValid()) {

			var email = widget.currentRow.email;
			var senha = widget.currentRow.senha;

			autenticar(email, senha).then(function (callback) {

				getPermission(callback.dataset.user.cdoperador).then(function (permission) {

					if (permission) {

						if (typeof currentCallback == 'function') {

							ScreenService.closePopup();

							currentCallback.apply(null, currentParameters);

						} else {

							throw 'callback is not a function';

						}

					} else {

						MessageService.showMessageByCode('lib#GEN_INVALID_OPERGROUP', null, { type: 'ERROR' } );

					}

				}, function (callback) {

					this.callbackError(callback);

				}.bind(self));

			}, function (callback) {

				this.callbackError(callback);

			}.bind(self));
		}

	};

	var autenticar = function (email, senha) {

		return restEngine.request('/Autenticar', {
			"requestType": "FilterData",
			"filter": [
				{
					name: "email",
					operator: "=",
					value: email
                },
				{
					name: "senha",
					operator: "=",
					value: senha
                }
            ]
		});

	};

	var getPermission = function (cdoperador) {

		return restEngine.request('/validateUserGroup', {
			"requestType": "Row",
			"row": {
				"idgrupoper": currentIdgrupoper,
				"cdoperador": cdoperador
			}
		}).then(function (callback) {

			return callback.dataset.permission.isValid;
		});

	};

	this.callbackError = function (callback) {

		if (callback.data) {

			ScreenService.showMessage(ScreenService.i18n(callback.data.error));

		} else {
			
			MessageService.showMessageByCode('lib#GEN_TIMEOUT', null, { type: 'ERROR' } );
		}
	};

	this.comparaPratoProduto = function (pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var menornv;
		if (nvinicial > nvfinal) {
			menornv = nvfinal;
		} else {
			menornv = nvinicial;
		}

		var resultado = true;
		var i = 0;

		while (pinicial[i] == pfinal[i] && i < menornv) {
			i++;
		}

		if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
			resultado = false;
		} else {
			resultado = true;
		}

		return resultado;
	};

	var reportWidget = {
		"id": 0,
		"name": "reportWidget",
		"label": "Tipo de Relatório",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};
	var reportFields = [
		{
			"id": 99999999,
			"name": "QR2TYPE",
			"label": "",
			"class": 12,
			"readOnly": false,
			"isVisible": true,
			"isVisibleForm": true,
			"isVisibleGrid": false,
			"template": "field/radio-group.html",
			"events": [],
			"valueField": "value",
			"descriptionField": "name",
			"defaultOption": 0,
			"dataSource": {
				"data": [
					{
						name: "pdf",
						value: "PDF"
					},
					{
						name: "xls",
						value: "XLS"
					},
					{
						name: "rtf",
						value: "RTF"
					},
					{
						name: "txt",
						value: "TXT"
					}
				]
			}
        }
    ];
	var reportActions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.generateReport(args.owner.widget, args);"
                }
            ]
        }
	];
	
	var reportCallback;
	var reportParameters;
	var reportWidgetParentName;

	this.openReportPopup = function (widget, callback, parameters, reportFormats) {

		reportCallback = callback;
		reportParameters = parameters || [];
		reportWidgetParentName = widget.name;
		reportWidget.container = widget.container;
		reportWidget.fields = reportFields;
		reportWidget.actions = reportActions;

		var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

		updateReportWidgetFormats(reportWidgetInitialized, reportFormats);

		ScreenService.openPopup(reportWidgetInitialized);
	};

	var updateReportWidgetFormats = function(reportWidget, reportFormats) {
		if (Array.isArray(reportFormats) && reportFormats.length) {
			var formatField = reportWidget.getField("QR2TYPE");
			formatField.dataSource.data = reportFormats;
		}
	};

	this.generateReport = function (widget, args) {

		var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

		reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

		args.row = reportWidgetParentCurrentRow;

		if (typeof reportCallback == 'function') {

			reportCallback.apply(null, reportParameters);
		}

		ScreenService.closePopup();

	};

	this.saveRow = function (args, closeSwipe) {
		args.owner.widget.saveRow().then(function () {
			var currentRow = args.owner.widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			args
				.owner
				.widget
				.reload();
			if (closeSwipe) {
				ScreenService.closeSwipe();
			}
		});
	};

	this.openSelectionOnce = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		if (!filterWidget.wasOpened) {
			filterWidget.edit();
			ScreenService.openPopup(filterWidget);
			filterWidget.wasOpened = true;
		}
	};

	this.openSelection = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		filterWidget.edit();
		ScreenService.openPopup(filterWidget);
	};

	this.getContainerName = function () {
		return templateManager.container.name;
	};

	this.backOrCancel = function (args) {
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(true);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}
		ScreenService.goBack();
		return deferred.promise;
	};

	this.edit = function (widget) {
		widget.backupRow = angular.copy(widget.currentRow);
		widget.edit();
	};

	this.modEdit = function (widget) {
		parent.edit(widget);
		var widgetParent = widget.widgets;
		if (widgetParent.length)
			parent.setVisibleforTabbed(widgetParent, false, widget);
		// widget.template = widget.formTemplate = widget.templateToForm;
	};

	this.cancelModEdit = function (args, widgetParent) {
		parent.backOrCancel(args).then(function (success) {
			var widget = args.owner.widget;
			var widgetParent = widget.widgets;
			if (widgetParent.length)
				parent.setVisibleforTabbed(widgetParent, true);
			// widget.template = widget.formTemplate = widget.templateToView;
		});
	};

	this.setVisibleforTabbed = function (widgets, isVisible, exception) {
		widgets.forEach(function (widget) {
			if (exception) {
				if (exception.name != widget.name)
					widget.isVisible = isVisible;
			} else {
				widget.isVisible = isVisible;
			}
		});
	};

	this.setContextVar = function (name, containerName, value) {
		localStorageEngine.setLocalVar(containerName + '_' + name, value);
	};

	this.getContextVar = function (name, containerName) {
		return localStorageEngine.getLocalVar(containerName + '_' + name);
	};

	var _getWidget = function (widgetName) {
		var isPopup = arguments[1] || null;
		if (!!templateManager.container) {
			var widget;
			if (!isPopup) {
				widget = templateManager.container.getWidget(widgetName);
			} else {
				widget = templateManager.container.getWidget(widgetName);
				if(widget)
					if(widget.hasOwnProperty('__filterPopup')) 
						widget = widget.__filterPopup;
					else{
						widget = widget.appliedFilter;
					}
			}
			if (widget) {
				return widget;
			}
		}
		return null;
	};

	var _getField = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var field = widget.getField(name);
		if (field) {
			return field;
		}
		return null;
	};

	var _getAction = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var action = widget.getAction(name);
		if (action) {
			return action;
		}
		return null;
	};

	this.getFieldValue = function (name, widgetName) {
		var isPopup = arguments[2] || null;
		var widget = _getWidget(widgetName, isPopup);
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		} else {
			return "";
		}
	};

	this.setEnableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = false;
	};

	this.setDisableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = true;
	};

	this.setVisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = true;
	};

	this.setInvisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = false;
	};

	this.setEnableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = false;
	};

	this.setDisableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = true;
	};

	this.setVisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = true;
	};

	this.setInvisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = false;
	};

	this.now = function () {
		return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
	};

	this.today = function () {
		return moment(new Date()).format('DD/MM/YYYY');
	};

	this.openReport = function (route, reportName, params) {
		params.__report_name = reportName;
		
		var lastActiveWidget = templateManager.container.getActiveWidget().lastActiveWidget;
	
		if (lastActiveWidget && lastActiveWidget.beforeOpenReport) {
		    lastActiveWidget.beforeOpenReport({
		        reportParams: params
		    });
		}
		
		return restEngine.post(route, {
			"requestType": "Row",
			"row": params
		}, null, null, null, null, 3000000).then(function (response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				var reportOpened = window.open(report.message);
				if(!reportOpened){
					MessageService.showMessageByCode('lib#GEN_POPUP_BLOCKER', null, { type: 'ERROR' } );
				}
			}
		}).catch(function (response) {
			if (response.data && response.data.error) {
				ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
			}
		});
	};

	// CRUD
	/**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function (widget, afterValid) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
					widget.reload().then(function () {
						deferred.resolve(success);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function (widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
					" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		} catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function (arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			} else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.deleteCrud = function (widget) {
		var deferred = ZHPromise.defer();
		MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};

    this.deleteCrudCheckedRows = function (widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

	this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}

	function preventEvent(e) {
		e.preventDefault();
	}

	function padLeft(value, length) {
		return (value.toString().length < length) ? padLeft("0" + value, length) : value;
	}

	function padRight(value, length) {
		return (value.toString().length < length) ? padRight(value + "0", length) : value;
	}

	this.getCurrentDate = function (format) {
		return moment().format(format);
	};

	this.setDateAttrToVarcharOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
	};

	this.setAttrOnCurrentRow = function (currentRow, attrName, value) {
		currentRow[attrName] = value;
	};

	this.setFloatAttrOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
	};

	this.valueToFloat = function (value) {
		if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
			if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
				return parseFloat(value);
			} else if (value.indexOf(',') > value.indexOf('.')) {
				return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
			} else if (value.indexOf(',') < value.indexOf('.')) {
				return parseFloat(value.replace(/\,/g, ''));
			}
		} else {
			return value;
		}
	};

	this.toLocalDate = function (element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	this.toLocalDateFormat = function (value, params) {
		if (value) {
			var valueDate = moment(value, params.format);
			value = valueDate.format(params.toFormat);
		}

		return value;
	};

	maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

	var maskFunction = function (element, viewValue, params, callBack) {
		// do something
	};

	var maskFormatFunction = function (value, params) {
		if (value) {
			var length = value.length;
			if (length === 11)
				value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
			else if (length === 14)
				value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
			return value;
		}
	};

	maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

	/*
	 * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
	 * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
	 * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
	 */

	var constAllWidgets;

	this.setVisibilityByProjectID = function (widgets) {
		if (widgets.length > 1) {
			widgets.forEach(function (childWidget) {
				parent.setVisibilityByProjectID(childWidget.widgets);
				if (childWidget.projects) {
					childWidget.projects.forEach(function (proj) {
						childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
					});
				}
			});

		}

	};

	this.validateFilterPopup = function (args) {
		if(args.owner.hasOwnProperty('__filterPopup')){
			var filterPopup = args.owner.__filterPopup;
		
		if (!filterPopup.isValid())
			args.e.preventDefault();
		}	
	};

	var timeFormats = {
		"hh": isValidHour,
		"mm": isValidMinutesOrSeconds,
		"ss": isValidMinutesOrSeconds,
		"hh:mm": isValidHourAndMinutes,
		"mm:ss": isValidMinutesAndSeconds,
		"hh:mm:ss": isValidTime,
	};

	function isValidHour(hour) {
		if (parseInt(hour) > 23 || parseInt(hour) < 0)
			return false;
		else
			return true;
	}

	function isValidMinutesOrSeconds(minuteOrSecond) {
		if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
			return false;
		else
			return true;
	}

	function isValidHourAndMinutes(hourAndMinute) {
		if (hourAndMinute) {
			var timeArray = hourAndMinute.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidMinutesAndSeconds(minutesAndSeconds) {
		if (minutesAndSeconds) {
			var timeArray = minutesAndSeconds.split(':');
			return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidTime(time) {
		if (time) {
			var timeArray = time.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
		}
		return true;
	}

	var validTime = function (value, format) {
		var validFunction;
		if (!format)
			throw 'format is not defined';
		if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
			validFunction = timeFormats[format.toLowerCase()];
		else
			throw 'time format is not registered';
		return {
			"valid": validFunction(value),
			"message": ScreenService.i18n("Não é um tempo válido.")
		};
	};

	this.registerTimeFormat = function (format, formatFunction) {
		if (typeof format !== 'string')
			throw 'time format must be an string';
		if (timeFormats.hasOwnProperty(format.toLowerCase()))
			throw 'time format already registered';
		timeFormats[format] = formatFunction;
	};

	ValidationEngine.registerValidation('isValidTime', validTime);

	/*
	 * Função que preenche um valor com um único caractere até um certo tamanho.
	 * Faz zerofill pela esquerda por default.
	 *
	 * @param string value
	 * @param int length
	 * @param boolean toRight
	 * @param char filler
	 *
	 * @return string
	 *
	 * Exemplos:
	 * General.fill('123', 10);            //'0000000123'
	 * General.fill('123', 10, true);      //'1230000000'
	 * General.fill('123', 10, null, '_'); //'_______123'
	 */
	this.fill = function (value, length, toRight, filler) {
		if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight)
				while (value.length + filler.length <= length) value = value + filler;
			else
				while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
	};

	/*
	 * Função que faz a validação de CNPJ.
	 *
	 * @param string CNPJ
	 *
	 * @return boolean
	 */
	function validCNPJ(CNPJ) {
		if (CNPJ === undefined || CNPJ === null)
			return false;

		CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

		if (CNPJ.length != 14)
			return false;

		if (CNPJ == "00000000000000" ||
			CNPJ == "11111111111111" ||
			CNPJ == "22222222222222" ||
			CNPJ == "33333333333333" ||
			CNPJ == "44444444444444" ||
			CNPJ == "55555555555555" ||
			CNPJ == "66666666666666" ||
			CNPJ == "77777777777777" ||
			CNPJ == "88888888888888" ||
			CNPJ == "99999999999999")
			return false;

		var tamanho = CNPJ.length - 2;
		var numeros = CNPJ.substring(0, tamanho);
		var digitos = CNPJ.substring(tamanho);
		var soma = 0;
		var pos = tamanho - 7;

		for (var i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(0)) //debugger;
			return false; //'resultado' é o 1º digito de validação do CNPJ

		tamanho = tamanho + 1;
		numeros = CNPJ.substring(0, tamanho);
		soma = 0;
		pos = tamanho - 7;

		for (i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(1)) //debugger;
			return false; //'resultado' é o 2º digito de validação do CNPJ

		return true;
	}

	this.validateCNPJ = validCNPJ;

	/*
	 * Função que faz a validação de CPF.
	 *
	 * @param string CPF
	 *
	 * @return boolean
	 */
	function validCPF(CPF) {
		if (CPF === undefined || CPF === null)
			return false;

		CPF = CPF.toString().replace(/[\D]+/g, '');

		if (CPF.length != 11)
			return false;

		if (CPF == "00000000000" ||
			CPF == "11111111111" ||
			CPF == "22222222222" ||
			CPF == "33333333333" ||
			CPF == "44444444444" ||
			CPF == "55555555555" ||
			CPF == "66666666666" ||
			CPF == "77777777777" ||
			CPF == "88888888888" ||
			CPF == "99999999999")
			return false;

		var soma = 0;
		var resto;

		for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(9, 10))) //debugger;
			return false; //'resto' é o 1º dígito de validação do CPF

		soma = 0;
		for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(10, 11))) //debugger;
			return false; //'resto' é o 2º dígito de validação do CPF

		return true;
	}

	this.validateCPF = validCPF;

	/*
	 * Função que faz a formatação de uma inscrição,
	 * seguindo o padrão de código genérico, CPF, CNPJ e
	 * inscrição genérica. Caso o parâmetro "force" for
	 * true, o método tenta forçar uma inscrição válida.
	 *
	 * Retorna um objeto com as seguintes propriedades:
	 *  * parameter //string de entrada contendo apenas números
	 *  * value     //string formatada conforme o tipo da inscrição
	 *  * tipo.cod  //char do código padrão do tipo da inscrição
	 *  * tipo.desc //string de descrição do tipo da inscrição
	 *  * valid     //boolean
	 *
	 * @param string inscricao
	 * @param boolean force
	 *
	 * @return object
	 */
	this.inscricaoFormat = function (inscricao, force) {
		if (inscricao === undefined || inscricao === null || inscricao === '')
			return {
				'parameter': '',
				'value': '',
				'tipo': {
					'cod': null,
					'desc': null
				},
				'valid': false
			};

		if (force === undefined || force === null)
			force = false;
		inscricao = inscricao.toString().replace(/[\D]+/g, '');

		var obj = {};
		obj.tipo = {};

		if (inscricao.length <= 5) {
			obj.tipo.cod = 'O';
			obj.tipo.desc = 'Código Genérico';
			obj.valid = true;
			obj.value = this.fill(inscricao, 5);
		} else {
			if (inscricao.length == 11) {
				obj.tipo.cod = 'F';
				obj.tipo.desc = 'Pessoa Física';
				obj.valid = this.validateCPF(inscricao);
				obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
			} else if (inscricao.length == 14) {
				obj.tipo.cod = 'J';
				obj.tipo.desc = 'Pessoa Jurídica';
				obj.valid = this.validateCNPJ(inscricao);
				obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
			} else force = true;

			if (force && obj.valid !== true) {
				obj.tipo.cod = 'I';
				obj.tipo.desc = 'Inscrição Internacional';

				if (inscricao.length <= 20) {
					obj.valid = true;
					obj.value = this.fill(inscricao, 20);
				} else {
					obj.valid = false;
					obj.value = inscricao.substr(0, 20);
				}
			}
		}

		obj.parameter = inscricao;
		return obj;
	};

	var CPFValidation = function (value, validationAttrs) {
		return {
			"valid": validCPF(value),
			"message": ScreenService.i18n('CPF Inválido')
		};
	};

	ValidationEngine.registerValidation('CPF', CPFValidation);

	var CNPJValidation = function (value, validationAttrs) {
		return {
			"valid": validCNPJ(value),
			"message": ScreenService.i18n('CNPJ Inválido')
		};
	};

	ValidationEngine.registerValidation('CNPJ', CNPJValidation);

	this.callRestEngine = function (dataSource, requestType, requestValue, disableLoader) {
		var defer = ZHPromise.defer();
		var params = {};
		
		switch (requestType) {
		case "Row":
			params = {
				"requestType": "Row",
				"row": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "DataSet":
			params = {
				"requestType": "DataSet",
				"dataset": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "FilterData":
			params = {
				"requestType": "FilterData",
				"filter": requestValue,
				"disableLoader": disableLoader
			};
			break;

		default:
			return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
		}

		return restEngine.request(dataSource, params);
	};

	/*
	 * Função que muda a acessibilidade de uma ou mais
	 * widgets.
	 *
	 * @param array(objects) widgets
	 * @param boolean enabled
	 *
	 * Exemplos:
	 * General.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
	 * General.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
	 */
	this.changeWidgetAcessibility = function (widgets, enabled) {
		widgets.forEach(function (widget) {
			widget.enabled = enabled;
		});

		templateManager.updateTemplate();
	};

	/*
	 * Função que altera o template de um form
	 * entre edição e visualização. Retorna um boolean
	 * que indica se a widget era do tipo form e a
	 * alteração foi realizada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.formWidgetSwitchTemplate = function (widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	/*
	 * Função que coloca uma widget em modo de edição,
	 * bloqueando as demais widgets. Retorna um boolean
	 * que indica se a alteração de modo foi executada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.widgetEdit = function (widget) {
		if (!widget.editing) {
			//Trata a edição de uma widget filha do tipo form
			if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
				var widgetsToDeactivate = widget.parent.widgets;

				this.changeWidgetAcessibility(
					widgetsToDeactivate.filter(function (subwidget) {
						return subwidget.name != widget.name;
					}).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
					false);
			} else //trata a edição dos demais casos
				this.changeWidgetAcessibility(widget.widgets, false);

			//propriedades a serem ignoradas na criação do backup
			var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

			//faz backup da currentRow para permitir a recuperação dos dados
			widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
			widget.edit();
			return true;
		} else return false;
	};

	/*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * própria widget. Provavelmente usado na action de
	 * 'voltar' do back_view.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToList = function (widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * widget parent. Provavelmente usado na action de
	 * 'voltar' do back_list.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToParent = function (widget) {
		ScreenService.closeSwipe().then(function () {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	/*
	 * Procedimento que habilita inserção em uma widget,
	 * bloqueando as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetNewRow = function (widget) {
		this.changeWidgetAcessibility(widget.widgets, false);

		widget.newRow();
	};

	/*
	 * Função que salva os dados de uma widget e retorna
	 * para o modo adequado de visualização ou listagem.
	 * Faz exibição de mensagens de aviso para o usuário
	 * conforme padronização e desbloqueia as demais
	 * widgets.
	 *
	 * Basicamente, um saveCrud adaptado ao padrão.
	 *
	 * @param object widget
	 *
	 * @return promise
	 */
	this.widgetSave = function (widget) {
		var deferred = ZHPromise.defer();

		if (widget.isValid()) {
			widget.currentRow.changed = true;

			if (!widget.currentRow.__is_new)
				widget.currentRow.__is_new = false;
			else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = widget.paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                	widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
			}

			widget.saveRow().then(function (response) {
				widget.currentRow.changed = false;
				delete widget.backupRow;

				if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
					widget.view();

					var widgets = widget.parent.widgets;
					this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
				} else {
					if (widget.currentRow.__is_new) {
						ScreenService.closeSwipe();
						widget.list();
					} else widget.view();

					this.changeWidgetAcessibility(widget.widgets, true);
				}

				if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
				else widget.reload();

				deferred.resolve(response);
			}.bind(this), function (error) {
				var errorMessage = 'Timeout';

				if (error.data) errorMessage = error.data.error;

				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
	};

	/*
	 * Função para validar containers abertos a partir de outros containers.
	 * Se o container atual não foi aberto a partir do container pai, a
	 * callback é executada. Se nenhuma callback for passada nos parâmetros,
	 * o container pai é aberto.
	 *
	 * @param: object {parentContainerName, callback, scope, params}
	 *
	 * Exemplos:
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
	 */
	this.checkParentContainer = function (containerProperties) {
		containers = templateManager.containers;
		if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
			if (angular.isFunction(containerProperties.callback)) {
				containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
			} else {
				ScreenService.openWindow(containerProperties.parentContainerName);
			}
		}
	};

	var messagesWidget = {
		"id": 0,
		"name": "messagesWidget",
		"label": "Mensagens",
		"messagesDataset": "",
		"template": "widget/form.html",
		"floatingControl": {
			"customizationAction": {
				"filter": false,
				"view": false,
				"layout": false
			}
		},
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};

	var messagesFields = [{
		"label": "",
		"name": "CONTENT",
		"readOnly": true,
		"template": "field/textarea.html",
		"textRows": 14,
		"isVisible": true,
		"class": 12,
		"descriptionOrder": true,
		"showOnView": true,
		"showOnForm": true,
		"showOnList": true
    }];

	var messagesActions = [{
		"name": "cancelar",
		"label": "Cancelar",
		"idVisible": true,
		"hideIcon": true,
		"showAsAction": "back",
		"events": [{
			"name": "ActionEvent",
			"code": "ScreenService.closePopup();"
        }]
    }, {
		"name": "exportar",
		"label": "Exportar",
		"idVisible": true,
		"icon": "export",
		"showAsAction": "always",
		"events": [{
			"name": "ActionEvent",
			"code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
        }]
    }];

	/* Função abre um popup com um Text Area a título de mostrar erros em lista.
	 * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
	 * @params: widget, dataset, popupName
	 */
	this.showErrorMessages = function (widget, dataset, popupName) {
		messagesWidget.container = widget.container;
		messagesWidget.fields = messagesFields;
		messagesWidget.actions = messagesActions;

		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

	/* Gera arquivo básico baseado em linhas de dataSet.
	 * @params: dataset, object {fileName}
	 * !!! fileExtension: por enquanto somente TXT;
	 * Exemplo:
	 * General.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
	 */

	this.generateFileByDataSet = function (dataset, fileProperties) {
		var content = "";
		var name = 'content';
		var extension = 'text/plain';

		if (fileProperties !== undefined) {
			if (fileProperties.fileName !== undefined)
				name = fileProperties.fileName;
		}

		dataset.forEach(function (line) {
			content += line + "\r\n";
		});

		var element = document.createElement('a');
		element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
		element.setAttribute('download', name);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

	/**
	 * Este método verifica se a widget foi filtrada usando o zh-grid-filter
	 * @param Widget widget
	 * @return boolean
	 */
	this.isWidgetFiltered = function (widget) {
		var isFiltered = true;
		var currentFilter;
		if(widget.hasOwnProperty('__filterPopup'))
			currentFilter = widget.__filterPopup.currentFilter;
		else
			currentFilter = widget.appliedFilter.FILTER_ROW;
			
		var requiredFields = widget.fields.filter(function (field){
			return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
		});
		if(requiredFields.length && Array.isArray(currentFilter)) {
			isFiltered = false;
		}
		return isFiltered;
	};

	function isRequiredFieldInFilter(field) {
		return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
	}

	function isRequiredFieldWithoutFilterProperties(field) {
		return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
	}

	/**
	 * Este método valida a inserção de registros na widget
	 * @param Widget widget
	 */
	this.newRow = function (widget) {
		if(this.isWidgetFiltered(widget)) {
			widget.newRow();
		} else {
			MessageService.showMessageByCode('lib#GEN_NOT_FILTER', null, { type: 'ERROR' } );
		}
	};

    /*
     * Função que adiciona à currentRow o(s) valor(es)
     * da widget parent necessários para salvar o registro.
     *
     * @param object widget
     * @param object parents
     *
     * @return promise
     */

    this.saveWithParent = function(widget, parents) {
        if(Array.isArray(parents)) {
            parents.forEach(function(parent){
                widget.currentRow[parent] = widget.parent.currentRow[parent];
            });
            return this.widgetSave(widget);
        }
        else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
    };
    
    /*
     * Aceita [- . , 0 1 2 3 4 5 6 7 8 9]
     *
     */
    
     this.justDigits = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonDigits;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justDigits(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonDigits(e) {
        var digits = [44, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var digitCode = e.which;
        if (digits.indexOf(digitCode) < 0) {
            preventEvent(e);
        }
    }

    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousands, precision) {
    	precision = precision || 2;
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousands": thousands,
            "precision": precision
        };
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    /**
     * Função responsável por aplicar uma máscara do type currency
     * no field.
     * 
     * @param {String} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.' || value[0] === ',')
                value = '0' + value;
            if (params.decimal === ',')
                value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            var counter = angular.copy(int.length / 3);

            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');
            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('maskCurrency', maskFunction, this.maskCurrency);
    
     /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} value - Valor numérico.
     */

    this.parseFloat = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/\,/g.test(value))
            value = value.toString().replace(',', '.');
        return parseFloat(value);
    };
    
    /**
     * Função que formata números float vindos do banco, ou em
     * padrão americano e passa para o formato brasileiro.
     * 
     * @param {String} value - Valor numérico.
     */
    
    var numberFormatBr = function(value) {
        if (/^(.*)(\,+).*(\.+)(.*)$/gm.test(value)) {
            var loop = value.toString().split(',');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace(',', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
		else if(/^(\d*)([\.]*)((\d|\.)*)$/.test(value)){
			value = value.split('.');
			value[value.length-1] = ','+value[value.length-1];
			value = value.join('');
		}
        return value.toString();
    };
    
    /**
     * Função criada para corrigir o erro da maxlength.
     * Em vez do maxlength deve ser usada a propriedade maxLengthCurrency,
     * que considera apenas os números inteiros como length.
     * 
     * Função deve ser usada no evento FieldOnKeydown.
     * 
     * @param {Args} args - objeto args.
     * @param {Field} field - field com a propriedade maxLengthCurrency.
     */
    
    this.lengthCurrency = function(args, field) {
        var value;
        var fieldValue;
        
        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = field.widget.currentRow[field.name + '_inline'];
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        } else {
            fieldValue = field.value();
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        }
    
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        
        if (realLength >= field.maxLengthCurrency && !/^(\d+)(\,){1}(\d*)$/gm.test(value)) {
            var allowedKeys = [8, 37, 38, 39, 40, 46, 110, 188, 190, 194];
            if (!allowedKeys.includes(args.e.keyCode))
                args.e.preventDefault();
        }
    };
    
    var treatResponse = function(response, Authenticator) { //
		var url = response.responseURL;
	    var responseURL = null;
	    var responseData = response.response;
	    
	    if (url.indexOf('index.php/') > -1) {
	        responseURL = url.slice(url.indexOf('index.php/') + 9);
	    }
	    
		if(responseURL){
			var responseObj = JSON.parse(responseData);
	        var errorCode = responseObj.error;
			if(Authenticator){
				Authenticator.run(responseURL, responseObj, errorCode);
			} else if(errorCode && errorCode.indexOf('AUTH_') > -1){// Exception: AUTH_INVALID_UNIT 
	            var cleanError = parent.getCleanError(errorCode);
				MessageService.showMessageByCode(cleanError);
			}
		}
	};
	
	this.getCleanError = function(msg){
		return msg.match(/AUTH_[^\s]+/g)[0];
	};
	
	/**
	 * Overwrite XMLHttpRequest.prototype.open
	 * and add a listener on it.
	 */
	this.initRoutesListener = function(Authenticator) {
	    (function(open) {
	        XMLHttpRequest.prototype.open = function(method, url, async, user, pass) {
	            this.addEventListener("readystatechange", function() {
	            	try{
	                	treatResponse(this, Authenticator);
	            	} catch(e){
	            		return false;
	            	}
	            }, false);
	
	            open.call(this, method, url, async, user, pass);
	        };
	    })(XMLHttpRequest.prototype.open);
	};
	
};

Configuration(function (ContextRegister) {
	ContextRegister.register('General', General);
});

// FILE: js/Util/LogDecorator.js
function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const PRODUCT_ID_FIELD_NAME         = 'PRODUCT_ID';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const TYPE_FIELD_NAME               = 'TYPE';
    const CONTAINER_FIELD_NAME_NAME     = 'CONTAINER_NAME';
    const WIDGET_NAME_FIELD_NAME        = 'WIDGET_NAME';
    const ORGANIZATION_ID_FIELD_NAME    = 'ORGANIZATION_ID';

    /* Field Labels */
    const USER_NAME_FIELD_LABEL    = 'User Name';
    const ROUTE_LABEL_FIELD_LABEL  = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE       = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE   = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';
    const DATE_INTERVAL_TEMPLATE   = 'zh-interval-component#field/interval.html';
    const DATETIME_TEMPLATE        = 'field/datetime.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate          = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userNameField       = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userNameField.name     = USER_NAME_FIELD_NAME;
        userNameField.label    = USER_NAME_FIELD_LABEL;
        routeLabelField.name   = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label  = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.moveField(widget, USER_ID_FIELD_NAME,        1);
        ZhuLogHelper.addField (widget, userNameField,             2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,           3);
        ZhuLogHelper.moveField(widget, CONTAINER_FIELD_NAME_NAME, 4);
        ZhuLogHelper.moveField(widget, WIDGET_NAME_FIELD_NAME,    5);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,          6);
        ZhuLogHelper.addField (widget, routeLabelField,           7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            },
            showOnList: true,
            maxWidth: '12%',
            class: 3
        });

        ZhuLogHelper.editField(widget, USER_ID_FIELD_NAME,        { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, USER_NAME_FIELD_NAME,      { showOnList: true, maxWidth: '15%', class: 4 });
        ZhuLogHelper.editField(widget, CONTAINER_FIELD_NAME_NAME, { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, WIDGET_NAME_FIELD_NAME,    { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TIME_FIELD_NAME,           { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, ROUTE_FIELD_NAME,          { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TYPE_FIELD_NAME,           { showOnList: true, maxWidth: '8%' , class: 3 });

        ZhuLogHelper.removeField(widget, PRODUCT_ID_FIELD_NAME);
        ZhuLogHelper.removeField(widget, ORGANIZATION_ID_FIELD_NAME);
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }
            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.editField(gridWidget, TIME_FIELD_NAME, {
            template: DATETIME_TEMPLATE,
            dateFormat: "DD/MM/YYYY",
            timeFormat: "HH:mm:ss",
            displayFormat: "DD/MM/YYYY HH:mm:ss",
            "datetime-dateFormat": "dd/mm/yy",
            "datetime-timeFormat": "HH:ii:ss",
            filterProperties: {
                isVisible: true,
                showOnForm: true,
                label: 'Date',
                template: DATE_INTERVAL_TEMPLATE,
                dateFormat: "DD/MM/YYYY",
                displayFormat: "DD/MM/YYYY",
                intervals: [ {
                    "label": "Última semana",
                    "period": "last-week"
                }, {
                    "label": "Último mês",
                    "period": "last-month"
                } ]
            }
        });

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, { class: 4 });
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, { class: 2 });
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       { class: 3 });
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP: {
                start : moment().add(-1, 'days').format('DD/MM/YYYY'),
                end   : moment().format('DD/MM/YYYY'),
                period: undefined
            }
        };
    };

    this.normalizeRows    = function(rows) { return rows; };
    this.normalizeFilters = function(filters) {};
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});


// FILE: js/Util/MenuBuilder.js
var MenuBuilder = function (restEngine, ZHPromise, templateManager, metaDataFactory, eventAggregator, $window, requestDataSourceEngine, ApplicationContext) {
    var localProductMenu;
    var attrToReplace = [
        "label",
        "showMenu",
        "showHeader",
        "showDescriptionOrder",
        "onRequestIdleTimeout",
        "mainWindow",
        "logLevel",
        "aclEnable",
        "showMenuSearch",
        "userDataActions",
        "reportServiceUrl",
        "reportsPath",
        "inDeveloperMode",
        "showDevtoolsInSameWindow",
        "shouldBackToMainWindowOnReload",
        "shortcutKeys"
    ];

    var requestProductMenu = function () {
        var modulesPackage = $window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/containers.json").then(function (response) {
            var productObjMenu = response.data;
            var keys = Object.keys(productObjMenu);
            // get first project menu found
            return productObjMenu[keys[0]];
        });
    };

    var removeEmptyMenuGroups = function(groupMenus) {
        return groupMenus.filter(function(group) {
            return (group.menus && group.menus.length != 0) || group.name == "zhHistory";
        });
    };

    var orderMenus = function(menus) {
        menus.map(function(menu) {
            if(menu.menus) {
                menu.menus = orderMenus(menu.menus);
            }
            return menu;
        });

        return sortMenus(menus);
    };

    var sortMenus = function(menus) {
        return menus.sort(function(menuA, menuB) {
            if (menuA.order && menuB.order) {
                if (menuA.order < menuB.order) {
                    return -1;
                }
                if (menuA.order > menuB.order) {
                    return 1;
                }
            }
            if (!menuA.order && menuB.order) {
                return -1;
            }
            if (menuA.order && menuB.order) {
                return 1;
            }
            return 0;
        });
    };

    var prepareMenus = function (response) {
        var groupMenus = response.dataset.menu;
        window.routesTranslate = response.dataset.routesTranslate;
        var zhuLogWidget = templateManager.container ? templateManager.container.getWidget('zhuLog') : false;
        if(zhuLogWidget) {
            zhuLogWidget.getField('ROUTE_LABEL').dataSource.data = window.routesTranslate;
            $(window).trigger('resize.grid');
        }
        groupMenus = orderMenus(removeEmptyMenuGroups(groupMenus));
        var groupMenuHistory = [templateManager.project.getMenu("zhHistory")];
        templateManager.project.groupMenu = metaDataFactory.groupMenuFactory(groupMenus.concat(groupMenuHistory));

        //Force framework to apply ACL Rules
        var aux = {};
        Object.keys(templateManager.aclRules).forEach(function(ruleKey){
            aux[ruleKey] = templateManager.aclRules[ruleKey];
        });
        templateManager.aclRules = aux;
        //Force framework to apply ACL Rules

        eventAggregator.publish('menuBuilded');
    };

    var replaceProductAttrOnMenu = function (productMenu) {
        attrToReplace.forEach(function (attr) {
            templateManager.project[attr] = productMenu[attr];
        });
    };

    var setLocalProductMenu = function () {
        if (!$window.modulesPackage) {
            requestProductMenu().then(function (productMenu) {
                localProductMenu = productMenu;
            });
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    if(!window.notUseMicroservice) {
        eventAggregator.subscribe('loginSuccess', function (loginResponse) {
            if (!$window.modulesPackage && !$window.keepTitle) {
                replaceProductAttrOnMenu(localProductMenu);
                templateManager.updateTitle(templateManager.project.label);
            }
            var filter = [{
                "name": "CONCATENATE_GROUP_MENU",
                "operator" : "=",
                "value" :  true
            }, {
                "name": "VISIBLE_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.VISIBLE_ELEMENTS || []
            }, {
                "name": "EXPIRED_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.EXPIRED_ELEMENTS || []
            }, {
                "name": "URL_ORIGIN",
                "operator" : "=",
                "value" :  window.location.origin
            }, {
                "name": "CURRENT_MODULE",
                "operator" : "=",
                "value" :  getCurrentModule()
            }, {
                "name": "CURRENT_LANGUAGE",
                "operator" : "=",
                "value" :  window.lang
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_buildMenu', filter)
                .then(prepareMenus)
                .catch(ApplicationContext.LibUtilities.showErrorMessage);
        });

        setLocalProductMenu();
    }
};

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuBuilder', MenuBuilder);
});

// FILE: js/Util/Messages.js
function LibMessages(){
    this.ALE_EXPIRED_SESSION = 'Sua sessão expirou.<br/>Você será redirecionado para efetuar login novamente.';
    this.ALE_MIGRATION_IS_PENDENT = 'ATENÇÃO!<br/><br/>Esse produto foi atualizado recentemente, com necessidade de alterações na estrutura do banco de dados, sem que essas alterações tenham sido aplicadas.<br/>A utilização do produto nessa situação poderá acarretar erros.<br/><br/>Solicite ao administrador do sistema a regularização da alteração na estrutura do bando de dados.', 'INFORMATION';

    this.ERR_ATTEMPT_END = 'Tentativas esgotadas. Redirecionando...';
    this.ERR_ATTEMPT_PARTIAL = 'Senha incorreta.<br/>Tentativa %1/3';
    this.ERR_EXPIRED_SESSION_LOGIN = 'Sessão expirada. Redirecionando...';
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibMessages', LibMessages);
});


// FILE: js/Util/ReportDecorator.js
function ReportDecorator(eventAggregator, restEngine, templateManager, ZhLogger, ScreenService, eventEngine) {
    
    this.onContainerOpen = function() {
        try {
            var container = templateManager.container;
            validContainer(container);
            if (container.customReportsConfig) {
                var customReportsConfig = container.customReportsConfig;
                validCustomReportsConfig(customReportsConfig);
                var widget = selectWidget(container, customReportsConfig.widgetName);
                var containerName = container.customReportsConfig ? container.customReportsConfig.reportCodeName : container.name;
                
                var fieldToChangeData = selectField(widget, customReportsConfig.fieldName);
                
                if (customReportsConfig.reportIsTheMainFunction === undefined || customReportsConfig.reportIsTheMainFunction === true) {
                    buildData(containerName, fieldToChangeData);
                } else {
                    const separator = "', '";
                    const eventCode = "ReportDecorator.triggerBuildData('" + containerName + separator + customReportsConfig.widgetName + separator + customReportsConfig.fieldName + "')";
                    
                    addEvent({
                        entity: widget,
                        event: {
                            name: 'WidgetOnEnterOnce',
                            code: eventCode,
                            bind: true,
                        }
                    });
                }
                widget.beforeOpenReport = beforeOpenReport;
            }
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    eventAggregator.subscribe('onContainerOpen',this.onContainerOpen);
    
    this.triggerBuildData = function(containerName, widgetName, fieldName) {
        const container = templateManager.container;
        
        validContainer(container);
        
        const widget = selectWidget(container, widgetName);
        const fieldToChangeData = selectField(widget, fieldName);
        
        try {
            buildData(containerName, fieldToChangeData);
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    
    function buildData(containerName, fieldToChangeData) {
        const route = getServiceUrlBasedOnTriggerPackage() + '#/lib_getDataReportField';
        
        restEngine.post(route, {
            "requestType": "Row",
            "row": [containerName.substr(0, 6)]
        }).then(function (callback) {
            try {
                var data = callback.dataset.dataReport;
                
                handleErrors(data);
                
                const formattedData = getFormattedData(fieldToChangeData, data);
                
                setDataSource(fieldToChangeData, formattedData);
                applyDefaultValue(fieldToChangeData);
                
                /**
                 * Força o select do item para que haja alteração visual.
                 * Operações visuais como updateTemplate e redraw não surtiram efeitos.
                 */
                setTimeout(function() {
                    clickOnSelectedOption(fieldToChangeData.name);
                }, 500);
                
                addEvent({
                    entity: fieldToChangeData.getParent(),
                    event: {
                        name: 'WidgetOnEnter',
                        code: "ReportDecorator.onEnterReportWidget('" + fieldToChangeData.name + "')",
                        bind: true,
                    }
                });
            } catch (e) {
                ZhLogger.warning(e);
            }
        }.bind(this), function(error) {
            if (error.data && error.data.error) {
                throw error.data.error;
            } else {
                throw 'Ocorreu um erro ao buscar a lista de relátorios da tela.';
            }
        });
    }
    
    function getServiceUrlBasedOnTriggerPackage() {
        const fileName = templateManager.container.fileName;
        const aliasSplited = fileName.split('#').shift().split('_');
        
        if (aliasSplited.length) {
            const packageName = aliasSplited.shift();
            
            if (packageName) return packageName;
        }
        
        return 'lib';
    }
    
    function getFormattedData(fieldToChangeData, data) {
        return data
                .filter(function(reportElement) {
                    return !Array.isArray(reportElement);
                })
                .map(function(reportElement) {
                    if (Array.isArray(reportElement)) return;
                    
                    const newElement = {};
                    
                    newElement[fieldToChangeData.valueField] = reportElement.value;
                    newElement[fieldToChangeData.descriptionField] = reportElement.description;
                    newElement[fieldToChangeData.strategyField] = reportElement.strategy;
                    
                    return newElement;
                });
    }
    
    function handleErrors(data) {
        const errorIndex = data.findIndex(function(reportElement) {
            return Array.isArray(reportElement);
        });
        
        if (errorIndex === -1) return;
            
        if (data.length === 1) {
            const message = 'Ocorreu um erro ao buscar a lista de relatórios da tela.';
            
            ScreenService.errorNotification(message);
            
            throw message + 'Consulte a requisição correspondente para mais detalhes.';
        } else {
            const isQr2Parametrized = data[errorIndex].findIndex(function(error) {
                return error === 'Servidor de relatórios QR2 não está parametrizado.';
            }) === -1;
            
            const isBirtParametrized = data[errorIndex].findIndex(function(error) {
                return error === 'Servidor de relatórios BIRT não está parametrizado.';
            }) === -1;
            
            if (isQr2Parametrized || isBirtParametrized) {
                ScreenService.errorNotification('Alguns relatórios podem não estar disponíveis.');
            }
            
        	ZhLogger.warning('Alguns relatórios podem não estar disponíveis. Consulte a requisição correspondente para mais detalhes.');
        }
    }
    
    function setDataSource(fieldToChangeData, data) {
        if (!fieldToChangeData.dataSource.data || fieldToChangeData.dataSource.data.length == 0) {
            fieldToChangeData.dataSource.data = data;
        } else {
            const valueField = fieldToChangeData.valueField;
            const descriptionField = fieldToChangeData.descriptionField;
            const strategyField = fieldToChangeData.strategyField;
            
            const removingDuplicatedReports = function(report, index, reports) {
                return index === reports.findIndex(function(r) {
                    return r[valueField] === report[valueField] &&
                           r[descriptionField] === report[descriptionField] &&
                           r[strategyField] === report[strategyField];
                });
            };
            
            const newData = fieldToChangeData.dataSource.data.concat(data);
            
            fieldToChangeData.dataSource.data = newData.filter(removingDuplicatedReports);
        }
    }
    
    function applyDefaultValue(fieldToChangeData) {
        if (!fieldToChangeData.defaultValue || fieldToChangeData.defaultValue === 'firstRow') {
            const firstRow = fieldToChangeData.dataSource.getFirstRow();
            
            fieldToChangeData.defaultValue = firstRow[fieldToChangeData.valueField];
        }
        
        fieldToChangeData.applyDefaultValue();
    }
    
    function clickOnSelectedOption(fieldName) {
        const baseSelector = "#" + fieldName + " + div";
        const baseElement = document.querySelector(baseSelector);
        
        if (!baseElement) {
            return setTimeout(function recursiveCallClickOnSelectedOption() {
                clickOnSelectedOption(fieldName);
            }, 500);
        }
        
        document.querySelector(baseSelector).click();
        
        const selectedElementSelector = baseSelector + ' li[class*="selected"]';
        
        document.querySelector(selectedElementSelector).click();
    }
    
    function selectWidget(container, widgetName) {
        var widget = container.getWidget(widgetName);
        
        if (widget) {
            return widget;
        } else {
            throw "A widget informada não existe dentro desse container.";
        }
    }
    
    function selectField(widget, fieldName) {
        var field = widget.getField(fieldName);
        
        if (field) {
            if (field.dataSource) {
                return field;
            } else {
                throw "O field informado não contém a propriedade dataSource.";    
            }
        } else {
            throw "O field informado não existe na widget informada.";
        }
    }
    
    function validContainer(container) {
        if (!container) {
            throw "Container não existe.";
        }
    }
    
    function validCustomReportsConfig(config) {
        if (!config.widgetName) {
            throw 'A propriedade widgetName não existe dentro de customReportsConfig.';
        }
        
        if (!config.fieldName) {
            throw 'A propriedade fieldName não existe dentro de customReportsConfig.';
        }
    }
    
    function eventExistsByCode(events, searchedCode) {
        return events.filter(function(event){
            return event.code.indexOf(searchedCode) != -1;
        }).length > 0;
    }
    
    function eventExistsByName(events, searchedName) {
        return events.findIndex(function(event){
            return event.name === searchedName;
        }) !== -1;
    }
    
    function addCodeOnEvent(events, eventName, eventCode, placeOn) {
        if (!placeOn) placeOn = 'right';
        
        const eventIndex = events.findIndex(function(event) {
            return event.name === eventName;
        });
        const event = events[eventIndex];
        const endSemicolonIndex = event.code.lastIndexOf(';');
        const codeHasEndSemicolon = endSemicolonIndex !== -1 && endSemicolonIndex === event.code.length - 1;
        
        if (!codeHasEndSemicolon) {
            event.code += ';';
        }
        
        if (placeOn === 'right') {
            events[eventIndex].code += eventCode;
        } else {
            const newCode = eventCode + ';' + events[eventIndex].code;
            
            events[eventIndex].code = newCode;
        }
    }
    
    function addEvent(params) {
        const entity = params.entity;
        const event = params.event;
        
        const name = event.name;
        const code = event.code;
        const events = entity.events;
        
        if (eventExistsByName(events, name)) {
            if (!eventExistsByCode(events, code)) {
                addCodeOnEvent(events, name, code, event.placeOn);
            }
        } else {
            events.push({
                name: name,
                code: code,
            });
            
            if (event.bind) {
                eventEngine.bindEvent(entity, name, code);
            }
        }
    }
    
    function getSelectedOption(fieldName) {
        const selector = '#' + fieldName + ' + div li[class*="selected"]';
        
        return document.querySelector(selector);
    }
    
    function getCurrent(fieldName) {
        const selector = '#' + fieldName + ' + div > span';
        
        return document.querySelector(selector).textContent;
    }
    
    this.onEnterReportWidget = function(fieldName) {
        const field = templateManager.container.getActiveWidget().getField(fieldName);
        
        if (!field.value()) {
            applyDefaultValue(field);
        }
        
        if (!getSelectedOption(fieldName)) {
            setTimeout(function runClickOnSelectedOption() {
                const selectedOption = getSelectedOption(fieldName);
                
                if (!selectedOption) {
                    return setTimeout(runClickOnSelectedOption, 500);
                }
                
                const current = getCurrent(fieldName);
                
                if (!current) {
                    return clickOnSelectedOption(fieldName);
                }
            }, 500);
        }
    };
    
    function beforeOpenReport(params) {
        const reportParams = params.reportParams;
        const reportNameSplitted = reportParams.__report_name.split('.');
        const reportName = reportNameSplitted[0];
        const strategy = reportNameSplitted[1];
        
        reportParams.__report_name = reportName;
        reportParams.STRATEGY = strategy;
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('ReportDecorator', ReportDecorator);
});

// FILE: js/Util/ShortcutDecorator.js
function ShortcutDecorator(ApplicationContext, templateManager) {

    /**
     *  Default keys for CRUD icons
     */
    const defaultCRUDShortcutKeys = {
        "add": "mod+a",
        "edit": "mod+e",
        "delete": "mod+d"
    };

    /**
     * 
     * Get the 'mod' key name according to OS (Ctrl or Command)
     * @returns String(Ctrl, Command) / Boolean(false)
     */
    function getTooltipModKey() {
        var platform = window.navigator.platform,
            commandPlatforms = ['Macintosh', 'MacIntel', 'MacPPC', 'Mac68K'],
            ctrlPlatforms = ['Win32', 'Win64', 'Windows', 'WinCE'],
            mod = false;

        if (commandPlatforms.indexOf(platform) !== -1) {
            mod = 'Command + ';
        } else if (ctrlPlatforms.indexOf(platform) !== -1 || /Linux/.test(platform)) {
            mod = 'Ctrl + ';
        }
        return mod;
    }

    /**
     * 
     * Get shortcut key from shortcutKeys object of widget, if not defined, 
     * from project property and if not defined, 
     * from defaultCRUDShortcutKeys const in this function.
     * @param {any} widget 
     * @param {any} shortcut 
     * @returns String(Shortcut key) / Undefined
     */
    function getShortcutKey(widget, shortcut) {
        return (
            Util.getNestedProperty(widget, ['shortcutKeys', shortcut]) ||
            Util.getNestedProperty(templateManager, ['project', 'shortcutKeys', shortcut]) ||
            defaultCRUDShortcutKeys[shortcut]
        );
    }
/**
 * 
 * Iterate recursively in widgets of container applying shortcut key in actions. 
 * @param {any} widgets 
 */
function applyShortcutOnAction(widgets) {
        var modTooltip = getTooltipModKey();
        widgets.forEach(function (widget) {
            if (widget.actions) {
                widget.actions.forEach(function (action) {
                    if (action.icon === 'plus' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'add');
                    } else if (action.icon === 'pencil' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'edit');
                    } else if (action.icon === 'trash' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'delete');
                    } else if (!action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, action.icon);
                    }
                    if (modTooltip && action.shortcutKey && !action.tooltip && templateManager.project.showShortcutKeyAsTooltip !== false) {
                        action.tooltip = action.shortcutKey.toUpperCase().replace('MOD+', modTooltip);
                    }
                });
            }
            if (widget.widgets && widget.widgets.length) {
                applyShortcutOnAction(widget.widgets);
            }
        });
    }

    /**
     * 
     * If project have the property enableCRUDShortcutKeys == true, applies the shortcut keys in actions
     * @param {any} containerName 
     * @param {any} container 
     * @returns Container(container)
     */
    this.apply = function (containerName, container) {
        if (templateManager.project.enableCRUDShortcutKeys == true) {
            applyShortcutOnAction(container.widgets);
        }
        return container;
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ShortcutDecorator', ShortcutDecorator);
    ContainerDecoratorProvider.registerProvider(new ShortcutDecorator(ApplicationContext, templateManager));
});


// FILE: js/Util/Utilities.js
function LibUtilities(ScreenService, eventAggregator, templateManager, requestDataSourceEngine, restEngine, MessageService) {
    var self = this;
    this.authenticationContainer = 'login#authentication';
    this.expiredSessionMethod = null;

    this.showErrorMessage = function(response) {
        return MessageService.getMessageByCode('lib#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode('lib#' + messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    return ScreenService.showMessage(message, 'ERROR');
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.redirectToLogin = function() {
        if(window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var validateExpiredSessionRoute = 'lib_validateExpiredSession';
        var logoutRoute = 'lib_logout';
        var loginRoute  = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if(route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if(window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if(eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if(route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;

        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'USER',
                value: localStorage.getItem('LOGIN_USER'),
                operator: '='
            }, {
                name: 'IS_LOGGED',
                value: localStorage.getItem('IS_LOGGED') || false,
                operator: '='
            }, {
                name: 'KEEP_CONNECTED',
                value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
                operator: '='
            } ];

            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter).then(function() {
                ScreenService.hideLoader();
                ScreenService.openLink(args.owner.url);
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        return requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    eventAggregator.subscribe("AboutOnClick", function () {
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: getCurrentModule()
            } ];
            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource('lib#/lib_getVersionInfo', filter).then(function (response) {
                ScreenService.hideLoader();
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.current = templateManager.versionData.dependencies.frontend;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                };
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    });

    eventAggregator.subscribe("EventOnRequestError", function (eventData) {
        if(eventData.e && eventData.e.e && eventData.e.e.data && eventData.e.e.data.data) {
            var error = eventData.e.e.data.data;
            var config = eventData.e.e.data.config;
            //Expired or Invalid Token
            if(error.errorCode == 101 || error.errorCode == 100)  {
                eventAggregator.publish('LibInvalidToken', error.errorCode);
            } else if(error.errorCode == 102){
                eventAggregator.publish('LibInvalidTokenSupportOperator');
            } else if(error.errorCode == 20){
                eventAggregator.publish('LibConcurrentAccess');
            }
        }
    });

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    /**
     * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
     * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
     * @param $productId      => Código do Produto (opcional).
     * @param $operatorCode   => Código do Operador (opcional).
     * @param $operatorName   => Nome do Operador (opcional).
     * @param $organizationId => Código do organização (opcional).
     */
    this.saveCollectedData = function(collectionIds, operationId, productId, operatorCode, operatorName, organizationId) {
        var row = {
            COLLECTION_IDS : collectionIds,
            OPERATION_ID   : operationId,
            PRODUCT_ID     : productId		|| null,
            OPERATOR_CODE  : operatorCode   || null,
            OPERATOR_NAME  : operatorName   || null,
            ORGANIZATION_ID: organizationId || null,
            changed        : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveCollectedData', [row]);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});


// FILE: js/Util/ViewFloatMask.js
var ViewFloatMask = function(maskEngine) {
    var keypressHandler = function (e, element, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var thousandsCode = opt.thousands.charCodeAt();
        var isThousands = (thousandsCode === e.which);
        var is_number = (e.which >= 48 && e.which <= 57);
        var is_dot = (e.which == 46);
        var is_comma = (e.which == 44);
        var is_minus = (e.which == 45);
        var hasSeparator = (element.val().indexOf(opt.decimal) != -1 || element.val().indexOf(opt.thousands) != -1);
        var hasNegative = (element.val().length > 0 || !opt.allowNegative);

        if ((hasSeparator && (is_dot || is_comma)) || (hasNegative && is_minus) || isThousands)
            e.preventDefault();

        if (is_dot || is_comma || is_number || is_minus)
            element.data('maskOptions', opt);
        else
            e.preventDefault();
    };

    var keyupHandler = function (e, element, params) {

        if(params.precision){
            var value = element[0].value;
            var decimals = value.substring(value.indexOf(params.decimal || ",")+1, value.length);
            if(value != decimals){
                if(decimals.length > params.precision){
                    element[0].value = value.substring(0, value.indexOf(params.decimal || ",")+(params.precision + 1));
                }

            }

        }

    };

    var viewFloat = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        params.showRound = !('showRound' in params) ? true : params.showRound;
        params.showSymbol = false;

        element.on("keypress", function(event) {
            keypressHandler(event, element, params);
        });

        element.on("keyup", function(event) {
            keyupHandler(event, element, params);
        });

        element.on("blur", function() {
            element.text(unMaskViewFloat(element.val(), params));
            element.val(viewFloatFormat(element.val(), params));
            params.callback(element.text());
        });

        if (!!params.showRound) {
            element.on("focus", function() {
                var value = element.text();
                var viewValue = viewFloatFormat(element.val(), params);
                var currentValue = viewFloatFormat(element.text(), params);

                if (currentValue !== viewValue)
                    value = element.val();

                element.val(viewFloatFormat(value, params));
                element.select();
            });
        }
    };

    var viewFloatFormat = function(value, params) {
        var defaultOptions = Util.clone($.currency.i18n[$.currency.lang]);

        var valueFormated = "";
        try {
            var options = Util.extend(defaultOptions, (params || {}), true);
            valueFormated = $.currency.string(value, options);
        } catch (e) {
            valueFormated = value;
        }
        return valueFormated;
    };

    var unMaskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var fltValue = opt.allowZero && !opt.allowEmpty ? 0 : "";
        var valueFormated = value;

        if (!Util.isEmptyOrBlank(valueFormated)) {
            var decimal = opt.decimal;
            var thousands = opt.thousands;

            if (typeof valueFormated === "string") {
                if(opt.showSymbol)
                    valueFormated = valueFormated.replace(opt.symbol, '');

                if(thousands)
                    valueFormated = valueFormated.replace(new RegExp('\\' + thousands, 'g'), '');

                valueFormated = valueFormated.replace(new RegExp('\\' + decimal, 'g'), '.');
                valueFormated = removeInvalidChar(valueFormated);
            }
            fltValue = parseFloat(valueFormated);

            if ((!opt.allowZero && fltValue === 0) || $.currency.isInvalidValue(fltValue, opt))
                valueFormated = "";
        }
        return valueFormated;
    };

    var removeInvalidChar = function(value) {
        var index = value.indexOf('.');
        return index + 1 === value.length ? value.slice(0, index) : value;
    };

    var maskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);

        if (value)
            value = value.replace(new RegExp('\\' + '.', 'g'), opt.decimal);

        return value;
    };

    this.register = maskEngine.registerMask('viewFloat', viewFloat, viewFloatFormat);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewFloatMask', ViewFloatMask);
});

// FILE: js/Util/WidgetsMetaData.js
function WidgetsMetaData() {
    this.getExpiredSession = function() {
        return {
            "name": "EXPIRED_SESSION",
            "label": "Sessão Expirada",
            "isVisible": true,
            "showDescriptionOrder": false,
            "showAsModal": true,
            "template": "widget/form.html",
            "tekLibExpiredSession": true,
            "actions": [
                {
                    "name": "CONFIRM",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventConfirm(args)"
                        }
                    ]
                },
                {
                    "name": "CANCEL",
                    "label": "Sair",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventCancel()"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": [],
            "fields": [
                {
                    "name": "OPERATOR",
                    "label": "Usuário",
                    "template": "field/text-edit.html",
                    "class": 12,
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "PASSWORD",
                    "label": "Senha",
                    "template": "field/password.html",
                    "class": 12,
                    "focused": true,
                    "validations":{
                        required: true
                    },
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true,
                    "events":[
                        {
                            "name": "FieldOnKeyup",
                            "code": "LibLoginController.expiredSessionFieldOnKeyupPassword(args)"
                        }
                    ]
                }
            ]
        };
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('WidgetsMetaData', WidgetsMetaData);
});


// FILE: js/Service/Login.js
function LibLoginService(requestDataSourceEngine){
    this.startSession = function(user) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'IS_LOGGED',
            value: localStorage.getItem('IS_LOGGED') || false,
            operator: '='
        }, {
            name: 'LOGIN_TOKEN',
            value: localStorage.getItem('LOGIN_TOKEN'),
            operator: '='
        }, {
            name: 'LOGIN_LANGUAGE',
            value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_startSession', filter, 1, undefined, undefined, undefined, 120000)
            .then(function(data) {
                if(typeof data == 'string') {
                    return {
                        isError : true,
                        message : data
                    };
                }
                return data && data.dataset ? data.dataset.system : false;
            });
    };

    this.validateUser = function(cdOperator, password, isSupportOperator) {
        var filter = [ {
            name: 'USER',
            value: cdOperator,
            operator: '='
        }, {
            name: 'PASSWORD',
            value: password,
            operator: '='
        }, {
            name: 'IS_SUPPORT_OPERATOR',
            value: isSupportOperator,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('login#/validateUser', filter);
    };

    this.logout = function() {
        var filter = [ {
            name: 'PROJECT',
            value: window.tekLoginProjectId,
            operator: '='
        }, {
            name: 'ERASE_HASH',
            value: localStorage.getItem('LOGIN_ERASE_HASH'),
            operator: '='
        }, {
            name: 'HASH',
            value: localStorage.getItem('LOGIN_HASH'),
            operator: '='
        } ];
        return requestDataSourceEngine.getDataSource('lib#/lib_logout', filter, 1, undefined, undefined, undefined, 120000);
    };

    this.getModulesVersion = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_getModulesVersion', []);
    };

    this.generateModulesVersionFile = function() {
        return requestDataSourceEngine.setDataSource('lib#/lib_generateModulesVersionFile', [ { changed: true } ]);
    };

    this.downloadModulesVersionFile = function(fileName) {
        var serviceUrl = window.serviceUrl;
        window.open(serviceUrl + "/lib_downloadModulesVersionFile?requestType=DataSet&dataset[0][FILE_NAME]=" + fileName);
    };

    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'ACCESS_PROFILE',
            value: accessProfile,
            operator: '='
        }, {
            name: 'OTHER_SESSION_VALUES',
            value: otherSessionValues,
            operator: '='
        }, {
            name: 'LOGIN_USER',
            value: localStorage.getItem('LOGIN_USER'),
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_updateSession', filter);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginService', LibLoginService);
});



// FILE: js/Controller/Login.js
function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var updateLanguage = function () {
        var language = localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var redirectToRedirectionProduct = function (redirectUrl, defer) {
        if (window.ZhNativeInterface) {
            window.ZhNativeInterface.redirect(redirectUrl);
        } else {
            window.location = redirectUrl;
        }
        defer.resolve({});
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var redirectUrl = localStorage.getItem('LOGIN_REDIRECT_URL');
            var authToken = updateToken();

            if(redirectUrl && !redirectUrl.includes(window.location.hostname)) {
                redirectToRedirectionProduct(redirectUrl, defer);
            } else if(!user && !authToken) {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            eventAggregator.publish('AboutOnClick');
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }

                if (response.MODULES_VERSION_ERROR) {
                    if(response.MODULES_VERSION_ERROR.MESSAGE_CODE) {
                        var modulesMessageCode   = response.MODULES_VERSION_ERROR.MESSAGE_CODE;
                        var modulesMessageParams = response.MODULES_VERSION_ERROR.MESSAGE_PARAMS;
                        MessageService.getMessageByCode('lib#' + modulesMessageCode).then(function (message) {
                            ZhLogger.warning(ScreenService.i18n(message, modulesMessageParams));
                        });
                    } else {
                        ZhLogger.warning(response.MODULES_VERSION_ERROR);
                    }
                }

                defer.resolve(response);
            }
        }
    };

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});

// FILE: bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js

// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-select-autocomplete';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
    return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
    window.templateUrls.push({
        'package': packageName,
        'baseUrl': './bower_components/' + packageName + '/dist/templates/'
    });
}

// FILE: src/controllers/zh-select-autocomplete.controller.js
function ZhSelectAutoCompleteController($scope, $rootScope, $timeout, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, EventChildPropagation, DecodeValueFieldService, $filter, metaDataFactory) {

	var self = this;
    var lastSearch = "";
    var propertyToSearch = [];
	var isMakingRequest = false;
	var shouldSelectAfterLoad = false;

	$scope.field.lazyLoadEvent = $scope.field.lazyLoadEvent || templateManager.project.lazyLoadEvent;

	$scope.isRequestBeingMade = function() {
		return isMakingRequest;
	};

	$scope.selectAfterLoad = function() {
		shouldSelectAfterLoad = true;
	};

    var dataInitialized = Util.isUndefinedOrNull($scope.field.dataSource.lazyLoad) ?
            true : !$scope.field.dataSource.lazyLoad;

	$scope.viewMore = ScreenService.i18n("View more...");
	$scope.filteredData = [];
	$scope.showDropdown = true;
	var filterUpdated = false;

	var oldFieldToggleDropdown = $scope.field.toggleDropdown;
    $scope.field.toggleDropdown = function(){
        $scope.showDropdown = !$scope.showDropdown;
    };

	var oldFieldInitializeField = $scope.field.initializeField;
	$scope.field.initializeField = function() {
		if(!dataInitialized){
			dataInitialized = true;
			updateDatasource(lastSearch, true);
		}
	};

	var parentReload = $scope.field.reload.bind($scope.field);

	/**
	 * Override the field function reload to initialize the field when called.
	 * @param {Object[]} [addictionalFilters] - An object of filters to be applied when reloading the field
	 * @returns {Promise.<Array>} Field data with the given fields.
	 */
	$scope.field.reload = function(addictionalFilters){
		$scope.field.initializeField();
		return parentReload(addictionalFilters).then(function(response){
			updateDatasource();
			return response;
		});
	};

	/**
	 * Get a row from dataSets
	 *
	 * @function getNewData
	 * @private
	 * @param   {Object} keys - DataSets returned by request
	 * @returns {Array} Returns a data array.
	 */
	function getNewData(dataSets){
		var data;
		result = dataSets.dataset;
		var keys = Object.keys(result);

		if (keys && keys.length > 0 && keys[0] !== '0') {
			var dataSetName = $scope.field.dataSource.name.split('/').pop();
			if (result[dataSetName]) {
				data = result[dataSetName];
			}
		} else {
			data = result;
		}

		return data;
	}

	/**
	 * Make a request to get an specific value.
	 *
	 * @function findNewValue
	 * @private
	 * @param   {Array} keys - CurrentRow's keys.
	 * @param   {Field} field - Field.
	 * @returns {Array} Returns a row.
	 */
	function findNewValue(keys, field, value){
		var filter = [];
		keys.forEach(function(key){
			filter.push({
				"name": key,
				"operator": '=',
				"value": value[key]
			});
		});
		var storageStrategy = field.dataSource.getStorageStrategy();
		return storageStrategy.getDataSource(field.dataSource.name, function(){}, filter, 1, 2, undefined, 0);
	}

	/**
	 * Prepares the value.
	 *
	 * @function findNewValue
	 * @private
	 * @param   {Array} valueFields - The field's valueFields
	 * @param   {String|Object} value - the value to be inserted.
	 * @returns {Object} The prepared value.
	 */
	function prepareValue(valueFields, value) {
		var valueBuilt = {};

		if(valueFields.length === 1 && typeof value !== 'object'){
			valueBuilt[valueFields[0]] =  value;
		} else {
			valueBuilt = value;
		}

		return valueBuilt;
	}

	/**
	 * Override the field function value to use valueFields.
	 * @param {Object} - The value to be set.
	 */
	var oldFieldValue = $scope.field.value;
	$scope.field.value = function(value){
		if(value){
			var shouldMakeRequest = $scope.field.dataSource.rest && $scope.field.hasPagination;
			currentRow = $scope.field.widget.currentRow;
			valueFields = $scope.field.valueFields;

			value = prepareValue(valueFields, value);

			if(shouldMakeRequest){
				findNewValue(valueFields, $scope.field, value).then(function(result){
					result = getNewData(result);
					if(result.length === 1){
						self.addItemAsSelected(result[0], $scope.field);
					} else {
						currentRow[$scope.field.name] = "";
					}
				});
			} else {
				var row = $scope.field.dataSource.data.filter(function(row){
					var result = true;

					valueFields.forEach(function(key){
						if(row[key] !== value[key]){
							result = false;
						}
					});

					return result;
				});

				if(row.length === 1){
					self.addItemAsSelected(row[0], $scope.field);
				} else {
					currentRow[$scope.field.name] = "";
				}
			}
		} else {
			return this.widget.currentRow[this.name];
		}
	};

	/**
	 * Extends the behavior of it's parent (ListController).
	 */
	function extendBehavior() {
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

    /**
     * Retrieves the code to be executed on FieldOnAdd event.
     * @returns {String} - Code to be executed.
     */
	function getOnAddCode(){
		var onAddEventCode = $scope.field.events.filter(function(event) {
			return event.name === "FieldOnAdd";
		});

		if(onAddEventCode.length > 0){
			return onAddEventCode[0].code;
		} else {
			return "args.owner.widget.onAdd()";
		}
	}

	var onAddCode = getOnAddCode();

	/**
	 * Build all actions of the widget that appears when the field's popup is opened.
	 *
	 * @function buildActions
	 * @private
	 * @param   {Field} field - Widget owner
	 * @returns {Array} - Actions of the widget
	 */
	function buildActions(field) {
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": onAddCode,
					"id": 9999
				}]
			});
		}

		return actions;
	}

    /**
     * Initialize the field.
     *
     * @function initialize
     */
    (function initialize() {
        initializeValuesOnCurrentRow();
        initializeFieldProperties();
        return $scope.field.dataSource.data;
    })();

    /**
     * Prepares the new clearValue function for the select-autocomplete's field.
     */
    $scope.field.clearValue = function() {
        clearFieldValue(this);
    };

    /**
     * Clears the select-autocomplete value and it's dependents.
     * @param {Field} field The select autocomplete.
     */
    function clearFieldValue(field) {
        $scope.resetPlaceholderValue();
        if (!field.isReadOnly()) {
            field.clearByOutData();
            field.clearByName();
            field.clearByOutDescription();
            field.dataSource.clearCheckedRows();
            setChanged(field);
            if (field.change) {
                field.change();
            }
            $scope.clearAutoCompleteListData();
            $scope.hideList();
            templateManager.updateTemplate();
            $scope.removeFocusOfInput();
        }
    }

	/**
     * Config which template of widget will appear when the field's popup is opened.
     *
     * @function configTemplate
     * @private
     * @param   {Field} field - Widget owner
     * @returns {String} template - Widget template
     */
	function configTemplate(field){
		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		}
		else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		}
		else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		return template;
	}

	/**
     * Create the fields to be inserted
     *
     * @function buildFields
     * @private
     * @param   {descriptionField} displayedField - Used to create the fields
     * @param   {Number} key - Array position
     */
	function buildFields(displayedField, key){
        this[key] = {
            "name": displayedField,
            "showLabel": false,
            "showOnList": true,
            "mask": this.mask
        };
    }

    /**
     * Insert labels on the fields.
     *
     * @function insertLabels
     * @private
     * @param   {String} label - label to be inserted
     * @param   {Number} key - Array position
     */
    function insertLabels(label, key){
        this[key].label = label;
        this[key].showLabel = true;
    }

	/**
     * Build all fields of the widget that appears when the field's popup is opened.
     *
     * @function buildSelectWidgetFields
     * @private
     * @param   {Field} field - Widget owner
     */
	function buildSelectWidgetFields(field) {
        var fields = [];

        if (field.fields && field.fields.length) {
            fields = field.fields;
        } else {
            field.descriptionFields.forEach(buildFields, fields);
        }

        if(field.displayedFieldsLabels){
            field.displayedFieldsLabels.forEach(insertLabels, fields);
        }

        fields.forEach(function(currentElement) {
            metaDataFactory.fieldFactory(currentElement, field.selectWidget, true);
        });
	}

	/**
     * Build all events of the widget that appears when the field's popup is opened.
     *
     * @function buildEvents
     * @private
     * @returns {Array} - Events of the widget
     */
	function buildEvents(){
		return [
			{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			},
			{
				"name": "WidgetAfterLoadDataSource",
				"code": "args.owner.updateDatasource()",
				"id": 9999
			}
		];
	}

	/**
	 * Set the args' row as selected.
	 *
	 * @function setSelected
	 * @private
	 * @param {Object} args - The window args.
	 */
	function setSelected(args) {
		if (!$scope.field.allowSelectOnlyLastChild || ($scope.field.allowSelectOnlyLastChild && this.isTree($scope.field.selectWidget) && isLastChild(args.row))) {
			ScreenService.closePopup();
			self.onAutoCompleteItemClick(args.row);
		}
	}

	/**
	 * Routine executed after the field's popup be closed.
	 *
	 * @function afterClosePopup
	 * @private
	 * @param {Field} field - The field that had it's popup closed.
	 */
	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	/**
	 * Build the widget that appears when the field's popup is opened.
	 *
	 * @function buildSelectWidget
	 * @private
	 * @param {Field} args - Widget owner.
	 * @param {Object} widgetConfig - Widget's propertys
	 */
	function buildSelectWidget(field, widgetConfig) {
		if (!field.selectWidget){
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 9000,
				"name": "Widget",
				"label": field.label,
				"template": widgetConfig.template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField,
					"customizationAction": {
						"layout": false,
						"filter": false,
						"view": false
					},
					"xlsAction": false,
					"pdfAction": false,
					"csvAction": false
				},
				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,
				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": [],
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage || 30,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"actions": widgetConfig.actions,
				"events": EventChildPropagation.propagate(widgetConfig.widgetEvents, field),
				"searchCriteria": field.searchCriteria
			});
			buildSelectWidgetFields(field);
		}
		field.selectWidget.setSelected = setSelected;
		field.selectWidget.updateDatasource = updateDatasource;
		field.selectWidget.afterClosePopup = afterClosePopup;
	}

	/**
     * Reload the field
     *
     * @function processReload
     * @private
     * @param   {boolean} isToReload
     * @param   {Field} field
     * @returns {Promise} - A resolved promise
     */
	function processReload(isToReload, field){
		var defer = ZHPromise.defer();

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.reload().then(defer.resolve, defer.resolve);
		}
		else {
			defer.resolve();
		}

		return defer.promise;
	}

	/**
     * Change the dataSource's data to the filtered by the given value.
     *
     * @function innerSearch
     * @private
     * @param {String} selectListSearch - The value to be searched on the field.
     * @param {Field} field - The field where will be searched the values and changed the data of the dataSource.
     */
    function innerSearch(selectListSearch, field) {
        if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
            var dataSource = field.dataSource;
            var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
            dataSource.filter(filter, function(data) {
                dataSource.data = data;
                templateManager.updateTemplate();
            });
        }
    }

    /**
     * Removes all filters from the dataSource's data
     *
     * @function  cleanFieldSearch
     * @private
     * @param {Field} field - The field where will be removed the filters of the dataSource's data.
     */
    function cleanFieldSearch(field) {
        innerSearch("", field);
    }

	/**
     * Build the Widget to be open
     *
     * @function buildFieldWidget
     * @private
     * @param   {Field} field - Widget owner
     * @returns {Promise} - A resolved promise
     */
	function buildFieldWidget(field) {
		cleanFieldSearch(field);

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));

		if (!field.selectWidget) {
			initializeSelectWidget(field);
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		return processReload(isToReload, field);
	}

	/**
     * Build configs to be used on the widget
     *
     * @function buildwidgetConfig
     * @private
     * @param   {Field} field - field used to build the configs
     * @returns {Object} Object with the configs
     */
	function buildWidgetConfig(field) {
		var widgetConfig = {};
		widgetConfig.template = configTemplate(field);
		widgetConfig.actions = buildActions(field);
		widgetConfig.widgetEvents = buildEvents();
		return widgetConfig;
	}

	/**
     * Initialize the field's SelectWidget propertys
     *
     * @function initializeSelectWidget
     * @private
     * @param   {Field} field - SelectWidget owner
     */
	function initializeSelectWidget(field) {
		buildSelectWidget(field, buildWidgetConfig(field));
		overrideReload(field);
	}

	var oldReload;
	function overrideReload(field) {
		oldReload = field.selectWidget.reload;
		field.selectWidget.reload = function() {
			if(dataInitialized){
				return oldReload.call(field.selectWidget);
			} else {
				return ZHPromise.when();
			}
		};
	}

	/**
	 * Call the beforeSelectOpen event if defined
	 *
	 * @function processBeforeSelectOpenEvent
	 * @private
	 * @param   {Field} field - Widget owner
	 */
	$scope.processBeforeSelectOpenEvent = function(field){
		if (field.beforeSelectOpen) {
			field.beforeSelectOpen();
		}
	};

	/**
     * Call the afterSelectOpen event if defined
     *
     * @function processAfterSelectOpenEvent
     * @param   {Field} field - Widget owner
     */
	$scope.processAfterSelectOpenEvent = function(field){
		if (field.afterSelectOpen) {
			field.afterSelectOpen();
		}
	};

	/**
     * Remove element Focus
     *
     * @function removeElementFocus
     * @private
     * @param   {Field} field - Field where the focus will be removed
     */
	function removeElementFocus(field) {
		if (field.element) {
			field.element.blur();
			window.getSelection().removeAllRanges(); //bug webkit
		}
	}

	/**
     * Open the popup of the field
     *
     * @function openField
     *
     * @param   {Field} field - Popup owner
     */
	$scope.openField = function(field, $event) {
        $event.stopPropagation();
        $scope.hideList();
		if (!field.isReadOnly()) {
			$scope.processBeforeSelectOpenEvent(field);
			removeElementFocus(field);
            $scope.inputWithEditableContent.click();
			buildFieldWidget(field).then(function() {
				$scope.clearSearchFilter();
				ScreenService.openPopup(field.selectWidget);
				$scope.processAfterSelectOpenEvent(field);
			});
		}
	};

	/**
	 * Open the popup of the field on the scope.
	 */
	var oldFieldOpenField = $scope.field.openField;
	$scope.field.openField = function() {
		$scope.openField($scope.field);
	};

    /**
     * Config the property valueField because the controller works only with valueFields.
     *
     * @function configValueFields
     * @private
     */
    function configValueFields(){
        var field = $scope.field;

        if(!field.valueFields){
            field.valueFields = [field.valueField];
        }
    }

    /**
     * Set the property descriptionField to avoid component break if not defined and config the property
     * descriptionField because the controller works only with descriptionFields.
     *
     * @function configDescriptionFields
     * @private
     */
    function configDescriptionFields() {
        var field = $scope.field;
        field.descriptionFields = field.descriptionFields ? field.descriptionFields : [field.descriptionField || field.name];
    }

	/**
	 * Clear the outdata related to the component.
	 */
	var oldFieldClearByOutData = $scope.field.clearByOutData;
	$scope.field.clearByOutData = (function() {
		for (var prop in this.outData) {
			this.widget.currentRow[this.outData[prop]] = "";
		}
	}).bind($scope.field);

	/**
	 * Clear the value of the selected row by the component from the current row.
	 *
	 * @function clearByName
	 * @public
	 */
	var oldFieldClearByName = $scope.field.clearByName;
	$scope.field.clearByName = (function () {
		this.widget.currentRow[this.name] = "";
	}).bind($scope.field);

	/**
	 * Clear the outDescription related to the component
	 *
	 * @function clearByOutDescription
	 * @public
	 */
	var oldFieldClearByOutDescription = $scope.field.clearByOutDescription;
	$scope.field.clearByOutDescription = (function() {
		this.widget.currentRow[this.outDescription] = "";
	}).bind($scope.field);

	if ($scope.selectField) {
		var oldSelectFieldOpenField = $scope.selectField.openField;
		$scope.selectField.openField = function() {
			$scope.openField($scope.selectField);
		};
	}

	/**
	 * Set the widget of the component as changed.
	 *
	 * @function setChanged
	 * @private
	 * @param {Field} field - The field to have it's current row set as changed.
	 */
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

    /**
     * Clear the filter of the component, and the values related to him from the current row.
     *
     * @function clearField
     * @public
     * @param {Field} field - The field to have it's values cleared.
     */
    $scope.clearField = function(field, $event) {
        $event.stopPropagation();
        clearFieldValue(field);
	};

	$scope.hasValueSetted = function() {
		var field = $scope.field;
		var currentRow = field.widget.currentRow;
		return !!currentRow[field.name];
	};

    $scope.$watch($scope.hasValueSetted, function(newVal){
    	$scope.showSpan = newVal;
    });

    /**
     * Searches for the given value if the value is valid.
     *
     * @function search
     * @public
     * @param {String} selectListSearch - The value to be searched. Should be bigger than 0.
     * @param {Field} field - The field that will have the data that will be searched.
     */
    $rootScope.search = function(selectListSearch, field) {
        var minLength = field.minimumInputLength || 0;
        if (selectListSearch.length > minLength) {
            innerSearch(selectListSearch, field);
        }
    };

	/**
	 * Add the given item as selected
	 *
	 * @function addItemAsSelected
	 * @public
	 * @param {Object} item - The selected item.
	 * @param {Field} field - The item's field, where the item will be set as selected.
	 */
	this.addItemAsSelected = function(item, field) {
		(field.outData ? changeByOut : changeByName)(field, item);
		if(field.outDescription){
			changeByDescription(field, item);
		}
		item.__isSelected = true;
		field.dataSource.checkedRows = [item];
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(item);
		}
		if (field.change) field.change();
	};

	/**
	 * Returns the mounted description of the selected value.
	 *
	 * @function getDescriptionFromSelectedValue
	 * @public
	 * @returns {String} The value that should be displayed of the selected row.
	 */
	$scope.getDescriptionFromSelectedValue = function() {
		var fieldName = $scope.field.name;
		var currentRow = $scope.field.widget.currentRow;

		return DecodeValueFieldService.model($scope.field, currentRow[fieldName]);
	};

	/**
	 * Returns the item's identifier.
	 *
	 * @function getItemIdentifier
	 * @public
	 * @param   {Object} item - The item to get it's identifier.
	 * @returns {String} The item's identifier.
	 */
	this.getItemIdentifier = function(item) {
		var identifier = [];
		var field = $scope.field;
		var valueFields = field.valueFields;

		for(var valueField in valueFields){
			identifier.push(item[valueFields[valueField]]);
		}

		return identifier.join(" | ");
	};

	/**
	 * Return the ordered data.
	 *
	 * @function orderDropdown
	 * @private
	 * @param {Object} data - The data to be ordered.
	 * @param {String} order - The selectWidget's order.
	 * @returns {Object} The ordered data.
	 */
	function orderDropdown(data, order){
		return $filter('orderBy')(data, order);
	}

    /**
     * Updates the dataSource, applying any typed filter.
     *
     * @function updateDatasourceRemote
     * @private
     * @param {DataSourceFilter} itemSearch - The field's input value, used to filter the dataSource.
     */
    function updateDatasourceRemote(itemSearch, isToReload){
        if($scope.field.dataSource.rest){
            if((itemSearch !== undefined) && itemSearch !== lastSearch || isToReload){
                if($scope.field.searchableFields){
					var filterSet = false;
					var fieldName = $scope.field.searchableFields.join("|");
					if(!$scope.filterToAdd){
						$scope.filterToAdd = {};
						filterSet = true;
					}
					$scope.filterToAdd.name = fieldName;
					$scope.filterToAdd.operator = "LIKE_ALL";
					$scope.filterToAdd.value = buildSearchValue(itemSearch);

					if(filterSet){
						$scope.field.dataSourceFilter = $scope.field.dataSourceFilter || [];
						$scope.field.dataSourceFilter.push($scope.filterToAdd);
					}
					isMakingRequest = true;
					if($scope.field.dataSource.disableLoader && $scope.showLoader){
						$scope.showLoader();
					}
					$scope.field.dataSource.load($scope.field.dataSourceFilter, 1, $scope.field.selectWidget.itemsPerPage).then(function(){
						if($scope.field.selectWidget.order){
							$scope.field.dataSource.data = orderDropdown($scope.field.dataSource.data, $scope.field.selectWidget.order);
							$scope.filteredData = $scope.field.dataSource.data;
						}

						if(shouldSelectAfterLoad && $scope.field.dataSource.data.length === 1){
							self.addItemAsSelected($scope.field.dataSource.data[0], $scope.field);
							shouldSelectAfterLoad = false;
						}

						isMakingRequest = false;
						if($scope.field.dataSource.disableLoader && $scope.hideLoader){
							$scope.hideLoader();
						}
					});
                    resetDropdown();
                } else {
                    throw new Error('Field ' + $scope.field.name + ' must have searchableFields. ');
                }
            }
        }
	}

	/**
	 * Deletes the filter add by searching in selectAutocomplete
	 */
	$scope.$on("$destroy", function(){
		if($scope.field.dataSourceFilter){
			$scope.field.dataSourceFilter.forEach(function(filter){
				if(angular.equals(filter, $scope.filterToAdd)){
					$scope.field.dataSourceFilter.splice($scope.field.dataSourceFilter.indexOf(filter), 1);
				}
			});
		}

		if ($scope.field.selectWidget) {
			if (oldReload) {
				$scope.field.selectWidget.reload = oldReload;
			}
		}

	    $scope.field.toggleDropdown = oldFieldToggleDropdown;
		$scope.field.initializeField = oldFieldInitializeField;
		$scope.field.value = oldFieldValue;
		$scope.field.openField = oldFieldOpenField;
		$scope.field.clearByOutData = oldFieldClearByOutData;
		$scope.field.clearByName = oldFieldClearByName;
		$scope.field.clearByOutDescription = oldFieldClearByOutDescription;
		$scope.field.reload = parentReload;
		if ($scope.selectField) {
			$scope.selectField.openField = oldSelectFieldOpenField;
		}
	});


    /**
     * Creates a filtered local copy of the Datasource.
     *
     * @function updateDatasourceLocal
     * @private
     * @param {DataSourceFilter} itemSearch - The field's input value, used to filter the dataSource copy.
     */
    function updateDatasourceLocal(itemSearch) {
        var expression = function(row) {
            var result = false;
            $scope.field.searchableFields.forEach(function(fieldName) {
                var rowValue = String(row[fieldName]).toLowerCase();
                switch ($scope.field.searchIn){
                    case 'beginning':
                        result = rowValue.startsWith(itemSearch.toLowerCase()) ? true : result;
                        break;
                    case 'end':
                        result = rowValue.endsWith(itemSearch.toLowerCase()) ? true : result;
                        break;
                    default:
                        result = rowValue.indexOf(itemSearch.toLowerCase()) !== -1 ? true : result;
                        break;
                }
            });

            return result;
        };

		$scope.filteredData = $filter('filter')($scope.field.dataSource.data, expression);
		if($scope.field.selectWidget.order){
			$scope.filteredData = orderDropdown($scope.filteredData, $scope.field.selectWidget.order);
		}
		resetDropdown();
		$timeout(function(){
            $scope.$digest();
        }, 0, false);
    }

    /**
     * Chose which method will be used to filter the data according with
     * the searchWithPagination property
     *
     * @function    updateDatasource
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
    function updateDatasource(itemSearch, isToReload){
        itemSearch = itemSearch || $scope.itemSearch || "";

        if($scope.shouldMakeRequest(itemSearch, isToReload)){
            updateDatasourceRemote(itemSearch, isToReload);
        }
        else {
            updateDatasourceLocal(itemSearch);
        }

        lastSearch = itemSearch;
    }

	/**
     * Updates the dataSource and updates the selected item in dropdown.
     *
     * @function onItemSearchChanged
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
	function onItemSearchChanged(itemSearch, isToReload){
		updateDatasource(itemSearch, isToReload);
		if($scope.updateDropdown){
			$scope.updateDropdown();
		}
	}

    /**
     * Verifies if the request should be made.
     *
     * @function shouldMakeRequest
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
	$scope.shouldMakeRequest = function(itemSearch, isToReload) {
		/* Checks if datasource is currently making requests */
		if($scope.field.searchWithPagination && dataInitialized && !isMakingRequest){
			/* Checks if the request has already been made */
			if(itemSearch == lastSearch && !filterUpdated){
				return false;
			} else {
				filterUpdated = false;
				var itemsPerPage = $scope.field.itemsPerPage || 30;
				var pageSize = 30 < itemsPerPage ? 30 : itemsPerPage;

				/* Checks if the request is a more specific version of last request.
				 *  If so, checks if the last request has already found all the data.
 				 */
				return (!(itemSearch.length > 0 && itemSearch.indexOf(lastSearch) !== -1 && self.getData() && self.getData().length < pageSize) || isToReload);
			}
		} else {
			return false;
		}
    };

    updateDatasource();

    $scope.$watch('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget, true);
    $scope.$watch('itemSearch', Util.buildDebounceMethod(onItemSearchChanged, 500, false));

    /**
     * Propagate the field's DataSourceFilter to the selectWidget.
     *
     * @function propagateFieldDataSourceFilterToSelectWidget
     * @private
     * @param {DataSourceFilter} fieldDataSourceFilter - The field's DataSourceFilter to be propagated.
     */
    function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		filterUpdated = true;
		if ($scope.field.selectWidget) {
            $scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
        }
		updateDatasource($scope.itemSearch, true);
    }

    /**
     * Defines the operator that will be used by filter.
     *
     * @function setOperator
     */
    function setOperator() {
        $scope.field.operator = $scope.field.operator || "=";
    }


    /**
     * Set hasPagination's default value if not set
     */
    function configHasPagination() {
        var field = $scope.field;
		if(field.hasPagination === undefined){
			field.hasPagination = true;
		}
    }

    /**
     * Initialize the field properties.
     *
     * @function initializeFieldProperties
     */
    function initializeFieldProperties() {
        configValueFields();
        configDescriptionFields();
        configHasPagination();
        buildPropertiesToSearch();
        setOperator();
        initializeSelectWidget($scope.field);
    }

    /**
     * Test the row to check if it should be displayed on the field's dropdown.
     *
     * @function searchFilter
     * @public
     * @param   {Object} row - The row to be tested.
     * @returns {Boolean} Result telling if the field should be displayed.
     */
    $scope.searchFilter = function(row) {
        return Util.isUndefinedOrNull($scope.itemSearch) || propertyToSearch.some(function(data) {
            return ~row[data].indexOfLatin($scope.itemSearch);
        });
    };

	/**
	 * Get the item with the given identifier.
	 *
	 * @function searchItem
	 * @private
	 * @param   {String} itemIdentifier - The desired item's identifier.
	 * @returns {Object} The item with the given identifier.
	 */
	function searchItem(itemIdentifier){
		var field = $scope.field;
		var data = field.dataSource.data;
		var valueFields = field.valueFields;
		var itemIdentifierArr = itemIdentifier.split(" | ");

		function findItem(item){
			var result = true;
			for(var valueField in valueFields){
				result &= item[valueFields[valueField]] == itemIdentifierArr[valueField];
			}
			return result;
		}

		item = data.find(findItem);
		return item;
	}

	/**
	 * Field's routine executed after one row be selected.
	 *
	 * @function onAutoCompleteItemSelect
	 * @public
	 * @param {String} selectedItemIdentifier - The identifier of the selected row.
	 */
	$scope.onAutoCompleteItemSelect = function(selectedItemIdentifier) {
		var item = searchItem(selectedItemIdentifier);
		self.clearEditableInput();

		if(!Util.isUndefinedOrNull(item))
			self.addItemAsSelected(item, $scope.field);

		$scope.removeFocusOfInput();
	};

	/**
	 * Set the item passed as clicked, clearing the component's last input and hiding it's list.
	 *
	 * @function onAutoCompleteItemClick
	 * @public
	 * @param {Object} clickedItem - The clicked item, with it's data.
	 */
	this.onAutoCompleteItemClick = function(clickedItem) {
		self.clearEditableInput();
		self.addItemAsSelected(clickedItem, $scope.field);
		$scope.removeFocusOfInput();
	};

	/**
	 * Tells if the given row is the last child.
	 *
	 * @function isLastChild
	 * @private
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result of the test of the row being the last child.
	 */
	var isLastChild = function(row) {
		return row._childCount === 0;
	};

	/**
	 * Change the values on the current row used to identify the selected row and, print the fields value on the screen.
	 *
	 * @function changeByName
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByName(field, row) {
		var valueFields = field.valueFields;
		var currentRow = field.widget.currentRow;
		var fieldName = field.name;
		currentRow[fieldName] = "";

		function buildCurrentRowValueFields(value, key){
			currentRow[value] = row[value];
		}

		valueFields.forEach(buildCurrentRowValueFields);
		currentRow[fieldName] = self.getItemDescription(row);
	}

	/**
	 * Change the given field current row value based on the configured outdata.
	 *
	 * @function changeByOut
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByOut(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				}
				else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}

	/**
	 * Change the given field current row value based on the configured outDescription.
	 *
	 * @function changeByDescription
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByDescription(field, row) {
		field.widget.currentRow[field.outDescription] = self.getItemDescription(row);
	}

	/**
	 * Get the component's actual value, on it's placeholder, returning the fieldRow, if it is passed.
	 *
	 * @function getSelectionText
	 * @public
	 * @param {Object} [fieldRow] - The clicked item, with it's data.
	 */
	this.getSelectionText = function getSelectionText(fieldRow) {
		if ($scope.field.placeholder && !fieldRow) {
			selectionText = $scope.field.placeholder;
		}
		else {
			selectionText = fieldRow || ScreenService.i18n("Search");
		}
		return selectionText;
	};

	/**
	 * Open a new window.
	 *
	 * @param  {Field} field - The field with the container's name that will be opened.
	 */
	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	/**
	 * Build the value to be used as filter.
	 *
	 * @function buildSearchValue
	 * @private
	 * @param {String} itemSearch - The field's input value, used to build the filter.
	 */
	function buildSearchValue(itemSearch) {
		switch ($scope.field.searchIn){
			case 'beginning':
				return itemSearch + "%";
			case 'end':
				return "%" + itemSearch;
			default:
				return "%" + itemSearch + "%";
		}
	}

	/**
	 * Clear the variables related to the filter.
	 *
	 * @function clearSearchFilter
	 * @public
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result telling if the field should be displayed.
	 */
	$scope.clearSearchFilter = function(){
		$scope.itemSearch = '';
	};

	/**
	 * Insert into the propertysToSearch the fields that the user can search on the field's dropdown.
	 *
	 * @function addFieldNamesToPropertyToSearch
	 * @private
	 * @param {Array.<Fields>} fields - The fields that should be avaible to search.
	 */
	function addFieldNamesToPropertyToSearch(fields) {
		fields.forEach(function(field){
			propertyToSearch.push(field);
		});
	}

	/**
	 * Build the properties that the user can search on the field's  dropdown.
	 *
	 * @function buildPropertiesToSearch
	 * @private
	 */
	function buildPropertiesToSearch(){
		var field = $scope.field;
		$scope.field.searchWithPagination = Util.isUndefinedOrNull($scope.field.searchWithPagination) ? true : $scope.field.searchWithPagination;
		var propertyToAdd = null;
		if (!Util.isUndefined(field.searchableFields)) {
			propertyToAdd = (field.searchableFields);
		} else if (!Util.isUndefined(field.displayedFields)) {
			propertyToAdd = (field.displayedFields);
		} else {
			propertyToAdd = (field.descriptionFields);
		}

		addFieldNamesToPropertyToSearch(propertyToAdd);
	}

	/**
	 * Initilize the field.name value on it's currentRow.
	 *
	 * @function initializeValuesOnCurrentRow
	 * @private
	 */
	function initializeValuesOnCurrentRow() {
		var currentRow = $scope.field.widget.currentRow;
		var fieldName = $scope.field.name;
		currentRow[fieldName] = currentRow[fieldName] || "";
	}

	/**
	 * Remove the item from the current row by it's index.
	 *
	 * @function removeSelectedItemByIndex
	 * @private
	 */
	function removeSelectedItemByIndex(index) {
		var widgetParent = $scope.field.widget;
		var fieldName = $scope.field.name;
		widgetParent.currentRow[fieldName].splice(index, 1);
	}

	/**
	 * Functions used by template
	 **/

	/**
	 * Get the component's label.
	 *
	 * @function getFieldLabel
	 * @public
	 * @returns {String} The component's label.
	 */
	this.getFieldLabel = function() {
		return $scope.field.label;
	};

	/**
	 * Get the selected item's values.
	 *
	 * @function getSelectedValues
	 * @public
	 * @returns {Object} The values of the selected item.
	 */
	this.getSelectedValues = function() {
		return $scope.field.widget.currentRow[$scope.field.name];
	};

	/**
	 * Get the selected item's description field.
	 *
	 * @function getSelectedItemDescription
	 * @public
	 * @returns {String} The component's data.
	 */
	this.getSelectedItemDescription = function(item) {
		return item[$scope.field.descriptionField];
	};

	/**
	 * Get the component's data.
	 *
	 * @function getData
	 * @public
	 * @returns {Array<Object>} The component's data.
	 */
	this.getData = function() {
		return $scope.filteredData;
	};

	$scope.$watchCollection(function() {
		return self.getData();
	}, function(newValues, oldValues) {
		if (newValues !== oldValues) {
			if($scope.changeDropdownListPosition){
				$scope.changeDropdownListPosition();
			}
		}
	});

	$scope.dropDownLimit = 10;

	/**
	 * Updates the number of items in the dropdown.
	 *
	 * @function loadMore
	 * @public
	 */
	$scope.loadMore = function($event){
		if($scope.dropDownLimit === 20){
			$scope.dropDownLimit += 10;
			$scope.viewMore = ScreenService.i18n("Open search widget");
			$scope.openList();
			$event.stopPropagation();
		} else if($scope.dropDownLimit >= 30){
			$timeout(function(){$scope.inputWithEditableContent.blur();}, 0, false);
			resetDropdown();
			$scope.openField($scope.field, $event);
		} else {
			$scope.dropDownLimit += 10;
			$scope.openList();
			$event.stopPropagation();
		}
	};

	/**
	 * Set the dropdown to it's initial state.
	 *
	 * @function resetDropdown
	 * @private
	 */
	function resetDropdown() {
		$scope.dropDownLimit = 10;
		$scope.viewMore = ScreenService.i18n("View more...");
	}

	/**
	 * Verify if the LoadMore should be shown on dropDown.
	 *
	 * @function shouldShowLoadMore
	 * @public
	 */
	$scope.shouldShowLoadMore = function(){
		if($scope.field.hasPagination) {
			return $scope.filteredData.length >= $scope.dropDownLimit;
		} else {
			return false;
		}
	};

	/**
	 * Translate the text of the item according to field's option and i18n.
	 *
	 * @param  {Field}  field The field with the options.
	 * @param  {Object} item  The item With the text to be translated.
	 * @return {String}       The translated text.
	 */
	function translateText(field, item){
		if(!field.getTemplateSimpleName){
			Util.extend(field, new Templatable());
		}
		var text = item[field.name];
		return DecodeValueFieldService.model(field, text);
	}

	/**
	 * Returns the given item's text to be shown on the dropdown.
	 *
	 * @function getItemText
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemText = function(item) {
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var displayedFields = $scope.field.displayedFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(displayedFields){
			if(fields){
				displayedFields.forEach(buildTranslatedItemDescription);
			}
			else {
				displayedFields.forEach(buildItemDescription);
			}
		}
		else {
			if(fields){
				descriptionFields.forEach(buildTranslatedItemDescription);
			}
			else {
				descriptionFields.forEach(buildItemDescription);
			}
		}
		return itemDescription.join(textSpacer);
	};


	/**
	 * Returns the given item's descriptionField if the field has an descriptionField.
	 *
	 * @function getItemDescription
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemDescription = function(item){
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(fields){
			descriptionFields.forEach(buildTranslatedItemDescription);
		}
		else {
			descriptionFields.forEach(buildItemDescription);
		}
		itemDescription = itemDescription.join(textSpacer);

		return itemDescription;
	};

	/**
	 * Remove the item with the given index from the parent field's current row.
	 *
	 * @function onRemoveSelectedItem
	 * @public
	 * @param {Integer} item - The index of the item to be removed.
	 */
	this.onRemoveSelectedItem = function(removedItemIndex) {
		removeSelectedItemByIndex(removedItemIndex);
	};

	/**
	 * Functions called from directive
	 *
	 **/

	/**
	 * Functions used internally
	 **/

	/**
	 * Clear the field's search parameter and it's list data.
	 *
	 * @function clearAutoCompleteListData
	 */
	$scope.clearAutoCompleteListData = function() {
		$scope.itemSearch = '';
		$scope.clearInputFieldText();
		$scope.autoCompleteListData = [];
		$scope.clearSelectedItem();
		$scope.showSpan = true;
	};

	/**
	 * Clear the value on the field's input.
	 *
	 * @function clearEditableInput
	 * @public
	 */
	this.clearEditableInput = function() {
		$scope.inputWithEditableContent.val('');
	};

	/**
	 * Tells if the component should display the span with the currentRow value.
	 *
	 * @function shouldShowSpan
	 * @public
	 * @returns {Boolean} Value telling if the span should be displayed.
	 */
	this.shouldShowSpan = function() {
		var isReadOnly = $scope.field.isReadOnly();
		return isReadOnly || $scope.showSpan;
	};
}

// FILE: src/directives/zh-select-autocomplete.directive.js
// register jQuery extension to select by TAB and Navation by press TAB
jQuery.extend(jQuery.expr[':'], {
    focusable: function(el) {
        return $(el).is('a, button, :input, [tabindex]');
    }
});

/**
 *
 */
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zhSelectAutocomplete', ['$timeout', 'eventEngine', zhSelectAutocomplete]);

    function zhSelectAutocomplete($timeout, eventEngine) {

        var EDITABLE_INPUT_MIN_WIDTH = 130;
        var ENTER = 13;
        var SPACE = 32;
        var DOWN = 40;
        var ESC = 27;
        var TAB = 9;
        var UP = 38;


        var linkFunction = function(theScope, theElement) {

            var $scope = theScope;
            var element = theElement;
            var selectedList = '';
            var lastMouseDownElement;
            var loaderElement = $('img.autocomplete-loading-icon');
            var namespace = $scope.$id;
            var documentEvents = [];
            var lazyLoadEventIndex;

            $scope.hideLoader = function() {
                loaderElement.hide();
            };
            $scope.showLoader = function() {
                loaderElement.css('display', 'initial');
            };

            $scope.itemSearch = $scope.itemSearch || "";
            $scope.selectAutoCompleteElement = $(element);
            $scope.inputWithEditableContent = $(element).find('.editable');

            var initialPlaceholderValue;

            $scope.spanValueField = $(element).find('span');

            $scope.inputWithEditableContent.change(function(e) {
                e.stopPropagation();
            });

            $scope.field.element.click(function(e) {
                e.stopPropagation();
            });

            if($scope.field.lazyLoadEvent){
                lazyLoadEventIndex = eventEngine.bindEvent($scope.inputWithEditableContent, $scope.field.lazyLoadEvent, function(){
                    $scope.field.initializeField();
                });
            }

            /*
            *   Prevent loader from canceling element click
            */
            $('div.zh-background-loading').mouseup( function(e){
            	if(lastMouseDownElement && lastMouseDownElement === $scope.inputWithEditableContent[0]){
                    $scope.inputWithEditableContent.click();
                }
            });

            $(document).on('mousedown.'+namespace, function(e) {
                lastMouseDownElement = e.target;
            });
            documentEvents.push('mousedown');

            createFocusListener();
            clickListener();
            KeyPressNavegationListener();

            /**
             * Creates the click clickListener.
             *
             * @function clickListener
             * @private
             */
            function clickListener() {
                var showSelect = $(element).find(".show-select");
                showSelect.click(function(event) {
                    event.stopPropagation();
                });
            }

            /**
             * Resets the placeholder value.
             *
             * @function resetPlaceholderValue
             * @private
             */
            $scope.resetPlaceholderValue = function() {
                if(initialPlaceholderValue){
                    $scope.inputWithEditableContent[0].placeholder = initialPlaceholderValue;
                } else {
                    initialPlaceholderValue = $scope.inputWithEditableContent[0].placeholder;
                }
            };

            /**
             * Creates the focus listener.
             *
             * @function createFocusListener
             * @private
             */
            function createFocusListener() {
                $scope.inputWithEditableContent.focus(function(event) {
                    $scope.inputWithEditableContent.val($scope.itemSearch);
                    $scope.resetPlaceholderValue();
                    $scope.inputWithEditableContent.select();
                    $scope.inputWithEditableContent.parent().scrollTop(0);
                    $scope.showSpan = false;
                    $scope.changeDropdownListPosition();
                    $scope.$apply();
                    $scope.processBeforeSelectOpenEvent($scope.field);
                    $scope.openList();
                    $scope.updateDropdown();
                    $scope.processAfterSelectOpenEvent($scope.field);
                    $scope.selectAutoCompleteElement.closest("[zh-scroller]").on("scroll." + $scope.$id, function(){
                        $scope.hideList();
                    });
                });
            }

            /**
             * Updates the dropdown and selects the first item
             *
             * @function updateDropdown
             *
             * @returns {Array.<Object>} The selected items list.
             */
            $scope.updateDropdown = function(){
                if($scope.selectAutoCompleteElement.find('.selected').length === 0 && $scope.itemSearch){
                    var spanList = $scope.selectAutoCompleteElement.find('.show-select span');

                    selectedList = spanList.eq(0).addClass('selected');
                }

                return selectedList;
            };

            /**
             * Creates the key press navigation listener.
             *
             * @function KeyPressNavegationListener
             * @private
             */
            function KeyPressNavegationListener() {
                var next;
                var navegationKeyPress = $scope.selectAutoCompleteElement;
                navegationKeyPress.on('keydown', 'input,select', function(event) {

                    var EVENTKEY = event;
                    var spanList = $scope.selectAutoCompleteElement.find('.show-select span');

                    //Fix cursor input 'auto' to change position
                    if(EVENTKEY.which === DOWN || EVENTKEY.which === UP ) {
                        EVENTKEY.preventDefault();
                    }

                    //Enter, Space or Tab
                    if (EVENTKEY.which === ENTER || EVENTKEY.which === TAB) {
                        EVENTKEY.preventDefault();

                        if (isShowMore(selectedList)) {
                            $scope.loadMore(event);
                        }
                        else {
                            if($scope.shouldMakeRequest($scope.itemSearch)) {
                                $scope.selectAfterLoad();
                            } else if(canSelect(selectedList)){
                                $scope.selectItem(selectedList);
                                $scope.inputWithEditableContent.val("");
                            }
                            if (getFocusableItems().length > 1) {
                                $scope.removeFocusOfInput();

                                $scope.focusOnNextItem();
                                $scope.hideList();
                            }
                        }

                    }
                    //Down
                    else if (EVENTKEY.which === DOWN) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.next();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.eq(0).addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.eq(0).addClass('selected');
                        }
                    }
                    //Up
                    else if (EVENTKEY.which === UP) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.prev();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.last().addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.last().addClass('selected');
                        }
                    }
                    //Esc
                    else if (EVENTKEY.which === ESC) {
                        $scope.hideList();
                        selectedList = '';
                        return false;
                    }
                    if (navegationKeyPress) {
                        var itemList = $scope.selectAutoCompleteElement.find('.list-items');
                        var item = navegationKeyPress.find('.selected');
                        var index;

                        if (item.offset() !== undefined) {
                            if (EVENTKEY.which === ENTER) {
                                $timeout(function() {
                                    spanList = $scope.selectAutoCompleteElement.find('.show-select span');
                                    index = $scope.dropDownLimit - 10;
                                    movimentation = (index + 3.7) * 32 + item.outerHeight() - itemList.innerHeight();
                                    animateItemFocus(itemList, movimentation);
                                    selectedList = spanList.eq(index);
                                    selectedList.addClass('selected');
                                }, 1000, false);
                            }
                            else {
                                index = parseInt(item.attr('data-item-index')) || parseInt(item.prev().attr('data-item-index')) + 1;
                                var movimentation = 0;
                                if (item.position().top + item.outerHeight() > itemList.innerHeight()) {
                                    movimentation = (index * 32 + item.outerHeight() - itemList.innerHeight());
                                    animateItemFocus(itemList, movimentation);
                                }
                                else if (item.position().top < 0) {
                                    movimentation = index * 32;
                                    animateItemFocus(itemList, movimentation);
                                }
                            }
                        }
                    }
                });
            }

            /**
             * Makes the animation to scroll the item for the one who has focus.
             *
             * @function animateItemFocus
             * @private
             * @param  {Object} itemList      - The element witch has the list of items
             * @param  {Number} movimentation - The value in pixels of the scrolling distance from the top.
             */
            function animateItemFocus(itemList, movimentation) {
                itemList.finish().animate({
                    scrollTop: movimentation
                }, 500);
            }

            /**
             * Get all the focusabel items.
             *
             * @function getFocusableItems
             * @private
             * @returns {Array.<Object>} The focusable items on the screen.
             */
            function getFocusableItems() {
                return $(':focusable');
            }

            /**
             * Test if the given selectedList is a valid row to be selected.
             *
             * @function canSelect
             * @private
             * @returns {Boolean} Result telling if the selectedList is a valid row to be selected.
             */
            function canSelect(selectedList) {
                return (typeof selectedList === "string" && selectedList !== "") ||
                    (typeof selectedList === "object" && selectedList.text() !== "") &&
                    selectedList.attr("data-item-identifier");
            }

            function isShowMore(selectedItem) {
                return selectedItem && selectedItem.hasClass("view-more");
            }

            /**
             * Open the field's dropdown list.
             *
             * @function openList
             */
            $scope.openList = function() {
                $(".zh-select-autocomplete .show-select").css("display", "none");
                $scope.selectAutoCompleteElement.find(".show-select").css("display", "block");
                $scope.selectAutoCompleteElement.find('.list-items').scrollTop(0);
            };


            /**
             * Change the dropdown list position on field.
             *
             * @function changeDropdownListPosition
             */
            $scope.changeDropdownListPosition = function() {
                var dropdown = $scope.selectAutoCompleteElement.find('.zh-select-autocomplete');
                var dropdownOffset = dropdown.offset().top;
                var listItemsElement = $scope.selectAutoCompleteElement.find('.list-items');

                updateListWidth(listItemsElement, dropdown);
                updateListPosition(listItemsElement, dropdownOffset, dropdown);
            };

            /**
             * Updates the position of the dropdown list.
             *
             * @function updateListPosition
             * @param {Element} listItemsElement - The list's DOM.
             * @param {Element} dropdownOffset - The dropdown's offset.
             * @param {Element} dropdown - The dropdown's DOM.
             */
            function updateListPosition(listItemsElement, dropdownOffset, dropdown) {
                var popupModal = $('.popup-modal');
                if (popupModal.length) {
                    dropdownOffset -= popupModal.offset().top;
                }
                $timeout(function() {
                    listItemsElement.css('top', dropdownOffset + dropdown.outerHeight() - getParentFixedTop(dropdown));
                    if (dropdown.offset().top + dropdown.outerHeight() + listItemsElement.outerHeight() + 10 > $(window).height()) {
                        listItemsElement.css('margin-top', -1 * (dropdown.outerHeight() + listItemsElement.outerHeight()));
                    }
                    else {
                        listItemsElement.css('margin-top', 0);
                    }
                }, 0, false);
            }

            /**
             * Updates the width of the dropdown list.
             *
             * @function updateListPosition
             * @param {Element} listItemsElement - The list's DOM.
             * @param {Element} dropdown - The dropdown's DOM.
             */
            function updateListWidth(listItemsElement, dropdown) {
                //Set the correct width field
                if(listItemsElement.width() !== dropdown.innerWidth()){
                    listItemsElement.width(dropdown.innerWidth());
                }
            }

            /**
             * Get the parent fixed top position element.
             *
             * @function getParentFixedTop
             * @param {String} el - A element of the DOM.
             */
            function getParentFixedTop(el) {
                var parents = el.parents();
                var top = 0;
                parents.each(function(idx, item) {
                    item = $(item);
                    if ((item.css('position') === "fixed" || item.css('position') === "absolute") &&
                        item.parent() && (item.parent().css('position') === "fixed" || item.parent().css('position') === "absolute")) {
                        top = item.offset().top;
                        return false;
                    }
                });

                return top;
            }

            /**
             * Clear the field input's text.
             *
             * @function clearInputFieldText
             */
            $scope.clearInputFieldText = function() {
                $scope.selectAutoCompleteElement.find('input:text').val('');
            };

            /**
             * Hide the fields's dropdown list.
             *
             * @function hideList
             */
            $scope.hideList = function() {
                $scope.selectAutoCompleteElement.find(".show-select").hide();
                $scope.selectAutoCompleteElement.find('.show-select span').removeClass('selected');
                $scope.selectAutoCompleteElement.closest("[zh-scroller]").off("scroll." + $scope.$id);
            };

            /**
             * Set as selected the given item.
             *
             * @function selectItem
             * @param {Object} selectedList - The DOM element of the selected row.
             */
            $scope.selectItem = function(selectedList) {
                if (!selectedList) return;
                selectedItem = selectedList.text();
                var itemIdentifier = selectedList.attr("data-item-identifier");
                $scope.onAutoCompleteItemSelect(itemIdentifier);
            };

            /**
             * Focus on the next item of the screen.
             *
             * @function focusOnNextItem
             */
            $scope.focusOnNextItem = function() {
                $scope.inputHasFocus = false;
                $canfocus = getFocusableItems();
                var index = $canfocus.index(document.activeElement) + 1;
                if (index >= $canfocus.length) index = 0;
                $canfocus.eq(index).focus();
            };

            $scope.focusOnInput = function() {
                $timeout(function() {
                    $scope.inputWithEditableContent.focus();
                }, 0, false);
            };

            $scope.removeFocusOfInput = function() {

                $timeout(function() {
                    $scope.inputWithEditableContent.blur();

                    if($scope.itemSearch){
                        $scope.inputWithEditableContent[0].placeholder = $scope.itemSearch;
                    }

                    $scope.hideList();
                    $scope.showSpan = $scope.hasValueSetted();
                    $scope.clearInputFieldText();
                });
            };

            $scope.clearSelectedItem = function() {
                var item = $scope.inputWithEditableContent.find('.selected');
                selectedList = '';
                item.removeClass('selected');
            };

            /**
             * Treats the click outside the component.
             */
            $(document).on("click." + namespace, function(event) {
                if (!$.contains($scope.selectAutoCompleteElement[0], event.target)){
                    $scope.removeFocusOfInput();
                }
            });
            documentEvents.push('click');

            $scope.$on("$destroy", function(){
                documentEvents.forEach(function(eventName){
                    $(document).off(eventName + "." + namespace);
                });

                $('div.zh-background-loading').off('mouseup');
                $scope.selectAutoCompleteElement.closest("[zh-scroller]").off("scroll." + $scope.$id);
                eventEngine.unbindEvent($scope.inputWithEditableContent, $scope.field.lazyLoadEvent, lazyLoadEventIndex);
            });

        };
        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();

// FILE: src/registers/zh-select-autocomplete.register.js
(function () {

	Configuration(registerSelectAutocomplete);

	function registerSelectAutocomplete(decodeTemplateFieldService) {
		decodeTemplateFieldService.registerViewTemplate(
			"zh-select-autocomplete#field/select-autocomplete.html",
			"zh-select-autocomplete#field/select-autocomplete-view.html"
		);
	}

})();